
/*************************************************************************
 *  made by : Kees Lemmens
 *  version : 0.1
 *  date    : May 1999
 *
 *  class name  : MouseHandler
 *  function    : Contains routines to handle mouse-events in the canvas
 ************************************************************************/

import java.io.*;
import java.awt.*;
import java.awt.event.*; // Listeners

final class MouseHandler
 implements MouseListener,MouseMotionListener
{
   ViewCanvas canvas;
   Scaler scaler;
   private Point ll,ur,old_ur;
   
   public MouseHandler(ScrollCanvas viewport) // constructor
   {
      super();
      
      this.canvas = viewport.canvas;
      this.scaler = viewport.scaler;
      
      ll     = new Point();
      ur     = new Point();
      old_ur = new Point();
   }

   // MouseListener interface methods :
   public void mousePressed(MouseEvent e)
   {
      ll.x = ur.x = old_ur.x = e.getX();
      ll.y = ur.y = old_ur.y = e.getY();
   }
   
   public void mouseReleased(MouseEvent e)
   {
      if (ll.x >= ur.x || ll.y <= ur.y)
	scaler.setScreenSize(canvas.getSize());
      else
	scaler.setZoom(ll,ur);
      
      canvas.repaint();
   }

   // and the unused ones :
   public void mouseClicked (MouseEvent e) {;}
   public void mouseEntered (MouseEvent e) {;}
   public void mouseExited  (MouseEvent e) {;}
   
   // MouseMotionListener interface methods :
   public void mouseDragged(MouseEvent e)
   {
      ur.x = e.getX();
      ur.y = e.getY();

      if(old_ur.x == ur.x && old_ur.y == ur.y) return;
      Graphics g = canvas.getGraphics();

      // NOTE : setXORMode seems to be sometimes buggy on MS-Windows 98 !
      // if you have to use that lousy platform then uncomment the "setColor"
      // instructions instead : they look less fancy, but at least it works.
      
      g.setXORMode(Color.white); 
      // g.setColor(Color.white);
      g.drawRect(ll.x,old_ur.y,old_ur.x-ll.x,ll.y-old_ur.y);
      // g.setColor(Color.black);
      g.drawRect(ll.x,    ur.y,    ur.x-ll.x,ll.y-    ur.y);
      old_ur.x=ur.x; old_ur.y=ur.y;
   }
   
   // and the unused ones :
   public void mouseMoved   (MouseEvent e) {;}
}
