      subroutine chsortlong ( keysrt, kgrad, npoint )
! ====================================================================== &
!
!        programmer    Jos van Kan/Onno Hoitinga
!        version  1.0  date 12-11-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This routine gets as input array keysrt of length npoint
!     on output array kgrad contains the indices to sort array keysrt
!     The Heap sort algorithm is used
!     keysrt is long integer
! **********************************************************************
!
!                       KEYWORDS
!
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kgrad(npoint)
      integer (kind=8) keysrt(npoint)

!     keysrt   i    sortkey (integer) on array to be sorted.
!     kgrad    o    contains permutation index that sorts keysrt
!                   in ascending order on output.
!     npoint   i    number of entries in keysrt & kgrad
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) q
      integer i, j, l, ir, kgradt

!     i       Counting variable
!     ir      ?
!     j       Counting variable
!     kgradt  ?
!     l       General loop variable
!     q       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     This routine originates from Numerical Recipes (fortran edition)
!     Cambridge University Press 1989
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      do j = 1, npoint
         kgrad(j) = j
      end do
      if ( npoint<=1 ) go to 1000
      l = npoint/2+1
      ir = npoint

200     if ( l>1 ) then
          l = l-1
          kgradt = kgrad(l)
          q = keysrt(kgradt)
        else
          kgradt = kgrad(ir)
          q = keysrt(kgradt)
          kgrad(ir) = kgrad(1)
          ir = ir-1
          if ( ir==1 ) then
            kgrad(1) = kgradt
            go to 1000
          end if
        end if
        i = l
        j = l+l
300     if ( j<=ir ) then
          if ( j<ir ) then
            if ( keysrt(kgrad(j))<keysrt(kgrad(j+1)) ) j = j+1
          end if
          if ( q<keysrt(kgrad(j)) ) then
            kgrad(i) = kgrad(j)
            i = j
            j = j+j
          else
            j = ir+1
          end if
        go to 300
        end if
        kgrad(i) = kgradt
      go to 200

1000  end
