      subroutine copydf ( ivec1, ivec2, kprob1, kprob2, idf1, idf2, &
                          kmesh )
! ====================================================================== &
!
!        programmer    Frans van de Vosse
!        version  2.1  date 09-11-2012 Long length ibuffr
!        version  2.0  date 26-12-1998 Replace body by copydfbf
!        version  1.0  date 07-11-1994
!
!   copyright (c) 1994-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   copydf copies the contents of one degree of freedom in array ivec1
!   belonging to kprob1 into another degree of freedom in array ivec2
!   belonging to kprob2
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1(*), ivec2(*), kprob1(*), kprob2(*), idf1, idf2, &
              kmesh(*)

!     idf1       i    degree of freedom in ivec1 to be copied
!     idf2       i    degree of freedom in ivec2 to be filled
!     ivec1      i    old array (must be an excisting SEPRAN array)
!     ivec2     i/o   new array (must be an excisting SEPRAN array)
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kprob1     i    kprob of old array
!     kprob2     i    kprob of new array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     COPYDFBF  Actual body of copydf
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'copydf' )
      call copydfbf ( ibuffr, buffer, ivec1, ivec2, kprob1, kprob2, &
                      idf1, idf2, kmesh )
      call erclos ( 'copydf' )
      end
