      subroutine delsepabf ( iarray )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 09-11-2012 Long length ibuffr
!        version  1.0  date 10-04-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete a SEPRAN array and all of its presence in IBUFFR
!
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iarray(5)

!     iarray  i   Array to be deleted
!                 At present the array may be of the following types
!                 solution vector or vector of special structure: 110, 115, 116
!                 matrix (103)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itype

!     itype   Indicates the type of vector to be deleted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INI066  delete temporay array
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1585   SEPRAN arrays of this type  are not yet allowed in this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      itype = iarray(2)
      if ( itype==110 .or. itype==115 .or. itype==116 ) then

!     --- solution array

         if ( iarray(1)>0 ) call ini066( ibuffr, iarray(1) )

      else if ( itype==103 ) then

!     --- matrix

         itype = abs(iarray(4))
         if ( itype>0 )  call ini066( ibuffr, itype )
         itype = abs(iarray(4))
         iarray(4) = 0
         if ( iarray(5)>0 ) call ini066( ibuffr, iarray(5) )
         if ( itype/=iarray(3) .and. iarray(3)>0 ) &
              call ini066( ibuffr, iarray(3) )

      else

!     --- Not yet available

         call eropen ( 'delsepabf' )
         call errint ( itype, 1 )
         call errsub ( 1585, 1, 0, 0 )
         call erclos ( 'delsepabf' )

      end if
      iarray = 0
      end
