      subroutine derivatives ( ibuffr, buffer, isolut, kmesh, kprob,
     +                         ivectr, inpsep, numvct )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 01-07-2011 Extra parameter numvct
!        version  1.1  date 02-03-2011 Input of ivec
!        version  1.0  date 08-12-2010
!
!   copyright (c) 2010-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes derived quantities of an input vector, either in the nodal points
!     or in the elements
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cbuffr'
      include 'SPcommon/clinsl'
      include 'SPcommon/cmacht'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isolut(5,*), kmesh(*), kprob(*), ivectr,
     +        inpsep(*), numvct
      double precision buffer(*)

!     buffer        i/o   Real buffer array in which all large data is stored
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    See subroutine readscderiv
!     isolut        i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     ivectr         i    Sequence number of output vector
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numvct        i/o   Largest sequence number of vector that has been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinder(15), ipiusr, ipuser, nbufol, iskpel(1), icurvs(1),
     +        lenhome, idummy(1), iprob, isubr
      double precision output(1)
      character (len=80) filenm
      logical debug, check

!     check          Indicates if the file exist (true) or not
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     filenm         Name of file
!     icurvs         dummy
!     idummy         Dummy parameter
!     iinder         User input array for subroutine DERIV
!     ipiusr         Starting address of IUSER
!     iprob          Actual problem number
!     ipuser         Starting address of IUSER
!     iskpel         dummy
!     isubr          Indicates the type of calling subroutine
!     lenhome        Number of characters in the name of the SEPRAN home
!                    directory
!     nbufol         help variable to store the old value of nbuffr
!     output         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPENDBUF   Copy existing array to end of BUFFER
!     INIDCOPENDBUF  Copy the contents of the array referred to by the memory
!                    management index mmindex to the end of IBUFFR
!                    and decrease the length of IBUFFR
!                    double precision array
!     INILEN         Compute length of array in IBUFFR
!     PRININ         print 1d integer array
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'derivatives' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from derivatives'
         call prinin ( inpsep, 8, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nbufol = nbuffr

      lenhome = index(sphome,' ')-1
      filenm = sphome(1:lenhome)//'/bin/update/wi211'
      inquire ( file=filenm, exist=check )

!     --- Fill array iinder

      iinder(1) = 3   ! call by derivs
      iinder(2:3) = 0 ! output vector of type of solution vector (real)
      iinder(4) = inpsep(6)  ! icheld
      iinder(5:6) = 1   ! ix, jdegfd
      iinder(7) = inpsep(8)   ! ivec
      iinder(8) = numvct   ! numvec

      iinder(9) = 0   ! clearvector
      iinder(10:11) = inpsep(7)  ! input vector
      iinder(12:15) = 0 ! allelements, itype_boun, irule_boun, inumcurves
      if ( debug ) call prinin ( iinder, 15, 'iinder' )

!     --- Fill arrays iuser and user if possible

      if ( iuser1>0 ) then

!     --- Copy arrays IUSER and USER to end of IBUFFR.  In this way DERIVS
!         cannot overwrite the positions used by these arrays

         if ( inpsep(1)>0 ) then

!        --- Sequence number of coefficient given

            isubr = 3
            iprob = inpsep(2)
            if ( iprob<=0 ) iprob = isolut(4,ivectr)/1000+1
            call erlins ( ibuffr, buffer, kprob,
     +                    kmesh, isubr, iprob, inpsep(1) )

         end if  ! ( inpsep(1)>0 )

         call inicopendbuf ( ibuffr, iuser1, ipiusr, idummy(1),
     +                       'iuser' )
         call inidcopendbuf ( ibuffr, iuser2, ipuser, idummy(1),
     +                        'user' )
         if ( debug )
     +      call prinin ( ibuffr(ipiusr), inilen(iuser1), 'iuser' )

      else

!     --- iuser and user have not been filled

         ipiusr = 1
         ipuser = 1

      end if

!     --- call actual subroutine

      call to0100 ( ibuffr, buffer, iinder, isolut(1,ivectr), kmesh,
     +              kprob, isolut, ibuffr(ipiusr), buffer(ipuser),
     +              output, iskpel, icurvs )
      numvct = max(numvct,ivectr)

      nbuffr = nbufol

1000  call erclos ( 'derivatives' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End derivatives'

      end if  ! ( debug )

      end
