      subroutine er0081 ( ibuffr, buffer, kmesh, kprob, ivectr, icurvs, &
                          idim, iquant, number, isolnw, indknw, &
                          ipoint, nparts, normal, indtan, iregion, iseq, &
                          isubr, nodes, nnodes )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  4.4  date 30-03-2015 Remove non positive nodes
!        version  4.3  date 09-11-2012 New calls to er0080/82
!        version  4.2  date 21-09-2012 New treatment reals
!
!   copyright (c) 1994-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store solution of region required into helparray
!     If necessary the tangential or normal component is computed
!     Warning: this subroutine uses memory management subroutines and
!               should be treated as such a subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)
      integer ivectr(*), kmesh(*), kprob(*), icurvs(*), idim, iquant, &
              isolnw, indknw, ipoint(*), nparts, normal, indtan, &
              number, iregion, iseq, isubr, nodes(*), nnodes
      double precision buffer(*)

!     buffer    i/o   Real buffer array in which all large data is stored
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     icurvs     i    Integer array containing the curve or surface numbers
!                     that must be printed.
!                     Pos. 1 indicates how array icurvs is filled.
!                     Possible values:
!                      0   only one curve or surface is used. The curve of
!                          surface number is stored in icurvs(2)
!                     -1   The solution is printed along the curves or surfaces
!                          icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                          icrv2 in icurvs(3).
!                     -2   See -1, however, the sequence of the printing is
!                          reversed. In case of curves this means that the
!                          curves are printed from the end point of icrv2 to
!                          the initial point of icrv2, then the end point of
!                          icrv2-1 to the begin point of icrv2-1 and so on.
!                          In case of surfaces ichois=-2 is treated as ichois=-1
!                     n>0  The curve numbers or surface numbers are stored in
!                          icurvs(2) to icurvs(n+1). That sequence is used.
!                     Remark: a negative curve number means that the curve is
!                          printed from end point to initial point, hence in
!                          reversed order. Negative surface numbers are not yet
!                          available.
!     idim       i    indicates which quantities must be printed.
!                     Possible values:
!                      0: The complete vector is printed
!                      1: The vector is only printed in user points.
!                         These user points must be defined in array ICURVS
!                      2: The vector is only printed in user curves.
!                         These curves must be defined in array ICURVS
!                      3: The vector is only printed in user surfaces.
!                         These surfaces must be defined in array ICURVS
!                      4: The vector is only printed in user volumes.
!                         These volumes must be defined in array ICURVS
!                     20: Zero level set
!     indknw     o    Infor entry with respect to the array containing the
!                     starting addresses of the nodal points in the new
!                     solution array
!                     If negative the number of unknowns in the array is
!                     constant and nunkp = -indknw
!                     Mark that this is always a new array, that should be
!                     deleted if no longer in use
!     indtan     o    Infor entry with respect to the array containing the
!                     tangential vectors along the boundaries
!                     This array is only used in case of curves
!                     Mark that this is always a new array, that should be
!                     deleted if no longer in use
!     ipoint     o    Array of length nparts+1 containing the starting
!                     addresses in array inodes of the various parts
!                     the boundary consists of.
!                     ipoint(1) = 1, &
!                     number of nodes at part i = ipoint(i+1)-ipoint(i)
!     iquant     i    indicates which quantity must be computed
!                     Possible values:
!                     0:  The contents of the vector are computed as stored in
!                         the vector corresponding to IVECTR i.e. without
!                         transformation.
!                     >0: The normal or tangential component of a vector along
!                         the curve or surface must be computed.
!                         The vector consists of two (R2) or three (R3)
!                         components
!                         The normal vector in the nodes is stored in array
!                         normal and the tangential vector in the array
!                         corresponding to indtan
!     iregion    i    indicates if the complete region must be printed
!                     (0) or that the printing is restricted to a sub region, &
!                     defined by the parameters xmin, xmax, ymin, ymax, zmin, &
!                     zmax in the comon block CPLOT.
!                     Possible values:
!                      0:  The complete region is printed
!                      1:  The region is restricted to xmin<=x<=xmax, &
!                          ymin<=y<=ymax, zmin<=z<=zmax
!                     -1:  The region is restricted to x=xmin
!                     -2:  The region is restricted to y=ymin
!                     -3:  The region is restricted to z=zmin
!                     -11: The region is restricted to x>=xmin
!                     -12: The region is restricted to y>=ymin
!                     -13: The region is restricted to z>=zmin
!                     -21: The region is restricted to x<=xmin
!                     -22: The region is restricted to y<=ymin
!                     -23: The region is restricted to z<=zmin
!     iseq       i    indicates the sequence in which the co-ordinates
!                     along the surface must be ordered in case idim=3
!                     Possible values:
!                     1:  increasing x, for fixed x increasing y, &
!                         for fixed y increasing z
!                     2:  increasing y, for fixed y increasing z, &
!                         for fixed z increasing x
!                     3:  increasing z, for fixed z increasing x, &
!                         for fixed x increasing y
!                     4:  sequence of nodal point numbers
!     isolnw     o    Infor entry with respect to new solution array
!                     Mark that this is always a new array, that should be
!                     deleted if no longer in use
!     isubr      i    Indicates which is the calling subroutine
!                     1    print subroutine, warning if too few points
!                     2    other subroutine, error
!     ivectr     i    Standard SEPRAN array referring to the vector to be
!                     printed
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kprob      i    Standard SEPRAN array, containing information of the
!                     problem
!     nnodes     o    Number of nodes
!     nodes      o    Contains the node numbers that must be filled
!     normal     o    Infor entry with respect to the array containing the
!                     normal vectors along the boundaries
!                     This array is only used in case of curves
!                     Mark that this is always a new array, that should be
!                     deleted if no longer in use
!     nparts     o    Number of parts the nodes array is subdivided into
!     number     i    indicates which degree of freedom in each point
!                     must be computed. This parameter must be seen in
!                     connection with iquant
!                     Possible values:
!                     0:  If iquant=0, all degrees of freedom are printed
!                         in the nodes defined by icurvs, &
!                         if iquant>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom 1, second degree of freedom 2 and third
!                         degree of freedom 3. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
!                     i>0: If iquant=0, the degree of freedom i is printed
!                         if iquant>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom i, second degree of freedom i+1 and third
!                         degree of freedom i+2. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipcoor, iptang, ipnorm, ipkprf, ipkprh, ipkprp, &
                       ipkpnw, ipusol
      integer indprf, indprh, indprp, nphys, nunkp, npoint, iplace, &
              ncurvs, nummax, ndim, iprob, length, ipkprb, error
      double precision, allocatable :: unew(:)
      logical debug, compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprh         indication if kprob part h has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipcoor         starting address of co-ordinates array in BUFFER
!     ipkpnw         Starting address of new array KPROB part f into array
!                    IBUFFR
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     ipkprf         Starting address of array KPROB part f in IBUFFR
!     ipkprh         starting address of array kprob part h
!     ipkprp         Starting address of array KPROB part p into array IBUFFR
!     iplace         Corrected value of idim
!                    Defines the type of input for prfillnodes
!     ipnorm         Starting address of array ANORM into array IBUFFR
!     iprob          Actual problem number
!     iptang         Starting address of array TANG into array IBUFFR
!     ipusol         Starting address of solution array in IBUFFR
!     length         Help parameter to indicate the length of arrays
!     ncurvs         Number of curves in mesh
!     ndim           Dimension of space
!     nphys          Number of physical unknowns, or maximal degrees of freedom
!                    per point
!     npoint         Number of nodal points
!     nummax         Maximum number of unknowns per point in new solution array
!     nunkp          Maximum number of unknowns in a point
!     unew           Array to store the new solution temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext
      integer iniprb

!     ER0076         Delete all points in inodes and ipoint not containing the
!                    required degree of freedom as given by number
!     ER0080         Find all degrees of freedom from the solution array or
!                    array of special structure as indicated by array inodes and
!                    store in output array. Real case
!     ER0082         Find all degrees of freedom from the solution array or
!                    array of special structure as indicated by array inodes and
!                    store in output array. Complex case
!     ER0083         Compute normal and tangential vectors along a boundary
!     ER0083D        Compute tangential vector along curves in R^3
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI068         compute pointers to degrees of freedom of vectors
!     INI080         Create permanent space in array BUFFER
!     INI081         Create permanent space in array IBUFFR
!     INIDCOPBUF     Activate or create the real array referred to by the
!                    memory management index mmindex and copy the contents
!                    of array to buffer
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INIPRB         Get value of ipkprb as function of iprob
!     PRFILLNODES    Fill array nodes with nodal points as defined in array
!                    inpsep
!     PRININ         print 1d integer array
!     PRREMNEGNODES  Remove all non-positive node numbers from array nodes
!     PRSW02         Restrict nodes to those in a specified region
!     PRSW04         Sort nodal points with respect to increasing co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     102   NUMBER has wrong value
!     387   too few degrees of freedom in nodal points in complete part of
!           boundary to integrate.
!    1578   quant>0 and idim<2 not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Activate ivectr and corresponding arrays
!    Compute nodal point numbers in which the vector must be computed
!    if ( idim=2 ) then
!       Reserve space and activate arrays
!       Create arrays "inodes" and "ipoint"
!    else ( idim=3 )
!       Reserve space and activate arrays
!       Create array "inodes"
!    delete nodal points without degree of freedom in "inodes"
!    Store quantities to be computed in usolnw
!    if ( iquant>0 ) then
!       Compute normal and tangential vectors
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'er0081' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0081'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate ivectr and corresponding arrays

      ndim = kmesh(6)
      call ini068 ( ibuffr, ivectr, kprob, indprf, indprh, indprp, &
                    nunkp, nphys )
      if ( number<0 .or. number>nphys ) then
         call errint ( number, 1 )
         call errint ( nphys, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( iquant>0 .and. number+idim>nphys ) then
         call errint ( number+idim, 1 )
         call errint ( nphys, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( iquant>0 .and. idim<2 ) then
         call errint ( iquant, 1 )
         call errint ( idim, 2 )
         call errwar ( 1578, 2, 0, 0 )
         iquant = 0
      end if
      compl = mod(ivectr(3),10)==1
      npoint = kmesh(8)
      ncurvs = kmesh(11)

      if ( debug ) then
         write(irefwr,1) 'ndim, number, nphys, idim, iquant, nunkp', &
                          ndim, number, nphys, idim, iquant, nunkp
         write(irefwr,1) 'npoint, ncurvs', npoint, ncurvs
         write(irefwr,*) 'compl ', compl
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute nodal point numbers in which the vector must be computed

      iplace = idim
      if ( idim==4 ) iplace = 7

      iprob = ivectr(4)/1000+1
      ipkprb = iniprb ( iprob, kprob )+1
      nnodes = 0

      call prfillnodes ( ibuffr, kmesh, iplace, 0, icurvs, &
                         nodes, nnodes, ipoint, nparts, kprob(ipkprb) )
      if ( ierror/=0 ) go to 1000

!     --- Remove all nodes with negative value

      call prremnegnodes ( nodes, nnodes )

      if ( debug ) then
         write(irefwr,1) 'iregion', iregion
         call prinin ( nodes, nnodes, 'nodes' )
      end if  ! ( debug )

      if ( iregion/=0 ) then

!     --- iregion # 0, restrict region

         call ini060text ( ibuffr, -kmesh(23), 'coor' )
         ipcoor = inidgttext ( kmesh(23), 'coor' )
         if ( ierror/=0 ) go to 1000
         call prsw02 ( nodes, ipoint, nparts, &
                       iregion, buffer(ipcoor), ndim, idim )
         if ( ierror/=0 ) go to 1000
         if ( idim==2 ) then
            nnodes = ipoint(nparts+1)-1
         else
            nnodes = nparts
         end if
         if ( debug ) write(irefwr,1) 'nnodes', nnodes

      end if

!     --- delete nodal points without degree of freedom in inodes

      if ( iquant==0 ) then
         nummax = 1
      else
         nummax = ndim
         number = max(1,number)
      end if

      if ( debug ) &
         write(irefwr,1) 'iquant, nummax, number, indprf, indprp', &
                          iquant, nummax, number, indprf, indprp
      ipkprf = 1
      ipkprh = 1
      ipkprp = 1
      if ( indprp>0 .and. nunkp/=nphys .and. number>0 .or. &
           indprf>0 ) then

!     --- indprf # 0, number of unknowns not constant in array or
!         indprp # 0, degrees of freedom may vary per point

         if ( indprf>0 ) ipkprf = inigettext ( indprf, 'kprobf' )
         if ( indprp>0 ) ipkprp = inigettext ( indprp, 'kprobp' )
         if ( ierror/=0 ) go to 1000
         call er0076 ( idim, nparts, ipoint, nodes, &
                       indprp, ibuffr(ipkprf), ibuffr(ipkprp), &
                       npoint, nphys, number, nummax )
         if ( debug ) write(irefwr,1) 'nparts', nparts
         if ( nparts<=0 ) then

!        --- No nodes found at curve or surfaces

            call errint ( nparts, 1 )
            if ( isubr==1 ) then
               call errwar ( 387, 1, 0, 0 )
               nnodes = 0
               go to 1000
            else
               call errsub ( 387, 1, 0, 0 )
            end if

         end if
         if ( idim==2 ) then
            nnodes = ipoint(nparts+1)-1
         else
            nnodes = nparts
         end if
         if ( debug ) write(irefwr,1) 'nnodes', nnodes

      end if
      if ( nnodes==0 ) then
         go to 1000
      end if
      if ( ierror/=0 ) go to 1000

!     --- Sort nodal points with respect to increasing co-ordinate

      if ( iseq<4 .and. (idim==0 .or. idim>=3) ) then

         call ini060text ( ibuffr, -kmesh(23), 'coor' )
         ipcoor = inidgttext ( kmesh(23), 'coor' )
         if ( ierror/=0 ) go to 1000
         call prsw04 ( iseq, nnodes, nodes, ndim, &
                       buffer(ipcoor) )
         if ( ierror/=0 ) go to 1000

      end if

!     --- Store quantities to be computed in usolnw

      indknw = 0
      if ( indprf==0 .or. number>0 ) then

!     --- Constant number of degrees of freedom in output vector

         ipkpnw = 1
         if ( number>0 ) then
            length = ndim*nnodes
         else
            length = nunkp*nnodes
         end if

      else

!     --- variable number of degrees of freedom in output vector

         call ini081 ( ibuffr, indknw, nnodes+1, 'nnodes' )
         length = nnodes*nphys

      end if
      isolnw = 0
      if ( compl ) length = 2*length

      allocate ( unew(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'unew' )
      if ( ierror/=0 ) go to 1000

      ipusol = inidgttext ( ivectr(1), 'vector' )
      if ( indprf>0 ) ipkprf = inigettext ( indprf, 'kprobf' )
      if ( indprh>0 ) ipkprh = inigettext ( indprh, 'kprobh' )
      if ( indprp>0 ) ipkprp = inigettext ( indprp, 'kprobp' )
      if ( indknw>0 ) ipkpnw = inigettext ( indknw, 'indknw' )
      if ( ierror/=0 ) go to 1000

      if ( compl ) then
         call er0082 ( indprf, indprh, indprp, ibuffr(ipkprf), &
                       ibuffr(ipkprh), ibuffr(ipkprp), npoint, nunkp, &
                       number, nummax, buffer(ipusol), unew, &
                       ibuffr(ipkpnw), nodes, nnodes, length )
      else
         call er0080 ( indprf, indprh, indprp, ibuffr(ipkprf), &
                       ibuffr(ipkprh), ibuffr(ipkprp), npoint, nunkp, &
                       number, nummax, buffer(ipusol), unew, &
                       ibuffr(ipkpnw), nodes, nnodes, length )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Make unew a permanent array in buffer

      call inidcopbuf ( buffer, ibuffr, isolnw, unew, &
                        length, 'usolnew' )
      if ( ierror/=0 ) go to 1000

      deallocate ( unew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'unew' )
      if ( ierror/=0 ) go to 1000

      indtan = 0
      normal = 0
      if ( iquant>0 ) then

!     --- iquant>0, compute tangential and normal component

         if ( idim==2 .and. ndim==2 ) then

!        --- idim=ndim=2, curves in R^2

            length = nnodes*2
            call ini080 ( ibuffr, normal, length, 'normal' )
            call ini080 ( ibuffr, indtan, length, 'tang' )
            call ini060text ( ibuffr, -kmesh(23), 'coor' )
            ipcoor = inidgttext ( kmesh(23), 'coor' )
            iptang = inidgttext ( indtan, 'indtan' )
            ipnorm = inidgttext ( normal, 'normal' )
            if ( ierror/=0 ) go to 1000
            call er0083 ( buffer(ipcoor), buffer(iptang), &
                          buffer(ipnorm), nodes, nparts, &
                          ipoint )

         else if ( idim==2 ) then

!        --- idim=2, ndim=3, curves in R^3, compute only the tangential vector

            length = nnodes*3
            call ini080 ( ibuffr, indtan, length, 'tang' )
            call ini060text ( ibuffr, -kmesh(23), 'coor' )
            ipcoor = inidgttext ( kmesh(23), 'coor' )
            iptang = inidgttext ( indtan, 'indtan' )
            if ( ierror/=0 ) go to 1000
            call er0083d ( buffer(ipcoor), buffer(iptang), &
                           nodes, nparts, ipoint )

         end if

      end if
      if ( ierror/=0 ) go to 1000
      if ( indknw==0 ) then
         if ( ndim==3 .and. idim==2 .and. &
              iquant>0 ) then
            indknw = -ndim
         else if ( number==0 ) then
            indknw = -nunkp
         else
            indknw = -max(1,iquant)
         end if
      end if

1000  call erclos ( 'er0081' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nparts', nparts
         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End er0081'

      end if  ! ( debug )

      end
