      subroutine er0082 ( indprf, indprh, indprp, kprobf, kprobh, &
                          kprobp, npoint, nunkp, number, nummax, usol, &
                          solnew, kprfnw, inodes, nnodes, length )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.0  date 09-11-2012 Extra parameter length
!        version  1.0  date 27-05-1994
!
!   copyright (c) 1994-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find all degrees of freedom from the solution array or array of special
!   structure as indicated by array inodes and store in output array
!   Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprf, indprh, indprp, kprobf(*), kprobh(*), npoint, &
              nunkp, kprobp(npoint,*), number, nummax, kprfnw(*), &
              nnodes, inodes(nnodes), length
      double complex usol(*), solnew(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     indprh         i    indication if kprob part h has been filled (>0)
!                         or not (0)
!     indprp         i    Indication if array kprob part p has been filled (1)
!                         or not (0)
!     inodes         i    Infor entry with respect to the array containing the
!                         nodal point numbers at output
!     kprfnw         o    New array KPROB part f with respect to array output
!                         Array kprfnw is only filled if number=0 and indprf>0
!                         It has exactly the same structure as kprobf, i.e.
!                         kprfnw(1) = 0
!                         number of degrees of freedom in inodes(i) = &
!                         kprfnw(i+1)-kprfnw(i)
!                         Length: nnodes+1
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     kprobh         i    KPROB part h: contains the new numbering of the
!                         unknowns
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unknowns per point
!     length        i/o   Length of solnew
!     nnodes         i    Number of nodes in inodes
!     npoint         i    Number of nodal points
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nummax         i    Maximum number of unknowns to be extracted in case
!                        number>0
!     nunkp          i    Maximum number of unknowns in a point
!     solnew         o    Output vector containing the solution in the nodes as
!                         stored in inodes
!                         If number>0, solnew may be considered as a
!                         two-dimensional maxnum x nnodes array with solnew(j,i)
!                         the j-th unknown compared to number in the
!                         i-th node of inodes
!                         If number=0 the solution vector is a one-dimensional
!                         vector containing all degrees of freedom in node
!                         inodes(1), followed by all degrees of freedom
!                         in node inodes(2), etcetera
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inunkp, inodp, k, ih, istart, lstart

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     istart         Starting address of node in array USOL
!     k              Counting variable
!     lstart         Starting address of node in array SOLNEW
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( indprf=0 or number>0 or nummax>1 ) then
!       For i := 1 (1) nnodes
!          Store nummax unknowns in solnew
!    else ( indprf>0, number=0 and nummax=1 )
!       For i := 1 (1) nnodes
!          Store all degrees of freedom in solnew
!          Fill array kprobf_new
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      lstart = 0
      if ( number>0 ) then

!     --- number>0

         if ( indprp>0 ) then

!        --- indprp>0, use KPROB part p

            if ( nummax==1 ) then

!           --- nummax = 1

               do i = 1, nnodes
                  inodp = inodes(i)
                  solnew(i-1+number) = usol(kprobp(inodp,number))
               end do  ! i = 1, nnodes
            else

!           --- nummax>1

               do k = number, number+nummax-1
                  ih = k-number+1
                  do i = 1, nnodes
                     inodp = inodes(i)
                     solnew((i-1)*nummax+ih) = usol(kprobp(inodp,k))
                  end do  ! i = 1, nnodes
               end do  !  k = number, number+nummax-1
            end if
         else

!        --- indprp = 0

            if ( indprf==0 ) then

!           --- indprf = 0

               if ( indprh==0 ) then

!              --- indprh = 0

                  if ( nummax==1 ) then

!                 --- nummax = 1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        solnew(i) = usol(nunkp*(inodp-1)+number)
                     end do  ! i = 1, nnodes
                  else

!                 --- nummax>1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        istart = nunkp*(inodp-1)+number-1
                        do k = 1, nummax
                           solnew((i-1)*nummax+k) = usol(istart+k)
                        end do  ! k = 1, nummax
                     end do  ! i = 1, nnodes
                  end if
               else

!              --- indprh>0

                  if ( nummax==1 ) then

!                 --- nummax = 1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        solnew(i) = usol(kprobh(nunkp*(inodp-1)+number))
                     end do  ! i = 1, nnodes
                  else

!                 --- nummax>1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        istart = nunkp*(inodp-1)+number-1
                        do k = 1, nummax
                           solnew((i-1)*nummax+k) = &
                              usol(kprobh(istart+k))
                        end do  ! k = 1, nummax
                     end do  ! i = 1, nnodes
                  end if
               end if
            else

!           --- indprf>0

               if ( indprh==0 ) then

!              --- indprh = 0

                  if ( nummax==1 ) then

!                 --- nummax = 1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        solnew(i) = usol(kprobf(inodp)+number)
                     end do  ! i = 1, nnodes
                  else

!                 --- nummax>1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        istart = kprobf(inodp)+number-1
                        do k = 1, nummax
                           solnew((i-1)*nummax+k) = usol(istart+k)
                        end do  ! k = 1, nummax
                     end do  ! i = 1, nnodes
                  end if
               else

!              --- indprh>0

                  if ( nummax==1 ) then

!                 --- nummax = 1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        solnew(i) = usol(kprobh(kprobf(inodp)+number))
                     end do  ! i = 1, nnodes
                  else

!                 --- nummax>1

                     do i = 1, nnodes
                        inodp = inodes(i)
                        istart = kprobf(inodp)+number-1
                        do k = 1, nummax
                           solnew((i-1)*nummax+k) = &
                              usol(kprobh(istart+k))
                        end do  ! i = 1, nnodes
                     end do  ! k = 1, nummax
                  end if
               end if
            end if
         end if
      else

!     --- number = 0, all unknowns

         if ( indprf==0 ) then

!        --- indprf = 0, constant number of degrees of freedom

            if ( indprh==0 ) then

!           --- indprh = 0, no renumbering

               do i = 1, nnodes
                  inodp = inodes(i)
                  do k = 1, nunkp
                     solnew((i-1)*nunkp+k) = usol((inodp-1)*nunkp+k)
                  end do
               end do
            else

!           --- indprh>0, renumbering

               do i = 1, nnodes
                  inodp = inodes(i)
                  do k = 1, nunkp
                     solnew((i-1)*nunkp+k) = &
                        usol(kprobh((inodp-1)*nunkp+k))
                  end do
               end do
            end if
         else

!        --- indprf>0, variable number of degrees of freedom

            lstart = 0
            if ( indprh==0 ) then

!           --- indprh = 0, no renumbering

               do i = 1, nnodes
                  inodp = inodes(i)
                  istart = kprobf(inodp)
                  inunkp = kprobf(inodp+1)-istart
                  solnew(lstart+1:lstart+inunkp) = &
                     usol(istart+1:istart+inunkp)
                  kprfnw(i) = lstart
                  lstart = lstart+inunkp
               end do ! i = 1, nnodes
               kprfnw(nnodes+1) = lstart
            else

!           --- indprh>0, renumbering

               do i = 1, nnodes
                  inodp = inodes(i)
                  istart = kprobf(inodp)
                  inunkp = kprobf(inodp+1)-istart
                  solnew(lstart+1:lstart+inunkp) = &
                     usol(kprobh(istart+1:istart+inunkp))
                  kprfnw(i) = lstart
                  lstart = lstart+inunkp
               end do ! i = 1, nnodes
               kprfnw(nnodes+1) = lstart
            end if
            length = lstart
         end if
      end if
      end
