      subroutine ercpdf01 ( vec1, kpf1, kph1, inkpf1, inkph1, nphys1, &
                            idf1, vec2, kpf2, kph2, inkpf2, inkph2, &
                            nphys2, idf2, npoint )
! ====================================================================== &
!
!        programmer     Frans van de Vosse
!        version 1.0    date   09-11-1994
!
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   ercpdf01 copies the contents of one degree of freedom in array vec1
!   belonging to kprob1 into another degree of freedom in array vec2
!   belonging to kprob2
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer inkpf1, inkph1, inkpf2, inkph2, kpf1(*), kph1(*), &
              kpf2(*), kph2(*), idf1, idf2, nphys1, nphys2, npoint
      double precision vec1(*), vec2(*)

!     idf1       i    degree of freedom of vector to be copied
!     idf2       i    degree of freedom of vector to be filled
!     inkpf1     i    index of kprob part f of problem 1
!     inkpf2     i    index of kprob part h of problem 1
!     inkph1     i    index of kprob part f of problem 2
!     inkph2     i    index of kprob part h of problem 2
!     kpf1       i    kprob part f of problem 1
!     kpf2       i    kprob part h of problem 1
!     kph1       i    kprob part f of problem 2
!     kph2       i    kprob part h of problem 2
!     nphys1     i    number of dof kprob1 (if constant)
!     nphys2     i    number of dof kprob2 (if constant)
!     npoint     i    Number of nodal points
!     vec1       i    input vector
!     vec2       o    output vector
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint, ndf1, ndf2, ipvec1, ipvec2

!    ipoint    Counter for node numbers
!    ipvec1    Pointer in vec1
!    ipvec2    Pointer in vec2
!    ndf1      Number of degrees of freedom
!    ndf2      Number of degrees of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ercpdf01' )

      if ( inkpf2.gt.0 .and. inkpf1.gt.0 ) then

!     --- not a constant number of degrees of freedom in vec2
!         not a constant number of degrees of freedom in vec1

         do 100 ipoint = 1, npoint
            ndf2 = kpf2(ipoint+1) - kpf2(ipoint)
            if ( idf2.le.ndf2 ) then
               ipvec2 = kpf2(ipoint)+idf2
               if ( inkph2.ne.0 ) ipvec2 = kph2(ipvec2)
               vec2(ipvec2) = 0d0
               ndf1 = kpf1(ipoint+1) - kpf1(ipoint)
               if ( idf1.le.ndf1 ) then
                  ipvec1 = kpf1(ipoint)+idf1
                  if ( inkph1.ne.0 ) ipvec1 = kph1(ipvec1)
                  vec2(ipvec2) = vec1(ipvec1)
               end if
            end if
  100    continue
      else if ( inkpf2.gt.0 .and. inkpf1.eq.0 ) then

!     --- not a constant number of degrees of freedom in vec2
!         a constant number of degrees of freedom in vec1

         ndf1 = nphys1
         do 150 ipoint = 1, npoint
            ndf2 = kpf2(ipoint+1) - kpf2(ipoint)
            if ( idf2.le.ndf2 ) then
               ipvec2 = kpf2(ipoint)+idf2
               if ( inkph2.ne.0 ) ipvec2 = kph2(ipvec2)
               vec2(ipvec2) = 0d0
               if ( idf1.le.ndf1 ) then
                  ipvec1 = (ipoint-1)*ndf1+idf1
                  if ( inkph1.ne.0 ) ipvec1 = kph1(ipvec1)
                  vec2(ipvec2) = vec1(ipvec1)
               end if
            end if
  150    continue

      else if ( inkpf2.eq.0 .and. inkpf1.gt.0 ) then

!     --- constant number of degrees of freedom in vec2
!         not a constant number of degrees of freedom in vec1

         ndf2 = nphys2
         do 200 ipoint = 1, npoint
            if ( idf2.le.ndf2 ) then
               ipvec2 = (ipoint-1)*ndf2+idf2
               if ( inkph2.ne.0 ) ipvec2 = kph2(ipvec2)
               vec2(ipvec2) = 0d0
               ndf1 = kpf1(ipoint+1) - kpf1(ipoint)
               if ( idf1.le.ndf1 ) then
                  ipvec1 = kpf1(ipoint)+idf1
                  if ( inkph1.ne.0 ) ipvec1 = kph1(ipvec1)
                  vec2(ipvec2) = vec1(ipvec1)
               end if
            end if
  200    continue
      else if ( inkpf2.eq.0 .and. inkpf1.eq.0 ) then

!     --- constant number of degrees of freedom in vec1
!         constant number of degrees of freedom in vec2

         ndf1 = nphys1
         ndf2 = nphys2
         do 250 ipoint = 1, npoint
            if ( idf2.le.ndf2 ) then
               ipvec2 = (ipoint-1)*ndf2+idf2
               if ( inkph2.ne.0 ) ipvec2 = kph2(ipvec2)
               vec2(ipvec2) = 0d0
               if ( idf1.le.ndf1 ) then
                  ipvec1 = (ipoint-1)*ndf1+idf1
                  if ( inkph1.ne.0 ) ipvec1 = kph1(ipvec1)
                  vec2(ipvec2) = vec1(ipvec1)
               end if
            end if
  250    continue
      end if

      call erclos ( 'ercpdf01' )
      end
