      subroutine erdump ( mesnum )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.4  date 13-06-2002 Remove trailing spaces
!        version  1.3  date 19-01-1999 include commons
!        version  1.2  date 17-10-1992 Removal of readi4
!        version  1.1  date 27-06-1992 Initialise idum
!        version  1.0  date 26-12-1991
!
!   copyright (c) 1991-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     erdump prints a message from the error message file, but without
!     extra texts. In this way it is fit for interactive input
!     Furthermore, no substitution of parameters takes place
! **********************************************************************
!
!                       KEYWORDS
!
!     message_manipulation
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmachnf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mesnum

!     mesnum   i    Message number.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer      i, ifrom, istart(21), ito ,maxerr, linenr, numlin, &
                   iref, idum, ilast
      parameter    ( iref=4 )
      logical      nxtlin
      character*80 errlin

!     errlin   One line of the message is stored in this parameter
!     i        loop variable
!     idum     Integer dummy parameter
!     ifrom    information in array istart is read from position ifrom
!     ilast    last non-blank character in line
!     iref     Reference to error message file
!     istart   integer array containing a part of the starting records
!              of the messages.
!     ito      information in array istart is read to position ito.
!     linenr   record number of message line
!     maxerr   1 + maximum number of messages that are stored
!              in the message file.
!     numlin   number of lines of a message
!     nxtlin   true: scan next line, false :don't scan next line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     READFL   Read from a SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!     erdump writes to the default output device.
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     read maximum message number from error text file ;
!     if (message number correct) then
!        read starting record number ;
!        calculate number of lines ;
!        if ( number of lines > 0 ) then
!           for each line do
!              read line ;
!              print line ;
!        else
!           print warning : message does not exist ;
!     else
!        print warning : message number out of range ;
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
!     --- read largest message number permitted

      ifrom = 1
      ito   = 1
      linenr= 1
      idum  = 0
      call readfl ( linenr, ifrom, ito, idum, iref, istart )
      maxerr = istart(1)

      if ( mesnum.gt.0 .and. mesnum.le.maxerr ) then

!     --- Read starting record of message

         linenr = 2 + (mesnum-1)/nrec4
         ito    = mod(mesnum,nrec4)+1
         if (ito.eq.1) ito = nrec4+1
         call readfl (linenr, ifrom, ito, idum, iref, istart)
         linenr = istart(ito-1)
         numlin = istart(ito) - linenr

         if ( numlin.gt.0 ) then

!        --- Read and print message

            i=linenr
            nxtlin=.true.

  100       if (nxtlin) then

!           --- read one message line

               read ( irefer, rec=i ) errlin
               ilast = 80
               do while ( errlin(ilast:ilast).eq.' ' )
                  ilast = ilast-1
               end do
               if ( icarcn.eq.0 ) then
                  write (irefwr,'(a)') errlin(2:ilast)
               else
                  write (irefwr,'(1x,a)') errlin(2:ilast)
               end if

               i = i + 1
               nxtlin = i.le.(linenr+numlin-1)
               goto 100

            end if

         else

!        --- Message not present in the file

            write (irefwr,1100) 'ERDUMP', mesnum

         end if

      else

!     --- Message number out of range

         write (irefwr,1100) 'ERDUMP', mesnum

      end if

 1100 format (' *** WARNING : '/ &
              ' ',a,' number ',i6,' has not yet been added', &
              ' to the error message file')

      end
