      subroutine erfillnodcont ( kprobv, iconct, kfill, nodes, nnodes, &
                                 kmesh )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for contact problems depending on kfill
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobv(*), iconct, kfill, nodes(*), nnodes, kmesh(*)

!     iconct         i    Sequence number of contact
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprobv         i    KPROB v contains information about the nodal points
!                         with contact.
!                         Length: 3*nsurfs+npoint
!                         KPROBV is organized as follows
!                         Part 1: contains information of the
!                                 surface with respect to contact
!                                 Pos. (i,1): number of contact points in
!                                             contact surface i
!                                 Pos. (i,2): number of points in contact
!                                             surface i that make nocontact
!                                 Pos. (i,3): starting address of contact points
!                                             for i^th contact surface in part 2
!                                             relative to part 2,.
!                                             Hence the smallest starting
!                                             address is 1
!                                 Length 3*nsurfs
!                         Part 2: contains the nodal points in the surfaces
!                                 that make contact in the sequence defined by
!                                 part 1 followed by the nodes that make no
!                                 contact
!     nnodes        i/o   Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nconctct, nocontct, istart, nsurfs
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     istart         First position used at a part of kprobv
!     nconctct       Number of points with contact
!     nocontct       Number of points that make no contact
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfillnodcont' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillnodcont'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      nsurfs = kmesh(12)
      nconctct = kprobv(iconct)

      if ( kfill==10 ) then

!     --- kfill = 10, Fill contact

         istart = kprobv(iconct+2*nsurfs)+3*nsurfs
         nodes(1:nconctct) = kprobv(istart:istart+nconctct-1)
         nnodes = nconctct

      else

!     --- kfill = 11, Fill no-contact

         nocontct = kprobv(iconct+nsurfs)
         istart = kprobv(iconct+2*nsurfs)+3*nsurfs+nconctct
         nodes(1:nocontct) = kprobv(istart:istart+nocontct-1)
         nnodes= nocontct

      end if  ! ( kfill==10 )

1000  call erclos ( 'erfillnodcont' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillnodcont'

      end if  ! ( debug )

      end
