      subroutine erfillnodobs ( ipointobst, iobst, nodes, nnodes, &
                                kfill )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for obstacle problems depending on kfill
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), iobst, kfill, nodes(*), nnodes

!     iobst          i    Obstacle sequence number
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numbndobst, numinnobst, numpntobst
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     istart         First position used at a part
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfillnodobs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillnodobs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      istart = ipointobst(iobst)
      if ( debug ) write(irefwr,1) 'iobst, istart', iobst, istart
      numpntobst = ipointobst(istart)

      select case ( kfill )

      case ( 16 )

!     --- kfill = 16, Fill all points in obstacle

         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         if ( debug ) &
            write(irefwr,1) 'numpntobst, numinnobst, numbndobst', &
                             numpntobst, numinnobst, numbndobst
         istart = istart+3
         nnodes = numpntobst+numinnobst+numbndobst
         if ( debug ) write(irefwr,1) 'istart, nnodes', istart, nnodes

      case ( 17 )

!     --- kfill = 17, Fill inner points in obstacle

         istart = istart+3
         nnodes = numpntobst

      case ( 18 )

!     --- kfill = 18, Fill boundary points in obstacle

         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         istart = istart+3+numpntobst
         nnodes = numinnobst+numbndobst

      case ( 19 )

!     --- kfill = 19, Fill boundary points on obstacle

         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         istart = istart+3+numpntobst+numinnobst
         nnodes = numbndobst

      end select  ! case ( kfill )
      nodes(1:nnodes) = ipointobst(istart+1:istart+nnodes)

1000  call erclos ( 'erfillnodobs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillnodobs'

      end if  ! ( debug )

      end
