      subroutine ernumnodcurvs ( icurnr, icurvs, kmesh, kmeshq, nnodes )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 23-10-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with the number of nodes along curve number icurnr
!     If the curve is a compound curve it is assumed that the curve is
!     contiguous. Double points are counted only once
! **********************************************************************
!
!                       KEYWORDS
!
!     compound_curve
!     nodes
!     curves
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, icurvs(*), kmesh(*), kmeshq(*), nnodes

!     icurnr         i    Curve sequence number
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     nnodes         o    Number of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubcv

!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find number of subcurves
!     Compute number of nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ernumnodcurvs' )

!     --- Find number of subcurves

      nsubcv = 0
      if ( kmesh(31).gt.0 ) nsubcv = kmeshq(icurnr+1) - kmeshq(icurnr)

      if ( icurnr.eq.1 ) then

!     --- icurnr = 1

         nnodes = icurvs(icurnr)

      else

!     --- icurnr > 1

         nnodes = icurvs(icurnr)-icurvs(icurnr-1)

      end if

      if ( nsubcv.gt.0 ) nnodes = nnodes-nsubcv+1

      call erclos ( 'ernumnodcurvs' )

      end
