      subroutine eropenfile ( namefile, iseq, iref )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 02-07-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Open file with name namefile and extension given by sequence number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     open_file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iref
      character (len=*) namefile

!     iref           i    File reference number
!     iseq           i    Sequence number
!     namefile       i    Base name of file
!                         The file gets extension .xxx, with xxx the sequence
!                         number iseq
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nchar, ios
      logical debug
      character (len=84) filename

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     filename       Name of file
!     ios            IOSTAT variable, returns from reading or writing
!     nchar          Number of characters in a word
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1317   error in opening of file
!    2770   Length of file name too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nchar := number of characters in name file
!     filename := namefile . sequence number
!     open file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'eropenfile' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eropenfile'

      end if
      if ( ierror/=0 ) go to 1000

!     --- nchar := number of characters in name file

      nchar =  index ( namefile, ' ' )-1
      if ( nchar<=0 ) nchar = len(namefile)

      if ( nchar>80 ) then

!     --- Number of characters in file name>80

         call errint ( nchar, 1 )
         call errint ( 80, 2 )
         call errchr ( namefile, 1 )
         call errsub ( 2770, 2, 0, 1 )
         go to 1000

      end if  ! ( nchar>80 )

!     --- filename := namefile . sequence number

      filename = namefile
      write(filename(nchar+1:nchar+4),200) iseq
200   format ( '.', i3.3 )
      nchar = nchar+4

!     --- open file

      open ( unit = iref, file = filename(1:nchar), iostat = ios )

      if ( ios/=0 ) then

!     --- error in opening file

         call errint ( ios, 1 )
         call errchr ( namefile(1:nchar), 1 )
         call errsub ( 1317, 1, 0, 1 )

      end if  ! ( ios/=0

1000  call erclos ( 'eropenfile' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eropenfile'

      end if

      end
