      subroutine erpres ( ibuffr, buffer, kmesh, kprob, isol, inpsep,
     +                    maxvec, numvec, istart, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 04-10-2010 Extra parameters
!        version  2.2  date 16-08-2004 keywords
!        version  2.1  date 17-11-2001 Reset default sequence number to 1
!
!   copyright (c) 1993-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine PRESBC for the essential boundary conditions
!    Fill array IINPRS with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), isol(5,*), inpsep(*), maxvec, numvec,
     +        ibuffr(*), istart
      double precision buffer(*), rinsep(*)

!     buffer        i/o   Real buffer array in which all large data is stored
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         1  ISEQNR     Sequence number corresponding to input
!                                       If iseqnr = -1, the next one will be
!                                       used
!                         2  IPROB      Problem sequence number
!                                       Not used
!                         3  IVECTOR    Vector sequence number (first vector)
!                                       If ivector = -1, the same value as iprob
!                                        will be used
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     istart        i/o   Starting address in array INPVEC
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     maxvec         i    Maximum number of vectors per time-step permitted
!     numvec        i/o   Largest sequence number of vector that has been filled
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, ivec, iseqnr, ivectr, iinprs(4)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iinprs         Input array defined by the user
!                    With this array the user may indicate which actions
!                    he wants to be performed by PRESBC
!                    IINPRS must be filled as follows:
!                    Pos. 1: Last entry number that has been filled by the
!                            user. If 0 or 1 only defaults are used
!                         2: IREAD Defines how the input must be read
!                                  Possible values:
!                                  0:  All SEPRAN input has been read by
!                                      subroutine SEPSTN until
!                                      END_OF_SEPRAN_INPUT
!                                      or end of file has been found
!                                      This input is used
!                                  1:  The input is read as described for PRESDF
!                                  2:  The last input read as described for
!                                      PRESDF is used
!                                      (equivalent with PRESTM ICTIME=1)
!                            Default value: 0
!                         3: ISEQNR, This parameter is only used if IREAD=0
!                            It indicates the sequence number of the input
!                            for essential boundary conditions to be used
!                            Default value: 1
!     iseq            Sequence number read
!     iseqnr          Sequence number with respect to the storage in array
!                     ISEPIN
!     ivec            Sequence number of vector of read
!     ivectr          Sequence number of first vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRESBCBF       Read essential boundary conditions
!     PRESCRIBEBC    Fill boundary conditions in vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivectr /0/
! ======================================================================
!
      call eropen ( 'erpres' )
      debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erpres'
         write(irefwr,1) 'istart', istart
         call prinin ( inpsep, 4, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqnr = 1
      iseq   = inpsep(1)
      if ( iseq>0 ) iseqnr = iseq

      if ( iseq/=-10 ) then

!     --- Standard case, use sequence number

         ivectr = min ( ivectr + 1, maxvec )
         ivec   = inpsep(3)
         if ( ivec/=-1 ) ivectr = ivec
         iinprs(1) = 4
         iinprs(2) = 0
         iinprs(3) = iseqnr
         iinprs(4) = ivectr
         call presbcbf ( ibuffr, buffer, kmesh, kprob, isol,
     +                   iinprs )
         istart = istart + 4

         numvec = max(numvec,iinprs(4))

      else

!     --- New case, all input in inpsep and rinsep

         call prescribebc ( ibuffr, buffer, kmesh, kprob, isol,
     +                      inpsep, rinsep, .true. )

         numvec = max(numvec,inpsep(3))
         istart = istart + 10 + inpsep(7)

      end if  ! ( iseq/=-10 )

1000  call erclos ( 'erpres' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End erpres'

      end if  ! ( debug )

      end
