      subroutine ersurfsnod2 ( nsurfaces, isurfaces, kmesht, kmesh, &
                               nodes, nnodes, kmeshc )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.1  date 03-01-2012 Use prfillnodes
!        version  1.0  date 28-12-2010
!
!   copyright (c) 2010-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the surfaces stored in array surfaces
!     in array nodes
!     It is assumed that kmesh part n has not been filled
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfaces, isurfaces(nsurfaces), kmesht(*), kmesh(*), &
              nodes(*), nnodes, kmeshc(*)

!     isurfaces      i    Array containing the surfaces from which the nodes
!                         must be found
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmesht         i    kmesh part t, contains element numbers of elements
!                         with highest dimensionality
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsurnr, istart, isurnr, inpelm, inelem, &
              error, npoint
      integer, allocatable :: iwork(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     istart         First position of a surface in kmeshn minus 1
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
!     jsurnr         Loop variable
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODSKMC1 Fill array nodes for elements 1 to inelem
!     EROPEN         Produces concatenated name of local subroutine
!     MSHEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ersurfsnod2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ersurfsnod2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npoint = kmesh(8)

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0

!     --- Loop over surfaces

      do jsurnr = 1, nsurfaces

         isurnr = isurfaces(jsurnr)
         call mshextkmesht ( isurnr, isurnr, kmesht, kmesh, &
                             istart, inpelm, inelem )

!        --- For all nodes inode in surface fill nodes

         call erfillnodskmc1 ( nodes, nnodes, kmeshc(istart), inelem, &
                               inpelm, iwork )

      end do  ! jsurnr = 1, nsurfaces

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'ersurfsnod2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ersurfsnod2'

      end if  ! ( debug )

      end
