******************************************************************************
*
*      File:  exam81.f
*
*      Contents:  Main program for the test example described
*                 in the SEPRAN user's manual 8.1
c                 In this program all steps are performed, i.e.
c                 mesh generation, computation and post-processing
*
*      Usage:     Compile and link this program with the SEPRAN libraries
*                 Run this program with input: exam81.dat
*
*      version 2.0    date   31-03-93 (Adapated to new subroutines)
*
******************************************************************************
*
      program exam81
      implicit none
      integer kmesh, kprob, intmat, matr,
     +        isol, irhsd, iuser(100), iinbld(1)
      integer istart, ifree, ioutp, itime, ichois, iinput, jmetod,
     +        iprob, massmt, islold, ipos, jdim, jpoint, i, number,
     +        jsmoot, istop, ncntln
      double precision user(100), contln(20)
      double precision rinput, xjdim, plotfm, yfact

c     --- Clear all arrays in order to prevent unwanted side effects

      kmesh = 0
      kprob = 0
      intmat= 0
      matr = 0
      isol = 0
      irhsd = 0
      iuser = 0
      user = 0d0
      contln = 0d0

c     --- Set first positions of variable length arrays

      contln(1) = 20
      iuser(1)  = 100
      user(1)   = 100

c     --- Start SEPRAN

      istart = 0
      ifree  = 1
      ioutp  = 0
      itime  = 0
      call start ( istart, ifree, ioutp, itime )

c     --- Mesh generation

      ichois  =  0
      call mesh ( ichois, iinput, rinput, kmesh )

c     --- Problem definition

      ichois = 0
      call probdf ( ichois, kprob, kmesh, iinput )

c     --- Define structure of matrix (symmetrical, profile storage)

      jmetod = 1
      call commat ( jmetod, kmesh, kprob, intmat )

c     --- Fill essential boundary conditions

      call presdf ( kmesh, kprob, isol )

c     --- Fill coefficients for the differential equation

      iprob = 1
      call filcof ( iuser, user, kprob, kmesh, iprob )

c     --- Build matrix and right-hand side (default situation)

      iinbld(1) = 0
      call build ( iinbld, matr, intmat, kmesh, kprob, irhsd,
     +             massmt, isol, islold, iuser, user )

c     --- Solve system of equations by a direct (profile) method
c         The matrix is positive definite

      ipos = 1
      call solve ( ipos, matr, isol, irhsd, intmat, kprob )

c     --- Print solution

      jdim = 1
      call prinov ( isol, kmesh, kprob, jdim, 'solution', xjdim,
     +              jpoint )

c    --- plot contour lines at level 0 , 0.1 , 0.2 , + . + , 1

      do 200 i = 1,11
         contln(i+5) = (i-1)*0.1
200   continue

      number = 1
      ncntln = 11
      plotfm = 10
      yfact  = 1
      jsmoot = 0
      call plotc1 ( number, kmesh, kprob, isol, contln, ncntln, plotfm,
     +              yfact, jsmoot )

c     --- Stop SEPRAN

      istop = 0
      call finish ( istop )

      end


c     --- function funccf for the definition of the coefficients


      function funccf ( ichois, x, y, z )
      implicit none
      integer ichois
      double precision funccf, x, y, z

      if ( ichois.eq.1 ) then

c     ---  du /dn  =  y

         funccf = y

      else

c     ---  u + du/dn  =  2x

         funccf = 2*x

      end if

      end
