      subroutine finish ( istop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 29-05-2014 Use timing
!        version  4.0  date 10-04-2014 Extension with mumps
!        version  3.1  date 15-11-2004 Removed the second delete of file 3
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     End of SEPRAN session
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cfile3f90'
      include 'SPcommon/cstandf90'
      include 'SPcommon/cmachnf90'
      include 'SPcommon/cmachtf90'
      include 'SPcommon/cpackagef90'
      include 'SPcommon/cmumpsf90'
      include 'SPcommon/cparallelf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istop

!     istop          i    Indication if file 2 must be closed and saved (#0)
!                         or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: check
      integer :: iref, isendarr(3)
      double precision timebefore(2)

!     check          Indicates if the file exist (true) or not
!     iref           File reference number
!     isendarr       Send array for mumps
!     timebefore     Time at entrance of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CLMACH         User closing subroutine
!     CLOSEMATLAB    Closes MATLAB engine
!     CLUNIX         Unix closing subroutine
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     INIBCASTMPI    Wrapper for MPI_BCAST
!     INIEXITMPI     Stop mpi
!     PLAFP6         Standard machine dependent plotting
!     WRITB1         Save administration to the backing storage file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   if ( file 3 is not scratch file ) then
!      Close file 3
!   Close SEPRAN
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( itime>0 ) call ersettime ( timebefore )

      if ( istop==1 ) call writb1

!     --- Close Files

      inquire ( iref1, opened=check )
      if ( check ) close ( iref1 )
      inquire ( iref2, opened=check )
      if ( check ) close ( iref2 )
      inquire ( iref3, opened=check )
      if ( check ) close ( iref3, status='delete' )
      inquire ( irefer, opened=check )
      if ( check ) close ( irefer )
      if ( irefre/=5 .and. irefre/=0 ) then
         inquire ( irefre, opened=check )
         if ( check ) close ( irefre )
      end if

      if ( itime.eq.1 ) call printtime ( 'finish', timebefore )

      if ( irefwr/=6 .and. irefwr/=0 ) then
        inquire ( irefwr, opened=check )
        if ( check ) close ( irefwr )
      end if

!     --- Files iref10, iref73 and iref74

      iref = abs(iref10)
      inquire ( iref, opened=check )
      if ( check ) close ( iref )
      iref = abs(iref73)
      inquire ( iref, opened=check )
      if ( check ) close ( iref )
      iref = abs(iref74)
      inquire ( iref, opened=check )
      if ( check ) close ( iref )

      if ( iplopn==1 ) then

!     --- The plot file has not been closed correctly

         call plafp6 ( 5, 0d0, 0d0 )

      end if

      if ( stoper(1:4)=='unix' ) then

!     --- UNIX operating system

         call clunix

      end if

!     --- Call machine dependent close

      call clmach

      if ( .not. firstmumps .and. inodenr==1 ) then

!     --- mumps must be closed
!         in order to stop the slave

         isendarr(1) = -1    ! send a stop to all processes
         isendarr(2:3) = 0
         call inibcastmpi ( isendarr, 3, SPINT )

      end if  ! ( .not. firstmumps )

!     --- Stop mpi

      if ( parallel .or. .not. firstmumps ) then

!     --- Parallel computing

         call iniexitmpi

      end if

!     --- Close MATLAB engine

      call closematlab
      end
