      subroutine getconvararray ( iseqarrglob, array, name, numcomps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the values of a constant or scalar array from cuscons into
!     a local array
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     array
!     constant
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqarrglob, numcomps
      double precision array(numcomps)
      character (len=*) name

!     array          o    array of length numcomps in which the values
!                         are stored
!     iseqarrglob    i    Sequence number of array as stored in inpsep
!     name           i    Name of array
!     numcomps       i    Number of components
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqarr
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseqarr        Local value of iseqarrglob
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3136   constant or scalar does not correspond to an array
!    3137   dimension of constant, vector or scalar not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getconvararray' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getconvararray'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqarr = mod(iseqarrglob,100000000)
      if ( iseqarr<5000 ) then

!     --- error: index does not belong to an array

         call errchr ( name, 1 )
         call errint ( iseqarr, 1 )
         call errsub ( 3136, 1, 0, 1 )
         go to 1000

      end if  ! ( iseqarr<5000 )

      iseqarr = iseqarr-5000

      if ( iseqarr>maxints+maxreals ) then

!     --- scalar array found

         iseqarr = iseqarr-maxints-maxreals
         if ( dimscals(iseqarr)/=numcomps ) then

!        --- dimension incorrect

            call errchr ( name, 1 )
            call errint ( dimscals(iseqarr), 1 )
            call errint ( numcomps, 2 )
            call errsub ( 3137, 2, 0, 1 )
            go to 1000

         end if  ! ( dimscals(iseqarr)/=numcomps )

         array(1:numcomps) = scalars(iseqarr:iseqarr+numcomps-1)

      else

!     --- constants array found

         if ( dimcons(iseqarr)/=numcomps ) then

!        --- dimension incorrect

            call errchr ( name, 1 )
            call errint ( dimcons(iseqarr), 1 )
            call errint ( numcomps, 2 )
            call errsub ( 3137, 2, 0, 1 )
            go to 1000

         end if  ! ( dimcons(iseqarr)/=numcomps )

         if ( iseqarr>maxints ) then

!        --- real found

            iseqarr = iseqarr-maxints
            array(1:numcomps) = rlcons(iseqarr:iseqarr+numcomps-1)

         else

!        --- integer found

            array(1:numcomps) = incons(iseqarr:iseqarr+numcomps-1)

         end if  ! ( iseqarr>maxints )

      end if  ! ( iseqarr>maxints+maxreals )

1000  call erclos ( 'getconvararray' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getconvararray'

      end if  ! ( debug )

      end
