      integer function getintparm ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with real defined by inpsep/rinsep
!     In case of a scalar, return with the scalar
! **********************************************************************
!
!                       KEYWORDS
!
!     real
!     variable
!     value
!     evaluate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep

!     getintparm     o    Returned value
!     inpsep         i    Entry of integer input array that defines the value
!                         If>-100000000, the value is stored in inpsep
!                         If<-100000000, the value is stored in
!                         scalars(-inpsep) or constants()
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpseploc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inpseploc      Local value of inpsep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getintparm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getintparm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpseploc = abs(inpsep)
      if ( inpseploc>100000000 ) then

!     --- scalar or constant found

         inpseploc = inpseploc-100000000
         if ( inpseploc<=maxints ) then
            getintparm = incons(inpseploc)
         else
            inpseploc = inpseploc-maxints
            if ( inpseploc<=maxreals ) then
               getintparm = nint(rlcons(inpseploc))
            else
               getintparm = nint(scalars(inpseploc-maxreals))
            end if  ! ( inpseploc<=maxreals ) then
         end if  ! ( inpseploc<=maxints )

      else

!     --- Integer value in inpsep

         getintparm = inpsep

      end if  !  ( inpsep<-100000000 )

1000  call erclos ( 'getintparm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getintparm'

      end if  ! ( debug )

      end
