/*
 *  Copyright (c) 1991  Philips Export B.V.
 * 
 *  Philips Research Laboratories
 *  Eindhoven, The Netherlands
 *
 *  Author    : G.F.M. Haenen
 *              F.W.G.M. Wouters
 *
 *  Date      : July, 1991
 *
 *  Modified  : Kees Lemmens, June 9, 1999
 *            - fixed compiler warnings and some small bugs
 *
 *  Function  : redirects plotting commands to the appropriate device
 *
 *  Comment   : watch the include field!
 * 
 *  Modified  : Kees Lemmens, June 1999
 *              - Ansified
 *  Modified  : Sander Gielen, March 1998
 *              - added fig plot functions
 *
*/

#include <stdio.h>
#include "plotfun.h"

/* Global variable : (for performance reasons ) */
static int output;

void color(int iclr)	/* set color */
{
   static int lastcolor = -1;
   
   if (iclr == lastcolor)
     return;
   
   lastcolor = iclr;
   
   if (iclr >= 0)
    {
       switch(output)
       {
        case PS_OUTPUT:
	  ps_color(iclr+1);
	  break;
        case HPGL_OUTPUT:
	  hpgl_color(iclr+1);
	  break;
        case FIG_OUTPUT:
	  fig_color(iclr+1);
	  break;
	case JB_OUTPUT:
	  break;
       }
    }
}

void move(double x1, double y1)
{
   switch(output)
   {
    case PS_OUTPUT:
    case HPGL_OUTPUT:
    case FIG_OUTPUT:
      break;
    case JB_OUTPUT:
      jb_move();
      break;
   }
}

void line(double x1, double y1, double x2, double y2)
{
   switch(output)
   {
    case PS_OUTPUT:
      ps_line(x1, y1, x2, y2);
      break;
    case HPGL_OUTPUT:
      hpgl_line(x1, y1, x2, y2);
      break;
    case FIG_OUTPUT:
      fig_line(x1, y1, x2, y2);
      break;
    case JB_OUTPUT:
      jb_line();
      break;
   }
}

void polyfill(double x[], double y[], int npoints)
{
   switch(output)
   {
    case PS_OUTPUT:
      ps_polyfill(x, y, npoints);
      break;
    case HPGL_OUTPUT:
      hpgl_polyfill(x, y, npoints);
      break;
    case FIG_OUTPUT:
      fig_polyfill(x, y, npoints);
      break;
    case JB_OUTPUT:
      jb_polyfill(x, y, npoints);
      break;
   }
}

void polyline(double x[], double y[], int npoints)
{
   switch(output)
   {
    case PS_OUTPUT:
      ps_polyline(x, y, npoints);
      break;
    case HPGL_OUTPUT:
      hpgl_polyline(x, y, npoints);
      break;
    case FIG_OUTPUT:
      fig_polyline(x, y, npoints);
      break;
    case JB_OUTPUT:
      jb_polyline(x, y, npoints);
      break;
   }
}

void text(double xin, double yin, double height, double  ang,char *strin)
{
   switch(output)
   {
    case PS_OUTPUT:
      ps_text(xin, yin, height, ang, strin);
      break;
    case HPGL_OUTPUT:
      hpgl_text(xin, yin, height, ang, strin);
      break;
    case FIG_OUTPUT:
      fig_text(xin, yin, height, ang, strin);
      break;
    case JB_OUTPUT:
      jb_text(strin);
      break;
   }
}

int init(window_t *dim,seprec_t *rec,filnam_t *fileinfo,int curoutput)
{
   int retval = -1;
   output = curoutput;
   
   switch(output)
   {
    case PS_OUTPUT:
      retval = ps_init(dim,fileinfo);
      break;
    case HPGL_OUTPUT:
      retval = hpgl_init(dim,fileinfo);
      break;
    case FIG_OUTPUT:
      retval = fig_init(dim,fileinfo);
      break;
    case JB_OUTPUT:
      retval = jb_init(rec,fileinfo);
      break;
    default :
      fprintf(stderr,"Unsupported output format : %d\n",curoutput);
      exit(1);
   }
   return(retval);
}

void pl_finish()
{
   switch(output)
   {
    case PS_OUTPUT:
      ps_finish();
        break;
    case HPGL_OUTPUT:
      hpgl_finish();
      break;
    case FIG_OUTPUT:
      fig_finish();
        break;
    case JB_OUTPUT:
      jb_finish();
        break;
   }
}

void symbol(double x, double y, int symb)
{
   switch(output)
   {
    case PS_OUTPUT:
      ps_symbol(x, y, symb);
      break;
    case HPGL_OUTPUT:
      hpgl_symbol(x, y, symb);
      break;
    case FIG_OUTPUT:
      fig_symbol(x, y, symb);
      break;
    case JB_OUTPUT:
      jb_symbol();
      break;
   }
}
