/*
 *		Copyright IBM Corporation 1989
 *
 *                      All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of IBM not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * University of Illinois at Urbana-Champaign
 * Department of Computer Science
 * 1304 W. Springfield Ave.
 * Urbana, IL	61801
 *
 * (C) Copyright 1987, 1988 by The University of Illinois Board of Trustees.
 * All rights reserved.
 *
 * Tool: X 11 Graphical Kernel System
 * Author: Gregory Scott Rogers
 * Author: Sung Hsien Ching Kelvin
 * Author: Yu Pan
 * 
 * Define file for : Standard GKS C-language binding 
 *
 * This header-file includes header-file <stdio.h>.
 * 
 * $Id: xgks.h,v 2.6 1993/03/27 04:26:53 steve Exp $
 * $__Header$
 */


#ifndef   GKS_H
#define   GKS_H

#include <stdio.h>			/* for FILE */

/* Gint - integer */
typedef int Gint;			/* this datatype must be defined by
					 * the implementation as a type
					 * suitable for containing integer
					 * values at least 16 bits long */

/* Glong - longer integer */
typedef int     Glong;


/* Gintlist - integer list */
typedef struct Gintlist {
    Gint            number;		/* number of integer in list */
    Gint           *integers;		/* array of integers */
} Gintlist;


/* Gchar - chatacter */
typedef char Gchar;			/* this datatype must be defined by
					 * the implementation as a type
					 * suitable for containing characters
					 * of at least 8 bits */


/* Gfloat - floating point number */
typedef float Gfloat;			/* this datatype must be defined by
					 * the implementation as a type
					 * suitable for containing float
					 * values at least 32 bits long */


/* Gredraw - workstation redraw state */
typedef enum Gredraw {
    GRD_GKS,				/* redraw due to GKS */
    GRD_X				/* redraw due to X */
}               Gredraw;


/* Gacf - attribute control flag */
typedef enum Gacf {
    GCURRENT,
    GSPECIFIED
} Gacf;


/* Gasf - aspect source flag */
typedef enum Gasf {
    GBUNDLED,
    GINDIVIDUAL
} Gasf;


/* Gasf - aspect source flags */
typedef struct Gasfs {
    Gasf            ln_type;		/* line type */
    Gasf            ln_width;		/* line width */
    Gasf            ln_colour;		/* line colour */
    Gasf            mk_type;		/* marker type */
    Gasf            mk_size;		/* marker size */
    Gasf            mk_colour;		/* marker colour */
    Gasf            tx_fp;		/* text font and precision */
    Gasf            tx_exp;		/* text expansion */
    Gasf            tx_space;		/* text character spacing */
    Gasf            tx_colour;		/* text colour */
    Gasf            fl_inter;		/* fill area interior style */
    Gasf            fl_style;		/* filll area style index */
    Gasf            fl_colour;		/* fill area colour */
} Gasfs;


/* Gattr - attributes used */
typedef enum Gattrs {
    GPOLYLINE,
    GPOLYMARKER,
    GTEXT,
    GFILLAREA
} Gattrs;


/* Gcstat - choice status */
typedef enum Gcstat {
    GC_OK,
    GC_NOCHOICE,
    GC_NONE
} Gcstat;


/* Gchoice - choice data */
typedef struct Gchoice {
    Gcstat          status;		/* choice status */
    Gint            choice;		/* choice number */
} Gchoice;


/* Gprflag - prompt flag */
typedef enum Gprflag {
    GPROFF,
    GPRON
}               Gprflag;


/* Gchoicerec - choice data record */
typedef union Gchoicerec {
    struct {
	Gchar          *data;		/* device/implementation dependent
					 * data */
    }               pet1;
    struct {
	Gint            number;		/* number of alternatives */
	Gprflag        *enable;		/* array of prompts */
	Gchar          *data;		/* device/implementation dependent
					 * data */
    }               pet2;
    struct {
	Gint            number;		/* number of choice strings */
	Gchar         **strings;	/* array of strings */
	Gchar          *data;		/* device/implementation dependent
					 * data */
    }               pet3;
    struct {
	Gint            number;		/* number of choice strings */
	Gchar         **strings;	/* array of strings */
	Gchar          *data;		/* device/implementation dependent
					 * data */
    }               pet4;
    struct {
	Gint            seg;		/* segment name */
	Gint            number;		/* number of alternatives */
	Gint           *pickids;	/* array of pick identifiers */
	Gchar          *data;		/* device/implementation dependent
					 * data */
    }               pet5;
    /*
     * .... more for implementation defined PETs
     */
} Gchoicerec;


/* Gimode - input mode */
typedef enum Gimode {
    GREQUEST,
    GSAMPLE,
    GEVENT
} Gimode;


/* Gesw - echo switch */
typedef enum Gesw {
    GECHO,
    GNOECHO
} Gesw;


/* Glimit - coordinate limits */
typedef struct Glimit {
    Gfloat          xmin;		/* x minimum limit */
    Gfloat          xmax;		/* x maximum limit */
    Gfloat          ymin;		/* y minimum limit */
    Gfloat          ymax;		/* y maximum limit */
} Glimit;


/* Gchoicest - choice state */
typedef struct Gchoicest {
    Gimode          mode;		/* mode */
    Gesw            esw;		/* echo switch */
    Gchoice         choice;		/* choice data */
    Gint            pet;		/* prompt and echo type */
    Glimit          e_area;		/* echo area */
    Gchoicerec      record;		/* choice data record */
} Gchoicest;


/* Gclip - clipping indicator */
typedef enum Gclip {
    GCLIP,
    GNOCLIP
} Gclip;


/* Gcliprect - clipping rectangle */
typedef struct Gcliprec {
    Gclip           ind;		/* clipping indicator */
    Glimit          rec;		/* clipping rectangle */
} Gcliprec;


/* Gclrflag - clear control flag */
typedef enum Gclrflag {
    GCONDITIONALLY,
    GALWAYS
} Gclrflag;


/* Gcoavail - colour availablity */
typedef enum Gcoavail {
    GCOLOUR,
    GMONOCHROME
} Gcoavail;


/* Gcobunbl - colour bundle */
typedef struct Gcobundl {
    Gfloat          red;		/* red intensity */
    Gfloat          green;		/* green intensity */
    Gfloat          blue;		/* blue intensity */
} Gcobundl;


/* Gcofac - colour facilities */
typedef struct Gcofac {
    Gint            colours;		/* number of colours */
    Gcoavail        coavail;		/* colour availability */
    Gint            predefined;		/* number of predefined bundles */
} Gcofac;


/* Gcovaild - colour values valid */
typedef enum Gcovalid {
    GABSENT,
    GPRESENT
} Gcovalid;


/* Gcsw - coordinate switch */
typedef enum Gcsw {
    GWC,				/* world coordinates */
    GNDC				/* normalized device coordinates */
} Gcsw;


/* Gdefchoice - default choice data */
typedef struct Gdefchoice {
    Gint            choices;		/* maximum number of choices */
    Gintlist        pets;		/* list of prompt and echo types */
    Glimit          e_area;		/* default echo area */
    Gchoicerec      record;		/* default choice data record */
} Gdefchoice;


/* Gdefmode - deferral mode */
typedef enum Gdefmode {
    GASAP,				/* as soon as possible */
    GBNIG,				/* before next interaction globally */
    GBNIL,				/* before next interaction locally */
    GASTI				/* at some time in */
} Gdefmode;


/* Girgmode - implicit regeneration mode */
typedef enum Girgmode {
    GSUPPRESSED,
