      subroutine ini003 ( ibuffr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.5  date 21-11-2012 Long integers
!        version  7.4  date 28-09-2012 Jump in case ierror#0
!        version  7.3  date 06-02-2002 Extension with eropen/erclos
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write all arrays with priority<10 to backing storage file 3
!     move other arrays to start of ibuffr from position ibufst
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*)

!    ibuffr  i/o  Integer array in which all large data is stored
!    iwork        work array of length 1500, in which the entries in infor
!                 are sorted for increasing starting address
!                 iwork(1) contains the index of the array with the smallest
!                          starting address, &
!                 iwork(2) with the next starting address etc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipfirs, ipnext
      integer iextra, numarr, i, inext, loclen, iprio, iext2, j, &
              iwork(iinfor)

!   i         General loop variable
!   iext2     iextra/2
!   iextra    Extra space needed at both sides of the array for control
!             integers
!   inext     Sequence number of next array in IBUFFR with respect to INFOR
!   ipfirs    Starting address of present array in IBUFFR
!   ipnext    Starting address of array inext in IBUFFR
!   iprio     Priority number
!   j         General loop variable
!   loclen    Length of present array to be considered
!   numarr    Number of arrays that are stored in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI100         write one array from IBUFFR to backing storage
!     INI102         Sort the sequence of the starting addresses in IBUFFR
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   write all arrays with priority<10 to file 3
!   Sort starting addresses of IBUFFR
!   Compress IBUFFR
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- write arrays with priority<10 to file 3

      call eropen ( 'ini003' )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
         iext2 = 0
      else
         iextra = intlen
         iext2 = 1
      end if
      do i = 1,iinfor
         iprio = infor1(2,i)
         if ( iprio<10 .and. infor(i)>0 ) call ini100(ibuffr,i)
      end do

!     --- make ibuffr compact
!         first sort starting positions

      call ini102 ( iwork, numarr )
      ipfirs = numarrs+1 + iextra
      do i = 1, numarr
         inext  = iwork(i)
         ipnext = infor(inext) - iextra
         loclen = infor1(1,inext) + 2*iextra
         if ( ipnext>ipfirs-iext2 ) then

!        --- move array inext to starting address ipfirs

            do j = 0, loclen-1
               ibuffr(ipfirs-iextra+j) = ibuffr(ipnext+j)
            end do
            infor(inext) = ipfirs
            if ( idebug>0 ) ibuffr(ipfirs-1) = inext

         end if
         ipfirs = ipfirs + loclen
         if ( intlen==2 .and. mod(ipfirs,2_8)==0 ) ipfirs=ipfirs+1

      end do

      ibfree = ipfirs
      if ( idebug>=2 ) then
         ibuffr(ibfree:nbuffr) = iinfin
      end if

1000  call erclos ( 'ini003' )

      end
