      subroutine ini009 ( isepi2, nsubr, maxi3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.25 date 15-02-2013 Extra debug
!        version  1.24 date 20-01-2010 Extension of number of solves
!        version  1.23 date 04-09-2006 Extension with bearing
!
!   copyright (c) 1993-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ISEPIN block 2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsubr, maxi3, isepi2(4,nsubr)

!     isepi2  o    Block 2 of array ISEPIN
!                  Contents:
!                   Pos. (1,i)  MAXSUB    maximum number of input arrays that
!                                         may be stored for subroutine i
!                   Pos. (2,i)  ICALL,    sequence number of next call of
!                                         subroutine i
!                   Pos. (3,i)  MAXCALL,  maximum number of input arrays that
!                                         have been read for subroutine i
!                   Pos. (4,i)  ISTART    Contains lowest number j of infor
!                                         entries for subroutine i
!                   The subroutines have a fixed position:
!                      1:  START  (not yet implemented)
!                      2:  PROBDF
!                      3:  COMMAT
!                      4:  CREATE
!                      5:  PRESDF
!                      6:  FILCOF
!                      7:  CHNCOF
!                      8:  SOLVE
!                      9:  OUTPUT
!                     10:  SEPCOMP
!                     11:  DERIVS
!                     12:  NLNPRB
!                     13:  INTEGR
!                     14:  INTBND
!                     15:  TIMPRB
!                     16:  ADAPBOUN
!                     17:  ADAPMESH
!                     18:  STATFREE
!                     19:  INSTFREE
!                     20:  CONTACT
!                     21:  LOOP
!                     22:  CHANGE_COORDINATES
!                     23:  EIGENVALUES
!                     24:  CAPCITY
!                     25:  INVERSE_PROBLEM
!                     26:  ENTHALPY_INTEGRATION
!                     27:  PROBLEM
!                     28:  REFINE
!                     29:  PRESSURE_CORRECTION
!                     30:  NAVIER-STOKES
!                     31:  LEVEL_SET
!                     32:  MAKE_OBSTACLE_MESH
!     maxi3   o    Maximum length of first parameter in two-dimensional array
!                  isepi3
!     nsubr   i    Number of different subroutines that may store their
!                  information in ISEPIN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nsubref
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     nsubref        Number of subroutines that are installed in this
!                    subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini009' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini009'
         write(irefwr,1) 'nsubr', nsubr
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      nsubref = 33
      if ( nsubr/=nsubref ) then

!     --- Error nsubr has incorrect value

         call errchr ( 'nsubr', 1 )
         call errint ( nsubr, 1 )
         call errint ( nsubref,  2 )
         call errint ( nsubref,  3 )
         call errsub ( 2147, 3, 0, 1 )
         go to 1000

      end if  ! ( nsubr/=28 )

      if ( debug ) write(irefwr,1) 'nsubref', nsubref

      isepi2(1,1) = 1
      isepi2(1,2) = 1
      isepi2(1,3) = 10
      isepi2(1,4) = 100
      isepi2(1,5) = 25
      isepi2(1,6) = 25
      isepi2(1,7) = 50
      isepi2(1,8) = 25   ! number of inputs for solve
      isepi2(1,9) = 10
      isepi2(1,10) = 1
      isepi2(1,11) = 100
      isepi2(1,12) = 10
      isepi2(1,13) = 25
      isepi2(1,14) = 100  ! number of boundary integrals
      isepi2(1,15) = 10
      isepi2(1,16) = 25
      isepi2(1,17) = 10
      isepi2(1,18) = 10
      isepi2(1,19) = 10
      isepi2(1,20) = 10
      isepi2(1,21) = 10
      isepi2(1,22) = 10
      isepi2(1,23) = 10
      isepi2(1,24) = 10
      isepi2(1,25) = 10
      isepi2(1,26) = 10
      isepi2(1,27) = 10  ! number of problems
      isepi2(1,28) = 10  ! number of inputs for refinement
      isepi2(1,29) = 10  ! number of inputs for pressure correction
      isepi2(1,30) = 10  ! number of inputs for navier-stokes
      isepi2(1,31) = 10  ! number of inputs for level_set
      isepi2(1,32) = 10  ! number of inputs for make_obstacle_mesh
      isepi2(1,33) = 10  ! number of inputs for bearing

      if ( debug ) call prinin1 ( isepi2, 33, 4, 'isepi2/1' )
      maxi3 = 0
      do i = 1, nsubr
         isepi2(2,i) = 1
         isepi2(3,i) = 0
         isepi2(4,i) = maxi3 + 1
         maxi3 = maxi3 + isepi2(1,i)
      end do
      maxi3 = 3*maxi3

1000  call erclos ( 'ini009' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( isepi2, nsubr, 4, 'isepi2/end' )
         write(irefwr,*) 'End ini009'

      end if

      end
