      subroutine ini060check ( ibuffr, mmindex, conpos1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 24-02-2012 Debug statements
!        version  1.1  date 04-12-2003 Call ini060text
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate array by subroutine ini060, and check first position
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     check
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmindex, conpos1

!     conpos1        i    Contents of position 1
!     ibuffr         i    Integer buffer array in which all large data is stored
!     mmindex        i    memory management sequence number of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipindex
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipindex        Starting address of array with memory management sequence
!                    number mmindex in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2425   Array does not correspond to mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini060check' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini060check'
         write(irefwr,1) 'mmindex', mmindex
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call ini060text ( ibuffr, mmindex, namarr(abs(mmindex)) )
      ipindex = inigettext ( abs(mmindex), namarr(abs(mmindex)) )

      if ( ibuffr(ipindex)/=conpos1 ) then

!     --- error: array does not correspond to present mesh

         call errchr ( namarr(abs(mmindex)), 1 )
         call errint ( ibuffr(ipindex), 1 )
         call errint ( conpos1, 2 )
         call errsub ( 2425, 2, 0, 1 )

      end if

1000  call erclos ( 'ini060check' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini060check'

      end if  ! ( debug )

      end
