      subroutine ini082 ( ibuffr, index, lengdb, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 17-05-2010 New parameter list
!        version  2.0  date 22-09-2003 Complete revision
!        version  1.6  date 12-02-2003 Adaptation error message
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend existing array such that the new length becomes LENGTH
!     The first part of the array has already been constructed in IBUFFR
!     from the temporary position IPCOPY. It must be copied to the new
!     positions.
!     Double precision arrays
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, lengdb, ibuffr(*)
      character (len=*) namear

!     ibuffr i/o Integer buffer array in which all large data is stored
!     index  i/o The value of index is the reference number of the
!                array of which the starting position must be computed with
!                respect to array INFOR in common block IINFOR
!                This means that the positions (i,INDEX) (i=1,2,3) in INFOR
!                contain all the information about the storage of the array
!     lengdb  i  length of the array in reals
!     namear  i  Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer length
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     length         Parameter lengdb translated in words
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI084         compute starting position of a new or already existing
!                    array
! **********************************************************************
!
!                       I/O
!
!   none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     Make IPCOPY semi permanent by putting information in array INFOR
!          in common block CARRAY, without raising IBFREE
!     Reserve space for new array ( subroutine INI091, starting address IPSTAR )
!     Recompute IPCOPY (It might have been reorganised)
!     If ( IPCOPY # IPSTAR ) then
!         Copy contents of first array to new positions
!     If ( INDEX>index of semi-permanent array ) then
!         INDEX = INDEX - 1
!     Decrease IINFOR
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Translate from BUFFER to IBUFFR

      call eropen ( 'ini082' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini082'
         write(irefwr,1) 'lengdb', lengdb
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length = intlen * lengdb
      call ini084 ( ibuffr, index, length, namear )

1000  call erclos ( 'ini082' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini082'

      end if  ! ( debug )

      end
