      subroutine iniactnmeshe ( ibuffr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 28-12-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates nmesh part e, for each element the nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipnmeshe, ipkmsc
      integer nelem, lnnmeshe, kelmaa, mmnmeshe

!     ipkmsc         Starting address of array KMESH part c in array
!                    IBUFFR
!     ipnmeshe       Starting address of array NMESHE in IBUFFR
!                    This array contains the node sequence numbers of the
!                    elements
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     lnnmeshe       length of array nmeshe
!     mmnmeshe       memory management sequence number of array NMESHE
!     nelem          Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen
      integer (kind=8) iniget

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060         Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIGET         Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     TOPEL01        Creates nmesh part e, for each element the nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate kmesh part c (topology)
!     Create space for array nmeshe and make this array permanent
!     Compute starting addresses
!     Actually fill nmeshe
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactnmeshe' )
      kelmaa = kmesh(48)-1   ! starting address of kmesh part aa
      mmnmeshe = 0           ! memory management number of nmeshe
      nelem = kmesh(9)       ! number of elements

!     --- Activate kmesh part c (topology)

      call ini060 ( ibuffr, -kmesh(17) )

!     --- Create space for array nmeshe and make this array permanent

      lnnmeshe = inilen(kmesh(17))+2*nelem
      call ini081 ( ibuffr, mmnmeshe, lnnmeshe, 'nmeshe' )
      kmesh(kelmaa+35) = mmnmeshe

!     --- Compute starting addresses

      ipnmeshe = iniget(mmnmeshe)   ! starting address nmeshe
      ipkmsc = iniget(kmesh(17))    ! starting address kmeshc

!     --- Actually fill nmeshe

      call topel01 ( ibuffr(ipnmeshe), ibuffr(ipkmsc), kmesh )
      call erclos ( 'iniactnmeshe' )
      end
