      subroutine inichnam
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 06-12-2013 Extension with strings and int vars
!        version  2.1  date 09-09-2011 Correction test on ilow
!        version  2.0  date 14-03-2009 Complete revision
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check the uniqueness of the names used in chincons and chrlcons
!     Do the same with respect to the names used in chscalars
! **********************************************************************
!
!                       KEYWORDS
!
!     check
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(maxints+maxreals+maxcls+maxvec+maxstrings+ &
                    maxintscals), &
              iseq(numints+numreals+numscals+numvec+numstrings+ &
                    numintscals), &
              iworkcp(numints+numreals+numscals+numvec+numstrings+ &
                    numintscals), &
              ihelp(numints+numreals+numscals+numvec+numstrings+ &
                    numintscals), &
              iprio(numints+numreals+numscals+numvec+numstrings+ &
                    numintscals)
      integer iseqch, i, numblank, i1, i2, ishift, jshift, &
              ilow, itop, ih, itopref, ih1, nprio, iunder, iupper, &
              startscals, startvecs, itype1, itype2, startintscals, &
              startstrings
      character ( len=1 ) textchar
      character ( len=32 ) text1, text2
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ihelp          integer work array
!     ilow           Lower index
!     iprio          integer work array of length maxints+maxreals+maxcls
!                    iprio contains the numbers 1 to maxints+maxreals or
!                    maxcls
!                    If the first iseqch letters of two words are
!                    identical they get the same number.
!                    Hence in first instance iprio is filled with 1
!     iseq           integer work array of length maxints+maxreals+maxcls
!                    iseq contains the sorted sequence of the names
!                    iseq(1) refers to the first name, (2) to the second one
!                    and so on.
!                    Integers are numbered 1 to 1000, reals 1001 to 2000
!                    and scalars 2001 to 3000
!     iseqch         Sequence number of character in word
!     ishift         Shift of sequence number
!     itop           Upper index
!     itopref        Number of items in iseq
!     itype1         Defines type of first word
!                    1: integer
!                    2: real
!                    3: variable
!                    4: vector name
!     itype2         Defines name of second word
!     iunder         Lower index
!     iupper         Upper index
!     iwork          integer work array of length maxints+maxreals+maxcls
!                    iwork is used to store the integer representation of
!                    one character per word
!     iworkcp        integer work array of length maxints+maxreals+maxcls
!                    This array is used for copying
!     jshift         Shift of sequence number in iwork
!     nprio          Value in iprio
!     numblank       Number of blanks
!     startintscals  Start of int scalars in arrays ihelp and iseq (-1)
!     startscals     Start of scalars in arrays ihelp and iseq (-1)
!     startstrings   Start of strings in arrays ihelp and iseq (-1)
!     startvecs      Start of vectors in arrays ihelp and iseq (-1)
!     text1          First text to be compared
!     text2          Next text to be compared
!     textchar       Is used to store a character
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     789   Double name found
! **********************************************************************
!
!                       PSEUDO CODE
!
!    First sort integers and reals:
!       iseqch := 1
!       iwork(i) := 10000 (i=1,numints+numreals)
!       iseq(i) := i (i=1,numints+numreals)
!       iprio(i) := 1 (i=1,numints+numreals)
!       Translate first letter of all names (if not blank) into integer
!       numblank := number of blanks found
!       Put blanks at the back of iseq and set corresponding iprio to 10000
!       ilow := 1
!       itop := numints+numreals-numblank
!       Sort the integers ilow to itop and make new sequence in iseq
!       While ( itop>ilow ) do
!          Fill array iprio(ilow to itop) corresponding to character iseqch
!          Adapt ilow and itop
!          iseqch := iseqch + 1
!          For all groups with equal priority do
!             Translate iseqch_th letter of names iseq(ilow to itop)
!             Put blanks at the beginning of a set whith equal priority
!             Sort rest of set
!          Adapt ilow and itop
!    Check uniqueness of integers and reals
!    Next perform the same actions for the variables
!    Filling of iprio:
!      ih := iprio(ilow)
!      ih1 := iwork(iseq(ilow))
!      nprio := ih
!      For i := ilow+1, itop
!         if ( iprio(i)=ih ) then
!            if ( iwork(iseq(i))#ih1 ) then
!               ih1 := iwork(iseq(i))
!               nprio := i
!            iprio(i) := nprio
!         else ( iprio#ih )
!            ih := iprio(i)
!            nprio := ih
!            ih1 := iwork(iseq(i))
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inichnam' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inichnam'
         write(irefwr,1) 'numints, numreals, numscals, numvec', &
                          numints, numreals, numscals, numvec
         write(irefwr,1) 'maxints, maxreals, maxcls, maxvec', &
                          maxints, maxreals, maxcls, maxvec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize arrays

      iwork = 100000
      iprio = 1
      do i = 1, numints+numreals+numscals+numvec+numstrings+numintscals
         ihelp(i) = i
      end do  ! i = 1, numints+numreals+numscals+numvec+numstrings+numintscals
      iseqch = 1
      startscals = maxreals+maxints     ! start of scalars (-1)
      startvecs  = startscals+maxcls    ! start of vectors (-1)
      startstrings  = startvecs+maxvec    ! start of strings (-1)
      startintscals  = startstrings+maxstrings    ! start of intscalars (-1)

!     --- Translate first letter of all names (if not blank) into integer
!         First integers

      numblank = 0
      do i = 1, numints
         if ( chincons(i)/=' ' ) then
            iwork(i) = ichar(chincons(i)(1:1))
         else
            numblank = numblank + 1
            ihelp(i) = 100000+numblank
         end if
      end do  ! i = 1, numints
      if ( debug ) call prinin ( iwork, numints, 'iwork_ints' )
      ishift = numints
      jshift = maxints

!     --- Next reals

      do i = 1, numreals
         if ( chrlcons(i)/=' ' ) then
            iwork(i+jshift) = ichar(chrlcons(i)(1:1))
         else
            numblank = numblank + 1
            ihelp(i+ishift) = 100000+numblank
         end if
      end do  ! i = 1, numreals
      if ( debug ) &
         call prinin ( iwork(1+jshift), numreals, 'iwork_reals' )
      ishift = ishift+numreals
      jshift = jshift+maxreals

!     --- Next variables

      do i = 1, numscals
         if ( chscalars(i)/=' ' ) then
            iwork(i+jshift) = ichar(chscalars(i)(1:1))
         else
            numblank = numblank + 1
            ihelp(i+ishift) = 100000+numblank
         end if
      end do  ! i = 1, numscals
      if ( debug ) &
         call prinin ( iwork(1+jshift), numscals, 'iwork_scalars' )
      ishift = ishift+numscals
      jshift = jshift+maxcls

!     --- Next vector names

      do i = 1, numvec
         if ( names(i)/=' ' ) then
            textchar = names(i)(1:1)
            call askupc ( textchar )
            iwork(i+jshift) = ichar(textchar)
         else
            numblank = numblank + 1
            ihelp(i+ishift) = 100000+numblank
         end if
      end do  ! i = 1, numvec
      ishift = ishift+numvec
      if ( debug ) then
         call prinin ( iwork(1+jshift), numvec, 'iwork_vectors' )
         call prinin ( ihelp, ishift, 'ihelp/1' )
      end if  ! ( debug )
      jshift = jshift+maxvec

!     --- Next strings

      do i = 1, numstrings
         if ( chstrings(i)/=' ' ) then
            iwork(i+jshift) = ichar(chstrings(i)(1:1))
         else
            numblank = numblank + 1
            ihelp(i+ishift) = 100000+numblank
         end if
      end do  ! i = 1, numstrings
      ishift = ishift+numstrings
      jshift = jshift+maxstrings

!     --- Finally integer scalars

      do i = 1, numintscals
         if ( chscalars(i+maxcls)/=' ' ) then
            iwork(i+jshift) = ichar(chscalars(i+maxcls)(1:1))
         else
            numblank = numblank + 1
            ihelp(i+ishift) = 100000+numblank
         end if
      end do  ! i = 1, numintscals
      ishift = ishift+numintscals
      jshift = jshift+maxintscals

!     --- Put blanks at the back of ihelp

      i1 = 0
      i2 = 0
      do i = 1, ishift
         if ( ihelp(i)>100000 ) then
            i2 = i2 + 1
            ihelp(i) = ishift-numblank+i2
         else
            i1 = i1 + 1
            ihelp(i) = i1
         end if
      end do
      if ( debug ) call prinin ( ihelp, ishift, 'ihelp/2' )

!     --- Store inverse of ihelp into iseq

      do i = 1, numints
         iseq(ihelp(i)) = i
      end do
      ishift = numints
      jshift = maxints
      do i = 1, numreals
         iseq(ihelp(i+ishift)) = i+jshift
      end do
      ishift = ishift+numreals
      jshift = jshift+maxreals
      do i = 1, numscals
         iseq(ihelp(i+ishift)) = i+jshift
      end do
      ishift = ishift+numscals
      jshift = jshift+maxcls
      do i = 1, numvec
         iseq(ihelp(i+ishift)) = i+jshift
      end do
      ishift = ishift+numvec
      jshift = jshift+maxvec
      do i = 1, numstrings
         iseq(ihelp(i+ishift)) = i+jshift
      end do
      ishift = ishift+numstrings
      jshift = jshift+maxstrings
      do i = 1, numintscals
         iseq(ihelp(i+ishift)) = i+jshift
      end do
      ishift = ishift+numintscals
      jshift = jshift+maxintscals
      if ( debug ) call prinin ( iseq, ishift, 'iseq/1' )

      itop = ishift-numblank
      if ( itop==0 ) go to 1000

      ilow = 1
      itopref = itop

!     --- Sort the integers

      do i = ilow, itop
         iworkcp(i-ilow+1) = iwork(iseq(i))
      end do
      call chsort ( iworkcp, ihelp, itop-ilow+1 )
      do i = 1, itop-ilow+1
         ihelp(i) = iseq(ilow+ihelp(i)-1)
      end do
      do i = ilow, itop
         iseq(i) = ihelp(i-ilow+1)
      end do
      if ( debug ) then
         write(irefwr,1) 'ilow, itop', ilow, itop
         call prinin ( ihelp, ishift, 'ihelp/3' )
         call prinin ( iseq, ishift, 'iseq/2' )
      end if  ! ( debug )

!     --- While loop itop>ilow

300   if ( itop>ilow ) then

!     --- Fill array iprio

         ih = iprio(ilow)
         ih1 = iwork(iseq(ilow))
         nprio = ih
         do i = ilow+1, itop
            if ( iprio(i)==ih ) then

!           --- priority equal to preceding one

               if ( iwork(iseq(i))/=ih1 ) then

!              --- Character found different from preceding one

                  ih1 = iwork(iseq(i))
                  nprio = i

               end if
               iprio(i) = nprio

            else

!           --- priority not equal to preceding one

               ih = iprio(i)
               nprio = ih
               ih1 = iwork(iseq(i))

            end if

         end do

!        --- adapt ilow and itop

350      if ( iprio(ilow+1)/=iprio(ilow) ) then
            ilow = ilow+1
            if ( ilow<itop ) go to 350
         end if

360      if ( iprio(itop-1)/=iprio(itop) ) then
            itop = itop-1
            if ( ilow<itop ) go to 360
         end if

         iseqch = iseqch+1

!        --- For all groups with equal priority do

         iunder = ilow
         iupper = ilow+1
         ih = iprio(iunder)
         if ( debug ) write(irefwr,1) 'iunder, iupper', iunder, iupper
400      if ( iunder<itop ) then

!        --- while not all groups have been treated

            if ( iprio(iupper)==ih ) then
               iupper = iupper+1
               if ( iupper<=itop ) go to 400
            end if
            iupper = iupper-1
            if ( debug ) &
               write(irefwr,1) 'iunder, iupper', iunder, iupper
            if ( iupper>iunder ) then

!           --- Group with equal priority found, translate and sort

               do i = iunder, iupper
                  if ( iseq(i)<startscals+1 ) then
                     iwork(iseq(i)) = &
                        ichar(chincons(iseq(i))(iseqch:iseqch))
                  else if ( iseq(i)<startvecs+1 ) then
                     iwork(iseq(i)) = &
                     ichar(chscalars(iseq(i)-startscals)(iseqch:iseqch))
                  else if ( iseq(i)<startstrings+1 ) then
                     textchar = names(iseq(i)-startvecs)(iseqch:iseqch)
                     call askupc ( textchar )
                     iwork(iseq(i)) = ichar(textchar)
                  else if ( iseq(i)<startintscals+1 ) then
                     iwork(iseq(i)) = ichar(chstrings(iseq(i) &
                                      -startstrings)(iseqch:iseqch))
                  else
                     iwork(iseq(i)) = ichar(chscalars(maxcls+iseq(i) &
                                      -startintscals)(iseqch:iseqch))
                  end if  ! ( iseq(i)<startscals )
               end do

!              --- Put blanks at the beginning and adapt priority

               numblank = 0
               do i = iunder, iupper
                  if ( iwork(iseq(i))==ichar(' ') ) then
                     ihelp(i) = 1
                     numblank = numblank+1
                  else
                     ihelp(i) = 0
                  end if
               end do
               if ( numblank>0 ) then

!              --- There are numblank blanks

                  do i = iunder, iupper
                     iworkcp(i) = iseq(i)
                  end do
                  nprio = iprio(iunder)
                  ih = nprio+numblank
                  i1 = iunder
                  i2 = iunder+numblank
                  if ( debug ) write(irefwr,1) 'i1, i2', i1, i2
                  do i = iunder, iupper
                     if ( ihelp(i)==0 ) then
                        iprio(i2) = ih
                        iseq(i2) = iworkcp(i)
                        i2 = i2+1
                     else
                        iprio(i1) = nprio
                        nprio = nprio+1
                        iseq(i1) = iworkcp(i)
                        i1 = i1+1
                     end if
                  end do

               end if

!              --- Sort rest

               iunder = iunder+numblank
               if ( iupper>iunder ) then
                  do i = iunder, iupper
                     iworkcp(i) = iwork(iseq(i))
                  end do
                  call chsort ( iworkcp(iunder), ihelp, &
                                iupper-iunder+1 )
                  do i = 1, iupper-iunder+1
                     ihelp(i) = iseq(iunder+ihelp(i)-1)
                  end do
                  do i = 1, iupper-iunder+1
                     iseq(iunder+i-1) = ihelp(i)
                  end do

               end if

            end if
            iunder = iupper+1
            iupper = iupper+2
            if ( iunder<itop ) ih = iprio(iunder)

!           --- end while iunder<itop

            go to 400

         end if

!        --- adapt ilow and itop

         if ( debug ) write(irefwr,1) 'ilow, itop', ilow, itop
         if ( ilow<numints+numreals+numscals+numvec+numstrings+ &
              numintscals ) then

!        --- ilow may be raised

500         if ( iprio(ilow+1)/=iprio(ilow) ) then
               ilow = ilow+1
               if ( ilow<itop ) go to 500
            end if

         end if  ! ( ilow<numints+numreals+numscals+numvec... )

510      if ( itop>1 ) then
            if ( iprio(itop-1)/=iprio(itop) ) then
               itop = itop-1
               if ( ilow<itop ) go to 510
            end if
         end if

!        --- end while itop>ilow

         go to 300

      end if

      if ( debug ) then
         call prinin ( ihelp, itopref, 'ihelp/4' )
         call prinin ( iseq, itopref, 'iseq/3' )
         call prinin ( iprio, itopref, 'iprio' )
         call prinin ( iwork, numints, 'iwork_ints' )
         call prinin ( iwork(maxints+1), numreals, 'iwork_reals' )
         call prinin ( iwork(1+startscals), numscals, 'iwork_scalars' )
         call prinin ( iwork(1+startvecs), numvec, 'iwork_vectors' )
         call prinin ( iwork(1+startstrings), numstrings, &
                       'iwork_strings' )
         call prinin ( iwork(1+startintscals), numintscals, &
                       'iwork_intscals' )
      end if  ! ( debug )

!     --- Check uniqueness of names

      if ( iseq(1)<=maxints ) then
         text1 = chincons(iseq(1))
         itype1 = 1
      else if ( iseq(1)<=startscals ) then
         text1 = chincons(iseq(1))
         itype1 = 2
      else if ( iseq(1)<=startvecs ) then
         text1 = chscalars(iseq(1)-startscals)
         itype1 = 3
      else if ( iseq(1)<=startstrings ) then
         text1 = names(iseq(1)-startvecs)
         call askupc ( text1 )
         itype1 = 4
      else if ( iseq(1)<=startintscals ) then
         text1 = chstrings(iseq(1)-startstrings)
         itype1 = 5
      else
         text1 = chscalars(maxcls+iseq(1)-startintscals)
         itype1 = 6
      end if  ! ( iseq(1)<=maxints ) then

      do i = 2, itopref

         if ( iseq(i)<=maxints ) then
            text2 = chincons(iseq(i))
            itype2 = 1
         else if ( iseq(i)<=startscals ) then
            text2 = chincons(iseq(i))
            itype2 = 2
         else if ( iseq(i)<=startvecs ) then
            text2 = chscalars(iseq(i)-startscals)
            itype2 = 3
         else if ( iseq(i)<=startstrings ) then
            text2 = names(iseq(i)-startvecs)
            call askupc ( text2 )
            itype2 = 4
         else if ( iseq(i)<=startintscals ) then
            text2 = chstrings(iseq(i)-startstrings)
            itype2 = 5
         else
            text1 = chscalars(maxcls+iseq(i)-startintscals)
            itype2 = 6
         end if  ! ( iseq(1)<=maxints ) then

         if ( text1==text2 ) then

!        --- Error: names are identical

            select case ( itype1 )
            case(1)
               call errchr ( 'integer constant', 1 )
               call errint ( iseq(i-1), 1 )
            case(2)
               call errchr ( 'real constant', 1 )
               call errint ( iseq(i-1)-maxints, 1 )
            case(3)
               call errchr ( 'variable', 1 )
               call errint ( iseq(i-1)-startscals, 1 )
            case(4)
               call errchr ( 'vector_name', 1 )
               call errint ( iseq(i-1)-startvecs, 1 )
            case(5)
               call errchr ( 'string', 1 )
               call errint ( iseq(i-1)-startstrings, 1 )
            case(6)
               call errchr ( 'intvar', 1 )
               call errint ( iseq(i-1)-startintscals, 1 )
            end select  ! case ( itype1 )

            select case ( itype2 )
            case(1)
               call errchr ( 'integer constant', 2 )
               call errint ( iseq(i), 2 )
            case(2)
               call errchr ( 'real constant', 2 )
               call errint ( iseq(i)-maxints, 2 )
            case(3)
               call errchr ( 'variable', 2 )
               call errint ( iseq(i)-startscals, 2 )
            case(4)
               call errchr ( 'vector_name', 2 )
               call errint ( iseq(i)-startvecs, 2 )
            case(5)
               call errchr ( 'string', 1 )
               call errint ( iseq(i-1)-startstrings, 2 )
            case(6)
               call errchr ( 'intvar', 1 )
               call errint ( iseq(i-1)-startintscals, 2 )
            end select  ! case ( itype2 )

            call errchr ( text1, 3 )
            call errsub ( 789, 2, 0, 3 )

         end if
         text1 = text2
         itype1 = itype2

      end do

1000  call erclos ( 'inichnam' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inichnam'

      end if  ! ( debug )

      end
