      subroutine iniclear ( ibuffr, buffer, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 20-09-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear existing SEPRAN double precision array by setting it to 0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     clear
!     array
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol
      integer length
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipusol         Starting address of solution array in BUFFER
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI006         clear real space
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate array
!     compute length and starting position
!     clear array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniclear' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniclear'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate array

      call ini060text ( ibuffr, isol(1), 'usol' )
      if ( ierror/=0 ) go to 1000

!     --- compute length and starting position

      length = inidln(isol(1))
      ipusol = inidgttext ( isol(1), 'usol' )
      if ( ierror/=0 ) go to 1000

!     --- clear array

      call ini006 ( buffer(ipusol), length )

1000  call erclos ( 'iniclear' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniclear'

      end if

      end
