      subroutine inicoparr ( ibuffr, mmarr1, mmarr2, namear1, namear2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 19-02-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one sepran array into the other, which may already have been
!     created before. Otherwise it is created
!     Only the memory management addresses are used, so it may also be a part
!     of a composite array like kmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmarr1, mmarr2
      character *(*) namear1, namear2

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     mmarr1         i    Memory managment sequence number of array to be copied
!     mmarr2        i/o   Memory managment sequence number of copied array
!     namear1        i    Name of array to be copied
!     namear2        i    Name of copied array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iparr1, iparr2
      integer length, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iparr1         Starting address of array with mm number mmarr1
!     iparr2         Starting address of array with mm number mmarr2
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1531   Invalid memory management number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicoparr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicoparr'

      end if
      if ( mmarr1<=0 .or. mmarr1>iinfor ) then

!     --- mmarr1 is not a valied mm number

         call errint ( mmarr1, 1 )
         call errint ( iinfor, 2 )
         call errchr ( namear1, 1 )
         call errsub ( 1531, 2, 0, 1 )

      end if
      if ( mmarr2<0 .or. mmarr2>iinfor ) then

!     --- mmarr1 is not a valied mm number

         call errint ( mmarr2, 1 )
         call errint ( iinfor, 2 )
         call errchr ( namear2, 1 )
         call errsub ( 1531, 2, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute length of array

      length = inilen(mmarr1)

!     --- Activate array

      call ini060text ( ibuffr, mmarr1, namear1 )
      if ( ierror/=0 ) go to 1000

!     --- Create new array

      call ini081 ( ibuffr, mmarr2, length, namear2 )
      if ( ierror/=0 ) go to 1000

!     --- get pointers

      iparr1 = inigettext ( mmarr1, 'arr1' )
      iparr2 = inigettext ( mmarr2, 'arr2' )
      if ( ierror/=0 ) go to 1000

!     --- Actual copying

      do i = 0, length-1
         ibuffr(iparr2+i) = ibuffr(iparr1+i)
      end do

1000  call erclos ( 'inicoparr' )
      if ( debug ) write(irefwr,*) 'End inicoparr'

      end
