      subroutine inicopbufend ( ibuffr, mmindex, ipcopy, length, &
                                namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 15-11-2012 Long integers
!        version  1.1  date 30-05-2006 Debug statements
!        version  1.0  date 30-12-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Activate or create the array referred to by the memory management
!    index mmindex to the end of IBUFFR and copy the contents
!    from the starting position ipindex at the end of ibuffr to the new array
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipcopy
      integer ibuffr(*), mmindex, length
      character *(*) namear

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ipcopy         i    Starting address of array before copying
!     length         i    length of the array in words
!     mmindex        i    Memory management sequence number of array to be
!                         copied
!     namear         i    Name of array
!                         This name is used in the table if the array did not
!                         yet exist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipindex
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipindex        Starting address of array with memory management sequence
!                    number mmindex in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI000         Copy integer array to other integer array possibly itself
!     INI081         Create permanent space in array IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create array
!     compute starting address
!     Copy array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicopbufend' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicopbufend'
         write(irefwr,1) 'mmindex, ipcopy, length', &
                          mmindex, ipcopy, length
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- create array

      call ini081 ( ibuffr, mmindex, length, namear )

!     --- compute starting address

      ipindex = inigettext(mmindex, namear)
      if ( debug ) write(irefwr,1) 'mmindex, ipindex', mmindex, ipindex

!     --- Copy array

      call ini000 ( ibuffr(ipcopy), ibuffr(ipindex), 1, 1, length, 0 )

1000  call erclos ( 'inicopbufend' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicopbufend'

      end if

      end
