      subroutine inicopendbuf ( ibuffr, mmindex, ipindex, nbufol, &
                                namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 17-08-2013 Jump in case of error
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 15-03-2004 Extra parameter namear
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy the contents of the array referred to by the memory management
!    index mmindex to the end of IBUFFR and decrease the length of IBUFFR
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipindex, nbufol
      integer ibuffr(*), mmindex
      character *(*) namear

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ipindex        o    Starting address of array after copying
!     mmindex        i    Memory management sequence number of array to be
!                         copied
!     namear         i    Name of array
!                         This name is used in the table if the array did not
!                         yet exist
!     nbufol         o    help variable to store the old value of nbuffr
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iptemp
      integer lenindex

!     iptemp         Starting address of temporary array in IBUFFR
!     lenindex       Length of array to be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI000         Copy integer array to other integer array possibly itself
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI069         Reserve space at the end of IBUFFR for a semi-permanent
!                    array
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate array
!     compute starting address and length
!     reserve space at the end of IBUFFR
!     Copy array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicopendbuf' )

!     --- activate array

      call ini060text ( ibuffr, mmindex, namear )

!     --- compute starting address and length

      ipindex = inigettext ( mmindex, namear )
      if ( ierror/=0 ) go to 1000
      lenindex = inilen(mmindex)

!     --- reserve space at the end of IBUFFR

      call ini069 ( ibuffr, iptemp, lenindex, nbufol )
      if ( ierror/=0 ) go to 1000

!     --- Copy array

      call ini000 ( ibuffr(ipindex), ibuffr(iptemp), 1, 1, lenindex, 0 )
      ipindex = iptemp

1000  call erclos ( 'inicopendbuf' )
      end
