      subroutine inidaxpy ( ibuffr, buffer, ivecx, ivecy, alpha )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 23-09-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the daxpy operation y = alpha x + y but now on two sepran
!     solution arrays
! **********************************************************************
!
!                       KEYWORDS
!
!     add
!     multiplication
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), ivecx(*), ivecy(*)
      double precision buffer(*), alpha

!     alpha          i    multiplication factor
!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ivecx          i    Vector x
!     ivecy          i    Vector y to be changed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipvecx, ipvecy
      integer itypvc, nusol
      logical debug, sepranvector

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipvecx         Starting address of array vecx
!     ipvecy         Starting address of array vecy
!     itypvc         Type of sepran vector
!     nusol          Total number of unknowns
!     sepranvector   if true the vector is a known sepran vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate array
!     Compute starting addresses
!     Perform operation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inidaxpy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inidaxpy'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check arrays

      itypvc = ivecx(2)
      sepranvector = itypvc==110 .or. itypvc==115 .or. &
                     itypvc==116 .or. itypvc==126 .or. &
                     itypvc==129
      if ( .not.sepranvector ) then

!     --- Not a known sepran vector

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      end if  ! ( .not.sepranvector )

      itypvc = ivecy(2)
      sepranvector = itypvc==110 .or. itypvc==115 .or. &
                     itypvc==116 .or. itypvc==126 .or. &
                     itypvc==129
      if ( .not.sepranvector ) then

!     --- Not a known sepran vector

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      end if  ! ( .not.sepranvector )

!     --- Activate arrays

      call ini060text ( ibuffr, ivecx(1), 'vecx' )
      call ini060text ( ibuffr, ivecy(1), 'vecy' )

!     --- Compute starting addresses

      ipvecx = inidgttext ( ivecx(1), 'vecx' )
      ipvecy = inidgttext ( ivecy(1), 'vecy' )
      nusol = ivecx(5)

!     --- Perform operation

      call daxpy ( nusol, alpha, buffer(ipvecx), 1, buffer(ipvecy), 1 )

1000  call erclos ( 'inidaxpy' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inidaxpy'

      end if

      end
