      function inidgt ( index )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 21-11-2012 Long integers
!        version  1.4  date 03-10-2007 Update of error message
!        version  1.3  date 03-02-2003 Update of error message
!
!   copyright (c) 1994-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with pointer corresponding to index, where pointer refers to
!     buffer instead of ibuffr. It is checked if pointer is available
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) inidgt
      integer index

!     index   i    index refers to the array sequence number from which
!                  the starting position must be computed
!     inidgt  o    Gives pointer in array BUFFER corresponding to index
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) istart
      integer iprio

!     iprio   Priority number
!     istart  Starting address in array IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     802   INDEX out of range
!    1446   array has not been activated proerly
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check if index between 1 and numarrs
!   Check if index has been activated
!   Set pointer
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      istart = numarrs+1
      if ( index<1 .or. index>numarrs ) then

!     --- index is not a valid index

         call eropen ( 'inidgt' )
         call errint ( index, 1 )
         call errint ( abs(index), 2 )
         call errint ( numarrs, 3 )
         call errchr ( ' ', 1 )
         call errsub ( 802, 3, 0, 1 )
         call erclos ( 'inidgt' )

      else

!     --- index in the correct range, check if array has been activated

         istart = infor(index)
         iprio  = infor1(2,index)
         if ( istart<0 .or. iprio<10 ) then

!        --- Array has not been activated properly

            call eropen ( 'inidgt' )
            call errint ( index, 1 )
            call errintlong ( istart, 2 )
            call errint ( iprio, 3 )
            call errchr ( namarr(index), 1 )
            call errchr ( ' ', 2 )
            call errsub ( 1446, 3, 0, 2 )
            call erclos ( 'inidgt' )

         end if

      end if

      if ( intlen==1 ) then
         inidgt = istart
      else
         inidgt = (istart-1)/intlen+1
      end if
      end
