      function inidln ( index )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 03-02-2003 Update of error message
!        version  1.1  date 26-02-1999 Use parameter numarrs
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with length of array corresponding to index in reals
!     It is checked if pointer is available
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cbuffrf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inidln, index

!     index   i    index refers to the array sequence number from which
!                  the starting position must be computed
!     inidln  o    Gives length of array corresponding to index
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer length

!     length  Length of array in inetegers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     802   INDEX out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check if index between 1 and numarrs
!   Set lengthr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      length = 0
      if ( index<1 .or. index>numarrs ) then

!     --- index is not a valid index

         call eropen ( 'inidln' )
         call errint ( index, 1 )
         call errint ( abs(index), 2 )
         call errint ( numarrs, 3 )
         call errchr ( ' ', 1 )
         call errsub ( 802, 3, 0, 1 )
         call erclos ( 'inidln' )

      else

!     --- index in the correct range, &

         length = infor1(1,index)

      end if

      inidln = length/intlen

      end
