      subroutine inihlp ( mesnum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-02-1996 Jump in case of error
!        version  1.0  date 18-06-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Print help menu
!    New pages in the help text must be indicated by ^ in column 2
! **********************************************************************
!
!                       KEYWORDS
!
!     menu
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cmachnf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mesnum

!     mesnum   i    Sequence number of help message in message file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer      i, ifrom, istart(21), ito ,maxerr, &
                   linenr, numlin, &
                   iref, idum, nchar, j
      parameter    ( iref=17)
      character * 80 line

!     i        loop variable
!     idum     Integer dummy parameter
!     ifrom    information in array istart is read from position ifrom
!     iref     Reference to error message file
!     istart   integer array containing a part of the starting records
!              of the messages.
!     ito      information in array istart is read to position ito.
!     j        loop variable
!     line    line buffer
!     linenr   record number of message line
!     maxerr   1 + maximum number of messages that are stored
!              in the message file.
!     nchar    number of characters in a line
!     numlin   number of lines of a message
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     INICLS    Clear screen
!     READFL  Read array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     read maximum message number from message text file ;
!     if (message number correct) then
!        read starting record number ;
!        calculate number of lines ;
!        if ( number of lines > 0 ) then
!           for each line do
!              read line ;
!              process and print line ;
!        else
!           print warning : message does not exist ;
!     else
!        print warning : message number out of range ;
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inihlp' )
      call inicls

!     --- read largest message number permitted

      ifrom = 1
      ito   = 1
      linenr= 1
      idum  = 0
      call readfl ( linenr, ifrom, ito, idum, iref, istart )
      if ( ierror.ne.0 ) go to 1000
      maxerr = istart(1)

      if ( mesnum.gt.0 .and. mesnum.le.maxerr ) then

!     --- Read starting record of message

         linenr = 2 + (mesnum-1)/nrec4
         ito    = mod(mesnum,nrec4)+1
         if (ito.eq.1) ito = nrec4+1
         call readfl (linenr, ifrom, ito, idum, iref, istart)
         if ( ierror.ne.0 ) go to 1000

         linenr = istart(ito-1)
         numlin = istart(ito) - linenr

         if ( numlin.gt.0 ) then

!        --- Read and print message

            do 200 i = linenr, linenr-1+numlin

!           --- read one message line

               read ( iref17, rec=i ) line
               if ( line(2:2).eq.'^' ) then

!              --- new page symbol found

                  write(irefwr,100)
100               format( ////' Press ENTER (RETURN) to continue ' )
                  read(*,*)
                  call inicls

               else

!              --- print line

                  nchar = 80
                  do 110 j = 80, 1, -1
                     if ( line(j:j).ne.' ' ) then
                        nchar = j
                        go to 120
                     end if
110               continue
120               write (irefwr,'(a)') line(1:nchar)

               end if

200         continue

         else

!        --- Message not present in the file

            write (irefwr,210) 'ERTEXT', mesnum
210         format (' *** WARNING : '/ &
                    ' ',a,' number ',i6,' has not yet been added', &
                    ' to the  message file')

         end if

      else

!     --- Message number out of range

         write (irefwr,210) 'ERTEXT', mesnum

      end if

      write(irefwr,100)
      read(*,*)

1000  call erclos ( 'inihlp' )
      end
