      subroutine ininodeelgrp ( ibuffr, kmesh, ielgrp, nodes, nnodes, &
                                map )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all nodes of element group ielgrp in array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), ielgrp, nodes(*), nnodes, &
              map(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     ielgrp         i    element group number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     map            o    Mapping array of length npoint to make an
!                         inverse mapping of array nodes
!                         map(i) = 0, means that node i is not in the
!                                     element group
!                         map(i)>0, defines position of node i in array nodes
!     nnodes        i/o   Number of nodes in array nodes
!                         At input declared length of nodes
!                         At output actual number of nodes
!     nodes          i    Contains the node numbers of the nodes in the
!                         element group
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inelem, inpelm, i, numnodes, lenkmeshc, npoint
      integer (kind=8) ipkmsc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     ipkmsc         Starting address of array KMESH part c
!     lenkmeshc      length of part of array kmeshc corresponding to
!                    element group
!     npoint         Number of nodal points
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnelmelgrp, getnpelmelgrp
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNELMELGRP   Fetch number of elements in elementgroup ielgrp
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     ININODEELGRP1  Fill all nodes of one element group in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodeelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodeelgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call ini060text ( ibuffr, kmesh(17), 'kmeshc' )

!     --- Compute starting addresses

      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )

!     --- Compute starting address of element group in kmeshc

      do i = 1, ielgrp-1

         inpelm = getnpelmelgrp ( kmesh, i )
         inelem = getnelmelgrp ( kmesh, i )
         ipkmsc = ipkmsc+inelem*inpelm

      end do  ! i = 1, ielgrp-1

!     --- Compute length of kmeshc for ielgrp

      inpelm = getnpelmelgrp ( kmesh, ielgrp )
      inelem = getnelmelgrp ( kmesh, ielgrp )

      lenkmeshc = inpelm*inelem
      npoint = kmesh(8)

      call ininodeelgrp1 ( ibuffr(ipkmsc), nodes, numnodes, lenkmeshc, &
                           map, npoint )

      call inichkarlen ( numnodes, nnodes, 'nodes' )
      nnodes = numnodes

1000  call erclos ( 'ininodeelgrp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininodeelgrp'

      end if  ! ( debug )

      end
