      function ininodezerlev1 ( ipointobst, ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-12-2011 Use inizerolev
!        version  1.0  date 19-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with node at boundary defined by zero levelset
!     1d only
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     node
!     level_set
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ininodezerlev1, ipointobst(*), ilev

!     ilev           i    Sequence number of level being considered
!     ininodezerlev1   o    Node number found
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numbndobst
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     istart         Starting address in ipointobst
!     numbndobst     Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3085   Number of nodes in boundary of zero level set is not 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodezerlev1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodezerlev1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ininodezerlev1 = 0  ! initialization

!     --- get starting address and number of points in boundary

      call inizerolev ( ipointobst, istart, numbndobst, ilev )

      if ( numbndobst/=1 ) then

!     --- Error: boundary consists not exactly of 1 point

         call errint ( numbndobst, 1 )
         call errsub ( 3085, 1, 0, 0 )

      else

!     --- exactly 1 point, find node

         ininodezerlev1 = ipointobst(istart+1)

      end if  ! ( numbndobst/=1 )

1000  call erclos ( 'ininodezerlev1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininodezerlev1'

      end if  ! ( debug )

      end
