      function iniprb ( iprob, kprob )
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 1.1    date   22-10-1995 Return with ipkprb=0 in case of error
!        version 1.0    date   30-04-1995
!
!
!
!   copyright (c) 1995-1995 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     Return with starting address of part iprob of array KPROB
!     in array KPROB minus one
!     if iprob is not available the routine returns with 0
! ********************************************************************
!
!                      KEYWORDS
!
!    memory_management
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer iniprb, iprob, kprob(*)

!     iniprb  o    Gives starting address of part array KPROB in array
!                  KPROB minus one
!     iprob   i    Problem sequence number
!     kprob   i    Standard SEPRAN array, containing information of the problem
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer ipkprb, nprob, k

!     ipkprb  Starting address of array KPROB for actual problem number
!     k       Counting variable
!     nprob   Number of various problems
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     699   Problem number incorrect
! ********************************************************************
!
!                    PSEUDO CODE
!
!   Compute nprob
!   Check iprob
!   Set ipkprb
! ======================================================================
!
      nprob = max ( 1, kprob(40) )
      if ( iprob.le.0 .or. iprob.gt.nprob ) then
         call eropen ( 'iniprb' )
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0 )
         call erclos ( 'iniprb' )
         ipkprb = 0
      else
         ipkprb = 1
         do 100 k = 2, iprob
            ipkprb = ipkprb + kprob ( 2+ipkprb )
100      continue
         iniprb = ipkprb - 1
      end if
      end
