      subroutine inisubsurface ( kmesh, kmeshr, isurnr, nsubsf, &
                                 subsurfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 03-11-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all subsurfaces of a compound surface and store in array
!     subsurfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     compound_surface
!     surfaces
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshr(*), isurnr, nsubsf, subsurfaces(*)

!     isurnr         i    Surface sequence number of surface of which the
!                         subsurfaces must be found
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     nsubsf         o    Number of subsurfaces
!     subsurfaces    o    Array of length nsubsf in which all the subsurfaces
!                         of surface isurnr are stored with positive sign
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isubsf

!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( kmeshr has not been filled ) then
!        nsubsf := 1;  subsurfaces(1) := isurnr
!     else ( kmeshr has been filled )
!        Compute nusbsf
!        Fill subsurfaces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inisubsurface' )
      if ( kmesh(32).eq.0 ) then

!     --- kmesh(32) = 0, hence kmeshr has not been filled
!         nsubsf := 1;  subsurfaces(1) := isurnr

         nsubsf = 1
         subsurfaces(1) = isurnr

      else

!     --- kmesh(32) # 0, hence kmeshr has been filled
!         Compute nusbsf
!         Fill subsurfaces

         istart = kmeshr(isurnr)
         nsubsf = kmeshr(isurnr+1) - istart
         if ( nsubsf.eq.0 ) then

!        --- There are no subsurfaces of isurnr
!            nsubsf := 1;  subsurfaces(1) := isurnr

            nsubsf = 1
            subsurfaces(1) = isurnr

         else

!        --- There are subsurfaces of isurnr

            do isubsf = 1, nsubsf
               subsurfaces(isubsf) = abs(kmeshr(istart+isubsf))
            end do

         end if

      end if

      call erclos ( 'inisubsurface' )

      end
