      subroutine initgs
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.15 date 29-08-2006 Raise recnr for file
!        version  3.14 date 05-01-2003 Extension for prisms
!        version  3.13 date 23-12-2002 Remove cedges
!        version  3.12 date 07-05-2002 Tijmen Gunther: extension 11/15pts Gauss
!        version  3.11 date 11-01-2001 Adaptation choice of / or \
!        version  3.10 date 22-12-2000 Extension with cedges
!        version  3.9  date 26-06-2000 Fill vertex arrays
!        version  3.8  date 13-01-2000 Adaptation 4 point Gauss for tetrahedr.
!        version  3.7  date 18-02-1999 Include commons
!        version  3.6  date 10-08-1995 Save commons
!        version  3.5  date 23-09-1993 Extension common cgaus
!        version  3.4  date 28-12-1992 Extension common ctyp3d
!        version  3.3  date 09-10-1991 Peter Slikkerveer:extension 7pts gauss
!        version  3.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.1  date 02-06-1991 New common ctyp3d
!        version  3.0  date 10-11-1988 Reorganization, extension of cgaus
!        version  2.0  date 14-12-1987 addition of ctyp3d
!        version  1.0  date 17-09-1987
!
!   copyright (c) 1987-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill common blocks cgaus and ctyp3d
!     Fill common block cgaus with values for the numerical integration
!     Gauss rules
!     Fill common block ctyp3d with information of subdivision of elements
!     in linear subelements.  This common block is used by interpolation
!     subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     numerical_integration
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cgausf90'
      include 'SPcommon/ctypdvf90'
      include 'SPcommon/ctyp3df90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, h1, h2, h3, alpha, beta, gamma, delta, &
                       alpha1, beta1
      integer i, ind(4), ind1(6), ind2(11), ind3(6), length

!     alpha          constant
!     alpha1         constant
!     beta           constant
!     beta1          multiplication factor
!     delta          constant
!     gamma          constant
!     h              help variable to store a quantity
!     h1             help variable to store a quantity
!     h2             help variable to store a quantity
!     h3             help variable to store a quantity
!     i              Counting variable
!     ind            index array to define cyclic permutation for three numbers
!     ind1           index array to define cyclic permutation for four numbers
!     ind2           index array to define cyclic permutation for four numbers
!     ind3           index array to define cyclic permutation for four numbers
!     length         Help parameter to indicate the length of arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     READFL  Read array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind /2,3,1,2/, ind1 /2,3,4,1,2,3/, &
           ind2 /2,3,4,5,6,7,8,9,10,11,1/, &
           ind3 /3,1,2,3,1,2/
! ======================================================================
!
!     --- Fill Gauss points
!         One-point Gauss rule for triangles

      wgs1 = .5d0
      h = 1d0 / 3d0
      do i = 1, 3
         alam1(i) = h
      end do

!     --- Four-point Gauss rule for triangles

      wgs2(1) = -.28125d0
      do i = 1, 3
         wgs2(i+1)           = .2604166666666667d0
         alam2(i,1)          = h
         alam2(i,1+i)        = .6d0
         alam2(i,1+ind(i))   = .2d0
         alam2(i,1+ind(i+1)) = .2d0
      end do

!     --- seven-point gauss rule for triangles

      wgs3(1) =  0.1125d0

      do i = 1, 3
         alam3(i,1) = h

         wgs3(i+1)           =  0.066197076394253d0
         alam3(i,1+i)        =  0.059715871789770d0
         alam3(i,1+ind(i))   =  0.470142064105115d0
         alam3(i,1+ind(i+1)) =  0.470142064105115d0

         wgs3(4+i)           =  0.0629695902724135d0
         alam3(i,4+i)        =  0.797426985353087d0
         alam3(i,4+ind(i))   =  0.101286507323456d0
         alam3(i,4+ind(i+1)) =  0.101286507323456d0
      end do

!     --- Two-point rule in one dimension

      h = sqrt(3d0)/6d0
      alam4(1) = 0.5d0 - h
      alam4(2) = 0.5d0 + h

!     --- Three-point rule in one dimension

      h = sqrt(15d0)/10d0
      alam5(1) = 0.5d0 - h
      alam5(2) = 0.5d0
      alam5(3) = 0.5d0 + h
      wgs5(1)  = 5d0/18d0
      wgs5(2)  = 8d0/18d0
      wgs5(3)  = wgs5(1)

!     --- One-point Gauss rule for tetrahedra

      wgs6 = 1d0/6d0
      do i = 1, 4
         alam6(i) = 0.25d0
      end do

!     --- Four-point Gauss rule for tetrahedra

      h = 1d0/24d0
      h1 = (5d0-sqrt(5d0))*0.05d0
      h2 = (5d0+3d0*sqrt(5d0))*0.05d0
      do i = 1, 4
         wgs7(i)            = h
         alam7(i,i)         = h2
         alam7(i,ind1(i))   = h1
         alam7(i,ind1(i+1)) = h1
         alam7(i,ind1(i+2)) = h1
      end do

!     --- Five-point Gauss rule for tetrahedra

      h = 3d0/40d0
      wgs8(1) = -2d0/15d0
      do i = 1, 4
         wgs8(i+1)  = h
      end do
      h = 1d0/6d0
      do i = 1, 4
         alam8(i,1)           = 0.25d0
         alam8(i,1+i)         = 0.5d0
         alam8(i,1+ind1(i))   = h
         alam8(i,1+ind1(i+1)) = h
         alam8(i,1+ind1(i+2)) = h
      end do

!     --- Eleven-point Gauss rule for Tetrahedra
!         Fill array with weights for 11-Gauss points

      h = -13155555555556d-15
      h1 =  7622222222222d-15
      h2 = 24888888888889d-15

      wgs9(1) = h
      do i = 1, 4
         wgs9(i+1) = h1
      end do
      do i = 1, 6
         wgs9(i+5) = h2
      end do

!     --- Fill array with barycentric coordinates of the gauss-points
!         according to alam9(i,k)=lambda (x )
!                                       i  k

      alpha = 785714285714286d-15
      beta  = 71428571428571d-15
      gamma = 399403576166799d-15
      delta = 100596423833201d-15

      do i = 1, 4
         alam9(i,1)           = 0.25d0
         alam9(i,1+i)         = alpha
         alam9(i,1+ind1(i))   = beta
         alam9(i,1+ind1(i+1)) = beta
         alam9(i,1+ind1(i+2)) = beta
         if ( i==1 ) then
            alam9(i,4+ind2(i))   = gamma
            alam9(i,4+ind2(i+1)) = gamma
            alam9(i,4+ind2(i+2)) = gamma
            alam9(i,4+ind2(i+3)) = delta
            alam9(i,4+ind2(i+4)) = delta
            alam9(i,4+ind2(i+5)) = delta
         else
            alam9(i,4+i)         = gamma
            alam9(i,8+ind3(1+i)) = gamma
            alam9(i,8+ind3(2+i)) = gamma
            alam9(i,7+i)         = delta
            alam9(i,5+ind3(1+i)) = delta
            alam9(i,5+ind3(2+i)) = delta
         end if
      end do

!     --- 15-point Gauss rule for Tetrahedra
!         Fill array with weights for 15-Gauss points

      h  = 30283678097089d-15
      h1 = 6026785714286d-15
      h2 = 11645249086029d-15
      h3 = 10949141561386d-15

      wgs10(1) = h
      do i = 1, 4
         wgs10(i+1) = h1
      end do
      do i = 1, 4
         wgs10(i+5) = h2
      end do
      do i = 1, 6
         wgs10(i+9) = h3
      end do

!     --- Fill array with barycentric coordinates of the gauss-points
!         according to alam10(i,k)=lambda (x )
!                                       i  k

      alpha1 = 66550153573664d-15
      beta1  = 433449846426336d-15

      do i = 1, 4
         alam10(i,1)           = 0.25d0
         alam10(i,1+i)         = 0d0
         alam10(i,1+ind1(i))   = 1d0/3d0
         alam10(i,1+ind1(i+1)) = 1d0/3d0
         alam10(i,1+ind1(i+2)) = 1d0/3d0
         alam10(i,5+i)         = 8d0/11d0
         alam10(i,5+ind1(i))   = 1d0/11d0
         alam10(i,5+ind1(i+1)) = 1d0/11d0
         alam10(i,5+ind1(i+2)) = 1d0/11d0

         alam10(i,9+i) = alpha1
         alam10(i,9+ind1(i)) = alpha1
         alam10(i,9+ind1(i+1)) = alpha1
         alam10(i,11+ind1(i)) = beta1
         alam10(i,11+ind1(i+1)) = beta1
         alam10(i,11+ind1(i+2)) = beta1
      end do
      alam10(2,10)=alam10(1,10)
      alam10(2,14)=alam10(1,10)
      alam10(2,15)=alam10(1,10)
      alam10(2,11)=alam10(1,15)
      alam10(2,12)=alam10(1,15)
      alam10(2,13)=alam10(1,15)
      alam10(3,10)=alam10(1,15)
      alam10(3,12)=alam10(1,15)
      alam10(3,14)=alam10(1,15)
      alam10(3,11)=alam10(1,10)
      alam10(3,13)=alam10(1,10)
      alam10(3,15)=alam10(1,10)
      alam10(4,10)=alam10(1,15)
      alam10(4,11)=alam10(1,15)
      alam10(4,15)=alam10(1,15)
      alam10(4,12)=alam10(1,10)
      alam10(4,13)=alam10(1,10)
      alam10(4,14)=alam10(1,10)

!     --- Fill arrays ityps and itetra etc in common block ctyp3d
!         by reading from file file 1, record number 10

      length = 24+352+24+142+4+8
      call readfl ( 20, 1, length, 1, 1, ityps )

!     --- Fill arrays numsel(20), ishstr(20) and ishsub(206) in common
!         block ctypdv by reading from file 1, record number 11

      length = 20+20+20+206
      call readfl ( 21, 1, length, 1, 1, numpel )

      end
