      function initrianedge ( iedge1, iedge2, nmeshh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-02-2009 Remove error message
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find triangle spanned by edges 1 and 2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer initrianedge, iedge1, iedge2, nmeshh(*)

!     iedge1         i    edge sequence number
!     iedge2         i    edge sequence number
!     initrianedge   o    triangle connected to iedge1 and iedge2
!                         if 0 no triangle found
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, itriangle1, itriangle2
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipstart        Starting address in array
!     itriangle1     First triangle connected to iedge1
!     itriangle2     Second triangle connected to iedge1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'initrianedge' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from initrianedge'

      end if
      if ( ierror/=0 ) go to 1000

      ipstart = nmeshh(iedge1)
      itriangle1 = nmeshh(ipstart+1)
      itriangle2 = nmeshh(ipstart+2)
      ipstart = nmeshh(iedge2)
      if ( nmeshh(ipstart+1)==itriangle1 .or. &
           nmeshh(ipstart+2)==itriangle1 ) then

!     --- triangle = itriangle1

         initrianedge = itriangle1

      else if ( nmeshh(ipstart+1)==itriangle2 .or. &
                nmeshh(ipstart+2)==itriangle2 ) then

!     --- triangle = itriangle2

         initrianedge = itriangle2

      else

!     --- no triangle found

         initrianedge = 0

      end if  ! ( nmeshh(ipstart+1)==itriangle1 )

1000  call erclos ( 'initrianedge' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End initrianedge'

      end if

      end
