      function inivectr ( ivectr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with sequence number of vector
!     indexed array is used if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inivectr, ivectr

!     inivectr       o    Result: adapted sequence number of vector
!     ivectr         i    sequence number of vector (might be indexed)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inivectr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inivectr'
         write(irefwr,1) 'ivectr', ivectr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ivectr>10000 ) then

!     --- ivectr>10000 hence indexed vector

         iseq = ivectr/10000
         inivectr = mod(ivectr,10000)+nint(scalars(iseq))-1
         if ( debug ) &
            write(irefwr,1) 'iseq, index, inivectr', &
                             iseq, nint(scalars(iseq)), inivectr

      else

!     --- ivectr<10000 hence standard vector

         inivectr = ivectr

      end if  ! ( ivectr>10000 )

1000  call erclos ( 'inivectr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inivectr', inivectr
         write(irefwr,*) 'End inivectr'

      end if  ! ( debug )

      end
