      subroutine intx00keyw ( code, ncode, nncode, ncodmx, icode, &
                              numchar, keyword, &
                              itype, nout, tabout, iposut, icountkey, &
                              table, readnx, tabmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 11-05-2014 Extra parameters
!        version  2.2  date 08-03-2013 Correction test
!        version  2.1  date 20-04-2012 Change of format
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string read is (main) keyword
!     If so return with icode >0 or finished true
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nncode, ncode(nncode), ncodmx(nncode), &
                             itype(nncode), nout, numchar
      integer, intent(inout) :: icode, iposut, icountkey(*)
      double precision, intent(inout) :: tabout(nout), table(nncode), &
                                         tabmax(nncode)
      logical, intent(inout) :: readnx
      character (len=*) :: code(nncode), keyword

!     code           i    Table with keywords
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     icountkey     i/o   Array that counts the number of times a keyword has been
!                         found
!     iposut        i/o   Position in array tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     keyword        i    Text containing the string without numbers
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     numchar        i    Number of characters in keyword
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug, check

!     check          Help parameter for if statement
!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTXKEYWORD    Check if string read is (main) keyword
!     INTXKEYWORD2   Check if string read is (main) keyword
!     INTXSCALAR     Check if one of the keywords is S or Scalar
!     INTXSTRING     Check if one of the keywords is String
!     INTXVECTOR     Check if one of the keywords is V or Vector
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx00keyw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx00keyw'
         write(irefwr,1) 'lastitem, lastitembef', lastitem, lastitembef
         write(irefwr,1) 'numchar, ityprd', numchar, ityprd
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d13.7) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if string is a standard keyword

      call intxkeyword ( code, ncode, nncode, ncodmx, icode, &
                         numchar, keyword )
      if ( debug ) write(irefwr,1) 'after intxkeyword', icode

!     --- keyitems(lastitem) = icode

      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not a standard keyword
!         Check if one of the keywords is V or Vector
!         In that case it might be a vectorname with number

      call intxvector ( code, ncode, nncode, ncodmx, icode, &
                        itype, nout, tabout, iposut, icountkey, &
                        table, readnx, tabmax )
      if ( debug ) write(irefwr,1) 'after intxvector', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not a standard keyword nor a vector name
!         Check if one of the keywords is S or Scalar
!         In that case it might be a scalar name with number

      call intxscalar ( code, ncode, nncode, ncodmx, icode )
      if ( debug ) write(irefwr,1) 'after intxscalar: icode', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not recognized yet
!         Might be a string

      call intxstring ( code, nncode, ncodmx, icode )
      if ( debug ) write(irefwr,1) 'after intxstring: icode', icode
      if ( icode>0 ) go to 1000  ! keyword found

!     --- String is not recognized yet
!         Might be a constant
!         If it is a constant and it is preceded by a minus sign, &
!         then multiply by -1

      call readstrconst
      check = lastitem>1 .and. (ityprd==2 .or. ityprd==3)
      if ( check ) check = typeitems(lastitem-1)==11
      if ( check ) then

         if ( index_rdarr(3)>0 ) then
            index_rdarr(3)=-index_rdarr(3)
         else
            realrd = -realrd
         end if  ! ( index_rdarr(3)>0 )

      end if  ! ( check )

!     --- In case string is not one of the previous options, check if it
!         is a keyword that is not spelled completely by comparing the
!         first characters only

      if ( ityprd==4 ) then

!     --- ityprd = 4, hence still a string

         call intxkeyword2 ( code, ncode, nncode, ncodmx, icode, &
                             numchar, keyword )

      end if  ! ( ityprd==4 )

1000  call erclos ( 'intx00keyw' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode, lastitem, ityprd, index_rdarr(3)', &
                          icode, lastitem, ityprd, index_rdarr(3)
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End intx00keyw'

      end if  ! ( debug )

      end
