      subroutine intx0215 ( icode, table, nout, tabout, minusg, &
                            tabmin, tabmax, code, finished, &
                            iposut, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 10-11-2014 Check tabout
!        version  1.6  date 18-01-2012 Extension with x_coor
!        version  1.5  date 12-09-2011 Extension with coor
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by an integer, repeatedly
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, minusg, ihelp(*)
      character (len=*) code(*)
      double precision table(*), tabmin(*), tabmax(*), tabout(nout)
      logical finished

!     code           i    Table with keywords
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     iposut         i    Position in array tabout
!     minusg         i    Indicates if a minus sign has been found (-1) or
!                         not (1)
!     nout           i    Number of entries in tabout
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivalrd, maxval
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ivalrd         Integer value read
!     maxval         Maximum value allowed for the integer read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1468   Array tabout too small
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0215' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0215'
         write(irefwr,1) 'icode, ihelp(icode)', icode, ihelp(icode)
         write(irefwr,1) 'lastitem, ityprd, minusg', &
                          lastitem, ityprd, minusg
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ihelp(icode)==0 ) then

!     --- ihelp(icode) = 0, keyword has been found for the
!         first time

         table(icode) = iposut + 1
         iposut = iposut + nint(tabmin(icode))
         if ( iposut>nout ) go to 1000

      end if

      if ( ityprd==4 ) call asknxt  ! read item only if preceded by string
      if ( debug ) write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd

      if ( ityprd==4 ) then

!     --- String found, check if it is a vector, scalar or constant

         if ( debug ) then
            write(irefwr,*) 'before readrealscal'
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         lastitem = lastitembef
         call readrealscal
         if ( ityprd==102 ) ityprd = 2  ! vector found

         if ( debug ) then
            write(irefwr,*) 'after readrealscal'
            write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         end if  ! ( debug )

      end if  ! ( ityprd==4 )

      if ( ityprd==2 .or. ityprd==11 ) then

!     --- Integer number found

         ivalrd = nint(realrd)
         if ( ityprd==11 ) ivalrd = ivalrd - 100000000
         maxval = nint(tabmax(icode))

         if ( debug ) write(irefwr,1) 'ivalrd, maxval', ivalrd, maxval

         if ( ivalrd>-1000 .and. ivalrd<1 .or. ivalrd>maxval ) then

!        --- Error integer out of range

            call errint ( ivalrd, 1 )
            call errint ( 1, 2 )
            call errint ( maxval, 3 )
            call errchr ( code(icode), 1 )
            call errsub ( 1870, 3, 0, 1 )
            call errcrd

         end if

         ivalrd = abs(ivalrd)  ! Make ivalrd positive (necessary for coor)

         if ( nint(table(icode))+ihelp(icode)>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! (      nint(table(icode))+ihelp(icode)>nout )

         tabout(nint(table(icode))+ihelp(icode)) = &
            minusg*ivalrd
         ihelp(icode) = ihelp(icode) + 1

      else

!     --- Not an integer number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         finished = .true.

      end if

1000  call erclos ( 'intx0215' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode, ihelp(icode)', icode, ihelp(icode)
         write(irefwr,*) 'End intx0215'

      end if

      end
