      subroutine intx0223 ( icountkey, icode, table, code, readnx, &
                            subkey, codemn, nmcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by possibly a special keyword that is skipped
!     table(icode) is set equal to 1
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, icountkey(*), nmcode(*)
      character (len=*) code(*), subkey, codemn(*)
      double precision table(*)
      logical readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     subkey         i    Sub (second) keyword
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, numkey
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ih             Help variable to store a constant temporarily
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1467   keyword has been found more than once
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0223' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0223'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      call asknxt
      if ( icountkey(icode).ne.0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      ih = nint(-table(icode))
      if ( ih.lt.0 ) then
         call errchr ( code(icode), 1 )
         call errsub ( 1467, 0, 0, 1 )
         call errcrd
         go to 1000
      end if
      subkey = codemn(ih)
      numkey = nmcode(ih)
      if ( ityprd.eq.4 ) then
         if ( string(1:numkey).ne.subkey(1:numkey) ) readnx = .false.
      else
         readnx = .false.
      end if
      table(icode) = 1d0

1000  call erclos ( 'intx0223' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0223'

      end if

      end
