      subroutine intx02expr ( nncode, itype, table, iretur, tabout, &
                              nout, iseqkeyw, nseqkeyw, readnx, iposut, &
                              finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 10-01-2015 New call to readreal
!        version  1.3  date 09-05-2014 Extra debug
!        version  1.2  date 08-12-2013 Extension with integer scalars
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Check if string read corresponds to expression
!      if so evaluate and return with number
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, itype(*), iretur, nout, &
              iseqkeyw(*), nseqkeyw, iposut
      double precision table(nncode), tabout(nout)
      logical readnx, finished

!     finished      i/o   Indicates if a main keyword has been found (true)
!                         or not (false)
!     iposut        i/o   Position in array tabout
!     iretur        i/o   Return value of subroutine INTX02
!                         See subroutine INTX02
!     iseqkeyw      i/o   Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                         iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                         -isekeyw(i) refers to position in
!                                         tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     nseqkeyw       i    Number of entries in iseqkeyw
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug, openfound

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
!     READSTRCONST   Check if name read corresponds to one the integer or
!                    real constants
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02expr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02expr'
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if the string corresponds to a constant

      ityprd = 4
      call readstrconst

      if ( debug ) write(irefwr,1) 'ityprd/1', ityprd

      if ( ityprd==2 .or. ityprd==3 ) then

!     --- string = constant, check if we are dealing with an expression
!         All open brackets before the keyword are also used

         lastitem = lastitembef ! decrease lastitem, since readreal
                                ! increases it by 1
         do while ( lastitem>0 .and. (typeitems(lastitem)==5 .or. &
                    typeitems(lastitem)==11) )
            lastitem = lastitem-1
         end do  ! while ( lastitem>0 .and. typeitems(lastitem)==5 )
         if ( debug ) write(irefwr,1) 'lastitem/1', lastitem
         openfound = .false.
         if ( debug ) write(irefwr,1) 'lastitem/2', lastitem
         call readreal ( openfound, .false. )
         if ( debug ) write(irefwr,2) 'realrd', realrd

         iposut = iposut+1
         tabout(iposut) = realrd
         iretur = iretur-1
         nseqkeyw = nseqkeyw+1
         iseqkeyw(nseqkeyw) = -iposut
         if ( iposut>nout ) then

!        --- iposut>nout

            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000

         end if  ! ( iposut>nout )
         finished = .true.

      end if  ! ( ityprd==2 .or. ityprd==3 )

!     --- not constant found, check if it is a variable

      call readstrscals

      if ( debug ) write(irefwr,1) 'ityprd/2', ityprd

      if ( ityprd==11 .and. &
           (itype(1)==16 .or. itype(1)==26 ) ) then

!     --- Special case of a scalar found in combination with a
!         coefficient

         table(1) = nint(realrd)
         nseqkeyw = nseqkeyw+1
         iseqkeyw(nseqkeyw) = 0
         readnx = .true.
         finished = .true.

      end if  ! ( ityprd==11 .and. ... )

1000  call erclos ( 'intx02expr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx02expr'

      end if  ! ( debug )

      end
