      subroutine intx02init ( nncode, maxcode, ihelp, icountkey, nout, &
                              pointar, numberkeyw, maxmultiple, iposut, &
                              ispecs, kposut, nseqkeyw, icount, itype )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters and perform some checks for intx02ext1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/creadinfof90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, maxcode, ihelp(maxcode), icountkey(maxcode), nout, &
              pointar(nncode), maxmultiple, numberkeyw(maxmultiple), &
              iposut, ispecs, kposut, nseqkeyw, icount, itype(*)

!     icount         o    Counter, number of keywords with itype = 8 or 9
!     icountkey      o    Array that counts the number of times a keyword has been
!                         found
!     ihelp          o    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     iposut         o    Last position in array tabout
!     ispecs         o    Help variable to save the value of ispecl
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     kposut         o    last position used in array strout
!     maxcode        i    Maximum length of array code allowed
!     maxmultiple    i    Declared size of array pointar
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     nseqkeyw       o    Number of entries in iseqkeyw
!     numberkeyw     o    Array of size maxmultiple containing the number of
!                         values stored in array valuekeyw for each type
!                         The sequence is defined by array pointar
!     pointar        o    array of size n containing the sequence numbers of
!                         all keywords with respect to arrays numberkeyw and
!                         valuekeyw
!                         if 0, no reference is necessary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    1439   Number of items more than 100
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02init' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02init'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iposut = 0
      kposut = 0
      nseqkeyw = 0
      ispecs = ispecl

      if ( nncode>maxcode ) then

!     --- Error nncode too large

         call errint ( nncode, 1 )
         call errint ( maxcode, 2 )
         call errsub ( 1439, 2, 0, 0 )
         go to 1000

      end if  ! ( nncode>maxcode )

      ihelp(1:nncode) = 0
      icountkey(1:nncode) = 0

      if ( nncode>lentable ) then

!     --- lentable too small

         call errint ( nncode, 1 )
         call errint ( lentable, 2 )
         call errchr ( 'infotable', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      if ( nout>lentabout ) then

!     --- lentabout too small

         call errint ( nout, 1 )
         call errint ( lentabout, 2 )
         call errchr ( 'infotabout', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      infotable(1:nncode) = 0
      infotabout(1:nout) = 0

      if ( ierror/=0 ) go to 1000

!     --- Initialize array pointar

      icount = 0
      do i = 1, nncode

         if ( itype(i)>=8 .and. itype(i)<=9 ) then

!        --- itype(i) = 8 or 9, hence this keyword may be present
!            more than once, raise icount and store in pointar

            icount = icount + 1
            pointar(i) = icount

         else

!        --- itype(i) # 6 or 8, hence this keyword is used only once
!            set pointar to 0

            pointar(i) = 0

         end if  ! ( itype(i)>=8 .and. itype(i)<=9 )

      end do  ! i = 1, n
      if ( debug ) then
         write(irefwr,1) 'icount', icount
         call prinin ( pointar, nncode, 'pointar' )
      end if  ! ( debug )

!     --- Check array length

      call inichkarlen ( icount, maxmultiple, 'numberkeyw' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize array numberkeyw

      numberkeyw(1:icount) = 0

1000  call erclos ( 'intx02init' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx02init'

      end if  ! ( debug )

      end
