      subroutine intx02nokeyw ( keywordfound, iretur, minusg, nout, &
                                iposut, iseqkeyw, tabout, nseqkeyw, &
                                finished, errorcode, code, codemn, &
                                nncode, itype, nmain, readnx, table )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-06-2014 extra debug
!        version  1.0  date 07-02-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Treat the situation in intx02ext1 that no keyword is found
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkeyw(*), nseqkeyw, iretur, minusg, nout, iposut, &
              errorcode, nncode, itype(*), nmain
      double precision tabout(nout), table(*)
      character (len=*) code(nncode), codemn(*)
      logical keywordfound, finished, readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     errorcode     i/o    If 0 at input an error message is given if a keyword
!                         is not recognized
!                         If>0, the subroutine does not give an error message, &
!                         but instead returns with -1
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     iposut        i/o   Position in array tabout
!     iretur        i/o   Return value of subroutine INTX02
!                         See subroutine INTX02
!     iseqkeyw      i/o   Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                         iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                         -isekeyw(i) refers to position in tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     keywordfound  i/o   If true a keyword has been found
!     minusg        i/o   Indicates if a minus sign has been found (-1) or
!                         not (1)
!     nmain          i    Number of entries in codemn
!     nncode         i    Number of entries in code
!     nout           i    Number of entries in tabout
!     nseqkeyw       i    Number of entries in iseqkeyw
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     table         i/o   Contains the numbers read for the various keywords
!     tabout        i/o   Contains series of numbers, see itype
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     INTX02EXPR     Check if string read corresponds to expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02nokeyw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02nokeyw'
         write(irefwr,*) 'keywordfound', keywordfound
         write(irefwr,1) 'iretur, lastitem', iretur, lastitem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( .not. keywordfound .and. iretur==-10000 ) then

!     --- iretur = -10000, so we must return with iretur = -10001
!         minus sign must be read again

         if ( minusg==-1 ) lastitem = lastitem-1
         lastitem = lastitembef  ! last item must be reread
         iretur = -10001
         finished = .true.

      else if ( iretur<=-10 .and. nchar<=32 ) then

!     --- iretur<=-10, check if it is an expression or constant

         if ( debug ) &
            write(irefwr,1) 'nout, iposut, nitems, lastitem', &
                             nout, iposut, nitems, lastitem

         call intx02expr ( nncode, itype, table, iretur, tabout, &
                           nout, iseqkeyw, nseqkeyw, readnx, &
                           iposut, finished )
         if ( ierror/=0 ) go to 1000
         if ( debug ) &
            write(irefwr,1) 'ityprd, minusg', ityprd, minusg

         if ( (ityprd==2 .or. ityprd==3) .and. minusg==-1 ) then

!        --- Number found, provided with minus sign

            minusg = 1
            tabout(-iseqkeyw(nseqkeyw)) = -realrd

         end if  ! ( ityprd==2 .or. ityprd==3 .and. minusg==-1 )

         if ( debug ) then
            write(irefwr,*) 'after intx02expr'
            write(irefwr,1) 'iretur, lastitem', iretur, lastitem
            write(irefwr,1) 'ityprd, nseqkeyw', ityprd, nseqkeyw
            write(irefwr,1) 'nout, iposut', nout, iposut
            write(irefwr,2) 'realrd', realrd
            if ( nseqkeyw>0 ) then
              write(irefwr,1) '-iseqkeyw(nseqkeyw)', -iseqkeyw(nseqkeyw)
              if ( iseqkeyw(nseqkeyw)<0 ) &
                 write(irefwr,2) 'tabout', tabout(-iseqkeyw(nseqkeyw))
            end if  ! ( nseqkeyw>0 )
            write(irefwr,*) 'finished', finished
         end if  ! ( debug )
         if ( ierror/=0 ) go to 1000

         if ( finished ) then

!        --- End of expression

            if ( lastitem>=nitems ) then

!           --- end of record found

               ityprd = 1

            else

!           --- There is more to read

               finished = .false.

            end if  ! ( lastitem>=nitems ) then

         else

!        --- When we reach this stage, the word is not recognized

           if ( errorcode>0 ) then
              errorcode = -1
              go to 1000  ! jump in case errorcode>0
           end if  ! ( errorcode>0 )

           call errchr ( string(1:nchar), 1 )
           do i = 1, abs(nmain)
              call errchr ( codemn(i), 1+i )
           end do
           do i = 1, nncode
              call errchr ( code(i), 1+i+abs(nmain) )
           end do
           call errsub ( 648, 0, 0, 1+abs(nmain)+nncode )
           call errcrd
           iretur = -1
           finished = .true.

         end if  ! ( finished )

      end if  ! ( iretur<=-10 )

1000  call erclos ( 'intx02nokeyw' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iretur', iretur
         write(irefwr,*) 'End intx02nokeyw'

      end if  ! ( debug )

      end
