      subroutine intxchkscal ( notfound, vectorfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-12-2014 Extra debug
!        version  1.2  date 10-04-2014 New call to readstrfunc
!        version  1.1  date 03-03-2014 Check also right-hand-side vector funcs
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if right-hand side is an expression containing only existing
!     scalars and vectors, operators and numbers and no other strings
! **********************************************************************
!
!                       KEYWORDS
!
!     vector_name
!     name
!     add
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical notfound, vectorfound

!     notfound       o    If true the right-hand side is not an expression
!     vectorfound    o    If true the right-hand side contains a vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numfun
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     numfun         if>0, string belongs to the set of functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READREALSCAL   Read integer, real or scalar
!     READSCALRHS    Check if right-hand side of equals sign corresponds to
!                    a known function that produces a scalar
!     READSTRFUNC    Check if name read corresponds to one of the known
!                    functions
!     READVECRHS     Check if right-hand side of equals sign corresponds to
!                    a known function that produces a vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxchkscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxchkscal'
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if right-hand side consists of scalars, constants and operators
!         only

      vectorfound = .false.
      notfound = .false.
      lastitem = lastitem+2

      do while (lastitem<=nitems)

         if ( debug ) &
            write(irefwr,1) 'lastitem, typeitems(lastitem)', &
                             lastitem, typeitems(lastitem)

         if ( typeitems(lastitem)==3 ) then

!        --- String found, check if it is a constant, scalar or vector

            lastitem = lastitem-1
            call readrealscal
            if ( debug ) &
               write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
            if ( ityprd==102 ) then

!           --- ityprd = 102, vector found

               vectorfound = .true.

            else if ( ityprd==2 .or. ityprd==3 .or. ityprd>=11 .and. &
                      ityprd<=14 ) then

!           --- ityprd = 2, 3, 11-14, number or scalar found
!               accepted

            else

!           --- It is possibly a function; check

               numfun = 0
               if ( lastitem<nitems-2 ) &
                  call readstrfunc ( numfun, string, nchar )
               if ( numfun==0 ) then

!              --- No function found, check if is something producing
!                  a scalar

                  call readscalrhs ( numfun )

                  if ( numfun==0 ) then

!                 --- No function found, return

                     call readvecrhs ( vectorfound )

                  end if  ! ( numfun==0 )
                  if ( vectorfound .or. numfun>0 ) exit

               end if  ! ( numfun==0 )

            end if  ! ( ityprd==102 )

         else if ( typeitems(lastitem)<=8 .or. &
                   typeitems(lastitem)==11 .or. &
                   typeitems(lastitem)==12 .or. &
                   typeitems(lastitem)==19 ) then

!        --- number, bracket, or operator found
!            accepted

         else

!        --- wrong character found
!            not accepted, jump

            exit

         end if  ! ( typeitems(lastitem)==3 )

         lastitem = lastitem+1

      end do  ! while (lastitem<=nitems)

1000  call erclos ( 'intxchkscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'notfound, vectorfound ', notfound, vectorfound
         write(irefwr,*) 'End intxchkscal'

      end if  ! ( debug )

      end
