      subroutine intxscalar ( code, ncode, nncode, ncodmx, icode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 08-12-2013 Extension with integer scalars
!        version  1.2  date 21-09-2012 Allow scalar 0 (time)
!        version  1.1  date 20-04-2012 Indexed scalars
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if one of the keywords is S or Scalar
!     In that case it might be a scalar name with number
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, ncode(nncode), ncodmx(nncode), icode
      character (len=*) code(nncode)

!     code           i    Table with keywords
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ivalrd, jcode, valmax
      logical debug, scalarfound

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ivalrd         Integer value read
!     jcode          Special value of icode corresponding to vector or scalar
!     scalarfound    If true vector or scalar is a keyword
!     valmax         Maximum value allowed for the integer read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSTRSCALS   Check if name read corresponds to one of the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxscalar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxscalar'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nncode

         scalarfound = ncode(i)==1 .and. code(i)=='S' .or. &
                       ncodmx(i)==6 .and. code(i)=='SCALAR'
         jcode = i
         if ( scalarfound ) exit

      end do  ! i = 1, nncode

      if ( debug ) then
         write(irefwr,1) 'jcode', jcode
         write(irefwr,*) 'scalarfound', scalarfound
      end if  ! ( debug )

      if ( scalarfound ) then

!     --- The keyword scalar is present

         if ( debug ) write(irefwr,*) 'scalarfound/2'
         call readstrscals
         if ( debug ) &
            write(irefwr,1) 'After readstrscals, ityprd', ityprd
         if ( ityprd==11 ) then

!        --- scalar name found

            icode = jcode
            ivalrd = nint(realrd-100000000d0)
            ivalrd = mod(ivalrd,10000)
            ivalrd = mod(ivalrd,5000)

            if ( ivalrd<=maxcls ) then

!           --- Real scalar found

               valmax = numscals
               if ( debug ) write(irefwr,1) 'ivalrd', ivalrd
               if ( ivalrd<0 .or. ivalrd>valmax ) then

!              --- Scalar number out of range

                  call errint ( ivalrd, 1 )
                  call errint ( valmax, 2 )
                  call errsub ( 2951, 2, 0, 0 )
                  call errcrd

               end if  ! ( ivalrd<=0 .or. ivalrd>numvec )

            else

!           --- Integer scalar found

               valmax = numintscals
               if ( debug ) write(irefwr,1) 'ivalrd', ivalrd
               if ( ivalrd-maxcls>valmax ) then

!              --- Scalar number out of range

                  call errint ( ivalrd, 1 )
                  call errint ( valmax, 2 )
                  call errsub ( 2951, 2, 0, 0 )
                  call errcrd

               end if  ! ( ivalrd<=0 .or. ivalrd>numvec )

            end if  ! ( ivalrd<maxcls )

         end if  ! ( ityprd==2 )

      end if  ! ( scalarfound )

1000  call erclos ( 'intxscalar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intxscalar'

      end if  ! ( debug )

      end
