      subroutine intxstorereal ( inpsep, rinsep, tabout, istrin, &
                                 iposut )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store value of real or information of the corresponding scalar
!     or expression in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     storage
!     real
!     scalar
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/creadinfof90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: istrin, iposut
      integer, intent(out) :: inpsep
      double precision, intent(in) :: tabout(*)
      double precision, intent(out) :: rinsep(*)

!     inpsep         o    Contains the integer part of the information read
!                         Possible values:
!                         0   value is 0
!                         >200000000  sequence number of expression + 200000000
!                         >100000000  sequence number of scalar + 100000000
!                         >0          sequence number in rinsep
!     iposut        i/o   Position in array tabout
!                         is raised by this subroutine
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!                         is raised if information is stored in istrin
!     rinsep        i/o   Array containing the real input for this subroutine
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxstorereal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxstorereal'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( infotabout(iposut)==0 ) then

!     --- Real found, store negative address in inpsep and value in rinsep

         rinsep(istrin) = tabout(iposut)  ! value
         inpsep = istrin                  ! address
         istrin = istrin+1                ! raise istrin

      else

!     --- expression or scalar found, store in inpsep

         inpsep = nint(tabout(iposut))

      end if  ! ( infotabout(iposut)==0 )

      iposut = iposut+1  ! raise iposut

1000  call erclos ( 'intxstorereal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intxstorereal'

      end if  ! ( debug )

      end subroutine intxstorereal
