      subroutine makef1subr
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  3.20 date 13-05-2009 Extension for shape 17
!        version  3.19 date 19-04-2008 Extension of itetra
!        version  3.18 date 11-04-2008 readsrin-->readsrin1
!        version  3.17 date 01-03-2007 Options -1019..-1021
!        version  3.16 date 29-08-2006 Raise recnr for last part
!
!   copyright (c) 1982-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  this program reads local file "file1.dat"
!  and processes the information on the standard problems
!  and fills file 1 (naming convention according to sepran.env)
!  file1 is read by probdf in init05
!  the eventual errors are reported on unit irefou (file1.out)
!      file 1  must be filled as follows:
! part 1)  records 1 up to krec2-1:  pointers for each standard problem
!          with 99<itype<maxtyp  (maximum type number) to part 2
!          For a certain type the pointer may be to another type
!          only if all the information is identical
! part 2)  records krec2,...,9 information on standard problems
!  for each type:
!    pos. 1   inunkp    number of degrees of freedom
!         2   numvsp    number of vectors of special structure
!         3   ishape    shape number of the element
!         4   nphys     physical variables of solution
!      if inunkp<0    number of degrees of freedom is constant (-inunkp)
!                       if inunkp=-1001 the number of degrees of freedom is
!                       ndim (dimension of space)
!                       if inunkp=-1002 the number of degrees of freedom is
!                       equal to the lower part of a two-tensor.
!                       Hence ndim = 1: 1,  ndim = 2: 3,  ndim = 3: 6
!                       if inunkp=-1003 the number of degrees of freedom is
!                       equal to the lower part of a two-tensor, the main
!                       diagonal excluded.
!                       Hence ndim = 1: 0,  ndim = 1: 3,  ndim = 3: 3
!                       if inunkp=-1004 the number of degrees of freedom is
!                       ndim**2
!                       if inunkp=-1005 the number of degrees of freedom is
!                       2*ndim+1
!                       if inunkp=-1006 the number of degrees of freedom is
!                       0 in case of ndim=2 and 3 in case of ndim=3
!                       if inunkp=-1007 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 6,7,8: 5
!                       ishape= 9:     1
!                       ishape= 14,16: 7
!                       ishape= 17:    1
!                       if inunkp=-1008 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 6,7,8: 5
!                       ishape= 9:     3
!                       ishape= 14,16: 7
!                       ishape= 17:    3
!                       if inunkp=-1009 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 9, 17: 0
!                       else         : 1
!                       if inunkp=-1010 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 9, 17: 0
!                       else         : 2
!                       if inunkp=-1011 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 9    : 1
!                       ishape= 17   : 0
!                       else         : 3
!                       if inunkp=-1012 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 17   : 1
!                       ishape= 14,16: 4
!                       else         : 0
!                       if inunkp=-1013 : ndim+1
!                       if inunkp=-1014 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : 1
!                       if inunkp=-1015 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : 2
!                       if inunkp=-1016 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : 3
!                       if inunkp=-1017 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : 6
!                       if inunkp=-1018 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : ndim
!                       if inunkp=-1019 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : same as -1002
!                       if inunkp=-1020 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : same as -1003
!                       if inunkp=-1021 the number of degrees of freedom
!                       depends on ishape:
!                       ishape= 7,17 : 0
!                       else         : same as -1004
!                >=0    number of degrees of freedom is not constant, &
!                       and then inunkp must be a pointer to the
!                       position in file 1 where information about the
!                       degrees of freedom can be found (computed by
!                       this program)
!    when numvsp>0:
!         4 + ivec:  inunkp corresponding to ivec th array of special
!                    structure  1<=ivec<=numvsp
!                    (same approach as for degrees of freedom of sol.)
!         5 + numvsp:   ncheld (number of possible values of icheld
!                                 in call of deriva)
!    when  ncheld>0:
!         6 + numvsp + 2 * icheld - 1:  ichder(icheld)
!         6 + numvsp + 2 * icheld - 2:  ivec(icheld)
!                  ichder=1 means vector is like the solution vector
!                         2       ivec'th vector of special structure
!                         3       vector defined per element;
!                                 then ivec is the number of degrees
!                                 of freedom
!                         4       ivec'th vector, but no averaging
!                         6       vector defined per element per node;
!                                 then ivec is the number of degrees
!                                 of freedom per node
!                         7       vector defined per integration point per node;
!                                 then ivec is the number of degrees
!                                 of freedom per node
!                         9       integer vector defined per element;
!                                 then ivec is the number of degrees
!                                 of freedom
!                         if ivec<0 then the vector is complex
!                         (see deriva)
!    followed by (if numvsp>0):
!         ichstr (stream)
!         ivec   (stream)
!                  ichstr=1 means stream function
!                         2       potential
!   when  nphys>0
!         iphys  inunkp corresponding to iphys th array of special
!                structure  1<=iphys<=nphys
!                (same approach as for degrees of freedom of sol.)
!   when any inunkp>0 then followed by
!         information of degrees of freedom per nodal point of
!         an element for solution, vectors of special sructure, &
!         and physical degrees of freedom
!    In case of global unknowns we need 4 positions:
!    1:  -numdegfd
!    2:  -1 to indicate that it concerns a global unknown
!    3:  -10 shape number
!    4:  0
!   Remark:
!   If ishape = -1 a special situation occurs. In that case inunkp>0 contains
!   a pointer to a position in file 1 with information about the number of
!   unknowns. The number of actual positions contained in that case is
!   always 10. These ten positions have the following meaning:
!   Pos. 1:  number of unknowns in the vertices of the elements
!        2:  number of unknowns in mid side points
!        3:  number of unknowns in the centroid
!        4:  number of unknowns in the centroids of faces
!        5-10:  not yet defined
! part 3)  records 10,..  Next some integer information concerning
!                         subdivision of elements is stored
! **********************************************************************
!
!                       KEYWORDS
!
!     file_1
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cstandf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,ih,ihh,numvsp,inunkp,ipos1,ipos2,ipos2n,ipw, &
              it,itype,nline, irefin,irefou, krec2, lnum,lenwrk, &
              maxtyp,ishape,ncheld,nnum,nphys,nrecnr, iunder, inpelm, &
              inum, itotalread
      parameter (lnum=1000,lenwrk=2000)
      integer ihelp(lnum), iwork(1000), iwork1(lenwrk), nnodes(17)
      character (len=80) text
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ih             Counter
!     ihelp          integer work array
!     ihh            Counter
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     inunkp         Number of degrees of freedom in isol1
!     ipos1          Position in array
!     ipos2          Position in array
!     ipos2n
!     ipw
!     irefin         Unit number to use for input
!     irefou         Unit number to use for output
!     ishape         Shape number of elements
!     it
!     itotalread     ?
!     itype          Type number of element
!     iunder         Lower bound of array
!     iwork          integer work array
!     iwork1         integer work array
!     krec2
!     lenwrk         Number of entries in array work
!     lnum           ?
!     maxtyp         Maximum number of types allowed
!     ncheld         Number of choices for derivative subroutine
!     nline
!     nnodes         Number of nodes for each ishape
!                    inpelm=nnodes(ishape+2), inpelm=nodes per element
!     nnum
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nrecnr         Number of records
!     numvsp         Number of vectors of special structure
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer init08

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FINISH         Stop SEPRAN
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIFIL         Open files
!     INIT08         Compute nunkp as function of inunkp and ndim for
!                    inunkp<-1000
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     READSRIN1      Read a series of integer numbers from the input
!     WRITFL         Write array to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!   INPUT (file=file1.dat) is required  ( free format)
!     first record: maxtyp = highest type number to be processed
!     For each type number:
!        first record:  itype, nnum
!                 with:
!                       itype: type number
!                       nnum:  if nnum>0 number of numbers that follow for
!                                          this type number
!                              if nnum<0 reference to type number -nnum with
!                                          exactly the same information)
!        second record: inunkp, numvsp, ishape, nphys
!                 with:
!                       inunkp: if inunkp<0, -number of unknowns per point
!                               -1001 means ndim unknowns per point
!                               -1002 the number of degrees of freedom is
!                                equal to the lower part of a two-tensor.
!                                Hence ndim = 1: 1,  ndim = 2: 3,  ndim = 3: 6
!                                -1003 the number of degrees of freedom is
!                                equal to the lower part of a two-tensor, &
!                                the main diagonal excluded.
!                                Hence ndim = 1: 0,  ndim = 1: 3,  ndim = 3: 3
!                                -1004 the number of degrees of freedom is
!                                ndim**2
!                                -1005 the number of degrees of freedom is
!                                2*ndim+1
!                                -1006 the number of degrees of freedom is
!                                0 if ndim<3 and 3 if ndim=3
!                               if inunkp>0, indication that the number of
!                               unknowns per point is not constant. The value
!                               of inunkp is not relevant
!                       numvsp: number of vectors of special structure
!                       ishape: shape number of element (-2<ishape<16)
!                       nphys:  number of physical unknowns
!        if numvsp>0 then:
!        next record:   inunkp  , inunkp  , + . + , inunkp
!                             1         2                 numvsp
!                       with inunkp the same meaning as before, but now for
!                       for each vector of special structure
!        next record:   ncheld
!                       number of allowed values of icheld for deriva
!        if ncheld>0 then:
!        next record:   ichder , ivec , ichder , ivec , + . + , ichder ,ivec
!                             1      1        2      2                n     n
!                       with n = ncheld
!                       and ichder and ivec the parameters for deriva
!        if numvsp>0 then:
!        next record:   ichstr, ivec for subroutine stream
!        if nphys>0 then:
!        next record:   inunkp  , inunkp  , + . + , inunkp
!                             1         2                 nphys
!                       with inunkp the same meaning as before, but now for
!                       for each physical degree of freedom
!        For each inunkp>0, in the sequence of their appearance above:
!          if  ishape>=0
!             inpelm positions with the number of unknowns per point
!             (inpelm is the number of nodes in the element)
!          if  ishape = -1
!             10 positions with the indication of the number of unknwons per
!             point as given above
!         So the minimum of input required is 5!
!   next record:  -1 -1 to indicate that end of information has been reached
!     Information of elements
!   next record:  -1 -1 to indicate that end of information has been reached
!     Comment records about type numbers
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
!    2396   Error in reading file
!    2397   Array length too small
!    2398   Incorrect end of input in file1.dat
!    2430   ipos1<nrec1*(krec2-1)
!    2431   numbers of items read is not the same as required
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data nnodes / 10,  1,    2,  3,  3,  6,  4,     9,  7, 10, 0,  0, &
                               4, 10,  8, 27, 14/
      data irefin,irefou /95,96/
      data iwork1/lenwrk*0/, ihelp/lnum*0/, iwork/1000*0/
! ======================================================================
!
!     --- Standard initialisations

      debug = .false.
      call initcb
      call initmd
      iinfin = 2000000000

!     --- Open the file 1 and error message file

      call inifil(-1)
      call inifil(4)
      call eropen ( 'makef1' )
      if ( debug ) then

!     --- debug information

         ioutp = 1

      else

!     --- No debug

         ioutp = -1

      end if  ! ( debug )

!     --- Open the file to be read , and open output file

      open ( unit=irefin, file='file1.dat' )
      open ( unit=irefou, file='file1.out' )
      irefre = irefin
      irefwr = irefou
      nline = 0

!     --- Read largest type number allowed

      ityprd = 1
      call asknxt
      maxtyp = nint(realrd)
      if ( debug ) write(irefwr,10) 'maxtype =', maxtyp
 10   format ( a, 10i6 )

      nline=nline+1
      if ( maxtyp<100 .or. maxtyp-99>lenwrk ) then

!     --- maxtyp too small or maxtyp out of range

         call errint ( maxtyp, 1 )
         call errint ( 100, 2 )
         call errint ( lenwrk+99, 3 )
         call errchr ( 'maxtyp', 1 )
         call errsub ( 1378, 3, 0, 1 )
         irefwr = irefou
         call errsub ( 1378, 3, 0, 1 )
         goto 10000
      end if

!     --- Compute number of record for part 2 of file 1

      krec2=(maxtyp-99)/nrec1 + 2

!     --- Initialize position of information for part 2, which must be stored
!         in part 1 of file 1

      ipos2  = (krec2-1)*nrec1+1
      ipos2n = ipos2+nrec1-1

!     --- Read and process information for standard problems (all types>99)

      nrecnr = krec2
      ipw    = 1
      do it = 100, maxtyp

!     --- Read type number and number of items to be read for this type

         ityprd = 1
         call readsrin1 ( ihelp, 2, inum, .false., 1000, 'type' )
         itype = ihelp(1)
         nnum = ihelp(2)
         if ( debug ) write(irefwr,10) 'itype, nnum', itype, nnum
         if ( nnum>0 ) then

!        --- nnum>0, read information

            itotalread = 0
            do while ( itotalread<nnum )
               call readsrin1 ( ihelp(itotalread+1), nnum-itotalread, &
                               inum, .false., 100000, 'contents' )
               nline=nline+1
               itotalread = itotalread+inum
            end do  ! while ( itotal<nnum )

         end if  ! ( nnum>0 ) then

!        --- Check itype

         if ( itype/=-1  .and. &
             (itype<100 .or. itype>maxtyp) ) then

!        --- itype out of range

            call errint ( itype, 1 )
            call errint ( 100, 2 )
            call errint ( maxtyp, 3 )
            call errchr ( 'itype', 1 )
            call errsub ( 1378, 3, 0, 1 )
            irefwr = irefou
            call errsub ( 1378, 3, 0, 1 )
            goto 10000

!           --- Check length of array ihelp

         else if ( nnum>lnum ) then

!        --- nnum too large

            call errint ( nnum, 1 )
            call errint ( lnum, 2 )
            call errint ( itype, 3 )
            call errchr ( 'ihelp', 1 )
            call errsub ( 2397, 3, 0, 1 )
            irefwr = irefou
            call errsub ( 2397, 3, 0, 1 )
            goto 10000

         else if (nnum==-1) then

!        --- Possible end of part 2 of the information

            if ( itype==-1 ) goto 210
            call errint ( itype, 1 )
            call errsub ( 2398, 1, 0, 0 )
            irefwr = irefou
            call errsub ( 2398, 1, 0, 0 )
            goto 10000

         else if (nnum<=-100) then

!        --- nnum<-100, copy information from type number nnum

            if ( debug ) write(irefwr,20) 'Type number', itype, &
               'is copy of type number', -nnum
 20         format ( 10(a,1x,i6,1x) )
            if (itype>100 .and. itype<maxtyp) then

!           --- Information for this type is equal to an earlier processed type

               ipos1=iwork1(-nnum-99)
               if ( ipos1<nrec1*(krec2-1) ) then

!              --- ipos1<nrec1*(krec2-1)

                  call errint ( itype, 1 )
                  call errint ( nnum, 2 )
                  call errsub ( 2430, 2, 0, 0 )
                  irefwr = irefou
                  call errsub ( 2430, 2, 0, 0 )
                  goto 10000
               else
                  iwork1(itype-99)=ipos1
                  goto 200
               end if
            end if
         end if

!        --- Check if actual record nrecnr has enough space left for nnum
!            numbers

         if ( ipw+nnum>nrec1 ) then
            if (ipw==1) then
               write(irefou,90) itype,nnum,nrec1
 90            format('Error in program makef1:'/ &
                 'for itype=',i8,' too many numbers nnum=',i8/ &
                 'while record length=',i8)
               goto 10000
             end if

!            --- Store the preceding information before processing the new one

             call writfl(nrecnr,1,nrec1,1,1,iwork)
             write(irefou,100) nrecnr,ipw-1,(iwork(i),i=1,ipw-1)
100          format('record nr=',i4,' used length=',i8, //(1x,10i6))
             do i=1,ipw-1
                iwork(i)=0
             end do
             ipw=1
             nrecnr=nrecnr+1
             ipos2  = ipos2n+1
             ipos2n = ipos2n+nrec1
          end if

!         --- Process the numbers that indicate a non-constant number of degrees
!             of freedom per nodal point

         if (nnum<4) then
            write(irefou,120) itype,nnum
120         format ('Error in program makef1:'/ &
              'for itype=',i8,' not enough numbers nnum=',i8/ &
              'is<5')
            goto 10000
         end if
         inunkp=ihelp(1)
         numvsp=ihelp(2)
         ishape=ihelp(3)
         nphys =ihelp(4)
         if ( nphys<-1000 ) nphys = init08 ( nphys, 3, 14 )
         ncheld=ihelp(5+numvsp)
         if ( debug ) then
            write(irefwr,10) 'number of unknowns', inunkp
            write(irefwr,10) 'number of vectors of special structure', &
                             numvsp
            write(irefwr,10) 'shape number', ishape
            write(irefwr,10) 'number of physical unknowns', nphys
            write(irefwr,10) 'number of types of derivatives', ncheld
         end if  ! ( debug )
         ih    =5+numvsp+2*ncheld+nphys
         if (numvsp>0) ih=ih+2
         if ( numvsp<0 .and. ishape/=-10 ) then
            write(irefou,130) itype,nnum
130         format ('Error in program makef1:'/ &
                    'for itype=',i8,' numvsp<0; numbers nnum=',i8)
            goto 10000
         end if
         if (nphys <0) then
            write(irefou,140) itype,nnum
140         format ('Error in program makef1:'/ &
                    'for itype=',i8,' nphys <0; numbers nnum=',i8)
            goto 10000
         end if
         if ( ishape/=-10 .and. ishape<-1 .or. ishape>15 .and. &
              ishape<101 .or. ishape>109 .and. ishape<134 .or. &
              ishape>134 ) then
            write(irefou,170) itype,ishape
170         format ( 'Error in program makef1:'/ &
              'for itype=',i8,' illegal mesh type =',i8 )
            goto 10000
         end if
         if ( ishape>100 ) then

!        --- ishape>100, number of nodes = ishape-100

            inpelm = ishape-100

         else if ( ishape>=-1 ) then

!        --- ishape>-1, number of nodes in nnodes

            inpelm = nnodes(ishape+2)
         else

!        --- ishape<-1, number of nodes unknown

            inpelm = 0
         end if
         if ( inunkp>=0 .and. ishape>=-1 ) then

!        --- Compute pointer to information about degrees of freedom

            ihelp(1)=ipos2+ipw-1+ih
            ih = ih+inpelm

         else if ( inunkp>=0 ) then

!        --- Error inunkp>= 0 and ishape=-10 is not permitted

            write(irefou,160) inunkp, ishape
160         format( 'Error in program makef1 '/ &
                    'inunkp>=0 and ishape<-1 not permitted'/ &
                    'inunkp = ',i8,'   ishape = ',i8 )

         end if

!        --- Compute pointer to degrees of freedom of ivec'th vector

         do i=1,numvsp
            inunkp=ihelp(4+i)
            if ( debug ) &
              write(irefwr,20) 'vector of special structure', i, &
              'inunkp', inunkp
            if (inunkp>=0 .and. ishape>=-1) then
               ihelp(4+i)=ipos2+ipw-1+ih
               ih        =ih+inpelm
            else if ( inunkp>=0 ) then

!           --- Error inunkp>= 0 and ishape=-10 is not permitted

               write(irefou,160) inunkp, ishape
            end if
         end do

!        --- Compute pointer to degrees of freedom of iphys 'th degree of
!            freedom

         ihh=5+numvsp+2*ncheld
         if (numvsp>0) ihh=ihh+2
         do i=1,nphys
            inunkp=ihelp(ihh+i)
            if ( debug ) &
              write(irefwr,20) 'physical unknown', i, &
              'inunkp', inunkp
            if (inunkp>=0) then
               ihelp(ihh+i)=ipos2+ipw-1+ih
               ih          =ih+inpelm
            end if
         end do

!        --- Check the number of numbers

         if ( nnum/=ih ) then

!        --- nnum # ih

            call errint ( itype, 1 )
            call errint ( ih, 2 )
            call errint ( nnum, 3 )
            call errsub ( 2431, 3, 0, 0 )
            irefwr = irefou
            call errsub ( 2431, 3, 0, 0 )
            goto 10000
         end if

!        --- Store ihelp in iwork and re-initialize ihelp

         do i=1,nnum
            iwork(ipw+i-1)=ihelp(i)
            ihelp(i)=0
         end do
         iwork1(itype-99)=ipos2+ipw-1
         ipw=ipw+nnum
200   end do

!     --- Store iwork1 in part 1 and (if present) iwork in part 2

210   if (ipw>1) then
         call writfl(nrecnr,1,nrec1,1,1,iwork)
         write(irefou,100) nrecnr,ipw-1,(iwork(i),i=1,ipw-1)
         do i=1,ipw-1
            iwork(i)=0
         end do
      end if
      do ih=1,krec2-1
         call writfl(ih,1,nrec1,1,1,iwork1((ih-1)*nrec1+1))
         ipw=min(nrec1,maxtyp-99-(ih-1)*nrec1)
         write(irefou,100) ih,ipw,(iwork1(i),i=1,ipw)
      end do
      close ( unit=irefin )

!     --- Read contents for common block ctyp3d and write to file 1
!         (starting from record number 10)

      open ( unit=irefin, file='ctyp3d.dat' )

!     --- Read and print first line

      read(irefin,300,err=900) text
      write(irefou,300) text
300   format ( a )

!     --- read dummy line

      read(irefin,300,err=900) text

!     --- read ityps: 24 integers

      ih = 24
      read(irefin,*,err=900) (iwork(i),i=1,ih)

!     --- read 2 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read itetra: 352 integers (used to be 292)

      iunder = ih+1
      ih = ih+352
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)

!     --- read 2 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read itypbs: 24 integers

      iunder = ih+1
      ih = ih+24
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)

!     --- read 3 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read iboun3: 142 integers

      iunder = ih+1
      ih = ih+142
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)

!     --- read 3 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read ivertx12: 4 integers

      iunder = ih+1
      ih = ih+4
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)

!     --- read 3 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read ivertx14: 8 integers

      iunder = ih+1
      ih = ih+8
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)
      nline=nline+1
      nline=nline+1
      if ( nrecnr>19 ) then
         write(irefou,305) nrecnr
305      format( 'Record number ', i3, ' is more than 19' )
         write(irefou,*) 'Process halted'
      end if
      nrecnr = 20
      write(irefou,310) nrecnr,(iwork(i),i=1,ih)
      call writfl(nrecnr,1,ih,1,1,iwork)
      close ( unit=irefin )

!     --- Read contents for common block ctypdv and write to file 1
!         (record number 21)

      open ( unit=irefin, file='ctypdv.dat' )
      read(irefin,300,err=900) text
      write(irefou,300) text

!     --- read dummy line

      read(irefin,300,err=900) text

!     --- read numpel: 20 integers

      ih = 20
      read(irefin,*,err=900) (iwork(i),i=1,ih)

!     --- read 2 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read numsel: 20 integers

      iunder = ih+1
      ih = ih+20
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)

!     --- read 2 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read ishstr: 20 integers

      iunder = ih+1
      ih = ih+20
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)

!     --- read 2 dummy lines

      read(irefin,300,err=900) text
      read(irefin,300,err=900) text

!     --- read ishsub: 206 integers

      iunder = ih+1
      ih = ih+206
      read(irefin,*,err=900) (iwork(i),i=iunder,ih)
      call inichkarlen ( ih, 1000, 'iwork' )
      if ( ierror/=0 ) go to 10000
      nline=nline+1
      nline=nline+1
      nrecnr = nrecnr+1
      write(irefou,310) nrecnr,(iwork(i),i=1,ih)
310   format(' record number',i3/(1x,10i6))
      call writfl(nrecnr,1,ih,1,1,iwork)
      goto 10000
900   call errint ( 1+nline, 1)
      call errchr ( 'file1.dat', 1 )
      call errsub ( 2396, 1, 0, 1 )
      irefwr = irefou
      call errsub ( 2396, 1, 0, 1 )
10000 call erclos ( 'makef1' )
      call finish(0)
      end
