      subroutine meshrdbf ( ibuffr, buffer, ichois, iref, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 29-05-2014 Extension of timebefore
!        version  1.0  date 21-03-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    read information of the mesh from the file indicated by iref
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), ichois, iref, kmesh(*)
      double precision buffer(*)

!     buffer i/o   Buffer array
!     ibuffr i/o   Integer buffer array in which all large data is stored
!     ichois  i    choice parameter,  possibilities:
!                 >=0    mesh is read,  Subroutine TOPOL is called with ichois
!                                         as first parameter
!                    -1    mesh is read,  Subroutine TOPOL is skipped
!                    -2    mesh is read,  Subroutine TOPOL is called
!                          Parameter ichois depends on version number of
!                          file meshoutput
!     iref    i    dataset reference number for the output
!                  when iref = 0 the standard output file is assumed
!                  when iref>=0 the file is supposed to be formatted, &
!                                 else an unformatted file is assumed
!     kmesh   o    standard sepran array containing information of the mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(10)
      double precision timebefore(2)

!     ihelp          Work array to store input from the input file
!     timebefore     Time at entrance of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'meshrdbf' )
      if ( itime>0 ) call ersettime ( timebefore )
      call msh071 ( ibuffr, buffer, ichois, iref, kmesh, ihelp )
      call erclmn ( 'meshrdbf', timebefore )
      end
