module sepmodulebuild

use sepmoduleelem
use sepmodulematr

!     --- Contains general information for subroutine build

logical :: addmat  ! indication whether information must be added to
                   ! the large matrix (true) or not (false)
logical :: addvec  ! indication whether information must be added to
                   ! the large vector (true) or not (false)
logical :: clrmat  ! indication whether the large matrix must be cleared
                   ! (true) or not (false)
logical :: clrlpv  ! indication whether last part of right-hand side must
                   ! be cleared (true) or not (false)
logical :: clrvec  ! indication whether the large vector must be cleared
                   ! (true) or not (false)
logical :: compli  ! true if the input vector is complex
logical :: complo  ! true if the output vector is complex
logical :: effbc   ! indication whether the effect of essential boundary
                   ! conditions must be taken into account or not
logical :: fillkmeshw  ! indicates if kmeshw must be filled (true) or not
logical :: matexi  ! If true the matrix exists
logical :: partad  ! indication whether all element matrices and or
                   ! vectors must be added to the large matrix or vector
                   ! (false) or not (true)
logical :: oldsol1 ! if true the vector vecold starts at position 1
logical :: skip    ! If true the building and assembly of element
                   ! matrices and vectors must be skipped for this group
logical :: skipbounels  ! if true skip all boundary elements
logical :: skipelements ! if true skip all elements of this group
logical :: skipinnels   ! if true skip all internal elements
logical :: symsubdiv    ! if true the elements are subdivided in simple linear
                        ! elements in a symmetric way
logical :: transform    ! If true a local transformation may be carried out,
                        ! if false not

integer :: iboun   ! boundary element group number
integer :: ibcmat  ! Indicates if the reaction forces are computed
integer :: ichderiv  ! parameter ichoice for subroutine deriv
integer :: ichnun  ! parameter ichoice for subroutine to0000
integer :: ichtst  ! value of ichoice for a standard problem (iype>=100)
!                    corresponding to icheld
integer :: icouno  ! number of entries in element vector (deriv)
integer :: ilocelem ! local element numbering used in flipflop array
integer :: invmtt  ! Help variable to determine invmat and ibcmat
integer :: ishapeorg ! Help parameter to store the original value of ishape
integer :: isubdv  ! Indicates if the quadratic elements must be
                   ! subdivided into linear ones (1) or not (0)
integer :: isubel  ! sub element sequence number
integer :: itypvo  ! Type of output vector
integer :: ivctst  ! value of ivec for a standard problem corresponding
                   ! to icheld
integer :: jmetd1  ! type of storage used for the large matrix
integer :: jmetd2  ! type of storage used for the large mass matrix
integer :: jnpelm  ! Number of points per element corresponding to
                   ! original element
integer :: lengpa  ! Length in integers required for first part of
                   ! matrix
integer :: lengpb  ! Length in integers required for second part of
                   ! matrix
integer :: lengpc  ! Length in integers required for first part of
                   ! mass matrix
integer :: lengpd  ! Length in integers required for second part
                   ! of mass matrix
integer :: lengrl  ! Length of real array rinput
                   ! (in double precision variables)
integer :: nsol    ! Number of degrees of freedom expressed in
                   ! reals
integer :: nsubel  ! number of sub elements an element is subdivided into
integer :: numns   ! number of element groups,including boundary
                   ! elements
integer :: numrhs  ! number of right-hand sides to be computed in
integer :: posinp  ! Position in input vector
integer :: posout  ! Position in output vector

integer, parameter :: leniinder=20  ! Length of array iinder
integer :: iinder(leniinder)  ! input array for derivative subroutine
                       ! See subroutine readdr for a description
!     iinder   i  Integer array in which the user indicates what computations
!                 must be performed by subroutine DERIV
!                 The entries of IINDER must be filled as follows
!                 1:  Indicates by which subroutine TO0100 is called
!                     Possible values:
!                     1:  DERIVA
!                     2:  DERIV
!                     3:  DERIVS
!                 2-20:  See subroutine readdr  iinder(1:19)

double precision :: sigma ! parameter sigma for boundary conditions of the
                          ! type u is constant
double precision :: valintegral ! value of integral

!     --- Declaration of arrays

integer, pointer :: flipflop(:)        ! flipflop
integer, pointer :: kprobaloc(:)       ! Part of array kproba
integer, pointer :: nodes(:)           ! kmesh part c or kprob part i

integer, allocatable :: iworktr(:)     ! array of length nusol output of to0016;
                                       ! contents:
                                       ! iworktr(i) = 0  no tranformation for
                                       ! degree of freedom i
                                       ! iworktr(i) = j  when i must be
                                       ! transformed
                                       ! j is the sequence number in kprobn
integer, pointer :: iwork(:)           ! work array of length nusol
                                       ! to add element matrices to large matrix

integer (kind=8), allocatable ::  istold(:,:)
                   ! Array of length numold x 2, in which some information
                   ! concerning the various arrays in islold is stored
                   ! Contents
                   ! istold(1,k)   Starting address of array
                   ! istold(2,k)   ipkprf for specific array

integer, allocatable :: istold1(:,:)
                   ! Array of length numold x 6, in whichsome information
                   ! concerning the various arrays in islold is stored
                   ! Contents
                   ! istold1(1,k)  Indication of the type of vector
                   !               Possible values:
                   !               0   Vector is of the same type as the
                   !                   solution vector
                   !              -1   Vector has not been filled
                   !               j   Vector has been filled and has the
                   !                   same structure as the jth vector in
                   !                   islold
                   ! istold1(2,k)  indprf for specific array
                   ! istold1(3,k)  iprob for this problem
                   ! istold1(4,k)  Problem number jprob for this array
                   ! istold1(5,k)  sequence number with respect to indprevsols
                   !               in module sepmoduleoldvecs
                   ! istold1(6,k)  mm-address for specific array

! --- temporary arrays

integer, pointer :: index1sub(:) ! array index1 in case of subdivision
integer, pointer :: icoeffarr(:)  ! integer coefficient array

double precision, pointer :: coeffarr(:) ! coefficient array
double precision, pointer :: weight(:) ! weight vector

! --- element matrices and vectors

double precision, pointer :: elemvc1(:) ! element vector 1
double precision, pointer :: elemvc2(:) ! element vector 2
double precision, pointer :: elemvc2d(:,:) ! element vector (2d)

double precision, pointer :: elemmt1(:,:) ! element matrix 1
double precision, pointer :: elemmt2(:,:) ! element matrix 2
double precision, pointer :: elemmt2d(:,:,:) ! element matrix (2d)

double precision, pointer :: elemms(:,:) ! element mass matrix

double precision, pointer :: elemwg(:) ! element weight vector

integer, pointer :: ielemvc(:)   ! integer element vector

   ! --- temporary arrays

   integer, pointer :: index3(:,:)  ! array index3
                                    ! (addresses in old solution)
   integer, pointer :: index1(:)    ! array index1 (node numbers of elements)
   integer, pointer :: index2(:)    ! array index2 (unknown numbers of elements)
   integer, pointer :: index3n(:)   ! new array index3 (local element)
   integer, pointer :: index4(:,:)  ! array index4 (addresses in index3)
   integer, pointer :: index4n(:)   ! array index4 (local element)
   integer, pointer :: index4tmp(:) ! array index4tmp (addresses in index2)
   integer, pointer :: index5(:)    ! array index5 (indexren periodical bc's)
   integer, pointer :: indexren(:)  ! array indexren (unknown numbers in matrix)
   integer, pointer :: index25(:)   ! work array
   integer, pointer :: indexh(:)    ! work array

   ! --- element matrices and vectors

   double precision, pointer :: elemvc(:) ! element vector
   double precision, pointer :: elemmt(:,:) ! element matrix

   ! --- Declaration of arrays

   double precision, pointer :: vecold(:)   ! prior solutions
   double complex, pointer :: cvecold(:)    ! prior solutions

end module sepmodulebuild
