module sepmodulecactl
   integer, parameter :: lengthmst=100

   integer :: icoor       ! With this parameter it is indicated whether the
                          ! arrays x and tang must be filled. Possible values:
                          !  0:  x nor tang are filled
                          !  1:  x is filled but tang is not filled
                          ! 10:  x is not filled but tang is filled
                          ! 11:  both x and tang are filled
   integer :: icount      ! Number of unknowns in element
   integer :: ideriv      ! Indication if derivatives must be computed (1)
                          ! or not (0)
   integer :: idim        ! number of components of the velocity vector u that
                          ! must be taken into account
   integer :: ielem       ! Element number
   integer :: ielgrp      ! Element group number
   integer :: ifirst      ! Indicator if the element subroutine is called for the
                          ! first time in a series (0) or not (1)
   integer :: igausp      ! Indication if Gauss points must be computed (1)
                          ! or not (0)
                          ! Is only applied when irule > 1
   integer :: inpelm      ! Number of nodes in element
   integer :: interp      ! Indication if the value of the basis functions in a
                          ! specific point must be computed (interpolation) (1)
                          ! or not (0)
   integer :: irelem      ! Relative element number with respect to element
                          ! group number ielgrp
   integer :: iphi        ! Indication if array phi must be filled (1)
                          ! or not (0)
                          ! Is only applied when irule > 1
   integer :: ipos        ! Position in array iuser
   integer :: irule       ! Consists of two parts intrule and interpol
                          ! according to
                          ! irule = intrule + 100*interpol, with
                          ! irule:
                          ! Indication of the type of integration rule to be
                          ! applied. Possibilities:
                          ! 1:  Newton-Cotes two-point rule
                          ! 2:  Gauss one-point rule
                          ! 3:  Gauss two-point rule
                          ! 4:  Gauss three-point rule
                          ! interpol:
                          ! Indicates how the coefficients must be interpolated
                          ! Possibilities:
                          ! 0:  Interpolation by the basis functions
                          ! 1:  Linear interpolation defined in integration
                          !     points
   integer :: isecdv      ! Not used
   integer :: istarw      ! When istarw > 0 it indicates the starting row
                          ! for some subroutines
   integer :: itype       ! Type number of element
   integer :: iweigh      ! Indication if weights and qmat must be filled or
                          ! not
                          ! iweigh consists of two parts iweigh_orig and iqmat
                          ! according to iweigh = iweigho+10*qmat
                          ! Possible values:
                          ! iweigh_orig:
                          !  0: weights must not be filled
                          !  1: weights must be filled
                          ! iqmat
                          !  0: qmat must not be filled
                          !  1: qmat must be filled by subroutine qclsys
                          !  2: qmat must be filled assuming a linear
                          !     transformation i.e. the surface elements are
                          !     in a plane
                          !  3: qmat must be filled assuming a general
                          ! transformation
   integer :: jadd        ! Indication if the matrix or vector must be added
                          ! to an already existing matrix resp element
                          ! vector (1) or not (0)
   integer :: jcart       ! Indication of the type of co-ordinates to be used
                          ! in the element.
                          ! jcart constists of two parts jcart_orig and
                          ! jtransform according to
                          ! jcart = jact_orig+10*jtransform
                          ! Possible values:
                          ! jcart_orig
                          ! 1:  Cartesian co-ordinates
                          ! 2:  Axi-symmetric co-ordinates
                          ! 3:  Polar co-ordinates
                          ! jtransform
                          ! 0:  standard, no special transformation
                          ! 1:  transformation from 3d to 2d, with z constant
                          !     first coordinate becomes x, second y
                          ! 2:  transformation from 3d to 2d, with x constant
                          !     first coordinate becomes y, second z
                          ! :  transformation from 3d to 2d, with y constant
                          !     first coordinate becomes x, second z

   integer :: jconsf      ! Information of the weight function w and the
                          ! function to be integrated g. Possibilities:
                          ! 1   g and w constant
                          ! 2   general
   integer :: jdercn      ! Indication if the derivatives of the basis
                          ! functions are constant (jdercn=0) or
                          ! variable (jdercn=1)
   integer :: jderiv      ! Indication if the value of the derivatives must be
                          !  computed.  Possibilities:
                          !  0   No derivatives
                          !  1   The gradient is computed in all integration points
                          !      in the sequence given by dudx
                          !  2   dudx in the vertices
                          !  3   dudy in the vertices
                          !  4   dudz in the vertices
                          !  5   The gradient is computed in the vertices
                          !      in the sequence given by dudx
   integer :: jdiag       ! If jdiag = 1 then the basis functions satisfy:
                          !          k                                        k
                          !  phi i (x ) = delta   for each integration point x
                          !                    ik
                          !  If jdiag = 2 they do not
   integer :: jind         ! indication of how the local vector u
                          !  must be found from usol. possibilities:
                          !  0:  the value of u in the nodal points must be
                          !      computed with the aid of array ind of
                          !      length n x ncomp
                          ! -1:  in each nodal point there are the same number
                          !      of components of u. the components ind(1),
                          !      ind(2), ... ind(ncomp) are stored
                          ! >0:  in each nodal point there are the same number
                          !      of components of u. the components jind,
                          !      jind+1,... jind+ncomp-1 must be stored in u.
   integer :: jpsi        !  If jpsi = 0 no linear basis functions are computed
                          !  if jpsi = 1, these basis functions are computed
   integer :: jsecnd      ! choice parameter. Possibilities:
                          ! 1   a12=a13=a23=0;  a11=a22=a33   a11 and w const
                          ! 2   a12=a13=a23=0;  a11=a22=a33
                          ! 3   a12#0 or a13#0 or a23#0   a11=a22=a33
                          ! 4   general
   integer :: jsol        ! Indication if the values of the local vector in the
                          ! nodal points or the integration points must be
                          ! computed (1) or not (0)
   integer :: jtrans      ! Indicates that a type of transformation must be
                          ! performed
                          ! jtrans consists of two parts jtrans_org and jinteg
                          ! in the following way:
                          ! jtrans_org:
                          !  0:  no transformation
                          !  1:  delete seventh point in case of extended
                          !      quadratic triangle
                          ! jinteg:
                          ! 0:   compute weights of integration in usual way
                          ! 1:   compute weights of integration in x-direction
                          ! 2:   compute weights of integration in y-direction
                          ! 3:   compute weights of integration in z-direction
   integer :: jzero       ! Choice parameter for subroutine el3002
                          ! Indicates the function to be integrated
                          ! Possibilities:
                          ! 1   The function b and the weights w are constant
                          ! 2   General
                          ! 3   The function b is constant, the weights w
                          !     are variable
   integer :: kstep       ! Indication if the elements must be treated normally
                          ! i.e. all integration points are stored sequentially
                          ! (1) or that a quadratic triangle with a 3-point
                          ! newton cotes rule must be considered (2)
   integer :: lrow        ! size of the element matrix
   integer :: m           ! Number of integration points
   integer :: massrule    ! integration rule for mass matrix
   integer :: metupw      !  Type of upwinding method to be chosen.
                          ! Possible values:
                          !  0:  No upwinding
                          !  1:  Classical upwind scheme
                          !  2:  Il'in scheme
                          !  3:  Doubly assymptotic scheme
                          !  4:  Critical approximation
                          !  5:  Hughes approximation
                          !  6:  Time-dependent approximation
                          ! >99: See metupw-100, however the mass matrix is not
                          !      weighted by the new basis functions
                          ! >999: See metupw-1000, however, now the
                          !       compressible variant of the convection term
                          !       is used
   integer :: mn          ! Number of positions the derivatives of the
                          ! basis functions needed for the derivative
                          ! in one direction
   integer :: mone        ! number of integration points in one dimension
   integer :: n           ! Number of nodes corresponding to type number
   integer :: ncomp       ! Number of components of the solution vector
   integer :: ndimlc      ! dimension of space, see dudx
   integer :: nelemcac    ! Number of elements in the mesh
   integer :: notmas      ! Indication if the mass matrix is equal to zero (1)
                          !  or not (0)
   integer :: notmat      ! Indicator if the element matrix is zero (1) or not (0)
   integer :: notvec      ! Indicator if the element vector is zero (1) or not (0)
   integer :: npsi        ! Number of basis functions psi
   integer :: npointcac   ! Number of nodes in the mesh
   integer :: nuser       ! Computed length of array user
   integer :: nusolcac    ! Number of degrees of freedom in the mesh
   integer :: nvert       ! Number of vertices in the element
   logical :: matrix      ! Indication if the element matrix must be computed
                          ! (true) or not (false)
   logical :: vector      ! Indication if the element vector must be computed
                          ! (true) or not (false)
   logical :: oldsol      ! Indication if array uold has been filled with a
                          ! preceding solution (true) or not (false)

   integer :: ich(lengthmst) ! In this array information concerning the
              ! coefficients is stored for the case IND(I)=3000 + IVEC
              ! The value of ich(i) has the following meaning:
              ! 1:   Array VECOLD(IVEC) has exactly one degree of freedom
              !      in all nodal points
              ! 2:   Array VECOLD(IVEC) has a constant number of degrees
              !      of freedom in all nodal points
              ! 3:   Array VECOLD(IVEC) has exactly one degree of freedom
              !      in all vertices and none in the other nodes
              !      Two-dimensional elements only
              ! 4:   Array VECOLD(IVEC) has a constant number of degrees
              !      of freedom in all vertices and none in the other nodes
              !      Two-dimensional elements only
              ! 5:   Array VECOLD(IVEC) contains the degree of freedom
              !      in all nodes
              ! 6:   Array VECOLD(IVEC) contains the degree of freedom
              !      in all vertices but not in all nodes
              !      Two-dimensional elements only
              ! 7:   Array VECOLD(IVEC) does not contain the degree of
              !      freedom in the centroid
              ! 8:   Array VECOLD(IVEC) does not contain the degree of
              !      freedom in the centroids of tetrahedrons
              ! >100:   Array VECOLD(IVEC) corresponds to a vector of
              !      special structure of type 116 with ich(i)-100 degrees
              !      of freedom per element
              ! >1000:  Array VECOLD(IVEC) corresponds to a vector of
              !      special structure of type 126 with ich(i)-1000 degrees
              !      of freedom per element per node
              ! >2000:  Array VECOLD(IVEC) corresponds to a vector of
              !      special structure of type 129 with ich(i)-20
              !      of freedom per element per integration point

   integer :: ind(lengthmst) ! In this array, of each coefficient it
              ! is stored whether the coefficient is 0 (ind(i) = 0),
              ! constant (ind(i)<0) or variable (ind(i)>0) See elusr0
              ! ind(lengthmst) has a special meaning:
              ! ind(lengthmst)=1 or 3 means the old solution has to be
              !           computed in the integration points
              ! ind(lengthmst)=2 or 3 means that array vecloc must be
              !     filled with all old solutions in the integration points
              ! ind(lengthmst)=0 means no filling takes place of old soluts

   integer :: ist(lengthmst) ! In this array it is indicated if a coefficient
              ! is a copy of preceding coefficient (>0) or not (=0)
              ! If ist(i) > 0 the value of ist(i) indicates the coefficient
              ! number from which the coefficient must be copied.

   integer :: ivecnums(lengthmst) ! In this array the the values of
              ! ivec are stored

   integer :: mst(lengthmst) ! In this array the start position of the
              ! real information of the ith coefficient in array user
              ! is stored. See elusr3
              ! If ist(i) = 3000+IVEC then mst(i) contains the degree of
              ! freedom j that must be used for this coefficient

   double precision :: alam1(3)
   double precision :: alam2(3,4)
   double precision :: alam3(3,7)
   double precision :: alam4(2)
   double precision :: alam5(3)
   double precision :: alam6(4)
   double precision :: alam7(4,4)
   double precision :: alam8(4,5)
   double precision :: alam9(4,11)
   double precision :: alam10(4,15)
   double precision :: wgs1
   double precision :: wgs2(4)
   double precision :: wgs3(7)
   double precision :: wgs5(3)
   double precision :: wgs6
   double precision :: wgs7(4)
   double precision :: wgs8(5)
   double precision :: wgs9(11)
   double precision :: wgs10(15)
!     Store weights and integration points for Gauss integration
!     Both integration on a line (1D) and on a triangle (2D)
!
!     List of available formulas:
!
!     1D (also to be used for repeated 1D):  (Interval [0,1])
!
!     1-point rule:   x = 0.5     w = 1
!     2-point rule:   xk = alam4(k)   w = 0.5
!     3-point rule:   xk = alam5(k)   wk = wgs5(k)
!
!     2D triangles (Co-ordinates to be expressed in terms of bary-centric
!                   co-ordinates)
!
!     1-point rule:   x  = alam1 (1/3)     w  = wgs1 (.5)
!     4-point rule:   xk = alam2(k)        wk = wgs2(k)
!     7-point rule:   xk = alam3(k)        wk = wgs3(k)
!
!     3D tetrahedra (Co-ordinates to be expressed in terms of bary-centric
!                    co-ordinates)
!
!     1-point rule:   x  = alam6 (1/4)     w  = wgs6 (1/6)
!     4-point rule:   xk = alam7(k)        wk = wgs7(k) (1/24)
!     5-point rule:   xk = alam8(k)        wk = wgs8(k)
!     11-point rule:  xk = alam9(k)        wk = wgs9(k)
!     15-point rule:  xk = alam10(k)       wk = wgs10(k)
!
!     alam1      Values of barycentric co-ordinates for one-point rule(2D)
!     alam2      Values of barycentric co-ordinates in integration points
!                for 4-point rule(2D)
!                                              k
!                alam2 ( i,k ) = lambda  (xgaus )
!                                      i
!     alam3      Values of barycentric co-ordinates in integration points
!                for 7-point rule(2D), see alam2
!     alam4      Values of Gauss co-ordinates for two-point rule (1D)
!     alam5      Values of Gauss co-ordinates for three-point rule (1D)
!     alam6      Values of barycentric co-ordinates for one-point rule(3D)
!     alam7      Values of barycentric co-ordinates in integration points
!                for 4-point rule(3D)
!     alam8      Values of barycentric co-ordinates in integration points
!                for 5-point rule(3D)
!     alam9      Values of barycentric co-ordinates in integration points
!                for 11-point rule(3D)
!     wgs1       Weight for one-point rule on a triangle
!     wgs2       Weights for four-point rule on a triangle
!     wgs3       Weights for seven-point rule on a triangle
!     wgs5       Weights for three-point rule on a line
!     wgs6       Weights for one-point rule on a tetrahedron
!     wgs7       Weights for four-point rule on a tetrahedron
!     wgs8       Weights for five-point rule on a tetrahedron
!     wgs9       Weights for eleven-point rule on a tetrahedron
!     wgs10      Weights for fifteen-point rule on a tetrahedron

end module sepmodulecactl

