module sepmodulekprob

   use sepmodulesol
   use sepmodulegen
   use sepmoduleprobinf

!  Some quantities for the problem definition

   integer :: icountkprob  ! counter for kprob to check whether it is new or old
   integer :: indpra       ! indication if kprob part a has been filled (>0)
                           ! or not (0) (also starting address)
   integer :: indprb       ! indication if kprob part b has been filled (>0)
                           ! or not (0) (also starting address)
   integer :: indprd       ! indication if kprob part d has been filled (>0)
                           ! or not (0) (also starting address)
   integer :: indpre       ! indication if kprob part e has been filled (>0)
                           ! or not (0) (also starting address)
   integer :: indprf       ! indication if kprob part f has been filled (>0)
                           ! or not (0)
   integer :: indprfi      ! indication if kprob part f_i has been filled (>0)
                           ! or not (0), i.e. kprobf for input vector
   integer :: indprflc     ! Local indprf
   integer :: indprfo      ! indication if kprob part f_o has been filled (>0)
                           ! or not (0), i.e. kprobf for output vector
   integer :: indprh       ! indication if kprob part h has been filled (>0)
                           ! or not (0)
   integer :: indprhi      ! Local indprh
   integer :: indprhlc     ! Local indprh
   integer :: indprho      ! Local indprh
   integer :: indpri       ! indication if KPROB part i has been filled (>0)
                           ! or not (0)
   integer :: indprj       ! indication if kprob part j has been filled (>0)
                           ! or not (0)
   integer :: indprp       ! indication if kprob part p has been filled (>0)
                           ! or not (0)
   integer :: indprpi      ! indication if kprob part p_i has been filled (>0)
                           ! or not (0), i.e. kprobp for input vector
   integer :: indprpo      ! indication if kprob part p_o has been filled (>0)
                           ! or not (0), i.e. kprobp for output vector
   integer :: indprr       ! indication if kprob part r has been filled (>0)
                           ! or not (0)
   integer :: indprx       ! Starting address of kprob part x
   integer :: indpry       ! Starting address of kprob part y
   integer :: indprz       ! indication if kprob part z has been filled (>0)
                           ! or not (0)
   integer :: indprab      ! sequence number of kprobab
   integer :: indprac      ! sequence number of kprobab
   integer :: ipstkprobpar ! Starting address of kprobpar in kprob
   integer :: iprob        ! problem number
   integer :: iskipelem    ! indicates if array skipelem is used
   integer :: kprelgrp     ! sum of number of nodes in element groups
   integer :: maxcontc     ! Largest sequence number of activated contact
   integer :: maxtrn       ! Maximum number of real variables per point with
                           ! respect to the local transformations
   integer :: nbcons       ! Number of boundary conditions of the type u
                           ! equals unknown constant
   integer :: nbound       ! Number of essential boundary conditions
   integer :: nboundi      ! local nbound
   integer :: nboundlc     ! local nbound
   integer :: ndtvec       ! Number of special structures
   integer :: nfictgrp     ! number of element groups corresponding to
                           ! fictitious unknowns
   integer :: nfictunk     ! Total number of fictitious unknowns
   integer :: nglcoupl     ! Number of free unknowns coupled to global unknowns
   integer :: nglgrp       ! number of element groups corresponding to global
                           ! unknowns
   integer :: nglunk       ! total number of global unknowns
   integer :: ninterface   ! Number of interface unknowns
   integer :: nkbnd        ! number of prescribed boundary conditions
   integer :: nkbndskip    ! number of prescribed boundary conditions
                           ! to be skipped
   integer :: npboun       ! Number of periodical boundary conditions
   integer :: npbouni      ! local npboun
   integer :: npltra       ! Number of local transformations
   integer :: nprob        ! Number of problems
   integer :: nrusol       ! Number of free unknowns
   integer :: nrusolloc    ! Local number of free unknowns (nusol-nbound)
   integer :: nrusolmax    ! Highest value of nrusol in node for petsc
   integer :: nphys        ! Number of physical unknowns, or maximal degrees of
                           ! freedom per point
   integer :: nphysi       ! Number of physical unknowns, or maximal degrees of
                           ! freedom per point (input vector)
   integer :: nphyso       ! Number of physical unknowns, or maximal degrees of
                           ! freedom per point (input vector)
   integer :: nphyslc      ! local nphys
   integer :: ntype        ! Number of different types
   integer :: numnat       ! Number of different boundary elements
   integer :: numpairs     ! number of pairs of periodical boundary conditions
   integer :: nunkp        ! Maximum number of unknowns in a point
   integer :: nunkpi       ! Maximum number of unknowns in a point
                           ! input vector
   integer :: nunkplc      ! local nunkp
   integer :: nunkpo       ! Maximum number of unknowns in a point
                           ! output vector
   integer :: nusol        ! Number of unknowns
   integer :: nusoli       ! local nusol
   integer :: nusolo       ! local nusol
   integer :: nusolglob    ! Global number of degrees of freedom

   integer, pointer :: kpiinput     ! sequence number of iinput  for kprob
   integer, pointer :: kprinput     ! sequence number of rinput  for kprob

   logical :: newstyle     ! New storage style for parallel environment
   logical :: nocallprobdf ! if false the subroutine is called by probdfext

   integer, pointer :: iinputkp(:)   ! array iinput wrt kprob
   integer, pointer :: intarprb(:)   ! See sepmoduleprobinf
   type(intarr), pointer :: kproba(:)  ! kprob part a
   integer, pointer :: kprobaa(:)    ! kprob part aa
   integer, pointer :: kprobab(:)    ! kprob part ab
   integer, pointer :: kprobac(:)    ! kprob part ac
   integer, pointer :: kprobad(:)    ! kprob part ad
   integer, pointer :: kprobatmp(:)  ! specific kprob part a
   integer, pointer :: kprobb(:)     ! kprob part b
   integer, pointer :: kprobc(:)     ! kprob part c
   integer, pointer :: kprobd(:)     ! kprob part d
   integer, pointer :: kprobe(:,:)     ! kprob part e
   integer, pointer :: kprobe2(:)    ! kprob part e last part
   integer, pointer :: kprobf(:)     ! kprob part f
   integer, pointer :: kprobfi(:)    ! kprob part f for input vector
   integer, pointer :: kprobflc(:)   ! local kprob part f
   integer, pointer :: kprobfo(:)    ! kprob part f for output vector
   integer, pointer :: kprobfh(:)    ! either kprobf, kprobfi or kprobfo
   integer, pointer :: kprobj(:,:)   ! kprob part j
   integer, pointer :: kprobg(:,:)   ! kprob part g
   integer, pointer :: kprobh(:)     ! kprob part h
   integer, pointer :: kprobhi(:)    ! kprob part h
   integer, pointer :: kprobho(:)    ! kprob part h
   integer, pointer :: kprobhinv(:)  ! kprob part h (inverse part)
   integer, pointer :: kprobhinvi(:) ! local kprob part hinv
   integer, pointer :: kprobhinvlc(:)! local kprob part hinv
   integer, pointer :: kprobhinvo(:) ! local kprob part hinv
   integer, pointer :: kprobhdd(:)   ! kprob part hdd (adapted for dd coupling)
   integer, pointer :: kprobhddinv(:) ! kprob part hdd (adapted for dd coupling)
   integer, pointer :: kprobi(:)     ! kprob part i
   integer, pointer :: kprobn(:,:)   ! kprob part n
   integer, pointer :: kprobr(:)     ! kprob part r
   integer, pointer :: kprobo(:)     ! kprob part o
   integer, pointer :: kprobp(:,:)   ! kprob part p
   integer, pointer :: kprobpi(:,:)  ! kprob part p for input vector
   integer, pointer :: kprobpo(:,:)  ! kprob part p for output vector
   integer, pointer :: kprobs(:)     ! kprob part s
   integer, pointer :: kprobvp1(:,:) ! kprob part v part 1
   integer, pointer :: kprobvp2(:)   ! kprob part v part 2
   integer, pointer :: kprobx(:)     ! kprob part x
   integer, pointer :: kproby(:)     ! kprob part y
   integer, pointer :: kprobwp1(:,:) ! kprob part w part 1
   integer, pointer :: kprobwp2(:)   ! kprob part w part 2
   integer, pointer :: kprobz(:)     ! kprob part u
   integer, pointer :: kprobfict(:,:) ! array of size nfictunk x 2 with the
                                     ! following contents
                                     ! kprobfictd(i,1) contains the structural
                                     ! element corresponding to fictitious
                                     ! unknown i
                                     ! kprobfictd(i,2) contains the fluid
                                     ! element corresponding to fictitious
                                     ! unknown i
   integer, pointer :: kprobper(:,:) ! Array of size 2 * numpairs containing the
                                     ! pairs of nodes with periodical
                                     ! boundary conditions
   integer, pointer :: kprobloc(:)   ! local array kprob referring to 1 problem

   type (probinfo), pointer :: kprobpart    ! Refers to sub problems
   type (realarr), pointer :: probzarr(:)   ! array corresponding to kprobz
   type (intarr), pointer :: kprobe3(:)     ! Contains subarrays kprobf
                                          ! corresponding to arrays of type ivec

   double precision, pointer :: coortr(:,:) ! kprob part n (real part) (rprobn)
   double precision, pointer :: rinputkp(:) ! array rinput wrt kprob
   double precision, pointer :: rprobs(:,:) ! kprob part s (real part)
   double precision, pointer :: rprobz(:) ! kprob part z (real part)

   integer, pointer :: kprobpar(:)   ! Part of kprob (start address kprob(65))
                                     ! containing extra information concerning
                                     ! parallel arrays
                                     !  Contents:
                                     !   1: nusolglob
                                     !   2: nrusolglob
                                     !   3: -
                                     !   4: kprobh dd ( special for dd parallel)
                                     !   5: kprobhglob (sepran numbering)
                                     !   6: onnz (petsc)
                                     !   7: nrusollocs (sepran numbering)
                                     !   8: nghosts (petsc)
                                     !   9: ghosts (petsc)
                                     !  10: petscrenum
                                     !  11: kprobhglob old numbering
                                     !  12: nnz (petsc)
                                     !  13: -
                                     !  14: -
                                     !  15: ninterfacdeg
                                     !  16: blockinfor
                                     !  17:
                                     !  18:
                                     !  19:
                                     !  20:
   integer, pointer :: kprobpz(:)    ! kprob part z
   integer, pointer :: skipelem(:)   ! skipelem: indicates if elements
                                     ! must be skipped (1) or not (0)

   integer, pointer :: blockinfor(:) ! Array containing information for parallel
                                     ! computing
                                     ! See subroutine probparblocks1 for a
                                     ! description
   integer, pointer :: kprobhglob(:) ! Array kprobh with global numbers of free
                                     ! degrees of freedom (length nrusol)
   integer, pointer :: ninterfdeg(:) ! contains number of free degrees of
                                     ! freedom along interface per block
                                     ! (accumulated) ninterfdeg(0) = 0
   integer, pointer :: nrusollocs(:) ! Contains largest degree of freedom per
                                     ! cpu
   integer, pointer :: nnz(:)        ! Contains number of non-zeros per row of
                                     ! the matrix, but only for the block stored
                                     ! on this cpu (petsc)
   integer, pointer :: onnz(:)       ! Contains number of non-zeros per row
                                     ! stored on other cpu's (in sepran sense)
   integer, allocatable :: kprobhglobinv(:) ! Inverse of array kprobhglob
   integer, pointer :: blockunkinfo(:)  ! See PG
   integer, pointer :: blockunkinfop(:) ! Same as blockunkinfo, but now with
                                        ! respect to the prescribed unknowns
   integer, pointer :: innercontr(:)    ! Array of size nrusol indicating if an
                          ! unknown in the  block must be used in a global inner
                          ! product or not
!                         innercontr(i) = 1: take unknown i into account
!                         innercontr(i) = 0: take unknown i not into account
!                         The unknown in the block with smallest number is taken
!                         into account
   integer, pointer :: sendinfo(:) ! Array containing information of the
!                         transport of unknowns between neighboring blocks
!                         See Programmers Guide Section 24.3 for a description
   integer, pointer :: sendinfopar(:) ! see sendinfo in parallel case
   integer, pointer :: innercontru(:) ! See innercontr with respect to velocity
   integer, pointer :: sendinfou(:)   ! See sendinfo with respect to velocity
   integer, pointer :: innercontrp(:) ! See innercontr with respect to pressure
   integer, pointer :: sendinfop(:)   ! See sendinfo with respect to pressure
   integer, pointer :: kprobhsend(:)   ! See sendinfo with respect to pressure
   type (intarr), pointer :: kprobfglobal(:)  ! see sepmoduleprobinf

   contains

      function sepgetinpelall ( ielgrp ) ! return with number of nodes per group
                                         ! boundary groups are included

      use sepmodulekmesh

      integer, intent(in) :: ielgrp
      integer :: sepgetinpelall

         if ( ielgrp>nelgrp ) then
            sepgetinpelall = kprobj(ielgrp-nelgrp,1)
         else if ( kelma>0 ) then
            sepgetinpelall = kmesha(ielgrp+1) - kmesha(ielgrp)
         else
            sepgetinpelall = npelm
         end if  ! ( kelma>0 )

      end function sepgetinpelall

      ! --- Compute number of degrees of freedom in node, as
      !     well as last position in usol before this node
      !     kprobf must be known

      function sepgetinusol ( nodp, inunkp ) result(inusol)
      implicit none
      integer, intent(in) :: nodp        ! node number
      integer, intent(out) :: inunkp     ! number of unknowns in node
      integer :: inusol                  ! position in usol before node
      if ( indprf==0 ) then

!     --- indprf = 0

         inusol = (nodp-1)*nunkp
         inunkp = nunkp

      else

!     --- indprf>0

         inusol = kprobf(nodp)
         inunkp = kprobf(nodp+1)-inusol

      end if
      end function sepgetinusol

      ! --- Compute max number of degrees of freedom in an element group
      !     for a solution vector of type 110 or 115
      !     as well as the sum of the unknowns in this group

      function sepgetinunel ( isol, ielgrp, isumnunkp ) result(nunkpmax)
      use sepmodulemain
      use sepmodulekmesh
      implicit none
      integer, intent(in) :: isol        ! solution vector
      integer, intent(in) :: ielgrp         ! element group number
      integer :: nunkpmax                   ! maximum of inunkp in group
      integer, intent(out) :: isumnunkp     ! sum over  inunkp in group


      integer :: i                          ! loop variable
      integer :: ikelm                      ! start address in kprobpa
      integer :: indpralc                   ! local value of indpra
      integer :: inpelm                     ! number of nodes in element
      integer :: inunkp                     ! number of unknowns nodes in node
      integer :: ipkpre                     ! start position kprob e
      integer :: iprobsav                   ! old value of ipron
      integer :: itypevc                    ! type of solution vector
                                            ! 110: solution vector
                                            ! 115: vector of special structure
      integer :: ivec                       ! type of vector of special
                                            ! structure
      integer :: kprelgrp                   ! length of part of kprob e

      itypevc = ks(isol)%typevec            ! get type of vector
      iprobsav = iprob                      ! save iprob
      iprob = ks(isol)%iprob                ! get problem number
      iseqkp = kprob
      kprobloc => kp(kprob)%kprobsub(iprob)%kprobloc

      if ( itypevc==110 ) then

!     --- Vector of type solution vector

         nunkpmax = kprobloc(4)       ! constant number of unknowns
         inpelm = sepgetinpelall ( ielgrp ) ! number of nodes in element
         isumnunkp = inpelm*nunkpmax

      else if ( itypevc==115 ) then

!     --- Vector of special structure

         nunkpmax = 0
         isumnunkp = 0
         if ( ielgrp<=nelgrp ) then

!        --- Only for internal elements

            ivec = ks(isol)%ivec  ! vector sequence number
            kprobe => kp(iseqkp)%kprobsub(iprob)%kprobe
            ikelm = sepgetkmesha ( ielgrp )  ! start address in kmesha
            inpelm = sepgetinpelm ( ielgrp ) ! number of nodes in element
            do i = 1, inpelm
               inunkp = kprobe(ikelm+i,ivec)
               nunkpmax = max(nunkpmax,inunkp)
               isumnunkp = isumnunkp+inunkp
            end do  ! i = 1, inpelm

         end if  ! ( ielgrp<=nelgrp )

      else

!     --- Type of vector not implemented

         call eropen ( 'sepgetinunel' )
         call errint ( ivec,1 )
         call errsub (2708, 1, 0, 0 )
         call erclos ( 'sepgetinunel' )

      end if  ! ( itypevc==110 )

      iprob = iprobsav                      ! reset iprob
      kprobloc => kp(kprob)%kprobsub(iprob)%kprobloc

      end function sepgetinunel

!     --- Compute number of elements in element vector as function
!         of jelgrp and type of vector

      function sepfillcountvc ( inpvec, jelgrp ) result(jcount)
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
      integer, intent(in) :: inpvec     ! input vector
      integer, intent(in) :: jelgrp     ! element group
      integer :: jcount                 ! length of element vector

      integer :: ishift                 ! shift for element group
      integer :: ivec                   ! type of vector of special structure
      integer :: nunkpi                 ! max number of unknowns per point
      integer :: iprobsav                   ! old value of ipron

      jcount=0
      inpelm = sepgetinpelm ( jelgrp )

      select case ( ks(inpvec)%typevec )

         case ( 110 )   !  vector is of type usol

            if ( indpra==0 ) then
               iprob = ks(inpvec)%iprob
               nunkpi = kp(iseqkp)%kprobsub(iprob)%kprobloc(4)
               jcount=nunkpi*inpelm
            else

               iprob = ks(inpvec)%iprob
               kproba => kp(iseqkp)%kprobsub(iprob)%kproba
               kprobatmp => kproba(jelgrp)%arr
               jcount = sum(kprobatmp)
            end if

         case ( 115 )   ! vector of special structure

            ivec=ks(inpvec)%ivec
            iprobsav = iprob
            iprob = ks(inpvec)%iprob
            kprobe => kp(iseqkp)%kprobsub(iprob)%kprobe
            kprobe2 => kp(iseqkp)%kprobsub(iprob)%kprobe2
            nunkpi = kprobe2(ivec)
            if ( nunkpi<0 ) then
               jcount = -nunkpi*inpelm
            else
               ishift = sepelshift ( jelgrp )
               jcount = sum(kprobe(ishift+1:ishift+inpelm,ivec))
            end if
            iprob = iprobsav

         case ( 116, 136 )  ! vector of special structure defined per element

            jcount = ks(inpvec)%n

         case ( 126 ) ! vector of special structure defined per element per node

            jcount = ks(inpvec)%n*inpelm

         case ( 129 )  ! vector of special structure is defined per element per
                       ! integration point

            jcount = ks(inpvec)%n*m

      end select  ! case ( ks(inpvec)%typevec )
      end function sepfillcountvc

!     --- Compte start address of kproba/e in kprob for one element group
!         and a specific vector

      function sepshiftkpra ( inpvec, ielgrp ) result(shiftpra)
      use sepmodulekmesh
      integer, intent(in) :: inpvec     ! input vector
      integer, intent(in) :: ielgrp     ! element group
      integer :: shiftpra               ! shift in kproba

      integer :: ivec                   ! type of vector of special structure
      integer :: inpelm                 ! number of nodes per element
      integer :: iprobsav                   ! old value of ipron

      shiftpra = max(1,indpra)
      if ( ks(inpvec)%typevec==110 ) then    ! solution vector
         if ( indpra>0 ) then
            shiftpra = sepelshift ( ielgrp )+indpra
            kprobatmp => kproba(ielgrp)%arr
         end if  ! ( indpra>0 )
      else if ( ks(inpvec)%typevec==115 ) then   ! vector of special structure
         inpelm = sepgetinpelm ( ielgrp )
         shiftpra = sepelshift ( ielgrp )
         ivec = ks(inpvec)%ivec
         iprobsav = iprob
         iprob = ks(inpvec)%iprob
         kprobe => kp(iseqkp)%kprobsub(iprob)%kprobe
         kprobatmp => kprobe(shiftpra+1:shiftpra+inpelm,ivec)
         shiftpra = shiftpra+1
         iprob = iprobsav
      end if

      end function sepshiftkpra

  !   --- List of functions
  !
  !   inpelm = sepgetinpelall ( ielgrp )     ! number of nodes per group
                                             ! includes boundary groups
  !   inusol = sepgetinusol ( nodp, inunkp ) ! computes inusol, inunkp
  !   nunkpmax = sepgetinunel ( isol, kprob, ielgrp, isumnunkp )
                              ! computes nunkmax and isumnunkp in ielgrp
  !   icount = sepfillcountvc ( inpvec, ielgrp )
                              ! number of elements in element vector
                              ! depending on input vector
  !   shiftpra = sepshiftkpra ( inpvec, ielgrp ) ! start address in kprob
                                             ! for kproba/e (1 element group)
  !   call sepactsolbuf ( isol, kprob, indprf, indprp, &
  !                       nunkplc, nphys, iprob )
  !   Activate isol and related arrays for series of sol vectors

end module sepmodulekprob
