      module sepmodulemat

      use sepmodulecomio
      use sepmodulematstr
      use sepmodulecpack

      integer :: itotit            ! total number of iterations
      integer :: nglob             ! number of global degrees of freedom
      integer :: minzerorhs        ! First non-zero component in right-hand side

      double precision, pointer :: sendwork(:)! Work array to be used to store
                                              ! quantities to be send and
                                              ! received
      double precision, pointer :: xglob(:)   ! Contains the part of the global
                                              ! solution that is required in
                                              ! this cpu

      type rmatrix

!     --- Defines a real symmetric compact matrix in SEPRAN style

         double precision, pointer :: S(:)  ! the internal matrix
         double precision, pointer :: MD(:) ! the diagonal of the preco mat
         double precision, pointer :: DLshift(:) ! diagonal for laplace shift
         integer, pointer :: diag(:)        ! position of diagonal elements
                                            ! for row-compact storage
         integer, pointer :: cols(:), rows(:)  ! columns and row definitions
         integer, pointer :: Mcols(:), Mrows(:) ! columns and row definitions
                                               ! for preconditioning matrix
         integer, pointer :: Mdiag(:)         ! position of diagonal elements
                                              ! for row-compact storage
         integer :: iseqmt ! Sequence number of matrix in array realmt
         integer :: iseqim ! Sequence number of integer matrix description
         integer :: istartsol  ! Control parameter concerning the start
                               ! of the conjugate process.
                               ! Possibilities:
                               ! 0: The process starts with vector 0
                               ! 1: The process starts with an explicitly
                               !    given start vector.
         integer :: keep                       ! See sepmodulesolve
         integer :: neq                        ! number of equations
         integer :: nnz                        ! number of non-zero elements
         integer :: rowmin                     ! lowest row number in subroutine
         integer :: rowmax                     ! highest row number in subroutine
         integer :: type_storage               ! storage tyoe
         integer :: type_prec                  ! type of preconditioning
         integer :: scaling                    ! 0 is not scaling
                                               ! 1: Only row scaling
                                               ! 2: Row and column scaling
                                               ! 3: Only column scaling
         integer :: matmultsp                  ! 0 is standard matrix vector
                                               !   multiplication
                                               ! 1: special for AL
         integer :: nblocks                    ! number of blocks
         integer, pointer :: iblocks(:)        ! blocks
         integer :: lmax                       ! second dimension of block
         double precision, pointer :: MDB(:,:) ! the diagonal of the
                                               ! block preco mat

!        --- Information with respect to projection vectors

         integer :: nproj                      ! number of projection vectors
         logical :: deflation                  ! if true deflation
         integer, pointer :: projcols(:), projrows(:)  ! def for proj vectors
         integer, pointer :: projcolsa(:), projrowsa(:)  ! def for proj vectors
         double precision, pointer :: projU(:)  ! sparse projection vectors
         double precision, pointer :: projAu(:)! sparse projection vectors Au
         double precision, pointer :: projE(:)  ! Cholesky decomposition
!                                               ! of the matrix E
         double precision, pointer :: workstorage(:), work1(:)  ! work arrays

!        --- In case of parallel computations the off-diagonal matrix
!            blocks are stored in rowsout, colsout and Aout

         integer, pointer :: colsout(:), rowsout(:)  ! columns and row
                                               ! definitions
         double precision, pointer :: Aout(:)  ! matrix elements
         integer :: nglob                      ! global number of unknowns

!        --- Information for linear solvers

         double precision :: eps      ! Local accuracy
         double precision :: eps1     ! Help variable for the termination
                                      ! criterion of the CG-process
                                      ! If eps1 = 0, the termination
                                      ! criterion is based upon the relative
                                      ! error
         double precision :: eps2     ! Help variable for the termination
                                      ! criterion of the CG-process
                                      ! If eps2 = 0, the termination
                                      ! criterion is based upon the absolute
                                      ! error
         double precision :: alammin  ! minimum eigenvalue to be used in
                                      ! termination criterion
         double precision :: adapt    ! parameter to adapt diagonal for preco
         double precision :: factor   ! factor for MILU process
         double precision :: maxlev   ! minimum value in case of constraint
                                      ! overrelaxation
         double precision :: minlev   ! minimum value in case of constraint
                                      ! overrelaxation
         double precision :: rowsum   ! Row norm of matrix
         double precision :: subsolve ! relative accuracy for subsolver
         integer :: iabserr           ! check on absolute error
         integer :: iaccuracy         ! defines how accuracy is stored
         integer :: iconstrain        ! indicates if constraints should be
                                      ! applied.
         integer :: idegfd            ! Indicates the degree of freedom that
                                      ! must be restricted
         integer :: iminimum          ! indicates how the minimum must be
                                      ! computed.
         integer :: imaximum          ! indicates how the maximum must be
                                      ! computed.
         integer :: iprint            ! print level
         integer :: irelerr           ! relative/abs error
         integer :: itotit            ! Accumulated number of iterations
         integer :: newmumps          ! if >0 the matrix structure is new
                                      ! otherwise an old structure is used
         integer :: nmax              ! maximum number of iterations
         integer :: ntreigv           ! used to estimate eigenvalues
         integer :: ntrunk            ! residual is made orthogonal to
                                      ! ntrunk vectors
         integer :: numiter           ! number of iterations executed
         integer :: itererr           ! error in case of no convergence
                                      !   0 no error
                                      !   2 error: no convergence
                                      !   4 error: maximal accuracy reached
         integer :: aterror           ! Defines what to do in case of an
                                      ! error: Possible values:
                                      !   0: stop
                                      !   1: return
         integer :: prlump            ! See sepmodulesolve
         integer :: startvec          ! Defines start vector
                                      !   0:  vector 0
                                      !   1:  given start vector.
                                      !   2:  random vector
         integer :: typesolv          ! Defines type of solver
                                      ! see inpsol(3) (read03)
         integer :: typeparsolv       ! Defines type of parallel solver
                                      !   1:  block jacobi
                                      !   2:  gcr
         integer :: ldim              ! parameter l in bicgstab(l)
         integer :: imax              ! Maximal dimension of Krylov space
         integer :: ninner            ! Number of inner iterations
         logical :: decomp            ! if true the decomposition of
                                      ! the matrix must be computed
         logical :: eistat            ! if true eisenstat preconditioning
         logical :: globshadow        ! if true the shadow vectors must be
                                      ! computed globally in the parallel
                                      ! case
         logical :: precon            ! if true standard preconditioning
         logical :: simple            ! if true the simple method is used
         logical :: symmetric         ! if true the matrix is symmetric

       ! Arrays with respect to parallel computations (N-N coupling)

         integer, pointer :: innercontr(:)
         integer, pointer :: sendinfo(:)

      end type rmatrix

!     --- Definition of boundary condition matrix S_ip

      type pmatrix

         integer :: nrow                       ! number of rows in
                                               ! internal matrix
         integer :: neq                        ! number of boundary conditions
         integer :: n                          ! size of complete vector
         integer, pointer :: cols(:), rows(:)  ! columns and row definitions
         integer, pointer :: colsp(:), rowsp(:) ! columns and row definitions
         double precision, pointer :: B(:)     ! boundary matrix
         double precision, pointer :: Bip(:)   ! boundary matrix S_ip
         double precision, pointer :: Bpi(:)   ! boundary matrix S_pi
         double precision, pointer :: Bpp(:)   ! boundary matrix S_pp

      end type pmatrix

      type matrixdiv

         double precision, pointer :: DIV(:)  ! the divergence matrix
         integer :: nu                        ! number of velocity unknowns
         integer :: np                        ! number of pressure unknowns
         integer, pointer :: cols(:), rows(:)  ! columns and row definitions
         logical :: simple = .false.

      end type matrixdiv

      type matrixgrad

         double precision, pointer :: GRAD(:) ! the gradient matrix
         integer :: nu                        ! number of velocity unknowns
         integer :: np                        ! number of pressure unknowns
         integer, pointer :: cols(:), rows(:)  ! columns and row definitions
         logical :: simple = .false.

      end type matrixgrad

!     --- Define matrix vector multiplication interface for this type

      interface operator (*)
         module procedure matvec     ! internal matrix vector multiplication
         module procedure cmatvec    ! matvec with complex vector
         module procedure dmatvec    ! matvec with D matrix in Stokes
         module procedure gmatvec    ! matvec with G matrix in Stokes
         module procedure pbmatvec    ! matvec with boundary condition matrix
      end interface

!     --- Define solution of incomplete system

      interface operator (/)
         module procedure matdiv  ! A y = x  or U y = x
         module procedure divmat  ! L y = x (Eisenstat)
      end interface

!     --- Define solution of system of linear equations

      interface operator (.div.)
         module procedure matsolve  ! A x = y
      end interface

!     --- Define matrix vector multiplication interface for
!         projection vectors
!         y = P x = A .P. x
!         y = P^T x = A .PT. x
!         y = AP x = A .AP. x
!         y = AP^T x = A .APT. x
!         y = E^-1 x = A .Einv. x
!         y = P E^-1 P^T x = A .CPROJ. x   ( coarse grid)
!         y = x - P E^-1 AP^T x = A .PROJ. x (deflation)
!         y = x - AP E^-1 P^T x = A .PROJT. x (deflation transpose)

      interface operator (.PROJ.)
         module procedure PROJ
      end interface
      interface operator (.PROJT.)
         module procedure PROJT
      end interface
      interface operator (.CPROJ.)
         module procedure CPROJ
      end interface

!     --- Define actual matrix vector multiplication

      contains

!     --- Matrix vector multiplication

      function matvec ( A, x ) result(y)  ! y = A*x

      implicit none
      type (rmatrix), intent(in) :: A     ! matrix
      double precision, intent(in) :: x(A%neq)      ! vector
      double precision :: y(A%neq)                  ! result

      if ( parallel .and. typecoupl>=1 ) then

!     --- parallel case
!         Copy xglob from other blocks and send x to other blocks

         call sepsolcopsol ( x, xglob )

!        --- Standard part matrix vector multiplication

         call sepmatvec ( A, x, y )

!        --- Extra part, due to parts of vector stored in neighbor blocks

         call sepmatoutvec ( A, xglob, y )

      else

!     --- serial case

         call sepmatvec ( A, x, y )

      end if  ! ( parallel )

      end function matvec

      function pbmatvec ( A, x ) result(y)  ! y = A*x, where A is boundary
                                           ! condition matrix

      implicit none
      type (pmatrix), intent(in) :: A          ! matrix
      double precision, intent(in) :: x(A%n)   ! vector
      double precision :: y(A%n)               ! result

      call sepmultbcond1 ( A, x, y )

      end function pbmatvec


!     --- Matrix vector multiplication

      function matsolve ( A, y ) result(x)  ! A x = y

      implicit none
      type (rmatrix), intent(in) :: A     ! matrix
      double precision, intent(in) :: y(A%neq)      ! right-hand side
      double precision :: x(A%neq)                  ! result

      call sepsolvemeth ( A, x, y )

      end function matsolve

!     --- Matrix vector multiplication

      function cmatvec ( A, x ) result(y)  ! y = A*x

      implicit none
      type (rmatrix), intent(in) :: A     ! matrix
      double complex, intent(in) :: x(A%neq)      ! vector
      double complex :: y(A%neq)                  ! result

      if ( parallel ) then

!     --- parallel case
!         Copy xglob from other blocks and send x to other blocks

         write(irefwr,*) 'Parallel not yet implemented in cmatvec'
         call ertrace
         call instop

      else

!     --- serial case

         call sepzmatvec ( A, x, y )

      end if  ! ( parallel )

      end function cmatvec

      function dmatvec ( D, u ) result(y)  ! y = D*u

      implicit none
      type (matrixdiv), intent(in) :: D     ! matrix
      double precision, intent(in) :: u(D%nu)     ! vector
      double precision :: y(D%np)                 ! result

         call sepdmatvec ( u, y )

      end function dmatvec

      function gmatvec ( G, p ) result(y)  ! y = G*p

      implicit none
      type (matrixgrad), intent(in) :: G     ! matrix
      double precision, intent(in) :: p(G%np)     ! vector
      double precision :: y(G%nu)                 ! result

         call sepgmatvec ( p, y )

      end function gmatvec

!     --- Inverse Matrix vector multiplication for a compact matrix
!         ILU

      function matdiv ( x, A ) result(y)

      implicit none
      type (rmatrix), intent(in) :: A      ! matrix
      double precision, intent(in) :: x(A%neq)      ! vector
      double precision :: y(A%neq)              ! result M y = x

      call sepmatdiv ( A, x, y )
      if ( A%nproj>0 .and. .not. A%deflation ) then

         A%workstorage = A .CPROJ. x
         y = y + A%workstorage

      end if  ! ( A%nproj>0 .and. .not. A%deflation )

      end function matdiv

      function divmat ( A, x ) result(y)

      implicit none
      type (rmatrix), intent(in) :: A      ! matrix
      double precision, intent(in) :: x(A%neq)      ! vector
      double precision :: y(A%neq)                  ! result M y = x

      call sepdivmat ( A, x, y )

      end function divmat

!     --- Norm of vector x

      function norm ( x, A )

      implicit none
      type (rmatrix), intent(in) :: A     ! matrix
      double precision, intent(in) :: x(A%neq)
      double precision :: norm
      double precision :: sum1
      integer :: i

      if ( parallel ) then

         if ( typecoupl==0 ) then ! dirichlet-dirichlet coupling
            sum1 = 0d0
            do i = 1, A%neq
               sum1 = sum1+A%innercontr(i)*x(i)**2
            end do  ! i = 1, A%neq
         else    ! neumann-neumann coupling
            sum1 = dot_product (x,x)
         end if  ! ( typecoupl==0 )
         call prparglsum ( sum1, norm, 1, SPDOUBLE )

      else   ! serial case

         norm = dot_product (x,x)

      end if  ! ( parallel )
      norm = sqrt(norm)

      end function norm

!     --- Global dot product of vectors x an y over all processors (parallel)

      function dotprod ( x, y, A )

      implicit none
      type (rmatrix), intent(in) :: A     ! matrix
      double precision, intent(in) :: x(A%neq), y(A%neq)
      double precision :: dotprod

      double precision :: sum1
      integer :: i

      if ( parallel ) then

         if ( typecoupl==0 ) then  ! dirichlet-dirichlet coupling
            sum1 = 0d0
            do i = 1, A%neq
               sum1 = sum1+A%innercontr(i)*x(i)*y(i)
            end do  ! i = 1, A%neq
         else   ! neumann-neumann coupling
            sum1 = dot_product (x,y)
         end if  ! ( typecoupl==0 )
         call prparglsum ( sum1, dotprod, 1, SPDOUBLE )

      else   ! serial case

         sum1 = dot_product (x,y)
         dotprod = sum1

      end if  ! ( parallel )

      end function dotprod

!     --- Computation of residual

      function residual ( A, x, b ) result(y)

      implicit none
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq), b(A%neq)
      double precision :: y(A%neq), work(A%neq)
      logical :: eistat
      integer :: iprec

      iprec = A%type_prec

      eistat = iprec==2 .or. iprec==4 .or. iprec==7

      y = A*x                     ! y = A x
      if ( eistat ) y = y+2d0*x   ! correction Eisenstat
      y = b-y                     ! residual

      if ( A%nproj>0 .and. A%deflation ) y =  A .projT. y
      if ( eistat ) y = A/y       ! Pre multiply residual by E^(-1)
                                  ! in case of Eisenstat

      end function residual

      function projT ( A, x ) result(y)
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: y(A%neq)

!     --- Deflation transpose:    y = x - AP E^-1 P^T x

      call sepprojT ( A, x, y )

      end function projT

      function proj ( A, x ) result(y)
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: y(A%neq)

      call sepproj ( A, x, y )

      end function proj

      function cproj ( A, x ) result(y)
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: y(A%neq)

!     --- Coarse grid  y = U E^-1 U^T x

      call sepprojc ( A, x, y )

      end function cproj

   end module sepmodulemat
