module sepmodulematstr

!  Some quantities for the matrix structure definition

   integer :: extrafillin  ! Defines the amount of extra fill-in
   integer :: iphys1       ! First physical variable
   integer :: iphys2       ! Last physical variable
   integer :: iprint       ! defines the print level
   integer :: jmethod      ! Type of storage scheme
                           ! Possible values
                           !  1    symmetric profile matrix
                           !  2    non-symmetric profile matrix
                           !  3    symmetric complex profile matrix
                           !  4    non-symmetric complex profile matrix
                           !  5    symmetric compact matrix
                           !  6    non-symmetric compact matrix
                           !  7    symmetric complex compact matrix
                           !  8    non-symmetric complex compact matrix
                           !  9    compact matrix without using the symmetry of
                           !       the structure
                           ! 20    full symmetric real matrix
                           ! 21    full non-symmetric real matrix
                           ! 22    full symmetric complex matrix
                           ! 23    full non-symmetric complex matrix
                           ! 49    hermitian complex profile matrix, no-split
                           ! 51    row compact for amg
                           ! 52    row compact for mumps (unsymmetric)
                           ! 53    row compact for mumps (symmetric)
                           ! 54    row compact for mumps (symmetric, pos def)
                           ! 55    row compact for mumps (petsc)
                           ! 56    row compact for mumps (petsc, no sepran mat)
   integer :: numdec       ! Number of unknowns that is considered to
                           ! be decoupled
   integer :: typecoupl    ! Defines type of coupling in parallel case
                           ! 0: neumann-neumann
                           ! 1: dirichlet-dirichlet

   ! --- special quantities for simple method

   integer :: nrusolpress  ! number of pressure unknowns (np)
   integer :: nrusolvel    ! number of velocity unknowns (nu)
   integer :: scheme       ! storage/renumbering scheme
                           ! 0: default
                           ! 1: profile
                           ! 2: nested_dissection
                           ! 3: minimum_degree
   integer :: sizedmat     ! size of D matrix (nd)
   integer :: sizedgmat    ! size of DG matrix (ndd)
   integer :: sizepmmat    ! size of pressure mass matrix (npm)
   integer :: sizeqmat     ! size of Q matrix (nq)
   integer :: sizedgmatpre ! size of preconditioning DG matrix ndgpre
   integer :: sizepmmatpre ! size of preconditioning PM matrix npmpre
   integer :: sizeqmatpre  ! size of preconditioning Q matrix

   logical :: fillbouncond ! If true the part related to the boundary conditions
                           ! must be filled

   integer, pointer :: decoupled(:)  ! Contains the degrees of freedom that are
                                     ! decoupled

   integer, pointer :: intdia(:)      ! Contains the positions of the diagonal
                                      ! of the large matrix
   integer, pointer :: intdiaQ(:)     ! Contains the positions of the diagonal
                                      ! of the velocity matrix Q

   integer, pointer :: icolsii(:)     ! Contains the column numbers of part ii
                                      ! of the matrix (internal unknowns)
   integer, pointer :: icolsDG(:)     ! Contains the column numbers of part ii
                                      ! of the dg matrix (simple)
   integer, pointer :: icolsDGpre(:)  ! Contains the column numbers of part ii
                                      ! of the dg preconditioning matrix
   integer, pointer :: icolsG(:)      ! Contains the column numbers of part ii
                                      ! of the gradient matrix (simple)
   integer, pointer :: icolsPM(:)     ! Contains the column numbers of part ii
                                      ! of the pressure mass matrix (simple)
   integer, pointer :: icolsPMpre(:)  ! Contains the column numbers of part ii
                                      ! of the preconditioned pressure mass
                                      ! matrix (simple)
   integer, pointer :: icolsQ(:)      ! Contains the column numbers of part ii
                                      ! of the velocity matrix (simple)
   integer, pointer :: icolsQpre(:)   ! Contains the column numbers of part ii
                                      ! of the velocity preconditioning matrix
   integer, pointer :: irowsii(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the matrix
                                      ! (accumulated, first pos. 0)
   integer, pointer :: irowsDG(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the dg
                                      ! matrix (simple)
   integer, pointer :: irowsDGpre(:)  ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the dg
                                      ! preconditioning matrix (simple)
   integer, pointer :: irowsG(:)      ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the gradient
                                      ! matrix (simple)
   integer, pointer :: irowsPM(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the pressure
                                      ! mass matrix (simple)
   integer, pointer :: irowsPMpre(:)  ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the
                                      ! preconditioned pressure mass matrix
   integer, pointer :: irowsQ(:)      ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the velocity
                                      ! matrix (simple)
   integer, pointer :: irowsQpre(:)   ! Contains the number of non-zero elements
                                      ! in the rows of part ii of the velocity
                                      ! preconditioning matrix (simple)
   integer, pointer :: icolsip(:)     ! Contains the column numbers of part ic
                                      ! of the matrix (essential boundary conds)
   integer, pointer :: irowsip(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part ip of the matrix
                                      ! (accumulated, first pos. 0)
   integer, pointer :: icolsic(:)     ! Contains the column numbers of part ip
                                      ! of the matrix (periodical boundary
                                      ! conditions)
   integer, pointer :: irowsic(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part ic of the matrix
                                      ! (accumulated, first pos. 0)
   integer, pointer :: icolsp(:)      ! Contains the column numbers of part p
                                      ! of the matrix (reaction forces)
   integer, pointer :: irowsp(:)      ! Contains the number of non-zero elements
                                      ! in the rows of part p of the matrix
                                      ! (accumulated, first pos. 0)
   integer, pointer :: icolsfi(:)     ! Contains the column numbers of part fi
                                      ! of the matrix (free-interface)
   integer, pointer :: irowsfi(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part fi of the matrix
                                      ! (accumulated, first pos. 0)
   integer, pointer :: icolsif(:)     ! Contains the column numbers of part if
                                      ! of the matrix (interface-free)
   integer, pointer :: irowsif(:)     ! Contains the number of non-zero elements
                                      ! in the rows of part if of the matrix
                                      ! (accumulated, first pos. 0)
   integer, pointer :: irowsout(:)    ! Contains the number of entries per
                                      ! column of matrix Siout
                                      ! accumulated (irowsout(0)=0)
   integer, pointer :: icolsout(:)    ! Contains the column numbers of
                                      ! matrix Siout
   integer, pointer :: storeinfo(:)   ! Array to store all information that
                                      ! is received from other blocks
                                      ! This array is filled as follows:
                                      ! Pos 1-iacnodes: starting addresses of
                                      !   all blocks in this array
                                      ! (0 means no information)
                                      ! This is followed by information received
                                      ! from all neighbor blocks with
                                      ! higher number
                                      ! This information consist of:
                                      ! number of interface unknowns (ninterf)
                                      ! ninterf global numbers of the interface
                                      ! unknowns
                                      ! array irowsfi corrected for other
                                      ! blocks, i.e. related to this block only
                                      ! Length ninterf+1 (first position 0)
                                      ! Next the corresponding array icolsfi
   integer, pointer :: sendunknowns(:) !Contains the sequence numbers of
                                      ! the internal unknowns that have to
                                      ! be send to other blocks
                                      ! (local numbering)
                                      ! Pos. 0 contains iacnodes
                                      !    (number of blocks)
                                      ! Pos. 1-iacnodes contain the start
                                      !  addresses of the information per block
                                      !  (-1)
                                      ! Hence: number of unknowns to be sent to
                                      ! block iblock = sendunknowns(iblock)- &
                                      ! sendunknowns(iblock-1) and the unknown
                                      ! numbers are stored from
                                      ! sendunknowns(iblock)+1
   integer, pointer :: recunknowns(:) ! Contains the sequence numbers of the
                                      ! internal unknowns that have to be
                                      ! received from other blocks
                                      ! (global numbering)
                                      ! Positions 2-iacnodes+1 contain start
                                      ! addresses
                                      ! If start address is 0 no information is
                                      ! received
                                      ! The start addresses may have an
                                      ! arbitrary order
                                      ! The unknown numbers are stored from
                                      ! starting address
                                      ! How many unknowns are received is
                                      ! given by subroutine inireceivempi
                                      ! Position 1 contains the number of
                                      ! neighbor blocks from
                                      ! which information is expected


end module sepmodulematstr
