module sepmodulemeshinf

use sepmodulegen
use sepmodulecons

type meshinfo           ! vectors of type kmesh
   integer :: icounter  ! sequence number
                        ! defines if array has been copied to module
   integer, pointer :: kmesh(:)
   integer, pointer :: blockinfo(:)  ! Array with global block information for
                              !  surface number isurnr
                              !  Contents:
                              !  Pos. 1  m ( number of neihgbor blocks)
                              !  Pos. 2*i   block number of i-th neighbor
                              !  Pos. 2*i+1 starting address of i-th neighbor in
                              !             array transinfo
   integer, pointer :: curveusrp(:)  ! user point numbers corresponding to
                                     ! curves
   integer, pointer :: globnodes(:)  ! contains the global node numbers of
                                     ! the nodes (parallel)
   integer, pointer :: iactobst(:)   ! Array of length numobst containing
!                         information about active obstacles
!                         iactobs(i) = 0, means obstacle not active
!                         iactobs(i)>0, means obstacle active
   integer, pointer :: icrossobst(:) ! information about the intersection of
                                     ! an obstacle with the mesh
   integer, pointer :: icurvsmsh(:)  ! information of the number of nodes in the
                                     ! curves
   integer, pointer :: icurvsobst(:,:) ! information of the active obstacles.
                                     ! The array corresponds to all curves
                                     ! separately
                                     ! icurvsobst(1,i) = 1 means that the
                                     ! corresponding point is active;
                                     ! 0 means not active
   integer, pointer :: icurvsobst2d(:,:) ! 2d version of icurvsobst
   integer, pointer :: ielemobst(:)  ! information about the elements in an
                                     ! obstacle
                                     ! See PG Section 24.2 for a description
   integer, pointer :: ielgrp_nodes(:)  ! contains all nodes corresponding
                                     ! to element groups
   integer, pointer :: iinputkm(:)   ! iinput
   integer, pointer :: intprop(:,:)  ! integer properties
   integer, pointer :: iobgrid3d(:,:,:,:)    ! In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin,
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
   integer, pointer :: iobscurv(:)   ! Array of length ncurvs indicating if
!                     a curve is on an obstacle or not
!                     iobscurv(i) = 0 means curve i not on an obstacle
!                                   1 points of curve i on an obstacle
!                                   2 only begin and end point on obstacle
!                                   3 only begin point on obstacle
!                                   4 only end point on obstacle! information
!                                     of the active obstacles.
   integer, pointer :: iouterbounseq(:) !kmesh part al
!                    Contains outer boundaries corresponding to a selected
!                    set of element groups
   integer, pointer :: ipointobst(:) ! information about the nodes in an
                                     ! obstacle
                                     ! See PG Section 24.2 for a description
   integer, pointer :: ipointobstpart(:) ! part of array ipointobst
   integer, pointer :: kmesha(:)     ! kmesh part a
   integer, pointer :: kmeshac(:)    ! kmesh part ac
   integer, pointer :: kmeshad(:)    ! kmesh part ad
   integer, pointer :: kmeshb(:)     ! kmesh part b
   integer, pointer :: kmeshc(:)     ! kmesh part c
   integer, pointer :: kmeshc2d(:,:) ! kmesh part c (2d)
   integer, pointer :: kmeshcglob(:) ! global kmesh part c (parallel)
   integer, pointer :: kmeshcobst(:,:) ! kmesh part c corresponding to obstacle
   integer, pointer :: kmeshcpoint(:)  ! contains pointers to elements in kmeshc
   integer, pointer :: kmeshd(:)     ! kmesh part d
   integer, pointer :: kmshd1(:)     ! kmesh part d1
   integer, pointer :: kmshd2(:)     ! kmesh part d2
   integer, pointer :: kmeshe(:)     ! kmesh part e
   integer, pointer :: kmshe1(:)     ! kmesh part e1
   integer, pointer :: kmshe2(:)     ! kmesh part e2
   integer, pointer :: kmeshf(:)     ! kmesh part f
   integer, pointer :: kmeshg(:)     ! kmesh part g
   integer, pointer :: kmeshh(:,:)   ! kmesh part h
   integer, pointer :: kmeshj(:)     ! kmesh part j
   integer, pointer :: kmeshk(:)     ! kmesh part k
   integer, pointer :: kmeshl(:)     ! kmesh part l
   integer, pointer :: kmeshlglob(:) ! global kmesh part l (parallel)
   integer, pointer :: kmeshm(:)     ! kmesh part m
   integer, pointer :: kmeshmglob(:) ! global array kmesh part m (parallel)
   integer, pointer :: kmeshn(:)     ! kmesh part n
   integer, pointer :: kmeshnglob(:) ! global kmesh part n (parallel)
   integer, pointer :: kmesho(:)     ! kmesh part o
   integer, pointer :: kmsho1(:)     ! kmesh part o part 1
   integer, pointer :: kmsho2(:)     ! kmesh part o part 2
   integer, pointer :: kmeshpar(:)   ! Array of size 10 in which information of
                      !  the global mesh in case of parallel computing is stored
                      !  Contents:
                      !  1: npoint_global
                      !  2: size of coor_global
                      !  3: size of kmeshl_global
                      !  4: size of kmeshm_global
                      !  5: size of kmeshn_global
                      !  6: size of mapping array mappingpar
   integer, pointer :: kmeshq(:)     ! kmesh part q
   integer, pointer :: kmeshr(:)     ! kmesh part r
   integer, pointer :: kmeshs(:)     ! kmesh part s
   integer, pointer :: kmeshs2(:,:)  ! kmesh part s part 2
   integer, pointer :: kmesht(:,:)   ! kmesh part t
   integer, pointer :: kmeshu(:)     ! kmesh part u
   integer, pointer :: kmeshv(:)     ! kmesh part v
   integer, pointer :: kmshv1(:)     ! kmesh part v1
   integer, pointer :: kmshv2(:)     ! kmesh part v2
   integer, pointer :: kmeshw(:)     ! kmesh part w
   integer, pointer :: kmeshx(:,:)   ! kmesh part x
   integer, pointer :: kmeshxsur(:)  ! surfaces corresponding to obstacle
   integer, pointer :: kmeshz(:)     ! kmesh part z
   integer, pointer :: levelstruc(:) ! contains series of levels
   integer, pointer :: mappingpar(:) ! mapping array (parallel)
   integer, pointer :: mappingparcp(:) ! copy of mappingpar
   integer, pointer :: nmesh(:)      ! nmesh
   integer, pointer :: nmesha(:)     ! nmesh part a
   integer, pointer :: nmeshb(:)     ! nmesh part a
   integer, pointer :: nmeshc(:)     ! nmesh part a
   integer, pointer :: nmeshd(:)     ! nmesh part a
   integer, pointer :: nmeshe(:)     ! nmesh part a
   integer, pointer :: nmeshf(:)     ! nmesh part a
   integer, pointer :: nmeshg(:)     ! nmesh part a
   integer, pointer :: nmeshh(:)     ! nmesh part a
   integer, pointer :: nmeshi(:)     ! nmesh part a
   integer, pointer :: nmeshj(:)     ! nmesh part a
   integer, pointer :: nmeshk(:)     ! nmesh part a
   integer, pointer :: nmeshl(:)     ! nmesh part a
   integer, pointer :: node_elgrps(:) ! contains all element groups to
                                      ! nodes
   integer, pointer :: obstfaces(:)  ! edges or faces of the boundary of an
                                     ! obstacle
   integer, pointer :: outerboun(:)  ! outer boundary
   integer, pointer :: outercurves(:)  ! outer curves
   integer, pointer :: pointnel(:)  ! length nelgrp+1
                                    ! pointnel(i) contains number of elements
                                    ! in element groups 1 to i-1
   integer, pointer :: pointkmsc(:) ! length nelgrp+1
                                    ! pointkmsc(i) contains start address
                                    ! of element group i in kmeshc (minus 1)
   integer, pointer :: pointlevel(:) ! pointers to levelstruc
   integer, pointer :: rennested(:) ! renumbering for nested dissection
   integer, pointer :: surfnodes(:) ! Contains the node numbers corresponding
!                         to the surface
!                         Pos. 1 ... nsurfs starting addresses to part 2
!                         Part2: for each surface:
!                         number of nodes followed by node numbers
   integer, pointer :: transinfo(:)  ! contains information of the transport of
                       ! unknowns between neighboring blocks
                       ! The starting address of each neighbor block is
                       ! stored in array blockinfo
                       ! For each neighbor block the following information
                       ! is stored
                       ! Pos. 1 number of nodes n on common side of both blocks
                       ! Pos. 2 ... n+1    local node numbers in block i
                       ! Pos. n+2 ... 2n+1 local node numbers in neighbor
                       !                   block j
   integer, pointer :: volume_surfs(:)  ! contains all sufaces corresponding
                                        ! to the volumes

   double precision, pointer :: coor(:,:)     ! coordinates
   double precision, pointer :: coorglob(:,:) ! global coordinates (parallel)
   double precision, pointer :: coorobst(:,:) ! coordinates of obstacle
   double precision, pointer :: curvesmsh(:,:) ! coordinates of the nodes in the
                                              ! curves
   double precision, pointer :: curvefacts(:) ! factors t along the curves
   double precision, pointer :: elsize(:,:)   ! 2d version of kmeshp
!                         elsize(1,i) element length of first element of curve i
!                         elsize(2,i) element length of last element of curve i
   double precision, pointer :: kmeshquant(:) ! contains some mesh quantities
                                              ! At this moment
                                              ! Pos 1:2  xmin, xmax
                                              ! Pos 3:4  ymin, ymax
                                              ! Pos 5:6  zmin, zmax
   double precision, pointer :: normal(:,:)   ! Contains the normals in the
!                         nodes of the surfaces in exactly the same
!                         sequence as in array surfnodes
   double precision, pointer :: realprop(:,:) ! real properties
   double precision, pointer :: rcrossobst(:) ! information about the
                                              ! intersection of an obstacle
                                              ! with the mesh
   double precision, pointer :: rinputkm(:)   ! rinput
   double precision, pointer :: rmeshac(:,:)  ! real part of kmeshac
   double precision, pointer :: rmeshx(:,:)   ! real part of kmesh part x

   type(intarr), pointer :: mappar(:) ! contains mapping arrays for parallel
                                      ! computing

! Temporary

   integer, pointer :: ichangeobst  ! if 1 a boundary has been changed
   integer, pointer :: ipstkmeshpar ! starting address of kmeshpar in kmesh
   integer, pointer :: kelmaa       ! indicates if nmesh has been created
   integer, pointer :: kelmac       ! memory management reference number
                                    ! of KMESH part ac
   integer, pointer :: kelmad       ! memory management reference number
                                    ! of KMESH part ad
   integer, pointer :: kelmae       ! memory management reference number
                                    ! of KMESH part ae
   integer, pointer :: kelmaf       ! memory management reference number
                                    ! of KMESH part af
   integer, pointer :: kelmag       ! memory management reference number
                                    ! of KMESH part ag
   integer, pointer :: kelmak       ! memory management reference number
                                    ! of KMESH part ak
   integer, pointer :: kelmal       ! memory management reference number
                                    ! of KMESH part al
   integer, pointer :: kelman       ! memory management reference number
                                    ! of KMESH part an
   integer, pointer :: kelmc        ! Indicates if kmesha has been filled
   integer, pointer :: kelmh        ! Indicates if icurvsobst has been filled
   integer, pointer :: kelmi        ! Indicates if kmeshi has been filled
   integer, pointer :: kelmj        ! Indicates if kmeshj has been filled
   integer, pointer :: kelmk        ! Indicates if kmeshk has been filled
   integer, pointer :: kelml        ! Indicates if kmeshl has been filled
   integer, pointer :: kelmm        ! Indicates if kmeshm has been filled
   integer, pointer :: kelmn        ! Indicates if kmeshn has been filled
   integer, pointer :: kelmo        ! Indicates if kmesho has been filled
   integer, pointer :: kelmp        ! Indicates if elsize has been filled
   integer, pointer :: kelmq        ! Indicates if kmeshq has been filled
   integer, pointer :: kelmr        ! Indicates if kmeshr has been filled
   integer, pointer :: kelms        ! Indicates if kmeshs has been filled
   integer, pointer :: kelmt        ! Indicates if kmesht has been filled
   integer, pointer :: kelmu        ! Indicates if kmeshu has been filled
   integer, pointer :: kelmv        ! Indicates if kmeshv has been filled
   integer, pointer :: kelmw        ! Indicates if kmeshw has been filled
   integer, pointer :: kelmx        ! Indicates if kmeshx has been filled
   integer, pointer :: kelmxrl      ! Indicates if rmeshx has been filled
   integer, pointer :: kelmz        ! Indicates if kmeshz has been filled

   integer, pointer :: kmiinput     ! sequence number of iinput  for kmesh
   integer, pointer :: kmrinput     ! sequence number of rinput  for kmesh
   integer, pointer :: meshadapt    ! if 0 the mesh has not been adapted
   integer, pointer :: mmnormal     ! Refers to array normal
   integer, pointer :: mmouter      ! refers to outerboundary
   integer, pointer :: mmsurfnodes  ! Refers to array surfnodes
   integer, pointer :: nlevel       ! number of levels

   type (kmeshobstacle), pointer :: kmobst(:)
   type (kmeshhsur), pointer :: kmhsur(:)

end type meshinfo

type kmeshobstacle

!  --- Information of the obstacles

   integer, pointer :: iobgrid2d1(:)
   integer, pointer :: iobgrid2d(:,:)
   integer, pointer :: iobgrid3d(:,:,:,:)

!  --- Only for 3d: coordinates and topology of surface

   integer, pointer :: kmeshcobst(:,:)
   double precision, pointer :: coorobst(:,:)

end type kmeshobstacle

type kmeshhsur

!  --- Information of surfaces as used to be stored in kmesh part h
!      volumes get volume number + nsurfs

       logical :: filled           ! if true the surface has been filled
       integer, pointer :: kmeshcsur(:)     ! local topology in surface
       integer, pointer :: maparsur(:)      ! global numbers of nodes in surface
       integer, pointer :: kbndptsur(:)     ! node numbers of boundary
       integer, pointer :: renumsur(:)      !
       double precision, pointer :: coorsur(:,:)  ! local coordinates of nodes
                                            ! in surface

end type kmeshhsur

integer :: iseqkm                           ! Sequence number of mesh array
integer :: lastmesh                         ! last position used in km
integer :: nextfreekm                       ! next free position in km

type (meshinfo) :: km(nummesh)              ! refers to set of mesh arrays
integer :: mesharr(nummesh)                 ! indicates if km has been
                                            ! filled:
                                            ! 0: km(i) is empty
                                            ! >0: km(i) has been filled

! Explanation of some arrays
!
! blockinfo      Array with global block information for surface
!                    number isurnr
!                    Contents:
!                    Pos. 1  m ( number of neihgbour blocks)
!                    Pos. 2*i   block number of i-th neighbour
!                    Pos. 2*i+1 starting address of i-th neighbour in
!                               array transinfo
!                    Length: 2*m+1
! globnodes      contains global node numbers of the nodes (parallel)
!                size: npoint_global + nblocks
!                The first nblocks positions contain the highest node number
!                used in part 2 for that block
!                Nodes in block 1 get number 1-npointloc
!                Nodes in block i not in blocks 1 to i-1 get new node numbers
!                So: suppose we have 2 blocks
!                Pos 1 of part 1 contains npointloc
!                Pos 2 of part 1 contains npoint_global
!                The points in block 1 get numbers 1 to globnodes(1)
!                These numbers are stored in part 2 on the positions
!                corresponding to the global nodes
!                The points in block 2 that are not part of block 1 get
!                the numbers globnodes(1)+1 to globnodes(2) in the sequence
!                of their local node numbers
!                These numbers are also stored in part 2
! kmeshj         the new numbering of the nodal points, when renumbering is
!                performed. Hence the first node to be used is KMESHJ(1),
!                the second one KMESHJ(2) and so on.
! levelstruc     Contains node numbers of all levels sequentially
!                Hence first all nodes of levelset 1 then of set 2
!                and so on
!                Starting addresses of each level can be found in pointlevel
!                Maximal length npoint
! mappar         contains the global node numbers as defined in
!                meshoutput_par.000
!                Array of size 0:iacnodes (number of blocks)
!                At the host (node 1), all subarrays are stored
!                At each other node inode, only mappar(inode) is filled
!                mappar(0)%arr contains the number of points at each block
!                (Only at the host)
! mappingpar     contains the global node numbers as defined in globnodes
!                for each local node
!                size: npointloc
! pointlevel     Contains starting addresses of each levelset in array
!                levelstruc
!                First position 1, last position: length of array
!                levelstruc+1
!                Size: nlevels+1 (<=npoint+1)
! rennested      See kmeshj, but now for nested dissection

end module sepmodulemeshinf
