module sepmodulepost

!  Contains variables and constants for post processing

   use sepmodulecons
   use sepmodulegen

   integer :: isol(maxsoluts)          ! temporary

   integer :: dcolrf                   ! Reference constant replacing the
                                       ! default value of dcolor
   integer :: iaxisr                   ! Reference constant replacing the
                                       ! default value of iaxis
   integer :: iboldr                   ! Reference constant replacing the
                                       ! default value of ibold
   integer :: ibounr                   ! Reference constant replacing the
                                       ! default value of ibounds
   integer :: ichels                   ! Choice parameter for subroutine STREAM
   integer :: ichint                   ! Choice parameter for subroutine INTP2D
   integer :: iconstpst(105)           ! store integer constants
   integer :: icontr                   ! Reference constant replacing the
                                       ! default value of icontr
   integer :: icurvs(101)              ! store information of curves
                                       ! or surfaces
   integer :: ielemr                   ! Reference constant replacing the
                                       ! default value of ielempl
   integer :: igroup(100)              ! used in case of skip groups
   integer :: iinputpst(1000)          ! input array for post processing
   integer :: iltime(3)                ! initial actual time, the end actual time
                                       ! and the time-step in integers
   integer :: imapsol(maxvc1,ntimvc)   ! mapping from 2d to 1d vectors in isol
   integer :: imarkr                   ! Reference constant replacing the
                                       ! default value of imark
   integer :: indcol(105)              ! color numbers for the
                                       ! first 100 colors from position 6
   integer :: indmsh(nummesh)          ! flags to indicate which meshes
                                       ! are active are stored
                                       ! 0, means mesh i is not active
                                       ! 1, means mesh i is active
   integer :: infocomponents(maxvc1,2) ! information of components of vectors
   integer :: ipost(10)                ! Special constants
   integer :: innerr                   ! Reference constant replacing the
                                       ! default value of inner
   integer :: inoder                   ! Reference constant replacing the
                                       ! default value of nodepl
   integer :: irefar(0:1000)           ! referencebetween the user numbers
                                       ! for the vectors and the actual numbers
   integer :: irotrf                   ! Reference constant replacing the
                                       ! default value of irotat
   integer :: isigxr                   ! Reference constant replacing the
                                       ! default value of isigx
   integer :: isigyr                   ! Reference constant replacing the
                                       ! default value of isigy
   integer :: isymrf                   ! Reference constant replacing the
                                       ! default value of ISYMBOL
   integer :: iskip                    ! Indication if element groups must be
                                       ! skipped (1) or not (0)
   integer :: ispec(maxvc1,4)          ! information of arrays
   integer :: jmaxrf                   ! Reference constant replacing the
                                       ! default value of jmax
   integer :: itimst                   ! Actual time step
   integer :: jpoint                   ! Nodal point number in which the stream
                                       ! function is given
   integer :: jtimst                   ! Help variable to store jtimes
                                       ! temporarily
   integer :: kiesxr                   ! Reference constant replacing the
                                       ! default value of kiesx
   integer :: kiesyr                   ! Reference constant replacing the
                                       ! default value of kiesy
   integer :: lastmapsol               ! Last entry in isol that has been used
   integer :: maxclr                   ! Reference constant replacing the
                                       ! default value of max colour
   integer :: minclr                   ! Reference constant replacing the
                                       ! default value of min colour
   integer :: nstxrf                   ! Reference constant replacing the
                                       ! default value of nstepx
   integer :: nstyrf                   ! Reference constant replacing the
                                       ! default value of nstepy
   integer :: ntext                    ! Number of characters to be plotted
                                       ! at the bottom of the plot
   integer :: ntextx                   ! Number of characters to be plotted
                                       ! along the x-axis
   integer :: ntexty                   ! Number of characters to be plotted
                                       ! along the y-axis
   integer :: ntime                    ! Number of time steps
   integer :: numfuncs                 ! Largest sequence number of vectors
                                       ! of type 117 referred to
                                       ! in ifuncs that are active
   integer :: numint(maxvc1)           ! Parameter number for subroutine INTP2D
                                       ! numint(ivec) refers to the ivec th
                                       ! array
   double precision :: constpst(10)    ! store real constants
   double precision :: contln(105)     ! contour levels as required by the
                                       ! PLOTC1 or PLOTC2. The levels are filled
                                       ! from position 6.
   double precision :: funcx(ntimvc+5) ! x-function values for subroutine PLOTFN
   double precision :: funcy(ntimvc+5) ! y-function values for subroutine PLOTFN
   double precision :: getal(100)      ! numbers read at input
   double precision :: heigrf          ! Reference value for HEIGHT
   double precision :: plotrf          ! Reference value for PLOTFM
   double precision :: psiphi          ! Value of stream function in jpoint
   double precision :: refvrf          ! Reference value for REFVAL
   double precision :: rinputpst(100)  ! input array for post processing
   double precision :: rpost(10)       ! Special constants
   double precision :: time(ntimvc)    ! actual time at the
                                       ! various time steps
   double precision :: valint(20,maxvc1) ! stores information for subroutine
                                       ! INTP2D
                                       ! valint(.,ivec) refers to the ivec th
                                       ! array
   double precision :: xeyerf          ! Reference x-value for eye point
   double precision :: xmaxrf          ! Reference value for xmax
   double precision :: xminrf          ! Reference value for xmin
   double precision :: xprorf          ! Reference value for x-projection point
   double precision :: yeyerf          ! Reference y-value for eye point
   double precision :: yfacrf          ! Reference value for YFACT
   double precision :: ymaxrf          ! Reference value for ymax
   double precision :: yminrf          ! Reference value for ymin
   double precision :: xorgid          ! X-origin of identification in cm's
   double precision :: yorgid          ! Y-origin of identification in cm's
   double precision :: yprorf          ! Reference value for y-projection point
   double precision :: values(10)      ! information for subroutine PLOTLN
   double precision :: zeyerf          ! Reference z-value for eye point
   double precision :: zmaxrf          ! Reference value for zmax
   double precision :: zminrf          ! Reference value for zmin
   double precision :: zprorf          ! Reference value for z-projection point

   type (realarr) :: ifuncs(0:maxfuncs) !  contains information  of the
                                       ! time-dependent quantities that are
                                       ! defined for each time step for one
                                       ! specific point
   integer, pointer :: iskiparr(:)

!   description

!     ifuncs         information  of the time-dependent quantities that are
!                    defined for each time step for one specific point
!                    ifuncs(0) refers to x-coordinates
!                    ifuncs(i) (i>0) to the function values
!                    Position 1 to 5 refer to standard information of vectors
!                    of type 117
!     igroup         Array igroup is used if element groups must be skipped
!                    The positions 1 to IINPUT(32) contain the element groups
!                    that must be skipped
!                    In case IINPUT(85)>0 it contains the volume numbers
!                    that must be used
!                    In case of particle tracing it contains the vector numbers
!                    that must be used
!     iinputout      Contains information read with respect to output written
!                    to files like sepcomp.out
!     iinputpst      Contains the actual integer information from
!                    the standard input file
!                    Length of the array: 100
!                    Array IINPUT is filled as follows:
!                    Pos.  1:  Type of SEPRAN subroutine the input refers to
!                              See below for the various possibilities
!                    Pos.  2:  Array number of the vector to be processed
!                              In case of the mach number computation it is the
!                              velocity vector sequence number
!                    Pos.  3:  Second array number of the vector to be processed
!                              In case of the mach number computation it is the
!                              enthalpy vector sequence number
!                    Pos.  4:  Parameter LAYOUT as present in SEPRAN output subr
!                    Pos.  5:  Parameter ICHOIS as present in SEPRAN output subr
!                    Pos.  6:  Parameter NCNTLN or NCOLOUR or NLEVEL as present
!                              in SEPRAN output subr.
!                    Pos.  7:  Parameter JSMOOT as present in SEPRAN output subr
!                    Pos.  8:  Parameter IDGFD1, NUMBER or IDEGFD as present
!                              in SEPRAN output subr.
!                    Pos.  9:  Parameter IDGFD2 as present in SEPRAN output subr
!                    Pos. 10:  Parameter JDIM as present in SEPRAN output
!                              subroutine
!                    Pos. 11:  Indication if the user has given the region to be
!                              plotted or printed (1) or not (0)
!                    Pos. 12:  Indication if the user has given the minimum
!                              contour level (1) or not (0)
!                    Pos. 13:  Indication if the user has given the maximum
!                              contour level (1) or not (0)
!                    Pos. 14:  Indication if the user has given the
!                              contour levels (1) or not (0)
!                    Pos. 15:  Indication if the quantity to be computed must be
!                              computed along a line (1) or along curves (2)
!                    Pos. 16:  Vector number for new function to be created
!                    Pos. 17:  Jpoint for stream function
!                    Pos. 18:  Lower curve number
!                    Pos. 19:  Upper curve number
!                    Pos. 20:  Parameter IPLOT to be used for time history
!                              Possible values:
!                               0   Print max Vi
!                               1   Print min Vi
!                               5   Print max abs( Vi )
!                               6   Print min abs( Vi )
!                              10   Plot max Vi
!                              11   Plot min Vi
!                              12   Plot point (x,y,z) Vi
!                              15   Plot max abs( Vi )
!                              16   Plot min abs( Vi )
!                    Pos. 21:  Parameter ISYMBOL for plotting of functions
!                    Pos. 22:  Parameter IPICT (Actual picture number)
!                    Pos. 23:  Parameter NPICT (number of pictures)
!                    Pos. 24:  Parameter LINOPT for PLOT3D
!                    Pos. 25:  Parameter NSTEP for PLOT3D
!                    Pos. 26:  Minimum colour
!                    Pos. 27:  Maximum colour
!                    Pos. 28:  Plot boundaries (1) or not (0)
!                    Pos. 29:  Contract elements (1) or not (0)
!                    Pos. 30:  Plot inner boundaries (1) or not (0)
!                    Pos. 31:  Plot outer boundaries double (1) or not (0)
!                    Pos. 32:  Skip element groups (>0) or not (0)
!                              The value gives the number of element groups to
!                              be skipped
!                    Pos. 33:  Plot element numbers (1) or not (0)
!                    Pos. 34:  Plot node numbers (1) or not (0)
!                    Pos. 35:  Mark nodes (1) or not (0)
!                    Pos. 36:  Renumbered nodes (1) or not (0)
!                    Pos. 37:  NTEXTX, i.e. number of characters in TEXTX
!                    Pos. 38:  NTEXTY, i.e. number of characters in TEXTY
!                    Pos. 39:  NTEXT, i.e. number of characters in TEXT
!                    Pos. 40:  IAXIS, indicates if axis must be plotted (1)
!                              or not (0)
!                    Pos. 41:  NUMBER, indicates if curves/points must be
!                              provided with numbers (1) or not (0)
!                    Pos. 42:  IOPLOT, help parameter for open plot.
!                              Possible values:
!                              0  Default: standard plot command
!                              1  open plot
!                              2  close plot
!                    Pos. 43:  NPOLYG, number of points in a polygon
!                    Pos. 44:  IIDENT, indicates if identification should be
!                              plotted or not. Possible values:
!                              0: Do not plot identification
!                              1: Plot identification (First time, read origin
!                                 from RINPUT)
!                              2: Plot identification (Next times, do not change
!                                 origin)
!                    Pos. 45:  NIDENT, i.e. number of characters in IDENTF
!                    Pos. 46:  NUNKP, i.e. number of unknowns for INTP3D
!                    Pos. 47:  IDEGFD, i.e. degree of freedom for INTP3D
!                    Pos. 48:  MESH,indicates whether mesh must be plotted (1)
!                                 or not (0)
!                    Pos. 49:  NPART,number of particle coordinates read
!                    Pos. 50:  PRINT,indicates whether track must be printed (1)
!                                 or not (0)
!                    Pos. 51:  NMARK,number of markers to be placed along track
!                    Pos. 52:  NVIEW, view-direction (1,2,3 or 4)
!                    Pos. 53:  IPROJT, i.e. type of projection
!                              0:  Parallel
!                              1:  Perspective
!                             10:  Parallel (The view point is computed by
!                                            default)
!                             11:  Perspective (The view point is computed by
!                                            default)
!                    Pos. 54:  Back ground colour
!                    Pos. 55:  Curve colour
!                    Pos. 56:  Mesh colour of visible part (0 = no plot)
!                    Pos. 57:  Mesh colour of invisible part (0 = no plot)
!                    Pos. 58:  Contour line colour of visible part (0 = no plot)
!                    Pos. 59:  Contour line colour of invisible part
!                              (0 = no plot)
!                    Pos. 60:  Curve or surface number to be indicated in plot
!                    Pos. 61:  Indicator whether special element is
!                                 curve             : 0
!                                 wireframe surface : 1
!                                 solid surface     : 2
!                    Pos. 62:  Minimum colour is given (1) or not (0)
!                    Pos. 63:  Maximum colour is given (1) or not (0)
!                    Pos. 64:  Normal component should be computed (1) or
!                              not (0)
!                    Pos. 65:  Tangential component should be computed (1) or
!                              not (0)
!                    Pos. 66:  Co-ordinates must be suppressed in printing (1)
!                              or not (0)
!                    Pos. 67:  Parameter IDIM as present in SEPRAN output
!                              subroutine
!                    Pos. 68:  Parameter BLOCK_MODE for 3d PLOT
!                    Pos. 69:  Parameter TRANSPARENT for 3d PLOT (1=transparent)
!                    Pos. 70:  Parameter SUPPRESS_HEADER (1=SUPPRESS_HEADER)
!                    Pos. 71:  Parameter PLOT_LEGENDA (1=noplot_legenda)
!                    Pos. 72:  Parameter PLOT_SCALES (1=noplot_scales)
!                    Pos. 73:  Parameter ARC_SCALE (1=arc scale found)
!                    Pos. 74:  Parameter POLAR_SYSTEM (3=polar coordinates)
!                    Pos. 75:  Parameter IVEC for output vector
!                    Pos. 76:  Parameter use_porosity (1=use,0 not)
!                    Pos. 77:  Parameter itype_scales
!                               0: linlin
!                               1: loglin
!                              10: linlog
!                              11: loglog
!                    Pos. 78:  Number of positive levels
!                    Pos. 79:  Number of negative levels
!                    Pos. 80:  If 1 all functions in one picture
!                    Pos. 81:  istep_colour
!                    Pos. 82:  istep_symbol
!                    Pos. 83:  dir_rotate
!                    Pos. 84:  num_rotations
!                    Pos. 85:  number of volumes or vectors stored in igroup
!                    Pos. 86:  type of function
!                              Possible values:
!                              1: log
!                    Pos. 87:  Indicates if tstep_print is used (1) or not (0)
!                    Pos. 88:  mesh_in
!                    Pos. 89:  mesh_out
!                    Pos. 90:  iseq_mesh
!                    Pos. 91:  iequidistant (if 1 an equidistant grid is used
!                              to print values)
!                    Pos. 92:  ICURVE_COLOUR
!                    Pos. 93:  ITYPE_INTERPOLATION
!                    Pos. 94:  IPLOT_ROWS
!                    Pos. 95:  IDEFSTART
!                    Pos. 96:  IFIRST
!                    Pos. 97:  ILAST
!                    Pos. 98:  ITYPE_TIME_INTEGRATION
!                    Pos. 99:  ICHOICE_INTP3D
!                    Pos.100:  use_file_name (1 use file name in text,
!                              0 do not use file name)
!                    Pos.101:  number of characters in text(2)
!                    Pos.102:  0:  plot user coordinates
!                              1:  plot fixed coordinates
!                    Pos.103:  Node numbers must be suppressed in printing (1)
!                              or not (0)
!
!                    The following numbers in IINPUT(1) are allowed and refer to
!                    the subroutine mentioned:
!
!                      0:  END record found, stop the process
!                      1:  Subroutine PRINRV, general print subroutine
!                      2:  Subroutine PRINOV, ordered print subroutine
!                      3:  Subroutine PRINBN, print along curves or surfaces
!
!                    101:  Subroutine PLOTM1/2/3 plot mesh
!                    102:  Subroutine PLOTCU, plot user points
!                    103:  Subroutine PLOTCU, plot curves
!                    104:  Subroutine PLOT3M, plot 3D coloured mesh
!                    110:  Subroutine PLOTVC, vector plot
!                    111:  Subroutine PLOTDT, plot distorted two-dimensional
!                          mesh
!                    112:  Subroutine PLOTC1, contour plot
!                    113:  Subroutine PLOTC2, coloured contour plot
!                    115:  Subroutine PLOT3D, plot three-dimensional function
!                                             given at two-dimensional region
!                                             with hidden lines
!                    116:  Subroutine PLOT3C, plot three-dimensional function
!                                             given at two-dimensional region
!                                             with hidden lines (coloured
!                                             surfaces)
!                    118:  Subroutine PFIELD, plot field line(s)
!                    119:  Subroutine PTRACK, plot track(s) of particles
!                    120:  Subroutine PLOTFN, plot one dimensional function
!                    121:  Subroutine PLOTLN, plot solution on line
!                    123:  Subroutine CONT3D, plot contour levels of a 3D
!                                             function
!                    124:  Subroutine CONT3C, plot contour surfaces of a 3D
!                                             function
!                    125:  Subroutine PLTEXT, plot text
!                    126:  Subroutine PLAF05, plot polygon
!                    127:  Subroutine PLAFP7, plot identification
!                    180:  Subroutine BOUNIV, compute boundary integrals
!                    181:  Subroutine VOLINT, compute volume integrals
!
!                    200:  Subroutine COMPCR, compute quantities along curves
!                    201:  Subroutine INTP2D, compute quantities along a line
!                    202:  Subroutine INTP3D, interpolate by a plane
!                    203:  Subroutine STREAM, compute stream function
!                    204:  Subroutine PRMACHNO, compute mach number
!                    205:  Subroutine PRTOTENTH, compute total enthalpy
!                    206:  Subroutine PRSTAGPRES, compute stagnation pressure
!                    207:  Subroutine PRCPCOEF, compute scaled pressure
!                    208:  Subroutine PRENTROPY, compute entropy
!                    209:  Subroutine ERINTPOL, interpolate
!                    210:  Subroutine , normal stress
!                    225:  Subroutine MANVECBF, compute vector of other vector
!
!                    300:  Time history
!
!     imapsol        Array of size maxvc1 x ntimvc defining a mapping from
!                    a two-dimensional storage to a one-dimensional storage
!                    with respect to array isol
!                    imapsol(i,j) refers to the i-th vector at time level j
!                    It contains an integer reference number k to isol(.,k),
!                    if this array is used and otherwise a 0
!     infocomponents Array of size maxvc1 x 2 containing information of the
!                    names of the components of each vector stored in names
!                    infocomponents(i,1) contains the number of components
!                    infocomponents(2,1) contains the starting address of the
!                    name of the component in array namecomponents, provided
!                    the number of components is larger than 1
!     ipost          special constants:
!                    1: Length ipost
!                    2: Length rpost
!                    3: profile flow (1) or not (0)
!                    4: icurve_upper (curve number upper surface
!                       of profile)
!                    5: icurve_lower (curve number lower surface
!                       of profile)
!     irefar         Input array of length 1001 that contains the reference
!                    between the user numbers for the vectors and the actual
!                    numbers from  1 to n
!                    If irefar(i) = 0, then the vector with reference number i
!                    does not exist; if irefar(i) > 0, then its value refers to
!                    the actual sequence number
!     ispec          Array of length maxvc1 x 3 which indicates for each array
!                    if it corresponds to a function to be computed or not, from
!                    which array it has been computed, and to which problem
!                    resp. mesh it belongs
!                    Contents:
!                    ispec(i,1) = 0,  the array does not correspond to any
!                                     function
!                    ispec(i,1) = 1,  the array corresponds to a stream function
!                    ispec(i,1) = 2,  the array corresponds to an interpolated
!                                     one-dimensional function
!                    ispec(i,1) = 3,  the array corresponds to an interpolated
!                                     two-dimensional function
!                    ispec(i,1) = 4,  the array corresponds to the mach number
!                    ispec(i,1) = 10, the array corresponds to normal_stress
!                    ispec(i,2) contains the solution array number from which
!                    the function must be computed
!                    ispec(i,3) contains the mesh and problem array number to
!                    which the function belongs. (Default one).
!                    So the mesh and problem corresponding to solution array i
!                    are given by KMESH(.,ispec(i,3)) resp. KPROB(.,ispec(i,3))
!                    if ispec(i,1) = 4, ispec(i,4) refers to the enthalpy vector
!                    if ispec(i,1) = 10 ispec(i,3) contains idim and ispec(i,3)
!                    the mm-sequence number for the curves
!    rinputpst  Contains the actual real information from
!               the standard input file
!               Length of the array: 100
!               RINPUT is filled as follows:
!               Pos.  1:  PLOTFM or LENGTH  (i.e. length of the plot)
!               Pos.  2:  YFACT  (multiplication factor in y-direction)
!               Pos.  3:  FACTOR (multiplication factor for vectors)
!               Pos.  4:  MINLEVEL  (minimum level  for contour line)
!               Pos.  5:  MAXLEVEL  (maximum level  for contour line)
!               Pos.  6:  PSIPHI (stream value for jpoint)
!               Pos.  7:  ORIGX  (x-origin of line)
!               Pos.  8:  ORIGY  (y-origin of line)
!               Pos.  9:  ANGLE  (angle of line or angle under which a
!                                 picture is visible)
!               Pos. 10:  XP     (x-co-ordinate, for example eye point)
!               Pos. 11:  YP     (y-co-ordinate)
!               Pos. 12:  ZP     (z-co-ordinate)
!               Pos. 13:  XP1    (x-co-ordinate, for example projection point
!               Pos. 14:  YP1    (y-co-ordinate)
!               Pos. 15:  ZP1    (z-co-ordinate)
!               Pos. 16:  HEIGHT (height of characters in cm's)
!               Pos. 17:  APLANE (parametes defining the plane
!               Pos. 18:  BPLANE     ax + by +cz = d
!               Pos. 19:  CPLANE     with APLANE = a, BPLANE = b
!               Pos. 20:  DPLANE          CPLANE = c, DPLANE = d )
!               Pos. 21:  TMAX   (maximum time for particle tracking)
!               Pos. 22:  GAMMA  specific heat ratio
!               Pos. 23:  XY_ANGLE
!               Pos. 24:  INTERSECT_ANGLE
!               Pos. 25:  GROUND_VALUE
!               Pos. 26:  ZSCALE
!               Pos. 27:  YSCALE
!               Pos. 28:  smin
!               Pos. 29:  smax
!               Pos. 30:  TSTEP_PRINT  time step for printing
!               Pos. 31:  DX
!               Pos. 32:  DY
!               Pos. 33:  DZ
!               Pos. 34:  EPS_TIME_STEP
!               Pos. 35:  ORIGZ (z-origin of plane)
!               Pos. 36:  TANG1_X (x-component of first tangential vector)
!               Pos. 37:  TANG1_Y (y-component of first tangential vector)
!               Pos. 38:  TANG1_Z (z-component of first tangential vector)
!               Pos. 39:  TANG2_X (x-component of second tangential vector)
!               Pos. 40:  TANG2_Y (y-component of second tangential vector)
!               Pos. 41:  TANG2_Z (z-component of second tangential vector)
!               Pos. 42:  XSCALE
!               Pos. 43:  TEXT_SIZE
!               Pos. 44:  TEXT_SIZE (levels)
!               Pos. 45:  TEXT_SIZE (numbers in levels)
!               Pos. 51-100:  Co-ordinates for polygon (At most 50, i.e.
!                             25 points)
!   rpost       special constants:
!                    1: gamma
!                    2: mach_ref (Mr)
!                    3: p_infinity
!                    4: h_infinity
!                    5: ||momentum_infinity||
!                    6: length airfoil
!                    7: angle of attack
!     isol           Starting SEPRAN solution array. Array ISOL has been
!                    constructed such that it may contain maxvc1 solution arrays
!                    for each time step.
!                    In order to save memory  isol is stored as a 5 x maxsoluts
!                    array. To get an entry in isol array imapsol must be used
end module sepmodulepost
