      subroutine msh021 ( kmesh, npelm, nboun, ishape, nelgrp, iwork, &
                          irefc, nelem, lengpc, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.11 date 30-03-2004 Extension with shape 134
!        version  4.10 date 14-03-2003 Extension with shapes > 100
!        version  4.9  date 18-03-2002 Extend first part of kmesh to lenkmeshfp
!        version  4.8  date 27-03-2000 Extend first part of kmesh to 60
!        version  4.7  date 21-07-1999 Tags included
!        version  4.6  date 18-09-1998 Remove kmesh(50)=1
!        version  4.5  date 04-10-1996 Use ihelp(2)
!        version  4.4  date 07-12-1995 Correction for spectral part
!        version  4.3  date 07-04-1995 Spectral part added
!        version  4.2  date 28-12-1994 Redefinition of ihelp
!        version  4.1  date 30-10-1991 Redefinition of ihelp
!        version  4.0  date 26-11-1989 Extension of parameter list, &
!                                      Addition of KMESH part t
!        version  3.1  date 24-09-1988 Unformatted read
!        version  3.0  date 22-12-1985
!
!   copyright (c) 1984-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill a part of array kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepintf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), iwork(*), npelm, nboun, ishape, nelgrp, irefc, &
              nelem, lengpc, ihelp(*)

!     ihelp         i   Work array in which only position 1 and position 4 are
!                       of interest
!                       if ihelp(1) > 0, then array KMESH part t must be filled
!                       if ihelp(4) > 0
!     irefc         i   dataset reference number for reading (>0 formatted, &
!                          <0 unformatted)
!     ishape        i   shape number of element when nelgrp = 1
!     iwork             work array of length nelgrp x 4
!     kmesh        i/o  standard sepran array
!     lengpc        i   length of kmesh part c
!     nboun         i   number of nodal points on the boundary of an element
!                       when nelgrp = 1
!     nelem         i   number of elements
!     nelgrp        i   number of element groups
!     npelm         i    Number of points per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmf, kelm, kelgrp, ih, ielgrp, jnodp, jboun, inpelm, &
              inboun, kelem, ishapet, inelem, n, ios
      double precision dummy(1)

!     dummy   Dummy parameter
!     i       Counting variable
!     ielgrp  element group number
!     ih      Help variable to store a constant temporarily
!     inboun  Number of points on boundary of element
!     inelem  Number of elements in a specific element group
!     inpelm  number of points per element
!     ios     IOSTAT variable, returns from reading or writing
!     ishapet Shape number read
!     jboun
!     jnodp   ?
!     kelem   New KMESH part c with respect to the surface elements
!             All elements have the same number of nodes
!     kelgrp  sum over all standard elements of number of points in standard
!             elements
!     kelm    ?
!     kelmf   Indicates if KMESH part f has been filled (>0) or not (0)
!             If kelmf>0, kelmf is the starting address of KMESH part f in KMESH
!     n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     190   Number of elements found is incorrect
!     192   ishape has incorrect value
!     193   the sum of the number of elements of the various standard elements
!           is not equal to number of elements
!     199   error in reading of dimension parameters
!     251   end of file found in reading of dimension parameters
!    1746   The number of nodes in the elements of element group is incorrect
!    1747   The number of internal nodes in the elements of element group
!           is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh021' )
      if ( kmesh(2).ne.100 ) then

!     --- kmesh (2) # 100   clear positions in kmesh

         kmesh(7) = 0
         do i = 14, lenkmeshfp
            kmesh(i) = 0
         end do

      end if

      kelmf = lenkmeshfp+1
      kmesh(20) = kelmf
      kmesh(41) = ihelp(2)
      kelm = kelmf+nelgrp
      if ( nelgrp.eq.1 ) then

!     --- nelgrp = 1  simple situation

         kelgrp = npelm
         kmesh(kelmf) = ishape
         if ( nboun.lt.npelm ) then

!        --- nboun < npelm  fill kmesh part b

            kmesh(16) = kelm
            kmesh(kelm) = nboun
            kelm = kelm+1
         end if
         lengpc = npelm*nelem

      else

!     --- nelgrp > 1,  read information

         ih = 0
         do ielgrp = 1, nelgrp
            if ( irefc.ge.0 ) then
               read ( irefc, *, iostat = ios ) (iwork(ih+i), &
                      i = 1, 4)
            else
               read ( -irefc, iostat=ios ) (iwork(ih+i), &
                      i = 1, 4)
            end if
            if ( ios.gt.0 ) then

!           --- ios > 0, Error in reading file

               call errint ( ios, 1 )
               call errsub (199, 1, 0, 0)
               go to 1000

            else if ( ios.lt.0 ) then

!           --- ios > 0, End of file reached when reading file 10

               call errsub ( 251, 0, 0, 0)
               go to 1000

            end if
            ih = ih+4

         end do

!        --- check information
!            first inpelm

         jnodp = 1
         jboun = 1
         do ielgrp = 1, nelgrp
            inpelm = iwork(ielgrp*4-3)
            inboun = iwork(ielgrp*4)
            if ( inpelm.ne.npelm ) jnodp = 0
            if ( inboun.ne.nboun ) jboun = 0
            if ( inpelm.lt.0 .or. inpelm.gt.npelm ) then
              call errint ( ielgrp, 1 )
              call errint ( inpelm, 2 )
              call errint ( npelm, 3 )
              call errsub ( 1746, 3, 0, 0 )
            end if
         end do
         kelgrp = inpelm*nelgrp

         if ( jnodp.eq.0 ) then

!        --- jnodp = 0   fill kmesh part a

            kmesh(15) = kelm
            kmesh(kelm) = 0
            kelgrp = 0
            do ielgrp = 1, nelgrp
               kelgrp = kelgrp+iwork(ielgrp*4-3)
               kmesh(kelm+ielgrp) = kelgrp
            end do
            kelm = kelm+nelgrp+1
         end if

!        --- then nboun

         if ( jboun.eq.0 ) then

!        --- jboun = 0   fill kmesh part b

           kmesh(16) = kelm
           do ielgrp = 1, nelgrp
              inboun = iwork(4*ielgrp)
              inpelm = iwork(ielgrp*4-3)
              kmesh(kelm+ielgrp-1) = inboun
              if ( inboun.lt.0 .or. inboun.gt.inpelm ) then
                 call errint ( ielgrp, 1 )
                 call errint ( inboun, 2 )
                 call errint ( inpelm, 3 )
                 call errsub ( 1747, 3, 0, 0 )
              end if
           end do
           kelm = kelm+nelgrp

         end if

!        --- ishape

         lengpc = 0
         kmesh(21) = kelm
         kelem = 0
         do ielgrp = 1, nelgrp
            ishapet = iwork(4*ielgrp-2)
            kmesh(kelmf+ielgrp-1) = ishapet
            if ( ishapet.gt.100 .and. ishapet.ne.109 .and. &
                 ishapet.ne.134 ) then
               call errint ( ishapet, 1 )
               call errint ( 100, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 192, 3, 0, 0 )
            end if
            inelem = iwork(ielgrp*4-1)
            kmesh(kelm+ielgrp-1) = inelem
            kelem = kelem+inelem
            inpelm = iwork(4*ielgrp-3)
            if ( inelem.lt.0 .or. inelem.gt.nelem ) then
               call errint ( inelem, 1 )
               call errint ( nelem, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 190, 3, 0, 0 )
            end if
            lengpc = lengpc+inelem*inpelm
         end do
         if ( kelem.ne.nelem ) then
            call errint ( kelem, 1 )
            call errint ( nelem, 2 )
            call errsub ( 193, 2, 0, 0 )
         end if
         kelm = kelm+nelgrp
      end if

      if ( ihelp(1).gt.0 ) then

!     --- ihelp(1) = 1, hence KMESH part t must be filled

         n = kmesh(13)
         if ( n.eq.0 ) n = kmesh(12)
         if ( n.eq.0 ) n = kmesh(11)
         kmesh(36) = kelm
         kelm = kelm + 2 * n

      end if
      if ( ihelp(4).gt.0 ) then

!     --- ihelp(4)>0, hence KMESH part u must be filled

         kmesh(38) = kelm
         kelm = kelm + 5

      end if

      kmesh(14) = kelgrp
      call prinlg ( 'kmesh', kelm, dummy, kmesh )
1000  call erclos ( 'msh021' )
      if ( ierror.gt.0 ) call instop
      end
