      subroutine mshsortelm ( ibuffr, buffer, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 21-11-2012 Long integers
!        version  1.3  date 22-08-2012 New calls to mshsortel2d/3d
!        version  1.2  date 12-02-2003 Replace ini060 by ini060text
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh parts intelmpos and realelmpos with information of in
!     which part region a node is positioned
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ibuffr(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmeshac, iprmeshac, ipcoor,ipkmsk, ipkmsj
      integer ndim, npoint, length, lenkmeshac, nsplit, lenrmeshac

!     ipcoor         starting address of co-ordinates array in BUFFER
!     ipkmeshac      Starting address of array kmesh part ac in array IBUFFR
!     ipkmsj         Starting address of array KMESH part j into array
!                    IBUFFR
!     ipkmsk         Starting address of array KMESH part k into array
!                    IBUFFR
!     iprmeshac      starting address of array rmesh part ac in BUFFER
!     length         Help parameter to indicate the length of arrays
!     lenkmeshac     Length of array kmeshac
!     lenrmeshac     Length of array rmeshac
!     ndim           Dimension of the space.
!     npoint         Number of nodal points
!     nsplit         Number of parts into which the mesh has been splitted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgt, inidbl, inisgl

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI083         Create new array in IBUFFR in the case that already a
!                    temporary copy has been stored in IBUFFR
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIDGT         Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INISGL         Transform pointer for BUFFER to pointer for IBUFFR
!     MSHSORTEL2D    Fill array kmesh part ac with information of in which part
!                    of the region an element is positioned (2D)
!     MSHSORTEL3D    Fill array kmesh part ac with information of in which part
!                    of the region an element is positioned (3D)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( ndim=2 ) then
!       perform algorithm for two-dimensional grid
!    else if ( ndim=3 ) then
!       perform algorithm for three-dimensional grid
!    else
!       skip
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsortelm' )
      ndim = kmesh(6)
      if ( ndim==1 ) go to 1000
      npoint = kmesh(8)
      nsplit = max(1,kmesh(44))
      call ini060text ( ibuffr, kmesh(23), 'coor' )
      if ( kmesh(24)>0 ) call ini060text ( ibuffr, kmesh(24), 'kmeshj' )
      if ( kmesh(25)>0 ) call ini060text ( ibuffr, kmesh(25), 'kmeshk' )

!     --- Reserve work space and space for arrays kmeshac and rmeshac
!         iwork                 3*npoint
!         kmeshac    estimated: npoint+4*nsplit+10000*nsplit
!         rmeshac               4*nsplit reals

      lenkmeshac = npoint+(2+ndim)*nsplit+10000*nsplit
      lenrmeshac = (2*ndim)*nsplit
      length = lenkmeshac+lenrmeshac*intlen
      call ini062 ( ibuffr, length, iprmeshac )
      ipkmeshac = iprmeshac+(2*ndim)*nsplit*intlen
      iprmeshac = inidbl(iprmeshac)
      ipcoor = inidgt(kmesh(23))
      if ( kmesh(24)>0 ) then
         ipkmsj = inigettext ( kmesh(24), 'kmeshj' )
      else
         ipkmsj = 1
      end if
      if ( kmesh(25)>0 ) then
         ipkmsk = inigettext ( kmesh(25), 'kmeshk' )
      else
         ipkmsk = 1
      end if
      if ( ndim==2 ) then

!     --- ndim = 2

         call mshsortel2d ( kmesh, buffer(ipcoor), &
                            ibuffr(ipkmsj), ibuffr(ipkmsk), &
                            ibuffr(ipkmeshac), buffer(iprmeshac), &
                            lenkmeshac )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call mshsortel3d ( kmesh, buffer(ipcoor), &
                            ibuffr(ipkmsj), ibuffr(ipkmsk), &
                            ibuffr(ipkmeshac), buffer(iprmeshac), &
                            lenkmeshac )
      end if

!     --- Make kmesh part ac permanent

      iprmeshac = inisgl(iprmeshac)
      length = (2*ndim)*nsplit*intlen+lenkmeshac
      call ini083 ( ibuffr, kmesh(51), length, iprmeshac, 'kmeshac' )

1000  call erclos ( 'mshsortelm' )
      end
