      subroutine prcoornode ( ibuffr, buffer, kmesh, node, coorloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 09-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the coordinates in a given node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), node
      double precision coorloc(*), buffer(*)

!     buffer        i/o   Buffer array
!     coorloc        o    Array with the coordinates of node
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     node           i    Node number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipcoor
      integer ndim
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipcoor         starting address of co-ordinates array
!     ndim           Dimension of the space.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     PRCOORNODE1    Get the value of the coordinates in a given node
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcoornode' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcoornode'
         write(irefwr,1) 'node', node
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call ini060text ( ibuffr, kmesh(23), 'coor' )
      ndim = kmesh(6)
      ipcoor = inidgttext ( kmesh(23), 'coor' )
      call prcoornode1 ( buffer(ipcoor), ndim, node, coorloc )

1000  call erclos ( 'prcoornode' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( buffer(ipcoor), kmesh(8), ndim, 'coor' )
         call prinrl ( coorloc, ndim, 'coorloc' )
         write(irefwr,*) 'End prcoornode'

      end if  ! ( debug )

      end
