      subroutine prcopsepvec3 ( func, usol, kprobp, npoint, idegfd, &
                                nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one degree of freedom of sepran vector in user vector
!     Only nodes in array nodes are used
!     The number of degrees of freedom per point is variable
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npoint, idegfd, kprobp(npoint,*), nnodes, &
                             nodes(nnodes)
      double precision, intent(in) :: usol(*)
      double precision, intent(out) :: func(nnodes)

!     func           o    Contains the function values
!     idegfd         i    Degree of freedom sequence number
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unknowns per point
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
!     npoint         i    Number of nodal points
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopsepvec3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopsepvec3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      func = usol(kprobp(nodes,idegfd))

1000  call erclos ( 'prcopsepvec3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopsepvec3'

      end if  ! ( debug )

      end
