      subroutine prcopyus2vec2 ( sepvector, inpvec, nnodes, idegfd, &
                                 kprobp, nodes, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy user vector to sepran vector
!     The user vector has a variable number of degrees of
!     freedom per point
!     The output vector is defined in all nodes of nodes and has exactly 1
!     degree of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npoint, idegfd, nnodes, nodes(nnodes), &
                             kprobp(npoint,*)
      double precision, intent(in) :: inpvec(nnodes)
      double precision, intent(inout) :: sepvector(*)

!     idegfd         i    Degree of freedom sequence number
!     inpvec         i    Input vector of length npoint containing the
!                         vector to be copied
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unknowns per point
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be filled
!     npoint         i    Number of nodal points
!     sepvector      o    sepran vector consisting of nunkp degrees of freedom
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyus2vec2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyus2vec2'
         write(irefwr,1) 'idegfd, nnodes, npoint', &
                          idegfd, nnodes, npoint
         call prinrl ( inpvec, nnodes, 'inpvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepvector(kprobp(nodes(1:nnodes),idegfd)) = inpvec(1:nnodes)

1000  call erclos ( 'prcopyus2vec2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopyus2vec2'

      end if  ! ( debug )

      end
