      subroutine prfillnodobs ( ibuffr, inpsep, kmesh, nodes, nnodes, &
                                kfill )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for obstacle problems depending on kfill
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), kmesh(*), nodes(*), nnodes, ibuffr(*), kfill

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipipointobst
      integer iobst
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iobst          Obstacle sequence number
!     ipipointobst   Starting address of array ipointobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLNODOBS   Fill array nodes for obstacle problems depending on kfill
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodobs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodobs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0
      if ( kmesh(42)==0 ) go to 1000  ! no obstacle found

!     --- Get parameters from inpsep

      iobst = inpsep(1)

!     --- activate arrays

      call ini060text ( ibuffr, kmesh(kmesh(52)), 'ipointobst' )

!     --- Compute starting addresses

      ipipointobst = inigettext ( kmesh(kmesh(52)), 'ipointobst' )
      if ( ierror/=0 ) go to 1000

!     --- actual call

      call erfillnodobs ( ibuffr(ipipointobst), iobst, nodes, nnodes, &
                          kfill )

1000  call erclos ( 'prfillnodobs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodobs'

      end if  ! ( debug )

      end
