      subroutine prfillnodobst ( ibuffr, inpsep, kmesh, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for obstacle problems (curves only)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     obstacle
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), kmesh(*), nodes(*), nnodes, ibuffr(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsm, ipicrvob
      integer error, npoint, iobst
      integer, allocatable :: iwork(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     iobst          Obstacle sequence number
!     ipicrvob       Starting address of array ICURVOBS
!     ipkmsm         Starting address of array KMESH part m
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODOBST  Fill array nodes for obstacle problems (curves only)
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodobst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodobst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0
      if ( kmesh(22)==0 .or. kmesh(42)==0 ) go to 1000  ! no obstacle found

!     --- Get parameters

      npoint = kmesh(8)

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- Get parameters from inpsep

      iobst = inpsep(1)

!     --- activate arrays

      call ini060text ( ibuffr, kmesh(22), 'icurvsobst' )
      call ini060text ( ibuffr, kmesh(27), 'kmeshm' )

!     --- Compute starting addresses

      ipicrvob = inigettext ( kmesh(22), 'icurvsobst' )
      ipkmsm = inigettext (  kmesh(27), 'kmeshm' )
      if ( ierror/=0 ) go to 1000

!     --- actual call

      call erfillnodobst ( ibuffr(ipkmsm), iobst, ibuffr(ipicrvob), &
                           kmesh, nodes, nnodes, iwork )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'prfillnodobst' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodobst'

      end if  ! ( debug )

      end
