      subroutine prfillposunk ( indprp, indprh, indprf, kprob, kprobp, &
                                kprobf, kprobh, npoint, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-05-2006 Debug statements
!        version  1.0  date 06-04-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array iwork with positions of an unknown with respect to
!    the nodes and also renumbered with respect to the matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, indprp, indprh, indprf, kprobp(npoint,*), &
              kprobf(*), kprobh(*), iwork(4,*), kprob(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     indprh         i    indication if kprob part h has been filled (>0)
!                         or not (0)
!     indprp         i    Indication if array kprob part p has been filled (1)
!                         or not (0)
!     iwork          o    integer work array of size 4*nusol
!                         In this array per unknown the following information
!                         is stored
!                         (1,.) node corresponding to unknown in standard seq
!                         (2,.) local degree of freedom in node corresponding
!                               to unknown in standard sequence
!                         (3,.) node corresponding to unknown if numbering
!                               with respect to matrix is used
!                         (4,.) local degree of freedom in node corresponding
!                               to unknown if numbering
!                               with respect to matrix is used
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     kprobh         i    KPROB part h: contains the new numbering of the
!                         unknowns
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unkowns per point
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nphys, nusol, i, j, inunkp, inusol, jnusol
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jnusol         Position of unknown in array USOL
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set constants
!     if ( indprp>0 ) then
!        use kprobp to fill iwork
!     else if ( indprf>0 ) then
!        use kprobf to fill iwork
!     else
!        fill iwork
!     Print the contents of iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillposunk' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillposunk'
         write(irefwr,1) 'indprf, indprh, indprp', &
                          indprf, indprh, indprp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      nphys = kprob(33)
      nusol = kprob(5)
      if ( debug ) write(irefwr,1) 'nphys, nusol', nphys, nusol

!     --- Clear iwork

      do i = 1, nusol
         do j = 1, 4
            iwork(j,i) = 0
         end do
      end do

      if ( indprp>0 ) then

!     --- indprp>0, use kprobp

         do i = 1, npoint

            do j = 1, nphys

               inusol = kprobp(i,j)
               if ( inusol>0 ) then

!              --- inusol>0, fill iwork

                  iwork(1,inusol) = i
                  iwork(2,inusol) = j
                  if ( indprh>0 ) then

!                 --- indprh>0, renumbering with respect to matrix

                     inusol = kprobh(inusol)
                     iwork(3,inusol) = i
                     iwork(4,inusol) = j

                  end if

               end if

            end do

         end do

      else if ( indprf>0 ) then

!     --- indprf>0, use kprobf

         do i = 1, npoint

            jnusol = kprobf(i)
            inunkp = kprobf(i+1)-jnusol

            do j = 1, inunkp

               inusol = jnusol+j
               if ( inusol>0 ) then

!              --- inusol>0, fill iwork

                  iwork(1,inusol) = i
                  iwork(2,inusol) = j
                  if ( indprh>0 ) then

!                 --- indprh>0, renumbering with respect to matrix

                     inusol = kprobh(inusol)
                     iwork(3,inusol) = i
                     iwork(4,inusol) = j

                  end if

               end if

            end do

         end do

      else

!     --- indprf = 0, &

         inunkp = kprob(4)
         do i = 1, npoint

            jnusol = (i-1)*inunkp
            do j = 1, inunkp

               inusol = jnusol+j
               iwork(1,inusol) = i
               iwork(2,inusol) = j
               if ( indprh>0 ) then

!              --- indprh>0, renumbering with respect to matrix

                  inusol = kprobh(inusol)
                  iwork(3,inusol) = i
                  iwork(4,inusol) = j

               end if

            end do

         end do

      end if

1000  call erclos ( 'prfillposunk' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( iwork, nusol, 4, 'iwork' )
         write(irefwr,*) 'End prfillposunk'

      end if  ! ( debug )

      end
