      subroutine prfilluser ( ibuffr, buffer, iuser, user, iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-12-2014 Extra parameter iprob
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 19-10-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy arrays iuser and user as stored in buffer into iuser and user
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/clinslf90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), iuser(*), iprob
      double precision buffer(*), user(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iprob          i    Actual problem number
!     iuser          o    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     user           o    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iptemp
      integer lenius, lenusr
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iptemp         Starting address of temporary array
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext
      integer inilen, inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilluser' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilluser'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenius = inilen(iuser1(iprob))
      lenusr = inidln(iuser2(iprob))

!     --- Guarantee that arrays IUSER and USER are placed into IBUFFR

      call ini060text ( ibuffr, -iuser1(iprob), 'iuser1(iprob)' )
      call ini060text ( ibuffr, -iuser2(iprob), 'iuser2(iprob)' )

!     --- Copy contents of IUSER and USER

      iptemp = inigettext ( iuser1(iprob), 'iuser1(iprob)' )
      iuser(1:lenius) = ibuffr(iptemp:iptemp+lenius-1)
      iptemp = inidgttext ( iuser2(iprob), 'iuser2(iprob)' )
      user(1:lenusr) = buffer(iptemp:iptemp+lenusr-1)

1000  call erclos ( 'prfilluser' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfilluser'

      end if  ! ( debug )

      end
