      subroutine prfillvec ( ibuffr, buffer, isol, iseqvec, kfun, nodes, &
                             nnodes, kmesh, kprob, idegfd, indprf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-01-2015 extension with prveccombinat7
!        version  1.0  date 10-03-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill vector in case of a vector expression
!     Real case
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     real
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kfun, iseqvec, nnodes, nodes(nnodes), &
                             kmesh(*), kprob(*), idegfd, indprf
      integer, intent(inout) :: ibuffr(*), isol(5,*)
      double precision, intent(inout) :: buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     idegfd         i    Degree of freedom sequence number
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     iseqvec        i    Sequence numbers of output vector in isol
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         In this case kfun refers to sequence number of vector
!                         expression plus 200000000
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numvecrhs, numrealrhs, icount, istore(100), &
                 vecs(100)
      double precision :: store(100)
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icount         Counter
!     istore         Contains the expression as stored in common
!                    cexpression, where all scalars and reals
!                    have been computed and stored in store
!                    References to these values are given by
!                    their position in store plus 1000
!                    The values of the vectors are stored in
!                    array vectors
!                    References to these vectors are given by
!                    their position in vectors plus 10000
!     numrealrhs     Number of reals stored in store
!     numvecrhs      Number of vectors in right-hand side
!                    Length of array vecs
!     store          Contains real information of the expression
!                    See subroutine READEXPR, array work
!     vecs           In this array of size numvecrhs the sequence numbers
!                    of the right-hand side vectors are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRVECCOMBINAT4 Evaluate right-hand side of vector expression
!                    a part of the real vector must be computed
!                    the number of unknowns per point is constant
!     PRVECCOMBINAT7 Evaluate right-hand side of vector expression
!                    a part of the real vector must be computed
!                    the number of unknowns per point is variable
!     PRVECSTRHS     Translate expression to a set of reals and a new set of
!                    temporary vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Translate expression to a set of reals and a new set of
!         temporary vectors

      call prvecstrhs ( kfun-200000000, numvecrhs, numrealrhs, icount, &
                        istore, store, vecs )
      call inichkarlen ( icount, 100, 'istore' )
      call inichkarlen ( numrealrhs, 100, 'store' )
      call inichkarlen ( numvecrhs, 100, 'vecs' )
      if ( ierror/=0 ) go to 1000

      if ( indprf==0 ) then

!     --- The number of degrees of freedom in each node is the same

         call prveccombinat4 ( ibuffr, buffer, isol, iseqvec, vecs, &
                               numvecrhs, kmesh, kprob, idegfd, &
                               istore, store, icount, numrealrhs, &
                               nodes, nnodes )

      else

!     --- The number of degrees of freedom is variable

         call prveccombinat7 ( ibuffr, buffer, isol, iseqvec, vecs, &
                               numvecrhs, kmesh, kprob, idegfd, &
                               istore, store, icount, numrealrhs, &
                               nodes, nnodes )

      end if  ! ( indprf==0 )

1000  call erclos ( 'prfillvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillvec'

      end if  ! ( debug )

      end
