      subroutine prfindcurvnod1 ( inode, kmeshm, ncurvs, numcurves, &
                                  curvesfound, kmeshq, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-08-2005 Correction single curves
!        version  1.0  date 06-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node inode is in one or more of the curves
!     and return with corresponding curve numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode, kmeshm(*), ncurvs, numcurves, curvesfound(*), &
              kmeshq(*), kmesh(*)

!     curvesfound    o    Array with curve numbers in which inode can be found
!     inode          i    Node sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     ncurvs         i    Number of curves in mesh
!     numcurves      o    Number of curves stored in array curvesfound
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, numnodes, istart
      logical single, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icurnr         Curve sequence number
!     istart         First position used at a part
!     numnodes       Number of nodal points in array nodes
!     single         If true the curve is a single curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inifindentry

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFINDENTRY   Check if integer number can be found in integer array
!                    and return with the position in the array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all single curves icurnr do
!        if ( node in icurnr ) then
!           store curve number
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindcurvnod1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindcurvnod1'

      end if
      if ( ierror/=0 ) go to 1000
      numcurves = 0

!     --- For all single curves icurnr do

      do icurnr = 1, ncurvs

         single = kmesh(31)==0
         if ( .not. single ) single = kmeshq(icurnr+1)-kmeshq(icurnr)==0
         if ( single ) then

!        --- Single curve
!            if ( node in icurnr ) then
!               store curve number

            numnodes = kmeshm(5+icurnr)-kmeshm(4+icurnr)
            istart = kmeshm(2)-1+kmeshm(4+icurnr)

            if ( inifindentry ( inode, kmeshm(istart), numnodes )>0 ) &
               then

!           --- node found in curve; store

               numcurves = numcurves+1
               curvesfound(numcurves) = icurnr

            end if  ! ( inifindentry ( inode, kmeshm(istart), numnodes )>0 )

         end if  ! ( single )

      end do  ! icurnr = 1, ncurvs

1000  call erclos ( 'prfindcurvnod1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindcurvnod1'

      end if

      end
