      subroutine prfindelm3 ( xgauss, igaussp, coor, kmeshc, &
                              kmeshac, rmeshac, kmshd1, kmshd2, iwork, &
                              iwork1, iwork2, isplit, inpelm, &
                              nvertices, ielem, ielemlow, ielemtop, &
                              kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 25-08-2000 Change of parameter list
!        version  1.0  date 25-03-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find element number in which point with coordinates (x,y,Z) is lying
!   Three-dimensional case
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     element
!     find
!     find_element
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/ctyp3df90'
      include 'SPcommon/celintf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xgauss(m,3), coor(3,*), rmeshac(6,*)
      integer          inpelm, kmeshc(inpelm,*), kmeshac(*), &
                       kmshd1(*), kmshd2(*), iwork(*), iwork1(*), &
                       iwork2(*), nvertices, isplit, ielem, &
                       ielemlow, ielemtop, kmesh(*), igaussp

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ielem          o    Element number
!     ielemlow       i    Smallest element number in group
!     ielemtop       i    Largest element number in group
!     igaussp        i    Sequence number of gauss point
!     inpelm         i    number of points per element
!     isplit         i    Sequence number of discontinuous part
!     iwork               integer work array of length npoint
!                         In this array all node numbers of a part are stored
!     iwork1              integer work array of length nelem
!                         iwork1(i) = 0, means element i has not yet been used
!                         iwork1(i) = 1, means element i has been used
!                         In this array all element numbers of a part are stored
!     iwork2              integer work array of length nelem
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshac        i    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         kmeshac is organized as follows:
!                         Part 1: length (ndim+2) x nsplit
!                               For each discontinuous part ndim+2 positions
!                               Pos. 1 number of nodes in part
!                               pos. 2-ndim+1: nx,ny,nz i.e. number of cells in
!                                              each direction
!                               pos. ndim+2: starting address w.r.t. part 2
!                         For each discontinuous part we have a separate
!                         part 2 and part 3
!                         Part 2: length 1+nx ny nz starting address of
!                               nodes in part 3 for each cell
!                               So in 2d the number of nodes in cell (i,j) is
!                               kmeshac_2(k+1)-kmeshac_2(k), with k=(i-1)*ny+j
!                         Part 3: length nnodes contains the node numbers
!                               in the sequence defined by part 2
!     kmeshc         i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     kmshd1         i    first part of kmesh part d, contains the number of
!                         elements corresponding to each nodal point accumulated
!                         kmshd1(1) = 0.
!                         Length: npoint+1
!     kmshd2         i    second part of kmesh part d
!                         Contains all the element numbers the nodal points are
!                         lying in sequentially
!     nvertices      i    Number of vertices
!     rmeshac        i    Real array of length 2 x ndim x nsplit
!                         contains xmin, xmax, ymin, ymax, zmin zmax
!                         for each subpart
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision phi(8), x(8), y(8), z(8), xmin, xmax, ymin, ymax, &
                       zmin, zmax, eps, distx, disty,  distz, xsi0, &
                       eta0, zeta0, xlocmax, xlocmin, &
                       ylocmax, ylocmin, zlocmax, zlocmin, &
                       xcoord, ycoord, zcoord
      integer          i, j, k, nodp, nodp1, nx, ny, indi, indj, indk, &
                       nelem, npoint, nnodes, istartpart2, istart, indl, &
                       indilow, inditop, indjlow, indjtop, l, icount, &
                       index1(8), ivertx(8), icell, jcell, kcell, &
                       indklow, indktop, nz
      logical          found

!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     distz          Distance in z-direction
!     eps            Local accuracy
!     eta0           Value of eta in previous iteration
!     found          If true already a possibility has been found
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     icount         Counter
!     index1         Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     indi           i-index
!     indilow        lower i-index
!     inditop        upper i-index
!     indj           j-index
!     indjlow        lower j-index
!     indjtop        upper j-index
!     indk           k-index
!     indklow        lower k-index
!     indktop        upper k-index
!     indl           l-index
!     istart         First position used at a part
!     istartpart2    First position used at part 2
!     ivertx         Array containing the position of the vertices in the
!                    elements
!     j              Counting variable
!     jcell          Cell number of cell under consideration
!     k              Counting variable
!     kcell          Cell number of cell under consideration
!     l              General loop variable
!     nelem          Number of elements
!     nnodes         Number of nodes
!     nodp           Nodal point sequence number
!     nodp1          Node sequence number
!     npoint         Number of nodal points
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     x              x co-ordinate of point
!     xcoord         x-coordinate of point
!     xlocmax        local maximum x coordinate
!     xlocmin        local minimum x coordinate
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xsi0           Value of xsi in previous iteration
!     y              y co-ordinate of point
!     ycoord         y-coordinate of point
!     ylocmax        local maximum y coordinate
!     ylocmin        local minimum y coordinate
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zcoord         z-coordinate of point
!     zeta0          Value of zeta in previous iteration
!     zlocmax        local maximum z coordinate
!     zlocmin        local minimum z coordinate
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP630INTPOL   Compute the values of the basis functions in a given
!                    point for a linear tetrahedron
!     ELP632INTPOL   Compute the values of the basis functions in a given
!                    point for a tri-linear hexahedron
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2150   no element found containg this point
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork1
!    Compute cell-index (i,j) in which point (x,y) is lying
!    finished := false
!    for k := 0 (1) nx, while ( not finished ) do
!       Find all nodes in cell (i+/-k,j+/-k), store in iwork
!       Find all elements in element groups ielgrp1 to ielgrp2 corresponding
!       to these nodes, store in iwork2
!       For all elements in the group, while ( not finished ) do
!         if ( point in element ) then
!            ielemfound := element number
!            finished := true
!
!    To check if a point is inside an element we distinguish between
!    triangles and quadrilaterals
!
!    Triangles:
!
!    The three vertices define three linear basis functions phi_i(x,y)
!    with:
!    phi_i(x,y) = a_0 + a_1 x + a_2 y
!    The computation of these basis functions is compted in elp620intpol
!    The point (xcoord,ycoord) is in the triangle if
!    0 <=phi_i(xcoord,ycoord) <= 1
!
!    Quadrilaterals:
!
!    The four vertices define four bi-linear basis functions phi_i(x,y)
!    The point xsi, eta defines the position of (xcoord,ycoord) in the
!    reference square
!    This is computed in elp621intpol
!    If xsi and eta in the interval (0,1) then (x,y) is in the quadrilateral
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindelm3' )

!     --- Store coordinates of point in (xcoord,ycoord,zcoord)

      xcoord = xgauss(igaussp,1)
      ycoord = xgauss(igaussp,2)
      zcoord = xgauss(igaussp,3)

!     --- Clear array iwork1

      nelem = kmesh(9)
      do i = 1, nelem
         iwork1(i) = 0
      end do

!     --- Compute cell-index (i,j) in which point (x,y) is lying

      xmin = rmeshac(1,isplit)
      xmax = rmeshac(2,isplit)
      ymin = rmeshac(3,isplit)
      ymax = rmeshac(4,isplit)
      zmin = rmeshac(5,isplit)
      zmax = rmeshac(6,isplit)
      eps = 1d-6
      nx = kmeshac((isplit-1)*5+2)
      ny = kmeshac((isplit-1)*5+3)
      nz = kmeshac((isplit-1)*5+4)
      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny
      distz = (zmax-zmin)/nz
      indi = min(nx,int((xcoord-xmin)/distx)+1)
      indj = min(ny,int((ycoord-ymin)/disty)+1)
      indl = min(nz,int((zcoord-zmin)/distz)+1)
      istart = kmeshac((isplit-1)*5+5)
      if ( inpelm.le.8 ) then

!     --- Linear elements

         do k = 1, nvertices
            ivertx(k) = k
         end do

      else if ( nvertices.eq.8 ) then

!     --- Triquadratic hexahedron

         do k = 1, nvertices
            ivertx(k) = ivertx14(k)
         end do

      else

!     --- Quadratic tetrahedron

         do k = 1, nvertices
            ivertx(k) = ivertx12(k)
         end do

      end if

      do k = 0, nx-1

!     --- for k := 0 (1) nx-1, while ( not finished ) do
!         Find all nodes in cell (i+/-k,j+/-k), store in iwork

         indilow = max(1,indi-k)
         inditop = min(nx,indi+k)
         indjlow = max(1,indj-k)
         indjtop = min(ny,indj+k)
         indktop = min(nz,indl+k)
         indklow = max(1,indl-k)
         nnodes = 0
         do icell = indilow, inditop
            do jcell = indjlow, indjtop
               do kcell = indklow, indktop
                  indk = (kcell-1)*nx*ny + (jcell-1)*nx+icell
                  istartpart2 = kmeshac(istart+indk-1)
                  npoint = kmeshac(istart+indk)-istartpart2
                  do icount = 1, npoint
                     iwork(nnodes+icount) = &
                        kmeshac(istartpart2+icount-1)
                  end do
                  nnodes = nnodes+npoint
               end do
            end do
         end do
         nelem = 0

         do i = 1, nnodes

!        --- Find all elements in element groups ielgrp1 to ielgrp2
!            corresponding to these nodes, store in iwork2

            nodp1 = iwork(i)
            do j = kmshd1(nodp1)+1, kmshd1(nodp1+1)
               ielem = kmshd2(j)
               if ( ielem.ge.ielemlow .and. ielem.le.ielemtop .and. &
                    iwork1(ielem).eq.0 ) then
                  nelem = nelem+1
                  iwork1(ielem) = 1
                  iwork2(nelem) = ielem
               end if
            end do

!           --- Clear array iwork1

            do j = 1, nelem
               iwork1(iwork2(j)) = 0
            end do

!           --- For all elements in the group, while ( not finished ) do
!               if ( point in element ) then
!                  ielemfound := element number
!                  finished := true

            do j = 1, nelem
               ielem = iwork2(j)
               xlocmin = xmax
               xlocmax = xmin
               ylocmin = ymax
               ylocmax = ymin
               zlocmin = zmax
               zlocmax = zmin
               do l = 1, nvertices
                  index1(l) = kmeshc(ivertx(l),ielem-ielemlow+1)
                  nodp = index1(l)
                  x(l) = coor(1,nodp)
                  y(l) = coor(2,nodp)
                  z(l) = coor(3,nodp)
                  xlocmin = min(xlocmin,x(l))
                  xlocmax = max(xlocmax,x(l))
                  ylocmin = min(ylocmin,y(l))
                  ylocmax = max(ylocmax,y(l))
                  zlocmin = min(zlocmin,z(l))
                  zlocmax = max(zlocmax,z(l))
                  index1(l) = nodp
               end do
               if ( xcoord.lt.xlocmin .or. xcoord.gt.xlocmax .or. &
                    ycoord.lt.ylocmin .or. ycoord.gt.ylocmax .or. &
                    zcoord.lt.zlocmin .or. zcoord.gt.zlocmax ) go to 100
               if ( nvertices.eq.4 ) then

!              --- tetrahedra

                  call elp630intpol ( coor, index1, phi, xcoord, &
                                      ycoord, zcoord )
                  found = phi(1).gt.-eps .and. phi(1).le.1d0+eps .and. &
                          phi(2).gt.-eps .and. phi(2).le.1d0+eps .and. &
                          phi(3).gt.-eps .and. phi(3).le.1d0+eps .and. &
                          phi(4).gt.-eps .and. phi(4).le.1d0+eps

               else

!              --- hexahedra
!                  Compute factors for trilinear interpolation
!                  x = a0 + a1 xsi + a2 eta + a3 zeta + a4 xsi eta zeta

                  call elp632intpol ( coor, index1, phi, xcoord, ycoord, &
                                      zcoord, xsi0, eta0, zeta0 )
                  found = xsi0.gt.-eps .and. xsi0.le.1d0+eps .and. &
                          eta0.gt.-eps .and. eta0.le.1d0+eps .and. &
                          zeta0.gt.-eps .and. zeta0.le.1d0+eps

               end if
               if ( found ) go to 1000

100         end do
         end do

      end do
      call errint ( ielemlow, 1 )
      call errint ( ielemtop, 2 )
      call erreal ( xcoord, 1 )
      call erreal ( ycoord, 2 )
      call errsub ( 2150, 2, 2, 0 )

1000  call erclos ( 'prfindelm3' )

      end
