      subroutine probmerge ( kprob1, kprob2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Merge arrays kprob1 and kprob2 into a new array kprob1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob1(*), kprob2(*)

!     kprob1        i/o   Standard SEPRAN array containing information of the
!                         problems
!                         At input the original kprob, at output extended
!                         with kprob2
!     kprob2         i    Contains problem description of new problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nprob, ipkprb, lastkprob, lastkprob2
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkprb         Starting address of array KPROB1 for actual problem
!                    number - 1
!     lastkprob      Last position in array kprob1
!     lastkprob2     Last position in array kprob2
!     nprob          Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIPRB         Get value of ipkprb as function of iprob
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probmerge' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probmerge'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nprob = kprob1(40) ! number of problems
      ipkprb = iniprb( nprob, kprob1 )    ! last address of problem nprob-1
      lastkprob = kprob1(ipkprb+3)        ! length of problem nporb
      ipkprb = lastkprob+ipkprb           ! last address of problem nprob

      kprob1(40) = kprob1(40)+1  ! raise nprob
      lastkprob2 = kprob2(3)
      call inichkarlen ( ipkprb+lastkprob2, kprob1(1), 'kprob1' )
      if ( ierror/=0 ) go to 1000
      kprob1(ipkprb+1) = kprob1(1)-ipkprb
      kprob1(ipkprb+2:ipkprb+lastkprob2) = kprob2(2:lastkprob2)
      if ( debug ) then
         write(irefwr,1) 'lastkprob, lastkprob2', lastkprob, lastkprob2
         call prinin ( kprob1, 40, 'kprob1' )
      end if  ! ( debug )

1000  call erclos ( 'probmerge' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probmerge'

      end if  ! ( debug )

      end
