      function prvalnode ( ibuffr, buffer, isol, kprob, node )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the solution vector in a given node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     value
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(*), kprob(*), node
      double precision prvalnode, buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     node           i    Node number
!     prvalnode      o    value of the solution vector in node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol, ipkprf
      integer nunkp, nphys, indprf, indprp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipkprf         Starting address of array KPROB part f
!     ipusol         Starting address of solution array
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI061         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3037   More than one degree of freedom in point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvalnode' )
      debug = .false. .and. ioutp>=0
      prvalnode = 0d0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvalnode'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and corresponding arrays

      call ini061 ( ibuffr, isol, kprob, indprf, indprp, &
                    nunkp, nphys )
      if ( ierror/=0 ) go to 1000
      ipusol = inidgttext ( isol(1), 'usol' )

      if ( indprf==0 ) then

!     --- indprf = 0, hence constant number of unknowns

         if ( nunkp/=1 ) then

!        --- Error: more than one degree of freedom in node

            call errint ( node, 1 )
            call errsub ( 3037, 1, 0, 0 )
            go to 1000

         end if  ! ( nunkp/=1 )
         prvalnode = buffer(ipusol-1+node)

      else

!     --- indprf>0, hence non-constant number of unknowns

         ipkprf = inigettext ( indprf, 'kprobf' )
         ipkprf = ipkprf+node-1
         nunkp = ibuffr(ipkprf+1)-ibuffr(ipkprf)

         if ( nunkp/=1 ) then

!        --- Error: more than one degree of freedom in node

            call errint ( node, 1 )
            call errsub ( 3037, 1, 0, 0 )
            go to 1000

         end if  ! ( nunkp/=1 )

         prvalnode = buffer(ipusol+ibuffr(ipkprf))

      end if  ! ( indprf==0 )

1000  call erclos ( 'prvalnode' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvalnode'

      end if  ! ( debug )

      end
