      subroutine putnelm ( kmesh, ielgrp, inelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put number of elements in lement group in kmesh part g
!
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ielgrp, inelem

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     kmesh         i/o   Standard SEPRAN array containing information of the
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kelmg, nelgrp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'putnelm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from putnelm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nelgrp = kmesh(5)
      kelmg = kmesh(21)

      if ( nelgrp==1 ) then

!     --- Only one element group

         kmesh(9) = inelem

      else

!     --- More element groupw

         kmesh(kelmg+ielgrp-1) = inelem

      end if  ! ( nelgrp==1 )

1000  call erclos ( 'putnelm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End putnelm'

      end if  ! ( debug )

      end
