      subroutine readb3 ( ibuffr, numarr, kprob, kmesh )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.3  date 15-11-2012 Long integers
!        version  4.2  date 04-04-2001 Corrections for more problems
!        version  4.1  date 27-06-2000 New definitions
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    readb3 reads array  kprob from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cfile3f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, kmesh(*), kprob(*), ibuffr(*)

!    ibuffr   i/o  Integer buffer array in which all large data is stored
!    kmesh     i   Standard SEPRAN array containing information of the mesh
!    kprob     o   Standard SEPRAN array containing information of the problem
!                  definition. See the Programmer's Guide 13.2 for a complete
!                  description
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) nbufol, ipwork, istart, ipkprm
      integer length, index, lenglc, kprbe1, &
              k, lengol, nprob, iprob, npoint, kelgrp, ipkprb, &
              nunkp, nsuper, nphys, ivec, kprobe, &
              isuper, ndtvec
      double precision dum(1)
      character *9 text

!     dum     Dummy array
!     index   Record number with respect to file 2 from which array
!             must be read
!     ipkprb  Starting address of array KPROB for actual problem number
!     ipkprm  Starting address of kprob part m in array IBUFFR
!     iprob   Problem number
!     ipwork  Starting position of part of array in IBUFFR
!     istart  Start position in array IBUFFR with respect to part of KMESH
!     isuper  Super element number
!     ivec    Indication of the structure of a vector of special structure
!     k       General loop variable
!     kelgrp  Sum over the nodes per element group for all element groups
!     kprbe1  Address in KPROB part e
!     kprobe  Starting address of KPROB part e
!     lenglc  Length of local array
!     lengol  Help variable to store KPROB(1) as stored by the user
!     length  Length of array KPROB and all corresponding arrays
!     nbufol  Original value of nbuffr
!     ndtvec  Number of vectors of special structure (type 115)
!     nphys   Number of physical unknowns, or maximal degrees of freedom
!             per point
!     npoint  Number of nodal points
!     nprob   Number of problems stored in array KPROB
!     nsuper  Number of super elements
!     nunkp   Maximal number of degrees of freedom in a point
!     text    String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INI000  Copy integer array to other integer array possibly itself
!     INI069  Reserve space at the end of IBUFFR for a semi-permanent array
!     INI081  Create permanent space in array IBUFFR
!     INIGET  Compute pointer of array in IBUFFR
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
!     READFL  Read array from a direct access SEPRAN file
!     WRITFL  Write array to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      88   KPROB(2) # 101
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb3' )
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

!     --- Reserve space in IBUFFR

      call ini069 ( ibuffr, ipwork, length, nbufol )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      istart = ipwork
      call readfl ( index, 1, length, 1, 2, ibuffr(ipwork) )
      if ( ierror/=0 ) go to 1000

!     --- Replace all information
!         First KPROB

      if ( ibuffr(ipwork+1)/=101 ) then
         call errint ( ibuffr(ipwork+1), 1)
         call errsub ( 88, 1, 0, 0)
         go to 1000
      end if

      nprob = max ( 1, ibuffr(ipwork+39) )
      length = 0
      do iprob = 1, nprob
         length = length+ibuffr(ipwork+2+length)
      end do

!     --- Save KPROB(1)

      lengol = kprob(1)
      call prinlg ( 'kprob', length, dum, kprob )
      if ( ierror/=0 ) go to 1000
      call ini000 ( ibuffr(ipwork), kprob, 1, 1, length, 1 )
      istart = istart + length

!     --- Loop over problems

      npoint = kmesh( 8)
      kelgrp = kmesh(14)
      ipkprb = 0
      do iprob = 1,nprob

!     --- Compute important constants

         nunkp  = kprob(ipkprb+4)
         ndtvec = kprob(ipkprb+6)
         nsuper = kprob(ipkprb+29)
         nphys  = kprob(ipkprb+33)

!        --- Kprob part e

         if ( ndtvec>0 ) then

!        --- ndtvec>0  kprob part e

            do ivec = 1,ndtvec
               kprbe1 = ipkprb+kprob(ipkprb+18)-1+kelgrp*ndtvec+ivec
               kprobe = kprob(kprbe1)
               if ( kprobe>0 ) then

!              --- kprobe>0  hence part in ibuffr or on file 3

                  lenglc = ibuffr(istart+1)
                  istart = istart + 2
                  kprob(kprbe1) = 0
                  write(text,110) ivec
110               format('kprobe', i3.3)
                  call ini081 ( ibuffr, kprob(kprbe1), lenglc, text )
                  ipwork = iniget(kprob(kprbe1))
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                lenglc, 1 )
                  istart = istart + lenglc

               end if

            end do

         end if

!        --- Kprob parts f, g

         do k = 19, 20

            if ( kprob(ipkprb+k)>0 ) then

               lenglc = ibuffr(istart+1)
               istart = istart + 2
               kprob(ipkprb+k) = 0
               if ( k==19 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobf' )
               else
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobg' )
               end if
               ipwork = iniget(kprob(ipkprb+k))
               if ( ierror/=0 ) go to 1000
               call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                             lenglc, 1 )
               istart = istart + lenglc

            end if

         end do

!        --- Kprob part i

         k = 22

         if ( kprob(ipkprb+k)>0 ) then

            lenglc = ibuffr(istart+1)
            istart = istart + 2
            kprob(ipkprb+k) = 0
            call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                          'kprobi' )
            ipwork = iniget(kprob(ipkprb+k))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc

         end if

!        --- Kprob parts k, l and m

         if ( nsuper>0 ) then

            lenglc = nsuper
            kprob(ipkprb+24) = 0
            call ini081 ( ibuffr, kprob(ipkprb+24), lenglc, 'kprobk' )
            ipwork = iniget(kprob(ipkprb+24))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc
            lenglc = nsuper
            kprob(ipkprb+25) = 0
            call ini081 ( ibuffr, kprob(ipkprb+25), lenglc, 'kprobl' )
            ipwork = iniget(kprob(ipkprb+25))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc
            lenglc = 2*nsuper
            kprob(ipkprb+26) = 0
            call ini081 ( ibuffr, kprob(ipkprb+26), lenglc, 'kprobm' )
            ipwork = iniget(kprob(ipkprb+26))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc
            ipkprm = ipwork
            do isuper = 1,nsuper
               length = ibuffr(ipkprm+2*isuper-2)
               index = ifree3
               ibuffr(ipkprm+2*isuper-1) = index
               call writfl(index,1,length,1,3,ibuffr(istart) )
               istart = istart+length
            end do

         end if
         if ( ierror/=0 ) go to 1000

!        --- Kprob part n

         if ( kprob(ipkprb+27)>0 ) then

            lenglc = ibuffr(istart)
            istart = istart + 1
            kprob(ipkprb+27) = 0
            call ini081 ( ibuffr, kprob(ipkprb+27), lenglc, 'kprobn' )
            ipwork = iniget(kprob(ipkprb+27))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc

         end if

!        --- Kprob part p

         if ( kprob(ipkprb+35)>0 ) then

            lenglc = npoint*max(nphys,nunkp)
            kprob(ipkprb+35) = 0
            call ini081 ( ibuffr, kprob(ipkprb+35), lenglc, 'kprobp' )
            ipwork = iniget(kprob(ipkprb+35))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc

         end if

!        --- Kprob parts r-s, iinput and rinput

         do k = 36, 38

            if ( kprob(ipkprb+k)>0 ) then

               lenglc = ibuffr(istart)
               istart = istart + 1
               kprob(ipkprb+k) = 0
               if ( k==36 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobq' )
               else if ( k==37 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobr' )
               else
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobs' )
               end if
               ipwork = iniget(kprob(ipkprb+k))
               if ( ierror/=0 ) go to 1000
               call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                             lenglc, 1 )
               istart = istart + lenglc

            end if

         end do

         do k = 41, 44

            if ( kprob(ipkprb+k)>0 ) then

               lenglc = ibuffr(istart)
               istart = istart + 1
               kprob(ipkprb+k) = 0
               if ( k==41 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'iinput-pr' )
               else if ( k==42 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'rinput-pr' )
               else if ( k==43 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobt' )
               else
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobu' )
               end if
               ipwork = iniget(kprob(ipkprb+k))
               if ( ierror/=0 ) go to 1000
               call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                             lenglc, 1 )
               istart = istart + lenglc

            end if

         end do

         k = 46
         if ( kprob(ipkprb+k)>0 ) then

            lenglc = ibuffr(istart)
            istart = istart + 1
            kprob(ipkprb+k) = 0
            call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, 'kprobv' )
            ipwork = iniget(kprob(ipkprb+k))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc

         end if

         do k = 50, 51

            if ( kprob(ipkprb+k)>0 ) then

               lenglc = ibuffr(istart)
               istart = istart + 1
               kprob(ipkprb+k) = 0
               if ( k==50 ) then
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kprobh' )
               else
                  call ini081 ( ibuffr, kprob(ipkprb+k), lenglc, &
                                'kproby' )
               end if
               ipwork = iniget(kprob(ipkprb+k))
               if ( ierror/=0 ) go to 1000
               call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                             lenglc, 1 )
               istart = istart + lenglc

            end if

         end do

         ipkprb = ipkprb + kprob(3+ipkprb)

      end do

1000  if ( ierror/=0 ) call instop

      call erclos ( 'readb3' )

!     --- Reset kprob(1)

      kprob(1) = lengol
      nbuffr = nbufol
      end
