      subroutine readco
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.13 date 06-12-2013 Extension with strings and int vars
!        version  1.12 date 11-10-2013 Use while
!        version  1.11 date 26-02-2011 Extension with strings
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read the list of constants and store the results in common blocks
!     cuscons and cusname
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/cpackagef90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, &
              iwork(maxints+maxreals+maxcls+maxvec+maxstrings+ &
                    maxintscals)
      logical finished, keyint, keyreal, keyvar, keynames, debug, &
              keystrings, keyintscals

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     iseq           Sequence number of vector in VECOLD/ISLOLD
!     iwork          integer work array
!     keyint         If true the keyword INTEGERS has been read
!     keyintscals    If true the keyword INT_VARIABLES has been read
!     keynames       If true the keyword VECTOR_NAMES has been read
!     keyreal        If true the keyword REALS has been read
!     keystrings     If true the keyword STRINGS has been read
!     keyvar         If true the keyword VARIABLES has been read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     INICHNAM       Check uniqueness of names in CUSNAMES
!     READCOINT      Read part integers of input block CONSTANTS
!     READCOINTVAR   Read part integer variables of input block CONSTANTS
!     READCONAME     Read part vector names of input block CONSTANTS
!     READCOREAL     Read part reals of input block CONSTANTS
!     READCOSTRING   Read part strings of input block CONSTANTS
!     READCOVAR      Read part variables of input block CONSTANTS
! **********************************************************************
!
!                       I/O
!
!    This subroutine reads all input between the keywords CONSTANTS and END
!    It is assumed that the keyword CONSTANTS has already been read.
!    The following input is recognized:
!    CONSTANTS
!       INTEGERS
!          1: name = value
!          3: name
!             name = value
!       REALS
!          1: name = value
!          3: name = value
!             name = value
!       STRINGS
!          1: name = value
!          3: name = value
!             name = value
!       VARIABLES
!          1: name = value
!          3: name = value
!             name
!       INT_VARIABLES
!          1: name = value
!          3: name = value
!             name
!       VECTOR_NAMES
!          1: name
!          3: name
!             name
!    END
!    A name of a constant consists of a sequence of letters, digits and
!    underscores.
!    All other characters including blank or end of line are treated as end
!    of name.
!    Only the first 32 characters are stored
!    The name must always start with a letter
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
!    1868   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
!   finished    := false
!   keyint   := false
!   keyreal  := false
!   keyvar   := false
!   keynames := false
!   keystrings := false
!   ispecl  := 9
!   while ( not finished ) do
!      read next record
!      interpret next record
!      if ( main keyword found ) then
!         finished := true
!      else if ( keyword = integers ) then
!         keyint := true
!         keyreal := false
!         keyvar  := false
!         keynames := false
!         keystrings := false
!      else if ( keyword = reals ) then
!         keyint := false
!         keyreal := true
!         keyvar  := false
!         keynames := false
!         keystrings := false
!      else if ( keyword = variables ) then
!         keyint := false
!         keyreal := false
!         keyvar  := true
!         keynames := false
!         keystrings := false
!      else if ( keyword = vector_names ) then
!         keyint := false
!         keyreal := false
!         keyvar  := false
!         keynames := true
!         keystrings := false
!      else if ( keyword = strings ) then
!         keyint := false
!         keyreal := false
!         keyvar  := false
!         keynames := false
!         keystrings := true
!      else if ( keyint ) then
!         put information into integer part
!      else if ( keyreal ) then
!         put information into real part
!      else if ( keyvar ) then
!         put information into scalar part
!      else if ( keynames ) then
!         put information into names (common cpost5)
!      else if ( keystrings ) then
!         put information into text part
!   ispecl := 0
!   Check uniqueness of integers, reals and names
!   Check uniqueness of variables
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readco' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readco'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make some initializations

      finished = .false.
      keyint   = .false.
      keyreal  = .false.
      keyvar   = .false.
      keynames = .false.
      keystrings = .false.
      keyintscals = .false.
      numvec   = 0
      iwork = 0

      do while ( .not. finished )

!     --- While loop
!         read and interpret next record

         ityprd = 1
         ispecl = 9
         call asknxt
         if ( ityprd==2 ) then

!        --- sequence number found, interpret and read rest

            iseq = nint(realrd)
            if ( debug ) write(irefwr,1) 'iseq', iseq
            call asknxt

         else

            iseq = 0

         end if
         if ( ityprd==4 ) then

!        --- keyword found, check type of keyword

            if ( debug ) write(irefwr,*) 'string ', string(1:nchar)
            if ( string(1:3)=='END' ) then

!           --- end found

               finished = .true.

            else if ( nchar<=8 .and. string(1:7)=='INTEGER' ) then

!           --- keyword = integers

               keyint  = .true.
               keyreal = .false.
               keyvar  = .false.
               keynames = .false.
               keystrings = .false.
               keyintscals = .false.

            else if ( nchar<=5 .and. string(1:4)=='REAL' ) then

!           --- keyword = reals

               keyint  = .false.
               keyreal = .true.
               keyvar  = .false.
               keynames = .false.
               keystrings = .false.
               keyintscals = .false.

            else if ( string(1:6)=='STRING' ) then

!           --- keyword = reals

               keyint  = .false.
               keyreal = .false.
               keyvar  = .false.
               keynames = .false.
               keystrings = .true.
               keyintscals = .false.

            else if ( string(1:8)=='VARIABLE' .or. &
                      string(1:7)=='SCALARS' ) then

!           --- keyword = variables

               keyint  = .false.
               keyreal = .false.
               keyvar  = .true.
               keynames = .false.
               keystrings = .false.
               keyintscals = .false.

            else if ( string(1:7)=='INT_VAR' .or. &
                      string(1:8)=='INT_SCAL' .or. &
                      string(1:12)=='INTEGER_SCAL' .or. &
                      string(1:11)=='INTEGER_VAR' ) then

!           --- keyword = integer variables

               keyint  = .false.
               keyreal = .false.
               keyvar  = .false.
               keynames = .false.
               keystrings = .false.
               keyintscals = .true.

            else if ( string(1:8)=='VECTOR_N' ) then

!           --- keyword = variables

               keyint  = .false.
               keyreal = .false.
               keyvar  = .false.
               keynames = .true.
               keystrings = .false.
               keyintscals = .false.

            else if ( keyint ) then

!           --- name of integer variable found

               call readcoint ( iseq, iwork )

            else if ( keyreal ) then

!           --- name of real variable found

               call readcoreal ( iseq, iwork )

            else if ( keyintscals ) then

!           --- name of integer variable found

               call readcointvar ( iseq, iwork )

            else if ( keyvar ) then

!           --- name of variable found

               call readcovar ( iseq, iwork )

            else if ( keynames ) then

!           --- name of vector found

               call readconame ( iseq, iwork )

            else if ( keystrings ) then

!           --- strings found

               call readcostring ( iseq, iwork )

            else

!           --- keyword not recognized

               call errchr ( string(1:nchar), 1 )
               call errchr ( 'INTEGERS', 2 )
               call errchr ( 'REALS', 3 )
               call errchr ( 'VECTOR_NAMES', 4 )
               call errchr ( 'VARIABLES', 5 )
               call errchr ( 'STRINGS', 6 )
               call errsub ( 648, 0, 0, 6 )
               call errcrd

            end if

         else

!        --- Record could not be interpreted

            call errsub ( 1868, 0, 0, 0 )
            call errcrd

         end if
         if ( ierror/=0 ) go to 1000

      end do  ! while ( .not. finished )

      ispecl = 0

!     --- Check uniqueness of integers and real names
!         Do the same for the variable names

      call inichnam

1000  call erclos ( 'readco' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'namevfound', namevfound
         write(irefwr,*) 'End readco'

      end if  ! ( debug )

      end
