      subroutine readconstant ( iwork, icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 10-04-2014 New call to readstrfunc
!        version  1.6  date 30-06-2011 Extension with abs
!        version  1.5  date 29-02-2008 Replace part of body by call subroutine
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read the name of a constant of function behind the \ sign and set ityprd
!     as well as realrd
!     The usual rules for separating names are valid
!     The name of the constant must be one of the defined names
!     At this moment the following names are allowed:
!     pi
!     The name of the function must be one of the defined names
!     At this moment the following names are allowed:
!     sqrt, sin , cos, exp, tan, atan, acos, asin
!     If this subroutine is adapted also change readexpr
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/constaf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), icount

!     icount         i    Counter
!     iwork         i/o   integer work array of length icount
!                         iwork contains the operators that precede the numbers
!                         according to the following code:
!                          -2:  division
!                          -1:  subtraction
!                           1:  addition
!                           2:  multiplication
!                           3:  raise to the power
!                          10:  sqrt
!                          11:  sin
!                          12:  cos
!                          13:  exp
!                          14:  tan
!                          15:  asin
!                          16:  acos
!                          17:  atan
!                          18:  log
!                          19:  abs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncode, ncode1
      parameter ( ncode=1, ncode1=13 )
      integer minusg, numfun, i
      character (len=10) code(ncode), code1(ncode1)
      logical debug, found

!     code           Table with keywords corresponding to special constants
!     code1          Table with keywords corresponding to special functions
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     found          If true already a possibility has been found
!     i              Counting variable
!     minusg         Indicates if a minus sign has been found (-1) or
!                    not (1)
!     ncode          Number of entries in code
!     ncode1         Number of entries in code1
!     numfun         Number of constants or functions checked
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     READNAME1      read the name of a constant behind the $ or % sign
!     READSTRFUNC    Check if name read corresponds to one the known functions
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1608   Name found after $ or \ contains more than 32 characters
!    1898   No opening bracket found
!    2743   The string following \ is not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'PI' /
      data code1 / 'SQRT', 'SIN', 'COS', 'EXP', 'TAN', 'ASIN', 'ACOS', &
                   'ATAN', 'LOG', 'ABS', 'ARCSIN', 'ARCCOS', 'ARCTAN' /
! ======================================================================
!
      call eropen ( 'readconstant' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readconstant'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read name following \-sign and detect if it is preceded by a - sign

      call readname1 ( minusg )
      if ( ierror/=0 ) go to 1000

!     --- name has been found, check the length

      if ( nchar>32 ) then
         call errchr ( string(1:nchar), 1 )
         call errint ( nchar, 1 )
         call errwar ( 1608, 1, 0, 1 )
         call errcrd
         nchar = 32
      end if

!     --- First the integers
!         The constants first

      found = .false.
      numfun = 0       ! number of constants checked
      do while ( numfun<ncode .and. .not. found )

!     --- Constant not yet found, raise numfun

         numfun = numfun+1
         if ( string(1:nchar)==code(numfun)(1:nchar) ) then

!        --- Constant found

            found = .true.
            if ( numfun>10 ) numfun = numfun - 5 ! reduce arc to a

         end if  ! ( string(1:nchar)==code(numfun)(1:nchar) )

      end do  ! while ( numfun<ncode .and. .not. found )

      if ( found ) then

!     --- Constant found

         select case ( numfun )

            case(1)

!           --- Constant pi found

            realrd = pi

         end select  ! case ( numfun )
         if ( minusg<0 ) realrd = -realrd
         ityprd = 2

      else

!     --- Constant not found, check if it is a function

         call readstrfunc ( numfun, string, nchar )

         if ( numfun>0 ) then

!        --- Function found, next item must be an opening bracket

            if ( minusg<0 ) then

!           --- minus sign found

               icount = icount+1
               iwork(icount) = -1

            end if  ! ( minusg<0 )
            if ( icount==1 .and. iwork(1)==0 ) then

!           --- Nothing before function found, insert addition

               iwork(1) = 1             ! add operator

            end if  ! ( icount==1 .and. iwork(1)==0 )

            icount = icount+1
            iwork(icount) = numfun          !  indicates type of operator
            if ( typeitems(lastitem+1)/=5 ) then

!           --- Not a bracket found after the keyword sqrt

               call errchr ( string(1:nchar), 1 )
               call errsub ( 1898, 0, 0, 1 )
               call errcrd

            end if  ! ( typeitems(lastitem)/=5 )
            found = .true.

         end if  ! ( numfun>0 )

      end if  ! ( found )

      if ( .not. found ) then

!     --- Constant not known

         call errchr ( string(1:nchar), 1 )
         do i = 1, ncode
            call errchr ( code(i), i+1 )
         end do  ! i = 1, ncode
         do i = 1, ncode1
            call errchr ( code1(i), i+1+ncode )
         end do  ! i = 1, ncode1
         call errsub ( 2743, 0, 0, 1+ncode+ncode1 )
         call errcrd

      end if

1000  call erclos ( 'readconstant' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount', icount
         call prinin ( iwork, icount, 'iwork' )
         write(irefwr,*) 'End readconstant'

      end if

      end
