      subroutine readexpress4 ( iwork, lenwork, ipositions, ishift )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove brackets around a single number, &
!     i.e. the combination ( + number ) is replaced by number
!     or in case it is the first number: by + number
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), lenwork, ipositions(2,*), ishift

!     ipositions     o    Indicates the positions of openings and closings
!                         brackets in iwork
!                         ipositions(1,i) position of i-th open bracket
!                         ipositions(2,i) position of corresponding close
!                         bracket
!     ishift         i    Defines shift in array iwork, once brackets are
!                         removed
!     iwork         i/o   integer work array, see subroutine readexpressnew
!     lenwork       i/o   Length of array work
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iopen, istart, ipos, j, jshift
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iopen          Number of open brackets that are still active
!                    If an opening bracket is found it is increased by 1, &
!                    if a closing bracket is found it is decreased by 1
!     ipos           Position in array
!     istart         First position used at a part
!     j              Counting variable
!     jshift         shift like ishift
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexpress4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexpress4'
  1      format ( a, 1x, (10i6) )
         call prinin ( iwork, lenwork, 'iwork/start' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iopen = 0
      do i = 1, lenwork-3

         if ( iwork(i)==-3 .and. iwork(i+1)==1 .and. iwork(i+3)==-4 ) &
            then

!        --- superfluous brackets found, mark

            iopen = iopen+1
            ipositions(1,iopen) = i

         end if  ! iwork(i)==-3 .and. iwork(i+1)==1 .and. iwork(i+3)==-4 )

      end do  ! i = 1, lenwork-3

!     --- Remove these brackets

      ishift = 0
      if ( iopen>0 ) then

!     --- iopen>0, hence brackets left

         istart = 0
         do j = 1, iopen

            ipos = ipositions(1,j)
            if ( ipos==1 ) then

!           --- position 1, keep the + &

               iwork(1) = 1
               iwork(2) = iwork(3)
               ishift = 2
               istart = 2

            else

!           --- other positions

               do i = istart+1, ipos-ishift-1
                  iwork(i) = iwork(i+ishift)
               end do  ! i = istart+1, ipos-ishift-1
               istart = ipos-ishift
               if ( iwork(istart-1)==-3 ) then

!              --- iwork(istart-1) = open bracket, must be followed by + &

                  iwork(istart) = 1
                  iwork(istart+1) = iwork(istart+ishift+2)
                  ishift = ishift+2
                  istart = istart+1

               else

!              --- iwork(istart-1) = other value

                  iwork(istart) = iwork(istart+ishift+2)
                  ishift = ishift+3

               end if  ! ( iwork(istart-1)==-3 )

            end if  ! ( ipos==1 )

         end do  ! j = 1, iopen

!        --- Copy final part

         lenwork = lenwork-ishift
         do i = istart+1, lenwork
            iwork(i) = iwork(i+ishift)
         end do  ! i = istart+1, lenwork

      end if  ! ( iopen>0 )

!     --- Next replace parts with ( number ) by number
!         Since ishift is used before and must be used as output parameter
!         we use jshift in this case

      iopen = 0
      do i = 1, lenwork-2

         if ( iwork(i)==-3 .and. iwork(i+2)==-4 ) then

!        --- superfluous brackets found, mark

            iopen = iopen+1
            ipositions(1,iopen) = i

         end if  ! iwork(i)==-3 .and. iwork(i+1)==1 .and. iwork(i+3)==-4 )

      end do  ! i = 1, lenwork-2

!     --- Remove these brackets

      jshift = 0
      if ( iopen>0 ) then

!     --- iopen>0, hence brackets left

         istart = 0
         do j = 1, iopen

            ipos = ipositions(1,j)
            do i = istart+1, ipos-jshift-1
               iwork(i) = iwork(i+jshift)
            end do  ! i = istart+1, ipos-jshift-1
            istart = ipos-jshift
            iwork(istart) = iwork(istart+jshift+1)
            jshift = jshift+2

         end do  ! j = 1, iopen

!        --- Copy final part

         lenwork = lenwork-jshift
         do i = istart+1, lenwork
            iwork(i) = iwork(i+jshift)
         end do  ! i = istart+1, lenwork

      end if  ! ( iopen>0 )

      ishift = jshift+ishift

1000  call erclos ( 'readexpress4' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, lenwork, 'iwork/end' )
         write(irefwr,1) 'ishift', ishift
         write(irefwr,*) 'End readexpress4'

      end if  ! ( debug )

      end
