      subroutine readinclude ( ninclude, irefresav )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 02-04-2008 Fortran 90
!        version  1.0  date 26-12-2002
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Read the file name following the include statement
!      and open include file
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     interpret
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cmachtf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchnewf90'
      include 'SPcommon/creffilesf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefresav, ninclude

!     irefresav     i/o   Help parameter to save the original value of irefre
!     ninclude      i/o   Level of include files
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character *200 filename, filenamecp
      integer lenname, lenhome, ios
      logical check

!     check          Indicates if the file exist (true) or not
!     filename       Name of file
!     filenamecp     Name of file
!     ios            IOSTAT variable, returns from reading or writing
!     lenhome        Number of characters in the name of the SEPRAN home
!                    directory
!     lenname        Length of name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1317   error in opening of file
!    1669   file does not exist
!    1879   Not a text found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readinclude' )
      if ( nitems<4 .or. typeitems(2)/=4 ) then

!     --- Not a quote found following INCLUDE

         call errchr ( string(1:nchar), 1 )
         call errsub ( 1879, 0, 0, 1 )
         firstcol = lastcol+1
         lastcol = maxcolum
         call errcrd
         go to 1000

      end if

!     --- Store starting position of text in firstcol

      firstcol = firstcolitems(2)
      numrecord = firstcol/maxcolum+1
      if ( numrecord>1 )  firstcol = mod(firstcol,maxcolum)
      firstcol = firstcol+1

!     --- Last item must be end quote and on the same record

      lastcol = firstcolitems(nitems)
      if ( lastcol/maxcolum+1/=numrecord ) then

!     --- Not an end quote found in the same line

         call errchr ( string(1:nchar), 1 )
         call errsub ( 1879, 0, 0, 1 )
         lastcol = maxcolum
         call errcrd
         go to 1000

      end if
      if ( numrecord>1 ) lastcol = mod(lastcol,maxcolum)
      lastcol = lastcol-1
      filename = cardimage(numrecord)(firstcol:lastcol)
      lenname = lastcol-firstcol+1
      if ( filename(1:7)=='SPHOME/' ) then

!     --- First 6 characters are SPHOME, replace by
!         SEPRAN home directory, followed by / &

         lenname = lenname-6
         filenamecp(1:lenname) = filename(7:lenname+6)
         lenhome = index(sphome,' ')-1
         if ( lenhome<=0 ) lenhome = 100
         filename(1:lenhome) = sphome(1:lenhome)
         filename(lenhome+1:lenhome+lenname) = filenamecp(1:lenname)

      end if
      if ( ninclude==0 ) irefresav = irefre
      irefre = inextref+ninclude
      inquire ( file=filename, exist=check )
      if ( .not. check ) then

!     --- File does not exist

         call errchr ( filename, 1 )
         call errsub ( 1669, 0, 0, 1 )
         call errcrd
         go to 1000

      end if
      open ( unit = irefre, file = filename, iostat=ios )
      if ( ios/=0 ) then

!     --- File can not be opened

         call errint ( ios, 1 )
         call errchr ( filename, 1 )
         call errsub ( 1317, 1, 0, 1 )
         call errcrd
         go to 1000

      end if
      ninclude = ninclude+1

1000  call erclos ( 'readinclude' )

      end
