      subroutine readscbounint ( istart, inpsep, tabout, icode,
     +                           nout, strout, iscalinp, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 29-06-2011 Extension with ichint
!        version  2.0  date 15-12-2010 Extra parameters
!        version  1.4  date 13-11-2010 error 1488 --> errkeywexcl
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   BOUNDARY_INTEGRAL
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cinout'
      include 'SPcommon/comcons1'
      include 'SPcommon/comcons2'
      include 'SPcommon/cuscons'
      include 'SPcommon/cusname'
      include 'SPcommon/cpost5'
      include 'SPcommon/cscalvec'
      include 'SPcommon/cwarn'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), icode, nout, iscalinp, kmesh(*)
      double precision tabout(*)
      character (len = *) strout(*)

!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: ISEQNR Sequence number boundary integrals
!                         3: IVECTOR  Sequence number input vector
!                         4: ISCALAR1 Sequence number first scalar of integral
!                         5: ISCALAR2 Sequence number second scalar of integral
!                         6: ISCALAR3 Sequence number third scalar of integral
!                         7  IPLACE   Position where boundary integrals must
!                                     be computed
!                                     1:  curves
!                                     2:  surfaces
!                         8  INUM     Number of curves
!                         9: ICHINT   Type of integral
!                         10:  Not yet used
!                         11-11+INUM    Curve numbers
!     iscalinp       i    Scalar number read before
!                         If 0, no scalar number has been read
!     istart        i/o   Next free position in inpsep
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nout           i    Number of entries in tabout
!     strout        i/o   Contains series of strings
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncode, ncodeextra
      parameter ( nncode=8, ncodeextra=1 )
      integer ncode(nncode), itype(nncode), ncodmx(nncode),
     +        ivec, iretur, iscal, i, nmcode(ncodeextra), m, iposut
      double precision table(nncode), tabmin(nncode), tabmax(nncode)
      character (len=20) code(nncode)
      character (len=1) codemn(ncodeextra)
      logical debug

!     code           Table with keywords
!     codemn         Table with main keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iposut         Position in array tabout
!     iretur         Return value of subroutine INTX02
!                    Possible values:
!                    -2:  end of file has been reached
!                    -1:  error has been found
!                     0:  normal return
!     iscal          Sequence number of scalar
!     itype          Indicates the type of action corresponding to keywords
!                    in code
!     ivec           Sequence number of vector with respect to array ISOL
!     m              number of items stored in tabout
!     ncode          Table containing the number of significant letters in
!                    code
!     ncodeextra     Number of keywords stored in nmcode
!     ncodmx         Table containing the number of maximum characters in
!                    the keyword table code
!     nmcode         Table containing the number of significant characters
!                    in the main keyword table codemn, including those of
!                    the extra texts
!     nncode         Number of entries in code
!     table          Contains values of items read corresponding to code
!     tabmax         Maximum value of items read corresponding to code
!     tabmin         Minimum value of items read corresponding to code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRWAR         Print warnings
!     INTX02         Interpret a record containing keywords followed by numbers
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!       BOUNDARY_INTEGRAL, options (all in one line)
!           where options may have the following contents
!               SEQ_BOUN_INTEGRAL = s (Default the next one)
!               VECTOR = s           (Default 1)
!               SCALAR1 = s          (Default 1)
!               SCALAR2 = s          (Default 1)
!               SCALAR3 = s          (Default 1)
!               CURVES = (Ci, Cj, Ck )
!               ICHINT = i           (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'SEQ_BOUN_INTEGRAL', 'VECTOR', 'SCALAR', 'SCALAR',
     +            'SCALAR', 'V', 'CURVES', 'ICHINT' /
      data ncode  /  6, 6, 1, 1, 1,   1, 3, 3 /
      data ncodmx / 17, 6, 6, 6, 6,   1, 6, 6 /
      data codemn / 'C' /
      data nmcode / 1 /

!     code   Table with sub keywords corresponding to keyword BOUNDARY_INTEGRAL
!             Values:
!             1:  SEQ_BOUN_INTEGRAL
!             2:  VECTOR
!             3:  SCALAR
!             4:  SCALAR
!             5:  SCALAR
!             6:  V
!             7:  CURVES
!             8:  ICHINT
! ======================================================================
!
      call eropen ( 'readscbounint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbounint'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = 0
      tabmin = 1d0
      tabmax = 0d0
      table = -1d0

      tabmax(1) = 1d2                ! SEQ_BOUN_INTEGRAL
      table(1) = 1d0

      tabmax(2) = dble(maxvectors)   ! VECTOR

      itype(3) = 12                  ! SCALAR
      tabmax(3) = 3d0
      table(3) = 1d0

      tabmax(4) = dble(maxscalars)   ! SCALAR
      table(4) = 2d0

      table(5) = 3d0                 ! SCALAR

      tabmax(6) = tabmax(2)          ! VECTOR

      itype(7) = 18                  ! curves
      tabmin(7) = dble(kmesh(11))    ! ncurvs
      tabmax(7) = tabmin(7)          ! ncurvs

      table(8) = 1d0                 ! ichint
      tabmax(8) = 8

      iretur = 0
      call intx02 ( code, ncode, nncode, itype, table, codemn,
     +              nmcode, 0, iretur, tabmin, tabmax, tabout,
     +              nout, strout, 1, ncodmx )
      if ( ierror/=0 ) go to 1000

      if ( table(2)>0d0 .and. table(6)>0d0 )
     +   call errkeywexcl ( code, ncodmx, 2, 6 )

      table(2) = max(1d0,table(2),table(6))
      inpsep(istart) = icode
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = nint(table(2))
      inpsep(istart+4) = mod(nint(table(4)),100000)
      inpsep(istart+5) = mod(nint(table(5)),100000)

      if ( iscalinp==0 ) then

!     --- Standard case, scalar has not been read before

         inpsep(istart+3) = mod(nint(table(3)),100000)

      else

!     --- Scalar number has been read before

         inpsep(istart+3) = iscalinp

      end if  ! ( iscal==0 )
      inpsep(istart+6:istart+8) = 0

      if ( table(7)>0d0 ) then

!     --- Curves found

         iposut = nint(table(7))! Position of curve numbers in tabout
         m = nint(tabmax(7))    ! number of curves read

         inpsep(istart+6) = 1   ! curves
         inpsep(istart+7) = m   ! number of curves
         inpsep(istart+10:istart+9+m) = nint(tabout(iposut:iposut+m-1)) ! curves

      end if  ! ( table(7)>0d0 )
      inpsep(istart+9) = nint(table(8))   ! ichint

      ivec = nint(table(2))
      if ( ivec>0 ) then

         if ( ivectors(ivec)==0 .and. nwarn<mxwarn ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if

      end if
      do i = 1, 3
         iscal = mod(nint(table(2+i)),100000)
         if ( iscal<=maxscalars ) iscalars(iscal) = 1
      end do

      istart = istart + 10 + inpsep(istart+7)

1000  call erclos ( 'readscbounint' )

      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, istart-1, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscbounint'

      end if  ! ( debug )

      end
