      subroutine readseqvec ( value, iinput, defvalue, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 02-01-2014 Correction in case no vectorname
!        version  1.0  date 13-05-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the sequence number read in iinput
!     If no sequence number is found, use default value
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision value
      integer iinput, defvalue
      character (len=*) name

!     defvalue       i    Default value
!     iinput         o    entry of integer input array to be filled with
!                         integer sequence number
!     name           i    Name of parameter for debugging
!     value          i    real value read containing the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqvec'
         write(irefwr,1) 'defvalue', defvalue
         write(irefwr,2) 'value', value
         write(irefwr,*) 'name ', name
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( value>1.99d8 ) then

!     --- vector expression

         iinput = nint(value)
         go to 1000

      else if ( value>0d0 ) then

!     --- vectorname is given

         iinput = mod(nint(value),5000)

      else

!     --- vectorname is not given, use default

         iinput = defvalue  ! default value

      end if  ! ( value(1)>0d0 )

      if ( debug ) write(irefwr,1) 'iinput', iinput

      if ( iinput>0 ) then

!     --- Vector name found

         if ( dimnames(iinput)>1 ) iinput = iinput+5000

      end if  ! ( iinput>0 )

1000  call erclos ( 'readseqvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput', iinput
         write(irefwr,*) 'End readseqvec'

      end if

      end
