      subroutine msh610 ( xn, yn, zn, xm, ym, zm, &
                          x1, y1, z1, x2, y2, z2, &
                          x3, y3, z3, t  )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.1  date 19-10-2006 Text added
!        version  2.0  date 13-04-2005 Update code
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to check whether line n - m has a common point with
!     triangle 1 - 2 - 3
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xn,yn,zn,xm,ym,zm,x1,y1,z1,x2,y2,z2,x3,y3,z3,t

!     t       o    if ( 0<t<1 ) then there is a common
!                  point , else t = 1000
!     x1      i    x-coordinate of vertex 1
!     x2      i    x-coordinate of vertex 2
!     x3      i    x-coordinate of vertex 3
!     xm      i    x-coordinate of point m
!     xn      i    x-coordinate of point n
!     y1      i    y-coordinate of vertex 1
!     y2      i    y-coordinate of vertex 2
!     y3      i    y-coordinate of vertex 3
!     ym      i    y-coordinate of point m
!     yn      i    y-coordinate of point n
!     z1      i    z-coordinate of vertex 3
!     z2      i    z-coordinate of vertex 3
!     z3      i    z-coordinate of vertex 3
!     zm      i    z-coordinate of point m
!     zn      i    z-coordinate of point n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision cval, dval, eval, fval, vpnt1, vpnt2, prod, xs, &
                       ys, zs

!     cval        helpvalue
!     dval        helpvalue
!     eval        helpvalue
!     fval        helpvalue
!     prod        product
!     vpnt1       value point1
!     vpnt2       value point2
!     xs          intersection point on line
!     ys          intersection point on line
!     zs          intersection point on line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH611  Is point (xs,ys,zs) inside angle given by three points
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      t   = 1000d0

      cval = ( z3-z1 )*( y2-y1 ) - ( z2-z1 )*( y3-y1 )
      dval = ( x3-x1 )*( z2-z1 ) - ( x2-x1 )*( z3-z1 )
      eval = ( y3-y1 )*( x2-x1 ) - ( y2-y1 )*( x3-x1 )
      fval = - cval*x1 - dval*y1 - eval*z1

      vpnt1 = cval * xn + dval * yn + eval * zn + fval
      vpnt2 = cval * xm + dval * ym + eval * zm + fval

      prod  = vpnt1 * vpnt2

      if ( prod>=0 ) return

!     --- determine t - value

      t = vpnt2 / ( vpnt2 - vpnt1 )

!     --- Point between points n and m ?

      if ( t<0   ) t = 1000d0
      if ( t>1d0 ) then
           t = 1000d0
           return
      end if

!     --- Point in triangle ?

      xs = xm + t * ( xn - xm )
      ys = ym + t * ( yn - ym )
      zs = zm + t * ( zn - zm )

!     --- Check all angles

      call msh611( x1, y1, z1, x2, y2, z2, x3, y3, z3, xs, ys, zs, t )
      if ( t>1d0 ) return
      call msh611( x2, y2, z2, x3, y3, z3, x1, y1, z1, xs, ys, zs, t )
      if ( t>1d0 ) return
      call msh611( x3, y3, z3, x1, y1, z1, x2, y2, z2, xs, ys, zs, t )
      end
