      subroutine msh613( i1, i2, i3, i4, coor, angle )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.0  date 07-04-2005 Update code
!        version  1.2  date 19-02-1993 New layout
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to compute the angle between triangle i1 - i2 - i3 and
!     triangle i4 - i2 - i3
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     angle
!     mesh
!     3d
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), angle
      integer i1, i2, i3, i4

!     angle   o    cos of angle between triangles
!     coor    i    array of length 3 x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     i1      i    Node considered
!     i2      i    Node considered
!     i3      i    Node considered
!     i4      i    Node considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xi1, yi1, zi1, xi2, yi2, zi2, xi3, yi3, zi3, &
                       xi4, yi4, zi4, e1, e2, e3, e4, e5, e6

!     e1       first  component vector in triangle i1-i2-i3
!     e2       second component vector in triangle i1-i2-i3
!     e3       third  component vector in triangle i1-i2-i3
!     e4       first  component vector in triangle i2-i3-i4
!     e5       second component vector in triangle i2-i3-i4
!     e6       third  component vector in triangle i2-i3-i4
!     xi1      x-coordinate i1
!     xi2      x-coordinate i2
!     xi3      x-coordinate i3
!     xi4      x-coordinate i4
!     yi1      y-coordinate i1
!     yi2      y-coordinate i2
!     yi3      y-coordinate i3
!     yi4      y-coordinate i4
!     zi1      z-coordinate i1
!     zi2      z-coordinate i2
!     zi3      z-coordinate i3
!     zi4      z-coordinate i4
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH614  Subroutine to determine direction (e1,e2,e3) of line in
!             plane of triangle i1 - i2 - i3 and perpendicular to line
!             i1 - i3
! **********************************************************************
!
!                       I/O
!
!    None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Line i2 - i3 is in common
!         Coordinates i1:

      xi1 = coor( 1, i1 )
      yi1 = coor( 2, i1 )
      zi1 = coor( 3, i1 )

!     --- Coordinates i2:

      xi2 = coor( 1, i2 )
      yi2 = coor( 2, i2 )
      zi2 = coor( 3, i2 )

!     --- Coordinates i3:

      xi3 = coor( 1, i3 )
      yi3 = coor( 2, i3 )
      zi3 = coor( 3, i3 )

!     --- Coordinates i4:

      xi4 = coor( 1, i4 )
      yi4 = coor( 2, i4 )
      zi4 = coor( 3, i4 )

!     --- Compute line in plane i1-i2-i3 through i1 and perpendicular to i2-i3:

      call msh614(xi3,yi3,zi3,xi1,yi1,zi1,xi2,yi2,zi2,e1,e2,e3)

!     --- Compute line in plane i2-i3-i4 through i4 and perpendicular to i2-i3:

      call msh614(xi3,yi3,zi3,xi4,yi4,zi4,xi2,yi2,zi2,e4,e5,e6)

!     --- Be sure that 3 nodes in the planes are not on one linesegment:

      if ( abs(e1) + abs(e2) + abs(e3)<0.1d0 .or. &
           abs(e4) + abs(e5) + abs(e6)<0.1d0 ) then

!     --- Set angle:

           angle = -1d0
      else

!     --- Compute angle:

           angle = e1*e4 + e2*e5 +e3*e6
      end if

      end
