      subroutine msh614( xi1, yi1, zi1, xi2, yi2, zi2, xi3, yi3, zi3, &
                         e1, e2, e3 )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 03-09-2010 New release
!        version  1.2  date 07-04-2005 Update code
!        version  1.1  date 30-12-1992 check length of vector e
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine direction of line in plane of triangle
!     i1 - i2 - i3 and perpendicular to line i1 - i3
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     perpendicular
!     mesh
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision e1, e2, e3, xi1, xi2, xi3, &
                       yi1, yi2, yi3, zi1, zi2, zi3

!     e1               first coordinate perpendicular vector
!     e2               second coordinate perpendicular vector
!     e3               third coordinate perpendicular vector
!     xi1              x-coordinate point i1
!     xi2              x-coordinate point i2
!     xi3              x-coordinate point i3
!     yi1              y-coordinate point i1
!     yi2              y-coordinate point i2
!     yi3              y-coordinate point i3
!     zi1              z-coordinate point i1
!     zi2              z-coordinate point i2
!     zi3              z-coordinate point i3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision er, rhs, p, xp, yp, zp

!     er      Euclidian length of vector
!     p       helpvariable to determine local position of point P
!     rhs     right-hand side term for computation
!     xp      x-coordinate
!     yp      y-coordinate
!     zp      z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   deconcatenate name from string of calling subroutines
!     EROPEN   concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('msh614')

      rhs = (xi3-xi1)*(xi2-xi1)+(yi3-yi1)*(yi2-yi1)+(zi3-zi1)*(zi2-zi1)
      p   = (xi3-xi1)*(xi3-xi1)+(yi3-yi1)*(yi3-yi1)+(zi3-zi1)*(zi3-zi1)
      if ( abs(p)<1d-20 ) then
         write(*,*) 'p=0 stop'
         p = 1d-20
         e1 = 0d0
         e2 = 0d0
         e3 = 0d0
         goto 1000
      end if
      p   =  rhs / p

!     --- Point P

      xp = xi1 + p * ( xi3-xi1 )
      yp = yi1 + p * ( yi3-yi1 )
      zp = zi1 + p * ( zi3-zi1 )

      e1 = xi2 - xp
      e2 = yi2 - yp
      e3 = zi2 - zp

!     --- Determine length

      er =  sqrt( e1*e1 + e2*e2 + e3*e3 )

!     --- Normalize vector if not too small

      if ( abs(er)>0 ) then
         e1 =  e1 / er
         e2 =  e2 / er
         e3 =  e3 / er
      end if

1000  continue
      call erclos('msh614')

      end
