      subroutine msh629( i1, i2, i3, istart, ibuur )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 08-09-2010 New release
!        version  2.0  date 15-04-2005 Update code
!        version  1.2  date 17-04-1992 declararions
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine node number i2 of the middle point of line i1 - i3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
!     midpoint
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart(*), ibuur(*), i1, i2, i3

!     i1        i   first node of line to be considered
!     i2        o   middle node to be determined
!     i3        i   last node number on line to be considered
!     ibuur     i   neighbour array
!     istart    i   startaddresses array for ibuur
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jstart, jeind, ib, j

!     ib
!     j
!     jeind
!     jstart
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     determine lowest number and corresponding start number in ibuur
!     and end number in ibuur
!     highest number is ib (i.e. neighbour)
!     run through all neighbours until middle point belonging to ib
!     is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Determine start and end of all neighbours and determine
!         endnode ib of linepiece

      if ( i1<i3 ) then
           jstart = istart(i1)
           jeind  = istart(i1+1) - 1
           ib     = i3
      else
           jstart = istart(i3)
           jeind  = istart(i3+1) - 1
           ib     = i1
      end if

!     --- Determine middle-point number

      do j = jstart, jeind, 2
          if ( ibuur(j)==ib ) then

!         --- Node found

               i2 = ibuur(j+1)

!              --- Ready for this line

               return
          end if
      end do

      end
