      subroutine msh631 ( kstapl, kstap, line, nlind, coor, xn, yn, zn, &
                          jn, itri, npoint, icheck )
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 08-09-2010 New release
!        version  2.6  date 30-08-2010 Comments, layout
!        version  2.5  date 02-04-2008 Accuracy adjustment
!        version  2.4  date 19-10-2006 Sharper accuracy for comparison
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether a line of the boundary has a common
!     point with one of the new triangles.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstapl(3,*), line(2,*), itri(*), kstap, nlind, jn, icheck, &
              npoint
      double precision coor(3,*), xn, yn, zn

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     icheck     o    if a line crosses the new element then
!                     icheck # 0
!                     if no point is "inside" the new element then
!                     icheck = -1 else icheck is nodenumber of point
!                     inside
!     itri       i    array containing for each point ipnt whether
!                     it belongs to the actual boundary , &
!                     ( itri(ipnt)>0 ) or not ( itri(ipnt) = 0 )
!     jn         i    point to be considered for tetrahedron
!     kstap      i    number of triangles in kstapl
!     kstapl     i    actual array with boundary triangles
!     line            help array
!     nlind      i    declared length of line
!     npoint     i    length of array itri
!     xn         i    x- coordinate of point to be considered if
!                     not yet created, i.e. if jn = 0
!     yn         i    y- coordinate of point to be considered if
!                     not yet created, i.e. if jn = 0
!     zn         i    z- coordinate of point to be considered if
!                     not yet created, i.e. if jn = 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a1, a2, a3, b1, b2, b3, c1, c2, c3, d1, d2, d3, &
                       det, eps, eta, ext, ski, t, xb, xd1, xd2, &
                       xd3, xj, xlm, xmax, xmin, xmind, ya, yb, yd1, &
                       yd2, yd3, yj, ylm, ymax, ymaxd, ymin, ymind, za, &
                       zb, zd1, zd2, zd3, zj, zlm, zmax, zmaxd, zmin, &
                       zmind, zta, xa, xmaxd
      integer i1, i2, i3, i4, ia, ib, il, j, je, k1, k2, k3, ks, nline

!     a1             help variable to store x difference
!     a2             help variable to store x difference
!     a3             help variable to store x difference
!     b1             help variable to store y difference
!     b2             help variable to store y difference
!     b3             help variable to store y difference
!     c1             help variable to store z difference
!     c2             help variable to store z difference
!     c3             help variable to store z difference
!     d1             extra difference x variables
!     d2             extra difference y variables
!     d3             extra difference z variable
!     det            determinant value
!     eps            accuracy
!     eta            barycentric coordinate
!     ext            help for barycentric coordinates
!     i1             node number first node
!     i2             node number second node
!     i3             node number third node
!     i4             node number fourth node
!     ia             first node on line
!     ib             second node on line
!     il             loop variable for lines
!     j              loop variable for points
!     je             help variable for indications
!     k1             first node triangle
!     k2             second node triangle
!     k3             third node triangle
!     ks             loopvariable
!     nline          number of lines
!     ski            barycentric coordinate
!     t              parameter along line
!     xa             x-coordinate
!     xb             x-coordinate
!     xd1            x-coordinate
!     xd2            x-coordinate
!     xd3            x-coordinate
!     xj             x-coordinate
!     xlm            x-coordinate
!     xmax           max of x-coordinates
!     xmaxd          blockvalue max of x-coordinates
!     xmin           min of x-coordinates
!     xmind          blockvalue min of x-coordinates
!     ya             y-coordinate
!     yb             y-coordinate
!     yd1            y-coordinate
!     yd2            y-coordinate
!     yd3            y-coordinate
!     yj             y-coordinate
!     ylm            y-coordinate
!     ymax           max of y-coordinates
!     ymaxd          blockvalue max of y-coordinates
!     ymin           min of y-coordinates
!     ymind          blockvalue min of y-coordinates
!     za             z-coordinate
!     zb             z-coordinate
!     zd1            z-coordinate
!     zd2            z-coordinate
!     zd3            z-coordinate
!     zj             z-coordinate
!     zlm            z-coordinate
!     zmax           max of z-coordinates
!     zmaxd          blockvalue max of z-coordinates
!     zmin           min of z-coordinates
!     zmind          blockvalue min of z-coordinates
!     zta            barycentric coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSH610         Determine intersection of triangle and line
!     MSH636         Compute determinant of matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh631' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh631'
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set accuracy

      eps = 10 * epsmac

!     --- Check whether an "old" boundary point is "inside" new element

      icheck = -1
      i4     = -1

!     --- Eventually jn is i4

      if ( jn>0 ) then
         xn = coor( 1, jn )
         yn = coor( 2, jn )
         zn = coor( 3, jn )

!        --- Set number

         i4 = jn
      end if

!     --- Find basis nodes

      i1 = kstapl( 1, 1 )
      i2 = kstapl( 2, 1 )
      i3 = kstapl( 3, 1 )

!     --- Coordinates node i1

      xd1 = coor( 1, i1 )
      yd1 = coor( 2, i1 )
      zd1 = coor( 3, i1 )

!     --- Coordinates node i2

      xd2 = coor( 1, i2 )
      yd2 = coor( 2, i2 )
      zd2 = coor( 3, i2 )

!     --- Coordinates node i3

      xd3 = coor( 1, i3 )
      yd3 = coor( 2, i3 )
      zd3 = coor( 3, i3 )

!     --- Determine bounding box around tetrahedron

      xmin = min(xd1,xd2,xd3,xn)
      ymin = min(yd1,yd2,yd3,yn)
      zmin = min(zd1,zd2,zd3,zn)
      xmax = max(xd1,xd2,xd3,xn)
      ymax = max(yd1,yd2,yd3,yn)
      zmax = max(zd1,zd2,zd3,zn)

!     --- Use for the bound a small extension in all directions

      xmind = xmin - 0.2d-4*(xmax-xmin)
      xmaxd = xmax + 0.2d-4*(xmax-xmin)
      ymind = ymin - 0.2d-4*(ymax-ymin)
      ymaxd = ymax + 0.2d-4*(ymax-ymin)
      zmind = zmin - 0.2d-4*(zmax-zmin)
      zmaxd = zmax + 0.2d-4*(zmax-zmin)

      a1 = xd2 - xd1
      a2 = yd2 - yd1
      a3 = zd2 - zd1

      b1 = xd3 - xd1
      b2 = yd3 - yd1
      b3 = zd3 - zd1

      c1 =  xn - xd1
      c2 =  yn - yd1
      c3 =  zn - zd1

      call msh636(a1,a2,a3,b1,b2,b3,c1,c2,c3,det)

!     --- Is triangle a line then return

      if ( abs(det)<eps ) go to 1000

!     --- Run through all nodes and check each

      do j = 1, npoint

!     --- Check whether point is in boundary

         if ( itri(j)==0 ) goto 100

!        --- Check whether point j node of tetrahedron

         if ( j==i1 ) goto 100
         if ( j==i2 ) goto 100
         if ( j==i3 ) goto 100
         if ( j==i4 ) goto 100

         xj = coor( 1, j )
         if ( xj<xmind ) goto 100
         if ( xj>xmaxd ) goto 100

         yj = coor( 2, j )
         if ( yj<ymind ) goto 100
         if ( yj>ymaxd ) goto 100

         zj = coor( 3, j )
         if ( zj<zmind ) goto 100
         if ( zj>zmaxd ) goto 100

         d1 = xj - xd1
         d2 = yj - yd1
         d3 = zj - zd1

         call msh636(d1,d2,d3,b1,b2,b3,c1,c2,c3,ski)
         ski = ski / det
         if ( ski<-eps .or. ski>1d0+eps ) goto 100
         call msh636(a1,a2,a3,d1,d2,d3,c1,c2,c3,eta)
         eta = eta / det
         if ( eta<-eps .or. eta>1d0+eps ) goto 100
         call msh636(a1,a2,a3,b1,b2,b3,d1,d2,d3,zta)
         zta = zta / det
         if ( zta<-eps .or. zta>1d0+eps ) goto 100

!        --- Last check

         ext = 1d0 - ski - eta - zta
         if ( ext<-eps .or. ext>1d0+eps ) goto 100

!        --- Point "inside" (at least near tetrahedron)

         if ( debug ) write(irefwr,*) 'Node',j,'is inside tet'
         icheck = j
         go to 1000
100   end do

!     --- None of the boundary points is inside
!         tetrahedron i1 - i2 - i3 - j
!         --- Fill array line

      nline = 0

!     --- Run through all triangles in boundary

      do ks = 2,kstap
         k1 = kstapl( 1, ks )
         k2 = kstapl( 2, ks )
         k3 = kstapl( 3, ks )

         if ( k1>k2 ) then
            nline = nline + 1
            line( 1, nline ) = k1
            line( 2, nline ) = k2
         end if

         if ( k2>k3 ) then
            nline = nline + 1
            line( 1, nline ) = k2
            line( 2, nline ) = k3
         end if

         if ( k3>k1 ) then
            nline = nline + 1
            line( 1, nline ) = k3
            line( 2, nline ) = k1
         end if
      end do

!     --- Length of array line enough ?

      call inichkarlen ( 2*nline, nlind, 'line' )
      if ( ierror/=0 ) go to 1000

!     --- Check whether all lines have no points in common with
!         new boundaries.
!         Run through all lines skip all lines outside "block" , and
!         check for the three faces

      do il = 1, nline

!     --- Nodes of line:

         ia = line( 1, il )
         ib = line( 2, il )

!        --- Coordinates start

         xa = coor( 1, ia )
         ya = coor( 2, ia )
         za = coor( 3, ia )

!        --- Coordinates end

         xb = coor( 1, ib )
         yb = coor( 2, ib )
         zb = coor( 3, ib )

!        --- Check

         xlm = min(xa,xb)
         if ( xlm>xmax ) goto 300
         xlm = max(xa,xb)
         if ( xlm<xmin ) goto 300
         ylm = min(ya,yb)
         if ( ylm>ymax ) goto 300
         ylm = max(ya,yb)
         if ( ylm<ymin ) goto 300
         zlm = min(za,zb)
         if ( zlm>zmax ) goto 300
         zlm = max(za,zb)
         if ( zlm<zmin ) goto 300

!        --- First triangle

         je = 1
         if ( ia==i1.or.ia==i2.or.ia==jn ) je = 0
         if ( ib==i1.or.ib==i2.or.ib==jn ) je = 0
         if ( je==1 ) then
            call msh610(xa,ya,za,xb,yb,zb,xd1,yd1,zd1,xd2,yd2,zd2, &
                          xn,yn,zn,t)
            if ( debug .and. t<1d0 ) then
               write(irefwr,*) ia,'-',ib,'intersects',i1,i2,jn
            end if
            if ( t<1d0 ) go to 1000
         end if

!        --- Second triangle

         je = 2
         if ( ia==i2.or.ia==i3.or.ia==jn ) je = 0
         if ( ib==i2.or.ib==i3.or.ib==jn ) je = 0
         if ( je==2 ) then
            call msh610(xa,ya,za,xb,yb,zb,xd2,yd2,zd2,xd3,yd3,zd3, &
                        xn,yn,zn,t)
            if ( debug .and. t<1d0 ) then
               write(irefwr,*) ia,'-',ib,'intersects',i2,i3,jn
            end if
            if ( t<1d0 ) go to 1000
         end if

!        --- Third triangle

         je = 3
         if ( ia==i3.or.ia==i1.or.ia==jn ) je = 0
         if ( ib==i3.or.ib==i1.or.ib==jn ) je = 0
         if ( je==3 ) then
            call msh610(xa,ya,za,xb,yb,zb,xd3,yd3,zd3,xd1,yd1,zd1, &
                        xn,yn,zn,t)
            if ( debug .and. t<1d0 ) then
               write(irefwr,*) ia,'-',ib,'intersects',i3,i1,jn
            end if
            if ( t<1d0 ) go to 1000
         end if
300   end do

      if (debug ) write(irefwr,*) 'No problems for this tet'
      icheck = 0

1000  call erclos ( 'msh631' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh631'
      end if  ! ( debug )

      if ( icheck/=0 .and. debug ) then
         if ( icheck==-1 ) then
            write(irefwr,*) 'intersection, je=',je
         else
            write(irefwr,*) 'node',icheck,'inside'
         end if
      end if

      end
