      subroutine msh639( kstapl, kstap, itri )
! ======================================================================
!
!        programmer    niek praagman
!        version  4.0  date 08-09-2010 New release
!        version  3.1  date 15-04-2005 Update code
!        version  3.0  date 24-02-2005 New norms
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check boundary array for double elements and adjust arrays KSTAPL
!    and ITRI if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstapl(3,*), itri(*), kstap

!     itri      i/o   array containing for each node nk whether
!                     that node is on the actual boundary ( i.e.
!                     itri(nk)>0 ) or not (i.e. itri(nk) = 0 )
!     kstap     i/o   number of elements in kstapl
!     kstapl    i/o   array containing the actual boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jstap, ib, i1, i2, i3, iweg, j, j1, j2, j3, ja, jb, jc

!     i1          first node of boundary triangle to be considered
!     i2          second node of boundary triangle to be considered
!     i3          third node of boundary triangle to be considered
!     ib          loopvariable
!     iweg        indicator whether triangle is double (1) or not (0)
!     j           loopvariable
!     j1          first node of comparing triangle
!     j2          second node of comparing triangle
!     j3          third node of comparing triangle
!     ja          indicator for first point
!     jb          indicator for second point
!     jc          indicator for third point
!     jstap       counter for new ordering of triangles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     None
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through boundary array and check for each triangle whether
!     it is unique. If not cancel both by placing zero values.
!     At the end remove all zero values.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
      jstap = 0

!     --- Check for double triangles and adjust kstapl and itri

      do ib = 1, kstap-1
         i1 = kstapl(1,ib)
         i2 = kstapl(2,ib)
         i3 = kstapl(3,ib)
         if ( i1/=0.and.i2/=0.and.i3/=0 ) then
            iweg = 0
            do j = ib+1 , kstap
               j1 = kstapl(1,j)
               j2 = kstapl(2,j)
               j3 = kstapl(3,j)
               ja = 1
               if ( j1==i1.or.j1==i2.or.j1==i3 ) ja = 0
               jb = 1
               if ( j2==i1.or.j2==i2.or.j2==i3 ) jb = 0
               jc = 1
               if ( j3==i1.or.j3==i2.or.j3==i3 ) jc = 0

!              --- Check whether all values are zero

               if ( ja+jb+jc==0 ) then

!              --- Triangle has to be excluded (two triangles of volumes
!                  situated "against" each other):

                  iweg = iweg+1

                  itri(j1) = itri(j1) - 1
                  itri(j2) = itri(j2) - 1
                  itri(j3) = itri(j3) - 1

                  kstapl(1,j) = 0
                  kstapl(2,j) = 0
                  kstapl(3,j) = 0
               end if
            end do

            if ( iweg==1 ) then
               itri(i1) = itri(i1) - 1
               itri(i2) = itri(i2) - 1
               itri(i3) = itri(i3) - 1
            else

!           --- Place triangle back in array kstapl:

               jstap = jstap + 1

               kstapl(1,jstap) = i1
               kstapl(2,jstap) = i2
               kstapl(3,jstap) = i3
            end if
         end if
      end do

!     --- No double triangles any longer, check last triangle:

      i1 = kstapl(1,kstap)
      i2 = kstapl(2,kstap)
      i3 = kstapl(3,kstap)

      if ( i1==0.or.i2==0.or.i3==0 ) then
           kstap = jstap
      else
           kstap = jstap + 1
           kstapl(1,kstap) = i1
           kstapl(2,kstap) = i2
           kstapl(3,kstap) = i3
      end if

!     --- Ready: boundary array cleared

      end
