      subroutine msh661 ( coor, i1, i2, i3, i4, ratio )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.1  date 26-10-2011 Correction division by 0
!        version  2.0  date 09-09-2010 New release
!        version  1.1  date 09-01-2008 Extra error message
!
!   copyright (c) 2006-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute radius of inner and outer sphere of four points in 3D
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     tetrahedron
!     radius
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, i4
      double precision coor(3,*), ratio

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     i1             i    node number of tetrahedron
!     i2             i    node number of tetrahedron
!     i3             i    node number of tetrahedron
!     i4             i    node number of tetrahedron
!     ratio          o    ratio of 3 * inner / outer radius
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a1, a2, a3, b1, b2, b3, &
                       c1, c2, c3, d1, d2, d3, &
                       det, radin, radom, term, &
                       t1, t2, t3, t123, s1, s2, s3, s123, &
                       u1, u2, u3, u123, v1, v2, v3, v4, v123, &
                       x1, x2, x3, x4, xc, &
                       y1, y2, y3, y4, yc, &
                       z1, z2, z3, z4, zc

!     a1             help coefficient
!     a2             help coefficient
!     a3             help coefficient
!     b1             help coefficient
!     b2             help coefficient
!     b3             help coefficient
!     c1             help coefficient
!     c2             help coefficient
!     c3             help coefficient
!     d1             help coefficient
!     d2             help coefficient
!     d3             help coefficient
!     det            determinant value
!     radin          radius of inner sphere
!     radom          radius of outer sphere
!     s1             help coefficient
!     s123           help variable
!     s2             help coefficient
!     s3             help coefficient
!     t1             help coefficient
!     t123           help variable
!     t2             help coefficient
!     t3             help coefficient
!     term           help variable
!     u1             help coefficient
!     u123           help variable
!     u2             help coefficient
!     u3             help coefficient
!     v1             help coefficient
!     v123           helpvariable
!     v2             help coefficient
!     v3             help coefficient
!     v4             help coefficient
!     x1             x-coordinate node 1
!     x2             x-coordinate node 2
!     x3             x-coordinate node 3
!     x4             x-coordinate node 4
!     xc             x-coordinate centre of sphere
!     y1             y-coordinate node 1
!     y2             y-coordinate node 2
!     y3             y-coordinate node 3
!     y4             y-coordinate node 4
!     yc             y-coordinate centre of sphere
!     z1             z-coordinate node 1
!     z2             z-coordinate node 2
!     z3             z-coordinate node 3
!     z4             z-coordinate node 4
!     zc             z-coordinate centre of sphere
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSH636         Compute determinant of matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     499   Declared length of arrays too small (Internal SEPRAN error)
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set values of coordinates
!
!     Compute matrix and righthandside for outer sphere
!     Determine centre and radius
!
!     Compute matrix and righthandside for inner sphere
!     Determine centre and radius
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh661' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh661'
         if ( i1==0 .or. i2==0 .or. i3==0 .or. i4==0 ) then

!        --- Internal error one of i1, i2, i3 or i4 = 0

            call errsub ( 499, 0, 0, 0 )
            go to 1000
         end if
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set the coordinates of the four nodes

      ratio = 0

!     --- Coordinates of nodes
!         i1:

      x1 = coor( 1, i1 )
      y1 = coor( 2, i1 )
      z1 = coor( 3, i1 )

!     --- i2:

      x2 = coor( 1, i2 )
      y2 = coor( 2, i2 )
      z2 = coor( 3, i2 )

!     --- i3:

      x3 = coor( 1, i3 )
      y3 = coor( 2, i3 )
      z3 = coor( 3, i3 )

!     --- i4:

      x4 = coor( 1, i4 )
      y4 = coor( 2, i4 )
      z4 = coor( 3, i4 )

!     --- Computation radius Outer sphere
!         Matrix and Rhsd

      a1 = x2 - x1
      a2 = x3 - x1
      a3 = x4 - x1

      b1 = y2 - y1
      b2 = y3 - y1
      b3 = y4 - y1

      c1 = z2 - z1
      c2 = z3 - z1
      c3 = z4 - z1

      call msh636(a1,a2,a3,b1,b2,b3,c1,c2,c3,det)

      if ( abs(det)<10 * epsmac ) go to 1000

      v1 = (x1*x1 + y1*y1 + z1*z1)
      v2 = (x2*x2 + y2*y2 + z2*z2)
      v3 = (x3*x3 + y3*y3 + z3*z3)
      v4 = (x4*x4 + y4*y4 + z4*z4)

      d1 = (v2 - v1)/2d0
      d2 = (v3 - v1)/2d0
      d3 = (v4 - v1)/2d0

      call msh636(d1,d2,d3,b1,b2,b3,c1,c2,c3,xc)

      xc = xc / det

      call msh636(a1,a2,a3,d1,d2,d3,c1,c2,c3,yc)

      yc = yc / det

      call msh636(a1,a2,a3,b1,b2,b3,d1,d2,d3,zc)

      zc = zc / det

      radom = sqrt((xc-x1)*(xc-x1) + (yc-y1)*(yc-y1) + (zc-z1)*(zc-z1))

!     --- Computation radius inner sphere
!         Matrix and Rhsd

      t1 = (y2-y1)*(z3-z1) - (z2-z1)*(y3-y1)
      t2 = (z2-z1)*(x3-x1) - (x2-x1)*(z3-z1)
      t3 = (x2-x1)*(y3-y1) - (y2-y1)*(x3-x1)

      t123 = sqrt(t1*t1+t2*t2+t3*t3)

      t1 = t1 / t123
      t2 = t2 / t123
      t3 = t3 / t123

      s1 = (y4-y1)*(z2-z1) - (z4-z1)*(y2-y1)
      s2 = (z4-z1)*(x2-x1) - (x4-x1)*(z2-z1)
      s3 = (x4-x1)*(y2-y1) - (y4-y1)*(x2-x1)

      s123 = sqrt(s1*s1+s2*s2+s3*s3)

      if ( s123>0d0 ) then
         s1 = s1 / s123
         s2 = s2 / s123
         s3 = s3 / s123
      end if  ! ( s123>0d0 )

      u1 = (y3-y1)*(z4-z1) - (z3-z1)*(y4-y1)
      u2 = (z3-z1)*(x4-x1) - (x3-x1)*(z4-z1)
      u3 = (x3-x1)*(y4-y1) - (y3-y1)*(x4-x1)

      u123 = sqrt(u1*u1+u2*u2+u3*u3)

      if ( u123>0d0 ) then
         u1 = u1 / u123
         u2 = u2 / u123
         u3 = u3 / u123
      end if  ! ( u123>0d0 )

      v1 = (y4-y2)*(z3-z2) - (z4-z2)*(y3-y2)
      v2 = (z4-z2)*(x3-x2) - (x4-x2)*(z3-z2)
      v3 = (x4-x2)*(y3-y2) - (y4-y2)*(x3-x2)

      v123 = sqrt(v1*v1+v2*v2+v3*v3)

      if ( v123>0d0 ) then
         v1 = v1 / v123
         v2 = v2 / v123
         v3 = v3 / v123
      end if  ! ( v123>0d0 )

      a1 = t1 - s1
      b1 = t2 - s2
      c1 = t3 - s3
      a2 = t1 - u1
      b2 = t2 - u2
      c2 = t3 - u3
      a3 = t1 - v1
      b3 = t2 - v2
      c3 = t3 - v3

      term = t1*x1 + t2*y1 + t3*z1

      d1   = - (s1*x4 + s2*y4 + s3*z4) + term

      d2   = - (u1*x4 + u2*y4 + u3*z4) + term

      d3   = - (v1*x4 + v2*y4 + v3*z4) + term

      call msh636(a1,a2,a3,b1,b2,b3,c1,c2,c3,det)

      if ( abs(det)<10 * epsmac ) go to 1000

      call msh636(d1,d2,d3,b1,b2,b3,c1,c2,c3,xc)

      xc = xc / det

      call msh636(a1,a2,a3,d1,d2,d3,c1,c2,c3,yc)

      yc = yc / det

      call msh636(a1,a2,a3,b1,b2,b3,d1,d2,d3,zc)

      zc = zc / det

      radin =  t1*(xc - x1) + t2*(yc - y1) + t3*(zc - z1)

      ratio = 3 * radin / radom

1000  call erclos ( 'msh661' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh661'
      end if

      end
