      subroutine msho36chkn ( npoint, npn, x, y, coor, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node with coordinates (x,y) is already stored in coor
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     mesh
!     mesh_generation
!     plaxis
!     internal_line
!     check
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: npoint
      integer, intent(out) :: npn
      double precision, intent(in) :: x, y, coor(2,*), eps

!     coor           i    array of length 2 x npoint containing the
!                         co-ordinates of the nodal points to be checked
!     eps            i    Local accuracy
!     npn            o    Refers to the node number in coor with
!                         coordinates (x,y)
!                         if no node can be found: 0
!     npoint         i    Number of nodal points on coor
!     x              i    x co-ordinate of point
!     y              i    y co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iext
      double precision :: dx, dy, dis

!     dis            distance between two points
!     dx             difference of vector in x-direction
!     dy             difference of vector in y-direction
!     iext           loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msho36chkn' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msho36chkn'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npn = 0

      do iext = 1, npoint

         dx = x - coor(1,iext)
         dy = y - coor(2,iext)

         dis = sqrt(dx*dx + dy*dy)

         if ( debug ) then
            write(irefwr,1) 'iext', iext
            write(irefwr,2) 'dis, eps', dis, eps
         end if  ! ( debug )

         if ( dis<eps ) then

!        --- Old point

            npn    = iext
            exit

         end if

      end do  ! iext = 1, npoint-1

1000  call erclos ( 'msho36chkn' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msho36chkn'

      end if  ! ( debug )

      end subroutine msho36chkn
