      subroutine er0081 ( ivectr, icurvs, idim, iquant, number, &
                          ipoint, nparts, iregion, iseqlc, &
                          isubr, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.4  date 30-03-2015 Remove non positive nodes
!        version  4.3  date 09-11-2012 New calls to er0080/82
!        version  4.2  date 21-09-2012 New treatment reals
!
!   copyright (c) 1994-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store solution of region required into help array
!     If necessary the tangential or normal component is computed
!     Warning: this subroutine uses memory management subroutines and
!               should be treated as such a subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, icurvs(*), idim, iquant, &
              ipoint(*), nparts, &
              number, iregion, iseqlc, isubr, nnodes

!     icurvs     i    Integer array containing the curve or surface numbers
!                     that must be printed.
!                     Pos. 1 indicates how array icurvs is filled.
!                     Possible values:
!                      0   only one curve or surface is used. The curve of
!                          surface number is stored in icurvs(2)
!                     -1   The solution is printed along the curves or surfaces
!                          icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                          icrv2 in icurvs(3).
!                     -2   See -1, however, the sequence of the printing is
!                          reversed. In case of curves this means that the
!                          curves are printed from the end point of icrv2 to
!                          the initial point of icrv2, then the end point of
!                          icrv2-1 to the begin point of icrv2-1 and so on.
!                          In case of surfaces ichois=-2 is treated as ichois=-1
!                     n>0  The curve numbers or surface numbers are stored in
!                          icurvs(2) to icurvs(n+1). That sequence is used.
!                     Remark: a negative curve number means that the curve is
!                          printed from end point to initial point, hence in
!                          reversed order. Negative surface numbers are not yet
!                          available.
!     idim       i    indicates which quantities must be printed.
!                     Possible values:
!                      0: The complete vector is printed
!                      1: The vector is only printed in user points.
!                         These user points must be defined in array ICURVS
!                      2: The vector is only printed in user curves.
!                         These curves must be defined in array ICURVS
!                      3: The vector is only printed in user surfaces.
!                         These surfaces must be defined in array ICURVS
!                      4: The vector is only printed in user volumes.
!                         These volumes must be defined in array ICURVS
!                     20: Zero level set
!     ipoint     o    Array of length nparts+1 containing the starting
!                     addresses in array inodes of the various parts
!                     the boundary consists of.
!                     ipoint(1) = 1, &
!                     number of nodes at part i = ipoint(i+1)-ipoint(i)
!     iquant     i    indicates which quantity must be computed
!                     Possible values:
!                     0:  The contents of the vector are computed as stored in
!                         the vector corresponding to IVECTR i.e. without
!                         transformation.
!                     >0: The normal or tangential component of a vector along
!                         the curve or surface must be computed.
!                         The vector consists of two (R2) or three (R3)
!                         components
!                         The normal vector in the nodes is stored in array
!                         normal and the tangential vector
!     iregion    i    indicates if the complete region must be printed
!                     (0) or that the printing is restricted to a sub region,
!                     defined by the parameters xmin, xmax, ymin, ymax, zmin,
!                     zmax in the comon block CPLOT.
!                     Possible values:
!                      0:  The complete region is printed
!                      1:  The region is restricted to xmin<=x<=xmax,
!                          ymin<=y<=ymax, zmin<=z<=zmax
!                     -1:  The region is restricted to x=xmin
!                     -2:  The region is restricted to y=ymin
!                     -3:  The region is restricted to z=zmin
!                     -11: The region is restricted to x>=xmin
!                     -12: The region is restricted to y>=ymin
!                     -13: The region is restricted to z>=zmin
!                     -21: The region is restricted to x<=xmin
!                     -22: The region is restricted to y<=ymin
!                     -23: The region is restricted to z<=zmin
!     iseqlc       i    indicates the sequence in which the co-ordinates
!                     along the surface must be ordered in case idim=3
!                     Possible values:
!                     1:  increasing x, for fixed x increasing y,
!                         for fixed y increasing z
!                     2:  increasing y, for fixed y increasing z,
!                         for fixed z increasing x
!                     3:  increasing z, for fixed z increasing x,
!                         for fixed x increasing y
!                     4:  sequence of nodal point numbers
!     isubr      i    Indicates which is the calling subroutine
!                     1    print subroutine, warning if too few points
!                     2    other subroutine, error
!     ivectr     i    Standard SEPRAN array referring to the vector to be
!                     printed
!     nnodes     o    Number of nodes
!     nparts     o    Number of parts the nodes array is subdivided into
!     number     i    indicates which degree of freedom in each point
!                     must be computed. This parameter must be seen in
!                     connection with iquant
!                     Possible values:
!                     0:  If iquant=0, all degrees of freedom are printed
!                         in the nodes defined by icurvs,
!                         if iquant>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom 1, second degree of freedom 2 and third
!                         degree of freedom 3. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
!                     i>0: If iquant=0, the degree of freedom i is printed
!                         if iquant>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom i, second degree of freedom i+1 and third
!                         degree of freedom i+2. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iplace, nummax, leng
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     iplace         Corrected value of idim
!                    Defines the type of input for prfillnodes
!     leng           Help variable to compute the length of some arrays
!     nummax         Maximum number of unknowns per point in new solution array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0076         Delete all points in inodes and ipoint not containing the
!                    required degree of freedom as given by number
!     ER0082         Find all degrees of freedom from the solution array or
!                    array of special structure as indicated by array inodes and
!                    store in output array. Complex case
!     ER0083         Compute normal and tangential vectors along a boundary
!     ER0083D        Compute tangential vector along curves in R^3
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRREMNEGNODES  Remove all non-positive node numbers from array nodes
!     PRSW02         Restrict nodes to those in a specified region
!     SEPACTSOLBF1   Activate sepran solution vector and related arrays
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
!     SEPSORTCOOR    Sort nodal points with respect to increasing co-ordinates
!     SEPSTORENEWSOL Find all degrees of freedom from the solution array or
!                    array of special structure as indicated by array inodes and
!                    store in output array. Real case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     102   NUMBER has wrong value
!     387   too few degrees of freedom in nodal points in complete part of
!           boundary to integrate.
!    1578   quant>0 and idim<2 not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Activate ivectr and corresponding arrays
!    Compute nodal point numbers in which the vector must be computed
!    if ( idim=2 ) then
!       Reserve space and activate arrays
!       Create arrays "inodes" and "ipoint"
!    else ( idim=3 )
!       Reserve space and activate arrays
!       Create array "inodes"
!    delete nodal points without degree of freedom in "inodes"
!    Store quantities to be computed in usolnw
!    if ( iquant>0 ) then
!       Compute normal and tangential vectors
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0081' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0081'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nullify(normalvec, unew, zunew, istore)

!     --- Activate ivectr and corresponding arrays

      call sepactsolbf1 ( ivectr )
      if ( number<0 .or. number>nphysi ) then
         call errint ( number, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( iquant>0 .and. number+idim>nphysi ) then
         call errint ( number+idim, 1 )
         call errint ( nphysi, 2 )
         call errsub ( 102, 2, 0, 0 )
      end if
      if ( iquant>0 .and. idim<2 ) then
         call errint ( iquant, 1 )
         call errint ( idim, 2 )
         call errwar ( 1578, 2, 0, 0 )
         iquant = 0
      end if
      compl = ks(ivectr)%compl==1

      if ( debug ) then
         write(irefwr,1) 'ndim, number, nphysi, idim, iquant, nunkpi', &
                          ndim, number, nphysi, idim, iquant, nunkpi
         write(irefwr,1) 'npoint, ncurvs', npoint, ncurvs
         write(irefwr,*) 'compl ', compl
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute nodal point numbers in which the vector must be computed

      iplace = idim
      if ( idim==4 ) iplace = 7

      nnodes = 0

      call sepfillnodes ( iplace, 0, icurvs, nodes, nnodes, ipoint, nparts )
      if ( ierror/=0 ) go to 1000

!     --- Remove all nodes with negative value

      call prremnegnodes ( nodes, nnodes )

      if ( debug ) then
         write(irefwr,1) 'iregion', iregion
         call prinin ( nodes, nnodes, 'nodes' )
      end if  ! ( debug )

      if ( iregion/=0 ) then

!     --- iregion # 0, restrict region

         if ( ierror/=0 ) go to 1000
         call prsw02 ( nodes, ipoint, nparts, &
                       iregion, coor, ndim, idim )
         if ( ierror/=0 ) go to 1000
         if ( idim==2 ) then
            nnodes = ipoint(nparts+1)-1
         else
            nnodes = nparts
         end if
         if ( debug ) write(irefwr,1) 'nnodes', nnodes

      end if

!     --- delete nodal points without degree of freedom in inodes

      if ( iquant==0 ) then
         nummax = 1
      else
         nummax = ndim
         number = max(1,number)
      end if

      if ( debug ) &
         write(irefwr,1) 'iquant, nummax, number, indprfi, indprpi', &
                          iquant, nummax, number, indprfi, indprpi
      if ( indprpi>0 .and. nunkpi/=nphysi .and. number>0 .or. &
           indprfi>0 ) then

!     --- indprfi # 0, number of unknowns not constant in array or
!         indprpi # 0, degrees of freedom may vary per point

         if ( ierror/=0 ) go to 1000

         call er0076 ( idim, nparts, ipoint, nodes, number, nummax )
         if ( debug ) write(irefwr,1) 'nparts', nparts
         if ( nparts<=0 ) then

!        --- No nodes found at curve or surfaces

            call errint ( nparts, 1 )
            if ( isubr==1 ) then
               call errwar ( 387, 1, 0, 0 )
               nnodes = 0
               go to 1000
            else
               call errsub ( 387, 1, 0, 0 )
            end if

         end if
         if ( idim==2 ) then
            nnodes = ipoint(nparts+1)-1
         else
            nnodes = nparts
         end if
         if ( debug ) write(irefwr,1) 'nnodes', nnodes

      end if
      if ( nnodes==0 .or. ierror/=0 ) go to 1000

!     --- Sort nodal points with respect to increasing co-ordinate

      if ( iseqlc<4 .and. (idim==0 .or. idim>=3) ) then

         if ( ierror/=0 ) go to 1000
         call sepsortcoor ( iseqlc, nnodes, nodes )
         if ( ierror/=0 ) go to 1000

      end if

!     --- Store quantities to be computed in usolnw

      indprfo = 0
      if ( indprfi==0 .or. number>0 ) then

!     --- Constant number of degrees of freedom in output vector

         if ( number>0 ) then
            leng = ndim*nnodes
         else
            leng = nunkpi*nnodes
         end if

      else

!     --- variable number of degrees of freedom in output vector

         allocate ( istore(nnodes+1) )
         leng = nnodes*nphysi
         indprfo = nnodes+1

      end if
      if ( compl ) then
         allocate ( zunew(leng), stat = error )
      else
         allocate ( unew(leng), stat = error )
      end if  ! ( compl )
      if ( error/=0 ) call eralloc ( error, leng, 'unew' )
      if ( ierror/=0 ) go to 1000

      usol => ks(ivectr)%sol
      if ( ierror/=0 ) go to 1000

      if ( compl ) then
         call er0082 ( number, nummax, usol, zunew, &
                       istore, nodes, nnodes, leng )
      else

         call sepstorenewsol ( number, nummax, nodes, nnodes )
      end if
      if ( ierror/=0 ) go to 1000

      if ( iquant>0 ) then

!     --- iquant>0, compute tangential and normal component

         allocate ( tang(ndim,nnodes) )
         if ( idim==2 .and. ndim==2 ) then

!        --- idim=ndim=2, curves in R^2

            allocate ( normalvec(2,nnodes) )
            call er0083 ( nodes, nparts, ipoint )

         else if ( idim==2 ) then

!        --- idim=2, ndim=3, curves in R^3, compute only the tangential vector

            leng = nnodes*3
            if ( ierror/=0 ) go to 1000
            call er0083d ( nodes, nparts, ipoint )

         end if

      end if
      if ( ierror/=0 ) go to 1000
      if ( indprfo==0 ) then
         if ( ndim==3 .and. idim==2 .and. &
              iquant>0 ) then
            indprfo = -ndim
         else if ( number==0 ) then
            indprfo = -nunkpi
         else
            indprfo = -max(1,iquant)
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nparts', nparts
         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End er0081'

      end if  ! ( debug )
      call erclos ( 'er0081' )

      end subroutine er0081
