      subroutine erdistlines2d ( x1, x2, x3, x4, distance )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the distance between two interval in 2d by
!     computing the distance of the line perpendicular to the mid point of
!     the first interval and the line through the other interval
! **********************************************************************
!
!                       KEYWORDS
!
!     distance
!     line_piece
!     line
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(2), x2(2), x3(2), x4(2), distance

!     distance       o    distance between the mid point of x1-x2 and the line
!                         through x3-x4
!     x1             i    coordinate of x1
!     x2             i    coordinate of x2
!     x3             i    coordinate of x3
!     x4             i    coordinate of x4
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision n(2), x(2), xn(2), fact1, fact2

!     fact1          multiplication factor with respect to intersection
!     fact2          multiplication factor with respect to intersection
!     n              normal vector
!     x              co-ordinates of point
!     xn             coordinates of new point on normal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERNORMAL2D     Compute normal vector perpendicular on the line through the
!                    points with coordinates (x1,y1) and (x2, y2)
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE2  Computes the intersection of two lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     n = normal on (x1-x2)
!     x = mid point on of x1 x2
!     compute intersection x - x+n, x3 - x4
!     distance := length of x to intersection point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erdistlines2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erdistlines2d'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- n = normal on (x1-x2)

      call ernormal2d ( x1, x2, n )

!     --- x = mid point on of x1 x2

      x(1) = 0.5d0*(x1(1)+x2(1))
      x(2) = 0.5d0*(x1(2)+x2(2))

!     --- compute intersection x - x+n, x3 - x4

      xn(1) = x(1)+n(1)
      xn(2) = x(2)+n(2)
      call mshcrossline2 ( x, xn, x3, x4, fact1, fact2 )

!     --- distance := length of x to intersection point

      if ( fact1.gt.-1d20 ) then
         distance = abs(fact1)
      else
         distance = rinfin
      end if   ! ( fact1.gt.-1d20 )

1000  call erclos ( 'erdistlines2d' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erdistlines2d'

      end if

      end
