      subroutine erfillarrcop ( nodes, kdeg, uold, unew, &
                                indprf, kprobf, nphys, numnodes )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 25-01-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of array uold into array unew for
!     all nodes stored in array nodes and the degrees of freedom indicated
!     in kdeg
!     uold is user array of length nnodes
!     unew is a sepran solution array
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), kdeg, indprf, kprobf(*), nphys, numnodes
      double precision uold(*), unew(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0) for the old vector
!     kdeg           i    Only the KDEGth degree of freedom
!     kprobf         i    KPROB part f for old vector
!     nodes          i    Contains the node numbers that must be filled
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point with respect to the old vector
!     numnodes       i    Number of nodal points in array nodes
!     unew          i/o   New solution vector
!     uold           i    Array containing the old vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, i

!     i              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1131   Degree of freedom required is larger than number of unknowns
!           per point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erfillarrcop' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillarrcop'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kdeg>nphys ) then
         call errint ( kdeg, 1 )
         call errint ( nphys, 2 )
         call errsub ( 1131, 2, 0, 0 )
         go to 1000
      end if  ! ( kdeg>nphys )

!     --- Distinguish between the various cases

      if ( indprf==0 ) then

!     --- constant number of degrees of freedom in uold

         do i = 1, numnodes
            node = nodes(i)
            unew((node-1)*nphys+kdeg) = uold(i)
         end do  ! i = 1, numnodes

      else

!     --- variable number of degrees of freedom in uold

         do i = 1, numnodes

            node = nodes(i)
            if ( kprobf(node+1)>=kprobf(node)+kdeg ) then

!           --- Degree of freedom available in uold

               unew(kprobf(node)+kdeg) = uold(i)

            end if  ! ( kprobf(node+1)>=kprobf(node)+kdeg )

         end do  ! i = 1, numnodes

      end if  ! ( indprf==0 )

1000  call erclos ( 'erfillarrcop' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillarrcop'

      end if  ! ( debug )

      end
