      subroutine ernormal2d ( x1, x2, n )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute normal vector perpendicular on the line through the
!     points with coordinates (x1,y1) and (x2, y2)
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(2), x2(2), n(2)

!     n              o    normal vector
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t(2), len

!     len            length of edge
!     t              tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     t = (x_2 - x_1) / ||x_2 - x_1||
!     n = (t_2, -t_1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
!     --- t = (x_2 - x_1) / ||x_2 - x_1||

      t(1) = x2(1)-x1(1)
      t(2) = x2(2)-x1(2)
      len = 1d0/sqrt(t(1)**2+t(2)**2)

!     --- n = (t_2, -t_1)

      n(1) = t(2)*len
      n(2) = -t(1)*len

      end
