      subroutine errormshsur1 ( nelh, npoh, nelem, npoint, isurnr )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 20-07-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare estimated size of npoint and nelem with actual size
!     Give error message if estimate is too small
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     error_manipulation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelh, npoh, nelem, npoint, isurnr

!     isurnr         i    Surface sequence number
!     nelem          i    Actual number of elements
!     nelh           i    Estimate of the number of elements
!     npoh           i    Estimate of the number of points
!     npoint         i    Actual number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     141   The actual generated number of nodal points is more than the
!           estimated number
!     157   The actual generated number of elements is more than the
!           estimated number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      if ( npoh<npoint ) then
         call errint ( isurnr, 1 )
         call errint ( npoint, 2 )
         call errint ( npoh  , 3 )
         call errsub ( 141, 3, 0, 0 )
      end if

      if ( nelh<nelem ) then
         call errint ( isurnr, 1 )
         call errint ( nelem , 2 )
         call errint ( nelh  , 3 )
         call errsub ( 157, 3, 0, 0 )
      end if

      end
