      function getconstseq ( namecof, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if namecof belongs to the set of constants, put the result
!     in getconstseq
!     If so put its value in value
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     constant
!     value
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: value
      logical :: getconstseq
      character (len=*) :: namecof

!     getconstseq    o    If namecof is one of the constants: true, else false
!     namecof        i    Name of constant to be checked
!     value          o    Value of constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number of constant in set of integers or reals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getnameint, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getconstseq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getconstseq'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      getconstseq = .false.  ! No yet found

      iseq =  getnameint ( namecof )
      if ( iseq>0 ) then

!     --- Integer found

         value = incons(iseq)
         getconstseq = .true.   ! value found

      else

         iseq =  getnamereal ( namecof )
         if ( iseq>0 ) then

!        --- Real found

            value = rlcons(iseq)
            getconstseq = .true.   ! value found

         end if  ! ( iseq>0 )

      end if  ! ( iseq>0 )

1000  call erclos ( 'getconstseq' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getconstseq'

      end if  ! ( debug )

      end function getconstseq
