      function getindint ( intname, loopname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the integer in the block constants with name intname
!     If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname, loopname
      integer getindint

!     getindint      o    Value of integer
!     intname        i    Name of integer
!     loopname       i    Name of loop variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intseqnm

!     intseqnm       Sequence number of integer with respect to array
!                    chincons
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameindint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINDINT  Get the sequence number corresponding to the integer in
!                    the block constants with name intname
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2155   integer name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chincons
!     If ( name in chincons ) then
!        getindint := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      intseqnm = getnameindint ( intname, loopname )
      getindint = 0
      if ( intseqnm>0 ) then
         getindint = incons(intseqnm)
      else if ( nwarn<=mxwarn ) then
         call eropen ( 'getindint' )
         call errchr ( intname, 1 )
         call errwar ( 2155, 0, 0, 1 )
         call erclos ( 'getindint' )
      end if  ! ( intseqnm>0 )
      end
