      subroutine getvector ( ivecin, ivectr, numarrays, newvector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-09-2016 New parameter list
!        version  1.0  date 04-01-2014
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with sequence number of vector in array isol
!     If vector consists of a vector expression a new vector is created
!     and a new sequence number made
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivecin, ivectr, numarrays
      logical newvector

!     ivecin         i    Sequence number of vector before reduction
!     ivectr         o    Actual sequence number of vector after reduction
!                         or computing the expression
!     newvector      o    If true a new vector is made, which should be
!                         removed at the end of the calling subroutine
!     numarrays      o    Number of indexed vectors in vector array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpsep(20), iseqcoor

!                    they are not
!     inpsep         Integer input array for subroutine prveccombinat0
!                    For a description see subroutine askvec
!                    All entries must be 0 except entry 8, which refers to
!                    the expression
!     iseqcoor       Sequence number of coordinates vector in isolut
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PRVECCOMBINAT0 Evaluate right-hand side of vector expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3167   Vector number to be created>maxvec
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getvector' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getvector'
         write(irefwr,1) 'ivecin, numvecmn', ivecin, numvecmn
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ivecin>1.99d8 ) then

!     --- Vector expression found
!         Create a new temporary position in isol

         numarrays = 1         ! at this moment not available for arrays
         numvecmn = numvecmn+1     ! raise numvecmn
         ivectr = numvecmn       ! new vector number
         iseqcoor = 0
         newvector = .true.    ! new vector is created

         if ( debug ) write(irefwr,1) 'numvecmn, ivectr', numvecmn, ivectr

         if ( numvecmn>maxvec ) then

!        --- Vector number to be created is too large

            call errint ( numvecmn, 1 )
            call errint ( maxvec, 2 )
            call errsub ( 3167, 2, 0, 0 )
            go to 1000

         end if  ! ( numvecmn>maxvec )

!        --- Create new vector and fill with expression

         inpsep = 0                    ! default values
         inpsep(8) = ivecin            ! sequence number of expression
         call prveccombinat0 ( inpsep, ivectr, iseqcoor )

      else

!     --- Standard case, compute actual vector sequence number

         call inireducevec ( ivecin, ivectr, numarrays )
         newvector = .false.    ! old vector is used

      end if  ! ( ivectr>1.99d8 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getvector'

      end if  ! ( debug )
      call erclos ( 'getvector' )

      end
