      subroutine inicurvusp ( userpoint, maxusp, numusp, ix )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 24-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all user points along ix axis with coordinates smaller than
!     that of userpoint and more than 0
!     order these user points in increasing sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer userpoint(*), maxusp, numusp, ix

!     ix             i    Defines direction of axis
!     maxusp         i    Declared length of array subuserp
!     numusp         o    Number of entries in array subuserp
!     userpoint     i/o   Contains the user point numbers along the
!                         axis sorted in ascending order
!                         userpoint(0) must contain userpoint at input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseq(maxusp), iwork(maxusp)
      double precision xcoor, x, work(maxusp)

!     i              Counting variable
!     iseq           Work array to store sequence numbers
!     iwork          integer work array to copy aray
!     work           Work array containing the coordinates of the points
!     x              x co-ordinate of point
!     xcoor          x-coordinate of userpoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICURVUSP1    Find all user points along ix axis with coordinates
!                    smaller than that of userpoint and more than 0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicurvusp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicurvusp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find user points

      call inicurvusp1 ( rinputkm(3), userpoint, &
                         maxusp, numusp, ix )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicurvusp'

      end if  ! ( debug )
      call erclos ( 'inicurvusp' )

      end subroutine inicurvusp
