      subroutine inifil ( ifile )
! ======================================================================
!
!        programmer   Guus Segal
!        version 6.7    date   20-04-1999 Inlcude commons
!        version 6.6    date   10-02-1994 new comments
!        version 6.5    date   22-09-1993 Extension with readonly for VAX
!
!   copyright (c) 1982-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!    The task of INIFIL is to open all files (except for standard input and
!    output if these files have unit number 5 and 6, or have no name)
!    Which files are opened depends on the parameter ifile
!    By a repeated number of calls more files may be opened
!
!      The subroutine is machine dependent.
!      In order to get one of the other machine dependent versions, it is
!      sufficient to remove comments in the first columns and to set
!      comments in the actual version
!
!      For example to get the VAX-version replace cvax by 4 spaces, etc.
!
!      Available versions:
!
!      IBM                   cibm
!      Cyber     NOS/VE      cybr
!      Apollo                capo
!      Standard Unix         cuni and cunx
!      Vax       VMS         cvax
!      Harris                char
!      HP 9000   Unix        cuni and chpu
!      CRAY                  cray
!      IBM PC 386            cipc
!      ALLIANT   Concentrix  cuni and cali
!      Convex    Unix        cuni and ccon
!
! ********************************************************************
!
!                      KEYWORDS
!
!     machine_dependent
!     open_file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      use sepmodulefiles
      implicit none

! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      integer ifile

!     ifile    i   Formal file number of file to be opened. In many practical
!                  situations this is also the actual file number
!                  Possible values for ifile are:
!                  1:  Standard element input file (iref1)
!                  2:  Standard backing storage file (iref2), reset information
!                 -2:  Standard backing storage file (iref2), old file
!                  3:  Standard scratch file (iref3)
!                  4:  Standard error message file (irefer)
!                  5:  Standard input file
!                  6:  Standard output file
!                 10:  File containing the mesh (iref10)  (meshoutput)
!                 17:  Standard menu message file (iref17)
!                 73:  File containing information of the problem (iref73)
!                      (sepcomp.inf)
!                 74:  File containing the solutions (iref74) (sepcomp.out)
!                  With respect to the files 10, 73 and 74 a
!                  negative value of ifile denotes that the file must already
!                  exist
!                  If the file does not exist an error message is given.
!                  If ierror = -1, ierror is set to 1 and control is given back
!                  to the calling subroutine , otherwise the program is stopped
!                  With respect to the files 1 4 and 17 a
!                  positive value of ifile denotes that the file must already
!                  exist and is not changed, a negative value means that the
!                  file may be new
! ********************************************************************
!
!
!                   COMMON BLOCKS
!

! ********************************************************************
!
!                       LOCAL PARAMETERS
!
!
      integer nr, nr1, nr4, i, ios, iref
      character (len=50) name
      character (len=10) facc, fstat
      logical check

!    check     Indication if file exists (true) or not (false)
!    facc      File access parameter
!    fstat     File status parameter
!    i         Counting variable
!    ios       IOSTAT status variable to be used for error messages
!    iref      File reference number
!    name      Name of file
!    nr        Record length of a direct access file in machine dependent
!              quantites ( bytes, words )
!    nr1       Record length of direct access file 1 in machine dependent
!              quantites ( bytes, words )
!    nr4       Record length of direct access file 4 in machine dependent
!              quantites ( bytes, words )
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     INSTOP  Stop the program
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   -
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!

      if ( abs(ifile).eq.1 ) then

!     --- Open the standard element input file

         irecp1=0
         irecp2=0

!        --- The record length is stored in nr1

         nr1 = nrec1 * lenwor

         if ( ifile.gt.0 ) then

!        --- old file, i.e. the file must be opened readonly

            fstat = 'old'

         else

!        --- old or new file, i.e. call by makef1

            fstat = 'unknown'

         end if

         if ( namef1(1:1).eq.' ' ) then

!        --- namef1 is blank, file = without name
!            If ifile > 0, the file must already exist

            if ( ifile.gt.0 ) then

!           --- old file, no update is allowed

               inquire ( unit=iref1, exist=check )
               if ( .not. check ) then
                  write(*,100) ' File 1 (standard element file)', iref1
100               format( a,' with reference number:', i3, &
                          ' does not exist'// &
                          ' execution terminated' )
                  stop
               end if
            end if
            open ( iref1, access='direct', recl=nr1, iostat=ios, &
                   status=fstat )
            if ( ios.ne.0 ) then
               write(*,110) ' File 1 (standard element file)',iref1,ios
110            format( ' Error in opening of', a, &
                       ' with reference number:', i3/ &
                       ' IOSTATUS parameter has value', i6// &
                       ' execution terminated' )
               stop

            end if

         else

!        --- namef1 is not blank, file has name
!            If ifile > 0, the file must already exist

            if ( ifile.gt.0 ) then
               inquire ( file=namef1, exist=check )
               if ( .not. check ) then
                  write(*,120) namef1
120               format( ' File ', a, &
                          ' does not exist'// &
                          ' execution terminated' )
                  stop
               end if
               open ( iref1, file=namef1, access='direct', recl=nr1, &
                      iostat=ios, status=fstat )
            else
               open ( iref1, file=namef1, access='direct', recl=nr1, &
                      iostat=ios, status=fstat )
            end if

            if ( ios.ne.0 ) then
               write(*,130) namef1, ios
130            format( ' Error in opening of file ', a/ &
                       ' IOSTATUS parameter has value', i6// &
                       ' execution terminated' )
               stop
            end if

         end if

      else if ( abs(ifile).eq.2 ) then

!     --- |ifile| = 2   Open file sepranback

!         define file 2
!         The record length is stored in nr

         nr = nrec2 * lenwor

         if ( namef2(1:1).eq.' ' ) then

!        --- namef2 is blank, file = without name

            if ( ifile.lt.0 ) then

!           --- ifile = -2, file must exist

               inquire ( unit=iref2, exist=check )
               if ( .not. check ) then
                  write(*,100) ' File 2 (SEPRAN backing storage file)', &
                               iref2
                  call instop
               end if

            end if

            open ( iref2, access='direct', recl=nr, iostat = ios )
            if ( ios.ne.0 ) then
               write(*,110) ' File 2 (SEPRAN backing storage file)', &
                            iref2, ios
               call instop
            end if

         else

!        --- namef2 is not blank, file has name

            if ( ifile.eq.-2 ) then

!           --- ifile = -2, file must exist

               inquire ( file=namef2, exist=check )
               if ( .not. check ) then
                  write(*,120) namef2
                  call instop
               end if

            end if

            open ( iref2, file=namef2, access='direct', recl=nr, &
                   iostat=ios)
            if ( ios.ne.0 ) then
               write(*,130) namef2, ios
               call instop
            end if

         end if

         if ( ifile.eq.2 ) then

!        --- ifile = 2, hence initialise cfile2

            inew   = 1
            ifree2 = 2 + 999 / nrec2
            do 200 i = 1,1000
               iwork3 (i) = 0
200         continue

         end if

      else if ( ifile.eq.3 ) then

!     --- Open file 3  (Scratch file)

         nr = nrec3 * lenwor

         if ( namef3(1:1).ne.' ' ) then

!        --- Scratch file has name

            if ( namef3(1:3).eq.'tmp' .and. namef3(4:4).eq.' ' ) then

!           --- Name of scratch file is tmp
!               The name will become /tmp/tmp.SEPxxxxxx,
!               where xxxxxx is the process id


               namef3(1:12) = '/tmp/tmp.SEP'
               namef3(13:18) = pidsep(1:6)


            else if ( namef3(1:7).eq.'usr/tmp' ) then

!           --- Name of scratch file is tmp/usr
!               The name will become /usr/tmp/tmp.SEPxxxxxx,
!               where xxxxxx is the process id


               namef3(1:16) = '/usr/tmp/tmp.SEP'
               namef3(17:22) = pidsep(1:6)

            end if

            open ( iref3, access='direct', file = namef3, recl = nr )

         else

!        --- standard version

            open ( iref3, status='scratch', access='direct', recl=nr )

         end if

         ifree3=1

      else if ( abs(ifile).eq.4 ) then

!     --- Open the error message file
!         The record length is stored in nr4

         nr4 = 80
         nrec4 = nr4 / lenwor
         if ( lenwor.eq.1 ) then

!        --- lenwor = 1, record length in words instead of bytes

            nr4 = 80 / numchr
            nrec4 = nr4

         else if ( lenwor.eq.3 ) then

!        --- lenwor = 3, Harris computer

            nr4 = 81
            nrec4 = 27

         end if

         if ( ifile.gt.0 ) then

!        --- old file, i.e. the file must be opened readonly

            fstat = 'old'

         else

!        --- old or new file, i.e. call by makef4

            fstat = 'unknown'

         end if

         if ( namef4(1:1).eq.' ' ) then

!        --- namef4 is blank, file = without name
!            If ifile > 0, the file must already exist

            if ( ifile.gt.0 ) then
               inquire ( unit=irefer, exist=check )
               if ( .not. check ) then
                  write(*,100) ' File 4 (error message file)', irefer
                  stop
               end if
               open ( irefer, access='direct', recl=nr4, iostat=ios, &
                      status=fstat )
            else
               open ( irefer, access='direct', recl=nr4, iostat=ios, &
                      status=fstat )
            end if
            if ( ios.ne.0 ) then
               write(*,110) ' File 4 (error message file)',irefer,ios
               stop
            end if

         else

!        --- namef4 is not blank, file has name
!            If ifile > 0, the file must already exist

            if ( ifile.gt.0 ) then
               inquire ( file=namef4, exist=check )
               if ( .not. check ) then
                  write(*,120) namef4
                  stop
               end if
            end if

            open ( irefer, file=namef4, access='direct', recl=nr4, &
                   iostat=ios, status=fstat )
            if ( ios.ne.0 ) then
               write(*,130) namef4, ios
               stop
            end if

         end if

      else if ( ifile.eq.5 ) then

!     --- Open the input file, if irefre not equal to 5, or namere has value

         if ( irefre.ne.5 .and. namere(1:1).eq.' ' ) then

!        --- namere is blank, file = without name
!            The file must already exist

            inquire ( unit=irefre, exist=check )
            if ( .not. check ) then
               write(*,100) ' Standard input file', irefre
               call instop
            end if
            open ( irefre, iostat=ios )
            if ( ios.ne.0 ) then
               write(*,110) ' Standard input file',irefre,ios
               call instop
            end if

         else if ( namere(1:1).ne.' ' ) then

!        --- namere is not blank, file has name

            inquire ( file=namere, exist=check )
            if ( .not. check ) then
               write(*,120) namere
               call instop
            end if

            open ( irefre, file=namere, iostat=ios )
            if ( ios.ne.0 ) then
               write(*,130) namere, ios
               call instop
            end if

         end if

      else if ( ifile.eq.6 ) then

!     --- Open the output file, if irefwr not equal to 6, or namewr has value
!         Define status parameter and access parameter depending on spappn
!         and the type of computer

         facc  = 'sequential'
         fstat = 'unknown'
         if ( spappn(1:1).eq.'y' .or. spappn(1:1).eq.'Y' ) then

!        --- File must be opened in append mode

            facc  = 'append'

         end if

         if ( irefwr.ne.6 .and. namewr(1:1).eq.' ' ) then

!        --- namewr is blank, file = without name
!            append mode is not possible

            open ( irefwr, iostat=ios )
            if ( ios.ne.0 ) then
               write(*,110) ' Standard output file',irefwr,ios
               call instop
            end if

         else if ( namewr(1:1).ne.' ' ) then

!        --- namewr is not blank, file has name

            open ( unit=irefwr, file=namewr, access=facc, status=fstat, &
                   iostat=ios)
            if ( ios.ne.0 ) then
               write(*,130) namewr, ios
               call instop
            end if

         end if

      else if ( abs(ifile).eq.17 ) then

!     --- Open the menu message file
!         The record length is stored in nr4

         nr4 = 80
         nrec4 = nr4 / lenwor
         if ( lenwor.eq.1 ) then

!        --- lenwor = 1, record length in words instead of bytes

            nr4 = 80 / numchr
            nrec4 = nr4

         end if

         if ( ifile.gt.0 ) then

!        --- old file, i.e. the file must be opened readonly

            fstat = 'old'

         else

!        --- old or new file, i.e. call by makef17

            fstat = 'unknown'

         end if

         if ( namems(1:1).eq.' ' ) then

!        --- namems is blank, file = without name
!            If ifile > 0, the file must already exist

            if ( ifile.gt.0 ) then
               inquire ( unit=irefer, exist=check )
               if ( .not. check ) then
                  write(*,100) ' File 17 (menu message file)', iref17
                  stop
               end if
            end if
            open ( iref17, access='direct', recl=nr4, iostat=ios, &
                   status=fstat )
            if ( ios.ne.0 ) then
               write(*,110) ' File 17 (menu message file)',iref17,ios
               stop
            end if

         else

!        --- namems is not blank, file has name
!            If ifile > 0, the file must already exist

            if ( ifile.gt.0 ) then
               inquire ( file=namems, exist=check )
               if ( .not. check ) then
                  write(*,120) namems
                  stop
               end if
            end if

            open ( iref17, file=namems, access='direct', recl=nr4, &
                   iostat=ios, status=fstat )
            if ( ios.ne.0 ) then
               write(*,130) namems, ios
               stop
            end if

         end if

      else

!     --- ifile = 10 (meshoutput), 73 (sepcomp.inf) or 74 (sepcomp.out)

         if ( abs(ifile).eq.10 ) then

            name = name10
            iref = iref10

         else if ( abs(ifile).eq.73 ) then

            name = name73
            iref = iref73

         else if ( abs(ifile).eq.74 ) then

            name = name74
            iref = iref74

         end if

         if ( ifile.lt.0 ) then

!        --- ifile < 0, file must exist

            if ( name(1:1).eq.' ' ) then

!           --- File has no name

               inquire ( unit=iref, exist=check )
               if ( .not. check ) then
                  write(*,100) ' File ', iref
                  if ( ierror.eq.-1 ) then
                     ierror = 1
                     go to 1000
                  else
                     call instop
                  end if
               end if

            else

!           --- File has a name

               inquire ( file=name, exist=check )
               if ( .not. check ) then
                  write(*,120) name
                  if ( ierror.eq.-1 ) then
                     ierror = 1
                     go to 1000
                  else
                     call instop
                  end if
               end if

            end if

         end if

         if ( iref.gt.0 ) then

!        --- iref > 0:  formatted

            if ( name(1:1).eq.' ' ) then

!           --- open file without name

               open ( unit = iref )

            else

!           --- open file with name

               open ( unit = iref, file = name )

            end if

         else

!        --- iref < 0, unformatted

            if ( name(1:1).eq.' ' ) then

!           --- open file without name

               open ( unit = -iref, form = 'unformatted' )

            else

!           --- open file with name

               open ( unit = -iref, file = name, form = 'unformatted' &
                    )

            end if

         end if

      end if
      ierror = 0
1000  nerror = 0
      end
