      subroutine ininodeedges ( ishape, iedge, nodesedges, nspecp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 22-05-2014 Extension with shape 22
!        version  2.0  date 01-01-2004 Extra parameter nspecp
!        version  1.1  date 14-09-2003 Extension for shape -199
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodesedges with the local node numbers of the nodes on the
!     iedge-th edge of an element with shape number ishape
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     edge
!     shape
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, iedge, nodesedges(*), nspecp

!     iedge          i    edge sequence number
!     ishape         i    Shape number of elements
!     nodesedges     o    Array of length nnode_edge containing the node
!                         numbers of the edges
!     nspecp         i    number of spectral points in one direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istartedge(18), edgesnodes(237), ininumnodedge, &
              edgenodeprism(18), numnodesedge(18), istart, i, &
              edgenodepris22(27)

!     edgenodepris22 Contains the local node numbers of the edges of the
!                    quadratic prisms
!     edgenodeprism  Contains the local node numbers of the edges of the
!                    linear prisms
!     edgesnodes     Contains the local node numbers of the edges of the
!                    elements with shape numbers smaller than 21
!     i              Counting variable
!     ininumnodedge  number of nodes per edge
!     istart         First position used at a part
!     istartedge     Starting position of edge in array NMESHD
!     numnodesedge   Array containing the number of nodes per edge as
!                    function of the shape number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data istartedge /  0,  2,  5, 11, 20,   28, 40, 49, 61, 69, &
                        75, 87,105,129,165,  183,201,225 /
      data edgesnodes /  1, 2,   1, 2, 3,   1, 2,  2, 3,  3, 1, &
                         1, 2, 3,   3, 4, 5,  5, 6, 1, &
                         1, 2,   2, 3,   3, 4,   4, 1, &
                         1, 2, 3,   3, 4, 5,   5, 6, 7,  7, 8, 1, &
                         1, 2, 3,   3, 4, 5,   5, 6, 1, &
                         1, 2, 3, 4,  4, 5, 6, 7,  7, 8, 9, 1, &
                         1, 2,   2, 3,   3, 4,  4, 1, &
                         1, 2,   2, 3,   3, 1, &
                         1, 2,   2, 4,   4, 1,  1, 3, 2, 3,  4, 3, &
                         1, 2, 3,  3, 8,10, 10, 7, 1, 1, 6, 5,  3, 4, 5, &
                        10, 9, 5, &
                         1, 2,  2, 6,  6, 5, 5, 1, &
                         4, 3,  3, 7,  7, 8, 8, 4, &
                         1, 4,  2, 3,  6, 7, 5, 8, &
                         1, 2, 3,  3,12,21,  21,20,19, 19,10, 1, &
                         7, 8, 9,  9,18,27,  27,26,25, 25,16, 7, &
                         1, 4, 7,  3, 6, 9,  21,24,27, 19,22,25, &
                         1, 2, 3,  3, 8,10,  10, 7, 1, 1, 6, 5, 3, 4, 5, &
                         10, 9, 5, &
                         1, 2, 3,  3, 8,10,  10, 7, 1, 1, 6, 5, 3, 4, 5, &
                         10, 9, 5, &
                         1, 2,  2, 6,  6, 5,  5, 1, &
                         4, 3,  3, 7,  7, 8,  8, 4, &
                         1, 4,  2, 3,  6, 7,  5, 8, &
                         1, 2,  2, 4,  4, 1,  1, 3, 2, 3,  4, 3 /
      data numnodesedge / 2, 3, 2, 3, 2,    3, 3, 4, 2, 2, &
                          2, 3, 2, 3, 3,    3, 2, 2 /
      data edgenodeprism / 1, 2, 2, 3, 3, 1, 4, 5, 5, 6, 6, 4, &
                           1, 4, 2, 5, 3, 6 /
      data edgenodepris22 / 1, 2, 3,   3, 4, 5,   5, 6, 1, &
                           13,14,15,  15,16,17,  17,18,13, &
                            1, 7,13,   3, 9,15,   5,11,17/
! ======================================================================
!
      if ( ishape==-10001 ) then

!     --- Elements of shape -10001

         nodesedges(1) = 1

      else if ( ishape==-9 ) then

!     --- Elements of shape -9

         nodesedges(1) = 1
         nodesedges(2) = 2

      else if ( nspecp>0 ) then

!     --- Spectral elements

         if ( ishape==1 ) then

!        --- 1d elements

            do i = 1, nspecp
               nodesedges(i) = i
            end do

         else if ( ishape==5 ) then

!        --- 2d elements

            if ( iedge==1 ) then
               do i = 1, nspecp
                  nodesedges(i) = i
               end do
            else if ( iedge==2 ) then
               do i = 1, nspecp
                  nodesedges(i) = i*nspecp
               end do
            else if ( iedge==3 ) then
               do i = 1, nspecp
                  nodesedges(i) = nspecp*nspecp-i+1
               end do
            else
               do i = 1, nspecp
                  nodesedges(i) = nspecp*nspecp-i*nspecp+1
               end do
            end if

         else if ( ishape==13 ) then

!        --- 3d elements

            if ( iedge==1 ) then
               do i = 1, nspecp
                  nodesedges(i) = i
               end do
            else if ( iedge==2 ) then
               do i = 1, nspecp
                  nodesedges(i) = (i-1)*nspecp*nspecp+nspecp
               end do
            else if ( iedge==3 ) then
               do i = 1, nspecp
                  nodesedges(i) = (nspecp-1)*nspecp*nspecp+nspecp-i+1
               end do
            else if ( iedge==4 ) then
               do i = 1, nspecp
                  nodesedges(i) = (nspecp-1)*nspecp*nspecp+1- &
                                  (i-1)*nspecp*nspecp
               end do
            else if ( iedge==5 ) then
               do i = 1, nspecp
                  nodesedges(i) = (nspecp-1)*nspecp+i
               end do
            else if ( iedge==6 ) then
               do i = 1, nspecp
                  nodesedges(i) = i*nspecp*nspecp
               end do
            else if ( iedge==7 ) then
               do i = 1, nspecp
                  nodesedges(i) = nspecp*nspecp*nspecp-i+1
               end do
            else if ( iedge==8 ) then
               do i = 1, nspecp
                  nodesedges(i) = (nspecp-1)*nspecp+ &
                                  (nspecp-1)*nspecp*nspecp+1- &
                                  (i-1)*nspecp*nspecp
               end do
            else if ( iedge==9 ) then
               do i = 1, nspecp
                  nodesedges(i) = 1+(i-1)*nspecp
               end do
            else if ( iedge==10 ) then
               do i = 1, nspecp
                  nodesedges(i) = nspecp+(i-1)*nspecp
               end do
            else if ( iedge==11 ) then
               do i = 1, nspecp
                  nodesedges(i) = (nspecp-1)*nspecp*nspecp+ &
                                  nspecp+(i-1)*nspecp
               end do
            else
               do i = 1, nspecp
                  nodesedges(i) = (nspecp-1)*nspecp*nspecp+1+ &
                                  (i-1)*nspecp
               end do
            end if

         else

!        --- Shape number not implemented

            call eropen ( 'ininodeedges' )
            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            call erclos ( 'ininodeedges' )
            ininumnodedge = -1

         end if

      else if ( ishape>=1 .and. ishape<=18 ) then

!     --- Standard elements of shapes between 1 and 18

         ininumnodedge = numnodesedge(ishape)
         istart = istartedge(ishape)+(iedge-1)*ininumnodedge
         do i = 1, ininumnodedge
            nodesedges(i) = edgesnodes(istart+i)
         end do

      else if ( ishape==21 ) then

!     --- Elements of shape 21

         ininumnodedge = 2
         istart = (iedge-1)*ininumnodedge
         do i = 1, ininumnodedge
            nodesedges(i) = edgenodeprism(istart+i)
         end do

      else if ( ishape==22 ) then

!     --- Elements of shape 22

         ininumnodedge = 3
         istart = (iedge-1)*ininumnodedge
         do i = 1, ininumnodedge
            nodesedges(i) = edgenodepris22(istart+i)
         end do

      else if ( ishape<=0 ) then

!     --- Elements that must be skipped

         ininumnodedge = -1

      else

!     --- Shape number not implemented

         call eropen ( 'ininodeedges' )
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         call erclos ( 'ininodeedges' )
         ininumnodedge = -1

      end if

      end
