      subroutine ininodeelgrp ( ielgrp, nodes, nnodes, map )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all nodes of element group ielgrp in array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielgrp, nodes(*), nnodes, map(*)

!     ielgrp         i    element group number
!     map            o    Mapping array of length npoint to make an
!                         inverse mapping of array nodes
!                         map(i) = 0, means that node i is not in the
!                                     element group
!                         map(i)>0, defines position of node i in array nodes
!     nnodes        i/o   Number of nodes in array nodes
!                         At input declared length of nodes
!                         At output actual number of nodes
!     nodes          i    Contains the node numbers of the nodes in the
!                         element group
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, numnodes, lenkmeshc, ikelmc

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     lenkmeshc      length of part of array kmeshc corresponding to
!                    element group
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     ININODEELGRP1  Fill all nodes of one element group in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodeelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodeelgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting address of element group in kmeshc

      ikelmc = 1
      do i = 1, ielgrp-1

         inpelm = Topgrp(i)%inpelm
         inelem = Topgrp(i)%inelem
         ikelmc = ikelmc+inelem*inpelm

      end do  ! i = 1, ielgrp-1

!     --- Compute length of kmeshc for ielgrp

      inpelm = Topgrp(ielgrp)%inpelm
      inelem = Topgrp(ielgrp)%inelem

      lenkmeshc = inpelm*inelem

      call ininodeelgrp1 ( kmeshc, nodes, numnodes, lenkmeshc, &
                           map, npoint )

      call inichkarlen ( numnodes, nnodes, 'nodes' )
      nnodes = numnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ininodeelgrp'

      end if  ! ( debug )
      call erclos ( 'ininodeelgrp' )

      end
