      function iniposgrid ( indi, nx, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Computes the starting position of cell defined by indi in grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     grid
!     cell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iniposgrid, ndim, indi(ndim), nx(ndim)

!     indi           i    contains ijk-index of cell in which point is lying
!     iniposgrid     o    starting address
!     ndim           i    Dimension of the space.
!     nx             i    number of points in ndim directions of the grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniposgrid' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniposgrid'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 ) then

!     --- ndim = 2

         iniposgrid = (indi(2)-1)*nx(1)+indi(1)

      else

!     --- ndim = 2

         iniposgrid = (indi(3)-1)*nx(1)*nx(2)+(indi(2)-1)*nx(1)+indi(1)

      end if  ! ( ndim==2

1000  call erclos ( 'iniposgrid' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniposgrid'

      end if  ! ( debug )

      end
