      function inishapesurvol ( ishapevolm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-05-2014 Extension with shape 22
!        version  1.1  date 29-12-2002 Extension with prisms
!        version  1.0  date 25-05-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with shape number of faces of volume element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     volume
!     surface
!     face
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inishapesurvol, ishapevolm

!     inishapesurvol o    shape number of faces of volume element
!     ishapevolm     i    shape number of volume element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishapesur(8)

!     ishapesur      shape number of surface element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1025   shape number has not yet been implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ishapesur / 3, 4, 5, 6, 7,   7, 5, 3 /
! ======================================================================
!
      call eropen ( 'inishapesurvol' )
      if ( ishapevolm>10 .and. ishapevolm<=18 ) then

!     --- standard case

         inishapesurvol = ishapesur(ishapevolm-10)

      else if ( ishapevolm==21 .or. ishapevolm==22 ) then

!     --- special case of prisms

         inishapesurvol = 0

      else

!     --- shape number not yet implemented

          call errint ( ishapevolm, 1 )
          call errchr ( 'topqu8', 1 )
          call errsub ( 1025, 1, 0, 1 )
          inishapesurvol = 0

      end if
      call erclos ( 'inishapesurvol' )
      end
