      function init08 ( inunkp, ndim, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.9  date 26-05-2014 Extension for shape 22
!        version  2.8  date 13-05-2009 Extension for shape 17
!        version  2.7  date 01-03-2007 Options -1019..-1021
!
!   copyright (c) 1995-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     init08 gets the value corresponding to inunkp and ndim
!     in the special cases that inunkp<-1000
! **********************************************************************
!
!                       KEYWORDS
!
!     file_1
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inunkp, ndim, init08, ishape

!     init08  o    Number of degrees of freedom
!                  init08 depends on inunkp and ndim
!                  The following values of inunkp give as result:
!                  -1001:  ndim
!                  -1002:  (1,3,6) for ndim=(1,2,3)
!                  -1003:  (0,1,3) for ndim=(1,2,3)
!                  -1004:  ndim^2
!                  -1005:  2*ndim+1
!                  -1006:  (0,0,3) for ndim=(1,2,3)
!                  -1007:  Depends on ishape:
!                          ishape 6, 7, 8, 14, 16  2*ndim+1
!                          ishape 9, 17, 18        1
!                  -1008:  Depends on ishape:
!                          ishape 6, 7, 8, 14, 16  2*ndim+1
!                          ishape 9, 17, 18        ndim+1
!                  -1009:  Depends on ishape:
!                          ishape 6, 7, 8, 14, 16  1
!                          ishape 9, 17, 18        0
!                  -1010:  Depends on ishape:
!                          ishape 6, 7, 8, 14, 16  2
!                          ishape 9, 17, 18        0
!                  -1011:  Depends on ishape:
!                          ishape 6, 7, 8, 14, 16  3
!                          ishape 9                1
!                          ishape  17, 18          0
!                  -1012:  Depends on ishape:
!                          ishape 6, 7, 14, 16     4
!                          ishape  17, 18          1
!                          rest                    0
!                  -1013:  ndim+1
!                  -1014   Depends on ishape:
!                          ishape= 7, 15, 16,17 : 0
!                          else         : 1
!                  -1015   Depends on ishape:
!                          ishape= 7, 15, 16,17 : 0
!                          else         : 2
!                  -1016   Depends on ishape:
!                          ishape= 7, 15, 16,17 : 0
!                          else         : 3
!                  -1017   Depends on ishape:
!                          ishape= 7, 15, 16,17 : 0
!                          else         : 6
!                  -1018   Depends on ishape:
!                          ishape= 7, 15, 16,17 : 0
!                          else         : ndim
!     inunkp  i    Number of degrees of freedom
!                  In this case it is supposed that inunkp<-1000
!     ishape  i    Shape number of element
!     ndim    i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    2867   Value of inunkp not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init08' )
      if ( ishape>18 .and. ishape<=20 .or. ishape>22 ) then

!     --- Shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if

      if ( inunkp>=-1000 ) then
         init08 = inunkp
         go to 1000
      end if  ! ( inunkp>=-1000 )

      select case ( inunkp )
      case ( -1001 )
         init08 = ndim
      case ( -1002 )
         if ( ndim==1 ) then
            init08 = 1
         else if ( ndim==2 ) then
            init08 = 3
         else
            init08 = 6
         end if
      case ( -1003 )
         if ( ndim==1 ) then
            init08 = 0
         else if ( ndim==2 ) then
            init08 = 1
         else
            init08 = 3
         end if
      case ( -1004 )
         init08 = ndim**2
      case ( -1005 )
         init08 = 2*ndim+1
      case ( -1006 )
         if ( ndim<=2 ) then
            init08 = 0
         else
            init08 = 3
         end if
      case ( -1007 )
         if ( ishape>=6 .and. ishape<=8 .or. ishape==14 .or. &
              ishape==16 .or. ishape==22 ) then
            init08 = 2*ndim+1
         else
            init08 = 1
         end if
      case ( -1008 )
         if ( ishape>=6 .and. ishape<=8 .or. ishape==14 .or. &
              ishape==16 .or. ishape==22 ) then
            init08 = 2*ndim+1
         else
            init08 = ndim+1
         end if
      case ( -1009 )
         if ( ishape==9 .or. ishape==17 .or. ishape==18 ) then
            init08 = 0
         else
            init08 = 1
         end if
      case ( -1010 )
         if ( ishape==9 .or. ishape==17 .or. ishape==18 ) then
            init08 = 0
         else
            init08 = 2
         end if
      case ( -1011 )
         if ( ishape==9 ) then
            init08 = 1
         else if ( ishape==17 .or. ishape==18 ) then
            init08 = 0
         else
            init08 = 3
         end if
      case ( -1012 )
         if ( ishape==17 .or. ishape==18 ) then
            init08 = 1
         else if ( ishape==6 .or. ishape==7 .or. &
                   ishape==14 .or. ishape==16 .or. ishape==22 ) then
            init08 = 4
         else
            init08 = 0
         end if
      case ( -1013 )
         init08 = ndim+1
      case ( -1014 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            init08 = 1
         end if
      case ( -1015 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            init08 = 2
         end if
      case ( -1016 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            init08 = 3
         end if
      case ( -1017 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            init08 = 6
         end if
      case ( -1018 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            init08 = ndim
         end if
      case ( -1019 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            if ( ndim==1 ) then
               init08 = 1
            else if ( ndim==2 ) then
               init08 = 3
            else
               init08 = 6
            end if
         end if
      case ( -1020 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            if ( ndim==1 ) then
               init08 = 0
            else if ( ndim==2 ) then
               init08 = 1
            else
               init08 = 3
            end if
         end if
      case ( -1021 )
         if ( ishape==7 .or. ishape>=15 .and. ishape<=17 ) then
            init08 = 0
         else
            init08 = ndim**2
         end if
      case default
         call errint ( inunkp, 1 )
         call errsub ( 2867, 1, 0, 0 )
      end select  ! case ( inunkp )

1000  call erclos ( 'init08' )

      end
