      subroutine msh071 ( ichois, iref, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 21-11-2012 Long integers
!        version  3.1  date 09-05-2012 Extra debug
!        version  3.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1991-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of the mesh from the file indicated by iref
!     This subroutine replaces the old subroutine MESHRD with a minor extension
!     (ihelp)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      use sepmodulemach
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iref, ihelp(10)

!     ichois  i    choice parameter,  possibilities:
!                 >=0    mesh is read,  Subroutine TOPOL is called with ichois
!                                         as first parameter
!                    -1    mesh is read,  Subroutine TOPOL is skipped
!                    -2    mesh is read,  Subroutine TOPOL is called
!                          Parameter ichois depends on version number of
!                          file meshoutput
!     ihelp   o    Contains the input written by MSH059 on the second record
!                  after nvolms
!     iref    i    dataset reference number for the output
!                  when iref = 0 the standard output file is assumed
!                  when iref>=0 the file is supposed to be formatted,
!                                 else an unformatted file is assumed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefc, i, ios, lengpc, nelgrpsav, &
              lengav, n, ivers, lenkmss, &
              jchois(10), lniinp, lnrinp, lenkmsk, &
              lenkmsh, lenkmsq, lenkmsr, &
              lenkmsv, leng, lenkmsx, index, iobst, lenin, &
              lenrl, nextra, ioutpsav, mmiinput
      double precision dummy(1)
      logical renumber
      character (len=80) text
      double precision :: timebefore(2)

!     dummy          Dummy parameter
!     i              Counting variable
!     index          Memory management sequence number
!     iobst          Obstacle sequence number
!     ios            IOSTAT variable, returns from reading or writing
!     ioutpsav       Help parameter to store the value of ioutp
!     irefc          dataset reference number for reading (>0 formatted, &
!                    <0 unformatted)
!     ivers          Version sequence number
!     jchois         Parameter ichois for TOPOL
!     leng           Dummy length
!     lengav         Available length for an array
!     lengpc         Length of KMESH part c
!     lenin          Number of integers in one row of kmeshx
!     lenkmsh        Length of array kmeshh
!     lenkmsk        Length of array kmeshk
!     lenkmsq        Length of array kmeshq
!     lenkmsr        Length of array kmeshr
!     lenkmss        Length of array kmeshs
!     lenkmsv        Length of array KMESH part v
!     lenkmsx        Length of array KMESH part x
!     lenrl          Number of reals in one row of rmeshx
!     lniinp         Available space for array IINPUT
!     lnrinp         Available space for array RINPUT
!     mmiinput       Memory management sequence number with respect to
!                    iinput
!     n              Help variable to store ncurvs, nsurfs or nvolms
!     nelgrpsav      Saves value of nelgrp
!     nextra         Number of extra variables stored in kmeshh(5,.)
!     renumber       If true standard renumbering must be applied
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: ininodeshape

!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     ININODESHAPE   Returns with number of nodes in element as function of
!                    the shape number
!     MSH000         Fill some parameters from array IINPUT
!     MSH021         Fill first part of array KMESH
!     MSH022         Read KMESH part c and COOR from a file
!     MSH070         read kmesh part j from the file indicated by irefc
!     MSHCHECKELMS   Check if two-dimensional mesh contains hanging nodes
!     MSHDELKMESHH   Delete kmesh part h and subarrays
!     MSHFILCR       Fill array icurvs and curves
!     MSHFILSR       Fill array kmesh part h with respect to surfaces
!     MSHFILVL       Fill array kmesh part h with respect to volumes
!     MSHFLKMP       Fill array kmesh part p
!     MSHOBSIN33     Fill information in kmesh part x concerning one obstacle,3D
!     MSHPOSJAC      Change sequence of nodes in elements with negative Jacobian
!     MSHRDMESHDIM   Read first records from file meshoutput
!     MSHREADKMESHT  Read kmesh part t from file meshoutput
!     MSHREADTEXT    Read text from file and compare with text that is expected
!     MSHUSERMESH    Create extra surfaces, curves and user points in a user
!                    mesh if necessary
!     PRININ         print 1d integer array
!     SEPCREAKMHSUR  allocate kmhsur
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLKMESHS  Make relation between kmeshs and kmeshs2
!     SEPFILLSURNODS Create and fill array surfnodes
!     SEPPOINTKMESHC Fill array pointkmsc with starting addresses of element
!                    groups in kmeshc (minus 1)
!     SEPRDKMESHLMN  Read kmesh parts l m and n from the file meshoutput
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       4   Number of element groups read is incorrect
!      20   nboun has wrong value
!      62   dimension of space (ndim) has wrong value
!      73   Maximal number of points per element is incorrect
!     119   The number of curves is incorrect
!     150   The number of surfaces is incorrect
!     192   MESHTP has incorrect value
!    1380   The number of user points is incorrect
!    1836   The number of elements is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ndim=2 ) then
!        Fill array KMESH part P
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh071' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh071'
         write(irefwr,1) 'iref, kelmt, ichois', iref, kelmt, ichois
         ioutpsav = ioutp
         ioutp = 1
  1      format ( a, 1x, (10i9) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( kmesh<=0 .or. kmesh>lastmesh ) kmesh = 0
      call sepcrmeshinfo ( kmesh )
      call seprelkmesh1
100   format ( a )
      irefc = iref
      if ( iref==0 ) irefc = irefre
      if ( debug ) write(irefwr,1) 'irefc', irefc

!     --- Read dimension parameters and first text records

      call mshrdmeshdim ( irefc, nboun, ihelp, ivers )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ndim, npelm, nboun, ishape, nelgrp, npoint', &
                          ndim, npelm, nboun, ishape, nelgrp, npoint
         write(irefwr,1) 'nelem, nuspnt, ncurvs, nsurfs, nvolms', &
                          nelem, nuspnt, ncurvs, nsurfs, nvolms
      end if  ! ( debug )

!     --- Check parameters

      if ( ndim<=0 .or. ndim>3 ) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
         go to 1000
      end if
      if ( npelm<=0 .or. npelm>5000 ) then
         call errint ( npelm, 1 )
         call errint ( 1000, 2 )
         call errsub ( 73, 2, 0, 0 )
      end if
      if ( nelgrp<=0 .or. nelgrp>10000 ) then
         call errint ( nelgrp, 1 )
         call errint ( 1000, 2 )
         call errsub ( 4, 2, 0, 0 )
      end if
      if ( nelem<1 ) then
         call errint ( nelem, 1 )
         call errint ( 100000, 2 )
         call errsub ( 1836, 2, 0, 0 )
      end if
      if ( nuspnt<0 .or. nuspnt>10*npoint ) then
         call errint ( nuspnt, 1 )
         call errint ( 10*npoint, 2 )
         call errsub ( 1380, 2, 0, 0 )
      end if
      if ( ncurvs<0 .or. ncurvs>10000 ) then
         call errint ( ncurvs, 1 )
         call errint ( 10000, 2 )
         call errsub ( 119, 2, 0, 0 )
      end if
      if ( nsurfs<0 .or. nsurfs> 10000) then
         call errint ( nsurfs, 1 )
         call errint ( 10000, 2 )
         call errsub ( 150, 2, 0, 0 )
      end if
      if ( nboun<0 .or. nboun>npelm ) then
         call errint ( nboun, 1 )
         call errint ( npelm, 2 )
         call errsub ( 20, 2, 0, 0 )
      end if
      if ( ishape<0 .or. ishape>100 ) then
         call errint ( ishape, 1 )
         call errint ( 100, 2 )
         call errint ( 0, 3 )
         call errsub ( 192, 3, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( debug .or. ioutp>=1 ) &
           write(irefwr,110) ndim, npelm, nboun, ishape, nelgrp, npoint, &
                             nelem, nuspnt, ncurvs, nsurfs, nvolms
110   format(' ndim =', i3, '   npelm =', i5, '   nboun =', i5, &
             '    ishape =', i4/ ' nelgrp =', i5, &
             ' npoint =', i8, '   nelem =', i8, '   nuspnt =', i6/ &
             ' ncurvs =', i6, '    nsurfs =', i6, '  nvolms =', i6 )

!     --- Fill the first part of array kmesh

      if ( debug ) write(irefwr,*) 'before msh021'
      call msh021 ( irefc, lengpc, ihelp, nboun )

      if ( km(iseqkm)%kmesh(23)>0 ) deallocate ( km(iseqkm)%coor )
      allocate ( km(iseqkm)%coor(ndim,npoint), km(iseqkm)%kmeshc(lengpc) )
      coor => km(iseqkm)%coor
      coorglob => coor
      kelmi = npoint*ndim
      kmeshc => km(iseqkm)%kmeshc
      kelmc = lengpc
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'before msh022'
      call msh022 ( irefc, lengpc, ivers )
      if ( ierror/=0 ) go to 1000

!     --- Read kmesh parts l, m and n

      if ( debug ) write(irefwr,*) 'before mshrdkmeshlmn'
      call seprdkmeshlmn ( irefc, ivers, ihelp )

      if ( ivers>=1 .and. ihelp(5)>0 ) then

!     --- Read kmesh part j

         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( debug ) write(irefwr,*) text
         end if
         if ( ichois==-2 ) then

!        --- ichois = -2, read kmeshj

            if ( associated(km(iseqkm)%kmeshj) ) &
               deallocate ( km(iseqkm)%kmeshj )
            allocate ( km(iseqkm)%kmeshj(npoint) )
            kmeshj => km(iseqkm)%kmeshj
            kelmj = npoint

            call msh070 ( kmeshj, npoint, irefc )

         else

!        --- Read dummy

            if ( irefc>=0 ) then
                read (irefc,*,iostat=ios) (n,i=1,npoint)
            else
                read(-irefc,iostat=ios) n
            end if

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- READ KMESH t if necessary

      if ( debug ) write(irefwr,1) 'kelmt', kelmt
      if ( kelmt/=0 ) then

!     --- kelmt/=0, read kmesh t

         if ( debug ) write(irefwr,*) 'before mshreadkmesht'
         call mshreadkmesht ( ios, irefc, ivers )

      end if
      if ( debug ) write(irefwr,*) 'after mshreadkmesht'

!     --- Read spectral part of KMESH if necessary

      if ( debug ) write(irefwr,1) 'kelmu, ivers', kelmu, ivers

      if ( kelmu>0 ) then

!     --- Read spectral part

         if ( irefc>=0 ) then

            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read(irefc, *) ( kmeshu(i), i = 1, 5 )

         else

            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc)  ( kmeshu(i), i = 1, 5 )

         end if
         if ( debug ) write(irefwr,*) text

      end if
      if ( ivers>1 ) then

!     --- At least version 2, read arrays iinput and rinput

         if ( irefc>=0 ) then
            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read(irefc,*) lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr, &
                          lenkmss
         else
            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc)  lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr, &
                          lenkmss
         end if
         if ( debug ) then
            write(irefwr,*) text
            write(irefwr,1) 'lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr', &
                             lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr
            write(irefwr,1) 'lenkmss', lenkmss
         end if  ! ( debug )
         if ( lniinp>0 ) then

!        --- Read iinput and rinput

            if ( associated ( km(iseqkm)%iinputkm ) ) &
               deallocate ( km(iseqkm)%iinputkm, km(iseqkm)%rinputkm )
            allocate ( km(iseqkm)%iinputkm(lniinp), &
                       km(iseqkm)%rinputkm(lnrinp) )
            iinputkm => km(iseqkm)%iinputkm
            rinputkm => km(iseqkm)%rinputkm
            kmiinput = lniinp
            kmrinput = lnrinp
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (iinputkm(i),i=1,lniinp)
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (rinputkm(i),i=1,lnrinp)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (iinputkm(i),i=1,lniinp)
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (rinputkm(i),i=1,lnrinp)
            end if
            if ( debug ) write(irefwr,*) text

         end if
         if ( lenkmsq>0 ) then
            if ( associated(km(iseqkm)%kmeshq) ) &
               deallocate ( km(iseqkm)%kmeshq )
            allocate ( km(iseqkm)%kmeshq(lenkmsq) )
            kmeshq => km(iseqkm)%kmeshq
            kelmq = lenkmsq
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (kmeshq(i),i=1,lenkmsq)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (kmeshq(i),i=1,lenkmsq)
            end if
            if ( debug ) write(irefwr,*) text
         end if
         if ( lenkmsr>0 ) then
            if ( associated(km(iseqkm)%kmeshr) ) &
               deallocate ( km(iseqkm)%kmeshr )
            allocate ( km(iseqkm)%kmeshr(lenkmsr) )
            kmeshr => km(iseqkm)%kmeshr
            kelmr = lenkmsr
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (kmeshr(i),i=1,lenkmsr)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (kmeshr(i),i=1,lenkmsr)
            end if
            if ( debug ) write(irefwr,*) text
         end if

         if ( lenkmss>0 ) then
            if ( associated(km(iseqkm)%kmeshs) ) &
               deallocate ( km(iseqkm)%kmeshs )
            allocate ( km(iseqkm)%kmeshs(lenkmss) )
            kmeshs => km(iseqkm)%kmeshs
            kelms = lenkmss
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (kmeshs(i),i=1,lenkmss)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (kmeshs(i),i=1,lenkmss)
            end if
            call sepfillkmeshs ( km(iseqkm)%kmeshs(nsurfs+2) )
            if ( debug ) write(irefwr,*) text
         end if

         if ( lenkmsh>0 ) then

!        --- create array kmesh part h

            length = nsurfs+nvolms+1
            if ( associated(km(iseqkm)%kmeshh) ) call mshdelkmeshh
            allocate ( km(iseqkm)%kmeshh(6,length) )
            kmeshh => km(iseqkm)%kmeshh
            kelmh = length

            call sepcreakmhsur

            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) kmeshh
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) kmeshh
            end if
            if ( debug ) write(irefwr,*) text
            n = kmeshh(5,1)
            length = ncurvs*2
            if ( associated(km(iseqkm)%icurvsmsh) ) &
               deallocate ( km(iseqkm)%icurvsmsh, km(iseqkm)%curvesmsh, &
                            km(iseqkm)%curvefacts )
            allocate ( km(iseqkm)%icurvsmsh(length), &
                       km(iseqkm)%curvesmsh(ndim,n), &
                       km(iseqkm)%curvefacts(n) )
            icurvsmsh => km(iseqkm)%icurvsmsh
            curvesmsh => km(iseqkm)%curvesmsh
            curvefacts => km(iseqkm)%curvefacts
            kmeshh(4,1) = 0
            kmeshh(6,1) = n

!           --- Fill arrays icurvs and curves

            icurvsmsh(ncurvs+1:2*ncurvs) = 0
            length = n
            if ( ivers<=2 ) then
               call mshfilcr ( icurvsmsh, curvesmsh, &
                               kmeshm, coor, ncurvs, ndim )
               curvefacts(1:n) = 0d0
            else
               if ( ivers<=5 ) then
                  lengav = ncurvs
               else
                  lengav = 2*ncurvs
               end if
               if ( irefc>=0 ) then
                  read(irefc,*) (icurvsmsh(i),i=1,lengav)
                  read(irefc,*) curvesmsh
                  if ( ivers>7 ) read(irefc,*) &
                     (curvefacts(i),i=1,n)
               else
                  read(-irefc) (icurvsmsh(i),i=1,lengav)
                  read(-irefc) curvesmsh
                  if ( ivers>7 ) read(-irefc) &
                     (curvefacts(i),i=1,n)
               end if
            end if
            if ( ihelp(8)==1 ) then

!           --- Fill surfaces

               do isurnr = 1, nsurfs
                  isurf = kmeshs(nsurfs+2+(isurnr-1)*8)
                  if ( isurf>0 .and. isurf/=7 .and. &
                     isurf/=11 .and. isurf/=12 .and. &
                     kmeshh(3,isurnr+1)>0 ) then
                     if ( irefc>=0 ) then
                        read(irefc,*) npointlc, inelem, nextra
                     else
                        read(-irefc)  npointlc, inelem, nextra
                     end if
                     if ( debug ) &
                        write(irefwr,1) 'npointlc, inelem, nextra', &
                                         npointlc, inelem, nextra
                     if ( npointlc>0 ) then

!                    --- Non-empty surface

                        if ( associated(kmhsur(isurnr)%coorsur) ) &
                           deallocate ( kmhsur(isurnr)%coorsur )
                        kmeshh(1,isurnr+1) = npointlc
                        allocate ( kmhsur(isurnr)%coorsur(ndim,npointlc) )

                        if ( kmhsur(isurnr)%filled ) &
                           deallocate(kmhsur(isurnr)%kmeshcsur)
                        allocate( kmhsur(isurnr)%kmeshcsur(inelem) )
                        ishape = kmeshs(nsurfs+3+(isurnr-1)*8)
                        inpelm = ininodeshape ( ishape )
                        kmeshh(6,isurnr+1) = inpelm
                        kmhsur(isurnr)%filled = .true.
                        if ( associated(kmhsur(isurnr)%maparsur) ) &
                           deallocate ( kmhsur(isurnr)%maparsur )
                        allocate ( kmhsur(isurnr)%maparsur(npointlc) )
                        kmeshh(3,isurnr+1) = npointlc
                        kmeshh(4,isurnr+1) = 0
                        if ( nextra>0 ) then
                           if ( associated(km(iseqkm)%kmhsur(isurnr)%renumsur) )&
                              deallocate(km(iseqkm)%kmhsur(isurnr)%renumsur)
                           allocate(km(iseqkm)%kmhsur(isurnr)%renumsur(nextra) )
                           kmeshh(5,isurnr+1) = nextra
                        else
                           kmeshh(5,isurnr+1) = 0
                        end if
                        call mshfilsr ( isurnr, irefc )

                     end if

                  end if

               end do

!              --- Fill volumes

               do ivolnr = 1, nvolms
                  isurnr = nsurfs+ivolnr
                  if ( kmeshh(3,isurnr+1)>0 ) then
                     if ( ivers<=7 ) then
                        npointlc = kmeshh(6,isurnr+1)
                        inelem =  kmeshh(4,isurnr+1)
                     else
                        if ( irefc>=0 ) then
                           read(irefc,*) npointlc, inelem
                        else
                           read(-irefc)  npointlc, inelem
                        end if
                     end if
                     if ( associated(km(iseqkm)%kmhsur(isurnr)%coorsur) ) &
                        deallocate ( km(iseqkm)%kmhsur(isurnr)%coorsur )
                     kmeshh(1,isurnr+1) = npointlc
                     allocate ( kmhsur(isurnr)%coorsur(ndim,npointlc) )
                     if ( associated(km(iseqkm)%kmhsur(isurnr)%kmeshcsur) ) &
                        deallocate ( kmhsur(isurnr)%kmeshcsur )
                     allocate ( km(iseqkm)%kmhsur(isurnr)%kmeshcsur(inelem) )
                     kmhsur(isurnr)%filled = .true.
                     if ( associated(kmhsur(isurnr)%maparsur) ) &
                        deallocate ( kmhsur(isurnr)%maparsur )
                     allocate ( kmhsur(isurnr)%maparsur(npointlc) )
                     kmeshh(3,isurnr+1) = npointlc
                     kmeshh(4,isurnr+1) = 0
                     kmeshh(5,isurnr+1) = 0
                     call mshfilvl ( ivolnr, irefc )

                  end if

               end do

            end if

         end if

      end if
      if ( debug ) write(irefwr,1) 'kelmm, ndim', kelmm, ndim
      if ( kelmm/=0 .and. ndim==2 ) then

!     --- ndim = 2
!         Fill array KMESH part P

         if ( associated(km(iseqkm)%elsize) ) deallocate ( km(iseqkm)%elsize )
         allocate ( km(iseqkm)%elsize(2,ncurvs) )
         elsize => km(iseqkm)%elsize
         kelmp = ncurvs
         call mshflkmp ( coor, kmeshm, elsize, &
                         ncurvs, lenkmsh, icurvsmsh, &
                         curvesmsh )

      end if
      if ( ivers>=4 ) then

!     --- Version 4, read two extra records

         if ( irefc>=0 ) then
            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read ( irefc, *, iostat = ios ) nlevel, nsplit
            read ( irefc, *, iostat = ios ) ihelp(1)
         else
            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc, iostat = ios) nlevel, nsplit
            read(-irefc, iostat = ios) ihelp(1)
         end if
         if ( debug ) then
            write(irefwr,*) text
            write(irefwr,*) 'nlevel, nsplit, ihelp(1)', &
                             nlevel, nsplit, ihelp(1)
         end if  ! ( debug )

         if ( nlevel>0 ) then

!        --- Read kmesh part k

            lenkmsk = nlevel+nsplit
            if ( associated(km(iseqkm)%kmeshk) ) &
               deallocate ( km(iseqkm)%kmeshk )
            allocate ( km(iseqkm)%kmeshk(lenkmsk) )
            kmeshk => km(iseqkm)%kmeshk
            kelmk = lenkmsk
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (kmeshk(i),i=1,lenkmsk)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (kmeshk(i),i=1,lenkmsk)
            end if
            if ( debug ) write(irefwr,*) text

         end if

      end if
      if ( ivers>=5 ) then

!     --- Version 5, read kmesh parts v and w if they exist

         if ( irefc>=0 ) then
            if ( ivers>=7 ) read (irefc,100,iostat=ios) text
            read(irefc,*) (ihelp(i),i=1,10)
         else
            if ( ivers>=7 ) read(-irefc,iostat=ios) text
            read(-irefc) (ihelp(i),i=1,10)
         end if
         if ( debug ) then
            write(irefwr,*) text
            call prinin ( ihelp, 10, 'ihelp' )
         end if  ! ( debug )

         if ( ihelp(1)>0 ) then

!        --- kmesh part v exists

            if ( irefc>=0 ) then
               read(irefc,*) lenkmsv
            else
               read(-irefc) lenkmsv
            end if
            if ( associated(km(iseqkm)%kmeshv) ) &
               deallocate ( km(iseqkm)%kmeshv )
            allocate ( km(iseqkm)%kmeshv(lenkmsv) )
            kmeshv => km(iseqkm)%kmeshv
            kelmv = lenkmsv
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) (kmeshv(i),i=1,lenkmsv)
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) (kmeshv(i),i=1,lenkmsv)
            end if
            if ( debug ) write(irefwr,*) text

         end if

         if ( ihelp(2)>0 ) then

!        --- kmesh part w exists

            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) nintprop, nrealprop
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) nintprop, nrealprop
            end if
            if ( debug ) write(irefwr,*) text
            if ( associated(km(iseqkm)%kmeshw) ) &
               deallocate ( km(iseqkm)%kmeshw )
            allocate ( km(iseqkm)%kmeshw(4) )
            kmeshw => km(iseqkm)%kmeshw
            if ( kelmw==0 ) then
               kmeshw(1) = nintprop
               kmeshw(2) = nrealprop
               kmeshw(3) = 0
               kmeshw(4) = 0
            end if
            kelmw = 4
            if ( associated(km(iseqkm)%intprop) ) &
               deallocate ( km(iseqkm)%intprop, km(iseqkm)%realprop )
            if ( nintprop>0 ) then

!           --- nintprop>0, Read integer properties

               allocate ( km(iseqkm)%intprop(nintprop,nelgrp) )
               intprop => km(iseqkm)%intprop
               kmeshw(3) = nintprop*nelgrp
               if ( irefc>=0 ) then
                  read(irefc,*) intprop
               else
                  read(-irefc) intprop
               end if

            else

               allocate ( km(iseqkm)%intprop(1,nelgrp) )
               intprop => km(iseqkm)%intprop

            end if
            if ( nrealprop>0 ) then

!           --- nrealprop>0, Read real properties

               allocate ( km(iseqkm)%realprop(nrealprop,nelgrp) )
               realprop => km(iseqkm)%realprop
               kmeshw(4) = nrealprop*nelgrp
               if ( irefc>=0 ) then
                  read(irefc,*) realprop
               else
                  read(-irefc) realprop
               end if

            else

               allocate ( km(iseqkm)%realprop(1,nelgrp) )
               realprop => km(iseqkm)%realprop

            end if

         end if
         if ( ihelp(3)>0 ) then

!        --- kmesh part x exists

            numobst = ihelp(3)
            lenrl = numobst
            lenin = 6*(ndim-1)
            lenkmsx = ihelp(6)/lenin
            if ( lenkmsx==0 ) lenkmsx = numobst

            if ( kelmx>0 ) &
               deallocate ( km(iseqkm)%kmeshx, km(iseqkm)%rmeshx )
            allocate ( km(iseqkm)%kmeshx(lenin,lenkmsx), &
                       km(iseqkm)%rmeshx(3*ndim,lenrl), &
                       km(iseqkm)%kmobst(numobst) )
            kmeshx => km(iseqkm)%kmeshx
            kelmx = lenkmsx*lenin
            km(iseqkm)%kmesh(47) = kelmx
            kelmxrl = lenrl*3*ndim
            km(iseqkm)%kmesh(68) = kelmxrl
            rmeshx => km(iseqkm)%rmeshx
            if ( irefc>=0 ) then
               if ( ivers>=7 ) read (irefc,100,iostat=ios) text
               read(irefc,*) kmeshx
               read(irefc,*) rmeshx
            else
               if ( ivers>=7 ) read(-irefc,iostat=ios) text
               read(-irefc) kmeshx
               read(-irefc) rmeshx
            end if
            if ( debug ) write(irefwr,*) text
            if ( ndim==3 ) then

!           --- ndim = 3, also read coordinates and topology of the obstacles

               numobst = numobst
               do iobst = 1, numobst
                  isurnr = nsurfs+1
                  inpelm = kmeshx(10,iobst)
                  inelem  = kmeshx(11,iobst)
                  npointlc = kmeshx(12,iobst)
                  allocate ( km(iseqkm)%kmobst(iobst)% &
                             kmeshcobst(inpelm,inelem), &
                             km(iseqkm)%kmobst(iobst)% &
                             coorobst(ndim,npointlc) )
                  kmeshx(8,iobst) = inpelm*inelem
                  kmeshx(9,iobst) = ndim*npointlc
                  kmeshcobst => km(iseqkm)%kmobst(iobst)%kmeshcobst
                  coorobst => km(iseqkm)%kmobst(iobst)%coorobst
                  if ( irefc>=0 ) then
                     read(irefc,*) kmeshcobst
                  else
                     read(-irefc) kmeshcobst
                  end if

!                 --- Reserve work space

                  if ( irefc>=0 ) then
                     read(irefc,*) coorobst
                  else
                     read(-irefc) coorobst
                  end if

!                 --- Fill part of rmeshx if necessary, because it has not
!                     been filled actually
!                     This is for example the case if the mesh is not created by
!                     sepmesh

                  call mshobsin33 ( iobst )

               end do

            end if

!           --- Clear reference to array iobgrid

            do i = 1, numobst
               kmeshx(ndim+3,i) = 0
            end do

         end if
         if ( ihelp(9)>0 ) then

!        --- ihelp(9)>0, Read volume_surfs
!            Warning this subroutine uses memory management

            text = '//Volume_surfs'
            if ( debug ) write(irefwr,*) text
            call mshreadtext ( irefc, text, name10 )

            if ( irefc>=0 ) then
               read(irefc,*) length
            else
               read(-irefc) length
            end if  ! ( irefc>=0 )
            if ( length>0 ) then
               allocate ( km(iseqkm)%volume_surfs(length) )
               volume_surfs => km(iseqkm)%volume_surfs
               kelmag = length
               if ( irefc>=0 ) then
                  read(irefc,*) volume_surfs
               else
                  read(-irefc) volume_surfs
               end if  ! ( irefc>=0 )
            end if  ! ( length>0 )

         end if
         if ( ihelp(7)>0 ) then

!        --- ihelp(7)>0, Read outer curves
!            Warning this subroutine uses memory management

            text = '//Outercurves'
            if ( debug ) write(irefwr,*) text
            call mshreadtext ( irefc, text, name10 )

            if ( irefc>=0 ) then
               read(irefc,*) length
            else
               read(-irefc) length
            end if  ! ( irefc>=0 )
            if ( length>0 ) then
               allocate ( km(iseqkm)%outerboun(length) )
               outerboun => km(iseqkm)%outerboun
               mmouter = length
               if ( irefc>=0 ) then
                  read(irefc,*) outerboun
               else
                  read(-irefc) outerboun
               end if  ! ( irefc>=0 )
            end if  ! ( length>0 )

         end if

         if ( ihelp(8)>0 ) then

!        --- ihelp(8)>0, Read normals
!            Warning this subroutine uses memory management

            text = '//Normals'
            if ( debug ) write(irefwr,*) text
            call mshreadtext ( irefc, text, name10 )
            if ( irefc>=0 ) then
               read(irefc,*) length
            else
               read(-irefc) length
            end if  ! ( irefc>=0 )
            if ( length>0 ) then
               if ( associated(km(iseqkm)%normal) ) &
                  deallocate ( km(iseqkm)%normal )
               allocate ( km(iseqkm)%normal(ndim,length/ndim) )
               normal => km(iseqkm)%normal
               mmnormal = length
               if ( irefc>=0 ) then
                  read(irefc,*) normal
               else
                  read(-irefc) normal
               end if  ! ( irefc>=0 )
            end if  ! ( length>0 )

            if ( nvolms>0 .and. mmnormal>0 ) then

!           --- There are volumes, fill array surfnodes

               call sepfillsurnods

            end if

         end if

         if ( ihelp(10)>0 ) then

!        --- ihelp(10)>0, Read curvesup
!            Warning this subroutine uses memory management

            text = '//Curveusp'
            if ( debug ) write(irefwr,*) text
            call mshreadtext ( irefc, text, name10 )
            if ( irefc>=0 ) then
               read(irefc,*) length
            else
               read(-irefc) length
            end if  ! ( irefc>=0 )
            if ( length>0 ) then
               if ( associated(km(iseqkm)%curveusrp) ) &
                  deallocate ( km(iseqkm)%curveusrp )
               allocate ( km(iseqkm)%curveusrp(length) )
               curveusrp => km(iseqkm)%curveusrp
               kelmak = length
               if ( irefc>=0 ) then
                  read(irefc,*) curveusrp
               else
                  read(-irefc) curveusrp
               end if  ! ( irefc>=0 )
            end if  ! ( length>0 )

         end if

      end if

      if ( .not. associated(pointkmsc) ) allocate ( pointkmsc(nelgrp+1) )
      call sepfillkmeshc   ! Make relation between kmeshc and Topgrp
      call seppointkmeshc  ! Fill array pointkmsc

      renumber = ichois/=-1
      if ( mainsubr==6 ) then

!     --- mainsubr = 6, call by sepmakeparmesh
!         Always a renumbering is applied based on Cuthill-McKee

         renumber = .true.

      end if  ! ( mainsubr==6 )

      if ( renumber ) then

!     --- Perform renumbering if ichois # -1

         if ( .not. parallel .and. ichois==-2 ) then
            jchois(1) = 1000
         else if ( ichois==-2 ) then
            jchois(1) = 0
         else
            jchois(1) = ichois
         end if

         if ( mainsubr==6 ) then

!        --- mainsubr = 6, call by sepmakeparmesh
!            Always a renumbering is applied based on Cuthill-McKee

            jchois(1) = -10   ! 10 positions are filled by the user
            jchois(2) = 1     ! Cuthill-Mckee algorithm
            jchois(3) = 2     ! Use only the renumbered algorithm
            jchois(4) = 0     ! iprint
            jchois(5) = 3     ! Priority for vertex nodes
            jchois(6) = 3     ! Priority for mid point nodes
            jchois(7) = 3     ! Priority for centroids
            jchois(8) = 0     ! no renumbering per level
            jchois(9) = 0     ! Automatic start
            jchois(10) = 1    ! start value (not used)

         end if  ! ( mainsubr==6 )

         call topol ( jchois )
         if ( kelme==0 ) call sepfillkmeshe

      end if

      if ( check_hanging_nodes ) then

!     --- check_hanging_nodes is true, check mesh (hanging nodes)

         call mshcheckelms

      end if

      if ( ndim==3 .and. mmouter==0 .and. &
          (ncurvs<nsurfs .and. ncurvs<3 .or. nuspnt<1) ) then

!     --- ncurvs<nsurfs or nuspnt<ncurvs
!         Special situation, presumably this mesh has been created by
!         a third party mesh generator
!         Create extra surfaces, curves and user points if necessary
!         Make sure that all elements have positive jacobian

         call mshposjac
         call mshusermesh

      end if

!     --- Fill array intarmsh partly

      if ( kmiinput<=0 ) go to 1000
      if ( size (iinputkm)<20 ) go to 1000  ! no input in iinput

      nelgrpsav = nelgrp
      call msh000 ( iinputkm, jchois, 1 )
      nelgrp = nelgrpsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh071'
         ioutp = ioutpsav

      end if
      call erclmn ( 'msh071', timebefore )

      end subroutine msh071
