      subroutine mshcop1 ( iseqkm1, iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-07-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of kmesh1 to kmesh2
!     Parts: a,b,f,g
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqkm1, iseqkm2

!     iseqkm1        i    Sequence number with respect to km of first mesh
!     iseqkm2        i    Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcop1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcop1'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( km(iseqkm1)%kmesh(15)/=0 ) then

!     --- kmesh part a

         if ( km(iseqkm2)%kmesh(15)>0 ) deallocate(km(iseqkm2)%kmesha)
         length = size(km(iseqkm1)%kmesha)
         allocate ( km(iseqkm2)%kmesha(length) )
         km(iseqkm2)%kmesha = km(iseqkm1)%kmesha
         km(iseqkm2)%kmesh(15) = length

      end if

      if ( km(iseqkm1)%kmesh(16)/=0 ) then

!     --- kmesh part b

         if ( km(iseqkm2)%kmesh(16)>0 ) deallocate(km(iseqkm2)%kmeshb)
         length = size(km(iseqkm1)%kmeshb)
         allocate ( km(iseqkm2)%kmeshb(length) )
         km(iseqkm2)%kmeshb = km(iseqkm1)%kmeshb
         km(iseqkm2)%kmesh(16) = length

      end if

      if ( km(iseqkm1)%kmesh(20)/=0 ) then

!     --- kmesh part f

         if ( km(iseqkm2)%kmesh(20)>0 ) deallocate(km(iseqkm2)%kmeshf)
         length = size(km(iseqkm1)%kmeshf)
         allocate ( km(iseqkm2)%kmeshf(length) )
         km(iseqkm2)%kmeshf = km(iseqkm1)%kmeshf
         km(iseqkm2)%kmesh(20) = length

      end if

      if ( km(iseqkm1)%kmesh(21)>0 ) then

!     --- kmesh part g

         if ( km(iseqkm2)%kmesh(21)>0 ) deallocate(km(iseqkm2)%kmeshg)
         length = size(km(iseqkm1)%kmeshg)
         allocate ( km(iseqkm2)%kmeshg(length) )
         km(iseqkm2)%kmeshg = km(iseqkm1)%kmeshg
         km(iseqkm2)%kmesh(21) = length

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcop1'

      end if  ! ( debug )
      call erclos ( 'mshcop1' )

      end subroutine mshcop1
