      subroutine mshcopred ( kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-05-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the essential parts of array kmesh1 into kmesh2
!     Only those parts needed for subroutine refine are copied
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2

!     kmesh1     i    KMESH array to be copied
!     kmesh2    i/o   Copy of array KMESH
!                     All positions must have been initialized before
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelmi, kelmm, kelmc, kelmk, kelml, kelmn, &
              mmiinput, mmrinput, kelmq, kelmr, kelms, iseqkm1, iseqkm2

!     i              Counting variable
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kelmc          Last position used in kmesh part c
!     kelmi          Memory management sequence number of array coor
!     kelmk          Memory management sequence number of array kmeshk
!     kelml          Memory management sequence number of array kmeshl
!     kelmm          Memory management sequence number of array kmeshm
!     kelmn          Memory management sequence number of array kmeshn
!     kelmq          If kelmq>0 array kmeshq has been filled
!     kelmr          Memory management sequence number of array kmeshr
!     kelms          Memory management sequence number of array kmeshs
!     mmiinput       Memory management sequence number with respect to
!                    iinput
!     mmrinput       Memory management sequence number with respect to
!                    rinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOP1        Copy part of kmesh1 to kmesh2 (a,b,f,g)
!     MSHCOP3        Copy part of kmesh1 to kmesh2 (q,r,s,iinput,rinput,
!                    kmesht, kmeshquant)
!     SEPCRMESHINFO  Create permanent space for mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check length of array kmesh2
!    Save memory management positions of old arrays of kmesh2
!    Copy parts of array kmesh
!    Copy kmesh part i (co-ordinates)
!    Copy kmesh part c (topology)
!    Copy kmesh part k (levels)
!    Copy kmesh part m (curve description)
!    Copy kmesh part n (surface description)
!    Copy kmesh parts q, r and s (description of curves and so on)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopred' )

!     --- Check length of array kmesh2

      call sepcrmeshinfo ( kmesh2 )

      iseqkm1 = kmesh1
      iseqkm2 = kmesh2

      km(iseqkm2)%kmesh(4:14) = km(iseqkm1)%kmesh(4:14)
      km(iseqkm2)%kmesh(42) = km(iseqkm1)%kmesh(42)
      km(iseqkm2)%kmesh(44) = km(iseqkm1)%kmesh(44)
      km(iseqkm2)%kmesh(64) = km(iseqkm1)%kmesh(64)

      call mshcop1 ( iseqkm1, iseqkm2 )

!     --- Copy coordinates, kmeshc, kmeshk, kmeshl, kmeshm, kmeshn

      km(iseqkm2)%kmesh(17) = -km(iseqkm1)%kmesh(17)
      km(iseqkm2)%kmeshc => km(iseqkm1)%kmeshc
      km(iseqkm2)%kmesh(23) = -km(iseqkm1)%kmesh(23)
      km(iseqkm2)%coor => km(iseqkm1)%coor
      km(iseqkm2)%kmesh(25) = -km(iseqkm1)%kmesh(25)
      km(iseqkm2)%kmeshk => km(iseqkm1)%kmeshk
      km(iseqkm2)%kmesh(26) = -km(iseqkm1)%kmesh(26)
      km(iseqkm2)%kmeshl => km(iseqkm1)%kmeshl
      km(iseqkm2)%kmesh(27) = -km(iseqkm1)%kmesh(27)
      km(iseqkm2)%kmeshm => km(iseqkm1)%kmeshm
      km(iseqkm2)%kmesh(28) = -km(iseqkm1)%kmesh(28)
      km(iseqkm2)%kmeshn => km(iseqkm1)%kmeshn

!     --- Copy kmeshq, kmeshr, kmeshs iinput, rinput, kmesht

      call mshcop3 ( iseqkm1, iseqkm2 )

      call erclos ( 'mshcopred' )

      end subroutine mshcopred
