      function prandm ( iseed )
      implicit none

      double precision prandm
      integer iseed

! **********************************************************************
!
!    This is the subroutine DRANDM adapted from the book
!
!    W.H.Press, B.P.Flannery, S.A.Teukolsky, W.T.Vetteling
!    Numerical Recipes, Cambridge University Press, 1987,
!    p.196-197, function RUN1
!
!    Bugs fixed - the variables ix1, ix2, ix3 and the array r(*)
!                 were not declared as "save" so they lost their values
!                 on return, the published function didn't work
!               - user's seed iseed<0 was rewritten by +1, thus destroyed
!               - possible overflow in the term  ic1-issed
!               - nonsence test j in 1..97 and the pause statement.
!    Logical flag is used instead of integer flag.
!
!        adapted by     Zdenek Vlasek
!
!        version 1.0    date   19-02-1991  for integers with minumum 32 bits
!
! **********************************************************************
!
!    It returns one random number from a sequence of uniformly distributed
!    random numbers in the interval [0,1) .
!
!    iseed  ... input parameter
!
!    iseed <>0  a new seeds based on the value abs(iseed) are set
!               and then the first random number is returned
!               ( you obtain a repeatable sequence of random numbers )
!
!    iseed = 0  if it is the first call in the program run:  some default
!               seeds are set and the first random number is returned
!               ( you obtain the same sequence every time the program runs )
!
!               not the first call:  a next random number is returned
!
!    To generate a non repeatable sequence of random numbers,
!    derive the first value iseed<>0  e.g. from current date and time.
!
!    Some random numbers are generated here by a simple congruent method,
!    to achive better distribution, they are kept in the array r(*)
!    and extracted from it in a random order.
!
! **********************************************************************

      integer m1, m2, m3, ia1, ia2, ia3, ic1, ic2, ic3
      double precision rm1, rm2
      parameter ( m1=259200, ia1=7141, ic1=54773, rm1=3.8580247e-6 )
      parameter ( m2=134456, ia2=8121, ic2=28411, rm2=7.4373773e-6 )
      parameter ( m3=243000, ia3=4561, ic3=51349)

      logical first
      integer ix1, ix2, ix3, j
      double precision r(97)
      save    ix1, ix2, ix3, r

      data first /.true./

      if ( first .or. iseed/=0 ) then

!       ... set the seeds ix1, ix2, ix3

!       ... avoid an overflow in ic1+iseed
        ix1=mod( ic1 + mod(abs(iseed),2000000000), m1 )
        ix1=mod(ia1*ix1+ic1,m1)
        ix2=mod(ix1,m2)
        ix1=mod(ia1*ix1+ic1,m1)
        ix3=mod(ix1,m3)
!       ... fill the array r(*)
        do 11 j=1,97
          ix1=mod(ia1*ix1+ic1,m1)
          ix2=mod(ia2*ix2+ic2,m2)
          r(j)=(dble(ix1)+dble(ix2)*rm2)*rm1
11      continue
        first = .false.
      end if

!     ... the next random number, use the saved  ix1,ix2,ix3  as seeds
      ix1=mod(ia1*ix1+ic1,m1)
      ix2=mod(ia2*ix2+ic2,m2)
      ix3=mod(ia3*ix3+ic3,m3)
      j=1+(97*ix3)/m3
!     ... pick a random number
      prandm=r(j)
!     ... fill the gap
      r(j)=(dble(ix1)+dble(ix2)*rm2)*rm1
      end
