      subroutine prfillnodnods ( inpsep, nodes, nnodes, npoint, iplace )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of nodal points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes, npoint, iplace

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node1, node2, istep, i

!     i              Counting variable
!     istep          Step size in nodes between node1 and node2
!     node1          Node of face
!     node2          Node of face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1120   First nodal point number in CREATE NODES (N1 to N2) is out of range
!    1121   Last nodal point number in CREATE NODES (N1 to N2) is out of range
!    1122   Step s in CREATE NODES (N1 to N2 with step s) negative
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodnods' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodnods'
         write(irefwr,1) 'iplace', iplace
         call prinin ( inpsep, 3, 'inpsep')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iplace==23 ) then

!     --- place = 23, series of nodes

         nnodes = inpsep(1)
         nodes(1:nnodes) = inpsep(2:nnodes+1)

      else

!     --- place = 4, nodes from node1 to node2 with step istep

         node1 = inpsep(1)
         node2 = inpsep(2)
         istep = inpsep(3)

         if ( node1<=0 .or. node1>npoint ) then
            call errint ( node1, 1 )
            call errint ( npoint, 2 )
            call errsub ( 1120, 2, 0, 0 )
         end if
         if ( node2<=0 .or. node2>npoint ) then
            call errint ( node2, 1 )
            call errint ( npoint, 2 )
            call errsub ( 1121, 2, 0, 0 )
         end if
         if ( istep<0 ) then
            call errint ( istep, 1 )
            call errsub ( 1122, 1, 0, 0 )
         end if
         nnodes = 0
         do i = node1, node2, max(1,istep)
            nnodes = nnodes + 1
            nodes(nnodes) = i
         end do

      end if  ! ( iplace==23 )

1000  call erclos ( 'prfillnodnods' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodnods'

      end if  ! ( debug )

      end
