      subroutine prfindelm2 ( xgauss, igaussp, kmeshclc, iwork, iwork1, &
                              iwork2, isplit, jnpelm, istep, nvertices, &
                              jelem, jelemlow, jelemtop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 12-02-2003 Debug statements
!        version  2.0  date 25-08-2000 Change of parameter list
!        version  1.0  date 25-03-2000
!
!   copyright (c) 2000-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find element number in which point with coordinates (x,y) is lying
!   Two-dimensional case
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     element
!     find
!     find_element
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xgauss(m,2)
      integer          jnpelm, kmeshclc(jnpelm,*), iwork(*), iwork1(*), &
                       iwork2(*), istep, nvertices, isplit, jelem, &
                       jelemlow, jelemtop, igaussp

!     jelem          o    Element number
!     jelemlow       i    Smallest element number in group
!     jelemtop       i    Largest element number in group
!     igaussp        i    Sequence number of gauss point
!     jnpelm         i    number of points per element
!     isplit         i    Sequence number of discontinuous part
!     istep          i    Step in loop
!     iwork               integer work array of length npointlc
!                         In this array all node numbers of a part are stored
!     iwork1              integer work array of length nelem
!                         iwork1(i) = 0, means element i has not yet been used
!                         iwork1(i) = 1, means element i has been used
!                         In this array all element numbers of a part are stored
!     iwork2              integer work array of length nelem
!     kmeshclc       i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     nvertices      i    Number of vertices
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision phi(3), x(4), y(4), xmin, xmax, ymin, ymax, eps, &
                       distx, disty, xcoord, ycoord, xsi0, eta0, &
                       xlocmax, xlocmin, ylocmax, ylocmin
      integer          i, j, k, nodp, nodp1, nx, ny, indi, indj, indk, &
                       nnodes, istartpart2, istart, npointlc, &
                       indilow, inditop, indjlow, indjtop, l, icell, &
                       jcell, index1(4)
      logical          found

!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     eps            Local accuracy
!     eta0           Value of eta in previous iteration
!     found          If true already a possibility has been found
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     index1         Array of length jnpelm containing the point numbers
!                    of the nodal points in the element
!     indi           i-index
!     indilow        lower i-index
!     inditop        upper i-index
!     indj           j-index
!     indjlow        lower j-index
!     indjtop        upper j-index
!     indk           k-index
!     istart         First position used at a part
!     istartpart2    First position used at part 2
!     j              Counting variable
!     jcell          Cell number of cell under consideration
!     k              Counting variable
!     l              General loop variable
!     nnodes         Number of nodes
!     nodp           Nodal point sequence number
!     nodp1          Node sequence number
!     npointlc       Local number of points
!     nx             number of points in x direction
!     ny             number of points in y direction
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    array phi is only filled when gauss integration is
!                    applied
!     x              x co-ordinate of point
!     xcoord         x-coordinate of point
!     xlocmax        local maximum x coordinate
!     xlocmin        local minimum x coordinate
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xsi0           Value of xsi in previous iteration
!     y              y co-ordinate of point
!     ycoord         y-coordinate of point
!     ylocmax        local maximum y coordinate
!     ylocmin        local minimum y coordinate
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP620INTPOL   Compute the values of the basis functions in a given
!                    point for a linear triangle
!     ELP621INTPOL   Compute the values of the basis functions in a given
!                    point for a bi-linear quadrilateral
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2150   no element found containg this point
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork1
!    Compute cell-index (i,j) in which point (x,y) is lying
!    finished := false
!    for k := 0 (1) nx, while ( not finished ) do
!       Find all nodes in cell (i+/-k,j+/-k), store in iwork
!       Find all elements in element groups ielgrp1 to ielgrp2 corresponding
!       to these nodes, store in iwork2
!       For all elements in the group, while ( not finished ) do
!         if ( point in element ) then
!            jelemfound := element number
!            finished := true
!
!    To check if a point is inside an element we distinguish between
!    triangles and quadrilaterals
!
!    Triangles:
!
!    The three vertices define three linear basis functions phi_i(x,y)
!    with:
!    phi_i(x,y) = a_0 + a_1 x + a_2 y
!    The computation of these basis functions is compted in elp620intpol
!    The point (xcoord,ycoord) is in the triangle if
!    0 <=phi_i(xcoord,ycoord) <= 1
!
!    Quadrilaterals:
!
!    The four vertices define four bi-linear basis functions phi_i(x,y)
!    The point xsi, eta defines the position of (xcoord,ycoord) in the
!    reference square
!    This is computed in elp621intpol
!    If xsi and eta in the interval (0,1) then (x,y) is in the quadrilateral
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindelm2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindelm2'
         call ertrace
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

!     --- Store coordinates of point in (xcoord,ycoord)

      xcoord = xgauss(igaussp,1)
      ycoord = xgauss(igaussp,2)
      if ( debug ) write(irefwr,2) 'xcoord, ycoord', xcoord, ycoord

!     --- Clear array iwork1

      do i = 1, nelem
         iwork1(i) = 0
      end do

!     --- Compute cell-index (i,j) in which point (x,y) is lying

      xmin = rmeshac(1,isplit)
      xmax = rmeshac(2,isplit)
      ymin = rmeshac(3,isplit)
      ymax = rmeshac(4,isplit)
      if ( debug ) write(irefwr,2) &
         'xmin, xmax, ymin, ymax',xmin, xmax, ymin, ymax
      eps = 1d-6
      nx = kmeshac((isplit-1)*4+2)
      ny = kmeshac((isplit-1)*4+3)
      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny
      indi = min(nx,int((xcoord-xmin)/distx)+1)
      indj = min(ny,int((ycoord-ymin)/disty)+1)
      istart = kmeshac((isplit-1)*4+4)
      if ( debug ) &
         write(irefwr,1) 'nx, ny, indi, indj, istart, isplit', &
                          nx, ny, indi, indj, istart, isplit

      do k = 0, nx-1

!     --- for k := 0 (1) nx-1, while ( not finished ) do
!         Find all nodes in cell (i+/-k,j+/-k), store in iwork

         indilow = max(1,indi-k)
         inditop = min(indi+k,nx)
         indjlow = max(1,indj-k)
         indjtop = min(indj+k,ny)
         nnodes = 0
         do icell = indilow, inditop
            do jcell = indjlow, indjtop
               indk = (jcell-1)*nx+icell
               istartpart2 = kmeshac(istart+indk-1)
               npointlc = kmeshac(istart+indk)-istartpart2
               do i = 1, npointlc
                  iwork(nnodes+i) = kmeshac(istartpart2+i-1)
               end do
               nnodes = nnodes+npointlc
            end do
         end do
         inelem = 0

         do i = 1, nnodes

!        --- Find all elements in element groups ielgrp1 to ielgrp2
!            corresponding to these nodes, store in iwork2

            nodp1 = iwork(i)
            do j = kmshd1(nodp1)+1, kmshd1(nodp1+1)
               jelem = kmshd2(j)
               if ( jelem>=jelemlow .and. jelem<=jelemtop .and. &
                    iwork1(jelem)==0 ) then
                  inelem = inelem+1
                  iwork1(jelem) = 1
                  iwork2(inelem) = jelem
               end if
            end do

!           --- Clear array iwork1

            do j = 1, inelem
               iwork1(iwork2(j)) = 0
            end do

!           --- For all elements in the group, while ( not finished ) do
!               if ( point in element ) then
!                  jelemfound := element number
!                  finished := true

            do j = 1, inelem
               jelem = iwork2(j)
               xlocmin = xmax
               xlocmax = xmin
               ylocmin = ymax
               ylocmax = ymin
               do l = 1, nvertices
                  nodp = kmeshclc((l-1)*istep+1,jelem-jelemlow+1)
                  x(l) = coor(1,nodp)
                  y(l) = coor(2,nodp)
                  xlocmin = min(xlocmin,x(l))
                  xlocmax = max(xlocmax,x(l))
                  ylocmin = min(ylocmin,y(l))
                  ylocmax = max(ylocmax,y(l))
                  index1(l) = nodp
               end do
               if ( xcoord<xlocmin .or. xcoord>xlocmax .or. &
                    ycoord<ylocmin .or. ycoord>ylocmax ) go to 100
               if ( nvertices==3 ) then

!              --- triangles

                  call elp620intpol ( coor, index1, phi, xcoord, &
                                      ycoord )
                  found = phi(1)>-eps .and. phi(1)<=1d0+eps .and. &
                          phi(2)>-eps .and. phi(2)<=1d0+eps .and. &
                          phi(3)>-eps .and. phi(3)<=1d0+eps

               else

!              --- quadrilaterals
!                  Compute factors for bilinear interpolation
!                  x = a0 + a1 xsi + a2 eta + a3 xsi eta

                  call elp621intpol ( coor, index1, phi, xcoord, ycoord, &
                                      xsi0, eta0, .false. )
                  found = xsi0>-eps .and. xsi0<=1d0+eps .and. &
                          eta0>-eps .and. eta0<=1d0+eps

               end if
               if ( found ) go to 1000

100         end do
         end do

      end do
      call errint ( jelemlow, 1 )
      call errint ( jelemtop, 2 )
      call erreal ( xcoord, 1 )
      call erreal ( ycoord, 2 )
      call errsub ( 2150, 2, 2, 0 )

1000  if ( debug ) write(irefwr,*) 'End prfindelm2'
      call erclos ( 'prfindelm2' )

      end
