      subroutine prplotzerolev
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 09-06-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot zero level set curve
!     It is assumed that the plot is already active and hence opened
!     The active color is used
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     level_set
!     plot
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilev, istart, numedges, k, iedge, ipstart, inpelm, node, i

!     i              Counting variable
!     iedge          edge sequence number
!     ilev           Sequence number of level set being considered
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     istart         First position used at a part
!     k              Counting variable
!     node           Node number
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotzerolev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotzerolev'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=2 ) go to 1000  ! only plot in case of ndim = 2
      if ( nlevelsets==0 .or. kelmad==0 ) go to 1000   ! no level_set found

!     --- Activate or create arrays

      if ( kmeshad(9)==0 ) call sepfillobfaces
      if ( nmesh(33)==0 ) call sepfillnmeshc

      ilev = 1  ! for the moment only one level set

      istart = obstfaces(ilev)
      numedges = obstfaces(istart)

      do k = 1, numedges     ! loop over edges

         iedge = obstfaces(istart+k)  ! edge
         ipstart = nmeshc(iedge)
         inpelm = nmeshc(ipstart)     ! number of nodes in edge
         node = nmeshc(ipstart+1)     ! first node of edge

!        --- plot with pen up to first node

         call plafep ( 4, coor(1,node), coor(2,node), 0d0, 0d0, 0 )

!        --- plot with pen down to other nodes

         do i = 2, inpelm

            node = nmeshc(ipstart+i)     ! next node of edge
            call plafep ( 5, coor(1,node), coor(2,node), 0d0, 0d0, 0 )

         end do  ! i = 2, inpelm

      end do  ! k = 1, numedges

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotzerolev'

      end if  ! ( debug )
      call erclos ( 'prplotzerolev' )

      end subroutine prplotzerolev
