      subroutine prputvalarr ( isol, nodepnts, &
                               nnodes, idegfd, uservec, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of a user array into a sepran solution array
!     for a set of nodes and a specific degree of freedom
! **********************************************************************
!
!                       KEYWORDS
!
!     put
!     value
!     variable
!     array
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodepnts(nnodes), idegfd, isol
      double precision uservec(nnodes)
      character (len=*) namear

!     idegfd         i    Degree of freedom sequence number
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!                         The values are copied in the corresponding array
!     namear         i    Name of array
!     nnodes         i    Number of nodes in array nodes
!     nodepnts       i    Contains the node numbers that must be filled
!     uservec        i    Contains the values that must be copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLARRCOP   Copy the contents of array uold into array unew for
!                    all nodes stored in array nodepnts and the degrees of
!                    freedom indicated in kdeg
!                    uold is a user array, unew a sepran array
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOL110  Activate or create arrays of type 110
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prputvalarr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prputvalarr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create array isol and corresponding arrays

      call sepcreasol110 ( isol, namear, 0 )

      if ( ierror/=0 ) go to 1000

      call erfillarrcop ( nodepnts, idegfd, uservec, usoltemp, &
                          indprfi, kprobfi, nphysi, nnodes )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prputvalarr'

      end if  ! ( debug )
      call erclos ( 'prputvalarr' )

      end subroutine prputvalarr
