      subroutine prremnegnodes ( nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove all non-positive node numbers from array nodes
!     Adapt nnodes
! **********************************************************************
!
!                       KEYWORDS
!
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: nnodes, nodes(nnodes)

!     nnodes        i/o   Number of nodes in nodes
!     nodes         i/o   Contains the node numbers
!                         At input all nodes
!                         At ouput only nodes with positive number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numpoints, i

!     i              Counting variable
!     numpoints      Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prremnegnodes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prremnegnodes'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numpoints = 0
      do i = 1, nnodes
         if ( nodes(i)>0 ) then
            numpoints = numpoints+1
            nodes(numpoints) = nodes(i)
         end if  ! ( nodes(i)>0 )
      end do  ! i = 1, nnodes

      nnodes = numpoints

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prremnegnodes'

      end if  ! ( debug )
      call erclos ( 'prremnegnodes' )

      end subroutine prremnegnodes
