      subroutine readb5 ( numarr, iarray )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.3  date 13-12-2017 New storage of intmat
!        version  5.2  date 04-06-2014 Allow -102 in intmat(2)
!        version  5.1  date 15-11-2012 Long integers
!
!   copyright (c) 1984-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   readb5 reads array of intmat type from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, iarray

!    iarray    o   Standard SEPRAN array containing information of the structure
!                  of the matrix
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer index, ihelp(2), k, istart
      character *8 text
      integer, allocatable :: iwork(:)

!     ihelp          Help array to store the length of the two parts of intmat
!     index          Record number with respect to file 2 from which array
!                    must be read
!     istart         Starting address in iwork
!     iwork          integer work array to store records read
!     k              Counting variable
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READFL         Read integer array from a direct access SEPRAN file
!     SEPCRINTMT     create a new array in intmt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb5' )
      write(irefwr,*) 'readb5 is not longer available'
      call ertrace
      call instop
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )
      allocate ( iwork(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      call readfl ( index, 1, length, 1, 2, iwork )
      if ( ierror/=0 ) go to 1000
      istart = 5
      intmat = iarray
      call sepcrintmt ( intmat )
  !    intmt(intmat)%intmatinf = iwork(8:7+iwork(7))

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'readb5' )
      end subroutine readb5
