      subroutine readbs ( numarr, iarray, ihelp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 14-04-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     reads array from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, iarray(*), ihelp(*)

!    iarray    o   Integer array containing information of the array that has
!                  been read
!    ihelp     i   Second integer user array, depending on the first one
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(2)

!     iwork       Work array that is used to store the first two positions
!                 of the record to be read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READB2         Reads arrays of KMESH type
!     READB3         Reads arrays of KPROB type
!     READB4         Reads arrays of type solution vector
!     READB5         Reads arrays of INTMAT type
!     READB6         Reads arrays of MATR type
!     READB7         Reads arrays of user type
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      82   array is not a sepran array
!     384   numarr has incorrect value
!    1242   Array IHELP is not of the type KPROB
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readbs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readbs'
         write(irefwr,1) 'numarr, ifree2', numarr, ifree2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( numarr<=0 .or. numarr>=ifree2 ) then

!     --- Error 384: NUMARR has incorrect value

         call errint ( numarr, 1 )
         call errint ( ifree2-1, 2 )
         call errsub ( 384, 2, 0, 0 )
         go to 1000

      end if

!     --- Read first two positions from record

      call readfl ( iwork3(2*numarr-1), 1, 2, 1, 2, iwork )
      if (debug ) call prinin ( iwork, 2, 'iwork' )

!     --- jump to right position

      if ( iwork(2)==100 ) then

!     --- iwork(2) = 100 , array is of type kmesh

         call readb2 ( numarr, iarray )

      else if ( iwork(2)==101 ) then

!     --- iwork(2) = 101 , array is of type kprob

         call readb3 ( numarr )

      else if ( abs(iwork(2))==110 .or. iwork(2)==115 .or. &
                iwork(2)==116 .or. iwork(2)==120 ) then

!     --- iwork(2) = 110, 115, 116 or 120   array is of type solution vector

         call readb4 ( numarr, iarray )

      else if (iwork(2)==102 ) then

!     --- iwork(2) = 102 , array is of type intmat

         call readb5 ( numarr, iarray )

      else if ( iwork(2)==103 ) then

!     --- iwork(2) = 103 , array is of type matr

!         call readb6 ( numarr, iarray )
         call ertrace
         call instop

      else if ( iwork(2)==200 .or. iwork(2)==300 ) then

!     --- iwork(2) = 200, 300 , array is user-array

         call readb7 ( numarr, iarray )

      else

!     --- Array is of unknown type

         call errsub ( 82, 0, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readbs'

      end if  ! ( debug )
      call erclos ( 'readbs' )

      end subroutine readbs
