      subroutine readc301 ( numrel, inpelm, relnods, rnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-10-1997
!
!   copyright (c) 1997-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check the numrel numbers stored in array relnods
!     Fill the relative nodes in array rnodes
!     Checking is with respect to uniqueness and whether all elements are
!     in the range 1 to inpelm
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numrel, inpelm
      double precision relnods(*), rnodes(*)

!     inpelm     i    number of points per element
!     numrel     i    Number of relative points that have been read
!     relnods    i    Array of length numrel containing the relative node
!                     numbers that have been read
!     rnodes     o    Array of length inpelm containing an indication
!                     whether a node is present in relnods (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, nodp, jnodp

!     jnodp      Nodal point sequence number
!     k          Counting variable
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCRD  Prints the last record read and underlines the errorneous part
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     268   Number of parameters larger than inpelm
!     269   Relative nodal point number in surface record incorrect
!     280   Double point found in surface record
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check numrel

      if ( numrel.gt.inpelm ) then
         call eropen ( 'readc301' )
         call errint ( numrel, 1 )
         call errint ( inpelm, 2 )
         call errsub ( 268, 2, 0, 0 )
         call errcrd
         call erclos ( 'readc301' )
      end if

      if ( numrel.gt.0 ) then

!     --- numrel > 0,  copy contents of resul numrel+1 positions forwards
!         clear rnodes

         do k = 1, inpelm
            rnodes(k) = 0
         end do

!        --- fill rest of resul

         do k = 1, numrel
            nodp  = nint(relnods(k))
            if ( nodp.le.0 .or. nodp.gt.inpelm ) then
               call eropen ( 'readc301' )
               call errint ( nodp, 1 )
               call errint ( inpelm, 2 )
               call errsub ( 269, 2, 0, 0 )
               call erclos ( 'readc301' )
            end if
            jnodp  = nint(rnodes(nodp))
            if ( jnodp.gt.0 ) then

!           --- jnodp  > 0  double point

               call eropen ( 'readc301' )
               call errint ( k, 1 )
               call errint ( nodp, 2 )
               call errsub ( 280, 2, 0 , 0)
               call errcrd
               call erclos ( 'readc301' )

            else

!           --- jnodp   =  0

               rnodes(nodp) = 1d0

            end if

         end do

      end if

      end
