      subroutine readcointvar ( iseq, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part variables of input block CONSTANTS
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iwork(*), intscals(maxintscals)

!     iseq           i    Sequence numberof vector in VECOLD/ISLOLD
!     iwork         i/o   integer work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, i, iseq1, numbscal

!     i              Counting variable
!     inum           Number of numbers read in a record
!     iseq1          Shifted sequence number
!     numbscal       Number of scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READARRINDEX   Check if the name of a constant vector or varibale is
!                    followed by an index
!     READSRINNOTEXT Read a series of integer numbers from the input
!                    No text between numbers allowed
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1612   Too much variables read
!    2143   Sequence number already in use
!    3100   Name of constant, variable or vector too long
!    3101   Index too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readcointvar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcointvar'
         write(irefwr,1) 'iseq, numintscals', iseq, numintscals
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseq==0 ) iseq = numintscals+1
      iseq1 = iseq+maxints+maxreals+maxintscals+maxvc1+maxstrings
      if ( iwork(iseq1)/=0 ) then

!     --- error, sequence number already in use

         call errint ( iseq, 1 )
         call errchr ( 'variables' , 1 )
         call errsub ( 2143, 1, 0, 1 )

      end if
      iwork(iseq1) = 1
      numintscals = max(numintscals,iseq)

      if ( debug ) &
         write(irefwr,1) 'iseq1, numintscals, maxintscals', &
                          iseq1, numintscals, maxintscals

      if ( numintscals>maxintscals ) then

!     --- Too much variables

         call errint ( numintscals, 1 )
         call errint ( maxintscals, 2 )
         call errsub ( 1612, 2, 0, 0 )
         call errcrd

      else

!     --- store name into chscalars

         chscalars(maxcls+iseq) = string(1:nchar)

!        --- Check if the scalar is indexed

         numbscal = readarrindex()

         dimscals(maxcls+iseq) =  numbscal ! dimension of scalar
         ncharscals(maxcls+iseq) = nchar   ! number of characters of scalstant

         if ( debug ) &
            write(irefwr,1) 'iseq, numbscal, nchar', &
                             iseq, numbscal, nchar

         if ( numbscal>1 ) then

!        --- numbscal>1, hence indexed scalar

            if ( nchar>29 ) then

!           --- Too many characters

               call errchr ( string, 1 )
               call errsub ( 3100, 0, 0, 1 )
               go to 1000

            end if  ! ( nchar>29 )
            do i = 1, min(numbscal,9)
               write ( chscalars(maxcls+iseq+i-1), 100 ) &
                   string(1:nchar), i
100            format ( a,'(',i1,')' )
            end do  ! i = 1, min(numbscal,9)

            if ( numbscal>9 ) then

               if ( nchar>28 ) then

!              --- Too many characters

                  call errchr ( string, 1 )
                  call errsub ( 3100, 0, 0, 1 )
                  go to 1000

               end if  ! ( nchar>29 )
               do i = 10, min(numbscal,99)
                  write ( chscalars(maxcls+iseq+i-1), 110 ) &
                   string(1:nchar), i
110               format ( a, '(',i2,')' )
               end do  ! i = 10, min(numbscal,99)

            end if  ! ( numbscal>9 )

            if ( numbscal>=100 ) then

!           --- The upper bound for index is 99 at this moment

               call errint ( numbscal, 1 )
               call errint ( 99, 2 )
               call errsub ( 3101, 2, 0, 0 )
               go to 1000

            end if  ! ( numbscal>100 )

         end if  ! ( numbscal>1 )

!        --- read numbers

         call readsrinnotext ( intscals(iseq), maxintscals-iseq+1, inum, &
                               .false., 2000000000, string(1:32) )
         numbscal = max(numbscal, inum)
         if ( inum>0 ) iscalars(maxcls+iseq) = 1
         if ( debug ) &
            call prinin ( intscals(iseq), numbscal, 'intscals' )
         numintscals = max(numintscals,iseq+numbscal-1)
         scalars(maxcls+iseq:maxcls+iseq+numbscal-1) = &
            intscals(iseq:iseq+numbscal-1)

         if ( debug ) &
            write(irefwr,1) 'inum, numbscal, numintscals', &
                             inum, numbscal, numintscals
         do i = 1, inum-1
            if ( iwork(iseq1+i)/=0 ) then

!           --- error, sequence number already in use

               call errint ( iseq+i, 1 )
               call errchr ( 'variables' , 1 )
               call errsub ( 2143, 1, 0, 1 )

            end if
            iwork(iseq1+i) = 1
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readcointvar'

      end if  ! ( debug )
      call erclos ( 'readcointvar' )

      end
