      subroutine readi4 ( iref, defchr, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 28-12-1992 Error correction reading more
!                                      strings)
!        version  2.0  date 17-10-1992 Complete new subroutine
!        version  1.0  date 01-01-1982
!
!   copyright (c) 1982-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Reads character defaults from the file with reference number iref
!   It is assumed that subroutine errchr has been called before with the
!   name of environment file as first character variable
! **********************************************************************
!
!                       KEYWORDS
!
!     default
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref, n
      character *(*) defchr(n)

!     defchr  o   Contains the defaults that have been reed from the file
!     iref    i   File reference number from which the defaults must be read
!     n       i   Number of defaults to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, irefsv, kcolsv
      logical ready

!    inum     Number of characters read
!    irefsv   Help variable to save irefre temporarily
!    kcolsv   Help parameter to save the old value of kcolum
!    ready    Indicates if a while loop must be terminated (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item in input file
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
!     ERTEXT  Write message from error message file to screen
! **********************************************************************
!
!                       I/O
!
!    Input is read from the environment file
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   1822:  environment file incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Save reference number for reading from screen
!   Set reading reference number for reading from environment file
!   Read strings from environment file
!   If ( not enough strings are available ) then
!      error: incorrect environment file
!   Reset reference number for reading from screen
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readi4' )

!     --- Make sure that end of record is skipped

      irefsv = irefre
      irefre = iref

      istop = 2
      ready  = .false.
      inum   = 0
      ispecl = 3
      kcolsv = kcolum

200   if ( .not. ready ) then

!     --- While loop
!           Read next item

         call asknxt

         if ( ityprd.eq.5 ) then

!        --- ityprd = 4, string found

            inum = inum + 1
            defchr(inum) = string(1:nchar)
            ityprd = 1

         else if ( ityprd.le.1 .or. ityprd.eq.7 ) then

!        --- ityprd = 0, 1 or 7, i.e. end of file, end of record or
!            ) sign found

            ready = .true.

         end if

         if ( inum.ge.n ) then

!        --- inum = n, stop reading

            ready = .true.

         end if

         go to 200

      end if
      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

      istop = 0

      if ( inum.lt.n ) then

!     --- Warning environment file is incorrect

         call errsub ( 1822, 0, 0, 1 )
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000

      end if

!     --- reset istop

1000  irefre = irefsv

      call erclos ( 'readi4' )
      end
