      subroutine readqt
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 02-04-2008 Fortran 90
!        version  2.0  date 01-01-2003 Complete revision
!        version  1.3  date 28-12-2000 Length of records extended to 240
!        version  1.2  date 10-02-1994 New comments
!        version  1.1  date 25-02-1993 Correction and quote
!        version  1.0  date 22-12-1990
!
!   copyright (c) 1990-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine READQT is used to read a text from the input file
!       This subroutine supposes that the parameter ICOLUM indicates the
!       first position of the text. This first position is a quote
!        a text is a number of symbols between quotes
!       After reading the text, icolum points to the next position to be read
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished
      integer iitem

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     iitem          Local sequence number of item
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1333   No end text quote found
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ready  := false
!   error  := false
!   nchar  := 0
!   kcolum := icolum
!   icolum := icolum+1
!   while ( not ready ) do
!      nchar := nchar+1
!      string(nchar) := cards(icolum)
!      icolum := icolum+1
!      if ( icolum>maxcolum+1 ) then
!         ready := true
!         error := true
!      else if ( string(nchar) # ' ) then
!         ready := true
!   nchar = nchar-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readqt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readqt'
         write(irefwr,1) 'lastitem, nitems', lastitem, nitems
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get first column and last column and store text in cardim

      firstcol = firstcolitems(lastitem)+1
      numrecord = firstcol/240+1
      firstcol = mod(firstcol,240)
      if ( debug ) write(irefwr,1) 'firstcol, numrecord', firstcol, numrecord
      if ( lastitem==nitems ) then

!     --- No item found after the quote

         lastcol = firstcol
         call errchr ( ' ', 1 )
         call errsub ( 1333, 0, 0, 1 )
         call errcrd
         go to 1000

      end if
      iitem = lastitem+1

      finished  = .false.

      do while ( .not. finished .and. iitem<=nitems )

!     --- Interpret next item on the same record

         icolum = firstcolitems(iitem)

         if ( debug ) &
            write(irefwr,1) 'iitem, icolum, typeitems(iitem)', &
                             iitem, icolum, typeitems(iitem)
         if ( icolum>240*numrecord ) then

!        --- Next item starts at new record and no end quote found

            lastcol = lastcolumn(numrecord)
            call errchr ( cardimage(numrecord)(firstcol:lastcol), 1 )
            call errsub ( 1333, 0, 0, 1 )
            call errcrd
            go to 1000

         else if ( typeitems(iitem)==4 ) then

!        --- end quote found

            lastitem = iitem
            lastcol = mod(lastcolitems(lastitem),240)-1
            nchar = lastcol-firstcol+1
            string(1:nchar) = cardimage(numrecord)(firstcol:lastcol)
            finished = .true.

         else

!        --- Not yet end quote found, raise iitem

            iitem = iitem+1

         end if

      end do  ! while ( .not. finished )
      if ( .not. finished ) then

!     --- no end quote found

         call errchr ( cardimage(numrecord)(firstcol:lastcol), 1 )
         call errsub ( 1333, 0, 0, 1 )
         call errcrd

      end if

!     --- End of while loop

1000  ityprd = 5
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readqt'

      end if  ! ( debug )
      call erclos ( 'readqt' )

      end
