      subroutine readrealvar ( rinput, iinput, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-09-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the variable read in rinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rinput(*)
      integer iinput, istrin

!     iinput         o    Entry of integer input array referring to variable
!                         If value stored in table(icode) iinput contains the
!                         entry in rinput where the value is stored
!                         If the variable is a scalar minus the entry
!                         in scalars is stored in iinput
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINPUT
!                         If a value is stored in rinput, istrin is raised
!     rinput        i/o   Real input array
!                         If a real is read the value is stored in
!                         rinput(istrin)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number in set of scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readrealvar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readrealvar'
         write(irefwr,1) 'icode, infotable(icode)', &
                          icode, infotable(icode)
         write(irefwr,2) 'table(icode), tabmin(icode)', &
                          table(icode), tabmin(icode)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( infotable(icode)==11 ) then

!     --- variable found

         iseq = nint(table(icode))-100000000
         iinput = -iseq

      else if ( table(icode)>=tabmin(icode) ) then

!     --- value is given

         rinput(istrin) = table(icode)
         iinput = istrin
         istrin = istrin+1

      else

!     --- value is given

         iinput = 0

      end if  ! ( infotable(icode)==11 )

1000  call erclos ( 'readrealvar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readrealvar'

      end if  ! ( debug )

      end
