      integer function sepchkscalvec()
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if item at right-hand side is scalar or vector
!     if so add left-hand side to the list of scalars or vectors
!     Old subroutine sepreadchkrhsvec
! **********************************************************************
!
!                       KEYWORDS
!
!     vector_name
!     name
!     add
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer scode, vcode, i, ncharsave, lastitemsav, stringcode
      logical ::  vectorfound, scalarfound, notfound, stringfound
      character (len=240) stringsave, stringimsave

!     i              Counting variable
!     lastitemsav    Help parameter to save the value of lastitem
!     ncharsave      Help parameter to save nchar at input
!     notfound       If true no expression ate right-hand side is found
!     scalarfound    If true the keyword scalar is in code
!     scode          sequence number of the keyword scalar in code
!     stringcode     sequence number of the string
!     stringfound    If true the keyword is in the list of strings
!     stringimsave   Help parameter to save string at input
!     stringsave     Help parameter to save string in capitals at input
!     vcode          sequence number of the keyword vector in code
!     vectorfound    If true the keyword vector is in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDSCAL        Add scalar name to the list of scalars
!     ADDSTRING      Add string name to the list of strings
!     ADDVEC         Add vector name to the list of vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCHKSTRING   Check if right-hand side corresponds to a string
!     SEPREADCHKRHS  Check if right-hand side is an expression containing
!                    only existing scalars and vectors, operators and numbers
!                    and no other strings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepchkscalvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepchkscalvec'
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = 0
      vcode = 0
      scode = 0
      stringcode = 0
      stringsave = string(1:nchar)
      stringimsave = stringim(1:nchar)
      ncharsave = nchar
      lastitemsav = lastitem
      do i = 1, nncode

         vectorfound = ncode(i)==1 .and. code(i)=='V' .or. &
                       ncodmx(i)==6 .and. code(i)=='VECTOR'
         vcode = i
         if ( vectorfound ) exit

      end do  ! i = 1, nncode

      do i = 1, nncode

         scalarfound = ncode(i)==1 .and. code(i)=='S' .or. &
                       ncodmx(i)==6 .and. code(i)=='SCALAR'
         scode = i
         if ( scalarfound ) exit

      end do  ! i = 1, nncode

      do i = 1, nncode

         stringfound = ncodmx(i)==6 .and. code(i)=='STRING'
         stringcode = i
         if ( stringfound ) exit

      end do  ! i = 1, nncode

      if ( debug ) &
         write(irefwr,1) 'scode, vcode, stringcode', &
                          scode, vcode, stringcode

      if ( stringcode>0 ) then

!     --- stringcode>0, possibly string = &
!         check right-hand side

         call sepchkstring ( notfound )

         if ( .not. notfound ) then

!        --- string found, add to list of strings

            nchar = ncharsave
            string(1:nchar) = stringsave
            stringim(1:nchar) = stringimsave(1:nchar)
            call addstring
            icode = stringcode
            if ( ierror/=0 ) go to 1000

         end if  ! ( .not. notfound )

      end if  ! ( stringcode>0 )

      if ( notfound .and. (scode>0 .or. vcode>0)  ) then

!     --- scode>0, possibly scalar =
!         check right-hand side

         call sepreadchkrhs ( notfound, vectorfound )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            write(irefwr,*) 'after sepreadchkrhs notfound, vectorfound', &
                             notfound, vectorfound
            write(irefwr,1) 'vcode', vcode
         end if  ! ( debug )

         nchar = ncharsave
         string(1:nchar) = stringsave(1:nchar)
         stringim(1:nchar) = stringimsave(1:nchar)

         if ( notfound ) then

!        --- No expression found

         else if ( vcode>0 .and. vectorfound ) then

!        --- Vector found, add vector to set of vectors

            if ( debug ) write(irefwr,*) 'vector found, addvec'
            icode = vcode   ! Hence v = &
            ityprd = 102
            call addvec
            if ( ierror/=0 ) go to 1000

         else if ( .not. vectorfound ) then

!        --- Scalar found, add scalar to set of scalars

            icode = scode   ! Hence s = &
            ityprd = 11
            call addscal
            if ( ierror/=0 ) go to 1000

         end if  ! ( .notfound )

      end if  ! ( scode>0 )

      nchar = ncharsave
      string(1:nchar) = stringsave(1:nchar)
      stringim(1:nchar) = stringimsave(1:nchar)
      lastitem = lastitemsav  ! reset lastitem

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,*) 'End sepchkscalvec'

      end if  ! ( debug )
      call erclos ( 'sepchkscalvec' )

      sepchkscalvec = icode

      end function sepchkscalvec
