      subroutine sepcreavec ( iold, inew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-05-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new sepran vector inew with the same properties as
!     The old vector iold
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iold
      integer, intent(inout) :: inew

!     inew      i/o   new array (gets the same structure as iold)
!     iold       i    old array (must be a SEPRAN array)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreavec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreavec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = ks(iold)%iseqkp
      length = size(ks(iold)%sol)
      call sepcrsolinfo ( inew, length, 0 )
      ks(inew)%typevec = ks(iold)%typevec
      ks(inew)%compl = ks(iold)%compl
      ks(inew)%nusol = ks(iold)%nusol
      ks(inew)%ivec = ks(iold)%ivec
      ks(inew)%n = ks(iold)%n
      ks(inew)%jcart = ks(iold)%jcart
      ks(inew)%nphys = ks(iold)%nphys
      ks(inew)%iprob = ks(iold)%iprob
      ks(inew)%iseqkp = ks(iold)%iseqkp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcreavec'

      end if  ! ( debug )
      call erclos ( 'sepcreavec' )

      end subroutine sepcreavec
