      subroutine sepcrmatpart1 ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-04-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new array in realmt without allocation of space
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: matr

!     matr         i/o   Defines matrix information by integer sequence number
!                        If it already exists the old info is removed
!                        New matr info is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM
      type (rmatrix), pointer :: Ar
      type (cmatrix), pointer :: Ac

!     Ac             Refers to realmt(matr) (complex matrix)
!     Ar             Refers to realmt(matr) (real matrix)
!     IM             Refers to intmt(intmat)
!     S              pointer to realmt(iseqmt)
!     i              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3223   all arrays in use
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrmatpart1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrmatpart1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( matr>0 .and. matr<=lastmatrix ) call seprmmatrix ( matr )

      if ( nextfreemat<=nummat ) then

!     --- nextfreemat in proper range

         matr = nextfreemat
         do i = matr+1, nummat
            if ( matarr(i)==0 ) then
               nextfreemat = i
               exit
            end if  ! ( matarr(i)==0 )
         end do  ! i = matr+1, nummat
         lastmatrix = max(lastmatrix, matr)

      else

!     --- no free entries left

         call errchr ( 'matrix', 1 )
         call errsub ( 3223, 0, 0, 1 )
         go to 1000

      end if  ! ( nextfreemat<=nummat )

      IM => intmt(intmat)
      jmethod = IM%jmethod
      iseqmt = matr

      S => realmt(matr)

      S%icounter = 0
      S%intmatref = intmat
      S%diagonal = .false.
      S%jmethod = jmethod
      S%complexmat = IM%complexm
      S%iprob = iprob
      S%decomp = .true.

      matarr(matr) = 1
      lastmatrix = max(lastmatrix, matr)

      if ( S%complexmat ) then

!     --- Complex matrix, initialize Ac

         Ac => S%Sc_i
         Ac%iseqmt = matr
         Ac%iseqim = intmat
         Ac%type_storage = jmethod
         Ac%symmetric = IM%symmetric

      else

!     --- Real matrix, initialize Ar

         Ar => S%S_i
         Ar%iseqmt = matr
         Ar%iseqim = intmat
         Ar%type_storage = jmethod
         Ar%symmetric = IM%symmetric

      end if  ! ( S%complexmat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcrmatpart1'

      end if  ! ( debug )
      call erclos ( 'sepcrmatpart1' )

      end subroutine sepcrmatpart1
