      subroutine sepedg06 ( lpnmpl )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.0  date 29-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates nmesh part l
!     Old subroutine topedg06
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lpnmpl

!     lpnmpl        i/o   length of array nmeshl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, iedge, indped, jnode, ipl, ipc, nelinode, &
              nedpnd, i, j, ipd, ielem, ipkmd2, medpnd
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     indped         Nodes per edge
!     inode          Node sequence number
!     ipc            Starting address in array
!     ipd            pointer in nmeshd
!     ipkmd2         Starting address of KMESH part d2 in array IBUFFR
!     ipl            pointer in nmeshl
!     iwork          integer work array of length nedges
!                    iwork(i)=0 means edge i ahs not yet been treated
!     j              Counting variable
!     jnode          node number on edge
!     medpnd         maximum number of edges connected to a node
!     nedpnd         number of edges connected to a node
!     nelinode       number of elements coupled to node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepedg06' )
      medpnd = 0

      allocate ( iwork(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      ipl = npoint + 1

!     --- Loop over the points

      do inode = 1, npoint

         nmeshl(inode) = ipl
         nedpnd = 0

!        --- Loop over the elements connected to this node

         ipkmd2 = kmshd1(inode)
         nelinode = kmshd1(inode+1)-ipkmd2

         do i = 1, nelinode

            ielem = kmshd2(ipkmd2+i)

!           --- Loop over the edges that define this element

            ipd = nmeshd(ielem)

            do j = 1, nmeshd(ipd)

               iedge = abs(nmeshd(ipd+j))
               if ( iwork(iedge).eq.0 ) then

                  ipc = nmeshc(iedge)
                  nndped = nmeshc(ipc)

!                 --- Loop over the nodes on this edge

                  do indped = 1, nndped

                     jnode = nmeshc(ipc+indped)
                     if ( jnode.eq.inode ) then

!                    --- Edge found : store in nmeshl

                        nedpnd = nedpnd + 1
                        nmeshl(ipl+nedpnd) = iedge
                        iwork(iedge) = 1

                     end if

                  end do

               end if

            end do

         end do

         nmeshl(ipl) = nedpnd

!        --- clear iwork

         do i = 1, nedpnd
            iwork(nmeshl(ipl+i)) = 0
         end do

         if (nedpnd.gt.medpnd) medpnd = nedpnd

         ipl = ipl + nedpnd + 1
         if ( ipl.gt.lpnmpl ) then

!        --- ipl too large

            ipl = ipl + 1000
            go to 1000

         end if

      end do

1000  lpnmpl = ipl -1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      nmesh(20) = medpnd
      call erclos ( 'sepedg06' )

      end subroutine sepedg06

