      subroutine sepfillkmeshd1 ( markelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 12-06-2015 New parameter list
!        version  2.5  date 28-08-2013 Use subroutines
!        version  2.4  date 18-08-2008 Fortran 90
!
!   copyright (c) 1983-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part d1 with pointers to kmesh part d2
!     kmesh part d contains the elements corresponding to each node
!     It concerns the special case of set of elements for one cpu
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     parallel
!     mesh
!     element
!     node
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: markelem(*)

!     markelem       i    Work array to mark elements that must be used
!                         0: element is skipped
!                         1: element is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, ielem

!     ielem          element number
!     ielgrp         element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIACCUMULARR  Accumulate contents of array iarray
!     PRININ         print 1d integer array
!     SEPFILLKMSHD1  Fill part of kmshd1 for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshd1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshd1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- clear kmesh part d1

      kmshd1(1:npoint) = 0

!     --- Store number of elements corresponding to nodes in kmesh d part 1

      ielem = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp


!        --- Loop over elements in element group

         call sepfillkmshd1 ( ielem, ielgrp, markelem )

      end do
      if ( debug ) call prinin ( kmshd1, npoint, 'kmeshd/1' )

!     --- Fill kmesh d part 1 by addition

      call iniaccumularr ( kmshd1, npoint )

1000  if ( debug ) then

!     --- Debug information

         if ( debug ) call prinin ( kmshd1, npoint+1, 'kmeshd/2' )
         write(irefwr,*) 'End sepfillkmeshd1'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshd1' )

      end subroutine sepfillkmeshd1
