      subroutine sepfillkmeshd2 ( markelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 12-06-2015 New parameter list
!        version  2.2  date 07-02-2014 Change format
!        version  2.1  date 28-08-2013 Use subroutines
!
!   copyright (c) 1983-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill second part of kmesh part d (d2)
!     kmesh part d contains the elements corresponding to each node
!     It concerns the special case of set of elements for one cpu
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     parallel
!     mesh
!     element
!     node
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: markelem(*)

!     markelem       i    Work array to mark elements that must be used
!                         0: element is skipped
!                         1: element is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inpelm, ielgrp, ikelmc, ielem
      integer, allocatable :: iwork(:)

!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iwork          integer work array
!                    iwork(i) contains the next sequence number of an
!                    element to be stored in kmeshd
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PR0012         Print accumulated array
!     SEPFILLKMSHD2  Fill array kmshd2 for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize parameters
!     Fill kmesh part d2 in kmshd2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshd2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshd2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Set iwork to 1

      iwork = 1

      ielem = 0
      ikelmc = 1

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!        --- Loop over elements in element group

         call sepfillkmshd2 ( ielgrp, iwork, ielem, markelem )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write ( irefwr, *) ' elements corresponding to nodal points'
         call pr0012 ( npoint, kmshd1, kmshd2 )
         write(irefwr,*) 'End sepfillkmeshd2'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshd2' )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end subroutine sepfillkmeshd2
