      subroutine sepfillkmesho1 ( lengo, lardim, linear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part o1
!     compute length of kmesh part o
!     only two and three-dimensional elements are permitted
!     Old subroutine toplm3
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbor
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lengo, lardim
      logical linear

!    lardim   o   largest dimension of the elements used
!    lengo    o   length of kmesh part o
!                 when 0 kmesh part o can not be created
!    linear   o   If true all elements are linear, otherwise quadratic
!                 elements have been found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iboun1(8), iboun2(8), ielgrp, nbound

!     iboun1         Number of boundaries of two-dimensional elements
!                    iboun1(i) corresponds to ishape-2
!                    1: linear triangle
!                    2: quadratic triangle
!                    3: bilinear quadrilateral
!                    4: biquadratic quadrilateral
!                    5: quadratic triangle with 7 points
!                    6: triangle with 8 points
!                    7: bilinear quadrilateral with 5 points
!                    8: triangle with 4 points
!     iboun2         Number of boundaries of three-dimensional elements
!                    iboun2(i) corresponds to ishape-10
!                    1: linear tetrahedron
!                    2: quadratic tetrahedron
!                    3: bilinear hexahedron
!                    4: biquadratic hexahedron
!                    5: quadratic tetrahedron with 10 points
!                    6: quadratic tetrahedron with 14 points
!                    7: bilinear hexahedron with 9 points
!                    8: linear tetrahedron with 5 points
!     ielgrp         element group number
!     nbound         Number of boundaries of an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     376   ishape = 0
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!   lardim := maximum dimension of elements
!   length_part_o := 0
!   For ielgrp := 1 (1) nelgrp do
!      ishape := kmesh_part_f(ielgrp)
!      inelem := number of elements in element group
!      nbound := number_of_boundaries of element
!      kmesh_part_o(ielgrp) := nbound
!      length_part_o := length_part_o + nbound*inelem
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iboun1 / 3, 3, 4, 4, 3, 3, 4, 3 /, &
           iboun2 / 4, 4, 6, 6, 4, 4, 6, 4 /
! ======================================================================
!
      call eropen ( 'sepfillkmesho1' )

!     --- Initializations

      linear = .true.

!     --- compute dimension of space

      lardim = 1
      lengo = 0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         if ( ishape>2 ) lardim = max(lardim,2)
         if ( ishape>10 .and. ishape<50 ) lardim = max(lardim,3)
         if ( ishape>18 .and. ishape/=21 .and. ishape.ne.50 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if
      end do
      if ( lardim>=2 ) then

!     --- fill kmesh part o1
!         compute length of kmesh part o

         lengo = nelgrp
         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            if ( lardim==2 ) then

!           --- Maximum dimension is equal to 2

               if ( ishape<3 .or. ishape>10 ) then
                  nbound = 0
               else
                  nbound = iboun1(ishape-2)
               end if
               if ( ishape==4 .or. ishape>=6 .and. ishape<9 ) linear = .false.

            else

!           --- Maximum dimension is equal to 3

               if ( ishape>=11 .and. ishape<=18 ) then

!              --- Standard tetrahedrons and hexahedrons

                  nbound = iboun2(ishape-10)

               else if ( ishape==21 ) then

!              --- Prism

                  nbound = 5

               else

!              --- Special type will be skipped

                  nbound = 0

               end if
               if ( ishape==12 .or. ishape>=14 ) linear = .false.

            end if
            kmsho1(ielgrp) = nbound
            inelem = Topgrp(ielgrp)%inelem
            if ( inelem>0 .and. ishape==0 ) then

!           --- Shape 0

               call errint ( ielgrp, 1 )
               call errwar ( 376, 1, 0, 0 )

            end if
            lengo = lengo+nbound*inelem

         end do  ! ielgrp = 1, nelgrp

      else

!     --- largest dimension is equal to 1

         lengo = nelgrp
         do ielgrp = 1, nelgrp
            ishape = kmeshf(ielgrp)
            if ( ishape==0 ) then
               nbound = 0
            else
               nbound = 2
            end if
            kmsho1(ielgrp) = nbound
            inelem = Topgrp(ielgrp)%inelem
            lengo = lengo+nbound*inelem
         end do  ! ielgrp = 1, nelgrp

      end if

      if ( lengo==nelgrp ) lengo = 0

1000  call erclos ( 'sepfillkmesho1' )

      end subroutine sepfillkmesho1
