      subroutine sepfillkprob
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 14-12-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters in module sepmodulekprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nelgrpkprob, i, ipos
      logical :: newcall

!     i              Counting variable
!     ipos           Help parameter in parinfo
!     nelgrpkprob    Number of element groups
!     newcall        If true it concerns a new situation, i.e. the array
!                    from which the module is filled has changed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPFILLKPROBPC Fill parameters wrt kprobpar in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkprob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkprob'
         write(irefwr,1) 'iprob, icountkprob',  iprob, icountkprob
         write(irefwr,1) 'kprobloc(80)',  kprobloc(80)
         call ertrace
    !     call prinin ( kprobloc, 80, 'kprob' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      indpry = kprobloc(51)
      numnat = kprobloc(12)
      ipstkprobpar = kprobloc(65)  ! starting address

      nunkp = kprobloc(4)
      nusol = kprobloc(5)
      nusoli = nusol
      ndtvec = kprobloc(6)
      ntype = kprobloc(7)
      nbound = kprobloc(10)
      nboundi = nbound
      indprx = kprobloc(11)
      npltra = kprobloc(13)
      indprf = kprobloc(19)
      indprh = kprobloc(50)
      nphys  = kprobloc(33)
      nphysi = nphys
      nunkpi = nunkp
      npboun = kprobloc(34)
      npbouni = npboun
      numpairs = kprobloc(62)
      nrusolloc = nusol-nbound

      indpra = kprobloc(14)
      nbcons = kprobloc(32)
      kprelgrp = kprobloc(39)
      maxtrn = kprobloc(45)
      maxcontc = kprobloc(48)
      nkbndskip = kprobloc(64)

      newstyle = kprobloc(69)>0
      indprfi = indprf
      indprhi = indprh

!     --- Dummies

      length = kprobloc(3)
      kprobf => kprobloc
      kprobfi => kprobloc
      kprobfo => kprobloc
      kprobhinvi => kprobhinv
      kprobhinvo => kprobhinv

      if ( indprx>0 ) then
         kprobx => kprobpart%kprobx
         nglgrp = kprobx(1)
         nglunk = kprobx(2)
         nglcoupl = kprobx(3)
      else
         nglunk = 0
         nglgrp = 0
         nglcoupl = 0
      end if
      nelgrpkprob = nelgrp+numnat+nglgrp

      nrusol = kprobloc(66)
      ninterface = kprobloc(67)

      if ( indpry>0 ) then
         nfictgrp = kproby(1)
         nfictunk = kproby(2)
      end if  ! ( indpry>0 )

      iinputkp => kp(iseqkp)%iinputkp
      rinputkp => kp(iseqkp)%rinputkp

      if ( indprf>0 ) kprobf => kprobpart%kprobf
      if ( indprh>0 ) kprobh => kprobpart%kprobh
      kprobr => kprobh
      indprr = kprobloc(37)
      if ( indprr>0 ) kprobr => kprobpart%kprobr

      kprobfi => kprobf
      kprobhi => kprobh
      kprobhinvi => kprobhinv
      kprobpi => kprobp

      if ( nocallprobdf ) call sepfillkprobpc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iprob', iprob
!         call prinin ( kprobloc, 40, 'kprob' )
         write(irefwr,*) 'End sepfillkprob'

      end if  ! ( debug )
      call erclos ( 'sepfillkprob' )

      end subroutine sepfillkprob
