      subroutine sepfillnmeshe
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates nmesh part e, for each element the nodes
!     Old subroutine iniactnmeshe
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lnnmeshe, ielem, ikelmc, inpelm, ipointer, irelem, ielgrp

!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     ipointer       pointer in array
!     irelem         Relative element sequence number with respect to
!                    element group
!     lnnmeshe       length of array nmeshe
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate kmesh part c (topology)
!     Create space for array nmeshe and make this array permanent
!     Compute starting addresses
!     Actually fill nmeshe
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshe' )

!     --- Create space for array nmeshe and make this array permanent

      lnnmeshe = size(kmeshc)+2*nelem
      allocate ( km(iseqkm)%nmeshe(lnnmeshe) )
      nmesh(35) = lnnmeshe
      nmeshe => km(iseqkm)%nmeshe

!     --- Actually fill nmeshe

      ipointer = nelem+1
      ielem = 0
      ikelmc = 0

!     --- For ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- Compute inelem and inpelm

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

!        --- For irelem := 1 (1) nelem do

         do irelem = 1, inelem

            ielem = ielem + 1
            nmeshe(ielem) = ipointer
            nmeshe(ipointer) = inpelm

            nmeshe(ipointer+1:ipointer+inpelm) = kmeshc(ikelmc+1:ikelmc+inpelm)
            ikelmc = ikelmc+inpelm
            ipointer = ipointer+inpelm+1

         end do

      end do

      call erclos ( 'sepfillnmeshe' )
      end
