      subroutine sepfillnmeshk
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmeshk with all faces connected to the nodes
!     Old subroutine topnodfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lennmeshk

!     lennmeshk      Length of array nmeshk
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLNMESHG  Fill array nmeshg
!     TOPNODFACES1   Fill array nmeshk with all faces connected to the nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters from kmesh
!     Activate arrays
!     Create temporary space to store nmeshk
!     Compute starting addresses
!     Create nmeshk
!     Make array nmeshk permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshk' )

!     --- Create nmeshg if it does not exist

      if ( nmesh(37)==0 ) call sepfillnmeshg
      nullify ( nmeshk )
      if ( associated(km(iseqkm)%nmeshk) ) deallocate(km(iseqkm)%nmeshk)

!     --- Create temporary space to store nmeshk
!         Estimated length nmeshk: nmeshg + 2*npoint

      lennmeshk = size ( nmeshg )+2*npoint

      allocate ( nmeshk(lennmeshk), stat = error )
      if ( error/=0 ) call eralloc ( error, lennmeshk, 'nmeshk' )
      if ( ierror/=0 ) go to 1000

!     --- Create nmeshk

      call topnodfaces1 ( npoint, nfaces, nmeshg, nmeshk, lennmeshk )

!     --- Make array nmeshk permanent

      allocate ( km(iseqkm)%nmeshk(lennmeshk) )
      km(iseqkm)%nmeshk = nmeshk(1:lennmeshk)
      nmesh(41) = lennmeshk

      deallocate ( nmeshk, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshk' )

      nmeshk => km(iseqkm)%nmeshk

1000  call erclos ( 'sepfillnmeshk' )

      end subroutine sepfillnmeshk
