      subroutine sepfillnodcon1 ( iconct, kfill, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for contact problems depending on kfill
!     Old subroutine erfillnodcont
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iconct, kfill, nodes(*), nnodes

!     iconct         i    Sequence number of contact
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     nnodes        i/o   Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nconctct, nocontct, istart

!     istart         First position used at a part of kprobv
!     nconctct       Number of points with contact
!     nocontct       Number of points that make no contact
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodcon1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodcon1'
         write(irefwr,1) 'kfill, iconct', kfill, iconct
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      nconctct = kprobvp1(iconct,1)

      if ( debug ) write(irefwr,1) 'nconctct', nconctct

      if ( kfill==10 ) then

!     --- kfill = 10, Fill contact

         istart = kprobvp1(iconct,3)
         nodes(1:nconctct) = kprobvp2(istart:istart+nconctct-1)
         nnodes = nconctct
         if ( debug ) then
            write(irefwr,1) 'istart', istart
            call prinin ( nodes, nnodes, 'nodes' )
         end if  ! ( debug )

      else

!     --- kfill = 11, Fill no-contact

         nocontct = kprobvp1(iconct,2)
         istart = kprobvp1(iconct,3)+nconctct
         nodes(1:nocontct) = kprobvp2(istart:istart+nocontct-1)
         nnodes= nocontct

      end if  ! ( kfill==10 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnodcon1'

      end if  ! ( debug )
      call erclos ( 'sepfillnodcon1' )

      end subroutine sepfillnodcon1
