      subroutine sepfillnodsrfs ( inum, inpsep, nodes, nnodes, iplace )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with nodal points as defined in array inpsep
!     Special case: nodes in surfaces
!     Old subroutine prfillnodsurfs
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes, inum, iplace

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     inum           i    See inpsep
!     iplace         i    Position where nodes must be filled
!                         See subroutine prfillnodes
!     nnodes         o    Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnod, isrf1, isrf2, nsur

!                    they are not
!     isrf1          First surface number
!     isrf2          Last surface number
!     nsur           Number of surfaces surrounding a volume
!     numnod         If>0, only numnod nodes per element must be used
!                    In that case ihelp is used
!                    If 0 all nodes are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLNODSRF1 Fill array nodes for set of surfaces
!     SEPFILLNODSRF2 Fill array nodes for set of surfaces
!                    Only a restricted number of nodes per element is used
!     SEPFILLNODSRF3 Fill array nodes for all outer surfaces
!     SEPFILLNODSRF5 Fill array nodes for set of surfaces
!                    Only a restricted number of nodes per element is used
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodsrfs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodsrfs'
         write(irefwr,1) 'inum', inum
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inum==-1 .or. iplace==24 ) then

!     --- inum = -1, special case for create
!         isrf1, 2 in inpsep(1:2)
!         information of skipping is stored in inpsep(3...)

         isrf1 = inpsep(1)
         isrf2 = inpsep(2)
         numnod = inpsep(3)
         if ( debug ) &
            write(irefwr,1) 'isrf1, isrf2, numnod', isrf1, isrf2, numnod

         if ( isrf1==-1 ) then

!        --- Special case isrf1 = -1: all outer boundaries
!            First activate or create the outer boundary (kmesh part ah)

            call sepfillnodsrf3 ( nodes, nnodes, inpsep(3), inpsep(4) )

         else if ( iplace==24 ) then

!        --- New definition of create input

            numnod = inpsep(2)
            if ( numnod==0 ) then

!           --- Standard case, all nodes in surfaces

               call sepsurfsnod ( inpsep(1), nodes, nnodes, inpsep(3) )

            else

!           --- Special case, restricted set of nodes in surfaces

               call sepfillnodsrf5 ( inpsep(1), inpsep(3), nodes, nnodes, &
                                     inpsep(2), inpsep(3+inpsep(1)) )

            end if  ! ( numnod==0 )

         else if ( numnod==0 ) then

!        --- Standard case, all nodes in surfaces isrf1 to isrf2

            call sepfillnodsrf1 ( isrf1, isrf2, nodes, nnodes )

         else

!        --- Special case, restricted set of nodes in surfaces isrf1 to isrf2

            call sepfillnodsrf2 ( isrf1, isrf2, nodes, nnodes, &
                                  inpsep(3), inpsep(4) )

         end if  ! ( isrf1==-1 )

      else if ( inum>0 ) then

!     --- inum>0, Use inpsep which contains the surface numbers

         call sepsurfsnod ( inum, nodes, nnodes, inpsep )

      else if ( inum==0 ) then

!     --- inum = 0, inpsep contains old array icurvs

         nsur = inpsep(1)
         if ( nsur==0 ) nsur = 1
         if ( debug ) then
            write(irefwr,1) 'nsur', nsur
            call prinin ( inpsep(2), nsur, 'surfaces' )
         end if  ! ( debug )

         if ( nsur>0 ) then

!        --- inpsep(1)>= 0, surface numbers stored from position 2

            call sepsurfsnod ( nsur, nodes, nnodes, inpsep(2) )

         else

!        --- inpsep(1)<0, surface numbers from inpsep(2 to 3)

            isrf1 = inpsep(2)
            isrf2 = inpsep(3)
            if ( debug ) write(irefwr,1) 'isrf1, isrf2', isrf1, isrf2

            call sepfillnodsrf1 ( isrf1, isrf2, nodes, nnodes )

         end if  ! ( ichoice>0 )

      end if  ! ( inum>0 )

1000  call erclos ( 'sepfillnodsrfs' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End sepfillnodsrfs'

      end if  ! ( debug )

      end subroutine sepfillnodsrfs
