      subroutine sepiinfsurf ( istart, inpelm, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find starting address of surface isurnr in array kmeshn (minus 1)
!     Also find number of element and number of points per element in isurnr
!     Old subroutine iniinfsurf
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istart, inpelm, isurnr

!     inpelm         o    number of points per element in surface isurnr
!     istart         o    Last position in kmeshn before surface isurnr
!     isurnr         i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepiinfsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepiinfsurf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find starting address minus 1

      istart = 2*nsurfs
      do i = 1, isurnr-1
         istart = istart+kmeshn(i)*kmeshn(i+nsurfs)
      end do  ! i = 1, isurnr-1

!     --- Find inpelm and inelem

      inpelm = kmeshn(isurnr)
      inelem = kmeshn(isurnr+nsurfs)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepiinfsurf'

      end if
      call erclos ( 'sepiinfsurf' )

      end
