      subroutine sepmainkeyw2 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string read is (main) keyword
!     If so return with icode >0 or finished true
!     Old subroutine intx02keyw
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if a main keyword has been found (true)
!                         or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ispeclsav

!     i              Counting variable
!     ispeclsav      Help parameter to save the value of ispecl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepmainkeywnn

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMAINKEYWNN  Check if string read is (main) keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmainkeyw2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmainkeyw2'
         write(irefwr,1) 'lastitem, ityprd, lastitembef, nmain', &
                          lastitem, ityprd, lastitembef, nmain
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- A keyword has been found, but at this moment we do not know
!         whether this keyword has been read with or without numbers
!         Therefore we read again

      if ( ityprd==100 ) then

!     --- ityprd = 100

         ityprd = 2
         icode = sepmainkeywnn()

      else

!     --- ityprd # 100

         lastitem = lastitembef

         ispeclsav = ispecl
         ispecl = 9
         call asknxt
         ispecl = ispeclsav

         if ( debug ) &
            write(irefwr,1) 'lastitem/1, ityprd', lastitem, ityprd

!        --- Next we have to check whether the keyword contains numbers
!            If so we have to split the keyword into a part without number
!            and a number
!            This is necessary to check if it is a keyword, since keywords do
!            not contain numbers

         numchar = sepsplitkeyw()
         if ( debug ) then
            write(irefwr,*) 'string ', string(1:nchar)
            write(irefwr,*) 'keyword ', keyword(1:numchar)
            write(irefwr,1) 'lastitem/2, ityprd', lastitem, ityprd
         end if  ! ( debug )

         do i = 1, abs(nmain)

            if ( keyword(1:ncodemain(i))==codemain(i)(1:ncodemain(i)) ) then

!           --- Main keyword found

               iretur = i
               lastitem = lastitembef+1   ! Only string is used
               if ( nmain<0 ) lastitem = lastitem-1
               finished = .true.
               if ( debug ) write(irefwr,*) 'main keyword'
               go to 1000

            end if  ! ( string(1:ncodemain(i))==codemain(i)(1:ncodemain(i)) )

         end do  ! i = 1, abs(nmain)

!        --- Check if string is a standard keyword
!            If not and it is a vector name return with ityprd = 2
!            If not and it is a scalar name return with ityprd = 11

         icode = sepmainkeywnn()

         if ( icode>0 ) then
            if ( iseqkeywitem(icode)==0 ) &
               iseqkeywitem(icode) = lastitem  ! store position
         end if  ! ( icode>0 )

      end if
      if ( ityprd==4 ) lastitem = lastitembef+1   ! Only string is used
      if ( debug ) write(irefwr,1) 'icode, ityprd', icode, ityprd

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode, iretur, ityprd, index_rdarr(3)', &
                          icode, iretur, ityprd, index_rdarr(3)
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,*) 'End sepmainkeyw2'

      end if  ! ( debug )
      call erclos ( 'sepmainkeyw2' )

      end subroutine sepmainkeyw2
