      function sepnodezerlev ( ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with node at boundary defined by zero levelset
!     1d only
!     Old subroutine ininodezerlev
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     node
!     level_set
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer sepnodezerlev, ilev

!     ilev           i    Sequence number of level being considered
!     sepnodezerlev  o    Node number found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numbndobst

!     istart         Starting address in ipointobst
!     numbndobst     Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3085   Number of nodes in boundary of zero level set is not 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnodezerlev' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnodezerlev'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      sepnodezerlev = 0  ! initialization

!     --- activate or create array ipointobst

      call  sepflpointobst

!     --- get starting address and number of points in boundary

      call inizerolev ( ipointobst, istart, numbndobst, ilev )

      if ( numbndobst/=1 ) then

!     --- Error: boundary consists not exactly of 1 point

         call errint ( numbndobst, 1 )
         call errsub ( 3085, 1, 0, 0 )

      else

!     --- exactly 1 point, find node

         sepnodezerlev = ipointobst(istart+1)

      end if  ! ( numbndobst/=1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnodezerlev'

      end if  ! ( debug )
      call erclos ( 'sepnodezerlev' )

      end
