      subroutine sepoutcr ( icurvind, isurfaces, nsurfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all curves surrounding a set of surfaces
!     Internal curves are indicated by -1, external curves by 1
!     Old subroutine inioutcr
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvind(*), isurfaces(*), nsurfaces

!     icurvind       o    Integer work array of length ncurvs that is filled as
!                         follows
!                         icurvind(i) = -1, at the boundary of the surfaces
!                         icurvind(i) = 1, or not in the surfaces icurvind(i)=0
!     isurfaces      i    Array containing the surfaces from which information
!                         must be found
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, istarcv, istart, isurf, isurnr, ncur, j

!     i              Counting variable
!     icurnr         Curve sequence number
!     istarcv        Start position of curves corresponding to specific surface
!                    in kmeshs minus 1
!     istart         First position used at a part
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     j              Counting variable
!     ncur           Number of curves surrounding a surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array icurvind
!     For isurnr := isrf1 (1) isrf2 do
!        For all external curves icurnr of surface do
!           if ( icurvind(icurnr)=0 ) then
!               icurvind(icurnr):=1
!           else ( icurvind(icurnr)#0 )
!               icurvind(icurnr):=-1
!        For all internal curves icurnr of surface do
!           icurvind(icurnr):=-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array icurvind

      icurvind(1:ncurvs) = 0

!     --- For isurnr := isrf1 (1) isrf2 do

      do j = 1, nsurfaces

!     --- First find type of surface, number of curves and number of inner
!         curves

         isurnr = isurfaces(j)
         istart = nsurfs+1+8*(isurnr-1)
         isurf = kmeshs(istart+1)
         if ( isurf==24 ) isurf = 7    ! compound surface
         istarcv = kmeshs(isurnr)
         ncur = kmeshs(isurnr+1) - istarcv
         if ( isurf==7 .or. isurf==11 ) then
            ninner = kmeshs(istart+4)
         else if ( isurf==12 ) then
            ninner = ncur-kmeshs(istart+3)-kmeshs(istart+4)- &
                     kmeshs(istart+5)-kmeshs(istart+6)
         else
            ninner = 0
         end if

!        --- For all external curves icurnr of surface do

         do i = 1, ncur-ninner

            icurnr = abs(kmeshs(istarcv+i))
            if ( icurvind(icurnr)==0 ) then

!           --- icurvind(icurnr)=0

                icurvind(icurnr) = 1

            else

!           --- icurvind(icurnr)#0

               icurvind(icurnr) = -1

            end if

         end do

!        --- For all internal curves icurnr of surface do

         do i = ncur-ninner+1, ncur

            icurnr = kmeshs(istarcv+i)
            icurvind(icurnr) = -1

         end do

      end do

      end
