      subroutine sepprobextend ( number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-12-2013 Correction part e not the last one
!        version  1.0  date 04-03-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend problem definition with vectors of special structure
!     of exactly number degrees of freedom
!     The extension takes place for problem iprob
! **********************************************************************
!
!                       KEYWORDS
!
!     structure
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: number

!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenprob1, istart, istart1, ishift, ih1, k
      double precision :: dummy(1)

!                    they are not
!     dummy          Dummy parameter
!     ih1            First position used at a part
!     ishift         Shift of specific variable with respect to diagonal
!     istart         First position used at a part
!     istart1        First position used at a part
!     k              Counting variable
!     kprobcopy      Work array to copy a part of kprob
!     lenprob1       Length of array kprob
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprobextend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprobextend'
         call prinin ( kprob, kprobloc(3), 'kprobloc' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create space for copy of kprob

      iseqkp = kprob
      nprob = kp(iseqkp)%nprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      lenprob1 = size(kprobloc)

!     --- get some parameters

      ndtvec = kprobloc(6)
      indpre = kprobloc(18)
      kprelgrp = kprobloc(39)

      if ( debug ) &
         write(irefwr,1) 'ndtvec, nprob, indpre, kprelgrp', &
                          ndtvec, nprob, indpre, kprelgrp

      if ( indpre==0 ) then

!     --- Simple case, we put kprob part e at the back of kprob
!         Consider first problem

         kprobloc(6) = 1
         indpre = kprobloc(3)+1         ! starting position kprob e
         kprobloc(18) = indpre
         kprobloc(3) = kprobloc(3)+kprelgrp+1  ! extension of kprob
         ndtvec = 1

!        --- Check length

         call prinlg ( 'kprob', kprobloc(3)+kprelgrp+1, dummy, kprobloc )
         if ( ierror/=0 ) go to 1000

         kprobloc(indpre:indpre+kprelgrp-1) = 0  ! not important
         kprobloc(indpre+kprelgrp) = -number     ! essential
         ishift = kprelgrp+1

      else

!     --- Complicated case, kprob part e already exists
!         Extend ndtvec by 1 and create new part e

         indpre = kprobloc(18)
         if ( kprobloc(3)-indpre+1==ndtvec*(kprelgrp+1) ) then

!        --- part e is last part and may be extended

            if ( debug ) write(irefwr,*) 'part e is last part'
            kprobloc(6) = ndtvec+1
            ishift = kprelgrp+1
            kprobloc(3) = kprobloc(3)+ishift
            call prinlg ( 'kprob', kprobloc(3)+ishift, dummy, kprobloc )
            if ( ierror/=0 ) go to 1000

!           --- First copy last ndtvec positions

            indpre = indpre+ndtvec*kprelgrp
            ih1 = indpre+kprelgrp
            kprobloc(ih1:ih1+ndtvec-1) = kprobloc(indpre:indpre+ndtvec-1) ! Copy
            kprobloc(indpre:ih1-1) = 0       ! not important
            kprobloc(ih1+ndtvec) = -number     ! essential
            ndtvec = ndtvec+1

         else

!        --- part e is not last part and is copied

            if ( debug ) write(irefwr,*) 'part e is not last part'
            kprobloc(6) = ndtvec+1
            ishift = (ndtvec+1)*(kprelgrp+1)

            if ( debug ) &
               write(irefwr,1) 'ndtvec_new, kprelgrp, ishift', &
                                kprobloc(6), kprelgrp, ishift
            call prinlg ( 'kprob', kprobloc(3)+ishift, dummy, kprobloc )
            if ( ierror/=0 ) go to 1000

            kprobloc(18) = kprobloc(3)+1
            istart1 = kprobloc(18)
            kprobloc(3) = kprobloc(3)+ishift

            if ( debug ) &
               call prinin ( kprobloc(indpre), ndtvec*(kprelgrp+1), &
                             'indpre_old' )

!           --- Copy old part e

            kprobloc(istart1:istart1+ndtvec*(kprelgrp+1)-1) = &
               kprobloc(indpre:indpre+ndtvec*(kprelgrp+1)-1)

!           --- Create new part

            indpre = kprobloc(18)+ndtvec*kprelgrp
            ih1 = indpre+kprelgrp
            kprobloc(ih1:ih1+ndtvec-1) = kprobloc(indpre:indpre+ndtvec-1) ! Copy
            kprobloc(indpre:ih1-1) = 0       ! not important
            kprobloc(ih1+ndtvec) = -number     ! essential
            ndtvec = ndtvec+1

            if ( debug ) then
               write(irefwr,1) 'istart1, ndtvec, kprelgrp', &
                                istart1, ndtvec, kprelgrp
               call prinin ( kprobloc(istart1), (ndtvec+1)*(kprelgrp+1), &
                             'indpre_new' )
            end if  ! ( debug )

         end if  ! ( kprobloc(3)-indpre+1==ndtvec*(kprelgrp+1) )

      end if  ! ( indpre==0 )
      kprobe => kprobpart%kprobe
      kprobe2 => kprobpart%kprobe2
      kprobe3 => kprobpart%kprobe3
      indpre = kprobloc(18)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobloc, size(kprobloc), 'kprob' )
         write(irefwr,*) 'End sepprobextend'

      end if  ! ( debug )
      call erclos ( 'sepprobextend' )

      end
