      subroutine sepreadsetsub ( inumlc, nmax, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret series of numbers read
!     Old subroutine intx02sub
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inumlc, nmax, ihelp(*)

!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     inumlc         i    Number of numbers read in a record
!     nmax           i    maximum number of numbers allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1463   The number of numbers found after the keyword too large
!    1464   Number found after keyword out of range
!    1468   Array tabout too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadsetsub' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadsetsub'
         write(irefwr,1) 'inumlc, nmax, icode', inumlc, nmax, icode
         write(irefwr,*) 'code(icode) ', code(icode)
         write(irefwr,2) 'tabmax', tabmax(icode)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inumlc==0 .or. inumlc>nmax ) then

!     --- Too few or too much numbers found after keyword

         call errchr ( code(icode), 1 )
         call errint ( inumlc, 1 )
         call errint ( nmax, 2 )
         call errsub ( 1463, 2, 0, 1 )
         call errcrd

      end if

      if ( iposut+inumlc>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      do i = 1, inumlc
         if ( infotabout(iposut+i)/=11 .and. &
              infotabout(iposut+i)/=19 .and. &
              abs(tabout(iposut+i))>tabmax(icode) ) then
            call errchr ( code(icode), 1 )
            call errint ( i, 1 )
            call erreal ( tabout(iposut+i), 1 )
            call erreal ( tabmax(icode), 2 )
            call errsub ( 1464, 1, 2, 1 )
            call errcrd
         end if
      end do
      iposut = iposut+inumlc
      ihelp(icode) = inumlc
      if ( ityprd==4 ) readnx = .false.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadsetsub'

      end if
      call erclos ( 'sepreadsetsub' )

      end subroutine sepreadsetsub
