      subroutine sepreadtype16
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a value or value = v, or func = i
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0216
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifunc
      double precision value(2)
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     ifunc          Parameter k after FUNC = &
!     value          Array containing the real or complex (2 positions)
!                    number read by readfn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFN         Read items of the shape number, VALUE=number or FUNC=i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype16' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype16'
         write(irefwr,1) 'icode, itype(icode)', icode, itype(icode)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      compl = itype(icode)==26
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      if ( debug ) write(irefwr,*) 'compl ', compl

      call readfn ( ifunc, value, compl )

      if ( ifunc>0 .or. ifunc<-100 ) then

!     --- function found

         table(icode) = ifunc

      else

!     --- value found

         if ( compl ) then
            iposut = iposut+2

            if ( iposut>nout ) then
               call errint ( nout, 1 )
               call errsub ( 1468, 1, 0, 0 )
               go to 1000
            end if  ! ( iposut>nout )

            table(icode) = -iposut+1
            tabout(iposut-1) = value(1)
            tabout(iposut) = value(2)
         else
            iposut = iposut+1

            if ( iposut>nout ) then
               call errint ( nout, 1 )
               call errsub ( 1468, 1, 0, 0 )
               go to 1000
            end if  ! ( iposut>nout )

            table(icode) = -iposut
            tabout(iposut) = value(1)
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype16'

      end if
      call erclos ( 'sepreadtype16' )

      end subroutine sepreadtype16
