      subroutine sepreadtype23 ( subkey )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by possibly a special keyword that is skipped
!     table(icode) is set equal to 1
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0223
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) :: subkey

!     subkey         i    Sub (second) keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, numkey

!     ih             Help variable to store a constant temporarily
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1467   keyword has been found more than once
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype23' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype23'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      call asknxt
      if ( icountkey(icode).ne.0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      ih = nint(-table(icode))
      if ( ih.lt.0 ) then
         call errchr ( code(icode), 1 )
         call errsub ( 1467, 0, 0, 1 )
         call errcrd
         go to 1000
      end if
      subkey = codesub(ih)
      numkey = ncodesub(ih)
      if ( ityprd.eq.4 ) then
         if ( string(1:numkey).ne.subkey(1:numkey) ) readnx = .false.
      else
         readnx = .false.
      end if
      table(icode) = 1d0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype23'

      end if
      call erclos ( 'sepreadtype23' )

      end subroutine sepreadtype23
