      subroutine seprmmatrix ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-12-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove array with sequence number matr from realmt
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix
!     remove
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: matr

!     matr     i/o     Defines matrix information by integer sequence number
!                        If it already exists the old info is removed
!                        New matr info is created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     S              pointer to realmt(iseqmt)
!     i         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmmatrix' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmmatrix'
         write(irefwr,1) 'matr, lastmatrix', matr, lastmatrix
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( matarr(matr)==0 ) then

!     --- Array has already been removed

         matr = 0
         go to 1000

      end if  ! ( matarr(matr)==0 )

      if ( matr==lastmatrix ) then

!     --- matr = lastmatrix, decrease lastmatrix

         lastmatrix = lastmatrix-1

         do i = lastmatrix, 1, -1
            if ( matarr(lastmatrix)==0 ) then
               lastmatrix = lastmatrix-1
            else
               exit
            end if  ! ( matarr(lastmatrix)==0 )
         end do  ! i = lastmatrix, 1, -1

      end if  ! ( matr==lastmatrix )
      matarr(matr) = 0
      nextfreemat = min(nextfreemat,matr)
      S => realmt(matr)

      IM => intmt(S%intmatref)

      if ( debug ) write(irefwr,*) 'before S%S_ii'
      if ( associated(S%S_ii) ) then
         deallocate ( S%S_ii, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'S_ii' )
         if ( ierror/=0 ) go to 1000
      end if  ! ( associated(S%S_ii) )

      if ( debug ) write(irefwr,*) 'before S%matinv'
      if ( associated(S%matinv) ) then

         deallocate ( S%matinv, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'matinv' )
         if ( ierror/=0 ) go to 1000
         nullify(S%S_i%MD)
      end if  ! ( associated(S%matinv) )

      if ( debug ) write(irefwr,*) 'before S%Dlapshift'
      if ( associated(S%Dlapshift) ) then

         deallocate ( S%Dlapshift, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'Dlapshift' )
         if ( ierror/=0 ) go to 1000
      end if  ! ( associated(S%Dlapshift) )

      if ( debug ) write(irefwr,*) 'before S%S_R'
      if ( associated(S%S_R) ) then

         deallocate ( S%S_R, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'S_R' )
         if ( ierror/=0 ) go to 1000
      end if  ! ( associated(S%S_R) )

      if ( debug ) write(irefwr,*) 'before S%S_Qprime'
      if ( associated(S%S_Qprime) ) then

         deallocate ( S%S_Qprime, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'S_Qprime' )
         if ( ierror/=0 ) go to 1000
      end if  ! ( associated(S%S_Qprime) )

      if ( debug ) write(irefwr,*) 'before nullify'
      nullify ( S%S_pi, S%S_ip, S%S_pp, S%S_ic, S%matinv, S%Dlapshift  )
      nullify ( S%S_Q, S%S_DIV, S%S_G, S%S_C, S%S_PM, S%S_PMD, S%S_UD  )
      nullify ( S%S_if, S%S_fi, S%S_iout )
      S%decomp = .true.

      matr = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'matr, lastmatrix', matr, lastmatrix
         write(irefwr,*) 'End seprmmatrix'

      end if  ! ( debug )
      call erclos ( 'seprmmatrix' )

      end subroutine seprmmatrix
