      subroutine seprmnmesh
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-07-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove subarrays of nmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmnmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmnmesh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( associated(km(iseqkm)%nmesha) ) then
         deallocate ( km(iseqkm)%nmesha, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmesha' )
         nullify(km(iseqkm)%nmesha)
      end if  ! ( associated(km(iseqkm)%nmesha) )

      if ( associated(km(iseqkm)%nmeshb) ) then
         deallocate ( km(iseqkm)%nmeshb, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshb' )
         nullify(km(iseqkm)%nmeshb)
      end if  ! ( associated(km(iseqkm)%nmeshb) )

      if ( associated(km(iseqkm)%nmeshc) ) then
         deallocate ( km(iseqkm)%nmeshc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshc' )
         nullify(km(iseqkm)%nmeshc)
      end if  ! ( associated(km(iseqkm)%nmeshc) )

      if ( associated(km(iseqkm)%nmeshd) ) then
         deallocate ( km(iseqkm)%nmeshd, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshd' )
         nullify(km(iseqkm)%nmeshd)
      end if  ! ( associated(km(iseqkm)%nmeshd) )

      if ( associated(km(iseqkm)%nmeshe) ) then
         deallocate ( km(iseqkm)%nmeshe, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshe' )
         nullify(km(iseqkm)%nmeshe)
      end if  ! ( associated(km(iseqkm)%nmeshe) )

      if ( associated(km(iseqkm)%nmeshf) ) then
         deallocate ( km(iseqkm)%nmeshf, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshf' )
         nullify(km(iseqkm)%nmeshf)
      end if  ! ( associated(km(iseqkm)%nmeshf) )

      if ( associated(km(iseqkm)%nmeshg) ) then
         deallocate ( km(iseqkm)%nmeshg, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshg' )
         nullify(km(iseqkm)%nmeshg)
      end if  ! ( associated(km(iseqkm)%nmeshg) )

      if ( associated(km(iseqkm)%nmeshh) ) then
         deallocate ( km(iseqkm)%nmeshh, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshh' )
         nullify(km(iseqkm)%nmeshh)
      end if  ! ( associated(km(iseqkm)%nmeshh) )

      if ( associated(km(iseqkm)%nmeshi) ) then
         deallocate ( km(iseqkm)%nmeshi, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshi' )
         nullify(km(iseqkm)%nmeshi)
      end if  ! ( associated(km(iseqkm)%nmeshi) )

      if ( associated(km(iseqkm)%nmeshj) ) then
         deallocate ( km(iseqkm)%nmeshj, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshj' )
         nullify(km(iseqkm)%nmeshj)
      end if  ! ( associated(km(iseqkm)%nmeshj) )

      if ( associated(km(iseqkm)%nmeshk) ) then
         deallocate ( km(iseqkm)%nmeshk, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshk' )
         nullify(km(iseqkm)%nmeshk)
      end if  ! ( associated(km(iseqkm)%nmeshk) )

      if ( associated(km(iseqkm)%nmeshl) ) then
         deallocate ( km(iseqkm)%nmeshl, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nmeshl' )
         nullify(km(iseqkm)%nmeshl)
      end if  ! ( associated(km(iseqkm)%nmeshl) )

      if ( associated(km(iseqkm)%nmesh) ) km(iseqkm)%nmesh(31:42) = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprmnmesh'

      end if  ! ( debug )
      call erclos ( 'seprmnmesh' )

      end subroutine seprmnmesh
