      subroutine sepvolmsnod1 ( nvolumes, nodes, nnodes, ivolumes, numnod, &
                                ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the volumes stored in array volumes
!     in array nodes
!     The number of nodes in nodes is stored in nnodes
!     Old subroutine ervolmsnod1
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nvolumes, nodes(*), ihelp(*), numnod, &
              nnodes, ivolumes(*)

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     ivolumes       i    Array containing the volumes from which the nodes
!                         must be found
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
!     nvolumes       i    Number of volumes in array ivolumes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, inpelm, ivolnr, jvolnr, &
              inode, irel, ielem
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ielem          Element number
!     inode          Node sequence number
!     inpelm         number of points per element
!     irel           Relative number node number in element
!     istart         First position used at a part
!     ivolnr         Volume sequence number
!     iwork          integer work array
!     jvolnr         Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1119   Relative nodal point number in element is out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    nnodes := 0
!    For all volumes ivolnr in ivolumes do
!       inelem := number of elements in ivolnr
!       inpelm := number of nodes in elements in ivolnr
!       istart := starting address of volume in kmeshc
!       For all nodes inode in volume do
!          if ( iwork(inode)=0 ) then
!             iwork(inode):=1
!             nnodes := nnodes+1
!             nodes(nnodes) := inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepvolmsnod1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepvolmsnod1'
         call prinin ( ivolumes, nvolumes, 'ivolumes' )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0

      do jvolnr = 1, nvolumes

         ivolnr = ivolumes(jvolnr)
         call sepextkmesht ( ivolnr, ivolnr, istart, inpelm )

!        --- check relative nodal points

         do i = 1, numnod

            irel = ihelp(i)

            if ( irel<=0 .or. irel>inpelm ) then

!           --- irel out of range

               call errint ( irel, 1 )
               call errint ( inpelm, 2 )
               call errint ( i, 3 )
               call errsub ( 1119, 3, 0, 0 )
               go to 1000

            end if

         end do  ! i = 1, numnod

!        --- For all nodes inode in volume

         do ielem = 1, inelem

            do i = 1, numnod

               inode = kmeshc(istart+ihelp(i))
               if ( inode<=0 ) exit
               if ( iwork(inode)==0 ) then

!              --- iwork(inode)=0

                  iwork(inode)=1
                  nnodes = nnodes+1
                  nodes(nnodes) = inode

               end if

            end do  ! i = 1, numnod

            istart = istart+inpelm

         end do  ! ielem = 1, inelem

      end do  ! jvolnr = 1, nvolumes

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'sepvolmsnod1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepvolmsnod1'

      end if

      end subroutine sepvolmsnod1
