      subroutine toordr2d ( irenum, coor, npoint, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map 2d array coor to new numbering, where only the second parameter
!     should be renumbered
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, irenum(npoint), ndim
      double precision coor(ndim,npoint)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     irenum         i    Renumbering array, i.e. array with new node numbers
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision, allocatable :: work(:)

!     i              Counting variable
!     work           Work array to store one component of coor at a time
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toordr2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toordr2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'work' )
      if ( ierror/=0 ) go to 1000

      do i = 1, ndim
         work(irenum(1:npoint)) = coor(i,:)
         coor(i,:) = work
      end do  ! i = 1, ndim

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'toordr2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toordr2d'

      end if  ! ( debug )

      end
