      subroutine topq03 ( nplsub, nelemlc, neleml )

! ======================================================================
!
!        programmer   Guus Segal
!
!        version 1.1    date   04-05-1994 Add possibility line elements
!        version 1.0    date   18-01-1993
!
!
!   copyright (c) 1993-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!     DESCRIPTION
!
!  Check if a three-dimensional mesh of surface elements consists of quads
!  only or that a subdivision into triangular elements must be made
!  The number of elements of the subdivided mesh is computed as well as the
!  number of points per element in the new mesh. Compute also the number
!  of line elements.
! ********************************************************************
!
!     KEYWORDS
!
!     3d
!     mesh
!     subdivision
!     surface
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! ********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS

      integer neleml, nplsub, nelemlc

!     neleml         o    Number of line elements to be used
!     nelemlc        i    Local number of elements
!     nplsub         o    Number of points in the submesh consisting of elements
!                         with vertex nodes only
! ********************************************************************
!
!     COMMON BLOCKS
!
! ********************************************************************
!
!     LOCAL PARAMETERS

      integer ielgrp
      logical triang

!     ielgrp  element group number
!     triang  Indicates if triangular elements have been found (true) or not
! ********************************************************************
!
!     SUBROUTINES CALLED
!
! ********************************************************************
!
!     I/O
!
! ********************************************************************
!
!     ERROR MESSAGES
!
! ********************************************************************
!
!     PSEUDO CODE
!
!
!     triangles := false
!     For ielgrp := 1 (1) nelgrp do
!        ishape := kmeshf(ielgrp)
!        if ( ishape = 3, 4 or 7 ) then
!           triangles := true
!     nelem := 0
!     neleml:= 0
!     For ielgrp := 1 (1) nelgrp do
!        ishape := kmeshf(ielgrp)
!        if ( ishape = 1 or 2 ) then
!           neleml := neleml + kelmg(ielgrp)
!        else if ( ishape = 3, 4 or 7 ) then
!           nelem  := nelem + kelmg(ielgrp)
!        else if ( ishape = 5 or 6 ) then
!           if ( triangles ) then
!              nelem := nelem + kelmg(ielgrp)*2
!           else ( quadrilaterals )
!              nelem := nelem + kelmg(ielgrp)
!
! ======================================================================
!
!     Check if there are triangles

      triang = .false.

      do 100 ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)
         if ( ishape==3 .or. ishape==4 .or. ishape==7 ) &
              triang = .true.

100   continue

!     Compute nelem and set nplsub

      if ( triang ) then
         nplsub = 3
      else
         nplsub = 4
      end if

      nelemlc  = 0
      neleml = 0

      do 200 ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)
         inelem = kmeshg(ielgrp)

         if ( ishape==1 .or. ishape==2 ) then

            neleml = neleml + inelem

         else if ( ishape==3 .or. ishape==4 .or. ishape==7 ) then

            nelemlc = nelemlc + inelem

         else if ( ishape==5 .or. ishape==6 ) then

            if ( triang ) then
               nelemlc = nelemlc + inelem*2
            else
               nelemlc = nelemlc + inelem
            end if

         end if

200   continue

      end
