      subroutine topqu3 ( itrian, neleml, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 16-02-2005 Correction connection elements
!                                      and hexahedrons
!        version  1.7  date 28-05-1997 Extension with hexahedrons
!        version  1.6  date 23-12-1996 Extension with 4-point triangles
!
!   copyright (c) 1988-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Divide quadratic triangular or quadrilateral elements (2 and 3D)
!     into elements of linear shape. Use info of array ITRIAN. If needed
!     divide line elements into linear elements.
! **********************************************************************
!
!                       KEYWORDS
!
!     subdivision
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itrian, neleml, inpelm

!     inpelm         Number of nodes in an element
!     itrian  i   Input array of length NELGRP in which information of the
!                 various element groups is stored with respect to the
!                 subdivision into linear triangles.
!                 itrian corresponds to the ielgrp th element
!                 group and its value has the following meaning:
!                 -3:   Subdivision of a quadratic line element
!                 -2:   Store linear line element
!                 -1:   Skip the element group
!                  0:   No subdivision required
!                  1:   Subdivision of a quadrilateral with 4 points into 2
!                       triangles
!                  2:   Subdivision of a triangle with 6 or 7 points into 4
!                       triangles
!                  3:   Subdivision of a triangle with 6 or 7 points into 1
!                       triangle
!                  4:   Subdivision of a quadrilateral with 9 points into 8
!                       triangles
!                  5:   Subdivision of a quadrilateral with 9 points into 8
!                       triangles. The centroid must be computed by
!                       interpolation
!                  6:   Subdivision of a quadrilateral with 9 points into 2
!                       triangles.
!     neleml  o   Number of line elements in kline
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, iielem, nsubel, j, k

!     i              Counting variable
!     iielem         Relative element number with respect to element group
!     istart         Start address minus one of information of an element either
!                    in array ISHSUB or in array ITETRA
!     j              Count helpvariable
!     k              Count helpvariable
!     nsubel         Product of number of subelements and number of points in a
!                    subelement of one element to be divided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all element groups
!       Compute for each group shape number, number of nodes in standard
!       element and number of elements
!       Check whether elements must be subdivided
!       If yes then perform subdivision
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topqu3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu3'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill some constants

      neleml = 0

!     --- Loop over elements, distinguish between elements to be subdivided
!         and elements that remain unchanged
!         The action depends on array itrian

     if ( itrian==0 ) then

!     --- Linear triangular element

         do i = 1, 3
            do iielem = 1,inelem

               kmeshctmp ((iielem-1)*3 + i ) = &
                  kmeshc ((iielem-1)*inpelm + i )

            end do
         end do

      else if ( itrian>0 ) then

!     --- Quadratic element or quadrilateral found, subdivide

         istart = ishstr(ishape)
         if ( itrian==-1 ) then
            nsubel =   0
         else if ( itrian==1 ) then
            nsubel =   6
            istart =  ishstr(5)
         else if ( itrian== 2 ) then
            nsubel =  12
            istart =  ishstr(4)
         else if ( itrian== 3 ) then
            nsubel =   3
            istart =  ishstr(4) + 12
         else if ( itrian== 4 .or. &
                   itrian== 5 ) then
            nsubel =  24
            istart =  istart + 20
         else if ( itrian== 6 ) then
            nsubel =   6
            istart =  istart + 44
         end if

         do i = 1, nsubel
            do iielem = 1, inelem

               kmeshctmp((iielem-1)*nsubel+i) = &
               kmeshc((iielem-1)*inpelm+ishsub(istart+i))

            end do
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'neleml', neleml
         write(irefwr,*) 'End topqu3'

      end if
      call erclos ( 'topqu3' )

      end
