      subroutine writb2 ( inum, numarr, namevc, kmesh, ihelp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.3  date 21-11-2012 Long integers
!        version  4.2  date 01-08-2001 new definition kmesh part h
!        version  4.1  date 04-04-2001 Adaptation for empty surfaces
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  array kmesh is written to backing storage
!  Warning: The information of KMESH part r3 is not correct in the
!           backing storage file, because the information of file 3
!           is not copied. As soon as this information will be used
!           in a subroutine different from MESH, an update is necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulexxx
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inum, numarr, kmesh, ihelp(*)
      character *(*) namevc

!    ihelp           Work array of length NREC2 which is used as a buffer
!    inum       i    Choice parameter at the input of subroutine WRITBS
!                    When INUM<=0, KMESH is written on the next free positions
!                    of file 2. When INUM = -1, the subroutine prints a text, as
!                    described in the Programmer's Guide.
!                    When INUM>0, KMESH is written on positions of a preceding
!                    written array, with array number INUM.
!    kmesh      i    Standard SEPRAN array containing information of the mesh
!    namevc     i    Name of the array to be written to file 2.
!                    Character variable, is used for printing only
!    numarr     o    At output NUMARR gets the sequence number of KMESH in
!                    file 2.
!                    NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprio, leng, lenglc, i, irecnr, iwork(2), iwork1(5)

!     i           Loop variable
!     iprio       Help variable to store the old priority temporarily
!                 The new priority is made equal to the old one
!     irecnr      Record number in a direct access file
!     iwork       Two-dimensional work array in which two positions for
!                 each part of the array to be written are stored
!                 The first position is always -1000001 to indicate that
!                 the new version of writbs is used, the second position
!                 contains the length of the array part
!                 iwork is used in order to be compatible with preceding
!                 versions of arrays KMESH that have been written
!     iwork1      integer work array
!     leng        Computed total length of the arrays to be written
!     lenglc      Local length of array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigpr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIGPR         Get priority of array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     WRIT00         writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The complete array is written to file 2
!     The subroutine finds which parts of KMESH have been filled and which
!     have not been filled.
!     The actual writing is performed by subroutine WRIT00, which
!     uses a buffer of length NREC2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writb2' )

!     --- compute length of complete array
!         First set some parameters

      call sepfillkmesh ( kmesh )
      iwork(1) =-1000003
      iwork(2) = 100
      leng   = 80

!     --- Add length of permanent parts to length of KMESH
!         Part c:  kmesh(17)
!         Part d:  kmesh(18)
!         Part e:  kmesh(19)
!         Part h:  kelmh
!         Part i:  kmesh(23)
!         Add length of auxiliary parts to length
!         Part j:  kmesh(24)
!         Part k:  kmesh(25)
!         Part m:  kmesh(27)
!         Part n:  kmesh(28)
!         Part o:  kmesh(29)
!         Part q:  kmesh(31)
!         Part r:  kmesh(32)
!         Part s:  kmesh(33)
!         Part iinput:  kmesh(34)
!         Part rinput:  kmesh(35)

      if ( kelmc>0 ) leng = leng + size ( kmeshc )+2
      if ( kelmd>0 ) leng = leng + size ( kmeshd )+2
      if ( kelme>0 ) leng = leng + size ( kmeshe )+2

      if ( kelmh>0 ) leng = leng + size ( kmeshh )+2

      if ( kelmi>0 ) leng = leng + size ( coor )*intlen+2
      if ( kelmj>0 ) leng = leng + size ( kmeshj )+2
      if ( kelmk>0 ) leng = leng + size ( kmeshk )+2
      if ( kelml/=0 ) leng = leng + size ( kmeshl )+2
      if ( kelmm/=0 ) leng = leng + size ( kmeshm )+2
      if ( kelmn/=0 ) leng = leng + size ( kmeshn )+2
      if ( kelmo>0 ) leng = leng + size ( kmesho )+2

      if ( kelmq/=0 ) leng = leng + size ( kmeshq )+2
      if ( kelmr/=0 ) leng = leng + size ( kmeshr )+2
      if ( kelms/=0 ) leng = leng + size ( kmeshs )+2
      if ( kmiinput>0 ) leng = leng + size ( iinputkm )+2
      if ( kmrinput>0 ) leng = leng + size ( rinputkm )+2

!     --- Add parts that are stored on backing storage, but whose reference
!         can not be found immediately in KMESH
!         Parts of KMESH part h

      if ( kelmh>10000000 ) then

!     --- kelmh>0, i.e. KMESH part h exists

         leng = leng + 2*ncurvs + (size ( curvesmsh ) + &
                  size ( curvefacts ))*intlen

         do i = 1,nsurfs+nvolms
            if ( kmeshh(1,i+1)>0 ) then
               leng = leng + kmeshh(1,i+1)*ndim*intlen + &
                        size ( kmhsur(i)%maparsur ) + &
                        abs ( kmeshh(4,i) ) + 5
               if ( kmeshh(5,i)>0 ) &
                  leng = leng + abs ( kmeshh(5,i) )
               if ( kmeshh(5,i)>0 ) &
                  leng = leng + abs ( kmeshh(5,i) )
            end if
         end do

      end if

!     --- write parts of arrays to backing storage
!         Array KMESH:

      km(iseqkm)%kmesh(2) = 100
      call writ00( inum, numarr, namevc, km(iseqkm)%kmesh, leng, 1, 80, &
                   ihelp )

!     --- KMESH part c,d and e

      lenglc = size ( kmeshc )
      iwork(2) = lenglc
      call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)

      call writ00 ( inum, numarr, namevc, kmeshc, leng, 2, &
                    lenglc, ihelp)
      lenglc = size ( kmeshd )
      iwork(2) = lenglc
      call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
      call writ00 ( inum, numarr, namevc, kmeshd, leng, 2, &
                    lenglc, ihelp)
      lenglc = size ( kmeshe )
      iwork(2) = lenglc
      call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
      call writ00 ( inum, numarr, namevc, kmeshe, leng, 2, &
                    lenglc, ihelp)
      if ( kelmh>100000000 ) then

!     --- KMESH part h

         lenglc = size ( kmeshh )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshh, leng, 2, &
                       lenglc, ihelp)

!        --- Read next parts of kmesh part h and write to file 2

         lenglc = 2*ncurvs
         call writ00 ( inum, numarr, namevc, icurvsmsh, leng, 2, &
                       lenglc, ihelp)
         lenglc = size ( curvesmsh )*intlen
         call writ00 ( inum, numarr, namevc, curvesmsh, leng, 2, &
                       lenglc, ihelp)
         lenglc = size ( curvefacts )*intlen
         call writ00 ( inum, numarr, namevc, curvefacts, leng, 2, &
                       lenglc, ihelp)

      end if

!     --- KMESH parts i - s, iinput, rinput

      lenglc = size ( coor )*intlen
      iwork(2) = lenglc
      call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
      call writ00 ( inum, numarr, namevc, coor, leng, 2, &
                    lenglc, ihelp)

      if ( kelmj>0  ) then
         lenglc = size ( kmeshj )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshj, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmj>0  )

      if ( kelmk>0  ) then
         lenglc = size ( kmeshk )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshk, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmk>0  )

      if ( kelml/=0  ) then
         lenglc = size ( kmeshl )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshl, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelml>0  )

      if ( kelmm/=0  ) then
         lenglc = size ( kmeshm )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshm, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmm/=0  )

      if ( kelmn/=0  ) then
         lenglc = size ( kmeshn )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshn, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmn/=0  )

      if ( kelmo>0  ) then
         lenglc = size ( kmesho )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmesho, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmo>0  )

      if ( kelmq/=0  ) then
         lenglc = size ( kmeshq )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshq, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmq/=0  )

      if ( kelmr/=0  ) then
         lenglc = size ( kmeshr )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshr, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelmr/=0  )

      if ( kelms/=0  ) then
         lenglc = size ( kmeshs )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, kmeshs, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kelms/=0  )

      if ( kmiinput>0  ) then
         lenglc = size ( iinputkm )
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, iinputkm, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kmiinput>0  )

      if ( kmrinput>0  ) then
         lenglc = size ( rinputkm )*intlen
         iwork(2) = lenglc
         call writ00 ( inum, numarr, namevc, iwork, leng, 2, 2, ihelp)
         call writ00 ( inum, numarr, namevc, rinputkm, leng, 2, &
                       lenglc, ihelp)
      end if  ! ( kmrinput>0  )

!     --- Close writing, by writing nothing but using ifirst = 3 in writ00

      call writ00 ( inum, numarr, namevc, rinputkm, leng, 3, 0, &
                    ihelp)
      call erclos ( 'writb2' )

      end subroutine writb2
