      subroutine writfd ( index, iunder, iupper, iref, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-06-2007 Fortran 90
!        version  1.0  date 14-01-1995 Double precision variant of writfd
!
!   copyright (c) 1995-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   write array from core to SEPRAN direct access file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direct_access
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulecomio
      use sepmoduleerr
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, iunder, iupper, iref
      double precision array(*)

!     array     array in which the input is written depending on the
!               values of ichois and iref
!     index     record number from which iarray has to be written
!     iref      dataset reference number indicating the type of file
!               iref = 1, 2, 3, 4 or 17
!     iunder    first element in iarray to be written
!     iupper    last element in iarray to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jref, nrec, n, iho, i, iht, k, ios

!    i       General loop variable
!    iho     lower index of array to be written in present record
!    iht     upper index of array to be written in present record
!    ios     IOSTATus variable
!    jref    reference number
!    k       General loop variable
!    n       number of records to be written
!    nrec    record length of file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1801   Error in writing file
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( iref==2 ) then

!     --- iref = 2

         jref = iref2
         nrec = nrec2/intlen
         irec2 = index+(iupper-iunder)/nrec+1
         ifree2 = max(ifree2,irec2)

      else if ( iref==3 ) then

!     --- iref = 3

         jref = iref3
         nrec = nrec3/intlen
         irec3 = index+(iupper-iunder)/nrec+1
         ifree3 = max(ifree3,irec3)

      end if

      if ( nrec==0 ) then
         write(*,110) iref
110      format ( ///' ************************************'// &
                     ' fatal error in subroutine writfd '/ &
                     ' File with symbolic reference number',i2/ &
                     ' has not been opened, whereas you try to'/ &
                     ' write to this file'// &
                     ' Execution is terminated' // &
                     ' ************************************')
         call instop
      end if
      n = (iupper-iunder)/nrec+1
      iho = iunder

      if ( ioutp>=2 ) then

!     --- Extra writes to print all record numbers that are used

         write(irefwr,120) namsub(levsub), iref, nrec, index, index+n-1
120      format(' Writing to file in subroutine ',a/ &
                ' File with symbolic ref. number',i2,'  record length', &
                i5, '  records', i4, ' to ', i4 )

      end if

      do 200 i = 1,n
         iht = min(iupper,iho+nrec-1)
         write(jref, rec=index+i-1, iostat=ios) (array(k),k=iho,iht)
         if ( ios/=0 ) then
            call eropen ( 'writfd' )
            call errint ( iref, 1 )
            call errint ( ios, 2 )
            call errsub ( 1801, 2, 0, 0 )
            call erclos ( 'writfd' )
         end if
         iho = iht+1
200   continue

      end subroutine writfd
