      subroutine addintscal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add integer scalar name to the list of scalars
!
! **********************************************************************
!
!                       KEYWORDS
!
!     scalar
!     name
!     add
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3159   No  space for scalars left
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'addintscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from addintscal'
         write(irefwr,1) 'numintscals', numintscals
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numintscals = numintscals+1   ! new sequence number of scalar

!     --- Check available space

      if ( numintscals>maxintscals ) then

!     --- numintscals>maxintscals

         call errint ( numintscals, 1 )
         call errint ( maxintscals, 2 )
         call errsub ( 3159, 2, 0, 0 )
         go to 1000

      end if  ! ( numintscals>maxintscals )

      chscalars(numintscals+maxcls) = stringim(1:nchar)    ! name of scalar
      dimscals(numintscals+maxcls) = 1         ! dimension of scalar name
      ncharscals(numintscals+maxcls) = nchar   ! number of characters of scalar
                                               ! name
      realrd = numintscals+maxcls+100000000    ! sequence number of scalar

1000  call erclos ( 'addintscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numintscals', numintscals
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End addintscal'

      end if  ! ( debug )

      end
