      subroutine askprisca ( inpsep, istart, icode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 12-02-2015 New call to readexpress
!        version  2.1  date 06-05-2014 Extension with append
!        version  2.0  date 22-03-2014 Use local inpsep
!
!   copyright (c) 2001-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword print_scalar
!     and store in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart, icode

!     icode   i    Code in inpsep defining the type of action (here 7)
!     inpsep  o    The integer input read is stored in array INPSEP
!                  Array INPSEP is filled as follows from position istart+ &
!                  0: icode
!                  1: Scalar sequence number or number of scalars
!                  2: Position in text array
!                  3: Number of characters in text
!                  4: inamefile (0: no filename)
!                  5: appendmode
!                      0: not
!                      1: append
!                  6:9: - &
!                 10...9+3*numscalars contain 3 positions per
!                     scalar:
!                      1: type (1=integer, 2=real, 3=scalar)
!                      2: position in incons rlcons or scalars
!                      3: if>0, index of scalar used as array index
!                  If icode = 7 and inpsep(2) = -1001 array kprob
!                  is printed by prinkprob
!                  If icode = 7 and inpsep(2) = -1002 array kmesh
!                  is printed by prinkmesh
!                  inpsep(3) defines ichois in prinkprob/kmesh
!     istart i/o   At input first position of inpsep to be filled
!                  At output next position of inpsep to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itext, jstart, numvarread, number, ipos, itypescal, &
              ispeclsav, lastitemsav, istartsav
      logical ::  openfound, closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     ipos           Position in scalar array
!     ispeclsav      Help variable to store the original value of ispecl
!     istartsav      Help variable to save the value of istart
!     itext          Text sequence number
!     itypescal      Type of scalar:
!                    1:  integer
!                    2:  real
!                    3:  scalar
!     jstart         First position used at a part
!     lastitemsav    Help parameter to save the value of lastitem
!     number         Sequence number of scalar
!     numvarread     Number of scalars read
!     openfound      Indicates if an openings bracket is found before
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ASKPRIKMESH    Read the information following print kmesh and
!                    store in inpsep
!     ASKPRIKPROB    Read the information following print kprob and
!                    store in inpsep
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READEXPRESS    Interprete part of input as number or expression
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
!     askprisca requires input from the standard input file
!     The following input is recognised:
!       PRINT_SCALAR i, options (all in one line)
!           where options may have the following contents
!           s!, s1, ...,  text = '...'
!       This program only interprets the part options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1614   Character could not be interpreted
!    1868   Text not recognized
!    1879   Not a text found after the keyword
!    1993   No file name found
!    3187   More than 1 text found in print statement
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askprisca' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprisca'
         write(irefwr,1) 'icode, istart', icode, istart
         call prinin ( typeitems, nitems, 'typeitems' )
         istartsav = istart
  1      format ( a, 1x, (5i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode     ! fill icode in first position
      inpsep(3:9) = 0
      numvarread = 0
      itext = 0
      jstart = 9
      ispeclsav = ispecl

!     --- Read next item, first check if it is an expression

100   openfound = .false.
      lastitemsav = lastitem
      call readexpress ( openfound, closefound, .true., &
                         .false., .false., vecfound )
      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

      if ( ityprd/=19 ) then

!     --- Not an expression found, reread

         lastitem = lastitemsav
         ispecl = 19
         call asknxt

      end if  ! ( ityprd/=19 )

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         write(irefwr,2) 'realrd/1', realrd
      end if  ! ( debug )

110   if ( ityprd==2 .or. ityprd>=11 .and. ityprd<=14 ) then

!     --- ityprd = 2 or 11-14 integer or variable found

         if ( ityprd==2 .or. ityprd==11 ) then

!        --- ityprd = 2 or 3 integer or variable found

            itypescal = 3
            number = mod(nint(realrd),100000000)
            ipos = 0
            numvarread = numvarread+1
            if ( debug ) &
               write(irefwr,1) 'itypescal, number, numvarread', &
                                itypescal, number, numvarread

         else

!        --- ityprd = 12-14 array found

            itypescal = ityprd-11
            number = index_rdarr(1)
            ipos = 0
            numvarread = numvarread+1
            ispecl = 15
            call asknxt
            if ( ityprd==8 ) then

!           --- opening bracket found, read next item

               ispecl = 10
               call asknxt
               if ( ityprd==2 .or. ityprd==12 ) then

!              --- integer found, check if this value is allowed

                  if ( ityprd==2 ) then
                     number = number+nint(realrd)-1
                  else
                     number = number+incons(index_rdarr(1))-1
                  end if

               else if ( ityprd==14 ) then

!              --- index found of type variable

                  ipos = index_rdarr(1)

               else

!              --- character could not be interpreted

                  call errchr ( string(1:nchar), 1 )
                  call errsub ( 1614, 0, 0, 1 )
                  call errcrd

               end if

            else

!           --- No bracket found

               inpsep(jstart+1) = itypescal
               inpsep(jstart+2) = number
               inpsep(jstart+3) = ipos
               jstart = jstart+3
               go to 110

            end if

         end if

         inpsep(jstart+1) = itypescal
         inpsep(jstart+2) = number
         inpsep(jstart+3) = ipos
         jstart = jstart+3

         go to 100

      else if ( ityprd==4 ) then

!     --- String found, must be text followed by text

         if ( debug ) write(irefwr,*) 'string ', string(1:nchar)
         if ( string(1:nchar)=='FILE' .and. typeitems(lastitem+1)==13 ) &
            then

!        --- File = found, read filename

            call asknxt

            if ( debug ) write(irefwr,1) 'FILE: ityprd', ityprd
            if ( ityprd==5 ) then

!           --- file name found

               if ( debug ) then
                  write(irefwr,1) 'nextxt', nextxt
                  write(irefwr,*) 'string ', string(1:nchar)
               end if  ! ( debug )

               texts(nextxt) = string(1:nchar)
               inpsep(4) = nextxt  ! position of name in array texts
               nextxt = nextxt + 1

            else

!           --- Error no file name found

               call errsub ( 1993, 0, 0, 0 )
               call errcrd
               go to 1000

            end if  ! ( ityprd==5 )
            if ( lastitem<nitems ) go to 100
            go to 800

         else if ( string(1:nchar)=='APPEND' ) then

!        --- string is not APPEND

            inpsep(5) = 1
            if ( lastitem<nitems ) go to 100
            go to 800

         else if ( string(1:4)/='TEXT' ) then

!        --- string is not TEXT
!            Check if it is a scalar name

            ispecl = 10
            call readstrscals
            if ( debug ) then
               write(irefwr,1) 'nchar, ityprd/2', nchar, ityprd
               write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

            if ( ityprd==14 .or. ityprd==18 ) then

!           --- scalar name found

               numvarread = numvarread+1

               inpsep(jstart+1) = 3
               inpsep(jstart+2) = index_rdarr(1)+10000*index_rdarr(3)
               inpsep(jstart+3) = 0
               jstart = jstart+3

               if ( debug ) then
                  write(irefwr,1) 'realrd, numvarread, jstart', &
                                   nint(realrd), numvarread, jstart
                  write(irefwr,1) 'index_rdarr(1)', index_rdarr(1)
               end if  ! ( debug )

               go to 100

            else if ( string(1:nchar)=='KPROB' ) then

!           --- Special text print kprob found

               call askprikprob ( inpsep )
               if ( lastitem<nitems ) go to 100
               go to 900

            else if ( string(1:nchar)=='KMESH' ) then

!           --- Special text print kmesh found

               call askprikmesh ( inpsep(0) )
               if ( lastitem<nitems ) go to 100
               go to 900

            else

               call errchr ( 'TEXT', 1 )
               call errsub ( 1868, 0, 0, 1 )
               call errcrd
               go to 1000

            end if  ! ( ityprd==11 )

         end if

         if ( itext>0 ) then

!        --- Error: more than one text found

            call errsub ( 3187, 0, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( itext>0 )

         call asknxt

         if ( debug ) write(irefwr,1) 'ityprd/3', ityprd

         if ( ityprd/=5 ) then

!        --- Not a text found

            call errchr ( 'TEXT', 1 )
            call errsub ( 1879, 0, 0, 1 )
            call errcrd
            go to 1000

         end if
         itext = nextxt
         texts(nextxt) = string(1:nchar)
         nextxt = nextxt + 1
         if ( lastitem<nitems ) go to 100

      else if ( ityprd==19 ) then

!     --- ityprd = 19, hence expression found

         inpsep(jstart+1) = 19
         inpsep(jstart+2) = nint(realrd)
         inpsep(jstart+3) = 0
         jstart = jstart+3
         numvarread = numvarread+1
         if ( lastitem<nitems ) go to 100

      else if ( ityprd/=0 .and. ityprd/=1 ) then

!     --- character could not be interpreted

         call errchr ( string(1:nchar), 1 )
         call errsub ( 1614, 0, 0, 1 )
         call errcrd

      end if

800   inpsep(1) = numvarread
      inpsep(2) = itext
      if ( itext>0 ) inpsep(3) = nchar

900   istart = istart+10 + 3*numvarread  ! raise istart

1000  call erclos ( 'askprisca' )
      ispecl = ispeclsav
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), istart-istartsav, 'inpsep' )
         write(irefwr,1) 'istart, numvarread', istart, numvarread
         write(irefwr,*) 'End askprisca'

      end if  ! ( debug )

      end
