      subroutine askprizerolev ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 16-04-2014 Extension with append
!        version  2.0  date 22-03-2014 Use local inpsep
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword print zero_levelset
!     and store in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 1    69 to indicate print zero_levelset
!                              2    levelset sequence number
!                              3    position of file name in array texts
!                              4    nchar number of characters of file name
!                              5    appendmode
!                                   0: not
!                                   1: append
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ilev

!     ilev           Sequence number of level being considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askprizerolev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprizerolev'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      code(1) = 'ZERO_LEVELSET'
      code(2) = 'FILE'
      code(3) = 'APPEND'
      ncode(1:2) = 4
      ncode(2) = 3
      ncodmx(1) = 13
      ncodmx(2) = 4
      ncodmx(3) = 6


      lastitem = lastitem-1  ! reread zero_levelset
      itype(1) = 3  ! integer or no integer
      itype(2) = 6  ! text after file
      table = -1d0
      table(3) = 0d0  ! append mode
      nmain = 0
      tabmin = 1d0
      tabmax = 1d0
      errorcode = 0
      iretur = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nncode, 'table' )
      end if  ! ( debug )

!     --- sequence number of level set

      if ( table(1)>0 ) then
         ilev = nint(table(1))
      else
         ilev = 1
      end if  ! ( table(1)>0 )

!     --- File name

      call intxstoretext ( inpsep(2), table(2), tabmax(2), strout )

      inpsep(0) = 69  ! print zero level set
      inpsep(1) = ilev  ! sequence number of level set
      inpsep(4) = nint(table(3))  ! append mode

1000  call erclos ( 'askprizerolev' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), 4, 'inpsep' )
         write(irefwr,*) 'End askprizerolev'

      end if  ! ( debug )

      istart = istart+5  ! raise istart

      end
