      subroutine asksca ( iscal, inpsep, rinsep, numscl, &
                          istrin, leninpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.12 date 12-02-2015 New call to readexpress
!        version  4.11 date 29-04-2014 Extension with boundary_sum
!        version  4.10 date 17-04-2014 Extension with intersection_integral
!
!   copyright (c) 1994-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword scalar i = and store in inpsep
!     and rinsep
!     If a new option is made also change READSCALRHS
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iscal, istrin, numscl, leninpsep, istart
      double precision rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         Array INPSEP is filled as follows:
!                          0:  leninpsep Length of this part of the array
!                          1: ISEQNR     Sequence number scalar
!                          2: ITYPE      Type of construction
!                                        Possible values:
!                                        1:  value
!                                        2:  func
!                                        3:  min
!                                        4:  max
!                                        5:  integer constant
!                                        6:  real constant
!                                        7:  variable
!                                        8:  time
!                                       12:  expression
!                          3: IPOS1      Position of value in rinsep (>0) or
!                                        parameter IFUNC in
!                                        FUNCSCAL ( IFUNC, SCALARS)
!                                        In case of min/max: number of
!                                        parameters to be compared
!                                        In case of itype = 5 to 7 starting
!                                        position of quantity in incons, &
!                                        rlcons or scalars
!                          4: IPOS2      In case of min/max: information of
!                                        first parameter:
!                                        >0: value in rinsep;
!                                        <0 -Scalar sequence number
!                                        -10000-length expression according to
!                                        the rules in readexpress
!                                        length = length of expression
!                                        Next positions integers for expression
!                                        In case of itype = 5 to 7 shift in
!                                        position because an array has been
!                                        found
!                                        If the shift itself is variable
!                                        ipos2 = -1 and the variable
!                                        sequence number is stored
!                                        in position 5
!                          5...          Rest of parameters in the same way
!                                        as ipos1
!                         In case si instead of the former compute_scalar is
!                         used, see askcos
!     iscal          i    Scalar sequence number
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Next free position in RINSEP
!     leninpsep      i    Length of array inpsep
!     numscl        i/o   Number of scalars read
!     rinsep         o    Real input array, the value is store in rinsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncodelc
      parameter ( nncodelc=10 )
      character (len=21) codelc(nncodelc)
      integer ncodelc(nncodelc), ncodmxlc(nncodelc), lcode, &
              istartsav, lastitemsav, icodelc
      logical :: openfound, closefound, notfound, vectorfound, &
                 vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     codelc         Table with keywords
!     icodelc        Local value of icode
!     istartsav      Help variable to save the value of istart
!     lastitemsav    Help parameter to save the value of lastitem
!     lcode          return code produced by intx00
!     ncodelc        Table containing the number of significant characters
!                    in the keyword table codelc
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc
!     nncodelc       Number of entries in codelc
!     notfound       if true no expression found in right-hand side
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     vecfound       If true the expression contains vectors
!     vectorfound    If true the right-hand side is a vector expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKCOS         Interpret options after compute_scalar
!     ASKNXT         Read next item from the standard input file
!     ASKSCA12       Interpret the part following scalar i = ... in case
!                    a constant or an array has been found
!     ASKSCA4        Interpret the part following scalar i = ... in case
!                    a keyword has been found
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READEXPRESS    Interprete part of input as number or expression
!     READSCBOUNINT  Read keywords after the keyword BOUNDARY_INTEGRAL
!     READSCINTEGRAL Read keywords after the keyword INTEGRAL
!     READSCINTERSIN Read keywords after the keyword intersection_integral
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADCHKRHS  Check if right-hand side is an expression containing
!                    only existing scalars and vectors, operators and numbers
!                    and no other strings
! **********************************************************************
!
!                       I/O
!
!     asksca requires input from the standard input file
!     The following input is recognized:
!     STRUCTURE    (has been read already be readal)
!         start of input
!      the next records are interpreted in the sequence they are read
!      the program is carried out in the same sequence
!       SCALAR i = definition (all in one line)
!           where definition may have one the following forms
!               value (real or integer value)
!               FUNC = k
!               MIN ( F1, F2 )
!               MAX ( F1, F2 ), &
!                  where F1 and F2 may be either a number or Sj, with
!                  j a scalar sequence number
!               INTEGRAL, options, see READSCINTEGRAL
!               BOUN_INTEGRAL, options, see READSCBOUNINT
!          also the options following compute_scalar are allowed
!       This program only interprets the part definition
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1487   Text not recognzied
!    1868   Text not recognized
!    3172   Right-hand side of expression is vector, the left-hand side is a scalar
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read next item
!     if ( item = number ) then
!        Fill rinsep and inpsep
!        Raise istrin
!     else if ( item = FUNC ) then
!        Fill inpsep
!     else if ( item = VALUE ) then
!        Read number
!        Fill rinsep and inpsep
!        Raise istrin
!     else if ( item = MIN or MAX ) then
!        Read rest of information
!        Fill rinsep and inpsep
!        Raise istrin if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc /'MIN_MAX', 'VALUE', 'FUNC', 'MIN', 'MAX', 'INTEGRAL', &
                   'BOUNDARY_INTEGRAL', 'GET_TIME', &
                   'INTERSECTION_INTEGRAL', 'BOUNDARY_SUM'  /
      data ncodelc  / 5, 5, 3, 3, 3,   6,12, 5, 6, 12 /
      data ncodmxlc / 7, 5, 4, 3, 3,   8,17, 8,21, 12 /

!     codelc         Table with main keywords
!                    1:  MIN_MAX
!                    2:  VALUE
!                    3:  FUNC
!                    4:  MIN
!                    5:  MAX
!                    6:  INTEGRAL
!                    7:  BOUNDARY_INTEGRAL
!                    8:  GET_TIME
!                    9:  INTERSECTION_INTEGRAL
!                   10:  BOUNDARY_SUM
! ======================================================================
!
      call eropen ( 'asksca' )
      debug = .false.  .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from asksca'
  1      format ( a, 1x, (10i12) )
         write(irefwr,1) 'iscal, istrin, maxcls, numscl, istart', &
                          iscal, istrin, maxcls, numscl, istart
         write(irefwr,1) 'lastitem, typeitems(lastitem)', &
                          lastitem, typeitems(lastitem)
         call prinin ( typeitems, nitems, 'typeitems' )

      end if
      if ( ierror/=0 ) go to 1000

      icodelc = 12  ! Default code for si = ...
      openfound = .false.
      leninpsep = 0

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )
      inpsep(istart+2) = iscal

!     --- Skip = sign if present

      lastitem = lastitem+1
      if ( typeitems(lastitem)==13 ) lastitem = lastitem+1
      if ( debug ) &
         write(irefwr,1) 'lastitem, typeitems(lastitem)/2', &
                          lastitem, typeitems(lastitem)

      if ( typeitems(lastitem)==5 ) then

!     --- ( found, raise lastitem and set openfound to true

         lastitem = lastitem+1
         openfound = .true.

      end if  ! ( typeitems(lastitem)==5 )

      if ( typeitems(lastitem)==8 .or. typeitems(lastitem)==1 .or. &
           typeitems(lastitem)==2 .or. typeitems(lastitem)==11 .or. &
           typeitems(lastitem)==17 .or. typeitems(lastitem)==5 ) then

!     --- One of the following characters found:
!         number, -, +, ( or {
!         This is considered to be an expression

         ityprd = 100

!        --- decrease lastitem, since it is raised in readexpress
!            except when { is found

         if ( typeitems(lastitem)/=17 ) lastitem = lastitem-1

      else

!     --- Read next item

         ispecl = 10
         iexpression = 1   ! indicate that asknxt must return in case of
                           ! an expression between brackets
         lastitem = lastitem-1  ! decrease lastitem
         call asknxt

      end if  ! ( typeitems(lastitem)==8 ... )
      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
      if ( ityprd==2 .or. ityprd==3 ) then

!     --- number found

         rinsep(istrin) = realrd
         inpsep(istart+3) = 1
         inpsep(istart+4) = istrin
         istrin = istrin + 1
         ispecl = 0
         leninpsep = 3

      else if ( ityprd>=12 .and. ityprd<=14 ) then

!     --- constant or array found, check which of the two

         call asksca12 ( inpsep(istart+2), leninpsep )

      else if ( ityprd==4 ) then

!     --- keyword found, interpret

         ierpri = 0
         ispecl = 0
         ierpri = 0

         lcode = sepsubkeyw()
         if ( debug ) write(irefwr,1) 'lcode', lcode

         if ( lcode==6 ) then

!        --- Special case, integral found

            icodelc = 11
            icode = icodelc
            call readscintegral ( istart, inpsep(istart), iscal )
            leninpsep = -1

         else if ( lcode==7 .or. lcode==10 ) then

!        --- Special case, boun_integral found

            icode = 16
            if ( lcode==10 ) icode = 17
            icodelc = icode
            call readscbounint ( istart, inpsep(istart), iscal )
            leninpsep = -1

         else if ( lcode==9 ) then

!        --- Special case, intersection_integral found

            icodelc = 110
            icode = icodelc
            call readscintersin ( istart, inpsep(istart), rinsep, &
                                  istrin, iscal )
            leninpsep = -1

         else if ( lcode>1 ) then

!        --- lcode>1, standard case

            call asksca4 ( inpsep(istart+2), rinsep, istrin, &
                           leninpsep, lcode-1 )

         else

!        --- lcode<=1, same as compute_scalar

            icodelc = 6
            iexpression = 0  ! reset iexpression to standard value
            ispecl = 0
            lastitem = lastitem-1
            iretur = -10000     ! return in case no keyword is found

            if ( debug ) then
               write(irefwr,*) 'before askcos'
               write(irefwr,1) 'istart, leninpsep', istart, leninpsep
               write(irefwr,1) 'icodelc, lcode, lastitem', &
                                icodelc, lcode, lastitem
            end if  ! ( debug )

            istartsav = istart
            call askcos ( iscal, inpsep(istart+1), numscl, istart )
            istart = istartsav

            if ( debug ) then
               write(irefwr,*) 'after askcos'
               write(irefwr,1) 'istart, iretur', istart, iretur
            end if  ! ( debug )

            if ( iretur==-10001 ) then

!           --- iretur = -10001, possibly an expression found

               inpsep(istart+3) = 8  ! to indicate expression
               closefound = .false.
               icodelc = 12
               if ( debug ) write(irefwr,*) 'before readexpress'

!              --- Check if right-hand side is vector

               lastitemsav = lastitem  ! save lastitem
               call sepreadchkrhs ( notfound, vectorfound )
               lastitem = lastitemsav  ! reset lastitem
               if ( debug ) &
                  write(irefwr,*) 'notfound, vectorfound', &
                                   notfound, vectorfound

               if ( vectorfound ) then

!              --- Error right-hand side of expression is vector, whereas
!                  left-hand side is scalar

                  call errsub ( 3172, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if  ! ( vectorfound )

!              --- Read expression

               call readexpress ( openfound, closefound, .true., &
                                  .false., .false., vecfound )

               if ( debug ) write(irefwr,1) 'ityprd', ityprd

               if ( ityprd==3 ) then

!              --- real found

                  rinsep(istrin) = realrd
                  inpsep(istart+3) = 1
                  inpsep(istart+4) = istrin
                  istrin = istrin + 1

               else if ( ityprd==20 ) then

!              --- Nothing found, give error

                  call errsub ( 1487, 0, 0, 0 )
                  call errcrd
                  go to 1000

               else

!              --- Other options

                  inpsep(istart+4) = nint(realrd)

               end if  ! ( ityprd==3 )
               leninpsep = leninpsep+3

            else

!           --- Standard case

               leninpsep = 20
               inpsep(istart+14) = 0

            end if  ! ( iretur==-10001 )

         end if  ! ( lcode>0 )

      else if ( ityprd==100 ) then

!     --- expression found, special case

         inpsep(istart+3) = 8
         closefound = .false.
         if ( debug ) &
            write(irefwr,1) 'before readexpress: lastitem', lastitem
         call readexpress ( openfound, closefound, .true., .false., &
                            .false., vecfound )

         if ( ityprd==3 ) then

!        --- real found

            rinsep(istrin) = realrd
            inpsep(istart+3) = 1
            inpsep(istart+4) = istrin
            istrin = istrin + 1
            leninpsep = 3

         else

!        --- expression found

            inpsep(istart+4) = nint(realrd)
            leninpsep = 3

         end if  ! ( ityprd==3 )

      else

!     --- Text not recognized

         call errchr ( string(1:nchar), 1 )
         call errsub ( 1868, 0, 0, 1 )
         call errcrd

      end if

      if ( iscal>0 .and. iscal<maxcls )  iscalars(iscal) = 1
      numscl = min ( maxcls, max(iscal,numscl) )

      if ( icodelc==12 ) then

!     --- icodelc = 12: s% = func or value

         iexpression = 0  ! reset iexpression to standard value
         inpsep(istart+1) = leninpsep
         leninpsep = leninpsep+1
         ispecl = 0

      end if  ! ( icodelc==12 )

!     --- Fill icodelc in inpsep
!         Raise istart except in case of integral

      icode = icodelc
      inpsep(istart) = icode

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode', icode
         write(irefwr,1) 'iscal, istrin, numscl, istart/end', &
                          iscal, istrin, numscl, istart
         if ( leninpsep>0 ) then
            call prininlarge ( inpsep(istart), leninpsep+1, 'inpsep' )
            call prinrl ( rinsep, istrin-1, 'rinsep' )
         end if  ! ( leninpsep>0 )
         write(irefwr,*) 'End asksca'

      end if

      istart = istart + leninpsep+1
      if ( debug ) write(irefwr,1) 'istart', istart
      call erclos ( 'asksca' )

      end subroutine asksca
