      subroutine askvec01 ( iretur, inpsep, tabout, rinsep, istrin, &
                            number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine corresponding to askvec dealing with iretur<-10, &
!     nseqkeyw==1 and iseqkeyw(1)<0
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iretur, inpsep(*), istrin, number
      double precision tabout(*), rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     iretur         i    Return value of subroutine INTX02
!                         See subroutine INTX02
!     istrin        i/o   Starting address of next positions in rinsep
!                         used (1) or not (0)
!     number         i    Vector sequence number
!     rinsep         o    Contains real input at output. The storage depends on
!                         the storage of inpsep
!     tabout         i    Contains the values found at output of intx02
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, i

!     i              Counting variable
!     n              Number of occurences of keyword found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2919   More than one number found following vi = ...
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvec01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvec01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = -iretur-10
      if ( n>1 ) then

!     --- More numbers than one found following vi
!         Not possible

         call errint ( n, 1 )
         do i = 1, n
            call erreal ( tabout(i), i )
         end do  ! i = 1, n
         call errsub ( 2919, 1, n, 0 )
         call errcrd
         go to 1000

      end if  ! ( n>1 )
      inpsep(3) = abs(number)    ! sequence number of vector
      inpsep(4) = 102            ! vector is constant
      inpsep(13) = istrin        ! position in rinsep
      rinsep(istrin) = tabout(1) ! value of constant
      istrin = istrin+1          ! raise istrin

1000  call erclos ( 'askvec01' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvec01'

      end if  ! ( debug )

      end
