      subroutine askvecgrad ( inpsep, istart, nprob, number, nelgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-01-2015 Extra parameter nelgrp
!        version  1.1  date 05-11-2014 Extension with stress
!        version  1.0  date 03-10-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret input following vi = grad or vi = stress
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     derivative
!     gradient
!     pressure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nprob, number, nelgrp
      integer, intent(inout) :: istart
      integer, intent(out) :: inpsep(0:*)

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 10
!                         Array INPSEP is filled as follows:
!                         0: Type of function to be called
!                             8:  derivatives
!                         1-13+m: see readscderiv
!     istart        i/o   Next free position in inpsep
!     nelgrp         i    Number of element groups
!     nprob          i    Number of various problems
!     number         i    Vector sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivec, m, istartnum

!                    they are not
!     istartnum      Starting address of a series of numbers in INPSEP
!     ivec           Sequence number of vector
!     m              number of values stored in tabout for keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READSCSKIPEL   Interpret information concerning skip element groups
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecgrad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecgrad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(3))
      m = nint(tabmax(3))
      ivec = -1
      if ( m>0 ) ivec = nint(tabout(iposut))   ! input vector

      if ( ivec==-1 ) ivec = nprob+1   !     --- Vector not given

!     --- At this moment we only allow computation through derivatives

      istartnum = 15
      inpsep(0) = 8   ! icode for derivatives
      inpsep(1:2) = 0
      inpsep(3) = number            ! output vector
      inpsep(4) = -10  ! Use derivatives
      inpsep(5) = 0
      if ( table(41)>0d0 ) then
         inpsep(6) = 2                 ! icheld for gradient
      else if ( table(49)>0d0 ) then
         inpsep(6) = 7                 ! icheld for pressure
         inpsep(0) = 113               ! icode for compute pressure
      else if ( table(50)>0d0 ) then
         inpsep(6) = 5                 ! icheld for curl
      else
         inpsep(6) = 6                 ! icheld for stress and flux
      end if  ! ( table(41)>0d0 )
      inpsep(7) = ivec              ! input vector
      inpsep(8:14) = 0              ! type of output vector

!     --- information about skip elements

      call readscskipel ( table(22), tabout, inpsep, istartnum, &
                          14, tabmax(22), nelgrp )

      if ( number>0 .and. number<=maxvec ) ivectors(number) = 1
      istart = istart + istartnum  ! Change also readscderiv

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecgrad'

      end if  ! ( debug )
      call erclos ( 'askvecgrad' )

      end
