      subroutine askveclincom ( code2, ncod2m, table2, iscal1, iretur, &
                                tabout, inpsep, rinsep, istrin, &
                                nseqkeyw, iseqkeyw, isignvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 18-01-2012 extra debug
!        version  1.1  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store coefficients for subroutine askvec
!     only for LIN_COMBINATION
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iscal1, ncod2m(*), iretur, inpsep(*), istrin, nseqkeyw, &
              iseqkeyw(*), isignvec(*)
      double precision table2(*), tabout(*), rinsep(*)
      character (len=*) code2(*)

!     code2          i    Table with keywords
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     iretur         i    Return value of subroutine INTX02
!                         See subroutine INTX02
!     iscal1         i    Sequence number of first scalar
!     iseqkeyw       i    Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                         iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                         -isekeyw(i) refers to position in
!                         tabout
!     isignvec       i    Contains the signs of the two vectors in case of
!                         linear combination
!     istrin        i/o   Starting address of next positions in rinsep
!                         used (1) or not (0)
!     ncod2m         i    Table containing the maximal number of  characters in
!                         the keyword table code2
!     nseqkeyw       i    Number of entries in iseqkeyw
!     rinsep         o    Contains real input at output. The storage depends on
!                         the storage of inpsep
!     table2         i    Contains values of items read corresponding to code2
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, i, icount
      double precision coef1, coef2

!     coef1          First coefficient read
!     coef2          Second coefficient read
!     i              Counting variable
!     icount         Counter
!     istart         Starting address in tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     589   Both coefficients in a linear combination of 2 vectors are zero
!    2378   Stand alone numbers found, in combination with coefficients.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askveclincom' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askveclincom'
         call prinin ( isignvec, 2, 'isignvec')
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = nint(table2(20))
      if ( debug ) then
         write(irefwr,*) 'linear combination'
         write(irefwr,1) 'istart', istart
      end if  ! ( debug )
      if ( istart>=1 ) then

!     --- istart>=1, i.e. coef i found

         if ( iscal1>0 ) call errkeywexcl ( code2, ncod2m, 8, 20 )
         if ( iretur<-10 ) then

!        --- Numbers found in combination with coef

            call errwar ( 2378, 0, 0, 0 )

         end if
         coef1 = tabout(istart)
         coef2 = tabout(istart+1)
         if ( debug ) then
            write(irefwr,2) 'coef1, coef2', coef1, coef2
         end if  ! ( debug )

!        --- If coef1 not given, set coef1 equal to 0

         if ( coef1<-1d29 ) coef1 = 0d0

!        --- If coef2 not given, set coef2 equal to 0

         if ( coef2<-1d29 ) coef2 = 0d0
         if ( abs(coef1)<=0d0 .and. abs(coef2)<=0d0 ) then

!        --- Both coefficients are zero

            call errsub ( 589, 0, 0, 0 )

         end if

         inpsep(13) = istrin
         rinsep(istrin) = coef1
         rinsep(istrin+1) = coef2
         istrin = istrin + 2

      else

!     --- Numbers found, instead of coef or scalar

         if ( debug ) &
            write(irefwr,1) 'numbers found, iretur, iscal1', &
                             iretur, iscal1
         if ( iretur<-10 .and. iscal1>0 ) then

!        --- The combination coef i and numbers are not allowed

            call errwar ( 2378, 0, 0, 0 )

         else if ( iscal1==0 ) then

!        --- Only numbers found, try to detect these numbers and couple
!            them to vectors by checking array iseqkeyw
!            As long as there are keywords left do

            i = 1
            icount = 0
            inpsep(13) = istrin
            rinsep(istrin) = 0d0
            rinsep(istrin+1) = 0d0
            do while ( i<=nseqkeyw .and. icount<=1 )

!           --- Still keywords available

               if ( debug ) &
                  write(irefwr,1) 'i, iseqkeyw, nseqkeyw', &
                                   i, iseqkeyw(i), nseqkeyw
               if ( iseqkeyw(i)<0 ) then

!              --- keyword is number, check if next item is vector

                  if ( debug ) &
                     write(irefwr,1) 'iseqkeyw(i+1)', iseqkeyw(i+1)

                  rinsep(istrin+icount) = tabout(-iseqkeyw(i))
                  if ( isignvec(icount+1)==-1 ) &
                     rinsep(istrin+icount) =-rinsep(istrin+icount)
                  if ( debug ) then
                     write(irefwr,1) 'istrin, icount', &
                                      istrin, icount
                     write(irefwr,2) 'rinsep(istrin+icount)', &
                                      rinsep(istrin+icount)
                  end if  ! ( debug )
                  icount = icount+1

                  if ( i>=nseqkeyw .or. iseqkeyw(i+1)/=3 ) then

!                 --- Keyword is not vector

                     if ( i==1 ) then

!                    --- i = 1, hence first vector does not exist

                        inpsep(9) = 0  ! no vector

                     else

!                    --- i = 3, hence second vector does not exist

                        inpsep(10) = -1  ! no vector

                     end if  ! ( i==1 )

                  end if  ! ( i>=nseqkeyw .or. iseqkeyw(i+1)/=3 )

                  i = i+1

               else if ( iseqkeyw(i)==3 ) then

!              --- keyword is vector, coefficient must be 1

                  rinsep(istrin+icount) = isignvec(icount+1)
                  icount = icount+1

               end if  ! ( iseqkeyw(i)==-2 )
               i = i+1

            end do  ! while ( i<nseqkeyw )
            istrin = istrin + 2

         end if  ! ( n>0 )

      end if  ! ( istart>=1 )

1000  call erclos ( 'askveclincom' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askveclincom'

      end if  ! ( debug )

      end
