      subroutine askvecsign ( inpsep, found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-05-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options of the form v = sign(V1) * V2
!     and store in inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      logical found

!     found         i/o   Indicates if one of the options following v = &
!                         has already been found
!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numitem

!     numitem        Sequence number of item in input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1903   From the possibilities: PHASE, MODULUS and LENGTH at least two are
!           found in the record
!    3075   Item read is not sign
!    3076   v = sign not followed by (
!    3077   u = sign(v not followed by )
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecsign' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecsign'
         write(irefwr,1) 'nitems', nitems
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( found ) call errsub ( 1903, 0, 0, 0 )
      if ( ierror/=0 ) go to 1000

      inpsep(4) = 107  ! Compute sign
      inpsep(7) = 1    ! ispec, corresponds to compute sign

      numitem = 2      ! number of items before sign
      if ( typeitems(2)==13 ) numitem = numitem+1
      if ( typeitems(numitem)==3 ) then

!     --- this must be sign

         numitem = numitem+1

      else

!     --- this can not be correct

         call errint ( numitem, 1 )
         call errsub ( 3075, 1, 0, 0 )
         go to 1000

      end if  ! ( typeitems(numitem)==3 )

      if ( typeitems(numitem)/=5 ) then

!     --- sign not followed by (

         call errint ( numitem, 1 )
         call errsub ( 3076, 1, 0, 0 )
         go to 1000

      end if  ! ( typeitems(numitem)/=5 ) then
      numitem = numitem+2

      if ( typeitems(numitem)/=6 ) then

!     --- sign(V not followed by )

         call errint ( numitem, 1 )
         call errsub ( 3077, 1, 0, 0 )
         go to 1000

      end if  ! ( typeitems(numitem)/=6 ) then
      numitem = numitem+1

      if ( numitem<nitems+1 ) then

!     --- Extra items read, this must be * Vector

         if ( typeitems(numitem)/=7 ) then

!        --- sign(V) not followed by * &

            call errint ( numitem, 1 )
            call errsub ( 3077, 1, 0, 0 )
            go to 1000

         end if  ! ( typeitems(numitem)/=7 ) then

      end if  ! ( numitem<nitems )

      found = .true.

1000  call erclos ( 'askvecsign' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecsign'

      end if  ! ( debug )

      end
