      subroutine askvecstvec ( inpsep, isignvec, number, numveclc, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 18-01-2012 Extension with x_coor
!        version  1.2  date 12-09-2011 Extension with coor
!        version  1.1  date 26-05-2011 Remove warning
!
!   copyright (c) 2010-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store vector numbers for subroutine askvec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), isignvec(*), number, numveclc
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     isignvec       o    Contains the signs of the two vectors in case of
!                         linear combination
!     number         i    Vector sequence number
!     numveclc        i/o   Number of vectors read
!     rinsep         o    Contains real input at output. The storage depends on
!                         the storage of inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, istart, iseq1, iseq2

!     iseq1          Sequence number of first input vector
!     iseq2          Sequence number of second input vector
!     istart         Starting address in tabout
!     n              Number of occurences of keyword found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1901   No (input) vector number found in the record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecstvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecstvec'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = nint(tabmax(3))
      istart = nint(table(3))
      isignvec(1) = 0
      isignvec(2) = 0
      if ( debug ) write(irefwr,1) 'n, istart', n, istart

      if ( n>=1 ) then

!     --- n>=1, at least one vector number found

         iseq1 = nint(tabout(istart))
         if ( debug ) write(irefwr,1) 'iseq1', iseq1

         if ( iseq1<0 ) then

!        --- Sequence number provided with minus sign

            isignvec(1) = -1
            iseq1 = -iseq1

         else

!        --- Sequence number not provided with minus sign

            isignvec(1) = 1

         end if

      else

         if ( inpsep(4)<=27 ) then
            iseq1 = abs(number)  ! standard case
         else
            call errsub ( 1901, 0, 0, 0 )
         end if

      end if

      if ( n>=2 ) then

!     --- n>=2, at least two vectors

         iseq2  = nint(tabout(istart+1))
         if ( debug ) write(irefwr,1) 'iseq2', iseq2

         if ( iseq2<0 ) then

!        --- Sequence number provided with minus sign

            isignvec(2) = -1
            iseq2 = -iseq2

         else

!        --- Sequence number not provided with minus sign

            isignvec(2) = 1

         end if

      else

!     --- Only one vector

         if ( inpsep(4)==107 ) then

!        --- sign(v) found

            iseq2 = 0

         else

!        --- standard case

            iseq2 = iseq1

         end if  ! ( inpsep(4)==107 )

      end if
      if ( debug ) then
         write(irefwr,1) 'iseq1, iseq2', iseq1, iseq2
         call prinin ( isignvec, 2, 'isignvec' )
      end if  ! ( debug )

      if ( number<=-1000 ) then

!     --- number = -1000 (hence coor)

         inpsep(3) = number

      else

!     --- standard case

         inpsep(3) = abs(number)

      end if  ! ( number==-1000 )
      inpsep(9) = iseq1
      inpsep(10) = iseq2
      if ( inpsep(9)<=-1000 ) inpsep(9) = maxvc1-inpsep(9)-1000
      if ( inpsep(10)<=-1000 ) inpsep(10) = maxvc1-inpsep(10)-1000
      if ( inpsep(9)>=1000 ) inpsep(9) = maxvc1+inpsep(9)-999
      if ( inpsep(10)>=1000 ) inpsep(10) = maxvc1+inpsep(10)-999
      numveclc = min ( maxvec, max(numveclc,number) )
      if ( number>0 .and. number<=maxvec ) ivectors(number) = 1
      if ( inpsep(4)==29 .or. inpsep(4)==30 ) rinsep(1) = table(5)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 10, 'inpsep' )
         call prinin ( isignvec, 2, 'isignvec' )
         write(irefwr,1) 'numveclc, number', numveclc, number
         write(irefwr,*) 'End askvecstvec'

      end if  ! ( debug )
      call erclos ( 'askvecstvec' )

      end
