      subroutine askwrt ( inpsep, number )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.2    date 21-01-1999 Include commons
!        version 1.1    date 16-03-1997 Set ivectors for reading
!        version 1.0    date 23-06-1995
!
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword write_vector i or read_vector i
!     and store in inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!   read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulemach
      use sepmoduleread1
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), number

!     inpsep  o    The integer input read is stored in array INPSEP
!                  Array INPSEP is filled as follows:
!                  1: Vector sequence number
!                  2: Sequence number at backing storage file
!     number  i    Scalar sequence number
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: skiperror

!     skiperror      If true errors must be skipped, since this is a
!                    parallel computation at a node with number>1 (slave)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     askwrt requires input from the standard input file
!
!     The following input is recognised:
!
!       WRITE_VECTOR i, options (all in one line) (or READ_VECTOR ...)
!           where options may have the following contents
!           sequence_number = i
!           save_administration
!
!       This program only interprets the part options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1580   Backing storage file has not been opened
!    1581   No arrays have been written to the backing storage file
!    1886   Vector has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      call eropen ( 'askwrt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askwrt'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(1) = 0

      skiperror = parallel .and. inodenr>1  ! slave

      code(1) = 'SEQUENCE_NUMBER'
      code(2) = 'SAVE_ADMINISTRATION'
      itype(1) = 0
      itype(2) = 2
      ncode(1) = 3
      ncode(2) = 8
      ncodmx(1) = 15
      ncodmx(2) = 19
      tabmin(1) = 1d0
      tabmin(2) = 0d0
      tabmax(1) = 1d3
      tabmax(2) = 0d0
      if ( inew.le.0 .and. .not. skiperror ) then

!     --- inew < 1, hence file has not been opened

         call errint ( iref2, 1 )
         call errchr ( namef2, 1 )
         call errsub ( 1580, 1, 0, 1 )

      end if
      if ( icode.eq.17 ) then

!     --- write_vector

         if ( ivectors(number).eq.0 ) then

!        --- vector does not exist

            call errint ( number, 1 )
            call errsub ( 1886, 1, 0, 0 )

         end if

      else

!     --- read_vector

         if ( inew.le.1 .and. .not. skiperror ) then

!        --- inew < 2, hence no arrays have been written to file 2

            call errint ( iref2, 1 )
            call errchr ( namef2, 1 )
            call errsub ( 1581, 1, 0, 1 )

         end if
         tabmax(1) = inew-1
         ivectors(number) = 1

      end if

      table(1) = -1d0
      table(2) =  0d0
      iretur = 0
      call sepreadsetkeyw

      inpsep(1) = number
      inpsep(2) = nint(table(1))
      inpsep(3) = nint(table(2))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askwrt'

      end if  ! ( debug )
      call erclos ( 'askwrt' )

      end subroutine askwrt
