      subroutine ercompvector1 ( iinput, iseqstress, iseqnormstress, &
                                 idim, maxsols )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 03-05-2010 Fortran 90
!        version  1.0  date 02-08-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract parameters from array iinput and check them
!     Help subroutine for ercompvector
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     stress
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqstress, iseqnormstress, idim, iinput(*), maxsols

!     idim           o    dimension of the space
!     iinput         i    integer input array that must have been as follows:
!                         Pos. 1 last entry that has been filled by the user
!                                must be at least 5
!                              2 iseqstress, sequence number of stress vector
!                                in isol
!                              3 iseqnormstress, sequence number of
!                                normal stress vectorin isol
!                              4 idim, dimension of space (at present 2)
!     iseqnormstress o    sequence number of normal stress vectorin isol
!     iseqstress     o    Sequence number of array stress in isol
!     maxsols        i    maximum number of solution vectors in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nfilled

!     nfilled        number of entries filled in iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1655   Element of array out of range
!    2665   Too few positions of array have been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check first position
!     Get parameters
!     Check parameters
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercompvector1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercompvector1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nfilled = iinput(1)

!     --- Check first position

      if ( nfilled<5 ) then

!     --- nfilled<5, too few positions have been filled

         call errint ( nfilled, 1 )
         call errint ( 5, 2 )
         call errchr ( 'iinput', 1 )
         call errsub ( 2665, 2, 0, 1 )
         go to 1000

      end if  ! ( nfilled<5 )

!     --- Get parameters

      iseqstress = iinput(2)
      iseqnormstress = iinput(3)
      idim = iinput(4)

!     --- Check parameters

      if ( iseqstress<=0 .or. iseqstress>maxsols ) then

!     --- iseqstress out of range

         call errint ( 2, 1 )
         call errint ( iseqstress, 2 )
         call errint ( 1, 3 )
         call errint ( maxsols, 4 )
         call errchr ( 'iinput', 1 )
         call errchr ( 'iseqstress', 2 )
         call errsub ( 1655, 4, 0, 2 )

      end if  ! ( iseqstress<=0 .or. iseqstress>maxsols )

      if ( iseqnormstress<=0 .or. iseqnormstress>maxsols ) then

!     --- iseqnormstress out of range

         call errint ( 3, 1 )
         call errint ( iseqnormstress, 2 )
         call errint ( 1, 3 )
         call errint ( maxsols, 4 )
         call errchr ( 'iinput', 1 )
         call errchr ( 'iseqnormstress', 2 )
         call errsub ( 1655, 4, 0, 2 )

      end if  ! ( iseqnormstress<=0 .or. iseqnormstress>maxsols )

      if ( idim<=1 .or. idim>2 ) then

!     --- idim out of range

         call errint ( 4, 1 )
         call errint ( idim, 2 )
         call errint ( 2, 3 )
         call errint ( 2, 4 )
         call errchr ( 'iinput', 1 )
         call errchr ( 'idim', 2 )
         call errsub ( 1655, 4, 0, 2 )

      end if  ! ( idim<=0 .or. idim>maxsols )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercompvector1'

      end if
      call erclos ( 'ercompvector1' )

      end subroutine ercompvector1
