      subroutine erfillcf701
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 701 (Poisson)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLCFSRHOCP Fill array iincof/rincof for source and rho cp part
!                    of elements
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The default input is:
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     coef 6 = source
!     coef 7 = density
!     coef 8 = heat_capacity
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf701' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf701'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Coefficient 6-8 (source, density and heat_capacity)

      call erfillcfsrhocp ( 6 )

!     --- integration rule and coordinate system

      call erfillcfrule

      iincof(iposin+1) = -2        ! end of information

1000  call erclos ( 'erfillcf701' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,*) 'End erfillcf701'

      end if  ! ( debug )

      end subroutine erfillcf701
