      subroutine erfillcf900
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 900
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icoefforce, i

!     i              Counting variable
!     icoefforce     Coefficient sequence number w.r.t. force vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFFORCE  Fill array iincof/rincof part force
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The default input is:
!     coef 6 = diffusion
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf900' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf900'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill model

      icoef = 2  ! MODEL

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

        call erfilldefcof4 ( codenavstoks(1) )

      end if  ! ( imark(icoef)==0 )

!     --- integration rule and coordinate system

      call erfillcfrule

      icoef = 5  ! LINEARIZATION

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

        call erfilldefcof4 ( codenavstoks(4) )

      end if  ! ( imark(icoef)==0 )

!     --- Coefficients 6 to 8

      do i = 5, 7

         icoef = i+1
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codenavstoks(i) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 5, 7

!     --- Force vector

      icoefforce = 9
      call erfillcfforce ( icoefforce )

      iincof(iposin+1) = -2        ! end of information

!     --- Coefficients 12 to 14

      do icoef = 12, 14

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codenavstoks(icoef) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 12, 14

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End erfillcf900'

      end if  ! ( debug )
      call erclos ( 'erfillcf900' )

      end subroutine erfillcf900
