      subroutine erfillcfforce ( icoefforce )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-01-2015 Correction for ndim=1
!        version  1.0  date 22-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof part force
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icoefforce

!     icoefforce     i    Coefficient sequence number w.r.t. force vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF2  Check if keyword is found in list of vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcfforce' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcfforce'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoef = icoefforce  ! FORCE

      if ( imark(icoef)==0 .and. ndim>1 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof2 ( codestress(6) )

      end if  ! ( imark(icoef)==0 )

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codestress(7) )

      end if  ! ( imark(icoef)==0 )

      icoef = icoefforce+1  ! Y_FORCE
      if ( ndim>1 .and. imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codestress(8) )

      end if  ! ( imark(icoef)==0 )

      icoef = icoefforce+2  ! Z_FORCE
      if ( ndim>2 .and. imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codestress(9) )

      end if  ! ( imark(icoef)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcfforce'

      end if  ! ( debug )
      call erclos ( 'erfillcfforce' )

      end subroutine erfillcfforce
