      subroutine erfillcfsrhocp ( icoefsource )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for source and rho cp part of elements
!     of type 800/7xx
!     3 consecutive positions are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      use sepmodulestrings
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icoefsource

!     icoefsource    i    Coefficient sequence number for source
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFRHOCP  Fill array iincof/rincof for rho cp part of elements
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcfsrhocp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcfsrhocp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Coefficient icoefsource (density)

      icoef = icoefsource
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codegen(1)(1:ncodegen(1)) )

      end if  ! ( imark(icoef)==0 )

!     --- Coefficients icoefsource+1/2 (density and heat capacity)

      icoef = icoef+1
      call erfillcfrhocp ( icoef )

1000  call erclos ( 'erfillcfsrhocp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcfsrhocp'

      end if  ! ( debug )

      end subroutine erfillcfsrhocp
