      subroutine erlins ( isubr, iproblc, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.6  date 24-12-2014 New definition clinsl
!        version  3.5  date 17-08-2013 Do not reuse iuseri
!        version  3.4  date 06-03-2013 Remove jump
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   ERLINS performs the actual calling of subroutine FILCOF for
!   subroutine LINSOL.
!   It fills the arrays IUSER and USER positions 1 to 5
!   Warning: this subroutine uses memory managment subroutines, so
!            all pointers must be recopied after the call of ERLINS
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iproblc, iseqnr

!    iproblc   i   Problem sequence number
!    iseqnr    i   Sequence number with respect to the storage in array ISEPIN
!    isubr     i   Indicates by which subroutine ERLINS is called indirectly
!                  Possible values:
!                  1:  LINSOL
!                  2:  LINSTM
!                  3:  LINPRB
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsubr, lenius, lenusr, ngroup

!     jsubr          Parameter isubr with respect to TO0093
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
!     ngroup         Total number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     TO0093         Body of subroutines FILCOF and CHNCOF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Estimate maximal length of arrays IUSER and USER
!   Reserve temporary space for these arrays
!   Fill first 5 positions of IUSER and USER
!   Fill rest of iuser and user through FILCOF (TO0093)
!   Make arrays IUSER and USER permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erlins' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erlins'
         call ertrace
         write(irefwr,1) 'iseqnr, iseqcoef, nitercom', &
                          iseqnr, iseqcoef, nitercom
         write(irefwr,1) 'iproblc', iproblc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iproblc==0 ) iproblc = iprobls
      iprob = iproblc
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indprf = kprobloc(19)
      kprobf => kprobpart%kprobf
      call sepfillkprob
      if ( debug )  write(irefwr,1) 'iprob', iprob

!     --- Jump in case coefficients are the same as prior ones

      if ( iseqnr>0 .and. iseqcoef==iseqnr .and. &
           nitercom>-1 .and. associated(iuser1(iprob)%iuser) ) go to 1000

!     --- Estimate maximal length of arrays IUSER and USER
!         Reserve temporary space for these arrays

      ngroup = nelgrp+2*numnat+nglgrp

!     --- allocate temporary arrays

      allocate ( iuser(5+101*ngroup) )
      allocate ( user(5+50*ngroup), stat = error )
      if ( error/=0 ) call eralloc ( error, 10+151*ngroup, 'iuser' )
      if ( ierror/=0 ) go to 1000

!     --- Fill first 5 positions of Iuser and user

      iuser(2:5) = 0
      user(2:5) = 0d0

!     --- Initialize first part of Iuser

      iuser(6:5+ngroup) = 0
      iuser(1) = 5 + 101 * ngroup
      user(1) = 5 + 50 * ngroup

!     --- Fill rest of iuser and user through FILCOF (TO0093)

      if ( isubr==3 ) then
         jsubr = 3
      else
         jsubr = 1
      end if

      if ( debug ) write(irefwr,1) 'before to0093: iseqnr', iseqnr
      call to0093 ( jsubr, iprob, iseqnr )

!     --- Make arrays Iuser and user permanent

      lenius = iuser(4)
      lenusr = nint(user(4))

      if ( associated(iuser1(iprob)%iuser) ) &
         deallocate ( iuser1(iprob)%iuser, iuser1(iprob)%user )
      allocate ( iuser1(iprob)%iuser(lenius), iuser1(iprob)%user(lenusr) )
      iuser1(iprob)%iuser = iuser(1:lenius)
      iuser1(iprob)%user = user(1:lenusr)

      if ( debug ) then
         call prinin ( iuser, lenius, 'iuser' )
         call prinrl ( user, lenusr, 'user' )
      end if  ! ( debug )

!     --- deallocate temporary arrays

      deallocate ( iuser, user, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iuser' )

      iseqcoef = iseqnr
      iprobls = iprob

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erlins'

      end if  ! ( debug )
      call erclos ( 'erlins' )

      end subroutine erlins
