      subroutine erloopfor ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 19-09-2012 Extension with cloop
!        version  1.2  date 06-02-2012 Debug
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 08-10-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform all actions necessary for the start of a for loop
!     Carry out some tests
! **********************************************************************
!
!                       KEYWORDS
!
!     loop
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleloop
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  istart, inpsep(*)

!     inpsep         i    Integer array in which all information about the
!                         structure is stored.
!                         See TO0094 for a description
!     istart        i/o   Position in inpsep referring to actual task
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibeginfor, istepfor, iendfor, ivar, nch, k
      logical newloop

!     ibeginfor      First value of for loop
!     iendfor        Last value of for loop
!     istepfor       Step of for loop
!     ivar           Sequence number of variable
!     k              Counting variable
!     nch            Number of characters in a string
!     newloop        If true a new loop is started
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2122   for variable has been changed in loop
!    2123   step for variable has been changed in loop
!    2124   end for variable has been changed in loop
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Get parameters istepfor and iendfor
!      Detect if a new loop is started
!      If ( not a new loop ) then
!         test some parameters
!      else
!         initialize infloop and parameters
!      Raise istart depending on end of loop reached
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erloopfor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erloopfor'
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters istepfor and iendfor

      iendfor = inpsep(istart+2)
      istepfor = inpsep(istart+3)
      if ( istepfor>100000000 ) &
         istepfor = nint(scalars(istepfor-100000000))
      if ( iendfor>100000000 ) &
         iendfor = nint(scalars(iendfor-100000000))
      ivar = inpsep(istart+4)

      if ( debug ) &
         write(irefwr,1) 'iendfor, istepfor, ivar', &
                          iendfor, istepfor, ivar

!     --- Detect if a new loop is started

      if ( iseqfor==0 ) then

!     --- Start of first for loop

         iseqfor = 1
         newloop = .true.

      else if ( infloop(1,iseqfor)/=istart ) then

!     --- Start of new for loop

         iseqfor = iseqfor+1
         newloop = .true.

      else

!     --- Old for loop

         newloop = .false.
         if ( infloop(3,iseqfor)/=nint(scalars(ivar)) ) then

!        --- Error loop variable has been changed

            call errint ( ivar, 1 )
            call errint ( infloop(3,iseqfor), 2 )
            call errint ( nint(scalars(ivar)), 3 )
            nch = index ( chscalars(ivar), ' ' ) -1
            if ( nch<0 ) nch = 32
            call errchr ( chscalars(ivar)(1:nch), 1 )
            call errsub ( 2122, 3, 0, 1 )

         end if
         if ( infloop(4,iseqfor)/=istepfor ) then

!        --- Warning loop step variable has been changed

            call errint ( ivar, 1 )
            call errint ( infloop(4,iseqfor), 2 )
            call errint ( istepfor, 3 )
            nch = index ( chscalars(ivar), ' ' ) -1
            if ( nch<0 ) nch = 32
            call errchr ( chscalars(ivar)(1:nch), 1 )
            call errwar ( 2123, 3, 0, 1 )

         end if
         if ( infloop(5,iseqfor)/=iendfor ) then

!        --- Warning loop step variable has been changed

            call errint ( ivar, 1 )
            call errint ( infloop(5,iseqfor), 2 )
            call errint ( iendfor, 3 )
            nch = index ( chscalars(ivar), ' ' ) -1
            if ( nch<0 ) nch = 32
            call errchr ( chscalars(ivar)(1:nch), 1 )
            call errwar ( 2124, 3, 0, 1 )

         end if

      end if
      if ( newloop ) then

!     --- New for loop, initialize infloop and scalars

         ibeginfor = inpsep(istart+1)
         if ( ibeginfor>100000000 ) &
            ibeginfor = nint(scalars(ibeginfor-100000000))
         infloop(1,iseqfor) = istart
         infloop(2,iseqfor) = ivar
         infloop(3,iseqfor) = ibeginfor-istepfor
         infloop(4,iseqfor) = istepfor
         infloop(5,iseqfor) = iendfor

      end if
      k = infloop(3,iseqfor)+istepfor
      scalars(ivar) = k
      infloop(3,iseqfor) = k
      if ( iendfor>=k ) then

!     --- for must be executed, go to next step

         istart = istart + 6

      else

!     --- for must not be executed, skip loop

         istart = inpsep(istart+5)
         iseqfor = iseqfor - 1

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erloopfor'

      end if  ! ( debug )
      call erclos ( 'erloopfor' )

      end
