      subroutine ernlns
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.4  date 26-08-2011 Simplification of subroutine
!        version  2.3  date 09-07-2001 Include common cseqnr
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the non-linear solver from the standard input file
!    Store the result in sepmoduleinput
!    Remark: if ernlns is adapted, also adapt ernlnsin !!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     non_linear
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iproblc, niinp, nrinp
      integer, allocatable :: iinput(:)
      double precision, allocatable :: rinput(:)
      save iproblc

!     iinput         integer input array
!     iproblc        Actual problem number
!     niinp          Length of integer input array
!     nrinp          Length of real input array
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READNL         Read card input for non-linear solver
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ03
!    The first record must have the following shape:
!    NONLINEAR_EQUATIONS [,PROBLEM = j], [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iproblc /0/
! ======================================================================
!
      call eropen ( 'ernlns' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinput and rinput

      niinp = 2000
      nrinp = 100

      allocate ( iinput(niinp), rinput(nrinp), stat = error )
      if ( error/=0 ) call eralloc ( error, niinp+nrinp, 'iinput' )
      if ( ierror/=0 ) go to 1000

      iseqnlns = iseqnlns + 1

      call readnl ( iinput, rinput, 2, iseqnlns, &
                    iproblc, nprob, niinp, nrinp )

!     --- Make input arrays permanent

      call ercopyinput ( 12, iseqnlns, 'NONLINEAR_EQUATIONS', iinput, &
                         rinput, iinput, niinp, nrinp, 0 )

      deallocate ( iinput, rinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  call erclos ( 'ernlns' )
      end
