      subroutine erprob ( iseqprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.3  date 22-11-2012 Long integers
!        version  2.2  date 26-08-2011 Simplification of subroutine
!
!   copyright (c) 2003-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input for subroutine PROBDF as well as sequence number and
!     problem number if available
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqprob

!     iseqprob      i/o   Sequence number of problem input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, nrinp, iseqprobrd, &
              lengin, lengrl, kread, nprob
      integer, allocatable :: iinput(:)
      double precision, allocatable :: rinput(:)

!                    they are not
!     iinput         integer input array
!     iseqprobrd     Value of iseqprob read by prorea
!     kread          Indication if first record has been read (1)
!                    or not (0)
!     lengin         Length of integer array iinput
!     lengrl         Length of real array rinput
!                    (in double precision variables)
!     niinp          Length of integer input array
!     nprob          Number of various problems
!     nrinp          Length of real input array
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PROREA         Read input from standard input file
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is read before by READAL
!    of the rest of the records is performed by subroutine PROREA
!
!    The first record must have the following shape:
!
!    PROBLEM [i], sequence_number = k
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     reserve space in buffer array for the arrays IINPUT and RINPUT
!     Activate arrays
!     Compute starting addresses
!     Read input with sequence number
!     Make input arrays permanent
!     Copy contents to final positions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erprob' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erprob'

      end if
      if ( ierror/=0 ) go to 1000

      niinp=10000     ! length of iinput
      nrinp=1000      ! length of rinput

      allocate ( iinput(niinp+5), rinput(nrinp+5), stat = error )
      if ( error/=0 ) call eralloc ( error, niinp+nrinp+10, 'iinput' )
      if ( ierror/=0 ) go to 1000

      iseqprob = iseqprob + 1
      kread = 11

!     --- Read input with sequence number

      call prorea ( iinput(6), lengin, nprob, kread, rinput(6), &
                    lengrl, niinp, nrinp, iseqprobrd )
      if ( iseqprobrd>0 ) iseqprob = iseqprobrd

!     --- Fill positions 1 to 5

      iinput(1) = lengin+5  ! length
      iinput(2) = 0
      iinput(3) = lengin+5  ! length
      iinput(4) = 0
      iinput(5) = nprob

      rinput(1) = dble(lengrl+5)  ! length
      rinput(2) = 0d0
      rinput(3) = dble(lengrl+5)  ! length
      rinput(4) = 0d0
      rinput(5) = 0d0

      lengin = lengin+5
      if ( lengrl>0 ) lengrl = lengrl+5

!     --- Make input arrays permanent

      call ercopyinput ( 27, iseqprob, 'PROBLEM', iinput, &
                         rinput, iinput, lengin, lengrl, 0 )
      if ( ierror/=0 ) go to 1000

      deallocate ( iinput, rinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erprob'

      end if
      call erclos ( 'erprob' )

      end
