      subroutine errchksub
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.0  date 07-12-92
!
!   copyright (c) 1992-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Checks if all error calls pass the right number of ints/reals/chars
!
! **********************************************************************
!
!                       KEYWORDS
!
!     error
!     check
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!     None.
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!     None.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer      i, ifrom, istart(21), ito ,maxerr, nrecwo, &
                   linenr, numint, numlin, numrea, lenspe, numspe, &
                   numcha, ich, iho, iht, ios, k, mesnum, n, nchar, &
                   nints, nreal, nwrong, iref
      logical      nxtlin
      character (len=80) errlin
      character (len=5) type

!     errlin   error line
!     i        loop variable
!     ich      pointer to character in errlin
!     ifrom    information in array istart is read from position ifrom
!     iho      first position of array istart that has not been read
!     iht      last position of array istart that has (to be/been) read
!     ios      iostatus variable
!     istart   integer array containing a part of the starting records
!              of the messages.
!     ito      information in array istart is read to position ito.
!     k        implied do-loop variable
!     lenspe   length of special message array.
!     linenr   record number of message line
!     maxerr   1 + maximum number of messages that are stored
!              in the message file.
!     mesnum   number of the message
!     n        number of records
!     nchar    number of character strings passed for the message
!     nints    number of character strings passed for the message
!     nreal    number of character strings passed for the message
!     nrecwo   number of words (integers) that can be stored
!              in one record of the message file
!     numcha   number of character strings in a message
!     numint   number of integers in a message
!     numlin   number of lines of a message
!     numrea   number of reals in a message
!     numspe   the number of the special error message.
!     nwrong   number of incorrect error message calls
!     nxtlin   true: scan next line, false :don't scan next line
!     type     type of message : 'error' / 'warning'
!              'error' is assumed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     None.
! **********************************************************************
!
!                       I/O
!
!         ERMESS writes to the default output device.
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    Message number not yet added to the Error Text File.
!    Number of integers, reals or strings in message not correct.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     read maximum message number from error text file ;
!     if (message number correct) then
!        read starting record number ;
!        calculate number of lines ;
!        if ( number of lines>0 ) then
!           for each line do
!              read line ;
!              process and print line ;
!        else
!           print warning : message does not exist ;
!     else
!        print warning : message number out of range ;
!     if (number of integers, reals and/or strings not correct) then
!        print warning ;
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- SET common blocks (machine-dependent)

      call initcb
      call initmd
      iref = 4

!     --- Open files (error message file)

      call inifil(4)
      open ( unit=2, file='errchk.tmp' )
      type='error'
      nwrong = 0
      lenspe = 6
      nrecwo = 20
      numspe = -1

!     --- read largest message number permitted

      ifrom = 1
      ito   = 1
      linenr= 1
      n=(ito-ifrom)/nrecwo+1
      iho=ifrom
      do 90 i=1,n
         iht=min(ito,iho+nrecwo-1)
         read (iref,rec=linenr+i-1,iostat=ios) (istart(k),k=iho,iht)
         if (ios/=0) then
            write (6,1000)
            stop
         end if
         iho=iht+1
90    continue
      maxerr = istart(1)

!     --- do until EOF in errdata.tmp

100   continue
         read ( 2, '(i5,3i6)', end=300 ) mesnum, nints, nreal, nchar
         if ( (mesnum>0 .and. mesnum<=maxerr) &
              .or.   mesnum==numspe  ) then

!        --- Read starting record of message

            if (mesnum==numspe) then

!           --- special error message: open failure of error text file

               numlin = lenspe
               linenr = 1
               numint=3
               numcha=1
               numrea=0
            else
               linenr = 2 + (mesnum-1)/nrecwo
               ito    = mod(mesnum,nrecwo)+1
               if (ito==1) ito = nrecwo+1
               n=(ito-ifrom)/nrecwo+1
               iho=ifrom
               do 110 i=1,n
                  iht=min(ito,iho+nrecwo-1)
                  read(iref,rec=linenr+i-1,iostat=ios) &
                      (istart(k),k=iho,iht)
                  if (ios/=0) then
                     write (6,1000)
                     stop
                  end if
                  iho=iht+1
110            continue
               linenr = istart(ito-1)
               numlin = istart(ito) - linenr
            end if
            if ( numlin>0 ) then
               i=linenr
               if ( mesnum==numspe ) then
                   nxtlin = .false.
               else
                  numint=0
                  numrea=0
                  numcha=0
                  nxtlin=.true.
               end if
120            if (nxtlin) then
                  read (iref,rec=i) errlin
                  ich = 0
130               if ( ich<80 ) then
                     ich = ich + 1
                     if ( errlin(ich:ich)=='$' ) then
                        numint = numint + 1
140                     ich = ich + 1
                        if ( ich<=80 ) then
                           if ( errlin(ich:ich)=='$' ) goto 140
                        end if
                     else if ( errlin(ich:ich)=='&' ) then
                        numrea = numrea + 1
150                     ich = ich + 1
                        if ( ich<=80 ) then
                           if ( errlin(ich:ich)=='&' ) goto 150
                        end if
                     else if ( errlin(ich:ich)=='%' ) then
                        numcha = numcha + 1
160                     ich = ich + 1
                        if ( ich<=80 ) then
                           if ( errlin(ich:ich)=='%' ) goto 160
                        end if
                     end if
                     goto 130
                  end if
                  i = i + 1
                  nxtlin = i<=(linenr+numlin-1)
                  goto 120
               end if

!              --- warning if number of integers, reals, or strings not correct
!                  (if error number exists)

               if ( numint/=nints .or. numrea/=nreal .or. &
                    numcha/=nchar ) then
                  nwrong = nwrong + 1
                  write (6,'(//1x,78(''*'')/)')
                  write (6,1400) type,mesnum,numint,nints,numrea,nreal, &
                                      numcha,nchar
                  write (6,'(/1x,78(''*'')//)')
               end if
            else

!           --- Message not present in the file

               nwrong = nwrong + 1
               write (6,1100) type, mesnum
            end if
         else

!        --- Message number out of range

            nwrong = nwrong + 1
            write (6,1100) type, mesnum
         end if
      goto 100
300   continue

!     --- write number of incorrect called error messages

      if ( nwrong==0 ) then
         write (6,'(a)') 'All error routine calls are correct'
      else
         write (6,'(a10,i3,a11,a20)') 'There are ',nwrong,' incorrect ', &
                        'error routine calls.'
      end if
 1000 format (/// &
       ' *** Module ERRCHK: '/ &
       ' Error while reading error text file '///)
 1100 format (' *** WARNING : '/ &
              ' ',a,' number ',i6,' has not yet been added', &
              ' to the error message file')
 1400 format ( &
       ' *** INTERNAL ERROR : number of integers, reals,', &
       ' or strings in message '/ &
       ' not equal to number of integers, reals, or strings', &
       ' in program.'/ &
       ' ',a,' number : ',i5/ &
       ' number of integers in message :',i4, &
       ', in program :',i4/ &
       ' number of reals in message    :',i4, &
       ', in program :',i4/ &
       ' number of strings in message  :',i4, &
       ', in program :',i4)
      end
