      subroutine erscaltxt ( iseqnr, textloc, lentxt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-05-2014 Extra debug
!        version  1.1  date 08-12-2013 Extension with integer scalars
!        version  1.0  date 21-12-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill string textloc with name of scalar depending on iseqnr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     text
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr, lentxt
      character (len=*) textloc

!     iseqnr         i    Sequence number of scalar
!     lentxt         o    length of textloc
!     textloc        o    String to store a local text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2952   Sequence number of scalar out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erscaltxt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erscaltxt'
         write(irefwr,1) 'maxcls, maxintscals', maxcls, maxintscals
         write(irefwr,1) 'iseqnr', iseqnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( iseqnr)

         case(0)

            textloc = 'time'
            lentxt = 4

         case(1:maxcls+maxintscals)

!        --- standard case

            textloc = chscalars(iseqnr)
            lentxt = index(textloc,'          ')-1

         case(maxcls+maxintscals+1)

            textloc = 'nelem'
            lentxt = 5

         case(maxcls+maxintscals+2)

            textloc = 'npoint'
            lentxt = 6

         case(maxcls+maxintscals+3)

            textloc = 'npelm'
            lentxt = 5

         case(maxcls+maxintscals+4)

            textloc = 'shape'
            lentxt = 5

         case default

!        --- error scalar number incorrect

            call errint ( iseqnr, 1 )
            call errint ( maxcls, 2 )
            call errsub ( 2952, 2, 0, 0 )
            go to 1000

      end select  ! case ( iseqnr)

      textloc(lentxt+1:lentxt+3) = ' = '
      lentxt = lentxt + 3

1000  call erclos ( 'erscaltxt' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erscaltxt'

      end if  ! ( debug )

      end
