      subroutine erscalval ( iseq, rvalue, ivalue, integerfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 22-03-2014 Allow expressions
!        version  2.1  date 08-12-2013 Extension with integer scalars
!        version  2.0  date 14-04-2013 Remove scalar from parameter list
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill value of scalars defined by ipos into either rvalue or ivalue
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulekmesh
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, ivalue
      double precision rvalue
      logical integerfound

!     integerfound   o    If true an integer has been found
!     iseq           i    Sequence number of scalar
!     ivalue         o    Integer value found
!     rvalue         o    Real value found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos

!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETSCALINDEX   Return with adapted index of scalar
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2952   Sequence number of scalar out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erscalval' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erscalval'
         write(irefwr,1) 'iseq', iseq
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      integerfound = .false.
      if ( iseq>200000000 ) then

!     --- Expression found

         call prevalexpr ( iseq, rvalue, 'print scalar' )
         go to 1000

      end if  ! ( iseq>200000000 )

      ipos = getscalindex ( iseq )

      if ( debug ) write(irefwr,1) 'ipos', ipos

      select case (ipos)

      case(0)

!     --- Sequence number = 0, hence time

         rvalue = t

      case (1:maxcls)

!     --- standard case

         rvalue = scalars(ipos)

      case (maxcls+1:maxcls+maxintscals)

!     --- integer scalar

         ivalue = nint(scalars(ipos))
         rvalue = ivalue
         integerfound = .true.

      case (maxcls+maxintscals+1)

!     --- 1001: nelem

         ivalue = nelem
         rvalue = ivalue
         integerfound = .true.

      case (maxcls+maxintscals+2)

!     --- 1002: npoint

         ivalue = npoint
         rvalue = ivalue
         integerfound = .true.

      case (maxcls+maxintscals+3)

!     --- 1003: npelm

         ivalue = npelm
         rvalue = ivalue
         integerfound = .true.

      case (maxcls+maxintscals+4)

!     --- 1004: shape

         ivalue = maxval(kmeshf)
         rvalue = ivalue
         integerfound = .true.

      case default

!     --- error scalar number incorrect

         call errint ( ipos, 1 )
         call errint ( maxcls, 2 )
         call errsub ( 2952, 2, 0, 0 )

      end select  ! case (iseqnr+ipos)

1000  if ( debug ) then

!     --- Debug information

         if ( integerfound ) then
            write(irefwr,1) 'ivalue', ivalue
         else
            write(irefwr,2) 'rvalue', rvalue
         end if  ! ( integerfound )

         write(irefwr,*) 'End erscalval'

      end if  ! ( debug )
      call erclos ( 'erscalval' )

      end
