      subroutine ertimd ( nooutput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 26-08-2016 use sepmoduleinput
!        version  5.1  date 28-10-2015 New call to readtm
!        version  5.0  date 26-08-2011 Extra parameters
!
!   copyright (c) 1995-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the time-dependent solver from the standard input
!    file  Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical nooutput

!     nooutput       o    If true no output is written to the file sepcomp.out
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniintim, niinp, nrinp, lnrinp
      integer, allocatable :: iinput(:), inforiinput(:)
      double precision, allocatable :: rinput(:)

!     iinput         integer input array
!     inforiinput    Array of length of rinput that contains a 0 if the
!                    number stored in rinput is a real and a reference
!                    to scalars if it is a scalar
!     lniintim       declared length of iintim
!     lnrinp         Length of real input array
!     niinp          Length of integer input array
!     nrinp          Available space for array RINPUT
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READTM         Read card input for time-dependent solver
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READTM
!    The first record must have the following shape:
!    TIME_INTEGRATION, [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ertimd' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iintim and rintim

      niinp = 1000
      nrinp = 190
      allocate ( iinput(niinp), inforiinput(nrinp), &
                 rinput(nrinp), stat=error )
      if ( error/=0 ) call eralloc ( error, niinp+2*lnrinp, 'iinput' )
      if ( ierror/=0 ) go to 1000

      iseqtimd = iseqtimd + 1

      lnrinp = 0
      call readtm ( iinput, rinput, iseqtimd, 1, ndim, &
                    niinp, lniintim, lnrinp )
      nooutput = iinput(19)==0

!     --- Make input arrays permanent

      call ercopyinput ( 15, iseqtimd, 'TIME_INTEGRATION', iinput, &
                         rinput, inforiinput, lniintim, lnrinp, 0 )

      deallocate ( iinput, inforiinput, rinput, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  call erclos ( 'ertimd' )
      end
