      subroutine fillinp ( isubr, iseqnr, iinput, &
                           rinput, leniinp, lenrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.11 date 24-12-2014 New definition clinsl
!        version  1.10 date 29-05-2014 Remove time routine
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill contents of arrays iinput and rinput into ibuffr/buffer and
!    store such that it can be found for a specific subroutine and sequence
!    number. This subroutine may be used instead of or additional to readal
!    iinput and rinput may be extended with default values if they are
!    filled partly only
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, iinput(*), leniinp, lenrinp
      double precision rinput(*)

!     iinput     i    integer input array for this specific subroutine
!                     For a description of all positions, see the
!                     specific subroutines
!     iseqnr     i    Sequence number of input with respect to the storage in
!                     array ISEPIN for this specific subroutine
!     isubr      i    Indicates the type of subroutine for which the input
!                     is meant
!                     Possible values
!                      1:  START
!                      2:  PROBDF
!                      3:  COMMAT
!                      4:  CREATE
!                      5:  PRESDF
!                      6:  FILCOF
!                      7:  CHNCOF
!                      8:  SOLVE
!                      9:  OUTPUT
!                     10:  SEPCOMP
!                     11:  DERIVS
!                     12:  NLNPRB
!                     13:  INTEGR
!                     14:  INTBND
!                     15:  TIMPRB
!                     16:  ADAPBOUN
!                     17:  ADAPMESH
!                     18:  STATFREE
!                     19:  INSTFREE
!                     20:  CONTACT
!                     21:  LOOP
!                     22:  CHANGE_COORDINATES
!     leniinp    i    Length of array iinput, i.e. number of positions that
!                     have been filled by the user. For all other positions
!                     default values are used.
!     lenrinp    i    Length of array rinput, i.e. number of positions that
!                     have been filled by the user. For all other positions
!                     default values are used.
!     rinput     i    Real input array for this specific subroutine
!                     For a description of all positions, see the
!                     specific subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERNLNSIN       Fill input concerning the non-linear solver from the arrays
!                    iinput and rinput
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSOLVIN       Fill input concerning the linear solver from the arrays
!                    iinput and rinput
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2040   Value of isubr has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!   if ( array isepin has not yet been filled ) then
!      Reserve space for first part of isepin
!      Fill blocks 2 and 3 of array ISEPIN and compute length of third part
!      Extend the length of isepin with block 3
!      clear isepin block 3
!   Activate array isepin and compute starting addresses
!   Copy information from iinput and rinput into ibuffr/buffer, depending on
!   isubr and iseqnr. Extend with defaults
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'fillinp' )
      if ( ierror/=0 ) go to 1000

!     --- Copy information from iinput and rinput into ibuffr/buffer, &
!         depending on isubr and iseqnr. Extend with defaults

      select case ( isubr )

         case ( :7, 9:11, 13: )

!        --- isubr has value that has not yet been implemented

            call errint ( isubr, 1 )
            call errsub ( 2040, 1, 0, 0 )

         case (8)

!        --- isubr = 8, input for linear solver

            call ersolvin ( iseqnr, iinput, rinput, leniinp, lenrinp )

         case (12)

!        --- isubr = 12, input for non-linear solver

            call ernlnsin ( iseqnr, iinput, rinput, leniinp, lenrinp )

      end select  ! case ( isubr )

!     --- end of subroutine, decrease priority and close

1000  call erclos ( 'fillinp' )

      end subroutine fillinp
