      function getconst ( realname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-04-2013 indexed constants
!        version  1.2  date 22-08-2005 Comment adapted
!        version  1.1  date 22-11-2004 Fortran 90
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the value of the real in the block constants with name realname
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) realname
      double precision getconst

!     getconst       o    Value of constant
!     realname       i    Name of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2156   real name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chrlcons
!     If ( name in chrlcons ) then
!        getconst := value
!     else
!        Find position of name in chincons
!        If ( name in chincons ) then
!           getconst := value
!        else
!           error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getconst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getconst'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = getnamereal ( realname )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>0 ) then
         getconst = rlcons(iseq)
      else
         iseq = getnameint ( realname )
         if ( iseq>0 ) then
            getconst = incons(iseq)
         else
            getconst = 0
            call errchr ( realname, 1 )
            call errwar ( 2156, 0, 0, 1 )
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'getconst', getconst
         write(irefwr,*) 'End getconst'

      end if  ! ( debug )
      call erclos ( 'getconst' )

      end
