      function getint ( intname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-04-2013 indexed constants
!        version  1.2  date 06-02-2012 Change w.r.t warnings
!        version  1.1  date 22-11-2004 Fortran 90
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the integer in the block constants with name intname
!     If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname
      integer getint

!     getint        o    Value of integer
!     intname       i    Name of integer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intseqnm

!     intseqnm       Sequence number of integer with respect to array
!                    chincons
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINT     Return with sequence number of integer in incons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2155   integer name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chincons
!     If ( name in chincons ) then
!        getint := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getint'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intseqnm = getnameint ( intname )

      if ( debug ) write(irefwr,1) 'intseqnm', intseqnm

      getint = 0
      if ( intseqnm>0 ) then
         getint = incons(intseqnm)
      else if ( nwarn<=mxwarn ) then
         call errchr ( intname, 1 )
         call errwar ( 2155, 0, 0, 1 )
      end if  ! ( intseqnm>0 )

1000  call erclos ( 'getint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'getint', getint
         write(irefwr,*) 'End getint'

      end if  ! ( debug )

      end
