      subroutine getintar ( intname, intar, ncomponents )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-09-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the values of a set of integers in the block constants with name
!    intname
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname
      integer ncomponents, intar(ncomponents)

!     intar          i    Contains the integers that are read
!     intname        o    Name of integer
!     ncomponents    i    Number of components
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, i

!     i              Counting variable
!     iseq       Sequence number of integer with respect to array
!                    chincons
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINT     Return with sequence number of integer in incons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2155   integer name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chincons
!     If ( name in chincons ) then
!        getintar := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iseq = getnameint ( intname )
      if ( iseq>0 ) then
         do i = 1, ncomponents
            intar(i) = incons(iseq+i-1)
         end do  ! i = 1, ncomponents
      else
         do i = 1, ncomponents
            intar(i) = 0
         end do  ! i = 1, ncomponents
         call eropen ( 'getintar' )
         call errchr ( intname, 1 )
         call errwar ( 2155, 0, 0, 1 )
         call erclos ( 'getintar' )
      end if
      end
