      function getnamevar ( varname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 02-05-2014 Correction dimension 1
!        version  1.5  date 08-12-2013 Extension with integer scalars
!        version  1.4  date 10-04-2013 Loop of indexed reals
!
!   copyright (c) 2000-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the scalar in the
!     block variables with name varname
!     If the name does not exist getnamevar is not changed
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) varname
      integer getnamevar

!     getnamevar    o   Sequence number of variable with respect to array
!                       chscalars
!     varname       i   Name of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1
      character (len=80) vname1, vname2

!     i              General loop variable
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
!     vname1         Name of variable translated to capitals
!     vname2         Name of variable translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3127   loop variable in scalars array zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numscals do
!        if ( varname = chscalars(i) ) then
!           getnamevar := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getnamevar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getnamevar'
         write(irefwr,*) 'varname ', varname
         write(irefwr,1) 'numscals, numintscals, maxcls', &
                          numscals, numintscals, maxcls
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      getnamevar = 0
      nch1 = index ( varname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(varname)
      vname1 = varname(1:nch1)
      call askupc(vname1)

      if ( debug ) write(irefwr,*) 'vname1 ', vname1(1:nch1)

      do i = 1, numscals

         nch = ncharscals(i)

         if ( nch1==nch ) then
            vname2 = chscalars(i)(1:nch1)
            call askupc(vname2)
            if ( vname1(1:nch1)==vname2(1:nch1) ) then

!           --- name of scalar found

               getnamevar = i

               if ( debug ) &
                  write(irefwr,1) 'i, dimscals(i)', i, dimscals(i)

               if ( dimscals(i)>1 ) then

!              --- Constant is an array

                  if ( iseqarray>0 ) then

!                 --- constants array

                     getnamevar = i+iseqarray-1

                  else if ( iseqarray<0 ) then

!                 --- constants array, but no sequence number given
!                     in this case an array is permitted because iseqarray<0

                     getnamevar = i+5000

                  else

!                 --- error loop varibale is 0

                     call errchr ( vname1(1:nch1), 1 )
                     call errsub ( 3127, 0, 0, 1 )
                     go to 1000

                  end if  ! ( iseqarray>0 )

               end if  ! ( dimscals(i)>1 )
               go to 1000

            end if
         end if

      end do  ! i = 1, numscals

      do i = maxcls+1, maxcls+numintscals

         nch = ncharscals(i)

         if ( debug ) write(irefwr,1) 'i, nch', i, nch

         if ( nch1==nch ) then
            vname2 = chscalars(i)(1:nch1)
            call askupc(vname2)

            if ( debug ) write(irefwr,*) 'vname2 ', vname2(1:nch1)
            if ( vname1(1:nch1)==vname2(1:nch1) ) then

!           --- name of scalar found

               getnamevar = i

               if ( dimscals(i)>0 ) then

!              --- Constant is an array

                  if ( iseqarray>0 ) then

!                 --- constants array

                     getnamevar = i+iseqarray-1

                  else if ( iseqarray<0 ) then

!                 --- constants array, but no sequence number given
!                     in this case an array is permitted because iseqarray<0

                     getnamevar = i+5000

                  else

!                 --- error loop varibale is 0

                     call errchr ( vname1(1:nch1), 1 )
                     call errsub ( 3127, 0, 0, 1 )
                     go to 1000

                  end if  ! ( iseqarray>0 )

               end if  ! ( dimscals(i)>1 )
               go to 1000

            end if
         end if

      end do  ! i = 1, numscals

1000  call erclos ( 'getnamevar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'getnamevar', getnamevar
         write(irefwr,*) 'End getnamevar'

      end if  ! ( debug )

      end
