      subroutine incv02 ( ibinas )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 04-03-2011 New call to readoutp
!        version  3.0  date 08-12-2006 New parameter list
!        version  2.3  date 13-10-1999 New definition of sepcomp.out
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    an interactive subroutine to make a conversion
!    of the SEPRAN sepcomp.out files from ASCII to binary or vice-versa
! **********************************************************************
!
!                       KEYWORDS
!
!     conversion
!     interactive
!     problem_definition
!     solution_vector
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmodulekprob
      use sepmoduleread
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibinas

!     ibinas   i    Indicates if a file must be translated from binary to ASCII
!                   or vice-versa
!                   Possible values:
!                   0:  Translate from binary to ASCII
!                   1:  Translate from ASCII to binary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniinp
      parameter ( lniinp=1500 )
      integer irefsv, iinputlc(lniinp), iref, ihelp(10), &
              ios, iseqlc, iter, irefnew, len74, ndtvecold
      logical endfil
      double precision t
      character (len=100) namesv

!     endfil         Indicates if end of file has been reached (true ) or
!                    not (false)
!     ihelp          Work array to store input from the input file
!     iinputlc       integer input array
!     ios            IOSTAT variable, returns from reading or writing
!     iref           File reference number
!     irefnew        Help variable to save reference number of new file
!     irefsv         Help variable to save reference number of old file
!     iseqlc           Sequence number of vector with respect to array isol
!     iter           iteration number
!     len74          Number of characters in name74
!     lniinp         Declared length of array iinputlc
!     namesv         Save of previous name
!     ndtvecold      Value of ndtvec at start of subroutine
!     t              Time read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERTEXT         Produce menu
!     INIFIL         Open files
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     PROUTPSUB      Write second part of information to file sepcomp.out
!     READ74SUB      Read names of vectors from the sepcomp.out file
!     READ74SUB1     Read first part of information of solution array from
!                    sepcomp.out
!     READOUTP       read real/complex sepran solution array from the
!                    file sepcomp.out
!     READPROBIINP   Read information concerning the input for probdfbf from
!                    file sepcomp.out and call subroutine probdfbf
!     SEPPRINOUTP    Write solution vectors to sepcomp.out
!     WRITPROBIINP   Write arrays iinput and rinput corresponding to kprob to
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
!    The files sepcomp.inf and sepcomp.out are read
!    The file sepcomp.inf is organized as follows:
!          First records array iinputlc as output from subroutine PROBDF
!          iinputlc is written by subroutine SEPSTR
!          Next records:
!          Rec. 1:  iversion, isubr
!          if ( isubr = 1 ) then
!             Rec. 2:  numvec
!             Rec. 3...: Sequence numbers of arrays that have been
!                        written to file sepcomp.out
!          else if ( isubr = 2 ) then
!             Rec. 2:  numvec
!             Rec. 3...: Sequence numbers of arrays that have been
!                        written to file sepcomp.out
!             Next records : each containing exactly one real number
!                        representing the time corresponding to the set of
!                        arrays written by one call of PROUTP
!          else if ( isubr = 3 ) then
!             Next records : each containing 10 numbers:
!                        sequence_number,iteration_number, time_level
!                        and 7 numbers 0 for future purposes
!          with
!              iversion: Version number of type of file
!                       -1: is finite volume version
!                       0: is first version
!                       present version: 2
!              isubr:   type of solution that is written.
!                       Possible values:
!                       1:  stationary case (one call of output only)
!                       2:  Time-dependent  (Information is written only
!                                            once for each time-level t )
!                       3:  general case: There are more calls of output
!                           possible, each with its own sequence number
!                           However, the sequence numbers must be disjunct
!                           Furthermore always a time-level and an iteration
!                           number is available
!              numvec:  number of vectors that have been written to the
!                       file sepcomp.out, for each time-step
!                       It is assumed that in each time-step exactly the
!                       same arrays will be written
!              output_sequence: Sequence number of output input
!              iteration_number: The output vector may be provided by a
!                       iteration number and a time-level
!   The arrays in the file sepcomp.out are written by the subroutine prinsl
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     769   End of file Reached of file sepcomp.out
!     770   Read error in file sepcomp.out
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incv02' )
      iinputlc(1) = lniinp
      iinputlc(3) = 0

!     --- Save old name of sepcomp.out and ask for actual name

      namesv = name74
      irefsv = iref74
      call errchr ( name74, 1 )
      call ertext ( 36+ibinas, 0, 0, 1 )

!     --- Read answer

      ityprd = 1
      ispecl = 3
      call asknxt
      if ( ityprd==5 ) name74 = string(1:nchar)
      ispecl = 0
      ndtvecold = ndtvec

!     --- Open old file

      if ( ibinas==0 ) then

!     --- ibinas = 0, binary file

         iref74 = -abs(iref74)

      else

!     --- ibinas = 1, ASCII file

         iref74 = abs(iref74)

      end if
      ierror = -1
      call inifil(-74)
      if ( ierror>0 ) then
         iref74 = irefsv
         name74 = namesv
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 1000
      end if
      irefsv = iref74

!     --- Read mesh file and close file

      write(*,135)
      write(*,*) 'The mesh will now be read'
      call inifil ( -10 )
      call msh071 ( -1, iref10, ihelp )
      iref = abs(iref10)
      close ( iref )

      write(*,*) 'The problem definition will now be read'
      call readprobiinp ( iinputlc, .false. )

      if ( ierror/=0 ) go to 1000

!     --- ask for name of new file

100   write(*,135)
135   format(////)
      call errchr ( 'sepcomp.new', 1 )
      call ertext ( 37-ibinas, 0, 0, 1 )

!     --- Read answer

      ityprd = 1
      ispecl = 3
      call asknxt
      len74 = index(name74,' ')-1
      if ( len74<0 ) len74 = len(name74)

      if ( len74==nchar .and. name74(1:len74)==string(1:nchar) ) then
         write(*,*) 'name of input and output file must be different'
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         go to 100
      end if  ! ( len74==nchar .and. name74(1:len74)==string(1:nchar) )

      if ( ityprd==5 ) name74 = string(1:nchar)
      ispecl = 0

!     --- Open new file

      iref74 = abs(iref74)+1

      if ( ibinas==0 ) then

!     --- ibinas = 0, binary file

         iref74 = abs(iref74)

      else

!     --- ibinas = 1, ASCII file

         iref74 = -abs(iref74)

      end if
      ierror = -1
      call inifil(74)
      if ( ierror>0 ) then
         iref74 = irefsv
         name74 = namesv
         go to 1000
      end if
      irefnew = iref74

!     --- Write first part to file sepcomp.out

      call writprobiinp

!     --- Read solution from file 74 with input from file 73

      write(*,*) 'File sepcomp.out is now read'

!     --- Read names of vectors

      iref74 = irefsv
      call read74sub

!     --- Write names of vectors

      iref74 = irefnew
      call proutpsub ( numvec, ipost, names, rpost )

      numvec = 0
      endfil = .false.

!     --- while loop end file

300   if ( .not. endfil ) then

!     --- Read next record in sepcomp.out

         iref74 = irefsv
         call read74sub1 ( iseqlc, iter, t, ios )

         if ( ios<0 ) then

!        --- end of file found

            endfil = .true.

         else if ( ios>0 ) then

!        --- error in reading of file

            go to 500

         else

!        --- Correct record found

            call readoutp ( isol, ndtvecold )
            if ( ierror/=0 ) go to 1000

         end if

!        --- write array

         iref74 = irefnew
         call sepprinoutp ( isol, iseqlc, iter, t )
         go to 300

      end if

      go to 1000

!     --- Error messages if ios # 0

500   if ( ios>0 ) then

!     --- ios>0, Error found in reading of file 74

         call errsub ( 770, 0, 0, 0 )

      else

!     --- ios<0, End of file found in reading of file 74

         call errsub ( 769, 0, 0, 0 )

      end if

1000  if ( ierror/=0 ) then
         call ertext ( 1, 0, 0, 0 )
         read(*,*)
         ierror = 0
      end if  ! ( ierror/=0 )

      call erclos ( 'incv02' )
      end subroutine incv02
