      subroutine insp04
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 25-11-2012 Long integers
!        version  2.7  date 30-08-2006 New definition of cpost5
!        version  2.6  date 14-10-2004 ini060 --> ini060text
!
!   copyright (c) 1990-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    INSP04 is used to compute the time-history of a time-dependent
!    solution and to print or plot it
!    subroutine INSP04 is a help subroutine of subroutine SEPPS1
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     plot
!     post_processing
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulepost
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iplot, ivec, ichois, ipoint, imesh, i, number, lastmapbef, &
              itim, ichvec, icplot(8), indsol, ndimlc
      double precision multfact, height, dum, rcplot(2), fval, xval
      logical intersect

!     dum            Dummy parameter
!     fval           level for which the x-value must be computed
!     height         Height of a plot
!     i              Loop variable
!     ichois         Standard choice parameter for several subroutines
!     ichvec         Choice parameter for subroutine vecman
!     icplot         Integer input array for plot subroutine
!     imesh          Actual mesh and problem number
!     indsol         Sequence number of vector in isol
!     intersect      if true fval is in the reach of the function
!     iplot          Parameter IPLOT, See iinputpst(20)
!     ipoint         Nodal point number in which a solution must be computed
!     itim           Counting help variable for time step
!     ivec           Sequence number of array to be processed
!     lastmapbef     Help variable to store the last value of lastmapsol
!     multfact       Multiplication factor
!     ndimlc         Local dimension of space
!     number         Parameter to indicate which unknown should be considered
!     rcplot         Double precision input array for plot subroutine
!     xval           computed x-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINDXY         Find nodal point closest to a specific point
!     INSP01         Compute quantities given by a compute statement
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLOTFN1        Plot function
!     PRINTB         Print information of one time-step
!     PRTIMHISLEV    Compute the x-value in the one-dimensional space where
!                    the function takes value fval
!     SEPALGEBR      algebraical operations on SEPRAN vectors
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPVECMAN      Sepran vector manipulations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'insp04' )

!     --- Store solution required into funcy and the corresponding time
!         into funcx from position 6

      ivec = iinputpst(2)
      iplot  = iinputpst(20) / 10
      ichois = iinputpst(20) - 10 * iplot

      if ( ichois==2 ) then

!     --- ICHOIS = 2, compute point ipoint closest to (x,y,z)

         do itimst = iltime(1), iltime(2)
            indsol = imapsol(ivec,itimst)
            if ( indsol>0 ) go to 100
         end do
         itimst = iltime(1)
         if ( indsol==0 ) then

!        --- Create new entry in isol

            lastmapsol = lastmapsol+1
            imapsol(ivec,1) = lastmapsol
            indsol = lastmapsol

         end if
100      if ( isolut2d(indsol)==0 ) call insp01
         imesh = ispec(ivec,3)
         kmesh => kmesh2d(imesh)
         call sepfillkmesh(kmesh)
         call findxy ( coor, rinputpst(10), rinputpst(11), rinputpst(12), &
                       npoint, ipoint, ndim, 3 )
      end if

      itim = 0
      lastmapbef = lastmapsol
      do itimst = iltime(1), iltime(2)
         indsol = imapsol(ivec,itimst)

         if ( indsol==0 ) then

!        --- Create new entry in isol

            lastmapsol = lastmapsol+1
            imapsol(ivec,itimst) = lastmapsol
            indsol = lastmapsol

         end if

         if ( isolut2d(indsol)==0 ) then

!        --- isol(indsol) = 0, so array isol has not been filled

            call insp01

         end if
         if ( isolut2d(indsol)==0 ) then

!        --- Array has not been filled

            if ( lastmapsol>lastmapbef ) then

!           --- A new array has been tried to create, reset

               lastmapsol = lastmapsol-1
               imapsol(ivec,itimst) = 0

            end if
            go to 200

         end if

         lastmapbef = lastmapsol
         itim = itim+1
         funcx (itim+5) = time (itimst)
         imesh = ispec(ivec,3)
         kmesh => kmesh2d(imesh)
         kprob => kprob2d(imesh)
         call sepfillkmesh(kmesh)
         ndimlc = ndim

         if ( ichois==3 ) then

!        --- ichois = 3,  find coordinate of given value along line

            fval = rinputpst(25)
            call prtimhislev ( isolut2d(indsol), fval, xval, intersect )
            if ( intersect ) then
               funcy (itim+5) = xval
            else
               itim = itim-1
            end if

         else if ( ichois==2 ) then

!        --- ichois = 2,  find solution in one point

            number = max(1,iinputpst(8))
            call sepalgebr ( 4, number, &
                            isolut2d(indsol), isolut2d, isolut2d, &
                            funcy(5+itim), dum, dum, dum, ipoint )
         else

!        --- ichois # 2,  Other situations

            if ( ichois<5 ) then
               ichvec = 11
            else
               ichvec = 12
            end if
            number = max(1,iinputpst(8))
            call sepvecman ( ichvec, number,isolut2d(indsol), isolut2d, &
                             isolut2d, constpst, iconstpst )

!           --- Fill funcy depending on iplot

            if ( ichois==0 .or. ichois==5 ) then
               funcy (itim+5) = constpst(2)
            else if ( ichois==1 .or. ichois==6  ) then
               funcy (itim+5) = constpst(1)
            end if

!           --- Compute ndim when itim = 1

            if ( itim==1 ) then
               if ( ks(isolut2d(indsol))%typevec==119 ) then

!              --- isol(2,....) = 119, array of type 119, i.e.
!                  one-dimensional

                  ndimlc = 1

               end if

            end if

         end if

!        --- Print result of time-step if iplot = 0

         if ( iplot==0 ) then

            if ( ichois==0 .or. ichois==5 ) then

!           --- ichois = 0 or 5

                call printb ( itim, ichois, funcx(itim+5), &
                              funcy(itim+5), ndimlc, constpst(6) )
            else if ( ichois==2 ) then

!           --- ichois = 2

                call printb ( itim, ichois, funcx(itim+5), &
                              funcy(itim+5), ndimlc, coor(1,ipoint) )
            else if ( ichois==3 ) then

!           --- ichois = 3, print time and position

                call printb ( itim, ichois, funcx(itim+5), &
                              funcy(itim+5), 0, constpst(6) )

            else

!           --- ichois # 0, 2, 3 or 5

                call printb ( itim, ichois,funcx(itim+5), &
                              funcy(itim+5), ndimlc, constpst(3) )
            end if

         end if

200   end do

      if ( iplot==1 ) then

!     --- iplot = 1,  Plot time history

         funcx(5) = itim
         funcy(5) = funcx(5)
         height = -rinputpst(2)
         if ( abs(height)<=0d0 ) height = -rinputpst(1)
         if ( iinputpst(71)==1 ) jtimes = 1

         number = 1

         multfact = rinputpst(3)

         if ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps ) then

!        --- multfact # 1, multiply solution

            do i = 1, iltime(2) - iltime(1) + 1
               funcy(i+5) = multfact * funcy(i+5)
            end do
         end if  ! ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps )

         multfact = rinputpst(42)  ! xscale

         if ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps ) then

!        --- multfact # 1, multiply time

            do i = 1, iltime(2) - iltime(1) + 1
               funcx(i+5) = multfact * funcx(i+5)
            end do
         end if  ! ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps )

         ntextx = iinputpst(37)
         ntexty = iinputpst(38)
         icplot(1) = 8
         icplot(2) = iinputpst(22)
         icplot(3) = iinputpst(23)
         icplot(4) = 1
         icplot(5) = iinputpst(21)
         icplot(6) = 0
         icplot(7) = number
         icplot(8) = iinputpst(77)
         rcplot(1) = rinputpst(1)
         rcplot(2) = height
         call plotfn1 ( icplot, rcplot, isolut2d, icurvs, funcx, funcy )

         if ( iinputpst(44)/=0 ) then
            if ( iinputpst(44)==1 ) then
               iinputpst(44) = 2
            end if
            call plafp7 ( xorgid, yorgid, identf, 0d0, .25d0, &
                          iinputpst(45) )
         end if

!        --- Update jtimes

         jtimes = jtimst
         if ( jtimes==11 .and. iinputpst(1)>=101 .and. &
               iinputpst(1)<130 ) jtimes = 12
         if ( jtimes==10 ) call plafep ( 2, 0d0, 0d0, 0d0, &
              0d0, 0 )

         if ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps ) then

!        --- multfact # 1, reset solution

            do i = 1, iltime(2) - iltime(1) + 1
               funcy(i+5) = funcy(i+5) / multfact
            end do
         end if

         multfact = rinputpst(42)  ! xscale

         if ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps ) then

!        --- multfact # 1, multiply time

            do i = 1, iltime(2) - iltime(1) + 1
               funcx(i+5) =  funcx(i+5)/multfact
            end do
         end if  ! ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps )

      end if
      call erclos ( 'insp04' )

      end subroutine insp04
