      subroutine intxstoretext ( inpsep, table, tabmax, strout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill reference to text and number of characters in inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     text
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision table, tabmax
      character (len=*) strout(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         Pos 1: sequence number of text in array texts
!                         Pos 2: number of characters in text
!     strout         i    Contains series of strings
!     table          i    Contains the numbers read for the keyword
!     tabmax         i    Contains the number of characters read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut

!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1887   Number of texts read too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxstoretext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxstoretext'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table>0 ) then

!     --- table>0, file name found

         iposut = nint(table)    ! position in tabout
         nchar  = nint(tabmax)   ! number of characters
         inpsep(2) = nchar
         if ( nextxt>ntexts ) then
            call errint ( ntexts, 1 )
            call errsub ( 1887, 1, 0, 0 )
            call errcrd
         else
            inpsep(1) = nextxt       ! sequence number of file
            texts(nextxt) = strout(iposut)  ! name of file
            nextxt = nextxt + 1
         end if

      else

!     --- table>0, file name found

         inpsep(1:2) = 0

      end if  ! ( table>0 )

1000  call erclos ( 'intxstoretext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intxstoretext'

      end if  ! ( debug )

      end
