      subroutine pravs2 ( nprev, iref, npoint, nunkp, usol )
! ======================================================================
!
!
!        programmer     Martien Hulsen
!        version 1.0    date   06-06-1993
!
!
!
!   copyright (c) 1993  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   Write solution array or array of special structure to file sepavs.xxx
!   for (x,y,t) data.
!   This routine should be called after PRAVS1.
!   This possibility is meant for a SEPRAN-AVS interface
!
!   The array should already have been mapped on a triangular mesh.
!   It is assumed that the solution vector contains only scalars in each
!   point or vectors
!   There are no points to be skipped
! ********************************************************************
!
!                      KEYWORDS
!
!   avs
!   output
!   vector
!   write
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer nprev, iref, nunkp, npoint
      double precision usol(nunkp,npoint)

!     nprev   i    number of array's that been written by previous calls
!                  to this routine.
!     iref    i    File reference number
!     npoint  i    Number of nodal points
!     nunkp   i    Maximum number of unknowns in a point
!     usol    i    Solution vector
! ********************************************************************
!
!                      COMMON BLOCKS
!
!    - &
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, j, k

!     i,j,k,  Counting variables
!
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    This subroutine writes the solution vector to the file indicated by iref.
!    The coordinates and topology (parts 1-5) have to be written by
!    pravs1 first.
!    This file is in standard formatted AVS format
!
!    Shape of the output:                 meaning
!
!    1  u1,1 u1,2 u1,3                  6. data values
!    2  u2,1 u2,2 u2,3
!          .
!          .
!          .
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   Write solution to file
! ======================================================================
!

!     --- Write solution to file

      k = nprev * npoint

      if ( nunkp.eq.1 ) then

!     --- nunkp = 1, scalar

         write(iref,420) ( i+k, usol(1,i), i=1,npoint )
420      format ( i6, e12.4 )

      else if ( nunkp.eq.2 ) then

!     --- nunkp = 2, 2D-vector

         write(iref,430) ( i+k, (usol(j,i), j=1,2), i=1,npoint )
430      format ( i6, 2e12.4, ' 0' )

      else

!     --- nunkp = 3, 3D-vector

         write(iref,440) ( i+k, (usol(j,i), j=1,3), i=1,npoint )
440      format ( i6, 3e12.4 )

      end if

      end
