      subroutine pravs3 ( iref, coor, ndim, npoint, nelem, npelm, &
                          kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write mesh information to file iref in AVS format
!
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     mesh
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref, ndim, npoint, nelem, npelm, kmeshc(npelm,nelem)
      double precision coor(ndim,npoint)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     iref    i    File reference number
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     ndim    i    Dimension of the space.
!     nelem   i    Number of elements
!     npelm   i    Number of points per element
!     npoint  i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     This subroutine writes the part co-ordinates and the topology
!     to the file indicated by iref.
!     This file is in standard formatted AVS format
!
!     Shape of the file:                 meaning
!
!     npoint, nelem, ndim, 0, 0          1. dimensions
!     1     x1 y1 z1                     2. node-id and co-ordinates
!     2     x2 y2 z2
!           .
!           .
!           .
!     1  1  cel-type i1 i2 i3 ...        3. node connectivity list
!     2  1  cel-type i1 i2 i3 ...
!           .
!           .
!           .
!     The rest of the file is filled in different subroutines
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Write dimension parameters to file
!     Write co-ordinates to file
!     Write topology to file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pravs3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pravs3'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Write co-ordinates to file

      if ( ndim==2 ) then

!     --- ndim = 2

         write(iref,200) ( i, (coor(j,i), j=1,2), i=1,npoint )
200      format ( i8, 2e12.4, ' 0' )

      else

!     --- ndim = 3

         write(iref,210) ( i, (coor(j,i), j=1,3), i=1,npoint )
210      format ( i8, 3e12.4 )

      end if

!     --- Write topology to file

      if ( npelm==3 ) then

!     --- triangles

         write(iref,300) ( i, (kmeshc(j,i), j=1,3), i=1,nelem )
300      format ( i8, ' 1 tri ', 3i8 )

      else if ( npelm==4 ) then

!     --- tetrahedra

         write(iref,310) ( i, (kmeshc(j,i), j=1,4), i=1,nelem )
310      format ( i8, ' 1 tet ', 4i8 )

      else

!     --- hexahedra

         write(iref,320) ( i, (kmeshc(j,i), j=1,8), i=1,nelem )
320      format ( i8, ' 1 hex ', 8i8 )

      end if

1000  call erclos ( 'pravs3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pravs3'

      end if

      end
