      subroutine prcompelem1 ( kmeshc1, kmeshc2, inpelm, inelem, &
                               ielgrp, same )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare elements in two meshes for one element group
!     node and element numbers may be different
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compare
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshc1(inpelm,inelem), ielgrp, &
              kmeshc2(inpelm,inelem)
      logical same

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     kmeshc1        i    Renumbered copy of kmeshc for mesh 1
!     kmeshc2        i    Renumbered copy of kmeshc for mesh 2
!     same           o    If true the arrays are the same
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT2DARR    Sort a 2-d array in increasing sequence
!                    First first entry, next second entry (if first are equal)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompelem1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompelem1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Reorder kmeshc1 and kmeshc2

      call chsort2darr ( kmeshc1, inelem, inpelm )
      call chsort2darr ( kmeshc2, inelem, inpelm )

!     --- Compare both arrays

      same = .true.
      do i = 1, inelem
         do j = 1, inpelm
            if ( kmeshc1(j,i)/=kmeshc2(j,i) ) same = .false.
         end do  ! j = 1, inpelm
         if ( .not. same ) then
            write(irefwr,*) 'element ', i, ' in element group ', ielgrp, &
                            ' is different for both meshes'
            call prinin ( kmeshc1(1,i), inpelm, 'kmeshc1' )
            call prinin ( kmeshc2(1,i), inpelm, 'kmeshc2' )
            go to 1000
         end if  ! ( .not same )
      end do  ! i = 1, inelem

1000  call erclos ( 'prcompelem1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompelem1'

      end if  ! ( debug )

      end
