      subroutine prcompuserp ( kmeshl2, inodes1, inodes2, same )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare user points in two meshes
!     nodes may be different
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compare
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshl2(*), inodes1(*), inodes2(*)
      logical same

!     inodes1        i    Ordering of coor1 for increasing coordinates
!     inodes2        i    Ordering of coor2 for increasing coordinates
!     kmeshl2        i    kmesh part l for mesh 2
!     same           o    If true the arrays are the same
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iuserp

!     iuserp         User point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompuserp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompuserp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      same = .true.

      do iuserp = 1, nuspnt

         if ( kmeshl(iuserp)>0 .and. kmeshl2(iuserp)>0 ) then

!        --- Both kmeshl and kmeshl2 have been filled

            if ( inodes1(kmeshl(iuserp))/=inodes2(kmeshl2(iuserp)) ) then

!           --- nodes have different sequence number

               write(irefwr,*) 'user point ', iuserp, &
                               ' is different for both meshes'
               same = .false.
               go to 1000

            end if  ! ( inodes1(kmeshl(iuserp))/=inodes2(kmeshl2(iuserp)) )

         else if ( kmeshl(iuserp)>0 .or. kmeshl2(iuserp)>0 ) then

!        --- Only one of the user points has been filled

            same = .false.
            write(irefwr,*) 'user point ', iuserp, &
                           ' has been filled for only one of the meshes'

         end if  ! ( kmeshl(iuserp)>0 .and. kmeshl2(iuserp)>0 )

      end do  ! iuserp = 1, nuspnt

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompuserp'

      end if  ! ( debug )
      call erclos ( 'prcompuserp' )

      end
