      subroutine prcopcoor ( iseqcoor, iseqinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 10-04-2013 extra debug
!        version  2.2  date 19-10-2012 extra debug
!        version  2.1  date 18-01-2012 extra debug
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates temporarily in solution vector stored in isolut
!     The sequence number in isolut is iseqcoor
!     The vector should be removed later on
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     coordinate
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqcoor, iseqinp

!     iseqcoor       o    Sequence number of coordinates vector in isolut
!                         This number is equal to numvct+2
!                         The vector must be removed at the end of the calling
!                         subroutine
!     iseqinp        i    Sequence number of input vector in isolut
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpsep(4), numvctcoor, istart

!                    they are not
!     inpsep         integer input array for subroutine PRCOPYCOOR
!     istart         dummy
!     numvctcoor     local copy of iseqcoor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPYCOOR     Copy coordinates into a vector
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopcoor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopcoor'
         write(irefwr,1) 'numvec, iseqinp, iseqcoor', &
                          numvec, iseqinp, iseqcoor
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set iseqcoor equal to numvct + 2
!         It must be removed later on

      iseqcoor = numvec+1
      numvec = numvec+1
      inpsep(1) = 0
      inpsep(2) = 1
      inpsep(3) = iseqcoor
      inpsep(4) = iseqinp
      numvctcoor = iseqcoor
      istart = 1
      if ( debug ) call prinin ( inpsep, 4, 'inpsep' )

      call prcopycoor ( inpsep, numvctcoor, istart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numvec', numvec
         write(irefwr,*) 'End prcopcoor'

      end if  ! ( debug )
      call erclos ( 'prcopcoor' )

      end
