      subroutine prcopycoor ( inpsep, numvct, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 03-12-2013 Add names to vectors
!        version  1.3  date 25-11-2012 Long integers
!        version  1.2  date 18-01-2012 Extension with x_coor
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates into a vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*), numvct, istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents
!                         1: sequence number (not used)
!                         2: problem number
!                         3: sequence number of output vector in isolut2d
!                         4: sequence number of input vector in isolut2d
!     istart        i/o   Next position in array inpsep
!                         istart is raised in this subroutine
!     numvct         i    Number of vectors stored in isolut2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ivec, ivect, number, iseqinp, iseqlc

!     iseqinp        Sequence number of input vector in isolut2d
!     iseqlc         Sequence number of vector
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     ivect          Sequence number of vector in isolut2d
!     number         Number of components
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PRCOPYCOOR1    Copy coordinates into a vector
!     PRCOPYCOOR2    Copy coordinates into a vector
!     PRININ         print 1d integer array
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set problem number and ndim
!     Find structure of vector with ndim degrees of freedom in nodes
!     with unknowns
!     Copy coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopycoor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopycoor'
         write(irefwr,1) 'ndtvec', ndtvec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set problem number and ndim

      ivect = inivectr(inpsep(3))   ! vector sequence number
      iprob = max(1,inpsep(2))  ! problem number
      iseqinp = inpsep(4)       ! defines which of the vectors is used
      numvct = max(numvct,ivect)  ! adapt numvct

      if ( debug ) then
         write(irefwr,1) 'iprob, ivect, ndim', iprob, ivect, ndim
         write(irefwr,1) 'iseqinp, numvct', iseqinp, numvct
      end if  ! ( debug )

!     --- Find structure of vector with ndim degrees of freedom in nodes
!         with unknowns

      itypvc = 110
      if ( iseqinp>0 .and. iseqinp<=maxvc1+1 .or. iseqinp==-1000 ) then

!     --- Coor is the vector

         number = ndim

      else

!     --- iseqinp = 1001 ... 1003 Hence one of the coordinates

         number = 1

      end if  ! ( abs(iseqinp)==1000

      if ( debug ) write(irefwr,1) 'number', number

      call sepcreastruc ( number, itypvc, ivec )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'itypvc, ivec, ndtvec', itypvc, ivec, ndtvec

!     --- Copy coordinates
!         First create array

      call sepcreasolbf1 ( isolut2d(ivect), itypvc, ivec, 'coor_cp', 0 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( isolut2d(ivect), 5, 'isolut2d(ivect)' )

!     --- Copy

      if ( iseqinp>0 .and. iseqinp<=maxvc1+1 .or. iseqinp==-1000 ) then

!     --- iseqinp = -1000 coordinates

         call prcopycoor1 ( usoltemp, kprobfo, indprfo, nphyso )

         names(ivect) = 'coor'

      else

!     --- iseqinp = -1001 ... -1003 Hence one of the coordinates

         if ( iseqinp<-1000 ) then
            iseqlc = -iseqinp-1000
         else
            iseqlc = iseqinp-maxvc1-1
         end if  ! ( iseqinp<-1000 )

         if ( debug ) write(irefwr,1) 'iseqlc, iseqinp', iseqlc, iseqinp

         call prcopycoor2 ( usoltemp, kprobfo, indprfo, &
                            nphyso, iseqlc )

         select case ( abs(iseqinp) )
            case(1)
               names(ivect) = 'x-coor'
            case(2)
               names(ivect) = 'y-coor'
            case(3)
               names(ivect) = 'z-coor'
         end select  ! case ( abs(iseqinp) )

      end if  ! ( iseqinp==-1000 ) then

      istart = istart+4   ! raise istart
1000  if ( debug ) then

!     --- Debug information

         call sepprintv ( isolut2d(ivect), 'coor' )
         write(irefwr,*) 'End prcopycoor'

      end if  ! ( debug )
      call erclos ( 'prcopycoor' )

      end
