      subroutine prcopymesh ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy mesh to different mesh number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscstoc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mesh1, mesh2, i

!                    they are not
!     i              Counting variable
!     mesh1          Mesh number of mesh to be copied
!     mesh2          Mesh number to which mesh must be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOPY        Copy important information of mesh
!     PRININ         print 1d integer array
!     PROCOPY        Copy important information of problem definition
!     SEPCOPYVC      Copy one SEPRAN vector into another one
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create structure for arry psi and initialize psi
!     Mark all nodes on start curve
!     While ( array psi has not been filled completely ) do
!        Find all neighbor elements in free surface that are not marked
!        Compute psi in these elements
!        Mark nodes in these elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopymesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopymesh'
         write(irefwr,1) 'istartmn', istartmn
         call prinin ( inpsep, 3, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants from inpsep

      mesh1  = inpsep(1)  ! Number of vectors
      mesh2  = inpsep(2)  ! dimension of problem
      if ( debug ) then
         write(irefwr,1) 'mesh1, mesh2', mesh1, mesh2
      end if  ! ( debug )

!     --- Copy mesh

      call mshcopy ( kmesh2d(mesh1), kmesh2d(mesh2) )

!     --- Copy problem definition

      call procopy ( kprob2d(mesh1), kprob2d(mesh2) )

!     --- Copy old solutions

      do i = 1, numvecmn
         if ( isolut3d(i,mesh1)>0 ) &
            call sepcopyvc ( isolut3d(i,mesh1), &
                            isolut3d(i,mesh2)  )
      end do  ! i = 1, numvecmn

!     --- Raise istartmn

      istartmn = istartmn+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopymesh'

      end if  ! ( debug )
      call erclos ( 'prcopymesh' )

      end
