      subroutine prcopyvecs1 ( uold, unew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-04-2003  Small adjustment (RvL)
!        version  1.0  date 24-01-2001
!
!   copyright (c) 2001-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy the contents of vector uold into vector unew with structure ivec
!    In those nodes where unew has less unknowns than uold entries of uold
!    are skipped
!    In those nodes where unew has more unknowns than uold the corresponding
!    entries are set equal to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision uold(*), unew(*)

!     unew           o    New solution vector
!     uold           i    Vector to be copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inusol, inunkp
      double precision u

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     u              solution to be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( indprpi>0 ) then
!        Use kprobpi to get variable from uold
!        Copy into unew
!     else if ( indprfi>0 ) then
!        Use kprobfi to get variable from uold
!        Copy into unew
!     else
!        Use nunkpi to get variable from uold
!        Copy into unew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( indprpi.gt.0 ) then

!     --- indprpi > 0, use kprobpi

         do i = 1, npoint

            do j = 1, nphysi

               inusol = kprobpi(i,j)
               if ( inusol.gt.0 ) then
                  u = uold(inusol)
               else
                  u = 0d0
               end if

               if ( indprpo.gt.0 ) then

!              --- indprpo > 0, use kprobpo

                  if ( j.le.nphyso ) then

!                 --- new degree of freedom exists

                     inusol = kprobpo(i,j)
                     if (inusol.gt.0) unew(inusol) = u

                  end if

               else if ( indprfo.gt.0 ) then

!              --- indprfo > 0, use kprobfo

                  inusol = kprobfo(i)
                  inunkp = kprobfo(i+1)-kprobfo(i)
                  if ( j.le.inunkp ) then

!                 --- new degree of freedom exists

                     unew(inusol+j) = u

                  end if

               else

!              --- indprfo = 0, use nphyso

                  if ( j.le.nphyso ) then

!                 --- new degree of freedom exists

                     inusol = (i-1)*nphyso+j
                     unew(inusol) = u

                  end if

               end if

            end do

         end do

      else if ( indprfi.gt.0 ) then

!     --- indprfi > 0, use kprobfi

         do i = 1, npoint

            inunkp = kprobfi(i+1)-kprobfi(i)
            do j = 1, inunkp

               inusol = kprobfi(i)+j
               if ( inusol.gt.0 ) then
                  u = uold(inusol)
               else
                  u = 0d0
               end if

               if ( indprpo.gt.0 ) then

!              --- indprpo > 0, use kprobpo

                  if ( j.le.nphyso ) then

!                 --- new degree of freedom exists

                     inusol = kprobpo(i,j)
                     unew(inusol) = u

                  end if

               else if ( indprfo.gt.0 ) then

!              --- indprfo > 0, use kprobfo

                  inusol = kprobfo(i)
                  inunkp = kprobfo(i+1)-kprobfo(i)
                  if ( j.le.inunkp ) then

!                 --- new degree of freedom exists

                     unew(inusol+j) = u

                  end if

               else

!              --- indprfo = 0, use nphyso

                  if ( j.le.nphyso ) then

!                 --- new degree of freedom exists

                     inusol = (i-1)*nphyso+j
                     unew(inusol) = u

                  end if

               end if

            end do

         end do

      else

!     --- indprfi = 0, use nunkpi

         do i = 1, npoint

            do j = 1, nunkpi

               inusol = (i-1)*nunkpi+j
               if ( inusol.gt.0 ) then
                  u = uold(inusol)
               else
                  u = 0d0
               end if

               if ( indprpo.gt.0 ) then

!              --- indprpo > 0, use kprobpo

                  if ( j.le.nphyso ) then

!                 --- new degree of freedom exists

                     inusol = kprobpo(i,j)
                     unew(inusol) = u

                  end if

               else if ( indprfo.gt.0 ) then

!              --- indprfo > 0, use kprobfo

                  inusol = kprobfo(i)
                  inunkp = kprobfo(i+1)-kprobfo(i)
                  if ( j.le.inunkp ) then

!                 --- new degree of freedom exists

                     unew(inusol+j) = u

                  end if

               else

!              --- indprfo = 0, use nphyso

                  if ( j.le.nphyso ) then

!                 --- new degree of freedom exists

                     inusol = (i-1)*nphyso+j
                     unew(inusol) = u

                  end if

               end if

            end do

         end do

      end if

      end
