      subroutine prcopyvecs2 ( uold, unew, nunkp, npoint, kprobf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-06-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy the contents of vector uold into vector unew with structure ivec
!    In those nodes where unew has more unknowns than uold the corresponding
!    entries are set equal to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  nunkp, npoint, kprobf(npoint+1)
      double precision uold(*), unew(nunkp,npoint)

!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     unew           o    New solution vector
!                         This one has exactly nunkp unknowns per point
!     uold           i    Old solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inunkp, istart

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyvecs2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyvecs2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array unew

      unew = 0d0

!     --- Copy uold into unew

      do i = 1, npoint

         istart = kprobf(i)
         inunkp = kprobf(i+1)-istart
         if ( inunkp>0 ) unew(1:inunkp,i) = uold(istart+1:istart+inunkp)

      end do  ! i = 1, npoint

1000  call erclos ( 'prcopyvecs2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopyvecs2'

      end if  ! ( debug )

      end
