      subroutine prcopyvectran1 ( usol, npoint, ucopy, nunkp, &
                                  nphysorig )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 09-04-2014 Extra parameter nphysorig
!        version  2.0  date 09-02-2007 Remove kprob
!        version  1.0  date 11-07-2005
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution from array usol stored in standard sepran sequence
!     to array ucopy organized in columns of unknowns
!     It is supposed that kprob part p has been filled
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nunkp, nphysorig
      double precision usol(*), ucopy(npoint,*)

!     nphysorig      i    Number of physical degrees of freedom before reduction
!     npoint         i    Number of nodal points
!     nunkp          i    number of unknowns to be copied
!     ucopy          o    array of length npoint x nunkp that is filled with
!                         the copied solution
!     usol           i    Solution vector in standard sequence
!                         kprobp must have been filled
!                         It is assumed that usol contains at least nunkp
!                         unknowns per point
!                         Only the first nunkp are copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyvectran1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyvectran1'

      end if
      if ( ierror/=0 ) go to 1000

      do i = 1 , npoint
         do j = 1, nunkp
            ucopy(i,j) = usol((i-1)*nphysorig+j)
         end do  ! j = 1, nunkp
      end do  ! i = 1 , npoint

1000  call erclos ( 'prcopyvectran1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopyvectran1'

      end if

      end
