      subroutine prcorrinterp2 ( coorold, interfaceold, nofaceold, &
                                 coornew, interfacenew, nofacenew, &
                                 oldnodes, meshvel, dt, lensto, &
                                 istore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-04-2013 New parameter list
!        version  1.0  date 15-11-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct interpolation in case nodes of the mesh crossed
!     the old object (2d)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     level_set
!     node
!     intersection
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nofaceold, interfaceold(nofaceold), nofacenew, &
              interfacenew(nofacenew), lensto, istore(*), &
              oldnodes(*)
      double precision coorold(2,*), coornew(2,*), meshvel(2,*), dt

!     coornew        i    Array containing the new coordinates of the nodes
!     coorold        i    Array containing the coordinates in the old mesh
!     dt             i    Time step
!     interfacenew   i    Contains node numbers of new interface
!     interfaceold   i    Contains node numbers of old interface
!     istore         i    work array of length npoint to store nodes for which
!                         phi1 and phi2 have different sign
!     lensto         i    Number of nodes in istore
!     meshvel       i/o   Contains mesh velocity
!     nofacenew      i    Number of nodes in interfacenew
!     nofaceold      i    Number of nodes in interfaceold
!     oldnodes       o    Contains the set of old nodes corresponding to
!                         the nodes in interfacenew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRLEVINTERFCP  Interpolate the concentration at the new interface from
!                    values at the old interface in case of a level set method
!                    Compute the corresponding mesh velocity
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcorrinterp2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcorrinterp2'
         write(irefwr,1) 'lensto', lensto
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill the concentration at the interface and
!         compute the mesh velocity at the interface

      if ( debug ) write(irefwr,*) 'first call to prlevinterfcp'
      call prlevinterfcp ( coorold, interfaceold, nofaceold, &
                           coornew, interfacenew, nofacenew, &
                           oldnodes, meshvel, dt )

      if ( lensto>0 ) then

!     --- Lensto>0, it must also be done for nodes outside the interface

         if ( debug ) write(irefwr,*) 'second call to prlevinterfcp'
         call prlevinterfcp ( coorold, interfaceold, nofaceold, &
                              coornew, istore, lensto, &
                              oldnodes(nofacenew+1), meshvel, dt )

      end if  ! ( lensto>0 )
1000  call erclos ( 'prcorrinterp2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcorrinterp2'

      end if  ! ( debug )

      end
