      subroutine preigval ( iineig )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-04-2012 Indexed scalars
!        version  1.0  date 05-04-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help subroutine to be called by sepcomp or sepfree
!     This subroutine calls the actual eigenvalue subroutine eigenvbf
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
!     eigenvector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemnsub
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iineig(*)

!     iineig     i    Array containing information about the eigenvalue
!                     problem
!                     Contents
!                     1: contains the sequence number for the input
!                        -1 = no input
!                     2: IVECTOR, vector sequence number of first eigenvector
!                        If 0 no eigenvctors are computed
!                     3: ISCALAR, scalar sequence number of first scalar
!                     4: IPROB, problem sequence number
!                     5: NEIGV, number of eigenvalues
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpeig(6), iprob, iuser(1), neigv, iscal
      double precision user(1)

!     inpeig     Local array containing information about the eigenvalue
!                problem as used by eigenvbf
!                Contents: see subroutine eigenvbf
!     iprob      Actual problem number
!     iscal          Sequence number of scalar
!     iuser      Integer user array to pass user information from
!                main program to subroutine. See STANDARD PROBLEMS
!     neigv      Number of eigenvalues
!     user       Real user array to pass user information from
!                main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex

!     EIGENVBF       computes eigenvalues and eigenvectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETSCALINDEX   Return with adapted index of scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2053   Too many eigenvalues
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'preigval' )

!     --- Fill array inpeig, i.e. local array iineig in eigenvb

      inpeig(1) = 6
      inpeig(2) = 0
      inpeig(3) = iineig(1)
      inpeig(4) = iineig(2)
      inpeig(5) = numvecmn
      iprob     = iineig(4)
      neigv     = iineig(5)
      inpeig(6) = neigv
      iscal = abs(getscalindex ( iineig(3) ))
      intmat => intmat1d(iprob)
      call eigenvbf ( inpeig, iuser, user, scalars(iscal) )
      numvecmn = inpeig(5)
      if ( iscal+neigv-1>maxcls ) then

!     --- Too many eigenvectors to store in isolut2d
!         Give warning and exclude computation of eigenvectors

         call errint ( neigv, 1 )
         call errint ( iineig(3), 2 )
         call errint ( maxcls, 3 )
         call errsub ( 2053, 3, 0, 0 )

      end if
      istartmn = istartmn + 6
      call erclos ( 'preigval' )
      end
