      subroutine prfindclosnod2 ( grid, cells, nactive, nnodes, istore, &
                                  nx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all nodes in the set of active cells and store in istore
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     cell
!     find_node
!     find
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nactive, cells(2,nactive), nnodes, istore(*), grid(*), nx

!     cells          i    Contains (i,j)-indices of active cells
!     grid           i    In this array the nodes of the interface are
!                         coupled to a rectangular grid of size nx * ny
!                         The decription of this array is the same as that of
!                         kmesh part ac subparts 2 and 3
!     istore         o    Contains the nodes in the acive cells
!     nactive        i    Number of active cells
!     nnodes         o    Number of nodes in istore
!     nx             i    number of points in x direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, indi, indj, ipos, nump

!     i              Counting variable
!     indi           i-index
!     indj           j-index
!     ipos           Position in array
!     nump           Number of points in a cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindclosnod2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindclosnod2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0
      do i = 1, nactive

         indi = cells(1,i)
         indj = cells(2,i)
         ipos = (indj-1)*nx+indi
         nump = grid(ipos+1)-grid(ipos)
         ipos = grid(ipos)-1
         istore(nnodes+1:nnodes+nump) = grid(ipos+1:ipos+nump)
         nnodes = nnodes+nump

         if ( debug ) then
            write(irefwr,1) 'i, indi, indj', i, indi, indj
            write(irefwr,1) 'ipos, nump', ipos, nump
         end if  ! ( debug )

      end do  ! i = 1, nactive

1000  call erclos ( 'prfindclosnod2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindclosnod2'

      end if  ! ( debug )

      end
