      subroutine prforcev ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.6  date 17-07-2015 Initialize matr
!        version  1.5  date 24-12-2014 New definition clinsl
!
!   copyright (c) 1998-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill the force vector with a right-hand side vector
!    This subroutine is meant for the case that a distributed load must be
!    translated to a point load
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekprob
      use sepmoduleinput
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)

!     inpsep  i    The integer input read is stored in array INPSEP
!                  1  ISEQCF     Sequence number corresponding to input for
!                                coefficients
!                                If iseqcf = -1, the next one will be used
!                  2  IPROB      Problem sequence number
!                  3  IVECTOR    Vector sequence number (first vector)
!                                If ivector = -1, the same value as iprob
!                                will be used
!                  4: LINEAR_SUB indicates if quadratic elements should be
!                                treated as a cluster of linear subelements
!                                0: no subelements
!                                1: subelements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ivec, iseqcf, ivectr, iinbldlc(15), &
              iprobr, ictime

!     ictime         dummy
!     iinbldlc       User input array for subroutine BUILD
!     iprobr         Problem number read
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqlc           Sequence number of vector in VECOLD/ISLOLD
!     ivec           Sequence number of vector of special structure
!     ivectr         standard sepran array denoting USOL or vector of
!                    special structure defined in nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILD       assemble a system of equations
!     SEPFILLCOEFFS  Fill coefficients at the end of the buffer array
!                    with starting addresses ipiusr and ipuser
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('prforcev')

      iseqcf = 1
      iprob  = 1
      iseqlc   = inpsep(1)
      if ( iseqlc/=-1 ) iseqcf = iseqlc
      iseqlc = iinputarrs(6)%maxcall
      iseqcf = min ( iseqcf, iseqlc )
      iprobr = inpsep(2)
      if ( iprobr/=-1 ) iprob = iprobr
      ivectr = iprob
      ivec   = inpsep(3)
      if ( ivec/=-1 ) ivectr = ivec

!     --- Fill coefficients in IUSER and USER

      call sepfillcoeffs ( 3, iseqcf, ictime )
      if ( ierror/=0 ) go to 1000

!     --- The right-hand side vector is built  (call of build)

      iinbldlc(1) = 15
      if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) &
         write ( irefwr, 150 ) ivectr
150   format ( ' Build right-hand side force vector', i3 )
      iinbldlc(2) = 2
      iinbldlc(3) = 1
      iinbldlc(4:8) = 0
      iinbldlc(9) = iprob
      iinbldlc(10) = numvecmn
      iinbldlc(11) = 0
      iinbldlc(12) = 1
      iinbldlc(13) = 1
      iinbldlc(14) = 0
      iinbldlc(15) = inpsep(4)
      call sepfillsmf ( iinbldlc, isolut2d(ivectr), isolut2d, isolut2d )
      call sepbuild

      numvecmn = max(numvecmn,ivectr)
      istartmn = istartmn + 5

1000  call erclos('prforcev')
      end
