      subroutine prinoutvec ( vector, nlines )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 01-03-2006 Correction format
!        version  2.0  date 01-04-2000 Extra parameter nfictunk
!        version  1.0  date 13-10-1999
!
!   copyright (c) 1999-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write sepran real solution vector to file sepcomp.out
!
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vector(*)
      integer nlines

!     nlines         i    number of lines that are printed
!     vector         i    solution vector to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, nusolclassic
      logical skip
      double precision locvalue(20)

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     locvalue       Local array to store some values
!     nusolclassic   Number of free classical unknowns, i.e ficitious unknowns
!                    and global unknowns excluded
!     skip           If true the next part must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinoutvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinoutvec'
         call ertrace
         write(irefwr,1) 'nlines, nphysi, indprfi, indprpi, nglunk, nusoli', &
                          nlines, nphysi, indprfi, indprpi, nglunk, nusoli
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprpi/=0 ) then

!     --- indprpi#0, use kprob part p

         do i = 1, nlines
            if ( indprfi==0 ) then
               skip = .false.
            else if ( kprobfi(i+1)-kprobfi(i)==0 ) then
               skip = .true.
            else
               skip = .false.
            end if
            if ( .not. skip ) then
               do j = 1, nphysi
                  if ( kprobpi(i,j)==0 ) then
                     locvalue(j) = -rinfin
                  else
                     locvalue(j) = vector(kprobpi(i,j))
                  end if
               end do

               if ( iref74>0 ) then
                  write(iref74,100) i, (locvalue(j),j=1,nphysi)
100               format(i6,20d17.9/(6x,20d17.9))
               else
                  write(-iref74) i, (locvalue(j),j=1,nphysi)
               end if

            end if
         end do

      else if ( indprfi/=0 ) then

!     --- indprfi#0, use kprob part f

         do i = 1, nlines
            inunkp = kprobfi(i+1)-kprobfi(i)
            if ( inunkp>0 ) then
               if ( iref74>0 ) then
                  write(iref74,100) i, (vector(j+kprobfi(i)),j=1,inunkp)
               else
                  write(-iref74) i, (vector(j+kprobfi(i)),j=1,inunkp)
               end if
            end if
         end do

      else

!     --- Standard situation, indprfi=0 and indprpi=0

         do i = 1, nlines
            if ( iref74>0 ) then
               write(iref74,100) i, (vector(j+(i-1)*nphysi),j=1,nphysi)
            else
               write(-iref74) i, (vector(j+(i-1)*nphysi),j=1,nphysi)
            end if
         end do

      end if

      nusolclassic = nusoli-nfictunk-nglunk
      if ( nfictunk>0 ) then

!     --- nfictunk>0, print fictitious unknowns

         if ( iref74>0 ) then
            write(iref74,100) i, (vector(j+nusolclassic),j=1,nfictunk)
         else
            write(-iref74) i, (vector(j+nusolclassic),j=1,nfictunk)
         end if

      end if

      nusolclassic = nusolclassic+nfictunk
      if ( nglunk>0 ) then

!     --- nglunk>0, print global unknowns

         if ( iref74>0 ) then
            write(iref74,100) i, (vector(j+nusolclassic),j=1,nglunk)
         else
            write(-iref74) i, (vector(j+nusolclassic),j=1,nglunk)
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinoutvec'

      end if  ! ( debug )
      call erclos ( 'prinoutvec' )

      end subroutine prinoutvec
