      subroutine prlevinterfst ( ipointobst, istart, numbndobst, ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting address of interface in array ipointobst
!
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     level_set
!     node
!     interface
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), istart, numbndobst, ilev

!     ilev           i    Sequence number of level being considered
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     istart         o    Starting address of interface in ipointobst
!     numbndobst     o    Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numpntobst, numinnobst

!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlevinterfst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlevinterfst'
         call ertrace
         write(irefwr,1) 'ilev, istart', ilev, istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = ipointobst(ilev)
      numpntobst = ipointobst(istart)
      numinnobst = ipointobst(istart+1)
      numbndobst = ipointobst(istart+2)
      istart = istart+4+numpntobst+numinnobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlevinterfst'

      end if  ! ( debug )
      call erclos ( 'prlevinterfst' )

      end
