      subroutine prmakesignvec3 ( uin, uout, n, umult, nunkp, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 18-07-2012 Check on division by zero
!        version  1.0  date 28-05-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new vector V by division of
!     input vector 1 by input vector 2 (componentwise)
! **********************************************************************
!
!                       KEYWORDS
!
!     sign
!     division
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nunkp, inpsep(*)
      double precision uout(nunkp,n), uin(nunkp,n), umult(n)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prmanv for a description
!     n              i    number of elements in sign vector
!     nunkp          i    Maximum number of unknowns in a point
!     uin            i    sign vector
!     umult          i    Multiplication vector
!     uout           o    Output vector (sign of uin times umult)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ispec, ivecmult, ivecsign

!     i              Counting variable
!     ispec          Parameter in inpsep denoting special options
!                      0:  Compute V1 * V2
!                      1:  Compute sign(V1) * V2
!     ivecmult       ivec for multiplication vector
!     ivecsign       ivec for sign vector
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3109   Subdivision by zero when two vectors are subdivided
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakesignvec3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakesignvec3'
         write(irefwr,1) 'n, nunkp', n, nunkp
         call prinrl ( umult, n, 'umult' )
         call prinrl1 ( uin, n, 2, 'uin' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ispec = inpsep(7)       ! ispec, defines type of multiplication
      if ( debug ) write(irefwr,1) 'ispec', ispec

      select case ( ispec )

      case (0)

!     --- V1 / V2

         do j = 1, nunkp

            do i = 1, n

               if ( umult(i)==0d0 ) then

                  if ( uin(j,i)/=0d0 ) then

!                 --- Error subdivision by zero

                     ivecmult = inpsep(10)    ! seq number of vector for div
                     ivecsign = inpsep(9)   ! seq number of vector for sign

                     call errchr ( names(ivecsign), 1 )
                     call errchr ( names(ivecmult), 2 )
                     call errchr ( names(ivecmult), 3 )
                     call errint ( i, 1 )
                     call errsub ( 3109, 1, 0, 3 )
                     go to 1000

                  end if  ! ( uin(j,i)/=0d0 )
                  uout(j,i) = uin(j,i)

               else

                  uout(j,i) = uin(j,i)/umult(i)

               end if  ! ( umult(i)==0d0 )

            end do  ! i = 1, n

         end do  ! j = 1, nunp

      end select  ! case ( ispec )

1000  call erclos ( 'prmakesignvec3' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakesignvec3'

      end if  ! ( debug )

      end
