      subroutine prmanv ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 04-09-2016 New parameter list
!        version  4.15 date 12-02-2015 New call to prstorevec
!        version  4.14 date 28-02-2014 Extension with constpstant vector
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call MANVEC for vector manipulation from program SEPCOMP
!     Use input from array INPSEP to make the choices
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP of
!                         length 20+nelgrp
!                         Array INPSEP is filled as follows:
!                          1  ISEQNR       Sequence number corresponding to
!                                          input.  If iseqnr = -1,
!                                          the next one will be used
!                          2  IPROB        Problem sequence number
!                                          If iprob = -1, the next one will be
!                                          used
!                          3  IVECTOR      Vector sequence number
!                                          (output vector)
!                                          If ivector = -1, the same value as
!                                          iprob will be used
!                          4: ichoice (See manvec)
!                          5: IDEGFD (See manvec)
!                          6: IPOINT (See manvec)
!                          7: ISPEC  (See manvec)
!                          8: ICHNRM (See manvec)
!                          9: Vector sequence number first input vector
!                         10: Vector sequence number second input vector
!                         11: Scalar sequence number first scalar
!                         12: Scalar sequence number second scalar
!                         13: Sequence number first position in RINSEP
!                         14: jskip  defines if element groups must be skipped
!                             (>0) or not (0)
!                             If >0 (21) positions jskip to jskip+nelgrp-1 are
!                             used to indicate if an element group is skipped
!                             or not
!                             0: do not skip
!                             1: skip
!     rinsep         i    Real input array (not yet used)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinvec(20+nelgrp), ivec2, ivec3, iscal1, iscal2, ichoice, &
              i, jskip, istrin, iseqarrsav, iseqarrmin, iseqarrmax, &
              iseqcoor, numvecsav, numarrays, numarray2, numarray3, &
              numscalars1, numscalars2, numarray1, numelgrps, ivectr, leng
      double precision rinvec(9)

!                    they are not
!     i              Counting variable
!     ichoice         Choice parameter, see iinvec
!     iinvec         In this integer input array it is indicated what type of
!                    manipulations must be preformed by subroutine MANVEC.
!                    For some cases IINVEC acts both as input and
!                    as output array.
!                    The entries of IINVEC must be filled by the user as follows
!                     1  Number of entries of IINVEC that are filled by the user
!                        IINVEC(1) must be at least equal to 2.
!                        For positions that are not filled by the user (except
!                        the positions 1 and 2) defaults are used.
!                     2  ichoice   General choice parameter indicating what type
!                                 of algebraic manipulations have to be carried
!                                 out.
!                                 ichoice must always have been given a value by
!                                 the user.
!                        Description of the possible values of ichoice
!                           1     Average: c = average (u1)
!                           2     Extract value of one degree of freedom in one
!                                 point from u1.
!                           3     Minimum and maximum value of u1 or | u1 |.
!                           4     Inner product:  c = ( u1 , u2 ) .
!                           5     || u1 ||  or || u1 - u2 ||
!                          26     Subtraction of constpstant:  u out = u1 - c
!                          27     Linear combination:  u out = c1 u1 +  c2 u2
!                          28     Put value of one degree of freedom in one
!                                 point into  u out
!                          29     Modulus of complex vector.
!                          30     Phase of complex vector.
!                          31     User defined function of two SEPRAN vectors:
!                                 u out =  f ( u1 , u2 ) .
!                          32     User defined function of  n  SEPRAN vectors
!                                 and the co-ordinates:
!                                 u out =  f ( u1 , u2 , + . + , u n, x ) .
!                          33     Length of vector composed by components:
!                                 u out (i) = sqrt ( u1 (i) ** 2 + u2 (i) ** 2 )
!                                 or
!                                 u out (i) = sqrt(u1(i)**2+u2(i)**2+u3(i)**2)
!                          34     u out (i) = conjg( u1(i) )
!                          35     SEPRAN function FUNADA of two SEPRAN vectors:
!                                 u out =  f ( u1 , u2 ) .
!                          36     Put degree of freedom IDEGFD into a vector
!                                 consisting of one unknown in each point
!                                 It is supposed that the original vector has
!                                 the degree of freedom in each point
!                          37     Real part of complex vector
!                          38     Imaginary part of complex vector
!                          51     U out is a map of the SEPRAN vector u1:, i.e.
!                                 the SEPRAN vector u1 is transformed to a new
!                                 structure defined by the parameters IPROB and
!                                 ISPEC (=IVEC). If ISPEC=0 the output vector is
!                                 of type 110, otherwise it is of type 115.
!                                 Restriction: the new and old vector must
!                                 correspond to the same number of nodes and the
!                                 number of degrees of freedom per node must be
!                                 the same for both vectors.
!                                 So in fact Uout is a rearranged u1
!                         101     U_out is mesh velocity (x^n+1 -x^n)/dt
!                                 ivec1 refers to x^n+1, ivec2 refers to x^n
!                         102     U_1 is constpstant
!                                 The value must be stored in rinsep
!                                 The vector U_1 refers to IVEC1 and this vector
!                                 must be an existing vector
!                         103     Transform from integrated to nodal values
!                         104     Transform to normal and tangential dirs
!                         105     Transform stress tensor along boundary from
!                                 Cartesian to vector using the normal
!                                 components along curves
!                         106     Make new vector out of two old vectors
!                                 The components of the first vector are the
!                                 first ones, the components of the second
!                                 vector are the last
!                         107     V1 = func(V2) or  V1 = func(V2) * V3
!                                 If ISPEC = 0:  FUNC = 1
!                                            1:  FUNC = sign
!                         108     V1 = V2 / V3
!                         109     mean_area
!                         110     min_area
!                         111     point_number
!                         112     mean value
!                         113     standard deviation
!                         114     transform to other coordinate system
!                                 System defined by ispec:
!                                 1:  polar coordinates
!                         115     vector expression
!                         116     vector = constpstant
!                     3  IDEGFD   Depending on the value of IDEGFD, the
!                                 algebraic manipulations will be performed with
!                                 respect to all degrees of freedom of the
!                                 vectors (IDEGFD=0), with all internal
!                                 degrees of freedom i.e. not with the
!                                 essential boundary conditions (IDEGFD=-1), &
!                                 or with the IDEGFD th  degree of
!                                 freedom only (IDEGFD>0).
!                                 Default value: IDEGFD = 0
!                     4  IPOINT   Nodal point number.
!                                 Default value: IPOINT = 1
!                     5  ISPEC    The use of ISPEC depends on the VALUE of
!                                 ichoice.
!                                 Default value: Depending on ichoice.
!                     6  ICHNRM   The use of ICHNRM depends on the VALUE of
!                                 ichoice.
!                                 Default value: ICHNRM = 3
!                                 In case ichoice = 115: sequence number of
!                                 expression
!                     7  ISKIP    Depending on the value of ISKIP element groups
!                                 must be skipped in the computation (ISKIP>20)
!                                 or not (ISKIP=0).
!                                 If ISKIP>20, the value of ISKIP denotes the
!                                 starting position in array IINVEC where
!                                 information concerning the elements to be
!                                 skipped is stored.
!                                 In that case it is necessary that the user has
!                                 filled the positions IINVEC(ISKIP) , + . + , &
!                                 ISKIP(ISKIP+ nelgrp -1), where  nelgrp
!                                 denotes the number of element groups. Only
!                                 element groups defined in the mesh are taken
!                                 into account, so boundary element groups are
!                                 not considered as special class. ISKIP in the
!                                 range 1<ISKIP<10 is not allowed.
!                                 If IINVEC(ISKIP+ ielgrp -1) = 0, element group
!                                 ielgrp  must not be skipped, if
!                                 IINVEC(ISKIP+ ielgrp -1) = 1, the element
!                                 group must be skipped.
!                                 Default value: ISKIP = 0
!                     8  IDEGFD1  First degree of freedom in a point.
!                                 Default value: IDEGFD1 = 1
!                     9  IDEGFD2  Second degree of freedom in a point.
!                                 Default value: IDEGFD1 = 2
!                    10  IDEGFD3  Third degree of freedom in a point.
!                                 Default value: IDEGFD1 = 3
!                    11  IPROB    Problem number
!                                 Default value: IPROB = 1
!                    12  IVEC1    Sequence number of input vector 1
!                                 (only used for error messages)
!                                 Default value: IVEC1 = 1
!                    13  IVEC2    Sequence number of input vector 2
!                                 (only used for error messages)
!                                 Default value: IVEC2 = 2
!     iscal1         Sequence number for first scalar
!     iscal2         Sequence number for second scalar
!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqcoor       Sequence number of coordinates vector is isolut
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     ivec2          Sequence number of second vector
!     ivec3          Sequence number of third vector
!     ivectr         Sequence number of first vector
!     jskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     leng           size of array
!     numarray1      Number of indexed vectors in vector array (ivectr)
!     numarray2      Number of indexed vectors in vector array (ivec1)
!     numarray3      Number of indexed vectors in vector array (ivec2)
!     numarrays      Number of indexed vectors or scalars
!     numelgrps      Number of element groups in inpsep
!     numscalars1    Number of indexed scalars in scalar array (iscal1)
!     numscalars2    Number of indexed scalars in scalar array (iscal2)
!     numvecsav      Help variable to store the value of numvec
!     rinvec         Double precision input and output array. The contents of
!                    RINVEC depend on the contents of IINVEC.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREDUCESCAL  Compute the reduced value of iscalin which indicates the
!                    first sequence number of a (series of) scalars, as well as
!                    the dimension of the series
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRCOPYCOORBCK  Copy coordinates stored in solution vector back
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRMANVSUB      Vector/scalar manipulation subroutine
!     PRSTOREVEC     Adapt vector sequence number and copy coor if necessary
!                    in new vector
!     PRVECCOMBINAT  Evaluate right-hand side of vector expression
!     PRVECCONST     Create vector in isol and give constpstant value
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3129   dimension of 2 series of vectors not the same
!    3130   dimension of 2 series of vectors or scalars not the same
!    3131   dimension of 2 series of vectors or scalars not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmanv' )

      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from prmanv'
         write(irefwr,1) 'numvecmn', numvecmn
         call prinin ( inpsep, 21, 'inpsep' )
         call prinrl ( rinsep, 3, 'rinsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000
      iseqcoor = 0
      numvecsav = numvec

      if ( inpsep(4)==115 ) then

!     --- Special case: vector is expression of vectors

         call prveccombinat ( inpsep )
         ivectr = inpsep(3)
         numarray1 = 0
         go to 900

      else if ( inpsep(4)==116 ) then

!     --- Special case: vector is constant

         call prvecconst ( inpsep, rinsep )
         ivectr = inpsep(3)
         numarray1 = 0
         go to 900

      end if  ! ( inpsep(4)==115 )

!     --- Compute ivectr and store coor if necessary

      call prstorevec ( iseqcoor, ivectr, inpsep(3) )
      ivectr = max(1,ivectr)

      numvecmn = max(ivectr, numvecmn)

      call inireducevec ( ivectr, ivectr, numarray1 )

      if ( debug ) write(irefwr,1) 'ivectr, numarray1', ivectr, numarray1

!     --- Fill array iinvec from inpsep

      iinvec(1) = 20+nelgrp
      do i = 2, 6
         iinvec(i) = inirepscal(inpsep(i+2))
      end do  ! i = 2, 6
      iinvec(8) = 1
      iinvec(9) = 2
      iinvec(10) = 3
      iinvec(11) = inirepscal(inpsep(2))
      rinvec(1) = 0d0
      rinvec(2) = 0d0

!     --- Get ivec2 and ivec3; if necessary store coordinates in vector

      call prstorevec ( iseqcoor, ivec2, inpsep(9) )
      ivec2 = max(1,ivec2)

      call inireducevec ( ivec2, ivec2, numarray2 )

      call prstorevec ( iseqcoor, ivec3, inpsep(10) )

      call inireducevec ( ivec3, ivec3, numarray3 )
      if ( debug ) &
         write(irefwr,1) 'ivectr, ivec2, ivec3, numarray2, numarray3', &
                          ivectr, ivec2, ivec3, numarray2, numarray3

      call inireducescal ( inpsep(11), iscal1, numscalars1 )
      call inireducescal ( inpsep(12), iscal2, numscalars2 )

      if ( debug ) &
         write(irefwr,1) 'numscalars1, numscalars2', &
                          numscalars1, numscalars2
      ichoice   = iinvec(2)
      jskip     = inirepscal(inpsep(14))
      istrin = inirepscal(inpsep(13))
      if ( debug ) write(irefwr,1) 'istrin', istrin
      if ( istrin>0 ) then
         rinvec(1) = rinsep(istrin)
         if ( ichoice/=102) rinvec(2) = rinsep(istrin+1)
      end if
      iinvec(7) = jskip
      if ( debug ) then
         call prinin ( iinvec, 12, 'iinvec' )
         write(irefwr,2) 'rinvec ', rinvec(1), rinvec(2)
      end if  ! ( debug )

      ivec3 = max(1,ivec3)  ! to prevent dimension problems

!     --- Clear positions 14 to 20+nelgrp
!         14-20 are presently not used

      iinvec(14:20) = 0          ! clear this part
      iinvec(21:20+nelgrp) = 1   ! skip all element groups

      if ( jskip>0 ) then

!     --- jskip>0, Copy information about element groups to be skipped

         numelgrps = inpsep(jskip)
         do i = 1, numelgrps
            iinvec(jskip+inpsep(jskip+i)-1) = 0  ! elements to be used
         end do  ! i = 1, numelgrps

      end if
      if ( debug ) write(irefwr,1) 'ichoice', ichoice

!     --- Loop over arrays

      numarrays = max(numarray1,numscalars1)

      if ( numarrays>1 ) then

!     --- numarrays>1, check dimensions

         if ( numarray1>1 .and. numarray1/=numarrays ) then
            call errint ( numarray1, 1 )
            call errint ( numarrays, 2 )
            call errchr ( names(ivectr), 1 )
            call errchr ( chscalars(iscal1), 2 )
            call errsub ( 3129, 2, 0, 2 )
         end if  ! ( numarray1>1 .and. numarray1/=numarrays )
         if ( numarray2>1 .and. numarray2/=numarrays ) then
            call errint ( numarray2, 1 )
            call errint ( numarrays, 2 )
            call errchr ( names(ivec2), 1 )
            call errsub ( 3130, 2, 0, 1 )
         end if  ! ( numarray2>1 .and. numarray2/=numarrays )
         if ( numarray3>1 .and. numarray3/=numarrays ) then
            call errint ( numarray3, 1 )
            call errint ( numarrays, 2 )
            call errchr ( names(ivec3), 1 )
            call errsub ( 3130, 2, 0, 1 )
         end if  ! ( numarray3>1 .and. numarray3/=numarrays )
         if ( numscalars1>1 .and. numscalars1/=numarrays ) then
            call errint ( numscalars1, 1 )
            call errint ( numarrays, 2 )
            call errchr ( chscalars(iscal1), 1 )
            call errsub ( 3131, 2, 0, 1 )
         end if  ! ( numscalars1>1 .and. numscalars1/=numarrays )
         if ( numscalars2>1 .and. numscalars2/=numarrays ) then
            call errint ( numscalars2, 1 )
            call errint ( numarrays, 2 )
            call errchr ( chscalars(iscal2), 1 )
            call errsub ( 3131, 2, 0, 1 )
         end if  ! ( numscalars2>1 .and. numscalars2/=numarrays )

      end if  ! ( numarrays>1 )
      if ( ierror/=0 ) go to 1000

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         iseqarrmin = 1
         iseqarrmax = numarrays

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray

      end if  ! ( iseqarray==0 )

      if ( debug ) &
         write(irefwr,1) 'iseqarrmin, iseqarrmax', iseqarrmin, iseqarrmax

      do iseqarray = iseqarrmin, iseqarrmax

         if ( ichoice>=25 ) then
            if ( iscal1>0 ) then
               rinvec(1) = scalars(iscal1)
            else if ( iscal1<0 ) then
               rinvec(1) = -scalars(-iscal1)
            end if  ! ( iscal1>0 )
            if ( iscal2>0 .and. ichoice/=102 ) then
               rinvec(2) = scalars(iscal2)
            else if ( iscal2<0 .and. ichoice/=102  ) then
               rinvec(2) = -scalars(iscal2)
            end if  ! ( iscal2>0 .and. ichoice/=102 )
         end if
         if ( debug ) then
            write(irefwr,1) 'iscal1, iscal2', iscal1, iscal2
            write(irefwr,2) 'rinvec ', rinvec(1), rinvec(2)
         end if  ! ( debug )
         iinvec(12) = ivec2
         iinvec(13) = ivec3

         call prmanvsub ( ivectr, ivec2, ivec3, inpsep, rinvec, &
                          ichoice, iscal1, iscal2, iinvec )

         if ( numarray1>1 ) ivectr = ivectr+1
         if ( numarray2>1 ) ivec2 = ivec2+1
         if ( numarray3>1 ) ivec3 = ivec3+1
         if ( numscalars1>1 ) iscal1 = iscal1+1
         if ( numscalars2>1 ) iscal2 = iscal2+1

      end do  ! iseqarray = iseqarrmin, iseqarrmax

      iseqarray = iseqarrsav  ! reset iseqarray
      if ( ivectr==iseqcoor ) then

!     --- ivectr = iseqcoor, copy new coordinates back

         call prcopycoorbck ( iseqcoor, inpsep(3) )

      end if  ! ( ivectr==iseqcoor )

      if ( iseqcoor>0 ) then

!     --- vector ivec2 contains coor

         call seprmsolinfo ( isolut2d(iseqcoor) )
         if ( numvecmn==iseqcoor ) numvecmn = numvecmn-1

      end if  ! ( iseqcoor>0 )

      if ( ivec3>0 .and. inpsep(10)>200000000 ) then

!     --- ivec3 corresponds to vector expression, remove

         call seprmsolinfo ( isolut2d(ivec3) )
         if ( numvecmn==ivec3 ) numvecmn = numvecmn-1

      end if  ! ( ivec3>0 .and. inpsep(10)>200000000 )

      if ( ivec2>0 .and. inpsep(9)>200000000 ) then

!     --- ivec2 corresponds to vector expression, remove

         call seprmsolinfo ( isolut2d(ivec2) )
         if ( numvecmn==ivec2 ) numvecmn = numvecmn-1

      end if  ! ( ivec2>0 .and. inpsep(9)>200000000 )

!     --- raise istartmn

900   leng = 21
      do i = 14, 15
         if ( inpsep(i)>0 ) leng = leng+inpsep(inpsep(i))+1
      end do  ! i = 14, 15

      istartmn = istartmn+leng

      numvec = numvecsav  ! restore numvec
      if ( numarray1>1 ) ivectr = ivectr-1

1000  if ( debug ) then

!     --- Debug information

         if ( ivectr>5000 ) ivectr = ivectr-5000
         write(irefwr,1) 'ivectr, numvecmn, istartmn, leng', &
                          ivectr, numvecmn, istartmn, leng

         call prinin ( isolut2d(ivectr), 5, names(ivectr) )
         write(irefwr,*) 'End prmanv'

      end if  ! ( debug )
      call erclos ( 'prmanv' )

      end subroutine prmanv
