      subroutine prmoveboun ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-02-2012 Allow indexed vectors
!        version  1.0  date 14-01-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute displacement vector for boundary due to change of
!     coordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmoduleintmat
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscmvboun
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  iseqdisp, iseqvecs(2), ireadadap

!     ireadadap      Sequence number of input for adaptation of boundary
!     iseqdisp       sequence number of displacement vector in ISOL
!     iseqvecs       Contains sequence numbers of vectors with respect to
!                    isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     MSHCOMPDISPL   Compute displacement vector for boundary due to change of
!                    coordinates
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmoveboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmoveboun'
         call prinin ( inpsep, 4, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqdisp = inivectr(inpsep(1))
      iseqvecs(1) = inivectr(inpsep(2))
      ireadadap = inpsep(3)
      iprob = inpsep(4)
      iseqvecs(2) = inivectr(inpsep(5))

      stiffmat => stiffm(iprob:maxprb)
      call mshcompdispl ( iseqdisp, ireadadap, iseqvecs )
      numvecmn = max(numvecmn,iseqdisp)
      istartmn = istartmn+6

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmoveboun'

      end if  ! ( debug )
      call erclos ( 'prmoveboun' )

      end subroutine prmoveboun
