      subroutine prplot3d ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-05-2011 Jump for certain time steps
!        version  1.0  date 11-03-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a 3d plot of a 2d scalar function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplot3d for a description
!     istart        i/o   Starting address in array INPSEP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iplots(8), ntext
      double precision rplots(13)
      character (len=80) textloc

!     iplots         Integer input array for plot subroutine
!     ntext          Number of characters in a text
!     rplots         Real input array for plot subroutine
!     textloc        String array to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTF3         Make 3D plot of function defined on 2D mesh using hidden
!                    lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplot3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplot3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itimloop/=0 .and. ioutact/=1 ) go to 900 ! Only at certain time steps

!     --- Fill array iplots etc

      iplots = 0       ! Default values
      rplots = 0d0     ! Default values
      iplots(1) = 8    ! number of entries filles
      iplots(5) = inpsep(2)   ! degfd

      if ( inpsep(4)>0 ) then

!     --- Text given by the user

         ntext = inpsep(4)
         textloc = texts(inpsep(3))(1:ntext)

      else

!     --- No text given by the user, use default text

         textloc(1:11) = '3D plot of '
         textloc(12:43) = names(inpsep(1))
         ntext = 43

      end if  ! ( inpsep(4)>0 )

      iplots(8) = ntext  ! Defines length of bottom text

!     --- Actual call

      isolut => isolut2d(inpsep(1))
      call plotf3 ( iplots, rplots, inpsep, textloc )

!     --- Raise istart

900   istart = istart+5

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplot3d'

      end if  ! ( debug )
      call erclos ( 'prplot3d' )

      end subroutine prplot3d
