      subroutine prreacnode ( isol, iseqreac, iseqresul )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-05-2011 Extra debug
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Replace the integrated reaction force by the nodal force
!     This is done by subdivision by a diagonal mass matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     mass_matrix
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5,*), iseqreac, iseqresul

!     iseqreac       i    Sequence number of reaction vector in array ISOL
!     iseqresul      i    Sequence number of result vector in array ISOL
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRDIVREAC      Subdivide the reaction force vector by the diagonal mass
!                    matrix defined by the surface
!     PRININ         print 1d integer array
!     PRMASSMAT      Fill mass matrix for all nodes that are on the outer
!                    boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreacnode' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreacnode'
         write(irefwr,1) 'iseqreac, iseqresul', iseqreac, iseqresul
         call prinin ( isol(1,iseqreac), 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute mass matrix

      call prmassmat

!     --- subdivide the reaction force (ndim components)

      call prdivreac ( isol, iseqreac, iseqresul )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prreacnode'

      end if
      call erclos ( 'prreacnode' )

      end subroutine prreacnode
