      subroutine prreadmesh ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 31-05-2013 Extra parameters intmat, meshseqs
!        version  2.0  date 30-05-2012 Extra parameters
!        version  1.1  date 22-11-2004 Fortran 90
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the mesh with sequence number stored in inpsep and name stored
!     in the combination inpsep texts from file and store the information
!     in array kmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecons
      use sepmodulemach
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents: see subroutine readscreadmesh from pos. 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, itypeprob, newmeshnr, npointold, nelemold, ihelp(10)

!     ihelp          integer work array
!     iref           File reference number
!     itypeprob      Defines how to deal with probdf
!                    Possible values:
!                    1:  no problem definition
!                    2:  default problem definition
!                    3:  old problem definition
!     nelemold       Number of elements in old mesh
!     newmeshnr      New mesh sequence number
!     npointold      Number of nodes in old mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIL         Open files
!     INIREPSCAL     Corrects integer value in case of scalars
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     PRININ         print 1d integer array
!     PROBSTRUC      Create problem definition and matrix structure for new mesh
!     PROCOPY1       Copy important information of problem definition
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set name of file in name10
!     Open file
!     Read mesh
!     Close file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreadmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreadmesh'
         call prinin ( inpsep, 3, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      newmeshnr =  abs(meshseqs(inirepscal(inpsep(1))))  ! newmeshnr

      iseqkm = kmesh2d(meshnr)

      npointold = km(iseqkm)%kmesh(8)
      nelemold = km(iseqkm)%kmesh(9)

      if ( debug ) write(irefwr,1) 'meshnr, newmeshnr ', meshnr, newmeshnr

!     --- Set name of file in name10

      if ( inpsep(2)>0 ) name10 = texts(inpsep(2))(1:50)

!     --- Open file

      call inifil ( -10 )

!     --- Read mesh

      kmesh => kmesh2d(newmeshnr)

      call msh071 ( -2, iref10, ihelp )

!     --- Close file

      iref = abs(iref10)
      close ( unit = iref )

      if ( ioutp>=0 ) then
         write(irefwr,100) inpsep(1), name10
100      format ( ' Mesh number ',i2, ' read from file ',a )
      end if

      itypeprob = inpsep(3)

      kmesh => kmesh2d(newmeshnr)
      call sepfillkmesh(kmesh)

      if ( debug ) then
          write(irefwr,1) 'itypeprob, npoint, npointold', &
                           itypeprob, npoint, npointold
          write(irefwr,1) 'nelem, nelemold', nelem, nelemold
      end if  ! ( debug )

      removesol = .false.

      select case ( itypeprob )

         case(1)

!        --- None, so no action at all

         case(2)

!        --- default, if problem is not defined, new problem definition default

            if ( newmeshnr==meshnr ) then

!           --- new mesh number is equal to old one

               if ( npoint/=npointold .or. nelem/=nelemold ) then

!              --- mesh has new definition, redefine kprob and intmat

                  call probstruc ( meshnr, meshnr )

               end if  ! ( npoint/=npointold .or. nelem/=nelemold )

            else

!           --- new mesh number is different from old one

               call probstruc ( meshnr, newmeshnr )

            end if  ! ( newmeshnr==meshnr )

         case(3)

!        --- old, use problem definition corresponding to old mesh

            if ( newmeshnr==meshnr ) then

!           --- new mesh number is equal to old one

               if ( npoint/=npointold .or. nelem/=nelemold ) then

!              --- mesh has new definition, redefine kprob2d

                  call probstruc ( meshnr, meshnr )

               end if  ! ( npoint/=npointold .or. nelem/=nelemold )

            else

!           --- new mesh number is different from old one
!               first copy old definition

               call procopy1 ( kprob2d(meshnr), kprob2d(newmeshnr) )

!              --- redefine kprob2d

               call probstruc ( meshnr, newmeshnr )

            end if  ! ( newmeshnr==meshnr )

      end select  ! case ( itypeprob )
      removesol = .true.

      istartmn = istartmn+4   ! raise istartmn

1000  call erclos ( 'prreadmesh' )
      end
