      subroutine prstocdiffvec ( iseqw1, iseqw0, iseqdiff, &
                                 investment, nodes1, numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 23-12-2011 replace mmnodes by nodes
!        version  1.0  date 16-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute difference vector v_diff = w1-w0-investment
!     at free surface
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     free_surface
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqw1, iseqw0, iseqdiff, nodes1(*), numnodes
      double precision investment

!     investment     i    Defines the investment
!     iseqdiff       i    sequence number of diff vector in isolut2d
!     iseqw0         i    sequence number of w0 in isolut2d
!     iseqw1         i    sequence number of w1 in isolut2d
!     nodes1         i    Contains the node numbers that must be filled
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: kprobf0(:)

!     kprobf0        kprob part f corresponding to iseqw0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSTOCDIFFVEC1 Compute difference vector v_diff = w1-w0-investment
!                    at free surface
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     sepcreasolbf1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstocdiffvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstocdiffvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy array w0 into difference vector and clear

      w0 => ks(isolut2d(iseqw0))%sol
      w1 => ks(isolut2d(iseqw1))%sol
      call sepcreasolbf1 ( isolut2d(iseqdiff), ks(isolut2d(iseqw0))%typevec, &
                           ks(isolut2d(iseqw0))%ivec, 'diffvec', 0 )
      diffvec => usoltemp
      diffvec = 0d0

!     --- activate arrays

      call sepactsolbf2 ( isolut2d(iseqw0) )
      kprobf0 => kprobfi
      call sepactsolbf2 ( isolut2d(iseqw1) )

!     --- Compute difference and store in array diffvec

      call prstocdiffvec1 ( w0, w1, diffvec, kprobf0, investment, kprobfi, &
                            indprfi, nodes1, numnodes )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstocdiffvec'

      end if  ! ( debug )
      call erclos ( 'prstocdiffvec' )

      end subroutine prstocdiffvec
