      subroutine prstocdiffvec1 ( w0, w1, diffvec, kprobfw0, investment, &
                                  kprobfw1, indprfw1, nodes, numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute difference vector v_diff = w1-w0-investment
!     at free surface
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     free_surface
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobfw0(*), kprobfw1(*), indprfw1, numnodes, &
              nodes(numnodes)
      double precision w0(*), w1(*), diffvec(*), investment

!     diffvec       i/o   difference vector
!                         Must be zero at input
!                         Contains w1-w0-investment in nodes at output
!     indprfw1       i    Indicates if the number of unknowns in w1 is constant
!                         0 or not (1)
!     investment     i    Defines the investment
!     kprobfw0       i    kprobf corresponding to w0
!     kprobfw1       i    kprobf corresponding to w1
!     nodes          i    Contains the node numbers of the free surface
!     numnodes       i    Number of nodal points in array nodes
!     w0             i    Contains the solution w0
!     w1             i    Contains the solution w1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, iposw0, iposw1

!     i              Counting variable
!     iposw0         Position of unknown in node number node in w0
!     iposw1         Position of unknown in node number node in w1
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstocdiffvec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstocdiffvec1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numnodes

         node = nodes(i)                ! node number
         iposw0 = kprobfw0(node)+1      ! position in w0 and diff vector
         if ( indprfw1>0 ) then

!        --- Non-constant number of degrees of freedom in w1

            iposw1 = kprobfw1(node)+1   ! position in w1 if indprfw1>0

         else

!        --- Constant number of degrees of freedom in w1

            iposw1 = node               ! position in w1 if indprfw1=0

         end if  ! ( indprfw1>0 )
         diffvec(iposw0) = w1(iposw1)-w0(iposw0)-investment

      end do  ! i = 1, numnodes

1000  call erclos ( 'prstocdiffvec1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstocdiffvec1'

      end if  ! ( debug )

      end
