      subroutine prstocreac2 ( iseqvec, iseqcoeff, iseqreac, &
                               elgrpw0, numelgrpw0, curves, numcurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 24-12-2014 New definition clinsl
!        version  1.3  date 17-08-2013 New call to erlins
!        version  1.2  date 25-11-2012 Long integers
!
!   copyright (c) 2009-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute reaction force for stochastic investment
!
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleuser
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvec, iseqcoeff, iseqreac, numelgrpw0, &
              elgrpw0(numelgrpw0), numcurves, curves(numcurves)

!     curves         i    Contains curves of free boundary
!     elgrpw0        i    Contains the element groups for which w0 is defined
!     iseqcoeff      i    Array containing sequence numbers of input for
!                         coefficients
!     iseqreac       i    Sequence number of reaction vector in array ISOL
!                         If <0 also the right-hand side contribution is added
!     iseqvec        i    Starting address of sequence numbers of vectors
!     numcurves      i    Number of curves in array curves
!     numelgrpw0     i    Number of eleemnt groups stored in array elgrpw0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable, dimension (:) :: skipel
      integer i

!                    they are not
!     i              Loop variable
!     skipel         Array containg the element groups to be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     TO0100         Performs the actual computation of the derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstocreac2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstocreac2'
         call prinin ( curves, numcurves, 'curves' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays iuser and user
!         First fill in ibuffr

      iprob = ks(isolut2d(iseqvec))%iprob
      call erlins ( 3, iprob, iseqcoeff )

!     --- Next copy to iuser and user

      iuser => iuser1(iprob)%iuser
      user => iuser1(iprob)%user

!     --- Fill array skipel

      skipel = 1   ! Clear array

      do i = 1, numelgrpw0
         skipel(elgrpw0(i)) = 0  ! mark element groups of set w0
      end do  ! i = 1, numelgrpw0
      if ( debug ) call prinin ( skipel, nelgrp, 'skipel' )

!     --- Fill array iinder

      iinder = 0            ! initialization
      iinder(1) = 4         ! call by prstoc
      iinder(2) = 7         ! reaction force
      iinder(16) = 2        ! use curves
      iinder(8) = numvecmn    ! Number of vectors
      iinder(10) = iseqvec  ! Vector sequence number
      iinder(11) = iseqvec  ! Vector sequence number
      iinder(12) = 1        ! Use array skipel
      iinder(15) = numcurves ! Number of curves

!     --- Compute reaction force

      call sepfillsmf1 ( isolut2d, maxvec )
      SMF%ioutvc =>  isolut2d(iseqreac)
      call to0100 ( skipel, curves )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstocreac2'

      end if  ! ( debug )
      call erclos ( 'prstocreac2' )

      end subroutine prstocreac2
