      subroutine prtimehis8 ( ipoints, lenipoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one time step in time history array for zero levelset
!     Mark that istephis must have been raised before the call of this
!     subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipoints(*), lenipoints

!     ipoints        i    Contains the nodal points where the time history
!                         must be stored
!     lenipoints     i    Declared length of array ipoints
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ilev, inode

!     i              Counting variable
!     ilev           Sequence number of level being considered
!     inode          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer sepnodezerlev

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     SEPNODEZERLEV  Return with node at boundary defined by zero levelset
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3117   More than 1 interface point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehis8' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehis8'
         write(irefwr,1) 'npoint_hist', npoint_hist
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( zerolev ) then

!     --- npoint_hist< 0, hence zero level set

         call inichkarlen ( npoint_hist, lenipoints, 'ipoints' )
         if ( ierror/=0 ) go to 1000

         if ( npoint_hist>1 ) then

!        --- Error: More than one interface node

            call errint ( npoint_hist, 1 )
            call errsub ( 3117, 1, 0, 0 )
            go to 1000

         end if  ! ( npoint_hist> 1 )

         do i = 1, npoint_hist
            ilev = ivecttim(i)
            inode = sepnodezerlev ( ilev )
            ipoints(i) = inode
         end do  ! i = 1, npoint_hist

         if ( debug ) write(irefwr,1) 'ilev, inode', ilev, inode

      else

!     --- npoint_hist>0, standard case

         call inichkarlen ( npoint_hist, lenipoints, 'ipoints' )
         if ( ierror/=0 ) go to 1000

         ipoints(1:npoint_hist) = ivecttim

      end if  ! ( zerolev )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimehis8'

      end if  ! ( debug )
      call erclos ( 'prtimehis8' )

      end
