      subroutine prtimhisnstep ( nstephis )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Estimate number of time steps
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: nstephis

!     nstephis      i/o   Estimated number of time steps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimhisnstep' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimhisnstep'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( toutstep>0d0 .and. toutstep<0.1d0*toutend ) then

!     --- toutstep>0, make estimate

         nstephis = nint((toutend-toutinit)/toutstep)+1
         nstephis = min ( 10000, max ( 50, nstephis ) )

      else if ( tstep>0d0 ) then

!     --- tstep>0, make estimate

         nstephis = min ( 10000, nint((tend-t0)/tstep) )

      else

!     --- no estimate possible, make a guess

         nstephis = 50

      end if  ! ( tstep>0d0 )
      nstephis = nstephis+1  ! also store initial step

1000  call erclos ( 'prtimhisnstep' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nstephis', nstephis
         write(irefwr,*) 'End prtimhisnstep'

      end if  ! ( debug )

      end subroutine prtimhisnstep
