      subroutine prtransfvect ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 11-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform vector to other coordinate system
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prmanv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecout, ispec, ivecin

!     ispec          Parameter ispec defining the type of transformation
!                    1:  Cartesian to polar  (2d only)
!     ivecin         Sequence number of input vector
!     ivecout        Sequence number of output vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRTRANSFVECT1  Transform vector to other coordinate system
!     sepcreasolbf1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1653   Number of unknowns not constant in node
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtransfvect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtransfvect'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters from inpsep

      ivecout = inpsep(13)
      ispec = inpsep(5)
      ivecin = inpsep(12)

      if ( debug ) &
         write(irefwr,1) 'ivecout, ivecin, ispec', &
                          ivecout, ivecin, ispec

      if ( ivecin<1 .or. ivecin>maxvc1 ) then

!     --- sequence number out of range

         call errint ( ivecin, 1 )
         call errint ( maxvc1, 2 )
         call errsub ( 2951, 2, 0, 0 )
         go to 1000

      end if  ! ( ivecin<1 .or. ivecin>maxvc1 )

      if ( ivecout<1 .or. ivecout>maxvc1 ) then

!     --- sequence number out of range

         call errint ( ivecout, 1 )
         call errint ( maxvc1, 2 )
         call errsub ( 2951, 2, 0, 0 )
         go to 1000

      end if  ! ( ivecout<1 .or. ivecout>maxvc1 )

!     --- Activate input vector

      vectin => ks(isolut2d(ivecin))%sol
      if ( ierror/=0 ) go to 1000

      if ( indprfi/=0 ) then

!     --- error: nuber of unknowns not constant per node

         call errsub ( 1653, 0, 0, 0 )
         go to 1000

      end if  ! ( indprfi/=0 )

!     --- Activate or create output vector

      call sepcreasolbf1 ( isolut2d(ivecout), ks(isolut2d(ivecin))%typevec, &
                           ks(isolut2d(ivecin))%ivec,'vecout', 0 )
      vectout => usoltemp
      if ( ierror/=0 ) go to 1000

!     --- Actual call

      call prtransfvect1 ( vectin, vectout, npoint, &
                           nunkpi, ispec )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtransfvect'

      end if  ! ( debug )
      call erclos ( 'prtransfvect' )

      end subroutine prtransfvect
