      subroutine prtransfvect1 ( vecin, vecout, nnodes, idim, ispec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform vector to other coordinate system
!     At this moment restricted to polar coordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, idim, ispec
      double precision vecin(idim,nnodes), vecout(idim,nnodes)

!     idim           i    dimension of the space
!     ispec          i    defines type of transformation
!                         1: Cartesian to polar coordinates (2d)
!     nnodes         i    Number of nodes
!     vecin          i    input vector
!     vecout         o    Output vector, result of the tranformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x, y

!     i              Counting variable
!     x              x component in point
!     y              y component in point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtransfvect1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtransfvect1'
         write(irefwr,1) 'nnodes, idim, ispec', nnodes, idim, ispec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ispec/=1 ) then

!     --- ispec has incorrect value

         call errint ( ispec, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errchr ( 'ispec', 1 )
         call errsub ( 1378, 3, 0, 1 )

      end if  ! ( ispec/=1 )

      if ( idim/=2 ) then

!     --- idim has incorrect value

         call errint ( idim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errchr ( 'idim', 1 )
         call errsub ( 1378, 3, 0, 1 )

      end if  ! ( ispec/=1 )
      if ( ierror/=0 ) go to 1000

      do i = 1, nnodes

         x = vecin(1,i)
         y = vecin(2,i)
         vecout(1,i) = sqrt(x**2+y**2)
         if ( abs(x)>0d0 ) then
            vecout(2,i) = atan(y/x)
         else
            vecout(2,i) = 0d0
         end if

      end do  ! i = 1, nnodes

1000  call erclos ( 'prtransfvect1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtransfvect1'

      end if  ! ( debug )

      end
