      subroutine putint ( intname, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 28-04-2000
!
!   copyright (c) 2000-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Put the value of the integer in the block constants with name intname
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname
      integer value

!     intname       i    Name of integer
!     value         i    Value of integer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer intseqnm

!     intseqnm       Sequence number of integer with respect to array
!                    chincons
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     GETNAMEINT     Return with sequence number of integer in incons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2155   integer name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chincons
!     If ( name in chincons ) then
!        put value in common
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      intseqnm = getnameint ( intname )
      if ( intseqnm.gt.0 ) then
         incons(intseqnm) = value
      else
         call eropen ( 'putint' )
         call errchr ( intname, 1 )
         call errsub ( 2155, 0, 0, 1 )
         call erclos ( 'putint' )
      end if
      end
