      subroutine read02_220
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 220
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 220
!                1: ELASTIC_MODULUS
!                2: SECOND_MOMENT
!                3: LOAD
!                4: INTEGRATION_RULE
!                5: COORDINATE_SYSTEM
! ======================================================================
!
      call eropen ( 'read02_220' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_220'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=5

      code(1:3) = codemech(1:3)
      ncode(1:3) = nscodemech(1:3)
      ncodmx(1:3) = ncodemech(1:3)
      code(4:5) = codegen(2:3)
      ncode(4:5) = nscodegen(2:3)
      ncodmx(4:5) = ncodegen(2:3)
      call sepcopycodesub ( codecsys, nncodecsys, nscodecsys )

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read

      tabmin(1:3) = 1d-20        ! ELASTIC_MODULUS, SECOND_MOMENT, LOAD

      itype(4) = 0               ! INTEGRATION_RULE
      tabmax(4) = 1d0

      itype(5) = 5   ! COORDINATE_SYSTEM
      tabmin(5) = 1d0
      tabmax(5) = tabmin(5)+nncodecsys-1

      nmain = 0
      iretur = -11  ! keyword has been read

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4
      icoef = 0

      if ( table(1)>0d0 ) then

!     --- ELASTIC_MODULUS found

         icoef = 6
         call read02fillc

      end if  ! ( table(1)>0d0 )

      if ( table(2)>0d0 ) then

!     --- SECOND_MOMENT found

         icoef = 7
         call read02fillc

      else if ( table(3)>0d0 ) then

!     --- LOAD found

         icoef = 8
         call read02fillc

      else if ( table(4)>0d0 ) then

!     --- INTEGRATION_RULE found

         icoef = 3
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(4))
         iposin = iposin+3

      else if ( table(5)>0d0 ) then

!     --- COORDINATE_SYSTEM found

         icoef = 4
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(5))-1
         iposin = iposin+3

      end if  ! ( table(2)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_220'

      end if  ! ( debug )
      call erclos ( 'read02_220' )

      end subroutine read02_220
