      subroutine read02_251
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 251
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 251
!                1: x_load
!                2: y_load
!                3: Z_LOAD
!                4: THICKNESS
!                5: STRESS_STRAIN_RELATION
!                6: ILOAD
!                7: INTEGRATION_RULE
!     codemain      contains subkeywords
!                1: PLANE_STRESS', &
!                2: PLANE_STRAIN
!                3: AXI_SYMMETRIC_STRESS
!                4: ORTHOTROPIC_PLANE_STRESS
!                5: ORTHOTROPIC_PLANE_STRAIN
!                6: STRATIFIED_AXI_SYMMETRIC_STRESS
!                7: USER_DEFINED_ELASTICITY_CART
!                8: USER_DEFINED_ELASTICITY_AXI
! ======================================================================
!
      call eropen ( 'read02_251' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_251'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codestressbn, nncodestressbn, nscodestressbn, &
                         ncodestressbn )
      call sepcopycodesub ( codestressmod, nncodestressmod, nscodestressmod )

      itype = 44
      table = 0d0
      tabmin = 0d0
      tabmax = 0d0

      itype(5) = 5               ! STRESS_STRAIN_RELATION
      tabmin(5) = 1d0
      tabmax(5) = tabmin(1)+7d0

      itype(6:7) = 0               ! ILOAD, INTEGRATION_RULE
      tabmax(6) = 4d0
      tabmax(7) = 3d0

      nmain = 0
      iretur = -11  ! keyword has been read

      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      if ( table(1)>0d0 ) then

!     --- x_load found

         icoef = 6
         call read02fillc

      else if ( table(2)>0d0 ) then

!     --- y_load found

         icoef = 7
         call read02fillc

      else if ( ndim>2 .and. table(3)>0d0 ) then

!     --- z_load found

         icoef = 8
         call read02fillc

      else if ( table(4)>0d0 ) then

!     --- THICKNESS found

         icoef = 9
         call read02fillc

      else if ( table(5)>0d0 ) then

!     --- STRESS_STRAIN_RELATION found

         icoef = 2
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(5))-1
         iposin = iposin+3

      else if ( table(6)>0d0 ) then

!     --- ILOAD found

         icoef = 1
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(6))
         iposin = iposin+3

      else if ( table(7)>0d0 ) then

!     --- INTEGRATION_RULE found

         icoef = 3
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(7))
         iposin = iposin+3

      end if  ! ( table(7)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_251'

      end if  ! ( debug )
      call erclos ( 'read02_251' )

      end subroutine read02_251
