      subroutine read02_803
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 803
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 703
!                 1: SOURCE
!                 2: INTEGRATION_RULE
!                 3: COORDINATE_SYSTEM
!                 4: ZERO_ORDER_COEF
!                 5: LINEARIZATION
!     codemain      contains subkeywords
!                 1: CARTESIAN
!                 2: AXI_SYMMETRIC
!                 3: POLAR
!                 4: PICARD
!                 5: NEWTON
! ======================================================================
!
      call eropen ( 'read02_803' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_803'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=5
      nncodesub=5

      code(1:3) = codegen(1:3)
      ncode(1:3) = nscodegen(1:3)
      ncodmx(1:3) = ncodegen(1:3)
      code(4) = codegen(7)
      ncode(4) = nscodegen(7)
      ncodmx(4) = ncodegen(7)
      code(5) = codenavstoks(4)
      ncode(5) = nscodenavstoks(4)
      ncodmx(5) = ncodenavstoks(4)
      codesub(1:3) = codecsys
      ncodesub(1:3) = nscodecsys
      codesub(4:5) = codelinear(2:3)
      ncodesub(4:5) = nscodelinear(2:3)

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read
      ityprd = 4    ! indicates that the first item is a string

      itype(2) = 0   ! INTEGRATION_RULE
      tabmin(2) = 0d0
      tabmax(2) = 4d0

      itype(3) = 5   ! COORDINATE_SYSTEM
      tabmin(3) = 1d0
      tabmax(3) = tabmin(3)+2d0

      itype(5) = 5   ! LINEARIZATION
      tabmin(5) = tabmax(3)+1d0
      tabmax(5) = tabmin(5)+1d0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      if ( table(1)>-1d19 ) then

!     --- SOURCE found

         icoef = 16
         call read02fillc
         go to 1000

      end if  ! ( table(1)>0d0 )

      do i = 2, 3

         if ( table(i)>0d0 ) then

!        --- INTEGRATION_RULE, COORDINATE_SYSTEM found

            icoef = i+1
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            if ( i==2 ) then
               iincof(iposin+3) = nint(table(i))  ! INTEGRATION_RULE
            else
               iincof(iposin+3) = nint(table(i))-1  ! COORDINATE_SYSTEM
            end if  ! ( i==2 )
            iposin = iposin+3
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 2, 3

      if ( table(4)>-1d19 ) then

!     --- table(4)> 0, ZERO_ORDER_COEF

         icoef = 15
         call read02fillc
         go to 1000

      end if  ! ( table(20)>0d0 )

      if ( table(5)>0d0 ) then

!     --- LINEARIZATION found

         icoef = 5
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(5))
         iposin = iposin+3
         go to 1000

      end if  ! ( table(5)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icoef', icoef
         write(irefwr,*) 'End read02_803'

      end if  ! ( debug )
      call erclos ( 'read02_803' )

      end subroutine read02_803
