      subroutine read02coef ( compl, icompl, ideg, ifunc, ival, itypel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 02-01-2014 Extra option vector
!        version  1.5  date 08-12-2013 Extension with integer scalars
!        version  1.4  date 16-04-2013 Correction comment
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following coef =
!     Store this information
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: icompl, ideg, ifunc, ival, itypel
      logical compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     icompl         i    Multiplication factor for complex numbers
!                         Real: icompl=1,   Complex: icompl=2
!     ideg           o    Degree of freedom read
!     ifunc          o    Choice parameter with respect to function calls
!     itypel        i/o   Type number of element
!     ival           o    Reference to position in rincof
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=15
      integer ih, ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2), ibef
      integer :: inex, ipos
      double precision :: tabmn2(nncode2), tabmx2(nncode2)
      character (len=17) code2 (nncode2)
      logical found, opfound

!     code2          Table with keywords
!     found          If true already a possibility has been found
!     ibef           Type of item before vector
!     ih             Help variable to store a constant temporarily
!     inex           Type of item following vector
!     ipos           Position of vector in record
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     opfound        If true an operator is found before or after a vector
!     tabmn2         Contains minimum values allowed in table
!     tabmx2         Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     738   Not one number read after elements, iref =
!     739   Not one number read after points, iref =
!     784   No number found after old solution ..., deg. of free
!    1293   More than one description found for coefficient: coef i = ...
!    3213   Superfluous information following vector name in coefficients block
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'COEF', 'VALUE', 'FUNC', 'OLD_SOLUTION', 'SOLUTION', &
                  'DEGREE_OF_FREEDOM', 'OF', 'FREEDOM', 'POINTS', &
                  'IREF', 'ELEMENTS', 'SOL_FUNC', 'OLD_VECTOR', 'OLD', &
                  'VECTOR' /
      data ncode2 /  3, 3, 3, 8, 5,  3,  2, 4, 3, 3, &
                    3, 5, 8, 3, 6 /
      data ncodmx2/  4, 5, 4,12, 8, 17,  2, 7, 6, 4, &
                    8, 8,10, 3, 6 /
      data itype2 /  0, 1, 0,38,38,  3, -1, 0, 2, 0, &
                    2, 1,38,-1,38 /
      data tabmn2 / 0d0, -1d30, 1d0, 1d0, 1d0,  1d0, 1d0, 1d0, 0d0,-1d2, &
                    0d0,   1d0, 1d0, 0d0, 1d0 /
      data tabmx2 / 0d0,  1d30, 1d5, 1d2, 1d2,  1d2, 1d2, 1d2, 0d0, 1d2, &
                    0d0,   1d4, 1d2, 0d0, 1d4 /

!     code2    Table with keywords
!             Values:
!             1:  COEF
!             2:  VALUE
!             3:  FUNC
!             4:  OLD_SOLUTION
!             5:  SOLUTION
!             6:  DEGREE_OF_FREEDOM
!             7:  OF
!             8:  FREEDOM
!             9:  POINTS
!            10:  IREF
!            11:  ELEMENTS
!            12:  SOL_FUNC
!            13:  OLD_VECTOR
!            14:  OLD
!            15:  VECTOR
! ======================================================================
!
      call eropen ( 'read02coef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02coef'
         write(irefwr,*) 'compl', compl
         write(irefwr,1) 'iposin', iposin
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      if ( compl ) then

!     --- Complex values

         itype2(1) = 26
         itype2(2) = 26

      else

!     --- Real values

         itype2(1) = 16
         itype2(2) = 16

      end if  ! ( compl )

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )

      table(1:nncode) = -1d20
      tabout(2) = 0d0
      found = .false.

      iretur = -10   ! No error message is given for single number
      nmain = 0      ! no main keywords

      if ( debug ) write(irefwr,*) 'before intx02ext'

      call sepreadsetkeyw
      if ( debug ) then
         write(irefwr,1) 'iretur, nseqkeyw, iposin', iretur, nseqkeyw, iposin
         call prinrl ( table, nncode, 'table' )
         call prinin ( typeitems, nitems, 'typeitems' )
         call prinin ( iseqkeywitem, nncode, 'iseqkeywitem' )
      end if  ! ( debug )

!     --- Set icoef as first integer for error messages

      call errint ( icoef, 1 )
      if ( iretur<=-11 .and. table(15)<0d0 ) then

!     --- Number found, i.e. COEF i = ....  (equivalent to value!)
!         Store information in rincof

         if ( found ) call errsub ( 1293, 1, 0, 0 )
         found = .true.
         iincof(iposin+3) = iposrn+1
         if ( compl ) then

!        --- Complex case

            rincof(iposrn+1) = tabout(1)
            rincof(iposrn+2) = tabout(2)
            iposrn = iposrn + 2
            itypel = 2

         else

!        --- Real case

            if ( debug ) call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )
            if ( iseqkeyw(1)==0 ) then

!           --- Scalar found

               iincof(iposin+3) = nint(table(1))
               itypel = 3

            else

!           --- Real found

               if ( debug ) write(irefwr,2) 'realrd', realrd
               rincof(iposrn+1) = realrd
               iposrn = iposrn + 1
               itypel = 2
               if ( index_rdarr(3)>0 ) then

!              --- Sign positive and indexed constant
!                  Store information
!                  index_rdarr(5) = 1 or 2 (integer or real)
!                  index_rdarr(3) position of index in scalars
!                  index_rdarr(4) position of constant

                  iincof(iposin+3) = index_rdarr(5)+ &
                                     1000*index_rdarr(3)+ &
                                     1000000*index_rdarr(4)

               else if ( index_rdarr(3)<0 ) then

!              --- Sign negative and indexed constant

                  iincof(iposin+3) = -index_rdarr(5)+ &
                                     1000*index_rdarr(3)- &
                                     1000000*index_rdarr(4)

               end if  ! ( index_rdarr(3)>0 )

            end if  ! ( iseqkeyw(1)==0 )

         end if
         if ( debug ) then
            write(irefwr,1) 'icoef, iposrn, ityprd, itypel', &
                             icoef, iposrn, ityprd, itypel
            write(irefwr,1) 'iincof(iposin+3)', iincof(iposin+3)
            if ( compl ) then
               call prinrl ( tabout, 2, 'tabout' )
            else
               write(irefwr,2) 'realrd, tabout(1)', realrd, tabout(1)
            end if  ! ( compl )
         end if  ! ( debug )

      else

!     --- Standard situation

         if ( table(1)>1d5 ) then

!        --- scalar found
!            Store information in iincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            iincof(iposin+3) = nint(table(1))
            itypel = 3

         else if ( table(1)>-1d19 .and. table(4)<-1d19 .and. &
                   table(5)<-1d19 ) then

!        --- coef i = coef j found
!            Store information in iincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            ih = nint(tabout(1))
            iincof(iposin+3) = ih
            itypel = 9
            if ( ih>=icoef ) then
               call errint ( ih, 1 )
               call errint ( icoef, 2 )
               call errsub ( 784, 2, 0, 0 )
               call errcrd
            end if
            if ( debug ) then
               write(irefwr,*) 'coef i = coef j'
               write(irefwr,1) 'ih, iposin, itypel', ih, iposin, itypel
            end if  ! ( debug )

         end if
         if ( table(2)>-1d19 ) then

!        --- value = .... found
!            Store information in rincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            iincof(iposin+3) = iposrn + 1
            if ( compl ) then
               rincof(iposrn+1) = tabout(1)
               rincof(iposrn+2) = tabout(2)
            else
               rincof(iposrn+1) = tabout(1)
            end if
            iposrn = iposrn + icompl + 1
            itypel = 2

         end if

         if ( table(3)>-1d19 ) then

!        --- func = ..... found
!            Store information in iincof
!            In case func = scalar found, we use a minus sign

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            iincof(iposin+3) = nint(table(3))
            if ( iincof(iposin+3)>100000000 ) &
               iincof(iposin+3) = -iincof(iposin+3)
            itypel = 3

         end if

         if ( table(4)>-1d19 .or. table(5)>-1d19 .or. table(13)>-1d19 &
              .or. table(15)>0d0 ) then

!        --- old solution j, degree of freedom k,
!            coef (value=v or function=f)  found
!            Store information in iincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            ih = nint(max(table(4),table(5),table(13),table(15)))

            if ( table(15)>0d0 .and. max(table(4),table(5),table(13))<0d0 ) then

!           --- vector name found, check if this name is followed by superfluous
!               information

               ipos = iseqkeywitem(15)    ! position of vector
               ibef = typeitems(ipos-1)   ! type of item before vector
               opfound = ibef==7 .or. ibef==8 .or. ibef==11 .or. ibef==12 .or. &
                         ibef==19  ! if true this is an operator
               if ( nitems>ipos ) then
                  inex = typeitems(ipos+1)   ! type of item after vector
                  opfound = opfound .or. inex==7 .or. inex==8 .or. inex==11 .or. &
                            inex==12 .or.inex==19  ! if true this is an operator
               end if  ! ( nitems>ipos )
               if ( opfound ) then

!              --- Error: operator found before or behind vector name

                  call errsub ( 3213, 0, 0, 0 )
                  call errcrd
                  go to 1000

               end if  ! ( nitems>iseqkeywitem(15) )
            end  if  ! ( table(15)>0d0 ... )
            iincof (iposin+3) = ih
            itypel = 6
            ideg = 0
            ival = 0
            ifunc = 0
            if ( table(6)>-1d19 .or. table(8)>-1d19 ) then

!           --- degree of freedom found

               ideg = nint(max(table(6),table(8)))

            end if

            if ( table(1)>-1d19 ) then

!           --- coef found

               if ( table(1)>0 ) then
                  ifunc = nint(table(1))
               else
                  ival = iposrn+1
                  if ( compl ) then
                     rincof(iposrn+1) = tabout(1)
                     rincof(iposrn+2) = tabout(2)
                  else

                     rincof(iposrn+1) = tabout(1)
                     if ( abs(tabout(1))<=0d0 ) then

!                    --- Coefficient equal to 0
!                        treat as given coefficient

                        iincof (iposin+3) = iposrn+1
                        itypel = 2   ! constant

                     end if  ! ( abs ( tabout(1)>0d0 )

                  end if
                  iposrn = iposrn + icompl + 1

               end if

            end if

         end if

         if ( table(9)>-1d19 ) then

!        --- Letter p found , i.e. points, iref = ...
!            Store information in iincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            if ( nint(table(10))==0 ) then
               call errsub ( 739, 0, 0, 0 )
               call errcrd
            else
               itypel = 4
               iincof(iposin+3) = nint(table(10))
            end if

         end if

         if ( table(11)>-1d19 ) then

!        --- Letter e found , i.e. elements, iref = ...
!            Store information in iincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            if ( table(10)<=0 ) then
               call errsub ( 738, 0, 0, 0 )
               call errcrd
            else
               itypel = 5
               iincof(iposin+3) = nint(table(10))
            end if

         end if

         if ( table(12)>-1d19 ) then

!        --- sol_func = k found
!            Store information in iincof

            if ( found ) call errsub ( 1293, 1, 0, 0 )
            found = .true.
            itypel = 10
            iincof(iposin+3) = nint(table(12))

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin+3, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,1) 'iposin', iposin
         write(irefwr,*) 'End read02coef'

      end if  ! ( debug )
      call erclos ( 'read02coef' )

      end
