      subroutine read02fill ( itypel, ideg, ifunc, ival )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information concerning coefficient into iincof
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: itypel
      integer, intent(in) :: ideg, ifunc, ival

!     ideg           i    Degree of freedom read
!     ifunc          i    Choice parameter with respect to function calls
!     itypel        i/o   Parameter itype in iincof
!     ival           i    Reference to position in rincof
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02fill' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02fill'
         write(irefwr,1) 'itypel, icoef, ideg, ival, ifunc, iposin', &
                          itypel, icoef, ideg, ival, ifunc, iposin
         call prinin ( iincof, iposin, 'iincof' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iincof(iposin+1) = itypel   ! Type input
      iincof(iposin+2) = icoef    ! Sequence number coefficient

      if ( itypel==6 ) then

!     --- itypel = 6, hence vector

         iincof(iposin+4) = ideg
         if ( ival>0 ) then
            iincof(iposin+5) = ival
            itypel = 7
         else if( ifunc>0 ) then
            iincof(iposin+5) = ifunc
            itypel = 8
         end if
         iincof(iposin+1) = itypel
         iposin = iposin + 4
         if ( itypel/=6 ) iposin = iposin + 1

      else

!     --- itypel # 6

         iposin = iposin + 3

      end if  ! ( itypel==6 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,1) 'iposin', iposin
         write(irefwr,*) 'End read02fill'

      end if  ! ( debug )
      call erclos ( 'read02fill' )

      end subroutine read02fill
