      subroutine read02fillv ( ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information concerning coefficient into iincof/rincof
!     Special case of a vector of coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ndim

!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ideg, ifunc, ival, itypel, i, ih

!     i              Counting variable
!     ideg           Degree of freedom read
!     ifunc          Choice parameter with respect to function calls
!     ih             Help variable to store a constant temporarily
!     itypel         Parameter itype in iincof
!     ival           Reference to position in rincof
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READ02COEF     Read information following coef = &
!     READ02FILL     Fill information concerning coefficient into iincof
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02fillv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02fillv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First reading of last part
!         iincof(iposin+3) and rincof are filled

      call read02coef ( .false., 1, ideg, ifunc, ival, itypel )
      imark(icoef) = 1

!     --- Next filling of iincof for all components

      ih = iincof(iposin+3)
      do i = 1, ndim

         ideg = i
         iincof(iposin+3) = ih
         call read02fill ( itypel, ideg, ifunc, ival )
         imark(icoef) = 1
         icoef = icoef+1

      end do  ! i = 1, ndim

      icoef = icoef-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02fillv'

      end if  ! ( debug )
      call erclos ( 'read02fillv' )

      end subroutine read02fillv
