      subroutine read03spectral ( isubeq, inpsol, rinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all information corresponding to the sub-keyword SPECTRAL
!     corresponding to the main keyword SOLVE
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     solve
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubeq, inpsol(*)
      double precision rinsol(*)

!     inpsol        i/o   Contains integer information concerning the solution
!                         process.
!                         For a description see subroutine READ03
!     isubeq         i    Sequence number of subequation
!     rinsol        i/o   Array containing the real user input if iread=-1
!                         For a description see subroutine READ03
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       SPECTRAL, options
!          with options empty or a combination of the following possibilities:
!          max_iter = i
!          accuracy = eps
!          print_level = i
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code   Table with the keywords following defect_correction
!             Values:
!             1:  max_iter
!             2:  accuracy
!             3:  print_level
! ======================================================================
!
      call eropen ( 'read03spectral' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read03spectral'

      end if
      if ( ierror/=0 ) go to 1000

      nncode = 3
      code(1) = 'MAX_ITER'
      code(2) ='ACCURACY'
      code(3) ='PRINT_LEVEL'
      ncode(1) = 6
      ncode(2) = 3
      ncode(3) = 4
      ncodmx(1:2) = 8
      ncodmx(3) = 11
      itype(1:nncode) = 0
      itype(2) = 1
      table(1:nncode) = 0d0
      tabmin(1:nncode) = 0d0
      tabmax(1) = 1d3
      tabmax(2) = 1d-1
      tabmax(3) = 2d0
      table(1) = 1d2
      table(2) = 1d-2
      table(3) = 1d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( iretur==-1 ) go to 1000
      inpsol(ninpsol1*isubeq+38) = nint(table(1))
      rinsol(nrinsol1*isubeq+8) = table(2)
      inpsol(ninpsol1*isubeq+37) = nint(table(3))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read03spectral'

      end if
      call erclos ( 'read03spectral' )

      end subroutine read03spectral
