      subroutine read74sub1 ( iseq, iter, t, ios )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read first part of information of solution array from sepcomp.out
!     Return with t and iseq
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iter, ios
      double precision t

!     ios            o    IOSTAT variable, returns from reading or writing
!     iseq           o    Sequence number of specific vector in solution
!                         vector as stored in sepcomp.out
!                         This is the actual sequence number minus one
!     iter           o    Iteration sequence number as stored in sepcomp.out
!     t              o    actual time as stored in sepcomp.out
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(7), i
      character (len=80) textloc

!     i              Counting variable
!     ihelp          integer work array to store values read
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read74sub1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read74sub1'

      end if
      if ( ierror/=0 ) go to 1000
100   format(a)

      if ( iref74>0 ) then

!     --- formatted

         read(iref74,100, iostat=ios) textloc
         if ( debug ) write(irefwr,*) textloc
         if ( ios/=0 ) go to 1000
         read(iref74,100,iostat=ios) textloc
         if ( debug ) write(irefwr,*) textloc
         if ( ios/=0 ) go to 1000
         read(iref74,100,iostat=ios) textloc
         if ( debug ) write(irefwr,*) textloc
         if ( ios/=0 ) go to 1000
         read(iref74,100,iostat=ios) textloc
         if ( debug ) write(irefwr,*) textloc
         if ( ios/=0 ) go to 1000
         read(iref74,100,iostat=ios) textloc
         if ( debug ) write(irefwr,*) textloc
         if ( ios/=0 ) go to 1000
         read(iref74,100,iostat=ios) textloc
         if ( debug ) write(irefwr,*) textloc
         if ( ios/=0 ) go to 1000
         read (iref74,*, iostat=ios) iseq, iter, t
         if ( debug ) write(irefwr,*) 'iseq, iter, t ', iseq, iter, t
         if ( ios/=0 ) go to 1000

      else

!     --- unformatted

         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read(-iref74, iostat=ios) textloc
         if ( ios/=0 ) go to 1000
         read(-iref74, iostat=ios) iseq, iter, t, (ihelp(i),i=1,7)
         if ( ios/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read74sub1'

      end if
      call erclos ( 'read74sub1' )

      end
