      subroutine readc0volm ( iwork, table, tabout, tabmax, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret all surface information read by readc0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*)
      double precision table(*), tabout(*), work(*), tabmax(*)

!     iwork         i/o   integer work array, see readc0
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
!     work          i/o   Work array, see readc0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut, m, inum, inpelm

!     inpelm         number of points per element
!     inum           Number of volumes
!     iposut         Position in array tabout
!     m              number of local nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READC301       Check the numrel numbers stored in array relnods
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     891   In one record more than one of the combinations:
!           POINTS, CURVES, SURFACES, NODES, ELEMENTS or GROUP is found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readc0volm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0volm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- VOLUMES RN1, RN2, ... (V1, V2, ... ) found

      if ( iwork(2)/=0 ) then
         call errsub ( 891, 0, 0, 0 )
         call errcrd
      end if

      iwork(2) = 25   ! indicates set of volumes

      if ( table(14)>0d0 ) then

!     --- table(14) >0, standard case

         iposut = nint(table(14))  ! starting address in tabout
         inum = nint(tabmax(14))   ! defines information in tabout
         m = inum/10000            ! number of surfaces
         inum = mod(inum,10000)    ! number of local nodes
         iwork(3) = inum     ! number of local nodes
         iwork(4) = m        ! number of surfaces
         iwork(5:4+m) = nint(tabout(iposut:iposut+m-1))  ! set of surfaces
         iposut = iposut+m

         if ( inum>0 ) then

!        --- inum>0, hence there are relative nodes

            work(100:99+inum) = tabout(iposut:iposut+inum-1)
            inpelm = sepgetinpelsurf ( iwork(5), 1 )

!           --- numrel>0,  copy contents of work numrel+1 positions forwards
!               clear work from position 101+numrel

            call readc301 ( inum, inpelm, work(100), work(101+inum) )

         end if  ! ( inum>0 )

      end if  ! ( table(14)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'm, inum', m, inum
         call prinin ( iwork, 4+inum, 'iwork' )
         if ( inum>0 ) call prinrl ( work(101+inum), inpelm, 'work' )
         write(irefwr,*) 'End readc0volm'

      end if  ! ( debug )
      call erclos ( 'readc0volm' )

      end
