      function readconstvar ( namecoeff, minval, maxval )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-12-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read constant (or variable) from  list of constants or variables
!     Return with value and sequence number
! **********************************************************************
!
!                       KEYWORDS
!
!     initialization
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision readconstvar, minval, maxval
      character ( len=* ) namecoeff

!     maxval         i    Maximum value allowed for the parameter read
!     minval         i    Minimum value allowed for the parameter read
!     namecoeff      i    Name of parameter (coefficient)
!     readconstvar   o    Value of constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision valcoef
      integer maxwarn, nwarnbef

!     maxwarn        Maximum number of warnings allowed
!     nwarnbef       Help paramter to save value of nwarn
!     valcoef        Value of constant
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getconst, getvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     GETCONST       Get the value of a constant in the input block constants
!     GETVAR         Get the value of a scalar in the input block variables
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1379   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readconstvar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readconstvar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxwarn = mxwarn
      nwarnbef = nwarn
      mxwarn = -1
      valcoef = getconst ( namecoeff )
      if ( nwarn>nwarnbef ) then

!     --- Coefficient is not a constant, may be a scalar?

         valcoef = getvar ( namecoeff )

      end if  ! ( nwarn>nwarnbef )
      mxwarn = maxwarn
      nwarn = nwarnbef

      if ( valcoef<minval .or. valcoef>maxval ) then

!     --- Error: value of coefficient out of range

         call erreal ( valcoef, 1 )
         call erreal ( minval, 2 )
         call erreal ( maxval, 3 )
         call errsub ( 1379, 0, 3, 0 )

      end if  ! ( valcoef<minval .or. valcoef>maxval )
      readconstvar = valcoef

1000  call erclos ( 'readconstvar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readconstvar'

      end if  ! ( debug )

      end
