      subroutine readintdef ( intname, intvalue, intvaldef )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-01-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if integer constant can be found in part constants of input file
!     If so, set constant otherwise use default value
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     constant
!     get_parameter
!     integer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intvalue, intvaldef
      character (len=*) intname

!     intname        i    Name of integer
!     intvaldef      i    Default value
!     intvalue       o    Value of integer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNAMEINT     Return with sequence number of integer in incons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readintdef' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readintdef'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get sequence number of real

      iseq = getnameint ( intname )
      if ( iseq>0 ) then

!     --- Sequence number>0, hence constant found, fill value

         intvalue = incons(iseq)

      else

!     --- Sequence number = 0, use default value

         intvalue = intvaldef

      end if  ! ( iseq>0 )

1000  call erclos ( 'readintdef' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readintdef'

      end if  ! ( debug )

      end
