      subroutine readnlfill
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-01-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays itype, tabmin, tabmax for subroutine readnl
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numsubcrit, numsuberr, numsubiter, stiter, sterror

!     numsubcrit     number of subkeywords corresponding to criterion
!     numsuberr      number of subkeywords corresponding to at_error
!     numsubiter     number of subkeywords corresponding to iteration_method
!     sterror        starting address of subkeyword at_error
!     stiter         starting address of subkeyword iteration_method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readnlfill' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnlfill'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numsubcrit = 8
      numsuberr = 3
      numsubiter = 3
      sterror = 1+numsubcrit
      stiter = sterror+numsuberr

!     --- Fill array code

      nncode = 16
      code(1) = 'MAXITER'
      code(2) = 'MINITER'
      code(3) = 'PRINT_LEVEL'
      code(4) = 'ACCURACY'
      code(5) = 'RELAXATION'
      code(6) = 'CRITERION'
      code(7) = 'LIN_SOLVER'
      code(8) = 'AT_ERROR'
      code(9) = 'OUTPUT'
      code(10) = 'ITERATION_METHOD'
      code(11) = 'SEQTOTAL_VEC'
      code(12) = 'ABS_RESIDUAL'
      code(13) = 'REL_RESIDUAL'
      code(14) = 'LIMIT_SOLUTION'
      code(15) = 'LINEAR_SUBELEMENT'
      code(16) = 'CALL_FUNC'

!     --- Fill array ncode

      ncode(1:2) = 3
      ncode(3) = 5
      ncode(4) = 3
      ncode(5) = 5
      ncode(6) = 4
      ncode(7) = 7
      ncode(8) = 5
      ncode(9) = 6
      ncode(10) = 4
      ncode(11) = 6
      ncode(12:13) = 7
      ncode(14) = 5
      ncode(15) = 10
      ncode(16) = 8

!     --- Fill array ncodmx

      ncodmx(1:2) = 7
      ncodmx(3) = 11
      ncodmx(4) = 8
      ncodmx(5) = 10
      ncodmx(6) = 9
      ncodmx(7) = 10
      ncodmx(8) = 9
      ncodmx(9) = 6
      ncodmx(10) = 16
      ncodmx(11) = 15
      ncodmx(12:13) = 21
      ncodmx(14) = 14
      ncodmx(15) = 17
      ncodmx(16) = 9

!     --- Fill array itype

      itype(1:nncode) = 0
      itype(4:5) = 1    ! accuracy, relaxation
      itype(6) = 5      ! criterion
      itype(8) = 5      ! at_error
      itype(10) = 5     ! iteration_method
      itype(12:13) = 1  ! abs/rel_residual_accuracy
      itype(14:15) = 2  ! limit_solution, linear_subelement

!     --- Fill array tabmin

      tabmin(1:nncode) = 0d0
      tabmin(1:2) = 1d0
      tabmin(6:7) = 1d0    ! start of subkeywords criterion
      tabmin(8) = sterror  ! start of subkeywords at_error
      tabmin(9) = 1d0
      tabmin(10) = stiter  ! start of subkeywords iteration_method
      tabmin(11) = 1d0

!     --- Fill array tabmax

      tabmax(1:nncode) = 1d1
      tabmax(1) = 1d3
      tabmax(2) = 1d2
      tabmax(3:4) = 1d9
      tabmax(5) = 2d0
      tabmax(6) = numsubcrit
      tabmax(8) = stiter-1
      tabmax(10) = stiter+numsubiter-1
      tabmax(11) = 1d2
      tabmax(14:15) = 0d0
      tabmax(16) = 1d2

!     --- Fill array codesub

      codesub(1) = 'ABSOLUTE'
      codesub(2) = 'RELATIVE'
      codesub(3) = 'RHS_ABSOLUTE'
      codesub(4) = 'RHS_RELATIVE'
      codesub(5) = 'ENERGY_ABS'
      codesub(6) = 'ENERGY_RELATIVE'
      codesub(7) = 'RES_ABSOLUTE'
      codesub(8) = 'RES_RELATIVE'
      codesub(sterror) = 'STOP'
      codesub(sterror+1) = 'RETURN'
      codesub(sterror+2) = 'RESUME'
      codesub(stiter) = 'STANDARD'
      codesub(stiter+1) = 'NEWTON'
      codesub(stiter+2) = 'INCREMENTAL'

!     --- Fill array ncodesub

      ncodesub = 3
      ncodesub(3:4) = 7
      ncodesub(5:6) = 10
      ncodesub(7:8) = 7
      ncodesub(sterror) = 4
      ncodesub(sterror+2) = 4
      ncodesub(stiter:stiter+2) = 4

1000  call erclos ( 'readnlfill' )
      if ( debug ) then

!     --- Debug information

         call prinin ( itype, nncode, 'itype' )
         call prinrl ( tabmin, nncode, 'tabmin' )
         call prinrl ( tabmax, nncode, 'tabmax' )
         write(irefwr,*) 'End readnlfill'

      end if  ! ( debug )

      end
