      subroutine readscboun1 ( inpsep, leng1, rinsep, istrin, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-10-2015 Remove ibuffr
!        version  1.5  date 11-03-2014 New call to readc0
!        version  1.4  date 18-02-2014 Jump in case of error
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input following prescribe_boundary_conditions vector
!     No sequence number found
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), leng1, istrin,nprob
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Positions 1 to 4 are filled by readscbouncond
!                         Information of rest of positions:
!                         5:  number of integers stored
!                         6:  number of reals stored
!                         7:  starting address in array rinsep
!                         followed by numint positions of actual information
!                         as read by readc0
!                         followed by aray iinputinf of length nreal
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     leng1          o    Length of array inpsep used in this part
!     nprob          i    Number of various problems
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: leninpcre=20, lenrincre=10
      integer inpcre(leninpcre), icompl, inforinpcre(lenrincre), numint, &
              nreal, iprob, ivec
      double precision rincre(lenrincre)

!     icompl         Indicates if vector is real (0) or complex (1)
!     inforinpcre    Array of length of rincre that contains a 0 if the
!                    number stored in rincre is a real and a reference
!                    to scalars if it is a scalar
!     inpcre         The integer input read is stored in array INPCRE from
!                    position 1.
!     iprob          Actual problem number
!     ivec           Sequence number of vector of special structure
!     leninpcre      Declared length of array inpcre
!     lenrincre      Declared length of array rincre
!     nreal          Number of reals stored
!     numint         Number of integers stored
!     rincre         Double precision input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     READC0         Read standard data records
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscboun1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscboun1'
         write(irefwr,1) 'istrin, leng1, lastitem', &
                          istrin, leng1, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read rest of record

      nreal = 0
      icompl = 0
      lastitem = lastitem-1
      iprob = nprob
      nmain = 0
      call readc0 ( inpcre, rincre, numint, nreal, icompl, &
                    inforinpcre, iprob, ivec, 2 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'iprob, ivec, nreal', iprob, ivec, nreal

      call inichkarlen ( numint, leninpcre, 'inpcre' )
      call inichkarlen ( nreal, lenrincre, 'rincre' )

      if ( debug ) write(irefwr,1) 'after readc0, ierror =', ierror
      if ( ierror/=0 ) go to 1000

!     --- Store information in inpsep and rinsep

      inpsep(1) = -10      ! seq number

      if ( iprob>0 ) then
         inpsep(2) = iprob    ! problem number
      else if ( inpsep(2)<=0 ) then
         inpsep(2) = 1        ! problem number
      end if  ! ( iprob>0 )

      inpsep(4) = ivec     ! type of special vector

      if ( debug ) then
         call prinin ( inpsep, 4, 'inpsep/1' )
      end if  ! ( debug )
      inpsep(5) = numint
      inpsep(6) = nreal
      inpsep(7) = istrin
      leng1 = 7
      inpsep(leng1+1:leng1+numint) = inpcre(1:numint)
      leng1 = leng1+numint
      if ( nreal>0 ) then
         inpsep(leng1+1:leng1+nreal) = inforinpcre(1:nreal)
         rinsep(istrin:istrin+nreal-1) = rincre(1:nreal)
         leng1 = leng1+nreal
         istrin = istrin+nreal
      end if  ! ( nreal>0 )
      leng1 = leng1-3  ! correction for part used before

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscboun1'

      end if  ! ( debug )
      call erclos ( 'readscboun1' )

      end subroutine readscboun1
