      subroutine readsccreatfvc ( istart, inpsep, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.2  date 22-11-2004 Fortran 90
!        version  1.1  date 12-12-2002 Use V or VECTOR
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   create_force_vector
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     create
!     force_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), nprob

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: contains the sequence number for the coefficients
!                         3  IPROB   Problem sequence number
!                                    Default: 1
!                         4  IVECTOR Vector sequence number (first vector)
!                                    If ivector = -1, the same value as iprob
!                                    will be used
!                         5: indicates if quadratic elements should be
!                            treated as a cluster of linear subelements
!                            0: no subelements
!                            1: subelements
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=5
      integer ncode1(nncod1), ncod1m(nncod1), ivec
      character (len=18) code1(nncod1)

!     code1          Table with keywords
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ivec           Sequence number of vector with respect to array ISOL
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       CREATE_FORCE_VECTOR, options (all in one line)
!           where options may have the following contents
!               SEQ_COEF = s         (Default the next one)
!               PROBLEM = s          (Default 1)
!               VECTOR = s           (Default the same as problem)
!               LINEAR_SUBELEMENTS
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2379   The number of stand alone numbers found in the record too large
!    2380   The stand alone number found is not the first item read
!    2381   Combination of stand alone number and vector not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQ_COEF', 'PROBLEM', 'VECTOR', &
                   'LINEAR_SUBELEMENTS', 'V' /
      data ncode1 / 6, 4, 6,  5, 1 /
      data ncod1m / 8, 7, 6, 18, 1 /

!     code1   Table with sub keywords corresponding to keyword
!             create_force_vector
!             Values:
!             1:  SEQ_COEF
!             2:  PROBLEM
!             3:  VECTOR
!             4:  LINEAR_SUBELEMENTS
!             5:  V
! ======================================================================
!
      call eropen ( ' readsccreatfvc' )

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:nncod1) = 0
      itype(4) = 2
      table(1:nncod1) = -1d0
      tabmin(1:nncod1) = 1d0
      tabmax(1) = 25d0
      tabmax(2) = nprob
      tabmax(3) = dble(maxvec)
      tabmax(4) = 0d0
      tabmax(5) = tabmax(3)
      iretur = -10
      nmain = 0
      call sepreadsetkeyw     ! interpret keywords
      if ( ierror/=0 ) go to 1000
      if ( table(3)>0d0 .and. table(5)>0d0 ) &
         call errkeywexcl ( code1, ncod1m, 3, 5)

      table(3) = max(table(3),table(5))

      if ( iretur<-10 ) then

!     --- Stand alone number found.
!         This is only allowed if there is only one, if it is the first item
!         read and moreover if not vector is found

         if ( iretur<-11 ) then

!        --- More than one stand alone number found

            call errint ( -(iretur+10), 1 )
            call errint ( 1, 2 )
            call errsub ( 2379, 2, 0, 0 )

         end if
         if ( iseqkeyw(1)>=0 ) then

!        --- Number found is not the first item read

            call errchr ( 'CREATE_FORCE_VECTOR', 1 )
            call errsub ( 2380, 0, 0, 1 )

         end if
         if ( table(3)>0d0 ) then

!        --- Combination of stand alone number and vector not allowed

            call errchr ( 'CREATE_FORCE_VECTOR', 1 )
            call errsub ( 2381, 0, 0, 1 )

         end if

!        --- Put number read into table(2), i.e. ivec

         if ( ierror==0 ) table(3) = tabout(-iseqkeyw(1))

      end if

      inpsep(istart+1:istart+3) = nint(table(1:3))
      inpsep(istart+4) = nint(table(4))
      ivec = nint(table(3))
      if ( ivec>0 ) then
         numvec = min ( max(numvec,ivec), maxvec )
         if ( ivec<=maxvec ) ivectors(ivec) = 1
      end if
      istart = istart + 5

1000  call erclos ( ' readsccreatfvc' )

      end
