      subroutine readscextend ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the information following extend_axi_symmetric_mesh and
!     store in inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     symmetric
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         0:  icode (115)
!                         1:  mesh_in
!                         2:  mesh_out
!     istart        i/o   Starting address in array INPSEP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=2
      integer ncode1(nncod1), itype1(nncod1), ncod1m(nncod1)
      double precision tabmn1(nncod1), tabmx1(nncod1)
      character (len=8) code1(nncod1)

!     code1          Table with keywords
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     extend_axi_symmetric_mesh options
!     The following options are available
!     input_mesh = i
!     output_mesh = j
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'MESH_IN', 'MESH_OUT' /
      data ncode1 /  7, 7 /
      data ncod1m /  7, 8 /
      data itype1 /  0, 0 /
      data tabmn1 / 1d0, 1d0 /
      data tabmx1 / 5d0, 5d0 /

!     code1   Table with sub keywords corresponding to keyword REFINE_MESH
!             Values:
!             1:  MESH_IN
!             2:  MESH_OUT
! ======================================================================
!
      call eropen ( 'readscextend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscextend'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode+3
      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      call sepcopytype ( nncod1, itype1, tabmn1, tabmx1 )

      table(1) = 1d0
      table(2) = 2d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      inpsep(1) = nint(table(1))    ! mesh_in
      inpsep(2) = nint(table(2))    ! mesh_out
      istart = istart+3  ! raise istart

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 3, 'inpsep' )
         write(irefwr,*) 'End readscextend'

      end if  ! ( debug )
      call erclos ( 'readscextend' )

      end
