      subroutine readscintermsh ( istart, inpsep, icode, readnx, &
                                  name_keyw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-11-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   INTERCHANGE_MESH
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), icode
      logical readnx
      character (len=*) name_keyw

!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: New sequence number of mesh 1
!                         3: New sequence number of mesh 2
!     istart        i/o   Next free position in inpsep
!     name_keyw      i    Name of keyword
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision table1(2)
      integer n

!     n              number of items read
!     table1         Contains values of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSERINT  Read a series of integers
! **********************************************************************
!
!                       I/O
!
!       INTERCHANGE_MESH = (i,j) (Default 1,2)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1061   Only even number of numbers allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscintermsh' )

      inpsep(istart) = icode
      inpsep(istart+1) = 1
      inpsep(istart+2) = 2
      readnx = .true.
      call sepreadserint ( table1, 2, n, .true., 5, name_keyw, 1 )
      if ( ierror/=0 ) go to 1000
      if ( n==2 ) then

!     --- 2 numbers found, store in inpsep

         inpsep(istart+1) = nint(table1(1))
         inpsep(istart+2) = nint(table1(2))

      else if ( n==1 ) then

!     --- Error: only one number read

         call errint ( n, 1 )
         call errchr ( name_keyw, 1 )
         call errsub ( 1061, 1, 0, 1 )

      end if
      istart = istart + 3

1000  call erclos ( 'readscintermsh' )

      end
