      subroutine readscintersec ( istart, inpsep, rinsep, numscl, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the option following the keyword INTERSECT
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), numscl, istrin
      double precision rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         Contents:
!                         1:  icode       Return code indicates intersection
!                         2:  intlinedef  Type of storage of line
!                         3:  iscal       Sequence number of scalar
!                         4:  icurnr      Curve sequence number
!                         5:  istrin      Starting address in RINSEP
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Next free position in rinsep
!     numscl        i/o   Number of scalars read
!     rinsep         o    Array containing the real input for this subroutine
!                         Contents:
!                         1: x_origin
!                         2: y_origin
!                         3: angle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=6
      integer ncode1(nncod1), itype1(nncod1), ncod1m(nncod1), i
      double precision :: tabmn1(nncod1), tabmx1(nncod1)
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     i              Counting variable
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following text is recognized
!
!     INTERSECTION LINE, curve Ci, origin = (o_x,o_y), angle = alpha, &
!                        scalar = j
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'LINE', 'CURVE', 'ORIGIN', 'ANGLE', 'SCALAR', 'C' /
      data ncode1 /  4, 3, 4, 4, 4, 1 /
      data ncod1m /  4, 5, 6, 5, 6, 1 /
      data itype1 /  2, 2,17, 1, 0, 0  /
      data tabmn1 / 0d0, 0d0, 2d0, -7d0, 1d0, 1d0 /
      data tabmx1 / 0d0, 0d0, 1d0,  7d0, 1d2, 1d0 /

!     code1   Table with sub keywords corresponding to keyword INTERSECTION
!             Values:
!             1:  LINE
!             2:  CURVE
!             3:  ORIGIN
!             4:  ANGLE
!             5:  SCALAR
!             6:  C
! ======================================================================
!
      call eropen ( 'readscintersec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscintersec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode     ! code with respect to tosepcomsub

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      call sepcopytype ( nncod1, itype1, tabmn1, tabmx1 )

!     --- Fill tables for intx02
!         The keywords LINE and CURVES are just skipped

      table(1:nncode) = -1d20
      table(4) = 0d0  ! Default  angle
      table(5) = 1d0  ! Default  scalar sequence number

      tabmax(5) = dble(maxcls)
      tabmax(6) = dble(ncurvs)

!     --- Read input

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

!     --- Fill array inpsep

      inpsep(istart+1) = 1       ! type of line
      inpsep(istart+2) = mod(nint(table(5)),100000000) ! scalar sequence number
      inpsep(istart+3) = nint(table(6))   ! curve sequence number
      inpsep(istart+4) = istrin  ! sequence number first position in RINSEP

!     --- Fill array rinsep

      if ( table(3)>0 ) then

!     --- Origin found, fill in positions 1 and 2

         iposut = nint(table(3))
         rinsep(istrin) = tabout(iposut)
         rinsep(istrin+1) = tabout(iposut+1)

      else

!     --- Origin not found, use default values

         rinsep(istrin) = 0d0
         rinsep(istrin+1) = 0d0

      end if  ! ( table(3)>0 ) then

      rinsep(istrin+2) = table(4)  ! angle

!     --- Raise parameters

      numscl = max(numscl,inpsep(istart+2))
      istart = istart+5
      istrin = istrin+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscintersec'

      end if
      call erclos ( 'readscintersec' )

      end subroutine readscintersec
