      subroutine readscmvboun ( inpsep, istart, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-03-2012 New call to readseqvector
!        version  1.0  date 13-01-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     MOVE_BOUNDARY
!     with respect to subroutine readsc
!     The input is stored in array inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart, nprob

!     inpsep         o    The integer input read is stored in array INPSEP
!                         INPSEP is filled as follows
!                          1: IDISP, sequence number of displacement in isol
!                          2: IVEL, sequence number of velocity in isol
!                          3: INPADAP, sequence number of adapt boundary
!                          4: IPROB, problem number for displacement
!                          5: IPSI, sequence number of stream function in isol
!     istart        i/o   Next position in array inpsep
!                         The parameter is raised in this subroutine
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=5
      integer ncode1(nncode1), ncodmx1(nncode1), ivec
      character (len=17) code1(nncode1)

!     code1         Table with keywords
!     ivec           Sequence number of vector of special structure
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       MOVE_BOUNDARY, options
!           where options may have the following contents
!           DISPLACEMENT_VECTOR = ...
!           VELOCITY_VECTOR = ...
!           SEQ_BOUN_INPUT = ...
!           PROB_DISPLACEMENT = ...
!           STREAM_FUNCTION = ...
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
!    2997   No displacement vector found in input for moving boundary
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'DISP_VECTOR', 'VEL_VECTOR', &
                  'SEQ_BOUN_INPUT', 'PROB_DISPLACEMENT', &
                  'STREAM_FUNCTION' /
      data ncode1  /  8, 6, 8, 4, 6 /
      data ncodmx1 / 11,10,14,17,15 /

!     code1   Table with sub keywords corresponding to keyword MOVE_BOUNDARY
!             Values:
!             1:  DISP_VECTOR
!             2:  VEL_VECTOR
!             3:  SEQ_BOUN_INPUT
!             4:  PROB_DISPLACEMENT
!             5:  STREAM_FUNCTION
! ======================================================================
!
      call eropen ( 'readscmvboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscmvboun'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode      ! standard first item

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      itype(1:nncode) = 38
      itype(3:4) = 0
      tabmin(1:nncode) = 0d0
      tabmin(3) =1d0
      tabmax(1:nncode) = 0d0
      tabmax(3) = 1d1

!     --- Clear table

      table(1:nncode) = -1d0

!     --- Clear inpsep

      inpsep(istart+2:istart+nncode) = 1   ! default values

      tabmax(1) = dble(maxvec)  ! maximum number of vectors
      tabmax(2) = dble(maxvec)  ! maximum number of vectors
      tabmax(4) = nprob             ! number of problems
      tabmax(5) = dble(maxvec)  ! maximum number of vectors
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

!     --- Displacement vector

      call readseqvector ( table(1), 'displacement', ivec, 2 )
      if ( ivec>0 ) then

!     --- displacement vector found

         inpsep(istart+1) = ivec
         numvec = max(numvec,ivec)
         if ( ivec<=maxvec ) ivectors(ivec) = 1

      else

!     --- Error: no displacement vector found

         call errsub ( 2997, 0, 0, 0 )
         call errcrd

      end if  ! ( ivec>0 )

!     --- velocity vector

      call readseqvector ( table(2), 'velocity', ivec, 1 )

      if ( ivec>0 ) then

!     --- velocity vector found

         inpsep(istart+2) = ivec

         if ( ivectors(ivec)==0 ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if

      end if  ! ( ivec>0 )

      if ( table(3)>0d0 ) inpsep(istart+3) = nint(table(3)) ! seq_boun_input
      if ( table(4)>0d0 ) inpsep(istart+4) = nint(table(4)) ! problem
      call readseqvector ( table(5), 'psi', inpsep(istart+5), 3 )  ! psi vector

      istart = istart+nncode+1    ! raise istart

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscmvboun'

      end if  ! ( debug )
      call erclos ( 'readscmvboun' )

      end
