      subroutine readscnewvec
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-05-2014 New call to intx02keyw
!        version  1.0  date 23-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if text following main keyword is not a keyword or a known vector
!     If it is none of them create a new vector with the text as name
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ::  check, finished

!     check          Indicates if an option is satisfied (true) or not
!     finished       dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDVEC         Add vector name to the list of vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPMAINKEYW2   Check if string read is (main) keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscnewvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscnewvec'
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      check = .false.
      if ( nitems>1 ) then

!     --- more than one item

         lastitem = 2
         if ( typeitems(2)==26 ) lastitem = 3  ! raise lastitem in case of , &
         check = nitems>=lastitem

      end if  ! ( nitems>1 )

      if ( check ) check = typeitems(lastitem)==3     ! second item is string

      if ( check ) then

!     --- second item is string, check if it is a main keyword

         lastitembef = 1      ! necessary to get the next keyword
         itype(1:nncode) = 0
         nmain = 0
         call sepmainkeyw2 ( finished )
         if ( debug ) &
            write(irefwr,1) 'lastitem, ityprd, icode', &
                             lastitem, ityprd, icode

         if ( icode==0 .and. ityprd==4 ) then

!        --- string found, which does not correspond to a keyword or an
!            existing vector

            call addvec

         end if  ! ( icode==0 .and. ityprd==4 )

      end if  ! ( check )

      lastitembef = 0
      lastitem = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscnewvec'

      end if  ! ( debug )
      call erclos ( 'readscnewvec' )

      end
