      subroutine readscplotint ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-01-2015 extra parameter nstrng
!        version  1.2  date 17-04-2014 Make inpsep local
!        version  1.1  date 03-07-2012 Use subroutine intxstoretext
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword PLOT_INTERSECTION
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     plot
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: istart, inpsep(0:*), istrin
      double precision, intent(inout) :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 0-6 are used:
!                          0:  ICODE
!                          1:  IVECTOR
!                          2:  IDEGFD
!                          3:  ISTXORIG; start address xorig
!                          4:  ISTYORIG; start address yorig
!                          5:  ISTANGLE; start address angle
!                          6:  ITEXTX: sequence number of x-text in TEXT array
!                          7:  NTEXTX: number of characters in x-text
!                          8:  ITEXTY: sequence number of y-text in TEXT array
!                          9:  NTEXTY: number of characters in y-text
!                         10:  NSURFACES
!                         11:  STARTSURFACES
!                              From position startsurfaces:
!                              surface numbers
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Next free position in rinsep
!     rinsep         o    Array containing the real input for this subroutine
!                         Contents:
!                         1:   x-coordinate origin
!                         2:   y-coordinate origin
!                         3:   angle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=7
      integer :: ncode1(nncode1), ncodmx1(nncode1)
      character (len=8) :: code1(nncode1)
      integer ::  m

!     code1          Table with keywords
!     m              number of values read
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSTOREREAL   Store value of real or information of the corresponding
!                    scalar or expression in inpsep and rinsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2233   number of reals read incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!       PLOT_INTERSECTION V1, options
!           where options may have the following contents
!           degfd=i
!           textx = '..'
!           texty = '..'
!           origin = (O_x,O_y)
!           angle = a
!           surfaces = (si)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'DEGFD', 'TEXTX', 'TEXTY', 'ORIGIN', 'ANGLE', 'V', &
                  'SURFACES' /
      data ncode1  /  3, 5, 5, 4, 3,   1, 4 /
      data ncodmx1 /  5, 5, 5, 6, 5,   1, 8 /
! ======================================================================
!
      call eropen ( 'readscplotint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplotint'
         write(irefwr,1) 'istart, istrin', istart, istrin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode

!     --- Input for subroutine sepreadsetkeyw

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      codesub(1) = 'S'
      ncodesub(1) = 1


      itype(1:nncode) = 0
      itype(2:3) = 6   ! textx, y
      itype(4) = 19    ! origin
      itype(5) = 19    ! angle

      tabmin(1:nncode) = 0d0
      tabmin(1) = 1d0  ! degfd
      tabmin(4) = 2d0  ! max number of coordinates
      tabmin(5) = 1d0  ! max number of angles
      tabmin(6) = 1d0  ! V

      tabmax(1:nncode) = 0d0
      tabmax(1) = 1d2  ! degfd
      tabmax(4) = 1d9  ! max value of coordinates
      tabmax(5) = 360d0  ! maximum angle
      tabmax(6) = 1d2  ! V

      table = 0d0

      itype(7) = 18    ! surfaces
      table(7) = -1d0  ! reference to code1
      tabmin(7) = 1d1
      tabmax(7) = nsurfs

      inpsep(1:11) = 0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, nncode, 'table' )

      if ( table(1)>0d0 ) then
         inpsep(2) = nint(table(1))   ! degfd
      else
         inpsep(2) = 1                ! default degfd
      end if

!     --- textx

      call intxstoretext ( inpsep(6), table(2), tabmax(2), strout )

!     --- texty

      call intxstoretext ( inpsep(8), table(3), tabmax(3), strout )

      inpsep(1) = nint(table(6))  ! sequence number of vector

      if ( table(4)>0d0 ) then

!     --- origin

         iposut = nint(table(4))
         m = nint(tabmax(4))

         if ( debug ) then
            write(irefwr,1) 'iposut, m', iposut, m
            call prinrl ( tabout(iposut), m, 'tabout' )
         end if  ! ( debug )

         if ( m/=2 ) then

!        --- Not two reals found after origin

            call errint ( m, 1 )
            call errint ( 2, 2 )
            call errchr ( code(4)(1:ncodmx(4)), 1 )
            call errsub ( 2233, 2, 0, 1 )
            go to 1000

         end if  ! ( m/=2 )

         call sepstorereal ( inpsep(3), rinsep, istrin )     ! istxorig
         call sepstorereal ( inpsep(4), rinsep, istrin )     ! istyorig

      end if  ! ( table(5)>0d0 )

      if ( table(5)>0d0 ) then

!     --- angle

         call sepstorereal ( inpsep(5), rinsep, istrin )     ! istangle

      end if  ! ( table(5)>0d0 )

      if ( table(7)>0d0 ) then

!     --- surfaces

         iposut = nint(table(7))   ! start in tabout
         m = nint(tabmax(7))       ! number of surfaces

         inpsep(10) = m
         inpsep(11) = 12           ! start of surfaces in inpsep

!        --- Store surfaces

         inpsep(12:11+m) = nint(tabout(iposut:iposut+m-1))

      end if  ! ( table(7)>0d0 )

!     --- Raise istart

      istart = istart+12+inpsep(10)

1000  if ( debug ) then

!     --- Debug information

         call prininlarge ( inpsep(0), 12+inpsep(10), 'inpsep' )
         write(irefwr,1) 'istart, istrin', istart, istrin
         write(irefwr,*) 'End readscplotint'

      end if  ! ( debug )
      call erclos ( 'readscplotint' )

      end subroutine readscplotint
