      subroutine readscplottim ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 28-03-2015 Extension with scalars
!        version  1.3  date 16-04-2014 Extension with idegfd
!        version  1.2  date 09-03-2013 Correction loop
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the options following the keyword PLOT_TIME_HISTORY
!     with respect to subroutine readsc
!     The input is stored in array inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_history
!     plot
!     scalar
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart, istrin
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 0    92 to indicate plot time_history
!                              1    vector sequence number
!                                   In case of scalars: -10000-num
!                                   with num the number f scalars
!                              2    Starting address in array rinsep
!                              3    number of colors (mcol)
!                              4    legenda
!                              5    degfd
!                              6    In case of scalars: iksipzero
!                                   Indicates if the first time step must
!                                   be skipped (1) or not
!                              7    starting address of scalars in inpsep
!                                   (istart)
!                             10-9+mcol    color sequence numbers
!                             istart-istart+num-1 scalar sequence numbers
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   The real input read is stored in array RINSEP
!                         Contents
!                         Pos. 1    PLOTFM, size of plot
!                         Pos. 2    YFACT, minus size of plot
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=6
      integer ncode1(nncod1), ncod1m(nncod1), &
              mleg, mcol, istartscal, num
      character (len=9) code1(nncod1)

!     code1          Table with keywords
!     istartscal     Start of scalars in inpsep
!     mcol           number of colors
!     mleg           number of legenda
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     num            Number of scalars read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following options are allowed:
!       PLOT_TIME_HISTORY, VECTOR = v
!       PRINT TIME_HISTORY, set of scalars, SKIP_ZERO
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2549   Both scalars and points in vectors not allowed in time history
!    3056   More legenda than colors
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTOR', 'COLORS', 'LEGENDA', 'DEGFD', 'SCALAR', &
                   'SKIP_ZERO' /
      data ncode1 / 1, 3, 3, 3, 3, 4 /
      data ncod1m / 6, 7, 7, 5, 6, 9 /
! ======================================================================
!
      call eropen ( 'readscplottim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplottim'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = 92   ! plot time_history

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      table(1:nncode) = 0d0
      itype(1) = 38   ! Vector or vector name
      itype(2) = 11   ! colors
      tabmin(2) = 20d0 ! maximum number of colors
      tabmax(2) =145d0 ! Largest number in color list
      itype(3) = 19   ! legenda
      tabmin(3) = 20d0 ! maximum number of values
      tabmax(3) = 1d10 ! largest value
      itype(4) = 0    ! degfd
      itype(5) = 45   ! Set of scalars or scalar names
      itype(6) = 2    ! Skip_zero
      tabmin(4) = 1d0  ! minimum value
      tabmax(4) = 1d1  ! largest value

      mcol = 0    ! no colours
      mleg = 0    ! no lengenda
      num = 0     ! number of scalars
      iretur = 0
      nmain = 0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncod1, 'table' )

      inpsep(1) = nint(table(1))  ! vector sequence number
      inpsep(2) = istrin  ! starting address in rinsep
      inpsep(3:10) = 0

      if ( table(2)>0d0 ) then

!     --- Colors found, fill in inpsep

         iposut = nint(table(2))  ! starting address in tabout
         mcol = nint(tabmax(2))    ! number of entries read
         inpsep(3) = mcol
         inpsep(11:10+mcol) = nint(tabout(iposut:iposut+mcol-1))

      end if  ! ( table(2)>0d0 )

      if ( table(3)>0d0 ) then

!     --- Legenda found, fill in rinsep

         iposut = nint(table(3))  ! starting address in tabout
         mleg = nint(tabmax(3))    ! number of entries read
         inpsep(4) = mleg
         rinsep(istrin+2:istrin+1+mleg) = tabout(iposut:iposut+mleg-1)

         if ( mleg>mcol ) then

!        --- More lengenda than colors

            call errint ( mleg, 1 )
            call errint ( mcol, 2 )
            call errsub ( 3056, 2, 0, 0 )

         end if  ! ( mleg>mcol )

      end if  ! ( table(3)>0d0 )

      inpsep(5) = 1
      if ( table(4)>0d0 ) inpsep(5) = nint(table(4))  ! degfd
      inpsep(6) = nint(table(6))                       ! skip zero

      if ( table(5)>0d0 ) then

!     --- Scalars found

         if ( table(1)>0d0 ) then

!        --- Error both scalars and vectors found (not allowed)

            call errsub ( 2549, 0, 0, 0 )

         end if  ! ( table(1)>0d0 )

         iposut = nint(table(5))
         num = nint(tabmax(5))  ! number of scalars
         inpsep(1) = -num-10000
         inpsep(6) = nint(table(6))
         istartscal = 10+mcol
         inpsep(7) = istartscal
         inpsep(istartscal:istartscal+num-1) = &
            mod(nint(tabout(iposut:iposut+num-1)),100000000)

      end if  ! ( table(5)>0d0 )

!     --- Fill rinsep

      rinsep(istrin) = 15d0      ! plotfm
      rinsep(istrin+1) = -15d0   ! yfact (- size of plot)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(istart), 11+mcol, 'inpsep' )
         call prinrl ( rinsep(istrin), 2+mleg, 'rinsep' )
         write(irefwr,*) 'End readscplottim'

      end if  ! ( debug )
      call erclos ( 'readscplottim' )

!     --- Raise istart

      istart = istart + 11+mcol+num
      istrin = istrin + 2+mleg

      end subroutine readscplottim
