      subroutine readscplotvc ( istart, inpsep, istrin, rinsep, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 17-05-2011 plot boundary zero levelset
!        version  1.1  date 20-12-2002 Allow plot_vector i
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read the input for the vector plot with respect to subroutine readsc
!  The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     plot_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin, ndim
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-7 are used:
!                         1:  ICODE
!                             39: plot vector
!                             57: plot tensor
!                         2:  IVECTOR
!                         3:  IREGION
!                             If IREGION#0, 4 positions in rinsep are used
!                         4:  IDEGFD1
!                         5:  IDEGFD2
!                         6:  IYFACT
!                             If IYFACT#0, 1 extra position in rinsep is used
!                         7:  IFACTOR
!                             If IFACTOR#0, 1 extra position in rinsep is used
!                         8:  IDEGFD3 (only for plot tensor)
!                        10:  NINNER: plot inner boundaries
!                             0: not
!                             1: plot
!                             +10:  plot zero level set boundary
!                        11:  IAXI: if 1 the axi-symmetric mesh is used
!                                   and the solution extended to this mesh
!                         See subroutine readsc
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     ndim           i    Dimension of the space.
!     rinsep        i/o   Array containing the real input for this subroutine
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=10
      integer ncode2(nncode2), ncodmx2(nncode2)
      character (len=20) code2(nncode2)
      integer i

!     code2          Table with keywords with respect to plot vector
!     i              Counting variable
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodmx2        Table containing the number of maximum characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!           zero_level_set
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!    2379   The number of stand alone numbers found in the record too large
!    2380   The stand alone number found is not the first item read
!    2381   Combination of stand alone number and vector not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'REGION', 'YFACT', 'DEGFD', 'DEGFD', 'DEGFD', &
                   'DEGFD', 'FACTOR' ,'V', 'ZERO_LEVEL_SET', &
                   'AXI_SYMMETRIC' /
      data ncode2  /  3, 3, 3, 3, 3, 3, 4, 1, 4, 3 /
      data ncodmx2 /  6, 5, 5, 5, 5, 5, 6, 1,14,13 /

!     code2  Table with sub keywords corresponding to keyword PLOT_VECTOR
!             Values:
!             1:  REGION
!             2:  YFACT
!             3:  DEGFD
!             4:  DEGFD
!             5:  DEGFD
!             6:  DEGFD
!             7:  FACTOR
!             8:  V
!             9:  ZERO_LEVEL_SET
!            10:  AXI_SYMMETRIC
! ======================================================================
!
      call eropen ( 'readscplotvc' )

      inpsep(0) = icode

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      itype(1:nncode) = 0
      itype(1) = 19  !  REGION
      itype(2) = 1   !  YFACT
      itype(3) = 10  !  DEGFD
      itype(7) = 1   !  FACTOR
      itype(9) = 2   !  ZERO_LEVEL_SET
      itype(10) = 2  !  AXI_SYMMETRIC

      tabmin(1:nncode) = 1d0
      tabmin(1) = 2d0*ndim   !  REGION
      tabmin(2) = 0d0   !  YFACT
      tabmin(7) = 1d-5  !  FACTOR
      tabmin(8) = 1d0   !  V

      tabmax(1:nncode) = 1d2
      tabmax(1) = 1d5   !  REGION
      if ( icode==57 ) then
         tabmax(3) = 4d0  ! plot_tensor
      else
         tabmax(3) = 2d0  ! plot_vector
      end if
      tabmax(7) = 1d5   !  FACTOR
      tabmax(8) = 1d2   !  V

      table = 0d0
      table(8) =-1d0   !  V
      if ( ndim/=2 ) then

!     --- Only ndim = 2 is allowed

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if
      iretur = -10
      nmain = 0
      call sepreadsetkeyw     ! interpret keywords
      if ( table(1)>0 ) then
         inpsep(2) = istrin
         do i = 0, 2*ndim-1
            rinsep(istrin+i) = tabout(i+1)
         end do
         istrin = istrin+2*ndim
      else
         inpsep(2) = 0
      end if
      if ( table(2)>0 ) then
         inpsep(5) = istrin
         rinsep(istrin) = table(2)
         istrin = istrin+1
      else
         inpsep(5) = 0
      end if
      if ( table(3)>0d0 ) then
         inpsep(3) = nint(table(3))
      else
         inpsep(3) = 1
      end if
      if ( table(4)>0d0 ) then
         inpsep(4) = nint(table(4))
      else
         inpsep(4) = inpsep(3)+1
      end if
      if ( table(5)>0d0 ) then
         inpsep(7) = nint(table(5))
      else
         inpsep(7) = inpsep(4)+1
      end if
      if ( table(6)>0d0 ) then
         inpsep(8) = nint(table(6))
      else
         inpsep(8) = inpsep(7)+1
      end if

      if ( table(7)>0d0 ) then
         rinsep(istrin) = table(7)
         inpsep(6) = istrin
      else
         rinsep(istrin) = 0d0
         inpsep(6) = 0
      end if
      istrin = istrin+1
      if ( iretur<-10 ) then

!     --- Stand alone number found.
!         This is only allowed if there is only one, if it is the first item
!         read and moreover if not vector is found

         if ( iretur<-11 ) then

!        --- More than one stand alone number found

            call errint ( -(iretur+10), 1 )
            call errint ( 1, 2 )
            call errsub ( 2379, 2, 0, 0 )

         end if
         if ( iseqkeyw(1)>=0 ) then

!        --- Number found is not the first item read

            call errchr ( 'PLOT_VECTOR', 1 )
            call errsub ( 2380, 0, 0, 1 )

         end if
         if ( table(8)>0d0 ) then

!        --- Combination of stand alone number and vector not allowed

            call errchr ( 'PLOT_VECTOR', 1 )
            call errsub ( 2381, 0, 0, 1 )

         end if

!        --- Put number read into table(8), i.e. ivec

         if ( ierror==0 ) table(8) = tabout(-iseqkeyw(1))

      end if

      inpsep(9) = 0   ! ninner
      inpsep(10) = nint(table(10))
      if ( table(9)>0d0 ) inpsep(9) = inpsep(9)+10 !zero_level

      inpsep(1) = max(1,nint(table(8)))
      istart = istart+11

      call erclos ( 'readscplotvc' )

      end
