      subroutine readscrefmesh ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 25-03-2005 Decrease input
!        version  1.2  date 22-11-2004 Fortran 90
!        version  1.1  date 25-10-2004 Extension with local refinement
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the keyword
!     REFINE_MESH
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  N (Number of refinements)
!                         3:  MESH_IN Sequence number of mesh to be refined
!                         4:  MESH_OUT Sequence number of refined mesh
!                         5:  ISEQUENCE Sequence number of input block refine
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=4
      integer ncode1(nncod1), itype1(nncod1), ncod1m(nncod1)
      double precision tabmn1(nncod1), tabmx1(nncod1)
      character (len=16) code1(nncod1)

!     code1          Table with keywords
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       REFINE_MESH, options (all in one line)
!           where options may have the following contents
!           TIMES = 2, Number of refinements (Default 1)
!           MESH_IN (Default 1)
!           MESH_OUT (Default MESH_IN)
!           SEQUENCE_NUMBER = i
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'TIMES', 'MESH_IN', 'MESH_OUT', 'SEQUENCE_NUMBER' /
      data ncode1 /  4, 7, 7, 3 /
      data ncod1m /  5, 7, 8,15 /
      data itype1 /  0, 0, 0, 0 /
      data tabmn1 / 1d0, 1d0, 1d0, 1d0 /
      data tabmx1 / 5d0, 5d0, 5d0,10d0 /

!     code1   Table with sub keywords corresponding to keyword REFINE_MESH
!             Values:
!             1:  TIMES
!             2:  MESH_IN
!             3:  MESH_OUT
!             4:  SEQUENCE_NUMBER
! ======================================================================
!
      call eropen ( 'readscrefmesh' )

      inpsep(istart) = icode
      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      call sepcopytype ( nncod1, itype1, tabmn1, tabmx1 )

      table(1) = -1d0
      table(2) = -1d0
      table(3) = -1d0
      table(4) = 1d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      inpsep(istart+1) = nint(table(1))    ! times
      inpsep(istart+2) = nint(table(2))    ! mesh_in
      inpsep(istart+3) = nint(table(3))    ! mesh_out
      inpsep(istart+4) = nint(table(4))    ! sequence number
      istart = istart+5

      call erclos ( 'readscrefmesh' )

      end subroutine readscrefmesh
