      subroutine readscskipel ( table, tabout, inpsep, istartnum, iskip, &
                                tabmax, nelgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 1-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret information concerning skip element groups and store
!     in inpsep(iskip) and from inpsep(istartnum)
!     raise istartnum
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     skip
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iskip, nelgrp
      integer, intent(inout) :: inpsep(0:*), istartnum
      double precision, intent(in) ::  table(*), tabout(*), tabmax(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         in position iskip istartnum
!                         From position istartnum nelgrp positions
!                         with 0 and 1
!     iskip          i    Position in inpsep
!     istartnum     i/o   Starting address of a series of numbers in INPSEP
!     nelgrp         i    Number of element groups
!     table          i    Contains information of skip element groups
!                         Stored in position 1 and 2
!     tabmax         i    Contains maximum values allowed in table
!                         Also referring to positions 1 and 2
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ipos

!     i              Counting variable
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1900   Both element groups and skip element groups given
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscskipel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscskipel'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table(1)>0d0 .or. table(2)>0d0 ) then

!     --- Element groups must be skipped

         inpsep(iskip) = istartnum
         if ( table(1)>0d0 ) then

!        --- element groups = given

            ipos  = nint(table(1))
            inpsep(istartnum:istartnum+nelgrp-1) = 0
            do i = 1, nint(tabmax(1))
               inpsep(istartnum-1+nint(tabout(i+ipos-1))) = 1
            end do  ! i = 1, nint(tabmax(1))

         else

!        --- skip element groups = given

            ipos  = nint(table(2))
            inpsep(istartnum:istartnum+nelgrp-1) = 1
            do i = 1, nint(tabmax(2))
              inpsep(istartnum-1+nint(tabout(i+ipos-1))) = 0
            end do  ! i = 1, nint(tabmax(2))

         end if  ! ( table(1)>0d0 )
         istartnum = istartnum+nelgrp

      end if  ! ( table(1)>0d0 .or. table(2)>0d0 )

      if ( table(1)>0d0 .and. table(2)>0d0 ) &
           call errsub ( 1900, 0, 0, 0 )

1000  call erclos ( 'readscskipel' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscskipel'

      end if  ! ( debug )

      end subroutine readscskipel
