      subroutine readseqint ( iinput, defvalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 28-12-2014 New parameter list
!        version  1.0  date 01-05-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the integer variable read in iinput
!     If no sequence number is found, check if the variable is known
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput, defvalue

!     defvalue       i    Default value
!     iinput         o    Entry of integer input array referring to variable
!                         If value stored in table(icode) iinput contains the
!                         entry in rinput where the value is stored
!                         If the variable is a scalar minus the entry
!                         in scalars is stored in iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number in set of scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnamevar, getnameint, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     GETNAMEVAR     Return with sequence number of variable in scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqint'
         write(irefwr,1) 'icode, infotable(icode)', &
                          icode, infotable(icode)
         write(irefwr,2) 'table(icode), tabmin(icode)', &
                          table(icode), tabmin(icode)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( infotable(icode)==11 ) then

!     --- variable found

         iseq = nint(table(icode))
         iinput = -iseq-maxints-maxreals

      else if ( table(icode)>=tabmin(icode) ) then

!     --- value is given

         iinput = nint(table(icode))

      else

!     --- value is not given, test if there is a variable with name
!         name

         iseq = getnamevar ( code(icode)(1:ncodmx(icode)) )
         if ( debug ) write(irefwr,1) 'iseq', iseq
         if ( iseq>0 ) then

!        --- iseq>0, variable name found

            iinput = -iseq-maxints-maxreals-100000000

         else

!        --- iseq = 0, variable name not found
!            Check if it is an integer or real

            iseq = getnameint ( code(icode)(1:ncodmx(icode)) )
            if ( iseq>0 ) then
               iinput = -iseq-100000000  ! integer found
            else
               iseq = getnamereal ( code(icode)(1:ncodmx(icode)) )
               if ( iseq>0 ) then
                  iinput = -iseq-maxints-100000000  ! real found
               end if  ! ( iseq>0 )
            end if  ! ( iseq>0 )

            if ( iseq==0 ) then

!           --- Use default value

               iinput = defvalue

            end if  ! ( iseq==0 )

         end if  ! ( iseq==0 )

      end if  ! ( infotable(icode)==11 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput', iinput
         write(irefwr,*) 'End readseqint'

      end if  ! ( debug )
      call erclos ( 'readseqint' )

      end subroutine readseqint
