      subroutine readseqscalar ( value, name, iinput, defvalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the sequence number read in iinput
!     If no sequence number is found, check if the scalar name is known
!     Also an integer or real may be present
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleinput
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision value
      integer iinput, defvalue
      character (len=*) name

!     defvalue       i    Default value
!     iinput         o    entry of integer input array to be filled with
!                         integer sequence number
!     name           i    Name of array
!     value          i    real value read containing the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscalnm

!     iscalnm        scalar sequencer number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint, getnamereal, getnamevar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     GETNAMEVAR     Return with sequence number of variable in scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3133   No vector with name name found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readseqscalar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readseqscalar'
         write(irefwr,1) 'defvalue', defvalue
         write(irefwr,2) 'value', value
         write(irefwr,*) 'name ', name
  1      format ( a, 1x, (10i9) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( value>0d0 ) then

!     --- scalarname is given

         iinput = nint(value)+100000000

      else

!     --- scalarname is not given, test if there is a scalar with name
!         name

         iseqarray = -1  ! indicates that arrays are permitted
         iscalnm = getnamevar ( name )  ! check scalar

         if ( debug ) write(irefwr,1) 'iscalnm/1', iscalnm

         if ( iscalnm==0 ) then

!        --- No scalar found, perhaps an integer?

            iscalnm = getnameint ( name )  ! check integer
            if ( debug ) write(irefwr,1) 'iscalnm/2', iscalnm

            if ( iscalnm==0 ) then
               iscalnm = getnamereal ( name ) ! or real
               if ( iscalnm>0  ) then
                  iscalnm = iscalnm+maxints
               end if  ! ( iscalnm>0 ) then
            end if   ! ( iscalnm>0 )

         else

!        --- Scalar name found, raise sequence with maxints+maxreals

            iscalnm = iscalnm+maxints+maxreals

         end if  ! ( iscalnm==0 )

         if ( iscalnm>0 ) then
            iinput = iscalnm+100000000  ! sequence number of scalar
         else if ( defvalue>0 ) then
            iinput = defvalue  ! default value
         else if ( defvalue==0 ) then
            iinput = defvalue  ! default value
            go to 1000         ! jump
         else

!        --- defvalue<0: No scalar with name name found

            call errchr ( name, 1 )
            call errsub ( 3133, 0, 0, 1 )
            go to 1000

         end if  ! ( iscalnm>0 )

      end if  ! ( value(1)>0d0 )

      iseqarray = 0  ! reset iseqarray

      if ( debug ) write(irefwr,1) 'iinput', iinput

1000  call erclos ( 'readseqscalar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinput', iinput
         write(irefwr,*) 'End readseqscalar'

      end if

      end
