      subroutine readstoc ( ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-12-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read parameters for stochastic investment problem and store
!     in common cstoc
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestoc
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim

!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision readconstvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READCONSTVAR   Read constant (or variable) from  list of constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readstoc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstoc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==1 ) then

!     --- ndim = 1, only one sigma and mu

         mu = readconstvar ( 'mu', 0d0, 1d0 )
         sigma = readconstvar ( 'sigma', 0d0, 1d0 )

      else

!     --- ndim = 2, sigma, rho and mu

         mux = readconstvar ( 'mux', 0d0, 1d0 )
         muy = readconstvar ( 'muy', 0d0, 1d0 )
         sigmax = readconstvar ( 'sigmax', 0d0, 1d0 )
         sigmay = readconstvar ( 'sigmay', 0d0, 1d0 )
         rho = readconstvar ( 'rho', 0d0, 1d0 )

      end if  ! ( ndim==1 )

      rstoc = readconstvar ( 'r', 0d0, 1d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readstoc'

      end if  ! ( debug )
      call erclos ( 'readstoc' )

      end
