      subroutine readsubelms ( nelgrp, inpsep, ireadseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the keyword
!     SUBDIVIDE_ELEMENT
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelgrp, inpsep(*), ireadseq(0:*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         1:  type of subdivision
!                             1: nodes on outer boundary
!                         2:  iseq: Sequence number outer boundary
!     ireadseq      i/o   Array of size nelgrp+1 indicating if sequence numbers
!                         are free (0) or already in use (>0)
!                         ireadseq(0) refers to the complete outer boundary
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     subdivide_elements nodes_on_outer_boundary iseq
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2227   Only one of a combination of keywords found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsubelms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsubelms'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- read information

      code(1) = 'NODES_ON_OUTER_BOUNDARY'
      ncode(1) = 12
      ncodmx(1) = 23
      itype(1) =  0
      iretur = 0
      nmain = 0
      tabmin(1) = 0d0
      tabmax(1) = nelgrp
      table(1) = -1d0

      call sepreadsetkeyw
      iseq = nint(table(1))

      if ( iseq.ge.0 ) then

!     --- iseq > 0, store input in inpsep

         ireadseq(iseq) = 1
         inpsep(1) = 1
         inpsep(2) = iseq

      else

!     --- Error no sequence number read

         call errchr ( 'SUBDIVIDE_ELEMENT', 1 )
         call errchr ( code(1)(1:ncodmx(1)), 2 )
         call errsub ( 2227, 0, 0, 2 )
         go to 1000

      end if  ! ( iposut.gt.0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readsubelms'

      end if
      call erclos ( 'readsubelms' )

      end subroutine readsubelms
