      subroutine readvecname
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-05-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read name of vector following keyword
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numchar, ivecnm

!     ivecnm         vector sequencer number with respect to isol
!     numchar        Number of characters in keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READNAMEVEC    Read name of vector in common cpost5
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!     READVECNUM     Read the vector number following the letter V
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readvecname' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readvecname'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item
!         In case it is a string, numbers are included

      ispecl = 9       ! include numbers in string
      call asknxt

      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

      if ( ityprd==4 ) then

!     --- ityprd = 4, string found
!         First test if this is V followed by number

         if ( debug ) write(irefwr,*) 'string ', string(1:nchar)

         if ( nchar==1 .and. string(1:1)=='V' .and. lastitem<nitems ) then

!        --- V found, check if it is followed by a %-sign or a number
!            In that case jump

            if ( typeitems(lastitem+1)==1 .or. &
                 typeitems(lastitem+1)==10 ) go to 100

         end if

!     --- we have to check whether the keyword contains numbers
!         If so we have to split the keyword into a part without number
!         and a number
!         This is necessary to check if it is a keyword, since keywords do
!         not contain numbers

         call readsplitkeyw ( numchar, keyword )
         if ( debug ) then
            write(irefwr,1) 'ityprd/1', ityprd
            write(irefwr,*) 'string ', string(1:nchar)
            write(irefwr,*) 'keyword ', keyword(1:numchar)
         end if  ! ( debug )

         call readnamevec      ! check if this is a vectorname

         if ( ityprd/=2 ) then

!        --- Not a vector name found, reset string

            string = keyword    ! name without number
            lastitem = lastitembef+1
            nchar = numchar
            call asknxt

         end if  ! ( iyprd/=2 ) then

      end if  ! ( ityprd==4 )

100   if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,2) 'realrd', realrd
      end if  ! ( debug )

      call readvecnum ( ivecnm )   ! interpret vector as number
      if ( ierror/=0 ) go to 1000

      iinputpst(2) = ivecnm           ! store in input
      ispecl = 0                   ! reset ispecl

1000  call erclos ( 'readvecname' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivecnm', ivecnm
         write(irefwr,*) 'End readvecname'

      end if  ! ( debug )

      end
