      subroutine sepcopcoorsymm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates of axi-symmetric mesh and fill a part of new kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     coordinate
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesymm
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPCOPCOORSYM1 Copy coordinates of old mesh into coornew and mirror
!                    coordinates of symmetric points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopcoorsymm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopcoorsymm'
         write(irefwr,1) 'npointsymm, nelem', npointsymm, nelem
         call prinrl1 ( coor, npoint, 2, 'coor' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      km(iseqkm)%kmesh(8) = npointsymm
      km(iseqkm)%kmesh(9) = 2*nelem           ! new nelem

!     --- Create new array coor

      allocate ( coortmp(2,npointsymm), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npointsymm, 'coortmp' )
      if ( ierror/=0 ) go to 1000

!     --- Fill new array coor

      call sepcopcoorsym1 ( coortmp )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coortmp, npointsymm, 2, 'coortmp' )
         write(irefwr,*) 'End sepcopcoorsymm'

      end if  ! ( debug )
      call erclos ( 'sepcopcoorsymm' )

      end subroutine sepcopcoorsymm
