      subroutine sepcopsolsymm1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution from original mesh to extended (axi-symmetric) mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     2d
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesymm
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, istarts, inunkp, i, inode

!     i              Counting variable
!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
!     istarts        First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopsolsymm1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopsolsymm1'
         call prinin ( nodesnew, npoint, 'nodesnew' )
         call prinrl ( usol, nusol, 'usol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      usolsymm(1:nusol) = usol(1:nusol)

      if ( indprf>0 ) then

!     --- indprf>0 use kprobf and kprobfsymm

         do i = 1, npoint

            inode = nodesnew(i)

            if ( inode>npoint ) then

!           --- Node in extended mesh

               istart = kprobf(i)
               istarts = kprobfsymm(inode)
               inunkp = kprobf(i)-istart
               select case (inunkp)
               case(0)

!              --- No action, since no unknowns

               case(1)

!              --- one unknown, standard

                  usolsymm(istarts+1) = usol(istart+1)

               case default

!              --- more than one, unknown
!                  we suppose that the first one is a vector

                  usolsymm(istarts+1) = -usol(istart+1)
                  usolsymm(istarts+2:istarts+inunkp) = &
                     usol(istart+2:istart+inunkp)

               end select  ! case (inunkp)

            end if  ! ( inode>npoint )

         end do  ! i = 1, npoint

      else

!     --- indprf=0 constant number of unknowns

         inunkp = nunkp
         do i = 1, npoint

            inode = nodesnew(i)

            if ( inode>npoint ) then

!           --- Node in extended mesh

               istart = (i-1)*nunkp
               istarts = (inode-1)*nunkp
               select case (inunkp)
               case(1)

!              --- one unknown, standard

                  usolsymm(istarts+1) = usol(istart+1)

               case default

!              --- more than one, unknown
!                  we suppose that the first one is a vector

                  usolsymm(istarts+1) = -usol(istart+1)
                  usolsymm(istarts+2:istarts+inunkp) = &
                     usol(istart+2:istart+inunkp)

               end select  ! case (inunkp)

            end if  ! ( inode>npoint )

         end do  ! i = 1, npoint

      end if  ! ( indprf>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usolsymm, istarts+inunkp, 'usolsymm' )
         write(irefwr,*) 'End sepcopsolsymm1'

      end if  ! ( debug )
      call erclos ( 'sepcopsolsymm1' )

      end subroutine sepcopsolsymm1
