      subroutine sepfemesh ( iref, iren )
! ======================================================================
!
!        programmer    Luc Timmermans/Frans van de Vosse
!        version  1.8  date 30-01-2016 Replace meshwrbf by meshwr
!        version  1.7  date 29-05-2014 Extension of timebefore
!        version  1.6  date 21-11-2012 Long integers
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates a linear element mesh based on the nodes
!     of a spectral element mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iref, iren

!     iref      i      reference number
!     iren      i      indicates whether to renumber the spectral element
!                      mesh according to the FE renumbering (used when
!                      preconditioning is performed)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leng, ielgrp, ihelp, irenum, icount, &
              inpelm, nprev, idummy(10), lastps, iseqkm1, iseqkm2
      double precision rinpelm
      character (len=100) filename
      integer, pointer :: kmesh1, kmesh2
      double precision :: timebefore(2)

!     filename       Name of file
!     icount         counting variable
!     idummy         dummy array
!     ielgrp         element group number
!     ihelp          help variable
!     inpelm         Number of points per element
!     irenum         indicator for type of renumbering for kmesh2
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmesh1         standard sepran array, corresponding to the
!                    spectral element mesh
!     kmesh2         standard sepran array, corresponding to the
!                    linear element mesh
!     lastps         Last non-blank position in filename
!     leng           help parameter to declare a length of an array
!     nprev          ?
!     rinpelm        Help variable to make inpelm real
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     FILLPC         ?
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     MSHRDPARMESH   Read information parallel information about global mesh
!                    from file meshoutput_par.000
!     PRININ         print 1d integer array
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPMESHWR      fill the file meshoutput with the mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1696   Number of nodes per element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- check input

      call eropen ('sepfemesh')
      if ( itime>0 ) call ersettime ( timebefore )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfemesh'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmesh1 => kmesh2d(1)
      kmesh2 => kmesh2d(2)

      if ( debug ) write(irefwr,1) 'kmesh1, kmesh2', kmesh1, kmesh2
      kmesh => kmesh1
      call sepfillkmesh(kmesh)
      iseqkm1 = iseqkm
      call sepcrmeshinfo ( kmesh2 )
      iseqkm2 = iseqkm
      iseqkm = iseqkm1

      if ( debug ) then
         write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2
         write(irefwr,1) 'kmesh1, kmesh2', kmesh1, kmesh2
      end if  ! ( debug )

!     --- copy kmesh1 into kmesh2

      km(iseqkm2)%kmesh(4:14) = km(iseqkm1)%kmesh(4:14)
      km(iseqkm2)%kmesh(7) = km(iseqkm1)%kmesh(7)
      km(iseqkm2)%kmesh(42) = km(iseqkm1)%kmesh(42)
      km(iseqkm2)%kmesh(44) = km(iseqkm1)%kmesh(44)
      km(iseqkm2)%kmesh(46) = 4
      allocate ( km(iseqkm2)%kmeshw(4) )
      km(iseqkm2)%kmeshw = 0
      km(iseqkm2)%kmesh(58) = km(iseqkm1)%kmesh(58)
      km(iseqkm2)%kmesh(61) = km(iseqkm1)%kmesh(61)

!     --- adjust first part of kmesh2 for linear elements
!         first the maximal numbers of nodes per element

      if ( ndim==1 ) km(iseqkm2)%kmesh(4) = 2
      if ( ndim==2 ) km(iseqkm2)%kmesh(4) = 4
      if ( ndim==3 ) km(iseqkm2)%kmesh(4) = 8
      km(iseqkm2)%kmesh(17) = 0

      if ( kelma/=0 ) then
         allocate ( km(iseqkm2)%kmesha(nelgrp+1) )
         km(iseqkm2)%kmesha = kmesha
         km(iseqkm2)%kmesh(15) = nelgrp+1
      end if  ! ( kelma/=0 )

!     --- fetch kmesh2 b

      allocate ( km(iseqkm2)%kmeshb(nelgrp), km(iseqkm2)%kmeshf(nelgrp) )
      km(iseqkm2)%kmeshf = kmeshf
      km(iseqkm2)%kmesh(16) = nelgrp
      km(iseqkm2)%kmesh(20) = nelgrp
      km(iseqkm2)%kmesh(23) = -km(iseqkm1)%kmesh(23)
      km(iseqkm2)%coor => coor
      iseqkm = iseqkm1

      if ( kelmk>0 ) then
         allocate ( km(iseqkm2)%kmeshk(size(kmeshk)))
         km(iseqkm2)%kmeshk = kmeshk
         km(iseqkm2)%kmesh(25) = size(kmeshk)
      end if  ! ( kelmk>0 )

      if ( kelml/=0 ) then
         allocate ( km(iseqkm2)%kmeshl(size(kmeshl)))
         km(iseqkm2)%kmeshl = kmeshl
         km(iseqkm2)%kmesh(26) = size(kmeshl)
      end if  ! ( kelml/=0 )

      if ( kelmm/=0 ) then
         allocate ( km(iseqkm2)%kmeshm(size(kmeshm)))
         km(iseqkm2)%kmeshm = kmeshm
         km(iseqkm2)%kmesh(27) = size(kmeshm)
      end if  ! ( kelmm/=0 )

      if ( kelmn/=0 ) then
         allocate ( km(iseqkm2)%kmeshn(size(kmeshn)))
         km(iseqkm2)%kmeshn = kmeshn
         km(iseqkm2)%kmesh(28) = size(kmeshn)
      end if  ! ( kelmn/=0 )

      if ( kelmp>0 ) then
         allocate ( km(iseqkm2)%elsize(2,size(elsize,2)))
         km(iseqkm2)%elsize = elsize
         km(iseqkm2)%kmesh(30) = size(elsize)
      end if  ! ( kelmp>0 )

      if ( kelmq/=0 ) then
         allocate ( km(iseqkm2)%kmeshq(size(kmeshq)))
         km(iseqkm2)%kmeshq = kmeshq
         km(iseqkm2)%kmesh(31) = size(elsize)
      end if  ! ( kelmq/=0 )

      if ( kelmr/=0 ) then
         allocate ( km(iseqkm2)%kmeshr(size(kmeshr)))
         km(iseqkm2)%kmeshr = kmeshr
         km(iseqkm2)%kmesh(32) = size(kmeshr)
      end if  ! ( kelmr/=0 )

      if ( kelms/=0 ) then
         allocate ( km(iseqkm2)%kmeshs(size(kmeshs)))
         km(iseqkm2)%kmeshs = kmeshs
         km(iseqkm2)%kmesh(33) = size(kmeshs)
      end if  ! ( kelms/=0 )

      if ( kmiinput>0 ) then
         allocate ( km(iseqkm2)%iinputkm(size(iinputkm)))
         km(iseqkm2)%iinputkm = iinputkm
         km(iseqkm2)%kmesh(34) = size(kmeshs)
      end if  ! ( kmiinput>0 )

      if ( kmrinput>0 ) then
         allocate ( km(iseqkm2)%rinputkm(size(rinputkm)))
         km(iseqkm2)%rinputkm = rinputkm
         km(iseqkm2)%kmesh(35) = size(kmeshs)
      end if  ! ( kmrinput>0 )

      if ( mmouter>0 ) then
         allocate ( km(iseqkm2)%outerboun(size(outerboun)))
         km(iseqkm2)%outerboun = outerboun
      end if  ! ( mmouter>0 )

      if ( mmnormal>0 ) then
         allocate ( km(iseqkm2)%normal(size(normal,1),size(normal,2)))
         km(iseqkm2)%normal = normal
      end if  ! ( mmnormal>0 )

!     --- next the number of elements
!         at this moment only spectral elements with the same number of
!         nodes in each direction are possible
!         also compute the length of kmesh2 part c and loads kmesh part a

      if ( ndim==1 ) then
         ihelp = npelm
         km(iseqkm2)%kmesh(9) = nelem*(ihelp-1)
         leng = km(iseqkm2)%kmesh(9)*2

!        --- load kmesh part b

         km(iseqkm2)%kmeshb(1) = 2

      end if
      if ( ndim==2 ) then
         leng = 0
         km(iseqkm2)%kmesh(9) = 0
         do ielgrp = 1, nelgrp

!        --- fetch mesh type, # elements and # nodes per element

            ishape = kmeshf(ielgrp)
            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm
            if ( ishape==-10001 ) then

!           --- connection elements

               nprev = km(iseqkm2)%kmesh(9)
               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9) + inelem
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*2

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 2

            else if ( ishape==1 ) then

!           --- line elements

               nprev = km(iseqkm2)%kmesh(9)
               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9) + inelem*(inpelm-1)
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*2

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 2

            else if ( ishape==5 ) then

!           --- surface elements

               nprev = km(iseqkm2)%kmesh(9)
               rinpelm = inpelm
               ihelp = int(sqrt(rinpelm))
               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9) + inelem*(ihelp - 1)*(ihelp - 1)
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*4

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 4

            end if
         end do

      end if

      if ( ndim==3 ) then
         leng = 0
         km(iseqkm2)%kmesh(9) = 0
         do ielgrp = 1, nelgrp

!        --- fetch mesh type, # elements and # nodes per element

            ishape = kmeshf(ielgrp)
            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm
            if ( ishape==-10001 ) then

!           --- connection elements

               nprev = km(iseqkm2)%kmesh(9)
               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9) + inelem
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*2

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 2

            else if ( ishape==1 ) then

!           --- line elements

               nprev = km(iseqkm2)%kmesh(9)
               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9) + inelem*(inpelm-1)
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*2

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 2

            else if ( ishape==5 ) then

!           --- surface elements

               nprev = km(iseqkm2)%kmesh(9)
               rinpelm = inpelm
               ihelp = int(sqrt(rinpelm))
               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9) +&
                  inelem*(ihelp - 1)*(ihelp - 1)
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*4

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 4

            else if ( ishape==13 .or. ishape==14 ) then

!           --- volume elements

               nprev = km(iseqkm2)%kmesh(9)
               rinpelm = inpelm
               ihelp = int(rinpelm**(1d0/3d0)+0.5d0)

               if ( ihelp**3/=inpelm ) then
                  call errint ( inpelm, 1 )
                  call errint ( ihelp, 2 )
                  call errint ( ihelp**3, 3 )
                  call errsub ( 1696, 3, 0, 0 )
               end if

               km(iseqkm2)%kmesh(9) = km(iseqkm2)%kmesh(9)+&
                  inelem*(ihelp-1)*(ihelp-1)*(ihelp-1)
               leng = leng + (km(iseqkm2)%kmesh(9)-nprev)*8

!              --- load kmesh part b

               km(iseqkm2)%kmeshb(ielgrp) = 8

            end if

         end do

      end if

!     --- create permanent space for kmesh2 part c

      allocate ( km(iseqkm2)%kmeshc(leng) )
      km(iseqkm2)%kmesh(17) = leng

!     --- fill kmesh2 part c

      iseqkm = iseqkm2
      call fillpc
      iseqkm = iseqkm1
      if ( ierror/=0 ) go to 1000

!     --- write kmesh2 to femesh.out

      if ( parallel ) then

!     --- parallel is true, hence parallel computing
!         Create file femesh_par.xxx

         filename = 'femesh_par.'
         write(filename(12:14),100) inodenr
100      format(i3.3)
         lastps = 14

      else

!     --- parallel is false
!         Create file femesh.out

         filename = 'femesh.out'
         lastps = 11

      end if  ! ( parallel )

      open ( iref, file=filename(1:lastps), status = 'unknown' )
      kmesh => kmesh2
      call sepfillkmesh(kmesh)
      if ( debug ) call prinin ( km(iseqkm)%kmesh, 80, 'kmesh2' )
      call sepmeshwr ( iref )

      if ( ierror/=0 ) go to 1000
      rewind(iref)

!     --- reset kmesh2

      km(iseqkm2)%kmesh = 0

      nelgrp = km(iseqkm1)%kmesh(5)
      ndim = km(iseqkm1)%kmesh(6)
      if ( ndim==1 ) km(iseqkm2)%kmesh(4) = 2
      if ( ndim==2 ) km(iseqkm2)%kmesh(4) = 4
      if ( ndim==3 ) km(iseqkm2)%kmesh(4) = 8

!     --- read kmesh2 from femesh.out

      irenum = 0
      if( iren>0 ) irenum = 2
      call msh071 ( irenum, iref, idummy )
      if ( ierror/=0 ) go to 1000
      close(iref)

!     --- copy the original coordinates into kmesh2 part i

      coor = km(iseqkm1)%coor

!     --- fetch kmesh1 and kmesh2 part j

      if ( iren>0 ) then

!     --- first check if a renumbering on kmesh2 is performed

         if ( kelmj/=0 ) then

!           --- reserve space for kmesh1 part j

            if ( associated(km(iseqkm1)%kmeshj) ) &
               deallocate(km(iseqkm1)%kmeshj)
            allocate(km(iseqkm1)%kmeshj(km(iseqkm1)%kmesh(8)))
            if ( ierror/=0 ) go to 1000

!           --- copy the renumbering into kmesh1 part j

            km(iseqkm1)%kmeshj = kmeshj
            km(iseqkm1)%kmesh(24) = km(iseqkm2)%kmesh(24)

         end if

      end if

      if ( parallel ) then

!     --- parallel is true, hence parallel computing
!         Read file meshoutput_par.000 and some information from meshoutput

         call mshrdparmesh

      end if

1000  km(iseqkm2)%kmesh(38) = -km(iseqkm1)%kmesh(38)
      call sepfillkmesh(kmesh)

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'kmesh1, kmesh2', kmesh1, kmesh2
         write(irefwr,*) 'End sepfemesh'

      end if  ! ( debug )
      call erclmn('sepfemesh', timebefore)

      end
