      subroutine sepfillcofelgr ( ielgrp, jparm, iwork, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays iuser and user from arrays iwork and work
!     internal elements
!     In this case iwork(k,1)>2001 is permitted
!     Array iwork is supposed to be a 4 x jparm array
!     Input may be read from a file with reference number iref (iwork(k,2))
!     This input is stored in the user array user
!     When iref<0 then the input is not read from the input file but
!     stored in array user by using subroutine funcfl
!     Old subroutine fil103
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulenparm
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielgrp, jparm, iwork(4,jparm)
      double precision work(jparm)

!     ielgrp   i   standard element number
!     iwork    i   User array of length 4 x NPARM to store information of the
!                  coefficients, together with array work
!                  iwork(1,i) contains the parameter ifunc
!                  iwork(2,i) the parameter iref or the sequence number
!                             of the old solution vector
!                  iwork(3,i) the degree of freedom in the old sol. vector
!                  iwork(4,i) information of the function by which the old
!                             solution vector must be multiplied
!                  with
!                  ifunc        Indication of the type of coefficients.
!                               Possible values:
!                  0:           coef i is a constant, stored in work(i)
!                  >0<1000:     coef i is given by the subroutine FUNCCF
!                  >1000<2000:  coef i is given by the subroutine FUNCC1
!                  2001:        coef i is given by the user in each point
!                               The values are read from file iref
!                  2002:        coef i is given by the user in each element
!                               The values are read from file iref
!                  2003:        coef i is given by the k th degree of freedom
!                               in the j th solution vector
!                               The number j is stored in iwork(2,i)
!                               The number k is stored in iwork(3,i)
!                  2004:        coef i is given by the k th degree of freedom
!                               in the j th solution vector multiplied by a
!                               function or constant
!                               The number j is stored in iwork(2,i)
!                               The number k is stored in iwork(3,i)
!                               If muliplied by a constant then iwork(4,i) = 0
!                               and work(i) is the constant otherwise iwork(4,i)
!                               refers to the function subroutine FUNCCF
!                  >10000:      coef i is given by the subroutine FUNCC3
!                  <0:          Coefficient i is equal to coefficient j, &
!                               where j = -iwork(1,i) and j<i
!                  iref         When>0 it indicates the input reference number
!                               of the file
!                               when<0,  it indicates that the coefficients to
!                               be stored in array user are created by the user
!                               subroutine funcfl
!                               iref<0 should only be used in combination with
!                               iwork(1,k)  =  2001
!     jparm    i   number of parameters is iwork
!     work     i   Real work array of length NPARM in which values of
!                  coefficients are stored. See iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, icount, &
              jcount, mcount, ifunc, iref, ih, kcount, itype
      double precision func

!                    they are not
!     func           Help variable to store the value of work(i)
!     i              Loop variable
!     icount         Counting variable, gives last position used in IUSER
!     ifunc          See iwork (input)
!     ih             Help variable to store a preceding value of iwork(1,j) for
!                    iwork(1,i)<0
!     iref           See iwork (input)
!     itype          Type number of element group
!     jcount         Counting variable, gives last position used in USER
!     k              Loop variable
!     kcount         Counting variable, gives last position used in preceding
!                    part of IUSER
!     mcount         Help variable to store the starting value of icount for
!                    this call of sepfillcofelgr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELF000         Actual filling of IUSER and USER for the simple cases
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCFL         User written subroutine to fill coefficients
!     PRININ1        print 2d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     536   value of iwork is not permitted in subroutine
!     571   iref<0 is not allowed in combination with iwork(k) = 2002
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Standard checks

      call eropen ( 'sepfillcofelgr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillcofelgr'
         call prinin1 ( iwork, jparm, 4, 'iwork' )
  1      format ( a, 1x, (10i10) )

      end if
      if ( ierror/=0 ) go to 1000
      kprobc => kprobpart%kprobc

      inelem = sepgetinelem ( ielgrp )  ! number of elements per group
      if ( ielgrp==1 .or. iuser(2)<=0 ) then

!     --- ielgrp  =  1

        icount = 5+nelgrp+numnat+nglgrp
        jcount = 5

      else

!     --- ielgrp>1

        icount = iuser(4)
        jcount = nint(user(4))

      end if

!     --- Store value of icount into mcount

      mcount = icount

!     --- fill arrays iuser and user

      iuser(ielgrp+5) = icount+1

      do i = 1, jparm
         ifunc = iwork(1,i)
         if ( debug ) write(irefwr,1) 'ifunc', ifunc
         if ( ifunc>=0 .and. ifunc<=2000 .or. abs(ifunc)>10000 ) then

!        --- Standard situation
!            ifunc>10000: scalar
!            ifunc<-10000: func = scalar

            func = work(i)
            call elf000 ( icount, jcount, ifunc, func, i, iuser, user )
            if ( jcount==2001 .or. jcount==2002 ) jcount = 2010

         else if ( ifunc==2001 ) then

!        --- ifunc  =  2001   read npoint function values from file iref

            icount = icount+1
            iuser(icount) = 2001
            icount = icount+1
            iuser(icount) = jcount+1
            call prinlg( 'user', -jcount-npoint, user, iuser )
            iref = iwork(2,i)
            if ( iref>0  ) then

!           --- iref>0  Read from file iref

               read(iref, *) (user(jcount+k), k = 1, npoint)

            else if ( iref<0 ) then

!           --- iref<0  Call subroutine funcfl

               call funcfl ( coor, npoint, -iref, user(jcount+1) )

            end if
            jcount = jcount+npoint
            if ( jcount==2001 .or. jcount==2002 ) jcount = 2010

         else if ( ifunc==2002 ) then

!        --- ifunc  =  2002   read nelem function values from file iref

            icount = icount+1
            iuser(icount) = 2002
            icount = icount+1
            iuser(icount) = jcount+1
            call prinlg ( 'user', -jcount-inelem, user, iuser )
            iref = iwork(2,i)
            if ( iref<0 ) then
               call errint ( iref, 1 )
               call errint ( ielgrp, 2 )
               call errint ( i, 3 )
               call errint ( iwork(1,i), 4 )
               call errint ( iwork(2,i), 5 )
               call errint ( iwork(3,i), 6 )
               call errint ( iwork(4,i), 7 )
               call errsub ( 571, 7, 0, 0 )
            end if
            if ( iref>0 ) read(iref,*) (user(jcount+k), k = 1, inelem)
            jcount = jcount+inelem
            if ( jcount==2001 .or. jcount==2002) jcount = 2010

         else if ( ifunc==2003 ) then

!        --- ifunc = 2003   Old vectors are used

            icount = icount+1
            iuser(icount) = ifunc
            iuser(icount+1) = iwork(2,i)
            iuser(icount+2) = iwork(3,i)
            icount = icount+2

         else if ( ifunc==2004 ) then

!        --- ifunc = 2004   Old vectors are used mulitplied by a function or
!                           constant

            icount = icount+1
            iuser(icount) = ifunc
            iuser(icount+1) = iwork(2,i)
            iuser(icount+2) = iwork(3,i)
            icount = icount+2
            call elf000 ( icount, jcount, iwork(4,i), work(i), &
                          i, iuser, user )

         else if ( ifunc<0 .and. ifunc>-i ) then

!        --- ifunc<0 and ifunc>-i  Coefficient  i is identical to
!                                      coefficient -iwork(1,i)

            kcount = mcount
            do k = 1, -ifunc
               ih = iwork(1,k)
               if ( ih>=0 .and. ih<=2000 .or. abs(ih)>10000 ) then
                  kcount = kcount+1
               else if ( ih>=2001 .and. ih<=2002 ) then
                  kcount = kcount+2
               else
                  kcount = kcount+3
               end if
               if ( debug ) &
                  write(irefwr,1) 'k, ih, kcount', &
                                   k, ih, kcount
            end do
            if ( ih>=0 .and. ih<=2000 .or. abs(ih)>10000 ) then
               icount = icount+1
               iuser(icount) = iuser(kcount)
            else if ( ih<=2002 ) then
               icount = icount+2
               iuser(icount-1) = ih
               iuser(icount) = iuser(kcount)
            else if ( ih==2003 ) then
               icount = icount + 3
               iuser(icount-2) = ih
               iuser(icount-1) = iuser(kcount-1)
               iuser(icount  ) = iuser(kcount)
            else
               icount = icount + 4
               iuser(icount-2) = ih
               iuser(icount-1) = iuser(kcount-1)
               iuser(icount  ) = iuser(kcount)
            end if
            do k = 1, 4
               iwork(k,i) = iwork(k,-ifunc)
            end do

         else

!        --- ifunc has wrong value

            call errint ( ifunc, 1 )
            call errint ( ielgrp, 2 )
            call errint ( i, 3 )
            call errint ( iwork(1,i), 4 )
            call errint ( iwork(2,i), 5 )
            call errint ( iwork(3,i), 6 )
            call errint ( iwork(4,i), 7 )
            call errsub ( 536, 7, 0, 0 )

         end if

      end do

      itype = kprobc(ielgrp) ! type number corresponding to ielgrp
      if ( itype>99 ) then
         do i = jparm+1, nparmarr(itype)    ! fill rest of coefficients
            icount = icount+1
            iuser(icount) = 0
         end do  ! i = jparm+1, nparmarr(itype)
      end if  ! ( itype>99 )
      iuser(4) = icount
      user(4)  = jcount
      iuser(2) = ielgrp
      call prinlg ( 'iuser', icount+1, user, iuser )
      call prinlg ( 'user', -jcount-1, user, iuser )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillcofelgr'

      end if
      call erclos ( 'sepfillcofelgr' )

      end subroutine sepfillcofelgr
