      subroutine sepfillinpsol5
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays inpsol and rinsol with default values
!     These values are extended with variables found before
!     Special case: compact storage (hence iterative)
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq, iprec

!     iseq           Sequence number
!     iprec          Type of preconditioner
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepfilldefint, sepgetnamevar, sepfilldefstr, sepitermeth
      double precision :: sepfilldefreal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLDEFINT  Give sepfilldefint either the default value defval or the
!                    value of the variable nameint if it exists
!     SEPFILLDEFREAL Give sepfilldefreal either the default value defval or the
!                    value of the variable namereal if it exists
!     SEPFILLDEFSTR  Give sepfilldefstr either the default value defval or the
!                    value of the variable namestring if it exists
!     SEPFILLINPSOL9 Fill arrays inpsol and rinsol with default values
!                    extended with given values in the form of scalars or reals
!                    Special case: jmethod = 9 or 10 (compact row storage)
!     SEPGETNAMEVAR  Get sequence number of scalar, real or integer
!     SEPITERMETH    Translate icode to value of iteration_method to be stored
!                    in inpsol
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillinpsol5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillinpsol5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = sepgetnamevar ( 'POSITIVE_DEFINITE' )
      if ( iseq>0 ) inpsol(2,1) = 1    ! positive_definite found

      inpsol(3,1) = sepfilldefstr ( 'SOL_ITER_METHOD', 4, codeitmeth, &
                                     nncodeitmeth, nscodeitmeth, ncodeitmeth )
      inpsol(3,1) = sepitermeth ( inpsol(3,1)+1 )
      iprec = sepfilldefstr ( 'SOL_PREC', 6, codeprec, &
                              nncodeprec, nscodeprec, ncodeprec )
      inpsol(5,1) = sepfilldefint ( 'SOL_MAXITER', 0 ) ! maxiter
      inpsol(6,1) = sepfilldefint ( 'SOL_PRINT', 0 ) ! print_level
      inpsol(8,1) = sepfilldefint ( 'SOL_KRYLOV', 20 ) ! dimension of krylov sp
      inpsol(9,1) = sepfilldefstr ( 'SOL_START', -1, codeprec(istcodestart), &
                                     nncodestart, nscodeprec(istcodestart), &
                                     ncodeprec(istcodestart) )-1
      inpsol(9,1) = max(0,inpsol(9,1))
      inpsol(11,1) = sepfilldefstr ( 'SOL_TERMCRIT', 10, codeprec(istcodeterm), &
                                     nncodeterm, nscodeprec(istcodeterm), &
                                     ncodeprec(istcodeterm) )
      inpsol(13,1) = sepfilldefint ( 'SOL_NTRUNK', 20 )

      if ( inpsol(3,1)==4 ) then
         inpsol(14,1) = sepfilldefint ( 'SOL_NINNER', 30 )  ! gmresr
      else
         inpsol(14,1) = sepfilldefint ( 'SOL_NINNER', 1 )  ! rest
      end if  ! ( inpsol(3,1)==4 )
      rinsol(1,1) = sepfilldefreal ( 'SOL_EPS', 1d-3 ) ! accuracy
      rinsol(3,1) = 0.95d0  ! factor for milu
      rinsol(5,1) = 1d-2    ! accuracy for projection vectors
      if ( iprec>0 .and. iprec<=5 ) then
         iprec = iprec - 1
      else if ( iprec==7 ) then
         inpsol(10,1) = 2
      end if
      if ( iprec>=11 ) iprec = iprec-3
      if ( iprec==-1 ) then
         if ( inpsol(3,1)==9 ) then
            iprec = 0   ! No preconditioner in case of ML-multigrid
         else
            iprec = 6   ! Default preconditioner
         end if  ! ( inpsol(3,1)==9 )
      end if  ! ( iprec==-1 ) then

      if ( iprec==14 ) inpsol(3,1) = 30  ! PRECONLY in case of mumps
      inpsol(4,1) = iprec

      if ( inpsol(3,1)==5 ) call sepfillinpsol9

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsol, 40, 'inpsol' )
         call prinrl ( rinsol, 5, 'rinsol ')
         write(irefwr,*) 'End sepfillinpsol5'

      end if  ! ( debug )
      call erclos ( 'sepfillinpsol5' )

      end subroutine sepfillinpsol5
