      function sepitermeth ( icode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Translate icode to value of iteration_method to be stored
!    in inpsol
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     solve
!     iteration
!     iterative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepitermeth
      integer, intent(in) :: icode

!     icode          i    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     sepitermeth    o    Translation of icode
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icgs

!     icgs           Indicates the type of iterative solver that is
!                    used. This is equal to inpsol(3)-1, see read03
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1307   Not CG found after ITERATION METHOD =
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepitermeth' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepitermeth'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepitermeth = 0

      select case ( icode )

         case(2)

!        --- CGS found

            icgs = 2

         case(3)

!        --- GMRESR found

            icgs = 4

         case(4)

!        --- GMRES found

            icgs = 3

         case(5)

!        --- CG  found

            icgs = 1

         case(6)

!        --- Overrelaxation  found

            icgs = 5

         case(7)

!        --- BiCGSTAb  found

            icgs = 17

         case(8)

!        --- GCR found, set icgs to 31

            icgs = 31

         case(9:19)

!        --- SIMPLE_GCR, SYMMLQ, SIMPLER_GCR, ML_multigrid, AL_GCR,
!            HSIMPLER_GCR, MSIMPLER_GCR, SCHUR, IDR

            icgs = icode-3

         case(20)

!        --- BLOCK_TRIANGULAR

            icgs = 18

         case(21:30)

!        --- Options for petsc

            icgs = icode

         case(31)

!        --- default

            icgs = 0

         case default

!        --- Not one of the keywords CG, CGS, GMRES or OVERRELAXATION found

            call errsub ( 1307, 0, 0, 0 )
            call errcrd
            go to 1000

      end select  ! case ( icode )

      sepitermeth = icgs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepitermeth'

      end if  ! ( debug )
      call erclos ( 'sepitermeth' )

      end function sepitermeth
