      subroutine seppresscorr ( finished, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one or more time steps with the pressure correction method
!     to solve the Navier-Stokes equations
!     Old subroutine prpresscorr
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulecons
      use sepmodulemain
      use sepmodulebuild
      use sepmoduletime
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinputlc(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     iinputlc         i    integer input array
!                         Contains specific input for the pressure correction
!                         process
!                         contents:
!                         1:  sequence number of pressure correction input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: niinpresscor=12, nrinpresscor=1
      integer :: iinpresscor(niinpresscor), &
                 iseqnr, leninp, lenrnp, ivector, iseqintmat
      double precision :: rinpresscor(nrinpresscor)
      logical, save :: first

!     first          If true it concerns the first time step
!     iinpresscor    Integer input array for the pressure correction process
!                    See subroutine readpresscor
!     iseqintmat     Sequence number with respect to array intmat
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     ivector        Sequence number of first vector
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     niinpresscor   Length of array iinpresscor
!     nrinpresscor   Length of array rinpresscor
!     rinpresscor    Real input array for the pressure correction process
!                    See subroutine readpresscor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     INIVECTR       Return with sequence number of vector
!     SEPFILLTIMINP  Fill module sepmoduletime with information stored in
!                    inpsep/rinsep and ibuffr
!     SEPPRESSCORR1  solves Navier-Stokes by pressure correction
!     SEPTIMEFR02    Adapt time parameters at end of time step for timefrbf
!     SEPTIMSEQINTM  Fill array intmat if necessary and initialize iseqintmat
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read input for pressure correction and store in arrays
!     iinpresscor, rinpresscor
!     Put parameters in iintim
!     Actual call to time subroutine
!     Adapt time parameters
!     Remove temporary created array at end of buffer
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data first /.true./
! ======================================================================
!
      call eropen ( 'seppresscorr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppresscorr'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Read input for pressure correction and store in arrays
!         iinpresscor, rinpresscor

      iseqnr = iinputlc(1)
      leninp = niinpresscor
      lenrnp = nrinpresscor
      call iinputseq ( 29, iseqnr, iinpresscor, &
                       rinpresscor, leninp, lenrnp, 'presscor' )
      if ( ierror/=0 ) go to 1000
      numvec = max(numvec,iinpresscor(1),iinpresscor(2))

!     --- Put parameters in iintim

      iseqnr = iinpresscor(5)
      ivector = max(1,inivectr(iinpresscor(1)))
      call sepfilltiminp ( iseqnr, isubr, ivector )
      if ( ierror/=0 ) go to 1000
      numoldtim = max(numoldtim,numvec)

!     --- Fill iseqintmat and intmat if necessary

      iprobtim = ks(isolut2d(ivector))%iprob
      intmatar => intmat1d
      stiffmat =>  stiffm(iprobtim:maxprb)
      massmat => massmt(iprobtim)
      call septimseqintm ( iseqintmat )

!     --- Actual call to time subroutine

      isubr = 3
      call seppresscorr1 ( finished, first, isubr, iinputlc(1), &
                           iinpresscor, rinpresscor )
      if ( ierror/=0 ) go to 1000

!     --- Adapt time parameters

      call septimefr02

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppresscorr'

      end if
      call erclos ( 'seppresscorr' )

      end
