      subroutine sepps2 ( posint, mmcoor, meshonly )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 30-01-2016 New parameter list
!        version  4.14 date 25-11-2012 Long integers
!        version  4.13 date 13-05-2010 Enlarge funcx(1)
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine SEPPS2 performs the actual post processing of SEPRAN files
!    SEPPS2 is called by program SEPPOST
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleread
      use sepmodulemach
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mmcoor(*)
      logical posint, meshonly

!     meshonly       i    If true only the mesh is plotted
!     mmcoor         i    Memory management sequence number with respect to
!                         coor
!     posint         i    true if seppost is called interactively
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irep, ivec, imesh, lastps, iactmesh, indsol
      character (len=60) namefl
      character (len=2) posold
      character (len=2) phcpcp
      double precision multfact
      logical new_solution

!     iactmesh       Actual mesh sequence number
!     imesh          Actual mesh and problem number
!     indsol         Sequence number of vector in isol
!     irep           Loop variable
!     ivec           Sequence number of array to be processed
!     lastps         Last non-blank position in namefl
!     multfact       Multiplication factor
!     namefl         Name of default SEPRAN neutral plotting file
!     new_solution   If true the solution array may have been changed
!     phcpcp         Help variable to store the default value of devhcp
!     posold         name of POSDEV from environment file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INITDF         Set defaults for program SEPPOST
!     INITMX         Initialize parameters xmin, xmax, etc and view point
!     INSP03         Define COMPUTE commands
!     INSP04         Carry out commands for time history
!     PLINDF         Remove all present SEPRAN neutral plotting files
!     POSWIN         Read input interactively
!     PRESET         Reset some of the defaults
!     PRININ         print 1d integer array
!     READ05         Read input from the standard input file
!     SEP2DVEC       Aassociate 2d array usoltemp2d to 1d vector
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPPS2INTERP   Interpolate solution from one mesh to another
!     SEPPS2INTERSCT Intersect a 3d mesh by a plane and create 2d mesh
!     SEPPS2MESHPL   Plot mesh quantities
!     SEPPS2OUTPUT   Output per timestep
!     SEPPS2READM    Read mesh in case of seppost
!     SEPPS2SETM     Set mesh number in seppost
!     SEPPS2WRITM    Write mesh to file
!     TO0093         Body of subroutines FILCOF and CHNCOF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2201   File sepcomp.out not found but postprocessing of solution required
!    2839   intersect_mesh not the first action
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2')
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2'
  1      format ( a, 1x, (10i6) )

      end if

      allocate ( iuser(105), user(105) )

!     --- Initialize some arrays

      funcx(1:5) = 0d0
      funcy(1:5) = 0d0
      indcol(1:5) = 0

      ispec(1:maxvc1,1:2) = 0
      ispec(1:maxvc1,3) = 1
      ispec(1:maxvc1,4) = 0

      indmsh(1) = 1
      indmsh(2:nummesh) = 0

!     --- Set first position of variable length arrays, except kmesh and kprob

      funcx(1)  = dble(ntimvc+5)
      funcy(1)  = funcx(1)
      indcol(1) = 105

!     --- Initialize some parameters

      jpoint = 0
      psiphi = 0d0
      textid(1) = ' '
      textx = 'x'
      texty = 'y'
      identf = ' '

      iltime(1) = 1
      iltime(2) = 1
      iltime(3) = 1
      funcx(4) = 1d0
      funcy(4) = 1d0
      iactmesh = 1
      imesh = 1
      new_solution = .false.

      nullify(iskiparr)

!     --- Initialize contln (positions 1 to 5)
!         Since contln is used as work array in sepst1, it is not possible to
!         perform this initialization before

      contln(1) = 105d0
      contln(2:5) = 0d0
      jkader = -1
      jtimes = 10
      itimst = 1

      kmesh => kmesh2d(1)
      call sepfillkmesh ( kmesh )
      if ( kelme==0 ) call sepfillkmeshe

      if ( posint ) then
         call initmx ( ndim, npoint, coor )
         if ( ierror/=0 ) go to 1000
      end if

      call preset(0)

      if ( posint ) then

!     --- interactive post processing, delete old plot files
!         and constpstruct name of new one

         call initdf
         posold = posdev
         call plindf ( namefl, phcpcp, lastps )
         if ( ierror/=0 ) go to 1000

      end if
      if ( mmcoor(1)>0 ) then

!     --- mmcoor(1)>0, hence the coordinates are read separately
!         Copy coordinates to kmesh

         usoltemp => ks(mmcoor(1))%sol
         call dcopy ( npoint*ndim, usoltemp, 1, coor, 1 )

      end if

!     --- Loop over the input records
!         Read one data record at a time
!         Perform the required actions immediately

      do irep = 1, 10000

!     --- Read a data record

         kmesh => kmesh2d(iactmesh)
         call sepfillkmesh ( kmesh )

         if ( debug ) &
            write(irefwr,1) 'irep, iactmesh, ndim, jtimes', &
                             irep, iactmesh, ndim, jtimes
         if ( posint ) then
            istop = 3
            call poswin ( posold, iuser, user )
         else
            call read05
         end if
         if ( ierror/=0 ) go to 1000

!        --- Manipulate the parameter jtimes in order to get the correct legenda

         if ( debug ) write(irefwr,1) 'jtimes/1', jtimes
         jtimst = jtimes
         if ( iinputpst(1)>0 ) jtimes = max(11,jtimes)
         if ( debug ) write(irefwr,1) 'jtimes/2, jtimst', jtimes, jtimst

!        --- Make a choice depending on iinputpst(1)
!            iinputpst(1) = 0, means finish the program

         if ( iinputpst(1)==0 ) goto 1000
         if ( meshonly .and. iinputpst(1)/=127 .and. &
              (iinputpst(1)<100 .or. iinputpst(1)>=110) ) then

!        --- meshonly found, so only meshes may be plotted

            call errchr ( name74, 1 )
            call errsub ( 2201, 0, 0, 1 )
            go to 1000

         end if
         if ( debug ) call prinin ( iinputpst, 10, 'iinputpst' )

         if ( iinputpst(1)==4 ) then

!        --- iinputpst(1) = 4, Write mesh

            call sepps2writm ( iactmesh )

         else if ( iinputpst(1)==5 ) then

!        --- iinputpst(1) = 5, Intersect mesh

            if ( new_solution ) then

!           --- Possibly a new solution has been created, not allowed

               call errsub ( 2839, 0, 0, 0 )
               go to 1000

            end if  ! ( new_solution )

            call sepps2intersct

         else if ( iinputpst(1)==20 ) then

!        --- iinputpst(1) = 20, Read mesh

            call sepps2readm ( iuser )

         else if ( iinputpst(1)==21 ) then

!        --- iinputpst(1) = 21, Set mesh

            call sepps2setm ( iinputpst, indmsh, iactmesh )

         else if ( iinputpst(1)==30 ) then

!        --- iinputpst(1) = 30, interpolate

            call sepps2interp
            new_solution = .true.

         else if ( iinputpst(1)==40 ) then

!        --- iinputpst(1) = 40, change coordinates

            imesh = iactmesh
            ivec = iinputpst(2)
            indsol = imapsol(ivec,imesh)
            multfact = rinputpst(3)
            isolut => isolut2d(indsol)
            usoltemp => ks(isolut)%sol
            length = size(usoltemp)
            call sep2dvec ( usoltemp, ndim, length/ndim )
            velocity2d => usoltemp2d
            coor = coor + multfact*velocity2d
            jtimes = jtimst

         else if ( iinputpst(1)==81 ) then

!        --- iinputpst(1) = 81, Fill coefficients for different element
!                            groups. Use routine TO0093

            user(1)   = 105d0
            iuser(1)  = 105

            kmesh => kmesh2d(1)
            kprob => kprob2d(1)
            call to0093 ( 1, 1, 1 )
            jtimes = jtimst

         else if ( iinputpst(1)==127 ) then

!        --- iinputpst(1) = 127, Define Plot identification

            ntext = iinputpst(39)
            if ( ntext==0 ) then
               ntext = 1
               textid(1)(1:1) = ' '
            end if
            iinputpst(45) = ntext
            iinputpst(44) = 1
            identf(1:ntext) = textid(1)(1:ntext)
            xorgid = rinputpst(7)
            yorgid = rinputpst(8)
            jtimes = jtimst

         else if ( iinputpst(1)>=100 .and. iinputpst(1)<110 ) then

!        --- 100<=iinputpst(1)<110, Make a plot of mesh quantities

            call sepps2meshpl ( iactmesh )
            new_solution = .true.

         else if ( iinputpst(1)==-3 .or. iinputpst(1)<200 ) then

!        --- iinputpst(1)<200, output per TIME STEP

            call sepps2output ( imesh, iactmesh, iuser, user, mmcoor )
            new_solution = .true.

         else if ( iinputpst(1)<300 ) then

!        --- iinputpst(1)<300, Definition of compute statements

            call insp03
            jtimes = jtimst

         else

!        --- Time history

            call insp04
            jtimes = jtimst

         end if
         if ( ierror/=0 ) go to 1000

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2'

      end if
      call erclos ( 'sepps2' )

      end subroutine sepps2
