      subroutine sepps2readm ( iuser )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 26-06-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read mesh in case of seppost
!     Command read mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleinput
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecons
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iuser(*)

!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mmiinputpst(1), ichois, iref, mesh_in, iseqprob, iseqkm2

!     ichois         Choice parameter
!     iref           File reference number
!     iseqkm2        Sequence number with respect to km of second mesh
!     iseqprob       Sequence number of problem input
!     mesh_in        Sequence number of input mesh
!     mmiinputpst       Memory management sequence number with respect to
!                    iinputpst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIFIL         Open files
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     PROBDFEXT      Reads and stores problem definition
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2226   Mesh structure differs from mesh with sequence number 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2readm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2readm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm2 = kmesh2d(1)

      mesh_in = iinputpst(88)
      name10 = textid(1)
      call inifil ( -10 )
      kmesh => kmesh2d(mesh_in)
      call msh071 ( -1, iref10, iuser )
      iref = abs(iref10)
      close ( unit = iref )
      call sepfillkmesh(kmesh)

!     --- Check if number of user points, curves, surfaces, volumes
!         and element groups of both meshes are the same

      if ( km(iseqkm2)%kmesh(5)/=nelgrp ) then

!     --- meshes are not the same

         call errint ( mesh_in, 1 )
         call errsub ( 2226, 1, 0, 0 )
         go to 1000

      end if

!     --- create array kprob2d corresponding to this mesh

      ichois = 4
      iseqprob = 1
      kprob => kprob2d(mesh_in)
      call probdfext ( ichois, iseqprob )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2readm'

      end if  ! ( debug )
      call erclos ( 'sepps2readm' )

      end
