      subroutine sepps2sub ( mmcoor, itimst, iactmesh, imesh, &
                             iinput, lastmapsol, imapsol, jtimst, &
                             iuser, user, ispec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 09-03-2013 Correction ivec=0
!        version  1.4  date 25-11-2012 Long integers
!        version  1.3  date 22-11-2006 Extra option for scaling
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the actual postprocessing for one time step
!
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulevecs
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mmcoor(*), itimst, &
              iactmesh, imesh, iinput(*), &
              lastmapsol, imapsol(maxvc1,ntimvc), jtimst, &
              iuser(*), ispec(maxvc1,4)
      double precision user(*)

!     iactmesh       i    Actual mesh sequence number
!     iinput         i    integer input array as stored in common cpost1
!     imapsol        i    Array of size maxvc1 x ntimvc defining a mapping from
!                         a two-dimensional storage to a one-dimensional storage
!                         with respect to array isol
!                         imapsol(i,j) refers to the i-th vector at time level j
!                         It contains an integer reference number k to isol(.,k)
!                         if this array is used and otherwise a 0
!     imesh          i    mesh sequence number
!     ispec          i    Array of length maxvc1 x 3 which indicates for each
!                         array if it corresponds to a function to be computed
!                         or not, from which array it has been computed, and to
!                         which problem resp. mesh it belongs
!                         See common cpost1
!     itimst         i    Integer time level
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jtimst         i    Integer time level
!     lastmapsol     i    Last entry in isol that has been used
!     mmcoor         i    Memory management sequence number with respect to
!                         coor
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, iseqkm1
      logical skip

!     iseqkm1        Sequence number with respect to km of first mesh
!     ivec           Sequence number of vector of special structure
!     skip           If true insp02 must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSP02         Carry out the command for plotting or printing
!     SEPPS2SUB1     Compute solution corresponding to compute statement
!     SEPPS2SUB2     Multiply vector by constant and add constant to vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2sub'
         write(irefwr,1) 'iinput(1)', iinput(1)
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinput(1)==120 .and. iinput(80)>0 ) then

!     --- iinput(1)=120 function plot
!         iinput(106) = 1, all in one picture

         if ( iinput(81)>0) dcolor = iinput(22)*iinput(81)
         if ( iinput(82)>0) &
            iinput(21) = iinput(22)*iinput(82)

      end if
      if ( iinput(1)>0 ) jtimes = max ( jtimes,11 )

      if ( mmcoor(itimst)>0 ) then

!     --- mmcoor(itimst)>0, hence the coordinates are read separately
!         Copy coordinates to kmesh

         iseqkm1 = kmesh2d(iactmesh)
         usoltemp => ks(mmcoor(itimst))%sol
         call dcopy ( km(iseqkm1)%kmesh(8)*km(iseqkm1)%kmesh(6), &
                      usoltemp, 1, km(iseqkm1)%coor, 1 )

      end if
      ivec = iinput(2)
      if ( debug ) &
         write(irefwr,1) 'itimst, ivec, iactmesh, imesh', &
                          itimst, ivec, iactmesh, imesh

      skip = .false.  ! initialize skip
      if ( iinput(1)>0 .and. iinput(1)<125 .or. iinput(1)>150 ) then

!     --- iinput(1)<125 or>150 so the special function
!         does not have to be computed

         call sepps2sub1 ( itimst, lastmapsol, imapsol, &
                           jtimst, ispec, ivec, skip )

      else if ( iinput(1)==-3 ) then

!     --- iinput(1) = -3, multiply solution by constant and constant

         call sepps2sub2 ( ivec )
         go to 1000

      end if  ! ( iinput(1)>0 .and. iinput(1)<125 .or. iinput(1)>150 )
      if ( ierror/=0 .or. skip ) go to 1000  ! jump in case skip true

      if ( ivec==0 ) then

!     --- No vector present, for example print text

          imesh = iactmesh

      else

!     --- Standard case

         imesh = ispec(ivec,3)
         if ( imesh==1 ) imesh = iactmesh

      end if  ! ( ivec==0 )

      if ( debug ) &
         write(irefwr,1) 'imesh, iactmesh', imesh, iactmesh

!     --- Perform the actual output operations

      kmesh => kmesh2d(imesh)
      kprob => kprob2d(imesh)
      call insp02 ( user, iuser )
      if ( debug ) &
         write(irefwr,1) 'imesh, iactmesh', imesh, iactmesh

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2sub'

      end if
      call erclos ( 'sepps2sub' )

      end subroutine sepps2sub
