      subroutine sepps2writm ( iactmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 30-01-2016 Replace meshwrbf by meshwr
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write mesh to file
!     in case of postprocessing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     post_processing
!     write
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulemain
      use sepmodulemach
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iactmesh

!     iactmesh       i    Actual mesh sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref10sav, imesh, ivec
      character (len=100) name10sav
      logical skip

!     imesh          Mesh sequence number
!     iref10sav      Help variable to store value of iref10
!     ivec           Sequence number of vector of special structure
!     name10sav      Help variable to store value of name10
!     skip           If true insp02 must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPMESHWR      fill the file meshoutput with the mesh
!     SEPPS2SUB1     Compute solution corresponding to compute statement
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( file name given ) then
!        give output file this name
!     else
!        use default file name
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2writm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2writm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store old file name

      name10sav = name10
      iref10sav = iref10
      if ( iinputpst(100)==1 ) then

!     --- File name is given, use this name

         name10 = textid(1)

      else

!     --- File name is not given, use default name

         name10 = 'mesh_post'

      end if  ! ( iinputpst(100)==1 )
      if ( debug ) then
         write(irefwr,*) 'name10 = ', name10
         write(irefwr,1) 'iinputpst(2)', iinputpst(2)
      end if  ! ( debug )

      if ( iinputpst(2)>0 ) then

!     --- Vi is given, compute interpolating mesh

         skip = .false.  ! initialize skip
         ivec = iinputpst(2)
         if ( debug ) write(irefwr,1) 'ivec', ivec
         call sepps2sub1 ( itimst, lastmapsol, imapsol, &
                           jtimst, ispec, ivec, skip )
         if ( ierror/=0 .or. skip ) go to 1000

         imesh = ispec(ivec,3)
         if ( imesh==1 ) imesh = iactmesh
         if ( debug ) &
            write(irefwr,1) 'imesh, iactmesh', imesh, iactmesh

      else

!     --- Vi is not given, use actual mesh

         imesh = iactmesh

      end if  ! ( iinputpst(2)>0 )

!     --- Write mesh, open and close file

      iref10 = inisetnewref()
      if ( iref10>0 ) then

!     --- iref10>0, formatted file

         open ( iref10, file = name10 )

      else

!     --- iref10<0, formatted file

         open ( iref10, file = name10, form = 'unformatted' )
         iref10 = -iref10

      end if  ! ( iref10>0 )

      if ( imesh/=iactmesh ) then
         kmesh => kmesh2d(imesh)
         call sepfillkmesh(kmesh)
      end if  ! ( imesh/=iactmesh )

      call sepmeshwr ( iref10 )

      close ( abs(iref10) )
      call inirmref ( abs(iref10) )

!     --- Reset old file name

      name10 = name10sav
      iref10 = iref10sav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2writm'

      end if  ! ( debug )
      call erclos ( 'sepps2writm' )

      end
