      subroutine sepreadvec ( isol, number, filenm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read vector field from a file in the sequence of the nodal points of
!     the mesh
!     Old subroutine readvec
! **********************************************************************
!
!                       KEYWORDS
!
!     field
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, number
      character (len=*) filenm

!     filenm     i    Name of file
!     isol       o    Standard SEPRAN array containing information of the
!                     solution
!     number     i    Indicates the number of degrees of freedom stored
!                     per point, &
!                     if<0 it indicates - number of degrees of freedom
!                     per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios, ivec, i, itype, iref

!     i              Counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     iref           File reference number
!     itype          Type of vector
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     INISTR1        find structure of vector
!     MSHEXINT       Interpolate solution to extra points
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     833   nphys too large
!    1317   error in opening of file
!    1992   ITYPE has wrong value
!    1994   End of file found while reading
!    1995   Error while reading file
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadvec' )
      if ( ierror/=0 ) go to 1000
      if ( number==0 .or. abs(number)>200 ) then
         call errint ( number, 1 )
         call errint ( 1, 2 )
         call errint ( 3, 3 )
         call errsub ( 1992, 3, 0, 0 )
      end if

!     --- Find type of vector corresponding to number

      if ( number>0 ) then

!     --- Vector is defined per node

         call inistr1 ( number, itype, ivec )
         if ( ierror/=0 ) go to 1000

!        --- Create vector

         call sepcreasolbf1 ( isol, itype, ivec, 'vector', 0 )
         length = number*(npoint-nextpnt)

      else

!     --- Vector is defined per element

         call sepcreasolbf1 ( isol, 116, -number, 'vector', 0 )
         length = ks(isol)%nusol

      end if

!     --- Open file filenm

      iref = inisetnewref()
      open ( unit=iref, file=filenm, iostat=ios )
      if ( ios/=0 ) then

!     --- Error in opening of file

         call errint ( ios, 1 )
         call errchr ( filenm, 1 )
         call errsub ( 1317, 1, 0, 1 )
         go to 1000

      end if

!     --- Read solution

      usol => ks(isol)%sol
      read(iref,*,iostat=ios) usol
      if ( ios<0 ) then

!     --- End of file found

         call errchr ( filenm, 1 )
         call errsub ( 1994, 0, 0, 1 )

      else if ( ios>0 ) then

!     --- Error in reading of file

         call errint ( ios, 1 )
         call errchr ( filenm, 1 )
         call errsub ( 1995, 1, 0, 1 )

      end if
      if ( nextpnt>0 .and. kelme>0 ) then

!     --- Extra points have been introduced, &
!         it is necessary to interpolate to these points

         if ( number>100 ) then
            call errint ( number, 1 )
            call errint ( 100, 2 )
            call errsub ( 833, 2, 0, 0 )
            go to 1000
         end if
         call mshexint ( usol, number )

      end if

      close(iref)
      call inirmref ( iref )

1000  call erclos ( 'sepreadvec' )
      end
