      subroutine seprelkmesh ( kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make the relation between kmesh and the pointers in sepmodulekmeshp
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmesh

!     kmesh        i    The value of kmesh is the reference number of the
!                         array in buf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPRELKMESH1   Make the relation between kmref and the pointers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprelkmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprelkmesh'
         call ertrace
         write(irefwr,*) 'parallel ', parallel
         write(irefwr,1) 'kmesh, icountmesh', kmesh, icountmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm = kmesh
      if ( debug ) write(irefwr,1) 'iseqkm', iseqkm

      call seprelkmesh1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndim', ndim

         write(irefwr,*) 'End seprelkmesh'

      end if  ! ( debug )
      call erclos ( 'seprelkmesh' )

      end subroutine seprelkmesh
