      subroutine to0089consts ( iinsol, isubr, iread, iseqcf, iseqsl, &
                                iprob, ivectr, maxvct, ivecreac, &
                                ivecfem, iprecfem, iinbld, ictime )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 13-03-2009 Correction iinbld(13)
!        version  2.1  date 13-01-2009 Rxtra debug
!        version  2.0  date 26-03-2007 Extra parametere ictime
!        version  1.1  date 21-02-2007 Extension with keep matrix
!        version  1.0  date 19-08-2005
!
!   copyright (c) 2005-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract some constants from array iinsol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     right_hand_side
!     solve
!     constant
!     extract
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*), isubr, iread, iseqcf, iseqsl, ivecreac, &
              iprob, ivectr, maxvct, ivecfem, iprecfem, iinbld(*), &
              ictime

!     ictime         i    Indication if information must be read from standard
!                         input file ( 0 ) or that information read before is
!                         used ( 1 ).  See to0089
!     iinbld         o    Input array for subroutine BUILDBF
!     iinsol         i    Input array defined by the user, see to0089
!     iprecfem       o    Indicates if a FEM preconditioning must be applied
!                         in case of a spectral mesh
!                         Possible values:
!                         0: no FEM preconditioning
!                         1: FEM preconditioning
!     iprob          o    Actual problem number
!     iread          o    Defines how the input must be read
!                         Possible values:
!                         -1:  No input is required
!                          0:  All SEPRAN input has been read by
!                              subroutine SEPSTN until
!                              END_OF_SEPRAN_INPUT or
!                              end of file has been found
!                              This input is used
!                          1:  The input is read as described for
!                              EIGENVAL
!     iseqcf         o    Sequence number corresponding to the coefficients
!     iseqsl         o    Sequence number corresponding to the linear solver
!     isubr          o    Indicates the type of calling subroutine
!                         Possible values
!                         1:    LINSOL
!                         2:    LINSTM
!                         3:    LINPRB
!     ivecfem        o    Sequence number of FEM solution vector
!     ivecreac       o    Sequence number of the reaction force vector with
!                         respect to array isol (if >0)
!     ivectr         o    Sequence number of the solution vector with respect to
!                         array isol
!     maxvct         o    maximum number of vectors that may be stored in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089consts' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089consts'
         call prinin ( iinsol, 18, 'iinsol' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract constants

      isubr    = iinsol(1)
      iread    = iinsol(2)
      iseqcf   = iinsol(3)
      iseqsl   = iinsol(4)
      iprob    = iinsol(5)
      ivectr   = iinsol(6)
      maxvct   = iinsol(7)
      ivecreac = iinsol(10)
      ivecfem  = iinsol(11)
      iprecfem = iinsol(12)

!     --- Fill iinbld

      iinbld(1) = 15

      if ( iinsol(15)<=1 .and. iinsol(16)<=1 .and. iinsol(18)<1 ) then

!     --- Build matrix and right-hand side

         if ( ictime<4 ) then
            iinbld(2) = 1
         else
            iinbld(2) = 2
         end if  ! ( ictime<4 ) then

      else if ( iinsol(16)<=1 .and. iinsol(18)<1 ) then

!     --- Build right-hand side

         iinbld(2) = 2

      else if (  iinsol(15)<=1 ) then

!     --- Build matrix

         iinbld(2) = 14

      else

!     --- Build nothing

         iinbld(2) = 4

      end if  ! ( iinsol(15)<=1 .and. iinsol(16)<=1 .and. iinsol(18)<1 )

      iinbld(3) = 0
      iinbld(4) = 0
      iinbld(5) = 0
      iinbld(6) = 0
      iinbld(7) = 0
      iinbld(8) = 0
      if ( ivecreac>0 ) iinbld(8) = 1
      iinbld(9) = iprob
      iinbld(11) = 0
      iinbld(10) = maxvct
      iinbld(12) = 1
      iinbld(13) = max(1,min(ivectr,maxvct))
      iinbld(14) = iinsol(8)
      iinbld(15) = iinsol(9)
      if ( maxvct==1 ) iinbld(5) = 1

1000  call erclos ( 'to0089consts' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinbld, iinbld(1), 'iinbld' )
         write(irefwr,*) 'End to0089consts'

      end if

      end
