      subroutine to0089semtst ( kmesh, iseqkmesh, iprecfem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check arrays kmesh and intmat in case of spectral element method
!     with finite element preconditioning
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulecons
      use sepmoduleintmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kmesh(2), iseqkmesh, iprecfem

!     iprecfem      i/o   Indicates if a FEM preconditioning must be applied
!                         in case of a spectral mesh
!                         Possible values:
!                         0: no FEM preconditioning
!                         1: FEM preconditioning
!     iseqkmesh     i/o   Sequence number of mesh in kmesh
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqkm1,  iseqkm2

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2108   FEM preconditioning required, but there is no spectral mesh
!    2110   FEM preconditioning required, but there is no FEM mesh
!    2113   FEM preconditioning required, but matrix structure is not for a FEM
!           grid
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089semtst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089semtst'

      end if
      if ( ierror/=0 ) go to 1000
      iseqkm1 = kmesh(1)
      iseqkm2 = kmesh(2)

      if ( km(iseqkm1)%kmesh(38)==0 ) then

!     --- warning no spectral mesh

         call errwar ( 2108, 0, 0, 0 )
         iprecfem = 0

      else

!     --- Check if next mesh in kmesh is fem mesh
         if ( km(iseqkm2)%kmesh(38)/=-km(iseqkm1)%kmesh(38) .or. &
              km(iseqkm2)%kmesh(8)/=km(iseqkm1)%kmesh(8) .or. &
              km(iseqkm2)%kmesh(9)<=km(iseqkm1)%kmesh(9) ) then

!        --- warning no fem mesh available

            call errwar ( 2110, 0, 0, 0 )
            iprecfem = 0
            iseqkmesh = 1

         else
            iseqkmesh = 2
         end if

      end if
      if ( intmt(intmat)%spectral ) then

!     --- intmat(2) is reset temporarily

         intmt(intmat)%spectral = .false.

      else

!     --- The structure of the matrix does not correspond to a fem grid

         call errwar ( 2113, 0, 0, 0 )
         iprecfem = 0
         iseqkmesh = 1

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089semtst'

      end if
      call erclos ( 'to0089semtst' )

      end
