      subroutine to0093 ( isubr, jprob, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 20-08-2016 use sepmoduleinput
!        version  4.0  date 12-11-2014 Split subroutine in several parts
!        version  3.11 date 30-10-2014 New call to read02
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the coefficients from the standard input file
!    or array IBUFFR. Fill coefficients in the arrays IUSER and USER
!    TO0093 is an under subroutine of FILCOF, CHNCOF, FILLCF and CHANCF
!    It performs the actual task of these subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      use sepmoduleread
      use sepmodulecoeffs
      use sepmoduleinput
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr, iseqnr, jprob

!     iseqnr         i    Sequence number with respect to the storage
!                         in array ISEPIN
!     isubr          i    Defines from which subroutine TO0093 is called
!                         Possible values:
!                          1:  FILCOF
!                          2:  CHNCOF
!                          3:  FILLCF
!                          4:  CHANCF
!     jprob          i    Actual problem number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcal, ipos, ngroup
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     ipos           Position of input arrays in sepmoduleinput
!     maxcal         Maximum number of coefficient inputs read
!     ngroup         Total number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READ02         Read input concerning the coefficients
!     SEPFILLIUSER   Fill information of coefficients as stored in arrays
!                    iincofarr and rincof into iuser/user
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPKMKPCOFS    Fill some quantities for coefficients corresponding to
!                    kmesh and kprob
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The actual reading is performed by subroutine READ02
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     628   First record read is not COEFFICIENTS
!    1290   Sequence number has not been defined
!    1913   Problem number for coefficients differs from the one for the solver
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check KMESH, KPROB and IPROB
!   Set NELGRP and NBNGRP
!   If ( isubr<3 ) then
!      read first record
!      read input from the standard input file and put in IINCOF, RINCOF
!   else
!      extract input from IBUFFR and put in IINCOF, RINCOF
!   Extract maxprm from IINCOF
!   if ( isubr=1 or 3 ) then
!      Clear IWORK, WORK and NPARM
!   else if ( isubr=2 or 4 ) then
!      Create arrays IWORK, WORK and NPARM from IUSER, USER
!   Expand IINCOF, RINCOF into IWORK, WORK and NPARM
!   Fill arrays IUSER and USER
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0093' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0093'
         call ertrace
         write(irefwr,1) 'isubr, iprob', isubr, jprob
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill quantities corresponding to kmesh and kprob

      iprob = jprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

      call sepkmkpcofs

!     --- Read input and store in arrays iincof and rincof

      ngroup = nelgrp+numnat+nglgrp
      if ( debug ) &
         write(irefwr,1) 'nelgrp, numnat, nglgrp', &
                          nelgrp, numnat, nglgrp

      nrcof = ngroup * 100           ! size of rincof
      nicof = 4 * nrcof              ! size of iincof

!     --- allocate temporary arrays

      allocate ( iincof(nicof), rincof(nrcof), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nicof+nrcof, 'iincof' )
      if ( ierror/=0 ) go to 1000

      if ( isubr<=2 ) then

!     --- isubr<=2, read input

         iincof = 0
         rincof = 0d0

!        --- Read first record

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then

!        --- First record does not start with a keyword

            call errsub ( 628, 0, 0, 0 )
            call errcrd
            go to 1000

         end if

         if ( string(1:6)=='CHANGE' ) call asknxt
         if ( string(1:4)=='COMP' ) then

!        --- Complex found, i.e. complex coefficients

            compl = .true.

!           --- Read next word

            call asknxt
            if ( ityprd/=4 ) then

!           --- First record does not start with a keyword

               call errsub ( 628, 0, 0, 0 )
               call errcrd
               go to 1000

            end if

         else

            compl = .false.

         end if
         if ( string(1:4)/='COEF' ) then
            call errsub ( 628, 0, 0, 0 )
            call errcrd
            go to 1000
         end if

         call read02 ( compl, isubr )

      else

!     --- isubr = 3 or 4, extract from IBUFFR

         call sepgetinpfinf ( isubr+3, iseqnr, ipos, maxcal )

         if ( maxcal==0 ) then

!        --- Coefficients array has not been created

            call errint ( iseqnr, 1 )
            call errchr ( 'coefficients', 1 )
            call errsub ( 1290, 1, 0, 1 )
            go to 1000

         end if

         length = size(in(ipos)%iinput)
         iincof(1:length) = in(ipos)%iinput
         if ( associated(re(ipos)%rinput) ) then
            length = size( re(ipos)%rinput )
            rincof(1:length) = re(ipos)%rinput
         end if  ! ( associated(re(ipos)%rinput)>0 )
         if ( iincof(2)==0 ) then
            compl = .false.
         else
            compl = .true.
         end if

         if ( iincof(3)/=iprob .and. maxcal>1 ) then

!        --- IPROB in linear solver differs from IPROB fill coefficients

            call errint ( iincof(3), 1 )
            call errint ( iprob, 2 )
            call errsub ( 1913, 2, 0, 0 )
            go to 1000

         end if

      end if

!     --- Fill arrays iuser/user from iincof and rincof

      call sepfilliuser ( isubr, compl )

!     --- deallocate temporary arrays

      deallocate ( iincof, rincof, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iincof' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iuser, iuser(3), 'iuser' )
         call prinrl ( user, nint(user(3)), 'user' )
         write(irefwr,*) 'End to0093'

      end if
      call erclos ( 'to0093' )

      end
