      subroutine tofreesb ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.52 date 30-01-2016 New call to instfree
!        version  3.51 date 24-12-2014 New definition clinsl
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Body of subroutine SEPFREE with exception of SEPRAN start and SEPRAN
!   reading of input
!   Warning!!!!!
!   Subroutines TO0094 and TOFREESB are nearly identical
!   If one of them is changed, the other one should be changed also
! **********************************************************************
!
!                       KEYWORDS
!
!     compute_all
!     driving_subroutine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulecomp
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision :: rinsep(*)

!     inpsep  i    The integer input read is stored in array INPSEP
!                  See subroutine sepcomsub for a description
!     rinsep  i    Real input array (Depends on inpsep)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, iseqfr
      logical :: freefinished

!     freefinished   If true the free boundary loop has been finished
!     iseqfr         Sequence number for input of subroutine STATFREE
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINC         Help subroutine to call CHNCOF, which changes coefficients
!                    in IUSER and USER
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     INSTFREE       Instationary free surface subroutine
!     SEPCOMSUB      Actual body of TO0094 and TOFREESB
!     SEPMAKEREL     Make relation between local and global arrays
!     STATFREE       Stationary free surface subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofreesb' )

      debug = .false.

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from tofreesb'

      end if

      istartmn = 2
      isubrmn  = 0
      iseqlp = 0
      ioutpsav = ioutp

!     --- Clear arrays

      do k = 1, maxmesh
         meshseqs(k) = k
      end do  ! k = 1, maxmesh

      numvecmn = 0
      nitercom = 0
      niterlp = 0
      jtimes = 10
      jkader = -1
      freefinished = .false.
      timefinished = .false.
      icalltimstr = 0
      meshnr = meshseqs(1)
      iinmap(1) = 2
      iinmap(2) = 1

      iseqif = 0
      checkif = .false.

100   if ( isubrmn>=0 ) then

!     --- isubrmn>=0, take next subroutine

         isubrmn  = inpsep(istartmn)
         if ( isubrmn==64 .or. ierror/=0 ) go to 1000

         call sepmakerel  ! make relation between global and local arrays

         if ( debug ) then

!        --- debug information

            write(irefwr,*) 'isubrmn = ', isubrmn
            write(irefwr,*) 'meshnr = ', meshnr

         end if

         if ( isubrmn==-1 ) go to 100

!        --- Call main subroutine for sepcomp

         if ( isubrmn==23 ) then

!        --- isubrmn = 23, Start stationary free boundary loop

            iseqfr = inpsep(istartmn+1)
            nitercom = 1
            istartmn = istartmn + 3
            ioutp = -1

            go to 100

         else if ( isubrmn==24 ) then

!        --- isubrmn = 24, call subroutine STATFREE and check on convergence

            call statfree ( nitercom, freefinished, iseqfr )
            iinmap(2) = 1
            if ( freefinished ) then
               istartmn = istartmn+2
               nitercom = 0
            else
               istartmn = inpsep(istartmn+1)
            end if
            go to 100

         else if ( isubrmn==25 ) then

!        --- isubrmn = 25, call subroutines ERLINC and ERLINS

         if ( inpsep(istartmn+3)>0 .and. &
              inpsep(istartmn+3)/=nitercom ) then

!        --- iteration number found that is different from nitercom
!            skip

            istartmn = istartmn + 5
            go to 100

         end if
         kmesh => kmesh2d(1)
         call erlins ( 3, inpsep(istartmn+4), inpsep(istartmn+1) )
         call erlinc ( 3, inpsep(istartmn+4), inpsep(istartmn+2) )
         istartmn = istartmn + 5
         go to 100

         else if ( isubrmn==26 ) then

!        --- isubrmn = 26, start instationary free boundary loop

            iseqfr = inpsep(istartmn+1)
            if ( .not. timefinished ) then

!           --- Not yet ready, go to next step

               istartmn = istartmn + 3
               icalltimstr = icalltimstr+1
               icalltim = 0

            else

!           --- End of loop

               istartmn = inpsep(istartmn+2)
               itimloop = 0
               icalltimstr = 0
               maxcalltim = 0
               icalltim = 0
               ioutp = ioutpsav

            end if
            go to 100

         else if ( isubrmn==27 ) then

!        --- isubrmn = 27, End instationary free_boundary loop

            istartmn = inpsep(istartmn+1)-3
            call instfree ( iseqfr )
            icalltimstr = 0
            maxcalltim = 0
            icalltim = 0
            iinmap(2) = 1
            ioutp = ioutpsav
            go to 100

         else if ( isubrmn==64 ) then

!        --- isubrmn = 64, stop

            go to 1000

         end if

         call sepcomsub ( inpsep, rinsep )
         if ( ierror/=0 ) go to 1000
         go to 100

      end if
1000  call erclos ( 'tofreesb' )
      end
