      subroutine msh003 ( iinput, rinput, curstr, &
                          iwork, irelcurves, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version 29.1  date 12-03-2015 Adaptation debug
!        version 29.0  date 11-12-2014 Extra parameters
!        version 28.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array curvesmsh
!     When this subroutine is changed also subroutine msh048
!     must be changed !!!!!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iwork(*), irelcurves(*), istartrel
      double precision :: rinput(*), curstr(*)

!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     iinput         i    standard integer input array for subroutine mesh
!                         it is supposed that the first five positions have been
!                         deleted
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     istartrel     i/o   next free position in irelcurves
!     iwork          o    Work array of length ncurvs containing a 0 if the
!                         curve is single and 1 if it is composite
!     rinput         i    standard real input array for subroutine mesh
!                         it is supposed that the first five positions have been
!                         deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposc, iinp, irnp, icurve, i, jinp, inpncv, jcurnr

!     i              Counting variable
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     inpncv         number of points along curve
!     iposc          last position used in array curves
!     irnp           position in array rinput
!     jcurnr         Curve sequence number
!     jinp           copy of iinp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSHCURCR  Create curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh003' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh003'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      irnp  = irnp1+(ndim+jcoars)*nuspnt+2*jcoars-1

!     --- Clear second part of icurvsmsh

      icurvsmsh(ncurvs+1:2*ncurvs) = 0

      do jcurnr = 1, ncurvs

         icurnr = iseqcurvs(jcurnr)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         if ( debug ) &
            write(irefwr,1) 'jcurnr, icurnr, iinp, icurve', &
                             jcurnr, icurnr, iinp, icurve

         if ( icurnr==1 ) then
            iposc = 1
         else
            iposc = icurvsmsh(icurnr-1)+1
         end if

         if ( icurve==0 ) then

!        --- Empty curve found: skip

            inpncv = 0

         else

!        --- Standard case

            jinp = iinp
            call mshcurcr ( iinput, rinput, curstr, &
                            icurve, jinp, irnp, &
                            iposc, icurnr, inpncv, jcoars, &
                            ipointcurv, irelcurves, irnp1, istartrel )

         end if
         if ( ierror/=0 ) go to 1000

      end do

!     --- Fill array iwork with a 0 if the curve is standard and a 1 if it is
!         a composite curve

      do i = 1, ncurvs
         icurve = iinput(ipointcurv(i))
         if ( icurve==0 .or. icurve==7 ) then
            iwork(i) = 1
            if ( icurvsmsh(i+ncurvs)==0 ) icurvsmsh(i+ncurvs) = 1
         else
            iwork(i) = 0
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh003'

      end if
      call erclos ( 'msh003' )

      end
