      subroutine msh005 ( ndim, curves, iinput, rinput, tau, n, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 11-12-2014 Complete new source
!        version  3.4  date 21-11-1993 Correction error message
!        version  3.3  date 17-12-1992 Correction error message
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of array curves for a straight line
!     n positions are filled
!     the spacing of the nodal points is given by array tau
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndim, n, icurnr
      double precision curves(ndim,n), rinput(ndim,*), tau(n)

!    curves   o  array to be filled from position 1
!    icurnr   i  curve number
!    iinput   i  the first two positions of this array contain the point
!                numbers of the initial and end points
!    n        i  number of points on the line
!    ndim     i  dimension of space
!    rinput   i  array containing the coordinates of the points
!    tau      i  array containing the spacing of the nodal points
!                output of subroutine msh004
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2
      double precision dx(ndim), x1(ndim), x2(ndim), eps, dlen(ndim)

!     dlen           |x1| + |x2|
!     dx             distance vector
!     eps            Local accuracy
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     x1             coordinates of first point in curve
!     x2             coordinates of last point in curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     470     initial and end point of line have the same co-ordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh005' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh005'
         write(irefwr,1) 'ndim, n', ndim, n
         call prinrl ( tau, n, 'tau' )

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get user point numbers

      i1 = iinput(1)
      i2 = iinput(2)

!     --- Store coordinates in x1 and x2 and the difference in d

      x1 = rinput(1:ndim,i1)
      x2 = rinput(1:ndim,i2)
      dx = x2-x1

      dlen = abs(x2)+abs(x1)    ! |x1| + |x2|

      eps = epsmac*dot_product(dlen,dlen)  ! local accuracy

      if ( dot_product(dx,dx)<eps ) then
         call errint ( icurnr, 1 )
         call erreal ( sqrt(dx), 1 )
         call errsub ( 470, 1, 1, 0 )
      end if  ! ( dot_product(dx,dx)<eps )

!     --- Fill curves

      do i = 1, ndim
         curves(i,1:n) = x1(i)+tau(1:n)*dx(i)
      end do  ! i = 1, ndim

1000  call erclos ( 'msh005' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh005'

      end if  ! ( debug )

      end subroutine msh005
