      subroutine msh018 ( iinput, isurnr, ifirst, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 07-11-2002 Layout
!        version  2.2  date 22-02-1999 Include common
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 30-01-1987
!
!   copyright (c) 1982-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute position of surface in array iinput
!     relative with respect to iinp3
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifirst, iinp, iinput(*), isurnr

!     ifirst     i    indication whether a new surface number is given
!                     (ifirst=0) or that the next one is needed (ifirst>0)
!     iinp       i    starting position in array iinput relative to iinp3
!                     when ifirst = 0  iinp is computed
!                     when ifirst > 0  iinp must have a value at input
!                     in that case it is raised for one surface
!                     when isurnr = 1  iinp=0
!     iinput     i    integer input array  must be stored from position iinp3
!     isurnr     i    surface number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, is

!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     is
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh018' )
      if ( ifirst.eq.0 ) then

!     --- ifirst = 0,  new surface

         iinp = 0
         ih = isurnr-1

      else

!     --- ifirst > 0  preceding surface has been treated

         ih = 1

      end if

!     --- ifirst  =  0  new surface

      iinp = iinp+1
      do i = 1, ih
         is = iinput(iinp)
         call msh049 ( iinput, iinp, is )
      end do
      iinp = iinp-1

      call erclos ( 'msh018' )

      end
