      subroutine msh027 ( iinput, inpold, ipoint, nvolms, leng1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.7  date 21-11-2014 Correction n
!        version  3.6  date 16-07-2008 Extension with internal surfaces
!        version  3.5  date 17-02-2006 Extension with old_general
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array iinput with information of volumes by rearranging the
!     information in inpold
!     !!!!!!!! If this subroutine is changed also change msh050 !!!!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     volume
!     input
!     mesh
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), inpold(*), ipoint(*), nvolms, leng1

!     iinput     o    new array iinput
!     inpold     i    old array iinput
!     ipoint     i    pointer array, see mshrd2
!     leng1      i    length of first part of array iinput corresponding to
!                     the curves
!     nvolms     i    number of volumes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, ivol, j, jinp, n

!     i              Counting variable
!     iinp           last position used in array iinput
!     ivol           Type of volume generator ot be used
!     j              Counting variable
!     jinp           last position used in array iinput with respect to iprec
!     n              Number of extra position in iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     298   code for generation of volumes incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh027' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh027'

      end if
      if ( ierror/=0 ) go to 1000

      jinp = leng1
      do i = 1, nvolms
         iinp = ipoint(i)
         if ( iinp<=0 ) then

!        --- volume i has not been defined

            iinput(jinp+1) = 0
            jinp = jinp + 1

         else

!        --- volume i has been defined

            ivol = inpold(iinp)

            if ( ivol<0 .or. ivol>11 ) then
               call errint ( ivol, 1 )
               call errsub ( 298, 1, 0, 0 )
               go to 1000
            end if

            n = 3+inpold(iinp+2)
            if ( ivol==1 ) then

!           --- Volume of type brick, 3 extra positions

               n = n+3

            else if ( ivol>=6 .and. ivol<=8 ) then

!           --- Volume of type copy, 1 extra position

               n = n+1

            else if ( ivol==10 ) then

!           --- Volume of type genral with internal points, extra positions
!               points, curves and surfaces

               n = n+inpold(iinp+n)+inpold(iinp+n+1)+inpold(iinp+n+2)+3

            end if  ! ( ivol==1 )
            iinp = iinp-1
            do j = 1, n
               iinput(jinp+j) = inpold(iinp+j)
            end do
            jinp = jinp+n

         end if

      end do

      iinput(5) = nvolms

1000  call erclos ( 'msh027' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinput, jinp, 'iinput' )
         write(irefwr,*) 'End msh027'

      end if

      end
