      subroutine msh030arcleng ( ndim, c2, radius, phi, sleng, jcentre, &
                                 x1, x2, xz, dx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of arc (part of circle)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jcentre, ndim
      double precision, intent(in) :: c2, radius, x1(ndim), x2(ndim), &
                                      xz(ndim), dx(ndim)
      double precision, intent(out) :: phi, sleng

!     c2             i    Helping constant (dx,dx)
!     dx             i    length of vector (x2-x1)
!     jcentre        i    user point number of centroid
!     ndim           i    Dimension of the space.
!     phi            o    angle from begin point to end point
!     radius         i    Radius of sphere or circle
!     sleng          o    length of the curve
!     x1             i    coordinates of first point
!     x2             i    coordinates of last point
!     xz             i    coordinates of centroid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: cs, arg, convex

!     arg            argument
!     convex         the value of this parameter defines whether the circle is
!                    convex or not
!     cs             help parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030arcleng' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030arcleng'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 ) then

!     --- ndim = 2

         arg = min ( 1d0, sqrt(c2)/(2d0*radius) )
         arg = max ( -1d0, arg )
         phi = 2d0*asin(arg)
         if ( jcentre>0) then
            convex = x1(1)*(xz(2)-x2(2))-xz(1)*dx(2)+ &
                     x2(1)*(x1(2)-xz(2))
         else
            convex = x2(1)*(xz(2)-x1(2))+xz(1)*dx(2)+ &
                     x1(1)*(x2(2)-xz(2))
         end if
         if ( convex>0d0 ) phi = 2d0*pi-phi
         sleng = radius*phi
         if ( jcentre<0 ) phi = -phi
         if ( debug ) &
            write(irefwr,2) 'arg, phi, convex, sleng', &
                             arg, phi, convex, sleng

      else

!     --- ndim = 3

         cs = 1d0-0.5d0*c2/radius**2
         phi = acos(cs)
         sleng = radius*phi

      end if  ! ( ndim==2 )

1000  call erclos ( 'msh030arcleng' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030arcleng'

      end if  ! ( debug )

      end subroutine msh030arcleng
