      subroutine msh045
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.1  date 02-11-2012 Change in debug
!        version  6.0  date 02-09-2010 Remove intarmsh
!        version  5.3  date 29-06-2006 Initialize spherefound
!
!   copyright (c) 1987-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array kmeshm for curves of curves and
!    kmeshn for surfaces of surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, i, nsubcv, jstart, ic, ih, j, ikelmc, &
              istmsr, nsubsf, jstmsr, isubsf, nelloc, npoloc, index, &
              ielem, k, inpncv, isurnrnew, ns, lengc, &
              icurnrcircle, jsurnr, icurvs(1)

      logical spherefound
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ic             Relative subcurve number
!     icurnrcircle   Curve number of circle
!     icurvs         Array with curve numbers for subroutine er0027
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     index
!     inpncv         Number of points in curve
!     istart         First position used at a part
!     istmsr
!     isubsf         Sequence number of subsurface
!     isurnrnew      New surface sequence number
!     iwork          integer work array of length npomxs+nelmax
!     j              Counting variable
!     jstart         starting position of curve i in kmeshm
!     jstmsr
!     jsurnr         Surface sequence number provided with sign
!     k              Counting variable
!     lengc          computed length of kmesh part c
!     nelloc         Local number of elements
!     npoloc         Number of nodes in subsurface
!     ns             Starting address of information of specific surface
!                    in array KMESHS
!     nsubcv         Number of subcurves of a curve
!     nsubsf         Number of subsurfaces
!     spherefound    If true a sphere must be created
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inidln

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIDLN         Compute length of array in BUFFER
!     PR0009         Print information of surfaces
!     PRININ         print 1d integer array
!     READFL         Read integer array from a direct access SEPRAN file
!     SEPER0027      Fill nodal points of curves as indicated by ichois and
!                    icurvs in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh045' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh045'
100      format ( a, 1x, (10i6) )

      end if

!     --- Fill kmeshm for curves of curves

      if ( nvolms==0 ) npomax = npomxs
      spherefound = .false.

      allocate ( iwork(nelmax+npomax), stat = error )
      if ( error/=0 ) call eralloc ( error, nelmax+npomax, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( kelmq/=0 ) then

!     --- kelmq # 0, hence kmesh part q has been filled, which means that
!                     there are curves of curves

         istart = kmeshm(2) - 1

!        --- Loop over curves

         do i = 1, ncurvs

            nsubcv = kmeshq(i+1) - kmeshq(i)
            jstart = istart + kmeshm(4+i) - 1
            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'i, istart, jstart, ncurvs, nsubcv ', &
                                i, istart, jstart, ncurvs, nsubcv

           end if

!          --- Loop over subcurves

            do ic = 1, nsubcv
               icurnr = kmeshq( kmeshq(i)+ic )
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'ic, icurnr ',ic, icurnr

               end if

               if ( icurnr>0 ) then

!              --- icurnr>0, copy curve forwards

                  ih = istart + kmeshm(4+icurnr) - 1
                  inpncv = kmeshm(5+icurnr) - kmeshm(4+icurnr)
                  kmeshm(jstart+1:jstart+inpncv) = &
                     kmeshm(ih+1:ih+inpncv)

               else

!              --- icurnr<0, copy curve backwards

                  ih = istart + kmeshm(5-icurnr)
                  inpncv = kmeshm(5-icurnr) - kmeshm(4-icurnr)
                  do j = 1, inpncv
                     kmeshm(jstart+j) = kmeshm(ih-j)
                  end do
               end if
               if ( kmeshm(jstart+1)<0 ) then

!              --- kmeshm(jstart+1) refers to user point
!                  check if user point contains node number and if so
!                  fill in kmeshm

                  ih = kmeshl(-kmeshm(jstart+1))
                  if ( ih>0 ) kmeshm(jstart+1) = ih

               end if
               if ( kmeshm(jstart+inpncv)<0 ) then

!              --- kmeshm(jstart+inpncv) refers to user point
!                  check if user point contains node number and if so
!                  fill in kmeshm

                  ih = kmeshl(-kmeshm(jstart+inpncv))
                  if ( ih>0 ) kmeshm(jstart+inpncv) = ih

               end if
               jstart = jstart + inpncv

            end do

         end do

      end if

      if ( nvolms>0 .and. kelmr/=0 ) then

!     --- kelmr # 0, hence kmesh part r has been filled, which means that
!                     there are surfaces of surfaces

         ikelmc = 2 * nsurfs

         do i = 1, nsurfs
            istmsr = kmeshr(i)
            nsubsf = kmeshr(i+1) - istmsr
            inpelm = kmeshn(i)
            nelemlc = kmeshn(i+nsurfs)
            isurf = kmeshs(nsurfs + 1 + (i-1)*8+1)
            ishape = kmeshs(nsurfs + 1 + (i-1)*8+2)
            if ( isurf==20 ) spherefound = .true.
            if ( debug ) then
                write(irefwr,100) 'i, istmsr, nsubsf, inpelm, nelemlc', &
                           i, istmsr, nsubsf, inpelm, nelemlc
                write(irefwr,100) 'isurf, ishape', isurf, ishape
            end if  ! ( debug )

            if ( nsubsf>0 ) then

!           --- nsubsf>0: surfaces of surfaces

               jstmsr = istmsr - nsurfs-1 + kmeshr(nsurfs+1)

!              --- Loop over subsurfaces

               do isubsf = 1, nsubsf

!              --- Read local mapping array into iwork

                  jsurnr = kmeshr(istmsr+isubsf)
                  isurnr = abs(jsurnr)
                  nelloc = kmeshn(isurnr+nsurfs)
                  npoloc = kmeshh(1,isurnr+1)
                  index = kmeshr(jstmsr+isubsf)
                  call readfl( index, 1, npoloc+nelloc, 1, 3, iwork )
                  if ( debug ) then
                     write(irefwr,100) 'isubsf, jsurnr, isurnr', &
                                        isubsf, jsurnr, isurnr
                     write(irefwr,100) 'nelloc, npoloc, index', &
                                        nelloc, npoloc, index
                     call prinin ( iwork, npoloc, 'nodes' )
                     call prinin ( iwork(npoloc+1), nelloc, 'elements')
                  end if  ! ( debug )
                  if ( isurf==11 ) then

!                 --- ordered surface of surfaces

                     nmes = kmeshs(nsurfs + 1 + (isurnr-1)*8+5)
                     mmes = kmeshs(nsurfs + 1 + (isurnr-1)*8+6)
                     if ( debug ) write(irefwr,100) 'n, m', nmes, mmes

                  else if ( isurf==12 ) then

!                 --- ordered surface of pipe surfaces

                     nmes = kmeshs(nsurfs + 1 + (isurnr-1)*8+7)
                     mmes = kmeshs(nsurfs + 1 + (isurnr-1)*8+8)
                     if ( debug ) write(irefwr,100) 'n, m', nmes, mmes

                  end if

!                 --- Copy contents from kmeshn corresponding to isurnr
!                     to surface i

                  istart = 2*nsurfs
                  do j = 1, isurnr-1
                     istart = istart + kmeshn(j)*kmeshn(j+nsurfs)
                  end do

!                 --- jsurnr>0, or not an ordered surface of surfaces

                  do j = 1, nelloc

                     ielem = iwork(npoloc+j)
                     jstart = ikelmc + (ielem-1)*inpelm
                     if ( jsurnr>0 ) then

!                    --- jsurnr>0, standard case, positive direction

                        do k = 1, inpelm
                           kmeshn(jstart+k) = kmeshn(istart+k)
                        end do

                     else

!                    --- jsurnr<0, special case, reversed direction

                        if ( ishape<=5 ) then

!                       --- ishape = 3, 4, 5 all points must be reversed

                           kmeshn(jstart+1) = kmeshn(istart+1)
                           do k = 2, inpelm
                              kmeshn(jstart+inpelm+2-k) = &
                                 kmeshn(istart+k)
                           end do

                        else

!                       --- ishape>5 last point must be kept

                           kmeshn(jstart+1) = kmeshn(istart+1)
                           do k = 2, inpelm-1
                              kmeshn(jstart+inpelm+1-k) = &
                                 kmeshn(istart+k)
                           end do
                           kmeshn(jstart+inpelm) = &
                              kmeshn(istart+inpelm)

                        end if

                     end if
                     istart = istart+inpelm

                  end do

               end do

            end if
            ikelmc = ikelmc + nelemlc*inpelm
         end do
      end if
      if ( spherefound ) then

!     --- a sphere has been found, perform the copying

         istart = 2*nsurfs
         do isurnr = 1,nsurfs
            ns = nsurfs + 2 + (isurnr-1)*8
            isurf = kmeshs(ns)
            lengc = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
            if ( isurf==20 ) then

!           --- Sphere, copy new surface info to old surface

               isurnrnew = kmeshs(ns+2)
               icurnr = kmeshs(ns+3)
               jstart = istart
               do i = isurnr, isurnrnew-1
                  jstart = jstart+kmeshn(i)*kmeshn(i+nsurfs)
               end do
               do i = 1, lengc
                  kmeshn(istart+i) = kmeshn(jstart+i)
               end do

!              --- Copy new circle to old circle

               jstart = kmeshs(isurnr)+1
               icurnrcircle = kmeshs(jstart)
               jstart = kmeshm(2) - 1
               jstart = jstart+kmeshm(4+icurnrcircle)-1
               inpncv = kmeshm(5+icurnrcircle) - kmeshm(4+icurnrcircle)
               icurvs(1) = icurnr
               call seper0027 ( 0, icurvs, inpncv, kmeshm(1+jstart) )

            end if
            istart = istart+lengc

         end do

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

!         call pr0008 ( kmeshm, ncurvs )

         if ( nvolms>0 ) &
            call pr0009 ( 0, kmeshn, iwork, iwork, iwork, iwork, &
                          nsurfs )

      end if

      call erclos ( 'msh045' )
      end
