      subroutine msh055surf12 ( kmeshsp2, isurnr, kmeshm, iprec, &
                                kmeshs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshsp2(8,*), isurnr, kmeshm(*), iprec, kmeshs(*)

!     iprec          i    Surface number of surface to be copied etc.
!     isurnr         i    Surface sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2      i/o   Second part of kmeshs, referring to surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, numsurfs, kcur, istart, j, i1, i2, ifir, lastpnt

!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     ifir           Help variable to store first point
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kcur           curve sequence number
!     lastpnt        last point of a curve
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     MSHFIRLPNTCRV  Find first and last point of a curve using the information
!                    stored in kmeshm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     640   Boundary of ordered surface of pipesurfaces not complete
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055surf12' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh055surf12'

      end if
      if ( ierror.ne.0 ) go to 1000

      istart = kmeshs(isurnr)
      lastpnt = 0
      ifir = 0

      do k = 1, 4
         numsurfs = kmeshsp2(k+2,iprec)
         do j = 1, numsurfs
            kcur = kmeshs(istart+j)

!           --- Compute initial and end point of curve

            call mshfirlpntcrv ( kcur, kmeshm, i1, i2 )

            if ( j.eq.1 .or. lastpnt.eq.ifir ) then
               lastpnt = i2
               ifir    = i1
            else if ( i2.eq.ifir ) then
               lastpnt = i2
               ifir    = i2
            else
               if ( i1.ne.lastpnt ) then
                  call errint ( abs(i1), 1 )
                  call errint ( abs(lastpnt), 2 )
                  call errint ( k, 3 )
                  call errint ( isurnr, 4 )
                  call errwar ( 640, 4, 0, 0 )
               end if
               lastpnt = i2
            end if
         end do
         istart = istart + numsurfs

      end do

1000  call erclos ( 'msh055surf12' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh055surf12'

      end if

      end
