      subroutine msh06311 ( kmeshr, kmeshs, isurnr, nsurfs, iinput, &
                            iinp, lstart, isurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-10-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all subsurfaces are of the same type
!     (rectangle or pipe surface)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshr(*), isurnr, nsurfs, kmeshs(*), iinput(*), iinp, &
              lstart, isurf

!     iinp           i    last position used in array iinput
!     iinput        i/o   integer input array containing information of the
!                         surfaces
!     isurf         i/o   Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     lstart         i    Position in kmeshs
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kstart, nsubsf, ksurnr, k, jsurf, irefsf

!     irefsf         ?
!     jsurf          Type of surface with respect to mesh generator
!     k              Counting variable
!     kstart         Start value of parameter
!     ksurnr         absolute value of surface sequence number
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     637   Subsurfaces of ordered surface are not all of the same type.
!     638   Subsurfaces of ordered surface are not of the type RECTANGLE or
!           PIPESURFACE.
!    2097   Surface number out of range
!    2098   Surface corresponds to empty surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh06311' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh06311'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- isurf = 11   ordened surface of surfaces
!         Check if all subsurfaces are of the same type
!         (rectangle or pipe surface)

      kstart = kmeshr(isurnr)
      nsubsf = kmeshr(isurnr+1) - kstart
      do k = 1, nsubsf
         ksurnr = abs( kmeshr(kstart+k) )
         if ( debug ) write(irefwr,*) 'k, ksurnr ', k, ksurnr
         if ( ksurnr<=0 .or. ksurnr>nsurfs ) then

!        --- surface number out of range

            call errint ( ksurnr, 1 )
            call errint ( nsurfs, 2 )
            call errint ( isurnr, 3 )
            call errsub ( 2097, 3, 0, 0 )

         else if ( kmeshs(nsurfs+2+(ksurnr-1)*8)==0 ) then

!        --- surface empty

            call errint ( ksurnr, 1 )
            call errint ( isurnr, 2 )
            call errsub ( 2098, 2, 0, 0 )

         end if
110      jsurf = kmeshs( nsurfs+2+(ksurnr-1)*8)
         if ( jsurf==8 .or. jsurf==9 .or. jsurf==14 .or. &
              jsurf==16 ) then

!        --- ksurnr corresponds to translate or rotate
!            Check preceding surface

            ksurnr = kmeshs(nsurfs+4+(ksurnr-1)*8)
            goto 110
         end if
         if ( jsurf==11 ) jsurf = 1
         if ( jsurf==12 ) jsurf = 10
         if ( k==1) then
            irefsf = jsurf
         else
            if ( jsurf/=irefsf ) then
               call errint ( jsurf, 1 )
               call errint ( irefsf, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 637, 3, 0, 0)
            end if
         end if
      end do
      if ( irefsf==10 ) then
         isurf = 12
         iinput(iinp) = isurf
         kmeshs(lstart+1) = isurf
      else if ( irefsf/=1 .and. irefsf/=13 .and. &
                irefsf/=15 ) then
         call errint ( isurnr, 1 )
         call errint ( irefsf, 2 )
         call errint ( ksurnr, 3 )
         call errsub ( 638, 3, 0, 0)
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh06311'

      end if  ! ( debug )
      call erclos ( 'msh06311' )

      end
