      subroutine msh06312 ( iinput, iinp, ncurvs, nsurfs, &
                            kmeshs, iworkcopy, iwork1, &
                            kmeshscopy, lstart, maxncursur, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-10-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmeshs part curves for surfaces of type 12
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, ncurvs, nsurfs, &
              kmeshs(*), kmeshscopy(*), lstart, maxncursur, iwork1(*), &
              iworkcopy(*), istart

!     iinp           i    last position used in array iinput
!     iinput         i    integer input array containing information of the
!                         surfaces
!     istart         i    Position in kmeshscopy
!     iwork1         i    integer work array of length nsurfs containing
!                         the inverse of iseqsurfs
!     iworkcopy      i    Work array of length nsurfs+1 containing a copy
!                         of the first part of kmeshs in the sequence of
!                         iseqsurfs
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshscopy     i    Work array of the same length as kmeshs part 2
!                         containing a copy of kmeshs part 2 in the sequence
!                         of iseqsurfs
!     lstart         i    Position in kmeshs
!     maxncursur     i    Estimate of the length of kmeshscopy
!     ncurvs         i    Number of curves in mesh
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(ncurvs), k, kstart, i, icurve, jcurve, irow, &
              j, kinp, nrow, ntel, jsurnr, ksurnr, n, n0, n1, &
              n2, nh, nkinp, ns, nsubsf

!     i              Counting variable
!     icurve         Curve number
!     irow           lopp variable for rows
!     iwork          integer work array of length ncurvs
!                    iwork(i) = 0 :  Curve i is not used
!                    iwork(i) = 1 :  Curve i has been used once
!                    iwork(i)>1 :  Curve i has been used more than once
!     j              Counting variable
!     jcurve         Curve number
!     jsurnr         Surface sequence number
!     k              Counting variable
!     kinp           starting address in iinput
!     kstart         Start value of parameter
!     ksurnr         absolute value of surface sequence number
!     n              length of part of array
!     n0
!     n1
!     n2
!     nh             number of nodes along a subcurve
!     nkinp          ?
!     nrow           Number of rows of surfaces in ordered surface of
!                    surfaces
!     ns             Starting address of information of specific surface
!                    in array KMESHS
!     nsubsf         Number of subsurfaces
!     ntel           ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh06312' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh06312'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- isurf = 12  Ordened surface of pipe surfaces
!                     Each pipe surface contains three or four curves
!         First construct under boundary

      iwork = 0

      kinp = iinp + 2
      nrow = iinput(kinp)
      nsubsf = iinput(kinp+1)
      ntel = 0
      do k = 1, nsubsf
         jsurnr = iinput(kinp+nrow+k)
         ksurnr = abs ( jsurnr )
         n = kmeshs(nsurfs+4+(ksurnr-1)*8)
         kstart = iworkcopy(iwork1(ksurnr))

!        --- Copy curves

         if ( istart+n>maxncursur ) then

!        --- Estimation of length of kmeshscopy too small

            call errint ( istart+n, 1 )
            call errint ( maxncursur, 2 )
            call errchr ( 'kmeshscopy', 1 )
            call errsub ( 550, 2 , 0, 1 )
            go to 1000

         end if
         if ( jsurnr>0 ) then

!        --- jsurnr>0,  Copy forwards

            do j = 1, n
               kmeshscopy(istart+j) = kmeshscopy(kstart+j)
            end do

         else

!        --- jsurnr<0,  Copy backwards

            do j = 1, n
               kmeshscopy(istart+j) = - kmeshscopy(kstart+n-j+1)
            end do

         end if
         do j = 1, n
            iwork(abs(kmeshscopy(istart+j))) = -1
         end do
         istart = istart + n
         ntel = ntel + n
      end do
      kmeshs(lstart+3) = ntel

!     --- Next construct upper boundary

      nsubsf = iinput(kinp+nrow)
      nkinp = kinp + nrow
      ntel = 0
      do k = 1, nrow-1
         nkinp = nkinp + iinput(kinp+k)
      end do
      do k = 1, nsubsf
         jsurnr = iinput(nkinp+k)
         ksurnr = abs ( jsurnr )
         ns = nsurfs+4+(ksurnr-1)*8
         kstart = iworkcopy(iwork1(ksurnr)) + kmeshs(ns)
         n = kmeshs(ns+1)
         if ( istart+n>maxncursur ) then

!        --- Estimation of length of kmeshscopy too small

            call errint ( istart+n, 1 )
            call errint ( maxncursur, 2 )
            call errchr ( 'kmeshscopy', 1 )
            call errsub ( 550, 2 , 0, 1 )
            go to 1000

         end if

!        --- Copy curves

         if ( jsurnr>0 ) then

!        --- jsurnr>0,  Copy forwards

            do j = 1, n
               kmeshscopy(istart+j) = kmeshscopy(kstart+j)
            end do

         else

!        --- jsurnr<0,  Copy backwards

            do j = 1, n
               kmeshscopy(istart+j) = - kmeshscopy(kstart+n-j+1)
            end do

         end if
         do j = 1, n
            iwork(abs(kmeshscopy(istart+j))) = -1
         end do
         istart = istart + n
         ntel = ntel + n
      end do
      kmeshs(lstart+4) = ntel

!     --- Finally fill pipe defining curves
!         First generating curve

      nkinp = kinp + nrow
      ntel = 0
      do irow = 1, nrow
         nsubsf = iinput(kinp+irow)
         jsurnr = iinput(nkinp+1)
         ksurnr = abs ( jsurnr )
         ns = nsurfs+4+(ksurnr-1)*8
         kstart = iworkcopy(iwork1(ksurnr)) + kmeshs(ns) + &
                  kmeshs(ns+1)
         n1 = kmeshs(ns+2)
         n2 = kmeshs(ns+3)
         if ( jsurnr>0 .or. n2==0 ) then
            n = n1
         else
            kstart = kstart + n1
            n = n2
         end if
         if ( istart+n>maxncursur ) then

!        --- Estimation of length of kmeshscopy too small

            call errint ( istart+n, 1 )
            call errint ( maxncursur, 2 )
            call errchr ( 'kmeshscopy', 1 )
            call errsub ( 550, 2 , 0, 1 )
            go to 1000

         end if
         do j = 1, n
            kmeshscopy(istart+j) = kmeshscopy(kstart+j)
         end do
         do j = 1, n
            iwork(abs(kmeshscopy(istart+j))) = -1
         end do
         ntel = ntel + n
         istart = istart + n
         nkinp = nkinp + nsubsf
      end do
      kmeshs(lstart+5) = ntel

!     --- Last generating curve and internal curves

      nkinp = kinp + nrow
      ntel = 0
      do irow = 1, nrow
         nsubsf = iinput(kinp+irow)
         do k = 1, nsubsf
            jsurnr = iinput(nkinp+k)
            ksurnr = abs ( jsurnr )
            ns = nsurfs+4+(ksurnr-1)*8
            kstart = iworkcopy(iwork1(ksurnr)) + kmeshs(ns)
            n0 = kmeshs(ns+1)
            n1 = kmeshs(ns+2)
            n2 = kmeshs(ns+3)
            if ( k==nsubsf ) then

!           --- k = nsubsf   Last surface in row

               if ( jsurnr>0 ) then
                  n = n2
                  nh = n1 + n0
               else
                  n = n1
                  nh = n0
               end if

               if ( istart+n>maxncursur ) then

!              --- Estimation of length of kmeshscopy too small

                  call errint ( istart+n, 1 )
                  call errint ( maxncursur, 2 )
                  call errchr ( 'kmeshscopy', 1 )
                  call errsub ( 550, 2 , 0, 1 )
                  go to 1000

               end if
               do j = 1, n

                  icurve = kmeshscopy(kstart+nh+j)
                  jcurve = abs(icurve)
                  istart = istart + 1
                  ntel = ntel + 1
                  kmeshscopy(istart) = icurve
                  iwork(jcurve) = -1

               end do

            end if

!           --- Fill array iwork

            kstart = iworkcopy(iwork1(ksurnr))
            do j = 1, iworkcopy(iwork1(ksurnr)+1) - kstart
               icurve = abs(kmeshscopy(kstart+j))
               if ( iwork(icurve)==0 ) iwork(icurve) = 1
            end do
         end do
         nkinp = nkinp + nsubsf
      end do
      kmeshs(lstart+6) = ntel
      do i = 1, ncurvs
         if ( iwork(i)==1 ) then
            istart = istart + 1
            kmeshscopy(istart) = i
         end if
      end do
      if ( istart>maxncursur ) then

!     --- Estimation of length of kmeshscopy too small

         call errint ( istart, 1 )
         call errint ( maxncursur, 2 )
         call errchr ( 'kmeshscopy', 1 )
         call errsub ( 550, 2 , 0, 1 )
         go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh06312'

      end if  ! ( debug )
      call erclos ( 'msh06312' )

      end
