      subroutine msh076 ( iinput, ikelmc, inodpbef, iinpsurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-03-2006 Extra parameters
!        version  1.0  date 04-03-1997
!
!   copyright (c) 1997-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kmeshc for interface elements
!     The checking of the input is performed in subroutine MSH038
!     That subroutine also computes the length of KMESHC
!     As a consequence a change in msh076 must always result in a change
!     in MSH075
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), ikelmc, inodpbef, iinpsurf

!     iinpsurf       o    Value of iinp at start of surfaces
!     iinput         i    integer input array containing information of the
!                         interface elements
!     ikelmc        i/o   last position used in kmesh part c
!     inodpbef       i    First nodal point to be used for extra points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurvs, iinp, isurfs


!     icurvs         Number of parts where curve interface elements are given
!     iinp           last position used in array iinput
!     isurfs         Number of parts where surface interface elements
!                    are given
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH076CURVS    fill array kmeshc for interface elements (curve elements)
!     MSH076SURFS    fill array kmeshc for interface elements (surface elements)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh076' )
      icurvs = iinput(2)
      isurfs = iinput(3)
      iinp = 4
      if ( icurvs>0 ) then

!     --- icurvs>0, there are curve interface elements

         call msh076curvs ( iinput, iinp, icurvs, inodpbef, ikelmc )

      end if  ! ( icurvs>0 )
      iinpsurf = iinp
      if ( isurfs>0 ) then

!     --- isurfs>0, there are surface interface elements
!                   The coordinates are filled in a separate subroutine
!                   because of the need of memeory management subroutines

         call msh076surfs ( iinput, iinp, isurfs, ikelmc )

      end if  ! ( isurfs>0 )

      call erclos ( 'msh076' )
      end
