      subroutine msh464 ( kelem, nelem, ibrpnt, istart, kelemh, &
                          ishape, coor, npoint, ndim, ja, &
                          factor, niedge, nisurf, nivolm, nponew )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.11 date 25-02-2004 Special treatment for shape 5
!        version  1.10 date 01-01-2004 Special treatment for shape 13
!        version  1.9  date 11-06-1997 Correction for internal points
!
!   copyright (c) 1995-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refill kmesh part c with new spectral elements after a spectral
!     refinement of the MESH. If needed surface and volume coordinates are
!     filled in in COOR .
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(*), nelem, ibrpnt(*), istart(*), kelemh(*), &
              ishape, npoint, ndim, ja, niedge, nisurf, nivolm, &
              nponew
      double precision coor(ndim,*), factor(0:*)

!     coor    i/o   coordinate array
!     factor   i    array with the distribution of nodes along a
!                   straight line from -1 to +1
!     ibrpnt   i    neighbour array
!     ishape   i    SEPRAN type of element
!     istart   i    array with starting addresses in ibrpnt
!     ja       i    indicator whether coordinates have to be computed
!                   ja = 1 or not ja = 0
!     kelem    i    old element array of basis mesh
!     kelemh   o    new element array of spectral mesh
!     ndim     i    dimension considered (2d or 3d)
!     nelem    i    number of elements
!     niedge   i    number of internal nodes on line part
!     nisurf   i    number of internal nodes on surface part
!     nivolm   i    number of internal nodes in volume part
!     npoint   i    number of nodes in original mesh
!     nponew  i/o   actual number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i(27), ie, it, iel, iele, ielpos, j, newplm, nisrf, &
              nivlm, inode(9), ia, ib, ic, ih, idim, npt, nisrftmp
      double precision x1, x2, x3

!     i         node array
!     ia        help node
!     ib        help node
!     ic        help node
!     idim      loop variable for dimension
!     ie        node number
!     iel       loop variable
!     iele      help variable
!     ielpos    startposition for element
!     ih        help node
!     inode     helparray to store characteristic surface nodes
!     it        node number
!     j         loop variable
!     newplm    number of nodes in new spectral element
!     nisrf     local computed number of nodes in surface
!     nisrftmp  help variable equal to 0 or nisrf
!     nivlm     local computed number of nodes in volume
!     npt       starting node for volume points
!     x1        co-ordinate of first vertex
!     x2        co-ordinate of second vertex
!     x3        co-ordinate of third vertex
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         unconcatenate name from calling sequence
!     EROPEN         concatenate name to calling sequence
!     ERRINT         place integer in error array
!     ERRSUB         submit error message
!     MSH403         get node number of new node
!     MSH416         determine diagonal to be preferred in quadrilateral
!     MSH461         sort the three nodesnumbers (of a triangle)
!     MSH475         place coordinates of internal points of spectral
!                    quadrilateral
!     MSH481         place coordinates of internal points of spectral
!                    hexahedron
!     MSH483         create spectral quadrilateral element
!     MSH484         create spectral hexahedron element
!     MSH490         check the elements created
!     MSH493         create spectral triangular element
!     MSH494         create spectral tetrahedronal element
!     MSHG21         create new coordinates in standard quadrilateral
!     MSHSPECREORD05 Reorder elements of shape 5 in case of spectral elements
!                    without fill-in (one element group only)
!     MSHSPECREORD13 Reorder elements of shape 13 in case of spectral elements
!                    without fill-in (one element group only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     952   Wrong element number specified, not available
!    1019   Internal points not yet implemented for this shape
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check which ishape has to be considered
!     Make new (spectral) elements and (if wanted) place coordinates
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh464' )

      if ( ishape.eq.1 ) then

!     --- Basic form is line element with two nodes

         newplm = 2 + niedge

         do iel = 1, nelem

            i(1) = kelem(2*iel-1)
            i(2) = kelem(2*iel)

!           --- Start node

            call msh403 ( npoint, i(1), i(2), ibrpnt, istart, i(3) )

!           --- Place niedge nodes between

            kelemh(newplm*(iel-1)+1) = i(1)

            if ( i(2).gt.i(1) ) then
               do j = 1, niedge
                  kelemh(newplm*(iel-1)+j+1) = i(3)+j-1
               end do
            else
               do j = 1, niedge
                  kelemh(newplm*(iel-1)+j+1) = i(3)+niedge-j
               end do
            end if

            kelemh(newplm*iel) = i(2)

         end do

      else if ( ishape.eq.3 ) then

!     --- Basic element is triangle with three nodes

         if ( nisurf.eq.-1 ) then

!        --- Determine number of internal nodes

            if ( niedge.eq.0 ) then

               nisrf = 0

            else

               nisrf = 1

            end if

         else

            nisrf = nisurf

         end if

!        --- Determine number of nodes in new element

         newplm = 3+3*niedge+nisrf

         do iel = 1, nelem

            i(1) = kelem(3*iel-2)
            i(3) = kelem(3*iel-1)
            i(5) = kelem(3*iel)

!           --- new nodal point numbers

            if ( niedge.gt.0 ) then

               call msh403 ( npoint, i(1), i(3), ibrpnt, &
                            istart, i(2) )
               call msh403 ( npoint, i(3), i(5), ibrpnt, &
                            istart, i(4) )
               call msh403 ( npoint, i(5), i(1), ibrpnt, &
                            istart, i(6) )

            end if

            if ( ndim.eq.3 .and. nisrf.gt.0 ) then

!           --- Sort the node numbers

               call msh461 ( i(1), i(3), i(5), ia, ib, ic )

               ih=-(ib+npoint*ic)

               call msh403 ( npoint, ia, ih, ibrpnt, istart, i(7) )

            end if

!           --- Determine new element and coordinates of internal
!               node numbers
!               Create new (spectral) triangle

            ielpos = newplm*(iel-1)+1
            if ( ndim.eq.2 ) then
               nisrftmp = 0
            else
               nisrftmp = nisrf
            end if

            call msh493 ( kelemh(ielpos), i, niedge, nisrftmp )
            if ( ndim.eq.2 .and. nisrf.gt.0 ) then
               i(7) = nponew+1
               do j = 1, nisrf
                  kelemh(ielpos+newplm-nisrf+j-1) = nponew+j
               end do
               nponew = nponew+nisrf
            end if

!           --- Place coordinates in the internal points

            if ( ja.eq.1 .and. nisrf.ne.0 ) then

               if ( nisrf.eq.1 ) then

!              --- Only barycentre

                  do idim = 1, ndim

                     coor(idim,i(7)) = &
                         ( coor(idim,i(1)) + &
                           coor(idim,i(3)) + &
                           coor(idim,i(5)) ) / 3d0

                  end do

               else if ( nisrf.eq.3 ) then

!              --- Place three points

                  do idim = 1, ndim

                     x1 = coor(idim,kelemh(ielpos+2))
                     x2 = coor(idim,kelemh(ielpos+6))
                     x3 = coor(idim,kelemh(ielpos+10))
                     coor(idim,i(7)) = 0.5d0*(x1+x3)
                     coor(idim,i(7)+1) = 0.5d0*(x2+x1)
                     coor(idim,i(7)+2) = 0.5d0*(x3+x2)

                  end do

               end if

            end if

         end do

      else if ( ishape.eq.5 ) then

!     --- Basic element is quadrilateral with four nodes

         if ( nisurf.eq.-1 ) then

!        --- Determine number of internal nodes following the standard
!            way

            if ( niedge.eq.0 ) then

               nisrf = 0

            else

               nisrf = niedge*niedge

            end if

         else

            nisrf = nisurf

         end if

         newplm = 4+4*niedge+nisrf

         if ( niedge.eq.0 ) then

!        --- Special case, renumber nodes

            do iel = 1, nelem*4
               kelemh(iel) = kelem(iel)
            end do

            call mshspecreord05 ( kelemh, nelem )

         else

            do iel = 1, nelem

               i(1) = kelem(4*iel-3)
               i(2) = kelem(4*iel-2)
               i(3) = kelem(4*iel-1)
               i(4) = kelem(4*iel )

               call msh403 ( npoint, i(1), i(2), ibrpnt, istart, i(5) )
               call msh403 ( npoint, i(2), i(3), ibrpnt, istart, i(6) )
               call msh403 ( npoint, i(3), i(4), ibrpnt, istart, i(7) )
               call msh403 ( npoint, i(4), i(1), ibrpnt, istart, i(8) )

!              --- Determine i(9)

               if ( nisrf.gt.0 ) then

                  call msh416( i(1), i(2), i(3), i(4), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(9) )

               else

                  i(9) = 0

               end if

!              --- Create new (spectral) quadrilateral

               ielpos = newplm*(iel-1)+1

               call msh483 ( kelemh(ielpos), i, niedge, nisrf )

!              --- Place coordinates in the internal points

               if ( ja.eq.1 .and. nisurf.ne.0 ) &
                    call mshg21( kelemh(ielpos), niedge, coor, ndim, &
                                 factor )

            end do

         end if  ! ( niedge.eq.0 )

      else if ( ishape.eq.11 ) then

!     --- Basic element is tetrahedron with four nodes

         if ( nisurf.eq.-1 .and. nivolm.eq.-1 ) then

!        --- Determine internal nodes following the standard way

            if ( niedge.eq.0 ) then

               nisrf = 0
               nivlm = 0

            else

               nisrf = 1
               nivlm = 1

            end if

         else

            nisrf = nisurf
            nivlm = nivolm

         end if

         newplm = 4+6*niedge+4*nisrf+nivlm

         do iel = 1, nelem

            i( 1) = kelem(4*iel-3)
            i( 3) = kelem(4*iel-2)
            i( 5) = kelem(4*iel-1)
            i(10) = kelem(4*iel )

!           --- new nodal point numbers

            if ( niedge.gt.0 ) then

               call msh403 ( npoint, i(1), i( 3), ibrpnt, istart, &
                            i( 2) )
               call msh403 ( npoint, i(3), i( 5), ibrpnt, istart, &
                            i( 4) )
               call msh403 ( npoint, i(5), i( 1), ibrpnt, istart, &
                            i( 6) )
               call msh403 ( npoint, i(1), i(10), ibrpnt, istart, &
                            i( 7) )
               call msh403 ( npoint, i(3), i(10), ibrpnt, istart, &
                            i( 8) )
               call msh403 ( npoint, i(5), i(10), ibrpnt, istart, &
                            i( 9) )

            end if

            if ( nisrf.gt.0 ) then

!           --- Determine the internal nodes of the faces

!               Sort the node numbers

               call msh461 ( i(1), i(3), i(5), ia, ib, ic )

               ih=-(ib+npoint*ic)

               call msh403 ( npoint, ia, ih, ibrpnt, istart, i(11) )

               call msh461 ( i(1), i(3), i(10), ia, ib, ic )

               ih=-(ib+npoint*ic)

               call msh403 ( npoint, ia, ih, ibrpnt, istart, i(12) )

               call msh461( i(3), i(5), i(10), ia, ib, ic )

               ih=-(ib+npoint*ic)

               call msh403 ( npoint, ia, ih, ibrpnt, istart, i(13) )

               call msh461( i(1), i(5), i(10), ia, ib, ic )

               ih=-(ib+npoint*ic)

               call msh403 ( npoint, ia, ih, ibrpnt, istart, i(14) )

            end if

            if ( nivlm.gt.0 ) then

                 i(15)  = nponew+1
                 nponew = nponew+nivlm

            end if

!           --- Determine new element and coordinates of internal
!               node numbers

!               Create new (spectral) tetrahedron

            ielpos = newplm*(iel-1)+1

            call msh494( kelemh(ielpos), i, niedge, nisrf, nivlm )

!           --- Place coordinates in the internal surface points

            if ( ja.eq.1 .and. nisrf.ne.0 ) then

!           --- Place barycentre

               if ( nisrf.ne.1 ) then

                  call errint ( ishape, 1 )
                  call errsub ( 1019, 1, 0, 0 )
                  go to 1000

               end if

!              --- Face 1

               do idim = 1, ndim

                   coor(idim,i(11)) = &
                         ( coor(idim,i(1)) + &
                           coor(idim,i(3)) + &
                           coor(idim,i(5)) ) / 3d0

               end do

!              --- Face 2

               do idim = 1, ndim

                   coor(idim,i(12)) = &
                         ( coor(idim,i( 1)) + &
                           coor(idim,i( 3)) + &
                           coor(idim,i(10)) ) / 3d0

               end do

!              --- Face 3

               do idim = 1, ndim

                   coor(idim,i(13)) = &
                         ( coor(idim,i( 3)) + &
                           coor(idim,i( 5)) + &
                           coor(idim,i(10)) ) / 3d0

               end do

!              --- Face 4

               do idim = 1, ndim

                   coor(idim,i(14)) = &
                         ( coor(idim,i( 1)) + &
                           coor(idim,i( 5)) + &
                           coor(idim,i(10)) ) / 3d0

               end do

            end if

            if ( ja.eq.1 .and. nivlm.ne.0 ) then

               if ( nivlm.ne.1 ) then

                  call errint ( ishape, 1 )
                  call errsub ( 1019, 1, 0, 0 )
                  go to 1000

               end if

!              --- Internal volume point

               do idim = 1, ndim

                   coor(idim,i(15)) = &
                         ( coor(idim,i( 1)) + &
                           coor(idim,i( 3)) + &
                           coor(idim,i( 5)) + &
                           coor(idim,i(10)) ) / 4d0

               end do

            end if

         end do

      else if ( ishape.eq.13 ) then

!     --- Basic element is hexahedron with 8 nodes

         if ( nisurf.eq.-1 .and. nivolm.eq.-1 ) then

!        --- Determine internal nodes following the standard way

            nisrf = niedge*niedge
            nivlm = niedge*niedge*niedge

         else

            nisrf = nisurf
            nivlm = nivolm

         end if

         newplm = 8+12*niedge+6*nisrf+nivlm
         if ( niedge.eq.0 ) then

!        --- Special case, renumber nodes

            call mshspecreord13 ( kelem, nelem )

         else

            do iel = 1, nelem

               iele = 8*(iel-1)

               do j = 1, 8

                   i(j) = kelem(iele+j)

               end do

!              --- new nodal point numbers
!                  Start nodes on lines

               call msh403 ( npoint, i(1), i(2), ibrpnt, istart, &
                            i( 9))
               call msh403 ( npoint, i(2), i(3), ibrpnt, istart, &
                            i(10) )
               call msh403 ( npoint, i(3), i(4), ibrpnt, istart, &
                            i(11) )
               call msh403 ( npoint, i(4), i(1), ibrpnt, istart, &
                            i(12) )
               call msh403 ( npoint, i(1), i(5), ibrpnt, istart, &
                            i(13) )
               call msh403 ( npoint, i(2), i(6), ibrpnt, istart, &
                            i(14) )
               call msh403 ( npoint, i(3), i(7), ibrpnt, istart, &
                            i(15) )
               call msh403 ( npoint, i(4), i(8), ibrpnt, istart, &
                            i(16) )
               call msh403 ( npoint, i(5), i(6), ibrpnt, istart, &
                            i(17) )
               call msh403 ( npoint, i(6), i(7), ibrpnt, istart, &
                            i(18) )
               call msh403 ( npoint, i(7), i(8), ibrpnt, istart, &
                            i(19) )
               call msh403 ( npoint, i(8), i(5), ibrpnt, istart, &
                            i(20) )

!              --- Six faces

               if ( nisrf.gt.0 ) then

!              --- Determine start nodes on faces

                  call msh416( i(1), i(2), i(3), i(4), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(21) )

                  call msh416( i(1), i(2), i(6), i(5), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(22) )

                  call msh416( i(2), i(3), i(7), i(6), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(23) )

                  call msh416( i(3), i(4), i(8), i(7), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(24) )

                  call msh416( i(1), i(4), i(8), i(5), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(25) )

                  call msh416( i(5), i(6), i(7), i(8), ie, it )
                  it = it*npoint
                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(26) )

!                 --- Place ( if wanted ) new coordinates

                  if ( ja.eq.1 ) then

                     inode(1) = i( 1)
                     inode(2) = i( 2)
                     inode(3) = i( 3)
                     inode(4) = i( 4)
                     inode(5) = i( 9)
                     inode(6) = i(10)
                     inode(7) = i(11)
                     inode(8) = i(12)
                     inode(9) = i(21)

                     call msh475( coor, inode, niedge, nisrf, &
                                  factor )

                     inode(1) = i( 1)
                     inode(2) = i( 2)
                     inode(3) = i( 6)
                     inode(4) = i( 5)
                     inode(5) = i( 9)
                     inode(6) = i(14)
                     inode(7) = i(17)
                     inode(8) = i(13)
                     inode(9) = i(22)

                     call msh475( coor, inode, niedge, nisrf, &
                                  factor )

                     inode(1) = i( 2)
                     inode(2) = i( 3)
                     inode(3) = i( 7)
                     inode(4) = i( 6)
                     inode(5) = i(10)
                     inode(6) = i(15)
                     inode(7) = i(18)
                     inode(8) = i(14)
                     inode(9) = i(23)

                     call msh475( coor, inode, niedge, nisrf, &
                                  factor )

                     inode(1) = i( 4)
                     inode(2) = i( 3)
                     inode(3) = i( 7)
                     inode(4) = i( 8)
                     inode(5) = i(11)
                     inode(6) = i(15)
                     inode(7) = i(19)
                     inode(8) = i(16)
                     inode(9) = i(24)

                     call msh475( coor, inode, niedge, nisrf, &
                                  factor )

                     inode(1) = i( 1)
                     inode(2) = i( 4)
                     inode(3) = i( 8)
                     inode(4) = i( 5)
                     inode(5) = i(12)
                     inode(6) = i(16)
                     inode(7) = i(20)
                     inode(8) = i(13)
                     inode(9) = i(25)

                     call msh475( coor, inode, niedge, nisrf, &
                                  factor )

                     inode(1) = i( 5)
                     inode(2) = i( 6)
                     inode(3) = i( 7)
                     inode(4) = i( 8)
                     inode(5) = i(17)
                     inode(6) = i(18)
                     inode(7) = i(19)
                     inode(8) = i(20)
                     inode(9) = i(26)

                     call msh475( coor, inode, niedge, nisrf, &
                                  factor )

                  end if

               end if

!              --- Special for main diagonal

               if ( nivlm.gt.0 ) then

                  ie = min( i(1), i(7) )
                  it = max( i(1), i(7) )

                  it = npoint*(npoint+1)+it

                  call msh403 ( npoint, ie, it, ibrpnt, istart, i(27) )

               end if

!              --- Create new (spectral) hexahedrons

               ielpos = newplm*(iel-1 )+1

               call msh484( kelemh(ielpos ), i, niedge, nisrf, &
                            nivlm )

!              --- Place coordinates in the interior points

               npt = i(27)

               call msh481( coor, kelemh(ielpos), npt, &
                            niedge, nisrf, nivlm, factor )

!              --- Check new created hexahedrons and coordinates

               call msh490( kelemh(ielpos ), coor, &
                            niedge, nisrf, nivlm )

            end do  ! iel = 1, nelem

         end if  ! ( niedge.eq.0 )

      else if ( ishape.eq.50 ) then

!     --- Interface element

         newplm = 2*(2+niedge)

         do iel = 1, nelem

            i(1) = kelem(4*iel-3)
            i(2) = kelem(4*iel-1)
            call msh403 ( npoint, i(1), i(2), ibrpnt, istart, i(3) )

!           --- Place niedge nodes between

            kelemh(newplm*(iel-1)+1) = i(1)
            if ( i(2).gt.i(1) ) then
               do j = 1, niedge
                  kelemh(newplm*(iel-1)+2*j+1) = i(3)+j-1
               end do
            else
               do j = 1, niedge
                  kelemh(newplm*(iel-1)+2*j+1) = i(3)+niedge-j
               end do
            end if
            kelemh(newplm*iel-1) = i(2)

            i(1) = kelem(4*iel-2)
            i(2) = kelem(4*iel)
            call msh403 ( npoint, i(1), i(2), ibrpnt, istart, i(3) )

!           --- Place niedge nodes between

            kelemh(newplm*(iel-1)+2) = i(1)
            if ( i(2).gt.i(1) ) then
               do j = 1, niedge
                  kelemh(newplm*(iel-1)+2*j+2) = i(3)+j-1
               end do
            else
               do j = 1, niedge
                  kelemh(newplm*(iel-1)+2*j+2) = i(3)+niedge-j
               end do
            end if
            kelemh(newplm*iel) = i(2)

         end do

      else if ( ishape.lt.0 ) then

!     --- Special element : Connected, do nothing

      else

         call errint ( ishape, 1 )
         call errsub ( 952, 1, 0, 0 )
         go to 1000

      end if

1000  call erclos( 'msh464' )

      end
