      subroutine msh481( coor, kelem, npt, niedge, nisurf, nivolm, &
                         factor )
! =======================================================================
!
!
!         programmer   niek praagman
!         version 1.1  date  21-11-1995 Revision computation internal
!                                       spectral points
!         version 1.0  date  07-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Compute coordinates internal spectral point(s) in a hexahedron
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     hexahedron
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none

      integer npt, niedge, nisurf, nivolm, kelem(*)
      double precision coor( 3, * ), factor(0:*)

!     coor    i,o  coordinates array
!     factor   i   array with distribution coordinates
!     kelem    i   array with all nodes of element
!     niedge   i   number of internal (spectral) nodes on line
!     nisurf   i   number of internal (spectral) nodes in faces
!     nivolm   i   number of internal (spectral) nodes in volume
!     npt      i   node number to start computation of internal nodes for
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer idim, i, j, k, ih, ieven, ipl, jpl, kpl, lpl, ipos, &
              inp(27), nplane
      double precision x1(27), x2(27), x3(27), xn(3), &
             zta, eta, ski

!     eta       second local coordinate
!     i         loop variable
!     idim      loop variable for dimension
!     ieven     indicator for number of extra sidepoints even or odd
!     ih        helpvariable
!     inp       helparray for nodes of transformation
!     ipl       pointer
!     ipos      pointer
!     j         loop variable
!     jpl       pointer
!     k         loop variable
!     kpl       pointer
!     lpl       pointer
!     nplane    number of nodes in one cross-section
!     ski       first local coordinate
!     x1        x-coordinates basis transformation
!     x2        y-coordinates basis transformation
!     x3        z-coordinates basis transformation
!     xn        array to store coordinates new point
!     zta       third local coordinate
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate subroutine name from calling sequence
!     EROPEN   Concatenate subroutine name to calling sequence
!     INSTOP   Stop execution of SEPRAN programm
!     MSHG24   Compute "world"-coordinates of submitted (ski,eta,zta) point
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Do some checks
!
!     Determine the basis for a quadratic isoparametric transformation
!
!     Run through all internal points and compute the coordinates of
!     the nodes in accordance with the values given in the distribution
!     array factor using the isoparametric transformation
!
! =======================================================================

      call eropen( 'msh481' )

!     Check first the situation

      if ( nisurf.ne.niedge * niedge .or. &
           nivolm.ne.niedge * nisurf ) then

         write(*,*) ' Not yet implemented possibility in MSH481 '
         write(*,*) ' Sidepoints = ',niedge
         write(*,*) ' Facepoints = ',nisurf
         write(*,*) ' Volmpoints = ',niedge
         write(*,*) ' Please inform SEPRA that you want this pos-'
         write(*,*) ' sibility to be included '
         write(*,*) ' Execution terminated '

         call instop

      end if

!     Run through all internal points and adjust the coordinates
!     utilizing an isoparametric projection for the corners and
!     the midpoints

      ih = 2 * int( niedge / 2 )
      nplane = ( 2 + niedge ) * ( 2 + niedge )

      if ( ih.eq.niedge ) then

         ieven = 2

      else

         ieven = 1

      end if

      if ( ieven.eq.1 ) then

!        Number of sidepoints is odd, simple situation

         ih = int( niedge / 2 ) + 1

         ipos   = 1
         inp(1) = kelem(ipos)
         ipos   = ipos + ih
         inp(2) = kelem(ipos)
         ipos   = ipos + ih
         inp(3) = kelem(ipos)
         ipos   = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         inp(4) = kelem(ipos)
         ipos   = ipos + ih
         inp(5) = kelem(ipos)
         ipos   = ipos + ih
         inp(6) = kelem(ipos)
         ipos   = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         inp(7) = kelem(ipos)
         ipos   = ipos + ih
         inp(8) = kelem(ipos)
         ipos   = ipos + ih
         inp(9) = kelem(ipos)
         ipos   = 1 + ih * nplane
         inp(10) = kelem(ipos)
         ipos   = ipos + ih
         inp(11) = kelem(ipos)
         ipos   = ipos + ih
         inp(12) = kelem(ipos)
         ipos   = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         inp(13) = kelem(ipos)
         ipos   = ipos + ih
         ipos   = ipos + ih
         inp(14) = kelem(ipos)
         ipos   = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         inp(15) = kelem(ipos)
         ipos   = ipos + ih
         inp(16) = kelem(ipos)
         ipos   = ipos + ih
         inp(17) = kelem(ipos)
         ipos   = 1 + ( 1 + niedge ) * nplane
         inp(18) = kelem(ipos)
         ipos   = ipos + ih
         inp(19) = kelem(ipos)
         ipos   = ipos + ih
         inp(20) = kelem(ipos)
         ipos   = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         inp(21) = kelem(ipos)
         ipos   = ipos + ih
         inp(22) = kelem(ipos)
         ipos   = ipos + ih
         inp(23) = kelem(ipos)
         ipos   = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         inp(24) = kelem(ipos)
         ipos   = ipos + ih
         inp(25) = kelem(ipos)
         ipos   = ipos + ih
         inp(26) = kelem(ipos)

!        Now the coordinates of the basis points can be computed

         do 110 i = 1 , 26

             jpl   =  inp(i)

             x1(i) = coor( 1, jpl )
             x2(i) = coor( 2, jpl )
             x3(i) = coor( 3, jpl )

110      continue

      else

!        Number of sidepoints is even, difficult situation
!        Compute the coordinates of the basis points

         ih = int( niedge / 2 )

         ipos   = 1
         jpl    = kelem( ipos )

         x1(1)  = coor( 1, jpl )
         x2(1)  = coor( 2, jpl )
         x3(1)  = coor( 3, jpl )

         ipos   = 1 + ih
         ipl    = kelem( ipos     )
         jpl    = kelem( ipos + 1 )

         x1(2)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(2)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(2)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = 2 + niedge
         jpl    = kelem(ipos)

         x1(3)  = coor( 1, jpl )
         x2(3)  = coor( 2, jpl )
         x3(3)  = coor( 3, jpl )

         ipos   = ih * ( niedge + 2 ) + 1
         ipl    = kelem( ipos )
         ipos   = ipos + niedge + 2
         jpl    = kelem( ipos )

         x1(4)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(4)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(4)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ih * ( niedge + 2 ) + 1 + ih

         ipl    = kelem( ipos )
         jpl    = kelem( ipos + 1 )
         kpl    = kelem( ipos + niedge + 2 )
         lpl    = kelem( ipos + niedge + 3 )

         x1(5)  = ( coor(1,ipl) + coor(1,jpl) + coor(1,kpl) + &
                    coor(1,lpl) ) / 4d0
         x2(5)  = ( coor(2,ipl) + coor(2,jpl) + coor(2,kpl) + &
                    coor(2,lpl) ) / 4d0
         x3(5)  = ( coor(3,ipl) + coor(3,jpl) + coor(3,kpl) + &
                    coor(3,lpl) ) / 4d0

         ipos   = ( ih + 1 ) * ( niedge + 2 )
         ipl    = kelem( ipos )
         ipos   = ipos + niedge + 2
         jpl    = kelem( ipos )

         x1(6)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(6)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(6)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = nplane - niedge - 1
         jpl    = kelem( ipos )

         x1(7)  = coor( 1, jpl )
         x2(7)  = coor( 2, jpl )
         x3(7)  = coor( 3, jpl )

         ipos   = ipos + ih
         ipl    = kelem( ipos     )
         jpl    = kelem( ipos + 1 )

         x1(8)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(8)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(8)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = nplane
         jpl    = kelem( nplane )

         x1(9)  = coor( 1, jpl )
         x2(9)  = coor( 2, jpl )
         x3(9)  = coor( 3, jpl )

         ipos   = ih * nplane + 1
         ipl    = kelem( ipos )
         jpl    = kelem( ipos + nplane )

         x1(10)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(10)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(10)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ipos + ih
         ipl    = kelem( ipos              )
         jpl    = kelem( ipos +          1 )
         kpl    = kelem( ipos + nplane     )
         lpl    = kelem( ipos + nplane + 1 )

         x1(11)  = ( coor(1,ipl) + coor(1,jpl) + coor(1,kpl) + &
                     coor(1,lpl) ) / 4d0
         x2(11)  = ( coor(2,ipl) + coor(2,jpl) + coor(2,kpl) + &
                     coor(2,lpl) ) / 4d0
         x3(11)  = ( coor(3,ipl) + coor(3,jpl) + coor(3,kpl) + &
                     coor(3,lpl) ) / 4d0

         ipos   = ipos + ih + 1
         ipl    = kelem( ipos )
         jpl    = kelem( ipos + nplane )

         x1(12)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(12)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(12)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos    = ipos + ( ih - 1 ) * ( niedge + 2 ) + 1
         ipl     = kelem( ipos )
         jpl     = kelem( ipos + niedge + 2 )
         kpl     = kelem( ipos + nplane )
         lpl     = kelem( ipos + nplane + niedge + 2 )

         x1(13)  = ( coor(1,ipl) + coor(1,jpl) + coor(1,kpl) + &
                     coor(1,lpl) ) / 4d0
         x2(13)  = ( coor(2,ipl) + coor(2,jpl) + coor(2,kpl) + &
                     coor(2,lpl) ) / 4d0
         x3(13)  = ( coor(3,ipl) + coor(3,jpl) + coor(3,kpl) + &
                     coor(3,lpl) ) / 4d0

         ipos   =  ipos + niedge + 1
         ipl    =  kelem( ipos )
         jpl     = kelem( ipos + niedge + 2 )
         kpl     = kelem( ipos + nplane )
         lpl     = kelem( ipos + nplane + niedge + 2 )

         x1(14)  = ( coor(1,ipl) + coor(1,jpl) + coor(1,kpl) + &
                     coor(1,lpl) ) / 4d0
         x2(14)  = ( coor(2,ipl) + coor(2,jpl) + coor(2,kpl) + &
                     coor(2,lpl) ) / 4d0
         x3(14)  = ( coor(3,ipl) + coor(3,jpl) + coor(3,kpl) + &
                     coor(3,lpl) ) / 4d0

         ipos   = ( ih+1 ) * nplane - niedge - 1
         ipl    = kelem( ipos )
         jpl    = kelem( ipos + nplane )

         x1(15)  = ( coor( 1, ipl ) + coor( 1, jpl ) ) / 2d0
         x2(15)  = ( coor( 2, ipl ) + coor( 2, jpl ) ) / 2d0
         x3(15)  = ( coor( 3, ipl ) + coor( 3, jpl ) ) / 2d0

         ipos   =  ipos + ih
         ipl    =  kelem( ipos )
         jpl     = kelem( ipos + 1 )
         kpl     = kelem( ipos + nplane )
         lpl     = kelem( ipos + nplane + 1 )

         x1(16)  = ( coor(1,ipl) + coor(1,jpl) + coor(1,kpl) + &
                     coor(1,lpl) ) / 4d0
         x2(16)  = ( coor(2,ipl) + coor(2,jpl) + coor(2,kpl) + &
                     coor(2,lpl) ) / 4d0
         x3(16)  = ( coor(3,ipl) + coor(3,jpl) + coor(3,kpl) + &
                     coor(3,lpl) ) / 4d0

         ipos   = ipos + ih + 1
         ipl    = kelem( ipos     )
         jpl    = kelem( ipos + nplane )

         x1(17)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(17)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(17)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ( niedge + 1 ) * nplane + 1
         jpl    = kelem( ipos )

         x1(18)  = coor( 1, jpl )
         x2(18)  = coor( 2, jpl )
         x3(18)  = coor( 3, jpl )

         ipos   = ipos + ih
         ipl    = kelem( ipos     )
         jpl    = kelem( ipos + 1 )

         x1(19)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(19)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(19)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ipos + ih + 1
         jpl    = kelem(ipos)

         x1(20)  = coor( 1, jpl )
         x2(20)  = coor( 2, jpl )
         x3(20)  = coor( 3, jpl )

         ipos   = ipos + (ih-1) * ( niedge + 2 ) + 1
         ipl    = kelem( ipos )
         jpl    = kelem( ipos + niedge + 2 )

         x1(21)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(21)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(21)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ipos + ih

         ipl    = kelem( ipos )
         jpl    = kelem( ipos + 1 )
         kpl    = kelem( ipos + niedge + 2 )
         lpl    = kelem( ipos + niedge + 3 )

         x1(22)  = ( coor(1,ipl) + coor(1,jpl) + coor(1,kpl) + &
                    coor(1,lpl) ) / 4d0
         x2(22)  = ( coor(2,ipl) + coor(2,jpl) + coor(2,kpl) + &
                    coor(2,lpl) ) / 4d0
         x3(22)  = ( coor(3,ipl) + coor(3,jpl) + coor(3,kpl) + &
                    coor(3,lpl) ) / 4d0

         ipos   = ipos + ih + 1
         ipl    = kelem( ipos )
         ipos   = ipos + niedge + 2
         jpl    = kelem( ipos )

         x1(23)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(23)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(23)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ( niedge + 2 ) * nplane - niedge - 1
         jpl    = kelem( ipos )

         x1(24)  = coor( 1, jpl )
         x2(24)  = coor( 2, jpl )
         x3(24)  = coor( 3, jpl )

         ipos   = ipos + ih
         ipl    = kelem( ipos     )
         jpl    = kelem( ipos + 1 )

         x1(25)  = ( coor(1,ipl) + coor(1,jpl) ) / 2d0
         x2(25)  = ( coor(2,ipl) + coor(2,jpl) ) / 2d0
         x3(25)  = ( coor(3,ipl) + coor(3,jpl) ) / 2d0

         ipos   = ( niedge + 2 ) * nplane
         jpl    = kelem( ipos )

         x1(26)  = coor( 1, jpl )
         x2(26)  = coor( 2, jpl )
         x3(26)  = coor( 3, jpl )

      end if

!     Finally the centroid of the hexahedron, which is not
!     given a priori, is needed

      x1(27) = 0d0
      x2(27) = 0d0
      x3(27) = 0d0

      x1(27) = x1(27) + 0.5d0 * &
             ( x1( 5) + x1(11) + x1(13) + x1(14) + x1(16) + x1(22) ) - &
             ( x1( 1) + x1( 3) + x1( 7) + x1( 9) + &
               x1(18) + x1(20) + x1(24) + x1(26) ) / 4d0
      x2(27) = x2(27) + 0.5d0 * &
             ( x2( 5) + x2(11) + x2(13) + x2(14) + x2(16) + x2(22) ) - &
             ( x2( 1) + x2( 3) + x2( 7) + x2( 9) + &
               x2(18) + x2(20) + x2(24) + x2(26) ) / 4d0
      x3(27) = x3(27) + 0.5d0 * &
             ( x3( 5) + x3(11) + x3(13) + x3(14) + x3(16) + x3(22) ) - &
             ( x3( 1) + x3( 3) + x3( 7) + x3( 9) + &
               x3(18) + x3(20) + x3(24) + x3(26) ) / 4d0

!     Next compute the new internal points

      do 240 k = 1 , niedge

         zta = factor(k)

         do 230 j = 1 , niedge

            eta = factor(j)

            do 220 i = 1 , niedge

               ski = factor(i)

!              Determine coordinates

               call mshg24( ski, eta, zta, x1, x2, x3, xn )

!              Place new determined coordinates with corrections in
!              array coor

               do 210 idim = 1 , 3

                  coor( idim, npt ) = xn(idim)

210            continue

               npt = npt + 1

220         continue

230      continue

240   continue

      call erclos( 'msh481' )

      end
