      subroutine msh482 ( kmeshm, istkrm, lenkmeshm, kmeshmnew, istkrh, &
                          istart, ibrpnt, npoint, niedge )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.1  date 03-03-2003 Small adjustment
!        version  4.0  date 16-10-2002 Complete revision
!        version  3.0  date 22-08-2001 Extra parameter curvefacts
!        version  2.3  date 12-07-1999 New call to msh006
!        version  2.2  date 03-12-1997 Extension with curves defined with
!                                      coarseness
!        version  2.1  date 21-02-1997 Adjustment for translation
!        version  2.0  date 31-01-1997 Adjustment for rotation
!        version  1.2  date 06-12-1995 Small change
!        version  1.1  date 10-11-1995 Keep correct ordering in curves
!        version  1.0  date 07-04-1995
!
!   copyright (c) 1995-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Place "refine points" as found in kmeshm part III in the curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), istkrm, lenkmeshm, kmeshmnew(*), istkrh, &
              istart(*), ibrpnt(*), npoint, niedge

!     ibrpnt         i    neighbour array
!     istart         i    start addresses neighbour array
!     istkrh         i    start position for present curve in kmeshmnew
!     istkrm         i    start position for present curve in kmeshm
!     kmeshm         i    KMESHM part III before refinement
!     kmeshmnew      o    new KMESHM part III
!     lenkmeshm      i    length KMESHM
!     niedge         i    number of points to be placed between the two
!                         end nodes
!     npoint         i    old number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, i1, i2, ih, nelm

!     i        loop variable
!     i1       first point of line piece
!     i2       last point of line piece
!     ih       refinement point of line piece
!     j        loop variable
!     nelm     number of elements along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSH403    Determine number refinement point in line piece
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through line, determine the pieces
!     determine the new starting point of each piece
!     compute the coordinates of all the new points
!     fill the new internal points in in the new array
!     in special cases map new nodes onto original curves
!        if curve = arc  compute mapping and map
!        if curve = rotation compute mapping and map
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh482' )

      nelm =lenkmeshm - 1
      do i = 1, nelm

          i1  =  kmeshm( istkrm -1 + i  )
          i2  =  kmeshm( istkrm    + i  )

          call msh403( npoint, i1, i2, ibrpnt, istart, ih )
          if ( ierror.ne.0 ) go to 1000

!         --- Place niedge nodes

          kmeshmnew( istkrh + (1+niedge) * (i-1) )  =  i1

          if ( i1.lt.i2 ) then

             do j = 1, niedge

                kmeshmnew( istkrh + (1+niedge) * (i-1) + j ) = &
                       ih + j - 1

             end do

          else

             do j = 1, niedge

                kmeshmnew( istkrh + (1+niedge) * (i-1) + j ) = &
                       ih + niedge - j

             end do

          end if

          kmeshmnew( istkrh + (1+niedge)*i )  =  i2

      end do

1000  call erclos( 'msh482' )

      end
