      subroutine msh490( kelem, coor, niedge, nisurf, nivolm )
! =======================================================================
!
!
!         programmer   Niek Praagman
!         version 1.0  date  06-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "Ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Check new spectral hexahedrons
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     spectral
! ***********************************************************************
!
!
!     INPUT / OUTPUT PARAMETERS

      implicit none
      integer kelem(*), niedge, nisurf, nivolm
      double precision coor(3,*)

!     coor     i    coordinate array
!     kelem    i    element array (spectral elements)
!     niedge   i    number of internal nodes on line part
!     nisurf   i    number of internal nodes on surface part
!     nivolm   i    number of internal nodes in volume part
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!

      integer ipos, i, j, k, in(8), nilin, nisrf

!     i         loop variable
!     in        node array
!     ipos      pointer
!     j         loop variable
!     k         loop variable
!     nilin     number of nodes in one line
!     nisrf     local computed number of nodes in surface
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS    unconcatenate name from calling sequence
!     EROPEN    concatenate name to calling sequence
!     MSH319    check value of volume of tetrahedron
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Determine the eight cornernodes of each subelement
!
!     Check for six tetrahedrons that there volume is positive
!
! =======================================================================

      call eropen( 'msh490' )

      nilin  =       2 + niedge
      nisrf  =   nilin * nilin

      if ( niedge.gt.0 .and. &
           nisurf.eq.niedge * niedge .and. &
           nivolm.eq.niedge * nisurf ) then

         do 300 k = 1 , niedge + 1

         do 200 j = 1 , niedge + 1

         do 100 i = 1 , niedge + 1

             ipos = i + (j-1) * nilin + (k-1) * nisrf

             in(1) = kelem( ipos                     )
             in(2) = kelem( ipos                 + 1 )
             in(3) = kelem( ipos + nilin             )
             in(4) = kelem( ipos + nilin         + 1 )
             in(5) = kelem( ipos         + nisrf     )
             in(6) = kelem( ipos         + nisrf + 1 )
             in(7) = kelem( ipos + nilin + nisrf     )
             in(8) = kelem( ipos + nilin + nisrf + 1 )

             call msh319( in(1), in(6), in(2), in(8), coor )
             call msh319( in(1), in(2), in(4), in(8), coor )
             call msh319( in(1), in(4), in(3), in(8), coor )
             call msh319( in(1), in(3), in(7), in(8), coor )
             call msh319( in(1), in(7), in(5), in(8), coor )
             call msh319( in(1), in(5), in(6), in(8), coor )

100      continue

200      continue

300      continue

      end if

      call erclos( 'msh490' )

      end
