      subroutine msh492( i1, i2, i3, i4, niedge, inode )
! =======================================================================
!
!
!         programmer   niek praagman
!         version 1.1  date  17-11-1995 Small error in ordering removed
!         version 1.0  date  06-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Determine the ordering of the internal spectral point(s) in
!     a quadrilateral element
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     quadrilateral
!     spectral
!     storage
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none

      integer i1, i2, i3, i4, inode(*), niedge

!     i1      i   first corner node
!     i2      i   second corner node
!     i3      i   third corner node
!     i4      i   fourth corner node
!     inode   i   array with the ordering (displacements) for the nodes
!     niedge  i   number of spectral nodes on line element
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer ipos, i, j, imn

!     i       loop variable
!     imn     node where
!     ipos    pointer in array inode
!     j       loop variable
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Determine the node number of the starting node and next the
!     way in which the nodes row by row are stored
!
! =======================================================================

!     Determine which situation and fill inode accordingly

      imn = min( i1, i2, i3, i4 )

      if ( imn.eq.i1 ) then

!        To the right or to the left

         if ( i2.lt.i4 ) then

              do 120 i = 1 , niedge
                 do 110 j = 1 , niedge

                    ipos = j + ( i-1 ) * niedge
                    inode(ipos) = j + ( i-1 ) * niedge

110              continue
120           continue

         else

              do 170 i = 1 , niedge
                 do 160 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = i + (j-1) * niedge
160              continue
170           continue

         end if

      else if ( imn.eq.i2 ) then

!        To the right or to the left

         if ( i3.lt.i1 ) then

              do 220 i = 1 , niedge
                 do 210 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = niedge * ( niedge - j ) + i
210              continue
220           continue


         else

              do 270 i = 1 , niedge
                 do 260 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = i * niedge - j + 1
260              continue
270           continue

         end if

      else if ( imn.eq.i3 ) then

!        To the right or to the left

         if ( i4.lt.i2 ) then

              do 320 i = 1 , niedge
                 do 310 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = ( niedge + 1 - i ) * niedge - &
                                     j + 1
310              continue
320           continue

         else

              do 370 i = 1 , niedge
                 do 360 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = niedge * niedge - (i-1) - &
                                   (j-1) * niedge
360              continue
370           continue

         end if

      else if ( imn.eq.i4 ) then

!        To the right or to the left

         if ( i1.lt.i3 ) then

              do 420 i = 1 , niedge
                 do 410 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = ( niedge + 1 - i ) + (j-1) * niedge
410              continue
420           continue

         else

              do 470 i = 1 , niedge
                 do 460 j = 1 , niedge
                     ipos = j + ( i-1 ) * niedge
                     inode(ipos) = ( niedge-i ) * niedge + j
460              continue
470           continue

         end if

      end if

      end
