      subroutine mshcheckobst ( obstfound, iinput, ipointcurv, &
                                infocurve )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-08-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if there are points on the obstacle that do not correspond to
!    a curve of type 10000, but are also part of a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
!     obstacle
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical obstfound
      integer iinput(*), infocurve(*), ipointcurv(*)

!     iinput         i    integer input array for the mesh
!     infocurve      o    Array of length ncurvs containing information about
!                         which curve is on a free surface and also
!                         part of an obstacle
!                         infocurve(i) = 0: standard curve
!                         infocurve(i) = iobst(>0): curve on free surface and
!                             also new part of obstacle iobst . The curve is not
!                             of type >= 10000
!                         infocurve(i) = -1: curve on obstacle that is made
!                            active
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curves in the part of iinput referring
!                         to curves
!     obstfound      o    If true an obstacle that is also part of a surface
!                         has become active
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, istart, iend, k, iinp, iobst, icurve, &
              istartsubcv, isub, isubcv, jsubcv, nsubcv, jcurnr

!     icurnr         Curve sequence number
!     icurve         Type of curve
!     iend           Last position used at a part
!     iinp           last position used in array iinput
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istartsubcv    ?
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isubcv         Subcurve sequence number with direction
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     jsubcv         |isubcv|
!     k              Counting variable
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    obstfound := false
!    for icurnr := 1 (1) ncurvs do
!       icurve := iinput(iinp)
!       if ( icurve#10000 and icurnr on free surface ) then
!          if ( there are nodes of the curve on an obstacle and
!               obstacle is part of surface ) then
!             obstfound := true
!       raise iinp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckobst' )
      obstfound = .false.
      do icurnr = 1, ncurvs
         infocurve(icurnr) = 0
      end do

!     --- loop over curves

      do icurnr = 1, ncurvs

         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         if ( icurve<10000 .and. &
              (kmeshz(icurnr)==-1 .or. kmeshz(icurnr)==2) ) then

!        --- icurve#10000 and icurnr on free surface ) then

            if ( icurnr==1 ) then
               istart = 1
            else
               istart = icurvsmsh(icurnr-1)+1
            end if
            iend = icurvsmsh(icurnr)
            do k = istart, iend
               iobst = icurvsobst2d(1,k)
               if ( iobst>0 ) then

!              --- there are nodes of the curve on an obstacle

                  if ( kmeshx(6,iobst)/=0 ) then

!                 --- obstacle is part of surface

                     obstfound = .true.
                     infocurve(icurnr) = iobst
                     jcurnr = kmeshx(2,iobst)
                     infocurve(jcurnr) = -1
                     if ( kelmq>0 ) then

!                    --- Possibly subcurves, mark subcurves also

                        istartsubcv = kmeshq(jcurnr)
                        nsubcv =  kmeshq(jcurnr+1)-istartsubcv
                        do isub = 1, nsubcv
                           isubcv = kmeshq(istartsubcv+isub)
                           jsubcv = abs(isubcv)
                           infocurve(jsubcv) = -1
                        end do

                     end if
                     go to 100

                  end if

               end if

            end do

         end if

100   end do

      call erclos ( 'mshcheckobst' )
      end
