      subroutine mshcirc3d ( iuserp1, iuserp2, rinput, curves, tau, &
                             inpncv, tang1, tang2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Define the mapping of a 3d plane to 2d using the 3 user points iuserp?
!    The mapping is defined by x = x1 + alpha t1 + beta t2
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     2d
!     3d
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuserp1, iuserp2, inpncv
      double precision rinput(3,*), tang1(3), tang2(3), curves(3,*), &
                       tau(*)

!     curves         o    array containing the coordinates of the curves
!     inpncv         i    Number of points in curve
!     iuserp1        i    First user point, centroid of circle
!     iuserp2        i    Second user point, first point on circle
!     rinput         i    Real input array
!     tang1          i    First tangential vector
!     tang2          i    Second tangential vector
!     tau            i    Work array of length n to store the relative distances
!                         between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision phi, radius, xicoor, etacoor

!     etacoor        eta coordinate
!     i              Counting variable
!     j              Counting variable
!     phi            angle (2 pi)
!     radius         Radius of sphere or circle
!     xicoor         xi coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      phi = 2d0*pi
      do j = 1, 3
         curves(j,1) = rinput(j,iuserp2)
      end do
      radius = sqrt ( (rinput(1,iuserp1)-rinput(1,iuserp2))**2 + &
                      (rinput(2,iuserp1)-rinput(2,iuserp2))**2 + &
                      (rinput(3,iuserp1)-rinput(3,iuserp2))**2 )
      do i = 2, inpncv-1
          xicoor  = radius * cos(tau(i)*phi)
          etacoor = radius * sin(tau(i)*phi)
          do j = 1, 3
             curves(j,i) = rinput(j,iuserp1)+ &
                           xicoor*tang1(j)+etacoor*tang2(j)
          end do
      end do
      do j = 1, 3
         curves(j,inpncv) = rinput(j,iuserp2)
      end do

      end
