      subroutine mshcompnelm ( p1, p2, rincor, edge, sleng, nelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of the number of elements given the length of
!     the curve, the unit length and the coarseness in the end points
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     curve
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: p1, p2
      integer, intent(out) :: nelm
      double precision, intent(in) :: rincor(*), sleng, edge

!     edge           i    Unit length defined for coarseness
!     nelm           o    number of elements along a curve
!     p1             i    user point number of first point
!     p2             i    user point number of second point
!     rincor         i    Contains the coarseness in the user points
!     sleng          i    length of the curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: coars1, coars2, edge1, edge2, edgemn

!     coars1         coarseness in the first user point
!     coars2         coarseness in the last user point
!     edge1          Length of first edge
!     edge2          Length of last edge
!     edgemn         Mean length of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompnelm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompnelm'
         write(irefwr,2) 'edge1, sleng', edge, sleng
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      coars1 = rincor(p1+1)
      coars2 = rincor(p2+1)

      edge1 = coars1*edge
      edge2 = coars2*edge

      if (abs(edge1/edge2-1d0)>=epsmac) then
         nelm = nint(sleng/(edge1-edge2)*log(edge1/edge2))
      else
         edgemn = 0.5d0*(edge1+edge2)
         nelm = nint(sleng/edgemn)
      end if

1000  call erclos ( 'mshcompnelm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nelm', nelm
         write(irefwr,2) 'coars1, coars2', coars1, coars2
         write(irefwr,2) 'edge1, edge2', edge1, edge2
         write(irefwr,*) 'End mshcompnelm'

      end if  ! ( debug )

      end subroutine mshcompnelm
