      subroutine mshd03 ( coor, kmeshc, itria, ipoly, ipolc, nr, ci, &
                          jpoly, jpoln, cj, ch, npt, coarse )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.1  date 25-11-2012 Long integers
!        version  5.0  date 30-12-1995 Correction for large differences in
!                                      coarseness due to Sander de Snoo
!        version  4.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1983-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine mshd03 divides a convex polygon into two new but
!    smaller convex polygons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     polygon
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), ipoly(*), nr(*), jpoly(*), itria, &
              ipolc, jpoln, npt
      double precision coor(2,*), ci(*), cj(*), ch(*), coarse(*)

!     ch         i
!     ci         i
!     cj         i
!     coarse     i    Array containing the coarseness per point
!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ipolc      i
!     ipoly      i
!     itria      i
!     jpoln      i
!     jpoly      i
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     npt        i    total number of points in the mesh
!     nr         i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, idir, ih, imog, j, j1, j2, jmog, &
              n, n1
      logical bforty, bbest
      double precision a, af1, af2, af3, af4, afwijk, ah, alpha, &
                       c1, c2, c3, c4, &
                       ch1, ch2, ch3, ch4, cmin, cmax, dx, dy, &
                       r1, r3, s, sl1, sl2, som, &
                       xi, yi, xi1, yi1, xi2, yi2, xi3, yi3, &
                       xi4, yi4, xj, yj, xj1, xj2, yj1, yj2, &
                       rimog, rjmog, slang

!     a          Help variable to store quantity
!     af1
!     af2
!     af3
!     af4
!     afwijk
!     ah
!     alpha
!     bbest
!     bforty
!     c1
!     c2
!     c3
!     c4
!     ch1
!     ch2
!     ch3
!     ch4
!     cmax
!     cmin
!     dx         length of vector in x-direction
!     dy         length of vector in y-direction
!     i          Counting variable
!     i1
!     i2
!     i3
!     i4
!     idir
!     ih         Help variable to store a constant temporarily
!     imog
!     j          Counting variable
!     j1
!     j2
!     jmog
!     n          length of polygon
!     n1
!     r1
!     r3
!     rimog
!     rjmog
!     s
!     sl1
!     sl2
!     slang
!     som
!     xi
!     xi1
!     xi2
!     xi3
!     xi4
!     xj
!     xj1
!     xj2
!     yi
!     yi1
!     yi2
!     yi3
!     yi4
!     yj
!     yj1
!     yj2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer eround

!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     EROUND    Round real to integer with minimum relative error
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     INSTOP    Stop the program
!     MSHD02    Fill array kmeshc for triangles
!     MSHG01    Compute the cosine of an angle
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     350   Number of points in boundary too small
!     351   mesh generator general terminated the execution of the program
!     375   the difference in coarseness in two locations is too large in to
!           relation their mutual distance
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshd03' )
      n=ipoly(ipolc)
      if ( n<3 ) then
        call errint ( n, 1 )
        call errint ( 3, 2 )
        call errsub ( 350, 2, 0, 0 )
        call instop
      end if

!     --- fill array with cosine values of angles

      do 100 i=1,n
          nr(i)=ipoly(ipolc+i)
          ch(i)=ci(ipolc+i)
100   continue

!     --- if n = 3 than ready

      if ( n==3 ) then
        call mshd02(kmeshc,itria,nr(1),nr(2),nr(3))
        go to 1000
      end if

!     --- take care of n=4 and next cut off triangles

200   continue
      if ( n==4 ) then

!     --- n = 4 a quadrangle
!         check whether there is an angle larger then 140 degrees
!         or smaller then 60 degrees:

        j=0
        cmax = 2d0
        cmin = -2d0
        do 210 i=1,4
          if (( ch(i)<-0.766d0 ).and.( ch(i)<cmax )) then
            cmax = ch(i)
            j=i
          else if ( (cmax>1d0).and. &
                   (ch(i)>0.5d0).and.(ch(i)>cmin) ) then
            cmin = ch(i)
            j=i
          end if
210     continue
        if (j>0) then
          if( (cmax<=1d0) ) then
            i = mod(j+1,2)
          else
            i = mod(j,2)
          end if
        else

!       --- all angles smaller then 140 and larger then 60 degrees:
!           create shortest diagonal
!           (note: sqrt not necessary to compare lengths

          sl1=(coor(1,nr(1))-coor(1,nr(3)))**2 &
             +(coor(2,nr(1))-coor(2,nr(3)))**2
          sl2=(coor(1,nr(2))-coor(1,nr(4)))**2 &
             +(coor(2,nr(2))-coor(2,nr(4)))**2
          if ( sl1>sl2 ) then
            i = 1
          else
            i = 0
          end if
        end if
        if (i==0) then
          call mshd02(kmeshc,itria,nr(1),nr(2),nr(3))
          call mshd02(kmeshc,itria,nr(3),nr(4),nr(1))
        else
          call mshd02(kmeshc,itria,nr(1),nr(2),nr(4))
          call mshd02(kmeshc,itria,nr(2),nr(3),nr(4))
        end if
        go to 1000
      end if

!     --- polygon has more than four cornerpoints
!         determine smallest angle

      cmin=-1d0
      do 220 j=1,n
        if ( ch(j)>=cmin ) then
          cmin=ch(j)
          i=j
        end if
220   continue

!     --- point i has smallest angle

      if ( ch(i)<0.174d0 ) goto 300

!     --- angle is smaller than 80 degrees
!         ( consider neighbouring points i-2,i-1,i+1 and i+2 )

      xi=coor(1,nr(i))
      yi=coor(2,nr(i))
      i1=i+1
      if ( i1>n ) i1=i1-n
      i2=i+2
      if ( i2>n ) i2=i2-n
      i3=i-1
      if ( i3<1 ) i3=i3+n
      i4=i-2
      if ( i4<1 ) i4=i4+n
      xi1=coor(1,nr(i1))
      yi1=coor(2,nr(i1))
      xi2=coor(1,nr(i2))
      yi2=coor(2,nr(i2))
      xi3=coor(1,nr(i3))
      yi3=coor(2,nr(i3))
      xi4=coor(1,nr(i4))
      yi4=coor(2,nr(i4))

!     --- c1 is the cosine of i-2,i-1,i+1
!         c2 is the cosine of i-1,i+1,i+2

      call mshg01(xi4,yi4,xi3,yi3,xi1,yi1,c1)
      call mshg01(xi3,yi3,xi1,yi1,xi2,yi2,c2)

!     --- ch(i)>50 degrees?

      if ( ch(i)<0.65d0 ) then

!     --- 50<ch(i)<80
!         Check new angles c1 and c2:

        if ( (c1>0.766d0).or.(c2>0.766d0) ) goto 300

!       --- Compute all other new angles
!           c3 is the cosine of i+1,i-1,i
!           c4 is the cosine of i-1,i+1,i

        call mshg01(xi1,yi1,xi3,yi3,xi,yi,c3)
        call mshg01(xi3,yi3,xi1,yi1,xi,yi,c4)
        if ( (c3>0.766d0).or.(c4>0.766d0) ) goto 300

!       --- all new angles are greater than 40 degrees
!           ( create this triangle )

      end if

!     --- Cut of triangle i3 - i - i1
!         determine length of line i1-i3

      sl1=sqrt((xi1-xi3)**2+(yi1-yi3)**2)
      r1=coarse(nr(i1))
      r3=coarse(nr(i3))

!     --- extra point necessary

      if (abs(r1/r3-1d0)>=1d-8) then
         a = sl1/(r1-r3)*log(r1/r3)
      else
         a = 2*sl1/(r1+r3)
      end if

!     --- a>sqrt(2)+e and angle i>70 degrees? then 2 triangles.

      if ( (a<1.43d0) .or. (ch(i)>0.342d0) ) then

!     --- only one triangle

        call mshd02(kmeshc,itria,nr(i3),nr(i),nr(i1))

!       --- angles

        ch(i3)=c1
        ch(i1)=c2

!       --- renumber and start again

        do 230 j=i,n-1
          ch(j)=ch(j+1)
          nr(j)=nr(j+1)
230     continue
        n=n-1
      else

!     --- cut off two triangles; a new point has to be created

        alpha = sqrt(r3/r1)
        npt=npt+1
        coor(1,npt)=( alpha*coor(1,nr(i1))+coor(1,nr(i3)) )/(1+alpha)
        coor(2,npt)=( alpha*coor(2,nr(i1))+coor(2,nr(i3)) )/(1+alpha)
        coarse(npt)=coarse(nr(i1))*alpha
        ch(i3)=c1
        ch(i1)=c2
        ch(i) =-1d0
        call mshd02(kmeshc,itria,nr(i3),nr(i),npt)
        call mshd02(kmeshc,itria,nr(i),nr(i1),npt)
        nr(i)=npt

!       --- number of points in polygon (n) remains the same

      end if
      goto 200

!     --- determine splitting line in polygon
!         first try to create angles which are larger than 40 degrees

300   continue
      bbest=.false.
      bforty=.true.
      afwijk=5d0

!     --- search for best possibility

310   continue
      do 350 i=1,n-2
        i1=i-1
        if ( i1<1 ) i1=i1+n
        i2=i+1
        xi=coor(1,nr(i))
        yi=coor(2,nr(i))
        xi1=coor(1,nr(i1))
        yi1=coor(2,nr(i1))
        xi2=coor(1,nr(i2))
        yi2=coor(2,nr(i2))
        if (i<n/2) then
          j=i+n/2
          idir=+1
        else
          j=n+1
          idir=-1
        end if
320     continue
          j=j+idir
          if ( j>n ) then
            j=i+n/2
            idir=-1
          end if
          j1=j-1
          j2=j+1
          if ( j2>n ) j2=j2-n
          xj=coor(1,nr(j))
          yj=coor(2,nr(j))
          xj1=coor(1,nr(j1))
          yj1=coor(2,nr(j1))
          xj2=coor(1,nr(j2))
          yj2=coor(2,nr(j2))

!         --- c1 and c2 are monotome functions.
!             if idir=-1 then c1 is growing, c2 is shrinking
!             if idir=+1 then c2 is growing, c1 is shrinking
!             So if idir =-1 and if c2 is below 60 degrees
!             and if c1 is above 120 afwijk will only increase.
!             first angle  = i1,i,j

          call mshg01(xi1,yi1,xi,yi,xj,yj,c1)
          if ( (c1>0.99d0) .or. &
               ( (c1-0.5)>afwijk ) .or. &
               ( (bforty).and.(c1>0.766d0) ) ) then

!         --- skip this point or rest of j (skip interval(j,n) )

            if (idir==+1) j=n+1
            goto 340
          end if

!         --- second angle = i2,i,j

          call mshg01(xi2,yi2,xi,yi,xj,yj,c2)
          if ( (c2>0.99d0) .or. &
               ( (c2-0.5d0)>afwijk ) .or. &
               ( (bforty).and.(c2>0.766d0) ) ) then

!         --- skip this point or rest of j (skip interval(j,i2,(-1)) )

            if (idir==-1) j=i2+1
            goto 340
          end if

!         --- check combination of c1 and c2

          if (idir==-1) then
            if ( (c2-0.5d0)-(c1+0.5d0)>afwijk ) then
              j=i2+1

!             --- skip rest of j (skip interval(j,i2,(-1)) )

              goto 340
            end if
          else
            if ( (c1-0.5d0)-(c2+0.5d0)>afwijk ) then
              j=n+1

!             --- skip rest of j (skip interval(j,n) )

              goto 340
            end if
          end if

!         --- compute sum of diferences with 60 degrees

          af1=min( abs(c1-0.5d0),abs(c1+0.5d0) )
          af2=min( abs(c2-0.5d0),abs(c2+0.5d0) )
          if (af1+af2>afwijk) goto 340

!         --- third angle = j1,j,i

          call mshg01(xj1,yj1,xj,yj,xi,yi,c3)
          if ( (c3>0.99d0) .or. &
               ( (bforty).and.(c3>0.766d0) ) ) goto 340
          af3=min( abs(c3-0.5d0),abs(c3+0.5d0) )

!         --- fourth angle = j2,j,i

          call mshg01(xj2,yj2,xj,yj,xi,yi,c4)
          if ( (c4>0.99d0) .or. &
               ( (bforty).and.(c4>0.7660d0) ) ) goto 340
          af4=min( abs(c4-0.5d0),abs(c4+0.5d0) )

!         --- compute sum of diferences with 60 degrees

          som=af1+af2+af3+af4
          if ( som<afwijk ) then

!         --- possible candidate i-j

            bbest=.true.
            afwijk=som
            imog=i
            jmog=j
            ch1=c1
            ch2=c2
            ch3=c3
            ch4=c4
          end if
340     continue
        if ( j/=i2+1 ) goto 320
350   continue
      if ( .not. bbest ) then

!     --- bbest = .false.  hence no candidate is found

        if ( bforty ) then

!       --- drop requirement of 40 degrees and restart

          bforty=.false.
          goto 310
        else

!       --- no possibility at all, stop

          call errsub ( 351, 0, 0, 0 )
          call instop
        end if
      end if

!     --- create splitting line imog - jmog
!         determine length, coarseness and number elements

      xi=coor(1,nr(imog))
      yi=coor(2,nr(imog))
      xj=coor(1,nr(jmog))
      yj=coor(2,nr(jmog))
      slang=sqrt((xj-xi)**2+(yj-yi)**2)
      rimog=coarse(nr(imog))
      rjmog=coarse(nr(jmog))

!     --- determine number of elements

      if (abs(rimog/rjmog-1d0)>=1d-8) then
         a = slang/(rimog-rjmog)*log(rimog/rjmog)
      else
         a = 2*slang/(rimog+rjmog)
      end if
      if (afwijk<0.6d0) then
        n1=eround(a)
      else
        n1=max(1,nint(a))
      end if

!     --- determine growth factor alpha

      alpha=(rjmog/rimog)**(1d0/n1)

!     --- correct n1 if alpha is large

      if ( alpha>2.5d0 .or. alpha<0.4d0 ) then
        n1=n1+1
        alpha=(rjmog/rimog)**(1d0/n1)
      end if
      if ( alpha>3.3d0 .or. alpha<0.3d0 ) call errwar ( 375, 0, 0, 0 )
      if ( n1>1 ) then

!     --- n1>1 ,  hence  ( n1-1 ) new points
!         --- compute length of first element

        if (abs(alpha-1)>1d-8) then
          s=(1-alpha**n1)/(1-alpha)
        else
          s=n1*(1+0.5d0*(n1-1)*(alpha-1))
        end if
        dx=(xj-xi)/s
        dy=(yj-yi)/s

!       --- place points in right positions

        s=1d0
        ah=alpha
        do 400 i=1,n1-1
            npt=npt+1
            coor(1,npt)=xi+s*dx
            coor(2,npt)=yi+s*dy
            coarse(npt)=rimog*ah
            ah=ah*alpha
            s=alpha*s+1d0
400     continue
      end if

!     --- fill array jpoly
!         {note: 1<=imog<imog+1<jmog<=n}

      ch(imog)=ch2
      ch(jmog)=ch3
      jpoln=jpoln+1
      jpoly(jpoln)=jmog-imog+n1
      ih=imog-1

!     --- do while (ih/=jmog)

500   continue
        ih=ih+1
        if ( ih>n ) ih=ih-n
        jpoln=jpoln+1
        jpoly(jpoln)=nr(ih)
        cj(jpoln)=ch(ih)
        if (ih/=jmog) goto 500
      do 510 i=1,n1-1
        jpoln=jpoln+1
        jpoly(jpoln)=npt+1-i
        cj(jpoln)=-1
510   continue
      ch(jmog)=ch4
      ch(imog)=ch1
      jpoln=jpoln+1
      jpoly(jpoln)=imog-jmog+n+n1
      ih=jmog-1

!     --- do while (ih/=imog)

520   continue
        ih=ih+1
        if ( ih>n ) ih=ih-n
        jpoln=jpoln+1
        jpoly(jpoln)=nr(ih)
        cj(jpoln)=ch(ih)
        if (ih/=imog) goto 520
      do 530 i=1,n1-1
        jpoln=jpoln+1
        jpoly(jpoln)=npt+i-(n1-1)
        cj(jpoln)=-1
530   continue
1000  call erclos ( 'mshd03' )
      end
