      subroutine mshd06 ( kmeshc, nelem, npoint, nbndpt, inpelm, &
                          iwork, jwork, ibuurp, jbuure, l3, l4, istep )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.2  date 02-05-1999 layout
!        version  1.1  date 02-05-1990 New programming norms
!
!   copyright (c) 1982-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill arrays iwork and jwork which contain the number of neighbour
!   points respectively elements accumulated
!   and fill arrays ibuurp and jbuure which contain the nodal point
!   numbers and element numbers
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem,npoint,nbndpt,iwork(*),jwork(*),ibuurp(*),jbuure(*), &
              inpelm,kmeshc(*),l3,l4,istep

!     ibuurp     o    array with the neighbour nodal points
!     inpelm     i    number of points per element
!     istep      i    linear (=1) or quadratic (=2) case
!     iwork      o    array containing pointers for array ibuurp
!     jbuure     o    array with the adjacent element numbers
!     jwork      o    array with pointers for array jbuure
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     l3         i    declared length array ibuurp
!     l4         i    declared length array jbuure
!     nbndpt     i    Number of nodal points in boundary, length of array kbndpt
!     nelem      i    Number of elements
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,j,jstart,ih,ij,isum,jsum,i1,i2,i3,i4

!     i          Counting variable
!     i1
!     i2
!     i3
!     i4
!     ih         Help variable to store a constant temporarily
!     ij
!     isum       Integer variable to compute a sum
!     j          Counting variable
!     jstart
!     jsum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     MSHD07    fill nodal point number ij in right position of array ibuurp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     364   too much space needed for the adjacent elements
!     365   error in jstart in jbuure
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshd06' )

!     --- Clear arrays iwork and jwork

      do i = 1 , npoint

          iwork(i) = 0
          jwork(i) = 0

      end do

!     --- Fill jwork(ij) with the number of elements point ij is situated in

      do i = 1 , nelem

          ih = inpelm * (i-1)

          do j = 1 , inpelm

             ij = kmeshc(ih+j)
             jwork(ij) = jwork(ij) + 1

          end do

      end do

!     --- Copy jwork into iwork

      do i = 1 , npoint

          iwork(i) = jwork(i)

      end do

!     --- All boundary points have one neighbour extra

      do i = 1 , nbndpt , istep

          iwork(i) = iwork(i) + 1

      end do

!     --- Rearrange and accumulate

      isum = 0
      jsum = 0

      do i = 1 , npoint

          iwork(i) = iwork(i) + isum

          isum     = iwork(i)

          jwork(i) = jwork(i) + jsum

          jsum     = jwork(i)

      end do

!     --- isum positions needed for the neighbour points and
!            jsum positions for the adjacent elements

      if ( isum.gt.l3 ) then
         call errint ( isum, 1 )
         call errint ( l3, 2 )
         call errchr ( 'ibuurp', 1 )
         call errsub ( 550, 2, 0, 1 )
      end if
      if ( jsum.gt.l4 ) then
         call errint ( jsum, 1 )
         call errint ( l4, 2 )
         call errsub ( 364, 2, 0, 0 )
      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array ibuurp

      do i = 1 , isum

          ibuurp(i) = 0

      end do

!     --- Fill array ibuurp

      if ( inpelm.eq.3 ) then

           do i = 1 , nelem

               ih = 3 * (i-1)
               i1 = kmeshc(ih+1)
               i2 = kmeshc(ih+2)
               i3 = kmeshc(ih+3)

               call mshd07(ibuurp,iwork,i1,i2)
               call mshd07(ibuurp,iwork,i1,i3)
               call mshd07(ibuurp,iwork,i2,i1)
               call mshd07(ibuurp,iwork,i2,i3)
               call mshd07(ibuurp,iwork,i3,i1)
               call mshd07(ibuurp,iwork,i3,i2)

           end do

      else if ( inpelm.eq.4 ) then

           do i = 1 , nelem

               ih = 4 * (i-1)
               i1 = kmeshc(ih+1)
               i2 = kmeshc(ih+2)
               i3 = kmeshc(ih+3)
               i4 = kmeshc(ih+4)

               call mshd07(ibuurp,iwork,i1,i2)
               call mshd07(ibuurp,iwork,i1,i4)
               call mshd07(ibuurp,iwork,i2,i1)
               call mshd07(ibuurp,iwork,i2,i3)
               call mshd07(ibuurp,iwork,i3,i2)
               call mshd07(ibuurp,iwork,i3,i4)
               call mshd07(ibuurp,iwork,i4,i3)
               call mshd07(ibuurp,iwork,i4,i1)

           end do

      end if

!     --- Clear array jbuure

      do i = 1 , jsum

          jbuure(i) = 0

      end do

!     --- Fill array jbuure

      do i = 1 , nelem

         do j = 1 , inpelm

            ih   =   kmeshc( inpelm*(i-1) + j )

            jstart = 0

            if ( ih.ne.1 ) jstart = jwork(ih-1)

205         jstart = jstart + 1

            if ( jstart.gt.jwork(ih) ) then
               call errint ( jstart, 1 )
               call errint ( jwork(ih), 2 )
               call errsub ( 365, 2, 0, 0 )
            end if

            if ( jbuure(jstart).ne.0 ) goto 205

          jbuure(jstart) = i

         end do
      end do

1000  call erclos ( 'mshd06' )
      end
