      subroutine mshfillcell ( istart, iobgrid1, ncell, maxcells, &
                               icell, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array icell for one grid cell
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, iobgrid1(*), ncell, maxcells, icell(*), nnodes

!     icell         i/o   Array containing the curve parts that must be
!                         considered for computing the cross sections
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     istart         i    Starting position in iobgrid1
!     maxcells       i    Declared length of icell
!     ncell         i/o   Number of curve parts that must be used to compute
!                         cross sections
!     nnodes         i    Number of nodes in obstacle curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurpart, ipart, ind1, i, ind

!     i              Counting variable
!     ind            Sequence number of curve part
!     ind1           Sequence number of curve part
!     ipart          Loop variable
!     ncurpart       Number of curve parts in cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillcell' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillcell'
         write(irefwr,1) 'istart, ncell, maxcells', istart, ncell, maxcells
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncurpart = iobgrid1(istart)
      if ( debug ) write(irefwr,1) 'ncurpart', ncurpart

      call inichkarlen ( ncurpart, ncell+maxcells, 'icell' )
      if ( ierror/=0 ) go to 1000

      do ipart = 1, ncurpart

         ind1 = iobgrid1(istart+ipart)
         do i = 1, ncell
            if ( icell(i)==ind1 ) go to 110
         end do
         ncell = ncell+1
         icell(ncell) = ind1
110      if ( ind1==1 ) then
            ind = nnodes-1
         else
            ind = ind1-1
         end if
         do i = 1, ncell
            if ( icell(i)==ind ) go to 120
         end do
         ncell = ncell+1
         icell(ncell) = ind
120      if ( ind1==nnodes-1 ) then
            ind = 1
         else
            ind = ind1+1
         end if
         do i = 1, ncell
            if ( icell(i)==ind ) go to 200
         end do
         ncell = ncell+1
         icell(ncell) = ind
         icell(ncell+2) = ind1

200   end do  ! ipart = 1, ncurpart

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillcell'

      end if  ! ( debug )
      call erclos ( 'mshfillcell' )

      end
