      subroutine mshflkmp ( coor, kmeshm, elsize, ncurvs, lenkmsh, &
                            icurvs, curves )
! ======================================================================
!
!        programmer    Peter Slikkerveer
!        version  2.2  date 26-10-2002 Debug statements
!        version  2.1  date 29-09-1997 Correct for empty curves
!        version  2.0  date 14-09-1997 Extra parameters icurvs, curves
!        version  1.0  date 11-08-1996
!
!   copyright (c) 1996-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill array elsize with the sizes of the elements at the beginning
!   and the end of the curves
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, kmeshm(*), lenkmsh, icurvs(*)
      double precision coor(2,*), elsize(2,ncurvs), curves(2,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     curves     i    array containing the coordinates of the curves
!     elsize     o    Contains kmesh part p i.e.
!                     elsize(1,i) element length of first element of curve i
!                     elsize(2,i) element length of last element of curve i
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     lenkmsh    i    Length of array kmeshh
!     ncurvs     i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istar3, icurnr, inodp1, inodp2
      double precision dx

!     dx             length of vector
!     icurnr         Curve sequence number
!     inodp1         First node in part
!     inodp2         Second node in part
!     istar3         Starting address of KMESHM part 3 in KMESH part M
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all curves do
!        inodp1 := first node in curve
!        inodp2 := second node in curve
!        dx := distance between inodp1 and inodp2
!        inodp1 := last node in curve
!        inodp2 := last node before last node in curve
!        dx := distance between inodp1 and inodp2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshflkmp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshflkmp'
         write(irefwr,*) 'lenkmsh, ncurvs ', lenkmsh, ncurvs

      end if
      if ( lenkmsh==0 ) then

!     --- lenkmsh = 0, icurvs and curves have not been filled

         istar3 = kmeshm(2) - 1
         do icurnr = 1, ncurvs

            if ( kmeshm(5+icurnr)-kmeshm(4+icurnr)>0 ) then

!           --- Curve contains nodes

               inodp1 = kmeshm(istar3 + kmeshm(4+icurnr)     )
               inodp2 = kmeshm(istar3 + kmeshm(4+icurnr) + 1  )
               if ( inodp1>0 .and. inodp2.gt.0 ) then
                  dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                              (coor(2,inodp1) - coor(2,inodp2))**2 )
                  elsize(1,icurnr) = dx
               else
                  elsize(1,icurnr) = 1d0
               end if
               inodp1 = kmeshm(istar3 + kmeshm(5+icurnr) - 1  )
               inodp2 = kmeshm(istar3 + kmeshm(5+icurnr) - 2  )
               if ( inodp1>0 .and. inodp2.gt.0 ) then
                  dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                              (coor(2,inodp1) - coor(2,inodp2))**2 )
                  elsize(2,icurnr) = dx
               else
                  elsize(2,icurnr) = 1d0
               end if

            else

               elsize(1,icurnr) = 1d0
               elsize(2,icurnr) = 1d0

            end if

         end do

      else

!     --- lenkmsh > 0, icurvs and curves have been filled

         istar3 = 1
         do icurnr = 1, ncurvs

            if ( debug ) &
               write(irefwr,*) 'icurnr, icurvs(icurnr), istar3 ', &
                                icurnr, icurvs(icurnr), istar3
            if ( icurvs(icurnr)>0 ) then
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'curves(istar3) ', &
                                   curves(1,istar3),curves(2,istar3)
                  write(irefwr,*) 'curves(istar3+1) ', &
                                   curves(1,istar3+1),curves(2,istar3+1)

               end if

               dx = sqrt ( (curves(1,istar3+1) - curves(1,istar3))**2 + &
                           (curves(2,istar3+1) - curves(2,istar3))**2 )
               if ( debug ) write(irefwr,*) 'dx ', dx
               elsize(1,icurnr) = dx
               istar3 = icurvs(icurnr)
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'istar3 ', istar3
                  write(irefwr,*) 'curves(istar3) ', &
                                   curves(1,istar3),curves(2,istar3)
                  write(irefwr,*) 'curves(istar3+1) ', &
                                   curves(1,istar3-1),curves(2,istar3-1)

               end if

               dx = sqrt ( (curves(1,istar3) - curves(1,istar3-1))**2 + &
                           (curves(2,istar3) - curves(2,istar3-1))**2 )
               if ( debug ) write(irefwr,*) 'dx ', dx
               elsize(2,icurnr) = dx
               istar3 = istar3+1

            else

               elsize(1,icurnr) = 1d0
               elsize(2,icurnr) = 1d0

            end if

         end do

      end if

      call erclos ( 'mshflkmp' )

      end
