      subroutine mshg01 ( x1, y1, x2, y2, x3, y3, cs )
! ======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.0    date   17-11-1995 Sander de Snoo: new faster formula
!        version 1.0    date   26-03-1984
!
!
!
!   copyright (c) 1984-1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!   Computes the cosine of the angle 1-2-3 using the inner product
!   cos(angle) = <a,b>/|a||b|
! ********************************************************************
!
!                      KEYWORDS
!
!     cosine
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      double precision x1, x2, y1, y2, x3, y3, cs

!     cs      o    cosine
!     x1      i    x-coordinate of first point
!     x2      i    x-coordinate of second point
!     x3      i    x-coordinate of third point
!     y1      i    x-coordinate of first point
!     y2      i    x-coordinate of second point
!     y3      i    x-coordinate of third point
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      double precision inprod, ab2

!     ab2     Length of vector
!     inprod  Inner product of vectors
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
!     INSTOP  Stop the program
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     351   mesh generator general terminated the execution of the program
! ********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
      inprod = (x2-x1)*(x2-x3)+(y2-y1)*(y2-y3)
      ab2 = ((x2-x1)**2+(y2-y1)**2)*((x2-x3)**2+(y2-y3)**2)
      if (ab2.gt.0) then
        cs = inprod/sqrt(ab2)
      else
        call eropen ( 'mshg01' )
        call errsub ( 351, 0, 0, 0 )
        call erclos ( 'mshg01' )
        call instop
      end if

      end
