      subroutine mshg23( ski, eta, x1, x2, x3, ndim, xn )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 02-11-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute coordinates of internal point ski,eta in case of a
!     isoparametric quadratic triangular transformation
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     isoparametric
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision ski, eta, x1(6), x2(6), x3(6), xn(3)
      integer ndim

!     eta       i     second coordinate local system
!     ndim      i     dimension (2D or 3D)
!     ski       i     first coordinate local system
!     x1        i     first coordinates of six basis points
!     x2        i     second coordinates of six basis points
!     x3        i     third coordinates of six basis points
!     xn        o     coordinates of transformed point (ski,eta)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision phi(6)

!     i          loop variable
!     phi        array to store values of the basis functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from string
!     EROPEN  Concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all basis functions and determine the value for
!     point (ski,eta)
!     Compute the coordinates of point (ski,eta)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshg23' )

!     --- Run through all basis functions and determine value

      phi(1) = (1d0-ski-eta) * (1d0-2d0*ski-2d0*eta)
      phi(2) = 4d0 * (1d0-ski-eta) * ski
      phi(3) = ski * (2d0*ski-1d0)
      phi(4) = 4d0 * ski * eta
      phi(5) = eta * (2d0*eta-1d0)
      phi(6) = 4d0 * (1d0-ski-eta) * eta

!     --- Compute the coordinates of the point (ski,eta) in the original
!         setting

      xn(1) = 0d0
      xn(2) = 0d0
      xn(3) = 0d0

      do 100 i = 1 , 6

               xn(1) = xn(1) + x1(i) * phi(i)
               xn(2) = xn(2) + x2(i) * phi(i)

          if ( ndim.eq.3 ) xn(3) = xn(3) + x3(i) * phi(i)

100   continue

      call erclos ( 'mshg23' )

      end
