      subroutine mshnewinp2 ( coorusr, infcur, infsur, &
                              ccoarse, factors, ctangent, &
                              iinput, rinput, iinputnew, &
                              rinputnew, lenpnt, lensurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 30-07-2012 New treatment of iinput/rinput
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 16-07-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend parts of local subarrays of iinput and rinput with extra
!     information
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputnew(*), lenpnt, &
              lensurfs, infcur(maxlencur,lencrv), &
              infsur(maxlensur,lensurfs), iinput(*)
      double precision coorusr(ndim,lenpnt), ccoarse(lenpnt), &
                       factors(lencrv), ctangent(2,lencrv), rinput(*), &
                       rinputnew(*)

!     ccoarse        i    Array containing the coarseness of the user points
!     coorusr        i    Array containing the co-ordinates of the user points
!                         x_i = coorusr(1,i);  y_i = coorusr(2,i)
!     ctangent       i    Array containing the factors t_0 and t_1 of the curves
!                         t_0_i = ctangent(1,i);  t_1_i = ctangent(2,i)
!     factors        i    Array containing the factors of the curves
!     iinput         i    integer input array
!     iinputnew      o    new integer input array
!     infcur         i    Array containing the information of the curves
!                         infcur(1,i) refers to curve i
!     infsur         i    Array containing the information of the surfaces
!                         infsur(1,i) refers to surface i
!     lenpnt         i    Number of entries in local point type arrays
!     lensurfs       i    Number of entries in local surface type arrays
!     rinput         i    Real input array
!     rinputnew      i    New real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurve, j, nshift, ncurvsnew, irnp, irnpold, &
              nsurfsnew, nuspntnew, iinp, jlast, &
              ivolm, iinpprev, istart, isurnrnew, ndefvl
      logical part4

!     i              Counting variable
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     iinpprev       Previous value of iinp
!     irnp           position in array rinput
!     irnpold        position in old array rinput
!     istart         First position used at a part
!     isurnrnew      New surface sequence number
!     ivolm          Type of volume with respect to mesh generator
!     j              Counting variable
!     jlast          iinput(20) refers to new part in iinput
!     ncurvsnew      Last curve sequence number used
!     ndefvl         Number of surfaces in volume
!     nshift         shift
!     nsurfsnew      Last surface sequence number used
!     nuspntnew      Last user point sequence number used
!     part4          If true rinput part 4 is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH046         Raise iinp as function of icurve
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     MSH050         Raise iinp as function of ivolm
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Copy first part of iinput
!    Copy part concerning curves
!    Copy part concerning surfaces
!    Copy last part of iinput
!    Copy first part of rinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnewinp2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnewinp2'
         write(irefwr,1) 'iinp8, irnp3', iinp8, irnp3
         call prinin ( iinput, iinp8, 'iinput' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nuspntnew = nuspnt
      ncurvsnew = ncurvs
      nsurfsnew = nsurfs
      nuspnt = iinput(2)
      ncurvs = iinput(3)
      nsurfs = iinput(4)
      nvolms = iinput(5)
      if ( debug ) then
         write(irefwr,1) 'nuspntnew, ncurvsnew, nsurfsnew', &
                          nuspntnew, ncurvsnew, nsurfsnew
         write(irefwr,1) 'nuspnt, ncurvs,nsurfs, nvolms', &
                          nuspnt, ncurvs,nsurfs, nvolms
         write(irefwr,1) 'iinp1, maxlenvol', iinp1, maxlenvol
      end if  ! ( debug )

!     --- Copy first part of iinput

      iinputnew(1:iinp1-1) = iinput(1:iinp1-1)
      if ( debug ) call prinin ( iinputnew, iinp1-1, 'iinputnew/1' )

!     --- Copy part concerning curves
!         First old part

      iinputnew(iinp1:iinp3-1) = iinput(iinp1:iinp3-1)
      if ( debug ) &
         call prinin ( iinputnew(iinp1), iinp3-iinp1, 'iinputnew/2' )

!     --- Next new part

      iinp = iinp3
      do icurnr = ncurvs+1, ncurvsnew
         iinputnew(iinp:iinp+maxlencur-1) = infcur(1:maxlencur,icurnr)
         icurve = iinputnew(iinp)
         call msh046 ( iinputnew, iinp, icurve )
      end do
      if ( debug ) &
         call prinin ( iinputnew(iinp3), iinp-iinp3, 'iinputnew/3' )
      iinp3 = iinp

!     --- Copy part concerning surfaces

      do isurnr = 1, nsurfsnew
         iinputnew(iinp:iinp+maxlensur-1) = infsur(1:maxlensur,isurnr)
         isurf = iinputnew(iinp)
         call msh049 ( iinputnew, iinp, isurf )
      end do
      nshift = iinp-iinp5
      if ( debug ) then
         istart = iinp3
         call prinin ( iinputnew(istart), iinp-istart, 'iinputnew/4' )
      end if  ! ( debug )

!     --- Check if paver is used

      part4 = .false.
      do isurnr = 1, nsurfsnew
         if ( infsur(1,isurnr)==19 ) part4 = .true.
      end do

!     --- Copy part concerning volumes
!         Replace spheres by new surface

      istart = iinp5
      iinp5 = iinp
      do ivolnr = 1, nvolms

         iinputnew(iinp:iinp+maxlenvol-1) = &
            iinput(istart:istart+maxlenvol-1)
         ndefvl = iinputnew(iinp+2)
         do i = 1, ndefvl

            isurnr = iinputnew(iinp+2+i)

            if ( infsur(1,isurnr)==20 ) then

!           --- Surface is of type sphere

               isurnrnew = infsur(7,isurnr)
               iinputnew(iinp+2+i) = isurnrnew

            end if

         end do
         ivolm = iinputnew(iinp)
         iinpprev = iinp
         call msh050 ( iinputnew, iinp, ivolm )
         istart = istart+iinp-iinpprev

      end do
      if ( debug ) &
         call prinin ( iinputnew(iinp5), istart-iinp5, 'iinputnew/5' )

!     --- Copy last part of iinput

      iinputnew(istart+nshift:iinp8+nshift) = &
         iinput(istart:iinp8)
      iinp = iinp8+nshift
      if ( iinputnew(9)<0 ) iinputnew(9) = iinputnew(9)-nshift
      iinputnew(2) = nuspntnew
      iinputnew(3) = ncurvsnew
      iinputnew(4) = nsurfsnew
      if ( iinputnew(14)>iinp1 ) iinputnew(14) = iinputnew(14)+nshift
      if ( iinputnew(15)>iinp1 ) then
         iinputnew(15) = iinputnew(15)+nshift
         jlast = iinputnew(15)-5
         do i = 4, 7
            if ( iinputnew(jlast+i)>0 ) &
               iinputnew(jlast+i) = iinputnew(jlast+i)+nshift
         end do
      end if
      if ( iinp6>0 ) iinp6 = iinp6+nshift
      if ( iinp7>0 ) iinp7 = iinp7+nshift
      if ( iinp8>0 ) iinp8 = iinp8+nshift
      if ( iinp9>0 ) iinp9 = iinp9+nshift
      if ( iinp10>0 ) iinp10 = iinp10+nshift
      if ( iinp11>0 ) iinp11 = iinp11+nshift
      if ( iinp12>0 ) iinp12 = iinp12+nshift

!     --- Copy first part of rinput

      rinputnew(1:2) = rinput(1:2)

!     --- Fill co-ordinates of user points

      do i = 1, nuspntnew
         do j = 1, ndim
            rinputnew(3+ndim*(i-1)+j-1) = coorusr(j,i)
         end do
      end do

      irnp = 3+ndim*nuspntnew
      irnpold = 3+ndim*nuspnt

      if ( debug ) then
         write(irefwr,1) 'ndim, nuspnt, nuspntnew, irnp, irnpold', &
                          ndim, nuspnt, nuspntnew, irnp, irnpold
         call prinrl1 ( rinputnew(3), nuspntnew, ndim, 'rinputnew' )
         call prinrl ( ccoarse, nuspntnew, 'ccoarse' )
      end if  ! ( debug )

!     --- Fill information about coarseness if jcoars=1

      if ( jcoars>0 ) then
         rinputnew(irnp) = rinput(irnpold)
         rinputnew(irnp+1:irnp+nuspntnew) = ccoarse(1:nuspntnew)
         irnp = irnp+nuspntnew+1
         irnpold = irnpold+nuspnt+1
      end if
      irnp2 = irnp

      if ( debug ) then
         write(irefwr,1) 'irnp, irnpold', irnp, irnpold
         write(irefwr,2) 'unit length', rinputnew(irnp-nuspntnew-1)
         call prinrl ( rinputnew(irnp-nuspntnew), nuspntnew, 'coarse' )
      end if  ! ( debug )

!     --- Fill information about factors

      rinputnew(irnp:irnp+ncurvsnew-1) = factors(1:ncurvsnew)
      irnp = irnp+ncurvsnew
      irnpold = irnpold+ncurvs

!     --- Fill information about tangents

      do i = 1, ncurvsnew
         rinputnew(irnp+2*i-2) = ctangent(1,i)
         rinputnew(irnp+2*i-1) = ctangent(2,i)
      end do
      irnp = irnp + 2*ncurvsnew-1
      irnpold = irnpold + 2*ncurvs-1

!     --- Copy last part of rinputold

      do i = 1, 4*nsurfs
         rinputnew(irnp+i) = rinput(irnpold+i)
      end do
      irnp = irnp+4*nsurfs
      irnpold = irnpold+4*nsurfs
      do i = 1, 4*(nsurfsnew-nsurfs)
         rinputnew(irnp+i) = 0d0
      end do
      irnp = irnp+4*(nsurfsnew-nsurfs)
      if ( part4 ) then

!     --- mesh generator paver is used

         do i = 1, 4*ncurvs
            rinputnew(irnp+i) = rinput(irnpold+i)
         end do
         irnp4 = irnp
         irnp = irnp+4*ncurvs
         do i = 1, 4*(ncurvsnew-ncurvs)
            rinputnew(irnp+i) = 0d0
         end do
         irnp = irnp+4*(ncurvsnew-ncurvs)
         irnpold = irnpold+4*ncurvs

      end if
      nshift = irnp3-irnpold
      if ( nshift>0 ) then
         do i = 1, nshift
            rinputnew(irnp+i) = rinput(irnp3-nshift+i)
         end do
         irnp = irnp+nshift
      end if
      irnp3 = irnp
      nuspnt = nuspntnew
      ncurvs = ncurvsnew
      nsurfs = nsurfsnew

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp8, irnp3', &
                          iinp8, irnp3
         call prinin ( iinputnew, iinp8, 'iinputnew/end' )
         call prinrl ( rinputnew, irnp3, 'rinputnew/end' )
         write(irefwr,*) 'End mshnewinp2'

      end if  ! ( debug )
      call erclos ( 'mshnewinp2' )

      end
