      subroutine msho53( bcord, coor, u, x, y, z, n )

! =======================================================================
!
!
!         programmer   niek praagman
!
!         version 2.0  date  09-03-1993 adjustment for coor
!         version 1.0  date  28-03-1990
!
!
!
!   copyright (c) 1990-1993  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Subroutine to compute (x,y,z) interpolation for value u along line
!     given by a 2d array COOR
!
! ***********************************************************************
!
!     KEYWORDS
!
!     interpolation
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS

      implicit none
      double precision bcord(3,*), coor(2,*), u, x, y, z
      integer n
!
!     bcord   i     3-D coordinate array
!     coor    i     2-D coordinate array
!     n       i     number of points on line
!     u       i     arclength along line
!     x       o     x-coordinate along line for arclength u
!     y       o     y-coordinate along line for arclength u
!     z       o     z-coordinate along line for arclength u
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!

      double precision ule, s, t, dx, dy, xp(2), yp(2), zp(2)
      integer i

!     dx             difference in first coordinate
!     dy             difference in second coordinate
!     i              loop variable
!     s              arclength
!     t              ratio parameter
!     ule            arclength
!     xp             global x-coordinates start and end point
!     yp             global y
!     zp             global z
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Run through all points of line as given by coor
!
!     While total arclength not enough
!      Determine arclength for each new point
!
!      Determine new coordinates in 3d
!
! =======================================================================

!     determine coordinates along line for arclength u

      ule = 0

      do 100 i = 2 , n

          s    = ule

          dx   = coor(1,i) - coor(1,i-1)
          dy   = coor(2,i) - coor(2,i-1)

          ule  = ule + sqrt( dx*dx + dy*dy )

          if ( ule.gt.u ) then

!              interpolate

               if ( u.lt.1.d-4) then

                    t = 0

               else

                    t = ( u - s ) / ( ule - s )

               end if

               xp(1) = bcord(1,i-1)
               yp(1) = bcord(2,i-1)
               zp(1) = bcord(3,i-1)

               xp(2) = bcord(1,i  )
               yp(2) = bcord(2,i  )
               zp(2) = bcord(3,i  )

               x  = xp(1) + t * ( xp(2) - xp(1) )
               y  = yp(1) + t * ( yp(2) - yp(1) )
               z  = zp(1) + t * ( zp(2) - zp(1) )

!              Ready, leave loop

               goto 200

          end if

100   continue

!     End of routine

200   continue

      end
