      subroutine msho75 ( xi, yi, xj, yj, x1, y1, x2, y2, ih )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 28-11-2013 Replaced by call to mshcrossline
!        version  2.0  date 06-11-2007 Extra checks added, "new" accuracy
!        version  1.0  date 27-06-1996
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to check whether line i-j and line 1-2 have a common
!     point.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ih
      double precision x1, x2, xi, xj, y1, y2, yi, yj

!     ih      o    indicator of result
!                     ih = 0 : there is a common point
!                     ih = 1 : there is no common point
!     x1      i    x-coordinate of point 1
!     x2      i    x-coordinate of point 2
!     xi      i    x-coordinate of point i
!     xj      i    x-coordinate of point j
!     y1      i    x-coordinate of point 1
!     y2      i    x-coordinate of point 2
!     yi      i    x-coordinate of point i
!     yj      i    x-coordinate of point j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xa(2), xb(2), xc(2), xd(2), fact1, fact2, eps

!     eps            small number
!     fact1          if between 0 and 1 the intesection is between x1 and x2
!     fact2          if between 0 and 1 the intesection is between xi and xj
!     xa             Contains the coordinates of x1
!     xb             Contains the coordinates of x2
!     xc             Contains the coordinates of xi
!     xd             Contains the coordinates of xj
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE   Computes the intersection of two lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msho75' )
      debug = .false. .and. ioutp>=0
      eps    = 10 * epsmac
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msho75'
         write(irefwr,2) 'xi, yi, xj, yj', xi, yi, xj, yj
         write(irefwr,2) 'x1, y1, x2, y2', x1, y1, x2, y2
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d14.6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xa(1) = x1
      xa(2) = y1
      xb(1) = x2
      xb(2) = y2
      xc(1) = xi
      xc(2) = yi
      xd(1) = xj
      xd(2) = yj
      call mshcrossline ( xa, xb, xc, xd, fact1, fact2, eps )

      if ( fact1>=0d0 .and. fact1<=1d0 .and. &
           fact2>=0d0 .and. fact2<=1d0 ) then
         ih = 0
      else
         ih = 1
      end if  ! ( fact1>=0d0 .and. fact1<=1d0 ... )

1000  call erclos( 'msho75' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ih', ih
         write(irefwr,2) 'fact1, fact2', fact1, fact2
         write(irefwr,*) 'End msho75'

      end if  ! ( debug )

      end
