      subroutine msho78 ( ndim, coor, npoint, rinput )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 15-09-1996 Compute boundary points
!                                      in a different way
!        version  1.0  date 27-06-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinate array in case of an unstructured mesh without a
!     given boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     unstructured
!     coordinate
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint
      double precision coor(ndim, npoint), rinput(ndim, npoint-4)

!     coor    o    array with all coordinates of the nodes
!     ndim    i    dimension of Euclidian space considered
!     npoint  i    total number of nodes in array coor
!     rinput  i    coordinates of unstructured nodes as given by user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i , ishift, j
      double precision xmin, xmax, ymin, ymax, zmin, zmax, dx, dy, dz

!     dx      max distance x-direction
!     dy      max distance y-direction
!     dz      max distance z-direction
!     i       loop variable
!     ishift  helpvariable to store shift value
!     j       loop variable
!     xmax    max x-value in user supplied coordinates
!     xmin    min x-value in user supplied coordinates
!     ymax    max y-value in user supplied coordinates
!     ymin    min y-value in user supplied coordinates
!     zmax    max z-value in user supplied coordinates
!     zmin    min z-value in user supplied coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Place all user supplied points in coor
!     Run through all nodes and compute extremal values
!     Place four extra (help)nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First place all user supplied points

      if ( ndim.eq.2 ) then
         ishift = 4
      else if ( ndim.eq.3 ) then
         ishift = 8
      end if
      do i = 1 , ndim
         do j = 1 , npoint - ishift
            coor( i, j + ishift ) = rinput( i, j )
         end do
      end do

!     --- Run through all nodes and compute the max and min coordinate
!         values

      xmin =   rinfin
      xmax =  -rinfin
      ymin =   rinfin
      ymax =  -rinfin
      zmin =   rinfin
      zmax =  -rinfin

      if ( ndim.eq.2 ) then

         do 300 i = 1+ishift , npoint

            if ( coor(1,i).lt.xmin ) xmin = coor(1,i)
            if ( coor(1,i).gt.xmax ) xmax = coor(1,i)
            if ( coor(2,i).lt.ymin ) ymin = coor(2,i)
            if ( coor(2,i).gt.ymax ) ymax = coor(2,i)

300      continue

         dx = xmax - xmin
         dy = ymax - ymin

         coor(1,1) = xmin - 0.1d0 * dx
         coor(2,1) = ymin - 0.1d0 * dy

         coor(1,2) = xmax + 0.1d0 * dx
         coor(2,2) = ymin - 0.1d0 * dy

         coor(1,3) = xmax + 0.1d0 * dx
         coor(2,3) = ymax + 0.1d0 * dy

         coor(1,4) = xmin - 0.1d0 * dx
         coor(2,4) = ymax + 0.1d0 * dy

      else if ( ndim.eq.3 ) then

         do 400 i = 1+ishift , npoint

            if ( coor(1,i).lt.xmin ) xmin = coor(1,i)
            if ( coor(1,i).gt.xmax ) xmax = coor(1,i)
            if ( coor(2,i).lt.ymin ) ymin = coor(2,i)
            if ( coor(2,i).gt.ymax ) ymax = coor(2,i)
            if ( coor(3,i).lt.zmin ) zmin = coor(3,i)
            if ( coor(3,i).gt.zmax ) zmax = coor(3,i)

400      continue

         dx = xmax - xmin
         dy = ymax - ymin
         dz = zmax - zmin

         coor(1,1) = xmin - 0.2d0 * dx
         coor(2,1) = ymin - 0.3d0 * dy
         coor(3,1) = zmin - 0.2d0 * dz

         coor(1,2) = xmax + 0.2d0 * dx
         coor(2,2) = ymin - 0.3d0 * dy
         coor(3,2) = zmin - 0.2d0 * dz

         coor(1,3) = xmax + 0.2d0 * dx
         coor(2,3) = ymax + 0.3d0 * dy
         coor(3,3) = zmin - 0.2d0 * dz

         coor(1,4) = xmin - 0.2d0 * dx
         coor(2,4) = ymax + 0.3d0 * dy
         coor(3,4) = zmin - 0.2d0 * dz

         coor(1,5) = xmin - 0.2d0 * dx
         coor(2,5) = ymin - 0.3d0 * dy
         coor(3,5) = zmax + 0.2d0 * dz

         coor(1,6) = xmax + 0.2d0 * dx
         coor(2,6) = ymin - 0.3d0 * dy
         coor(3,6) = zmax + 0.2d0 * dz

         coor(1,7) = xmax + 0.2d0 * dx
         coor(2,7) = ymax + 0.3d0 * dy
         coor(3,7) = zmax + 0.2d0 * dz

         coor(1,8) = xmin - 0.2d0 * dx
         coor(2,8) = ymax + 0.3d0 * dy
         coor(3,8) = zmax + 0.2d0 * dz

      end if

      end
