      subroutine msho79 ( coor, npoint, kelem, nelem, nbound )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.0    date   27-06-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove the extra coordinates and elements in case of an unstructured
!     mesh with artificial constructed boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     unstructured
!     coordinate
!     2d
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer npoint, nelem, nbound, kelem(3,nelem)
      double precision coor(2,npoint)

!     coor   i,o   array with all coordinates of the nodes
!     kelem  i,o   element array
!     nbound  o    number of boundary pieces in kbound
!     nelem  i,o   total number of elements
!     npoint i,o   total number of nodes in array coor
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i , ielem, jelem, ibound, i1, i2, i3

!     i       loop variable
!     i1      node number
!     i2      node number
!     i3      node number
!     ibound  new number of boundary pieces
!     ielem   loop variable for elements
!     jelem   new number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Rearrange coor, i.e. cancel the help points
!
!     Rearrange kelem
!
!     Determine boundary pieces
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Rearrange coor : (cancel the "help" points)

      do 100 i = 1 , npoint - 4

         coor( 1, i ) = coor( 1, i+4 )
         coor( 2, i ) = coor( 2, i+4 )

100   continue

!     --- Rearrange kelem

      jelem  = 0
      ibound = 0

      do 200 ielem = 1 , nelem

         i1 = kelem( 1, ielem )
         i2 = kelem( 2, ielem )
         i3 = kelem( 3, ielem )

         if ( i1.lt.5 .or. i2.lt.5 .or. i3.lt.5 ) then

!        --- Cancel element

            if ( i1.gt.4 .and. i2.gt.4 ) then

                 ibound = ibound + 1

            else if ( i2.gt.4 .and. i3.gt.4 ) then

                 ibound = ibound + 1

            else if ( i3.gt.4 .and. i1.gt.4 ) then

                 ibound = ibound + 1

            end if

         else

!        --- Element is ok, store in new array

            jelem = jelem + 1

            kelem( 1, jelem ) = i1 - 4
            kelem( 2, jelem ) = i2 - 4
            kelem( 3, jelem ) = i3 - 4

         end if

200   continue

      npoint = npoint - 4
      nelem  = jelem
      nbound = ibound

      end
