      subroutine msho83( nx, ny, nz, kmeshm )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.0    date   28-06-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part m in case of a structured mesh in 3D where nx, ny
!     and nz are the number of points in the coordinate-directions
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nx, ny, nz, kmeshm(*)

!     kmeshm  o    part M of array KMESH see SEPRAN PG
!     nx      i    number of nodal points in x-direction
!     ny      i    number of nodal points in y-direction
!     nz      i    number of nodal points in z-direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ikn(12)

!     i       loop variable
!     ikn     helparray for starting points of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill all positions as indicated in the PG
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill the array positions as indicated in the Programmers Guide

      kmeshm(  1 ) = 0
      kmeshm(  2 ) = 18
      kmeshm(  3 ) = 18 + 4 * ( nx + ny + nz )
      kmeshm(  4 ) = 0
      kmeshm(  5 ) = 1
      kmeshm(  6 ) = kmeshm(  5 ) + nx
      kmeshm(  7 ) = kmeshm(  6 ) + ny
      kmeshm(  8 ) = kmeshm(  7 ) + nx
      kmeshm(  9 ) = kmeshm(  8 ) + ny
      kmeshm( 10 ) = kmeshm(  9 ) + nz
      kmeshm( 11 ) = kmeshm( 10 ) + nz
      kmeshm( 12 ) = kmeshm( 11 ) + nz
      kmeshm( 13 ) = kmeshm( 12 ) + nz
      kmeshm( 14 ) = kmeshm( 13 ) + nx
      kmeshm( 15 ) = kmeshm( 14 ) + ny
      kmeshm( 16 ) = kmeshm( 15 ) + nx
      kmeshm( 17 ) = kmeshm( 16 ) + ny

      do 100 i = 1 , 12

         ikn( i ) = 16 + kmeshm( 4 + i )

100   continue

!     --- Run along the x-axis directions (curves 1, 3, 9, 11)

      do 200 i = 1 , nx

          kmeshm( ikn(  1 ) + i ) = i
          kmeshm( ikn(  3 ) + i ) = nx * ny + 1 - i
          kmeshm( ikn(  9 ) + i ) = ( nx * ny ) * ( nz - 1 ) + i
          kmeshm( ikn( 11 ) + i ) = nx * ny * nz + 1 - i

200   continue

!     --- Run along the y-axis directions (curves 2, 4, 10, 12)

      do 300 i = 1 , ny

          kmeshm( ikn(  2 ) + i ) = i * nx
          kmeshm( ikn(  4 ) + i ) = nx * ( ny - 1 ) + 1 - ( i-1 ) * nx
          kmeshm( ikn( 10 ) + i ) = nx * ny * ( nz-1 ) + i * nx
          kmeshm( ikn( 12 ) + i ) = nx * ny * nz - nx + 1 - ( i-1 ) * nx

300   continue

!     --- Run along the z-axis directions

      do 400 i = 1 , nz

          kmeshm( ikn(  5 ) + i ) = 1 + ( nx * ny ) * ( i - 1 )
          kmeshm( ikn(  6 ) + i ) = nx + ( nx * ny ) * ( i - 1 )
          kmeshm( ikn(  7 ) + i ) = nx * ny + ( nx * ny ) * ( i - 1 )
          kmeshm( ikn(  8 ) + i ) = nx * ( ny-1 ) + 1 + &
                                         ( nx * ny ) * ( i - 1 )

400   continue

!     --- All boundary nodenumbers have been placed

      end
