      subroutine mshobsfl1 ( nx, ny, nz, iobgrid, i, j, k, ielem, &
                             leniobgrid, leniobgridact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 06-11-2012 Extra debug
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid,leniobgridact
!        version  1.0  date 15-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the combination of cell i,j,k and surface element ielem into
!     iobgrid if it has not already been stored
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, leniobgrid, i, j, k, leniobgridact, &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,leniobgrid), ielem

!     i              i    i-index in iobgrid
!     ielem          i    Element number
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     j              i    j-index in iobgrid
!     k              i    k-index in iobgrid
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     leniobgridact i/o   Actual length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer l

!     l              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2253   Size of iobgrid too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all available entries in iobgrid do
!        if ( pos(i,j,k) = 0 ) then
!           Fill ielem in obgrid and return
!        if ( pos(i,j,k) = ielem ) then
!           return
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfl1'
         write(irefwr,1) 'leniobgrid, i, j, k', leniobgrid, i, j, k
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do l = 1, leniobgrid

         leniobgridact = max(leniobgridact,l)
         if ( iobgrid(i,j,k,l).eq.0 ) then

!        --- New entry in iobgrid found

            iobgrid(i,j,k,l) = ielem
            go to 1000

         else if ( iobgrid(i,j,k,l).eq.ielem ) then

!        --- Entry already in iobgrid

            go to 1000

         end if

      end do

!     --- More than  entries necessary: error

      call errint ( leniobgrid+1, 1 )
      call errint ( leniobgrid, 2 )
      call errsub ( 2253, 2, 0, 0 )
      write(irefwr,100) iobgrid(i,j,k,1:leniobgrid)
100   format ( 'Elements in iobgrid'/(10i6))

1000  call erclos ( 'mshobsfl1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'leniobgridact', leniobgridact
         write(irefwr,*) 'End mshobsfl1'

      end if  ! ( debug )

      end

