      subroutine mshobsflcurv ( iobgrid, nx, ny, i, j, iobgrid1, &
                                icurbef, nparts, istorecurves, &
                                maxparts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store part of boundary in column into array istorecurves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), i, j, iobgrid1(*), &
              icurbef, nparts, maxparts, istorecurves(maxparts)

!     i              i    Counting variable
!     icurbef        i    Previous curve part used
!     iobgrid        i    Contains for each cell i,j either 0 or a starting
!                         address in iobgrid1 referring to the curve parts
!                         crossing this cell
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     istorecurves  i/o   Contains the curve parts for one column in
!                         the grid of cells
!     j              i    Counting variable
!     maxparts       i    Declared length of array istorecurves
!     nparts         i    number of parts the obstacle consists of
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, istart, numedges, icurpart, k

!     icurpart       Sequence number of curve part
!     iedge          Loop number referring to curve parts stored in cell
!     istart         First position used at a part
!     k              Counting variable
!     numedges       number of curve parts stored in cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsflcurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsflcurv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = iobgrid(i,j)
      numedges = iobgrid1(istart)

      if ( debug ) &
         write(irefwr,1) 'i, j, istart, numedges', &
                          i, j, istart, numedges

      do iedge = 1, numedges

         icurpart = iobgrid1(istart+iedge)

         if ( debug ) &
            write(irefwr,1) 'iedge, icurpart, icurbef/1', &
                             iedge, icurpart, icurbef

        if ( icurpart/=icurbef ) then

!       --- new boundary part part, raise nparts and
!           store in istorecurves

           do k = 1, nparts-1
              if ( icurpart==istorecurves(k) ) go to 100
           end do
           nparts = nparts+1

           call inichkarlen ( nparts, maxparts, 'istorecurves' )
           if ( ierror/=0 ) go to 1000

           icurbef = icurpart
           istorecurves(nparts) = icurpart

100     end if

      end do  ! iedge = 1, numedges

1000  call erclos ( 'mshobsflcurv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsflcurv'

      end if  ! ( debug )

      end
