      subroutine mshobstcopmsh ( meshorig, meshobst, maploc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete some parts of new mesh and copy part l from kmesh2d
!     Part of prmshlevms cooresponding to new and old mesh coinciding
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshorig, meshobst
      type (mapping) :: maploc

!     maploc        i/o   contains information of the mapping
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqkm1, iseqkm2

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOPY        Copy important information of mesh
!     MSHCOPYKMESHAD Copy kmesh part ad from one mesh to another
!     MSHDELKMESHAD  Delete kmesh part ad
!     MSHOBSTMARKELM Recompute array ielemobst in the sense that only elements
!                    inside or outside the obstacle are marked
!     MSHOBSTMS08    Delete some parts of kmeshnew and copy parts l and m
!                    from kmesh
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPINITKMESHAD Initialize kmesh ad
!     SEPRMIBUF      delete temporary array from ibuffr
!     SEPRMNMESH     Remove subarrays of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstcopmsh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstcopmsh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = kmesh2d(meshorig)

!     --- Delete some parts of kmesh2dnew and copy parts l and m from kmesh2d

      call mshobstms08 ( kmesh2d(meshobst) )  ! delete

!     --- Copy essential parts of original mesh into new one

      call mshcopy ( kmesh2d(meshorig), kmesh2d(meshobst) )    ! copy
      kmesh => kmesh2d(meshobst)
      call sepfillkmesh( kmesh )
      iseqkm2 = iseqkm
      if ( km(iseqkm2)%kmesh(48)>0 ) then
         call seprmnmesh
         km(iseqkm2)%kmesh(48) = 0  ! kmeshaa
      end if  ! ( km(iseqkm2)%kmesh(48)>0 )

      call sepfillkmeshaa

      if ( km(iseqkm2)%kmesh(52)>0 ) then
         call mshdelkmeshad
         km(iseqkm2)%kmesh(52) = 0  ! kmeshad
      end if   ! ( km(iseqkm2)%kmesh(52)>0 )

      call sepinitkmeshad

!     --- Copy kmesh2d part ad of original mesh into new one

      call mshcopykmeshad ( kmesh2d(meshorig), &
                            kmesh2d(meshobst) ) ! copy  kmesh2d ad

!     --- Remove old array map if it exists

      if ( associated(maploc%imap) ) then
         deallocate ( maploc%imap, maploc%rmap )
         call sepclear ( maploc )
      end if  ! ( associated(maploc%imap) )

!     --- Refill array ielemobst: only elements inside the obstacle are
!         marked

      call mshobstmarkelm

!     --- Print and plot information depending on plotobst and printobst

      if ( plotobst .or. printobst ) then

!     --- debug = true, plot nodes and elements with a colour

         call plotobstacle ( debug_mshobstmesh, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstcopmsh'

      end if  ! ( debug )
      call erclos ( 'mshobstcopmsh' )

      end
