      subroutine mshobstmarkelm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 10-11-2011 Use ini060test
!        version  1.0  date 24-10-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Recompute array ielemobst in the sense that only elements inside or
!     outside the obstacle are marked
!     Elements that cross the boundary are marked as external
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: ielemobstcp(:)
      integer lenielemobst

!     ielemobstcp    copy of old ielemobst
!     lenielemobst   Computed length of new array ielemobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMARKEL5 Recompute array ielemobst in the sense that only elements
!                    inside or outside the obstacle are marked
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate array
!     Compute starting address
!     Refill array ielemobst
!     Make ielemobst permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstmarkelm' )

      length = size(ielemobst)

      allocate ( ielemobstcp(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'ielemobstcp' )
      if ( ierror/=0 ) go to 1000

      ielemobstcp = ielemobst

!     --- Refill array ielemobst

      call mshobstmarkel5 ( ielemobstcp, lenielemobst )

!     --- Make ielemobst permanent

      if ( associated(km(iseqkm)%ielemobst) ) &
         deallocate ( km(iseqkm)%ielemobst )
      allocate ( km(iseqkm)%ielemobst(lenielemobst) )
      ielemobst => km(iseqkm)%ielemobst
      ielemobst = ielemobstcp(1:lenielemobst)
      kmeshad(2) = lenielemobst

      deallocate ( ielemobstcp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ielemobstcp' )

1000  call erclos ( 'mshobstmarkelm' )
      end
