      subroutine mshr01rect ( resul, iseqkeyw, nseqkeyw, ncur, tabout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-05-2005 Compute ncur
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill n, m and curve numbers for rectangle in case no keywords
!     are given
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     rectangle
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision resul(*), tabout(*)
      integer iseqkeyw(*), nseqkeyw, ncur

!     iseqkeyw       i    Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                      iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                      -isekeyw(i) refers to position in tabout
!     ncur          i/o   Number of curves surrounding a surface
!     nseqkeyw       i    Number of entries in iseqkeyw
!     resul         i/o   work array in which the numbers read are stored, &
!                         see mshr01
!     tabout         i    Contains series of numbers read by intx02ext
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount

!     i              Counting variable
!     icount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr01rect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr01rect'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0
      if ( resul(1)<=0d0 .and. resul(2)<=0d0 ) then

!     --- n and m have not been filled
!         It is possible that only numbers are given, not preceded by
!         keywords
!         This is only for old fashioned input files

          if ( iseqkeyw(1)<=0d0 .and. iseqkeyw(2)<=0d0 ) then

!         --- First two items are numbers hence n and m

             resul(2) = tabout(1)
             resul(3) = tabout(2)
             icount = 2

          end if  ! ( iseqkeyw(1)<=0d0 .and. iseqkeyw(2)<=0d0 )

      end if  ! ( resul(1)<=0d0 .and. resul(2)<=0d0 ) then

      if ( ncur==0 ) then

!     --- No curve numbers given, hence only numbers

         do i = icount+1, nseqkeyw
            resul(24+i-icount) = tabout(i)
         end do  ! i = icount+1, nseqkeyw
         ncur = nseqkeyw-icount

      end if  ! ( ncur==0 )

1000  call erclos ( 'mshr01rect' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshr01rect'

      end if

      end
