      subroutine mshr15 ( kmeshc, numnp1, numnp2, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-09-2010 Remove work arrays
!        version  1.2  date 22-08-1994 Extension with inpelm = 5
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 10-01-1983
!
!   copyright (c) 1983-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill elements in array kmesh part c, in the case of quadrilaterals
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), numnp1, numnp2

!     inpelm  i    number of points per element
!     kmeshc  o    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     numnp1  i    number of points in first direction minus one
!     numnp2  i    number of points in second direction minus one
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, in2, j, k, n1, itel(9)

!     i       Counting variable
!     ielem   Element number
!     in2     ?
!     itel    Contains shift of relative node numbers
!     j       Counting variable
!     k       Counting variable
!     n1      ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ielem=1
      n1=numnp1+1
      if ( inpelm==4 ) then

!     --- inpelm = 4,  linear elements

         itel(1)=0
         itel(2)=1
         itel(3)=1+n1
         itel(4)=n1
         do i=1,numnp2
            in2=(i-1)*n1
            do j=1,numnp1
               do k=1,4
                  kmeshc(k,ielem)=in2+j+itel(k)
               end do
               ielem = ielem + 1
            end do
         end do
      else if ( inpelm==5 ) then

!     --- inpelm = 5,  linear elements extended with centroid

         itel(1)=0
         itel(2)=1
         itel(4)=2*n1-1
         itel(3)=2*n1
         itel(5)=n1
         do i=1,numnp2
            in2=(i-1)*(2*n1-1)
            do j=1,numnp1
               do k=1,5
                  kmeshc(k,ielem)=in2+j+itel(k)
               end do
               ielem = ielem + 1
            end do
         end do
      else

!     --- inpelm = 9,  quadratic elements

         itel(1)=0
         itel(2)=1
         itel(3)=2
         itel(4)=2+n1
         itel(5)=2+2*n1
         itel(6)=1+2*n1
         itel(7)=2*n1
         itel(8)=n1
         itel(9)=n1+1
         do i=1,numnp2,2
            in2=(i-1)*n1
            do j=1,numnp1,2
               do k=1,9
                  kmeshc(k,ielem)=in2+j+itel(k)
               end do
               ielem = ielem + 1
            end do
         end do
      end if
      end
