      subroutine mshrd1 ( iinput, rinput, &
                          ipoint, inpmax, iinp, rinpt1, &
                          coorusp, rincor, cunit )
! ======================================================================
!
!        programmer    Guus Segal
!        version 27.1  date 20-11-2014 Split in subroutines
!        version 27.0  date 14-05-2014 Extra parameters
!        version 26.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    read cards with curves until a card with mesh or with surfaces
!    is found
!    fill arrays iinput and rinput with the information of the
!    curves and the factors respectively
!    iinput and rinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipoint(maxcrv), kcode, &
              inpmax, iinput(inpmax), iinp
      double precision rinput(maxcrv), rinpt1(2,maxcrv), &
                       coorusp(ndim,nuspnt), rincor(nuspnt), cunit

!     coorusp        i    contains coordinates of user points
!     cunit          i    Unit length for coarseness
!     iinp           o    next free position in iinput
!     iinput         o    integer input array
!     inpmax         i    maximal space for iinput
!     ipoint         o    pointer array: contains the starting positions of the
!                         curves in array iinput
!     rincor         i    Contains the coarseness in the user points
!     rinpt1         o    in this array the initial and end parameters t0 and t1
!                         for the curves are stored temporarily
!                         storage: t0 for curve i in rinpt1(1,i)
!                                  t1 for curve i in rinpt1(2,i)
!     rinput         o    real input array;  factors are filled from position 1, &
!                         hence rinput must be started from 8 + ndim * nuspnt
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurtp
      parameter ( ncurtp=22 )
      character (len=10) codelc(ncurtp)
      integer maxnum, icurve, kcurvs, &
              ncodelc(ncurtp), ncodmxlc(ncurtp)

!     codelc         In this table the significant characters of the various
!                    types
!                    of curves are stored.
!     icurve         Type of curve (See codelc)
!     kcurvs         Actual number of curves found (<= ncurvs)
!     maxnum         Maximal number of numbers to be read from the input
!     ncodelc        Contains the significant number of characters per word in
!                    codelc
!     ncodmxlc       Contains the maximum number of characters per word in
!                    codelc
!     ncurtp         Number of various curve types
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHRD1CUR      Read information for curves of curves
!     MSHRD1CURNR    Read first part of curve record and detect curve number
!     MSHRD1REFLECT  Read information for reflect curve
!     MSHRD1REST     Read rest of information for curve record
!     MSHRD1SHAPE    Read or set shape number of curve
!     MSHRD1TRANS    Read information for translate curve
!     MSHRD1TYPE     Detect type of curve
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!     Records with respect to the curves are read.
!     The following records are recognized:
!
!     Ci = line j (....)
!     Ci = user j (....)
!     Ci = arc j (....)
!     Ci = spline j (....)
!     Ci = cline j (....)
!     Ci = carc j (....)
!     Ci = curves j (....)
!     Ci = parameter j (....)
!     Ci = cparam j (....)
!     Ci = translate j (....)
!     Ci = rotate j (....)
!     Ci = cspline j (....)
!     Ci = reflect j (....)
!     Ci = spcurve j (....)
!     Ci = profile j (....)
!     Ci = cprofile j (....)
!     Ci = circle j (....)
!     Ci = ccircle j (....)
!     Ci = own_curve j (....)
!     Ci = framecurve j (....)
!     Ci = ellips j (....)
!     Ci = ell_arc j (...)
!     Ci = cell_arc j (...)
!
!     A part of the information between the brackets is interpreted by mshr00
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'LINE', 'USER', 'ARC', 'SPLINE', 'CLINE', 'CARC', &
                    'CURVES', 'PARAMETER', 'CPARAM', 'TRANSLATE', &
                    'ROTATE', 'CSPLINE', 'REFLECT', 'SPCURVE', &
                    'PROFILE', 'CPROFILE', 'CIRCLE', 'OWN_CURVE', &
                    'FRAMECURVE', 'ELL_ARC', 'CELL_ARC', 'CCIRCLE' /
      data ncodelc  / 3, 3, 3, 4, 4,    4, 4, 5, 5, 5, &
                      5, 5, 6, 5, 4,    5, 4, 6, 6, 6, &
                      7, 4 /
      data ncodmxlc / 4, 4, 3, 6, 6,    4, 6, 9, 6, 9, &
                      6, 7, 7, 7, 7,    8, 6, 9,10, 7, &
                      8, 7 /

!     codelc  In this table the significant characters of the various types
!             of curves are stored.
!             The following types of curves are available:
!             1  line
!             2  user
!             3  arc
!             4  spline
!             5  cline
!             6  carc
!             7  curves
!             8  parameter
!             9  cparam
!            10  translate
!            11  rotate
!            12  cspline
!            13  reflect
!            14  spcurve
!            15  profile
!            16  cprofile
!            17  circle
!            18  own_curve
!            19  framecurve
!            20  ell_arc
!            21  cell_arc
!            22  ccircle
! ======================================================================
!
      call eropen ( 'mshrd1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize factors and  array ipoint

      ipoint(1:maxcrv) = 0
      rinput(1:maxcrv) = 1d0
      rinpt1(1:2,1:maxcrv) = 0d0

      ncurvs = 0
      kcurvs = 0
      iinp = 1

      kcode = -1

!     --- while loop read curve records

      do while ( kcode<0 )

!     --- read next card

         if ( debug ) write(irefwr,1) 'kcode', kcode
         jcode = kcode
         call mshrd1curnr ( iinp, inpmax, ipoint, maxcrv, icurnr, kcurvs )
         kcode = icode
         if ( debug ) &
            write(irefwr,1) 'kcode, icurnr, ncurvs', kcode, icurnr, ncurvs
         if ( ierror/=0 .or. kcode>=0 ) go to 1000

!        --- Detect type of curve

         call sepcopycode ( codelc, ncurtp, ncodelc, ncodmxlc )
         call mshrd1type ( iinput, iinp, icurnr, icurve, jcoars )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,1) 'icurve, kcode', icurve, kcode

!        --- Read rest of records depending on the type of curve

         select case ( icurve )

         case(0)

!        --- No action

         case(7,14)

!        --- curves of curves

         call mshrd1cur ( maxnum, iinput, iinp, maxcrv, codelc, &
                          icurve, icurnr )

         case(10,11)

!        --- Translate or Rotate
!            First read Cj

            call mshrd1trans ( maxnum, iinput, iinp, maxcrv, codelc, &
                               icurve, icurnr, nuspnt )

         case(13)

!        --- Reflect
!            Read number of curve to be reflected

            call mshrd1reflect ( iinput, iinp, ndim, nuspnt, icurnr )

         case default

!        --- Other cases
!            determine shape number

            call mshrd1shape ( iinput, iinp )

!           --- Detect rest of record

            call mshrd1rest ( maxnum, iinput, iinp, ndim, cunit, nuspnt, &
                              icurve, icurnr, rinput, rinpt1, rincor, &
                              coorusp, maxcrv )

         end select  ! case ( icurve )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,1) 'kcode', kcode

      end do  ! while ( kcode<0 )

1000  iinp = iinp + ncurvs - kcurvs

      if ( debug ) then

!     --- Debug information

         call prinin ( iinput, iinp-1, 'iinput' )
         write(irefwr,*) 'End mshrd1'

      end if
      call erclos ( 'mshrd1' )

      end
