      subroutine mshrd1cur ( maxnum, iinput, iinp, maxcrv, clcode, &
                             icurve, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information for curves of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: maxcrv, icurve, icurnr
      integer, intent(inout) :: iinput(*), iinp
      integer, intent(out) :: maxnum
      character (len=*) :: clcode(*)

!     clcode         i    Indicates what text must be checked.
!     icurnr         i    Curve sequence number
!     icurve         i    Type of curve
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
!     maxcrv         i    Maximal number of curves allowed
!     maxnum         o    Maximal number of numbers to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inum, i, n
      integer :: iwork(1500)

!     i              Counting variable
!     inum           Number of numbers read in a record
!     n              number read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSERKEYW    Read number of integers preceded by the same keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     601   No curves found after curves of curves
!    1199   More than 1000 numbers found in curve record
!    1236   Curve number in curve of curves is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1cur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1cur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- curves of curves

      maxnum = 1500
      call readserkeyw ( iwork, maxnum, inum, .false., maxcrv, &
                         clcode(icurve), 'C' )
      if ( inum<=0 ) then
         call errsub ( 601, 0, 0, 0 )
         call errcrd
      end if
      if ( inum>=1500 ) then
         call errsub ( 1199, 0, 0, 0 )
         call errcrd
      end if
      iinput(iinp+1) = 1
      if ( icurve==14 ) then

!     --- icurve = 14, 4 extra positions corresponding to nelm, iratio
!                      begin point and end point

         iinput(iinp+2) = 0
         iinput(iinp+3) = 2
         iinput(iinp+4) = 0
         iinput(iinp+5) = 0
         iinp = iinp+3

      end if

      iinput(iinp+3) = inum
      do i = 1, inum
         n = iwork(i)
         iinput(iinp+3+i) = n
         n = abs(n)
         if ( n<=0 ) then

!        --- Error 1236: curve in curves of curves<1

            call errint ( icurnr, 1 )
            call errint ( n, 2 )
            call errsub ( 1236, 2, 0, 0 )

         end if
      end do
      iinp = iinp+4+inum

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd1cur'

      end if  ! ( debug )
      call erclos ( 'mshrd1cur' )

      end subroutine mshrd1cur
