      subroutine mshrd1trans ( maxnum, iinput, iinp, maxcrv, clcode, &
                               icurve, icurnr, nuspnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information for translate curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: maxcrv, icurve, icurnr, nuspnt
      integer, intent(inout) :: iinput(*), iinp
      integer, intent(out) :: maxnum
      character (len=*) :: clcode(*)

!     clcode         i    Indicates what text must be checked.
!     icurnr         i    Curve sequence number
!     icurve         i    Type of curve
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
!     maxcrv         i    Maximal number of curves allowed
!     maxnum         o    Maximal number of numbers to be read
!     nuspnt         i    Number of user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inum, i, j
      integer :: iwork(1500)

!     i              Counting variable
!     inum           Number of numbers read in a record
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSERKEYW    Read number of integers preceded by the same keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     494   No user point found in curve to be translated
!     495   Less than 3 user points found in curve to be rotated
!     498   Curve number of curve to be translated or rotated too large
!    1483   Coinciding user points at one curve
!    2944   No curve number found following the keyword translate or rotate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1trans' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1trans'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Translate or Rotate
!         First read Cj

      maxnum = 1
      call readserkeyw ( iwork, maxnum, inum, .true., maxcrv, &
                         clcode(icurve), 'C' )
      if ( inum<0 ) then

!     --- No curve number found following translate or rotate

         call errint ( icurnr, 1 )
         call errsub ( 2944, 1, 0, 0 )

      end if  ! ( table1(1)<0d0 )

!     --- Next read point numbers

      maxnum = 1499
      call readserkeyw ( iwork(2), maxnum, inum, .false., nuspnt, &
                         clcode(icurve), 'P' )

      iinput(iinp+1) = 1
      iinput(iinp+3) = inum
      inum = inum+1
      do i = 1, inum
         iinput(iinp+3+i) = iwork(i)
      end do

      if ( icurve==10 ) then
         if ( inum<1) then
            call errsub ( 494, 0, 0, 0 )
            call errcrd
         end if
      else
         if ( inum<3) then
            call errint ( inum, 1 )
            call errsub ( 495, 1, 0, 0 )
            call errcrd
         end if
      end if
      if ( iinput(iinp+4)<=0 ) then
         call errint ( iinput(iinp+4), 1 )
         call errint ( icurnr, 2 )
         call errsub ( 498, 2, 0, 0 )
         call errcrd
      end if

      do i = 1, inum-2
         do j = i+1, inum-1
            if ( abs(iinput(iinp+i+4))==abs(iinput(iinp+j+4)) .and. &
                 .not. (i==1 .and. j==inum-1) ) then

!           --- Error 1483: Coinciding user points

               call errint ( i, 1 )
               call errint ( icurnr, 2 )
               call errint ( iinput(iinp+i+4), 3 )
               call errint ( j, 4 )
               call errsub ( 1483, 4, 0, 0 )

            end if

         end do
      end do

      iinp = iinp+4+inum

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd1trans'

      end if  ! ( debug )
      call erclos ( 'mshrd1trans' )

      end subroutine mshrd1trans
