      subroutine mshrd2 ( iinput, rinput, &
                          ipoint, inpmax, iinp, icurvs,  composrf, rpin, &
                          part4, adaptiinput, lenipoint, changeinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version 26.0  date 05-12-2014 Extra parameter changeinp
!        version 25.2  date 21-11-2014 Split subroutine in parts
!        version 25.1  date 25-11-2013 Make iwork local
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards with surfaces until a card with mesh or with volumes
!     is found
!     fill array iinput with the information of the surfaces
!     iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipoint(maxsrf), &
              inpmax, iinput(inpmax), iinp, lenipoint, &
              icurvs(lenipoint)
      double precision rinput(4,maxsrf), rpin(4,ncurvs)
      logical composrf, part4, adaptiinput, changeinp

!     adaptiinput   i/o   If true iinput must be adapted
!     changeinp      o    if true extra curves have been created
!     composrf       o    If true composite elements have been found
!     icurvs         i    icurvs(i) = 0, means curve i does not exist, &
!                         icurvs(i)>0, means curve i has been filled
!     iinp          i/o   next free position in iinput
!     iinput        i/o   integer input array of length iinpmax
!     inpmax         i    maximal space for iinput
!     ipoint              pointer array: contains the starting positions of the
!                         curves in array iinput
!     lenipoint      i    Length of array ipoint
!     part4          o    If true rinput part 4 is used
!     rinput         o    part of real input array, length 4*maxsrf
!     rpin           o    information conserning thickness and factor for paver
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=23
      character (len=15) codelc(nncodelc)
      integer icopy, ksurfs, kcode, &
              ncodmxlc(nncodelc), ncodelc(nncodelc), iwork(maxsrf)
      logical :: trian

!     codelc         In this table the significant characters of the various
!                    types of surfaces are stored.
!     icopy          Surface number of surface to be copied
!     iwork          integer work array of length maxsrf to store the
!                    subsurfaces read
!     kcode          if <0, a main keyword has been found
!     ksurfs         Actual number of surfaces
!     ncodelc        Table containing the number of significant characters in
!                    the keyword table codelc
!     ncodmxlc       Table containing the number of maximum characters in
!                    the keyword table codelc
!     nncodelc       Number of surface types; length of array codelc
!     trian          Indication whether triangles (true) or quadrilaterals
!                    are created (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHRD2ORDSUR   Read rest of surface for ordered surface of surfaces
!     MSHRD2REST     Read rest of surface input
!     MSHRD2SHAPE    Read or set shape number of surface
!     MSHRD2SURNR    Read first part of surface record and detect surface number
!     MSHRD2SURSUR   Read rest of surface for surface of surfaces
!     MSHRD2TRANS    Read rest of surface for translate, rotate, reflect or
!                    similar
!     MSHRD2TYPE     Read type of surface
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2089   ndim<2
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'RECTANGLE', 'GENERAL', 'TRIANGLE', 'QUADRILATERAL', &
                    'USER', 'INPUT', 'SURFACES', 'TRANSLATE', &
                    'ROTATE', 'PIPESURFACE', 'ORDERED_SURFACE', &
                    'ORDERED_SURFACE', 'COONS', 'SIMILAR', 'PARSURF', &
                    'REFLECT', 'RECTANGLE', 'ISOPAR', 'PAVER', &
                    'SPHERE', 'COPY_SURFACE', 'FRAMESURF', 'SQUARE' /
      data ncodelc / 3, 3, 3, 4, 3,  3, 4, 5, 4, 4, &
                     4,12, 4, 5, 3,  4, 7, 3, 3, 4, &
                     8, 5, 4 /
      data ncodmxlc / 9, 7, 8,13, 6,  5, 8, 9, 6,11, &
                     15,15, 5, 7, 7,  7, 9, 6, 5, 6, &
                     12, 9, 6 /

!     codelc         In this table the significant characters of the various
!                    types of surfaces are stored.
!                    The following types of surfaces are available:
!                     1:  rectangle
!                     2:  general
!                     3:  triangle
!                     4:  quadrilateral
!                     5:  meshus
!                     6:  input
!                     7:  surfaces
!                     8:  translate
!                     9:  rotate
!                    10:  pipesurface
!                    11:  ordered surface
!                    12:  -
!                    13:  coons
!                    14:  similar
!                    15:  parsurf
!                    16:  reflect
!                    17:  rectangle with smoothing
!                    18:  isopar
!                    19:  paver
!                    20:  sphere
!                    21:  copy_surface
!                    22:  framesurf
!                    23:  square
! ======================================================================
!
      call eropen ( 'mshrd2' )
      debug = .false.
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2'
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize array ipoint

      composrf = .false.
      changeinp = .false.
      if ( ndim<2 ) then

!     --- ndim too small

         call errint ( ndim, 1 )
         call errsub ( 2089, 1, 0, 0 )
         go to 1000

      end if
      part4 = .false.

      ipoint = 0

!     --- put rpin to - infinity

      rpin = -1d20

      nsurfs = 0
      ksurfs = 0
      iinp = 1
      kcode = -1

!     --- while loop read surface records

      do while ( kcode<0 )

!     --- read next record (surface number)

         icode = kcode
         call mshrd2surnr ( iinp, inpmax, nsurfs, ksurfs, ipoint, maxsrf, &
                            isurnr )
         kcode = icode
         if ( ierror/=0 .or. kcode>=0 ) go to 1000

!        --- detect type of surface

         call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )

         call mshrd2type ( iinput, iinp, isurnr, isurf, adaptiinput )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,1) 'isurnr, isurf', isurnr, isurf

!        --- Read rest of records depending on the type of surface

         select case ( isurf )

         case(0)

!        --- No action

         case(7)

!        --- Surface of surfaces

            call mshrd2sursur ( iinp, iinput, maxsrf, iwork, codelc, &
                                isurnr, composrf )

         case(11)

!        --- isurf = 11: ordered surface consisting of subsurfaces

            call mshrd2ordsur ( iinp, iinput, maxsrf, iwork, codelc, &
                                isurnr, composrf )

         case(8, 9, 14, 16, 21)

!        --- isurf = 8, 9, 14 or 16 (TRANSLATE, ROTATE, REFLECT or SIMILAR)

            call mshrd2trans ( icopy, isurnr, isurf )

         case default

!        --- Other types of surfaces
!            determine shape number

            call mshrd2shape ( iinput, iinp, trian )
            if ( ierror/=0 ) go to 1000

         end select ! case ( isurf )

         if ( ierror/=0 ) go to 1000

         if ( isurf/=0 .and. isurf/=7 .and. isurf/=11 ) then

!        --- Detect rest of record

            call mshrd2rest ( iinp, iinput, maxsrf, isurf, isurnr, &
                              icopy, icurvs, ncurvs, nuspnt, rinput, &
                              rpin, part4, trian, nsurfs, ipoint, &
                              codelc, ndim, ishape, lenipoint, &
                              changeinp )
            if ( ierror/=0 ) go to 1000

         end if  ! ( isurf/=0 .and. isurf/=7 .and. isurf/=11 )

      end do  ! while ( kcode<0 )

1000  iinp = iinp + nsurfs - ksurfs

      if ( debug ) then

!     --- Debug information

         call prinin ( iinput, iinp-1, 'iinput' )
         write(irefwr,*) 'End mshrd2'

      end if
      call erclos ( 'mshrd2' )

      end subroutine mshrd2
