      subroutine mshrd2coons ( iinp, iinput, idiag, isurnr, iblend, &
                               icurve, jinp, ncur, isurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information for coons or quadrilateral in array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icurve, isurnr, iblend, jinp, ncur, isurf
      integer, intent(inout) :: iinp, iinput(*), idiag

!     iblend         i    User parameter for blending
!     icurve         i    Type of curve
!     idiag         i/o   Position of diagonal element in matrix
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     isurf          i    Type of surface with respect to mesh generator
!     isurnr         i    Surface sequence number
!     jinp           i    last position used in array iinput with respect to
!                         iprec
!     ncur           i    Number of curves surrounding a surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCARD         Mark part in a record by underlining with ***
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     971   Mesh generator only available for fixed number of constituting
!           curves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2coons' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2coons'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ncur/=4 ) then
         call errint ( 4, 1 )
         call errint ( ncur, 2 )
         call errint ( isurnr, 3 )
         if ( isurf==4 ) then
            call errchr ( 'quadrilateral', 1 )
         else
            call errchr ( 'coons', 1 )
         end if
         call errsub ( 971, 3, 0, 1 )
         call ercard ( kcolum, icolum )
      end if
      if ( idiag/=2 .or. isurf==4 ) idiag = 0
      iinput(iinp)    = iblend
      iinput(iinp+1)  = icurve
      iinp            = iinp + 2
      iinput(jinp+2)  = iinput(jinp+2)+10000*idiag

1000  call erclos ( 'mshrd2coons' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2coons'

      end if  ! ( debug )

      end subroutine mshrd2coons
