      subroutine mshrd2frame ( iinp, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information for framesurf or internal points in array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinp, iinput(*)

!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine mshrea
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numpoints, numcurves, i

!     i              Counting variable
!     numcurves      Number of curves in array icurves
!     numpoints      Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2frame' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2frame'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numpoints = nint(work(23))  ! number of internal points
      numcurves = nint(work(22))  ! number of internal curves
      iinput(iinp) = numpoints    ! number of internal points
      iinput(iinp+1) = numcurves  ! number of internal curves
      iinp = iinp+2
      do i = 0, numpoints-1
         iinput(iinp+i) = nint(work(1001+i))  ! internal point
      end do  ! i = 1, iinput(iinp)
      iinp = iinp+numpoints  ! raise iinp
      do i = 0, numcurves-1
         iinput(iinp+i) = nint(work(1251+i))  ! internal curve
      end do  ! i = 1, iinput(iinp)
      iinp = iinp+numcurves  ! raise iinp
      if ( debug ) then
         write(irefwr,1) 'numpoints, numcurves', numpoints, numcurves
         write(irefwr,1) 'iinp_points', iinp-numcurves-numpoints
         write(irefwr,1) 'iinp_curves', iinp-numcurves
         if ( numpoints>0 ) &
            call prinin ( iinput(iinp-numcurves-numpoints), &
                          numpoints, 'extra points' )
         if ( numcurves>0 ) &
             call prinin ( iinput(iinp-numcurves), numcurves, &
                           'extra curves' )
      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2frame'

      end if  ! ( debug )
      call erclos ( 'mshrd2frame' )

      end subroutine mshrd2frame
