      subroutine mshrd2sphere ( iinp, iinput, isurnr, nsur, nsurfs, &
                                ncur, icentre, ipoint, itypesurf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information for sphere in array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ncur, icentre, itypesurf, nsur
      integer, intent(inout) :: iinp, iinput(*), ipoint(*), isurnr, &
                                nsurfs

!     icentre        i    User point number of centroid
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     ipoint        i/o   Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     isurnr         i    Surface sequence number
!     itypesurf      i    Defines type of surface to be created
!     ncur           i    Number of curves surrounding a surface
!     nsur           i    Number of surfaces surrounding a volume
!     nsurfs        i/o   number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCARD         Mark part in a record by underlining with ***
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     144   Surface number has already been read before
!     971   Mesh generator only available for fixed number of constituting
!           curves
!    2093   Number of subsurfaces read in surface record more than 500
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2sphere' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2sphere'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ncur/=1 ) then
         call errint ( 1, 1 )
         call errint ( ncur, 2 )
         call errint ( isurnr, 3 )
         call errchr ( 'SPHERE', 1 )
         call errsub ( 971, 3, 0, 1 )
         call ercard ( kcolum, icolum )
      end if
      iinput(iinp  ) = icentre
      iinput(iinp+1) = itypesurf
      iinput(iinp+2) = 0
      iinput(iinp+3) = 0
      iinput(iinp+4) = 0
      iinput(iinp+5) = 0
      if ( nsur>2 ) then

!     --- Too many subsurfaces

         call errint ( nsur, 1 )
         call errint ( 2, 2 )
         call errint ( isurnr, 3 )
         call errchr ( 'SPHERE', 1 )
         call errsub ( 2093, 3, 0, 1 )

      else

!     --- Copy subsurfaces

         do i = 1, nsur
            isurnr = nint(work(1000+i))
            iinput(iinp+3+i) = isurnr
            if ( ipoint(isurnr)/=0 ) then

!           --- Surface isurnr has already been filled

               call errint ( isurnr, 1 )
               call errsub ( 144, 1, 0, 0 )
               call errcrd

            end if
            ipoint(isurnr) = -1
            nsurfs = max ( nsurfs, isurnr )

         end do

      end if
      iinp = iinp + 6

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2sphere'

      end if  ! ( debug )
      call erclos ( 'mshrd2sphere' )

      end subroutine mshrd2sphere
