      subroutine mshrd2square ( iinp, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information for square in array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinp, iinput(*)

!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine mshrea
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numpoints, numcurves, iinpsav

!     iinpsav        Help variable to save the value of iinp
!     numcurves      Number of curves
!     numpoints      Number of user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2square' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2square'
         iinpsav = iinp

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numpoints = nint(work(23))  ! number of points
      numcurves = nint(work(1))  ! number of curves
      iinput(iinp) = numpoints
      iinput(iinp+1) = numcurves
      iinput(iinp+2) = nint(work(2))  ! n
      iinput(iinp+3) = nint(work(3))  ! m
      iinp = iinp+4

      if ( numpoints>0 ) then

!     --- numpoints>0, fill user points

         iinput(iinp:iinp+numpoints-1) = nint(work(1001:1000+numpoints))
         iinp = iinp+numpoints

      end if  ! ( numpoints>0 )
      if ( numcurves>0 ) then

!     --- numcurves>0, fill curves

         iinput(iinp:iinp+numcurves-1) = nint(work(25:24+numcurves))
         iinp = iinp+numcurves

      end if  ! ( numcurves>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput(iinpsav), iinp-iinpsav, 'iinput' )
         write(irefwr,*) 'End mshrd2square'

      end if  ! ( debug )
      call erclos ( 'mshrd2square' )

      end subroutine mshrd2square
