      subroutine mshrd2sursur ( iinp, iinput, maxsrf, iwork, clcode, &
                                isurnr, composrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of surface for surface of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: maxsrf, isurnr
      integer,intent(inout) :: iinp, iinput(*), iwork(*)
      character (len=*) clcode(*)
      logical,intent(inout) :: composrf

!     clcode         i    In this table the significant characters of the
!                         various types of surfaces are stored.
!     composrf      i/o   If true composite elements have been found
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!     isurnr         i    Surface sequence number
!     iwork          i    integer work array, see mshrd2
!     maxsrf         i    Maximal number of surfaces allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inum, i, maxnum

!     i              Counting variable
!     inum           Number of numbers read in a record
!     maxnum         Maximum row number for part of decomposition
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSERKEYW    Read number of integers preceded by the same keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     560   Surface consisting of subsurfaces contains no subsurface numbers.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2sursur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2sursur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxnum = maxsrf
      call readserkeyw ( iwork, maxnum, inum, .false., maxsrf, &
                         clcode(7), 'S' )
      if ( inum<=0 ) then
          call errint ( isurnr, 1 )
          call errsub ( 560, 1, 0, 0 )
          call errcrd
      end if
      if ( inum<=0 ) then
          call errint ( isurnr, 1 )
          call errsub ( 560, 1, 0, 0 )
          call errcrd
      end if
      iinput(iinp+2) = inum
      iinp = iinp + 2
      do i = 1, inum
         iinput(iinp+i) = iwork(i)
      end do
      iinp = iinp + 1 + inum
      composrf = .true.

1000  call erclos ( 'mshrd2sursur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inum, iinp', inum, iinp
         write(irefwr,*) 'End mshrd2sursur'

      end if  ! ( debug )

      end subroutine mshrd2sursur
