      subroutine mshrd8 ( iinput, input1, nelgrp, inpmax, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.0  date 21-11-2014 New parameter list
!        version  9.4  date 29-11-2012 Debug statements
!        version  9.3  date 06-08-1995 Extra error messages because of j>0
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards with special connected elements until a card with
!   renumber, plot or end is found
!   fill array iinput with the information of these connected elements
!   iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpmax, iinput(inpmax), nelgrp, input1(*), iinp

!     iinp           i    last position used in array iinput
!     iinput        i/o   integer input array
!                         in this array the input of the mesh must be stored
!                         from position 1
!     inpmax         i    Available space for array iinput
!     input1        i/o   array iinput with respect to special connected
!                         elements
!     nelgrp        i/o   Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nlcode=8
      character (len=8) clcode(nlcode)
      integer ipoint, icurv, isurf, inum, ielgrp, ifirst, ilast, &
              ishape, nscode(nlcode), &
              ipos, ncodlm(nlcode), iexcl, nuspnt, &
              ncurvs, nsurfs
      logical ::  check

!     check          Indicates if a new element group has been found, or still
!                    the first one is investigated
!     clcode         Contains the texts that are recognized after the part
!                    CELM i = &
!                    Values:
!                    1:  POINTS
!                    2:  CURVES
!                    3:  SURFACES
!                    4:  EXCLUDE
!                    5:  ALL
!                    6:  P
!                    7:  C
!                    8:  S
!     icurv          Number of curves that are connected
!     ielgrp         element group number
!     iexcl          Parameter exclude
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     inum           Number of numbers read in a record
!     ipoint         Number of user points that are connected
!     ipos           Position in work
!     ishape         Shape of element
!     isurf          Number of surfaces that are connected
!     ncodlm         Table containing the number of maximum characters in
!                    the keyword table clcode
!     ncurvs         Number of curves in mesh
!     nlcode         Number of entries in clcode
!     nscode         Number of significant characters in clcode
!     nsurfs         number of surfaces
!     nuspnt         Number of user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following input of the standard input file is recognized
!    - main keywords, see icode
!    - Input for meshconnect:
!         CELMi = POINTS(P1,P2)
!         CELMi = CURVES l (C1,C2)
!         CELMi = SURFACES j (S1,S2)[, EXCLUDE ( [ALL] [Ci, Ck, ... ] )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      68   element group number not equal to preceding element group number or
!           preceding element group number plus one
!     151   first curve number has incorrect value
!     152   last curve number has incorrect value
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!     310   record with velm (or celm) contains more than 3 volume numbers
!     311   record with selm (or celm) contains more than three curve numbers
!     316   record with pelm found, whereas already records with celm or selm
!           are read for connected elements.
!     317   record with pelm contains more than two user points
!     318   record with celm found, whereas already records with selm are read
!           for connected elements.
!     424   first user point number (ifirst) has incorrect value
!     425   last point number (ilast) has incorrect value
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
!    1184   No number found, whereas at these positions a number is expected
!    1348   Record does not start with a character
!    1362   Data record for MESHCONNECT does not start with the letter C.
!    1432   User point connected to itself
!    1490   No curves found after exclude
!    1583   j>0 and iexclude#0, not allowed
!    1837   Present element group number is equal to preceding one, whereas a
!           new type of elements is introduced
!    3120   no input found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data clcode / 'POINTS', 'CURVES', 'SURFACES', 'EXCLUDE', 'ALL', &
                    'P', 'C', 'S' /
      data nscode / 3, 3, 3, 3, 3, 1, 1, 1 /
      data ncodlm / 6, 6, 8, 7, 3, 1, 1, 1 /
! ======================================================================
!
      call eropen ( 'mshrd8' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd8'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype(1) = 2
      itype(2:3) = 4
      itype(4:5) = 2
      itype(6:8) = 14

      tabmin(1:nlcode) = 0d0
      tabmin(6) = 2d0
      tabmin(7) = nout
      tabmin(8) = 2d0
      tabmax(1:nlcode) = 0d0
      tabmax(2) = 300d0
      tabmax(3) = 1d0

!     --- Get parameters

      nuspnt = iinput(2)
      ncurvs = iinput(3)
      nsurfs = iinput(4)

      iinp = 3
      iinput(12)=1
      ipoint = 0
      icurv = 0
      isurf = 0
      check = .false.

!     --- read next record

200   ityprd = 1
      call asknxt

      if ( debug) write(irefwr,1) 'ityprd', ityprd
      if ( ityprd/=4 ) then

!     --- error 1348: Record does not start with a character

         call errsub ( 1348, 0, 0, 0 )
         call errcrd
         go to 200

      else

!     --- Detect the type of record
!         If a main keyword has been found, return to calling subroutine

         if ( debug) write(irefwr,*) 'string ', string(1:nchar)
         ierpri = 0
         icode = sepmainkeyw() - 1
         if ( debug ) write(irefwr,1) 'icode', icode

         if ( icode>=0 ) then

            if ( iinp<=3 ) then

!           --- No input found after mesh connect

               call errchr ( 'meshconnect', 1 )
               call errsub ( 3120, 0, 0, 1 )

            end if  ! ( iinp<=0 )
            go to 1000

         end if  ! ( icode>=0 )

      end if
      if ( string(1:1)/='C' ) then
         call errchr ( string(1:1), 1 )
         call errsub ( 1362, 0, 0, 1 )
         call errcrd
         go to 200
      end if

!     --- detect the connection card

      call asknxt
      if ( ityprd/=2 ) then

!     --- No integer number found after CELM

         call errsub ( 1184, 0, 0, 0 )
         call errcrd
         go to 200
      end if
      ielgrp = nint(realrd)
      input1(iinp+1) = ielgrp

      if ( debug ) write(irefwr,1) 'ielgrp', ielgrp
      if ( .not. check ) then

!     --- No records with respect to connect elements read, so nelgrp must
!         be raised

         if ( ielgrp==nelgrp ) then

!        --- Error 1837: ielgrp = nelgrp

            call errint ( ielgrp, 1 )
            call errsub ( 1837, 1, 0, 0 )
            call errcrd

         end if
         nelgrp = nelgrp + 1

      else if ( ielgrp>nelgrp ) then

!     --- Records have been read and ielgrp>nelgrp, raise nelgrp

         nelgrp = nelgrp+1

      end if

      if ( ielgrp/=nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp-1, 2 )
         call errsub ( 68, 2, 0, 0 )
         call errcrd
      end if
      check = .true.

!     --- Read type of record

      call sepcopycode ( clcode, nlcode, nscode, ncodlm )

      nmain = 0
      table = 0d0
      tabmax(6) = nuspnt
      tabmax(7) = ncurvs
      tabmax(8) = nsurfs
      tabout(1) = 0d0
      tabout(2) = 0d0
      iretur = 0
      call sepreadsetkeyw

      if ( debug ) write(irefwr,1) 'iretur', iretur

      if ( iretur==-1 ) go to 200

      if ( nint(table(1))==1 ) then

!     --- user points must be connected

         if ( icurv>0 .or. isurf>0 ) then
            call errsub ( 316, 0, 0, 0 )
            goto 200
         end if
         inum = nint(tabmax(6))
         if ( inum/=2 ) then
            call errint ( inum, 1 )
            call errsub ( 317, 1, 0, 0 )
            call errcrd
         end if
         ifirst = nint(tabout(1))
         if ( ifirst<=0 .or. ifirst>nuspnt ) then
            call errint ( ifirst, 1 )
            call errint ( nuspnt, 2 )
            call errsub ( 424, 2, 0, 0 )
            call errcrd
         end if
         ilast = nint(tabout(2))
         if ( ilast<=0 .or. ilast>nuspnt ) then
            call errint ( ilast, 1 )
            call errint ( ifirst, 2 )
            call errint ( nuspnt, 3 )
            call errsub ( 425, 3, 0, 0 )
            call errcrd
         end if
         if ( ilast==ifirst ) then
            call errint ( ilast, 1 )
            call errsub ( 1432, 1, 0, 0 )
            call errcrd
         end if

         input1(iinp+2) = ifirst
         input1(iinp+3) = ilast
         iinp = iinp+3
         ipoint = ipoint+1
         goto 200

      else if ( nint(table(2))/=0 ) then

!     --- curves must be connected

         if ( isurf>0) then
            call errsub ( 318, 0, 0, 0 )
            goto 200
         end if
         inum = nint(tabmax(7))
         if ( inum/=2 ) then
            call errint ( inum, 1 )
            call errsub ( 311, 1, 0, 0 )
            call errcrd
         end if
         if ( table(2)<-1d1 ) then
            ishape = 0
         else
            ishape = nint(tabout(1))
         end if
         iexcl = ishape/100
         if ( ishape>0 .and. ishape-100*iexcl/=0 .and. iexcl>0 ) then
            call errint ( iexcl, 1 )
            call errint ( ishape, 2 )
            call errsub ( 558, 2, 0, 0 )
         end if
         ipos = max(1,nint(table(7)))
         ifirst = nint(tabout(ipos))
         if ( ifirst==0 .or. abs(ifirst)>ncurvs ) then
            call errint ( ifirst, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 151, 2, 0, 0 )
            call errcrd
         end if
         ilast = nint(tabout(ipos+1))
         if ( ilast==0 .or. abs(ilast)>ncurvs ) then
            call errint ( ilast, 1 )
            call errint ( ifirst, 2 )
            call errint ( ncurvs, 3 )
            call errsub ( 152, 3, 0, 0 )
            call errcrd
         end if

         input1(iinp+2) = ishape
         input1(iinp+3) = ifirst
         input1(iinp+4) = ilast
         iinp = iinp+4
         icurv = icurv+1
         goto 200

      else if ( nint(table(3))/=0 ) then

!     --- surfaces must be connected

         inum = nint(tabmax(8))
         if ( inum/=2 ) then
            call errint ( inum, 1 )
            call errsub ( 310, 1, 0, 0 )
            call errcrd
         end if
         if ( table(3)<-1d1 ) then
            ishape = 0
         else
            ishape = nint(tabout(1))
         end if
         ipos = max(1,nint(table(8)))
         ifirst = nint(tabout(ipos))
         if ( ifirst<=0 .or. ifirst>nsurfs ) then
            call errint ( ifirst, 1 )
            call errint ( nsurfs, 2 )
            call errsub ( 155, 2, 0, 0 )
            call errcrd
         end if
         ilast = nint(tabout(ipos+1))
         if ( ilast<0 .or. ilast>nsurfs ) then
            call errint ( ifirst, 1 )
            call errint ( ilast, 2 )
            call errint ( nsurfs, 3 )
            call errsub ( 156, 3, 0, 0 )
            call errcrd
         end if

         input1(iinp+2) = ishape
         input1(iinp+3) = ifirst
         input1(iinp+4) = ilast
         input1(iinp+5) = 0
         iinp = iinp+5
         if ( nint(table(4))==1 ) then

!        --- EXCLUDE found

            if ( ishape/=0 ) then
               call errint ( ishape, 1 )
               call errsub ( 1583, 1, 0, 0 )
            end if
            if ( nint(table(5))==1 ) then

!           --- ALL found

               input1(iinp) = -1

            else

!           --- Curve numbers found

               inum = nint(tabmax(7))
               if ( inum==0 ) then

!              --- error no curve numbers found after exclude

                  call errint ( ielgrp, 1 )
                  call errsub ( 1490, 1, 0, 0 )
                  call errcrd
               end if
               input1(iinp) = inum
               ipos = nint(table(7))
               input1(iinp+1:iinp+inum) = nint(tabout(ipos:ipos+inum-1))
               iinp = iinp + inum

            end if

         end if
         isurf = isurf+1
         goto 200

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd8'

      end if  ! ( debug )
      call erclos ( 'mshrd8' )

      input1(1) = ipoint
      input1(2) = icurv
      input1(3) = isurf

      end
