      subroutine mshrda ( ndim, cunit, maxpnt, maxcrv, maxsrf, maxvlm, &
                          jcoars, ireadr, ispecial, iscale, &
                          itype_scale, maxratio )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.1  date 20-11-2014 Extension with TYPE_ELEMENTS
!        version  8.0  date 30-07-2012 Remove ifinvl from parameter list
!        version  7.0  date 23-09-2009 Extra parameter maxratio
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine MSHRDA reads the first part of the input for subroutine MESH
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, maxpnt, maxcrv, maxsrf, maxvlm, jcoars, &
              ireadr, ispecial, iscale, itype_scale
      double precision cunit, maxratio

!     cunit          o    Unit length for coarseness
!     ireadr         i    If 1 the first record has been read by READSP
!     iscale         o    Indicates the type of scaling that must be applied
!                         Possible values:
!                         0: no scaling
!                         1: always scaling
!                         2: scaling for specified sufraces and volumes
!     ispecial       o    Indicates if the standard mesh input is read with
!                         points or the special one with nodal_points
!                         Possible values:
!                         0    Standard case
!                         1    Special case
!     itype_scale    o    Indicates how the scaling must be performed
!                         Possible values:
!                         0: scaling in x, y and z directions
!                         1: scaling in x-direction only
!                         2: scaling in y-direction only
!                         3: scaling in z-direction only
!                         4: scaling depends on direction of mesh and size of
!                            boundary elements
!     jcoars         o    If 1 coarseness is given, if 0 not
!     maxcrv         o    Maximal number of curves allowed
!     maxpnt         o    Maximal number of nodal points allowed
!     maxratio       o    Maximal ratio between succeeding elements
!     maxsrf         o    Maximal number of surfaces allowed
!     maxvlm         o    Maximal number of volumes allowed
!     ndim           o    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=13, nsub=31
      integer :: ncode2(nncode2), ncodmx2(nncode2), nmcode(nsub)
      character (len=14) code2(nncode2)
      character (len=29) codemn(nsub)

!     code2         Table with keywords
!     codemn        Table with sub keywords
!     ncode2        Table containing the number of significant characters in
!                   the keyword table code2
!     ncodmx2       Table containing the maximal number of  characters in
!                   the keyword table code2
!     nmcode        Table containing the number of significant characters in
!                   the keyword table code, including those of the extra texts
!     nncode2       Number of entries in code2
!     nsub          Number of subkeywords stored in codemn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHFILLSHAPES  Fill default shapes
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following input from the standard input file is expected:
!    MESHnD   (n=1,2 or 3)  (Mandatory)
!    TYPE_ELEMENTS = ..
!    Default records (arbitrary order):
!      COARSE (unit=u, maxratio = m)
!      MAXPOINTS = mp (Default 1000)
!      MAXCURVES = mc (Default 1000)
!      MAXSURFACES = ms (Default 1000)
!      MAXVOLUMES = mv (Default 500)
!      FINITE_VOLUMES
!      ISNAS
!      SCALING = s    (NO/ALWAYS/DEPENDENT)
!      TYPE_SCALING = t  (ALL/X_DIR/Y_DIR/Z_DIR/DETECT)
!   POINTS (mandatory)
!   or alternatively NODAL_POINTS, options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!     130   first record for subroutine mesh not correct
!    1599   No coarseness found after coarse
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Read mesh card and ndim
!   Check ndim
!   nextcard = blank
!   While ( nextcard not POINTS ) do
!      read nextcard
!      interpret nextcard (allowed are MAX... COARSE and POINTS)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'COARSE', 'MAXPOINTS', 'MAXCURVES', 'MAXSURFACES', &
                  'MAXVOLUMES', 'FINITE_VOLUMES', 'ISNAS', &
                  'UNIT_LENGTH', 'LISS', 'SCALING', 'TYPE_SCALING', &
                  'MAXRATIO', 'TYPE_ELEMENTS' /
      data ncode2  / 3, 4, 4,  4,  4,   4, 5, 4, 3, 4, &
                    7, 6, 7 /
      data ncodmx2 / 6, 9, 9, 11, 10,  14, 5,11, 4, 7, &
                   13, 8,13 /
      data codemn /  'NO', 'ALWAYS', &
                    'DEPENDENT', 'ALL', 'X_DIR', 'Y_DIR', 'Z_DIR', &
                    'DETECT', 'TRIANGLE', &
                    'QUADRILATERAL', 'TETRAHEDRON', 'HEXAHEDRON', &
                    'LINEAR_TRIANGLE', 'QUADRATIC_TRIANGLE', &
                    'LINEAR_TETRAHEDRON', 'QUADRATIC_TETRAHEDRON', &
                    'LINEAR_HEXAHEDRON', 'QUADRATIC_HEXAHEDRON', &
                    'LINEAR_QUADRILATERAL', 'QUADRATIC_QUADRILATERAL', &
                    'LINEAR', 'QUADRATIC', &
                    'EXTENDED_LINEAR_TRIANGLE', &
                    'EXTENDED_LINEAR_QUADRILATERAL', &
                    'EXTENDED_QUADRATIC_TRIANGLE', &
                    'EXTENDED_TETRAHEDRON14', 'EXTENDED_TETRAHEDRON15', &
                    'EXTENDED_LINEAR_HEXAHEDRON', &
                    'EXTENDED_LINEAR_TETRAHEDRON', &
                    'LINEAR_WEDGE', 'QUADRATIC_WEDGE' /
      data nmcode /  2, 6, 5,   3, 5, 5, 5, 5, 3, 7, &
                     3, 3,10,  13,10,13,10,13, 10,13, &
                     3, 7,19,  19,14,22,22,19, 19,10, &
                    13 /

!     code2           Table with keywords
!                    The following keywords are available:
!                     1: COARSE
!                     2: MAXPOINTS
!                     3: MAXCURVES
!                     4: MAXSURFACES
!                     5: MAXVOLUMES
!                     6: FINITE_VOLUMES
!                     7: ISNAS
!                     8: UNIT_LENGTH
!                     9: LISS
!                    10: SCALING
!                    11: TYPE_SCALING
!                    12: MAXRATIO
!                    13: TYPE_ELEMENTS
!     codemn         Table with sub keywords
!                    The following keywords are available:
!                     1: NO
!                     2: ALWAYS
!                     3: DEPENDENT
!                     4: ALL
!                     5: X_DIR
!                     6: Y_DIR
!                     7: Z_DIR
!                     8: DETECT
!                     9: TRIANGLE
!                    10: QUADRILATERAL
!                    11: TETRAHEDRON
!                    12: HEXAHEDRON
!                    13: LINEAR_TRIANGLE
!                    14: QUADRATIC_TRIANGLE
!                    15: LINEAR_TETRAHEDRON
!                    16: QUADRATIC_TETRAHEDRON
!                    17: LINEAR_HEXAHEDRON
!                    18: QUADRATIC_HEXAHEDRON
!                    19: LINEAR_QUADRILATERAL
!                    20: QUADRATIC_QUADRILATERAL
!                    21: LINEAR
!                    22: QUADRATIC
!                    23: EXTENDED_LINEAR_TRIANGLE
!                    24: EXTENDED_LINEAR_QUADRILATERAL
!                    25: EXTENDED_QUADRATIC_TRIANGLE
!                    26: LINEAR_WEDGE
!                    27: QUADRATIC_WEDGE
! ======================================================================
!
      call eropen ( 'mshrda' )

!     --- Initialize arrays

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopycodesub ( codemn, nsub, nmcode )

      nmain = 2
      codemain(1) = 'POINTS'
      codemain(2) = 'NODAL_POINTS'
      ncodemain(1) = 3
      ncodemain(2) = 9

      itype(1:nncode) = 0
      tabmin(1:nncode) = 0d0
      tabmax(1:nncode) = 0d0

      itype(1) = 2      ! COARSE

      tabmin(2:5) = 1d1 ! MAXPOINTS, .. , MAXVOLUMES
      tabmax(2:5) = 1d6

      itype(6:7) = 2    ! FINITE_VOLUMES, ISNAS
      itype(8) = 1      ! UNIT_LENGTH
      tabmin(8) = 1d-20
      tabmax(8) = 1d20

      itype(9) = 2      ! LISS
      itype(10:11) = 5  ! SCALING, TYPE_SCALING
      itype(12) = 1     ! MAXRATIO
      tabmax(12) = 1d1

      itype(13) = 5     ! TYPE_ELEMENTS

!     --- read first card  (mesh.d)

      if ( ireadr/=1 ) then

!     --- ireadr # 1, standard situation
!         Read first record and interpret MESH part

         ityprd = 1
         call asknxt
         if ( ityprd/=4 .or. nchar<3 .or. string(1:3)/='MES' ) then
            call errsub ( 130, 0, 0, 0 )
            call errcrd
         end if
         call asknxt
         ndim = nint(realrd)
         if ( ndim<=0 .or. ndim>3 ) then
            call errint ( ndim, 1 )
            call errsub ( 62, 1, 0, 0 )
            call errcrd
         end if

      end if

!     --- Read next records until a record with POINTS has been found

      table(1:nncode) = -1d0
      tabmin(10) = 1d0        ! SCALING
      tabmax(10) = tabmin(10)+2d0
      tabmin(11) = tabmax(10)+1d0   ! TYPE_SCALING
      tabmax(11) = tabmin(11)+4d0
      tabmin(13) = tabmax(11)+1d0   ! TYPE_ELEMENTS
      tabmax(13) = tabmin(13)+23d0
      ityprd = 1
      istop  = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0

!     --- First set defaults:

      jcoars = 0
      maxpnt = 1000
      maxcrv = 1000
      maxsrf = 1000
      maxvlm = 500
      cunit = 1d0
      maxratio = 0d0
      iscale = 0
      itype_scale = 0

      if ( table(1)>0d0 ) then

!     --- COARSE  found, read number

         cunit = table(8)
         if ( cunit<=0d0 ) call errsub ( 1599, 0, 0, 0 )
         maxratio = max(0d0,table(12))
         jcoars = 1

      end if

      if ( table(2)>0d0 ) maxpnt = nint(table(2))
      if ( table(3)>0d0 ) maxcrv = nint(table(3))
      if ( table(4)>0d0 ) maxsrf = nint(table(4))
      if ( table(5)>0d0 ) maxvlm = nint(table(5))

      if ( table(10)>0d0 ) iscale = nint(table(10))-1
      if ( table(11)>0d0 ) itype_scale = nint(table(11))-1
      ispecial = iretur-1
      if ( ispecial==1 .and. ndim==1 ) call errsub ( 1989, 0, 0, 0 )

      if ( table(13)>0d0 ) call mshfillshapes ( nint(table(13)) )

      call erclos ( 'mshrda' )
      end
