      subroutine mshsp1 ( iinput, rinput, iinp, n, icurnr, ipointcurv )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 14-10-2002 Extra parameters kmesh, kmeshq
!        version  2.0  date 13-12-1999 Extra parameters ipointcurv, icurnr
!                                      Correction for reflection of reflection
!        version  1.0  date 25-03-1994
!
!   copyright (c) 1994-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute co-ordinates of curve by reflection of the points of curve
!     iprec
!     The reflection line or plane is defined by the first ndim user
!     points of the new curve
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     reflection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, n, icurnr, ipointcurv(*)
      double precision rinput(ndim,*)

!     icurnr         i    Curve sequence number
!     iinp           i    starting position in array iinput with respect to new
!                         curve
!     iinput         i    new array iinput
!                         The first 5 positions have been skipped
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curves in the part of iinput referring
!                         to curves
!     n              o    number of points in curve
!     rinput        i/o   new array rinput  |
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ip, iprec, ipc1, ipc2, ipc3, ipc4, nuser, j, &
              jposc, ipos, istart, ic, nparts, nsubcv, isubcv
      double precision eps, xp(3), xn(3), x1(3), x2(3), dx(3), xm(3), &
                       dist
      logical errfnd

!     dist           distance
!     dx             direction vector for reflection line / plane
!     eps            accuracy
!     errfnd         If true an error has been found
!     i              loop variable
!     ic             Relative subcurve number
!     ip             loop variable for dimension loops
!     ipc1           first user point on original curve
!     ipc2           first user point for reflection
!     ipc3           second user point for reflection
!     ipc4           third user point for reflection
!     ipos           local position
!     iprec          pointer
!     istart         First position used at a part
!     isubcv         Subcurve sequence number with direction
!     j              Counting variable
!     jposc          position
!     nparts         number of nodes along a subcurve
!     nsubcv         Number of subcurves of a curve
!     nuser          number of user points
!     x1             work array to store coordinates of first point
!     x2             work array to store coordinates of second point
!     xm             point in reflection line or plane
!     xn             coordinates : result of reflection of xp
!     xp             coordinates of original point to be reflected
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHSP2         Compute coordinates of reflected point
!     MSHSP3         Compute characteristics of reflection plane
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     964   Prescribed user point is not reflection of original user point
!    2368   Curve to be copied is not coniguous
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find starting positions of curve iprec in array iinput
!     and array curvesmsh
!     iprec is array to be reflected, n is number of nodes on that curve
!     Determine characteristics of curve iprec
!     Curve icurnr:  first (user) point: ipc2
!     If ndim = 2 : Determine reflection line and tangent of normal
!     Else if ndim = 3 : Determine reflection plane and tangent of normal
!     Determine point on reflection line or plane
!     Reflect all co-ordinates of curve iprec into curve icurnr
!     Copy or check co-ordinates of user points on the curve icurnr
!        Determine user point numbers, first the "image"
!        Check whether image is correct for original point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsp1' )

      eps = 10 * epsmac

!     --- Find starting positions of curve iprec in array iinput (kinp)
!         and array curves (jposc)
!         iprec is array to be reflected, n is number of nodes on that curve

      nsubcv = 0
      iprec = iinput(iinp)

      if ( kelmq>0 ) nsubcv = kmeshq(iprec+1)-kmeshq(iprec)

      if ( iprec==1 ) then
         jposc = 0
         n = icurvsmsh(iprec)
      else
         jposc = icurvsmsh(iprec-1)
         n = icurvsmsh(iprec) - jposc
      end if

!     --- Correct n if there are more subcurves due to double points

      if ( nsubcv>1 ) n = n-nsubcv+1

!     --- Characteristics of curve iprec
!             nuser is number of user points
!             ipc1 is first user point in curve iprec
!             (with this point the reflection line is determined)

      istart = curveusrp(iprec)
      nuser = curveusrp(iprec+1)-istart
      if ( iinput(ipointcurv(iprec))==13 ) then
         istart = istart+ndim
         nuser = nuser-ndim
      end if

      ipc1 = curveusrp(istart+1)

!     --- Curve icurnr:  first (user) point: ipc2

      ipc2 = iinput(iinp+1)
      ipc3 = iinput(iinp+2)

      if ( ndim==3 ) ipc4 = iinput(iinp+3)

      if ( ndim==2 ) then

!     --- Determine reflection line and tangent of its normal

         dx(1) = -( rinput(2,ipc2) - rinput(2,ipc3) )
         dx(2) =  ( rinput(1,ipc2) - rinput(1,ipc3) )

      else if ( ndim==3 ) then

!     --- Determine reflection plane and tangent of its normal

         call mshsp3 ( rinput, ipc2, ipc3, ipc4, dx )

      end if

!     --- Determine point on reflection line or plane

      do i = 1, ndim
         xm(i) = rinput(i,ipc2)
      end do

!     --- Reflect all co-ordinates of curve iprec into curve icurnr

      ipos = icurvsmsh(icurnr-1)
      nsubcv = max(1,nsubcv)

!     --- For all subcurves do

      do ic = 1, nsubcv

         if ( nsubcv>1 ) then

!        --- There are more subcurves,
!            find subcurve isubcv and number of nodes along isubcv

            isubcv = abs(kmeshq(kmeshq(iprec)+ic))
            if ( abs(isubcv)==1 ) then
               nparts = icurvsmsh(isubcv)
            else
               nparts = icurvsmsh(isubcv)-icurvsmsh(isubcv-1)
            end if

         else

!        --- No subcurves

            nparts = n

         end if

         do i = 1, nparts

            do ip = 1, ndim
               xp(ip) = curvesmsh(ip,jposc+i)
            end do

            call mshsp2( ndim, xp, xm, dx, xn )

            do ip = 1, ndim
               curvesmsh(ip,ipos+i) = xn(ip)
            end do

         end do

         if ( ic>1 ) then

!        --- not the first subcurve, check if first points is equal to
!            last one in previous subcurve
!            Store begin coordinates in x1

            errfnd = .false.
            do j = 1, ndim
               x1(j) = curvesmsh(j,jposc+1)
               if ( abs(x2(j)-x1(j))>&
                    sqreps*(abs(x1(j))+abs(x2(j))) ) errfnd = .true.
            end do
            if ( errfnd ) then

!           --- The curve is not contiguous

               call errint ( iprec, 1 )
               call errint ( icurnr, 2 )
               call errint ( ic, 3 )
               call errsub ( 2368, 3, 0, 0 )
               call prinrl ( x1, ndim, 'first node' )
               call prinrl ( x2, ndim, 'second node' )

            end if

         end if

         if ( nsubcv>1 ) then

!        --- Store end coordinates in x2

            do j = 1, ndim
               x2(j) = curvesmsh(j,jposc+nparts)
            end do

         end if

!        --- Raise pointers, take double point into account

         jposc = jposc+nparts
         ipos = ipos+nparts-1

      end do

!     --- Copy or check co-ordinates of user points on the curve icurnr

      i = 0
      ipos = icurvsmsh(icurnr-1)

500   if ( i<nuser ) then

         i = i + 1

!        --- Determine user point numbers, first the "image"

         ipc2 = iinput( iinp + ndim + i )

         if ( ipc2<0 ) then

!        --- Indication of user that this point is image of
!            last user point

            i = nuser
            ipc2 = -ipc2
            iinput( iinp + ndim + i ) = ipc2

         end if

         ipc1 = abs(curveusrp( istart + i ))

!        --- Check whether image is correct

         do ip = 1, ndim
            xp(ip) = rinput(ip,ipc1)
         end do

         call mshsp2 ( ndim, xp, xm, dx, xn )

!        --- Check norm of point ipc2

         dist = 0d0

         do ip = 1, ndim
            xp(ip) = rinput(ip,ipc2)
            dist  = dist + xp(ip) * xp(ip)
         end do

         dist = sqrt( dist )

         if ( dist<eps ) then

!        --- Point has not yet been filled

            do ip = 1, ndim
               rinput(ip,ipc2) = xn(ip)
            end do

         else

!        --- Check whether point is image

            dist = 0d0

            do ip = 1, ndim
               xp(ip) = rinput(ip,ipc2) - xn(ip)
               dist   = dist + xp(ip) * xp(ip)
            end do

            dist = sqrt( dist )

            if ( dist>eps ) then

               call errint ( curveusrp(istart+1), 1 )
               call errint ( ipc2, 2 )
               call errint ( icurnr, 3 )
               call erreal ( xm(1), 1 )
               call erreal ( xm(2), 2 )
               if ( ndim==3 ) then
                  call erreal ( xm(3), 3 )
               else
                  call erreal ( 0d0, 3 )
               end if
               call erreal ( rinput(1,ipc2), 4 )
               call erreal ( rinput(2,ipc2), 5 )
               if ( ndim==3 ) then
                  call erreal ( rinput(3,ipc2), 6 )
               else
                  call erreal ( 0d0, 6 )
               end if
               call erreal ( xn(1), 7 )
               call erreal ( xn(2), 8 )
               if ( ndim==3 ) then
                  call erreal ( xn(3), 9 )
               else
                  call erreal ( 0d0, 9 )
               end if
               call errsub ( 964, 3, 9, 0 )
               go to 1000

            end if

         end if

!        --- Go for next check

         goto 500

      end if

1000  call erclos ( 'mshsp1' )

      end
