      subroutine mshtra ( bcorol, bcornw, nbndpt, ndim, coor, npoint, &
                          isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-05-2002 Adaptation accuracy
!        version  1.2  date 23-10-1998 Layout
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 21-01-1987
!
!   copyright (c) 1987-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array coor by translation with respect to preceding array coor
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcorol(*), bcornw(*), coor(*)
      integer nbndpt, ndim, npoint, isurnr

!     bcornw     i    array containing the co-ordinates of the new boundary
!     bcorol     i    array containing the co-ordinates of boundary of surface
!                     to be copied
!     coor      i/o   at input coor contains the co-ordinates of the surface
!                     to be copied
!                     at output coor contains the co-ordinates of the new
!                     surface
!     isurnr     i    surface number
!     nbndpt     i    Number of nodal points in bcornw
!     ndim       i    Dimension of the space.
!     npoint     i    Number of nodal points in coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx(3), x1, x2, lendx
      integer i, ic

!     dx         Translation vector
!     i          Counting variable
!     ic         Counting variable
!     lendx      Length of array dx
!     x1         coordinate of point in first curve
!     x2         coordinate of point in last curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     527   The new curves in the surface that must be constructed by
!           translation are not a translation of the curves of the surface
!           to be copied
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtra' )

!     --- compute translation

      do i = 1, ndim
         dx(i)=bcornw(i)-bcorol(i)
      end do
      lendx = 0d0
      do i = 1, ndim
         lendx = lendx + dx(i)**2
      end do
      lendx = sqrt(lendx)

!     --- Check co-ordinates of bcornw

      do ic = 2, nbndpt
         do i = 1, ndim
            x1=bcorol((ic-1)*ndim+i)+dx(i)
            x2=bcornw((ic-1)*ndim+i)
            if ( abs(x2-x1).gt.sqreps*(abs(x1)+abs(x2)+lendx) ) then
               call errint ( isurnr, 1 )
               call errint ( ic, 2 )
               call erreal ( x2-x1, 1 )
               call errsub ( 527, 2, 1, 0 )
            end if
         end do
      end do

!     --- Construct co-ordinates of new array coor by translating preceding one

      do ic = 1, npoint
         do i = 1, ndim
            coor((ic-1)*ndim+i)=coor((ic-1)*ndim+i)+dx(i)
         end do
      end do
      call erclos ( 'mshtra' )
      end
