      subroutine mshv10 ( coor, kmeshc, nelem )
! ======================================================================
!
!        programmer    Berent Wolters
!        version  1.0  date 08-12-2003  Adapted from mshv9.f
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    correct centroid position for biquadratic quadrilaterals
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     centre
!     mesh
!     mesh_generation
!     quadrilateral
!     transformation
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*)
      integer kmeshc(9,*), nelem

!     coor      i/o   array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!                     The coordinates of the centroids may be changed
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     nelem      i    number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, i8, i9
      double precision x1, y1, x2, y2, x3, y3, x4, y4, x5, y5, x6, y6, &
                       x7, y7, x8, y8

!     i              element counting variable
!     i1             First node of element
!     i2             Second node of element
!     i3             Third node of element
!     i4             Fourth node of element
!     i5             Fifth node of element
!     i6             Sixth node of element
!     i7             Seventh node of element
!     i8             Eight node of element
!     i9             Ninth node of element
!     x1             x-coordinate of node 1
!     x2             x-coordinate of node 2
!     x3             x-coordinate of node 3
!     x4             x-coordinate of node 4
!     x5             x-coordinate of node 5
!     x6             x-coordinate of node 6
!     x7             x-coordinate of node 7
!     x8             x-coordinate of node 8
!     y1             y-coordinate of node 1
!     y2             y-coordinate of node 2
!     y3             y-coordinate of node 3
!     y4             y-coordinate of node 4
!     y5             y-coordinate of node 5
!     y6             y-coordinate of node 6
!     y7             y-coordinate of node 7
!     y8             y-coordinate of node 8
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over the elements

      do i = 1, nelem

!     --- Get node numbers of element

           i1 = kmeshc(1,i)
           i2 = kmeshc(2,i)
           i3 = kmeshc(3,i)
           i4 = kmeshc(4,i)
           i5 = kmeshc(5,i)
           i6 = kmeshc(6,i)
           i7 = kmeshc(7,i)
           i8 = kmeshc(8,i)
           i9 = kmeshc(9,i)

!          --- Get point coordinates

           x1 = coor(1,i1)
           y1 = coor(2,i1)
           x2 = coor(1,i2)
           y2 = coor(2,i2)
           x3 = coor(1,i3)
           y3 = coor(2,i3)
           x4 = coor(1,i4)
           y4 = coor(2,i4)
           x5 = coor(1,i5)
           y5 = coor(2,i5)
           x6 = coor(1,i6)
           y6 = coor(2,i6)
           x7 = coor(1,i7)
           y7 = coor(2,i7)
           x8 = coor(1,i8)
           y8 = coor(2,i8)

!          --- Set centroid position by means of serendipity shape functions

           coor(1,i9) = (x2+x4+x6+x8) / 2d0 - (x1+x3+x5+x7) / 4d0
           coor(2,i9) = (y2+y4+y6+y8) / 2d0 - (y1+y3+y5+y7) / 4d0

      end do  ! i = 1, nelem

      end
