      subroutine mshv8(coor,kmeshc,nelem,npoint,npt,npti,nbndpt, &
             iwork,jwork,ibuurp,jbuure)
      implicit none
!
! ***********************************************************************
!                                                                       * &
!                                                                       * &
!         programmer     Niek Praagman                                  * &
!
!         version 2.3    date  29-06-1990  extra boundary check for midpoints
!         version 2.2    date  03-05-1990  new programming norms
!                                                                       * &
!                                                                       * &
!                                                                       * &
!   copyright (c) 1984,1990  "Ingenieursbureau SEPRA"                   * &
!   permission to copy or distribute this software or documentation     * &
!   in hard or soft copy granted only by written license obtained       * &
!   from "Ingenieursbureau SEPRA".                                      * &
!   all rights reserved. no part of this publication may be reproduced, * &
!   stored in a retrieval system (e.g. , in memory, disk or core)       * &
!   or be transmitted by any means, electronic, mechanical, photocopy,  * &
!   recording, or otherwise, without written permission from the        * &
!   publisher.                                                          * &
!                                                                       * &
! ***********************************************************************
! * &
! *   Make 8-node quadrangles out of 4-node quadrangles
! *   Create all extra internal mid points
! * &
! ***********************************************************************
! * &
! *   GLOBAL parameters
! * &
      double precision coor(*)
      integer kmeshc(*),nelem,npoint,npt,npti,nbndpt
      integer iwork(*),jwork(*),ibuurp(*),jbuure(*)
! * &
! *   Meaning of parameters
! * &
! *   coor    i,o :  array containing the coordinates of the points
! *   kmeshc  i,o :  at input array containing 3-node element-information
! *                  at output 6-node element information
! *   nelem    i  :  number of elements
! *   npoint   i  :  number of points in mesh before creation of six- &
! *                  node elements
! *   npt      i  :  at input npoint
! *                  at output total number of points in mesh after creation
! *                  of six-node elements
! *   npti     i  :  number of points on boundary +1
! *   nbndpt   i  :  number of points on boundary
! *   iwork    i  :  pointer array for neighbour nodes
! *   jwork    i  :  pointer array for neighbour elements
! *   ibuurp   i  :  array with neighbour points sequentially
! *   jbuure   i  :  array with neighbour elements sequentially
! * &
! *************************************************************************
! * &
! *   LOCAL parameters
! * &
      integer i,j,i1,i2,i3,i4,itotal,ikn,jstart,jeind,jkn,ipunt
      integer iel,ielem,jh,istart,ieind,i5,i6,i7,i8,iapp
! * &
! ***********************************************************************
! * &
! *   Run through all elements and construct new places
! * &
      do 100 i = 1 , nelem
! * &
           j   = 4 * (nelem+1-i)
! * &
           i1  = kmeshc(j-3)
           i2  = kmeshc(j-2)
           i3  = kmeshc(j-1)
           i4  = kmeshc(j)
! * &
           j   = 2 * j
! * &
           kmeshc(j-7) = i1
           kmeshc(j-6) =  0
           kmeshc(j-5) = i2
           kmeshc(j-4) =  0
           kmeshc(j-3) = i3
           kmeshc(j-2) =  0
           kmeshc(j-1) = i4
           kmeshc(j  ) =  0
! * &
100   continue
! * &
! *   Element array kmeshc ready for mid points
! * &
      itotal = npoint - 1
! * &
      do 200 ikn = 1 , itotal
! * &
! *       Run through all neighbour points
! * &
          jstart = 0
! * &
          if ( ikn.ne.1 ) jstart = iwork(ikn-1)
! * &
          jstart = jstart + 1
          jeind  = iwork(ikn)
! * &
      do 200 i = jstart , jeind
! * &
          jkn = ibuurp(i)
          if ( jkn.le.ikn ) goto 200
! * &
          ipunt  =  0
          istart =  jwork(jkn-1)+1
          ieind  =  jwork(jkn)
! * &
          iapp   =  0
! * &
! *       Compute number of appearances of line ikn - jkn
! * &
          do 105 iel = istart , ieind
! * &
              ielem = jbuure(iel)
              j     = 8 * ielem
              i1    = kmeshc(j-7)
              i2    = kmeshc(j-5)
              i3    = kmeshc(j-3)
              i4    = kmeshc(j-1)
! * &
              if ((ikn.eq.i1).and.(jkn.eq.i2)) iapp = iapp + 1
              if ((ikn.eq.i1).and.(jkn.eq.i4)) iapp = iapp + 1
              if ((ikn.eq.i2).and.(jkn.eq.i1)) iapp = iapp + 1
              if ((ikn.eq.i2).and.(jkn.eq.i3)) iapp = iapp + 1
              if ((ikn.eq.i3).and.(jkn.eq.i2)) iapp = iapp + 1
              if ((ikn.eq.i3).and.(jkn.eq.i4)) iapp = iapp + 1
              if ((ikn.eq.i4).and.(jkn.eq.i3)) iapp = iapp + 1
              if ((ikn.eq.i4).and.(jkn.eq.i1)) iapp = iapp + 1
! * &
105       continue
! * &
          do 110 iel = istart , ieind
! * &
              ielem = jbuure(iel)
              j     = 8 * ielem
              i1    = kmeshc(j-7)
              i2    = kmeshc(j-5)
              i3    = kmeshc(j-3)
              i4    = kmeshc(j-1)
! * &
              jh = 0
! * &
              if ((ikn.eq.i1).and.(jkn.eq.i2)) jh = j - 6
              if ((ikn.eq.i1).and.(jkn.eq.i4)) jh = j
              if ((ikn.eq.i2).and.(jkn.eq.i1)) jh = j - 6
              if ((ikn.eq.i2).and.(jkn.eq.i3)) jh = j - 4
              if ((ikn.eq.i3).and.(jkn.eq.i2)) jh = j - 4
              if ((ikn.eq.i3).and.(jkn.eq.i4)) jh = j - 2
              if ((ikn.eq.i4).and.(jkn.eq.i3)) jh = j - 2
              if ((ikn.eq.i4).and.(jkn.eq.i1)) jh = j
! * &
              if ( jh.eq.0 ) goto 110
! * &
! *           Fill kmeshc(jh)
! * &
              if ( jkn.ge.npti ) then
                   ipunt = 1
                   kmeshc(jh) = npt + 1
              else if ( jkn.eq.ikn+2 ) then
                   kmeshc(jh) = ikn + 1
              else
! * &
! *                Special boundary situation
! * &
                   if ( iapp.eq.1 ) then
                        kmeshc(jh) = jkn + 1
                   else
                        ipunt = 1
                        kmeshc(jh) = npt + 1
                   end if
! * &
              end if
! * &
110       continue
! * &
          if ( ipunt.eq.1 ) npt = npt + 1
! * &
200   continue
! * &
! *   Run through all elements and fill coordinate array
! * &
      do 300 i = 1 , nelem
! * &
          j   =  8 * i
! * &
          i1  =  kmeshc(j-7)
          i2  =  kmeshc(j-6)
          i3  =  kmeshc(j-5)
          i4  =  kmeshc(j-4)
          i5  =  kmeshc(j-3)
          i6  =  kmeshc(j-2)
          i7  =  kmeshc(j-1)
          i8  =  kmeshc(j  )
! * &
          coor(2*i2-1) = (coor(2*i1-1)+coor(2*i3-1))/2d0
          coor(2*i4-1) = (coor(2*i3-1)+coor(2*i5-1))/2d0
          coor(2*i6-1) = (coor(2*i5-1)+coor(2*i7-1))/2d0
          coor(2*i8-1) = (coor(2*i7-1)+coor(2*i1-1))/2d0
          coor(2*i2  ) = (coor(2*i1  )+coor(2*i3  ))/2d0
          coor(2*i4  ) = (coor(2*i3  )+coor(2*i5  ))/2d0
          coor(2*i6  ) = (coor(2*i5  )+coor(2*i7  ))/2d0
          coor(2*i8  ) = (coor(2*i7  )+coor(2*i1  ))/2d0
! * &
300   continue
! * &
      end
