      subroutine mshv9 ( coor, kmeshc, nelem, npt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 06-09-2003 Correction centroid quadrilateral
!        version  1.1  date 03-05-1990 new programming rules
!
!   copyright (c) 1984-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Place extra mid point in 8-node element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     centre
!     mesh
!     mesh_generation
!     rectangle
!     transformation
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*)
      integer kmeshc(*), nelem, npt

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
!     npt           i/o   actual number of points in the mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xs, ys
      integer i, j, i1, i2, i3, i4, i5, i6, i7, i8

!     i              Counting variable
!     i1             Node number of first node in element
!     i2             Node number of second node in element
!     i3             Node number of third node in element
!     i4             Node number of fourth node in element
!     i5             Node number of fifth node in element
!     i6             Node number of sixth node in element
!     i7             Node number of seventh node in element
!     i8             Node number of eigths node in element
!     j              Position in kmeshc
!     xs             x-coordinate of centroid
!     ys             y-coordinate of centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshv9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshv9'

      end if

!     --- For all elements do

      do i = nelem, 1, -1

!     --- Find 8 nodes of present element
!         Start with last element to prevent overwriting of elements
!         that have not been treated

         j  = 8 * i

         i1 = kmeshc(j-7)
         i2 = kmeshc(j-6)
         i3 = kmeshc(j-5)
         i4 = kmeshc(j-4)
         i5 = kmeshc(j-3)
         i6 = kmeshc(j-2)
         i7 = kmeshc(j-1)
         i8 = kmeshc(j  )

!        --- Compute coordinates of centroid

         xs = 0.5d0 * ( coor(1,i2) + coor(1,i4) + &
                        coor(1,i6) + coor(1,i8) ) - &
              0.25d0* ( coor(1,i1) + coor(1,i3) + &
                        coor(1,i5) + coor(1,i7) )
         ys = 0.5d0 * ( coor(2,i2) + coor(2,i4) + &
                        coor(2,i6) + coor(2,i8) ) - &
              0.25d0* ( coor(2,i1) + coor(2,i3) + &
                        coor(2,i5) + coor(2,i7) )

!        --- Store centroid in new point

         npt = npt + 1
         coor(1,npt) = xs
         coor(2,npt) = ys

!        --- Refill kmeshc with 9-point element

         j = 9 * i

         kmeshc(j-8)   = i1
         kmeshc(j-7)   = i2
         kmeshc(j-6)   = i3
         kmeshc(j-5)   = i4
         kmeshc(j-4)   = i5
         kmeshc(j-3)   = i6
         kmeshc(j-2)   = i7
         kmeshc(j-1)   = i8

         kmeshc(j  )   = npt

      end do

      call erclos ( 'mshv9' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshv9'

      end if

      end
