      subroutine mshz03 ( trian, npelm, nbpelm, npedge, mrec, iQ, &
                          nelem, melem, kmeshc2d, &
                          mpoint, npoint, Coor, Ref, &
                          nbcord, icurve, Tau, iblend )
! ======================================================================
!
!       programmer      Zdenek Vlasek
!       version 1.1     date 18-06-1990  comment
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990-1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     To create one element, all its boundary nodenumbers are in iQ
!     but inner nodes ( if any ) are not yet known, they must be
!     created here.
!     The orientation of boundary must be positive, &
!     but a non-natural sequence of iQ(15..) is not excluded.
!     If demanded, apply the straightening of the element edges.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      logical trian
      integer npelm, nbpelm, npedge, mrec, iq ( mrec ), &
              nelem, melem, kmeshc2d ( npelm, melem ), &
              mpoint, npoint, nbcord, icurve, iblend
      double precision coor ( 2, mpoint ), ref ( 2, mpoint ), &
                       tau ( 2, nbcord )

!     coor    i/o   Real array (2,npoint).
!                   Physical coordinates of all meshpoints.
!                   If  npoint  grows in this subroutine then
!                   the corresponding values in array Coor will be
!                   filled here
!     iblend   i    User's choice, see mshzde
!     icurve   i    User's choice, see mshzde
!     iq       i    Integer array of length mrec. This record describes
!                   the boundary of the new element in the same way as
!                   if it is a quadrilateral region ( see mshz21 ).
!                   For triangular element, the fourth edge is "empty":
!                   iQ(9)=iQ(8), iQ(13)=iQ(12) .
!     kmesh   i/o   Integer array (npelm,melem). Global nodenumbers for
!                   all elements created.  One column should be filled
!                   here.
!     melem    i    Upper bound for nelem
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     nbpelm   i    Number of element-boundary points per element
!     nelem   i/o   Current number of elements created, it should be
!                   incremented by 1 here.
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npelm    i    Number of points per element
!     npoint  i/o   Current number of all meshpoints. If the element
!                   has an inner points then  npoint  grows here.
!     ref     i/o   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!     trian    i    .true.  = triangular element
!                   .false. = quadrilateral element
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, jstore, layer, incr, iblen2, jfrom, jto, kfrom, kto, &
              nbvert, ibeg, i1, i2, j1
      double precision db, xfrom, yfrom, dx, dy, Olddis(9), sum, ds, &
                       dshort, x1, y1, x2, y2, x1sh, y1sh, x2sh, y2sh

!     db         Spacing of layers of inner points in the
!                quadrilateral element, in the reference plane
!     ds         Length of a cross-section in the triangle
!     dshort     Length of the shortes of the cross-sections
!     dx         Increment of new (straight) coordinates per
!                unit length of the old edge
!     dy         See dx
!     i          Edge of the triangle
!     i1         Edge of the triangle
!     i2         Edge of the triangle
!     ibeg       Where in iQ the edge starts
!     iblen2     The choice of mapping for inner points of the
!                element. The same as the input parameter iblend, &
!                but after straightening it is set to 0 ( local
!                mapping ).
!     incr       Number of intervals along one element edge
!     j          Local nodenumbers to determine a row in kMesh
!     j1         Local nodenumbers to determine a row in kMesh
!     jfrom      Local nodenumber of the first point of the edge
!     jstore     Local nodenumbers to determine a row in kMesh
!     jto        Local nodenumbers of the last point of the edge
!     kfrom      Global nodenumbers corresponding to jfrom, jto
!     kto        Global nodenumbers corresponding to jfrom, jto
!     layer      It counts layers of the inner points
!     nbvert     It counts number of points of the edge which are
!                on the boudary of the original quadrilateral
!     olddis     Real array of length at least npedge-1. Distance
!                measured along the edge before straightening.
!     sum        Length of the arc measured along the edge
!     x1         Physical coordinates of endpoints of cross section
!     x1sh       Physical coordinates of endpoints of the
!                shortest of the cross sections
!     x2         Physical coordinates of endpoints of cross section
!     x2sh       Physical coordinates of endpoints of the
!                shortest of the cross sections
!     xfrom      Physical coordinates of the first point of edge
!     y1         Physical coordinates of endpoints of cross section
!     y1sh       Physical coordinates of endpoints of the
!                shortest of the cross sections
!     y2         Physical coordinates of endpoints of cross section
!     y2sh       Physical coordinates of endpoints of the
!                shortest of the cross sections
!     yfrom      Physical coordinates of the first point of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshz02

!     MSHZ01    Double-faced subroutine:
!               To check an element created: angles along its boundary
!               To determine the orientation of the whole boundary
!     MSHZ02    Real function.  Distance of two meshpoints
!     MSHZ04    To display an error message and to stop the run
!     MSHZ07    Creates all new intermediate points between two given
!               points, using the mapping chosen
!     MSHZ17    Local (using the current element-boundary) interpolation
!               on the boundary
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The value nelem is incremented.
!     The given element boundary meshpoints are recorded into kMesh.
!     If icurve = 0 or 1 then the element edges are eventually
!     recalculated to become straight, the relative distances of the
!     meshpoints are kept.
!     For quadrilaterals: the inner layers of meshpoints are found.
!     For triangle: the eventual inner point is placed in the center
!     of shortest of the three cross-sections. The strategy was
!     chosen due to long curved triangles.
!     npoint, Coor and kMesh are updated.
!     The local nodes are ordered this way:
!         10----9----8----7                  7
!          |              |                 / \
!         11   15...16    6                8   6
!          |              |               /     \
!         12   13...14    5              9  10   5
!          |              |             /         \
!          1----2----3----4            1---2---3---4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

!     --- check the orientation

      if ( iQ(14)/=1 ) call mshz04 ( '# mshz03: negative orientation' )

!     --- increment nelem

      if ( nelem>=melem ) then
         call mshz04 ( 'mshz03: melem overflow' )
      else
         nelem = nelem + 1
      end if

      incr = npedge - 1

!     --- fill kMesh by nodenumbers of the element-boundary nodes
!         ( global nodenumbers in iQ may be reverse ordered )

      if ( iQ ( 6 )<iQ ( 10 ) ) then

!     --- normal order

         do 10 j = 1, nbpelm
            kmeshc2d ( j, nelem ) = iQ ( 14 + j )
   10    continue
      else

!     --- reverse order

         do 11 j = 1, nbpelm
            kmeshc2d ( j, nelem ) = iQ ( 16 + nbpelm - j )
   11    continue
      end if

!     --- don't use iQ any more in here

!         - straightening of edges

!         ( due to lack of information about an element on the other side of
!          the edge, the straightening will be done twice )

      if ( icurve<=1 ) then

!     --- all inner edges must be straight
!          when  icurve=0
!            even the edges connected to boundary must be straight

         if ( incr>1 ) then

!        --- there are inner points on the edge

!             keep the lengths of the arc for inner-edge points in
!             the array Olddis

            if ( npedge-1>9 ) &
               call mshz04 ( 'mshz03: array Olddis(9) overflow' )

!           --- loop over edges in kMesh ( *, nelem ), &
!                the first point on the edge has local nodenumber  jfrom

            do 15 jfrom = 1, nbpelm - 1, incr

!           --- local nodenumber of the endpoint of the edge

               jto = jfrom + incr

!              --- jto must be in the range 1..nbpelm

               if ( jto>nbpelm ) jto = 1

!              --- global nodenumbers

               kfrom = kmeshc2d ( jfrom, nelem )
               kto   = kmeshc2d ( jto, nelem )

!              --- count the nodes on the boundary of original
!                         quadrilateral

               nbvert = 0
               do 12 j = jfrom, jfrom + incr - 1
                  if ( kmeshc2d ( j, nelem )<nbcord ) nbvert = nbvert + 1
   12          continue
               if ( kmeshc2d ( jto, nelem )<nbcord ) nbvert = nbvert + 1

               if ( nbvert<npedge ) then

!              --- at least one node is  n o t  on the boundary

                  if ( icurve==0 .or. nbvert.eq.0 ) then

!                 --- (icurve=0) or (icurve=1 and nbvert=0 )

!                      apply straightening
!                      to keep the same density of points, &
!                      store the lenghts in Olddis

                     sum = 0d0
                     do 13 j = jfrom, jfrom + incr - 1
                        if ( j<nbpelm ) then
                           j1 = j + 1
                        else
                           j1 = 1
                        end if
                        sum = sum + &
                           mshz02 ( kmeshc2d(j,nelem), kmeshc2d(j1,nelem), &
                                    npoint, Coor )

!                       --- store the accumulated length from jfrom to j1:

                        Olddis ( j - jfrom + 1 ) = sum
  13                 continue

!                    --- now, sum is the accumulated length of the edge
!                        recalculate Coor

                     xfrom = Coor ( 1, kfrom )
                     yfrom = Coor ( 2, kfrom )

!                    --- dx,dy is new increment of x,y
!                        per unit length of the old edge

                     dx = ( Coor ( 1, kto ) - xfrom ) / sum
                     dy = ( Coor ( 2, kto ) - yfrom ) / sum

                     do 14 j = jfrom + 1, jfrom + incr - 1
                        Coor ( 1, kmeshc2d ( j, nelem ) ) = &
                           xfrom + Olddis ( j - jfrom ) * dx
                        Coor ( 2, kmeshc2d ( j, nelem ) ) = &
                           yfrom + Olddis ( j - jfrom ) * dy
   14                continue

                  end if
               end if
   15       continue
         end if
      end if

!     --- inner points
!         if npelm>6 then some inner points should be calculated, &
!         after straightening: only the local blending inside the element
!                              is reasonable

      if ( icurve<=1 ) then
         iblen2 = 0
      else
         iblen2 = iblend
      end if

      if ( npelm<=6 ) then

!     --- no inner points, no action

      else if ( .not. trian ) then

!     --- quadrilateral, npedge >= 3, incr >=2

!          numbering of inner points starts from the bottom layer, &
!          in each layer goes from left to right
!          where to store the next inner point

         jstore = nbpelm + 1

         db = 1d0 / incr

         do 30 layer = 1, incr - 1

!        --- the Fortran columnwise ordering of array kMesh is used here
!            there is no need to fill Ref for the inner points

            call mshz07 ( 0d0, db * layer, 1d0, db * layer, mrec, iQ, &
               incr, kmeshc2d ( jstore, nelem ), &
               mpoint, npoint, Coor, &
               kmeshc2d ( nbpelm + 1 - layer, nelem ), &
               kmeshc2d ( npedge + layer, nelem ), &
               Ref, nbcord, Tau, iblen2 )

            jstore = jstore + incr - 1

   30    continue

      else

!     --- a triangle with the central point
!         the solution suitable also for long curved triangles:
!         put the central point on the  s h o r t e s t
!         of the three cross sections
!         which are parallel to edges in 1/3 of height
!         initialize  i1 to point to edge 2, initialize dshort

         i1 = 2
         dshort = 1d30

         do 50 i = 1, 3

!        --- compare the cross section parallel to the edge i

!             interpolation on edge i+1

            ibeg = iQ ( 5 + i1 )

            call mshz17 ( 1d0/3d0, ibeg, ibeg + incr, mrec, iQ, &
               npoint, Coor, x1, y1 )

!           --- interpolation on edge i+2

            i2 = i1 + 1
            if ( i2==4 ) i2 = 1
            ibeg = iQ ( 5 + i2 )

            call mshz17 ( 2d0/3d0, ibeg, ibeg + incr, mrec, iQ, &
               npoint, Coor, x2, y2 )

            ds = (x2-x1)**2 + (y2-y1)**2
            if ( ds<dshort ) then
               dshort = ds
               x1sh = x1
               y1sh = y1
               x2sh = x2
               y2sh = y2
            end if

!           --- next  i1

            i1 = i2

   50    continue

         if ( npoint>=mpoint ) then
            call mshz04 ( 'mshz07: mpoint overflow' )
         else
            npoint = npoint + 1
         end if

         Coor ( 1, npoint ) = ( x1sh + x2sh ) * 0.5d0
         Coor ( 2, npoint ) = ( y1sh + y2sh ) * 0.5d0

!        --- there is no nedd to fill Ref

         kmeshc2d ( npelm, nelem ) = npoint

      end if

!     --- check the element boundary inner angles and their sum

      call mshz01 ( nelem, nbpelm, incr, npelm, &
         kmeshc2d ( 1, nelem ), npoint, Coor )

      end
