      subroutine mshz08 ( a, b, mpoint, npoint, Coor, Ref, &
         nbcord, Tau, mrec, iQ )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   When npoint<0 : initialization of values to be saved.
!
!   When npoint>=0: to create one new meshpoint
!                   corresponding to point [a,b] in the reference plane, &
!                   when the  g l o b a l   mapping is used.
!
!                   Results: updated  npoint, Coor, Ref
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      implicit none
      integer mpoint, npoint, mrec, nbcord, iQ ( mrec )

      double precision a, b, Coor ( 2, mpoint ), Ref ( 2, mpoint ), &
         Tau ( 2, nbcord )

!     a, b     i    Global reference coordinates of the point
!     Coor     io   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iQ       i    Integer array of length mrec. This record describes
!                   boundary of the quadrilateral region ( see mshz21 ).
!                   It is needed during the initialization only. For
!                   non-negative npoint: iQ is ignored
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     npoint   io   Current number of all meshpoints
!     Ref      io   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     Tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!
! **********************************************************************
!
!                  LOCAL VARIABLES

!--- saved local, they keep information on the original quadrilateral
!                 and on the last found boundary intervals
!                 ( usually the next [a,b] is close to previous [a,b], &
!                   which helps to accelerate the search )

      integer kbeg1, kbeg2, kbeg3, kbeg4, kend1, kend2, kend3, kend4
      save    kbeg1, kbeg2, kbeg3, kbeg4, kend1, kend2, kend3, kend4

      double precision x00, x10, x11, x01, y00, y10, y11, y01
      save             x00, x10, x11, x01, y00, y10, y11, y01

!--- local, not saved

      double precision a1, b1, xi0, xi1, xj0, xj1, yi0, yi1, yj0, yj1, &
         tau0, tau1, sig0, sig1, &
         dum, aa, bb

!     a1, b1      1-a resp. 1-b
!     aa, bb      Modified reference coordinates
!     dum         Dummy variable
!     kbeg1, kbeg2, kbeg3, kbeg4   Global nodenumbers of the beginning
!                                  of the s/e/n/west edge of iQ
!     kend1, kend2, kend3, kend4   Global nodenumbers of the end
!                                  of the s/e/n/west edge of iQ
!     x00, y00    Physical coordinates corresponding to ref.coord. [0,0]
!     x01, y01    Physical coordinates corresponding to ref.coord. [0,1]
!     x11, y11    Physical coordinates corresponding to ref.coord. [1,1]
!     x10, y10    Physical coordinates corresponding to ref.coord. [1,0]
!     xi0, yi0    Physical coordinates coresponding to [a,0]
!     xi1, yi1    Physical coordinates coresponding to [a,1]
!     xj0, yj0    Physical coordinates coresponding to [0,b]
!     xj1, yj1    Physical coordinates coresponding to [1,b]
!     tau0, tau1, sig0, sig1       Reference noneqidistant coordinates, &
!                                  on boundary of "Tau", &
!                                  see mshzde, iblend, case iblend=1
!
! **********************************************************************
!
!                  SUBROUTINES CALLED
!
!     mshz04:   To display an error message and to stop the run
!     mshz09:   Global ( using the original boundary ) interpolation, &
!               with the option: one of coordinates or both.
!
! **********************************************************************
!
!                  ERROR MESSAGES are self-explaining
!
! **********************************************************************
!
!                  METHOD
!
! (i) Non-modified interpolation by blending:
!
!     reference plane: a,b           physical plane: z=x+i*y
!     -------------------- ----------------------- &
!
!      0,1-----a,1-----1,1           z(k01)---zi1------z(k11)
!       |               |             |                 |
!      0,b     a,b     1,b           zj0      zab=?    zj1
!       |               |             |                 |
!      0,0-----a,0-----1,0           z(k00)---zi0------z(k10)
!
!     The sum of linear interpolation in a-direction and b-direction, &
!     minus the bilinear interpolation defined by mapping of corners
!
!     x(a,b) = combination of ( 1, a, b, a*a, a*b, b*b )
!     y(a,b) = combination of ( 1, a, b, a*a, a*b, b*b )
!
!     The coefficients depend   n o n l i n e a r i l y   on a,b
!         ( nonlinearity comes via  zi0, zi1, zj0, zj1 )
!
!     The interpolation is exact on the boundary and in the interior
!     is as smooth as the boundary mapping.
!
!     Algorithm:  a1 = 1 - a
!                 b1 = 1 - b
!
!                 zab = + z(j0) * a1
!                       + z(j1) * a
!                       + z(i0) * b1
!                       + z(i1) * b
!                       - ( z(k00) * b1 + z(k01) * b ) * a1
!                       - ( z(k10) * b1 + z(k11) * b ) * a
!
! (ii)  Approach used in other Sepran parts, adviced by Guus:
!
!     The modified method tries to improve the results if the boundary
!     points on the opposite sides in the physical plane are distributed
!     non-synchonously, like "xy" in the following:
!
!          x     x   x x       x x x x       x  x xx
!          x           x       x     x       x     x
!          x           x       x     x       x     x
!          x  x  x     x       x x x x       xx x  x
!
!              "xy"             "Ref"         "Tau"
!
!     The boundary nodes in "xy" are mapped onto the reference
!     rectangle "Tau"   n o n e q u i d i s t a n t l y, &
!     proportionaly to the length of arc.
!
!     At the same time they are mapped onto the
!     reference rectangle "Ref" equidistantly, then
!     the Coon's formula ( see above ) is used to map "Ref" into the
!     physical "xy" plane, but the coefficients  a, b, a1=1-a, b1=1-b
!     are pre-transformed this way:
!
!     For given position [a,b] in "Ref" we find the
!     four projections onto the boundary in "Ref"
!     [a,    0], [a,    1], [0,    b], [1,    b]
!     and then the four corresponding points
!     [tau0, 0], [tau1, 1], [0, sig0], [1, sig1]
!     on the boundary in "Tau", &
!           ( it may happen tau0<>tau1, sig0<>sig1 ), &
!     now in "Tau" we connect the opposite points by
!     straight lines and the coordinates [ aa, bb ]
!     of their intersection point in "Tau"
!     are used in Coon's formula instead of the [a,b].
!
! **********************************************************************
!

      if ( npoint.lt.0 ) then

!       * initialize values to be saved, nothing else
!       * in iQ must be the   g l o b a l   quadrilateral

!       * nodenumbers for beginnings and ends of edges, &
!        points in  iQ, Coor, Ref  are stored in natural order

         kbeg1 = 1
         kbeg2 = iQ ( iQ ( 7 ) )
         kbeg3 = iQ ( iQ ( 8 ) )
         kbeg4 = iQ ( iQ ( 9 ) )

         kend1 = kbeg2
         kend2 = kbeg3
         kend3 = kbeg4

!       * the following  kend4  differs from kbeg1, to have each edge
!       * as one sub-array, &
!       * the "useless" nodenumber  kend4  helps in interpolations
         kend4 = abs (npoint)

!       * coordinates of the corners in the physical plane

         x00 = Coor ( 1, kbeg1 )
         y00 = Coor ( 2, kbeg1 )

         x10 = Coor ( 1, kbeg2 )
         y10 = Coor ( 2, kbeg2 )

         x11 = Coor ( 1, kbeg3 )
         y11 = Coor ( 2, kbeg3 )

         x01 = Coor ( 1, kbeg4 )
         y01 = Coor ( 2, kbeg4 )

      else

!       * use saved values, do the blending interpolation

         a1 = 1d0 - a

         call mshz09 ( a,  kbeg1, kend1, npoint, Coor, 0, xi0, yi0 )
         call mshz09 ( a1, kbeg3, kend3, npoint, Coor, 0, xi1, yi1 )

         call mshz09 ( a,  kbeg1, kend1, nbcord, Tau, 1, tau0, dum )
         call mshz09 ( a1, kbeg3, kend3, nbcord, Tau, 1, tau1, dum )

         b1 = 1d0 - b

!       * the equidistant linear interpolation on edge 4 and 2 of "Ref"
!       *                 to get xj0, yj0, xj1, yj1, sig0, sig1

         call mshz09 ( b1, kbeg4, kend4, npoint, Coor, 0, xj0, yj0 )
         call mshz09 ( b,  kbeg2, kend2, npoint, Coor, 0, xj1, yj1 )

         call mshz09 ( b1, kbeg4, kend4, nbcord, Tau,  2, dum, sig0 )
         call mshz09 ( b,  kbeg2, kend2, nbcord, Tau,  2, dum, sig1 )

!       * preprocessing ( Guus ), transformes [a,b] to [aa,bb], &
!       * which is intersection of the two lines

         dum = sig1 - sig0
         bb = ( tau0 * dum + sig0 ) / ( 1d0 + ( tau0 - tau1 ) * dum )
         aa = bb * tau1 + ( 1d0 - bb ) * tau0

!       * check if we are close to the rectangle "Ref"

         if ( min ( aa, bb ).lt.-1d-6 .or. &
              max ( aa, bb ).gt.1d0+1d-6 ) &
            call mshz04 ( '# mshz08:  aa  or  bb  out of range' )

!       * eliminate numerical errors

         aa = max ( 0d0, min ( 1d0, aa ) )
         bb = max ( 0d0, min ( 1d0, bb ) )

         a1 = 1d0 - aa
         b1 = 1d0 - bb

         if ( npoint.ge.mpoint ) then
            call mshz04 ( 'mshz08: mpoint overflow' )
         else
            npoint = npoint + 1
         end if

!       * store new meshpoint

         Coor ( 1, npoint ) = &
                xj0 * a1  +  xj1 * aa  +  xi0 * b1  +  xi1 * bb &
            - ( x00 * b1  +  x01 * bb ) * a1 &
            - ( x10 * b1  +  x11 * bb ) * aa

         Coor ( 2, npoint ) = &
                yj0 * a1  +  yj1 * aa  +  yi0 * b1  +  yi1 * bb &
            - ( y00 * b1  +  y01 * bb ) * a1 &
            - ( y10 * b1  +  y11 * bb ) * aa

         Ref ( 1, npoint ) = a
         Ref ( 2, npoint ) = b

      end if

      end
