      subroutine mshz09 ( a, kbeg, kend, npoint, Coor, ivar, x, y )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   Interpolation on the boudary of the original quadrilateral
!
!   The get one or both components of the point [x,y] corresponding
!   to the reference point placed in the relative distance "a" on the
!   edge of the reference rectangle. The edge consists of meshpoints
!   numbered from kbeg to kend, they are supposed to be distributed
!   on the reference rectangle equidistantly.
!
!   Piecewise linear interpolation used.
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      integer kbeg, kend, npoint, ivar
      double precision a, Coor ( 2, npoint ), x, y

!     a        i    Relative position on the edge:   a=0 cooresponds
!                   to the meshpoint  kbeg, a=1 corresponds to  kend
!     Coor     i    Real array (2,npoint).
!                   Physical coordinates of meshpoints, both kbeg and
!                   kend  must be in the range 1..npoint
!     ivar     i    Choice:  ivar=0  ... find both x and y
!                                 1  ... find x only
!                                 2  ... find y only
!     kbeg,kend  i  Nodenumbers of endpoints.
!     npoint   i    Current number of all meshpoints
!     x, y     o    Coordinates in the physical plane, one of them may
!                   stay unfilled, depending on ivar
!
! **********************************************************************
!
!                  LOCAL VARIABLES

      integer kleft, kright
      double precision exactk, rest

!     kleft, kright   Meshpoints defining the subinterval for the
!                     linear interpolation
!     exactk          Real "meshpoint" cooresponding to "a", not the
!                     whole number
!     rest            exactk - kleft
!
! **********************************************************************
!

      exactk = dble ( kbeg ) + a * ( kend - kbeg )

!    * normal ordering

      kleft = min ( kend - 1, max ( kbeg, int ( exactk ) ) )
      kright = kleft + 1
      rest = exactk - kleft

!    * calculate the relative position between  kleft, kright

      if ( ivar.ne.2 )&
         x = Coor ( 1, kleft ) + &
            rest * ( Coor ( 1, kright ) - Coor ( 1, kleft ) )
      if ( ivar.ne.1 )&
         y = Coor ( 2, kleft ) + &
            rest * ( Coor ( 2, kright ) - Coor ( 2, kleft ) )

      end
