      subroutine mshz10 ( mrec, iQ, Coor, Ref, &
                          nbcord, Tau )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   To initialize the boundary values in Ref(2,nbcord), Tau(2,nbcord).
!   It is called from mshzde.
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      integer mrec, iQ ( mrec ), nbcord
      double precision Coor ( 2, nbcord ), Ref ( 2, nbcord ), &
                       Tau ( 2, nbcord )

!     Coor     i    Real array (2,nbcord).
!                   Physical coordinates of user's boundary points
!                   ( the point number nbcord is the same as number 1, &
!                     but its nodenumber is not used in kMesh )
!     iQ       i    Integer array of length mrec.  The record describes
!                   boundary of the original quadrilateral region
!                   ( see mshz21 ).
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!                   including the intermediate points, &
!                   including the last point equal to the first
!                   nbcord = (n1+n2+n3+n4)*(npedge-1) + 1
!     Ref      o    Real array (2,nbcord). Reference coordinates of
!                   user's boundary points, see mshz21.
!     Tau      o    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!
! **********************************************************************
!
!                  LOCAL VARIABLES

      integer i, ifix, ivar, jfrom, jto, jstep, j, k, k1
      double precision const, h, sum

!     const        Value of the fixed variable on the current edge
!     h            Step in the equidistant case
!     i            Subscript
!     ifix, ivar   Which coordinate is fixed and which is variable
!                  on the current edge of the reference rectangle
!     j, jfrom, jto   Pointers into the record iQ
!     jstep        Step ( +-1 ) in j-loops
!     k, k1        Global nodenumbers
!     sum          Accumulated length of the edge in the physical plane
!
! **********************************************************************
!
!                  SUBROUTINES CALLED
!
!     mshz04   To display an error message and to stop the run
!
! **********************************************************************
!
!                  ERROR MESSAGES are self-explaining
!
! **********************************************************************
!
!-- initialize Ref (*,*), nodes are equidistant
!-- initialize Tau (*,*), nodes are distributed proportionaly

!--- initialize south ( b=0 ), east ( a=1 ), north ( b=1 ), west ( a=0 )
!               i=1            i=2           i=3            i=4
!
!    * fill the "useless" point

      Ref ( 1, nbcord ) = 0d0
      Ref ( 2, nbcord ) = 0d0
      Tau ( 1, nbcord ) = 0d0
      Tau ( 2, nbcord ) = 0d0

      do 300 i = 1, 4

         if ( i.eq.1 .or. i.eq.3 ) then
!          * 2nd coordinate is constant: b = 0 resp 1
            ifix = 2
            ivar = 1
         else
!          * 1st coordinate is constant: a = 1 resp. 0
            ifix = 1
            ivar = 2
         end if

         if ( i.eq.1 .or. i.eq.4 ) then
!          * south or west, the constant is  0
            const = 0d0
         else
            const = 1d0
         end if

!       * pointers into  iQ  for endpoints

         if ( i.eq.1 .or. i.eq.2 ) then

!          * the variable coordinate is increasing
            jfrom = iQ ( 5 + i )
            jto = iQ ( 9 + i )
            jstep = 1
         else

!          * the variable coordinate is decreasing
!                            due to the anti-clockwise orientation
!          * be carefull: the order of calculation will be reverse
!                            if compared with the order in which
!                            these points are stored in Coor

            jfrom = iQ ( 9 + i )
            jto = iQ ( 5 + i )
            jstep = -1
         end if

!       * fill Ref, but not the endpoint ( to avoid 1 +- epsilon)
!        ( point [1,1] as endpoint for both north and east in Ref
!          will be missing, we do it now )

         if ( i.eq.2 ) then
            k = iQ ( jto )
            Ref ( 1, k ) = 1d0
            Ref ( 2, k ) = 1d0
            Tau ( 1, k ) = 1d0
            Tau ( 2, k ) = 1d0
         end if

!       * nodes in Ref are equidistant

         h = 1d0 / dble ( abs ( jfrom - jto ) )

         do 50 j = jfrom, jto - jstep, jstep
!          * pointer into Ref
            k = iQ ( j )
            Ref ( ifix, k ) = const
            Ref ( ivar, k ) = dble ( abs ( j - jfrom ) ) * h
   50    continue

!       * nodes are not equidistant in Tau plane, &
!           spacing is proportional to the length of arc

!       * in  sum  we accumulate length of the arc

         sum = 0d0

         do 100 j = jfrom, jto - jstep, jstep

!          *  pointer into  Tau, fill Tau
            k = iQ ( j )
            Tau ( ifix, k ) = const
            Tau ( ivar, k ) = sum

!          * new sum, from this point to the next point
            k1 = iQ ( j + jstep )
            sum = sum + &
               sqrt ( ( Coor ( 1, k1 ) - Coor ( 1, k ) ) ** 2 &
                    + ( Coor ( 2, k1 ) - Coor ( 2, k ) ) ** 2 )
  100    continue
!       * keep  sum  for normalization

!       * normalize the length for inner points, &

         if ( sum.le.0d0 ) &
            call mshz04 ( 'mshz10: zero length of one of the sides' )

         do 150 j = jfrom + jstep, jto - jstep, jstep
            k = iQ ( j )
            Tau ( ivar, k ) = Tau ( ivar, k ) / sum
  150    continue

  300 continue

      end
