      subroutine mshz12 ( new, trian, mrec, iq, ir, is, it, iu, &
                          npelm, npedge, mpoint, melem, &
                          npoint, nelem, coor, ref, kmeshc2d, &
                          nbcord, icurve, tau, iblend )
! ======================================================================
!
!       programmer      Zdenek Vlasek
!       version 2.0     date 31-07-1990  parameters n1..n4 for mshz23
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990-1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Submesh generator for 2-dim quadrilateral region in some special
!   cases of n1, n2, n3, n4, when the mesh can be found directly.
!   It is called from mshz21 or from mshz25.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new, trian
      integer  mrec, iq ( mrec ), ir ( mrec ), is ( mrec ), &
               it ( mrec ), iu ( mrec ), &
               npelm, npedge, mpoint, melem, npoint, nelem, &
               kmeshc2d ( npelm, melem ), &
               nbcord, icurve, iblend
      double precision coor ( 2, mpoint ), ref ( 2, mpoint ), &
                       tau ( 2, nbcord )

!     coor    i/o   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iblend   i    User's choice, see mshzde
!     icurve   i    User's choice, see mshzde
!     iq            Integer  s c r a t c h  array of length mrec.
!                   The record must be initialized by calling subroutine
!                   to describe boundary of a quadrilateral region
!                   ( see mshz21 ), the region will be subdivided here.
!                   Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!                   On return, the record iQ may contain a garbage.
!     ir            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines
!     is            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines
!     it            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines
!     iu            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines
!     kmeshc2d   i/o   Integer array (npelm,melem). Global nodenumbers for
!                   all elements created.
!     melem    i    Upper bound for nelem
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     nelem   i/o   Current number of elements created
!     new      i    .true.  = new topology of the mesh
!                   .false. = user supports an old topology of the mesh
!                             in the array  kMesh, untouched !!!
!                             ( this option is not available for
!                               quadrilateral elements )
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npelm    i    Number of points per element
!     npoint  i/o   Current number of all meshpoints
!     ref     i/o   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!     trian    i    .true.  = triangular elements
!                   .false. = quadrilateral elements
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical part4
      integer n1, n2, n3, n4, n5, n6, incr, i, &
              isg, k1, k2, inew, ito1, ito2, itemp
      double precision ak1, ak2, bk1old, bk1, bk2, height, b1, b2, b
      double precision, parameter :: pihalf=0.5d0*3.14159265358979d0

!     ak1        Reference coordinates of the new vertex  k1
!     ak2        Reference coordinates of the new vertex  k2
!     b          Temporary values used in the iterative solution of the
!                nonlinear equation to optimize angles around node k1
!     b1         See b
!     b2         See b
!     bk1        Reference coordinates of the new vertex  k1
!     bk1old     Second reference coordinate of the new vertex  k1, before
!                bk1 was changed to achieve better angles around k1
!     bk2        Reference coordinates of the new vertex  k2
!     height     Estimation of height of the quadrilateral iQ in the
!                physical plane ( heuristic )
!     i          Counting variable
!     incr       Number of intervals along one element edge
!     inew       Current position when iR or iS or iT or iU is filled
!     isg        Signum of iQ(10)-iQ(6), the loopstep in many loops
!     itemp      Position in iQ of a temporary interesting node
!     ito1       Positions in iQ of the two split-nodes, namely the
!                left and right end of the north edge of iT, see diagrams
!     ito2       See ito1
!     k1         Global nodenumbers of new vertices, see diagrams below
!     k2         See k1
!     n1         Number of elements along the  south  side
!                of the current quadrilateral
!     n2         Number of elements along the  east  side
!                of the current quadrilateral
!     n3         Number of elements along the  north side
!                of the current quadrilateral
!     n4         Number of elements along the  west side
!                of the current quadrilateral
!     n5         Number of elements along edge 4 of iR, see diagrams below
!     n6         Number of elements along edge 3 of iS, see diagrams below
!     part4      .true.   when the fourth part "iU" will be created
!                .false.  when the are only three parts
!     pihalf     pi / 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshz02
      double precision fun
      fun ( b ) = pihalf + atan ( ( bk1old - b ) * height / ( 1d0 - ak1 ) ) &
                  - 2d0 * atan ( b * height / ak1 )

!     FUN       LOCAL FUNCTION
!               special-purpose function used to find a better position of the
!               common vertex of several sub-regions
!     MSHZ02    Real function.  Distance of two meshpoints
!     MSHZ04    To display an error message and to stop the run
!     MSHZ07    Creates all new intermediate points between two given
!               points, using the mapping chosen
!     MSHZ16    Local ( using the boundary of iQ ) blending transform
!     MSHZ22    To divide a quadrilateral region regularily, &
!               into quadrilaterals, applicable when n2=n4 and n1=n3, &
!               into triangles,      applicable when n2=n4
!     MSHZ23    To change the inner pointers in the record like iQ, &
!               due to the change of the orientation, change iQ(14), &
!               the first vertex remains the same
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   PREPOSITIONS
!   ------------ &
!   if called from mshz25:
!                           n1 = n4 = 1,  1 <  n2 <= n3
!   if called from mshz21:
!                           n1 = n4 = 1,  1 <= n2 <= n3,  n3 >= 2
!                      or:  n4 = n2 = 1,  2 <= n1 < n3
!                      or ( an "easy" algorithm ):
!                           n1 <= n3,  n4 <= n2,  n2-n4 <= n3-n1, &
!                           (n1<n3) or ( n4<n2 )
!   notice, that in all calls from mshz25/QuadriE there is:
!           n1 <= n3,  n4 <= n2,  n2-n4 <= n3-n1,  (n1<n3) or ( n4<n2 )
!   METHOD for QUADRILATERAL ELEMENTS
!   --------------------------------- &
!   Set  na = n2-n4,                                   it is na>=0, &
!        nb = (n3-n1-na)/2 = ( (n3-n1)-(n2-n4) ) / 2,  it is nb>=0, &
!   at least one of na, nb is positive.
!   The following diagram shows how the divide the quadrilateral
!   into 6 parts ( minus two parts when na=0,  minus three when nb=0 ), &
!   each of them can be farther divided regularily.
!             |----na---|----nb---|----n1-----|---nb----|
!             |         |         |           |         |
!             |         |         |           |         |
!             |         |         n4          n4        |
!             |         n4        |           |         n4
!             |         |         |----n1-----|         |
!             |         |       /               \       |
!             n4        |    nb                   nb    |
!             |         |   /                       \   |
!             |         | /                           \ |
!             |         |--------------n1---------------|
!             |       /                                 |
!             |     /                                   |
!             |   na                                    na
!             | /                                       |
!             |------------------------n1---------------|
!   The aproach is not suitable when nb is much larger then n2, &
!   ( e.g. n1=10, n2=1, n3=40, n4=1  ==>  nb=15 ), because
!   in this case it produces two many horizontal layers. Usually, &
!   a better solution is first to split the south and the north side
!   by mshz24, which results in a recursive algorithm.
!   Here, in mshz12, the subdivision described above will be done
!   in no more then two steps.
!         case n2=n4:   ( it is  na = 0 )
!                       then n3-n1 >= 2, n3-n1 is even )
!                       set n5 = (n3-n1)/2  (=nb), &
!                       split  iQ  into four parts stored in
!                       iR, iS, iT, iU ( see the diagram below ), &
!                       each of them can be subdivided regularily, &
!                       by mshz22:
!                       |----n5---|----n1-----|---n5----|
!                       |         |           |         |
!                       |         |   =iT     |         |
!                       |         n2          n2        |
!                       |   =iR   |           |   =iU   |
!                   n4=n2        (k1)--n1----(k2)       n2
!                       |       /               \       |
!                       |    n5                   n5    |
!                       |   /         =iS           \   |
!                       | /                           \ |
!                       |--------------n1---------------|
!         case n2>n4:   set n5=n2-n4 (=na), n6=n1,  ( n6 is introduced
!                           for compatibility with triangular elements )
!                       split iQ into three parts stored in iR, iS, iT, &
!                       iR, iS can be subdivided regularily by mshz22, &
!                       iT can be subdivided again here ( if n3-n5>1 )
!                               or regularily by mshz22 ( if n3-n5=1 ):
!                       |----n5---|------(n3-n5)--------|
!                       |         |                     |
!                       |         n4       =iT          n4
!                       |   =iR   |                     |
!                       |         |                     |
!                       n4       (k1)-----n6=n1--------(k2)
!                       |       /                       |
!                       |    n5                         |
!                       |   /           =iS             n5=n2-n4
!                       | /                             |
!                       |---------------n1--------------|
!   METHOD for TRIANGULAR ELEMENTS
!   ------------------------------ &
!         case  n2=n4:  it does not happen, &
!         case  n2>n4:  let n5 = n2-n4, &
!                       let n6 = something between n1 and n3-n5
!                              ( a suitable interpolation is used ), &
!                       split  iQ  in three parts stored in iR, iS, iT, &
!                       each can be subdivided regularily by mshz22:
!                       |----n5---|------(n3-n5)--------|
!                       |         |                     |
!                       |         n4       =iT          n4
!                       |   =iR   |                     |
!                       |         |                     |
!                       n4       (k1)-------n6---------(k2)
!                       |       /                       |
!                       |    n5                         |
!                       |   /           =iS             n5=n2-n4
!                       | /                             |
!                       |---------------n1--------------|
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill  incr

      incr = npedge - 1

!     --- loop using "goto 10"
!         the control is returned here only when the quadrilateral  iT
!         should be divided also by this algorithm, &
!         in such case the record iQ was filled by iT, before the jump

   10 continue

!     --- fill n1, n2, n3, n4

         n1 = iQ(2)
         n2 = iQ(3)
         n3 = iQ(4)
         n4 = iQ(5)

!        --- estimate the relative height of the iQ, in the physical plane

         height = ( mshz02 ( iQ(iQ(7)), iQ(iQ(11)), npoint, Coor ) &
                  + mshz02 ( iQ(iQ(9)), iQ(iQ(13)), npoint, Coor ) ) &
                / ( mshz02 ( iQ(iQ(6)), iQ(iQ(10)), npoint, Coor ) &
                  + mshz02 ( iQ(iQ(8)), iQ(iQ(12)), npoint, Coor ) )

!        --- replace a low symmetric quadrilateral iQ    |----|
!            by high quadrilateral:      |--------|      |    |
!                                        |        | ---> |    |
!                                        |--------|      |----|
!            the reason: it becomes easier to place the new vertex k1
!                       into the interior of iQ

         if ( height<1d0 &
           .and. iQ(2)==iQ(5) .and. iQ(3).eq.iQ(4) ) then
            call mshz23 ( iQ, n1, n2, n3, n4 )
            height = 1d0 / height
         end if

!        --- remember order in which nodenumbers are stored in iQ

         isg = sign ( 1, iQ ( 10 ) - iQ ( 6 ) )

!        --- four parts or three parts ?

         part4 = .not. trian .and. n2==n4

!        --- number of elements on edge 4 of iR

         if ( n4==n2 ) then
            n5 = ( n3 -n1 ) / 2
         else
            n5 = n2 - n4
         end if

!        --- pointers  ito1, ito2  point into iQ, &
!            they define how to split the north edge of iQ:
!                       iQ(12)----------ito1-----------ito2--------iQ(8)
!                         |               |              |           |
!            or
!                       iQ(12)----------ito1---------------------ito2=iQ(8
!                         |               |                          |

         ito1 = iQ ( 12 ) - n5 * isg * incr
         if ( part4 ) then
            ito2 = ito1 - n1 * isg * incr
         else
            ito2 = iQ ( 8 )
         end if

!        --- p r o b l e m :  what looks nice in the diagrams above
!                             ( and even in the reference plane "ab" )
!                             can result in bad angles in the physical
!                             plane "xy"
!            what follows is a heuristic attempt to place the point k1 well
!            first proposal of the relative position of new point k1 in iQ

         if ( n5==0 ) call mshz04 ( '# Irreg: called for n5 = 0' )

         ak1 = dble ( n5 ) / dble ( n3 )

         if ( part4 ) then

!        --- any  bk1  is possible
!            first attempt: layers in iS and iT have the same thickness

            bk1old = dble ( n5 ) / dble ( n5 + n4 )

!           --- correction, if the quadrilateral iQ is too high, &
!               an attempt to create angles 135 + 135 + 90 degrees

            bk1 = min ( bk1old, ak1 / height )

         else

!        --- proposed point k1 will be connected with existing point k2
!            which is on edge 2 of  iQ, &
!            too low  bk1  is unacceptable ( bad  iS )

!             first attempt: layers in  iS  are parallel to a-ax

            bk1old = dble ( n2 - n4 ) / dble ( n2 )
            bk1 = bk1old

!           --- correction of bk1  to achieve better angles

            if ( height>=1d0 ) then

               if ( atan ( bk1old * height / ak1 )>0.5d0 * pihalf ) then

!              --- solve  fun ( bk1 ) = 0  on [ ak1*bk1old, bk1old]
!                         by bisection,  fun is decreasing

                  b1 = bk1old * ak1
                  b2 = bk1old
                  if ( fun ( b1 )<0d0 .or. fun ( b2 )>0d0 ) &
                     call mshz04 ( '# mshz12: function fun is wrong' )

   15             if ( b2 - b1>1d-3 ) then
                     b = ( b1 + b2 ) * 0.5d0
                     if ( fun ( b )<0d0 ) then
                        b2 = b
                     else
                        b1 = b
                     end if
                     goto 15
                  end if

                  bk1 = b1

               end if

            end if

         end if

!        --- finaly, create the new point, remember its nodenumber  k1

         call mshz16 ( ak1, bk1, mrec, iQ, .true., &
                       mpoint, npoint, Coor )

         if ( mod ( iblend, 2 )==1 ) then

!        --- fill also Ref  ( danger: mshz16 increments npoint again )

            npoint = npoint - 1
            call mshz16 ( ak1, bk1, mrec, iQ, .true., &
                          mpoint, npoint, Ref )
         end if

         k1 = npoint

!        --- first part: fill iR, see diagrams above
!            the ends of the diagonal are iQ(6), k1

         iR ( 2 ) = n4
         iR ( 3 ) = n5
         iR ( 4 ) = n4
         iR ( 5 ) = n5
         iR ( 14 ) = iQ ( 14 )

         inew = 14

!        --- edge 1 of iR is reverse edge 4 of iQ, copy also both ends

         iR ( 6 ) = 15

         do 20 i = iQ ( 9 ), iQ ( 13 ), isg
            inew = inew + 1
            iR ( inew ) = iQ ( i )
   20    continue

         iR ( 10 ) = inew

!        --- edge 2 of iR is new, create inner points

         iR ( 7 ) = inew

         if ( n5 * incr>1 ) then
            call mshz07 ( 0d0, 0d0, ak1, bk1, mrec, iQ, &
               n5 * incr, &
               iR ( inew + 1 ), mpoint, npoint, Coor, &
               iQ ( iQ ( 6 ) ), k1, Ref, nbcord, Tau, iblend )

            inew = inew + n5 * incr - 1
         end if

!        --- put  k1  into iR

         inew = inew + 1
         iR ( inew ) = k1

         iR ( 11 ) = inew

!        --- edge 3 of iR is new, from  k1  to  ito1, &
!            create inner points, if any

         iR ( 8 ) = inew

         if ( n4 * incr>1 ) then
            call mshz07 ( ak1, bk1, ak1, 1d0, mrec, iQ, &
               n4 * incr, iR ( inew + 1 ), mpoint, npoint, Coor, &
               k1, iQ ( ito1 ), Ref, nbcord, Tau, iblend )
            inew = inew +  n4 * incr - 1
         end if

         iR ( 12 ) = inew + 1

!        --- edge 4 of iR is reverse part of edge 3 of iQ, &
!            copy also both ends

         iR ( 9 ) = iR ( 12 )

         do 30 i = ito1, iQ ( 12 ), isg
            inew = inew + 1
            iR ( inew ) = iQ ( i )
   30    continue

         iR ( 13 ) = inew
         iR ( 1 ) = inew

!        --- subdivide  iR  now, scratch arrays iS, iT are empty
!            keep  iR  untouched for filling of iS and iT !!!

         call mshz22 ( new, trian, mrec, iR, iS, iT, &
                  npelm, npedge, &
                  mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                  nbcord, icurve, Tau, iblend )

!        --- second part: fill iS, see diagrams above

         iS ( 2 ) = n1
         iS ( 3 ) = n5

         if ( trian ) then

!        --- select number of elements along edge 3 of iS
!            proportional to the avarage of old bk1 and new bk1

            n6 = 1 + &
               int ( 0.5d0 * ( bk1old + bk1 ) * ( n3 - n2 + 1 ) )
         else
            n6 = n1
         end if

         iS ( 4 ) = n6
         iS ( 5 ) = n5
         iS ( 14 ) = iQ ( 14 )

         inew = 14

!        --- edge 1 of iS is edge 1 of iQ, copy also both ends

         iS ( 6 ) = 15

         do 40 i = iQ ( 6 ), iQ ( 10 ), isg
            inew = inew + 1
            iS ( inew ) = iQ ( i )
   40    continue

         iS ( 10 ) = inew

!        --- edge 2 of iS

         iS ( 7 ) = inew

         if ( part4 ) then

!        --- edge 2 is new, create the endpoint, &
!            remember in  k2  its nodenumber
!            and in  ak2, bk2  its local position

            ak2 = 1d0 - ak1
            bk2 = bk1

            call mshz16 ( ak2, bk2, mrec, iQ, .true., &
                          mpoint, npoint, Coor )

            if ( mod ( iblend, 2 )==1 ) then

!           --- fill also Ref  ( danger: mshz16 increments npoint )

               npoint = npoint - 1
               call mshz16 ( ak2, bk2, mrec, iQ, .true., &
                             mpoint, npoint, Ref )
            end if

            k2 = npoint

!           --- pointers in iQ to the ends of diagonal are iQ(7), k2

            if ( n5 * incr>1 ) then
               call mshz07 ( 1d0, 0d0, ak2, bk2, mrec, iQ, &
                  n5 * incr, &
                  iS ( inew + 1 ), mpoint, npoint, Coor, &
                  iQ ( iQ ( 7 ) ), k2, Ref, nbcord, Tau, iblend )

               inew = inew + n5 * incr - 1
            end if

!           --- put  k2  into iS

            inew = inew + 1
            iS ( inew ) = k2

         else

!        --- edge 2 of iS is part of edge 2 of iQ, skip the first point

            itemp = iQ ( 11 ) - n4 * isg * incr

            k2 = iQ ( itemp )
            ak2 = 1d0
            bk2 = bk1old

            do 50 i = iQ ( 7 ) + isg, itemp, isg
               inew = inew + 1
               iS ( inew ) = iQ ( i )
   50       continue

         end if

         iS ( 11 ) = inew

!        --- edge 3 of iS is new, from node  k2  to  k1, &
!            create inner points, if any

         iS ( 8 ) = inew

         if ( n6 * incr>1 ) then
            call mshz07 ( ak2, bk2, ak1, bk1, mrec, iQ, &
               n6 * incr, iS ( inew + 1 ), mpoint, npoint, Coor, &
               k2, k1, Ref, nbcord, Tau, iblend )
            inew = inew + ( n6 * incr - 1 )
         end if

         iS ( 12 ) = inew + 1

!        --- edge 4 of iS is reverse edge 2 of iR, copy also both ends
!            ( the nodes in iR are ordered normaly )

         iS ( 9 ) = iS ( 12 )

         do 60 i = iR ( 11 ), iR ( 7 ), -1
            inew = inew + 1
            iS ( inew ) = iR ( i )
   60    continue

         iS ( 13 ) = inew

         iS ( 1 ) = inew

!        --- subdivide iS now, scratch arrays iT, iU are empty
!            keep  iS  untouched for  iT  and eventually for  iU  !!

         call mshz22 ( new, trian, mrec, iS, iT, iU, &
                  npelm, npedge, &
                  mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                  nbcord, icurve, Tau, iblend )

!        --- third part: fill iT, see diagrams above
!            create in iT a quadrilateral  k1, k2, ito2, ito1, &

         iT ( 2 ) = n6
         iT ( 3 ) = n4
         if ( part4 ) then
            iT ( 4 ) = n1
         else
            iT ( 4 ) = n3 - ( n2 - n4 )
         end if
         iT ( 5 ) = n4
         iT ( 14 ) = iQ ( 14 )

         inew = 14

!        --- edge 1 of iT is reverse edge 3 of iS, copy also both ends
!            ( nodes in iS are ordered naturaly )

         iT ( 6 ) = 15

         do 70 i = iS ( 12 ), iS ( 8 ), -1
            inew = inew + 1
            iT ( inew ) = iS ( i )
   70    continue

         iT ( 10 ) = inew

!        --- edge 2 of iT

         iT ( 7 ) = inew

         if ( part4 ) then

!        --- edge 2 of iT is new, from  k2  to  ito2, &
!            create inner points, if any

            if ( n4 * incr>1 ) then
               call mshz07 ( ak2, bk2, ak2, 1d0, mrec, iQ, &
                  n4 * incr, iT ( inew + 1 ), mpoint, npoint, Coor, &
                  k2, iQ ( ito2 ), Ref, nbcord, Tau, iblend )
               inew = inew + n4 * incr - 1
            end if

         else

!        --- edge 2 of iT is part of edge 2 of iQ, from  itemp, &
!            do not copy the ends

            do 80 i = itemp + isg, iQ ( 11 ) - isg, isg
               inew = inew + 1
               iT ( inew ) = iQ ( i )
   80       continue

         end if

         iT ( 11 ) = inew + 1

!        --- edge 3 of iT is part of edge 3 of iQ, from  ito2  to  ito1, &
!            copy also both ends

         iT ( 8 ) = iT ( 11 )

         do 90 i = ito2, ito1, isg
            inew = inew + 1
            iT ( inew ) = iQ ( i )
   90    continue

         iT ( 12 ) = inew

!        --- edge 4 of iT is reverse edge 3 of iR, &
!            do not copy the first point

         iT ( 9 ) = inew

         do 100 i = iR ( 12 ) - 1, iR ( 8 ), -1
            inew = inew + 1
            iT ( inew ) = iR ( i )
  100    continue

         iT ( 13 ) = inew
         iT ( 1 ) = inew

!        --- keep  iT   untouched, postpone its processing for a while !!
!            fourth part, if any: fill iU, see diagrams above

         if ( part4 ) then

!        --- create in iU a quadrilateral  iQ ( 7 ), iQ ( 8 ), ito2, k2

            iU ( 2 ) = n4
            iU ( 3 ) = n5
            iU ( 4 ) = n4
            iU ( 5 ) = n5

            iU ( 14 ) = iQ ( 14 )

            inew = 14

!           --- edge 1 of iUis edge 2 of iQ, copy also both ends

            iU ( 6 ) = 15

            do 110 i = iQ ( 7 ), iQ ( 11 ), isg
               inew = inew + 1
               iU ( inew ) = iQ ( i )
  110       continue

            iU ( 10 ) = inew

!           --- edge 2 of iU is part of edge 3 of iQ, &
!               do not copy the first point

            iU ( 7 ) = inew

            do 120 i = iQ ( 8 ) + isg, ito2, isg
               inew = inew + 1
               iU ( inew ) = iQ ( i )
  120       continue

            iU ( 11 ) = inew

!           --- edge 3 of iU is reverse edge 2 of iT
!               do not copy the first point

            iU ( 8 ) = inew

            do 130 i = iT ( 11 ) - 1, iT ( 7 ), -1
               inew = inew + 1
               iU ( inew ) = iT ( i )
  130       continue

            iU ( 12 ) = inew

!           --- edge 4 of iU is reverse edge 2 of iS
!               do not copy the first point

            iU ( 9 ) = inew

            do 140 i = iS ( 11 ) - 1, iS ( 7 ), -1
               inew = inew + 1
               iU ( inew ) = iS ( i )
  140       continue

            iU ( 13 ) = inew
            iU ( 1 ) = inew

         end if

!        --- iR,  iS  are not needed any more,  divide  iT, iU

         if ( part4 ) then

!        --- subdivide iU now , scratch arrays are  iR, iS

            call mshz22 ( new, trian, mrec, iU, iR, iS, &
               npelm, npedge, &
               mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
               nbcord, icurve, Tau, iblend )

         end if

         if ( trian .or. iT(2)==iT(4) ) then

!        --- subdivide iT now, scratch arrays are  iR, iS

            call mshz22 ( new, trian, mrec, iT, iR, iS, &
                     npelm, npedge, &
                     mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                     nbcord, icurve, Tau, iblend )

         else

!        --- iR, iS, iU were succesfully subdivided into elements
!            and iT can be subdivided by the this subroutine mshz12

!             fill iQ by iT and make the next loop

            do 150 i = 1, iT ( 1 )
               iQ ( i ) = iT ( i )
  150       continue

         end if

      if ( .not. trian .and. iT(2)/=iT(4) ) goto 10

      end
