      subroutine mshz15 ( new, trian, npelm, nbpelm, npedge, &
                          mrec, iq, iu, &
                          nelem, melem, kmesh, &
                          mpoint, npoint, coor, ref, &
                          nbcord, icurve, tau, iblend )
! ======================================================================
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990-1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Generate triangular elements when the quadrilateral in iQ is just
!   one layer of elements:  n2 = n4 = 1.
!   It may be  n1 <> n3 .
!   Nodenumbers in  iQ  are in the natural order: iQ(6) < iQ(0) .
!   Orientation in  iQ  is positive: iQ(14)=1 .
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new, trian
      integer npelm, nbpelm, npedge, mrec, iQ ( mrec ), iU ( mrec ), &
              nelem, melem, kMesh ( npelm, melem ), &
              mpoint, npoint, nbcord, icurve, iblend
      double precision Coor ( 2, mpoint ), Ref ( 2, mpoint ), &
                       Tau ( 2, nbcord )

!     coor    i/o   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iblend   i    User's choice, see mshzde
!     icurve   i    User's choice, see mshzde
!     iq       i    Integer array of length mrec.
!                   The record must be initialized by calling subroutine
!                   to describe boundary of a quadrilateral region
!                   ( see mshz21 ), the region will be subdivided here.
!                   Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!     iu       s    Integer scratch array of length mrec
!     kmesh   i/o   Integer array (npelm,melem). Global nodenumbers for
!                   all elements created.
!     melem    i    Upper bound for nelem
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     nbpelm   i    Number of element-boundary points per element
!     nelem   i/o   Current number of elements created
!     new      i    .true.   if the topology is new
!                   .false.  if the topology is given by kMesh
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npelm    i    Number of points per element
!     npoint  i/o   Current number of all meshpoints
!     ref     i/o   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!     trian    i    .true.  = triangular elements ( not here )
!                   .false. = quadrilateral elements
!                   ( the information is passed to called subroutines )
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical down
      integer i, i1, i1i, i1last, i3, i3i, i3last, ntotal, iel, incr, &
              jbeg, j, i1firs, i3firs, k
      double precision da1, da3

!     da1        Step on the edge 1 in the ref.plan
!     da3        Step on the edge 3 in the ref.plan
!     down       .true.  The vertex i1i was chosen
!                .false. The vertex i3i was chosen
!     i          Counting variable
!     i1         Pointers into edge 1 of iQ, see the diagram below
!     i1firs     Pointers into edge 1 of iQ, see the diagram below
!     i1i        Pointers into edge 1 of iQ, see the diagram below
!     i1last     Pointers into edge 1 of iQ, see the diagram below
!     i3         Pointers into edge 3 of iQ, see the diagram below
!     i3firs     Pointers into edge 3 of iQ, see the diagram below
!     i3i        Pointers into edge 3 of iQ, see the diagram below
!     i3last     Pointers into edge 3 of iQ, see the diagram below
!     iel        Loop control, local counter of elements
!     incr       Number of intervals along element edge
!     j          Pointers into iU
!     jbeg       Pointers into iU
!     k          Global nodenumber of the third vertex
!     ntotal     Number of triangles to be created here
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshz00

!     MSHZ00    Real function.  The angle in range [ 0, 2 * pi ), &
!                            defined by a triplet of nodenumbers
!     MSHZ03    From given boundary points of one element
!               creates the whole element
!     MSHZ04    To display an error message and to stop the run
!     MSHZ07    Creates all new intermediate points between two given
!               points, using the mapping chosen
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The layer is divided from left to right and for each of the new
!   triangles ( not for the last of all ) there is a simple problem
!   to be solved:
!   the third vertex in the triangle above the base i3---i1
!                    is i1i or i3i, see the diagram:
!      i3firs------------------------i3--------i3i------------i3last
!        |                           | .     .                 |
!        |                          |    ..                    |
!        |       d o n e           |   .   .                   |
!        |                        | .        .                 |
!      i1firs--------------------i1-----------i1i-------------i1last
!   If the new topology is created, the strategy is to maximize the
!   inner angle at the third vertex.
!   If the topology is given by the user, then the answer is found
!   in kMesh ( and checked ). In this case only the physical coordinates
!   of the internal points should be re-calculated.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      if ( .not. trian ) call mshz04 ( '# mshz15: called for quadrilaterals' )

      if ( iQ(14).ne.1 ) call mshz04 ( '# mshz15: wrong orientation' )

      incr = npedge - 1

!     --- step on edge 1 and 3 in the reference plane

      da1 = 1d0 / ( iQ ( 2 ) * incr )
      da3 = 1d0 / ( iQ ( 4 ) * incr )

!     --- range of pointers into iQ for edges 1 and 3, &
!            both from left to right, so i3firs > i3last

      i1firs = iQ ( 6 )
      i1last = iQ ( 10 )

      i3firs = iQ ( 12 )
      i3last = iQ ( 8 )

!     --- number of triangles to be created

      ntotal = iQ ( 2 ) + iQ ( 4 )

!     --- fill the fixed part of the record iU

      iU ( 1 ) = 15 + nbpelm

      iU ( 2 ) = 1
      iU ( 3 ) = 1
      iU ( 4 ) = 1
      iU ( 5 ) = 0

      iU ( 6 ) = 15
      iU ( 7 ) = 15 + incr
      iU ( 8 ) = 15 + 2 * incr
      iU ( 9 ) = 15 + 3 * incr

      iU ( 10 ) = iU ( 7 )
      iU ( 11 ) = iU ( 8 )
      iU ( 12 ) = iU ( 9 )
      iU ( 13 ) = iU ( 9 )

      iU ( 14 ) = 1

!     --- initialize i3, i1 ( they define edge 1 of the next element )

      i3 = i3firs
      i1 = i1firs

      do 500 iel = 1, ntotal

!     --- create in  iU  the boundary of the next triang.element

         if ( iel.eq.1 ) then

!        --- edge 1 of iU is the edge 4 of iQ, copy also both endpoints

            do 10 i = iQ ( 9 ), iQ ( 13 )
               iU ( 15 + i - iQ (9 ) ) = iQ ( i )
   10       continue

         else

!        --- edge 1 of iU is reverse edge 3 or reverse edge 2
!            of the previous iU, namely  iU (jbeg...jbeg+incr)

            do 20 j = jbeg + incr, jbeg, -1
               iU ( 15 + incr + jbeg - j ) = iU ( j )
   20       continue

         end if

         i1i = i1 + incr
         i3i = i3 - incr

         if ( i1.eq.i1last ) then

!        --- i1i is out of range, the choice is i3i

            down = .false.

         else if ( i3.eq.i3last ) then

!        --- i3i is out of range, the choice is i1i

            down = .true.

         else if ( new ) then

!        --- maximize the angle

            down = mshz00 ( iQ(i1), iQ(i1i), iQ(i3), npoint, Coor ) &
             .ge.mshz00 ( iQ(i1), iQ(i3i), iQ(i3), npoint, Coor )

         else

!        --- use the old decission, it is in  kMesh ( *, nelem+1 )

            k = kMesh ( 1 + 2 * incr, nelem + 1 )

            if ( k.eq.iQ ( i1i ) ) then
               down = .true.
            else if ( k.eq.iQ ( i3i ) ) then
               down = .false.
            else
               call mshz04 ( 'mshz15: old topology, disorder in kMesh' )
            end if

         end if

!        --- edge 2 ( without its first point ) and edge 3

         if ( down ) then

!        --- second edge of  iU  is  iQ ( i1..i1i )

            do 30 i = i1 + 1, i1i
               iU ( 15 + incr + i - i1 ) = iQ ( i )
   30       continue

            if ( iel.lt.ntotal ) then

!           --- down, third edge is new, from i1i to i3

!                inner points

               if ( incr.gt.1 ) then
                  call mshz07 ( da1 * ( i1i - i1firs ), 0d0, &
                                da3 * ( i3firs - i3 ), 1d0, mrec, iQ, &
                     incr, iU ( 16 + 2 * incr ), &
                     mpoint, npoint, Coor, &
                     iQ ( i1i ), iQ ( i3 ), Ref, nbcord, Tau, iblend )
               end if

!              --- last point is the same as  i3

               iU ( 15 + 3 * incr ) = iQ ( i3 )

            else

!           --- down, third edge is old from i1last to i3last

               do 33 i = i1last + 1, i3last
                  iU ( 15 + 2 * incr + i - i1last ) = iQ ( i )
   33          continue

            end if

!           --- down, prepare values for next element

            i1 = i1i
            jbeg = 15 + 2 * incr

         else

!        --- not down, edge 2 and 3

            if ( iel.lt.ntotal ) then

!           --- not down, second edge of  iU  is new, &
!                         from i1 to i3i
!               inner points

               if ( incr.gt.1 ) then
                  call mshz07 ( da1 * ( i1 - i1firs ), 0d0, &
                                da3 * ( i3firs - i3i ), 1d0, mrec, iQ, &
                     incr, iU ( 16 + incr ), &
                     mpoint, npoint, Coor, &
                     iQ ( i1 ), iQ ( i3i ), Ref, nbcord, Tau, iblend )
               end if

            else

!           --- last element in this slice
!               not dowm, second edge of  iU  is old, &
!                                         iQ ( i1last .. i3last )
!               inner points

               do 44 i = i1last + 1, i3last - 1
                  iU ( 15 + incr + i - i1last ) = iQ ( i )
   44          continue

            end if

!           --- not down, third edge of  iU  is iQ ( i3i..i3 )

            do 40 i = i3i, i3
               iU ( 15 + 2 * incr + i - i3i ) = iQ ( i )
   40       continue

!           --- not down, prepare values for next element

            jbeg = 15 + incr
            i3 = i3i

         end if

         call mshz03 ( trian, npelm, nbpelm, npedge, mrec, iU, &
            nelem, melem, kMesh, &
            mpoint, npoint, Coor, Ref, &
            nbcord, icurve, Tau, iblend )

  500 continue

      end
