      subroutine mshz17 ( a, ib, ie, mrec, iQ, &
         npoint, Coor, x, y )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   Piecewice linear equidistant interpolation.
!
!   The mapping is defined by its values in integer points
!          i = ib .. ie    or   i = ie .. ib, &
!                i ---> Coor ( iQ (i) )
!
!   Value  Coor ( iQ (ib+a*(ie-ib)) ) is to be put in [x,y], &
!   so "a" is the relative position in the ib..ie.
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      integer ib, ie, mrec, iQ ( mrec ), npoint
      double precision a, Coor ( 2, npoint ), x, y

!     a        i    Relative position:  a=0 cooresponds to ib
!                                       a=1 corresponds to ie
!     Coor     i    Real array (2,npoint).
!                   Physical coordinates of meshpoints
!     ib, ie   i    Pointers into iQ, endpoints.
!                   It may happen  ib > ie.
!     iQ       i    Integer array of length mrec. The record
!                   describes boundary of a quadrilateral region
!                   ( see mshz21 ).
!     mrec     i    Upper bound for length of iQ
!     npoint   i    Current number of all meshpoints
!     x, y     o    Coordinates in the physical plane
!
! **********************************************************************
!
!                  LOCAL VARIABLES

      integer ileft, iright, kl, kr
      double precision exacti, rest

!     ileft, iright   Pointers into iQ defining the subinterval for the
!                     linear interpolation
!     kl, kr          Global nodenumbers corresponding to ileft, iright
!     exacti          Real "subscript" in the range ib..ie, which
!                     corresponds to the position "a"
!     rest            exactk - ileft
!
! **********************************************************************
!

      exacti = dble ( ib ) + a * ( ie - ib )

      if ( ib.lt.ie ) then
!       * normal ordering
         ileft = min ( ie - 1, max ( ib, int ( exacti ) ) )
      else
!       * reverse ordering
         ileft = min ( ib - 1, max ( ie, int ( exacti ) ) )
      end if
      iright = ileft + 1
!    * calculate the relative position between  ileft, iright
      rest = exacti - ileft

!    * pointers into Coor
      kl = iQ ( ileft )
      kr = iQ ( iright )

      x = Coor ( 1, kl ) + rest * ( Coor ( 1, kr ) - Coor ( 1, kl ) )
      y = Coor ( 2, kl ) + rest * ( Coor ( 2, kr ) - Coor ( 2, kl ) )

      end
