      subroutine mshz21 ( new, trian, npelm, npedge, &
                          mpoint, melem, mrec, mstk, &
                          npoint, nelem, kmeshc2d, &
                          istk, iq, ir, is, it, iu, coor, ref, &
                          nbcord, icurve, tau, iblend )
! ======================================================================
!
!       programmer      Zdenek Vlasek
!       version 2.1     date 03-07-1992  Comment removed from if statement
!       version 2.0     date 31-07-1990  n1,n2,n3,n4 are now local
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990-1992  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Submesh generator for 2-dim quadrilateral region, &
!   for quadrilateral elements with 4, 9, 16, 25, ... nodes.
!   User gives all boundary nodes, type of element and the number of
!   elements to be created along each side of the quadrilateral.
!   It is not suitable for too curved quadrilateral regions
!   or for very irregular distribution of boundary nodes, &
!   in which cases some general mesh generator should be used.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new, trian
      integer npelm, npedge, &
              mpoint, melem, mrec, mstk, &
              npoint, nelem, kmeshc2d ( npelm, melem ), &
              istk ( mstk, mrec ), &
              iq ( mrec ), ir ( mrec ), is ( mrec ), it ( mrec ), &
              iu ( mrec ), nbcord, icurve, iblend
      double precision coor ( 2, mpoint ), ref ( 2, mpoint ), &
                       tau ( 2, nbcord )

!     coor    i/o   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!                   ( the points number 1..npoint must be initialized
!                   when mshz21 is called, at that time npoint=nbcord )
!     iblend   i    User's choice, see mshzde
!     icurve   i    User's choice, see mshzde
!     iq       i    Integer array of length mrec.  The record must be
!                   i n i t i a l i z e d   by calling subroutine
!                   to describe boundary of a quadrilateral region, &
!                   the region will be subdivided here.
!                   Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!                   On return, the record iQ may contain a garbage.
!                   The items of the record are:
!                   iq(1)    actual length of the record
!                   iq(2)    number of elements along south
!                   iq(3)    number of elements along east
!                   iq(4)    number of elements along north
!                   iq(5)    number of elements along west
!                   iq(6..9)    point into iQ, they tell where in iQ
!                               is the global nodenumber
!                               of the  f i r s t  point of the s/e/n/west
!                   iq(10..13)  pointers into iQ, they tell where in iQ
!                               is the global nodenumber
!                               of the  l a s t  point of the s/e/n/west
!                               thus, &
!                               if iQ(6) < iQ(10) then
!                                  the south nodenumbers occupy part
!                                  iQ ( iQ(6)..iQ(10) )
!                                  and are ordered  n o r m a l l y
!                               else if iQ(6) > iQ(10) then
!                                  the south nodenumbers occupy part
!                                  iQ ( iQ(10)..iQ(6) )
!                                  and are  r e v e r s e  ordered
!                   iq(14)   +1  when quadrilateral is positive
!                                    ( anticlockwise ) oriented
!                            -1  when it is negative oriented
!                   iq(15.. )  global nodenumbers of the boundary
!                              points of the quadrilateral
!                              they start from one of the corners
!                              ( not necessery from the southwest )
!                              they are ordered in the positive or
!                              in the negative direction ( see iQ(14) )
!                              the last is the same as the first:
!                              iQ(iQ(1)) = iQ(15)
!     ir            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines, &
!                   the structure is as at iQ
!     is            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines, &
!                   the structure is as at iQ
!     istk          Integer scratch array (mstk,mrec).
!                   The stack which can keep upto mstk records, &
!                   the current number of records is nstk.
!                   Each record represents one quadrilateral boundary
!                   and is built like the iQ(mrec), see mshz21, &
!                   the records wait in the stack for processing
!     it            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines, &
!                   the structure is as at iQ
!     iu            Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines, &
!                   the structure is as at iQ
!     kmeshc2d    o    Integer array (npelm,melem). Global nodenumbers for
!                   all elements created.
!     melem    i    Upper bound for nelem, see mshzde
!     mpoint   i    Upper bound for npoint, see mshzde
!     mrec     i    Upper bound for length of iQ
!     mstk     i    Upper bound for number of records on the stack
!     nbcord   i    Number of user's boundary points, see mshzde
!     nelem    o    Number of elements created
!     new      i    .true.  New topology, see mshzde
!                   ( for subroutines common with the subpack mshz25 )
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npelm    i    Number of points per element
!     npoint  i/o   Current number of all meshpoints.
!                   It must be initialized on input, see mshzde
!     ref      i-   Real array (2,mpoint).  Reference coordinates of all
!                   meshpoints, the inner-element points are not needed.
!                   The coordinates of points in the reference
!                   quadrilateral [0,1] x [0,1], &
!                   in the same order as in Coor(2,mpoint), &
!                   the boundary points are  e q u i d i s t a n t
!                   on each of the four sides, &
!                   -- it is used only in the case  iblend=1 or 3
!                      and then it must be initialized on input
!     tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!                   The coordinates of  b o u n d a r y  points
!                   in the reference quadrilateral [0,1] x [0,1], &
!                   in the same order as in bCord(2,nbcord), &
!                   The points are  n o n - e q u i d i s t a n t
!                   on each of the four sides, in accordance with
!                   the relative length of the arc meassured in the
!                   physical plane, &
!                   -- it is used only in the case  iblend=1 or 3
!                      and then it must be filled on input
!     trian    i    .true.  = triangular elements
!                   .false. = quadrilateral elements
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nstk, n1, n2, n3, n4

!     n1         Number of elements along the  south  side
!                of the current quadrilateral
!     n2         Number of elements along the  east  side
!                of the current quadrilateral
!     n3         Number of elements along the  north side
!                of the current quadrilateral
!     n4         Number of elements along the  west side
!                of the current quadrilateral
!     nstk       The current number of items on the stack
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHZ12    decomposition of one quadrilateral with irregular but
!               elementary values n1,n2,n3,n4,  into three or more parts, &
!               each of them is processed directly, without stack
!     MSHZ18    To change the inner pointers in the record like iQ, &
!               due to a choice of new first vertex, &
!               the orientation is not influenced
!     MSHZ19    To pop one record iQ from the stack, for the processing
!     MSHZ22    To divide a quadrilateral region regularily, &
!               into quadrilaterals, applicable when n2=n4 and n1=n3, &
!               into triangles,      applicable when n2=n4
!     MSHZ23    To change the inner pointers in the record like iQ, &
!               due to the change of the orientation, change iQ(14), &
!               the first vertex remains the same
!     MSHZ24    to split a quadrilateral region into two quadrilateral
!               parts, both parts are put on stack, &
!               the cut leads from south to north, &
!               this subroutine is the only source or  r e c u r s i o n
!               ( but no recursive call is used )
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   see mshzde
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize the stack and  nelem

      nstk = 0
      nelem = 0

!     --- stack loop ( repeat ... until stack_is_empty ), by "goto 10"

   10 continue

         if ( nstk>0 ) then

!        --- get one record from the stack

            call mshz19 ( mstk, mrec, nstk, iQ, iStk )

         else

!        --- iQ is the input parameter, initialized by mshzde

         end if

         n1 = iQ(2)
         n2 = iQ(3)
         n3 = iQ(4)
         n4 = iQ(5)

!        --- if possible, divide regularily

         if ( n1==n3 .and. n2.eq.n4 ) then

            call mshz22 ( new, trian, mrec, iQ, iR, iU, &
               npelm, npedge, &
               mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
               nbcord, icurve, Tau, iblend )

         else

!        --- some irregular division needed
!            ensure  n1<=n3  to reduce the branching

            if ( n1>n3 ) then
               call mshz18 ( iQ, 3, n1, n2, n3, n4 )
            end if

!           --- ensure  n4<=n2  to reduce the branching

            if ( n4>n2 ) then
               call mshz18 ( iQ, 2, n1, n2, n3, n4 )
               call mshz23 ( iQ, n1, n2, n3, n4 )
            end if

!           --- ensure  n2/n4 <= n3/n1
!               if the ratios are equal, then ensure  n1<=n4, &
!                                        to reduce the branching

            if ( n2 * n1>n3 * n4 .or. ( n2 * n1==n3 * n4 ) &
               .and. ( n1>n4 ) ) then
               call mshz23 ( iQ, n1, n2, n3, n4 )
            end if

            if ( iblend==2 .or. iblend.eq.3 &
               .or. ( n1==n4 .and. n2.eq.n3 ) &
               .or. ( n2==1 .and. n2 + abs ( n3-n1-n2+n4) / 2<=&
                                      ( n3+n1-1)/(2*n1) + 1 ) ) then

!           --- user's choice: the "easy" algorithm
!               or the diagonal symmetry is present
!               or n4=n2=1 and "easy" will produce no more layers
!                              then subroutine mshz24
!                      number of layers:
!                      solve   2k-1 < n3/n1 <= 2k+1
!                              k-0.5 < n3/(2*n1) <= k+0.5
!                              k >= n3/(2*n1)-0.5 = (n3-n1)/(2*n1)
!                      k = (n3-n1+2*n1-1)/(2*n1) = (n3+n1-1)/(2*n1)
!                      mshz24 does k+1 layers
!               apply the "easy" nonrecursive algorithm
!               ensure  n2-n4 <= n3-n1

               if ( n2 - n4>n3 - n1 ) &
                  call mshz23 ( iQ, n1, n2, n3, n4 )

               call mshz12 ( new, trian, mrec, iQ, iR, iS, iT, iU, &
                  npelm, npedge, &
                  mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                  nbcord, icurve, Tau, iblend )

!              --- take next record from the stack

               goto 900

            end if

!           --- apply a recursive splitting algorithm
!               remember !!!  1 <= n1 <= n3,  1 <= n4 <= n2,  1 <= n2/n4 <= &
!               n3/n1 !!!

            if ( n1==1 ) then

!           --- n1=1, 1<=n2/n4<=n3, n3>1 ( otherwise n2=n4, n3=n1 )

               if ( n4==1 ) then

!              --- n1=n4=1, 1<=n2<=n3, n3>1
!                  examples:   1 1 3 1    1 2 2 1   1 2 4 1  1 5 9 1

                  call mshz12 ( new, trian, mrec, iQ, iR, iS, iT, iU, &
                     npelm, npedge, &
                     mpoint, melem, npoint, nelem, Coor, Ref, kmeshc2d, &
                     nbcord, icurve, Tau, iblend )

               else

!              --- n1=1, 2<=n4<=n2, n3>1
!                  examples:   1 3 2 2    1 2 3 2   1 9 2 7
!                  split it in two parts, &
!                  the cut leads from west to east

                  call mshz23 ( iQ, n1, n2, n3, n4 )

!                 --- now the cut leads from south to north

                  call mshz24 ( trian, mrec, iQ, iR, &
                     npedge, mstk, nstk, iStk, &
                     mpoint, npoint, Coor, Ref, nbcord, Tau, iblend )

               end if

            else

!           --- remaining cases:
!               2 <= n1 <= n3,  2 <= n2,  1 <= n4 <= n2
!               split it in two, in horizontal or in vertical direction

               if ( n4>=2 ) then

!              --- both splits are possible, but we prefere
!                  the cut leading from west to east

                  call mshz23 ( iQ, n1, n2, n3, n4 )

!                 --- now the cut should lead from south to north

               end if

!              --- split it in two parts, &
!                  the cut leads from south to north

               call mshz24 ( trian, mrec, iQ, iR, &
                  npedge, mstk, nstk, iStk, &
                  mpoint, npoint, Coor, Ref, nbcord, Tau, iblend )

            end if

         end if

  900    continue

!        --- no case was left !!!  use the stack

      if ( nstk>0 ) goto 10

      end
