      subroutine mshz22 ( new, trian, mrec, iq, ir, iu, &
                          npelm, npedge, mpoint, melem, &
                          npoint, nelem, coor, ref, kmeshc2d, &
                          nbcord, icurve, tau, iblend )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  2.0  date 31-07-1990  parameters n1..4 for mshz18,...
!        version  1.0  date 14-02-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   The quadrilateral region described in iQ has the nice property n2=n4
!   and in the case of quadrilateral elements also the property n1=n3.
!   The region is divided here into  n2  horizontal layers, each
!   layer is divided regularily by mshz14 or mshz15.
!   iQ may be  n o t  be changed here
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new, trian
      integer  mrec, iq ( mrec ), ir ( mrec ), iu ( mrec ), &
               npelm, npedge, mpoint, melem, npoint, nelem, &
               kmeshc2d ( npelm, melem ), nbcord, icurve, iblend
      double precision coor ( 2, mpoint ), ref ( 2, mpoint ), &
                       tau ( 2, nbcord )

!     coor    i/o   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iblend   i    User's choice, see mshzde
!     icurve   i    User's choice, see mshzde
!     iq       i-   Integer  s c r a t c h  array of length mrec.
!                   The record must be initialized by calling subroutine
!                   to describe boundary of a quadrilateral region
!                   ( see mshz21 ), the region will be subdivided here.
!                   Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!                   On return, the record iQ may contain a garbage.
!     ir       -    Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines
!     iu       -    Integer scratch arrays, each of length mrec, used to
!                   pass the sub-quadrilaterals to called subroutines
!     kmeshc2d   i/o   Integer array (npelm,melem). Global nodenumbers for
!                   all elements created.
!     melem    i    Upper bound for nelem
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     nelem   i/o   Current number of elements created
!     new      i    .true.  = new topology of the mesh
!                   .false. = user supports an old topology of the mesh
!                             in the array  kmeshc2d, untouched !!!
!                             ( this option is not available for
!                               quadrilateral elements )
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npelm    i    Number of points per element
!     npoint  i/o   Current number of all meshpoints
!     ref     i/o   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!     trian    i    .true.  = triangular elements
!                   .false. = quadrilateral elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical back
      integer level, i, inew, incr, isg, &
              n1, n2, n3, n4, ibeg2, iend2, &
              ibeg4, iend4, nbpelm, n1r, n3r, j
      double precision quoti, exn3r, db

!     back       To remember that iQ was changed here and thus it must
!                be restored before return
!     db         Step per one horizontal layer in the reference plane
!     exn3r      Real term in the geometric progression, n3r will
!                be close to it
!     i          Counting variable
!     ibeg2      Pointers into iQ, where is in iQ edge 2 of iR, &
!                see the diagram below
!     ibeg4      Pointers into iQ, where is in iQ edge 4 of iR, &
!                see the diagram below
!     iend2      Pointers into iQ, where is in iQ edge 2 of iR, &
!                see the diagram below
!     iend4      Pointers into iQ, where is in iQ edge 4 of iR, &
!                see the diagram below
!     incr       Number of intervals along one element edge
!     inew       Current position when iR is filled
!     isg        Signum of iQ(10)-iQ(6), the loopstep in many loops
!     j          Counting variable
!     level      Loop control variable, local level
!     n1         Number of elements along the  south  side
!                of the current quadrilateral
!     n1r        Number of elements along edge 1 of iR, &
!                see the diagram below
!     n2         Number of elements along the  east  side
!                of the current quadrilateral
!     n3         Number of elements along the  north side
!                of the current quadrilateral
!     n3r        Number of elements along edge 3 of iR, &
!                see the diagram below
!     n4         Number of elements along the  west side
!                of the current quadrilateral
!     nbpelm     Number of element-boundary points per element
!     quoti      Quotient in the geometric progression from n1 to n3
!                used to interpolate number of elements along the
!                horizontal cuts
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHZ07    Creates all new intermediate points between two given
!               points, using the mapping chosen
!     MSHZ13    Swap two integers
!     MSHZ14    Decomposition of one quadrilateral when n2=n4=1, n1=n3, &
!               resulting into one layer of quadrilateral elements
!     MSHZ15    Decomposition of one quadrilateral when n2=n4=1, &
!               resulting into one layer of quadrilateral elements
!     MSHZ18    To change the inner pointers in the record like iQ, &
!               due to a choice of new first vertex, &
!               the orientation is not influenced
!     MSHZ23    To change the inner pointers in the record like iQ, &
!               due to the change of the orientation, change iQ(14), &
!               the first vertex remains the same
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!  The number of elements along the horizontal lines
!  is close to the geometric progression from n1 to n3.
!  Each layer is described as quadrilateral in iR and handed to be
!  processed by mshz14 or mshz15.
!  To achieve positive orientation, the iQ may be reversed here
!  and restored before return.
!              |------n3-------|
!              |               |
!              |               |
!              |               |
!              |               |
!              |               |
!            ibeg4----n3r----iend2    - \
!              |               |          \
!              |               |           >  the layer to be done now
!              |               |          /
!            iend4----n1r----ibeg2    - /
!              |               |
!             ...             ...
!              |               |
!            iQ(13)---n1-------iQ(7)
!            iQ(6)            iQ(10)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill n1, n2, n3, n4

      n1 = iQ(2)
      n2 = iQ(3)
      n3 = iQ(4)
      n4 = iQ(5)

!     --- ensure the positive orientation ( iQ(14)=1 ), with  new n1 = old n1
!         remember that the change must be restored before return

      if ( iQ ( 14 )<0 ) then
         back = .true.
         call mshz18 ( iQ, 2, n1, n2, n3, n4 )
         call mshz23 ( iQ, n1, n2, n3, n4 )
      else
         back = .false.
      end if
      incr = npedge - 1
      if ( trian ) then
         nbpelm = 3 * incr
      else
         nbpelm = 4 * incr
      end if

!     --- step in the reference plane per one level

      db = 1d0 / n2

!     --- notice, the positive orientation does not guarante a positive isg

      isg = sign ( 1, iQ ( 10 ) - iQ ( 6 ) )

!     --- pointers ( into iQ ) for the corners of the first layer
!         ( edge 2 and 4 of iR is extracted from edge 2 and 4 of iQ )

      ibeg2 = iQ ( 7 )
      iend2 = ibeg2 + isg * incr
      iend4 = iQ ( 13 )
      ibeg4 = iend4 - isg * incr
      n1r = n1
      if ( trian ) then

!     --- quoti  is the ideal ratio  n3r/n1r  for layers
!         exn3r  is exact,  n3r  is rounded ( for the first layer )

         quoti = ( dble(n3) / dble(n1) ) ** ( 1d0 / n2 )
         exn3r = n1 * quoti
         n3r = int ( exn3r + 0.4999d0 )
      else

!     --- no need: exn3r = 1d0

         n3r = n1
      end if

!     --- loop over layers

      do 100 level = 1, n2
         inew = 15

!        --- edge 1 of iR, including the endpoints

         iR ( 6 ) = 15
         if ( level==1 ) then

!        --- edge 1 of iR is edge 1 of iQ

            do 10 i = iQ ( 6 ), iQ ( 10 ), isg
               iR ( inew ) = iQ ( i )
               inew = inew + 1
   10       continue
         else

!        --- edge 1 of iR is reverse edge 3 of previous iR
!              ( take care when  trian:  new edge 1 may be longer
!                then the old edges 1 and 2 together, so
!                new edge 1 may collide with the old edge 3:
!                x  x   x            x   x      vertices
!                .......123456789abcde....      old nodenumbers
!            method used:
!                .......edcba987654321....      reverse sequence
!                edcba987654321                 after shift
!            reverse the sequence of nodenumbers on the old edge 3

            j = iR ( 12 )
            do 19 i = iR ( 8 ), ( iR ( 8 ) + iR ( 12 ) - 1 ) / 2
               call mshz13 ( iR ( i ), iR ( j ) )
               j = j - 1
   19       continue

!           --- shift the sequence to the left

            do 20 i = iR ( 8 ), iR ( 12 )
               iR ( inew ) = iR ( i )
               inew = inew + 1
   20       continue
         end if
         iR ( 10 ) = inew - 1

!        --- edge 2, without the first point, but including the last point

         iR ( 7 ) = iR ( 10 )
         do 30 i = ibeg2 + isg, iend2, isg
            iR ( inew ) = iQ ( i )
            inew = inew + 1
   30    continue
         iR ( 11 ) = inew - 1

!        --- edge 3, without endpoints

         iR ( 8 ) = iR ( 11 )
         if ( level<n2 ) then

!        --- inner points of edge 3 do not exist yet, do them

            if ( n3r * incr>1 ) then
               call mshz07 ( 1d0, db * level, 0d0, db * level, mrec, iQ, &
                  n3r * incr, iR ( inew ), mpoint, npoint, Coor, &
                  iQ ( iend2 ), iQ ( ibeg4 ), Ref, nbcord, Tau, iblend )
               inew = inew + ( n3r * incr - 1 )
            end if
         else

!        --- edge 3 of iR is edge 3 of iQ, without endpoints

            do 40 i = iQ ( 8 ) + isg, iQ ( 12 ) - isg, isg
               iR ( inew ) = iQ ( i )
               inew = inew + 1
   40       continue
         end if
         iR ( 12 ) = inew

!        --- edge 4, including both endpoints

         iR ( 9 ) = iR ( 12 )
         do 50 i = ibeg4, iend4, isg
            iR ( inew ) = iQ ( i )
            inew = inew + 1
   50    continue
         iR ( 13 ) = inew - 1
         iR ( 1 ) = iR ( 13 )
         iR ( 2 ) = n1r
         iR ( 3 ) = 1
         iR ( 4 ) = n3r
         iR ( 5 ) = 1

!        --- nodenumbers are in the natural order

         iR ( 14 ) = 1
         if ( trian ) then

!        --- mshz15  must not destroy iR, it may use iU as scratch !!

            call mshz15 ( new, trian, npelm, nbpelm, npedge, &
                          mrec, iR, iU, nelem, melem, kmeshc2d, &
                          mpoint, npoint, Coor, Ref, &
                          nbcord, icurve, Tau, iblend )
         else

!        --- mshz14  must not change iR, it is pos.oriented !!

            call mshz14 ( trian, mrec, iR, iU, npelm, npedge, &
                          mpoint, melem, npoint, nelem, Coor, Ref, &
                          kmeshc2d, nbcord, icurve, Tau, iblend )
         end if

!        --- prepare for the next layer

         ibeg2 = ibeg2 + isg * incr
         iend2 = iend2 + isg * incr
         ibeg4 = ibeg4 - isg * incr
         iend4 = iend4 - isg * incr
         if ( trian ) then
            n1r = n3r
            exn3r = exn3r * quoti
            n3r = int ( exn3r + 0.4999d0 )
         end if
  100 continue
      if ( back ) then

!     --- restore iQ

         call mshz18 ( iQ, 2, n1, n2, n3, n4 )
         call mshz23 ( iQ, n1, n2, n3, n4 )
      end if
      end
