      subroutine mshz24 ( trian, mrec, iQ, iR, &
         npedge, mstk, nstk, iStk, &
         mpoint, npoint, Coor, Ref, nbcord, Tau, iblend )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                  PURPOSE
!
!   To split the quadrilateral iQ in two parts,
!   roughly half to half,
!   number of elements along the cut set close to sqrt(n2*n4)
!
!   The cut leads from south to north.
!
!   Both parts are to be put on the stack.
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      logical trian

      integer mrec, iQ ( mrec ), iR ( mrec ), &
         npedge, mstk, nstk, iStk ( mstk, mrec ), &
         mpoint, npoint, nbcord, iblend

      double precision Coor ( 2, mpoint ), Ref ( 2, mpoint ), &
         Tau ( 2, nbcord )

!     Coor     io   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iblend   i    User's choice, see mshzde
!     iQ       i    Integer array of length mrec.
!                   It describes boundary of a quadrilateral region
!                   ( see mshz21 ), the region will be subdivided here.
!                   Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!     iR       -    Integer scratch array of length mrec, used to
!                   pass the sub-quadrilaterals to subroutine mshz20
!     iStk     io   Integer array (mstk,mrec).
!                   The stack which can keep upto mstk records, &
!                   the current number of records is nstk.
!                   Each record represents one quadrilateral boundary
!                   and is built like the iQ(mrec), see mshz21, &
!                   the records wait in the stack for processing
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     mstk     i    Upper bound for number of records in the stack
!     nbcord   i    Number of user's boundary points, see mshzde
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npoint   io   Current number of all meshpoints
!     nstk     io   Current number of records on the stack
!     Ref      io   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     Tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!     trian    i    .true.  = triangular elements
!                   .false. = quadrilateral elements
!
! **********************************************************************
!
!                  LOCAL VARIABLES

      integer n1, n2, n3, n4, n1r, n2r, n3r, &
         incr, isg, inew, i, ifrom, ito, &
         ibeg1, iend3
      double precision relat, a1, a2

!     a1, a2         Local ( with respect to iQ ) reference coordinates
!                    of the points "ifrom" and "ito" are [a1,0], [a2,1].
!     i              Loop control variable, pointer into iQ or iR
!     ibeg1, iend3   Pointers into iQ, where is in iQ edge 2 of iR, &
!                    see the diagram below
!     ifrom, ito     Pointers into iQ, where is in iQ beggining and
!                    the end of the cut, see the diagram below
!     incr           Number of intervals along one element edge
!     inew           Current position when iR is filled
!     isg            Signum of iQ(10)-iQ(6), the loopstep in many loops
!     n1,n2,n3,n4    Number of elements along s/e/n/west side of iQ
!     n1r,n2r,n3r    Number of elements along s/e/north side 1 of the
!                    left iR, see the diagram below
!     relat          the ratio n1r/n1
!
! **********************************************************************
!
!                  SUBROUTINES CALLED

      logical mshz06

!     mshz04   To display an error message and to stop the run
!
!     mshz06   Logical function. Is the number even ?
!     mshz07   Creates all new intermediate points between two given
!              points, using the mapping chosen
!     mshz20   To push one record onto the stack, for later processing
!
! **********************************************************************
!
!                  ERROR MESSAGES are self-explaining
!
! **********************************************************************
!
!                  METHOD
!
!    As usual, the ideal values of n1r, n2r, n3r are not integer
!    and after rounding may break the solvability condition.
!
!    To achieve nice results, the way of defining the values  n1r, n3r
!    and then n2r is rather long and heuristic, but not blind.
!
!    Surely, there are many strategies.
!
!    Danger: bad changes may cause unfinite recursion due to growth
!    of the number of elements along the parts
!
!
!    iend3-------n3r----------ito----(n3-n3r)-----|
!      |                       /                  |
!      |                      /                   |
!      |     =left iR        /                    |
!      n4                  n2r                    n2
!      |                   /      =right iR       |
!      |                  /                       |
!    ibeg1----n1r-------ifrom-----(n1-n1r)--------|
!
! **********************************************************************
!

!-- significant values

      n1 = iQ ( 2 )
      n2 = iQ ( 3 )
      n3 = iQ ( 4 )
      n4 = iQ ( 5 )

      ibeg1 = iQ ( 6 )
      iend3 = iQ ( 12 )

      incr = npedge - 1
      isg = sign ( 1, iQ ( 10 ) - iQ ( 6 ) )

!    * split iQ roughly in half

!    * rounding up instead of down helps when n2r is incremented, &
!    *    more elements is usually not worse then less elements, &

!    * the stack on return looks like this:
!    *                     right part of iQ  ... the top of stack
!    *                     left part of iQ
!    *                     ...               ... old top of stack
!    *    if the left part is bigger then the right part ( or equal ), &
!    *    measured by their n1+n2+n3+n4, then the stack will become
!    *    later not too deep:
!    *    we avoid a deep stack containing many small quadrilaterals

      n1r = ( n1 + 1 ) / 2

!       * if e.g. n1 = 3 then n1r is far from center, &
!       *    look for opposite point ( see case  3 1 9 1 ), round up

      n3r = ( n1r * n3 + n1 - 1 ) / n1

!    * fill n2r an accordance with this n3r, &
!    * interpolate in the range n4..n2, on log scale, &
!         see the "bad" demonstration case  3 27 9 1

      relat = dble (n1r) / n1
      n2r = int ( 0.5d0 + exp ( ( 1d0 - relat ) * log ( dble(n4) ) &
         + relat * log ( dble(n2) ) ) )

!    * solvability

      if ( .not. trian &
         .and. .not. mshz06 ( n1r + n2r + n3r + n4 ) ) then

         if ( n2.eq.1 ) then
!          * also n4=1, layer of quadrilaterals, &
!               keep n2r=1, change n3r ( it was rounded up )
            n3r = n3r - 1
            if ( n3r.eq.0 ) call mshz04 ( '# mshz24: n3r=0' )
         else
!          * enlarge nr2
            n2r = n2r + 1
         end if

      end if

!-- position in iQ of points to be connected

      ifrom = iQ ( 6 ) + n1r * isg * incr
      ito = iQ ( 12 ) - n3r * isg * incr

!------------------------------------ the left part, one half or more

      inew = 14

!-- edge 1, take it from edge 1 of iQ

      iR ( 6 ) = 15

      do 10 i = iQ ( 6 ), ifrom, isg
         inew = inew + 1
         iR ( inew ) = iQ ( i )
   10 continue

      iR ( 10 ) = inew

!-- edge 2, the inner points no not exist

      iR ( 7 ) = inew

      if ( n2r * incr.gt.1 ) then
         a1 = dble ( abs (ifrom - ibeg1 ) ) / ( n1 * incr )
         a2 = dble ( abs (iend3 - ito ) ) / ( n3 * incr )

         call mshz07 ( a1, 0d0, a2, 1d0, mrec, iQ, &
            n2r * incr, iR ( inew + 1 ), mpoint, npoint, Coor, &
            iQ ( ifrom ), iQ ( ito ), Ref, nbcord, Tau, iblend )

         inew = inew + n2r * incr - 1

      end if

      iR ( 11 ) = inew + 1

!-- edge 3, take it from edge 3 of iQ

      iR ( 8 ) = iR ( 11 )

      do 20 i = ito, iQ ( 12 ), isg
         inew = inew + 1
         iR ( inew ) = iQ ( i )
   20 continue

      iR ( 12 ) = inew

!-- edge 4 of iR is edge 4 of iQ

      iR ( 9 ) = inew

      do 30 i = iQ ( 9 ) + isg, iQ ( 13 ), isg
         inew = inew + 1
         iR ( inew ) = iQ ( i )
   30 continue

      iR ( 13 ) = inew

!-- finish this record

      iR ( 1 ) = inew

      iR ( 2 ) = n1r
      iR ( 3 ) = n2r
      iR ( 4 ) = n3r
      iR ( 5 ) = n4

      iR ( 14 ) = iQ ( 14 )

      call mshz20 ( mstk, mrec, nstk, iR, iStk )

!------------------------------ the right part, one half or less

      inew = 14

!-- edge 1, use here edge 2 of the previous iR

      iR ( 6 ) = 15

      do 40 i = iR ( 7 ), iR ( 11 )
         inew = inew + 1
!        ... no danger here
         iR ( inew ) = iR ( i )
   40 continue

      iR ( 10 ) = inew

!-- edge 2 is part of reverse edge 3 of iQ

      iR ( 7 ) = inew

      do 50 i = ito - isg, iQ ( 8 ), - isg
         inew = inew + 1
         iR ( inew ) = iQ ( i )
   50 continue

      iR ( 11 ) = inew

!-- edge 3 is reverse edge 2 of iQ

      iR ( 8 ) = inew

      do 60 i = iQ ( 11 ) - isg, iQ ( 7 ), - isg
         inew = inew + 1
         iR ( inew ) = iQ ( i )
   60 continue

      iR ( 12 ) = inew

!-- edge 4 is part of the reverse edge 1 of iQ

      iR ( 9 ) = inew

      do 70 i = iQ ( 10 ) - isg, ifrom, - isg
         inew = inew + 1
         iR ( inew ) = iQ ( i )
   70 continue

      iR ( 13 ) = inew

!-- finish this record

      iR ( 1 ) = inew

      iR ( 2 ) = n2r
      iR ( 3 ) = n3 - n3r
      iR ( 4 ) = n2
      iR ( 5 ) = n1 - n1r

      iR ( 14 ) = -iQ ( 14 )

      call mshz20 ( mstk, mrec, nstk, iR, iStk )

      end
