      subroutine plaxis01 ( iinput, rinput, coorusr, infcur, &
                            infsur, ccoarse, &
                            factors, ctangent, lenpnt, lensurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 30-07-2012 calls to eropen/erclos
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 16-07-1999 Extra parameter ndim
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Split arrays iinput and rinput into various parts
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     plaxis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lenpnt, lensurfs, &
              infcur(maxlencur,lencrv), infsur(maxlensur,lensurfs)
      double precision rinput(*), coorusr(ndim,lenpnt), ccoarse(lenpnt), &
                       factors(lencrv), ctangent(2,lencrv)

!     ccoarse        o    Array containing the coarseness of the user points
!     coorusr        o    Array containing the co-ordinates of the user points
!                         x_i = coorusr(1,i);  y_i = coorusr(2,i)
!     ctangent       o    Array containing the factors t_0 and t_1 of the curves
!                         t_0_i = ctangent(1,i);  t_1_i = ctangent(2,i)
!     factors        o    Array containing the factors of the curves
!     iinput         i    Standard integer input array for subroutine MESH
!     infcur         o    Array containing the information of the curves
!                         infcur(1,i) refers to curve i
!     infsur         o    Array containing the information of the surfaces
!                         infsur(1,i) refers to surface i
!     lenpnt         i    Number of entries in local point type arrays
!     lensurfs       i    Number of entries in local surface type arrays
!     rinput         i    Real input array for subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinpsav, j, icurve, iinp

!     i              Counting variable
!     icurve         Type of curve
!     iinp           Position in array iinput
!     iinpsav        Help variable to save the value of iinpx
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH046         Raise iinp as function of icurve
!     MSH049         Compute starting address of specific surface in
!                    array IINPUT
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays
!     Copy co-ordinates from rinput into coorusr
!     Copy coarseness from rinput into ccoarse
!     Copy information of curves into infcur
!     Copy factors from rinput into factors
!     Copy tangents from rinput into ctangent
!     Copy information of surfaces into infsur
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plaxis01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plaxis01'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'nuspnt, ncurvs, nsurfs, irnp1, jcoars', &
                          nuspnt, ncurvs, nsurfs, irnp1, jcoars

!     --- Clear arrays

      coorusr = 0d0
      ccoarse = 1d0
      infcur = 0
      ctangent = 0d0
      factors = 0d0
      infsur = 0

!     --- Copy co-ordinates from rinput into coorusr

      do i = 1, nuspnt
         do j = 1, ndim
            coorusr(j,i) = rinput(irnp1+ndim*(i-1)+j-1)
         end do
      end do
      irnp1 = irnp1+ndim*nuspnt
      if ( debug ) write(irefwr,1) 'irnp1', irnp1

      if ( jcoars==1 ) then

!     --- Copy coarseness from rinput into ccoarse

        ccoarse(1:nuspnt) = rinput(irnp1+1:irnp1+nuspnt)

      end if
      if ( debug ) call prinrl ( ccoarse, nuspnt, 'ccoarse' )

!     --- Copy information of curves into infcur

      iinp = iinp1
      do i = 1, ncurvs
         icurve = iinput(iinp)
         iinpsav = iinp
         call msh046(iinput,iinp,icurve)
         do j = 1, iinp-iinpsav
            infcur(j,i) = iinput(iinpsav+j-1)
         end do
      end do

!     --- Copy factors from rinput into factors
!         Copy tangents from rinput into ctangent

      do i = 1, ncurvs
         factors(i) = rinput(irnp2+i-1)
         ctangent(1,i) = rinput(irnp2+ncurvs+2*i-2)
         ctangent(2,i) = rinput(irnp2+ncurvs+2*i-1)
      end do

!     --- Copy information of surfaces into infsur

      iinp = iinp3
      do i = 1, nsurfs
         isurf = iinput(iinp)
         iinpsav = iinp
         call msh049 ( iinput, iinp, isurf )
         do j = 1, iinp-iinpsav
            infsur(j,i) = iinput(iinpsav+j-1)
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( ccoarse, 3*nuspnt, 'ccoarse' )
         write(irefwr,*) 'End plaxis01'

      end if  ! ( debug )
      call erclos ( 'plaxis01' )

      end
