      subroutine sepfillelemobs
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ielemobst (2d)
!     Old subroutine mshfillelemobs
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqob, iobst, lencurvs, lenielemobst, nx, ny
      integer, allocatable :: iwork(:), istore(:), ielemobstcp(:)

!     ielemobstcp    copy of array ielemobst
!     iobst          Obstacle sequence number
!     iseqob         Sequence number
!     istore         Contains the last level set
!     iwork          integer work array
!     lencurvs       Overestimate of the number of points along the new
!                    curves
!     lenielemobst   Computed length of array ielemobst
!     nx             number of cells in x direction
!     ny             number of cells in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPINITKMESHAD Initialize kmesh ad
!     SEPOBSTMARKEL  Mark all elements in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!     SEPOBSTMARKEL1 Find all elements that intersect the obstacle but do not
!                    have a node inside the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2207   There are no obstacles
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillelemobs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillelemobs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      if (debug)  write(irefwr,1) 'numobst', numobst
      if ( numobst==0 ) then

!     --- No obstacles

         call errwar ( 2207, 0, 0, 0 )
         go to 1000

      end if

      if ( kelmad==0 ) call sepinitkmeshad
      if ( kmeshad(1)==0 ) call sepflpointobst

      iseqob = kmeshad(5)

      if ( kelmaa==0 )  call sepfillnmeshc
      if ( nmesh(38)==0 ) call sepfillnmeshh

!     --- Reserve temporary space for array ipelemobst and
!         ipelemobstcp

      lenielemobst = 6*numobst+nelem
      allocate ( iwork(nelem), istore(nedges), ielemobst(lenielemobst), &
                 ielemobstcp(lenielemobst), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem+nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- First the elements are created from the nodes only

      call sepobstmarkel ( lenielemobst )

!     --- Next it is checked if there are extra elements that have no node
!         in common with the obstacle but still intersect the obstacle

      lenielemobst = numobst+1
      lencurvs = size(curvesmsh)
      do iobst = 1, numobst
         nx = kmeshx(3,iobst)
         ny = kmeshx(4,iobst)
         iobgrid2d => kmobst(iobst)%iobgrid2d
         iobgrid2d1 => kmobst(iobst)%iobgrid2d1
         if ( debug ) then
            write(irefwr,1) 'iobst, nx, ny', iobst, nx, ny
            call prinin ( iobgrid2d, nx*ny, 'iobgrid' )
         end if  ! ( debug )
         call sepobstmarkel1 ( iwork, lenielemobst, istore, &
                               nx, ny, ielemobstcp, iobst, lencurvs )
         if ( ierror/=0 ) go to 1000
      end do
      lenielemobst = lenielemobst-1

!     --- Make array ielemobst permanent

      deallocate ( ielemobst )
      if ( associated(km(iseqkm)%ielemobst) ) &
         deallocate ( km(iseqkm)%ielemobst )
      allocate ( km(iseqkm)%ielemobst(lenielemobst) )
      ielemobst => km(iseqkm)%ielemobst
      ielemobst = ielemobstcp(1:lenielemobst)

      kmeshad(2) = lenielemobst
      kmeshad(7) = iseqob

      deallocate ( iwork, istore, ielemobstcp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillelemobs'

      end if  ! ( debug )
      call erclos ( 'sepfillelemobs' )

      end subroutine sepfillelemobs
