      subroutine sepmeshwr ( iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write information of the mesh to the file indicated by iref
!     If iref>=0, then formatted output is used with at most 80 columns
!     If iref<0 unformatted output
!     Old subroutine meshwr
! **********************************************************************
!
!                       KEYWORDS
!
!     write_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref

!     iref    i    dataset reference number for the output
!                  when iref = 0  the standard output file is assumed
!                  when iref>0, the output is written to the file with
!                  reference number iref with formatted output
!                  When iref<0, the output is written to the file with
!                  reference number -iref with unformatted output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp(10)
      double precision :: timebefore(2)

!     ihelp          Work array to store input from the input file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     MESHWRPAR      Write mesh to file in case of parallel mesh
!     SEPMSH059      Write information of the mesh to the file indicated by iref
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmeshwr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmeshwr'
         write(irefwr,*) 'parallel ', parallel

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      if ( itime>0 ) call ersettime ( timebefore )

!     --- Initialize array ihelp

      ihelp(2) = 0
      ihelp(3) = 0
      if ( parallel ) then

!     --- Parallel case, write only from node 1

         call meshwrpar

      else

!     --- Standard case

         call sepmsh059 ( iref, ihelp )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmeshwr'

      end if  ! ( debug )
      call erclmn ( 'sepmeshwr', timebefore )

      end subroutine sepmeshwr
