      subroutine sepmshflkmp
! ======================================================================
!
!        programmer    Peter Slikkerveer
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elsize with the sizes of the elements at the beginning
!     and the end of the curves
!     Old subroutine mshflkmp
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istar3, icurnr, inodp1, inodp2
      double precision dx
      logical filled

!     dx             length of vector
!     filled         If true the surface has been filled
!     icurnr         Curve sequence number
!     inodp1         First node in part
!     inodp2         Second node in part
!     istar3         Starting address of KMESHM part 3 in KMESH part M
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all curves do
!        inodp1 := first node in curve
!        inodp2 := second node in curve
!        dx := distance between inodp1 and inodp2
!        inodp1 := last node in curve
!        inodp2 := last node before last node in curve
!        dx := distance between inodp1 and inodp2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmshflkmp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmshflkmp'
         write(irefwr,*) 'ncurvs ', ncurvs

      end if
      filled = kelmh>0
      if ( filled ) filled = kmeshh(3,1)>0
      if ( filled ) then

!     --- lenkmsh = 0, icurvs and curves have not been filled

         istar3 = kmeshm(2) - 1
         do icurnr = 1, ncurvs
            if ( kmeshm(5+icurnr)-kmeshm(4+icurnr)>0 ) then

!           --- Curve contains nodes

               inodp1 = kmeshm(istar3 + kmeshm(4+icurnr)     )
               inodp2 = kmeshm(istar3 + kmeshm(4+icurnr) + 1  )
               if ( inodp1>0 .and. inodp2>0 ) then
                  dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                              (coor(2,inodp1) - coor(2,inodp2))**2 )
                  elsize(1,icurnr) = dx
               else
                  elsize(1,icurnr) = 1d0
               end if
               inodp1 = kmeshm(istar3 + kmeshm(5+icurnr) - 1  )
               inodp2 = kmeshm(istar3 + kmeshm(5+icurnr) - 2  )
               if ( inodp1>0 .and. inodp2>0 ) then
                  dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                              (coor(2,inodp1) - coor(2,inodp2))**2 )
                  elsize(2,icurnr) = dx
               else
                  elsize(2,icurnr) = 1d0
               end if

            else

               elsize(1,icurnr) = 1d0
               elsize(2,icurnr) = 1d0

            end if

         end do

      else

!     --- mmicurvs> 0, icurvs and curves have been filled

         istar3 = 1
         do icurnr = 1, ncurvs

            if ( debug ) &
               write(irefwr,*) 'icurnr, icurvsmsh(icurnr), istar3 ', &
                                icurnr, icurvsmsh(icurnr), istar3
            if ( icurvsmsh(icurnr)>0 ) then
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'curvesmsh(istar3) ', &
                                   curvesmsh(1,istar3),curvesmsh(2,istar3)
                  write(irefwr,*) 'curvesmsh(istar3+1) ', &
                                   curvesmsh(1,istar3+1),curvesmsh(2,istar3+1)

               end if

               dx = sqrt ( (curvesmsh(1,istar3+1) - curvesmsh(1,istar3))**2 + &
                           (curvesmsh(2,istar3+1) - curvesmsh(2,istar3))**2 )
               if ( debug ) write(irefwr,*) 'dx ', dx
               elsize(1,icurnr) = dx
               istar3 = icurvsmsh(icurnr)
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'istar3 ', istar3
                  write(irefwr,*) 'curvesmsh(istar3) ', &
                                   curvesmsh(1,istar3),curvesmsh(2,istar3)
                  write(irefwr,*) 'curvesmsh(istar3+1) ', &
                                   curvesmsh(1,istar3-1),curvesmsh(2,istar3-1)

               end if

               dx = sqrt ( (curvesmsh(1,istar3) - curvesmsh(1,istar3-1))**2 + &
                           (curvesmsh(2,istar3) - curvesmsh(2,istar3-1))**2 )
               if ( debug ) write(irefwr,*) 'dx ', dx
               elsize(2,icurnr) = dx
               istar3 = istar3+1

            else

               elsize(1,icurnr) = 1d0
               elsize(2,icurnr) = 1d0

            end if

         end do

      end if

      call erclos ( 'sepmshflkmp' )

      end
