      subroutine msh047bn ( inpada, kprobf, indprf, &
                            iwork, coorol, anorm, icurves, &
                            iwork1, ncurves, iellist, &
                            closed, nbefor, jcount, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 16-06-2005 Set n
!        version  2.0  date 14-10-2002 Extra parameter kmeshq
!        version  1.0  date 09-02-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iwork with nodal points corresponding to the boundaries to be
!     updated
!     The normal vector is computed and stored in anorm
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), iwork(*), iwork1(*), jcount, &
              inpada(*), indprf, icurves(*), nbefor, n, &
              ncurves, iellist(*)
      double precision :: coorol(2,*), anorm(2,*)
      logical closed

!     anorm          o    In this array the normal vector is stored
!                         for all nodes in the boundary
!     closed         o    If true the boundary is closed
!     coorol         o    Array containing the old co-ordinates along the
!                         boundary
!     icurves        i    Integer array containing the curve numbers that
!                         must be adapted. It is assumed that these curves form
!                         one continuous curve. Compound curves must have
!                         been represented by their subcurves
!     iellist        o    Array of length npoint-1 containing a
!                         list of elements that are adjacent to the boundary
!                         element iellist(i) contains both nodal points
!                         iwork(i) and iwork(i+1)
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inpada         i    User input array for subroutine ADAPBOUN
!                         See subroutine msh047
!     iwork          o    integer work array of length npoint containing the
!                         nodal points of the boundary at ouptut
!     iwork1         o    integer work array of length npoint containing
!                         iwork1(i) = 0, means node i does not correspond
!                         to user point
!                         iwork1(i)>0, means node i corresponds to user
!                         point iwork1(i)
!     jcount         o    Contains number of nodes stored in iwork
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     n              o    number of nodes in boundary
!     nbefor         o    number fo oiginal nodes in boundary
!     ncurves        i    Number of curves that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer linear, i, inodp, j, jcurve, istar3, &
              icount, istart, icurve, idirec, iadapt, number

!     i              Counting variable
!     iadapt         Indicates how the boundary must be adapted, see INPADA
!     icount         Counter
!     icurve         Absolute value of jcurve
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     inodp          Nodal point number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     j              Counting variable
!     jcurve         Curve sequence number provided with direction sign
!     linear         Parameter linear as described in inpada(7)
!     number         The number-th unknown in each nodal point of the array
!                    is considered.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0033         Check if the (first) degree of freedom number is present in
!                    all nodal points of array iwork
!     ER0047         Compute normal to coorol and store in array anorm
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH067         Fill array iellist with element numbers
!     SEPER0027      Fill nodal points of curves as indicated by ichois and
!                    icurvs in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     385   boundary contains less than two points
!     474   the (first) degree of freedom number is not present in all
!           nodal points of array iwork
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh047bn' )
      iadapt = inpada(4)
      idirec = inpada(5)
      number = inpada(6)
      linear = inpada(7)
      do i = 1, npoint
         iwork1(i) = 0
      end do

!     --- first fill array iwork with nodal point numbers of boundary

      if ( iadapt/=8 ) then

!     --- Standard method, nodal points are used

         n = npoint
         call seper0027 ( ncurves, icurves, n, iwork )
         if ( iadapt>=6 .and. iadapt/=9 ) then

!        --- Special method, requiring the gradient in mid points of the
!            cells

            call msh067 ( iwork, n, iellist, iwork1 )

         end if

         nbefor = n
         if ( n<2 ) then
            call errint ( n, 1 )
            call errsub ( 385, 1, 0, 0 )
         end if

!        --- check if the (first) degree of freedom number is present in all
!            nodal points of array iwork

         if ( indprf>0 .and. iadapt/=2 .and. iadapt/=6 .and. &
              iadapt/=6 ) then

            call er0033 ( iwork, n, number, kprobf )
            if ( n/=nbefor ) then
               call errint ( number, 1 )
               call errint ( icurves(1), 2 )
               call errint ( icurves(ncurves), 3 )
               call errint ( n, 4 )
               call errint ( nbefor, 5 )
               call errsub ( 474, 5, 0, 0 )
            end if

         end if

!        --- Fill array coorol

         do i = 1, n
            inodp = iwork(i)
            coorol(1,i) = coor(1,inodp)
            coorol(2,i) = coor(2,inodp)
         end do

!        --- compute normal to coorol and store in array anorm
!            First component nx, second one ny

         if ( iadapt/=4 .and. iadapt/=6 ) &
            call er0047 ( linear, coorol, n, anorm )

         if ( iwork(1)==iwork(n) ) then

!        --- First and last point of the curves are the same, hence the
!            curve is closed

            if ( iadapt/=6 ) n = n-1
            closed = .true.

         else

!        --- not a closed curve

            closed = .false.

         end if

      else

!     --- iadapt = 8, i.e. only the co-ordinates of the boundary are required

         icount = 0
         jcount = 0
         istar3 = kmeshm(2)-1

!        --- Put information about node numbers of user points into iwork1
!            iwork1(i) = 0, means node i does not correspond to user point
!            iwork1(i)>0, means node i corresponds to user point iwork1(i)

         do i = 1, nuspnt
            if ( kmeshl(i)>0 ) then
               iwork1(kmeshl(i)) = i
            end if
         end do

         do j = 1, ncurves

            jcurve = icurves(j)
            icurve = abs(jcurve)
            n = kmeshm(5+icurve)-kmeshm(4+icurve)
            if ( jcurve>0 ) then
               istart = kmeshm(4+icurve)
               idirec = 1
            else
               istart = kmeshm(5+icurve)-1
               idirec = -1
            end if
            do i = 1, n
               coorol(:,icount+i) = curvesmsh(:,istart+idirec*(i-1))
            end do

!           --- Fill array iwork with information concerning user points
!               iwork(2*i-1) contains the user point number
!               iwork(2*i)   contains the position in coorol/nw

            do i = 1, n
               inodp = kmeshm(istar3+istart+idirec*(i-1))
               if ( inodp<0 ) then

!              --- User point without node number

                  iwork(jcount+1) = -inodp
                  iwork(jcount+2) = icount+i
                  jcount = jcount+2

               else if ( inodp>0 ) then

!              --- Point with node number

                  if ( iwork1(inodp)>0 ) then

!                 --- User point

                     iwork(jcount+1) = iwork1(inodp)
                     iwork(jcount+2) = icount+i
                     jcount = jcount+2

                  end if

               end if

            end do
            icount = icount+n-1

         end do
         n = icount+1
         closed = .false.

      end if
      call erclos ( 'msh047bn' )
      end
