      subroutine msh047upd ( inpada, rinada, nunkp, curves, kprobf, &
                             indprf, usol, iwork, coornw, &
                             coorol, anorm, iellist, &
                             icurvs, closed, n, uold, maxphys, numvecs )
! ======================================================================
!
!        programmer    Guus Segal and Jaap van der Zanden
!        version  2.1  date 06-11-2009 Minor correction text
!        version  2.0  date 06-01-2009 Removal of work
!        version  1.0  date 05-11-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     recompute co-ordinates of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), iwork(*), maxphys, inpada(*), nunkp, indprf, &
              iellist(*), icurvs(*), n, numvecs
      double precision coornw(2,*), usol(*), curves(2,*), &
                       coorol(2,*), anorm(2,*), rinada(*), uold(*)
      logical closed

!     anorm      i    real work array containing the norms in the points
!                     on the curves in icurves
!     closed     i    If true the boundary is closed
!     coornw    i/o   array containing the new co-ordinates of the
!                     points on the curves in icurves
!     coorol     i    array containing the old co-ordinates of the
!                     points on the curves in icurves
!     curves     o    in this array the old and new co-ordinates of the
!                     curves are stored
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     iellist         Array of length npoint-1 containing a
!                     list of elements that are adjacent to the boundary
!                     element iellist(i) contains both nodal points
!                     iwork(i) and iwork(i+1)
!     indprf     i    indication whether kprob part f is used or not
!     inpada     i    Integer array to be filled by the user, indicating how the
!                     boundary must be adapted.
!     iwork      i    integer work array containing the nodal points of the
!                     curves in icurves at output
!     kprobf     i    KPROB part f: contains the number of unknowns per
!                     point accumulated
!     maxphys    i    Maximum number of unknowns per point
!     n          i    number of nodes in boundary
!     numvecs    i    Number of vectors in iseqvecs
!     nunkp      i    maximal number of degrees of freedom in nodal points
!     rinada     i    Real input array with data as described by inpada
!     uold       i    Array containing the old solutions
!     usol       i    solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer linear, iadapt

!     iadapt         Indicates how the boundary must be adapted, see INPADA
!     linear         Parameter linear as described in inpada(7)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH072         Correct end points of free boundary
!     MSHADCURVS     Adapt coordinates at boundary (2d)
!     MSHUPDQUAD     Adapt updated curve for quadratic elements
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make initializations
!     compute new co-ordinates and store in array coornw
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      call eropen ( 'msh047upd' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in msh047upd'
         call prinin ( inpada, 15, 'inpada' )

      end if
      if ( ierror/=0 ) go to 1000

      iadapt = inpada(4)
      linear = inpada(7)

!     --- Adapt coordinates at boundary

      call mshadcurvs ( usol, iwork, n, indprf, kprobf, &
                        nunkp, iadapt, rinada, coorol, &
                        coornw, anorm, inpada, closed, maxphys, &
                        numvecs, uold, iellist )

!     --- correct end points for free separation points

      call msh072 ( coorol, coornw, inpada, rinada, n, icurvs, curves )
      if ( debug ) call  prinrl1(coornw, n, 2, 'coornw after msh072')

!     --- this part calculates the shortest distance between every
!         'mid' and the line between the endpoints of an element side.
!         this 'mid' nodal point is put back to the 'mid' shift
!         parallel to the line between the end-points.
!         meaning of some variables used in this part:
!                !                X  nodal point p
!                !      a        /
!         y-coor !      | \     /     X    new position of point p
!                !      |   \  /
!                !  dy  |     \--> line between a and b
!                !      |       \
!                !      |   dx    \
!                !      |___________b
!                0------------------------------- &
!                                           x-coor

      if ( iadapt/=8 .and. linear==2 ) then

!     --- quadratic elements

         call mshupdquad ( coornw, n )

      end if

1000  call erclos ( 'msh047upd' )
      if ( debug ) call  prinrl1(coornw, n, 2, 'coornw end msh047upd')

      end
