      subroutine msh300 ( kmeshc, coor, nelem, npoint, inpelm )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date   28-12-1996 Extra parameter inpelm
!        version 1.0    date   01-09-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill the centroid point for hexahedrons with 9 points and tetrahedrons
!   with 4 points
!   Both coor and kmeshc will be filled
! **********************************************************************
!
!                       KEYWORDS
!
!   mesh
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nelem, npoint, inpelm, kmeshc(inpelm,nelem)
      double precision coor(3,*)

!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!                  Only the centroids have not yet been filled
!     inpelm  i    Number of nodes per element
!     kmeshc i/o   kmesh part c, contains the nodal points corresponding to the
!                  elements
!                  Only the centroids have not yet been filled
!     nelem   i    Number of elements
!     npoint i/o   Number of nodal points
!                  At input npoint is the number of nodes without the
!                  centroids, at output it is equal to the total number
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, npelm
      double precision sum, w

!     i       Counting variable
!     j       Counting variable
!     k       Counting variable
!     npelm   Number of points per element minus 1
!     sum     Help parameter to compute a sum
!     w       Multiplication factor for averaging
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill kmeshc with nodal point numbers npoint+1 to npoint+nelem
!     Fill array coor by taking the average of the co-ordinates of the
!     vertices
!     Raise npoint
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill kmeshc with nodal point numbers npoint+1 to npoint+nelem

      do i = 1, nelem
         kmeshc(inpelm,i) = npoint+i
      end do

!     --- Fill array coor by taking the average of the co-ordinates of the
!         vertices

      npelm = inpelm-1
      w = 1d0/npelm
      do j = 1, 3
         do i = 1, nelem
            sum = 0d0
            do k = 1, npelm
               sum = sum + coor(j,kmeshc(k,i))
            end do
            coor(j,npoint+i) = w*sum
         end do
      end do

!     --- Raise npoint

      npoint = npoint + nelem
      end
