      subroutine msh308 ( bcord, nx, ny, nz, ivolnr, inpelm, kbndpt )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 4.0    date   05-06-1998 Extra parameters inpelm, kbndpt
!        version 3.0    date   02-02-1997 Extra parameter intarmsh
!        version 2.2    date   26-12-1994 New commons cmessc and cmessg
!        version 2.1    date   23-06-1992 New common cmessg
!        version 2.0    date   12-02-1990 New call
!        version 1.0    date   07-01-1985
!
!
!
!   copyright (c) 1985-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    check whether the input by bcord of mshr3d is correct
!
! **********************************************************************
!
!                       KEYWORDS
!
!   mesh
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      implicit none 
      double precision bcord(*)
      integer nx, ny, nz, ivolnr, inpelm, kbndpt(*)

!     bcord      i    array containing the coordinates of the boundaries
!     inpelm     i    Number of points per element
!     ivolnr     i    Volume sequence number
!     kbndpt     i    array with the node numbers of the boundary elements
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, dist, ds, eps
      integer i, i1, i2, i4, i5, ist1, ist2, ist3

!     afst    ?
!     dist    ?
!     ds      ?
!     eps     Local accuracy
!     i       Counting variable
!     i1      First point of curve in KMESH part m
!     i2      Last point of curve in KMESH part m
!     i4      ?
!     i5      ?
!     ist1    ?
!     ist2    ?
!     ist3    ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     MSH304  determine distance of nodal points n and m
!     MSH307  ?
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     299   Surfaces have not been created in the correct sequence
!    1099   Diagonal for triangle has wrong direction
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      call eropen ( 'msh308' )
      call errint ( ivolnr, 1 )

!     --- determine value for eps

      i1 = 1
      i2 = nx
      i4 = (ny-1)*nx+1
      i5 = nx*ny+nx*nz-nz+1
      call msh304(i1,i2,bcord,dist)
      afst = dist/nx
      call msh304(i1,i4,bcord,dist)
      afst = dist/ny+afst
      call msh304(i1,i5,bcord,dist)
      afst = dist/nz+afst
      eps = sqreps*afst*100d0

!     --- point 1

      ist1 = 0
      ist2 = 3*nx*ny
      ist3 = 3*nx*ny+6*nx*nz+3*ny*nz
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 1, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 2

      ist1 = 3*(nx-1)
      ist2 = 3*nx*ny+ist1
      ist3 = 3*nx*(ny+nz)
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 2, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 3

      ist1 = 3*(nx*ny-1)
      ist2 = 3*nx*(ny+nz)+3*(ny-1)
      ist3 = 3*(nx*ny+nx*nz+ny*nz+nx-1)
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 3, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 4

      ist1 = 3*nx*(ny-1)
      ist2 = 3*(nx*ny+nx*nz+ny*nz)
      ist3 = ist2+3*(nx*nz+ny-1)
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 4, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 5

      ist1 = 3*nx*(ny+nz-1)
      ist2 = 3*(nx*ny+2*nx*nz+ny*nz+ny*(nz-1))
      ist3 = 3*nx*ny+6*nx*nz+6*ny*nz
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 5, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 6

      ist1 = 3*(nx*ny+nx*nz-1)
      ist2 = 3*(nx*ny+nx*nz+ny*nz-ny)
      ist3 = 3*(nx*ny+2*nx*nz+2*ny*nz+nx-1)
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 6, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 7

      ist1 = 3*(nx*ny+nx*nz+ny*nz-1)
      ist2 = 3*(nx*ny+2*nx*nz+ny*nz-1)
      ist3 = 6*(nx*ny+nx*nz+ny*nz)-3
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 7, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if

!     --- point 8

      ist1 = 3*(nx*ny+nx*nz+ny*nz+nx*nz-nx)
      ist2 = 3*(nx*ny+2*nx*nz+2*ny*nz-1)
      ist3 = 6*(nx*ny+nx*nz+ny*nz)-3*nx
      call msh307 ( bcord, ist1, ist2, ist3, ds )
      if ( ds.gt.eps ) then

          call errint ( 8, 2 )
          call errint ( ist1/3+1, 3 )
          call errint ( ist2/3+1, 4 )
          call errint ( ist3/3+1, 5 )
          do i = 1, 3
             call erreal ( bcord(ist1+i), i )
             call erreal ( bcord(ist2+i), i+3 )
             call erreal ( bcord(ist3+i), i+6 )
          end do
          call errsub ( 299, 5, 9, 0)

      end if
      if ( inpelm.eq.4 ) then

!     --- linear tetrahedrons, test direction of diagonal at side

         if ( kbndpt(2).ne.1 .or. kbndpt(4).ne.2 .or. kbndpt(6).ne.1 ) &
            then
            call errint ( 1, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = (nx-1)*(ny-1)*6
         if ( kbndpt(ist1+2).ne.1 .or. kbndpt(ist1+4).ne.2 .or. &
              kbndpt(ist1+6).ne.1 ) then
            call errint ( 2, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (nx-1)*(nz-1)*6
         if ( kbndpt(ist1+2).ne.1 .or. kbndpt(ist1+4).ne.2 .or. &
              kbndpt(ist1+6).ne.1 ) then
            call errint ( 3, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (ny-1)*(nz-1)*6
         if ( kbndpt(ist1+2).ne.1 .or. kbndpt(ist1+4).ne.2 .or. &
              kbndpt(ist1+6).ne.1 ) then
            call errint ( 4, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (nx-1)*(nz-1)*6
         if ( kbndpt(ist1+2).ne.1 .or. kbndpt(ist1+4).ne.2 .or. &
              kbndpt(ist1+6).ne.1 ) then
            call errint ( 5, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (ny-1)*(nz-1)*6
         if ( kbndpt(ist1+2).ne.1 .or. kbndpt(ist1+4).ne.2 .or. &
              kbndpt(ist1+6).ne.1 ) then
            call errint ( 6, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if

      else if ( inpelm.eq.10 ) then

!     --- quadratic tetrahedrons, test direction of diagonal at side

         if ( kbndpt(3).ne.1 .or. kbndpt(7).ne.3 .or. kbndpt(11).ne.1 ) &
            then
            call errint ( 1, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = (nx-1)*(ny-1)*3
         if ( kbndpt(ist1+3).ne.1 .or. kbndpt(ist1+7).ne.3 .or. &
              kbndpt(ist1+11).ne.1 ) then
            call errint ( 2, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (nx-1)*(nz-1)*3
         if ( kbndpt(ist1+3).ne.1 .or. kbndpt(ist1+7).ne.3 .or. &
              kbndpt(ist1+11).ne.1 ) then
            call errint ( 3, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (ny-1)*(nz-1)*3
         if ( kbndpt(ist1+3).ne.1 .or. kbndpt(ist1+7).ne.3 .or. &
              kbndpt(ist1+11).ne.1 ) then
            call errint ( 4, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (nx-1)*(nz-1)*3
         if ( kbndpt(ist1+3).ne.1 .or. kbndpt(ist1+7).ne.3 .or. &
              kbndpt(ist1+11).ne.1 ) then
            call errint ( 5, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if
         ist1 = ist1 + (ny-1)*(nz-1)*3
         if ( kbndpt(ist1+3).ne.1 .or. kbndpt(ist1+7).ne.3 .or. &
              kbndpt(ist1+11).ne.1 ) then
            call errint ( 6, 2 )
            call errsub ( 1099, 2, 0, 0 )
         end if

      end if

!     --- end of test

      call erclos ( 'msh308' )
      if ( ierror.ne.0 ) call instop
      end
