      subroutine msh4152darc ( curves, curvnw, n, iinput, icurve, &
                               rinput, j, jn )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 14-08-2001 1d version of old msh415
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to compute the coordinates and place the midpoints
!     in all parts of an arc in 2d
! **********************************************************************
!
!                       KEYWORDS
!
!     midpoint
!     polygon
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, iinput(*), j, jn, icurve
      double precision curves(2,*), curvnw(2,*), rinput(2,*)

!     curves         i    old coordinates array
!     curvnw         o    new coordinates array
!     icurve         i    Type of curve
!     iinput         i    integer input array
!     j             i/o   Counting variable for old array curves
!     jn            i/o   Counting variable for new array curves
!     n              i    number of points in local polygon
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jpnt, iuserp
      double precision x1, x2, xn, y1, y2, yn, xc, yc, r, r1

!     iuserp         User point sequence number
!     jpnt           last point number used
!     r              radius
!     r1             computed radius if point is in the middle of two neighbours
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     xc             x-coordinate of centroid
!     xn             x coordinate of new point
!     y1             y-coordinate of first point in curve
!     y2             y-coordinate of last point in curve
!     yc             y-coordinate of centroid
!     yn             y-coordinate of new point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     PSEUDO CODE
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( icurve.eq.17 ) then
         iuserp = abs(iinput(1))
      else
         iuserp = abs(iinput(3))
      end if
      xc = rinput(1,iuserp)
      yc = rinput(2,iuserp)
      x1 =  curves(1,j+1)
      y1 =  curves(2,j+1)
      r = sqrt((xc-x1)**2+(yc-y1)**2)
      do jpnt = 1, n-1
         j = j + 1
         x1 = curves(1,j)
         y1 = curves(2,j)
         x2 = curves(1,j+1)
         y2 = curves(2,j+1)
         xn = ( x1+x2 ) * 0.5d0
         yn = ( y1+y2 ) * 0.5d0
         r1 = r/sqrt((xc-xn)**2+(yc-yn)**2)
         xn = xc+r1*(xn-xc)
         yn = yc+r1*(yn-yc)
         jn = jn + 1
         curvnw(1,jn) = x1
         curvnw(2,jn) = y1
         jn = jn + 1
         curvnw(1,jn) = xn
         curvnw(2,jn) = yn
      end do
      j    = j + 1
      jn   = jn+1
      curvnw(1,jn) = x2
      curvnw(2,jn) = y2
      end
