      subroutine msh446 ( kelem , nelem , kelemh, coorh , npoint, &
                          ibrpnt, istart, npnold, nextra )
! ======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.1    date   27-04-1995 New basis for triangles
!        version 2.0    date   14-02-1995 Complete redesign
!        version 1.0    date   27-03-1992
!
!
!
!   copyright (c) 1992 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Place a new point in centre of each tetrahedron, store coordina- &
!     tes in array coorh and create new elements in kelemh
!
! ***********************************************************************
!
!     KEYWORDS
!
!     3d
!     create_point
!     create_element
!     mesh
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none

      integer kelem(*), nelem, nextra, npoint, kelemh(*), &
              ibrpnt(*), istart(*), npnold

      double precision coorh(*)

!     coorh   o   new coordinate array
!     ibrpnt  i   neighbour array
!     istart  i   array with starting positions in ibrpnt
!     kelem   i   element array
!     kelemh  o   new element array
!     nelem   i   number of elements
!     nextra  i   variable to add to node number of barycenter in
!                 order to take into account other groups of elements
!     npnold  i   number of points in old mesh
!     npoint  i   number of nodes after all "lines" have been filled
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11, i12, i13, &
              i14, i15, ia, ib, ic, ih, nelhel

!     i         loop variable
!     i1        node number
!     i10       node number
!     i11       node number
!     i12       node number
!     i13       node number
!     i14       node number
!     i15       node number
!     i2        node number
!     i3        node number
!     i4        node number
!     i5        node number
!     i6        node number
!     i7        node number
!     i8        node number
!     i9        node number
!     ia        help node number
!     ib        help node number
!     ic        help node number
!     ih        help node number
!     nelhel    local element number
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     MSH403    Determine node number of new middle point
!     MSH418    Fill element in element array, linear hexahedron
!     MSH461    Order the nodes of a triangle
!
! ***********************************************************************
!
!     I / O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all elements
!
!       Determine for each element the node numbers of the points
!       in the middle of the edges
!       Determine for each face the node number of the barycenter
!
!       Fill coorh for centre point and determine node number
!
!       Fill new elements (in total 4 hexaheders)
!
! =======================================================================

!     Check each element; place coordinates of new point in array coorh
!     and nodenumbers of new elements in array kelemh

      nelhel = 0

      do 100 i = 1 , nelem

          i1 = kelem(4*i-3)
          i2 = kelem(4*i-2)
          i3 = kelem(4*i-1)
          i4 = kelem(4*i  )

!         Compute middle node numbers

          call msh403( npnold, i1, i2, ibrpnt, istart, i5 )
          call msh403( npnold, i2, i3, ibrpnt, istart, i6 )
          call msh403( npnold, i3, i1, ibrpnt, istart, i7 )
          call msh403( npnold, i2, i4, ibrpnt, istart, i8 )
          call msh403( npnold, i3, i4, ibrpnt, istart, i9 )
          call msh403( npnold, i1, i4, ibrpnt, istart, i10 )

!         Compute barycenter of faces node numbers

          call msh461( i1, i2, i3, ia, ib, ic )

          ih = - ( ib + npnold * ic )

          call msh403( npnold, ia, ih, ibrpnt, istart, i11 )

          call msh461( i1, i2, i4, ia, ib, ic )

          ih = - ( ib + npnold * ic )

          call msh403( npnold, ia, ih, ibrpnt, istart, i12 )

          call msh461( i2, i3, i4, ia, ib, ic )

          ih = - ( ib + npnold * ic )

          call msh403( npnold, ia, ih, ibrpnt, istart, i13 )

          call msh461( i1, i3, i4, ia, ib, ic )

          ih = - ( ib + npnold * ic )

          call msh403( npnold, ia, ih, ibrpnt, istart, i14 )

!         Fill coorh for point i15 (adjust with nextra)

          i15 = npoint + nextra + i

          coorh(3*i15-2) = ( coorh(3*i1-2) + coorh(3*i2-2) + &
                             coorh(3*i3-2) + coorh(3*i4-2) ) / 4d0

          coorh(3*i15-1) = ( coorh(3*i1-1) + coorh(3*i2-1) + &
                             coorh(3*i3-1) + coorh(3*i4-1) ) / 4d0

          coorh(3*i15  ) = ( coorh(3*i1  ) + coorh(3*i2  ) + &
                             coorh(3*i3  ) + coorh(3*i4  ) ) / 4d0

!         Fill new elements (in total 4 hexahedrons)

          nelhel = nelhel + 1
          call msh418( kelemh, nelhel, i4, i10, i14, i9, i8, i12, &
                       i15, i13 )
          nelhel = nelhel + 1
          call msh418( kelemh, nelhel, i1, i5, i11, i7, i10, i12, &
                       i15, i14 )
          nelhel = nelhel + 1
          call msh418( kelemh, nelhel, i5, i2, i6, i11, i12, i8, &
                       i13, i15 )
          nelhel = nelhel + 1
          call msh418( kelemh, nelhel, i7, i11, i6, i3, i14, i15, &
                       i13, i9 )

100   continue

      end
