      subroutine msh648( i, j, k, l, m, acoor, t )

! =======================================================================
!
!
!         programmer   niek praagman
!
!         version 1.0    date  09-09-1992
!
!
!
!   copyright (c) 1992,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     Subroutine to determine whether line i-j has a point in common
!     with triangle k l m.
!
!
! ***********************************************************************
!
!     KEYWORDS
!
!     3d
!     intersection
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!

      implicit none

      integer i, j, k, l, m
      double precision acoor(3,*), t

!
!     acoor      (i)  :  coordinate array
!
!     i          (i)  :  node number
!
!     j          (i)  :  node number
!
!     k          (i)  :  node number
!
!     l          (i)  :  node number
!
!     m          (i)  :  node number
!
!     t          (o)  :  relative intersection coordinate
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      double precision px1, py1, pz1, px2, py2, pz2, &
                       pxa, pya, pza, pxb, pyb, pzb, pxc, pyc, pzc

!     px1
!     px2    x-coordinates
!     pxa
!     pxb
!     pxc
!
!     py1
!     py2    y-coordinates
!     pya
!     pyb
!     pyc
!
!     pz1
!     pz2    z-coordinates
!     pza
!     pzb
!     pzc
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSH610    Computes intersection coordinate t
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     trivial
!
! =======================================================================

      call eropen( 'msh648' )

!     Compute coordinates

!     Points on line

      px1 = acoor(1,i)
      py1 = acoor(2,i)
      pz1 = acoor(3,i)

      px2 = acoor(1,j)
      py2 = acoor(2,j)
      pz2 = acoor(3,j)

!     Nodes in Triangle

      pxa = acoor(1,k)
      pya = acoor(2,k)
      pza = acoor(3,k)

      pxb = acoor(1,l)
      pyb = acoor(2,l)
      pzb = acoor(3,l)

      pxc = acoor(1,m)
      pyc = acoor(2,m)
      pzc = acoor(3,m)

!     Compute t-value

      call msh610( px1, py1, pz1, px2, py2, pz2, &
                   pxa, pya, pza, pxb, pyb, pzb, pxc, pyc, pzc, t )

      call erclos( 'msh648' )

      end
