      subroutine mshadd3dnod ( tetnode, tetnodenew, npointnew, ielemnew, &
                               coornew, iwork, iedges, nmeshc, &
                               icrossobst, rcrossobst, ielem, ikelmcnew, &
                               istarticross, skipnod, &
                               eps, inpelm, ielmfaces, nmesha, nmeshe, &
                               fluidnd, inelem1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 05-04-2005 Fortran 90
!        version  3.1  date 23-06-2004 Update by RvL
!        version  3.0  date 07-04-2004 Extra parameters
!        version  2.0  date 19-10-2001 Remove parameters
!        version  1.0  date 07-03-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron if necessary into subtetareders and
!     indicate which of these subtetareders is in or out the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, tetnode(inpelm), tetnodenew(inpelm,8), &
              npointnew, ielemnew, iwork(*), fluidnd(*), inelem1, &
              iedges(3,*), nmeshc(*), nmeshe(*), &
              icrossobst(*), ielem, ikelmcnew, istarticross, &
              skipnod(*), ielmfaces(4,*), nmesha(*)
      double precision coornew(3,*), rcrossobst(*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     eps            i    Local accuracy
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iedges        i/o   Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i, &
!                         with sequence number iedges(i)
!     ielem          i    Element number
!     ielemnew      i/o   Sequence number of element in new mesh
!     ielmfaces      i    ?
!     ikelmcnew     i/o   Starting address of next element in kmeshcnew
!     inelem1        i    ?
!     inpelm         i    number of points per element
!     istarticross   i    Starting address of obstacle in array icrossobst
!     iwork          i    integer work array of size nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!     nmesha         i    nmesh part a
!     nmeshc         i    nmesh part c
!     nmeshe         i    nmesh part e contains the node sequence numbers of
!                         the elements
!     npointnew     i/o   Sequence number of last generated node
!     rcrossobst     i    Real array corresponding to icrossobst
!                         For each cross section a real factor f is stored
!                         defining the intersection of the edge with the
!                         obstacle in the following way
!                         x_intersect = x1 + f * (x2-x1)
!                         where x1 and x2 are the first and last point of the
!                         edge respectively
!     skipnod        i    Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer edgenode(5,6), istart, icross, nodebegin, nodeend, &
              ipstart, i, iedge, nextranodes, ireledge, idx(4), &
              nelemnew, centroid(12), midfacenodes(4), facenode(4,4), &
              idx1(3), idx2(3), idx3(3), idx4(3), istart1, j, k, l, &
              ifaces(3,4), nsection(4), npointnewsave, nextranodes1, &
              maxnpoint
      double precision eps, fact

!     centroid       ?
!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     facenode       ?
!     fact           Interpolation factor
!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     idx            ?
!     idx1           ?
!     idx2           ?
!     idx3           ?
!     idx4           ?
!     iedge          Edge sequence number
!     ifaces         ?
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     maxnpoint      ?
!     midfacenodes   ?
!     nelemnew       Number of elements in new mesh
!     nextranodes    Number of extra nodes in element
!     nextranodes1   ?
!     nodebegin      node at the begin of a crossed edge
!     nodeend        node at the end of a crossed edge
!     npointnewsave  ?
!     nsection       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS3DQ1  Subdivide each tetrahedron into the right number of
!                    subtetrahedrons, quadratic case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     eps := 0.2
!     nextranodes = 0
!     do i = 1,number of intersections
!        iedge := i-th edge which is intersected
!        fact := interpolation factor
!        if ( cross section with edge close to end point of edge ) then
!           mark corresponding node as on boundary
!        else ( i-th edge is intersected )
!           nextranodes := nextranodes+1
!           edgenode(1,i) := first point of edge
!           edgenode(2,i) := last point on edge
!           if ( first time edge is intersected) then
!              mark edge as intersected
!              introduce the intersection node
!              calculate its coordinates
!              mark it as boundary node
!              store node interpolation data
!           end if
!           edgenode(3,i) := intersection node
!        end if
!     end do
!     if ( nextranodes/=0 ) then
!        count(i) := number of intersected edges which contain tetnode(i)
!        ledgenode(1:2,j) are the local numbers of the corner nodes which lay
!                        on the same edge as intersection node j.
!        isort := the sort index of count, in other words
!                 if ( isort(i)>isort(j) then count(i)>count(j)
!     end if
!     subdivide tetrahedron into the right number of subtetrahedrons, &
!     if ( checksubtetrahedrons) then
!        check volumes of subtetrahedrons
!     end if
!     update pointers ikelmcnew and ielemnew
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx /4, 1, 2, 3/
      data idx1 /2, 4, 6/
      data idx2 /2, 7, 8/
      data idx3 /4, 8, 9/
      data idx4 /7, 9, 6/
! ======================================================================
!
      call eropen ( 'mshadd3dnod' )

!     --- nextranodes := 0

      nextranodes = 0

!     --- loop over the cut edges which
!         1. checks whether the crossing point is far enough from the
!            tetrahedron nodes
!         2. If so, introduces a new node
!         3. Set/adjust node properties

      istart = iwork(ielem)

!     --- find for every face the middle nodes of the edges

      istart1 = nmeshe(ielem)
      do i=1,3
         ifaces(i,1)=nmeshe(istart1+idx1(i))
         ifaces(i,2)=nmeshe(istart1+idx2(i))
         ifaces(i,3)=nmeshe(istart1+idx3(i))
         ifaces(i,4)=nmeshe(istart1+idx4(i))
      end do

      do i=1,4
         nsection(i)=0
      end do

!     --- do i = 1,nintersect

      do i = 1,icrossobst(istart)
         ireledge = icrossobst(istart+i)
         icross  = istarticross+1+2*(ireledge-1)

!        --- iedge := edge which is intersected

         iedge =            icrossobst(icross)
         fact  = rcrossobst(icrossobst(icross+1))
         ipstart = nmeshc(iedge)
         nodebegin=nmeshc(ipstart+1)
         if ( inpelm==4 ) then
             nodeend=nmeshc(ipstart+2)
         else
             nodeend=nmeshc(ipstart+3)
         end if
         if ( skipnod(nodebegin)/=3 .and. &
              skipnod(nodeend)/=3 ) then

            if ( fact<=1d0-eps .and. fact>=eps ) then

!           --- i-th edge is intersected
!               increment nextranodes
!               edgenode(1,nextranodes) := first point of edge
!               edgenode(2,nextranodes) := last point on edge

               nextranodes = nextranodes+1
               edgenode(1,nextranodes) = nodebegin
               edgenode(2,nextranodes) = nodeend

               if ( iedges(1,iedge)==0 ) then

!              --- New point
!                  npointnew := npointnew+1
!                    Fill coordinates of cross section in coornew(npointnew)
!                    mark node as boundary node

                  if (inpelm==4) then
                     npointnew = npointnew+1
                     iedges(1,iedge) = npointnew
                  else if (inpelm==15) then
                     iedges(2,iedge) = npointnew+1
                     iedges(3,iedge) = npointnew+2
                     npointnew=npointnew+2
                     npointnewsave = npointnew
                     npointnew = nmeshc(ipstart+2)
                     iedges(1,iedge) = npointnew
                  end if

                  coornew(1,npointnew) = (1d0-fact)* &
                                coornew(1,edgenode(1,nextranodes))+ &
                                fact*coornew(1,edgenode(2,nextranodes))
                  coornew(2,npointnew) = (1d0-fact)* &
                                coornew(2,edgenode(1,nextranodes))+ &
                                fact*coornew(2,edgenode(2,nextranodes))
                  coornew(3,npointnew) = (1d0-fact)* &
                                coornew(3,edgenode(1,nextranodes))+ &
                                fact*coornew(3,edgenode(2,nextranodes))
                  npointnew=npointnewsave

               end if

!              --- edgenode(3,nextranodes) := cross section node

               if (inpelm==4) then

                  edgenode(3,nextranodes) = iedges(1,iedge)

               else if (inpelm==15) then

                  edgenode(3,nextranodes) = iedges(2,iedge)
                  edgenode(4,nextranodes) = iedges(1,iedge)
                  edgenode(5,nextranodes) = iedges(3,iedge)

               end if

               do l=1 ,4 !loop over faces

                  k=0
                  do j=1,3 !loop over midedgenodes of the faces
                     if (ifaces(j,l)==iedges(1,iedge)) k=1
                  end do
                  if (k==1) nsection(l)=nsection(l)+1

               end do

            end if

         end if

      end do  ! i = 1,icrossobst(istart)

      if (nextranodes/=0) then

         istart1 = nmeshe(ielem)
         do j=1,4
            midfacenodes(j)=nmeshe(istart1+10+j)
         end do

!        --- loop over faces per element

         istart1 = nmesha(ielem)
         do i=1,4

!        --- face has already been filled

            if (ielmfaces(1,abs(nmesha(istart1+idx(i))))/=0) then

!           --- loop over extra nodes filled per face

               do j=1,4
                  facenode(j,i)=ielmfaces(j,abs(nmesha(istart1+idx(i))))
               end do

            else

!           --- face has not yet been filled

               if (nsection(i)/=0) then

                  if (nsection(i)==1) nextranodes1=2
                  if (nsection(i)==2) nextranodes1=4

!                 --- loop over extra nodes to be filled per face

                  do j=1,nextranodes1
                     npointnew=npointnew+1
                     facenode(j,i)=npointnew
                     ielmfaces(j,abs(nmesha(istart1+idx(i))))=npointnew
                  end do

               end if

            end if

         end do

!        --- find the number of extra nodes to be stored in
!            array 'centroid' depending on the number of
!            intersection in the element

!            --- maximum length of centroid is 12

         do i=1, 12
            npointnew=npointnew+1
            centroid(i)=npointnew
         end do

      end if

!     --- Create new elements

      call mshobstms3dq1 ( tetnode, tetnodenew, coornew, &
                           nextranodes, nelemnew, edgenode, &
                           inpelm, centroid, midfacenodes, &
                           facenode, fluidnd(inelem1*7+1), inelem1 )

      maxnpoint=0

      if (inpelm==4) then

         do i=1, nelemnew

            do j=1,inpelm
               imap(tetnodenew(j,i),1)=-1
            end do

         end do

      else if (inpelm==15) then

         do i=1, nelemnew

            do j=1,15

               imap(tetnodenew(j,i),1)=-1

               if (tetnodenew(j,i)>maxnpoint) then
                  maxnpoint=tetnodenew(j,i)
               end if

            end do

         end do

      end if

!     --- reset npointnew

      if (nelemnew>1) then
         npointnew=maxnpoint
      end if

!     --- update pointers ielemnew and ikelmcnew

      ielemnew = ielemnew+nelemnew-1
      ikelmcnew = ikelmcnew+inpelm*nelemnew

      call erclos ( 'mshadd3dnod' )
      end
