      subroutine msharea ( x1, x2, x3, area )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the area of the triangle spanned by x1-x2-x3
!     The sign of the are defines the direction
! **********************************************************************
!
!                       KEYWORDS
!
!     area
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision area, x1(*), x2(*), x3(*)

!     area           o    Area of triangle
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
!     x3             i    coordinates of third point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The area is defined as:
!
!     cos(phi) = (a,b)/||a||||b||, &
!
!     with a = x3-x1 and b = x2-x1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msharea' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msharea'

      end if
      area = 0.5d0 * ( &
         (x3(1)-x2(1))*(x1(2)-x2(2)) - (x2(2)-x3(2))*(x2(1)-x1(1)) )

      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msharea'

      end if
      call erclos ( 'msharea' )

      end
