      subroutine mshcoor3dquad ( coor, npoint, nedges, nfaces, nelem, &
                                 ishape, nmeshb, nmeshc, kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-05-2005 Fortran 90
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coor in the specific case of a pipe and quadratic tetrahedrons of
!     shape 15 or 16
!     It has been assumed that coor has been filled before for the
!     corresponding linear tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     quadratic
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nedges, nfaces, nelem, ishape, nmeshb(*), &
              kmeshc(4,*), nmeshc(*)
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         volume
!                         At input the linear tetrahedrons with corresponding
!                         node numbers
!                         At output the tetrahedrons of shape 15/16 with the new
!                         numbering
!     ishape         i    Shape number of elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements (linear case)
!     nedges         i    number of edges of volume
!     nelem          i    Number of elements of volume
!     nfaces         i    Number of faces of volume
!     nmeshb         i    nmesh part b
!     nmeshc         i    nmesh part c
!     npoint         i    Number of nodal points of volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, istart, nod1, nod2, nod3, j, npointedges, iface, &
              iedge1, iedge2, iedge3, nod4, ielem, nodc
      double precision onethird

!     iedge          edge sequence number
!     iedge1         edge sequence number
!     iedge2         edge sequence number
!     iedge3         edge sequence number
!     ielem          Element number
!     iface          Face sequence number
!     istart         First position used at a part
!     j              Counting variable
!     nod1           Node number of vertex 1
!     nod2           Node number of vertex 2
!     nod3           Node number of vertex 3
!     nod4           Node number of vertex 4
!     nodc           Node number of centroid
!     npointedges    Number of vertices + number of edges in volume
!     onethird       1/3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcoor3dquad' )
      onethird = 1d0/3d0

!     --- Fill co-ordinates of edges

      do iedge = 1, nedges

         istart = nmeshc(iedge)
         nod1 = nmeshc(istart+1)
         nod2 = nmeshc(istart+2)
         nod3 = npoint+iedge
         do j = 1, 3
            coor(j,nod3) = 0.5d0*(coor(j,nod1)+coor(j,nod2))
         end do

      end do
      npointedges = npoint+nedges

!     --- Fill co-ordinates of faces

      do iface = 1, nfaces

         istart = nmeshb(iface)
         iedge1 = abs(nmeshb(istart+1))+npoint
         iedge2 = abs(nmeshb(istart+2))+npoint
         iedge3 = abs(nmeshb(istart+3))+npoint
         nod4 = npointedges+iface
         do j = 1, 3
            coor(j,nod4) = onethird* &
                          (coor(j,iedge1)+coor(j,iedge2)+coor(j,iedge3))
         end do

      end do
      npointedges = npointedges+nfaces

      if ( ishape==16 ) then

!     --- Nodes in centroids

         do ielem = 1, nelem
            nod1 = kmeshc(1,ielem)
            nod2 = kmeshc(2,ielem)
            nod3 = kmeshc(3,ielem)
            nod4 = kmeshc(4,ielem)
            nodc = npointedges+ielem
            do j = 1, 3
               coor(j,nodc) = 0.25d0* (coor(j,nod1)+coor(j,nod2)+ &
                                       coor(j,nod3)+coor(j,nod4))
            end do
         end do

      end if

      call erclos ( 'mshcoor3dquad' )
      end
