      subroutine mshcr04 ( coorol, coornw, usol, npoint, amult )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  3.1  date 29-12-2008 Fortran 90
!        version  3.0  date 14-03-2003 Complete revision
!        version  2.0  date 20-09-2001 Remove jcart
!        version  1.2  date 28-02-1999 Include commons
!        version  1.1  date 07-09-1997 Correction axi-symmetric
!        version  1.0  date 08-04-1996
!
!   copyright (c) 1996-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the co-ordinates of the new boundary in msh047 in the case
!   iadapt = 4 i.e. x_new = x_old + factor u
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint
      double precision coorol(2,npoint), coornw(2,npoint), &
                       usol(npoint,2), amult

!     amult          i    Multiplication factor
!     coornw         o    Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     npoint         i    Number of nodal points
!     usol           i    Array of size npoint x 2 containing the
!                         velocity in the points of the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + factor u(i)
!    For all points i along the boundary do
!       x_new(i) := x_old(i) + u(i) * factor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      coornw(1,:) = coorol(1,:) + amult*usol(:,1)
      coornw(2,:) = coorol(2,:) + amult*usol(:,2)

      end
