      subroutine mshfil ( iinput, icurvs, iposi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.0  date 25-11-2013 Split in subroutines
!        version  8.1  date 15-11-2012 Long integers
!        version  8.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1987-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array KBNDPT for a composite surface
!     and write to file 3
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), iinput(*), iposi

!     icurvs       i/o    Array of length ncurvs containing information of the
!                         curves with respect to kbndptlc
!                         icurvs is filled as follows:
!                         icurvs(i) = 0    Curve i is not present
!                         icurvs(i)>0    Curve i has not been used before
!                                          The value gives the starting position
!                                          in array kbndptlc
!                         icurvs(i)<0    Curve i has been used before
!                                          Minus the value gives the starting
!                                          position in array kbndptlc
!     iinput        i     Array containing input for the composite surface
!     iposi         i     next free position of iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint, neltot, nwork, nptot, iuserpnt(nuspnt)
      integer, allocatable :: iwork(:)

!     ipoint
!     iuserpnt       Integer work array of length nuspnt
!                    In this array the local node number of the user points
!                    are stored once they are found
!                    Initially the array is set equal to 0 in mshfil
!                    iuserpnt is used by mshfl0
!     iwork          Integer work array
!     neltot         Total number of elements in composite surface
!     nptot          Total number of nodes in composite surface, &
!                    centroids excluded
!     nwork          Last position used by first part of IWORK
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFIL1112     fill array KBNDPT for a composite surface and write to
!                    file 3 for surfaces of type 11 and 12
!     MSHFIL12       fill array KBNDPT for a composite surface and write to
!                    file 3 for surfaces of type 12
!     MSHFIL7        fill array KBNDPT for a composite surface and write to
!                    file 3 for surfaces of type 7
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfil' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfil'
         write(irefwr,1) 'isurnr, isurf', isurnr, isurf
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear kbndptlc

      kbndptlc(1:nbndpt) = 0

!     --- Clear iuserpnt

      iuserpnt(1:nuspnt) = 0

!     --- Loop over the subsurfaces

      ipoint = npomxs + nelmax
      nwork = ipoint+nsurfs
      leniwk = 2*(npomxs+nelmax)+nsurfs+ncurvs
      if ( debug ) &
         write(irefwr,1) 'npomxs, nelmax, nwork', npomxs, nelmax, nwork

      allocate ( iwork(leniwk), stat = error )
      if ( error/=0 ) call eralloc ( error, leniwk, 'iwork' )
      if ( ierror/=0 ) go to 1000

      npointlc = 0
      nelemlc = 0
      nptot = 0
      if ( isurf==12 ) then

!     --- isurf = 12, ordered surface of pipe surfaces.
!                     if ishape = 7, we need the total number of elements

         call mshfil12 ( iinput, iposi, nptot, neltot )

      end if

      if ( isurf==11 .or. isurf==12 ) then

!     --- isurf = 11, 12  ordered surfaces of surfaces

         call mshfil1112 ( iinput, icurvs, kbndptlc, iwork, iposi, &
                           leniwk, nptot, neltot )

      else if ( isurf==7 ) then

!     --- isurf = 7, non-ordered surfaces of surfaces

         call mshfil7 ( iinput, icurvs, kbndptlc, iwork, &
                        iuserpnt, leniwk )

      end if

!     --- Fill information in kmeshh

      kmeshh(1,isurnr+1) = -npointlc
      kmeshh(2,isurnr+1) = nelemlc
      kmeshh(6,isurnr+1) = inpelm
      npomxs = max(npomxs,npointlc)
      nelmax = max(nelmax,nelemlc)

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

         write(irefwr,*) 'End mshfil'

      end if  ! ( debug )
      call erclos ( 'mshfil' )
      end
