      subroutine mshfil7 ( iinput, icurvs, &
                           kbndpt, iwork, iuserpnt, leniwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2014 Change isurf 23 in 99
!        version  1.0  date 25-11-2013
!
!   copyright (c) 2013-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array KBNDPT for a composite surface
!     and write to file 3
!     Type surface: 7 (not ordered)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), icurvs(*), iwork(*), iinput(*), &
              iuserpnt(*), leniwork

!     icurvs       i/o    Array of length ncurvs containing information of the
!                         curves with respect to kbndpt
!                         icurvs is filled as follows:
!                         icurvs(i) = 0    Curve i is not present
!                         icurvs(i)>0    Curve i has not been used before
!                                          The value gives the starting position
!                                          in array kbndpt
!                         icurvs(i)<0    Curve i has been used before
!                                          Minus the value gives the starting
!                                          position in array kbndpt
!     iinput        i     Array containing input for the composite surface
!     iuserpnt            Integer work array of length nuspnt
!                         In this array the local node number of the user points
!                         are stored once they are found
!                         Initially the array is set equal to 0 in mshfil7
!                         iuserpnt is used by mshfl0
!     iwork               Work array of length 2*(npoloc+nelloc)+nsurfs
!                         npoloc = number of points in surface
!                         nelloc = number of elements in surface
!                         With respect to mshfl0, iwork is extended with ncurvs
!                         positions
!     kbndpt         o    Array containing the boundary points of the
!                         composite surface;curves are counted for only once
!     leniwork       i    Computed length of array iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, index, ipoint, &
              istart, isub, isubsf, jstarr, jstart, &
              jsurf, leng3, ndefsf, nelloc, &
              ngboun, npoloc, nwork, &
              jinp, k, kinp, numcurs,  bouncurvs(ncurvs)
      integer, allocatable :: kbndlc(:)

!     bouncurvs      Array in which the boundary curves are stored, &
!                    in case of internal curves
!     i              Counting variable
!     index          Index array to be used if 1000<=ind(number)<2000 and
!                    ichois = 2
!     ipoint
!     istart         Starting address in array VECOLD
!     isub           Loop variable
!     isubsf         Subsurface sequence number
!     jinp           Starting address in array iinput with respect to surface
!     jstarr
!     jstart         Starting address in KMESH part r
!     jsurf
!     k              Counting variable
!     kbndlc         Local array KBNDPT corresponding to surface
!     kinp           Starting address in array iinput with respect to surface
!     leng3
!     ndefsf         Number of subsurfaces in surface
!     nelloc         Number of elements in local subsurface
!     ngboun         Number of curves the boundary of the surface of surface
!                    consists of
!     npoloc         Number of points in local subsurface
!     numcurs        Number of curves along surface
!     nwork          Last position used by first part of IWORK
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     MSHFL0         Fill array KBNDPT for a non-ordered surface
!     PRININ         print 1d integer array
!     WRITFL         Write array to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfil7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfil7'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ipoint = npomxs + nelmax
      nwork = ipoint+nsurfs

      jstart = kmeshr(isurnr)
      ndefsf = kmeshr(isurnr+1) - jstart
      if ( debug ) write(irefwr,1) 'jstart, ndefsf', jstart, ndefsf
      jstarr = kmeshr(nsurfs+1) - nsurfs-1 + kmeshr(isurnr)
      do isub = 1, ndefsf
         isubsf = abs( kmeshr(jstart+isub) )
         if ( debug ) write(irefwr,1) 'isub, isubsf', isub, isubsf
         jinp = iinp3    ! starting address of surfaces in iinput

!        --- Find starting address of subsurface in iinput

         do k = 1, isubsf-1
            jsurf = iinput(jinp)
            call msh049 ( iinput, jinp, jsurf )
         end do  ! k = 1, isubsf-1
         jsurf = iinput(jinp)

         numcurs = iinput(jinp+2)
         kinp = jinp+numcurs+2
         if ( jsurf==22 .or. jsurf==99 ) then
            nextrapnts = iinput(kinp+1)
            nextracrvs = iinput(kinp+2)
            kinp = kinp+2+nextrapnts
         else
            nextrapnts = 0
            nextracrvs = 0
         end if  ! ( jsurf==22 .or. jsurf==99 )

         if ( debug ) then
            write(irefwr,1) 'jinp, kinp, jsurf, numcurs', &
                             jinp, kinp, jsurf, numcurs
            write(irefwr,1) 'nextrapnts, nextracrvs', &
                             nextrapnts, nextracrvs
         end if  ! ( debug )

!        --- Fill local array kbndpt into kbndlc

         npoloc = abs(kmeshh(1,isubsf+1))
         nelloc = kmeshh(2,isubsf+1)
         if ( debug ) then
            write(irefwr,1) 'npoloc, nelloc, npointlc, nelemlc', &
                             npoloc, nelloc, npointlc, nelemlc
            write(irefwr,1) 'isubsf, kmeshh(4,isubsf+1)', &
                             isubsf, kmeshh(4,isubsf+1)
         end if  ! ( debug )

         if ( kmeshh(4,isubsf+1)>0 ) then

!        --- kmeshh(4,isubsf+1)>0, the boundary exists
!            copy kbndpt

            leng3  = kmeshh(4,isubsf+1)
            lenextracrvs = kmhsur(isubsf)%kbndptsur(1+leng3)-nextrapnts
            if ( debug ) &
               write(irefwr,1) 'lenextracrvs', lenextracrvs

            allocate ( kbndlc(leng3), stat = error )
            if ( error/=0 ) call eralloc ( error, leng3, 'kbndlc' )
            if ( ierror/=0 ) go to 1000

            kbndlc(1:leng3) = kmhsur(isubsf)%kbndptsur(1:leng3)
            if ( debug ) then
               write(irefwr,1) 'leng3', leng3
               call prinin ( kbndlc, leng3, 'kbndlc' )
            end if  ! ( debug )
            lendebug = leng3   ! just for debuging

!           --- Fill array iwork with the signs of the curve numbers as
!               present in array kmeshs

            ngboun = kmeshs(isurnr+1) - kmeshs(isurnr)
            if ( nwork+ngboun>leniwork ) then

!           --- Array iwork too small

               call errint ( nwork+ngboun, 1 )
               call errint ( leniwork, 2 )
               call errchr ( 'iwork', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if
            index = kmeshs(isurnr)
            do i = 1, ngboun
               iwork(nwork+abs(kmeshs(index+i))) = &
                     kmeshs(index+i)
            end do

            if ( debug ) &
               call prinin ( kmeshs(index+1), ngboun, 'kmeshs' )

!           --- Fill array kbndpt for surface isubsf

            if ( nextracrvs==0 ) then

!           --- Standard case, no extra curves

               istart = kmeshs(isubsf)
               nboun = kmeshs(isubsf+1) - istart
               if ( debug ) &
                  call prinin ( kmeshs(istart+1), nboun, &
                                'curves' )
               call mshfl0 ( kbndlc, kbndpt, icurvs, iwork, npoloc, &
                             nelloc, kmeshs(istart+1), &
                             iwork(nwork+1), iuserpnt )
            else

!           --- Internal curves are present
!               Fill curves in bouncurvs

               bouncurvs(1:numcurs) = iinput(jinp+3:jinp+numcurs+2)
               bouncurvs(numcurs+1:numcurs+nextracrvs) = &
                  iinput(kinp+1:kinp+nextracrvs)
               nboun = numcurs+nextracrvs
               if ( debug ) &
                  call prinin ( bouncurvs, nboun, &
                                'curves' )

               call mshfl0 ( kbndlc, kbndpt, icurvs, iwork, npoloc, &
                             nelloc, bouncurvs, &
                             iwork(nwork+1), iuserpnt )
            end if  ! ( nextracrvs==0 )

         else

!        --- No boundary fill iwork in standard sequence

            do i = 1, npoloc
               iwork(i) = npointlc+i
            end do  ! i = 1, npoloc
            do i = 1, nelloc
               iwork(i+npoloc) = nelemlc+i
            end do  ! i = 1, npoloc
            npointlc = npointlc+npoloc
            nelemlc = nelemlc+nelloc

         end if  ! ( kmeshh(4,isubsf+1)==0 )

         index = ifree3

         if ( debug ) then

!        --- debug information

            write(irefwr,1) 'npointlc, nelemlc, index', &
                             npointlc, nelemlc, index
            call prinin ( iwork, npoloc, 'iwork_nodes' )
            call prinin ( iwork(npoloc+1), nelloc, 'iwork_elements' )

         end if  ! ( debug )

         call writfl ( index, 1, npoloc+nelloc, 1, 3, iwork )
         jstarr = jstarr+1
         kmeshr(jstarr) = index

         deallocate ( kbndlc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kbndlc' )
         if ( ierror/=0 ) go to 1000

      end do

1000  if ( debug ) then

         write(irefwr,*) 'End mshfil7'

      end if  ! ( debug )
      call erclos ( 'mshfil7' )

      end subroutine mshfil7
