      subroutine mshfilcompsurf ( activatm, iinput, iposi, rinput, jnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 27-11-2012 Long integers
!        version  3.1  date 30-07-2012 New treatment of iinput/rinput
!        version  3.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information with respect to compound surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compound_surface
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi
      double precision rinput(*)
      logical activatm, jnew

!     activatm       i    If true kmesh part m must be activated
!     iinput         i    integer input array, standard for subroutine MESH
!                         The starting address in MESH is 6
!     iposi          i    next free position of iinput
!     jnew           i    indication whether the mesh is new (true) or old
!     rinput         i    Real input array, standard for subroutine MESH
!                         The starting address in MESH is 6
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoloc, nelloc, icurvstmp(ncurvs)

!     icurvstmp      Work array to store a temporary version of icurvs
!     nelloc         Local number of elements
!     npoloc         Number of nodes in subsurface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFIL         fill array KBNDPT for a composite surface and write to
!                    file 3
!     MSHFL1         fill array icurvs with starting positions in array kbndpt
!                    with respect to composite surface
!     MSHFL2         Compute length of various arrays for subroutine mshfil
!     MSHPLCOMPSURF  Plot composite surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfilcompsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfilcompsurf'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      nsurfs = iinput(4)
      if ( debug ) then
         write(irefwr,1) 'ndim, nuspnt, ncurvs, numsub, nbndpt, inpelm', &
                          ndim, nuspnt, ncurvs, numsub, nbndpt, inpelm
         write(irefwr,1) 'nbnmax, nsurfs', nbnmax, nsurfs
      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'before mshfl1'
      call mshfl1 ( icurvstmp )
      if ( debug ) write(irefwr,1) 'nbndpt', nbndpt

!     --- Finally fill array kbndpt for a complete surface
!         First compute length of arrays

      if ( debug ) write(irefwr,*) 'before mshfl2'
      call mshfl2 ( npoloc, nelloc )
      if ( debug ) &
         write(irefwr,1) 'npoloc, nelloc', npoloc, nelloc

      allocate ( kbndptlc(nbndpt), stat = error )
      if ( error/=0 ) call eralloc ( error, nbndpt, 'kbndptlc' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'before mshfil'
      call mshfil ( iinput, icurvstmp, iposi )
      if ( ierror/=0 ) go to 1000
      nbnmax = max ( nbnmax, nbndpt )

!     --- Store array kbndpt as permanent array in kmeshh

      if ( nbndpt>0 ) then
         if ( kmeshh(4,isurnr+1)>0 ) deallocate(kmhsur(isurnr)%kbndptsur)
         allocate ( km(iseqkm)%kmhsur(isurnr)%kbndptsur(nbndpt+1) )
         kmeshh(4,isurnr+1) = nbndpt
         kmhsur(isurnr)%kbndptsur(1:nbndpt) = kbndptlc(1:nbndpt)
         kmhsur(isurnr)%kbndptsur(nbndpt+1) = 0
      end if  ! ( nbndpt>0 )

      deallocate ( kbndptlc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kbndptlc' )
      if ( ierror/=0 ) go to 1000

      if ( ndim==3 .and. jnew .and. iplot/=0 .and. &
           isurnr>numsub ) then

!     --- Plot of compound surfaces has only been implemented for compound
!         surfaces

         npointlc = max(abs(kmeshh(1,isurnr+1)),npomxs)
         nelemlc  = max(kmeshh(2,isurnr+1),nelmax)

         if ( debug ) write(irefwr,*) 'before mshplcompsurf'
         call mshplcompsurf ( rinput )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfilcompsurf'

      end if
      call erclos ( 'mshfilcompsurf' )

      end subroutine mshfilcompsurf
