      subroutine mshfillmsub ( ncurvs, kmeshm, kmeshq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part m for composite curves
!     It is supposed that single curves have already been filled in kmeshm
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     refine
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, kmeshm(*), kmeshq(*)

!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         i    KMESH part q, contains the subcurve numbers of
!                         each curve
!     ncurvs         i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, nsubcv, i, jstart, ic, ih, inpncv, icurnr, j

!     i              Counting variable
!     ic             Relative subcurve number
!     icurnr         Curve sequence number
!     ih             Help variable to store a constant temporarily
!     inpncv         Number of points in curve
!     istart         First position used at a part
!     j              Counting variable
!     jstart         starting position of curve i in kmeshm
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillmsub' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillmsub'

      end if

      istart = kmeshm(2) - 1

!     --- Loop over curves

      do i = 1, ncurvs

         nsubcv = kmeshq(i+1) - kmeshq(i)
         jstart = istart + kmeshm(4+i) - 1
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'i, istart, jstart, ncurvs, nsubcv ', &
                             i, istart, jstart, ncurvs, nsubcv

         end if

!        --- Loop over subcurves

         do ic = 1, nsubcv
            icurnr = kmeshq( kmeshq(i)+ic )
            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'ic, icurnr ',ic, icurnr

            end if

            if ( icurnr.gt.0 ) then

!           --- icurnr > 0, copy curve forwards

               ih = istart + kmeshm(4+icurnr) - 1
               inpncv = kmeshm(5+icurnr) - kmeshm(4+icurnr)
               do j = 1, inpncv
                  kmeshm(jstart+j) = kmeshm(ih+j)
               end do

            else

!           --- icurnr < 0, copy curve backwards

               ih = istart + kmeshm(5-icurnr)
               inpncv = kmeshm(5-icurnr) - kmeshm(4-icurnr)
               do j = 1, inpncv
                  kmeshm(jstart+j) = kmeshm(ih-j)
               end do
            end if

            jstart = jstart + inpncv

         end do

      end do

      call erclos ( 'mshfillmsub' )

      end
