      subroutine mshfillvertsp2 ( kmeshc_loc, ishape, nvertices, ivert, &
                                  nside )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the node numbers of the vertices of a 2d element in array ivert
!     and store number of vertices in nvertices
!     Special case for spectral elements
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     vertex
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc_loc(*), ishape, nvertices, ivert(*), nside

!     ishape         i    Shape number of elements
!     ivert          o    Array of size nvertices containing the node numbers
!                         of the vertices of an element
!     kmeshc_loc     i    Local array kmeshc for one element
!                         Contains the node numbers of the element
!     nside          i    number of points along side of spectral element
!     nvertices      o    Number of vertices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillvertsp2' )

      if ( ishape.eq.3 ) then

!     --- Linear triangle

         nvertices = 3
         ivert(1) = kmeshc_loc(1)
         ivert(2) = kmeshc_loc(nside)
         ivert(3) = kmeshc_loc(nside*2-1)

      else if ( ishape.eq.5 ) then

!     --- Bi-linear quadrilateral

         nvertices = 4
         ivert(1) = kmeshc_loc(1)
         ivert(2) = kmeshc_loc(nside)
         ivert(3) = kmeshc_loc(nside*nside)
         ivert(4) = kmeshc_loc(nside*(nside-1)+1)

      else

!     --- shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )

      end if

      call erclos ( 'mshfillvertsp2' )

      end
