      subroutine mshfillvolsur ( iinput, lenvolsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 27-05-2002 Correction negative surfaces
!        version  1.0  date 08-03-2002
!
!   copyright (c) 2002-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array volume_surfs with all single surfaces enclosing the volume
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     surface
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lenvolsurfs

!     iinput         i    Standard integer input array for subroutine MESH
!                         Must be called with starting address 6
!     lenvolsurfs   i/o   Length of array volume_surfs
!                         At input declared length
!                         At output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ndefvl, icount, i, jsurnr, nsubsf, j, ivolm, iinp

!     i              Counting variable
!     icount         Counter
!     iinp           Position in array iinput
!     istart         First position used at a part
!     ivolm          Type of volume with respect to mesh generator
!     j              Counting variable
!     jsurnr         Surface sequence number
!     ndefvl         Number of surfaces in volume
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH050         Raise iinp as function of ivolm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     istart := nvolms+1
!     for all volumes ivolnr do
!        volume_surfs(ivolnr) := istart
!        ndefvl := number of surfaces corresponding to volume
!        icount := 0
!        for all surfaces jsurnr corresponding to volume do
!           if ( jsurnr is single surface ) then
!              icount := icount+1; volume_surfs(istart+icount) := jsurnr
!           else ( jsurnr is composite surface ) then
!              For all subsurfaces isurnr do
!                 icount := icount+1; volume_surfs(istart+icount) := isurnr
!        volume_surfs(istart) := icount
!        istart := istart+icount+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillvolsur' )

!     --- istart := nvolms+1

      istart = nvolms+1
      nsubsf = 0
      iinp = iinp5

!     --- for all volumes ivolnr do

      do ivolnr = 1, nvolms

!     --- volume_surfs(ivolnr) := istart
!         ndefvl := number of surfaces corresponding to volume
!         icount := 0

         volume_surfs(ivolnr) = istart
         ivolm = iinput(iinp)
         if ( ivolm==0 ) then
            ndefvl = 0
         else
            ndefvl = iinput(iinp+2)
         end if
         icount = 0

!        --- for all surfaces jsurnr corresponding to volume do

         do i = 1, ndefvl

            jsurnr = mod(abs(iinput(iinp+2+i))-1,nsurfs)+1
            if ( kelmr>0 ) nsubsf = kmeshr(jsurnr+1)-kmeshr(jsurnr)
            if ( nsubsf<=1 ) then

!           --- jsurnr is single surface
!               icount := icount+1; volume_surfs(istart+icount) := jsurnr

               icount = icount+1
               volume_surfs(istart+icount) = jsurnr

            else

!           --- jsurnr is composite surface
!               For all subsurfaces isurnr do

               do j = 1, nsubsf

                  isurnr =mod(abs(kmeshr(kmeshr(jsurnr)+j))-1,nsurfs)+1

!                 --- icount := icount+1; volume_surfs(istart+icount) := isurnr

                  icount = icount+1
                  volume_surfs(istart+icount) = isurnr

               end do

            end if

         end do

!        --- volume_surfs(istart) := icount
!            istart := istart+icount+1

         volume_surfs(istart) = icount
         istart = istart+icount+1
         if ( istart-1>lenvolsurfs ) then

!        --- Array length too small

            call errint ( istart-1, 1 )
            call errint ( lenvolsurfs, 2 )
            call errchr ( 'volume_surfs', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

!        --- Raise iinp

         call msh050 ( iinput, iinp, ivolm )

      end do
      lenvolsurfs = istart-1

1000  call erclos ( 'mshfillvolsur' )

      end
