      subroutine mshmetis4 ( iprocessor, nprocessors, numelms, &
                             iwork, elements, elperseg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add all elements of higher processor number that have all nodes
!     in this block to present processor
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
!     metis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iprocessor, nprocessors, iwork(*)
      integer, intent(inout) :: numelms, elperseg(*), elements(*)

!     elements      i/o   Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     elperseg      i/o   Array of size nelem containing the sequence
!                         number of the processor to which it is added
!     iprocessor     i    Processor sequence number
!     iwork          i    integer work array of size npoint
!                         iwork(i) = 0 means node is not present in this block
!                         iwork(i) = 1 means node is present in this block
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     numelms       i/o   Number of elements in processor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ielem, istart, ielgrp, inpelm, j, iielem
      logical :: inside

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     inside         if true all nodes of element belong to processor
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmetis4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmetis4'
         call prinin ( elements, numelms, 'elements' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all processors with lower number

      do j = 1, iprocessor-1

!     --- For all elements ielem do

         ielem  = 0
         istart = 0
         do ielgrp = 1, nelgrp

            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm

            do iielem = 1, inelem

               ielem = ielem + 1

               if ( elperseg(ielem)==j ) then

!              --- element processor j found

                  inside = .true.
                  do i = 1, inpelm

                     if ( iwork(kmeshc(istart+i))/=1 ) then

!                    --- Node not in element, jump

                        inside = .false.
                        exit

                     end if  ! ( iwork(kmeshc(istart+i))/=1 )

                  end do

                  if ( inside ) then

!                 --- All nodes of element in processor iprocessor
!                     Add element to this processor

                     numelms = numelms+1
                     elements(numelms) = ielem
                     elperseg(ielem) = iprocessor

                  end if  ! ( inside )

               end if
               istart = istart+inpelm

            end do  ! iielem = 1, inelem

         end do  ! ielgrp = 1, nelgrp

      end do  ! j = 1, iprocessor-1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( elements, numelms, 'elements' )
         write(irefwr,*) 'End mshmetis4'

      end if  ! ( debug )
      call erclos ( 'mshmetis4' )

      end subroutine mshmetis4
