      subroutine mshnegordsurf ( isurnr, isubsf, nsurfs, npoloc, nelloc, &
                                 kmeshr, ishape, kbndpt, inplmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 13-11-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder array kbndpt for negative subsurfaces of a compound
!     ordered surface
! **********************************************************************
!
!                       KEYWORDS
!
!     compound_surface
!     ordering
!     reordering
!     surfaces
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, nsurfs, npoloc, nelloc, &
              isubsf, kmeshr(*), ishape, kbndpt(*), inplmt

!     inplmt         i    Number of points in element of surface corresponding
!                         to volume
!     ishape         i    Shape number of elements
!     isubsf         i    Sequence number of subsurface
!     isurnr         i    Surface sequence number
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary for surface isurnr
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     nelloc         i    Local number of elements
!     npoloc         i    Number of nodes in subsurface
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint(npoloc+nelloc), index, jstmsr

!     index          Record number of information in file 3
!     ipoint         Array in which the local nodal point numbers and element
!                    numbers are stored of the subsurface with respect to
!                    array kbndpt
!     jstmsr         Position in kmeshr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHNEGORDSURF1 Reorder array kbndpt for negative subsurfaces of a
!                    compound ordered surface
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnegordsurf' )

!     --- Read sequence of elements with respect to compound surface
!         into array ipoint from position ipoint(npoloc+1)

      jstmsr = kmeshr(nsurfs+1) - nsurfs - 1 + kmeshr(isurnr)
      index = kmeshr(jstmsr+isubsf)
      call readfl ( index, 1, npoloc+nelloc, 1, 3, ipoint )

!     --- Negative subsurface of compound surface

      call mshnegordsurf1 ( ipoint(npoloc+1), kbndpt, ishape, nelloc, &
                            inplmt )
      call erclos ( 'mshnegordsurf' )
      end
