      subroutine mshnorm3d
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 27-11-2012 Long integers
!        version  2.2  date 15-09-2006 Use mshfillsurnod
!        version  2.1  date 06-04-2004 New call to mshfillsurnods
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute normals on boundary in R^3 and store in array normal
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     normal_vector
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHNORS0       Compute normals on boundary in R^3 and store in array
!                    normal
!     PRINRL1        Print 2d real vector
!     SEPFILLSURNODS Create and fill array surfnodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     Fill array surfnodes
!     make array normal a permanent array
!     compute starting addresses
!     Compute the normals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnorm3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnorm3d'

      end if

!     --- Fill array surfnodes

      call sepfillsurnods

!     --- make array normal a permanent array
!         the length of normal is 3 times the length of (surfnodes-2*nsurfs)

      length = size ( surfnodes )-2*nsurfs
      allocate ( km(iseqkm)%normal(3,length) )
      normal => km(iseqkm)%normal
      mmnormal = length*3
      if ( ierror/=0 ) go to 1000

!     --- Compute the normals

      call mshnors0
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( normal, length, 3, 'normals' )

      end if
1000  call erclos ( 'mshnorm3d' )
      end subroutine mshnorm3d
