      subroutine mshobstboun2d2 ( iworkedge, npointnew, iworkshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-05-2004 Complete revision, part of original
!        version  2.1  date 07-01-2004 Do a more subtle subdivision
!        version  2.0  date 14-10-2002 Extra parameters kmeshq, lencurvs
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new points in mesh by subdividing some edges of
!     elements that have all points on the boundary of the mesh and/or
!     the obstacle
!     First part initialize some arrays and parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     subdivision
!     edge
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iworkedge(*), npointnew, iworkshape(*)

!     iworkedge      o    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     iworkshape     o    Work array of length nelem that is filled as follows
!                         iworkshape(ielem) contains the shape number of the
!                         element ielem
!     npointnew      o    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, i, ielgrp

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iworkshape with the shape numbers of the elements
!     Clear array iworkedge
!     Copy old arrays imap and rmap into imap2 and rmap2
!     npointnew := npoint
!     For all elements ielem with all nodes on boundary do
!        For all edges iedge corresponding to ielem do
!           nneigh := number of triangles corresponding to iedge
!           if ( nneigh=2 ) then
!              create new point
!     For all elements ielem with all nodes on obstacle do
!        For all edges iedge corresponding to ielem do
!           compute coordinates of centroid of edge
!           if ( centroid outside obstacle ) then
!              create new point
!     For all elements ielem with some nodes on obstacle and all others on
!         boundary do
!        For all edges iedge corresponding to ielem do
!           if ( one endpoint on boundary and other on obstacle ) then
!              create new point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun2d2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstboun2d2'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Fill array iworkshape with the shape numbers of the elements

      if ( nelgrp.eq.1 ) then

!     --- nelgrp = 1, only one element group

         ishape = kmeshf(1)
         if ( ishape.ne.3 ) then

!        --- Shape number not allowed

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if
         do ielem = 1, nelem
            iworkshape(ielem) = 3
         end do

      else

!     --- nelgrp > 1

         ielem = 0
         do ielgrp = 1, nelgrp
            inelem = Topgrp(ielgrp)%inelem
            ishape = kmeshf(ielgrp)
            if ( ishape.ne.1 .and. ishape.ne.3 ) then

!           --- Shape number not allowed

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if
            do i = 1, inelem
               iworkshape(ielem+i) = ishape
            end do
            ielem = ielem+inelem
         end do

      end if

!     --- Clear array iworkedge

      do i = 1, nedges
         iworkedge(i) = 0
      end do

!     --- Copy old arrays imap and rmap into imap2 and rmap2

      do i = 1, npoint
         imap2(i,1) = imap(i,1)
         imap2(i,2) = imap(i,2)
         imap2(i,3) = imap(i,3)
         rmap2(i,1) = rmap(i,1)
         rmap2(i,2) = rmap(i,2)
      end do

!     --- npointnew := npoint

      npointnew = npoint

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstboun2d2'

      end if
      call erclos ( 'mshobstboun2d2' )

      end
