      subroutine mshobstboun2d4 ( iworkedge, kmeshcloc, kmeshcnewloc, &
                                  nmeshdloc, nelemloc, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into 2, 3 or 4 subtriangles, depending on the
!     number of edges with extra points
!     The new subdivision is stored in kmeshcnewloc
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     subdivision
!     triangle
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iworkedge(*), kmeshcloc(*), kmeshcnewloc(*), &
              nmeshdloc(*), nelemloc
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iworkedge      i    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     kmeshcloc      i    Local array kmeshc containing the node numbers of one
!                         element
!     kmeshcnewloc   o    New local array kmeshc containing the node numbers
!                         of the new elements
!     nelemloc       o    Number of new elements in the original one
!     nmeshdloc      i    Local array nmeshd containing the edge sequence
!                         numbers of the actual element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nsub_edges, iedge, isub_edges(3), intersect, nodp1, &
              nodp2, nodp3, nodp4, nodp5, nodp6, raise(5)
      double precision t1(2), t2(2), phi1, phi2, len1, len2

!     i              Counting variable
!     iedge          edge sequence number
!     intersect      sequence number of first local edge that is intersected
!     isub_edges     Array of length nsub_edges containing the sequence
!                    numbers of the local edges that are intersected
!     len1           Euclidian length of vector
!     len2           Euclidian length of vector
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     nodp3          Node sequence number
!     nodp4          Node sequence number
!     nodp5          Node sequence number
!     nodp6          Node sequence number
!     nsub_edges     Number of edges that are intersected in the element
!     phi1           angle
!     phi2           angle
!     raise          Raise-operator
!     t1             First tangential vector
!     t2             Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nsub_edges := number of edges that are intersected
!     isub_edges := local sequence numbers of edges that are intersected
!     if ( nsub_edges=1 ) then
!        rotate element such that first edge is intersected
!        subdivide element as follows
!
!               3
!              /|\
!             / | \
!            /  |  \             elements:  1-4-3
!           /   |   \                       2-3-4
!          /    |    \
!         /     |     \
!        x------x------x
!        1      4      2
!
!     else if ( nsub_edges=2 ) then
!        rotate element such that first edge is intersected
!        then either edge 2 or edge 3 is intersected
!        subdivide element as follows
!
!               3                           3
!              /|\                         /|\
!             / | \                       / | \
!         5  x  |  \                     /  |  x 5
!           / \ |   \                   /   | / \
!          /   \|    \                 /    |/   \
!         /     \     \               /     /     \
!        x------x------x             x------x------x
!        1      4      2             1      4      2
!
!        elements                     elements
!        1-4-5                        4-2-5
!        4-2-3 and 3-5-4              1-4-3 and 4-5-3
!   or   4-2-5 and 2-3-5         or   1-4-5 and 5-3-1
!
!     else if ( nsub_edges=3 ) then
!        subdivide element as follows
!
!               3
!              / \
!             /   \
!         6  x --- x 5               elements
!           / \    /\                1-4-6
!          /   \  /  \               4-2-5
!         /     \/    \              5-3-6
!        x------x------x             4-5-6
!        1      4      2
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise / 2, 3, 1, 2, 3 /
! ======================================================================
!
      call eropen ( 'mshobstboun2d4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstboun2d4'
         call prinin ( nmeshdloc, 10, 'nmeshdloc' )
         call prinin ( iworkedge, 10, 'iworkedge' )
         call prinin ( kmeshcloc, 3, 'kmeshcloc' )
         call prinrl1 ( coor, 4, 2, 'coor' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- nsub_edges := number of edges that are intersected
!         isub_edges := local sequence numbers of edges that are intersected

      nsub_edges = 0
      do i = 1, 3
         iedge = abs(nmeshdloc(i))
         if ( iworkedge(iedge)>0 ) then

!        --- edge is splitted into two parts

            nsub_edges = nsub_edges+1
            isub_edges(nsub_edges) = i

         end if

      end do

      if ( debug ) write(irefwr,1) 'nsub_edges', nsub_edges

      if ( nsub_edges==1 ) then

!     --- nsub_edges=1
!         rotate element such that first edge is intersected
!         subdivide element into two elements

         intersect = isub_edges(1)
         nodp1 = kmeshcloc(intersect)
         nodp2 = kmeshcloc(raise(intersect))
         nodp3 = kmeshcloc(raise(intersect+1))
         nodp4 = iworkedge(abs(nmeshdloc(intersect)))
         kmeshcnewloc(1) = nodp1
         kmeshcnewloc(2) = nodp4
         kmeshcnewloc(3) = nodp3

         kmeshcnewloc(4) = nodp2
         kmeshcnewloc(5) = nodp3
         kmeshcnewloc(6) = nodp4
         nelemloc = 2

      else if ( nsub_edges==2 ) then

!     --- nsub_edges=2
!         rotate element such that first edge is intersected
!         subdivide element into three elements

         intersect = isub_edges(1)
         nodp1 = kmeshcloc(intersect)
         nodp2 = kmeshcloc(raise(intersect))
         nodp3 = kmeshcloc(raise(intersect+1))
         nodp4 = iworkedge(abs(nmeshdloc(intersect)))
         nodp5 = iworkedge(abs(nmeshdloc(isub_edges(2))))

         if ( isub_edges(2)-isub_edges(1)==1 ) then

!        --- edge 2 is alo intersected

            kmeshcnewloc(1) = nodp4
            kmeshcnewloc(2) = nodp2
            kmeshcnewloc(3) = nodp5

            t1(1) = coor(1,nodp3)-coor(1,nodp2)
            t1(2) = coor(2,nodp3)-coor(2,nodp2)
            t2(1) = coor(1,nodp1)-coor(1,nodp3)
            t2(2) = coor(2,nodp1)-coor(2,nodp3)
            len1 = t1(1)**2+t1(2)**2
            len2 = t2(1)**2+t2(2)**2
            phi1 = (t1(1)*t2(1)+t1(2)*t2(2))/sqrt(len1*len2)

            t1(1) = t2(1)
            t1(2) = t2(2)
            t2(1) = coor(1,nodp2)-coor(1,nodp1)
            t2(2) = coor(2,nodp2)-coor(2,nodp1)
            len1 = t1(1)**2+t1(2)**2
            len2 = t2(1)**2+t2(2)**2
            phi2 = (t1(1)*t2(1)+t1(2)*t2(2))/sqrt(len1*len2)

            if ( phi1>phi2 ) then

!           --- phi1 > phi2, subdivide angle 2-3-1

               kmeshcnewloc(4) = nodp1
               kmeshcnewloc(5) = nodp4
               kmeshcnewloc(6) = nodp3

               kmeshcnewloc(7) = nodp4
               kmeshcnewloc(8) = nodp5
               kmeshcnewloc(9) = nodp3

            else

!           --- phi2 > phi1, subdivide angle 3-1-2

               kmeshcnewloc(4) = nodp1
               kmeshcnewloc(5) = nodp4
               kmeshcnewloc(6) = nodp5

               kmeshcnewloc(7) = nodp5
               kmeshcnewloc(8) = nodp3
               kmeshcnewloc(9) = nodp1

            end if

         else

!        --- edge 3 is alo intersected

            kmeshcnewloc(1) = nodp1
            kmeshcnewloc(2) = nodp4
            kmeshcnewloc(3) = nodp5

            t1(1) = coor(1,nodp2)-coor(1,nodp1)
            t1(2) = coor(2,nodp2)-coor(2,nodp1)
            t2(1) = coor(1,nodp3)-coor(1,nodp2)
            t2(2) = coor(2,nodp3)-coor(2,nodp2)
            len1 = t1(1)**2+t1(2)**2
            len2 = t2(1)**2+t2(2)**2
            phi1 = (t1(1)*t2(1)+t1(2)*t2(2))/sqrt(len1*len2)

            t1(1) = t2(1)
            t1(2) = t2(2)
            t2(1) = coor(1,nodp1)-coor(1,nodp3)
            t2(2) = coor(2,nodp1)-coor(2,nodp3)
            len1 = t1(1)**2+t1(2)**2
            len2 = t2(1)**2+t2(2)**2
            phi2 = (t1(1)*t2(1)+t1(2)*t2(2))/sqrt(len1*len2)

            if ( phi1>phi2 ) then

!           --- phi1 > phi2, subdivide angle 1-2-3

               kmeshcnewloc(4) = nodp4
               kmeshcnewloc(5) = nodp2
               kmeshcnewloc(6) = nodp5

               kmeshcnewloc(7) = nodp2
               kmeshcnewloc(8) = nodp3
               kmeshcnewloc(9) = nodp5

            else

!           --- phi2 > phi1, subdivide angle 2-3-1

               kmeshcnewloc(4) = nodp4
               kmeshcnewloc(5) = nodp2
               kmeshcnewloc(6) = nodp3

               kmeshcnewloc(7) = nodp3
               kmeshcnewloc(8) = nodp5
               kmeshcnewloc(9) = nodp4

            end if

         end if

         nelemloc = 3

      else

!     --- nsub_edges=3
!         subdivide element into four elements

         nodp1 = kmeshcloc(1)
         nodp2 = kmeshcloc(2)
         nodp3 = kmeshcloc(3)
         nodp4 = iworkedge(abs(nmeshdloc(isub_edges(1))))
         nodp5 = iworkedge(abs(nmeshdloc(isub_edges(2))))
         nodp6 = iworkedge(abs(nmeshdloc(isub_edges(3))))

         kmeshcnewloc(1) = nodp1
         kmeshcnewloc(2) = nodp4
         kmeshcnewloc(3) = nodp6

         kmeshcnewloc(4) = nodp4
         kmeshcnewloc(5) = nodp2
         kmeshcnewloc(6) = nodp5

         kmeshcnewloc(7) = nodp5
         kmeshcnewloc(8) = nodp3
         kmeshcnewloc(9) = nodp6

         kmeshcnewloc(10) = nodp4
         kmeshcnewloc(11) = nodp5
         kmeshcnewloc(12) = nodp6

         nelemloc = 4

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstboun2d4'

      end if  ! ( debug )
      call erclos ( 'mshobstboun2d4' )

      end subroutine mshobstboun2d4
