      subroutine mshobstms2d05a ( trianglenode, trianglenodenew, &
                                  edgenode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 19-05-2005 Reset of edgenode
!        version  1.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  1.0  date 27-12-2001 Copied from mshobstms2d05
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into three triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(3), trianglenodenew(9), edgenode(3,2)

!     edgenode        i   Array with nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     trianglenode    i   Array of nodal point numbers of the corners of
!                         the original triangle
!     trianglenodenew o   Array of nodal point numbers of the corners of
!                         the new triangles.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer raise(4), i, node1, node2, node3, node4, node5, iseq

!     i              Counting variable
!     iseq           Sequence number
!     node1          Node of face
!     node2          Node of face
!     node3          Node of face
!     node4          Node of face
!     node5          Node of face
!     raise          Raise-operator
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     node1 := common node of edgenode(1,1), edgenode(2,1), edgenode(1,2)
!              and edgenode(2,2)
!     iseq := sequence number of node1 in trianglenode
!     node2 := next node in trianglenode
!     node3 := last node in trianglenode
!     if ( node2 in ( edgenode(1,1),edgenode(2,1) ) then
!        node4 := edge(3,1)
!        node5 := edge(3,2)
!     else ( node2 in ( edgenode(1,2),edgenode(2,2) )
!        node4 := edge(3,2)
!        node5 := edge(3,1)
!     triangle1 := (node1,node4,node5)
!
!      1                      1
!      |\                     |\
!      | \                    | \                 5                     5
!      |  \5                  |  \ 5            /\                    /\
!      |  /\                  |  /            /   \                 / / \
!      |/   \                 |/          4  |\    \            4  |     \
!   4  |     \             4  |              |   \  \              | /    \
!      |      \  -->                         |       \     or      |       \
!      |       \                             |      \ \            |/       \
!      |        \                            ---------- ---------- &
!      ---------- 2          3          2          3
!      2          3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise / 2, 3, 1, 2 /
! ======================================================================
!
!     --- node1 := common node of inode1, inode2, inode3, inode4

      if ( edgenode(1,1)==edgenode(1,2) .or. &
           edgenode(1,1)==edgenode(2,2) ) then
         node1 = edgenode(1,1)
      else
         node1 = edgenode(2,1)
      end if

!     --- iseq := sequence number of common node node1 in triangle node

      do i = 1, 3
         if ( trianglenode(i)==node1 ) then

!        --- Node found

            iseq = i
            go to 100
         end if
      end do
 100  continue

!     --- node2 := next node in trianglenode
!         node3 := last node in trianglenode

      node2 = trianglenode(raise(iseq))
      node3 = trianglenode(raise(iseq+1))

      if ( node2==edgenode(1,1) .or. node2==edgenode(2,1) ) then

!     --- node2  = edgenode(1,1) or edgenode(2,1))
!         node4 := edgenode(3,1)
!         node5 := edgenode(3,2)

         node4 = edgenode(3,1)
         node5 = edgenode(3,2)

      else

!     --- node2 in (edgenode(1,2),edgenode(2,2))
!         node4 := edgenode(3,2)
!         node5 := edgenode(3,1)

         node4 = edgenode(3,2)
         node5 = edgenode(3,1)

      end if

!     --- triangle1 := (node1,node4,node5)

      trianglenodenew(1) = node1
      trianglenodenew(2) = node4
      trianglenodenew(3) = node5

      if ( node4<node5 ) then

!     --- option 1
!         triangle2 := (node4,node2,node3)

         trianglenodenew(4) = node4
         trianglenodenew(5) = node2
         trianglenodenew(6) = node3

!        --- triangle3 := (node3,node5,node4)

         trianglenodenew(7) = node3
         trianglenodenew(8) = node5
         trianglenodenew(9) = node4

      else

!     --- option 2
!         triangle2 := (node5,node4,node2)

         trianglenodenew(4) = node5
         trianglenodenew(5) = node4
         trianglenodenew(6) = node2

!        --- triangle3 := (node2,node3,node5)

         trianglenodenew(7) = node2
         trianglenodenew(8) = node3
         trianglenodenew(9) = node5

      end if

      end
