      subroutine mshobstms3d06a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  2.1  date 19-05-2005 Reset of edgenode
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with three crossed edges which have one
!     single node in common into four tetrahedrons.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4),tetnodenew(16), edgenode(3,3), isort(4), &
              ledgenode(2,3), nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, &
              trans(4), transi(4),transtemp, transposj, transposmax, &
              tetnodet(4), edgenodet(6), &
              edgenodemin, edgenodemax, nodetwo
      integer pedgenodet(4,4)
      logical oddpairintchan

!     edgenodemax    edge node with lowest node number
!     edgenodemin    edge node with largest node number
!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     j              Counting variable
!     nodetwo        local tet number which should be transformed to tetnode 2
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transposmax    The transformed local edge node number of edgenode
!                    edgenodemax
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find intersection point with lowest and highest node number.
!
!     Define transformation(= rotation) to move node isort(4) to node 1 and
!     the intersection point with lowest node number to edge position 1.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.).
!
!     Split tetrahedron into four subtetrahedrons, taking into account the
!     position of the intersection point with the highest node number.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     pedgenodet(i,j) = the local edge number = local edgenode number
!                   for the edge [local tetnode i, local tetnode j]
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
      call eropen ( 'mshobstms3d06a' )

!     --- Find intersection point with lowest and highest node number

      if( edgenode(3,1)<edgenode(3,2) ) then
         edgenodemin = 1
         edgenodemax = 2
      else
         edgenodemin = 2
         edgenodemax = 1
      end if
      if( edgenode(3,3)>edgenode(3,edgenodemax) ) then
         edgenodemax = 3
      else if ( edgenode(3,3)<edgenode(3,edgenodemin) ) then
         edgenodemin = 3
      end if

!     --- Determine which of the edgenodemin segment end nodes is the shared
!         node isort(4), which will be transformed to node 1.
!         The other one will be transformed to node 2.

      if( ledgenode(1,edgenodemin)==isort(4) ) then
         nodetwo = ledgenode(2,edgenodemin)

      else
         nodetwo = ledgenode(1,edgenodemin)

      end if

!     --- Define transformation to move node isort(4) to node 1
!                              and node nodetwo to node 2
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(4) to node 1

      if( isort(4)/=trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(4)
         trans(isort(4)) = transtemp
         oddpairintchan = .not.oddpairintchan

       end if

!      --- move node nodetwo to node 2

      if (nodetwo/=trans(2) ) then
         transtemp = trans(2)
         trans(2) = nodetwo
         trans(trans(nodetwo)) = transtemp
         oddpairintchan = .not.oddpairintchan

      end if

!     --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(3)
         trans(3) = trans(4)
         trans(4) = transtemp
      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Transform the intersection points to their transformed positions

      do j = 1,3
         transposj = pedgenodet(transi(ledgenode(1,j)), &
                                transi(ledgenode(2,j)))
         edgenodet(transposj) = edgenode(3,j)
      end do
      transposmax = pedgenodet(transi(ledgenode(1,edgenodemax)), &
                               transi(ledgenode(2,edgenodemax)))

!     --- Split tetrahedron

      tetnodenew(1) = tetnodet(1)
      tetnodenew(2) = edgenodet(1)
      tetnodenew(3) = edgenodet(3)
      tetnodenew(4) = edgenodet(4)
      tetnodenew(5) = edgenodet(1)
      tetnodenew(6) = edgenodet(3)
      tetnodenew(7) = edgenodet(4)
      if( transposmax==4 ) then
         tetnodenew(8) = tetnodet(4)
         tetnodenew(9) = edgenodet(3)
      else if( transposmax==3) then
         tetnodenew(8) = tetnodet(3)
         tetnodenew(9) = edgenodet(4)
      else

!     --- transposmax has wrong value

         call errint ( transposmax, 1 )
         call errchr ( 'transposmax', 1 )
         call errsub ( 2270, 1, 0, 1 )
         go to 1000

      end if

      tetnodenew(10) = edgenodet(1)
      tetnodenew(11) = tetnodet(3)
      tetnodenew(12) = tetnodet(4)
      tetnodenew(13) = edgenodet(1)
      tetnodenew(14) = tetnodet(3)
      tetnodenew(15) = tetnodet(4)
      tetnodenew(16) = tetnodet(2)

      nelemnew = 4

1000  call erclos ( 'mshobstms3d06a' )

      end
