      subroutine mshobstms3d09 ( tetnode, tetnodenew, edgenode, &
                                 ledgenode, nelemnew, nedgenode )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  3.0  date 19-05-2005 Extra parameter nedegenode
!        version  2.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron with four crossed edges into eight tetrahedrons.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4),tetnodenew(32), nedgenode, &
              edgenode(nedgenode,6), ledgenode(2,6), nelemnew
      integer pedgenodet(4,4)

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nedgenode      i    Length of first index of edgenode
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, transposj, edgenodet(6)

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     j              Counting variable
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     transposj      The transformed local edge node number of edge node j
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the local positions of the edge nodes.
!
!     Split tetrahedron into eight subtetrahedrons.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
      call eropen ( 'mshobstms3d09' )

!     --- Determine for the edge nodes the local positions

      do j = 1,6
         transposj = pedgenodet(ledgenode(1,j),ledgenode(2,j))
         edgenodet(transposj) = edgenode(3,j)
      end do

!     --- Split tetrahedron

      tetnodenew(1) = tetnode(1)
      tetnodenew(2) = edgenodet(1)
      tetnodenew(3) = edgenodet(3)
      tetnodenew(4) = edgenodet(4)
      tetnodenew(6) = edgenodet(2)
      tetnodenew(7) = edgenodet(1)
      tetnodenew(8) = edgenodet(3)
      tetnodenew(5) = edgenodet(4)
      tetnodenew(9)  = tetnode(2)
      tetnodenew(10) = edgenodet(2)
      tetnodenew(11) = edgenodet(1)
      tetnodenew(12) = edgenodet(5)
      tetnodenew(14) = edgenodet(4)
      tetnodenew(15) = edgenodet(2)
      tetnodenew(16) = edgenodet(1)
      tetnodenew(13) = edgenodet(5)
      tetnodenew(17) = tetnode(3)
      tetnodenew(18) = edgenodet(3)
      tetnodenew(19) = edgenodet(2)
      tetnodenew(20) = edgenodet(6)
      tetnodenew(22) = edgenodet(4)
      tetnodenew(23) = edgenodet(3)
      tetnodenew(24) = edgenodet(2)
      tetnodenew(21) = edgenodet(6)
      tetnodenew(25) = tetnode(4)
      tetnodenew(26) = edgenodet(4)
      tetnodenew(27) = edgenodet(6)
      tetnodenew(28) = edgenodet(5)
      tetnodenew(30) = edgenodet(2)
      tetnodenew(31) = edgenodet(4)
      tetnodenew(32) = edgenodet(6)
      tetnodenew(29) = edgenodet(5)

      nelemnew = 8

      call erclos ( 'mshobstms3d09' )

      end
