      subroutine mshobstrenum ( jrenum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-05-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array jrenum, depending on iinput in case of obstacles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jrenum(10)

!     jrenum         o    Parameter jrenum for TOPOL
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, irenum, method

!     i              Counting variable
!     irenum         First position in iinput with respect to renumbering
!     method         Defines the type of renumbering to be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     fill default values in jrenum
!     if ( iinput has been filled ) then
!         adapt these values according to iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstrenum' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstrenum'

      end if

!     --- First fill default values in jrenum
!         The last position is filled in mshobstms

      jrenum(1) = -10  ! 10 entries are used
      jrenum(2) = 2    ! Sloan
      jrenum(3) = 0    ! smallest profile
      jrenum(4) = 0    ! Minimal level for printing
      jrenum(5) = 3    ! Priority for vertex nodes
      jrenum(6) = 3    ! Priority for mid point nodes
      jrenum(7) = 3    ! Priority for centroids
      jrenum(8) = 0    ! ilevel
      jrenum(9) = 4    ! Start point given

!     --- Next adapt these positions depending on iinput

      if ( kmiinput>0 ) then

!     --- kmiinput > 0, hence iinput has been filled

         irenum = iinputkm(10)
         if ( irenum>0 ) then

!        --- irenum > 0

            method = mod(irenum,100)
            if ( irenum>=100 ) then
               jrenum(5) = 1    ! Priority for vertex nodes
               jrenum(6)  = 1   ! Priority for mid point nodes
            end if
            if ( method<9 ) then
               jrenum(3) = (irenum-1)/3     ! Preference
               method = irenum-3*jrenum(3)
            end if
            jrenum(2) = method    ! Type of renumbering method

         else if ( irenum<0 ) then

!        --- irenum < 0, complete array given, only the first 8 positions are
!            copied

            do i = 2, 8
               jrenum(i) = iinputkm(i)
            end do

         end if

      end if

      if ( debug ) write(irefwr,*) 'End mshobstrenum'
      call erclos ( 'mshobstrenum' )

      end
