      subroutine mshobstsurf ( kmeshnnew, lenkmsnnew, iedges, ielmfaces, &
                               idimedges, nedgenode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 22-11-2011 Removal of iwork from list
!        version  5.1  date 15-09-2006 Debug information
!        version  5.0  date 19-05-2005 Extra parameter nedgenode
!
!   copyright (c) 2001-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new array kmeshn in the case that extra points at the surface
!     are created when a new mesh is created due to intersection with an
!     obstacle (3D only)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnnew(*), idimedges, nedgenode, iedges(idimedges,*), &
              lenkmsnnew, ielmfaces(4,*)

!     idimedges      i    Length of first dimension of array iedges
!                         (linear:1, quadratic: 3)
!     iedges         i    Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i,
!                         with sequence number iedges(i)
!     ielmfaces      i    Contains faces of elements
!     kmeshnnew      o    New array kmeshn
!     lenkmsnnew    i/o   Length of array kmshnnew
!                         Ar input: declared length
!                         Ar output: computed length
!     nedgenode      i    Length of first index of edgenode
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm

!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSTSURF3   Create new array kmeshn in the case that extra points at
!                    the surface  are created when a new mesh is created due to
!                    intersection with an obstacle (3D, linear triangles)
!     MSHOBSTSURF7   Create new array kmeshn in the case that extra points at
!                    the surface  are created when a new mesh is created due to
!                    intersection with an obstacle (3D, quadratic triangles)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2751   This number of nodes per element in surface is not programmed yet
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.
      call eropen ( 'mshobstsurf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstsurf'

      end if
      if ( ierror/=0 ) go to 1000

      inpelm = maxval(kmeshn(1:nsurfs))

      select case ( inpelm )
      case(3)

!     --- Linear triangle

         call mshobstsurf3 ( kmeshnnew, lenkmsnnew, &
                             iedges, idimedges, nedgenode )

      case(7)

!     --- Extended quadratic triangle

         call mshobstsurf7 ( kmeshnnew, lenkmsnnew, &
                             iedges, ielmfaces, idimedges, nedgenode )

      case default

         call errint ( inpelm, 1 )
         call errint ( 1, 2 )
         call errsub ( 2751, 2, 0, 0 )

      end select  ! case ( inpelm )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstsurf'

      end if
      call erclos ( 'mshobstsurf' )

      end
