      subroutine mshouterfind3d ( lenoutersurfs, lensurfcur, lencurvsur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 27-11-2011 Extra debug
!        version  1.4  date 09-01-2008 Debug information
!        version  1.3  date 22-09-2007 Fortran 90
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find the outer boundary of a three-dimensional region and store the
!    surface numbers in array outerboun
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenoutersurfs, lensurfcur, lencurvsur

!     lencurvsur     i    Declared length of array curve_surfs
!     lenoutersurfs i/o   Length of array  outerboun
!                         At input estimated length
!                         At output computed length
!     lensurfcur     i    Declared length of array surface_curvs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nsurfs), i, isurnr, inpelm, istart, j, &
              ipstart, numsrf, iworkvolms(nvolms), ivolumes(nvolms), &
              iwork1(nsurfs), ipointer(nvolms), k, &
              nleft, npart, nsubsf, ivolnr, istartpart, nside, &
              surface_curvs(lensurfcur), curve_surfs(lencurvsur), &
              istartsurf, iworksurf(nsurfs), jvolnr, numvolms
      logical filled, finished, found

!     curve_surfs    Contains all outer surfaces to the "single" curves
!                    provided the curve contains exactly 2 outer surfaces
!                    Pos. 1 to ncurvs: starting addresses in part 2
!                    Contents of each curve in part 2
!                    Pos. 1 number of outer surfaces coupled to curve
!                    Followed by surface numbers
!     filled         If true the surface is filled
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     found          If true already a possibility has been found
!     i              Counting variable
!     inpelm         number of points per element
!     ipointer       Array of length nvolms containing pointers corresponding
!                    to array ivolumes
!                    ipointer(ipart) contains the last position used by
!                    volumes in ivolumes in part ipart
!     ipstart        Starting address in array
!     istart         First position used at a part
!     istartpart     Starting address in array ivolumes
!     istartsurf     Starting position in array
!     isurnr         Surface sequence number
!     ivolnr         Volume sequence number
!     ivolumes       Array containing the volumes stored per part
!     iwork          integer work array of length nsurfs
!                    if ( iwork(i)=-1, surface i must be excluded
!                    if ( iwork(i)= 0, surface i is still free
!                    if ( iwork(i)= 1, surface i has already been treated
!     iwork1         integer work array of length nsurfs
!                    Is used to mark outer surfaces in volumes
!     iworksurf      integer work array of length nsurfs
!                    Is used to store number of volumes connected to surface
!     iworkvolms     integer work array of length nvolms
!                    Is used to indicate if a volume is used already
!                    -1: volume is not filled
!                     0: volume is not yet used
!                     1: volume has already been treated
!     j              Counting variable
!     jvolnr         Volume sequence number
!     k              Counting variable
!     nleft          Number of volumes or surfaces left
!     npart          Number of parts
!     nside          number of points along side of spectral element
!     nsubsf         Number of subsurfaces
!     numsrf         number of surfaces
!     numvolms       number of volumes
!     surface_curvs  Contains the number of "single" curves coupled
!                    to all outer surfaces in the following way
!                    Pos. 1 to nsurfs: starting addresses for second part
!                    For each surface in second part:
!                    Pos. 1: number of single curves
!                    Followed by curve numbers (all positive)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCURVSURFS   Store all outer surfaces coupled to a curve in array
!                    curv_surfaces.
!                    Only single curves in a surface are considered
!     MSHOUTERFIND01 Provide all surfaces in array outerboun with a sign in
!                    order to have all outer surfaces directed in the same way
!     MSHOUTERFIND02 Provide all surfaces in array outerboun with a sign in
!                    order to have all outer surfaces directed in the same way
!                    Special case: spectral elements
!     MSHSURFCURVS   Store all "single" curves corresponding to outer surfaces
!                    in array surface_curvs
!     PRININ         print 1d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     PRINOUTSRFS    Print the contents of array outerboun
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2413   Internal error: nleft<0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find volume element with lowest number
!     Clear arrays iwork, iwork1
!     Set iwork(i) equal to -1 for all composite surfaces
!     Set iwork(i) equal to -1 for all surfaces that have not been filled
!     Set iwork(i) equal to -1 for all inner surfaces
!     Set iwork1(i) = 0 for all surfaces
!     Set iworkvolms(i) = 0 if volume filled and -1 if not filled
!     Subdivide volumes in parts which are connected:
!        npart := 0; istartpart := 0
!        While ( there are still volumes left in iworkvolms ) do
!           ivolnr := first free volume in iworkvolms
!           npart := npart+1; istartpart := istartpart+1
!           ivolumes(istartpart) := ivolnr; iworkvolms(ivolnr) := 1
!           Set iwork1 equal to 1 for all surfaces in ivolnr
!           For all volumes not treated yet do
!              jvolnr := next volume
!              if ( iwork1(isurnr)>0 for at least one the surfs in jvolnr ) then
!                 iworkvolms(jvolnr) := npart; istartpart := istartpart+1
!                 ivolumes(istartpart) := jvolnr; iworkvolms(jvolnr) := 1
!                 Set iwork1 equal to 1 for all surfaces in jvolnr
!           ipointer(npart) := istartpart
!     Fill array outerboun from array istore
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterfind3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterfind3d'
         call prinin ( kmeshm, 5, 'kmeshm' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays iwork, iworkfaces and iwork1

      iwork = 0
      iworksurf = 0
      iwork1 = 0

!     --- Set iwork(i) equal to -1 for all composite surfaces

      ninner = 0
      if ( kelmr>0 ) then

!     --- kelmr>0, hence there are composite surfaces

         do isurnr = 1, nsurfs
            nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
            if ( nsubsf>0 ) then
               iwork(isurnr) = -1
               ninner = ninner+1
            end if

         end do

      end if

!     --- Set iwork(i) equal to -2 for all surfaces that have not been filled

      istart = 2*nsurfs
      do isurnr = 1, nsurfs

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)

         if ( iwork(isurnr)==0 ) then

!        --- Surface has not been marked yet

            filled = inelem>0
            do j = 1, inpelm
               if ( kmeshn(istart+j)<=0 ) filled = .false.
            end do
            if ( .not. filled ) then

!           --- surface has not been filled

               ninner = ninner+1
               iwork(isurnr) = -2

            end if

         end if
         istart = istart+inpelm*inelem

      end do

!     --- Set iwork(i) equal to -1 for all inner surfaces
!         First fill number of volumes connected to surface in iworksurf
!         Set iworkvolms(i) = 0 if volume filled and -1 if not filled

      nleft = nvolms
      do ivolnr = 1, nvolms
         istart = volume_surfs(ivolnr)
         numsrf = volume_surfs(istart)
         filled = .false.
         do i = 1, numsrf
            isurnr = volume_surfs(istart+i)
            iworksurf(isurnr) = iworksurf(isurnr)+1
            if ( iwork(isurnr)>-2 ) filled = .true.
         end do
         if ( filled ) then

!        --- Volume is filled

            iworkvolms(ivolnr) = 0

         else

!        --- Volume is not filled

            iworkvolms(ivolnr) = -1
            nleft = nleft-1

         end if

      end do

      do isurnr = 1, nsurfs
         if ( iwork(isurnr)==0 .and. iworksurf(isurnr)/=1 ) then

!        --- Surface does not correspond to one volume only

            ninner = ninner+1
            iwork(isurnr) = -1

         end if

      end do

      if ( debug ) then

!     --- debug true, debug statements

         write(irefwr,*) 'In mshouterfind3d: '
         write(irefwr,*) 'nsurfs, ninner ', nsurfs, ninner
         call prinin(iwork,nsurfs,'iwork')

      end if

!     --- Subdivide volumes in parts which are connected:
!         npart := 0; istartpart := 0

      npart = 0
      istartpart = 0

!     --- While ( there are still volumes left in iworkvolms ) do

      do while ( nleft>0 )

!     --- ivolnr := first free volume in iworkvolms

         do i = 1, nvolms
            if ( iworkvolms(i)==0 ) then

!           --- free outer surface found

               ivolnr = i
               go to 100

            end if
         end do

!        --- npart := npart+1; istartpart := istartpart+1
!            ivolumes(istartpart) := ivolnr; iworkvolms(ivolnr) := 1

100      npart = npart+1
         istartpart = istartpart+1
         ivolumes(istartpart) = ivolnr
         iworkvolms(ivolnr) = 1
         nleft = nleft-1

!        --- Set iwork1 equal to 1 for all surfaces in ivolnr

         ipstart = volume_surfs(ivolnr)
         numsrf = volume_surfs(ipstart)
         iwork1(volume_surfs(ipstart+1:ipstart+numsrf)) = 1

         if ( debug ) then

!        --- debugging information

            write(irefwr,*) 'istart,ivolnr,istartpart,npart ', &
                             istart,ivolnr,istartpart,npart

         end if

!        --- For all volumes not treated yet do
!              jvolnr := next volume
!              if ( iwork1(isurnr)>0 for at least one the surfs in jvolnr ) then
!                 iworkvolms(jvolnr) := npart; istartpart := istartpart+1
!                 ivolumes(istartpart) := jvolnr; iworkvolms(jvolnr) := 1
!                 Set iwork1 equal to 1 for all surfaces in jvolnr

         finished = .false.
         do while ( .not. finished )

!        --- Still volumes found

            found = .false.
            do i = ivolnr+1, nvolms

               if ( iworkvolms(i)==0 ) then

!              --- volume not treated yet

                  ipstart = volume_surfs(i)
                  numsrf = volume_surfs(ipstart)
                  do j = 1, numsrf
                     isurnr = volume_surfs(ipstart+j)

                     if ( iwork1(isurnr)>0 ) then

!                    --- Volume is connected to previous volumes

                        found = .true.
                        jvolnr = i
                        go to 200

                     end if
                  end do
               end if

            end do

!           --- end checking volume

200         if ( found ) then

!           --- Still volume found

               istartpart = istartpart+1
               ivolumes(istartpart) = jvolnr
               iworkvolms(jvolnr) = 1
               nleft = nleft-1

!              --- Set iwork1 equal to 1 for all surfaces in jvolnr

               ipstart = volume_surfs(jvolnr)
               numsrf = volume_surfs(ipstart)
               iwork1(volume_surfs(ipstart+1:ipstart+numsrf)) = 1

               if ( debug ) then

!              --- debugging information

                  write ( irefwr,* ) ' istartpart, ivolnr ', &
                                     istartpart, ivolnr

               end if
               if ( nleft<0 ) then

!              --- nleft<0, internal error

                  call errint ( nleft, 1 )
                  call errsub ( 2413, 1, 0, 0 )
                  go to 1000

               end if

            else

!           --- No surface available anymore

               finished = .true.

            end if

         end do

!        --- ipointer(npart) := istartpart

         ipointer(npart) = istartpart

      end do

      if ( debug ) then

!     --- debugging information

         write(irefwr,*) 'npart ', npart
         istartpart = 0
         do i = 1, npart
            numvolms = ipointer(i)-istartpart
            write(irefwr,*) 'part ', i
            call prinin ( ivolumes(istartpart+1), numvolms, 'volumes' )
            istartpart = istartpart+numvolms
         end do

      end if

!     --- Fill array outerboun from arrays ipointer and ivolumes

      outerboun(1) = npart
      istart = npart+2
      istartsurf = 1
      istartpart = 0
      do i = 1, npart

         outerboun(1+i) = istart
         numvolms = ipointer(i)-istartpart
         nleft = 0
         do j = 1, numvolms

            ivolnr = ivolumes(istartpart+j)
            ipstart = volume_surfs(ivolnr)
            numsrf = volume_surfs(ipstart)
            do k = 1, numsrf

               isurnr = volume_surfs(ipstart+k)
               if ( iwork(isurnr)==0 ) then

!              --- surface is outer surface, add to outerboun

                  nleft = nleft+1
                  outerboun(istart+nleft) = isurnr

               end if

            end do

         end do
         outerboun(istart) = nleft
         istartsurf = istartsurf+nleft+1
         istart = istart+nleft+1
         if ( istart>lenoutersurfs ) then

!        --- array outerboun too small

            call errint ( istart, 1 )
            call errint ( lenoutersurfs, 2 )
            call errchr ( 'outerboun', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         istartpart = istartpart+numvolms

      end do

      if ( debug ) then

!     --- debugging information

         call prinoutsrfs ( outerboun )

      end if
      lenoutersurfs = istart-1
      if ( kelmm>0 ) then

!     --- There are curves, hence the next part can be carried out
!         Store all "single" curves corresponding to a surface in surface_curvs

         call mshsurfcurvs ( surface_curvs, lensurfcur, nsurfs, &
                             ncurvs, kmeshs, iwork )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then

!        --- debug true, debug statements

            write(irefwr,*) 'Single curves corresponding to surfaces'
            call prinnmeshx ( nsurfs, surface_curvs )

         end if

!        --- Store all outer surfaces corresponding to a "single" curve in
!            curve_surfs

         call mshcurvsurfs ( curve_surfs, lencurvsur, &
                             surface_curvs, ncurvs, nsurfs )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then

!        --- debug true, debug statements

            write(irefwr,*) 'Outer surfaces corresponding to curves'
            call prinnmeshx ( ncurvs , curve_surfs )

         end if

!        --- Provide each surface in outerboun with a sign so that
!            all surfaces in a part are directed in the same way

         if ( kelmu==0 ) then

!        --- kelmu = 0, standard elements

            call mshouterfind01 ( nsurfs, outerboun, surface_curvs, &
                                  kmeshm, kmeshn, curve_surfs, &
                                  kmeshs(nsurfs+2) )

         else

!        --- kelmu>0, spectral elements

            nside = 2+kmeshu(2)
            call mshouterfind02 ( nsurfs, outerboun, surface_curvs, &
                                  kmeshm, kmeshn, curve_surfs, &
                                  kmeshs(nsurfs+2), nside )

         end if  ! ( kelmu==0 )
         if ( ierror/=0 ) go to 1000

      end if  ! ( kelmm>0 )

      if ( debug ) then

!     --- debugging information

         call prinoutsrfs ( outerboun )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshouterfind3d'

      end if  ! ( debug )
      call erclos ( 'mshouterfind3d' )

      end
