      subroutine mshparelems3 ( elements, iprocessor, numelms, points, &
                                numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-03-2015 Use inicopbuf
!        version  1.0  date 05-05-2004
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store arrays elements and points into ibuffr and store the corresponding
!     memory management sequence numbers into elempointers and nodepointers
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer elements(*), iprocessor,numelms, points(*), &
              numnodes

!     elements            Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     iprocessor     i    Processor sequence number
!     numelms        i    Number of elements
!     numnodes       i    Number of nodal points
!     points         i    Array of size npointloc containing the node
!                         numbers corresponding to the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2432   The number of elements in block of the parallel mesh is 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelems3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelems3'
         write(irefwr,1) 'iprocessor', iprocessor
         call prinin ( elements, numelms, 'elements' )
         call prinin ( points, numnodes, 'points' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( numelms==0 ) then

!     --- No elements in block

         call errint ( iprocessor, 1 )
         call errsub ( 2432, 1, 0, 0 )
         go to 1000

      end if

!     --- Create subarrays in ibuffr containing these elements
!         and nodes, store the memory management numbers in
!         elempointers and nodepointers

      allocate ( kmhpar(iprocessor)%elempointers(numelms), &
                 kmhpar(iprocessor)%nodepointers(numnodes) )
      kmhpar(iprocessor)%elempointers = elements(1:numelms)
      kmhpar(iprocessor)%nodepointers = points(1:numnodes)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparelems3'

      end if
      call erclos ( 'mshparelems3' )

      end
