      subroutine mshparelsurfs ( isurnr, elemacts, numelms )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-03-2015 New parameter list
!        version  1.1  date 05-04-2013 use getroutines
!        version  1.0  date 03-02-2003
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elements with the elements corresponding to one block
!     and also the corresponding array points with the point numbers
!     This subroutine is used if a block is identical to a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, elemacts(*), numelms

!     elemacts      i/o   Array of size nelem which indicates if an element
!                         has already been supplied to a block (1) or not (0)
!     isurnr         i    Surface sequence number
!     numelms        o    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inpelm, ielem1

!     ielem1         First element number in surface
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find starting address, number of elements and so on for surface isurnr
!     Fill array elements(1:numelms) with all elements in surface
!     Add all line elements in the surface to elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelsurfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelsurfs'
         write(irefwr,*) 'isurnr ', isurnr

      end if
      if ( ierror/=0 ) go to 1000

!     --- Find starting address, number of elements and so on for surface isurnr

      call sepextkmesht ( isurnr, isurnr, istart, inpelm )

!     --- Fill array elements(1:numelms) with all elements in surface

      ielem1 = kmesht(1,isurnr)
      elemacts(ielem1:ielem1+inelem-1) = isurnr
      numelms = inelem

1000  if ( debug ) write(irefwr,*) 'End mshparelsurfs'
      call erclos ( 'mshparelsurfs' )

      end
