      subroutine mshparelvolms ( ivolnr, elemacts, numelms )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-03-2015 New parameter list
!        version  1.0  date 28-01-2004
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elements with the elements corresponding to one block
!     and also the corresponding array points with the point numbers
!     This subroutine is used if a block is identical to a volume
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivolnr, elemacts(*), numelms

!     elemacts      i/o   Array of size nelem which indicates if an element
!                         has already been supplied to a block (1) or not (0)
!     ivolnr         i    volume sequence number
!     numelms        o    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inpelm, ielem1, ielgrp, ielem

!     ielem          Element number
!     ielem1         First element number in volume
!     ielgrp         element group number
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find starting address, number of elements and so on for volume ivolnr
!     Fill array elements(1:numelms) with all elements in volume
!     Add all surface elements in the volume to elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelvolms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelvolms'
         write(irefwr,*) 'ivolnr ', ivolnr

      end if

!     --- Find starting address, number of elements and so on for volume ivolnr

      call sepextkmesht ( ivolnr, ivolnr, istart, inpelm )

!     --- Fill array elements(1:numelms) with all elements in volume

      ielem1 = kmesht(1,ivolnr)
      elemacts(ielem1:ielem1+inelem-1) = ivolnr
      numelms= inelem

!     --- Add all surface elements in the volume to elements

      istart = 0
      ielem  = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         if ( ishape==0 .or. ishape>=11 ) then

!        --- element is volume element or empty, skip

            ielem = ielem+inelem
            istart = istart+inelem*inpelm

         else

!        --- element is surface element, check if it has to be added to this
!            block

            ielem = ielem+1
            if ( elemacts(ielem)==0 ) then

!           --- element has not been added to a block

               elemacts(ielem) = ivolnr
               numelms = numelms+1

            end if
            istart = istart+inpelm

         end if  ! ishape==0 .or. ishape>=3

      end do  ! ielgrp = 1, nelgrp

      if ( debug ) write(irefwr,*) 'End mshparelvolms'
      call erclos ( 'mshparelvolms' )

      end
