      subroutine mshpavbndchk1 ( ibndnr, maxbounds, nowbnd, nloop, &
                                 bndcoornew, bndcoorold, layer, &
                                 lnnowbnd, nodenrs, imark, crd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if lines from old boundary to new boundary intersects (or is close
!     to ) other boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     check
!     boundary
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibndnr, maxbounds, nowbnd(maxbounds,*), nloop, layer, &
              lnnowbnd, nodenrs(*), imark(*)
      double precision bndcoorold(2,*), bndcoornew(4,lnnowbnd,*), &
                       crd(4,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd
!     imark         i/o   Indicates if markers must be set in the plot (>0)
!                         or not (0)
!     layer          i    Present layer sequence number
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     maxbounds      i    Max number of loops in nowbnd
!     nloop          i    number of boundaries in nowbnd
!     nodenrs        i    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode1, inode2, jbndnr, numnodes, jedge, j, jedge1, &
              ntimes , itimes, jnd
      double precision x1(2), x2(2), x3(2), x4(2), xmin, xmax, ymin, &
                       ymax, eps, xminnew, xmaxnew, yminnew, ymaxnew, &
                       fact1, fact2, n1(2), h, len
      logical changed, outbox

!     changed        If true a boundary has been changed
!     eps            Local accuracy
!     fact1          multiplication factor
!     fact2          multiplication factor
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     inode1         First node in edge
!     inode2         Second node in edge
!     itimes         Loop counter
!     j              Counting variable
!     jbndnr         jth boundary in nowbnd
!     jedge          edge sequence number
!     jedge1         last node of edge jedge
!     jnd            jth node in jbndnr
!     len            length of edge
!     n1             Contains normal in a point
!     ntimes         number of times a loop must be carried out
!     numnodes       Number of nodal points in array nodes
!     outbox         if true the edge lies outside the rectangle
!     x1             coordinates of inode1
!     x2             coordinates of inode2
!     x3             coordinates of inode3
!     x4             coordinates of inode4
!     xmax           Maximum x-value
!     xmaxnew        Maximum x-value of edge
!     xmin           Minimum x-value
!     xminnew        Minimum x-value of edge
!     ymax           Maximum y-value
!     ymaxnew        Maximum y-value of edge
!     ymin           Minimum y-value
!     yminnew        Minimum y-value of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE   Computes the intersection of two lines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numnodes do
!        x1 := coordinates of node on old boundary
!        x2 := coordinates of node on new boundary
!        Extend the line x1-x2 by a factor 1/4
!        For all boundaries jbndnr except ibndnr do
!           For all edges iedge on jbndnr do
!              x3 := coordinates of first node on edge
!              x4 := coordinates of last node on edge
!              if ( line x1-x2 intersects line x3-x4 ) then
!                 mark i as old node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbndchk1' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      numnodes = nowbnd(ibndnr,1)
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbndchk1'
         write(irefwr,1) 'layer, ibndnr', layer, ibndnr
         call prinin ( imark, numnodes, 'imark' )
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinin ( nowbnd, ibndnr, 'nowbnd' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
      end if
      if ( ierror/=0 ) go to 1000
      changed = .false.

!     --- For all nodes on the boundary do

      do i = 1, numnodes

!     --- x1 := coordinates of node on old boundary
!         x2 := coordinates of node on new boundary

         inode1 = nowbnd(ibndnr,i+1)
         inode2 = nodenrs(i)
         x1(1) = bndcoorold(1,i)
         x1(2) = bndcoorold(2,i)
         if ( imark(i)<=1 ) then

!        --- Standard case

            x2(1) = bndcoornew(1,i,ibndnr)
            x2(2) = bndcoornew(2,i,ibndnr)
            ntimes = 1

         else

!        --- Special case of concave angle

            ntimes = 2

         end if

         if ( debug ) &
            write(irefwr,1) 'i, inode2, ntimes', i, inode2, ntimes
         do itimes = 1, ntimes

            if ( ntimes>1 ) then

!           --- Compute x2 depending on itimes

               if ( itimes==1 ) then
                  jnd = i-1
                  if ( jnd==0 ) jnd = numnodes
                  n1(1) = -(bndcoorold(2,i)-bndcoorold(2,jnd))   ! -t1(2)
                  n1(2) =  (bndcoorold(1,i)-bndcoorold(1,jnd))   !  t1(1)
               else
                  jnd = i+1
                  if ( jnd>numnodes ) jnd = 1
                  n1(1) = -(bndcoorold(2,jnd)-bndcoorold(2,i))  ! -t1(2)
                  n1(2) =  (bndcoorold(1,jnd)-bndcoorold(1,i))  !  t1(1)
               end if
               len = 1d0/sqrt(n1(1)**2+n1(2)**2)
               n1(1) = n1(1)*len
               n1(2) = n1(2)*len
               h = crd(3,inode1)*crd(4,inode1)  ! width corresponding to inode1
               x2(1) = x1(1)+h*n1(1)
               x2(2) = x1(2)+h*n1(2)

            end if

            if ( inode1/=inode2 ) then

!           --- Define box along nodes

               xmin = min(x1(1),x2(1))
               xmax = max(x1(1),x2(1))
               ymin = min(x1(2),x2(2))
               ymax = max(x1(2),x2(2))
               eps = max(xmax-xmin,ymax-ymin)*sqreps

!              --- We do not want a node ending close to another boundary
!                  Therefore the line is increased by a factor 2 (i.e. +h)

               x2(1) = x2(1)+2d0*(x2(1)-x1(1))
               x2(2) = x2(2)+2d0*(x2(2)-x1(2))

!              --- For all boundaries jbndnr do except ibndnr do

               do jbndnr = 1, nloop

                  if ( jbndnr==ibndnr ) go to 100

!                 --- For all edges on jbndnr do

                  do jedge = 1, nowbnd(jbndnr,1)

!                 --- x3 := coordinates of first node on edge
!                     x4 := coordinates of last node on edge

                     jedge1 = jedge+1
                     if ( jedge1>nowbnd(jbndnr,1) ) jedge1 = 1
                     x3(1) = bndcoornew(1,jedge,jbndnr)
                     x3(2) = bndcoornew(2,jedge,jbndnr)
                     x4(1) = bndcoornew(1,jedge1,jbndnr)
                     x4(2) = bndcoornew(2,jedge1,jbndnr)
                     xminnew = min(x3(1),x4(1))
                     xmaxnew = max(x3(1),x4(1))
                     yminnew = min(x3(2),x4(2))
                     ymaxnew = max(x3(2),x4(2))
                     outbox = xmaxnew.lt.xmin .or. xminnew>xmax .or. &
                              ymaxnew.lt.ymin .or. yminnew>ymax

                     if  (outbox ) outbox = &
                        max(xmaxnew-xminnew,ymaxnew-yminnew)>eps

                     if ( debug ) then
                        write(irefwr,1) 'jbndnr, jedge', jbndnr, jedge
                        write(irefwr,2) 'xminnew, xmaxnew, yminnew, ymaxnew',  &
                                         xminnew, xmaxnew, yminnew, ymaxnew
                        write(irefwr,*) 'outbox ', outbox
                     end if  ! ( debug )

                     if ( .not. outbox ) then

!                    --- edge in rectangle
!                        compute intersection of edges

                        call mshcrossline ( x1, x2, x3, x4, fact1, &
                                            fact2, sqreps )
                        if ( debug ) &
                           write(irefwr,2) 'fact1, fact2', fact1, fact2
                        if ( fact2.ge.-sqreps .and. &
                             fact2<=1d0+sqreps ) then

!                       --- line from old boundary to new boundary intersects
!                           other boundary
!                           Reset bndcoornew

                           bndcoornew(1,i,ibndnr) = bndcoorold(1,i)
                           bndcoornew(2,i,ibndnr) = bndcoorold(2,i)
                           bndcoornew(3,i,ibndnr) = 0d0
                           imark(i) = 0
                           changed = .true.
                           do j = 1, numnodes

                              if ( nodenrs(j)==inode2 ) then

!                             --- node2 found

                                 nodenrs(j) = nowbnd(ibndnr,j+1)
                                 bndcoornew(1,j,ibndnr) = &
                                    bndcoorold(1,j)
                                 bndcoornew(2,j,ibndnr) = &
                                    bndcoorold(2,j)
                                 imark(j) = 0

                              end if  ! ( nodenrs(j)==inode2 )

                           end do  ! j = 1, numnodes
                           go to 200  ! jump if node is excluded

                        end if  ! ( fact2.ge.-sqreps .and. fact2<=1d0+sqreps )

                     end if  ! ( .not. outbox )

                  end do  ! jedge = 1, numnodes

100            end do  ! jbndnr = 1, nloop

            end if  ! ( inode1/=inode2 )

         end do  ! itimes = 1, ntimes

200   end do  ! i = 1, numnodes

      if ( debug ) write(irefwr,*) 'changed ', changed
      if ( changed ) then

!     --- The boundary has been changed, remove all entries of imark > 1

         do i = 1, numnodes
            imark(i) = min(1,imark(i))
         end do

      end if

1000  call erclos ( 'mshpavbndchk1' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs' )
         write(irefwr,*) 'End mshpavbndchk1'

      end if

      end
