      subroutine mshpavmark ( numnodes, bndcoorold, bndcoornew, crd, &
                              bndangle, layer, nowbnd, maxbounds, &
                              ibndnr, imark, ncrd, nodenrs, &
                              nbndpt, nloop, kmeshc, nel, &
                              markboun, lnnowbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-07-2004 new call to plotmeshinter
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all nodes along the closed boundary part in the following way:
!      -1: new node is skipped  (sharp corner)
!       0: reuse old node
!       1: standard case: a new node is created
!     n>1: n new nodes are created (concave case)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, layer, maxbounds, nowbnd(maxbounds,*), ibndnr, &
              imark(*), ncrd, nodenrs(*), nbndpt, &
              nloop, kmeshc(3,*), nel, lnnowbnd, &
              markboun(lnnowbnd,nloop,*)
      double precision bndcoorold(2,*), bndcoornew(4,*), bndangle(*), &
                       crd(4,*)

!     bndangle       i    Array containing the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    real work array containing the
!                         boundary coordinates of the i'th boundary
!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd being looked at
!     imark          o    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     markboun       i    Three dimensional array of size lnnowbnd x nloop x 3
!                         with the following contents
!                         markboun(i,j,k) refers to relative node i along the
!                         j-th boundary in nowbnd
!                         k has the following meaning.
!                         if Pos. 1 = 0 then the node is free otherwise (1)
!                         it is clcoe to another node.
!                         In that case position 2 contains the boundary number
!                         of that other node and position 3 the relative
!                         position  of the node along that boundary
!     maxbounds      i    Max number of loops in nowbnd
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     ncrd          i/o   Last node number used
!     nel            i    number of elements in the topology
!     nloop          i    number of boundaries in nowbnd
!     nodenrs        o    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imarkinters(2,numnodes)
      logical changed, plotdebug

!     changed        If true double points have been created
!     imarkinters    array of size 2 x numnodes indicating if two
!                    lines from old boundary to new boundary intersect
!                    imarkinters(1,i) > 0 means that the lines i-1 and i
!                    intersect (0 not)
!                    imarkinters(2,i) > 0 means that the lines i and i+1
!                    intersect (0 not)
!     plotdebug      If true plots must be made for debugging
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVMNINTER0 Smooth boundary by removing edges that are essentially
!                    smaller than in the previous layer
!     MSHPAVMNINTER1 mark all lines from old to new point if they intersect with
!                    previous line and next line
!     MSHPAVMNINTER2 fill array imark for all nodes along the boundary
!     MSHPAVMNINTER3 Check if subsequent nodes on the new boundary have
!                    distance less than hiref/2
!                    If so take average value and make both identical
!     MSHPAVMNINTER4 Smooth final new boundary
!                    Check if no intersections in the boundary are present
!     MSHPAVMNINTER5 Check if 3 subsequent edges on the new boundary have
!                    distance less than 0.7 times the old distance
!                    If so reduce the middle edge to zero by taking mean values
!     MSHPAVMNINTER6 Check if 2 subsequent edges on the new boundary have common
!                    length that is smaller than the length of each of its
!                    direct neighbours
!                    If so make one edge of the two, except when the angle
!                    is more than pi
!     MSHPAVMNINTER7 Give all nodes with the same number the same coordinates
!     PLOTBOUNNODES  Plot boundary nodes
!     PLOTBOUNOLD    Plot old boundary nodes
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PLOTNOWBND     Plot the actual boundary during mesh generation which is
!                    defined in array nowbnd, used in mshpav
!     PRININ         print 1d integer array
!     PRINNOWBND     Print array nowbnd which is used in mshpav
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First mark all lines from old to new point if they intersect with previous
!     line and next line
!     Next fill array imark for all nodes along the boundary
!     Check if subsequent nodes have distance less than previous distance
!     If so take average value and make both identical
!     Check if three subsequent edges on the new boundary have length
!     that is less than 0.7 the length at the old boundary
!     If so remove the middle edge
!     if ( layer >= 2 ) then
!        Smooth the boundary in the following way:
!          if two adjacent edges have common length that is less than the
!          length of both neighbour edges these 2 edges are replaced by 1
!     if ( at least one of the coordinates has been changed ) then
!        Be sure that all new nodes with the same number have the same
!        coordinates
!     Check and smooth final boundary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmark' )
      debug = .false.
      plotdebug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( layer.ge.10000 ) plotdebug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmark'
         write(irefwr,1)'ibndnr ', ibndnr
         call prinrl1 ( bndcoornew, numnodes, 4, 'bncoornew_start' )
         call prinrl ( bndangle, numnodes, 'bndangle' )
         call prinnowbnd ( nowbnd, nloop, maxbounds, layer )
  1      format ( a, 1x, (10i6) )

      end if

      if ( plotdebug ) then

!     --- Debug information

         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpavinters/1' )
         jtimes = 3
         call plotmeshinter ( nel, kmeshc, crd, &
                              layer, 4, numnodes, 'mshpavinters/1' )
         jtimes = 0

      end if
      if ( ierror/=0 ) go to 1000

!     --- First mark all lines from old to new point if they intersect with
!         previous line and next line

      call mshpavmninter1 ( numnodes, bndcoorold, bndcoornew, crd, &
                            bndangle, nowbnd, maxbounds, &
                            ibndnr, imarkinters )
      if ( ierror/=0 ) go to 1000

!     --- Next fill array imark for all nodes along the boundary

      call mshpavmninter2 ( numnodes, bndcoorold, bndcoornew, crd, &
                            bndangle, nowbnd, maxbounds, &
                            ibndnr, imark, ncrd, nodenrs, &
                            imarkinters, layer, nloop, markboun, &
                            lnnowbnd )
      if ( debug ) call prinin ( nodenrs, numnodes, 'nodenrs/2' )
      if ( plotdebug ) then
         jtimes = 1
         call plotbounnodes ( numnodes, bndcoornew, 1 )
         jtimes = 3
         call plotbounold ( numnodes, bndcoorold, 10, 'nodenrs/2' )
         jtimes = 0
      end if
      if ( ierror/=0 ) go to 1000

!     --- Check if subsequent nodes have distance less than previous distance
!         If so take average value and make both identical

      changed = .true.
      do while ( changed )
         call mshpavmninter3 ( numnodes, bndcoornew, imark, nodenrs, &
                               changed, layer, bndcoorold, nbndpt )
      end do   ! while ( changed )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( nodenrs, numnodes, 'nodenrs/3' )
      if ( plotdebug ) then
         jtimes = 1
         call plotbounnodes ( numnodes, bndcoornew, 1 )
         jtimes = 3
         call plotbounold ( numnodes, bndcoorold, 10, 'nodenrs/9' )
         jtimes = 0
      end if

!     --- Check if three subsequent edges on the new boundary have length
!         that is less than 0.7 the length at the old boundary
!         If so remove the middle edge

      call mshpavmninter5 ( numnodes, bndcoornew, imark, nodenrs, &
                            changed, layer, bndcoorold, nbndpt )
      if ( ierror/=0 ) go to 1000

      if ( plotdebug ) then
         jtimes = 1
         call plotbounnodes ( numnodes, bndcoornew, 1 )
         jtimes = 3
         call plotbounold ( numnodes, bndcoorold, 10, 'nodenrs/5' )
         jtimes = 0
      end if
      if ( debug ) call prinin ( nodenrs, numnodes, 'nodenrs/5' )

      if ( ierror/=0 ) go to 1000

      if ( layer.ge.2 ) then

!     --- Layer > 1, the boundary may be smoothed
!         At this moment smoothing is very simple:
!         if two adjacent edges have common length that is less than the
!         length of both neighbour edges these 2 edges are replaced by 1

         call mshpavmninter6 ( numnodes, bndcoornew, imark, nodenrs, &
                               changed, layer, bndangle )
         if ( debug ) call prinin ( nodenrs, numnodes, 'nodenrs/6' )

         call mshpavmninter0 ( numnodes, bndcoornew, imark, nodenrs, &
                               layer, bndcoorold )

      end if
      if ( ierror/=0 ) go to 1000

      if ( changed ) then

!     --- At least one of the coordinates has been changed
!         Be sure that all new nodes with the same number have the same
!         coordinates

         call mshpavmninter7 ( numnodes, bndcoornew, nodenrs, &
                               layer, nbndpt, nowbnd, maxbounds, &
                               ibndnr )
         if ( ierror/=0 ) go to 1000

      end if  ! ( changed )
      if ( plotdebug ) then
         jtimes = 1
         call plotbounnodes ( numnodes, bndcoornew, 1 )
         jtimes = 3
         call plotbounold ( numnodes, bndcoorold, 10, 'nodenrs/7' )
         jtimes = 0
      end if

!     --- Check and smooth final boundary

      call mshpavmninter4 ( numnodes, bndcoornew, nodenrs, &
                            bndcoorold, layer, ibndnr, nowbnd, &
                            maxbounds, imark )
      if ( ierror/=0 ) go to 1000
      if ( plotdebug ) then
         jtimes = 1
         call plotbounnodes ( numnodes, bndcoornew, 1 )
         jtimes = 3
         call plotbounold ( numnodes, bndcoorold, 10, 'nodenrs/4' )
         jtimes = 0
      end if

1000  call erclos ( 'mshpavmark' )

      if ( debug ) then

!     --- Debug information

         call prinin ( imark, numnodes, 'imark' )
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinrl1 ( bndcoornew, numnodes, 4, 'bncoornew_end' )
         write(irefwr,*) 'End mshpavmark'

      end if

      end
