      subroutine mshpavmaxloop ( nlayers, nloop, maxloop, maxbounds, &
                                 nowbnd, crd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-04-2004 Change default from 5 to 3
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Detect the maximum number of layers that may be constructed before
!     triangle is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlayers, nloop, maxloop, maxbounds, nowbnd(maxbounds,*)
      double precision crd(4,*)

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     maxbounds      i    Max number of loops in nowbnd
!     maxloop        o    maximum number of layers that may be constructed
!                         before triangle is applied
!     nlayers        i    Maximum number of layers for application of paver
!                         algorithm
!                         The rest of the region is filled by triangle
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode1, inode2

!     inode1         First node on first boundary
!     inode2         First node on second boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( nlayers > 0 ) then
!        maxloop := nlayers
!     else if ( nloop = 1 ) then
!        maxloop := infinity
!     else if ( nloop = 2 and all boundaries contain layers ) then
!        maxloop := infinity
!     else
!        maxloop := 3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmaxloop' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmaxloop'

      end if
      if ( ierror.ne.0 ) go to 1000
      if ( nlayers.gt.0 ) then

!     --- nlayers given, we perform nlayers steps
!         with paver and the rest is filled with triangle

         maxloop = nlayers

      else if ( nloop.eq.1 ) then

!     --- nloop = 1, closed boundary
!         maxloop := infinity

         maxloop = 100000

      else if ( nloop.eq.2 ) then

!     --- nloop = 2
!         Check thickness along the boundaries

         inode1 = nowbnd(1,2)  ! first node on first boundary
         inode2 = nowbnd(2,2)  ! first node on second boundary
         if ( crd(3,inode1).le.0d0 .or. crd(3,inode2).le.0d0 ) then

!        --- One of the two boundaries has thickness 0, so we are at a
!            outer boundary

            maxloop = 3

         else  ! ( crd(3,inode1).gt.0d0 .and. crd(3,inode2).gt.0d0 )

!        --- Both boundaries have given thickness, we are at an inner region

            maxloop = 100000

         end if  ! ( crd(3,inode1).le.0d0 .or. crd(3,inode2).le.0d0 )

      else  !  (nloop > 2 )

!     --- nloop > 2, outer region
!         maxloop := 3

         maxloop = 3

      end if  ! ( nlayers.gt.0 )

1000  call erclos ( 'mshpavmaxloop' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmaxloop'

      end if

      end
