      subroutine mshpavmndist3 ( idx, idy, ix, iy, nx, ny, grid, &
                                 gridinfo, finished, maxbounds, &
                                 nowbnd, inode, layer, crd, nwcr, nxcr, &
                                 h, len, ind, ibndnr, jbndnr, &
                                 jind, n, maxlen )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find edge in boundary that is close to given node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     boundary
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idx, idy, ix, iy, nx, ny, grid(0:nx,0:ny), gridinfo(*), &
              maxbounds, nowbnd(maxbounds,*), inode, layer, ind, &
              ibndnr, jind, jbndnr
      double precision crd(4,*), nwcr(2), nxcr(2), h, len, &
                       n(2), maxlen
      logical finished

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     finished       o    If true an edge close to the node is found
!     grid           i    Work array of size at most 101 x 101 to store
!                         information about the boundary in relation with
!                         arrays gridinfo and rgridinfo
!                         The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                         to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                         ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                         if grid(ix,iy) = 0, no boundary node is stored in cell
!                         (ix,iy), if > 0 information about the boundary cells
!                         is stored in array grdinfo from position
!                         grid(ix,iy)
!     gridinfo       i    In this array the boundary points corresponding to a
!                         specific grid cell in grid are stored
!                         istart = grid(ix,iy)
!                         In gridinfo(istart) the number of boundary points of
!                         cell (ix,iy) are stored (if > 0). This is followed
!                         by 2 positions for each boundary point
!                         The pair consists of the loop sequence number in the
!                         boundary where loop refers to a closed loop and
!                         a sequence number in the loop
!     h              i    Width of layer in node
!     ibndnr         i    i'th boundary in nowbnd
!     idx            i    max search length in x-dir
!     idy            i    max search length in y-dir
!     ind            i    Sequence number of node in ibndnr
!     inode          i    Node sequence number
!     ix             i    x index of cell containing inode
!     iy             i    y index of cell containing inode
!     jbndnr         o    jth boundary in nowbnd
!     jind           o    Sequence number of node in jbndnr
!     layer          i    Present layer sequence number
!     len            i    maximum search length
!     maxbounds      i    maximum loops in nowbnd in paver
!     maxlen         i    Maximum length of edges in the surface
!     n              i    normal in node
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     nwcr           i    coordinates of indnr
!     nx             i    number of cells in x direction
!     nxcr           i    coordinates of i+1th node in jbndnr
!     ny             i    number of cells in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idist, indx, indy, istart, nnodes, k, jloop, jnode2, jnd1, &
              jnode1, jnd, jnbndnod
      double precision factor, fact2, factmin, ds, dsmin, t(2), &
                       thresh
      logical check, found, intersect, correctdir

!     check          If true the edge to be investigated is in the right range
!     correctdir     If true normals have oppsoite signs hence the nodes
!                    may be considered as close if they are at short distance
!     ds             distance between two points
!     dsmin          Minimum value of ds
!     fact2          multiplication factor in intersection
!                    indicates if two lines intersect if 0 <= fact2 <= 1
!     factmin        Parameter to find the node with minimal distance to inode
!     factor         factor in intersection
!                    indicates if two lines intersect if 0 <= factor <= 1
!     found          If true a node close to inode has been found
!     idist          Loop parameter
!     indx           Loop parameter
!     indy           Loop parameter
!     intersect      if true two lines intersect
!     istart         First position used at a part
!     jloop          Boundary number
!     jnbndnod       number of coordinates of nodes in bndcrj
!     jnd            jth node in jbndnr
!     jnd1           jnd+1-th node in jbndnr
!     jnode1         first node in an edge
!     jnode2         second node in an edge
!     k              Counting variable
!     nnodes         Number of nodes
!     t              tangential vector
!     thresh         threshold value, i.e. maximum length for connecting nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVMNDIST1  Compute intersection of edge jnode1-jnode2 with line
!                    nwcr, nxcr
!     MSHPAVMNDIST2  check if the edge we are considering is allowed
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmndist3' )
      debug = .false.
      if ( layer>=10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmndist3'
         write(irefwr,1) 'inode', inode
         write(irefwr,2) 'h', h
         call prinrl ( crd(1,inode), 4, 'crd' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror.ne.0 ) go to 1000
      factmin = rinfin
      dsmin = rinfin
      factor = 0d0
      if ( layer>2 ) then
         thresh = min(16d0*h**2, maxlen**2) !maximum length for connecting nodes
      else
         thresh = 0d0
      end if
      if ( debug ) write(irefwr,2) 'thresh', thresh

!     --- For all cells around node within range do

      do idist = 0, max(idx,idy)

         do indx = max(0,ix-idist), min(ix+idist,nx)

            do indy = max(0,iy-idist), min(iy+idist,ny)

               check = indx>=ix-idx .and. indx<=ix+idx
               if ( check ) &
                  check = indy>=iy-idy .and. indy<=iy+idy
               if ( check ) &
                  check = abs(ix-indx).eq.idist .or. &
                          abs(iy-indy).eq.idist
               if ( check ) then

!              --- Check = true, we are in the right range
!                  Find loop jloop and index jind of nodes in cell

                  istart = grid(indx,indy)

                  if ( istart>0 ) then

!                 --- There is at least one node in the cell

                     nnodes = gridinfo(istart)
                     if ( debug ) then
                        write(irefwr,1) 'idist, indx, indy', &
                                         idist, indx, indy
                        write(irefwr,2) 'factmin', factmin
                     end if
                     do k = 1, nnodes

                        jloop = gridinfo(istart+2*(k-1)+1)  ! boundary part
                        jnd = gridinfo(istart+2*(k-1)+2)    ! relative position

!                       --- First look at edge before node "jnd"

                        found = .false.
                        jnode2 = abs(nowbnd(jloop,jnd+1))  ! second node on edge
                        jnbndnod = nowbnd(jloop,1)         ! numnodes on jloop
                        jnd1 = jnd-1
                        if ( jnd1.eq.0 ) jnd1 = jnbndnod
                        jnode1 = abs(nowbnd(jloop,jnd1+1))  ! first node
                        if ( jnode1.ne.inode .and. jnode2.ne.inode ) &
                           then

!                       --- Candidate for intersection found
!                           Compute intersection of corresponding edge with
!                           line nwcr, nxcr

                           call mshpavmndist1 ( crd, jnode1, jnode2, &
                                                nwcr, nxcr, h, len, &
                                                found, factor, fact2, &
                                                layer, intersect )
                           if ( debug .and. found ) then
                              write(irefwr,1) 'ind, ibndnr, inode', &
                                ind, ibndnr, inode
                              write(irefwr,*) 'factor, fact2', &
                                               factor, fact2
                              write(irefwr,1) &
                                 'k, jloop, jnd, jnode1, jnode2 ', &
                                  k, jloop, jnd, jnode1, jnode2
                              write(irefwr,*) 'intersect ', intersect
                           end if

                           if ( .not. found ) then

!                          --- No intersection of normal with edge found, &
!                              but it is still possible that the distance
!                              between node and nodes on edge is small enough
!                              First check if normal points in the right
!                              direction, i.e the normal in inode must point to
!                              the edge and the normal from the edge must point
!                              in the direction of the node
!                              The normal in the node is defined by n
!                              The normal on the edge by (-t(2), t(1))

                              t(1) = crd(1,jnode2)-crd(1,jnode1)
                              t(2) = crd(2,jnode2)-crd(2,jnode1)
                              ds = 1d0/sqrt(t(1)**2+t(2)**2)
                              t(1) = t(1)*ds
                              t(2) = t(2)*ds
                              correctdir = n(2)*t(1)-n(1)*t(2)<-0.7d0
                              if ( correctdir ) then

!                             --- The normals have opposite signs, &
!                                 now check if the line from inode to edge
!                                 has same direction as n

                                 t(1) = crd(1,jnode1)-crd(1,inode)
                                 t(2) = crd(2,jnode1)-crd(2,inode)
                                 ds = 1d0/sqrt(t(1)**2+t(2)**2)
                                 t(1) = t(1)*ds
                                 t(2) = t(2)*ds
                                 correctdir = &
                                    n(1)*t(1)+n(2)*t(2)>0.7d0
                              end if
                              if ( correctdir ) then

!                             --- Normals have different direction so we
!                                 are at opposite sides

                                 ds = (crd(1,inode)-crd(1,jnode1))**2+ &
                                      (crd(2,inode)-crd(2,jnode1))**2
                                 if ( ds<dsmin .and. ds<=thresh ) &
                                    then
                                    jbndnr = jloop
                                    dsmin = ds
                                    jind = jnd1
                                    found = .true.
                                    if ( debug ) write(irefwr,1) &
                                       '1: inode, jnode1, jloop', &
                                           inode, jnode1, jloop
                                 end if

                                 ds = (crd(1,inode)-crd(1,jnode2))**2+ &
                                      (crd(2,inode)-crd(2,jnode2))**2
                                 if ( ds<dsmin .and. ds<=thresh ) &
                                    then
                                    jbndnr = jloop
                                    dsmin = ds
                                    jind = jnd
                                    found = .true.
                                    if ( debug ) write(irefwr,1) &
                                       '2: inode, jnode2, jloop', &
                                           inode, jnode2, jloop
                                 end if

                              end if  ! ( -n(1)*t(2)+n(2)*t(1)<0d0 )

                           end if  ! ( found )

                           if ( found ) then

!                          --- An intersection has been found, finished

                              if ( factor<factmin-sqreps ) then

!                             --- Check if this is the closest intersection

                                 factmin = factor

                                 if ( found .and. jloop.eq.ibndnr ) then

!                                --- An intersection has been found, check if
!                                    the edge we are considering is allowed

                                    call mshpavmndist2 ( maxbounds, &
                                        nowbnd, ind, nwcr, ibndnr, crd, &
                                        found, fact2, jnode1, jnode2, &
                                        layer, intersect )

                                 end if  ! ( found )
                                 if ( found ) then

!                                --- edge is allowed

                                    finished = .true.
                                    jbndnr = jloop
                                    if ( jloop.ne.ibndnr .and. &
                                         fact2>=0.5d0 ) then
                                       jind = jnd
                                    else
                                       jind = jnd1
                                    end if

                                 end if
                                 if ( debug .and. found ) then
                                    write(irefwr,2) &
                                       'before: factor, fact2', &
                                        factor, fact2
                                    write(irefwr,1) 'jind', jind
                                 end if

                              end if  ! ( factor<factmin )

                           end if  ! ( found )

                        end if  ! ( jnode1.ne.inode .and. jnode2.ne.inode )

                        if ( .not. found ) then

!                       --- Next loop at edge after node "jnd"

                           found = .false.
                           jnode1 = abs(nowbnd(jloop,jnd+1))! first node on edge
                           jnd1 = jnd+1
                           if ( jnd1>jnbndnod ) jnd1 = 1
                           jnode2 = abs(nowbnd(jloop,jnd1+1))      ! second node
                           if ( jnode1.ne.inode .and. jnode2.ne.inode ) &
                              then

!                          --- Candidate for intersection found
!                              Compute intersection of corresponding edge with
!                              line nwcr, nxcr

                              call mshpavmndist1 ( crd, jnode1, jnode2, &
                                                   nwcr, nxcr, h, len, &
                                                   found, factor, fact2, &
                                                   layer, intersect )

                              if ( debug .and. found ) then

                                 write(irefwr,1) &
                                    'after: k, jloop, jnd ', &
                                            k, jloop, jnd
                                 write(irefwr,1) 'ind, ibndnr, inode ', &
                                                  ind, ibndnr, inode
                                 write(irefwr,1) &
                                    'jnode1, jnode2',jnode1, jnode2
                                 write(irefwr,2) 'fact2, factor', &
                                                  fact2, factor
                                 write(irefwr,*) 'intersect ', intersect

                              end if

                              if ( .not. found ) then

!                             --- No intersection of normal with edge found, &
!                                 but it is still possible that the distance
!                                 between node and nodes on edge is small enough

                                 t(1) = crd(1,jnode2)-crd(1,jnode1)
                                 t(2) = crd(2,jnode2)-crd(2,jnode1)
                                 ds = 1d0/sqrt((t(1)**2+t(2)**2))
                                 t(1) = t(1)*ds
                                 t(2) = t(2)*ds
                                 correctdir = &
                                    n(2)*t(1)-n(1)*t(2)<-0.7d0
                                 if ( correctdir ) then

!                                --- The normals have opposite signs, &
!                                    now check if the line from inode to edge
!                                    has same direction as n

                                    t(1) = crd(1,jnode1)-crd(1,inode)
                                    t(2) = crd(2,jnode1)-crd(2,inode)
                                    ds = 1d0/sqrt(t(1)**2+t(2)**2)
                                    t(1) = t(1)*ds
                                    t(2) = t(2)*ds
                                    correctdir = &
                                       n(1)*t(1)+n(2)*t(2)>0.7d0
                                 end if
                                 if ( correctdir ) then

!                                --- Normals have different direction so we
!                                    are at opposite sides

                                    ds = (crd(1,inode)-crd(1,jnode1))**2 &
                                        +(crd(2,inode)-crd(2,jnode1))**2
                                    if ( ds<dsmin .and. &
                                         ds<=thresh ) then
                                       jbndnr = jloop
                                       dsmin = ds
                                       jind = jnd
                                       found = .true.
                                       if ( debug ) write(irefwr,1) &
                                          '3: inode, jnode1, jloop', &
                                              inode, jnode1, jloop
                                    end if

                                    ds = (crd(1,inode)-crd(1,jnode2))**2 &
                                        +(crd(2,inode)-crd(2,jnode2))**2
                                    if ( ds<dsmin .and. &
                                         ds<=thresh )  then
                                       jbndnr = jloop
                                       dsmin = ds
                                       jind = jnd1
                                       found = .true.
                                       if ( debug ) write(irefwr,1) &
                                          '4: inode, jnode2, jloop', &
                                              inode, jnode2, jloop
                                    end if

                                 end if  ! ( -n(1)*t(2)+n(2)*t(1)<0d0 )

                              end if  ! ( found )

                              if ( found ) then

!                             --- An intersection has been found, finished

                                 if ( factor<factmin-sqreps ) then

!                                --- Check if this is the closest intersection

                                    if ( found .and. jloop.eq.ibndnr ) &
                                       then

!                                   --- An intersection has been found, check if
!                                       the edge we are considering is allowed

                                       call mshpavmndist2 ( maxbounds, &
                                           nowbnd, ind, nwcr, ibndnr, &
                                           crd, found, fact2, jnode1, &
                                           jnode2, layer, intersect )

                                    end if  ! ( found )
                                    if ( found ) then

!                                   --- edge is allowed

                                       finished = .true.
                                       jbndnr = jloop
                                       if ( jloop.eq.ibndnr .or. &
                                            (jloop.ne.ibndnr .and. &
                                             fact2<=0.5d0) ) then
                                          jind = jnd
                                       else
                                          jind = jnd1
                                       end if
                                       factmin = factor

                                    end if

                                    if ( debug .and. found ) then
                                       write(irefwr,2) &
                                          'after: factor, fact2', &
                                           factor, fact2
                                       write(irefwr,1) 'jind', jind
                                    end if

                                 end if

                              end if  ! ( found )

                           end if  ! ( jnode1.ne.inode .and. jnode2.ne.inode )

                        end if

                        if ( finished ) go to 1000

                     end do  ! k = 1, nnodes

                  end if  ! ( istart>0 )

               end if  ! ( check )

            end do  ! indy = max(0,iy-idist), min(iy+idist,ny)

         end do  ! indx = max(0,ix-idist), min(ix+idist,nx)

      end do  ! idist = 0, max(idx,idy)

1000  call erclos ( 'mshpavmndist3' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmndist3'

      end if

      end
