      subroutine mshpavmninter3 ( numnodes, bndcoornew, imark, nodenrs, &
                                  changed, layer, bndcoorold, nbndpt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if subsequent nodes on the new boundary have distance less than
!     1/2 of the old distance
!     If so take average value and make both identical
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, imark(*), nodenrs(*), layer, nbndpt
      double precision bndcoornew(4,*), bndcoorold(2,*)
      logical changed

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     changed        o    If true double points have been created
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     layer          i    Present layer sequence number
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind, iprev, i, inode, inode1, inode2
      double precision x1(2), x2(2), ds, dsold, x1old(2), x2old(2)

!     ds             distance between two points on new boundary
!     dsold          distance between two points on old boundary
!     i              Counting variable
!     ind            Sequence number along boundary loop
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     iprev          Previous node number
!     x1             coordinates of first point of edge on new boundary
!     x1old          coordinates of first point of edge on old boundary
!     x2             coordinates of second point of edge on new boundary
!     x2old          coordinates of second point of edge on old boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter3' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter3'
         call prinrl1 ( bndcoorold, numnodes, 2, 'bndcoorold' )
         call prinrl1 ( bndcoornew, numnodes, 4, 'bndcoornew' )
         call prinin ( imark, numnodes, 'imark' )
         call prinin ( nodenrs, numnodes, 'nodenrs' )

      end if
      changed = .false.
      if ( imark(numnodes).gt.0 ) then

!     --- Last node is new

         iprev = numnodes
         x1(1) = bndcoornew(1,numnodes)
         x1(2) = bndcoornew(2,numnodes)

      else

!     --- Last node is old

         iprev = 0

      end if

      x1old(1) = bndcoorold(1,numnodes)
      x1old(2) = bndcoorold(2,numnodes)
      do ind = 1, numnodes

         x2old(1) = bndcoorold(1,ind)
         x2old(2) = bndcoorold(2,ind)
         if ( imark(ind).gt.0 ) then

!        --- New node, check distance

            if ( iprev.gt.0 ) then

!           --- Previous node on the new boundary

               if ( nodenrs(iprev).ne.nodenrs(ind) .and. &
                    nodenrs(ind).gt.nbndpt .and. &
                    nodenrs(iprev).gt.nbndpt ) then

!              --- New node is different from previous one

                  x2(1) = bndcoornew(1,ind)
                  x2(2) = bndcoornew(2,ind)
                  x2old(1) = bndcoorold(1,ind)
                  x2old(2) = bndcoorold(2,ind)
                  ds = (x2(1)-x1(1))**2+(x2(2)-x1(2))**2
                  dsold = (x2old(1)-x1old(1))**2+(x2old(2)-x1old(2))**2

                  if ( ds.le.0.25d0*dsold ) then

!                 --- Distance smaller than h/2

                     bndcoornew(1,ind) = 0.5d0*(x1(1)+x2(1))
                     bndcoornew(2,ind) = 0.5d0*(x1(2)+x2(2))
                     inode1 = nodenrs(ind)
                     inode2 = nodenrs(iprev)
                     inode = min(inode1,inode2)

!                    --- All nodes with node numbers inode1 or inode2 get number
!                        inode

                     do i = 1,  numnodes
                        if ( nodenrs(i).eq.inode1 .or. &
                             nodenrs(i).eq.inode2 ) then
                           nodenrs(i) = inode
                           bndcoornew(1,i) = bndcoornew(1,ind)
                           bndcoornew(2,i) = bndcoornew(2,ind)
                        end if
                     end do
                     changed = .true.
                     if ( debug ) then
                        write(irefwr,*) 'ind, iprev, ds, dsold ', &
                                         ind, iprev, ds, dsold

!                       --- write(irefwr,*) 'x1, x2 ', x1, x2

                     end if

                  end if  ! ( ds.le.0.25d0*dsold )
                  x1(1) = x2(1)
                  x1(2) = x2(2)
                  iprev = ind

               end if  ! ( nodenrs(iprev).ne.nodenrs(ind) )

            else

!           --- iprev = 0

               x1(1) = bndcoornew(1,ind)
               x1(2) = bndcoornew(2,ind)
               iprev = ind

            end if  ! ( iprev.gt.0 )

         else if ( imark(ind).eq.0 ) then

!        --- Previous node is old

            iprev = 0

         end if
         x1old(1) = x2old(1)
         x1old(2) = x2old(2)

      end do  ! ind = 1, numnodes

      call erclos ( 'mshpavmninter3' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinrl1 ( bndcoornew, numnodes, 4, 'bndcoornew' )
         write(irefwr,*) 'End mshpavmninter3'

      end if

      end
