      subroutine mshpavmninter6 ( numnodes, bndcoornew, imark, nodenrs, &
                                  changed, layer, bndangle )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if 2 subsequent edges on the new boundary have common length
!     that is smaller than the length of each of its direct neighbours
!     If so make one edge of the two, except when the angle is more than pi
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, imark(*), nodenrs(*), layer
      double precision bndcoornew(4,*), bndangle(*)
      logical changed

!     bndangle       i    Array containing the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     changed        o    If true double points have been created
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     layer          i    Present layer sequence number
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ind, iprev
      double precision xold, yold, xnew, ynew, ds, h, &
                       edgelength(-1:numnodes+2)

!     ds             distance between two points along a curve
!     edgelength     Contains the length of the edges along the present
!                    boundary (squared)
!     h              width of leayer in node
!     i              Counting variable
!     ind            Sequence number along boundary loop
!     iprev          Previous node number
!     xnew           New x-coordinate
!     xold           Previous x-coordinate
!     ynew           New y-coordinate
!     yold           Previous y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTREAL    Check control integer around a real work array
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set control integers
!     Store length of subsequent edges in array edgelength
!     Check if the sum of length of two adjacent edges is less than the
!     length of both neighbour edges
!     If so make one edge of the two
!     Check control integers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter6' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter6'
         write(irefwr,*) 'layer ', layer
         call prinin ( nodenrs, numnodes, 'nodenrs' )

      end if
      if ( numnodes.le.5 ) go to 1000

!     --- Set control integers

      edgelength(-1) = 1004d0
      edgelength(numnodes+2) = 1004d0

!     --- Store length of subsequent edges in array edgelength

      xold = bndcoornew(1,numnodes)
      yold = bndcoornew(2,numnodes)
      iprev = nodenrs(numnodes)
      do i = 1, numnodes

         xnew = bndcoornew(1,i)
         ynew = bndcoornew(2,i)
         if ( nodenrs(i).eq.iprev .or. bndangle(i).le.-0.01d0 ) then

!        --- New and old node the same or angle > pi

            edgelength(i) = 0d0

         else

!        --- New and old node different

            ds = (xnew-xold)**2+(ynew-yold)**2
            edgelength(i) = sqrt(ds)

         end if
         xold = xnew
         yold = ynew
         iprev = nodenrs(i)

      end do  ! i = 1, numnodes
      edgelength(0) = edgelength(numnodes)
      edgelength(numnodes+1) = edgelength(1)
      if ( debug ) call prinrl ( edgelength(1), numnodes,'edgelength' )

!     --- Check if the sum of length of two adjacent edges is less than the
!         length of both neighbour edges
!         If so make one edge of the two

      ind = 0
      iprev = numnodes
      do while ( ind.lt.numnodes-2 )

!     --- While loop over boundary

         ind = ind+1
         if ( imark(ind).gt.0 .and. imark(ind+1).gt.0 .and. &
              imark(ind+2).gt.0 .and. edgelength(ind).gt.0d0 .and. &
              edgelength(ind+1).gt.0d0 ) then

!        --- Both edges on the new boundary and not one of them has been
!            skipped

            ds = edgelength(ind)+edgelength(ind+1)
            h = abs(bndcoornew(3,ind))*1.4d0
            if ( ds.lt.edgelength(ind-1) .and. ds.lt.edgelength(ind+2) ) &
               then

!           --- Sum of edge length less than length of neighbours

               nodenrs(ind) = min(nodenrs(iprev),nodenrs(ind))
               nodenrs(iprev) = nodenrs(ind)
               edgelength(ind) = 0d0
               imark(ind) = 1
               ind = ind+2
               changed = .true.
               if ( debug ) write(irefwr,*) '1: ind, iprev, h, ds ', &
                  ind, iprev, h, ds

            else if ( ds.lt.h ) then

!           --- Sum of edge length less than 1.4 h
!               Remove middle point and adapt coordinates

               nodenrs(ind) = min(nodenrs(iprev),nodenrs(ind))
               nodenrs(iprev) = nodenrs(ind)
               edgelength(ind) = 0d0
               xold = bndcoornew(1,iprev)
               yold = bndcoornew(2,iprev)
               xnew = bndcoornew(1,ind+1)
               ynew = bndcoornew(2,ind+1)
               bndcoornew(1,iprev) = xold + 0.15d0*(xnew-xold)
               bndcoornew(2,iprev) = yold + 0.15d0*(ynew-yold)
               bndcoornew(1,ind+1) = xold + 0.85d0*(xnew-xold)
               bndcoornew(2,ind+1) = yold + 0.85d0*(ynew-yold)
               imark(ind) = 1
               ind = ind+2
               changed = .true.
               if ( debug ) write(irefwr,*) '2: ind+1, iprev, h ', &
                  ind+1, iprev, h

            end if  ! ( ds.lt.edgelength(ind-1) ... )

         end if  ! ( imark(ind).gt.0 ... )
         iprev = ind

      end do  ! while ( ind.lt.numnodes-2 )

!     --- Check control integers

      call errcontreal ( edgelength(-1), edgelength(numnodes+2), &
                         1004d0, 'edgelength' )

1000  call erclos ( 'mshpavmninter6' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinin ( imark, numnodes, 'imark' )
         call prinrl ( edgelength(1), numnodes, 'edgelength' )
         write(irefwr,*) 'End mshpavmninter6'

      end if

      end
