      subroutine mshpavmnmesh ( numnodes, bndcoorold, bndcoornew, crd, &
                                layer, nowbnd, maxbounds, ibndnr, imark, &
                                nel, kmeshc, ncrd, newbnd, lennewbnd, &
                                nloop, maxlen, nodenrs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-07-2004 new call to plotmeshinter
!        version  1.0  date 16-07-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a row of elements along a closed boundary using the paver algorithm
!     Special care is required for sharpe corners and concave parts
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, layer, maxbounds, nowbnd(maxbounds,*), ibndnr, &
              imark(*), nel, kmeshc(3,*), ncrd, newbnd(*), lennewbnd, &
              nloop, nodenrs(*)
      double precision bndcoorold(2,*), bndcoornew(4,*), crd(4,*), &
                       maxlen

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold         i    real work array containing the
!                         boundary coordinates of the i'th boundary
!     crd           i/o   array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     ibndnr         i    i'th boundary in nowbnd being looked at
!     imark          i    Array indicating for each point on a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     lennewbnd      i    Declared length of array newbnd
!     maxbounds      i    maximum loops in nowbnd in paver
!     maxlen         i    Maximum length of edges in the surface
!     ncrd          i/o   Sequence number of last node that has been created
!     nel           i/o   number of elements in the topology
!     newbnd              temp array containing the boundary nodes for one loop
!     nloop          i    number of boundaries in nowbnd
!     nodenrs        i    Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd        i/o   array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer prevnode, icount, ind, ntri, i, inode, j, jnd, nelprev, &
              node, nellast
      double precision n1(2), n2(2), len, hi, fact, n(2), norm, maxleng
      logical plotdebug

!     fact           factor with which the thickness grows
!     hi             height of the mesh in node indnr
!     i              Counting variable
!     icount         Defines the direction of the diagonal in case of
!                    a rectangular cell (0 or 1)
!                    This direction is changed subsequently
!     ind            Sequence number along boundary loop
!     inode          Node sequence number
!     j              Counting variable
!     jnd            jth node in jbndnr
!     len            length of edge
!     maxleng         Maximum length of edges
!     n              average normal
!     n1             normal on an edge before node
!     n2             normal on an edge after node
!     nellast        Last element created until
!     nelprev        Number of elements stored in kmeshc at the start
!                    of the subroutine
!     node           Node sequence number of new node
!     norm           length of normal vector
!     ntri           number of extra triangles
!     plotdebug      If true plots must be made for debugging
!     prevnode       Previous node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAV1TRIAN   Create 1 triangle from the three nodes inode1-3 which form
!                    the closed boundary in that sequence
!     MSHPAV2TRIAN   Create 2 triangles form the four nodes inode1-4 which form
!                    the closed boundary in that sequence
!                    The diagonal is chosen such that the largest angle is
!                    subdivided
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PLOTNOWBND     Plot the actual boundary during mesh generation which is
!                    defined in array nowbnd, used in mshpav
!     PRININ         print 1d integer array
!     PRININ1PART    print 2d integer array partly
!     PRINNOWBND     Print array nowbnd which is used in mshpav
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     prevnode := 0
!     For ind := 1 (1) number_of_nodes_along_the_boundary do
!        if ( imark(ind) = 0 ) then
!           add old node to new boundary
!           prevnode := old node
!           if ( ind > 1 and imark(ind-1) # 0 ) then
!              add one triangle
!        else if ( imark(ind) > 0 ) then
!           if ( imark(ind) > 1 ) then
!              adapt new point
!           add new node to boundary
!           if ( ind > 1 ) then
!              if ( imark(ind-1) = 0 ) then
!                 create one triangle
!              else if ( imark(ind-1) > 0 ) then
!                 create 2 triangles
!              else if ( imark(ind-1) = -1 ) then
!                 if ( ind > 2 ) then
!                    create 2 triangles
!                 else if ( imark(last) > 0 ) then
!                    create 2 triangles
!                 else if ( imark(last) = -1 ) then
!                    create one triangle
!           if ( imark(ind) > 1 ) then
!              create new points and triangles
!           prevnode := new node
!        else if ( imark(ind) < 0 and ind > 1 ) then
!           create one triangle
!     Fill last edge depending on imark(1) and imark(last):
!        if ( imark(last) = 0 ) then
!           if ( imark(1) > 0 ) then
!              add one extra triangle from last node to first node
!        else ( imark(last) # 0 )
!           if ( imark(1) <= 0 ) then
!              add one extra triangle from last node to first node
!           else ( imark(1) > 0 )
!              add two extra triangles from last node to first node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnmesh' )
      debug = .false.
      plotdebug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( layer.ge.10000 ) plotdebug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnmesh'
         write(irefwr,1) 'layer, ibndnr, nel', layer, ibndnr, nel
         call prinnowbnd ( nowbnd, nloop, maxbounds, layer )
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinrl1 ( bndcoornew, nowbnd(ibndnr,1), 4, 'bndcoornew' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

!     --- prevnode = 0

      prevnode = 0
      lennewbnd = 1
      icount = 0
      nelprev = nel
      newbnd(1) = 0
      maxleng = maxlen/1.4d0

!     --- Loop over all edges

      do ind = 1, numnodes

         inode = abs(nowbnd(ibndnr,ind+1))
         fact = crd(4,inode)
         hi   = min(maxleng,crd(3,inode))   ! width corresponding to inode
         if ( debug ) then
            write(irefwr,1) 'ind, inode, imark', ind, inode, imark(ind)
            write(irefwr,2) 'fact, hi', fact, hi
         end if  ! ( debug )
         nellast = nel
         if ( imark(ind)==0 ) then

!        --- imark(ind) = 0: the old node must be reused
!            add old node to new boundary
!            prevnode := old node

            if ( newbnd(lennewbnd)/=inode ) then
               lennewbnd = lennewbnd+1
               newbnd(lennewbnd) = inode
               if ( debug ) write(irefwr,1) 'inode, lennewbnd/1', inode, lennewbnd
            end if
            if ( ind>1 ) then
               if ( imark(ind-1)/=0 ) then

!              --- ind > 1 and imark(ind-1) # 0
!                  add one triangle

                  if ( inode/=prevnode ) then

!                 --- different nodes

                     call mshpav1trian ( abs(nowbnd(ibndnr,ind)), &
                                         inode, prevnode, nel, kmeshc )
                     if ( debug ) &
                        call prinin1part ( kmeshc, nel, 3, nel, &
                                           'kmeshc/1' )
                     if ( plotdebug ) then
                        jtimes = 1
                        call plotnowbnd ( nowbnd, nloop, maxbounds, &
                                          layer, crd, 'mshpavmnmesh/1' )
                        jtimes = 3
                        call plotmeshinter ( 1, kmeshc(1,nel), crd, &
                           layer, 4, 3, 'mshpavmnmesh/1' )
                     end if

                  end if  ! ( inode/=prevnode )

               end if  ! ( imark(ind-1)/=0 )

            end if  ! ( ind>1 )
            prevnode = inode

         else if ( imark(ind)>0 ) then

!        --- imark(ind) > 0: there is at least one new node

            if ( imark(ind)>1 ) then

!           --- imark(ind) > 1: more than one point must be created
!               adapt new point
!               Reset normal to normal on previous boundary part

               jnd = ind-1
               if ( jnd==0 ) jnd = numnodes
               n1(1) = -(bndcoorold(2,ind)-bndcoorold(2,jnd))   ! -t1(2)
               n1(2) =  (bndcoorold(1,ind)-bndcoorold(1,jnd))   !  t1(1)
               len = 1d0/sqrt(n1(1)**2+n1(2)**2)
               n1(1) = n1(1)*len
               n1(2) = n1(2)*len
               if ( debug ) then
                   write(irefwr,1) 'ind', ind
                   write(irefwr,2) 'coor-bef', bndcoornew(1,ind), bndcoornew(2,ind)
               end if  ! ( debug ) then

!              --- Recompute coordinates

               fact = crd(4,inode)
               hi   = min(maxleng,crd(3,inode))   ! width corresponding to inode
               bndcoornew(1,ind) = bndcoorold(1,ind)+hi*n1(1)
               bndcoornew(2,ind) = bndcoorold(2,ind)+hi*n1(2)
               bndcoornew(3,ind) = min(maxleng,crd(3,inode)*fact)
               bndcoornew(4,ind) = fact
               if ( debug ) then
                  write(irefwr,2) 'fact, hi, n1 ', fact, hi, n1
                  write(irefwr,2) 'coor-aft ', &
                                   bndcoornew(1,ind), bndcoornew(2,ind)
               end if

            end if  ! ( imark(ind)>1 )

!           --- add new node to boundary

            node = nodenrs(ind)
            crd(1,node) = bndcoornew(1,ind)
            crd(2,node) = bndcoornew(2,ind)
            crd(3,node) = bndcoornew(3,ind)
            crd(4,node) = bndcoornew(4,ind)
            if ( newbnd(lennewbnd)/=node ) then
               lennewbnd = lennewbnd+1
               newbnd(lennewbnd) = node
               if ( debug ) &
                  write(irefwr,1) 'inode, node, ind, lennewbnd/2', &
                                   inode, node, ind, lennewbnd
            end if

            if ( ind>1 ) then

!           --- ind > 1, i.e. not the first point

               if ( imark(ind-1)==0 ) then

!              --- imark(ind-1) = 0: previous node is on old boundary
!                  create one triangle

                  call mshpav1trian ( prevnode, inode, node, nel, &
                                      kmeshc )
                  if ( debug ) &
                     call prinin1part ( kmeshc, nel, 3, nel, 'kmeshc/2')
                  if ( plotdebug ) then
                     jtimes = 1
                     call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                       crd, 'mshpavmnmesh/2' )
                     jtimes = 3
                     call plotmeshinter ( 1, kmeshc(1,nel), crd, &
                          layer, 4, 3, 'mshpavmnmesh/2' )
                  end if

               else if ( imark(ind-1)/=0 ) then

!              --- imark(ind-1) > 0: previous node is on new boundary
!                  create 2 triangles

                  call mshpav2trian ( abs(nowbnd(ibndnr,ind)), inode, &
                                      node, prevnode, crd, nel, &
                                      kmeshc, icount )
                  if ( debug ) call prinin1part ( kmeshc, &
                      nel, 3, nellast+1, 'kmeshc/3' )
                  if ( plotdebug ) then
                     jtimes = 1
                     call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                       crd, 'mshpavmnmesh/3' )
                     jtimes = 3
                     call plotmeshinter ( nel-nellast, &
                          kmeshc(1,nellast+1), crd, &
                          layer, 4, 4, 'mshpavmnmesh/3' )
                  end if

               else if ( imark(ind-1)==-1 ) then

!              --- imark(ind-1) = -1: The previous node is skipped

                  print *, 'hier kan hij niet komen'
                  stop

               end if  ! ( imark(ind-1)==0 )

            end if  ! ( ind>1 )

            if ( imark(ind)>1 ) then

!           --- imark(ind) > 1: more than one point must be created
!               create new points and triangles
!               n2 is normal on next edge

               ntri = imark(ind)-1
               jnd = ind+1
               if ( jnd>numnodes ) jnd = 1
               n2(1) = -(bndcoorold(2,jnd)-bndcoorold(2,ind))  ! -t2(2)
               n2(2) =  (bndcoorold(1,jnd)-bndcoorold(1,ind))  !  t2(1)
               len = 1d0/sqrt(n2(1)**2+n2(2)**2)
               n2(1) = n2(1)*len
               n2(2) = n2(2)*len
               do j = 1, ntri

!              --- first the nodes

                  prevnode = node
                  ncrd = ncrd + 1
                  lennewbnd = lennewbnd+1
                  newbnd(lennewbnd) = ncrd
                  if ( debug ) write(irefwr,1) 'lennewbnd/3', lennewbnd
                  do i = 1, 2
                     n(i) = (ntri-j)*n1(i) + j*n2(i)
                  end do
                  norm = 1d0/sqrt(n(1)**2+n(2)**2)
                  do i = 1, 2
                     crd(i,ncrd) = bndcoorold(i,ind) + (n(i)*norm)*hi
                  end do
                  crd(3,ncrd) = min(maxleng,hi*fact)
                  crd(4,ncrd) = fact
                  if ( debug ) then
                     write(irefwr,1) 'j, ncrd, ntri ', j, ncrd, ntri
                     write(irefwr,2) 'n, hi, fact ', n, hi, fact
                     call prinrl ( crd(1,ncrd), 4, 'crd' )
                  end if

!                 --- now add the elements

                  call mshpav1trian ( inode, ncrd, prevnode, nel, &
                                      kmeshc )
                  if ( debug ) &
                     call prinin1part ( kmeshc, nel, 3, nel, 'kmeshc/7')
                  if ( plotdebug ) then
                     jtimes = 1
                     call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                       crd, 'mshpavmnmesh/6' )
                     jtimes = 3
                     call plotmeshinter ( 1, kmeshc(1,nel), crd, &
                          layer, 4, 3, 'mshpavmnmesh/6' )
                     jtimes = 0
                  end if
                  node = ncrd

               end do  ! j = 1, ntri

            end if  ! ( imark(ind)>1 )

!           --- prevnode := new node

            prevnode = node

         else if ( ind>1 ) then

!        --- imark(ind) = -1 and ind > 1

            if ( imark(ind-1)>0 ) then

!           --- imark(ind-1) > 0 create one triangle

               if ( inode/=prevnode ) then

!              --- different nodes

                  call mshpav1trian ( abs(nowbnd(ibndnr,ind)), inode, &
                                      prevnode, nel, kmeshc )
                  if ( debug ) &
                     call prinin1part (kmeshc, nel, 3, nel, 'kmeshc/10')
                  if ( plotdebug ) then
                     jtimes = 1
                     call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                       crd, 'mshpavmnmesh/7' )
                     jtimes = 3
                     call plotmeshinter ( 1, kmeshc(1,nel), crd, &
                          layer, 4, 3, 'mshpavmnmesh/7' )
                     jtimes = 0
                  end if

               end if

            end if

         else

!        --- imark(ind) = -1 and ind = 1, set prevnode

            prevnode = nodenrs(1)

         end if  ! ( imark(ind)==0 )

      end do  ! ind = 1, numnodes

!     --- Fill last edge depending on imark(1) and imark(last)

      inode = abs(nowbnd(ibndnr,numnodes+1))
      if ( imark(numnodes)==0 ) then

!     --- imark(last) = 0: last point on old boundary

         if ( imark(1)>0 ) then

!        --- imark(1) > 0, i.e. first node on new boundary
!            add one extra triangle from last node to first node

            if ( inode/=newbnd(2) ) then

!           --- different nodes

               call mshpav1trian ( inode, abs(nowbnd(ibndnr,2)), &
                                   newbnd(2), nel, kmeshc )
               if ( debug ) &
                  call prinin1part ( kmeshc, nel, 3, nel,  'kmeshc/8' )
               if ( plotdebug ) then
                  jtimes = 1
                  call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                    crd, 'mshpavmnmesh/8' )
                  jtimes = 3
                  call plotmeshinter ( 1, kmeshc(1,nel), crd, &
                       layer, 4, 3, 'mshpavmnmesh/8' )
                  jtimes = 0
               end if

            end if

         end if  ! ( imark(1)>0 )

      else  ! not ( imark(numnodes)==0 )

!     --- imark(last) # 0: last point on new boundary

         if ( imark(1)==0 ) then

!        --- imark(1) = 0: first point on old boundary
!            create one triangle

            if ( inode/=prevnode ) then

!           --- different nodes

               call mshpav1trian ( inode, abs(nowbnd(ibndnr,2)), &
                                   prevnode, nel, kmeshc )
               if ( debug ) &
                  call prinin1part ( kmeshc, nel, 3, nel, 'kmeshc/9' )

               if ( plotdebug ) then
                  jtimes = 1
                  call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                    crd, 'mshpavmnmesh/9' )
                  jtimes = 3
                  call plotmeshinter ( 1, kmeshc(1,nel), crd, &
                       layer, 4, 3, 'mshpavmnmesh/9' )
                  jtimes = 0
               end if

            end if

         else  ! not ( imark(1)==0 )

!        --- imark(1) # 0: first or second point on new boundary
!            add two extra triangles

            call mshpav2trian ( inode, abs(nowbnd(ibndnr,2)), &
                                newbnd(2), prevnode, &
                                crd, nel, kmeshc, icount )
            if ( debug ) call prinin1part ( kmeshc, nel, 3, &
                nellast+1, 'kmeshc/10' )
            if ( plotdebug ) then
               jtimes = 1
               call plotnowbnd ( nowbnd, nloop, maxbounds, layer, &
                                 crd, 'mshpavmnmesh/10' )
               jtimes = 3
               call plotmeshinter ( nel-nellast, &
                    kmeshc(1,nellast+1), crd, &
                    layer, 4, 4, 'mshpavmnmesh/10' )
               jtimes = 0
            end if

         end if  ! ( imark(1).le.0 )

      end if  ! ( imark(numnodes)==0 )
      lennewbnd = lennewbnd-1
      if ( newbnd(2)==newbnd(lennewbnd+1) ) lennewbnd = lennewbnd-1
      newbnd(1) = lennewbnd

      if ( debug ) then
         write(irefwr,1) 'layer, nel ', layer, nel
         call prinin1part ( kmeshc, nel, 3, nelprev+1, &
                            'kmeshc-2 elements' )
      end if
      if ( plotdebug ) then
         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpavmnmesh/11' )
         jtimes = 3
         call plotmeshinter ( nel-nelprev, kmeshc(1,nelprev+1), crd, &
              layer, 4, 6, 'mshpavmnmesh/11' )
         jtimes = 0
      end if

      call erclos ( 'mshpavmnmesh' )
      if ( debug ) then
         write(irefwr,1) 'nel ', nel
         call prinin ( newbnd, lennewbnd, 'newbnd' )
         write(irefwr,*) 'End mshpavmnmesh'
      end if

      end
