      subroutine mshpavmnrempnt ( iremove, icount, numnodes, bndcoornew, &
                                  nodenrs, layer, bndcoorold, &
                                  edgelength )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove point from new boundary if possible
!     This is only possible if the distance from old point to the new
!     is not too small
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  iremove(*), icount, numnodes, nodenrs(*), layer
      double precision bndcoornew(4,*), bndcoorold(2,*), edgelength(*)

!     bndcoornew    i/o  Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     edgelength     i    Contains the length of the edges along the present
!                         boundary (squared)
!     icount         i    number of nodes in array iremove
!     iremove        i    Contains node numbers of nodes that may be removed
!     layer          i    Present layer sequence number
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    Number of nodal points in array bndcoornew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jcount, ind, indprev, indnext
      logical check, check1

!     check          If true we have jcount > 1 and
!                    iremove(jcount) = iremove(jcount-1)+1
!     check1         If true we havejcount < icount and
!                    iremove(jcount+1) = iremove(jcount)+1
!     ind            Sequence number along boundary loop
!     indnext        Sequence number of next node along boundary
!     indprev        Sequence number of previous node along boundary
!     jcount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVREMNODE  Remove node from new boundary if allowed
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     jcount := 0
!     while ( jcount < icount ) do
!        jcount := jcount+1
!        if ( jcount > 1 and iremove(jcount) = iremove(jcount-1)+1 ) then
!           jcount := jcount+1
!        else if ( jcount < icount and iremove(jcount+1) = iremove(jcount)+1 )
!           then
!           remove node jcount+1 if allowed
!           jcount := jcount+1
!        else if ( jcount = 1 and iremove(1) = 1 and iremove(icount) = numnodes)
!           then
!           remove node 1 if allowed
!           icount := icount-1
!        else
!           check which node in the edge can be removed
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnrempnt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnrempnt'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- jcount := 0

      jcount = 0

!     --- while ( jcount < icount ) do

      do while ( jcount.lt.icount )

!     --- jcount := jcount+1

         jcount = jcount+1
         check = jcount.gt.1
         if ( check ) check = iremove(jcount).eq.iremove(jcount-1)+1
         if ( .not. check ) then
            check1 = jcount.lt.icount
            if ( check1 ) &
               check1 = iremove(jcount+1).eq.iremove(jcount)+1
         end if

         if ( check ) then

!        --- jcount > 1 and iremove(jcount) = iremove(jcount-1)+1
!            jcount := jcount+1

            jcount = jcount+1

         else if ( check1 ) then

!        --- jcount < icount and iremove(jcount+1) = iremove(jcount)+1
!            remove node jcount+1 if allowed
!            jcount := jcount+1

            call mshpavremnode ( numnodes, bndcoornew, nodenrs, &
                                 layer, bndcoorold, iremove(jcount+1) )
            jcount = jcount+1

         else if ( jcount.eq.1 .and. iremove(1).eq.1 .and. &
                   iremove(icount).eq.numnodes ) then

!        --- jcount = 1 and iremove(1) = 1 and iremove(icount) = numnodes
!            remove node 1 if allowed
!            icount := icount-1

            call mshpavremnode ( numnodes, bndcoornew, nodenrs, &
                                 layer, bndcoorold, iremove(1) )
            icount = icount-1

         else

!        --- normal case, check which node in the edge can be removed
!            first we check what the smallest neighbour is

            ind = iremove(jcount)
            indprev = ind-1
            if ( indprev.le.0 ) indprev = numnodes
            indnext = ind+1
            if ( indnext.gt.numnodes ) indnext = 1
            if ( edgelength(indprev).lt.edgelength(indnext) ) then

!           --- previous edge is the smallest, we remove ind

               call mshpavremnode ( numnodes, bndcoornew, &
                                    nodenrs, layer, bndcoorold, ind )

            else

!           --- next edge is the smallest, we remove ind+1

               call mshpavremnode ( numnodes, bndcoornew, &
                                    nodenrs, layer, bndcoorold, &
                                    indnext )

            end if  ! ( edgelength(ind-1).lt.edgelength(ind+1) )

         end if  ! ( check )

      end do  ! while ( jcount.lt.icount )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmnrempnt'

      end if
      call erclos ( 'mshpavmnrempnt' )

      end
