      subroutine mshpavmnsmoot1 ( numnodes, crd, layer, ibndnr, nel, &
                                  kmeshc, newbnd, nelprev, nbndpt, &
                                  deleted, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-05-2004 Remove collapsed elements
!        version  1.0  date 05-11-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Smooth the layer of elements created by removing small elements
!     if possible
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     smoothing
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, layer, ibndnr, nel, kmeshc(3,*), &
              newbnd(*), imark(*), nelprev, nbndpt
      double precision crd(4,*)
      logical deleted

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     deleted       i/o   If true an element has been deleted
!     ibndnr         i    i'th boundary in nowbnd
!     imark          i    array of size ncrd indicating if a node is
!                         on new boundary or old boundary
!                         0: not on boundary or on outer boundary
!                         -1: on old boundary
!                         >0: on new boundary
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nel            i    number of elements in the topology
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     newbnd        i/o   temp array containing the boundary nodes for one loop
!     numnodes       i    Number of nodal points in array newbnd
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, i, inode, icount, iseq, &
              iplus(4), inode1, inode2, inode3
      double precision angle
      logical remove

!     angle          angle between two edges
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     inode          Node sequence number
!     inode1         First node in triangle
!     inode2         Second node in triangle
!     inode3         First node in second triangle
!     iplus          Array containing i+1 modulo 3
!     iseq           Sequence number of node corresponding to old boundary
!                    in array kmeshc(.,ielem)
!     remove         if true element is removed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHANGLE       Compute the cosine of the angle between the nodes with
!                    coordinates x1, x2 and x3, where the angle is defined
!                    as x3-x1-x2
!     MSHPAVDELCOLEL Remove collapsed elements in kmeshc by setting
!                    corresponding entries to 0
!     MSHPAVMNSMOOT0 Check if element may be removed without problem
!     MSHPAVMNSMOOT3 Remove double entries from array newbnd
!     PRININ         print 1d integer array
!     PRININ1PART    print 2d integer array partly
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all new elements do
!        Compute angles of triangle
!        if ( cos(angle) > 0.9 ) then
!           if ( element can be removed, without effecting neighbour elements )
!              remove element
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iplus / 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshpavmnsmoot1' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnsmoot1'
         write(irefwr,*) 'layer, ibndnr, numnodes, nbndpt ', &
                          layer, ibndnr, numnodes, nbndpt
         write(irefwr,*) 'nelprev, nel ', nelprev, nel
         call prinin ( newbnd(2), newbnd(1), 'newbnd' )
         call prinin1part ( kmeshc, nel, 3, nelprev+1, 'kmeshc/1' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- For all new elements do

      do ielem = nelprev+1, nel

!     --- Compute angles of triangle
!         It is only possible to remove an edge if both nodes are on the
!         new boundary

         if ( debug ) then
            write(irefwr,*) 'ielem ', ielem
            call prinin ( kmeshc(1,ielem), 3, 'element' )
         end if
         if ( kmeshc(1,ielem).gt.0 ) then

!        --- kmeshc(1,ielem) > 0, hence element is not removed

            icount = 0
            do i = 1, 3
               inode = kmeshc(i,ielem)
               if ( imark(inode).gt.0 ) then

!              --- Node one new boundary, raise icount

                  icount = icount+1

               else  ! ( imark(inode).ne.1 )

!              --- Node on old boundary, store sequence number

                  iseq = i

               end if  !  ( imark(inode).eq.1 )

            end do  ! i = 1, 3
            if ( debug ) write(irefwr,*) 'icount ', icount
            if ( icount.eq.2 ) then

!           --- two nodes on new bounday

               inode1 = kmeshc(iseq,ielem)
               inode2 = kmeshc(iplus(iseq), ielem )
               inode3 = kmeshc(iplus(iseq+1), ielem )
               if ( inode1.eq.inode2 .or. inode1.eq.inode3 .or. &
                    inode2.eq.inode3 ) then

!              --- Element is collapsed, so remove

                  kmeshc(1,ielem) = 0
                  kmeshc(2,ielem) = 0
                  kmeshc(3,ielem) = 0
                  go to 100

               end if  !( inode1.eq.inode2 .or. inode1.eq.inode3 ... )

!              --- Compute angle inode3-inode1-inode2

               call mshangle ( crd(1,inode1), crd(1,inode2), &
                               crd(1,inode3), angle, 2 )
               remove = angle.gt.0.9d0
               if ( remove ) then

!              --- cos(angle) > 0.9, hence angle < 30 degrees
!                  Check if element may be removed without problem
!                  This means that we want to identify inode2 and inode3 and
!                  replace them by a new coordinate, which is the mean value of
!                  the two

                  call mshpavmnsmoot0 ( crd, layer, ibndnr, nel, &
                                        kmeshc, newbnd, nelprev, &
                                        deleted, imark, ielem, iseq )

               end if  ! ( remove )

100         end if  ! ( icount.eq.2 )

         end if  ! ( kmeshc(1,ielem).gt.0 )

      end do  ! ielem = nelprev+1, nel

!     --- Remove double entries in newbnd

      call mshpavmnsmoot3 ( newbnd )

!     --- Remove double entries in kmeshc

      call mshpavdelcolel ( kmeshc, nelprev, nel )

1000  call erclos ( 'mshpavmnsmoot1' )

      if ( debug ) then

!     --- Debug information

         call prinin1part ( kmeshc, nel, 3, nelprev+1, 'kmeshc/2' )
         call prinin ( newbnd(2), newbnd(1), 'newbnd' )
         write(irefwr,*) 'End mshpavmnsmoot1'

      end if

      end
