      subroutine mshpavtrianct ( iwork, work, ipcube, ipitri, ipjcube, &
                                 ipkstapl, lniwork, lencube )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check control integers around the work arrays in mshpavtrian
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ipcube, ipitri, ipjcube, &
              ipkstapl, lniwork, lencube
      double precision work(*)

!     ipcube         i    Starting address of array chelp
!     ipitri         i    Starting address of array itri
!     ipjcube        i    Starting address of array jcube
!     ipkstapl       i    Starting address of array kstapl
!     iwork          i    integer work array in which all subarrays are stored
!                         The control integers in this array are checked
!     lencube        i    Length of array cube
!     lniwork        i    Declared length of array iwork
!     work           i    Work array in which all subarrays are stored
!                         The control integers in this array are checked
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1509   Leading control integer incorrect
!    1510   Trailing control integer incorrect
!    2475   Leading control value incorrect
!    2476   Trailing control value incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavtrianct' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavtrianct'

      end if
      if ( ierror.ne.0 ) go to 1000
      if ( iwork(1).ne.1001 ) then

!     --- iwork(1) # 1001

         call errint ( iwork(1), 1 )
         call errint ( 1001, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 1509, 2, 0, 1 )

      end if
      if ( iwork(ipitri-2).ne.1001 ) then

!     --- iwork(ipitri-2) # 1001

         call errint ( iwork(ipitri-2), 1 )
         call errint ( 1001, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 1510, 2, 0, 1 )

      end if
      if ( iwork(ipitri-1).ne.1002 ) then

!     --- iwork(ipitri-1) # 1002

         call errint ( iwork(ipitri-1), 1 )
         call errint ( 1002, 2 )
         call errchr ( 'itri', 1 )
         call errsub ( 1509, 2, 0, 1 )

      end if
      if ( iwork(ipjcube-2).ne.1002 ) then

!     --- iwork(ipjcube-2) # 1002

         call errint ( iwork(ipjcube-2), 1 )
         call errint ( 1002, 2 )
         call errchr ( 'itri', 1 )
         call errsub ( 1510, 2, 0, 1 )

      end if
      if ( iwork(ipjcube-1).ne.1003 ) then

!     --- iwork(ipjcube-1) # 1003

         call errint ( iwork(ipjcube-1), 1 )
         call errint ( 1003, 2 )
         call errchr ( 'jcube', 1 )
         call errsub ( 1509, 2, 0, 1 )

      end if
      if ( iwork(ipkstapl-2).ne.1003 ) then

!     --- iwork(ipkstapl-2) # 1003

         call errint ( iwork(ipkstapl-2), 1 )
         call errint ( 1003, 2 )
         call errchr ( 'jcube', 1 )
         call errsub ( 1510, 2, 0, 1 )

      end if
      if ( iwork(ipkstapl-1).ne.1004 ) then

!     --- iwork(ipkstapl-1) # 1004

         call errint ( iwork(ipkstapl-1), 1 )
         call errint ( 1004, 2 )
         call errchr ( 'kstapl', 1 )
         call errsub ( 1509, 2, 0, 1 )

      end if
      if ( iwork(lniwork).ne.1004 ) then

!     --- iwork(lniwork) # 1004

         call errint ( iwork(lniwork), 1 )
         call errint ( 1004, 2 )
         call errchr ( 'kstapl', 1 )
         call errsub ( 1510, 2, 0, 1 )

      end if
      if ( abs(work(1)-1001d0).gt.1d-10 ) then

!     --- work(1) # 1001

         call erreal ( work(1), 1 )
         call erreal ( 1001d0, 2 )
         call errchr ( 'chelp', 1 )
         call errsub ( 2475, 0, 2, 1 )

      end if
      if ( abs(work(ipcube-2)-1001d0).gt.1d-10 ) then

!     --- work(1) # 1001

         call erreal ( work(ipcube-2), 1 )
         call erreal ( 1001d0, 2 )
         call errchr ( 'chelp', 1 )
         call errsub ( 2476, 0, 2, 1 )

      end if
      if ( abs(work(ipcube-1)-1002d0).gt.1d-10 ) then

!     --- work(1) # 1002

         call erreal ( work(ipcube-1), 1 )
         call erreal ( 1002d0, 2 )
         call errchr ( 'cube', 1 )
         call errsub ( 2475, 0, 2, 1 )

      end if
      if ( abs(work(ipcube+lencube)-1002d0).gt.1d-10 ) then

!     --- work(1) # 1002

         call erreal ( work(ipcube+lencube), 1 )
         call erreal ( 1002d0, 2 )
         call errchr ( 'cube', 1 )
         call errsub ( 2476, 0, 2, 1 )

      end if

1000  call erclos ( 'mshpavtrianct' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavtrianct'

      end if

      end
