      subroutine mshpipbouni ( iinput, iposi, kbndpt, kmeshslc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 14-11-2002 Extra parameter kmeshslc
!        version  1.1  date 27-05-2002 Correction for negative pipesurfaces
!
!   copyright (c) 2002-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorders array KBNDPT in case of negative subsurfaces
!     Only for pipe
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     surface
!     pipe
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi, kbndpt(*), kmeshslc(8,*)

!     iinput         i    integer input array    see subroutine mesh
!                         iinput contains only the part with respect to the
!                         definition of the volumes
!     iposi          i    next free position of iinput
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         elements of the boundary
!     kmeshslc       i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubsf, istart, jsur, &
              isubsf, isur, leng2, nelloc, npoloc, &
              ikelmc, nelbot, nelpip
      logical check

!     check          extra debug parameter
!     ikelmc         position in kmesh part c
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     jsur           surface number provided with sign
!     leng2          length indicator elements
!     nelbot         Number of elements in bottom surface
!     nelloc         Local number of elements
!     nelpip         Number of elements in pipe surface
!     npoloc         Number of nodes in subsurface
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIDLN         Compute length of array in BUFFER
!     MSHNEGORDSURF  Reorder array kbndpt for negative subsurfaces of a
!                    composite ordered surface
!     MSHNEGSURF     Reorder array kbndpt for negative subsurfaces of a
!                    composite surface
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read information from bottom surface and store in iwork positions
!     1 to iboun1-1
!     Read boundary of surface and store in iwork positions
!     iboun1 to ipkmsp-1
!     Read kmeshc corresponding to top surface
!     Store temporarily in iwork from position ipkmsp
!     Compare top and bottom
!     Read kmeshc corresponding to pipe surface
!     Store in iwork from position ipkmsp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpipbouni' )

!     --- Get parameters from intarmsh

      check = .false.
      ishape = ishape-8

!     --- Reorder kbndpt of bottom surface

      isurnr = mod(iinput(iposi+3)-1,nsurfs)+1

      nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
      isurf = kmeshslc(1,isurnr)
      ikelmc = 0
      nelbot = kmeshh(2,isurnr+1)
      length = inplmt*nelbot
      if ( nsubsf==0 ) then

!     --- Surface consists of subsurfaces, raise ikelmc

         ikelmc = ikelmc+length

      else if ( isurf==11 .or. isurf==12 ) then

!     --- Ordered surface of subsurfaces

         istart = kmeshr(isurnr)
         do isubsf = 1, nsubsf

            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            nelloc = kmeshh(2,isur+1)
            if ( jsur<0 ) then

!           --- reversed direction, change sequence in kbndpt

               npoloc = kmeshh(1,isur+1)
               call mshnegordsurf ( isurnr, isubsf, nsurfs, npoloc, &
                                    nelloc, kmeshr, ishape, &
                                    kbndpt(ikelmc+1), inplmt )

            end if

         end do
         ikelmc = ikelmc+length

      else

!     --- Surface consists of subsurfaces

         istart = kmeshr(isurnr)
         do isubsf = 1, nsubsf
            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            nelloc = kmeshh(2,isur+1)
            leng2 = nelloc * inplmt
            if ( jsur>0 ) then

!           --- standard direction, raise ikelmc

               ikelmc = ikelmc+leng2

            else

!           --- reversed direction, change sequence in kbndpt

               call mshnegsurf ( nelloc, ishape, inplmt, kbndpt, &
                                 ikelmc )

            end if

         end do

      end if

!     --- Reorder kbndpt of top surface

      isurnr = mod(iinput(iposi+4)-1,nsurfs)+1
      isurf = kmeshslc(1,isurnr)

      nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
      length = kmeshh(2,isurnr+1)*kmeshh(6,isurnr+1)

      if ( nsubsf==0 ) then

!     --- Surface consists of subsurfaces, raise ikelmc

         ikelmc = ikelmc+length

      else if ( isurf==11 .or. isurf==12 ) then

!     --- Ordered surface of subsurfaces

         istart = kmeshr(isurnr)
         do isubsf = 1, nsubsf

            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            nelloc = kmeshh(2,isur+1)
            if ( jsur<0 ) then

!           --- reversed direction, change sequence in kbndpt

               npoloc = kmeshh(1,isur+1)
               call mshnegordsurf ( isurnr, isubsf, nsurfs, npoloc, &
                                    nelloc, kmeshr, ishape, &
                                    kbndpt(ikelmc+1), inplmt )

            end if

         end do
         ikelmc = ikelmc+length

      else

!     --- Surface consists of subsurfaces

         istart = kmeshr(isurnr)
         do isubsf = 1, nsubsf
            jsur = kmeshr(istart+isubsf)
            isur = abs(jsur)
            nelloc = kmeshh(2,isur+1)
            leng2 = nelloc*inplmt

            if ( jsur>0 ) then

!           --- standard direction, raise ikelmc

               ikelmc = ikelmc+leng2

            else

!           --- reversed direction, change sequence in kbndpt

               call mshnegsurf ( nelloc, ishape, inplmt, kbndpt, &
                                 ikelmc )

            end if

         end do

      end if

!     --- Reorder kbndpt corresponding to pipe surface

      isurnr = mod(iinput(iposi+5)-1,nsurfs)+1
      nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
      isurf = kmeshslc(1,isurnr)
      nelpip = kmeshh(2,isurnr+1)
      length = inplmt*nelpip

      if ( nsubsf==0 ) then

!     --- nsubsf = 0, No subsurfaces

         ikelmc = ikelmc+length

      else

!     --- Surface consists of subsurfaces

         ikelmc = ikelmc+length

      end if

      if ( check ) then

!     --- Check = true, print some information

         write(irefwr,*) 'KBNDPT after reordering'
         call prinin1(kbndpt,nelbot,inplmt,'bottom surface')
         call prinin1(kbndpt(1+inplmt*nelbot),nelbot,inplmt, &
                      'top surface')
         call prinin1(kbndpt(1+inplmt*nelbot*2),nelpip,inplmt, &
                      'pipe surface')

      end if
      ishape = ishape+8

      call erclos ( 'mshpipbouni' )

      end
