      subroutine mshpipcopboun ( nelem, inpelm, kbndptold, kbndptnew, &
                                 coor, bcord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates back from the boundary array bcord into array coor
!     The node numbers of the elements are stored in kbndptold (bcord)
!     and kbndptnew (coor)
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh
!     coordinate
!     copy
!     boundary
!     pipe
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, inpelm, kbndptold(inpelm,*), kbndptnew(inpelm,*)
      double precision coor(3,*), bcord(3,*)

!     bcord          i    array containing the coordinates of the boundaries
!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     kbndptnew      i    Nodal point numbers of the boundary elements with
!                         respect to array bcord
!     kbndptold      i    Nodal point numbers of the boundary elements with
!                         respect to array coor
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, ielem, nodeold, nodenew

!     ielem          Element number
!     j              Counting variable
!     k              Counting variable
!     nodenew        New nodal point number with respect to coor
!     nodeold        Old nodal point number with respect to bcord
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpipcopboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpipcopboun'

      end if

!     --- Loop over elements

      do ielem = 1, nelem

!     --- Loop over nodes in element

         do j = 1, inpelm

            nodeold = kbndptold(j,ielem)
            nodenew = kbndptnew(j,ielem)

            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'ielem, j, nodeold, nodenew ', &
                                ielem, j, nodeold, nodenew
               call prinrl ( bcord(1,nodeold), 3, 'bcord' )
               call prinrl ( coor(1,nodenew), 3, 'coor' )

            end if

            do k = 1, 3
               coor(k,nodenew) = bcord(k,nodeold)
            end do

         end do   ! j = 1, inpelm

      end do   ! ielem = 1, nelem

      call erclos ( 'mshpipcopboun' )
      if ( debug ) write(irefwr,*) 'End mshpipcopboun'

      end
