      subroutine mshpipe ( nsurbt, nelz, npobot, bcord, &
                           ipipetopol, nelbot, &
                           kbndpt, ibottopol, ibotboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-05-2014 Extra debug
!        version  3.0  date 24-08-2010 Removal of iwork1
!        version  2.1  date 05-05-2005 Fortran 90
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Generate volume elements with mesh generator PIPE
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     hexahedron
!     coordinate
!     element
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurbt, nelz, npobot, ipipetopol(*), nelbot, &
              kbndpt(*), ibottopol(*), ibotboun(*)
      double precision bcord(*)

!     bcord          i    array containing the coordinates of the boundaries
!     ibotboun       i    Contains node numbers of boundary of bottom surface
!     ibottopol      i    Contains topology of bottom surface
!     ipipetopol     i    Contains topology of pipe surface
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     nelbot         i    Number of elements in bottom surface
!     nelz           i    Number of line elements in vertical direction
!     npobot         i    Number of points of bottom surface
!     nsurbt         i    Number of elements in boundary of bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPIP         Subroutine to create pipe volume elements
!     MSHPIP15       Subroutine to create pipe volume elements for elements
!                    of shape 15 and 16
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpipe' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpipe'
         write(irefwr,1) 'nsurbt, ishape', nsurbt, ishape
  1      format ( a, 1x, (10i8) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ishape<15 .or. ishape>16 ) then

!     --- standard case, just a call to mshpip

         call mshpip ( nsurbt, nelz, npobot, bcord, &
                       bcord(npobot*3+1), bcord(npobot*6+1), coor3d, &
                       ibottopol, nelbot, ibotboun, npointlc, nelemlc, &
                       kmeshc, ivolnr, kbndpt, nbndpt, ipipetopol, &
                       ivol, ishape )

      else

!     --- ishape = 15 or 16, special case

         call mshpip15 ( nsurbt, nelz, npobot, bcord, &
                         ipipetopol, nelbot, kbndpt, &
                         ibottopol, ibotboun )

      end if

1000  if ( debug ) then

         write(irefwr,*) 'End mshpipe'

      end if
      call erclos ( 'mshpipe' )

      end
