      subroutine mshpiplinprism ( npobot, nelbot, kmeshc, ikelm, &
                                  inpelm, kmscbt, istep, inpelmbot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-12-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill bottom layer of elements in case of a pipe consisting of
!     linear prisms, i.e. prisms that are linear in the "z-direction"
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     element
!     topology
!     pipe
!     prism
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istep, npobot, nelbot, inpelm, kmeshc(inpelm,*), ikelm, &
              inpelmbot, kmscbt(inpelmbot,*)

!     ikelm         i/o   Last position used in kmeshc
!     inpelm         i    number of points per element
!     inpelmbot      i    Number of nodes per element of bottom surface and
!                         top surface of pipe
!     istep          o    Step size of nodes in one row to the new row of
!                         elements
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the bottom row of elements
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     nelbot         i    Number of elements in bottom surface
!     npobot         i    Number of points of bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, iielem

!     iielem         Relative element number in element group
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpiplinprism' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpiplinprism'
         write(irefwr,*) 'npobot, nelbot, inpelm, ikelm, inpelmbot ', &
                          npobot, nelbot, inpelm, ikelm, inpelmbot

      end if

      istep = npobot
      do iielem = 1, nelbot

!     --- Loop over elements in bottom cross-section

         do k = 1, inpelmbot
            kmeshc(k,iielem) = kmscbt(k,iielem)
            kmeshc(k+inpelmbot,iielem) = kmscbt(k,iielem)+istep
         end do

      end do
      ikelm = ikelm+inpelm*nelbot

      call erclos ( 'mshpiplinprism' )
      if ( debug ) write(irefwr,*) 'End mshpiplinprism'
      end
