      subroutine mshrdparmesh
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 30-01-2016 New call to mshrdparmesh1
!        version  2.3  date 01-05-2015 Also fill globnodes
!        version  2.2  date 27-11-2012 Long integers
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information about global mesh from file meshoutput
!     Only used if parallel computing is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     parallel
!     neighbor
!     read
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulemach
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lentransinfo, lenblockinfo, npointsav, nelemsav
      integer :: npoint_glob, i

!     lenblockinfo   Length of array blockinfo
!     lentransinfo   Length of array transinfo
!     nelemsav       Help parameter to save the old value of nelem
!     npoint_glob    Number of nodes in global mesh
!     npointsav      Help parameter to save the old value of npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHRDPARMESH1  Read information about global mesh from file
!                    meshoutput_par.000
!     MSHRDPARMSH1   Read information parallel information about global mesh
!                    from file meshoutput_par.000
!     PRINKMESHPAR1  Print the contents of array kmeshpar
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create temporary space to store transportinfo and blockinfo
!     Fill arrays transportinfo and blockinfo by reading from file
!     Make arrays transportinfo and blockinfo permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdparmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdparmesh'
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extend array kmesh with kmesh part parallel
!         starting address kmesh(65)

      km(iseqkm)%kmesh(65) = 10  ! starting address kmesh parallel
      if ( associated(km(iseqkm)%kmeshpar) ) &
         deallocate(km(iseqkm)%kmeshpar)
      allocate(km(iseqkm)%kmeshpar(10))
      kmeshpar => km(iseqkm)%kmeshpar

      if ( debug ) write(irefwr,1) 'kmesh(65)', km(iseqkm)%kmesh(65)

!     --- Clear kmeshpar

      kmeshpar = 0

!     --- Read information from file meshoutput and store in kmeshpar

      npointsav = npoint
      nelemsav = nelem
      call mshrdparmesh1 ( iref10 )
      npoint_glob = kmeshpar(1)

!     --- Create array mapping (local --> global nodal points)
!         Lenght npoint_local (npoint)

      npoint = npointsav
      if ( associated(km(iseqkm)%mappingpar) ) &
         deallocate(km(iseqkm)%mappingpar)
      allocate(km(iseqkm)%mappingpar(npoint))
      mappingpar => km(iseqkm)%mappingpar
      kmeshpar(6) = npoint

!     --- Create temporary space to store transportinfo and blockinfo

      nelem = nelemsav
      lentransinfo = npoint*4
      lenblockinfo = 2*iacnodes**2

      allocate ( blockinfo(lenblockinfo), transinfo(lentransinfo+4), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lentransinfo+lenblockinfo, 'blockinfo' )
      if ( ierror/=0 ) go to 1000

      length = npoint_glob+iacnodes
      if ( associated(km(iseqkm)%globnodes) ) &
         deallocate(km(iseqkm)%globnodes)
      allocate(km(iseqkm)%globnodes(length))
      globnodes => km(iseqkm)%globnodes
      kmeshpar(8) = length

!     --- Fill arrays transportinfo, blockinfo and mapping by reading from file
!         Fill array globnodes with the node numbering corresponding to
!         the parallel case
!         First all nodes in cpu 1, then in cpu 2 etc.

      if ( .not. associated(km(kmesh)%mappar) ) then
         allocate(km(kmesh)%mappar(0:iacnodes))
         allocate(km(kmesh)%mappar(0)%arr(iacnodes))
         do i = 1, iacnodes
            nullify(km(kmesh)%mappar(i)%arr)
         end do  ! i = 1, iacnodes
      end if  ! ( .not. associated(km(kmesh)%mappar) )

      mappar => km(kmesh)%mappar

      if ( debug ) write(irefwr,*) 'before mshrdparmsh1'
      call mshrdparmsh1 ( lenblockinfo, lentransinfo, &
                          npoint_glob, globnodes, globnodes(1+iacnodes) )
      if ( ierror/=0 ) go to 1000

!     --- Make a copy of mapping and store mm address in kmeshpar(9)

      if ( associated(km(iseqkm)%mappingparcp) ) &
         deallocate(km(iseqkm)%mappingparcp)
      allocate(km(iseqkm)%mappingparcp(npoint))
      mappingparcp => km(iseqkm)%mappingparcp
      kmeshpar(9) = npoint
      mappingparcp = mappingpar

!     --- Make arrays transportinfo and blockinfo permanent

      if ( associated(km(iseqkm)%transinfo) ) &
         deallocate(km(iseqkm)%transinfo)
      allocate(km(iseqkm)%transinfo(lentransinfo))
      km(iseqkm)%transinfo = transinfo(1:lentransinfo)
      km(iseqkm)%kmesh(53) = lentransinfo

      if ( associated(km(iseqkm)%blockinfo) ) &
         deallocate(km(iseqkm)%blockinfo)
      allocate(km(iseqkm)%blockinfo(lenblockinfo))
      km(iseqkm)%blockinfo = blockinfo(1:lenblockinfo)
      km(iseqkm)%kmesh(54) = lenblockinfo

      deallocate ( transinfo, blockinfo, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'transinfo' )
      if ( ierror/=0 ) go to 1000

!     --- Force sepmodulekmesh to be filled again

      call seprelkmesh ( kmesh )

1000  if ( debug ) then

!     --- Debug information

         call prinkmeshpar1
         write(irefwr,*) 'End mshrdparmesh'

      end if  ! ( debug )
      call erclos ( 'mshrdparmesh' )

      end subroutine mshrdparmesh
