      subroutine mshreacheckl ( checkl, icheck )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read and interprete part of input corresponding to check_levels
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheck
      logical checkl

!     checkl        i/o   If true check_level has been found
!     icheck         o    Level of check of elements
!                         0:  no special checks (default)
!                         1:  check volume of elements
!                         2:  check volume of elements and print minimum and
!                             maximum value per surface/volumeIndication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1542   Check_level has incorrect value
!    1552   keyword found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreacheckl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreacheckl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( checkl ) then

!     --- check_level has been found more than once

         call errchr ( string(1:nchar), 1 )
         call errwar ( 1552, 0, 0, 1 )

      end if
      checkl = .true.
      call asknxt
      if ( ityprd/=2 ) then

!     --- not an integer found

         call errchr ( 'CHECK_LEVEL', 1 )
         call errsub ( 574, 0, 0, 1 )

      else
         icheck = nint(realrd)
         if ( icheck<0 .or. icheck>4 ) then

!        --- wrong value of icheck

            call errint ( icheck, 1 )
            call errint ( 0, 2 )
            call errint ( 3, 3 )
            call errwar ( 1542, 3, 0, 0 )
            icheck = 3

         end if
      end if

1000  call erclos ( 'mshreacheckl' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreacheckl'

      end if  ! ( debug )

      end
