      subroutine mshrefkmeshc2 ( kmeshc, kmeshcnew, nelem, nodenumbers, &
                                 nelprev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 2, i.e. quadratic line elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     quadratic
!     1d
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), kmeshcnew(3,*), nelem, nodenumbers(*), &
              nelprev

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, iielem

!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iielem         Relative element number in element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        kmeshcnew(1,ielemnew) := kmeshc(1,ielem)
!        kmeshcnew(2,ielemnew) := nodenumbers(ielem)
!        kmeshcnew(3,ielemnew) := kmeshc(2,ielem)
!        ielemnew := ielemnew+1
!        kmeshcnew(1,ielemnew) := kmeshc(2,ielem)
!        kmeshcnew(2,ielemnew) := nodenumbers(ielem)+1
!        kmeshcnew(3,ielemnew) := kmeshc(3,ielem)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc2'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do iielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         kmeshcnew(1,ielemnew) := kmeshc(1,ielem)
!         kmeshcnew(2,ielemnew) := nodenumbers(ielem)
!         kmeshcnew(3,ielemnew) := kmeshc(2,ielem)
!         ielemnew := ielemnew+1
!         kmeshcnew(1,ielemnew) := kmeshc(2,ielem)
!         kmeshcnew(2,ielemnew) := nodenumbers(ielem)+1
!         kmeshcnew(3,ielemnew) := kmeshc(3,ielem)

         ielem = iielem+nelprev
         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshc(1,iielem)
         kmeshcnew(2,ielemnew) = nodenumbers(ielem)
         kmeshcnew(3,ielemnew) = kmeshc(2,iielem)
         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshc(2,iielem)
         kmeshcnew(2,ielemnew) = nodenumbers(ielem)+1
         kmeshcnew(3,ielemnew) = kmeshc(3,iielem)

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshc2'

      end if

      end
