      subroutine mshrefkmeshc3 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                 nmeshd, nodenumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 3, i.e. linear triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     linear
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), kmeshcnew(3,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*)

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(3), i

!     edge           Contains the edge node numbers on the 3 edges
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFKMESHC3E Refine one element in case of a complete refinement
!                    Element of shape 3, i.e. linear triangles
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc3'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev
         call prinin1 ( kmeshc, nelem, 3, 'kmeshc/surface' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         fill first element
!         ielemnew := ielemnew+1
!         fill second element
!         ielemnew := ielemnew+1
!         fill third element
!         ielemnew := ielemnew+1
!         fill fourth element

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 3
            iedge = abs(nmeshd(ipstart+i))
            edge(i) = nodenumbers(iedge)
         end do

         call mshrefkmeshc3e ( kmeshc(1,ielem), kmeshcnew, ielemnew, &
                               edge )

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc3' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 3, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc3'

      end if

      end
