      subroutine mshrefkmeshcs ( kmeshcnew, nodenumbers, lenkmeshc, nelemnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!        version  1.1  date 27-11-2011 use function getshapeelgrp
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc case that the complete mesh is refined
!     In this case we have surface elements, line elements but no volume
!     elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), nodenumbers(*), nelemnew, lenkmeshc

!     kmeshcnew      o    New kmesh part c
!     lenkmeshc      i    length of array kmeshc
!     nelemnew       i    Number of elements in new mesh
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelprev, ielgrp, i, &
              ikelmc, ikelmcnew, inpelm, nsub

!     i              Counting variable
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmcnew      Starting address of next element in kmeshcnew
!     inpelm         number of points per element
!     nelprev        Number of elements stored in kmeshc at the start
!                    of the subroutine
!     nsub           Number of elements an element is subdivided in
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFKMESHC10 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 10, i.e. extended linear triangles
!     MSHREFKMESHC3  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 3, i.e. linear triangles
!     MSHREFKMESHC4  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 4, i.e. quadratic triangles
!     MSHREFKMESHC5  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 5, i.e. bi-linear quadrilaterals
!     MSHREFKMESHC6  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 6, i.e. bi-quadratic quadrilaterals
!     MSHREFKMESHC7  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 7, i.e. extended quadratic triangles
!     MSHREFKMESHC9  Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 9, i.e. extended bi-linear quadrilaterals
!     MSHREFKMESHCL1 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 1, i.e. linear line elements
!     MSHREFKMESHCL2 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 2, i.e. quadratic line elements
!     MSHREFKMESHCL9 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape -9, i.e. large line element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshcs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshcs'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      nelemnew = 0
      nelprev = 0
      ikelmcnew = 1

      ikelmc = 1
      do ielgrp = 1, nelgrp

!     --- ishape := shape number of element

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem, inpelm', &
                             ielgrp, ishape, inelem, inpelm

!        --- for all elements in group do
!            fill kmeshcnew

         if ( ishape==1 ) then

!        --- ishape = 1, linear line elements

            call mshrefkmeshcl1 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers )
            nsub = 2

         else if ( ishape==2 ) then

!        --- ishape = 2, quadratic line elements

            call mshrefkmeshcl2 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers )
            nsub = 2

         else if ( ishape==3 ) then

!        --- ishape = 3, linear triangles

            call mshrefkmeshc3 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                 inelem, nelprev, nmeshd, nodenumbers )
            nsub = 4

         else if ( ishape==4 ) then

!        --- ishape = 4, quadratic triangles

            call mshrefkmeshc4 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                 inelem, nelprev, nmeshd, nodenumbers, &
                                 nedges )
            nsub = 4

         else if ( ishape==5 ) then

!        --- ishape = 5, bi-linear quadrilaterals

            call mshrefkmeshc5 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                 inelem, nelprev, nmeshd, nodenumbers, &
                                 nedges )
            nsub = 4

         else if ( ishape==6 ) then

!        --- ishape = 6, bi-quadratic quadrilaterals

            call mshrefkmeshc6 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                 inelem, nelprev, nmeshd, nodenumbers, &
                                 nedges )
            nsub = 4

         else if ( ishape==7 ) then

!        --- ishape = 7, extended quadratic triangles

            call mshrefkmeshc7 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                 inelem, nelprev, nmeshd, nodenumbers, &
                                 nedges )
            nsub = 4

         else if ( ishape==9 ) then

!        --- ishape = 9, extended bi-linear quadrilaterals

            call mshrefkmeshc9 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                 inelem, nelprev, nmeshd, nodenumbers, &
                                 nedges )
            nsub = 4

         else if ( ishape==10 ) then

!        --- ishape = 10, extended linear triangles

            call mshrefkmeshc10 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers, &
                                  nedges )
            nsub = 4

         else if ( ishape==-9 ) then

!        --- ishape -9, large line element consisting of all nodes along
!            a set of curves

            call mshrefkmeshcl9 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inpelm, nelprev, nmeshd, nodenumbers )
            nsub = 0
            nelemnew = nelemnew+1
            ikelmcnew = ikelmcnew+inpelm*2-1
            if ( nelgrp>1 ) kmeshg(ielgrp) = inelem
            do i = ielgrp, nelgrp
               kmesha(i+1) = kmesha(i+1)+inpelm-1
            end do

         else if ( ishape==-10001 ) then

!        --- ishape = -10001, connection element, is filled later on

            nsub = 0

         else

!        --- shape not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if  ! ( ishape==1 )
         ikelmc = ikelmc+inelem*inpelm
         ikelmcnew = ikelmcnew+inelem*nsub*inpelm
         nelemnew = nelemnew+nsub*inelem
         nelprev = nelprev+inelem
         if ( nsub>0 .and. nelgrp>1 ) kmeshg(ielgrp) = nsub*inelem
         if ( ikelmcnew-1>lenkmeshc ) then

!        --- Array too short

            call errint ( ikelmcnew-1, 1 )
            call errint ( lenkmeshc, 1 )
            call errchr ( 'kmeshc_new', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         if ( debug ) write(irefwr,1) 'ikelmcnew', ikelmcnew

      end do  ! ielgrp = 1, nelgrp
      lenkmeshc = ikelmcnew-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshcs'

      end if
      call erclos ( 'mshrefkmeshcs' )

      end
