      subroutine mshrefkmeshmcr ( kmeshm, kmeshmnew, leng, &
                                  nodenumbers, nmeshl, nmeshc, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-07-2004 Use iniedge
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy information of old curve as stored in kmesh part m into
!     new curve extended with intermediate point in case a mesh is refined
!     completely
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), kmeshmnew(*), leng, nodenumbers(*), &
              nmeshl(*), nmeshc(*), nedges

!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshmnew      o    New array kmeshm
!     leng           i    Help parameter to indicate the length of arrays
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c
!     nmeshl         i    nmesh part l
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ipstart, inode, inode1, inode2, irel, iedge, &
              iwork(nedges), numedges, numnodes

!     i              Counting variable
!     iedge          edge sequence number
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     irel           Relative position of two nodes on edge
!     iwork          integer work array of length nedges
!                    if iwork(i) = 1 node i is connected to edge, 0 not
!     j              Counting variable
!     numedges       number of edges
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2460   No edge found corresponding to two nodes
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all sets of 2 nodes on curve do
!        iedge := common edge for both nodes
!        compute relative position of two nodes on edge
!        determine new node number
!        copy first node and new node to kmeshmnew
!     copy last node to kmeshmnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshmcr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshmcr'
         write(irefwr,*) 'nedges, leng ', nedges, leng
         call prinin ( kmeshm, leng, 'kmeshm/curve' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array iwork

      do i = 1, nedges
         iwork(i) = 0
      end do

!     --- For all sets of 2 nodes on curve do

      do i = 1, leng-1

!     --- iedge := common edge for both nodes
!         compute relative position of two nodes on edge
!         determine new node number
!         copy first node and new node to kmeshmnew

         inode1 = kmeshm(i)           ! first node in set
         inode2 = kmeshm(i+1)         ! second node in set
         ipstart = nmeshl(inode1)     ! start position of inode1 in nmeshl
         numedges = nmeshl(ipstart)   ! number of edges connected to inode1
         if ( debug ) &
            write(irefwr,*) 'inode1, inode2, numedges ', &
                             inode1, inode2, numedges

!        --- find edge number connected to inode1 and inode2

         iedge = iniedge ( inode1, inode2, nmeshl, iwork )
         if ( iedge.eq.0 ) then

!        --- error: no edge found

            call errint ( inode1, 1 )
            call errint ( inode2, 2 )
            call errsub ( 2460, 2, 0, 0 )
            go to 1000

         end if

!        --- compute relative position of two nodes on edge

         ipstart = nmeshc(iedge)     ! start position of iedge in nmeshc
         numnodes = nmeshc(ipstart)  ! number of nodes connected to iedge
         irel = 2
         if ( numnodes.gt.2 ) then

!        --- number of nodes more than 2 find last position

            do j = 1, numnodes

               inode = nmeshc(ipstart+j)
               if ( inode.eq.inode1 .or. inode.eq.inode2 ) then

!              --- Node belongs to edge, find largest entry

                  irel = max(j,irel)

               end if

            end do  ! j = 1, numnodes-1
            if ( debug ) &
               call prinin ( nmeshc(ipstart+1), numnodes, 'edge' )

         end if  ! ( numnodes.gt.2 )

!        --- determine new node number
!            copy first node and new node to kmeshmnew

         inode = nodenumbers(iedge)+irel-2
         kmeshmnew(2*i-1) = inode1
         kmeshmnew(2*i) = inode
         if ( debug ) write(irefwr,*) 'i ,inode, inode1, irel ', &
            i ,inode, inode1, irel

      end do  ! i = 1, leng-1

!     --- copy last node to kmeshmnew

      kmeshmnew(2*leng-1) = inode2

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmnew, 2*leng-1, 'kmeshmnew/curve' )
         write(irefwr,*) 'End mshrefkmeshmcr'

      end if
      call erclos ( 'mshrefkmeshmcr' )

      end
