      subroutine mshshiftnod5 ( coornew, indelm, map )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 26-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift the intermediate nodes onto their faces according to
!     the vertex nodes. Straight triangle/tetrahedron are formed.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer map(*), indelm(nelemcac)
      double precision  coornew(ndim,*)

!     coornew        i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     indelm         i    Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     map            i    Mapping array that is filled if ichoice = 2 or must
!                         be filled if ichoice = 3
!                         map is an integer array of length 5 with the following
!                         contents:
!                         1: mmimap  memory management reference to array imap
!                         2: 130 to indicate the type of array
!                         3: mmrmap  memory management reference to array rmap
!                         4: mmcoor1  memory management reference to array coor1
!                         5: mmcoor2  memory management reference to array coor2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  j, jnode(15), istart, idx(11)

!     idx            Array containing the indices of the non-vertices
!     istart         First position used at a part
!     j              Counting variable
!     jnode          Array containing the node numbers of an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
         data idx /2, 4, 6, 7, 8, 9, 11, 12, 13, 14, 15/
! ======================================================================
!
      call eropen ( 'mshshiftnod5' )
      istart=0

!     --- loop over element groups

      do ielgrp = 1, nelgrp

!        --- in case of extended quadratic triangle

         if (ishape==7) then

!        --- loop over elements

            do ielem=1,inelem
               inpelm = Topgrp(ielgrp)%inpelm
               inelem = Topgrp(ielgrp)%inelem
               ishape = kmeshf(ielgrp)
               if (indelm(ielem)==1) then

!              --- get node numbers of element

                  do j=1,inpelm
                     jnode(j)= kmeshc((ielem-1)*inpelm+j)
                  end do

                  do j=1,7
                     if (map(jnode(j))>0) then
                        map(jnode(j))=0
                     end if
                  end do

!                 --- place midside nodes and centroid based on
!                     vertex positions

                  do j=1,ndim
                     coornew(j,jnode(2)) = (coornew(j,jnode(1))+ &
                                         coornew(j,jnode(3)))*0.5d0
                     coornew(j,jnode(4)) = (coornew(j,jnode(3))+ &
                                         coornew(j,jnode(5)))*0.5d0
                     coornew(j,jnode(6)) = (coornew(j,jnode(5))+ &
                                         coornew(j,jnode(1)))*0.5d0
                     coornew(j,jnode(7)) = ( coornew(j,jnode(1))+ &
                                          coornew(j,jnode(3))+ &
                                          coornew(j,jnode(5)) )/3d0
                  end do
               end if
            end do

!           --- in case of extended tetrahedron

         else if (ishape==16) then

!        --- loop over elements

            do ielem=1,inelem
               if (indelm(ielem)==1) then

!              --- get node numbers of element

                  do j=1,inpelm
                     jnode(j)= kmeshc((ielem-1)*inpelm+j)
                  end do

                  do j=1,11
                     if (map(jnode(idx(j)))>0) then
                       map(jnode(idx(j)))=0
                     end if
                  end do

!                 --- place midside, midplane nodes and centroid based on
!                     vertex positions

                  do j=1,ndim
                     coornew(j,jnode(2))= (coornew(j,jnode(1))+ &
                                        coornew(j,jnode(3)))*0.5d0
                     coornew(j,jnode(4))= (coornew(j,jnode(3))+ &
                                        coornew(j,jnode(5)))*0.5d0
                     coornew(j,jnode(6))= (coornew(j,jnode(5))+ &
                                        coornew(j,jnode(1)))*0.5d0
                     coornew(j,jnode(7))= (coornew(j,jnode(1))+ &
                                        coornew(j,jnode(10)))*0.5d0
                     coornew(j,jnode(8))= (coornew(j,jnode(3))+ &
                                        coornew(j,jnode(10)))*0.5d0
                     coornew(j,jnode(9))= (coornew(j,jnode(4))+ &
                                        coornew(j,jnode(10)))*0.5d0
                     coornew(j,jnode(11))= ( coornew(j,jnode(1))+ &
                                          coornew(j,jnode(3))+ &
                                          coornew(j,jnode(5)) )/3d0
                     coornew(j,jnode(12))= ( coornew(j,jnode(1))+ &
                                          coornew(j,jnode(3))+ &
                                          coornew(j,jnode(10)) )/3d0
                     coornew(j,jnode(13))= ( coornew(j,jnode(3))+ &
                                          coornew(j,jnode(5))+ &
                                          coornew(j,jnode(10)) )/3d0
                     coornew(j,jnode(14))= ( coornew(j,jnode(1))+ &
                                          coornew(j,jnode(5))+ &
                                          coornew(j,jnode(10)) )/3d0
                     coornew(j,jnode(15))= ( coornew(j,jnode(1))+ &
                                          coornew(j,jnode(3))+ &
                                          coornew(j,jnode(5))+ &
                                          coornew(j,jnode(10)) )*2.5d-1
                  end do
               end if
            end do
            istart=istart+inelem*inpelm
            jnode(j)= kmeshc(istart+(ielem-1)*inpelm+j)
         end if

      end do
      call erclos('mshshiftnod5')

      end
