      subroutine mshshiftnod6 ( coor, kmeshd, nmeshe, npoint, kmeshx, &
                                map, icurvs, curves, shifted, kmeshe, &
                                bndnod, fluidnd, ilast )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  2.0  date 23-04-2003  Complete revision
!        version  1.0  date 26-11-2002
!
!   copyright (c) 2002-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Shift the midside nodes of the fluid element onto the obstacle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshe(*), kmeshd(*), npoint, kmeshx(6,*), map(*), &
              icurvs(*), fluidnd(*), ilast, shifted(*), kmeshe(*), &
              bndnod
      double precision coor(2,*), curves(2,*)

!     bndnod         i    Node that lies on boundary of the fluid domain and
!                         on the obstacle boundary
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curves         i    Array containing the coordinates of the curves
!     fluidnd        o    Array that contains the nodal numbers of the coupling
!                         elements in case of fluid-structure computations
!     icurvs         i    Array containing the number of points in the curves
!                         accumulated
!     ilast          o    Number of coupling elements in case of fluid-structure
!                         computations
!     kmeshd         i    KMESH part d
!     kmeshe         i    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     kmeshx       i    Integer part of kmesh part x (i = obstacle number)
!                         kmeshx(1,i) = type of obstacle (1=curve, 2=surface)
!                         kmeshx(2,i) = curve/surface number of obstacle
!                         kmeshx(3,i) = nx
!                         kmeshx(4,i) = ny
!                         kmeshx(5,i) = mm sequence number of iobgrid
!                         kmeshx(6,i) = starting address surfaces corresponding
!                                       to obstacle
!                         In R^3:
!                         kmeshx(5,i) = nz
!                         kmeshx(6,i) = mm sequence number of iobgrid
!                         kmeshx(7,i) = starting address volumes corresponding
!                                       to obstacle
!                         kmeshx(8,i) = mm sequence number of kmeshc_iobst
!                         kmeshx(9,i) = mm sequence number of coor_iobst
!                         kmeshx(10,i) = inpelm
!                         kmeshx(11,i) = nelem
!                         kmeshx(12,i) = npoint
!     map            i    Mapping array that is filled if ichoice = 2 or must
!                         be filled if ichoice = 3
!                         map is an integer array of length 5 with the following
!                         contents:
!                         1: mmimap  memory management reference to array imap
!                         2: 130 to indicate the type of array
!                         3: mmrmap  memory management reference to array rmap
!                         4: mmcoor1  memory management reference to array coor1
!                         5: mmcoor2  memory management reference to array coor2
!     nmeshe         i    nmesh part e contains the node sequence numbers of
!                         the elements
!     npoint         i    Number of nodal points
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, i1, j, k, kk, istart, icurnr, inpobst, inode(3), &
               nrelem, ielem, jelem(2), node1, node2, idx(4), &
               midpoint, oppnode(2), idx1(3), jnode, r, &
               midold, istart1, iobst, ind, cent
      double precision x1(2), x2(2), xlow(2), xupp(2), fact1, fact2, &
                       eps, minl, l, xobst(2,3)
      logical found

!     cent           Node number of the centroid
!     eps            Local accuracy
!     fact1          Defines the factor for the intersection point
!                    The intersection point on the face is defined by
!                    x1 + fact1 * (x2-x1)
!     fact2          Defines the factor for the intersection point with respect
!                    to the obstacle curve
!                    There is only an intersection if 0 <= fact2 <= 1
!     found          If true already a possibility has been found
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     icurnr         Curve sequence number
!     idx            Help array to store indices
!     idx1           Array containing the indices of the vertices
!     ielem          Element number
!     ind            Node number of obstacle node that is closest to midpoint
!     inode          Node sequence number
!     inpobst        Number of nodal points in obstacle
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     j              Counting variable
!     jelem          do loop variable with respect to the elements
!     jnode          Array containing the node numbers of an element
!     k              Counting variable
!     kk             Counting variable
!     l              General loop variable
!     midold         Former found midpoint
!     midpoint       Node number of the node between two vertices
!                    (quadratic curve)
!     minl           Minimal distance
!     node1          First node of fluid curve
!     node2          Last node of fluid curve
!     nrelem         Number of elements
!     oppnode        Node number of node opposite (considering a quadratic
!                    triangular element) to midpoint
!     r              Counting variable
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     xlow           First coordinate of obstacle part
!     xobst          Contains the coordinats of the 3 points of a quadratic
!                    obstacle curve element
!     xupp           Last coordinate of obstacle part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCROSSCURVE  Determine intersection of straight edge with curved
!                    obstacle edge
!     MSHCROSSLINE   Computes the intersection of two lines
!     MSHSHIFTNOD6A  Determine the consequent order of fluid nodes on the
!                    obstacle
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     3000           Midpoint is not found
!     3001           Intersection is not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!         call to mshshiftnod6a -> find fluid nodes that lie on the
!                                  obstacle in consequent order
!         loop over fluid nodes that lie on the obstacle
!           find midside node
!           loop over obstacle nodes
!             find obstacle node closest to midside node
!           end loop
!           loop over 2 elements connected to midside node
!             loop over 2 obstacle segments connected to obstacle node
!               determine intersection (coarse)
!               determine intersection (exact)
!               move midside point onto the obstacle
!             end loop
!           end loop
!         end loop
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data  idx1 /1,3,5/
! ======================================================================
!
      call eropen ( 'mshshiftnod6' )

      debug=.false.

!     --- get the fluid vertices in consequent order

      call mshshiftnod6a( shifted, kmeshe, npoint, bndnod, fluidnd, &
                          ilast )

!     --- set some variables

      eps=1d-1
      midold=0
      node2=0

!     --- loop over fluid points on obstacle boundary

      do i=1,ilast-2, 2
         midpoint=0
         node1=fluidnd(i)
         node2=fluidnd(i+2)
         if (debug) write(*,*)'node1, node2', node1, node2

         jelem(1)=0
         jelem(2)=0
         kk=1
         nrelem=kmeshd(node1+1)-kmeshd(node1)

!        --- loop over elements connected to node1

         do k=1,nrelem
            ielem=kmeshd(npoint+1+kmeshd(node1)+k)
            if (jelem(2).eq.0) then
               if (jelem(1).eq.0) then

!              --- find first element with 2 nodes on obstacle

                  istart1=nmeshe(ielem)

!                 --- loop over vertices of element

                  do j=1,3

                     inode(j)=nmeshe(istart1+idx1(j))

!                    --- check if element has 2 nodes on obstacle

                     if (inode(j).eq.node2) then

!                    --- 2 nodes in element found

                        jelem(kk)=ielem
                        idx(2)=idx1(j)
                     end if

                  end do

               else

!              --- find second element

                  kk=2
                  istart1=nmeshe(ielem)

!                 --- loop over vertices of element

                  do j=1,3
                     inode(j)=nmeshe(istart1+idx1(j))

!                    --- check if element has 2 nodes on obstacle

                     if ( inode(j).eq.node2 ) then
                        jelem(kk)=ielem
                        idx(2)=idx1(j)
                     end if
                  end do

               end if

               if (jelem(kk).ne.0) then

!              --- find index of first node

                  do j=1,3
                     if (inode(j).eq.node1) idx(1)=idx1(j)
                  end do

!                 --- midpoint found

                  if (idx(1)+idx(2).eq.4) idx(3)=2
                  if (idx(1)+idx(2).eq.8) idx(3)=4
                  if (idx(1)+idx(2).eq.6) idx(3)=6
                  midpoint=nmeshe(istart1+idx(3))
                  if (midpoint.ne.midold) then

!                 --- find node opposite to midpoint

                     if (idx(3).eq.2) idx(4)=5
                     if (idx(3).eq.4) idx(4)=1
                     if (idx(3).eq.6) idx(4)=3
                     oppnode(kk)=nmeshe(istart1+idx(4))
                  else
                     jelem(kk)=0
                  end if
               end if
            end if !end if jelem.eq.0

         end do !end loop over elements around node

         if (midpoint.le.0.or.midpoint.gt.npoint) then
            call errint ( node1, 1 )
            call errint ( node2, 2 )
            call errsub ( 3000, 2, 0, 0 )
            goto 1000
         end if

         iobst  = 1
         icurnr = kmeshx(2,iobst)
         if ( icurnr.eq.1 ) then

!        --- icurnr = 1, first curve

            inpobst = icurvs(icurnr)
            istart = 1
         else

!        --- icurnr > 1

            inpobst = icurvs(icurnr)-icurvs(icurnr-1)
            istart = icurvs(icurnr-1)+1
         end if
         minl=rinfin

!        --- loop over obstacle nodes

         do jnode=1,inpobst-1

!        --- determine distance between

            l=sqrt( (curves(1,istart-1+jnode)- &
                                   coor(1,midpoint))**(2d0) + &
                    (curves(2,istart-1+jnode)- &
                                   coor(2,midpoint))**(2d0) )

!           --- find obstacle node that is closest to the midpoint

            if (minl.gt.l) then
               minl=l
               ind=jnode
            end if
         end do
         found=.true.

!        --- loop over 2 elements the midpoint is connected to

         do k=1,2

!        --- loop over obstacle segments connected to node ind

            do j=-3,3

               if (ind-1+j.ge.1.and.ind+j.le.inpobst) then

!              --- if a midpoint is found

                  if (found) then
                     xlow(1)=curves(1,istart-1+ind-1+j)
                     xlow(2)=curves(2,istart-1+ind-1+j)
                     xupp(1)=curves(1,istart-1+ind+j)
                     xupp(2)=curves(2,istart-1+ind+j)
                     x1(1)=coor(1,midpoint)
                     x1(2)=coor(2,midpoint)
                     x2(1)=coor(1,oppnode(k))
                     x2(2)=coor(2,oppnode(k))
                     if (midpoint.ne.0) map(midpoint)=-1

                     call mshcrossline ( x1, x2, xlow, xupp, fact1, &
                                         fact2, eps )

                     if (debug) then
                        write(*,*)'xobst1, yobst1  ', xlow(1), xlow(2)
                        write(*,*)'xobst2, yobst2  ', xupp(1), xupp(2)
                        write(*,*)'element ',k, jelem(k)
                        write(*,*)'nodenr, x1, y1  ', midpoint, &
                                           x1(1), x1(2)
                        write(*,*)'nodenr, x2, y2  ', oppnode(k), &
                                           x2(1), x2(2)
                        write(*,*)'fac1, fac2  ', fact1, fact2
                     end if

                     if ( fact2.ge.0 .and. fact2.le.1d0 ) then

!                    --- find coordinates of element

                        if (mod(ind-1+j,2).gt.0) then
                           xobst(1,1) = curves(1,istart-1+ind-1+j)
                           xobst(2,1) = curves(2,istart-1+ind-1+j)
                           xobst(1,2) = curves(1,istart-1+ind+j)
                           xobst(2,2) = curves(2,istart-1+ind+j)
                           xobst(1,3) = curves(1,istart-1+ind+1+j)
                           xobst(2,3) = curves(2,istart-1+ind+1+j)
                        else
                           xobst(1,1) = curves(1,istart-1+ind-2+j)
                           xobst(2,1) = curves(2,istart-1+ind-2+j)
                           xobst(1,2) = curves(1,istart-1+ind-1+j)
                           xobst(2,2) = curves(2,istart-1+ind-1+j)
                           xobst(1,3) = curves(1,istart-1+ind+j)
                           xobst(2,3) = curves(2,istart-1+ind+j)
                        end if

!                       --- intersection must be determined more accurately

                        call mshcrosscurve ( x1, x2, xobst, fact1 )
                        found=.false.

                        if (debug) then
                           write(*,*)'fac1, fac2  ', fact1, fact2
                        end if

!                       --- place midpoint in the right position

                        coor(1,midpoint)= &
                                (1d0-fact1) * coor(1,midpoint)+ &
                                      fact1 * coor(1,oppnode(k))
                        coor(2,midpoint)= &
                                (1d0-fact1) * coor(2,midpoint)+ &
                                      fact1 * coor(2,oppnode(k))
                        midold=midpoint

!                       --- position centroid for both elements
!                           the midpoint is connected to

                        do i1=1,2
                           istart1=nmeshe(jelem(i1))
                           cent=nmeshe(istart1+7)
                           do r=1,2
                              coor(r,cent)=(1d0/9d0)* &
                                     ( -coor(r,nmeshe(istart1+1)) &
                                       -coor(r,nmeshe(istart1+3)) &
                                       -coor(r,nmeshe(istart1+5))+ &
                                   4d0*(coor(r,nmeshe(istart1+2))+ &
                                        coor(r,nmeshe(istart1+4))+ &
                                        coor(r,nmeshe(istart1+6))) )
                           end do
                        end do
                        fluidnd(i+1)=midpoint
                     end if
                  end if
               end if
            end do ! end loop obstacle segments
         end do ! end loop elements

         if (fluidnd(i+1).eq.0) then
            call errint ( midpoint, 1 )
            call errsub ( 3001, 1, 0, 0 )
            goto 1000
         end if

      end do ! end loop i (npoint)

      if (debug) then
         do i=1,ilast
            write(*,*)'fluidedge node  ', fluidnd(i)
         end do
      end if

1000  call erclos ( 'mshshiftnod6' )
      end
