      subroutine mshsl3 ( coor, ncoor, curves, iinp, iinput, rinput, &
                          tau, icurve, ispinp, ndim, nuspnt, &
                          icurnr )
! ======================================================================
!
!        programmer    Sander de Snoo
!        version  3.1  date 12-01-2009 Remove array work
!        version  3.0  date 02-02-1997 Extra parameters
!        version  2.0  date 30-10-1996 Extra parameter ispinp
!        version  1.1  date 15-09-1996 Correction for line through 2 points
!        version  1.0  date 19-04-1996
!
!   copyright (c) 1996-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to compute coordinates of a curve for a free boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, iinput(*), icurve, ncoor, ispinp(*), ndim, nuspnt, &
              icurnr
      double precision rinput(*), coor(*), curves(*), tau(*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     curves     o    array containing the coordinates of the curves
!     icurnr     i    Curve sequence number
!     icurve     i    Type of curve
!     iinp       i    last position used in array iinput
!     iinput     i    integer input array
!     ispinp          Work array of length 2+ncoor to store spline information
!     ncoor      i    number of coordinates of old curve
!     ndim       i    Dimension of the space.
!     nuspnt     i    Number of user points
!     rinput     i    Real input array
!     tau             Work array of length n (number of nodes to be created)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelm, iratio, j1, j2, irnp, j, ishape, n, jcoars
      double precision factor, edge1, edge2, dx, dy, rinploc(1)

!     dx         distance in x-direction
!     dy         distance in y-direction
!     edge1      coarseness at begin curve
!     edge2      coarseness at end curve
!     factor     Multiplication factor
!     iratio     Ratio as defined in input
!     irnp
!     ishape     Shape number of elements
!     j          Counting variable
!     j1         begin userpoints of curve
!     j2         end userpoints of curve
!     jcoars     If 1 coarseness is given, if 0 not
!     n          Number of points along curve
!     nelm       Number of elements along curve
!     rinploc    Local array rinput containing the parameter alpha
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSH004    Fill array tau
!     MSHC01    Fill array curves with splines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsl3' )

      ishape   = iinput(iinp+1)
      nelm     = iinput(iinp+2)
      iratio   = iinput(iinp+3)
      jcoars = iinput(11)

      n = nelm*ishape+1

      irnp = ndim*nuspnt
      if ( jcoars==1 ) irnp = irnp+nuspnt
      factor = rinput(irnp+icurnr)

      if ( icurve==5 .or. icurve==6 .or. icurve==9 .or. &
           icurve==12 ) then

!     --- compute coarseness

         if (icurve==1 .or. icurve==3 .or. icurve==5 .or. &
             icurve==6 .or. icurve==8 .or. icurve==9) then
           j1=iinput(iinp+4)
           j2=iinput(iinp+5)
         else if (icurve==4 .or. icurve==12) then
           j1=iinput(iinp+6)
           j2=iinput(iinp+iinput(iinp+5)-1)
         end if

         edge1=rinput(1+ndim*nuspnt+j1)*rinput(1+ndim*nuspnt)
         edge2=rinput(1+ndim*nuspnt+j2)*rinput(1+ndim*nuspnt)

!        --- compute factor

         factor = (edge2/edge1)**(1d0/nelm)
         iratio = 2

      end if

!     --- compute tau

      call msh004 ( tau, ishape, nelm, n, iratio, factor, 0 )

!     --- generate new spline following old curve
!         ispinp = itype=0, n, + [1..n]
!         rinput = coor

      if ( ncoor==2 ) then

!     --- Only two points, i.e. straight line

         dx = coor(3)-coor(1)
         dy = coor(4)-coor(2)
         do j = 1, n
            curves(2*j-1) = coor(1)+dx*tau(j)
            curves(2*j) = coor(2)+dy*tau(j)
         end do

      else

!     --- more than two points: spline

         ispinp(1)=22
         ispinp(2)=ncoor
         do j = 1, ncoor
            ispinp(2+j)=j
         end do
         rinploc(1) = 0.5d0
         call mshc01 ( ispinp, coor, curves, ndim, n, tau, &
                       rinploc, icurnr )
      end if

      call erclos ( 'mshsl3' )

      end
