      subroutine mshsmooth3d1 ( npoint, ndim, inpelm,coor, dtdx, &
                                ang, compgrad, knode, kmeshc, &
                                kmeshd, length )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 26-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!         Compute the angles of the elements and their derivatives
!         in case of 3D tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     smoothing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint, compgrad, knode, length, &
              kmeshc(*), kmeshd(*), inpelm
      double precision  coor(ndim,*), ang(length), dtdx(length,ndim)

!     ang            o    Angle
!     compgrad       i    Parameter for computing derivatives of angle (1)
!                         or not (0)
!     coor           i    Array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dtdx           o    Array containing the derivatives of the angles
!     kmeshc         i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     kmeshd         i    KMESH part d, contains the elements corresponding to
!                         the nodal points
!     knode          i    Shifting node
!     length         i    Help parameter to indicate the length of arrays
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i,j,k,l,ii, ielem, kk, idx(3,3), inode, jnode(4), &
               nrelem, idx1(4)
      double precision  sidelength(8,3), deltax, ang1(3,length), &
                        veca(3,3), vecb(3,3), vecc(3,3,3), &
                        vecd(3,3,3), sum

!     ang1           Angle
!     deltax         Estimate of unit length of element
!     i              Counting variable
!     idx            Array used for roulating the indices
!     idx1           Array for finding the vertices
!     ielem          Element number
!     ii             Counting variable
!     inode          Node sequence number
!     inpelm         number of points per element
!     j              Counting variable
!     jnode          Array containing the node numbers of the vertices
!     k              Counting variable
!     kk             Counting variable
!     l              General loop variable
!     nrelem         Number of elements around node
!     sidelength     Array containing the lengths of edges
!     sum            Help parameter to compute a sum
!     veca           Array containing 3 edges of an element which have
!                    the same node
!     vecb           Array containing the three normals to each couple
!                    of edges in veca
!     vecc           Array containing 3 edges of an element which have
!                    the same node. One node of the element is shifted
!                    slightly
!     vecd           Array containing the three normals to each couple
!                    of edges in veca. One node of the element is shifted
!                    slightly
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1,2,3, 2,3,1, 3,1,2  /
      data idx1 / 1,3,5,10 /
! ======================================================================
!
      call eropen ( 'mshsmooth3d1' )

      kk=1
      deltax=1d-5

!     --- determine the nr of elements around knode

       nrelem=kmeshd(knode+1)-kmeshd(knode)

!      --- loop over the elements

       do k=1,nrelem
          ielem= kmeshd(npoint+1+kmeshd(knode)+k)

!         --- determine which nodes are involved in element

          do j=1,4
             if (inpelm.eq.4) then
                jnode(j) = kmeshc((ielem-1)*inpelm+j)
             else if (inpelm.eq.15) then
                jnode(j) = &
                   kmeshc((ielem-1)*inpelm+idx1(j))
             end if
          end do

!         --- determine the lengths of the elementsides
!             first loop over nodes

          do inode=1,4
            ii=1

!           --- second loop over nodes

            do j=1,4
               if(j.ne.inode) then
                  do i=1,ndim

!                 --- determine the edges of the element connected to nodeloop
!                     over nodes

                     veca(i,ii)= coor(i,jnode(j))-coor(i,jnode(inode))
                     vecc(1,i,ii)= coor(i,jnode(j))-coor(i,jnode(inode))
                     vecc(2,i,ii)= coor(i,jnode(j))-coor(i,jnode(inode))
                     vecc(3,i,ii)= coor(i,jnode(j))-coor(i,jnode(inode))

                  end do

                  if (compgrad.eq.1) then
                     if ( jnode(j).eq.knode) then
                        vecc(1,1,ii)=coor(1,jnode(j))+deltax- &
                                     coor(1,jnode(inode))
                        vecc(2,2,ii)=coor(2,jnode(j))+deltax- &
                                     coor(2,jnode(inode))
                        vecc(3,3,ii)=coor(3,jnode(j))+deltax- &
                                     coor(3,jnode(inode))
                     else if (jnode(inode).eq.knode) then
                        vecc(1,1,ii)=coor(1,jnode(j))- &
                                    (coor(1,jnode(inode))+deltax)
                        vecc(2,2,ii)=coor(2,jnode(j))- &
                                    (coor(2,jnode(inode))+deltax)
                        vecc(3,3,ii)=coor(3,jnode(j))- &
                                    (coor(3,jnode(inode))+deltax)
                     end if
                  end if

                  ii=ii+1
               end if
            end do

!           --- compute the normals to the plane and the lengths of
!               these normals and the lengths of the edges

            do j=1,3
               do i=1,ndim
                  vecb(i,j)=veca(idx(i,2),j)*veca(idx(i,3),idx(j,2))- &
                            veca(idx(i,3),j)*veca(idx(i,2),idx(j,2))
               end do
               sidelength(1,j) = sqrt ( veca(1,j)**2d0+ &
                                        veca(2,j)**2d0+ &
                                        veca(3,j)**2d0 )
               sidelength(2,j) = sqrt ( vecb(1,j)**2d0+ &
                                        vecb(2,j)**2d0+ &
                                        vecb(3,j)**2d0 )
            end do

            if (compgrad.eq.1) then

!           --- compute the normals to the plane and the lengths of
!               these normals and the lengths of the edges

               do j=1,3
                  do l=1,ndim
                     do i=1,ndim
                        vecd(l,i,j) = vecc(l,idx(i,2),j)* &
                                      vecc(l,idx(i,3),idx(j,2))- &
                                      vecc(l,idx(i,3),j)* &
                                      vecc(l,idx(i,2),idx(j,2))
                     end do
                     sidelength(l-1+3,j)= sqrt( vecd(l,1,j)**2d0+ &
                                                vecd(l,2,j)**2d0+ &
                                                vecd(l,3,j)**2d0 )
                     sidelength(l-1+6,j)= sqrt( vecc(l,1,j)**2d0+ &
                                                vecc(l,2,j)**2d0+ &
                                                vecc(l,3,j)**2d0 )
                  end do
               end do
            end if

!           --- compute the angles: ang(x)

            do j=1,3
               sum=0d0
               do i=1,ndim
                  sum=sum+veca(i,idx(j,3))*vecb(i,j)
               end do
               ang(kk-1+j)=acos(sum/(sidelength(1,idx(j,3))* &
                                       sidelength(2,j)))

!              --- compute angle depending on the direction of the normal

               if (ang(kk-1+j).le.(0.5d0*pi)) then
                   ang(kk-1+j)=(0.5d0*pi)-ang(kk-1+j)
               else
                  ang(kk-1+j)=ang(kk-1+j)-(0.5d0*pi)
               end if
            end do

!           --- compute the angles: ang1(x+dx)

            if (compgrad.eq.1) then

               do j=1,3
                  do l=1,ndim
                     sum=0d0
                     do i=1,ndim
                         sum=sum+vecc(l,i,idx(j,3))*vecd(l,i,j)
                     end do
                     ang1(l,kk-1+j) = &
                            acos(sum/(sidelength(l-1+6,idx(j,3))* &
                                      sidelength(l-1+3,j)))
                     if (ang1(l,kk-1+j).le.(0.5d0*pi)) then
                        ang1(l,kk-1+j)=(0.5d0*pi)-ang1(l,kk-1+j)
                     else
                        ang1(l,kk-1+j)=ang1(l,kk-1+j)-(0.5d0*pi)
                     end if
                  end do
               end do

!              --- compute the derivatives of the angles

               do j=1,3
                  do i=1,ndim
                     dtdx(kk-1+j,i)=(ang1(i,kk-1+j)-ang(kk-1+j))/deltax
                  end do
               end do

            end if

            kk=kk+3

         end do  ! end if first node loop

      end do  ! end element loop

      call erclos ( 'mshsmooth3d1' )
      end
