      subroutine mshspecreorkn
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder elements of shape 5 in kmesh part n in case of spectral elements
!     without fill-in
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     reordering
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, inpelm, istart

!     inpelm         number of points per element
!     istart         First position used at a part
!     isurnr         Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSPECREORD05 Reorder elements of shape 5 in case of spectral elements
!                    without fill-in (one element group only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspecreorkn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspecreorkn'

      end if
      if ( ierror/=0 ) go to 1000
      istart = 2*nsurfs+1

      do isurnr = 1, nsurfs

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)
         if ( inpelm==4 .and. inelem>0 ) then

!        --- Quads of 4 points, reorder

            call mshspecreord05 ( kmeshn(istart), inelem )

         end if
         istart = istart+inpelm*inelem

      end do  ! isurnr = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspecreorkn'

      end if
      call erclos ( 'mshspecreorkn' )

      end
