      subroutine mshsurfaces2 ( nouterfaces, outerfaces, &
                                surffaces, nleft, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     If there are outer faces not belonging to a surface then
!     create new surface
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     outer_boundary
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nouterfaces, outerfaces(nouterfaces), &
              surffaces(*), nleft, inpelm

!     inpelm         i    number of points per element
!     nleft          o    Number of outer faces that do not belong to a surface
!     nouterfaces    i    Number of outer faces (length of outerfaces)
!     outerfaces     i    Array containing the face numbers of the outer faces
!     surffaces      o    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, isurnr, iielem, iface, iwork(nfaces)

!     i              Counting variable
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length nfaces
!                    for each outer face iface iwork(iface) is set to -1
!                    if iface belongs to a surface it is reset to 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:nfaces)
!     For all faces iface in array outerfaces do
!        iwork(iface) := -1
!     For all surfaces stored in kmeshn and surffaces do
!        For all faces iface in surface do
!           iwork(iface) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces2'

      end if
      if ( ierror/=0 ) go to 1000
      inpelm = kmeshn(1)

!     --- Clear array iwork(1:nfaces)

      iwork = 0

!     --- For all faces iface in array outerfaces do
!            iwork(iface) := -1

      iwork(outerfaces) = -1

!     --- For all surfaces stored in kmeshn and surffaces do

      istart = 0
      nleft = nouterfaces
      do isurnr = 1, nsurfs

         inelem = kmeshn(isurnr+nsurfs)

!        --- For all faces iface in surface do

         do iielem = 1, inelem

!        --- iwork(iface) := 0

            iface = surffaces(istart+iielem)
            if ( iwork(iface)<0 ) then

!           --- Outer surface found

               iwork(iface) = 0
               nleft = nleft-1

            end if

         end do  ! iielem = 1, inelem
         istart = istart+inelem

      end do  ! isurnr = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nleft ', nleft
         write(irefwr,*) 'End mshsurfaces2'

      end if
      call erclos ( 'mshsurfaces2' )

      end
