      subroutine mshtransinfo3 ( iwork, neighbours, nnodes, points, &
                                 istore, n_ij, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of nodes n_ij on common boundary of ineigh and
!     iblock with iwork(i) > 0 and store the nodes in array istore
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     topology
!     neighbour
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), neighbours(*), nnodes, points(*), istore(*), &
              n_ij, npoint

!     istore         o    Work array of length n_ij to store the
!                         common neighbour nodal points of two neighbour blocks
!                         of blocknr
!     iwork          i    integer work array of length npoint that is filled
!                         in the following way:
!                         iwork(i) = -1:  point i is in block iblock
!                         iwork(i) =  0:  point i is not a neighbour of iblock
!                         iwork(i) =  1:  point i is a neighbour of iblock
!     neighbours     i    Array of length npoint that marks all nodes in block
!                         blocknr in the follwoing way
!                         neighbours(i) = 0, node not in ineigh
!                         neighbours(i) > 0 node is the neighbours(i)-th
!                         local node of blocknr
!     nnodes         i    Number of nodes
!     n_ij           o    nodes in the interface of iblock and ineigh that are
!                         also neighbour nodes of blocknr
!     npoint         i    Number of nodal points (only used for debugging)
!     points         i    Array of size npointloc containing the node
!                         numbers corresponding to the ineigh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode

!     i              Counting variable
!     inode          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     n_ij := 0
!     For i := 1(1) nnodes do
!        inode := points(i)
!        if ( inode neighbour of blocknr and inode in iblock ) then
!           n_ij := n_ij+1;
!           istore(n_ij) := neighbours(inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtransinfo3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtransinfo3'
         call prinin ( iwork, npoint, 'iwork' )
         call prinin ( neighbours, npoint, 'neighbours' )
         call prinin ( points, nnodes, 'points' )

      end if

!     --- n_ij := 0

      n_ij = 0

!     --- For i := 1(1) nnodes do

      do i = 1, nnodes

!     --- inode := points(i)

         inode = points(i)
         if ( iwork(inode).gt.0 .and. neighbours(inode).gt.0 ) then

!        --- inode neighbour of blocknr and inode in iblock )
!            n_ij := n_ij+1;
!            istore(1,n_ij) := neighbours(inode);  istore(2,n_ij) := i

            n_ij = n_ij+1
            istore(n_ij) = neighbours(inode)

         end if  ! ( iwork(inode).gt.0 .and. neighbours(inode).gt.0 )

      end do  ! i = 1, nnodes

      call erclos ( 'mshtransinfo3' )

      if ( debug ) then

!     --- Debug information

         call prinin ( istore, n_ij, 'istore' )
         write(irefwr,*) 'End mshtransinfo3'

      end if

      end
