      subroutine mshtriancentr ( kmeshc, npoint, nelem, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Place extra centre point in quadratic triangle or quadratic tetrahedron
!     The number of points is extended
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     centre
!     node
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), npoint, nelem, inpelm

!     inpelm         i    Number of nodes per element
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         At input the 6-point triangles must have been stored
!                         At output the 7-point triangles are stored
!     nelem          i    Number of elements
!     npoint        i/o   Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, istartold, istartnew, inpelm1

!     i              Counting variable
!     ielem          Element number
!     inpelm1        inpelm-1
!     istartnew      Starting address of old element in kmeshc
!     istartold      Starting address of new element in kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For ielem := nelem (-1) 1
!        Compute old starting address in kmeshc of element ielem
!        Compute new starting address in kmeshc of element ielem
!        Copy old triangle to new positions
!        Put node in centroid
!        Raise node number
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtriancentr' )
      inpelm1 = inpelm-1

!     --- For ielem := nelem (-1) 1

      do ielem = nelem, 1, -1

!     --- Compute old starting address in kmeshc of element ielem
!         Compute new starting address in kmeshc of element ielem

         istartold = (ielem-1)*inpelm1
         istartnew = (ielem-1)*inpelm

!        --- Copy old triangle to new positions

         do i = inpelm1, 1, -1
            kmeshc(istartnew+i) = kmeshc(istartold+i)
         end do

!        --- Put node in centroid
!            Raise node number

         npoint = npoint+1
         kmeshc(istartnew+inpelm) = npoint

      end do

      call erclos ( 'mshtriancentr' )

      end
