      subroutine mshwrplot ( iinput, rinput, irefmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 30-06-2008 Extra debug
!        version  1.0  date 01-02-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write parts of arrays iinput and rinput corresponding to plot
!   in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh
      double precision rinput(*)

!     iinput         i    integer input array
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilast, indent
      character *1000 wrtrecord

!     ilast          Last curve (surface) number read
!     indent         number of spaces for the indent
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTRL       Put real number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshwrplot' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwrplot'
  1      format ( a, 1x, (10i6) )

      end if
      iplot = iinput(9)
      ndim = iinput(1)
      ilast = 0
      indent = 3

      call iniputxt ( 'plot,', wrtrecord, ilast )
      if ( iplot>0 ) then

!     --- write plot record, old definition

         jmark = mod(iplot,10)
         numsub = iplot/10
         if ( ndim>2 ) then
            i3d = 1
         else
            i3d = 0
         end if
         call iniputxt ( ' jmark = ', wrtrecord, ilast )
         call iniputin ( jmark, wrtrecord, ilast )
         if ( numsub>0 ) then

!        --- numsub>0

            call iniputxt ( ', numsub = ', wrtrecord, ilast )
            call iniputin ( numsub, wrtrecord, ilast )

         end if

      else if ( iplot<0 ) then

!     --- write plot record, new definition

         if ( debug ) then
            write(irefwr,1) 'numsub, jcurvp, jcurvn', &
                             numsub, jcurvp, jcurvn
            write(irefwr,1) 'juserp, i3d, irenpl', &
                             juserp, i3d, irenpl
            write(irefwr,1) 'nomesh, iorien', nomesh, iorien
         end if  ! ( debug )
         call iniputxt ( ' jmark = ', wrtrecord, ilast )
         call iniputin ( jmark, wrtrecord, ilast )
         if ( numsub>0 ) then

!        --- numsub>0

            call iniputxt ( ', numsub = ', wrtrecord, ilast )
            call iniputin ( numsub, wrtrecord, ilast )

         end if
         if ( jcurvp/=2 ) then

!        --- jcurvp # 2

            call iniputxt ( ', curve = ', wrtrecord, ilast )
            call iniputin ( jcurvp, wrtrecord, ilast )

         end if
         if ( jcurvn/=0 ) then

!        --- jcurvn # 0

            call iniputxt ( ', nodes = ', wrtrecord, ilast )
            call iniputin ( jcurvn, wrtrecord, ilast )

         end if
         if ( juserp/=0 ) then

!        --- juserp # 0

            call iniputxt ( ', userpoints = ', wrtrecord, ilast )
            call iniputin ( juserp, wrtrecord, ilast )

         end if
         if ( dcolor/=0 ) then

!        --- dcolor # 0

            call iniputxt ( ', colour = ', wrtrecord, ilast )
            call iniputin ( dcolor, wrtrecord, ilast )

         end if
         if ( nomesh/=0 ) &
            call iniputxt ( ', nomesh ', wrtrecord, ilast )
         if ( irenpl/=0 ) &
            call iniputxt ( ', ren_plot ', wrtrecord, ilast )
         if ( i3d/=0 ) then

!        --- i3d # 0

            call iniputxt ( ', orientation = ', wrtrecord, ilast )
            call iniputin ( iorien, wrtrecord, ilast )

         end if

      end if

      if ( rinput(1)>0d0 ) then

!     --- plotfm>0: length

         call iniputxt ( ', length = ', wrtrecord, ilast )
         call iniputrl ( rinput(1), wrtrecord, ilast )

      else if ( rinput(1)<0d0 ) then

!     --- plotfm<0: scale

         call iniputxt ( ', scale = ', wrtrecord, ilast )
         call iniputrl ( -rinput(1), wrtrecord, ilast )

      end if
      if ( rinput(2)>0d0 ) then

!     --- yfact>0: yfact

         call iniputxt ( ', yfact = ', wrtrecord, ilast )
         call iniputrl ( rinput(2), wrtrecord, ilast )

      end if

      if ( i3d>0 ) then

!     --- i3d>0, eye point

         if ( abs(rinput(irnp3-2))<=0d0 .and. &
              abs(rinput(irnp3-1))<=0d0 .and. &
              abs(rinput(irnp3))<=0d0 ) then

!        --- eye point is zero, set to large value

            rinput(irnp3-2) = 1d3
            rinput(irnp3-1) = 1d3
            rinput(irnp3) = 2d3

         end if  ! ( abs(rinput(irnp3-2))<=0d0 ... )
         call iniputxt ( ', eyepoint = (', wrtrecord, ilast )
         call iniputrl ( rinput(irnp3-2), wrtrecord, ilast )
         call iniputxt ( ' , ', wrtrecord, ilast )
         call iniputrl ( rinput(irnp3-1), wrtrecord, ilast )
         call iniputxt ( ' , ', wrtrecord, ilast )
         call iniputrl ( rinput(irnp3), wrtrecord, ilast )
         call iniputxt ( ' )', wrtrecord, ilast )
      end if

      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

      if ( debug ) write(irefwr,*) 'End mshwrplot'
      call erclos ( 'mshwrplot' )

      end
