      subroutine mshwrvolumes ( iinput, irefmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 08-08-2008 Extension with extra curves/surfaces
!        version  1.0  date 01-02-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write parts of arrays iinput and rinput corresponding to volumes
!   in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh

!     iinput         i    integer input array
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nvols
      parameter ( nvols=11 )
      integer ntxtvol(nvols), ilast, indent, i, ivolm, ndefsf, &
              iinp, j, iinpsav
      character (len=11) txtvolms(nvols)
      character (len=1000) wrtrecord

!     i              Counting variable
!     iinp           last position used in array iinput
!     iinpsav        Help variable to save the value of iinpx
!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     ivolm          Type of volume with respect to mesh generator
!     j              Counting variable
!     ndefsf         Number of curves in surface
!     ntxtvol        Help array to store the length of the names of the
!                    volume generators
!     nvols          Number of volume generators
!     txtvolms       Help array to store the names of the volume
!                    generators
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
!     MSH050         Raise iinp as function of ivolm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data txtvolms / 'brick', 'user', ' ','pipe', 'general', &
                      'translate', 'rotate', 'reflect', 'channel', &
                      'general', 'old_general' /
      data ntxtvol / 5, 4, 1, 4, 7,   9, 6, 7, 7, 7, &
                    11 /
! ======================================================================
!
      call eropen ( 'mshwrvolumes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwrvolumes'

      end if
      indent = 3
      ilast = 0

      if ( nvolms>0 ) then

!     --- nvolms>0
!         write record with volumes

         call iniputxt ( 'volumes', wrtrecord, ilast )
         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

      end if

!     --- write volumes

      indent = 6
      iinp = iinp5
      do i = 1, nvolms
         ivolm = iinput(iinp)
         call iniputxt ( 'v', wrtrecord, ilast )
         call iniputin ( i, wrtrecord, ilast )
         call iniputxt ( ' = ', wrtrecord, ilast )
         call iniputxt ( txtvolms(ivolm)(1:ntxtvol(ivolm)), &
                         wrtrecord, ilast )
         ndefsf = mod(iinput(iinp+2),10000)

!        --- write shape number

         call iniputxt ( ' ', wrtrecord, ilast )
         call iniputin ( iinput(iinp+1), wrtrecord, ilast )
         call iniputxt ( ' ( ', wrtrecord, ilast )

!        --- write surface numbers

         do j = 1, ndefsf
            if ( iinput(iinp+2+j)<0 ) &
               call iniputxt ( '-', wrtrecord, ilast )
            call iniputxt ( 's', wrtrecord, ilast )
            call iniputin ( abs(iinput(iinp+2+j)), wrtrecord, ilast )
            call iniputxt ( ', ', wrtrecord, ilast )
         end do
         ilast = ilast-2

         if ( ivolm==10 ) then

!        --- ivolm = 10, extra surfaces, curves and user points

            iinpsav = iinp+3
            nextrapnts = iinput(iinpsav+1)  ! number of internal points
            nextracrvs = iinput(iinpsav+2)  ! number of internal curves
            nextrasrfs = iinput(iinpsav+3)  ! number of internal surfaces
            iinpsav = iinpsav+3

            if ( nextrapnts>0 ) then

!           --- nextrapnts>0, internal points

               call iniputxt ( ', internal_points = (', wrtrecord, &
                               ilast )
               do j = 1, nextrapnts
                  call iniputxt ( 'p', wrtrecord, ilast )
                  call iniputin ( iinput(iinpsav+j), wrtrecord, ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do  ! j = 1, nextrapnts
               ilast = ilast-2
               call iniputxt ( ')', wrtrecord, ilast )

            end if  ! ( nextrapnts>0 )
            iinpsav = iinpsav+nextrapnts

            if ( nextracrvs>0 ) then

!           --- nextracrvs>0, internal points

               call iniputxt ( ', internal_curves = (', wrtrecord, &
                               ilast )
               do j = 1, nextracrvs
                  call iniputxt ( 'c', wrtrecord, ilast )
                  call iniputin ( iinput(iinpsav+j), wrtrecord, ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do  ! j = 1, nextracrvs
               ilast = ilast-2
               call iniputxt ( ')', wrtrecord, ilast )

            end if  ! ( nextracrvs>0 )
            iinpsav = iinpsav+nextracrvs

            if ( nextrasrfs>0 ) then

!           --- nextrasrfs>0, internal surfaces

               call iniputxt ( ', internal_surfaces = (', &
                                wrtrecord, ilast )
               do j = 1, nextrasrfs
                  call iniputxt ( 's', wrtrecord, ilast )
                  call iniputin ( iinput(iinpsav+j), wrtrecord, ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do  ! j = 1, nextrasrfs
               ilast = ilast-2
               call iniputxt ( ')', wrtrecord, ilast )

            end if  ! ( nextrasrfs>0 )

         end if  ! ( ivolm==10 )

         call iniputxt ( ' )', wrtrecord, ilast )

         call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )
         call msh050 ( iinput, iinp, ivolm )

      end do

      if ( debug ) write(irefwr,*) 'End mshwrvolumes'
      call erclos ( 'mshwrvolumes' )

      end
