      subroutine sepmshnorm2d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute normals on boundary in R^2 and store in array normal
!     Old subroutine mshnorm2d
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     normal_vector
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenpart3

!     lenpart3       Length of kmeshm part 3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMSHNORC0    Compute normals on boundary in R^2 and store in array
!                    normal
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     make array normal a permanent array
!     compute starting addresses
!     Compute the normals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmshnorm2d' )

!     --- make array normal a permanent array
!         the length of normal is 2 times part 3 of kmeshm

      lenpart3 = kmeshm(3)-kmeshm(2)
      if ( km(iseqkm)%kmesh(61)>0 ) deallocate(km(iseqkm)%normal)
      allocate(km(iseqkm)%normal(2,lenpart3))
      km(iseqkm)%kmesh(61) = lenpart3
      normal => km(iseqkm)%normal

!     --- Compute the normals

      call sepmshnorc0

      call erclos ( 'sepmshnorm2d' )
      end
