      subroutine sepmshnorm3d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute normals on boundary in R^3 and store in array normal
!     Old subroutine mshnorm3d
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
!     normal_vector
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPFILLSURNODS Create and fill array surfnodes
!     SEPMSHNORS0    Compute normals on boundary in R^3 and store in array
!                    normal
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     Fill array surfnodes
!     make array normal a permanent array
!     compute starting addresses
!     Compute the normals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmshnorm3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmshnorm3d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill array surfnodes

      call sepfillsurnods

!     --- make array normal a permanent array
!         the length of normal is 3 times the length of (surfnodes-2*nsurfs)

      length = size ( surfnodes )-2*nsurfs
      if ( km(iseqkm)%kmesh(61)>0 ) deallocate(km(iseqkm)%normal)
      allocate(km(iseqkm)%normal(3,length))
      km(iseqkm)%kmesh(61) = length
      normal => km(iseqkm)%normal

!     --- Compute the normals

      call sepmshnors0

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( normal, length, 3, 'normals' )

      end if
1000  call erclos ( 'sepmshnorm3d' )
      end subroutine sepmshnorm3d
