      subroutine meshwrpar
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-01-2016 New parameter list
!        version  1.1  date 02-11-2013 Use iref10
!        version  1.0  date 09-05-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write mesh to file in case of parallel mesh
!     This means that a new global mesh is written to meshoutput
!     and that the files meshoutput_par.xxx are rewritten
! **********************************************************************
!
!                       KEYWORDS
!
!     write_mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulemach
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenname, ihelp(9)
      character (len=100) name10sav

!     ihelp          integer work array, see msh059
!     lenname        Length of name
!     name10sav      Help variable to store value of name10
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIL         Open files
!     MESHWRPAR1     Write information of refined mesh to file
!                    meshoutput_par.000
!     MESHWRPAR2     Write arrays transinfo, blockinfo and mapping to node 1
!     MESHWRPAR3     Write global array kmesh to meshoutput in case of
!                    refinement. Parallel case
!     SEPMSH059      Write information of the mesh to the file indicated by iref
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     write new meshoutput_par.xxx
!     if ( nodenr==1 ) then
!        write new meshoutput_par.000
!        create  new kmesh
!        write new meshoutput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'meshwrpar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from meshwrpar'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check if name ends on _par.xxx

      lenname = index(name10,' ')-1
      if ( name10(lenname-7:lenname-3)=='_par.' ) then

!     --- name of file ends on _par.xxx, remove end positions

         lenname = lenname-8

      else

!     --- name of file does not end on _par.xxx, add end positions

         write(name10(lenname+1:lenname+8),100) inodenr
100      format('_par.',i3.3)

      end if  ! ( name10(lenname-7:lenname-3)=='_par.' )

      call inifil(10)      ! open file

!     --- write new meshoutput_par.xxx

      ihelp = 0
      call sepmsh059 ( iref10, ihelp )

      close(abs(iref10))

      if ( inodenr==1 ) then

!     --- Node number 1
!         write new meshoutput_par.000

         call inifil(10)      ! open file

         call meshwrpar1

         close(24)

!        --- create  new kmesh
!            write new meshoutput

         name10sav = name10             ! save name10
         name10 = ' '
         name10 = name10sav(1:lenname)  ! reduce name10

         if ( debug ) then
            write(irefwr,1) 'lenname', lenname
            write(irefwr,*) 'name10', name10
         end if  ! ( debug )

         call inifil(10)      ! open file

         call meshwrpar3

         close(24)

         name10 = name10sav  ! reset name10

      else

!     --- Other node number, write information to block 1

         call meshwrpar2

      end if  ! ( inodenr==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End meshwrpar'

      end if  ! ( debug )
      call erclos ( 'meshwrpar' )

      end
