      subroutine meshwrpar2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-01-2016 New parameter list
!        version  1.0  date 15-05-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write arrays transinfo, blockinfo and mapping to node 1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lentransinfo, icontext, lenblockinfo
      integer, dimension(:), pointer :: send_buffer
      integer, save :: send_buffer_size

!     icontext       Tag for sending and receiving, is used as check
!     lenblockinfo   Length of array blockinfo
!     lentransinfo   Length of array transinfo
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'meshwrpar2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from meshwrpar2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


!     --- Compute starting addresses and length

      lentransinfo = size(transinfo)
      lenblockinfo = size(blockinfo)

      send_buffer_size = 4*(lentransinfo+lenblockinfo+npoint)

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, 1 )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

!     --- write to block 1

      icontext = 31
      call inibsendmpi ( transinfo, lentransinfo, SPINT, 1, icontext )

      icontext = 32
      call inibsendmpi ( blockinfo, lenblockinfo, SPINT, 1, icontext )
      icontext = 33
      call inibsendmpi ( mappingparcp, npoint, SPINT, 1, icontext )

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End meshwrpar2'

      end if  ! ( debug )
      call erclos ( 'meshwrpar2' )

      end
