      subroutine msh034sub ( iwork, icurv, jcurv, index, kmeshm, &
                             istar3, kbndpt, i, ngen )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-08-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array iwork for the positions corresponding to curves
!     with the negative values of the corresponding positions in kmeshm
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), icurv, jcurv, index, kmeshm(*), istar3, &
              kbndpt(*), i, ngen

!     i              i    Counting variable with respect to local curves
!     icurv          i    Curve number with direction
!     index         i/o   Position in array kbndpt
!     istar3         i    Starting address of KMESHM part 3 in KMESH part M
!     iwork         i/o   integer work array
!     jcurv          i    Absolute value of curve number
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     ngen           i    Number of separate definining curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k1, k2, ih, idirec, nodp, k, ifirst, ilast
      save ilast

!     idirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     ifirst         First node in boundary part
!     ih             Help variable to store a constant temporarily
!     ilast          Last node in boundary part
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh034sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh034sub'
         write(irefwr,1) 'icurv, jcurv, index, i, ngen', &
                          icurv, jcurv, index, i, ngen
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      k1 = kmeshm(4+jcurv)
      k2 = kmeshm(5+jcurv)-1
      if ( icurv>0 ) then
         ifirst = kmeshm(istar3+k1)
      else
         ifirst = kmeshm(istar3+k2)
      end if

      if ( ngen==1 .and. i>1 ) then
         if ( ifirst/=ilast ) index = index+1
      end if

      if ( icurv>0 ) then
         ilast = kmeshm(istar3+k2)
      else
         ilast = kmeshm(istar3+k1)
      end if
      if ( debug ) then
         write(irefwr,1) 'k1, k2, ifirst, ilast', k1, k2, ifirst, ilast
         call prinin ( kbndpt(index), k2-k1+1, 'kbndpt' )
      end if  ! ( debug )

      if ( icurv>0 ) then

!     --- icurv>0   copy forwards

         ih = index-k1
         idirec = 1

      else

!     --- icurv<0   copy backwards

         ih = index+k2
         idirec = -1

      end if
      do k = k1, k2
         nodp = kbndpt(ih+k*idirec)
         if ( iwork(nodp)==0 ) iwork(nodp) = -k-istar3
      end do
      index = index+k2-k1

1000  call erclos ( 'msh034sub' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh034sub'

      end if  ! ( debug )

      end
