      subroutine msh035sub ( j, kmeshl, kmeshm, kmeshn, kbndpt, mapar, &
                             npoint, istart, kstart, isurnr, ivolnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-03-2009 Update mapar in case of userpoint (ws)
!        version  1.0  date 23-07-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array mapar and the arrays kmeshn, kmeshm, kmeshl for
!     one node in surface corresponding to volume
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer j, kmeshl(*), kmeshn(*), kbndpt(*), istart, kstart, &
              mapar(*), npoint, isurnr, ivolnr, kmeshm(*)

!     istart         i    Reference to nodal point is stored in kmeshn(istart+j)
!     isurnr         i    Surface sequence number
!     ivolnr         i    Volume sequence number
!     j              i    See istart, kstart (counting variable in surface
!                         element)
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary (local numbering)
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshn        i/o   KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kstart         i    Reference to nodal point is stored in kbndpt(kstart+j)
!     mapar         i/o   mapping array: contains the mapping from local to
!                         global numbering of the nodal points
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jnodp, knodp, nodp

!     jnodp          Nodal point sequence number
!     knodp          Nodal point sequence number
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH036         fill array mapar position iknpt with jknpt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh035sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh035sub'
         write(irefwr,1) 'npoint, istart', npoint, istart
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jnodp = kmeshn(istart+j)
      knodp = kbndpt(kstart+j)
      if ( debug ) &
         write(irefwr,1) 'j, jnodp, knodp, istart, kstart', &
                          j, jnodp, knodp, istart, kstart

      if ( jnodp<-1000000000 ) then

!     --- jnodp<-1000000000, hence user point

         nodp = kmeshl(-jnodp-1000000000)
         if ( debug ) &
             write(irefwr,1) 'kmeshl: nodp', nodp, -jnodp-100000000

         if ( nodp>0 ) then

!        --- nodp>0, fill in kmeshn

            kmeshn(istart+j) = nodp
            mapar(knodp)     = nodp

         else

!        --- nodp = 0   ( set mapar equal to - position in
!                         kmeshn - npoint )

            if ( mapar(knodp)==0 ) mapar(knodp) = -npoint-istart-j

            if ( debug ) write(irefwr,1) 'mapar(knodp)', mapar(knodp)

         end if  ! ( nodp>0 )

      else if ( jnodp<0 ) then

!     --- negative value in kmeshn: debug

         nodp = kmeshm(-jnodp)
         if ( debug ) write(irefwr,1) 'nodp', nodp

         if ( nodp>0 ) then

!        --- nodp>0   fill in mapar

            call msh036 ( mapar, knodp, nodp, npoint, &
                          isurnr, ivolnr )
            kmeshn(istart+j) = nodp

         else if ( nodp<0 ) then

!        --- nodp<0   debug in kmeshl

            nodp = kmeshl(-nodp)
            if ( debug ) write(irefwr,1) 'kmeshl: nodp', nodp

            if ( nodp>0 ) then

               kmeshm(-jnodp) = nodp
               call msh036 ( mapar, knodp, nodp, npoint, &
                             isurnr, ivolnr )
               kmeshn(istart+j) = nodp

            else

!           --- nodp = 0   ( set mapar equal to - position in
!                            kmeshn - npoint )

               if ( mapar(knodp)==0 ) mapar(knodp) = -npoint-istart-j
               if ( debug ) write(irefwr,1) 'mapar(knodp)', &
                  mapar(knodp)

            end if

         else

!        --- nodp = 0   ( set mapar equal to - position in
!                         kmeshn - npoint )

            if ( mapar(knodp)==0 ) mapar(knodp) = -npoint-istart-j

         end if  ! ( jnodp<-1000000000 )

         if ( debug )  write(irefwr,1) 'nodp', nodp

      else if ( jnodp>0 ) then

!     --- jnodp>0

         jnodp = kmeshn(istart+j)
         call msh036 ( mapar, knodp, jnodp, npoint, isurnr, ivolnr )

      else

!     --- jnodp = 0   ( set mapar equal to - position in
!                      kmeshn - npoint )

         if ( mapar(knodp)==0 ) mapar(knodp) = -npoint-istart-j

      end if  !  ( jnodp<-1000000000 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh035sub'

      end if  ! ( debug )
      call erclos ( 'msh035sub' )

      end
