      subroutine msh044surf ( kmeshm, kmeshl, kmeshn, nsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill negative values in kmeshm and kmeshn if possible
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), kmeshl(*), kmeshn(*), nsurfs

!     kmeshl         i    KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshn        i/o   KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, nodp, jnodp

!     j              Counting variable
!     jnodp          Nodal point sequence number
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh044surf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh044surf'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill kmeshm

      do j = kmeshm(2), kmeshm(3)-1
         nodp=kmeshm(j)
         if (nodp<0) then
            jnodp=kmeshl(-nodp)
            if(jnodp>0) kmeshm(j)=jnodp
         end if
      end do

!     --- Fill kmeshn
!         First compute length of second part

      length = 2*nsurfs
      do j = 1, nsurfs
         length = length+kmeshn(j)*kmeshn(j+nsurfs)
      end do  ! i = 1, nsurfs

      do j = 2*nsurfs+1, length
         nodp = kmeshn(j)

         if ( nodp<0 ) then

!        --- nodp<0, check if kmeshm has been filled

            jnodp = kmeshm(nodp)
            if ( jnodp>0 ) kmeshn(j) = jnodp

         end if  ! ( nodp<0 )

      end do  ! i = 2*nsurfs+1, length

1000  call erclos ( 'msh044surf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh044surf'

      end if

      end
