      subroutine msh058interf ( iinput, iinp, ielgeg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 10-03-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part f for interface elements
!     Adapt iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     interface_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, ielgeg(*)

!     ielgeg         i    array with number of nodes for each element shape
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array for subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurvs, isurfs, ielgrp, ifirst, ilast

!     i              Counting variable
!     icurvs         Number of parts where curve interface elements are given
!     ielgrp         element group number
!     ifirst         First surface
!     ilast          Second surface
!     isurfs         Number of parts where surface interface elements are given
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH058SURFS    Find shape number and number of points of element of a
!                    series of surfaces
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     323   element group number for connected element is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh058interf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh058interf'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iinp = iinp11
      nelgrp = nelgrp+1
      icurvs = iinput(iinp11+1)
      isurfs = iinput(iinp11+2)
      iinp = iinp+3
      jnodp = 0

      do i = 1, icurvs
         ielgrp = iinput(iinp)
         ishape = iinput(iinp+1)
         if ( ielgrp<nelgrp ) then
            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errsub ( 323, 2, 0, 0)
         else
            if ( ielgrp/=nelgrp ) nelgrp = nelgrp+1
            if ( ielgrp/=nelgrp ) then
               call errint ( ielgrp, 1 )
               call errint ( nelgrp, 2 )
               call errsub ( 323, 2, 0, 0)
            end if
            inpelm = 2*(ishape+1)
            if ( inpelm/=npelm) jnodp = 0
            npelm = max(inpelm,npelm)
            kmeshf(ielgrp) = ishape
         end if
         iinp = iinp+4
      end do

      do i = 1, isurfs
         ielgrp = iinput(iinp)
         ifirst = iinput(iinp+2)
         ilast = iinput(iinp+3)
         if ( debug ) &
            write(irefwr,1) 'i, ielgrp, ishape, ifirst, ilast', &
                             i, ielgrp, iinput(iinp+1), ifirst, ilast
         call msh058surfs ( iinput, ielgeg, ifirst, ilast, ielgrp )
         if ( debug ) write(irefwr,1) 'inpelm, ishape', inpelm, ishape
         inpelm = 2*inpelm
         if ( ielgrp<nelgrp ) then
            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errsub ( 323, 2, 0, 0)
         else
            if ( ielgrp/=nelgrp ) nelgrp = nelgrp+1
            if ( ielgrp/=nelgrp ) then
               call errint ( ielgrp, 1 )
               call errint ( nelgrp, 2 )
               call errsub ( 323, 2, 0, 0)
            end if
         end if  ! ( ielgrp<nelgrp )
         npelm = max(inpelm,npelm)
         kmeshf(ielgrp) = 70+ishape
         iinput(iinp+1) = 70+ishape  ! adapted shape number
         if ( debug ) call prinin ( iinput(iinp), 4, 'iinput_loc' )
         iinp = iinp+4

      end do  ! i = 1, isurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh058interf'

      end if
      call erclos ( 'msh058interf' )

      end
