      subroutine msh058surfs ( iinput, ielgeg, ifirst, ilast, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 10-03-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find shape number and number of points of element of a series
!     of surfaces
!     Check if they are all the same
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     check
!     number_of_nodes
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ielgeg(*), ifirst, ilast, ielgrp

!     ielgeg         i    array with number of nodes for each element shape
!     ielgrp         i    element group number
!     ifirst         i    First surface number
!     iinput         i    integer input array for subroutine mesh
!     ilast          i    Second surface number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, jinp

!     j              Counting variable
!     jinp           last position used in array iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1584   shape numbers in case of mesh connect not equal for surfaces
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh058surfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh058surfs'

      end if
      if ( ierror/=0 ) go to 1000

      jinp   = iinp3
      do j = 1, ifirst-1
         isurf = iinput(jinp)
         call msh049 ( iinput, jinp, isurf )
      end do
      ishape = iinput(jinp+1)
      inpelm = ielgeg(ishape)
      jinp   = iinp3
      do j = 1, ilast-1
         isurf = iinput(jinp)
         call msh049 ( iinput, jinp, isurf )
      end do
      if ( ishape/=iinput(jinp+1) ) then
         call errint ( ifirst, 1 )
         call errint ( ishape, 2 )
         call errint ( ilast, 3 )
         call errint ( iinput(jinp+1), 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1584, 5, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh058surfs'

      end if
      call erclos ( 'msh058surfs' )

      end
