      subroutine msh075surfs1 ( kmeshl, kmeshm, kmeshn, inodp, istart, &
                                inelem, inpelm, kmeshcloc, npoloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmeshn with new node numbers in case of surface interface elements
!     Raise inodp if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshl(*), kmeshm(*), kmeshn(*), inodp, istart, inelem, &
              inpelm, kmeshcloc(*), npoloc

!     inelem         i    Number of elements in surface
!     inodp         i/o   Next nodal point number
!     inpelm         i    number of points per element in surface
!     istart         i    First position of surface in kmeshn
!     kmeshcloc      i    Local array kmeshc containing the node numbers of one
!                         surface (local numbering)
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshn        i/o   KMESH part n, contains nodal point numbers of the
!                         surfaces
!     npoloc         i    Number of nodes in surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inode, mnode, lnode, jnode, inodeloc, iwork(npoloc)

!     inode          Node sequence number
!     inodeloc       Local node number
!     iwork          integer work array of length npoint
!                    If a local node has been used it gets the
!                    corresponding global node number, otherwise 0
!     j              Counting variable
!     jnode          Node sequence number
!     lnode          Node sequence number as stored in kmeshl
!     mnode          Node sequence number as stored in kmeshm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh075surfs1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh075surfs1'
         write(irefwr,1) 'inelem, inpelm, istart', &
                          inelem, inpelm, istart
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork (1:npoloc)

      do j = 1, npoloc
         iwork(j) = 0
      end do  ! j = 1, npoloc

!     --- Loop over all nodes in surface

      do j = 1, inelem*inpelm

         inode = kmeshn(istart+j)  ! node in surface
         inodeloc = kmeshcloc(j)   ! local node number
         if ( debug ) &
            write(irefwr,1) 'j, inode, inodeloc', j, inode, inodeloc

         if ( iwork(inodeloc)>0 ) then

!        --- Node has already been filled

            jnode = iwork(inodeloc)
            kmeshn(istart+j) = jnode
            if ( inode<0 ) then

!           --- inode<0, node refers to kmeshm
!               fill references in kmeshm and kmeshl

               mnode = kmeshm(-inode)
               kmeshm(-inode) = jnode
               if ( mnode<0 ) kmeshl(-mnode) = jnode

            end if  ! ( inode<0 )

         else

!        --- Node has already been filled

            if ( inode<0 ) then

!           --- inode<0, node refers to kmeshm

               mnode = kmeshm(-inode)
               if ( debug ) write(irefwr,1) 'mnode', mnode
               if ( mnode<0 ) then

!              --- kmeshm(-inode)<0, node refers to kmeshl

                  lnode = kmeshl(-mnode)
                  if ( debug ) write(irefwr,1) 'lnode', lnode
                  if ( lnode==0 ) then

!                 --- kmeshl(inode) = 0, new node

                     kmeshl(-mnode) = inodp
                     mnode = inodp
                     inodp = inodp+1

                  else

!                 --- kmeshl(inode)>0, old node

                     mnode = kmeshl(-mnode)

                  end if  ! ( kmeshl(inode)==0 )
                  kmeshm(-inode) = mnode

               else if ( mnode==0 ) then

!              --- kmeshm(-inode) = 0, new node

                  kmeshm(-inode) = inodp
                  mnode = inodp
                  inodp = inodp+1

               end if  ! ( kmeshm(-inode)<0 )

               inode = mnode

            else if ( inode==0 ) then

!           --- inode = 0, new node

               inode = inodp
               inodp = inodp+1

            end if  ! ( inode<0 ) then
            kmeshn(istart+j) = inode
            iwork(inodeloc) = inode

         end if  ! ( iwork(inodeloc)>0 ) then

      end do  ! j = 1, inelem

1000  call erclos ( 'msh075surfs1' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshn(istart+1), inelem, inpelm, 'kmeshn' )
         write(irefwr,*) 'End msh075surfs1'

      end if

      end
