      subroutine mshadapcent14 ( coor, kmeshc, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of 3d mesh once the mesh has been deformed
!     Goal is to keep mid-side points ate the centre of the sides
!     and centre points in the centroid
!     One element of shape 14 only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(27), imark(*)
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     imark         i/o   Indicates if nodes may be moved (0) or not (1)
!     kmeshc         i    contains the nodal point number
!                         corresponding to the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapcent14' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapcent14'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First put all nodes on the mid sides of the edges in the centre

      if ( imark(kmeshc(2))==0 ) coor(1:3,kmeshc(2)) = &
         0.5d0*(coor(1:3,kmeshc(1))+coor(1:3,kmeshc(3)))
      if ( imark(kmeshc(6))==0 ) coor(1:3,kmeshc(6)) = &
         0.5d0*(coor(1:3,kmeshc(3))+coor(1:3,kmeshc(9)))
      if ( imark(kmeshc(8))==0 ) coor(1:3,kmeshc(8)) = &
         0.5d0*(coor(1:3,kmeshc(7))+coor(1:3,kmeshc(9)))
      if ( imark(kmeshc(4))==0 ) coor(1:3,kmeshc(4)) = &
         0.5d0*(coor(1:3,kmeshc(7))+coor(1:3,kmeshc(1)))

      if ( imark(kmeshc(20))==0 ) coor(1:3,kmeshc(20)) = &
         0.5d0*(coor(1:3,kmeshc(19))+coor(1:3,kmeshc(21)))
      if ( imark(kmeshc(24))==0 ) coor(1:3,kmeshc(24)) = &
         0.5d0*(coor(1:3,kmeshc(21))+coor(1:3,kmeshc(27)))
      if ( imark(kmeshc(26))==0 ) coor(1:3,kmeshc(26)) = &
         0.5d0*(coor(1:3,kmeshc(25))+coor(1:3,kmeshc(27)))
      if ( imark(kmeshc(22))==0 ) coor(1:3,kmeshc(22)) = &
         0.5d0*(coor(1:3,kmeshc(25))+coor(1:3,kmeshc(19)))

      if ( imark(kmeshc(10))==0 ) coor(1:3,kmeshc(10)) = &
         0.5d0*(coor(1:3,kmeshc(19))+coor(1:3,kmeshc(1)))
      if ( imark(kmeshc(12))==0 ) coor(1:3,kmeshc(12)) = &
         0.5d0*(coor(1:3,kmeshc(21))+coor(1:3,kmeshc(3)))
      if ( imark(kmeshc(18))==0 ) coor(1:3,kmeshc(18)) = &
         0.5d0*(coor(1:3,kmeshc(9))+coor(1:3,kmeshc(27)))
      if ( imark(kmeshc(16))==0 ) coor(1:3,kmeshc(16)) = &
         0.5d0*(coor(1:3,kmeshc(25))+coor(1:3,kmeshc(7)))

!     --- Next centers of the faces

      if ( imark(kmeshc(5))==0 ) coor(1:3,kmeshc(5)) = &
         0.25d0*(coor(1:3,kmeshc(2))+coor(1:3,kmeshc(8))+ &
                 coor(1:3,kmeshc(4))+coor(1:3,kmeshc(6)))
      if ( imark(kmeshc(23))==0 ) coor(1:3,kmeshc(23)) = &
         0.25d0*(coor(1:3,kmeshc(20))+coor(1:3,kmeshc(26))+ &
                 coor(1:3,kmeshc(22))+coor(1:3,kmeshc(24)))
      if ( imark(kmeshc(11))==0 ) coor(1:3,kmeshc(11)) = &
         0.25d0*(coor(1:3,kmeshc(10))+coor(1:3,kmeshc(12))+ &
                 coor(1:3,kmeshc(20))+coor(1:3,kmeshc(2)))
      if ( imark(kmeshc(15))==0 ) coor(1:3,kmeshc(15)) = &
         0.25d0*(coor(1:3,kmeshc(6))+coor(1:3,kmeshc(24))+ &
                 coor(1:3,kmeshc(12))+coor(1:3,kmeshc(18)))
      if ( imark(kmeshc(17))==0 ) coor(1:3,kmeshc(17)) = &
         0.25d0*(coor(1:3,kmeshc(16))+coor(1:3,kmeshc(18))+ &
                 coor(1:3,kmeshc(8))+coor(1:3,kmeshc(26)))
      if ( imark(kmeshc(13))==0 ) coor(1:3,kmeshc(13)) = &
         0.25d0*(coor(1:3,kmeshc(4))+coor(1:3,kmeshc(22))+ &
                 coor(1:3,kmeshc(10))+coor(1:3,kmeshc(16)))

!     --- Finally centroid

      if ( imark(kmeshc(14))==0 ) coor(1:3,kmeshc(14)) = &
         0.25d0*(coor(1:3,kmeshc(5))+coor(1:3,kmeshc(23))+ &
                 coor(1:3,kmeshc(11))+coor(1:3,kmeshc(17)))

!     --- Mark nodes

      imark(kmeshc) = 1

1000  call erclos ( 'mshadapcent14' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapcent14'

      end if  ! ( debug )

      end
