      subroutine mshadapcent17 ( coor, kmeshc, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of 3d mesh once the mesh has been deformed
!     Goal is to keep mid-side points ate the centre of the sides
!     and centre points in the centroid
!     One element of shape 17 only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(9), imark(*)
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     imark         i/o   Indicates if nodes may be moved (0) or not (1)
!     kmeshc         i    contains the nodal point number
!                         corresponding to the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapcent17' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapcent17'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First put all nodes on the mid sides of the edges in the centre

      coor(1:3,kmeshc(9)) = 0.125d0 * ( coor(1:3,kmeshc(1)) + &
                                        coor(1:3,kmeshc(2)) + &
                                        coor(1:3,kmeshc(3)) + &
                                        coor(1:3,kmeshc(4)) + &
                                        coor(1:3,kmeshc(5)) + &
                                        coor(1:3,kmeshc(6)) + &
                                        coor(1:3,kmeshc(7)) + &
                                        coor(1:3,kmeshc(8)) )

!     --- Mark nodes

      imark(kmeshc) = 1

1000  call erclos ( 'mshadapcent17' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapcent17'

      end if  ! ( debug )

      end
