      subroutine mshbackgrid2d ( nx, ny, ox, oy, dx, dy, coor, &
                                 kmeshc, kmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create mesh for background rectangular grid in R^2
!     Arrays kmesh, kmeshc and coor are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, kmeshc(4,(nx-1)*(ny-1)), kmeshm(*)
      double precision coor(2,nx,ny), ox, oy, dx, dy

!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dx             i    step size in x-direction
!     dy             i    step size in y-direction
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshm         o    KMESH part m, contains nodal point numbers of the
!                         curves
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     ox             i    x-coordinate of origin
!     oy             i    y-coordinate of origin
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielem, jshift, istart

!     i              Counting variable
!     ielem          Element number
!     istart         First position used at a part
!     j              Counting variable
!     jshift         shift in j-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHRECT_GRID   Create coordinates for rectangular grid in R^2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill coordinates
!     Fill kmeshc
!     Fill kmeshm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshbackgrid2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshbackgrid2d'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Fill coordinates

      call mshrect_grid ( coor, nx, ny, ox, oy, dx, dy)

!     --- Fill kmeshc

      ielem = 0
      jshift = 0
      do j = 1, ny-1  ! loop over rows

         do i = 1, nx-1  ! loop over columns

            ielem = ielem+1
            kmeshc(1,ielem) = i+jshift
            kmeshc(2,ielem) = i+1+jshift
            kmeshc(3,ielem) = i+nx+1+jshift
            kmeshc(4,ielem) = i+nx+jshift

         end do  ! i = 1, nx-1

         jshift = jshift+nx

      end do  ! j = 1, ny-1

!     --- Fill kmeshm

      kmeshm(1) = 0             ! ninner
      kmeshm(2) = 7             ! start part 3
      kmeshm(3) = 2*(nx+ny)+4   ! start part 4
      kmeshm(4) = 0             ! length part 4
      kmeshm(5) = 1             ! start of first curve
      kmeshm(6) = 2*(nx+ny)-2   ! end of first curve + 1

!     --- Fill first part of boundary

      istart = 6
      do i = 1, nx
         kmeshm(istart+i) = i
      end do  ! i = 1, nx
      istart = istart+nx-1

!     --- Fill second part of boundary

      do i = 1, ny
         kmeshm(istart+i) = i*nx
      end do  ! i = 1, nx
      istart = istart+ny-1

!     --- Fill second part of boundary

      do i = 1, nx
         kmeshm(istart+i) = ny*nx-i+1
      end do  ! i = 1, nx
      istart = istart+nx-1

!     --- Fill fourth part of boundary

      do i = 1, ny
         kmeshm(istart+i) = 1+(ny-i)*nx
      end do  ! i = 1, nx

1000  call erclos ( 'mshbackgrid2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshbackgrid2d'

      end if

      end
