      subroutine mshcentrehalfc ( coor, curvenodes, numpoints, origin, &
                                  radius, isurnr, startpoints, &
                                  endpoints, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute centroid of a half circle as well as radius
!     Check if all nodes not on the mid line are positioned on circle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     plane
!     surface
!     centre
!     radius
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), origin(3), radius, eps
      integer curvenodes(*), numpoints, isurnr, startpoints(*), &
              endpoints(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curvenodes     i    Contains the nodes of the boundary curve
!     endpoints      i    Array containing the relative end points of the
!                         straight lines on the part of the circle (if any)
!                         relative with respect to array curvenodes
!     eps           i/o   Local accuracy
!     isurnr         i    Surface sequence number
!     numpoints      i    Number of points
!     origin         o    Contains the coordinates of the origin of the circle
!     radius         o    Radius of sphere or circle
!     startpoints    i    Array containing the relative start points of the
!                         straight lines on the part of the circle (if any)
!                         relative with respect to array curvenodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode1, inode2

!     i              Counting variable
!     inode1         First node
!     inode2         Second node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHECKCIRCLE Check if node is lying on circle defined by origin
!                    and radius
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute radius and centroid of circle as midpoint of straight line
!     Check if other nodes are on circle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcentrehalfc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcentrehalfc'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute radius and centroid of circle as midpoint of straight line

      inode1 = curvenodes(startpoints(1))
      inode2 = curvenodes(endpoints(1))
      if ( debug ) write(irefwr,*) 'nodes ', inode1, inode2
      do i = 1, 3
         origin(i) = 0.5d0*(coor(i,inode1)+coor(i,inode2))
      end do  ! i = 1, 3

      radius = 0d0  ! square of radius
      do i = 1, 3
         radius = radius + (coor(i,inode1)-origin(i))**2
      end do  ! i = 1, 3
      eps = radius*eps

      if ( debug ) &
         write(irefwr,*) 'origin, radius ', origin, sqrt(radius)

!     --- Check if other nodes are on circle

      if ( startpoints(1).lt.endpoints(1) ) then

!     --- startpoint < end point: standard situation

         do i = 1, startpoints(1)-1

            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )

         end do  ! i = 1, startpoints(1)-1

         do i = endpoints(1)+1, numpoints

            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )

         end do  ! i = endpoints(1)+1, numpoints

      else

!     --- startpoint > end point: special case

         do i = endpoints(1)+1, startpoints(1)-1

            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )

         end do  ! i = endpoints(1)+1, numpoints

      end if  ! ( startpoints(1).lt.endpoints(1) )

      radius = sqrt(radius)
      if ( debug ) write(irefwr,*) 'radius ', radius

1000  call erclos ( 'mshcentrehalfc' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcentrehalfc'

      end if

      end
