      subroutine mshcorrend ( coorol, coornw, inpada, rinada, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct end points of free boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), n
      double precision coorol(2,*), coornw(2,*), rinada(*)

!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     inpada         i    User input array for subroutine ADAPBOUN
!     n              i    number of points along the boundary
!     rinada         i    User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iadapt, idirec, iexcl1, iexcl2, iangl1, iangl2
      double precision angle1, angle2

!     angle1         Angle given at first point
!     angle2         Angle given at last point
!     iadapt         Indicates how the boundary must be adapted, see INPADA
!     iangl1         Indicates how the starting point must be adapted, see
!                    INPADA
!     iangl2         Indicates how the end point must be adapted, see
!                    INPADA
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     iexcl1         Parameter iexclude_1 as described in inpada(9)
!     iexcl2         Parameter iexclude_2 as described in inpada(10)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1476   The option angle has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcorrend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcorrend'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extract parameters from inpada and rinada

      iadapt = inpada(4)
      idirec = inpada(5)
      iexcl1 = inpada(9)
      iexcl2 = inpada(10)
      iangl1 = inpada(13)
      iangl2 = inpada(14)
      if ( iangl1==1 ) angle1 = rinada(3)
      if ( iangl2==1 ) angle2 = rinada(4)

!     --- Adapt end points as function of iexcl1/2

      if ( iadapt==3 .and. idirec>0 .and. iexcl1/=3 ) then
         coornw(1,1) = coornw(1,1)*2 - coorol(1,1)
         coornw(2,1) = coornw(2,1)*2 - coorol(2,1)
      else if ( iadapt==3 .and. idirec<0 .and. iexcl2/=3 ) then
         coornw(1,n) = coornw(1,n)*2 - coorol(1,n)
         coornw(2,n) = coornw(2,n)*2 - coorol(2,n)
      end if
      if ( iexcl1==3 ) then
         coornw(1,1) = coorol(1,1)
         coornw(2,1) = coorol(2,1)
      else if ( iexcl1==2 ) then
         coornw(2,1) = coorol(2,1)
      else if ( iexcl1==1 ) then
         coornw(1,1) = coorol(1,1)
      end if
      if ( iexcl2==3 ) then
         coornw(1,n) = coorol(1,n)
         coornw(2,n) = coorol(2,n)
      else if ( iexcl2==2 ) then
         coornw(2,n) = coorol(2,n)
      else if ( iexcl2==1 ) then
         coornw(1,n) = coorol(1,n)
      end if

!     --- check if angles have been prescribed

      if ( iangl1==1 ) then

!     --- first point has prescribed angle

         if ( iexcl1==1 ) then
            if ( abs(angle1)<=epsmac .or. &
                 abs(angle1-180d0)<=epsmac ) then
               coornw(2,2)=(coornw(2,1)+coornw(2,2))*0.5d0
               coornw(2,1)=coornw(2,2)
            else if ( abs(angle1-90d0)<=epsmac) then
               coornw(1,2)=coornw(1,1)
            else
               call errchr ( 'adapboun', 1 )
               call errsub ( 1476, 0, 0, 1 )
            end if
         else if ( iexcl1==2 ) then
            if ( abs(angle1)<=epsmac .or. &
                 abs(angle1-180d0)<=epsmac ) then
               coornw(2,2)=coornw(2,1)
            else if ( abs(angle1-90d0)<=epsmac ) then
               coornw(1,2)=(coornw(1,1)+coornw(1,2))*0.5d0
               coornw(1,1)=coornw(1,2)
            else
               call errchr ( 'adapboun', 1 )
               call errsub ( 1476, 0, 0, 1 )
            end if
         else if ( iexcl1==3 ) then
            if ( abs(angle1)<=epsmac .or. &
                 abs(angle1-180d0)<=epsmac ) then
               coornw(2,2)=coornw(2,1)
            else if (abs(angle1-90d0)<=epsmac ) then
               coornw(1,2)=coornw(1,1)
            else
               call errchr ( 'adapboun', 1 )
               call errsub ( 1476, 0, 0, 1 )
            end if
         end if
      end if

      if ( iangl2==1 ) then

!     --- last point has prescribed angle

         if ( iexcl2==1 ) then
            if ( abs(angle2)<=epsmac .or. &
                 abs(angle2-180d0)<=epsmac ) then
               coornw(2,n-1)=(coornw(2,n)+coornw(2,n-1))*0.5d0
               coornw(2,n)=coornw(2,n-1)
            else if (abs(angle2-90d0)<=epsmac) then
               coornw(1,n-1)=coornw(1,n)
            else
               call errchr ( 'adapboun', 1 )
               call errsub ( 1476, 0, 0, 1 )
            end if
         else if ( iexcl2==2 ) then
            if ( abs(angle2)<=epsmac .or. &
                 abs(angle2-180d0)<=epsmac ) then
               coornw(2,n-1)=coornw(2,n)
            else if ( abs(angle2-90d0)<=epsmac ) then
               coornw(1,n-1)=(coornw(1,n)+coornw(1,n-1))*0.5d0
               coornw(1,n)=coornw(1,n-1)
            else
               call errchr ( 'adapboun', 1 )
               call errsub ( 1476, 0, 0, 1 )
            end if
         else if ( iexcl2==3 ) then
            if ( abs(angle2)<=epsmac .or. &
                 abs(angle2-180d0)<=epsmac ) then
               coornw(2,n-1)=coornw(2,n)
            else if (abs(angle2-90d0)<=epsmac) then
               coornw(1,n-1)=coornw(1,n)
            else
               call errchr ( 'adapboun', 1 )
               call errsub ( 1476, 0, 0, 1 )
            end if
         end if
      end if

1000  call erclos ( 'mshcorrend' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcorrend'

      end if  ! ( debug )

      end
