      subroutine mshcrtinterf ( iinput, ikelmc, inodpbef )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill interface elements in kmeshc
!     Adapt array coor if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
!     coordinate
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), ikelmc, inodpbef

!     iinput         i    integer input array containing information of the
!                         interface elements
!     ikelmc        i/o   last position used in kmesh part c
!     inodpbef       i    First nodal point to be used for extra points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinpsurf, iinp, isurnr1, isurnr2

!     i              Counting variable
!     iinp           last position used in array iinput
!     iinpsurf       Value of iinp at start of surfaces
!     isurnr1        Surface sequence number
!     isurnr2        Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH076         fill array kmeshc for interface elements
!     MSHINTERFCOOR  Fill the new coordinates created by interface elements
!                    into array coor
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrtinterf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrtinterf'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill kmeshc and (a part of) coor

      call msh076 ( iinput, ikelmc, inodpbef, iinpsurf )

      if ( iinput(3)>0 .and. inodpbef<=npoint ) then

!     --- isurfs>0 and inodpbef<=npoint, hence possibly new coordinates
!         created in surfaces
!         to get the coordinates we have to look in the original
!         surfaces as stored in kmeshh
!         This means that we have to use memory management subroutines
!         Loop over the surfaces

         iinp = iinpsurf
         do i = 1, iinput(3)

            isurnr1 = iinput(iinp+2)    ! first surface number
            isurnr2 = iinput(iinp+3)    ! second surface number

            if ( debug ) then
               write(irefwr,1) 'inodpbef, isurnr1, isurnr2', &
                                inodpbef, isurnr1, isurnr2
               call prinin1 ( kmhsur(isurnr1)%kmeshcsur, &
                              kmeshn(isurnr1), kmeshn(isurnr1+nsurfs), &
                              'kmeshc_loc1' )
               call prinin1 ( kmhsur(isurnr2)%kmeshcsur, &
                              kmeshn(isurnr2), kmeshn(isurnr2+nsurfs), &
                              'kmeshc_loc2' )
            end if  ! ( debug )

!           --- Fill coordinates

            call mshinterfcoor ( inodpbef, kmhsur(isurnr1)%kmeshcsur, &
                                 kmhsur(isurnr2)%kmeshcsur, &
                                 kmhsur(isurnr1)%coorsur, &
                                 kmhsur(isurnr2)%coorsur, isurnr1, isurnr2 )

            iinp = iinp+4  ! raise iinp

         end do  ! i = 1, iinput(3)

      end if  ! ( iinput(3)>0 .and. inodpbef<=npoint )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcrtinterf'

      end if
      call erclos ( 'mshcrtinterf' )

      end subroutine mshcrtinterf
