      subroutine mshcurvbox ( xbox, iwork, ipointcurv, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a box around all single curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iwork(*), ipointcurv(*), iinput(*)
      double precision xbox(ndim,ncurvs,2)

!     iinput         i    integer input array
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curves in the part of iinput referring
!                         to curves
!     iwork          o    integer work array of length ncurvs
!                         iwork(i) = -1: curve is empty
!                         iwork(i) = 0: curve is single
!                         iwork(i) = 1: curve is compound
!     xbox           o    Is used to store the minimum and maximum coordinates
!                         of all curves
!                         xbox(i,j,1)  minimum i^th coordinate of curve j
!                         xbox(i,j,2)  maximum i^th coordinate of curve j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, istart, nnodes
      double precision kcoor

!     i              Counting variable
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kcoor          k-th coordinate in point
!     nnodes         Number of nodes in curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize array xbox(1:ncurvs)
!     Clear array iwork(1:ncurvs)
!     Set iwork(i) = 1 for all compound curves
!     For all non-empty curves do
!        Compute min and max coordinates and store in xbox
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurvbox' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurvbox'
         call ertrace
         call prinin ( ipointcurv, ncurvs, 'ipointcurv' )
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize array xbox(1:ncurvs)

      do i = 1, ncurvs
         do k = 1, ndim
            xbox(k,i,1) = rinfin   ! minimum = infinity
            xbox(k,i,2) = -rinfin  ! maximum = -infinity
         end do  ! k = 1, ndim
      end do  ! i = 1, ncurvs

!     --- Clear array iwork(1:ncurvs)

      do i = 1, ncurvs

         if ( debug ) then

!        --- Debug information

            write(irefwr,100) 'i, ipointcurv(i), iinput(i) ', &
                               i, ipointcurv(i), iinput(ipointcurv(i))

         end if
         if ( iinput(ipointcurv(i))==14 ) then

!        --- Curve of splines, set to 1

            iwork(i) = 1

         else if ( iinput(ipointcurv(i))/=0 ) then

!        --- Non-empty curve, set to 0

            iwork(i) = 0

         else

!        --- empty curve, set to -1

            iwork(i) = -1

         end if  ! ( iinput(ipointcurv(icurnr))/=0 )

      end do  ! i = 1, ncurvs

!     --- Set iwork(i) = 1 for all compound curves

      if ( kelmq/=0 ) then

!     --- kelmq/=0, hence there are compound curves

         do i = 1, ncurvs

            if ( kmeshq(i+1)-kmeshq(i)>0 ) then

!           --- compound curve found, set iwork to 1

               iwork(i) = 1

            end if  ! ( kmeshq(i+1)-kmeshq(i)>0 )

         end do  ! i = 1, ncurvs

      end if  ! ( kelmq/=0 )
      if ( debug ) call prinin ( iwork, ncurvs, 'iwork' )

!     --- For all non-empty curves do
!            Compute min and max coordinates and store in xbox

      istart = 0    ! last address used in curves
      do i = 1, ncurvs

         nnodes = icurvsmsh(i)-istart  ! actual number of nodes
         if ( iwork(i)==0 ) then

!        --- Non-empty curve, fill coordinates

            do j = 1, nnodes
               do k = 1, ndim
                  kcoor = curvesmsh(k,istart+j)   ! k-th coordinate
                  xbox(k,i,1) = min(xbox(k,i,1),kcoor) ! min coordinate
                  xbox(k,i,2) = max(xbox(k,i,2),kcoor) ! max coordinate
               end do  ! k = 1, ndim
            end do  ! j = 1, nnodes

         end if  ! ( iwork(i))==0 )
         istart = icurvsmsh(i)  ! last address used in curves

      end do  ! i = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurvbox'

      end if
      call erclos ( 'mshcurvbox' )

      end
