      subroutine mshcurves41 ( kmeshmold, kmeshmtmp, ncurvsnew, ipoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-03-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Merge old array kmeshm with new part and create a new array kmeshm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmold(*), kmeshmtmp(*), ncurvsnew, ipoint(*)

!     ipoint         i    Array of length ncurvsnew containing the starting
!                         address of the next curve in kmeshmtmp
!                         So in ipoint(1) the number of nodes for the first new
!                         curve in kmeshmtmp plus one is stored etc.
!     kmeshmold      i    Old array kmeshm
!     kmeshmtmp      i    Temporary array kmeshm in which the nodes of the new
!                         curves are stored
!     ncurvsnew      i    Number of new curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurvsold, i, istart

!     i              Counting variable
!     istart         First position used at a part
!     ncurvsold      Old number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( kmeshm old exists ) then
!        Copy old array kmeshm
!        Copy new array kmeshm
!     else
!        Copy new array kmeshm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves41' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves41'
         write(irefwr,*) 'ncurvsold, ncurvsnew ', ncurvs, ncurvsnew
         if ( ncurvs>0 ) call pr0008 ( kmeshmold, ncurvs )
         call prinin ( ipoint, ncurvsnew, 'ipoint' )

      end if
      if ( ierror/=0 ) go to 1000

      ncurvsold = ncurvs        ! old number of curves
      if ( ncurvs>0 ) then

!     --- ncurvs > 0, old array kmeshm exists
!         Set first positions kmeshm

         ncurvsold = ncurvs        ! old number of curves
         ncurvs = ncurvsold+ncurvsnew ! new number of curves
         ninner = kmeshmold(1)        ! old number of inner boundaries
         kmeshm(1) = ninner+ncurvsnew ! new number of inner boundaries
         kmeshm(2) = 6+ncurvs         ! starting address part 2
         kmeshm(3) = kmeshmold(3)+ncurvsnew+ipoint(ncurvsnew)-1
                                      ! starting address part 3
         kmeshm(4) = ninner+ncurvsnew ! length part 4
         if ( debug ) write(irefwr,*) 'ncurvsold ', ncurvsold

!        --- Copy old array kmeshm
!            First part 2

         do i = 0, ncurvsold
            kmeshm(5+i) = kmeshmold(5+i)
         end do  ! i = 1, ncurvsold+1

!        --- Next part 3

         do i = 1, kmeshmold(5+ncurvsold)-1
            kmeshm(kmeshm(2)-1+i) = kmeshmold(kmeshmold(2)-1+i)
         end do  ! i = 1, kmeshm(5+ncurvs)-1

!        --- Finaly part 4

         do i = 1, kmeshmold(1)
            kmeshm(kmeshm(3)-1+i) = kmeshmold(kmeshmold(3)-1+i)
         end do  ! i = 1, kmeshm(1)

!        --- Copy new array kmeshm
!            First part 2

         istart = 5+ncurvsold
         do i = 1, ncurvsnew
            kmeshm(istart+i) = kmeshm(istart)+ipoint(i)-1
         end do

!        --- Next part 3

         istart = kmeshm(2)-1+kmeshmold(5+ncurvsold)-1
         do i = 1, ipoint(ncurvsnew)-1
            kmeshm(istart+i) = kmeshmtmp(i)
         end do  ! i = 1, kmeshm(5+ncurvs)-1

!        --- Finally part 4

         do i = 1, ncurvsnew
            kmeshm(kmeshm(3)+kmeshmold(1)-1+i) = ncurvsold+i
         end do  ! i = 1, kmeshm(1)

      else

!     --- Copy new array kmeshm

         ncurvs = ncurvsnew           ! new number of curves
         kmeshm(1) = ncurvsnew        ! new number of inner boundaries
         kmeshm(2) = 6+ncurvs         ! starting address part 2
         kmeshm(3) = 4+ncurvsnew+ipoint(ncurvsnew)
                                      ! starting address part 3
         kmeshm(4) = ncurvsnew        ! length part 4

!        --- Fill part 2

         kmeshm(5) = 1
         do i = 1, ncurvsnew
            kmeshm(5+i) = ipoint(i)
         end do

!        --- Fill part 3

         istart = kmeshm(2)-1
         do i = 1, ipoint(ncurvsnew)-1
            kmeshm(istart+i) = kmeshmtmp(i)
         end do  ! i = 1, kmeshm(5+ncurvs)-1

!        --- Finally part 4

         do i = 1, ncurvsnew
            kmeshm(kmeshm(3)-1+i) = i
         end do  ! i = 1, kmeshm(1)

      end if  ! ( kelmm>0 )
      ncurvs = ncurvsold+ncurvsnew

1000  if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'End mshcurves41'

      end if
      call erclos ( 'mshcurves41' )

      end
