      subroutine mshelemref ( elements, iedges, iseqkm2, kmeshcnew, &
                              coornew, lenkmscnew, intnodes, parentnodes, &
                              refmapold, refmapnew, ireflvl, refarray, &
                              nodeextrael, enthref, npoint_basis, &
                              enthwork, lnintnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 22-11-2011 Use mshfindlowsur
!        version  2.0  date 26-10-2005 extra parameter lnintnodes (JB)
!        version  1.3  date 18-10-2005 refarray integer
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Create a new topology consisting of the set kmeshc/g/t
!      in case of a local refinement
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer elements(*), iedges(*), iseqkm2, &
              kmeshcnew(*), lenkmscnew, &
              intnodes(*), parentnodes(2,*), refmapold(2,*), &
              refmapnew(2,*), ireflvl, refarray(*), nodeextrael(*), &
              npoint_basis, enthwork(*), lnintnodes
      double precision coornew(*)
      logical enthref

!     coornew        i    Array containing the new coordinates of the nodes
!     elements       i    Array of length levels(nlevels) (<=nelem)
!                         containing the reordered element sequence numbers
!                         So elements(1) contains the first element number to
!                         be used, elements(2) the second one and so on
!     enthref        i    logical, when true, enthalpy method is used
!     enthwork      i/o   work array to keep track of added nodes
!     iedges         i    Work array of size nedges
!     intnodes       i    integer array containing all nodes at the interface
!                         of an element group
!     ireflvl        i    Current level of refinement
!     iseqkm2        i    Sequence number with respect to km of second mesh
!     kmeshcnew      i    New kmesh part c
!     lenkmscnew     i    Length of array kmshcnew
!     lnintnodes     i    Declared length of array intnodes
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     npoint_basis   i    npoint of the basis mesh
!     parentnodes   i/o   Array with for all added nodes the node
!                         numbers of the parents
!     refarray       i    Reference array containing the required level
!                         of refinement per element of the basis mesh
!     refmapnew     i/o   Mapping array
!     refmapold      i    Mapping array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielemnew, inpelm, ikelmc, ikelmcnew, &
              ilow, i, isurnr, ielembot, ielemtop, ielgrp, &
              irelem, inelem2, edgenode(3,6), nelemnew, nedgenode, &
              ielemold, ielemlast, nextranode, ncursur

!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     i              Counting variable
!     ielem          Element number
!     ielembot       Element sequence number of element with lowest number in
!                    surface isurnr
!     ielemlast      Element number of latest considered basis
!     ielemnew       Sequence number of element in new mesh
!     ielemold       Previous value of ielemnew
!     ielemtop       Largest element number in group
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmcnew      Starting address of next element in kmeshcnew
!     ilow           elements are numbered from element number ilow
!     inelem2        Number of elements in a specific element group of
!                    new mesh
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     isurnr         Surface sequence number
!     ncursur        ncurvs, nsurfs or nvolms depending on dimension
!     nedgenode      Length of first index of edgenode
!     nelemnew       Number of elements in new mesh
!     nextranode     number of extra nodes added
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHELEMREF1D   Subdivide an 1D element if necessary into two elements
!     MSHELEMREF2D   Subdivide a triangle if necessary into two, three or
!                    four subtriangles
!     MSHOBSTMS3D01  Subdivide each tetrahedron into the right number of
!                    subtetrahedrons
!     MSHREFEDGE     Fill array edgenode with information concenrning edges
!                    to be subdivided
!     MSHREFINTNODES Fill array intnodes
!     MSHREFMAP      Fill array refmap
!     MSHREFPARENTS  Fill array parentnodes
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPFINDLOWSUR  Find surface (curve) with lowest non-zero entry in kmesht
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For ielgrp := 1 (1) nelgrp do
!        inelem := number of elements in group ielgrp
!        inpelm := number of nodes per element in group ielgrp
!        For irelem := 1 (1) inelem do
!           ielem := ielem+1
!           ielemnew := ielemnew+1
!           if ( ielem is not intersected ) then
!              copy element from kmeshc to kmeshcnew
!           else ( ielem is intersected )
!              create new elements
!              copy these elements to kmeshcnew
!              Raise ielemnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshelemref' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshelemref'
         write(irefwr,*) 'enthref ', enthref
         call prinin ( elements, nelem, 'elements' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem = 0
      ielemnew = 0

      length = size(kmesht,2)
      if ( associated(km(iseqkm2)%kmesht) ) &
         deallocate ( km(iseqkm2)%kmesht )
      allocate ( km(iseqkm2)%kmesht(2,length) )

!     --- For ielgrp := 1 (1) nelgrp do

      select case ( ndim )
      case(1)
         ncursur = ncurvs
      case(2)
         ncursur = nsurfs
      case(3)
         ncursur = nvolms
      end select  ! case ( ndim )

      ikelmc = 0
      ikelmcnew = 0
      inelem2 = 0

!     --- Find curve/surface/volume with lowest positive element number
!         This is necessary to fill kmesh part t

      call sepfindlowsur ( ncursur, ielembot, ielemtop, isurnr )

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in group ielgrp
!         inpelm := number of nodes per element in group ielgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, inelem, ishape', &
                             ielgrp, inpelm, inelem, ishape

!        --- For irelem := 1 (1) inelem do

         if (enthref) then
            ielemlast = 0
            nextranode = 0
         end if

         do irelem = 1, inelem

!        --- ielem := ielem+1
!            ielemnew := ielemnew+1

            ielem = ielem+1
            if ( enthref ) ielemold = ielemnew
            ielemnew = ielemnew+1

            if ( ielem==ielembot ) then

!           --- Lowest element of surface, refill kmesht

               km(iseqkm2)%kmesht(1,isurnr) = ielemnew

            end if

!           --- if ( ielem is not intersected ) then
!               copy element from kmeshc to kmeshcnew

            if ( elements(ielem)==0 ) then

!           --- ielem is not intersected

               do i = 1, inpelm
                  kmeshcnew(ikelmcnew+i) = kmeshc(ikelmc+i)
               end do
               ikelmc = ikelmc+inpelm
               ikelmcnew = ikelmcnew+inpelm

            else

!           --- else ( ielem is intersected )
!               create new elements
!               copy these elements to kmeshcnew
!               Raise ielemnew

               if ( ishape/=1 .and. ishape/=3 .and. ishape/=11 ) then

!              --- shape number not 3 or 11, not yet implemented

                  call errint ( ishape, 1 )
                  call errsub ( 800, 1, 0, 0 )
                  go to 1000

               end if  ! ( ishape/=1 .and. ishape/=3 .and. ishape/=11 )

!              --- Fill array edgenode

               call mshrefedge ( nmeshd, ielem, edgenode, nmeshc, &
                                 iedges, nedgenode )

               if (debug) call prinin(edgenode, 3*nedgenode, 'edgenode')

               if ( enthref ) then

!              --- Fill array intnodes

                  call mshrefintnodes (intnodes, edgenode, ielem, &
                                       nedgenode, ireflvl, nodeextrael, &
                                       refmapold, ielemlast, &
                                       nextranode, enthwork, &
                                       npoint_basis, lnintnodes )

!                 --- Fill array parentnodes

                  call mshrefparents (parentnodes, edgenode, nedgenode, &
                                      npoint_basis)

                  if (debug) then
                     call prinin(intnodes, 48, 'intnodes')
                     call prinin(parentnodes, 39*2, 'parentnodes')
                  end if

               end if  ! ( enthref )

               if ( ndim==1 ) then

!              --- ndim = 1

                  call mshelemref1d ( elements(ielem), kmeshc(ikelmc+1), &
                                      kmeshcnew(ikelmcnew+1), edgenode, &
                                      ikelmcnew, ielemnew )

               else if ( ndim==2 ) then

!              --- ndim = 2

                  call mshelemref2d ( elements(ielem), kmeshc(ikelmc+1), &
                                      kmeshcnew(ikelmcnew+1), edgenode, &
                                      coornew, ikelmcnew, ielemnew )
               else

!              --- ndim = 3

                  call mshobstms3d01 ( kmeshc(ikelmc+1), &
                                       kmeshcnew(ikelmcnew+1), coornew, &
                                       elements(ielem), nelemnew, &
                                       edgenode, inpelm )
                  ielemnew = ielemnew+nelemnew-1
                  ikelmcnew = ikelmcnew+inpelm*nelemnew
                  if ( debug ) call prinin1 ( kmeshcnew(ikelmcnew+1), &
                     nelemnew, 4, 'tetnodenew')

               end if  ! ( ndim==2 ) then

               ikelmc = ikelmc+inpelm

            end if  ! ( elements(ielem)==0  ...
            if ( ielem==ielemtop ) then

!           --- Top element of surface, refill kmesht

               km(iseqkm2)%kmesht(2,isurnr) = ielemnew

!              --- Raise ielembot and ielemtop and recompute isurnr

               ilow = nelem
               do i = 1, ncursur
                  if ( km(iseqkm2)%kmesht(1,i)<ilow .and. &
                       km(iseqkm2)%kmesht(1,i)>ielemtop ) then

!                 --- new surface found

                     isurnr = i
                     ilow = km(iseqkm2)%kmesht(1,i)

                  end if
               end do
               ielembot = km(iseqkm2)%kmesht(1,isurnr)
               ielemtop = km(iseqkm2)%kmesht(2,isurnr)

            end if

            if ( enthref ) &
               call mshrefmap( ielemold, ielemnew, refmapold, refmapnew, &
                               refarray, ireflvl, ielem )

         end do
         if ( kelmg>0 ) then

!        --- Raise kmesh part g

            km(iseqkm2)%kmeshg(ielgrp) = ielemnew-inelem2
            inelem2 = ielemnew

         end if

      end do !end loop over element groups
      km(iseqkm2)%kmesh(9) = ielemnew
      lenkmscnew = ikelmcnew
      if (debug) then
         call prinin(refmapold, 18*2, 'refmapold')
         call prinin(refmapnew, 32*2, 'refmapnew')
      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshelemref'

      end if
      call erclos ( 'mshelemref' )

      end
