      subroutine mshelemref2d ( nextranodes, trianglenode, &
                                trianglenodenw, edgenode, coornew, &
                                ikelmcnew, ielemnew )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.1  date 03-03-2005 Extension with 3d
!        version  1.0  date 21-10-2004
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a triangle if necessary into two, three or four subtriangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     intersection
!     linear
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nextranodes, trianglenode(*), trianglenodenw(*), &
              edgenode(3,*), ikelmcnew, ielemnew
      double precision coornew(2,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     ielemnew      i/o   Sequence number of element in new mesh
!     ikelmcnew     i/o   Starting address of next element in kmeshcnew
!     nextranodes    i    Number of extra nodes in element
!     trianglenode   i    Array of nodal point numbers of the corners of
!                         the original triangle.
!     trianglenodenw o    Array of nodal point numbers of the corners of
!                         the new subtriangles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS2D04  Subdivide triangle into two triangles
!     MSHOBSTMS2D05  Subdivide triangle into three triangles
!     MSHOBSTMS2D06  Subdivide triangle into four triangles
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshelemref2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshelemref2d'
         write(irefwr,1) 'nextranodes', nextranodes
  1      format ( a, 1x, (10i6) )
         call prinin ( trianglenode, 3, 'trianglenode' )

      end if
      if ( ierror/=0 ) go to 1000
      if ( nextranodes==0 ) then

!     --- nextranodes=0
!         copy triangle

         do i = 1, 3
            trianglenodenw(i) = trianglenode(i)
         end do

      else if ( nextranodes==1 ) then

!     --- nextranodes=1
!         subdivide triangle into 2 triangles

          call mshobstms2d04 ( trianglenode, trianglenodenw, edgenode )

      else if ( nextranodes==2 ) then

!     --- nextranodes=2
!         subdivide triangle into 3 triangles

          call mshobstms2d05 ( trianglenode, trianglenodenw, edgenode, &
                               coornew, 3 )

      else

!     --- nextranodes=3
!         subdivide triangle into 4 triangles

         call mshobstms2d06 ( trianglenode, trianglenodenw, edgenode, &
                              coornew, 3 )

      end if

      ikelmcnew = ikelmcnew+(nextranodes+1)*3
      ielemnew = ielemnew+nextranodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshelemref2d'

      end if
      call erclos ( 'mshelemref2d' )

      end
