      subroutine mshfillkmeshs3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-01-2006 Correction in case of closed curves
!        version  1.0  date 18-10-2004
!
!   copyright (c) 2004-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorganize curves stored in kmeshs part 1 in order to get closed
!     curves if possible
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     curve
!     ordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imarkcurves(ncurvs), i, istart, &
              ncurves, ifirst, ilast, nopen, iopen, istar3, &
              icount, jcount, istorecurves(ncurvs), icurves(ncurvs)
      integer, allocatable :: imarknodes(:)

!     i              Counting variable
!     icount         Counter
!     icurves        array containing a series of curve numbers
!     ifirst         Position of first node in curve
!     ilast          Position of last node in curve
!     imarkcurves    Work array of length ncurves
!                    if>0, curve i is in boundary
!     imarknodes     Work array of length npoint
!                    In imarknodes(i) it is stored how many times node i
!                    is an initial or end point of a curve
!     iopen          Loop variable
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istorecurves   Work array of length ncurves
!                    Contains the curve numbers of the boundary
!     jcount         Counter
!     ncurves        Number of curves that must be adapted
!     nopen          Number of open boundaries
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFILLKMESHSC Store closed curves in kmeshs
!     MSHFILLKMESHSO Store open curves in kmeshs
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array imarkcurves, imarknodes
!     For isurnr = 1 (1) nsurfs do
!        ncurves := number of curves surrounding isurnr
!        Copy curves from kmeshs into istorecurves
!        Set array imarkcurves = 1 for all curves in isurnr
!        For all begin and end points of all curves raise imarknodes by 1
!        nopen := number of open curves; icount := 0
!        For iopen := 1 (1) nopen do
!           Start with open curve
!           Find all next curves, store in icurves
!           raise icount and mark curves
!        while ( icount<ncurves ) do
!           Store closed curves in kmeshs
!        Copy open curves into kmeshs part 1
!        Reset array imarkcurves
!        Reset array imarknodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshs3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshs3'
         call pr0008 ( kmeshm, ncurvs )
         call prinin ( kmeshs, nsurfs+1, 'kmeshs_1' )
100      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      istar3 = kmeshm(2)-1

      allocate ( imarknodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'imarknodes' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array imarkcurves, imarknodes

      do i = 1, ncurvs
         imarkcurves(i) = 0
      end do  ! i = 1, ncurvs

      do i = 1, npoint
         imarknodes(i) = 0
      end do  ! i = 1, npoint

!     --- For isurnr = 1 (1) nsurfs do

      do isurnr = 1, nsurfs

!     --- ncurves := number of curves surrounding isurnr

         istart = kmeshs(isurnr)   ! starting address of isurnr in kmeshs part 3
                                   ! minus 1
         ncurves = kmeshs(isurnr+1) - istart  ! number of curves surrounding
                                              ! isurnr
         if ( debug ) &
            write(irefwr,100) 'isurnr, ncurves, istart', &
                               isurnr, ncurves, istart
         if ( ncurves>0 ) then

!        --- Copy curves from kmeshs into istorecurves

            do i = 1, ncurves
               istorecurves(i) = kmeshs(istart+i)
            end do  ! i = 1, ncurves
            if ( debug ) &
               call prinin ( istorecurves, ncurves, 'istorecurves' )

!           --- Set array imarkcurves = 1 for all curves in isurnr

            do i = 1, ncurves
               imarkcurves(istorecurves(i)) = 1
            end do  ! i = 1, ncurves

!           --- For all begin and end points of all curves raise imarknodes by 1

            do i = 1, ncurves
               icurnr = istorecurves(i)    ! curve sequence number
               ifirst = kmeshm(4+icurnr)   ! position of first node
               ilast = kmeshm(5+icurnr)-1  ! position of last node
               imarknodes(kmeshm(ifirst+istar3)) = &
                  imarknodes(kmeshm(ifirst+istar3))+1
               imarknodes(kmeshm(ilast+istar3)) = &
                  imarknodes(kmeshm(ilast+istar3))+1

!              --- If a node is more than 2 times in the curves, sorting is not
!                  possible  and we skip this surface

               if ( imarknodes(kmeshm(ifirst+istar3))>2 ) go to 200
               if ( imarknodes(kmeshm(ilast+istar3))>2 ) go to 200

            end do  ! i = 1, ncurves
            if ( debug ) then
               call prinin ( imarkcurves, ncurvs, 'imarkcurves' )
               call prinin ( imarknodes, ncurvs, 'imarknodes' )
            end if  ! ( debug )

!           --- nopen := number of open curves; icount := 0

            nopen = 0
            do i = 1, ncurves
               icurnr = istorecurves(i)    ! curve sequence number
               ifirst = kmeshm(4+icurnr)   ! position of first node
               ilast = kmeshm(5+icurnr)-1  ! position of last node
               if ( imarknodes(kmeshm(ifirst+istar3))==1 ) &
                  nopen = nopen+1
               if ( imarknodes(kmeshm(ilast+istar3))==1 ) &
                  nopen = nopen+1
            end do  ! i = 1, ncurves
            nopen = nopen/2
            icount = 0
            if ( debug ) write(irefwr,*) 'nopen ', nopen

!           --- For iopen := 1 (1) nopen do

            do iopen = 1, nopen

!           --- Start with open curve
!               Find all next curves, store in icurves
!               raise icount and mark curves

               call mshfillkmeshso ( ncurves, istorecurves, imarkcurves, &
                                     kmeshm, imarknodes, icount, &
                                     icurves )
               if ( ierror/=0 ) go to 1000
               if ( debug ) then
                  write(irefwr,100) 'iopen, icount', iopen, icount
                  call prinin ( icurves, ncurves, 'icurves' )
               end if  ! ( debug )

            end do  ! iopen = 1, nopen

            jcount = 0

!           --- while ( icount<ncurves ) do

            do while ( icount+jcount<ncurves )

!           --- Store closed curves in kmeshs

               call mshfillkmeshsc ( ncurves, istorecurves, imarkcurves, &
                                     kmeshm, jcount, kmeshs(istart+1) )
               if ( ierror/=0 ) go to 1000
               if ( debug ) then
                  write(irefwr,100) 'jcount, icount', jcount, icount
               end if  ! ( debug )

            end do  ! while ( icount<ncurves )

!           --- Copy open curves into kmeshs part 3
!               Closed curves have already been filled in position 1 to jcount

            do i = 1, icount
               kmeshs(istart+i+jcount) = icurves(i)
            end do  ! i = 1, icount

!           --- Reset arrays imarknodes and  imarkcurves

200         do i = 1, ncurves
               icurnr = istorecurves(i)    ! curve sequence number
               ifirst = kmeshm(4+icurnr)   ! position of first node
               ilast = kmeshm(5+icurnr)-1  ! position of last node
               imarknodes(kmeshm(ifirst+istar3)) = 0
               imarknodes(kmeshm(ilast+istar3)) = 0
            end do  ! i = 1, ncurves

            do i = 1, ncurves
               imarkcurves(istorecurves(i)) = 0
            end do  ! i = 1, ncurves

         end if  ! ( ncurves>0 )

      end do  ! isurnr = 1, nsurfs

      deallocate ( imarknodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imarknodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillkmeshs3'

      end if
      call erclos ( 'mshfillkmeshs3' )

      end
