      subroutine mshfillkmeshsc ( ncurves, istorecurves, imarkcurves, &
                                  kmeshm, jcount, kmeshs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store closed curves in kmeshs
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     curve
!     ordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurves, istorecurves(*), imarkcurves(*), kmeshm(*), &
              jcount, kmeshs(*)

!     imarkcurves   i/o   Work array of length ncurves
!                         if > 0, curve i is in boundary and not yet used
!     istorecurves   i    Work array of length ncurves
!                         Contains the curve numbers of the boundary
!     jcount        i/o   Number of closed curves stored
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     ncurves        i    Number of curves that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, ifirst, ilast, nextnode, firstnode, istar3
      logical finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     firstnode      First node on curve
!     i              Counting variable
!     icurnr         Curve sequence number
!     ifirst         Position of first point of curve
!     ilast          Position of last point of curve
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     nextnode       next node to be treated
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2738   jcount > ncurves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshsc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshsc'
         call prinin ( istorecurves, ncurves, 'istorecurves' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000
      istar3 = kmeshm(2)-1

      do i = 1, ncurves

         icurnr = istorecurves(i)    ! curve sequence number

         if ( imarkcurves(icurnr).ne.0 ) then

!        --- New curve

            ifirst = kmeshm(4+icurnr)   ! position of first node
            ilast = kmeshm(5+icurnr)-1  ! position of last node
            nextnode = kmeshm(ilast+istar3)    ! next node in row
            go to 100

         end if  ! ( imarkcurves(icurnr).eq.1 )

      end do  ! i = 1, ncurves

100   imarkcurves(icurnr) = 0   ! curve is marked as used
      jcount = jcount+1
      kmeshs(jcount) = icurnr
      firstnode = kmeshm(ifirst+istar3)
      if ( debug ) &
         write(irefwr,*) 'icurnr, jcount, nextnode, firstnode ', &
                          icurnr, jcount, nextnode, firstnode
      if ( nextnode.eq.firstnode ) go to 1000   ! curve icurnr is closed itself

!     --- Find all next curves, store in icurves
!         raise icount and mark curves

      finished = .false.
      do while ( .not. finished )

         do i = 1, ncurves
            icurnr = istorecurves(i)    ! curve sequence number
            if ( imarkcurves(icurnr).ne.0 ) then

!           --- Free curve found
!               check if it starts or ends with nextnode

               ifirst = kmeshm(4+icurnr)   ! position of first node
               ilast = kmeshm(5+icurnr)-1  ! position of last node
               if ( kmeshm(ifirst+istar3).eq.nextnode ) then

!              --- First node is the next one

                  nextnode = kmeshm(ilast+istar3)
                  go to 200

               else if ( kmeshm(ilast+istar3).eq.nextnode ) then

!              --- Last node is the next one

                  nextnode = kmeshm(ifirst+istar3)
                  icurnr = -icurnr
                  go to 200

               end if  ! ( kmeshm(ifirst+istar3).eq.nextnode )

            end if  ! ( imarkcurves(icurnr).ne.1 )

         end do  ! i = 1, ncurves

200      imarkcurves(abs(icurnr)) = 0        ! curve is marked as used
         jcount = jcount+1
         kmeshs(jcount) = icurnr
         if ( nextnode.eq.firstnode ) finished = .true.
         if ( debug ) &
            write(irefwr,*) 'icurnr, jcount, nextnode ', &
                             icurnr, jcount, nextnode

         if ( jcount.gt.ncurves ) then

!        --- Internal error, jcount to large

            call errint ( jcount, 1 )
            call errint ( ncurves, 2 )
            call errsub ( 2738, 2, 0, 0 )
            go to 1000

         end if  ! ( jcount.gt.ncurves )

      end do  ! while ( .not. finished )

1000  call erclos ( 'mshfillkmeshsc' )

      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshs, jcount, 'kmesh_surface_part' )
         write(irefwr,*) 'End mshfillkmeshsc'

      end if

      end
