      subroutine mshgetelm ( nmeshe, kmeshd, fluidnd, inelem1, npoint, &
                             inode, indelm, shifted )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.1  date 22-11-2011 Debug information
!        version  1.0  date 22-06-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Check for coupling elements (2D)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer fluidnd(3,*), nmeshe(*), inelem1, kmeshd(*), npoint, &
              inode, indelm(*), shifted(*)

!     fluidnd        o    Array that contains the nodal numbers of the coupling
!                         elements in case of fluid-structure computations
!     indelm         i    Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     inelem1       i/o   Counter for number of coupling elements
!     inode          i    Node sequence number
!     kmeshd         i    KMESH part d
!     nmeshe         i    nmesh part e contains the node sequence numbers of
!                         the elements
!     npoint         i    Number of nodal points
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, j, k, ielem, nrelem, idx1, idx2, idx(3), istart
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     idx            Array with indices of the vertices
!     idx1           Help parameter to store an index
!     idx2           Help parameter to store an index
!     ielem          Element number
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     nrelem         Number of elements around node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data  idx /1,3,5/
! ======================================================================
!
      call eropen ( 'mshgetelm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshgetelm'
         write(irefwr,1) 'inode', inode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- loop over elements connected to node i

      nrelem=kmeshd(inode+1)-kmeshd(inode)
      do k=1,nrelem
         ielem=kmeshd(npoint+1+kmeshd(inode)+k)
         istart=nmeshe(ielem)
         idx1=0
         idx2=0

!        --- loop over vertices of element

         do j=1,3
            if( shifted(nmeshe(istart+idx(j)))==1.and. &
                nmeshe(istart+idx(j))/=inode) then
               idx2=j
            else if(nmeshe(istart+idx(j))==inode) then
               idx1=j
            end if
         end do

         if ( debug ) &
            write(irefwr,1) 'k, ielem, idx1, idx2', k, ielem, idx1, idx2

         if (idx1>0.and.idx2>0) then

            found=.false.
            do i=1,inelem1
               if ( (nmeshe(istart+idx(idx1))==fluidnd(1,i).and. &
                     nmeshe(istart+idx(idx2))==fluidnd(3,i)).or. &
                    (nmeshe(istart+idx(idx1))==fluidnd(3,i).and. &
                     nmeshe(istart+idx(idx2))==fluidnd(1,i)) ) &
                  found=.true.
            end do

            if (.not.found) then

               inelem1=inelem1+1
               fluidnd(1,inelem1)=nmeshe(istart+idx(idx1))
               fluidnd(3,inelem1)=nmeshe(istart+idx(idx2))
               if (idx1==1) then
                  if (idx2==2) then
                     fluidnd(2,inelem1)=nmeshe(istart+2)
                  else if (idx2==3) then
                     fluidnd(2,inelem1)=nmeshe(istart+6)
                  end if
               else if (idx1==2) then
                  if (idx2==1) then
                     fluidnd(2,inelem1)=nmeshe(istart+2)
                  else if (idx2==3) then
                     fluidnd(2,inelem1)=nmeshe(istart+4)
                  end if
               else if (idx1==3) then
                  if (idx2==1) then
                     fluidnd(2,inelem1)=nmeshe(istart+6)
                  else if (idx2==2) then
                     fluidnd(2,inelem1)=nmeshe(istart+4)
                  end if
               end if

            end if

         end if

         indelm(ielem)=1

      end do

1000  call erclos ( 'mshgetelm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshgetelm'

      end if  ! ( debug )

      end
