      subroutine mshinterface ( iinput, lengc, activatm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 30-07-2012 New treatment of iinput/rinput
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 24-03-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     checks the input for interface elements. It computes the
!     required length of these elements with respect to KMESH part c
!     The corresponding part of kmeshn is filled
!     inodp is raised if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengc
      logical activatm

!     activatm       i    If true kmesh part m must be activated
!     iinput         i    integer input array with respect to the mesh
!     lengc         i/o   computed length of kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurfs

!     isurfs         Number of parts where surface interface elements
!                    are given
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH075         Check the input for interface elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Reserve work space of length npoint
!     Activate some subarrays of kmeshh
!     Compute starting addresses
!     Actual call
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshinterface' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshinterface'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get and store some constants

      isurfs = iinput(2+iinp11)   ! number of parts with surface
                                 ! interface elements
      if ( debug ) write(irefwr,1) 'isurfs', isurfs

!     --- Actual call

      call msh075 ( iinput(iinp11), lengc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshinterface'

      end if
      call erclos ( 'mshinterface' )

      end
