      subroutine mshmapar35 ( mapar, kmeshl, userpoints, &
                              nextrapnts, kbndpt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-07-2008 Completely rewritten
!        version  1.0  date 24-01-2005
!
!   copyright (c) 2005-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put nodal point numbers of extra points into array kmeshl
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mapar(*), kmeshl(*), userpoints(*), nextrapnts, &
              kbndpt(*)

!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary (local numbering with respect to surface)
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points (global numbering)
!     mapar          i    mapping array: contains the mapping from local to
!                         global numbering of the nodal points
!     nextrapnts     i    Number of internal points in surface
!     userpoints     i    integer array of length nextrapnts containing the
!                         user point numbers of the extra points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nodp

!     i              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all internal points in surface do
!        get nodal point number
!        get user point number
!        Put nodal point number into kmeshl
!
!     The boundary points are all stored in kbndpt with length nbndpt
!     The next nextracrvs points contain the nodes of the extra curves
!     Finally the next extra nextrapnts points contain the nodes of the extra
!     user points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmapar35' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmapar35'
         call prinin ( kbndpt, nextrapnts, 'kbndpt' )
         call prinin ( userpoints, nextrapnts, 'userpoints' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For all internal points in surface do

      do i = 1, nextrapnts

!     --- get nodal point number
!         get user point number
!         Put nodal point number into kmeshl

         nodp = mapar(kbndpt(i))
         kmeshl(userpoints(i)) = nodp
         if ( debug ) &
            write(irefwr,1) 'i, nodp, userpoints(i)', &
                             i, nodp, userpoints(i)

      end do  ! i = 1, nextrapnts

1000  call erclos ( 'mshmapar35' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmapar35'

      end if

      end
