      subroutine mshmarkedges ( iedges, nedges, refarray, nmeshd, nelem, &
                                nsubdiv, npoint, ireflvl, refedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 25-04-2012 Extra parameter refedges
!        version  1.2  date 15-09-2006 Debug information
!        version  1.1  date 27-09-2005 Added ireflvl (J.H. Brusche)
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all edges that must be sudivided
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nedges, iedges(nedges), nmeshd(*), nelem, nsubdiv, npoint, &
              refarray(*), ireflvl, refedges(*)

!     iedges         o    Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     ireflvl        i    Refinement level
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     npoint         i    Number of nodal points
!     nsubdiv        o    Number of edges that are subdivided, hence number
!                         of extra nodes to be created
!     refarray       i    Array of size nelem that is filled with a 0
!                         if the element ielem must not be subdivided and a 1
!                         if it must
!     refedges       o    Contains the edge numbers of the edges that
!                         must be subdivided
!                         Length: nsubdiv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ipstart, edge

!     edge           edge sequence number
!     i              Counting variable
!     ielem          Element number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iedges(1:nedges)
!     For all elements ielem do
!        if ( refarray(ielem)>= ireflvl ) then
!           Mark all edges of ielem
!     Compute number of subdivisions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkedges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkedges'
         write(irefwr,1) 'ireflvl', ireflvl
  1      format ( a, 1x, (10i6) )

         call prininnonzero ( refarray, nelem, 'refarray' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iedges(1:nedges)

      iedges = 0
      nsubdiv = 0

!     --- For all elements ielem do

      do ielem = 1, nelem

         if ( refarray(ielem)>=ireflvl ) then

!        --- refarray(ielem)>0
!            Mark all edges of ielem

            ipstart = nmeshd(ielem)
            do i = 1, nmeshd(ipstart)

               edge = abs(nmeshd(ipstart+i))  ! edge sequence number

               if ( debug ) &
                  write(irefwr,1) 'edge, iedges(edge)', &
                                   edge, iedges(edge)

               if ( iedges(edge)==0 ) then

!              --- new edge found

                  iedges(edge) = 1
                  nsubdiv = nsubdiv+1
                  refedges(nsubdiv) = edge

               end if  ! ( iedges(edge)==0 )

            end do  ! i = 1, nmeshd(ipstart)

            if ( debug ) then
               write(irefwr,1) 'ielem', ielem
               call prinin ( nmeshd(ipstart+1), nmeshd(ipstart), &
                             'edges' )
            end if  ! ( debug )

         end if  ! ( refarray(ielem)>=ireflvl )

      end do  ! ielem = 1, nelem

!     --- Compute number of subdivisions

      nsubdiv = 0
      do i = 1, nedges
         if ( iedges(i)>0 ) then
            nsubdiv = nsubdiv+1
            iedges(i) = nsubdiv+npoint
         end if  ! ( iedges(i)>0 )
      end do  ! i = 1, nedges

1000  call erclos ( 'mshmarkedges' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nsubdiv', nsubdiv
         call prinin ( iedges, nedges, 'iedges' )
         call prinin ( refedges, nsubdiv, 'refedges' )
         write(irefwr,*) 'End mshmarkedges'

      end if

      end
