      subroutine mshmprsur ( iinput, lengc, kbndpt, mapar, jnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 12-03-2015 Correction extra curves
!        version  2.2  date 30-11-2014 Change isurf 23 in 99
!        version  2.1  date 15-11-2012 Long integers
!
!   copyright (c) 2006-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill the arrays mapar, kmeshl, kmeshm for surface elements
!     in case surface elements are connected to an element group
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      integer iinput(*), kbndpt(*), mapar(*),  lengc

!     iinput         i    integer input array for subroutine mesh
!     jnew           i    indication whether the mesh is new (true) or old
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     lengc         i/o   computed length of kmesh part c
!     mapar         i/o   mapping array: contains the mapping from local to
!                         global numbering of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, i, ielgrp, ifirst, &
              ilast, jinp, itype, leng, istart, k, index, &
              isubsf, isursf, nsubsf, lengsf, kinp,  jsurnr, &
              nel1,  nelloc, jsursf, numcurs, numnodes, iposiinppnt, &
              iposiinpcrv
      integer, allocatable :: kmeshcsurf(:)

!     i              Counting variable
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     index          Help parameter to store a value
!     iposiinpcrv    Starting address in array iinput with respect to the
!                    extra user curves (internal curves)
!     iposiinppnt    Starting address in array iinput with respect to the
!                    extra user user points (internal points)
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isursf         Surface sequence number
!     itype          Type number of element
!     jinp           position in array iinput
!     jsurnr         Surface sequence number
!     jsursf         Surface sequence number with sign
!     k              Counting variable
!     kinp           position in array iinput
!     kmeshcsurf     Work array to store kmeshc for surface temporary
!     leng           Help variable to compute the length of some arrays
!     lengsf
!     nel1           First element sequence number in subsurface
!     nelloc         Number of elements in subsurface
!     nsubsf         Number of subsurfaces
!     numcurs        Number of curves along surface
!     numnodes       Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH017         Fill the arrays mapar and kbndpt with the global numbers of
!                    the nodal points for a surface
!     MSH018         compute position of surface in array iinput
!     MSHMAPAR35     Put nodal point numbers of extra points into array kmeshl
!     MSHMAPCURVS    Put nodal point numbers of extra curves into array kmeshm
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmprsur' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmprsur'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iinp = iinp4
      if ( debug ) write(irefwr,1) 'inodp, iselem', inodp, iselem

      do i = 1, nsurel

         ielgrp = iinput(iinp+1)
         ifirst = iinput(iinp+2)
         ilast = iinput(iinp+3)
         iinp = iinp+4
         if ( kelma>0 ) inpelm = kmesha(ielgrp+1) -  kmesha(ielgrp)
         inelem = 0

         if ( debug ) then

!        --- debug = true, carry out debug statements

            write(irefwr,*) 'surfaces in mshmpr'
            write(irefwr,1) 'i, ielgrp, ifirst, ilast', &
                             i, ielgrp, ifirst, ilast

         end if

         do jsurnr = ifirst, ilast

            isurnr = jsurnr
            if ( debug ) write(irefwr,1) 'jsurnr', jsurnr
            if ( kmeshs(nsurfs+1+8*(isurnr-1)+1)==20 ) then

!           --- Surface is of type sphere, use the surface from which it must
!               be copied

               isurnr = kmeshs(nsurfs+1+8*(isurnr-1)+3)

            else if ( kmeshs(nsurfs+1+8*(isurnr-1)+1)==0 ) then

!           --- Empty surface, skip

               jinp = jinp+1
               go to 150

            end if
            if ( debug ) &
               write(irefwr,1) 'jsurnr, isurnr', jsurnr, isurnr

            ishape = kmeshs(nsurfs+1+8*(isurnr-1)+2)
            if ( debug ) then

!           --- debug = true, carry out debug statements

               write(irefwr,1) 'isurnr, ishape', isurnr, ishape

            end if
            if ( jnew) then

!           --- jnew = true   new mesh

               call msh018 ( iinput(iinp3), isurnr, 0, jinp )
               itype  = iinput(jinp+iinp3)
               numnodes = abs(kmeshh(1,isurnr+1))

!              --- since kbndpt is extended by one position we have to decrease
!                  nbndpt by 1

               if ( kmeshh(4,isurnr+1)/=0 ) then
                  nbndpt = abs(kmeshh(4,isurnr+1))
               else
                  nbndpt = 0
               end if  ! ( kmeshh(4,isurnr+1)>0 )
               inelem =  kmeshh(2,isurnr+1)
               leng = inpelm*inelem

               if ( nvolms==0 ) then

!              --- nvolms = 0, fill array KMESH part t with surfaces

                  kmesht(1,isurnr) = iselem
                  kmesht(2,isurnr) = iselem+inelem-1
                  kmesht(1,jsurnr) = iselem
                  kmesht(2,jsurnr) = iselem+inelem-1

               end if

               lengc = lengc+leng
               index = kmeshh(4,isurnr+1)
               if ( debug ) then

!              --- debug = true, carry out debug statements

                  write(irefwr,1) 'numnodes, nbndpt, index, itype', &
                                   numnodes, nbndpt, index, itype
                  write(irefwr,1) 'inelem, inpelm', inelem, inpelm

               end if

               if ( itype==22 .or. itype==99 ) then

!              --- For type 99, we have to fill the internal nodes into
!                  and curves
!                  Compute position of userpoints and curves in iinput

                  numcurs = iinput(iinp3+jinp+2)
                  kinp = iinp3+jinp+numcurs+2
                  nextrapnts = iinput(kinp+1)
                  nextracrvs = iinput(kinp+2)
                  iposiinppnt = kinp+3
                  iposiinpcrv = iposiinppnt+nextrapnts

               else

                  nextrapnts = 0
                  nextracrvs = 0
                  iposiinppnt = 1
                  iposiinpcrv = 1

               end if  ! ( itype==99 )

               if ( debug ) then

!              --- debug = true, carry out debug statements

                  write(irefwr,1) 'nextrapnts, nextracrvs', &
                                   nextrapnts, nextracrvs
                  write(irefwr,1) 'iposiinppnt, iposiinpcrv', &
                                   iposiinppnt, iposiinpcrv

               end if

               if ( index<=0 ) then

!              --- index<0, hence surface has not been treated yet

                  if ( itype==7 .or. itype==11 .or. itype==12 ) then

!                 --- itype = 7, 11 or 12  surface of subsurfaces
!                     Perform a loop over the subsurfaces

                     nel1 = iselem
                     nsubsf = kmeshr(isurnr+1) - kmeshr(isurnr)
                     do isubsf = 1, nsubsf
                        jsursf = kmeshr(kmeshr(isurnr)+isubsf)
                        isursf = abs(jsursf)
                        call msh018 ( iinput(iinp3), isursf, 0, kinp )
                        itype  = iinput(kinp+iinp3)
                        numnodes = kmeshh(1,isursf+1)
                        nbndpt = abs(kmeshh(4,isursf+1))
                        lengsf = size ( kmhsur(isursf)%kmeshcsur )
                        index = kmeshh(4,isursf+1)
                        if ( debug ) then

!                       --- debug = true, carry out debug statements

                           write(irefwr,1) &
                            'isubsf, jsursf, numnodes, nbndpt, index, itype', &
                             isubsf, jsursf, numnodes, nbndpt, index, itype

                        end if

                        if ( index<0 ) then

!                       --- index<0, hence surface has not been treated yet

                           index = - index
                           kmeshh(4,isursf+1) = index
                           kbndpt(1:nbndpt) = kmhsur(isursf)%kbndptsur(1:nbndpt)
                           npoint = numnodes
                           call msh017 ( kbndpt, mapar, iinput(kinp+iinp3), &
                                         isursf )

                           if ( nvolel>0 ) then

!                          --- nvolel>0, fill kmesh part n

                              kbndpt(1:lengsf) = &
                                 kmhsur(isursf)%kmeshcsur(1:lengsf)
                              istart = 2*nsurfs
                              do k = 1, isursf-1
                                 istart = istart + kmeshn(k)* &
                                                   kmeshn(k+nsurfs)
                              end do

                              do k = 1, lengsf
                                 kmeshn(istart+k) = mapar(kbndpt(k))
                              end do

                           end if
                           kmhsur(isursf)%maparsur(1:numnodes) = &
                              mapar(1:numnodes)
                           if ( nvolms==0 ) then

!                          --- nvolms = 0, fill array KMESH part t with surfaces

                              nelloc = lengsf/inpelm
                              kmesht(1,isursf) = nel1
                              kmesht(2,isursf) = nel1+nelloc-1
                              nel1 = nel1+nelloc

                           end if

                        end if

                     end do

                  else

!                 --- itype # 7, 11 or 12  Standard surface

                     if ( nbndpt>0 ) then

!                    --- There is a boundary

                        index = -index
                        kmeshh(4,isurnr+1) = index
                        kbndpt(1:nbndpt) = kmhsur(isurnr)%kbndptsur(1:nbndpt)

!                       --- Store actual value of nbndpt
!                           The last position of kbndpt contains lenextracrvs
!                           See msh012

                        lenextracrvs = kmhsur(isurnr)%kbndptsur(1+nbndpt)
                        nbndpt =nbndpt-lenextracrvs
                        lenextracrvs = lenextracrvs-nextrapnts

                     else

                        lenextracrvs = 0

                     end if  ! ( nbndpt>0 )

                     nbndpt = nbndpt+lenextracrvs
                     npoint = numnodes
                     if ( debug ) then
                        call prinin ( kbndpt, nbndpt+lenextracrvs, &
                                     'kbndpt' )
                        write(irefwr,1) 'nbndpt, numnodes, inodp', &
                                         nbndpt, numnodes, inodp
                        write(irefwr,1) 'lenextracrvs', lenextracrvs
                     end if  ! ( debug )
                     call msh017 ( kbndpt, mapar, &
                                   iinput(jinp+iinp3), isurnr )
                     if ( debug ) then

!                    --- debug = true, carry out debug statements

                        call prinin ( kmeshl, nuspnt, 'kmeshl' )
                        call prinin ( kbndpt, nbndpt, 'kbndpt' )
                        call prinin ( mapar, numnodes, 'mapar' )

                     end if

                     if ( nvolel>0 ) then

!                    --- There are volume elements, fill kmeshn

                        allocate ( kmeshcsurf(leng), stat = error )
                        if ( error/=0 ) &
                           call eralloc ( error, leng, 'kmeshcsurf' )
                        if ( ierror/=0 ) go to 1000

                        kmeshcsurf(1:leng) = &
                           kmhsur(isurnr)%kmeshcsur(1:leng)
                        istart = 2*nsurfs
                        do k = 1, isurnr-1
                           istart = istart + kmeshn(k)*kmeshn(k+nsurfs)
                        end do
                        kmeshn(istart+1:istart+leng) = &
                            mapar(kmeshcsurf(1:leng))
                        if ( debug ) then

!                       --- debug = true, carry out debug statements

                           call prinin1 ( kmeshn(istart+1), &
                                          kmeshn(isurnr+nsurfs), &
                                          kmeshn(isurnr), 'kmeshn' )

                        end if

                        deallocate ( kmeshcsurf, stat = error )
                        if ( error/=0 ) &
                           call erdealloc ( error, 'kmeshcsurf' )
                        if ( ierror/=0 ) go to 1000

                     end if

                     if ( nextrapnts>0 ) then

!                    --- nextrapnts>0, we have to fill the internal nodes into
!                        array kmeshl

                        call mshmapar35 ( mapar, kmeshl, &
                                          iinput(iposiinppnt), &
                                          nextrapnts, &
                                          kbndpt(nbndpt+lenextracrvs+1))

                     end if  ! ( nextrapnts>0 )

                     if ( .false. .and. nextracrvs>0 ) then

!                    --- nextracrvs>0, we have to fill the internal curves into
!                        array kmeshm
!                        This part is not used since msh017 is adapted

                        if ( debug ) then
                           write(irefwr,1) 'nbndpt', nbndpt
                           call prinin ( kbndpt, nbndpt+1, 'kbndpt' )
                        end if  ! ( debug )
                        call mshmapcurvs ( nextracrvs, &
                                           iinput(iposiinpcrv), kmeshm, &
                                           kmeshl, kbndpt(nbndpt+1), &
                                           mapar, isurnr )

                     end if  ! ( nextracrvs>0 )

                     kmhsur(isurnr)%maparsur(1:numnodes) = mapar(1:numnodes)

                  end if
                  iselem = iselem+inelem

               end if

            end if

150      end do ! jsurnr = ifirst, ilast

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inodp, iselem ', inodp, iselem
         call prinin ( kmeshl, nuspnt, 'kmeshl' )
         write(irefwr,*) 'End mshmprsur'

      end if
      call erclos ( 'mshmprsur' )

      end subroutine mshmprsur
