      subroutine mshobst2d ( kmeshnew, coornew, ielemobstnew, &
                             ipointobstnew, kmeshcnew, kmeshmnew, &
                             iinput, rinputusr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 05-11-2013 Extra parameter rinputusr
!        version  1.4  date 05-05-2013 Delete kmesh part ad
!        version  1.3  date 24-10-2012 Use plotinters
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements, &
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     2d case
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnew, ielemobstnew(*), &
              ipointobstnew(*), kmeshcnew(*), &
              kmeshmnew(*), iinput(*)
      double precision coornew(*), rinputusr(*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     ielemobstnew  i/o   New array ielemobst
!     iinput        i/o   integer input array
!                         See subroutine mshobstms23d
!     ipointobstnew  i    New array ipointobst
!     kmeshcnew      o    New kmesh part c
!     kmeshmnew      o    New array kmeshm
!     kmeshnew      i/o   New array kmesh
!     rinputusr      i    See subroutine mshobstms
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointnew, iseqkm1

!     iseqkm1        Sequence number with respect to km of first mesh
!     npointnew      Sequence number of last generated node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHDELKMESHAD  Delete kmesh part ad
!     MSHOBSTMS2D    Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (2d)
!     PR0008         Print information of curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Reserve temporary space
!     Compute starting addresses
!     Create new mesh
!     Make new arrays permanent
!     Compute some extra arrays by topol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobst2d' )
      debug = debug_mshobstmesh
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobst2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

!     --- Delete kmeshnew part ad if it exists

      iseqkm = kmeshnew
      call mshdelkmeshad

      iseqkm = iseqkm1

!     --- Create new mesh

      npointnew = iinput(4)
      call mshobstms2d ( kmeshcnew, coornew, kmeshnew, ipointobstnew, &
                         ielemobstnew, iinput, npointnew, &
                         kmeshmnew, rinputusr )

1000  if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshmnew, km(iseqkm)%kmesh(11) )
         write(irefwr,*) 'End mshobst2d'

      end if  ! ( debug )
      iseqkm = iseqkm1
      call erclos ( 'mshobst2d' )

      end
