      subroutine mshobstcurv ( kmeshmnew, lenkmsmnew, iedges, kstep, &
                               idimedges )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  3.2  date 19-09-2013 Extra debug
!        version  3.1  date 24-04-2012 Correction zero node numbers
!        version  3.0  date 22-11-2011 Removal of iwork from list
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new array kmeshm in the case that extra points at the surface
!     are created when a new mesh is created due to intersection with an
!     obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     surface
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmnew(*),  &
              idimedges, iedges(idimedges,*), lenkmsmnew, kstep

!     idimedges      i    Length of first dimension of array iedges
!                         (linear:1, quadratic: 3)
!     iedges         i    Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i, &
!                         with sequence number iedges(i)
!     kmeshmnew      o    New array kmeshm
!     kstep          i    Indicates if linear elements are used (1) or
!                         quadratic (2)
!                         Is used to loop over end points of elements on
!                         curve only
!     lenkmsmnew    i/o   Length of array kmshnnew
!                         Ar input: declared length
!                         Ar output: computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, istartnew, inpelm, inode1, &
              inode2, iedge, i, inpelmnew, icurv, j, &
              edgenode(2), nsubcurv, isubcurv, jcurv
      integer, allocatable :: iwork(:)
      logical found

!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     found          If true already a possibility has been found
!     i              Counting variable
!     icurv          Curve number with direction
!     iedge          edge sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     inpelm         number of points per element
!     inpelmnew      ?
!     istart         First position used at a part
!     istartnew      Last position used in kmeshnnew
!     isubcurv       ?
!     iwork          integer work array of length nedges
!                    if iwork(i)=0, edge i does not correspond to inode1
!                    if iwork(i)=1, edge i corresponds to inode1
!     j              Counting variable
!     jcurv          Absolute value of curve number
!     nsubcurv       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     PR0008         Print information of curves
!     SEPFILLNMESHL  Fill array nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstcurv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstcurv'
         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,1) 'kstep, lenkmsmnew', kstep, lenkmsmnew
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( nmesh(42)==0 ) call sepfillnmeshl

      if ( debug ) write(irefwr,1) 'nedges', nedges

      allocate ( iwork(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      istart = kmeshm(2)-1
      inpelm=0

!     --- fill parts 1, 2 and 4 of new kmeshm

      kmeshmnew(1) = kmeshm(1)
      kmeshmnew(2) = kmeshm(2)
      kmeshmnew(4) = kmeshm(4)
      kmeshmnew(5) = 1
      istartnew = kmeshm(2)-1

!     --- loop over curves

      do jcurv = 1, ncurvs

         inpelmnew = 1
         inpelm = kmeshm(5+jcurv)-kmeshm(5+jcurv-1)
         if ( debug ) then
            write(irefwr,1) 'jcurv, inpelm, istart, istartnew', &
                             jcurv, inpelm, istart, istartnew
            write(irefwr,1) 'kmeshm(istart+1)', kmeshm(istart+1)
         end if  ! ( debug )

         if ( inpelm>0 ) then

!        --- check for subcurves

            if ( kelmq>0 ) then

!           --- kelmq>0, hence kmeshq has been filled

               nsubcurv = kmeshq(jcurv+1)-kmeshq(jcurv)
               if (nsubcurv==0) nsubcurv = 1

            else

!           --- kelmq>0, hence kmeshq has been filled

               nsubcurv = 1

            end if  ! ( kmesh(31>0 )

!           --- loop over subcurvs

            inpelmnew = 1
            do isubcurv = 1, nsubcurv

               if ( nsubcurv>1 ) then
                  icurv = abs(kmeshq(kmeshq(jcurv)+isubcurv))
               else
                  icurv = jcurv
               end if

               inpelm = kmeshm(5+icurv)-kmeshm(5+icurv-1)
               if ( debug ) &
                  write(irefwr,1) 'isubcurv, jcurv, icurv, inpelm', &
                                   isubcurv, jcurv, icurv, inpelm

!              --- loop over nodes per element per curve

               do j = 1, inpelm-kstep, kstep

                  inode1 = kmeshm(istart+j)
                  inode2 = kmeshm(istart+j+kstep)

                  if ( debug ) &
                  write(irefwr,1) 'istart, j, inode1, inode2', &
                                   istart, j, inode1, inode2

                  if ( inode1<=0 .or. inode2<=0 ) then

!                 --- One of the two nodes is empty, no edge

                     found = .false.

                  else

!                 --- find edge corresponding to inode1 and inode2

                     iedge = iniedge ( inode1, inode2, nmeshl, iwork )
                     if ( debug ) &
                        write(irefwr,1) 'iedge, inode1, inode2', &
                                         iedge, inode1, inode2

                     found = iedge>0
                     if ( found ) found = iedges(1,iedge)>0

                  end if  ! ( inode1==0 .or. inode2==0 )
                  if ( found ) then

!                 --- create new elements in kmeshmnew
!                     raise istartnew

                     if ( kstep==1 ) then

!                    --- linear case

                        kmeshmnew(istartnew+1) = inode1
                        kmeshmnew(istartnew+2) = iedges(1,iedge)
                        kmeshmnew(istartnew+3) = inode2

                        istartnew = istartnew+2
                        inpelmnew = inpelmnew+2

                     else if ( kstep==2 ) then

!                    --- quadratic case

                        if (nmeshc(nmeshc(iedge)+1)==inode2) then
                           edgenode(1) = iedges(3,iedge)
                           edgenode(2) = iedges(2,iedge)
                        else if (nmeshc(nmeshc(iedge)+1)==inode1) then
                           edgenode(1) = iedges(2,iedge)
                           edgenode(2) = iedges(3,iedge)
                        end if

                        kmeshmnew(istartnew+1) = inode1
                        kmeshmnew(istartnew+2) = edgenode(1)
                        kmeshmnew(istartnew+3) = iedges(1,iedge)
                        kmeshmnew(istartnew+4) = edgenode(2)
                        kmeshmnew(istartnew+5) = inode2

                        istartnew = istartnew+4
                        inpelmnew = inpelmnew+4

                     end if

                  else

!                 --- no extra elements added in kmeshm

                     do i = 1, kstep+1
                        kmeshmnew(istartnew+i) = kmeshm(istart+j-1+i)
                     end do

                     istartnew = istartnew+kstep
                     inpelmnew = inpelmnew+kstep

                  end if

               end do !end loop over elements

!              --- Fill first part of kmeshn

               kmeshmnew(5+jcurv) = kmeshmnew(5+jcurv-1) + inpelmnew
               istartnew = istartnew+1
               istart = istart+inpelm
               inpelmnew = inpelmnew+1

            end do !end loop over subcurves

         else

!        --- curve not filled

            kmeshmnew(5+jcurv) = kmeshmnew(5+jcurv-1)

            istartnew = istartnew + inpelm
            istart = istart+inpelm

         end if

      end do !end loop over curves

!     --- update kmeshm part 3

      kmeshmnew(3) = kmeshmnew(2) + (kmeshmnew(5+ncurvs) - kmeshmnew(5))

      if ( debug ) then
         write(irefwr,1) 'ncurvs', ncurvs
         write(irefwr,1) 'kmeshmnew(2)', kmeshmnew(2)
         write(irefwr,1) 'kmeshmnew(5)', kmeshmnew(5)
         write(irefwr,1) 'kmeshmnew(5+ncurvs)', kmeshmnew(5+ncurvs)
         write(irefwr,1) 'kmeshmnew(3)', kmeshmnew(3)
      end if  ! ( debug )

      do i = 1, kmeshm(4)
         kmeshmnew(kmeshmnew(3)-1+i) = kmeshm(kmeshm(3)-1+i)
      end do

      lenkmsmnew = kmeshmnew(3)+kmeshmnew(4)-1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenkmsmnew', lenkmsmnew
         call pr0008 ( kmeshmnew, ncurvs )
         write(irefwr,*) 'End mshobstcurv'

      end if
      call erclos ( 'mshobstcurv' )

      end
