      subroutine mshobstms023 ( npoint, iworkpoint, nmap, ipointobst, &
                                numobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-10-2011 Extension with 1d
!        version  1.1  date 08-11-2004 Set iworkpoint(.,2)
!        version  1.0  date 20-05-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill iworkpoint with information about nodal points in and outside
!     obstacle and corresponding obstacle sequence numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nmap, iworkpoint(nmap,2), ipointobst(*), &
              numobst

!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     iworkpoint     o    Work array of length npointnew*2
!                         This array is filled as follows:
!                         iworkpoint(i,1) = 0 if node i outside obstacle
!                         iworkpoint(i,1) = 1 if node i inside obstacle
!                         iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                             boundary)
!                         iworkpoint(i,1) = 3 if node i on obstacle
!                         iworkpoint(i,2) contains the corresponding obstacle
!                         sequence number iobst
!     nmap           i    length of first parameter of imap and rmap
!     npoint         i    Number of nodal points
!     numobst        i    number of obstacles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, istart, numpntobst, numinnobst, numbndobst

!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iworkpoint
!     For all obstacles do
!        Fill iworkpoint for that obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms023' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms023'
         write(irefwr,1) 'numobst', numobst
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iworkpoint

      iworkpoint(1:npoint,1:2) = 0

!     --- For all obstacles do
!            Fill iworkpoint for that obstacle

      do iobst = 1, numobst

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         istart = istart+3

         if ( debug ) &
            write(irefwr,1) 'iobst, numpntobst, numinnobst, numbndobst', &
                             iobst, numpntobst, numinnobst, numbndobst

         iworkpoint(ipointobst(istart+1:istart+numpntobst),1) = 1
         iworkpoint(ipointobst(istart+1:istart+numpntobst),2) = iobst

         istart = istart+numpntobst

         iworkpoint(ipointobst(istart+1:istart+numinnobst),1) = 2
         iworkpoint(ipointobst(istart+1:istart+numinnobst),2) = iobst

         istart = istart+numinnobst

         iworkpoint(ipointobst(istart+1:istart+numbndobst),1) = 3
         iworkpoint(ipointobst(istart+1:istart+numbndobst),2) = iobst

      end do  ! iobst = 1, numobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms023'

      end if
      call erclos ( 'mshobstms023' )

      end
