      subroutine mshobstms06 ( meshin, meshout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-10-2006 Remove mshdelsel
!        version  1.0  date 25-10-2004
!
!   copyright (c) 2004-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove some subarrays of kmeshnew that are no longer valid
!     Recompute topology
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, meshout

!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jrenum(10), iseqkm1, iseqkm2
      integer, pointer :: kmeshnew

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     jrenum         Array jrenum for TOPOL
!     kmeshnew       New array kmesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     MSHCOPSEL      Copy a selected set of subarrays of kmesh
!     MSHOBSTRENUM   Fill array jrenum, depending on iinput in case of obstacles
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms06' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms06'
         call ertrace
         write(irefwr,1) 'meshin, meshout', meshin, meshout
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmesh => kmesh2d(meshin)
      if ( meshout==meshin ) then
         kmeshnew => kmesh2d(maxmesh+1)
      else
         kmeshnew => kmesh2d(meshout)
      end if  ! ( meshout==meshin )

      call sepfillkmesh(kmesh)
      iseqkm1 = iseqkm
      iseqkm2 = kmeshnew

      if ( debug ) then
         write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2
         write(irefwr,1) 'nelem1, nelem2', nelem, km(iseqkm2)%kmesh(9)
      end if  ! ( debug )

      if ( nelem/=km(iseqkm2)%kmesh(9) ) then

!     --- Number of elements in new and old mesh different
!         Compute some extra arrays by topol

         if ( debug ) write(irefwr,1) 'kelmj', kelmj
         if ( kelmj>0 ) then

!        --- kmesh(24) = 0, old mesh has been renumbered
!            Check if array iinput has been filled
!            If so get type of renumbering from iinput. otherwise use
!            Sloan

            call mshobstrenum ( jrenum )

!           --- The last two positions are always new, since we have already
!               an "optimal" numbering for the old mesh

            jrenum(10) = kmeshj(1)   ! Start with renumbered node 1
            kmesh => kmeshnew
            call sepfillkmesh ( kmesh )
            call topol ( jrenum )
            kmesh => kmesh2d(meshin)
            call sepfillkmesh ( kmesh )
            if ( ierror/=0 ) go to 1000

         end if  ! ( kmesh(24)>0 )

      else

!     --- Number of elements in old and new mesh the same
!         Copy some parts of old mesh to new one

         call mshcopsel ( kmesh, kmeshnew )

      end if

!     --- Copy outer boundary

      if ( mmouter>0 ) then

!     --- mmouter>0, outer boundary is known
!         Copy old array outerboun to new mesh

         if ( associated(km(iseqkm2)%outerboun) ) &
            deallocate(km(iseqkm2)%outerboun)
         length = size(km(iseqkm1)%outerboun)
         allocate(km(iseqkm2)%outerboun(length))
         km(iseqkm2)%outerboun = km(iseqkm1)%outerboun

      end if  ! ( mmouter>0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms06'

      end if
      call erclos ( 'mshobstms06' )

      end
