      subroutine mshobstms2dsh ( meshorig, meshobst, maploc, iinputusr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 19-09-2013 New call to subroutines
!        version  1.2  date 24-10-2012 New call to princrossobst
!        version  1.1  date 22-11-2011 New call to mshobstms03
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements, &
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     2d and 3d case with shift
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: meshorig, meshobst, iinputusr(*)
      type (mapping) :: maploc

!     iinputusr      i    Integer input array defining which actions must
!                         be undertaken
!                         At this moment iinputusr is under construction
!                         Contents
!                         Pos. 1  Last entry that is filled by the calling
!                                 program
!                         Pos. 2  shft: Defines whether nodes must be
!                                       shifted to the obstacle (1) or not (<1)
!                                       if -1 do not call mshobstbounxx
!                                       Default value: 0
!                         Pos. 3  smth: Defines type of smoothing;
!                                       Laplacian smoothing (1), angle
!                                       optimization (2), both (3), none (0)
!                                       Only allowed if shft=1
!     maploc         o    contains information of the mappin
!     meshobst       i    Sequence number of obstacle mesh to be used
!     meshorig       i    Sequence number of original mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinput(20),  npointnew, linear, iseqkm1, iseqkm2, &
              lenkmscnew, nelemnew, lenkmsnnew, &
              idimedges,  lenkmsmnew, lenipointobst, lenkmsmnewsv, &
              lenielemobst, lencoornew, lenkmsnnewsv
      double precision volold(3), volumes(3), voltot, voltmp
      integer, pointer :: kmeshcnew(:), kmeshnnew(:), iedges(:), &
                              ipointobstnew(:), kmeshmnew(:), &
                              ielemobstnew(:), ielmfaces(:), imaplc(:,:)
      double precision, pointer :: coornew(:), rmaplc(:,:)
      integer, pointer :: kmeshnew

!     coornew        Array containing the new coordinates of the nodes
!     idimedges      Length of first dimension of array iedges
!                    (linear:1, quadratic: 3)
!     iedges         Work array of size nedges
!     ielemobstnew   New array ielemobst
!     ielmfaces      Contains faces of elements
!     iinput         integer array to pass information from mshobstms01
!                    See subroutine mshobstms23d
!     ipointobstnew  New array ipointobst
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnew       New array kmesh
!     kmeshnnew      New array kmeshn
!     lencoornew     Estimated length of array coornew
!     lenielemobst   Computed length of array ielemobst
!     lenipointobst  length of array ipointobst
!     lenkmscnew     Length of array kmshcnew
!     lenkmsmnew     Length of array kmshmnew
!     lenkmsmnewsv   Help parameter to store original value of lenkmsmnew
!     lenkmsnnew     Length of array kmshnnew
!     lenkmsnnewsv   Help parameter to store original value of kmshnnew
!     linear         Indicates if linear elements are used (1) or
!                    quadratic (2)
!     nelemnew       Number of elements in new mesh
!     npointnew      Sequence number of last generated node
!     volold         Old computed value of volume of region
!     voltmp         Temporary computed value of volume of region
!     voltot         Total volume of region
!     volumes        Array of length 3 containing the volume of the
!                    following subparts:
!                    1: all elements completely inside the obstacle
!                    2: all elements partly inside the obstacle
!                    3: all elements outside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     MSHOBST01      Compute some parameters for subroutine mshobstms
!     MSHOBSTMARKBT  Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
!     MSHOBSTMS03    Subdivide a triangle if necessary into two or three
!                    subtriangles (points are moved to the obstacle)
!     MSHOBSTMS04    Put control integers around the temporary arrays in
!                    mshobstsms
!     MSHOBSTMS05    Update kmeshm and kmeshn if necessary
!     MSHOBSTMS06    Remove some subarrays of kmeshnew that are no longer valid
!                    Recompute topology
!     MSHOBSTMS07    Make new arrays permanent for subroutines mshobstms
!                    and refineloc
!     MSHOBSTMS10    Check the control integers of the work arrays defined
!                    in MSHOBSTMESH
!     MSHOBSTPARM    Estimate number of nodes and elements in new mesh
!     MSHOBSTVOL3D   Compute volume of part of region inside, outside and
!                    near obstacle
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     PRIN3DOBST     Write the coordinates and topology of the obstacle to a
!                    file in openview format
!     PRININ         print 1d integer array
!     PRINMAP1       Actual printing of array imap
!     SEPCOPIBUF     Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents
!                    of iarray to ibuffr
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHE  Fill array nmeshe
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPINITKMESHAD Initialize kmesh ad
!     SEPPRCROSSOBST Print the contents of the arrays icrossobst and rcrossobst
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2268   The volume of the original tetrahedron is not equal to the sum of
!           the volumes of the new tetrahedron
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Reserve temporary space
!     Compute starting addresses
!     Create new mesh
!     Make new arrays permanent
!     Compute some extra arrays by topol
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms2dsh' )
      debug = debug_mshobstmesh
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms2dsh'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug .and. ndim==3 ) then

!     --- debug true, compute volumes of parts of mesh, store in array volold
!         and print

         call mshobstvol3d ( volold )
         voltot = volold(1)+volold(2)+volold(3)
         write(irefwr,100) volold, voltot
100      format ( 'Before subdivision:'/ &
                  'Volume of elements inside obstacle  ', d17.8/ &
                  'Volume of elements near obstacle    ', d17.8/ &
                  'Volume of elements outside obstacle ', d17.8/ &
                  'Volume of complete region           ', d17.8 )

      end if  ! ( debug .and. ndim==3 )

      kmesh => kmesh2d(meshorig)
      iseqkm1 = iseqkm
      kmeshnew =>kmesh2d(meshobst)

!     --- Activate arrays

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( kelmad==0 ) call sepfillkmeshad ( iseqobst )
      if ( nmesh(31)==0 ) call sepfillnmesha
      if ( nmesh(35)==0 ) call sepfillnmeshe

      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( kmeshad(1)==0 ) call sepflpointobst
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3
      if ( nmesh(42)==0 ) call sepfillnmeshl

      if ( debug ) then
         call sepprcrossobst
         call sepprinipobst ( ipointobst )
      end if  ! ( debug )


!     --- Compute some parameters

      call mshobst01 ( iinput )

!     --- Compute length of temporary arrays and estimate npointnew

      call mshobstparm ( iinput )

      npointnew = iinput(4)
      lenkmsmnew = iinput(7)
      lenkmsnnew = iinput(8)
      lenkmscnew = iinput(12)
      lencoornew = iinput(14)
      lenipointobst = iinput(15)
      lenielemobst = iinput(17)

      lenkmsmnewsv = lenkmsmnew
      lenkmsnnewsv = lenkmsnnew


!     --- Reserve space for temporary arrays

      allocate ( coornew(0:lencoornew+1), rmaplc(npointnew,2), &
                 kmeshcnew(0:lenkmscnew+1), kmeshnnew(0:lenkmsnnew+1), &
                 iedges(3*nedges), ipointobstnew(0:lenipointobst+1), &
                 ielemobstnew(0:lenielemobst+1), &
                 imaplc(npointnew,3), ielmfaces(4*nfaces), &
                 kmeshmnew(0:lenkmsmnew), &
                 stat = error )
      length = lencoornew+lenkmscnew+lenkmsnnew+ &
               nedges+lenipointobst+lenielemobst+npointnew*5+ &
               4*nfaces+lenkmsmnew+12*2
      if ( error/=0 ) call eralloc ( error, length, 'coornew' )
      if ( ierror/=0 ) go to 1000

      iedges = 0
      imap => imaplc
      rmap => rmaplc

!     --- Put control integers around these arrays

      call mshobstms04 ( kmeshcnew, kmeshnnew, &
                         ipointobstnew, ielemobstnew, &
                         kmeshmnew, coornew, iinput )

!     --- Create new mesh

      npointnew = iinput(4)
      call mshobstms03 ( kmeshcnew(1), coornew(1), iedges, kmeshnew, &
                         ipointobstnew(1), ielemobstnew(1), &
                         iinput, npointnew, iinputusr, ielmfaces )

      if ( ierror/=0 ) go to 1000
      iseqkm2 = kmeshnew

!     --- Update kmeshm and kmeshn if necessary

      idimedges = 3
      call mshobstms05 ( kmeshnnew(1), lenkmsnnew, iedges, linear, &
                         ielmfaces, npointnew, kmeshnew, &
                         idimedges, kmeshmnew(1), lenkmsmnew )

!     --- Check control integers

      iinput(7) = lenkmsmnewsv
      iinput(8) = lenkmsnnewsv

      call mshobstms10 ( kmeshcnew, kmeshnnew, &
                         ipointobstnew, ielemobstnew, &
                         kmeshmnew, coornew, iinput )

!     --- Make new arrays permanent

      nelemnew = km(iseqkm2)%kmesh(9)
      call mshobstms07 ( kmeshnew, coornew(1), kmeshcnew(1), &
                         kmeshnnew(1), kmeshmnew(1), iinput )

!     --- Fill part 4 of ipointsobst of nodes on the boundary of the
!         obstacle which are only in elements completely inside
!         the obstacle

      call sepfillkmesh(kmeshnew)
      call sepfillkmeshd  ! fill kmeshnew part d
      call sepfillkmesh(kmesh)
      iseqkm = iseqkm2
      call mshobstmarkbt ( ipointobstnew(1), nelemnew, &
                           npointnew, km(iseqkm2)%kmshd1, km(iseqkm2)%kmshd2, &
                           kmeshcnew(1), iinput(2) )
      iseqkm = iseqkm1

!     --- Make arrays permanent and copy to these permanent positions

      call sepinitkmeshad

      if ( associated(km(iseqkm)%ipointobst) ) &
         deallocate(km(iseqkm)%ipointobst)
      allocate(km(iseqkm)%ipointobst(iinput(2)))
      kmeshad(1) = iinput(2)
      kmeshad(6) = iseqobst
      ipointobst => km(iseqkm)%ipointobst
      ipointobst = ipointobstnew(1:iinput(2))

      if ( associated(km(iseqkm)%ielemobst) ) &
         deallocate(km(iseqkm)%ielemobst)
      allocate(km(iseqkm)%ielemobst(iinput(3)))
      kmeshad(2) = iinput(3)
      ielemobst => km(iseqkm)%ielemobst
      ielemobst = ielemobstnew(1:iinput(3))
      if ( associated(maploc%imap) ) deallocate ( maploc%imap, maploc%rmap )
      allocate ( maploc%imap(npointnew,3) )
      maploc%imap = imaplc(1:npointnew,1:3)
      allocate(maploc%rmap(npointnew,2))
      maploc%rmap = rmaplc(1:npointnew,1:2)

      maploc%iseqkm1 = iseqkm1
      maploc%iseqkm2 = iseqkm2

      if ( debug ) then
         call sepprinipobst ( ipointobst )
         call sepprinielobst ( ielemobst )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug .and. ndim==3 ) then

!     --- debug true, compute volumes of parts of mesh, store in array volold
!         and print

         kmesh => kmeshnew
         call sepfillkmesh ( kmesh )
         call mshobstvol3d ( volumes )
         voltmp = volumes(1)+volumes(2)+volumes(3)
         if ( abs(voltot-voltmp)>1d-13*abs(voltot) ) then

!        --- Error volume after subsdivision incorrect

            call erreal ( voltot, 1 )
            call erreal ( voltmp, 2 )
            call errsub ( 2268, 0, 2, 0 )

         end if
         write(irefwr,200) volumes, voltmp
200      format ( 'After subdivision:'/ &
                  'Volume of elements inside obstacle  ', d17.8/ &
                  'Volume of elements near obstacle    ', d17.8/ &
                  'Volume of elements outside obstacle ', d17.8/ &
                  'Volume of complete region           ', d17.8 )

      end if
      iseqkm = iseqkm1

!     --- Remove some subarrays of kmeshnew that are no longer valid
!         Recompute topology

      call mshobstms06 ( meshorig, meshobst )
      kmesh => kmeshnew
      call sepfillkmesh ( kmesh )

      if ( debug .or. plotobst .or. printobst ) then

!     --- debug = true, plot nodes and elements with a colour
!         First print arrays

         call plotobstacle ( debug_mshobstmesh, 0 )

         if ( debug ) then

!        --- debug true, print map

            call prinmap1 ( npointnew )

         end if

      end if

      if ( write_geomview ) then
         call prin3dobst
      end if

!     --- Remove temporary arrays

      deallocate ( kmeshcnew, kmeshnnew, &
                   iedges, ipointobstnew, ielemobstnew, &
                   imaplc, coornew, rmaplc, ielmfaces, kmeshmnew, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coornew' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms2dsh'

      end if  ! ( debug )
      call erclos ( 'mshobstms2dsh' )

      end subroutine mshobstms2dsh
