      subroutine mshobstsurf7 ( kmeshnnew, lenkmsnnew, &
                                iedges, ielmfaces, &
                                idimedges, nedgenode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 22-11-2011 Removal of iwork from list
!        version  5.1  date 15-09-2006 Debug information
!        version  5.0  date 19-05-2005 Extra parameter nedgenode
!
!   copyright (c) 2001-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new array kmeshn in the case that extra points at the surface
!     are created when a new mesh is created due to intersection with an
!     obstacle (3D, 7 point triangle)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnnew(*), &
              idimedges, nedgenode, iedges(idimedges,*), lenkmsnnew, &
              ielmfaces(4,*)

!     idimedges      i    Length of first dimension of array iedges
!                         (linear:1, quadratic: 3)
!     iedges         i    Work array of size nedges
!                         If iedges(i) = 0, no point has been created in edge i
!                         If iedges(i)>0, a point has been created in edge i, &
!                         with sequence number iedges(i)
!     ielmfaces      i    Contains faces of elements
!     kmeshnnew      o    New array kmeshn
!     lenkmsnnew    i/o   Length of array kmshnnew
!                         Ar input: declared length
!                         Ar output: computed length
!     nedgenode      i    Length of first index of edgenode
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, istartnew, isurnr, inpelm, ielem, &
              ncross, ireledge, inode1, inode2, iedge, raise(3), &
              i, nnode_edge, ipstart, edgenode(nedgenode,3), &
              nextraelms, jpstart, jnode_edge, idummy(1), idx(3), &
              ifacenode(4), iface1
      integer, allocatable :: iwork(:)
      double precision dummy(1)

!     dummy          Dummy parameter
!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     i              Counting variable
!     idummy         Dummy parameter
!     idx            Array containing the indices of the non-vertices
!     iedge          edge sequence number
!     ielem          Element number
!     iface1         ?
!     ifacenode      ?
!     inode1         First node in edge
!     inode2         Second node in edge
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istartnew      Last position used in kmeshnnew
!     isurnr         Surface sequence number
!     iwork          integer work array of length nedges
!                    if iwork(i)=0, edge i does not correspond to inode1
!                    if iwork(i)=1, edge i corresponds to inode1
!     jnode_edge     number of nodes corresponding to edge
!     jpstart        Starting address in array
!     ncross         Number of edges that are intersected
!     nextraelms     Number of extra elements
!     nnode_edge     number of nodes corresponding to edge
!     raise          Raise-operator
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHOBSTMS2D04S Subdivide quadratic triangle into two triangles
!     MSHOBSTMS2D05S Subdivide quadratic triangle into three triangles
!     PR0009         Print information of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     istart := 2*nsurfs
!     For all surfaces do
!        For all elements in surface do
!           ncross := 0
!           For ireledge := 1 (1) number_of_edges_in_element
!              iedge := edge number
!              if ( iedge contains extra point ) then
!                 ncross := ncross+1
!                 store edge information in edgenode
!           if ( ncross>0 ) then
!              create new elements in kmeshnnew
!              raise istart
!           else
!              copy old element from kmeshn into kmeshnnew
!              raise istart
!        Fill first part of kmeshn
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise / 2, 3, 1 /
      data idx / 1, 3, 5 /
! ======================================================================
!
      call eropen ( 'mshobstsurf7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstsurf7'
  1      format ( a, 1x, (10i6) )

         call pr0009 ( 0, kmeshn, dummy, idummy, &
                       idummy(1), idummy, nsurfs )

      end if
      if ( ierror/=0 ) go to 1000

!     --- istart := 2*nsurfs

      istart = 2*nsurfs
      istartnew = 2*nsurfs

      allocate ( iwork(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- For all surfaces do

      do isurnr = 1, nsurfs

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)
         if ( debug ) &
            write(irefwr,1) 'isurnr, inpelm, inelem', &
                             isurnr, inpelm, inelem
         nextraelms = 0

!        --- if ( kmeshn(istart+1)<=0 ) then

         if ( kmeshn(istart+1)<=0 ) then

!        --- Surface has not been filled, may be an obstacle surface
!            Copy old information to new surface
!            raise istart

            kmeshnnew(istartnew+1:istartnew+inpelm*inelem) = &
               kmeshn(istart+1:istart+inpelm*inelem)
            istartnew = istartnew+inpelm*inelem
            istart = istart+inpelm*inelem
            kmeshnnew(isurnr) = inpelm
            kmeshnnew(isurnr+nsurfs) = inelem

         else

!        --- For all elements in surface do

            do ielem = 1, inelem

!           --- ncross := 0

               ncross = 0

!              --- For ireledge := 1 (1) number_of_edges_in_element

               do ireledge = 1, 3

!              --- iedge := edge number
!                  Both node numbers of edge are stored in inode1 and inode2

                  inode1 = kmeshn(istart+idx(ireledge))
                  inode2 = kmeshn(istart+idx(raise(ireledge)))

!                 --- if ordered surfaces are used, sometimes 0 nodes
!                     are found in the surface

                  if ( inode1>0 .and. inode2>0 ) then

!                 --- Positions in iwork corresponding to edges connected to
!                     inode1 are set to 1

                     ipstart = nmeshl(inode1)
                     if ( ipstart==0 ) exit
                     nnode_edge = nmeshl(ipstart)
                     iwork(nmeshl(ipstart+1:ipstart+nnode_edge)) = 1

!                    --- The edge is found by finding the common edge in the set
!                        of all edges connected to inode2

                     jpstart = nmeshl(inode2)
                     if ( jpstart==0 ) exit
                     jnode_edge = nmeshl(jpstart)
                     do i = 1, jnode_edge
                        iedge = nmeshl(jpstart+i)
                        if ( iwork(iedge)==1 ) exit
                     end do

!                    --- iwork is cleared

                     iwork(nmeshl(ipstart+1:ipstart+nnode_edge)) = 0

                     if ( iedges(1,iedge)>0 ) then

!                    --- iedge contains extra point
!                        ncross := ncross+1
!                        store information in edgenode

                        ncross = ncross+1
                        edgenode(1,ncross) = nmeshc(nmeshc(iedge)+1)
                        edgenode(2,ncross) = nmeshc(nmeshc(iedge)+3)
                        edgenode(3,ncross) = iedges(2,iedge)
                        edgenode(4,ncross) = iedges(1,iedge)
                        edgenode(5,ncross) = iedges(3,iedge)

                     end if

                  end if

               end do

               inode1=kmeshn(istart+7)
               ipstart=nmeshk(inode1)
               iface1=nmeshk(ipstart+1)

               ifacenode(1:4)= ielmfaces(1:4,iface1)

               if ( debug ) &
                  write(irefwr,*) 'ielem, ncross ', ielem, ncross

               if ( ncross>0 ) then

!              --- ncross>0
!                  create new elements in kmeshnnew
!                  raise istart

                  if ( ncross==1 ) then

!                 --- Only one cross-section, i.e. two triangles

                     call mshobstms2d04s ( kmeshn(istart+1), &
                                           kmeshnnew(istartnew+1), &
                                           edgenode, ifacenode )

                     istartnew = istartnew+2*inpelm
                     nextraelms = nextraelms+1

                  else if ( ncross==2 ) then

!                 --- Two cross-section, i.e. three triangles

                     call mshobstms2d05s ( kmeshn(istart+1), &
                                           kmeshnnew(istartnew+1), &
                                           edgenode, ifacenode )

                     istartnew = istartnew+3*inpelm
                     nextraelms = nextraelms+2

                  else

!                 --- Three cross-section, i.e. four triangles
!                     Option not yet implemented

                     call errint ( inpelm, 1 )
                     call errsub ( 800, 1, 0, 0 )
                     go to 1000

                  end if

               else

!              --- No crossing found
!                  copy old element from kmeshn into kmeshnnew
!                  raise istart

                  kmeshnnew(istartnew+1:istartnew+inpelm) = &
                     kmeshn(istart+1:istart+inpelm)
                  istartnew = istartnew+inpelm

               end if
               istart = istart+inpelm

            end do

!           --- Fill first part of kmeshn

            kmeshnnew(isurnr) = inpelm
            kmeshnnew(isurnr+nsurfs) = inelem+nextraelms

         end if

      end do

!     --- Check array length

      call inichkarlen ( istartnew, lenkmsnnew, 'kmeshnnew' )

      lenkmsnnew = istartnew

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call pr0009 ( 0, kmeshnnew, dummy, idummy, &
                          idummy(1), idummy, nsurfs )
         write(irefwr,*) 'End mshobstsurf7'

      end if
      call erclos ( 'mshobstsurf7' )

      end
