      subroutine mshparnormal ( maparinv, normloc, lennormloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-02-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array normloc with normals on outer surfaces for one processor
!     3d only
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     3d
!     outer_boundary
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maparinv(*), lennormloc
      double precision normloc(3,*)

!     lennormloc     o    Computed number of nodes in array normloc
!     maparinv       i    Contains mapping from global numbering to local
!                         numbering for one block
!     normloc        o    Contains the normal in the local nodes in the
!                         same sequence as in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer locstart, isurnr, ipstart, nnodes, istart, i, &
              node, locnode, k

!     i              Counting variable
!     ipstart        Starting address in array
!     istart         First position in array
!     isurnr         Surface sequence number
!     k              Counting variable
!     locnode        Node number in local mesh
!     locstart       Counter for the number of nodes at the surfaces
!                    of the local mesh
!     nnodes         Number of nodes
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all surfaces do
!        find node number of surface in global and local mesh
!        if ( local node number>0 ) then
!           Copy normal into array normals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparnormal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparnormal'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      locstart = 0

!     --- For all surfaces do

      do isurnr = 1, nsurfs

!     --- find node number of surface in global and local mesh

         ipstart = surfnodes(isurnr)
         nnodes = surfnodes(ipstart)
         istart = ipstart-nsurfs-isurnr

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'isurnr', isurnr
            call prinin ( surfnodes(ipstart+1), nnodes, 'nodes' )

         end if  ! ( debug )

         do i = 1, nnodes

            node = surfnodes(ipstart+i)
            locnode = maparinv(node)

            if ( locnode>0 ) then

!           --- local node number>0
!               Copy normal into array normals

               locstart = locstart+1
               do k = 1, 3
                  normloc(k,locstart) = normal(k,istart+i)
               end do  ! k = 1, 3
               if ( debug ) &
                  write(irefwr,1) 'node, locnode', node, locnode

            end if  ! ( locnode>0 )

         end do  ! i = 1, nnodes

      end do  ! isurnr = 1, nsurfs
      lennormloc = locstart

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparnormal'

      end if  ! ( debug )
      call erclos ( 'mshparnormal' )

      end
