      subroutine mshpavbnquad5 ( inode2, inode, itrian1, itrian2, &
                                 iedge3, iedge5, nmeshl, nmeshd, &
                                 iwork, nmeshh, iworkedge, markedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     A quadrilateral has been found with base inode1 and inode2 and two
!     other nodes on the opposite boundary
!     One of these nodes is inode and the edge iedge connecting inode2 and inode
!     is part of itrian1
!     The other triangle in the quadrilateral is itrian2
!     It is however possible that also another triangle is coupled to iedge
!     with both nodes on the oppsoite boundary
!     If that is the case this triangle is more suited than the previous one
!     and the quadrilateral is formed by these new triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     edge
!     triangle
!     quadrilateral
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode2, inode, itrian1, itrian2, iedge3, iedge5, &
              nmeshl(*), iwork(*), nmeshh(*), iworkedge(*), nmeshd(*), &
              markedges(*)

!     iedge3        i/o   edge sequence number (boundary edge)
!     iedge5        i/o   edge sequence number (diagonal of quadrlateral)
!     inode          i    Node sequence number of node on opposite boundary
!     inode2         i    Second node in edge
!     itrian1       i/o   Sequence number of first triangle
!     itrian2       i/o   Sequence number of second triangle
!     iwork         i/o   integer work array of length nedges
!                         At start iwork must be set to zero
!                         After each call iwork is reset to 0
!     iworkedge     i/o   work array of length nedges to be used to mark
!                         edges
!                          0:  not marked yet
!                         -1:  boundary edge that has been used already
!                         -2:  diagonal of quadrilateral that may be removed
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iedge, itrian, ipstart, iedge1

!     i              Counting variable
!     iedge          edge sequence number
!     iedge1         edge sequence number
!     ipstart        Starting address in array
!     itrian         triangle sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE        Returns with edge number corresponding to 2 nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Given inode1, inode2, inode and itrian1
!     iedge := edge spanned by inode and inode2
!     itrian := other triangle coupled to iedge
!     inode4 := last node on itrian2
!     if ( inode4 = boundary node and edge inode,inode4 is not used ) then
!        itrian2 := itrian; iedge5 := iedge; iedge3 := (inode,inode4)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbnquad5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbnquad5'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Given inode1, inode2, inode and itrian1
!         iedge := edge spanned by inode and inode2

      iedge = iniedge ( inode2, inode, nmeshl, iwork )

!     --- itrian := other triangle coupled to iedge

      ipstart = nmeshh(iedge)
      if ( itrian1==nmeshh(ipstart+1) ) then

!     --- itrian := nmeshh(ipstart+2)

         itrian = nmeshh(ipstart+2)

      else

!     --- itrian := nmeshh(ipstart+1)

         itrian = nmeshh(ipstart+1)

      end if  ! ( itrian1==nmeshh(ipstart+1) )
      if ( debug ) write(irefwr,*) 'iedge, itrian1, itrian ', &
         iedge, itrian1, itrian

!     --- inode4 := last node on itrian2
!         if ( inode4 = boundary node and edge inode,inode4 is not used ) then
!           itrian2 := itrian; iedge5 := iedge; iedge3 := (inode,inode4)

      ipstart = nmeshd(itrian)
      do i = 1, 3

         iedge1 = abs(nmeshd(ipstart+1))  ! edge of trian
         if ( debug ) then
            write(irefwr,*) 'i, iedge1 ',i, iedge1
            write(irefwr,*) ' markedges(iedge1), iworkedge(iedge1) ', &
                              markedges(iedge1), iworkedge(iedge1)
         end if  ! ( debug )
         if ( markedges(iedge1)==-1 ) then

!        --- markedges(iedge) = 1, edge is boundary edge

            if ( iworkedge(iedge1)==0 ) then

!           --- Boundary edge not yet used, set new triangle and edges

               itrian2 = itrian
               iedge5 = iedge
               iedge3 = iedge1

            end if  ! ( iworkedge(iedge)==0 )
            go to 1000

         end if  ! ( markedges(iedge)==-1 )

      end do  ! i = 1, 3

1000  call erclos ( 'mshpavbnquad5' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavbnquad5'

      end if

      end
