      subroutine mshpavintedges ( nedges, markedges, nmeshc, &
                                  nbounpoints, found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all internal edges  of the mesh that go from one boundary point
!     to another boundary point by a -2
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nedges, markedges(*), nmeshc(*), nbounpoints
      logical found

!     found          o    If true at least one internal edge going from one
!                         boundary point to another boundary point has been
!                         found
!     markedges     i/o   array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nbounpoints    i    Number of boundary nodes
!                         All nodes with node number larger than nbounpoints
!                         are internal, the rest is on the boundary
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c, contains nodes of edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, ipstart, i, nodp
      logical mark

!     i              Counting variable
!     iedge          edge sequence number
!     ipstart        Starting address in array
!     mark           if true, all nodes of the edge are on the boundary
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For iedge := 1 (1) nedges do
!        if ( edge iedge is internal edge ) then
!           if ( both end nodes are on the boundary ) then
!              markedges(iedge) = -2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavintedges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavintedges'

      end if
      if ( ierror.ne.0 ) go to 1000
      found = .false.

!     --- For iedge := 1 (1) nedges do

      do iedge = 1, nedges

         if ( markedges(iedge).eq.0 ) then

!        --- edge iedge is internal edge
!            if ( both end nodes are on the boundary ) then
!               markedges(iedge) = -2
!            At this moment we restrict ourselves to linear elements

            ipstart = nmeshc(iedge)  ! start in nmeshc
            mark = .true.
            do i = 1, 2
               nodp = nmeshc(ipstart+i)  ! nodal point
               if ( nodp.gt.nbounpoints ) mark = .false.  ! not on boundary
            end do  ! i = 1, 2
            if ( mark ) then
               markedges(iedge) = -2
               found = .true.
            end if  ! ( mark )

         end if  ! ( markedges(iedge).eq.0 )

      end do  ! iedge = 1, nedges

1000  call erclos ( 'mshpavintedges' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavintedges'

      end if

      end
