      subroutine mshpavsub1tria ( iedge, nmeshc, itrian, kmeshc, nel, &
                                  nodeextra )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide one triangle into 2, because one of the edges is splitted
!     into 2 parts
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedge, nmeshc(*), itrian, kmeshc(3,*), nel, nodeextra

!     iedge          i    edge sequence number of edge that must be splitted
!     itrian         i    Sequence number of triangle
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in the topology
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nodeextra      i    Node number of extra node on edge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, edgenode(3), trianglenodenw(6), i

!     edgenode       Array with all nodes on the edge
!                    edgenode(1) first node on edge iedge
!                    edgenode(2) last node on edge iedge
!                    edgenode(3) new node on edge iedge
!     i              Counting variable
!     ipstart        Starting address in array
!     trianglenodenw Array of nodal point numbers of the corners of
!                    the new subtriangles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS2D04  Subdivide triangle into two triangles
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavsub1tria' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavsub1tria'
         write(irefwr,*) 'nel, itrian, iedge ', nel, itrian, iedge
         call prinin ( kmeshc(1,itrian), 3, 'element before' )

      end if
      if ( ierror.ne.0 ) go to 1000

      ipstart = nmeshc(iedge)           ! edge to be subdivided
      edgenode(1) = nmeshc(ipstart+1)   ! first node on iedge
      edgenode(2) = nmeshc(ipstart+2)   ! last node on iedge
      edgenode(3) = nodeextra           ! extra node on iedge

!     --- subdivide

      call mshobstms2d04 ( kmeshc(1,itrian), trianglenodenw, &
                           edgenode )

      if ( debug ) then
         call prinin ( edgenode, 3, 'edgenode ')
         call prinin ( trianglenodenw, 6, 'trianglenodenw')
      end if  ! ( debug )

!     --- Store one of the triangles in old place and one in new part

      nel = nel+1
      do i = 1, 3
         kmeshc(i,itrian) = trianglenodenw(i)
         kmeshc(i,nel) = trianglenodenw(i+3)
      end do  ! i = 1, 3

1000  call erclos ( 'mshpavsub1tria' )

      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshc(1,itrian), 3, 'element 1 after' )
         call prinin ( kmeshc(1,nel), 3, 'element 2 after' )
         write(irefwr,*) 'End mshpavsub1tria'

      end if

      end
