      subroutine mshpavsub3tria ( iedgediv, nmeshc, itrian, kmeshc, nel, &
                                  inodediv, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 19-05-2005 New calls to subroutines
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into 3 triangles because 2 edges have been splitted
!     into 2 parts
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedgediv(2), nmeshc(*), itrian, kmeshc(3,*), nel, &
              inodediv(2)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iedgediv       i    Contains edge numbers of the 2 edges to be subdivided
!     inodediv       i    Contains extra node numbers on the 2 edges to be
!                         subdivided
!     itrian         i    Sequence nuber of triangle
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel           i/o   number of elements in the topology
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, edgenode(3,2), trianglenodenw(9), i

!     edgenode       Array with all nodes on the edges to be intersected
!                    edgenode(1,i) first node on edge
!                    edgenode(2,i) last node on edge
!                    edgenode(3,i) new node on edge
!     i              Counting variable
!     ipstart        Starting address in array
!     trianglenodenw Array of nodal point numbers of the corners of
!                    the new subtriangles
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMS2D05  Subdivide triangle into three triangles
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavsub3tria' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavsub3tria'
         write(irefwr,*) 'nel, itrian ', nel, itrian
         call prinin ( kmeshc(1,itrian), 3, 'element before' )
         call prinin ( iedgediv, 2, 'iedgediv' )
         call prinin ( inodediv, 2, 'inodediv' )

      end if
      if ( ierror/=0 ) go to 1000

      ipstart = nmeshc(iedgediv(1))     ! edge to be subdivided
      edgenode(1,1) = nmeshc(ipstart+1) ! first node on iedge
      edgenode(2,1) = nmeshc(ipstart+2) ! last node on iedge
      edgenode(3,1) = inodediv(1)       ! extra node on iedge

      ipstart = nmeshc(iedgediv(2))     ! edge to be subdivided
      edgenode(1,2) = nmeshc(ipstart+1) ! first node on iedge
      edgenode(2,2) = nmeshc(ipstart+2) ! last node on iedge
      edgenode(3,2) = inodediv(2)       ! extra node on iedge

      if ( debug ) then
         call prinin ( edgenode(1,1), 3, 'edgenode_1' )
         call prinin ( edgenode(1,2), 3, 'edgenode_2' )
      end if  ! ( debug )

!     --- subdivide

      call mshobstms2d05 ( kmeshc(1,itrian), trianglenodenw, &
                           edgenode, coor, 3 )

!     --- Store one of the triangles in old place and one in new part

      nel = nel+1
      do i = 1, 3
         kmeshc(i,itrian) = trianglenodenw(i)
         kmeshc(i,nel) = trianglenodenw(i+3)
         kmeshc(i,nel+1) = trianglenodenw(i+6)
      end do  ! i = 1, 3
      nel = nel+1

1000  call erclos ( 'mshpavsub3tria' )

      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshc(1,itrian), 3, 'element 1 after' )
         call prinin ( kmeshc(1,nel-1), 3, 'element 2 after' )
         call prinin ( kmeshc(1,nel), 3, 'element 3 after' )
         write(irefwr,*) 'End mshpavsub3tria'

      end if

      end
