      subroutine mshprojend ( coorol, coornw, icurnr, ipoint, kmeshm, &
                              coorold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     correct end points for free separation points by projection
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  icurnr, ipoint, kmeshm(*)
      double precision coorol(2,*), coornw(2,*), coorold(2,*)

!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     coorold        i    Contains the coordinates of the old mesh
!     icurnr         i    Curve sequence number
!     ipoint         i    Point at boundary that must be adapted
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, k1, k2, icurvs(1), nodp, numnodes, k
      double precision , allocatable :: curves(:,:)

!     curves         array containing the coordinates of curve icurnr
!     error          Return value of allocate or deallocate
!     icurvs         arrayof length 1, containing the number of nodes in icurnr
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     numnodes       Number of nodal points in icurnr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH073         Correct end points of free boundary
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshprojend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshprojend'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy coordinates of curve icurnr into array curves

      istar3 = kmeshm(2)-1   ! starting address of node numbers
      k1 = kmeshm(4+icurnr)
      k2 = kmeshm(5+icurnr)-1
      numnodes = k2-k1+1
      if ( debug ) &
         write(irefwr,1) 'icurnr, k1, k2, numnodes', &
                          icurnr, k1, k2, numnodes

      allocate(curves(2,numnodes), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( 2*numnodes, 1 )
         call errint ( error, 2 )
         call errchr ( 'curves', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

      icurvs(1) = numnodes
      do k = 1, numnodes
         nodp = kmeshm(istar3+k+k1-1)
         curves(1:2,k) = coorold(1:2,nodp)
      end do  ! k = k1, k2
      if ( debug ) call prinrl1 ( curves, numnodes, 2, 'curves' )
      call msh073 ( coorol, coornw, 1, ipoint, icurvs, curves )

 900  deallocate ( curves, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'curves', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  call erclos ( 'mshprojend' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshprojend'

      end if  ! ( debug )

      end
