      subroutine mshrdmeshdim ( irefc, nboun, ihelp, ivers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read first records form file meshoutput
!     It concerns the dimension parameters and some comment records
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, nboun, ihelp(10), ivers

!     ihelp          o    Contains the input written by MSH059 on the second
!                         record after nvolms
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     ivers          o    Version sequence number
!     nboun          o    Number of boundary points at an element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ios
      character (len=80) text

!     i              Counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     765   End of file Reached of file containing mesh output
!     766   Read error in file containing mesh output
! **********************************************************************
!
!                       PSEUDO CODE
!
!     765   End of file Reached of file containing mesh output
!     766   Read error in file containing mesh output
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdmeshdim' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdmeshdim'
         call ertrace
         write(irefwr,1) 'irefc', irefc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize array ihelp

      do i = 1, 9
         ihelp(i) = 0
      end do

!     --- read dimension parameters

      ndim = 10000
      if ( irefc>=0 ) then
         read (irefc,*,iostat=ios) ndim, npelm, nboun, ishape, &
                          nelgrp, npoint, nelem, nuspnt, ncurvs, nsurfs
      else
         read(-irefc,iostat=ios) ndim, npelm, nboun, ishape, &
                      nelgrp, npoint, nelem, nuspnt, ncurvs, nsurfs
      end if
      if ( debug ) &
         write(irefwr,1) 'ndim, npelm, nboun, ishape', &
                          ndim, npelm, nboun, ishape

      if ( ndim<0 ) then

!     --- ndim<0, so 10 extra positions should be read

         if ( irefc>=0 ) then
            read ( irefc, *, iostat = ios ) nvolms, (ihelp(i),i=1,9)
         else
            read(-irefc, iostat = ios) nvolms, (ihelp(i),i=1,9)
         end if
         ndim = -ndim

      end if
      if ( debug ) then
         write(irefwr,1) 'nvolms', nvolms
         call prinin ( ihelp, 9, 'ihelp' )
      end if  ! ( debug )

      ivers = ihelp(9)
      if ( debug ) write(irefwr,1) 'ivers', ivers
      if ( ivers>=7 .and. ivers<1000 ) then

!     --- Read and skip comment records

         if ( irefc>=0 ) then
            read (irefc,100,iostat=ios) text
            if ( debug ) write(irefwr,*) text
            read (irefc,100,iostat=ios) text
            if ( debug ) write(irefwr,*) text
            read (irefc,100,iostat=ios) text
            if ( debug ) write(irefwr,*) text
            if ( nelgrp>1 ) then
               read (irefc,100,iostat=ios) text
               if ( debug ) write(irefwr,*) text
            end if  ! ( nelgrp>1 )
100         format ( a )
         else
            read(-irefc,iostat=ios) text
            if ( debug ) write(irefwr,*) text
            read(-irefc,iostat=ios) text
            if ( debug ) write(irefwr,*) text
            read(-irefc,iostat=ios) text
            if ( debug ) write(irefwr,*) text
            if ( nelgrp>1 )  then
               read(-irefc,iostat=ios) text
               if ( debug ) write(irefwr,*) text
            end if  ! ( nelgrp>1 )
         end if
      end if

      if ( ios>0 ) then

!     --- ios>0, Error in reading file

         call errint ( ios, 1 )
         call errsub ( 766, 1, 0, 0 )
         go to 1000

      else if ( ios<0 ) then

!     --- ios>0, End of file reached when reading file 10

         call errsub ( 765, 0, 0, 0)
         go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrdmeshdim'

      end if  ! ( debug )
      call erclos ( 'mshrdmeshdim' )

      end
