      subroutine mshrect_grid3d ( coorrect, nx, ny, nz, ox, oy, oz, &
                                  dx, dy, dz )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create coordinates for rectangular grid in R^2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     coordinate
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz
      double precision coorrect(3,nx,ny,nz), ox, oy, oz, dx, dy, dz

!     coorrect       o    Coordinates of rectangle
!     dx             i    step size in x-direction
!     dy             i    step size in y-direction
!     dz             i    step size in z-direction
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     ox             i    x-coordinate of origin
!     oy             i    y-coordinate of origin
!     oz             i    z-coordinate of origin
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision x, y, z

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     x              x co-ordinate of point
!     y              y co-ordinate of point
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for i = 0 (1) nx-1 do
!        x := ox+i*dx
!        for j = 0 (1) ny-1 do
!           y := oy+j*dy
!           for k = 0 (1) nz-1 do
!              z := oz+k*dz
!              coorrect(1,i+1,j+1,k+1 ) := x;  coorrect(2,i+1,j+1,k+1 ) := y
!              coorrect(3,i+1,j+1,k+1 ) := z
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrect_grid3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrect_grid3d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- for i = 0 (1) nx-1 do
!           x := ox+i*dx

      do i = 0, nx-1

         x = ox+i*dx

!        --- for j = 0 (1) ny-1 do
!               y := oy+j*dy

         do j = 0, ny-1

            y = oy+j*dy

!           --- for k = 0 (1) nz-1 do
!                  z := oz+k*dz
!                  coorrect(1,i+1,j+1,k+1 ) := x;  coorrect(2,i+1,j+1,k+1 ) := y
!                  coorrect(3,i+1,j+1,k+1 ) := z

            do k = 0, nz-1

               z = oz+k*dz

               coorrect(1,i+1,j+1,k+1) = x
               coorrect(2,i+1,j+1,k+1) = y
               coorrect(3,i+1,j+1,k+1) = z

            end do  ! k = 0, nz-1

         end do  ! j = 0, ny-1

      end do  ! i = 0, nx-1

1000  call erclos ( 'mshrect_grid3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrect_grid3d'

      end if

      end
