      subroutine mshrediscur1 ( nnodes, numnodes, icurnr, coornew, &
                                coorold, nodes, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find number of nodes of the old curve that must be used to
!     define the spline
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, numnodes, icurnr, nodes(*), ndim
      double precision coornew(ndim,*), coorold(ndim,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     coorold        i    Array containing the old coordinates of the nodes
!     icurnr         i    Curve sequence number (provided with sign)
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers of the curve
!     numnodes       o    Number of nodal points that can be used to define the
!                         spline
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ifound, i, istart, idir, nodp1, nodp2
      double precision x(ndim), alpha, tang(ndim), norm

!     alpha          Defines the projection of x onto an element x_1 - x_2
!                    if<0, the point x is before the element
!                    if between 0 and 1 it is in the element
!                    otherwise after the element
!     i              Counting variable
!     idir           Direction of curve
!     ifound         First relative node number of curve to be used
!     istart         First position in curve
!     nodp1          Node sequence number (first node)
!     nodp2          Node sequence number (second node)
!     norm           Lenght of tangential vector
!     tang           Tangential vector
!     x              contains coorindates of new start point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2991   Internal error, no mapping possible
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ifound := 0
!     For i := 1 (1) nnodes-1 do
!        Project end point onto element i
!        If ( projection before end point of element ) then
!           ifound := i
!           jump out of loop
!     numnodes := nnodes+1-ifound
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrediscur1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrediscur1'
         write(irefwr,1) 'nnodes', nnodes
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ifound := 0

      if ( icurnr>0 ) then

!     --- icurnr>0, positive direction

         istart = 1                     ! start of curve
         idir = 1                       ! direction of curve

      else

!     --- icurnr<0, negative direction

         istart = nnodes                ! start of curve
         idir = -1                      ! direction of curve

      end if  ! ( icurnr>0 )
      if ( debug ) &
         write(irefwr,1) 'icurnr, idir, istart', icurnr, idir, istart

      x = coornew(1:ndim,nodes(istart))   ! coordinates of start point
      if ( debug ) write(irefwr,2) 'x', x

!     --- For i := 1 (1) nnodes-1 do

      do i = 1, nnodes-1

!     --- Project end point onto element i

         nodp1 = nodes(istart)    ! first node of element
         istart = istart+idir
         nodp2 = nodes(istart)    ! second node of element

!        --- Compute tangential vector

         tang = coorold(1:ndim,nodp2)-coorold(1:ndim,nodp1)
         norm = dot_product(tang,tang) !  length of vector squared
         alpha = dot_product(x-coorold(1:ndim,nodp1),tang)/norm
         if ( debug ) then
            write(irefwr,1) 'nodp1, nodp2', nodp1, nodp2
            write(irefwr,2) 'alpha', alpha
         end if  ! ( debug )

         if ( alpha<1d0 ) then

!        --- projection before end point of element
!            ifound := i
!            jump out of loop

            ifound = i
            go to 100

         end if  ! ( alpha<1 )

      end do  ! i = 1, nnodes-1

!     --- if the loop ends without finding a node there is some
!         serious error

      call errint ( abs(icurnr), 1 )
      call errsub ( 2991, 1, 0, 0 )
      go to 1000

!     --- numnodes := nnodes+1-ifound

100   numnodes = nnodes+1-ifound

1000  call erclos ( 'mshrediscur1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numnodes', numnodes
         write(irefwr,*) 'End mshrediscur1'

      end if  ! ( debug )

      end
