      subroutine mshreflocalent ( kmeshcnew, coornew, iedges, iseqkm2, &
                                  elements,  refarray, lenkmscnew, intnodes, &
                                  parentnodes, refmapold, refmapnew, &
                                  ireflvl, nodeextrael, npoint_basis, &
                                  enthwork, lnintnodes, kmeshnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 25-04-2012 New call to mshreflocal
!        version  1.4  date 05-03-2012 Clear kmesh part h
!        version  1.3  date 26-10-2005 extra parameter lnintnodes (JB)
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine an existing mesh locally and store the result in a
!     new array kmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), iedges(*), kmeshnew, &
              elements(*), intnodes(*), &
              lenkmscnew, ireflvl, nodeextrael(*), &
              parentnodes(*), refmapold(*), refmapnew(*), refarray(*), &
              npoint_basis, enthwork(*), lnintnodes, iseqkm2
      double precision coornew(*)

!     coornew        o    Array containing the new coordinates of the nodes
!     elements      i/o   Work array of length nelem
!                         elements(ielem) edges of element ielem are subdivided
!     enthwork      i/o   work array to keep track of added nodes
!     iedges        i/o   Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     intnodes      i/o   integer array containing all nodes at the interface
!                         of an element group
!     ireflvl        i    Current level of refinement
!     iseqkm2        i    Sequence number with respect to km of second mesh
!     kmeshcnew      o    New kmesh part c
!     kmeshnew       i    New array kmesh
!     lenkmscnew     o    Length of array kmshcnew
!     lnintnodes     i    Declared length of array intnodes
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     npoint_basis   i    npoint of the basis mesh
!     parentnodes   i/o   Array with for all added nodes the node
!                         numbers of the parents
!     refarray       i    Reference array containing the required level
!                         of refinement per element of the basis mesh
!     refmapnew     i/o   Mapping array
!     refmapold      i    Mapping array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointnew, nsubdiv
      integer, allocatable :: refedges(:)

!     npointnew      Sequence number of last generated node
!     nsubdiv        number of extra nodes that are created
!     refedges       Contains the edge numbers of the edges that
!                    must be subdivided
!                    Length: nsubdiv
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOORREF     Extend array coornew with the coordinates of the extra
!                    points that are created by subdividing the edges
!     MSHELEMREF     Create a new topology constisting of the set kmeshc/g/t
!                    in case of a local refinement
!     MSHMARKEDGES   Mark all edges that must be sudivided
!     MSHMARKELEMS   Mark all elements with the number of edges
!                    that are subdivided
!     MSHOBSTMS022   Copy part of kmesh to kmeshnew n case of obstacles
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     SEPPRINKMESHC  print kmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy part of kmesh to kmeshnew
!     Copy coordinates from coor to coornew
!     Mark all edges of the elements to be subdivided
!     Mark all elements that must be subdivided
!     Create new nodes and put in array coornew
!     Create new elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreflocalent' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreflocalent'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinrl1 ( coor, npoint, ndim, 'coor' )

      allocate ( refedges(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'refedges' )
      if ( ierror/=0 ) go to 1000

!     --- Copy part of kmesh to kmeshnew
!         Copy coordinates from coor to coornew
      call mshobstms022 ( kmeshnew, iseqkm2, coornew )

      km(iseqkm2)%kmesh(22) = 0
      km(iseqkm2)%kmesh(34) = 0
      km(iseqkm2)%kmesh(35) = 0
      km(iseqkm2)%kmesh(62) = 0

      if ( ierror/=0 ) go to 1000

!     --- Mark all edges of the elements to be subdivided

      if (debug) then
         call prinin(nmeshd, 20, 'nmeshd')
         write(irefwr,*) 'nelem, npoint', nelem, npoint
      end if

      call mshmarkedges ( iedges, nedges, refarray, nmeshd, nelem, &
                          nsubdiv, npoint, ireflvl, refedges )
      if ( ierror/=0 ) go to 1000

      if (debug) call prinin(iedges, nedges, 'iedges')

!     --- Mark all elements that must be subdivided

      call mshmarkelems ( iedges, elements, nmeshd, nelem )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( elements, nelem, 'elements' )

!     --- Create new nodes and put in array coornew

      call mshcoorref ( iedges, nedges, coornew, nmeshc, &
                        ndim, .false. )
      if ( ierror/=0 ) go to 1000
      npointnew = npoint+nsubdiv
      if ( debug ) then
         call prinrl1 ( coornew, npointnew, ndim, 'coornew' )
         call sepprinkmeshc
      end if  ! ( debug )

!     --- Create new elements

      call mshelemref ( elements, iedges, iseqkm2, kmeshcnew, &
                        coornew, lenkmscnew, intnodes, parentnodes, &
                        refmapold, refmapnew, ireflvl, refarray, &
                        nodeextrael, .true., npoint_basis, enthwork, &
                        lnintnodes )

      if ( ierror/=0 ) go to 1000
      km(iseqkm2)%kmesh(8) = npointnew

      deallocate ( refedges, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'refedges' )

1000  if ( debug ) then

!     --- Debug information

    !     call  pr0003 ( kmeshnew, kmeshcnew )
         write(irefwr,*) 'End mshreflocalent'

      end if
      call erclos ( 'mshreflocalent' )

      end
