      subroutine mshreflocchk1 ( kmeshcnew, kmeshnnew, &
                                 coornew, kmeshmnew, lencoornew, &
                                 lenkmscnew,  lenkmsnnew, lenkmsmnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 22-11-2011 New parameter list
!        version  2.0  date 20-09-2006 Extra parameter
!        version  1.0  date 20-10-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check control integers around the temporary arrays in refineloc
!     In this way overwriting of arrays can be checked
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(0:*), kmeshnnew(0:*), &
              kmeshmnew(0:*), lenkmsmnew, &
              lencoornew, lenkmscnew, lenkmsnnew
      double precision coornew(0:*)

!     coornew        i    Array containing the new coordinates of the nodes
!     kmeshcnew      i    New kmesh part c
!     kmeshmnew      i    New array kmeshm
!     kmeshnnew      i    New array kmeshn
!     lencoornew     i    Estimated length of array coornew
!     lenkmscnew     i    Length of array kmshcnew
!     lenkmsmnew     i    Length of array kmshmnew
!     lenkmsnnew     i    Length of array kmshnnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRLEADING     Check leading control integer in case of temporary array
!     ERRLEADINGD    Check leading control integer in case of real array
!     ERRTRAILING    Check trailing control integer in case of temporary array
!     ERRTRAILINGD   Check trailing control integer in case of real array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreflocchk1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreflocchk1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call errleading  ( kmeshcnew(0), 10002, 'kmscnew' )
      call errtrailing ( kmeshcnew(lenkmscnew+1), &
                         10002, 'kmscnew' )

      call errleading  ( kmeshnnew(0), 10003, 'kmsnnew' )
      call errtrailing ( kmeshnnew(lenkmsnnew+1), 10003, 'kmsnnew' )

      call errleadingd  ( coornew(0), 10006, 'coornew' )
      call errtrailingd ( coornew(lencoornew+1), 10006, 'coornew' )

      call errleading  ( kmeshmnew(0), 10011, 'kmsmnew' )
      call errtrailing ( kmeshmnew(lenkmsmnew+1), 10011, 'kmsmnew' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreflocchk1'

      end if
      call erclos ( 'mshreflocchk1' )

      end
