      subroutine mshrefmapping1 ( npoint_basis, tempnew, tempold, &
                                  nelembasis, refineold, refinenew, &
                                  reflvl, intnodesold, intnodesnew, &
                                  nodeextrael, parentnodes )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.1  date 29-09-2005 Debug: ipold --> ipnew in block
!                                      refineold(i)<refinenew(i)
!        version  1.0  date 25-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map the solution vector from the old mesh to the new mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intnodesnew(*), intnodesold(*), nelembasis, &
              refineold(*), refinenew(*), reflvl, &
              npoint_basis, parentnodes(2,*), nodeextrael(0:*)
      double precision tempold(*), tempnew(*)

!     intnodesnew   i/o   array of added internal nodes
!     intnodesold   i/o   array of added internal nodes
!     nelembasis     i    number of elements of the basis mesh
!     nodeextrael    i    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
!     npoint_basis   i    Number of nodes of the basis mesh
!     parentnodes   i/o   Array with for all added nodes the node
!                         numbers of the parents
!     refinenew      i    refinement array for the new mesh
!     refineold      i    refinement array for the old mesh
!     reflvl         i    Level of refinement
!     tempnew        o    Temperature at the current time level
!     tempold        i    Temperature at the previous time level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, parents(2), ipnew, ipold, j, npoints, npoint_new

!     i              Counting variable
!     ipnew          Starting location in array intnodesnew
!     ipold          Starting location in array intnodesold
!     j              Counting variable
!     npoint_new     Number of points of the new mesh
!     npoints        Number of points
!     parents        Integer array containing the nodal numbers in the
!                    new mesh that are the 'parents' of an added node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     copy all values for the nodes of the basis mesh
!     for each element of the basis mesh and all points of the old mesh
!     that are not mapped yet
!        copy nodes in case refinement old mesh>=refinement new mesh
!        interpolate nodes when refinement new mesh<refinement old mesh
!     end
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefmapping1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefmapping1'
         write(irefwr,1) 'nelembasis, npoint_basis', nelembasis, npoint_basis
         call prinin(refineold, nelembasis, 'refineoldloc')
         call prinin(refinenew, nelembasis, 'refinenewloc')
         call prinin(nodeextrael, 5, 'nodeextrael')
         call prinin(intnodesold, 750, 'intnodesold')
         call prinin(intnodesnew, 750, 'intnodesnew')
         call prinin(parentnodes, 750, 'parentnodes')
         call prinrl ( tempold, npoint_basis, 'tempold' )
  1      format ( a, 1x, (10i6) )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Both the new as well as the old refined mesh contain all the nodes
!         of the basis mesh. We can thus perform a one-on-one mapping of
!         these nodes.

      do i = 1, npoint_basis
         tempnew(i) = tempold(i)
      end do

!     --- For each of the elements of the basis mesh the remaining mapping
!         is derived based on the number of internal nodes, which is linked
!         to the level of refinement of the basis mesh element.

      npoint_new = npoint_basis

      do i = 1, nelembasis

!     --- obtain the starting locations of the arrays intnodes* &

         ipnew = intnodesnew(i)
         ipold = intnodesold(i)

         if (debug) print *, 'ipnew, ipold', ipnew, ipold

!        --- In case the level of refinement in the old mesh is larger or equal
!            to that of the new mesh, interpolation is not needed.

         if (refineold(i)>=refinenew(i)) then

!        --- set the number of added nodes

            if (debug) print *, 'refinenew(',i,')', refinenew(i)
            npoints = nodeextrael(min(refinenew(i), reflvl))
            if (debug) print *, '>= npoints', npoints

            do j = 1, npoints
               tempnew(intnodesnew(ipnew+j-1)) = &
                       tempold(intnodesold(ipold+j-1))
               npoint_new = npoint_new + 1
               if (debug) print *, 'tempnew(', &
                                    intnodesnew(ipnew+j-1),')', &
                                    tempnew(intnodesnew(ipnew+j-1)), &
                                    'j, intnodesold(ipold+j-1) ', &
                                    j, intnodesold(ipold+j-1)
            end do
         end if
      end do

!     --- npoint_new = npoint_basis

      do i = 1, nelembasis

!     --- obtain the starting locations of the arrays intnodes* &

         ipnew = intnodesnew(i)
         ipold = intnodesold(i)

         if (debug) print *, 'ipnew, ipold', ipnew, ipold

         if (refineold(i)<refinenew(i)) then

!        --- refineold(i)<refinenew(i)

            npoints = nodeextrael(min(refineold(i), reflvl))
            if (debug) print *, '< npoints', npoints

            do j = 1, npoints
                   ! ipold --> ipnew in next line!
               tempnew(intnodesnew(ipnew+j-1)) = &
                                    tempold(intnodesold(ipold+j-1))
               npoint_new = npoint_new + 1
               if (debug) print *, 'tempnew(', &
                   ! ipold --> ipnew in next line!
                                    intnodesnew(ipnew+j-1),')', &
                                    tempold(intnodesold(ipold+j-1))
            end do

            ipnew = ipnew+npoints

!           --- the remaining internal nodes of the new mesh need to be inter- &
!               polated. First compute the remaining number of nodes.

            npoints = nodeextrael(min(refinenew(i), reflvl)) - &
                         nodeextrael(min(refineold(i), reflvl))

            if (debug) print *, '< npoints 2', npoints

!           --- perform the interpolation of the remaining nodes

            do j = 0, npoints-1
               parents(1) = parentnodes(1,intnodesnew(ipnew+j) - &
                            npoint_basis)
               parents(2) = parentnodes(2,intnodesnew(ipnew+j) - &
                            npoint_basis)
               tempnew(intnodesnew(ipnew+j)) = 0.5d0*( &
                       tempnew(parents(1)) + tempnew(parents(2)) )
               npoint_new = npoint_new + 1
               if (debug) then
                  print *, 'parents', parents(1), parents(2)
                  print *,'tempnew(parents)', tempnew(parents(1)), &
                                              tempnew(parents(2))
                  print *,'tempnew(',intnodesnew(ipnew+j),')', &
                           tempnew(intnodesnew(ipnew+j))
                  print *, 'ielembasis, tempnew(114)', i, tempnew(114)
               end if
            end do

         end if
      end do

1000  call erclos ( 'mshrefmapping1' )
      if ( debug ) then

!     --- Debug information

         print *, 'npoint_new', npoint_new
         call prinrl(tempnew, npoint_new, 'tempnew (mshrefmapping1)')
         call prinrl(tempold, npoint_new, 'tempold (mshrefmapping1)')
         call prinin(refineold, nelembasis, 'refineoldloc')
         call prinin(refinenew, nelembasis, 'refinenewloc')

         write(irefwr,*) 'End mshrefmapping1'

      end if

      end
