      subroutine mshrefpar11 ( iseqkm2, kmeshcglobnew, kmeshccopy, &
                               kmeshgcopy, lenkmeshcglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new kmeshcglob for the parallel case if refinement
!     is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     topology
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm2, kmeshcglobnew(*), kmeshccopy(iacnodes,*), &
              kmeshgcopy(iacnodes,*), lenkmeshcglob

!     iseqkm2        i    Sequence number with respect to km of second mesh
!     kmeshccopy     i    Array of size iacnodes x lenkmeshcglob containing
!                         the arrays kmeshc of each block
!                         The global numbering is used
!     kmeshcglobnew  o    Array kmeshcglob contains the topology of the global
!                         mesh in case of parallel computing
!                         The first nelgrp positions contain the number of
!                         elements per element group
!                         The next positions contain the node numbers of the
!                         elements
!     kmeshgcopy     i    Array of size iacnodes x nelgrp containing
!                         the arrays kmeshg of each block
!     lenkmeshcglob i/o   Length of array kmeshcglob
!                         At input: declared length
!                         At output: computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iblock, ielgrp, ipkmscloc, &
              inpelm, lenkmeshc, ipkmscglob, startblock(iacnodes)

!     iblock         Block sequence number
!     ielgrp         element group number
!     inpelm         number of points per element
!     ipkmscglob     Starting address in array kmeshcglob
!     ipkmscloc      Starting address of array KMESH part c
!     lenkmeshc      length of array kmeshc
!     startblock     Array of size iacnodes
!                    Contains starting addresses in kmeshccopy for each
!                    next element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnpelmelgrp

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3104   shape numbers of lower dimension have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar11'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nelgrp==1 ) then

!     --- nelgrp = 1, nelem elements

         kmeshcglobnew(1) = sum(kmeshgcopy(1:iacnodes,1))  ! nelem

      else

!     --- nelgrp > 1, more element groups
!         First check the shapes of the elements

         if ( km(iseqkm2)%kmesh(20)/=0 ) then

!        --- kmeshnew(20) # 0, hence different shapes

            do ielgrp = 1, nelgrp

               ishape = km(iseqkm2)%kmeshf(ielgrp)
               if ( ndim==2 .and. ishape<3 .or. &
                    ndim==3 .and. ishape<11 ) then

!              --- Shape numbers of lower dimension have not yet been
!                  implemented

                  call errint ( ishape, 1 )
                  call errint ( ndim, 2 )
                  call errsub ( 3104, 2, 0, 0 )
                  go to 1000

               end if  ! ( ndim==2 .and. ishape<3 .or. ...

            end do  ! i = 1, nelgrp

         end if  ! ( kmeshnew(20)/=0 )

         do ielgrp = 1, nelgrp
            kmeshcglobnew(ielgrp) = sum(kmeshgcopy(1:iacnodes,ielgrp))  ! nelem
         end do  ! ielgrp = 1, nelgrp

      end if  ! ( nelgrp==1 )

      if ( debug ) call prinin ( kmeshcglobnew, nelgrp, 'kmeshcglob/1' )

!     --- Store new kmesh part c

      ipkmscglob = nelgrp+1
      startblock = 1

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         do iblock = 1, iacnodes

            lenkmeshc = inpelm*kmeshgcopy(iblock,ielgrp)

!           --- Check array length

            call inichkarlen ( ipkmscglob+lenkmeshc, lenkmeshcglob, &
                               'kmeshcglob' )
            if ( ierror/=0 ) go to 1000

            ipkmscloc = startblock(iblock)
            kmeshcglobnew(ipkmscglob:ipkmscglob+lenkmeshc) = &
               kmeshccopy(iblock,ipkmscloc:ipkmscloc+lenkmeshc)
            ipkmscloc = ipkmscloc+lenkmeshc
            startblock(iblock) = ipkmscloc
            ipkmscglob = ipkmscglob+lenkmeshc

         end do  ! iblock = 1, iacnodes

      end do  ! ielgrp = 1, nelgrp

      lenkmeshcglob = ipkmscglob

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefpar11'

      end if  ! ( debug )
      call erclos ( 'mshrefpar11' )

      end
