      subroutine mshrefupdatend ( ndim, nodeextrael, maxreflvl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodeextrael with number of extra nodes as function
!     of the refinement level
!     Let k denote the level of refinement, then the number of extra
!     nodes is given by:
!     1D           2^k - 1
!     2D           (2^k+1)*(2^(k-1)+1) - 3
!     3D           (2^k+1)*(2^k+2)*(2^k+3)/6 - 4
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     mesh
!     enthalpy
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, maxreflvl, nodeextrael(0:maxreflvl)

!     maxreflvl      i    Maximum value of number of refinements
!     ndim           i    Dimension of the space.
!     nodeextrael    o    Array of size maxreflvl that contains the number
!                         of extra nodes per element defined as function
!                         of the refinement level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefupdatend' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefupdatend'

      end if
      if ( ierror/=0 ) go to 1000
      if ( maxreflvl/=3 ) then

!     --- maxrefvl out of range

         call errint ( maxreflvl, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errchr ( 'maxreflvl', 1 )
         call errsub ( 1378, 3, 0, 1 )
         go to 1000

      end if  ! ( maxreflvl>3 )

      nodeextrael(0) = 0

      if ( ndim==1 ) then

!     --- ndim = 1

         nodeextrael(1) = 1
         nodeextrael(2) = 3
         nodeextrael(3) = 7

      else if ( ndim==2 ) then

!     --- ndim = 2

         nodeextrael(1) = 3
         nodeextrael(2) = 12
         nodeextrael(3) = 42

      else ! ( ndim==3 )

!     --- ndim = 3

         nodeextrael(1) = 6
         nodeextrael(2) = 31
         nodeextrael(3) = 161

      end if  ! ( ndim==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefupdatend'

      end if
      call erclos ( 'mshrefupdatend' )

      end
