      subroutine mshshift3d1 ( iworkpoint, skipnod )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark nodes that ly on surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh
!     obstacle
!     intersection
!     tetrahedron
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iworkpoint(*), skipnod(*)

!     iworkpoint     i    Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodenr, i, nnodes
      integer, allocatable :: nodes(:), isurfaces(:)

!     i              Counting variable
!     isurfaces      Array containing the surfaces from which information
!                    must be found (Here all surfaces)
!     nnodes         Number of nodes
!     nodenr         Node sequence number
!     nodes          Contains the node numbers that must be filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSURFSNOD1   Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift3d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift3d1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- allocate nodes array

      allocate ( nodes(npoint), isurfaces(nsurfs), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nsurfs, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- Find all nodes in the surfaces and store in

      do i = 1, nsurfs
         isurfaces(i) = i
      end do  ! i = 1, nsurfs

      call sepsurfsnod1 ( nsurfs, isurfaces, nodes, nnodes )

      do i = 1, nnodes

         nodenr = nodes(i)
         if ( nodenr>0 ) then

!        --- Node exists

            if ( iworkpoint(nodenr)==3 ) then

!           --- node on surface and obstacle

               skipnod(nodenr) = 2

            else

!           --- node on surface

               skipnod(nodenr) = 1

            end if  ! ( iworkpoint(nodenr)==3 )

         end if  ! ( nodenr>0 ) then

      end do  ! i = 1, nnodes

      deallocate ( nodes, isurfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( skipnod, npoint, 'skipnod' )
         write(irefwr,*) 'End mshshift3d1'

      end if  ! ( debug )
      call erclos ( 'mshshift3d1' )

      end
