      subroutine mshsmoothmat ( irhsdlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Build matrix and zero right-hand side for surface smoothing
!
! **********************************************************************
!
!                       KEYWORDS
!
!     build
!     2d
!     3d
!     smoothing
!     free_surface
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irhsdlc(5)

!     irhsdlc       i/o   Standard SEPRAN array, containing information of the
!                         large vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinbldlc(3), iusol(5)

!     iinbldlc       User input array for subroutine BUILD
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     iusol          Position in solution vector
!     user           Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILD       assemble a system of equations
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPPRINTMAT    Print matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays

      irhsdlc = 0
      iusol = 0

!     --- Fill iinbld

      iinbldlc(1) = 3             ! number of entries filled
      iinbldlc(2) = 3             ! matrix is build, rhsd = 0
      iinbldlc(3) = 1             ! no effect of boundary conditions

      call sepfillsmf ( iinbldlc, irhsdlc, iusol, iusol )
      call sepbuild

1000  if ( debug ) then

!     --- Debug information

         call sepprintmat ( stiffmat )
         write(irefwr,*) 'End mshsmoothmat'

      end if  ! ( debug )
      call erclos ( 'mshsmoothmat' )

      end
