      subroutine mshsmoothsolv ( isol, irhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-05-2014 New call to sepsolvel
!        version  1.0  date 05-02-2009
!
!   copyright (c) 2009-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system corresponding to displacement of surface nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     smoothing
!     free_surface
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmoduleintmat
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, irhsd

!     irhsd         i/o   Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPINITINPSOL  initialize arrays inpsol and rinsol for seplinsol
!     SEPLINSOL      Body of linear solver
!     SEPMAVER       perform matrix vector multiplication
!     SEPUPDINPSOL   Update inpsol for special situations
!                    Put parameters in sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothsolv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothsolv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear right-hand side

      rhsd => ks(irhsd)%sol
      rhsd = 0d0

!     --- put contribution of boundary conditions into right-hand side

      call sepmaver ( stiffmat, isol, irhsd, 7 )

!     --- Solve system of equations
!
      call sepinitinpsol
      inpsol(3,1 ) = 1          ! cg
      inpsol(4,1 ) = 3          ! ilu preconditioning
      inpsol(5,1 ) = 100        ! maxiter
      inpsol(6,1 ) = -1         ! iprint
      if ( associated(realmt(stiffmat(1))%matinv) ) then
         inpsol(10,1 ) = 2      ! reuse preconditioning
      else
         inpsol(10,1 ) = 1      ! keep preconditioning
      end if  ! ( associated(realmt(stiffmat(1))%matinv) )
      rinsol(1,1) = 0.01d0

      iseqsol = 1
      iseqreac = 0
      iseqintmat = 1
      keepmat = 0
      iseqsolut = 1
      iseqmt = stiffmat(1)
      call sepupdinpsol  ! Update inpsol for special situations
      call seplinsol ( stiffmat, isol, irhsd )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsmoothsolv'

      end if  ! ( debug )
      call erclos ( 'mshsmoothsolv' )

      end
