      subroutine mshsmoothsrprb ( isurnr, kmeshs, kmeshsp2, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kprob with respect to surface smoothing
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     smoothing
!     free_surface
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekprob
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, kmeshs(*), kmeshsp2(8,*), ishape

!     ishape         i    Shape number of elements
!     isurnr         i    Surface sequence number for which kprob must be filled
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2       i    Second part of kmeshs, referring to surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leniinput
      parameter (leniinput=200)
      integer ichprb, iinput(leniinput), numcurvs, surstart, i, istart, &
              iseqprob

!     i              Counting variable
!     ichprb         Choice parameter for subroutine PROBDF
!     iinput         integer input array for subroutine probdf
!     iseqprob       Sequence number of problem input
!     istart         Starting address information in iinput
!     leniinput      Length of array iinput
!     numcurvs       Number of curves in surface
!     surstart       Start address of curves in kmeshs wrt isurnr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothsrprb' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothsrprb'
         write(irefwr,1) 'isurnr', isurnr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numcurvs = kmeshs(isurnr+1)-kmeshs(isurnr)
      surstart = kmeshs(isurnr)
      ishape = kmeshsp2(2,isurnr)
      if ( debug ) then
         write(irefwr,1) 'numcurvs, surstart', numcurvs, surstart
         call prinin ( kmeshs(surstart+1), numcurvs, 'curves' )
      end if  ! ( debug )

!     --- Fill array iinput

      iinput = 0
      iinput(1) = leniinput  ! declared length
      iinput(3) = leniinput  ! declared length
      iinput(5) = 1          ! nprob
      iinput(6) = 1          ! ntype
      iinput(7) = 1          ! numnatbnd
      iinput(10) = 1         ! numnatsrf
      iinput(12) = numcurvs  ! numesscrv
      iinput(20) = 10        ! ivers
      istart = 66            ! starting address information
      iinput(istart) = 0     ! type number 0 for volumes
      iinput(istart+1) = 812 ! type number 812 for surface

!     --- Boundary element

      iinput(istart+2) = 1       ! boundary group number
      iinput(istart+3) = 1       ! number of surfaces
      iinput(istart+4) = isurnr  ! first surface number

!     --- essential boundary conditions

      istart = istart+4
      do i = 1, numcurvs
         iinput(istart+1) = 1       ! bit pattern
         iinput(istart+2) = 0       ! jstep
         iinput(istart+3) = 1       ! number of curves
         iinput(istart+4) = abs(kmeshs(surstart+i))  ! curve number
         istart = istart+4
      end do  ! i = 1, numcurvs
      call inichkarlen ( istart, leniinput, 'iinput' )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( iinput, istart, 'iinput' )

      ichprb = 1    ! User defines input in iinput
      iseqprob = 0  ! Sequence number of input
      nprob = 1

      allocate ( iinputtmp(istart-5) )
      iinputtmp = iinput(6:istart)
      allocate(rinputtmp(1))

      call probdfext ( ichprb, iseqprob )

      deallocate(iinputtmp)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprob, 40, 'kprob' )
         write(irefwr,*) 'End mshsmoothsrprb'

      end if  ! ( debug )
      call erclos ( 'mshsmoothsrprb' )

      end
