      subroutine mshsrfframecp ( rinput, coorusrp, iinput, ncoar, &
                                 rinputcoarse, jcoars, nuspnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-07-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates of user points from rinput into array coorusrp
!     Special case for framesurf
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     copy
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ncoar, nuspnt, jcoars
      double precision rinput(3,*), coorusrp(4,*), rinputcoarse(*)

!     coorusrp       o    array containing the coordinates as well as the
!                         local coarseness of internal points to be placed
!                         in the surface
!                         Length 4 x ncoar
!                         (1,i):  x-coordinate of point i
!                         (2,i):  y-coordinate of point i
!                         (3,i):  z-coordinate of point i
!                         (4,i):  coarseness of point i
!     iinput         i    integer input array containing the user point numbers
!                         of the extra internal points
!     jcoars         i    If 1 coarseness is given, if 0 not
!     ncoar          i    Number of internal points in surface
!     nuspnt         i    Number of user points
!     rinput         i    Real input array, see MESHBODY
!                         This part contains the coordinates of the user points
!     rinputcoarse   i    Real input array
!                         This part contains the coarseness of the user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iuserp
      double precision unitlength

!     i              Counting variable
!     iuserp         User point sequence number
!     unitlength     Unit length for coarseness
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsrfframecp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsrfframecp'
         call prinin ( iinput, ncoar, 'user points' )
         write(irefwr,1) 'jcoars, nuspnt', jcoars, nuspnt
  1      format ( a, 1x, (10i6) )
         if ( jcoars==1 ) &
            call prinrl ( rinputcoarse(2), nuspnt, 'coarseness' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( jcoars==1 ) unitlength = rinputcoarse(1)

!     --- just copy

      do i = 1, ncoar
         iuserp = iinput(i)
         coorusrp(1:3,i) = rinput(1:3,iuserp)
         if ( jcoars==1 ) then
            coorusrp(4,i) = rinputcoarse(iuserp+1)*unitlength
         else
            coorusrp(4,i) = 0d0
         end if  ! ( jcoars==1 )
      end do  ! i = 1, ncoar

1000  call erclos ( 'mshsrfframecp' )

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coorusrp, ncoar, 3, &
                        'coordinates of user points' )
         write(irefwr,*) 'End mshsrfframecp'

      end if

      end
