      subroutine mshsurfaces6 ( istorelc, nfaces, leniwork2, nsurfsold, &
                                nsurfs, kmeshn, surffaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iwork1 and iwork2 with surface numbers corresponding to
!     faces
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istorelc(*), nfaces, leniwork2, &
              nsurfsold, nsurfs, kmeshn(*), surffaces(*)

!     istorelc            Work array of length nfaces
!                         istorelc(i) gives number of surfaces corresponding to
!                         face i that have been stored in iwork2
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     leniwork2      i    Length of array iwork2
!     nfaces         i    Number of faces
!     nsurfs         i    number of surfaces
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, i, isurnr, istart, ipstart, iielem, inelem

!     i              Counting variable
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inelem         Number of elements in a specific element group
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iwork1 with starting addresses in array iwork2
!     by accumulating information stored in iwork
!     Clear array istorelc(1:nfaces)
!     Fill array iwork2 with surface numbers corresponding to faces
!     Use istorelc to keep track the number of surfaces already filled
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces6'
         write(irefwr,1) 'nfaces', nfaces
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Fill array iwork1 with starting addresses in array iwork2
!         by accumulating information stored in iwork

      ipstart = 1

      do iface = 1, nfaces

         if ( iwork(iface)>1 ) then
            iwork1(iface) = ipstart
            ipstart = ipstart+iwork(iface)
         else
            iwork1(iface) = 0
         end if

      end do
      ipstart = ipstart-1
      if ( ipstart>leniwork2 ) then

!     --- Estimated length too short

         call errint ( ipstart, 1 )
         call errint ( leniwork2, 2 )
         call errchr ( 'iwork2', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      leniwork2 = ipstart

      if ( debug ) write(irefwr,1) 'leniwork2', leniwork2

!     --- Clear array istorelc(1:nfaces)

      istorelc(1:nfaces) = 0

!     --- Fill array iwork2 with surface numbers corresponding to faces
!         Use istorelc to keep track the number of surfaces already filled

      istart = 0
      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)

!        --- For all faces iface in surface do

         do iielem = 1, inelem

!        --- iwork(iface) := iwork(iface)+1

            iface = surffaces(istart+iielem)
            if ( iwork1(iface)>0 ) then

!           --- Face in several surfaces

               ipstart = iwork1(iface)+istorelc(iface)
               iwork2(ipstart) = isurnr
               istorelc(iface) = istorelc(iface) + 1

            end if  ! ( iwork1(iface)>0 )

         end do  ! iielem = 1, inelem
         istart = istart+inelem

      end do  ! isurnr = 1, nsurfsold

      if ( debug ) then

!     --- Debug information

         call prinin ( iwork1, nfaces, 'iwork1' )
         call prinin ( iwork2, leniwork2, 'iwork2' )
         call prinin ( istorelc, nfaces, 'istorelc' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces6'

      end if
      call erclos ( 'mshsurfaces6' )

      end
