      subroutine mshtransmat ( t1, t2, t3, trans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the inverse transformation matrix to be used to map
!     a 3d plane onto 2d coordinates, given 2 tangential vectors
!     and the normal
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     transformation
!     plane
!     3d
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision t1(3), t2(3), t3(3), trans(3,3)

!     t1             i    first tangential vectors in plane (normalized)
!     t2             i    second tangential vectors in plane (normalized
!                         and orthogonal  to t1)
!     t3             i    normal vector on the plane (normalised)
!     trans          o    array containing the inverse of the transformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ind(5), i1, i2

!     i              Counting variable
!     i1             i+1 (cyclic)
!     i2             i+2 (cyclic)
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 1, 2, 3, 1, 2 /
! ======================================================================
!
      call eropen ( 'mshtransmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtransmat'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute inverse of transformation

      do i = 1, 3
         i1 = ind(i+1)
         i2 = ind(i+2)
         trans(1,i) = t2(i1)*t3(i2)-t2(i2)*t3(i1)
         trans(2,i) = t3(i1)*t1(i2)-t3(i2)*t1(i1)
         trans(3,i) = t1(i1)*t2(i2)-t1(i2)*t2(i1)
      end do

1000  call erclos ( 'mshtransmat' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshtransmat'

      end if

      end
