      subroutine mshupdquad ( coornw, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt updated curve for quadratic elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n
      double precision coornw(2,n)

!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     n              i    number of nodes in boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision ax, ay, px, py, bx, by, dx, dy, cx, cy, clamb

!     ax             x_1 of point a
!     ay             x_2 of point a
!     bx             x_1 of point b
!     by             x_2 of point b
!     clamb          normalized projection of (a,p) on (a,b)
!     cx             x_1 of c
!     cy             x_2 of c
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     px
!     py
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshupdquad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshupdquad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, n-1, 2

         ax=coornw(1,i)
         ay=coornw(2,i)

!        --- compute a,b,p,(a-b)=d

         px=coornw(1,i+1)
         py=coornw(2,i+1)
         bx=coornw(1,i+2)
         by=coornw(2,i+2)
         dx=ax-bx
         dy=ay-by

!        --- clambda=((p-a).d)/(d.d)

         clamb=((px-ax)*dx+(py-ay)*dy)/(dx*dx+dy*dy)

!        --- compute intersection vector c=a+clamb*d

         cx=ax+clamb*dx
         cy=ay+clamb*dy

!        --- shift point

         coornw(1,i+1)=0.5d0*(ax+bx)+px-cx
         coornw(2,i+1)=0.5d0*(ay+by)+py-cy

      end do

1000  call erclos ( 'mshupdquad' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshupdquad'

      end if  ! ( debug )

      end
