      subroutine mshwritirinp ( iinput, rinput, lengin, lengrl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 16-04-2004
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write the arrays iinput and rinput created by sepmesh to a file with
!     name meshinprinp
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     mesh
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengin, lengrl
      double precision rinput(*)

!     iinput         i    integer input array with input for the mesh
!                         generator as described in the programmers guide
!     lengin         i    Length of integer array iinput
!     lengrl         i    Length of real array rinput
!                         (in double precision variables)
!     rinput         i    Real input array with input for the mesh
!                         generator as described in the programmers guide
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iref
      double precision help(5)

!     help           help array to create the first 5 positions of array rinput
!     i              Counting variable
!     iref           File reference number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshwritirinp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwritirinp'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Open file meshinprinp

      iref = inisetnewref()
      open ( iref, file = 'meshinprinp' )

!     --- Write number of integers in iinput + 5

      write(iref,100) lengin+5
100   format ( 10i8 )

!     --- Write 5 positions followed by iinput

      write(iref,100) lengin+5, 0, lengin+5, 0, 0, &
                      (iinput(i),i=1,lengin)

!     --- Write number of reals in rinput + 5

      write(iref,100) lengrl+5

!     --- Create 5 extra positions for rinput in array help

      help(1) = dble(lengrl+5)
      help(2) = 0d0
      help(3) = help(1)
      help(4) = 0d0
      help(5) = 0d0

!     --- Write 5 positions followed by rinput

      write(iref,110) help, (rinput(i),i=1,lengrl)
110   format ( 5d15.8 )

!     --- Write 3 extra zeros for plaxmesh

      write(iref,100) 0
      write(iref,100) 0
      write(iref,100) 0
      close(iref)
      call inirmref ( iref )

1000  call erclos ( 'mshwritirinp' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshwritirinp'

      end if

      end
