      subroutine prmarkskipnods ( imarknode, imarkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark array imarknode for all nodes that are completely inside
!     the obstacle (i.e. lying in elements that are all completely inside
!     the obstacle.
!     It is supposed that imarknode has been initialized before
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     node
!     mark
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imarknode(*), imarkelem(*)

!     imarkelem      i    Marker array of length nelem
!                         imarkelem(i) = 0: element i not inside obstacle
!                         imarkelem(i) = 1: element i inside obstacle
!     imarknode      i    Marker array of length npoint
!                         imarknode must have been initialized by 0
!                         imarknode(i) = 0: node i not inside obstacle
!                         imarknode(i) = 1: node i inside obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inode, ielem

!     ielem          Element number
!     inode          Node sequence number
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarkskipnods' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarkskipnods'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- For all nodes do

      do inode = 1, npoint

!     --- find all elements corresponding to node
!           if ( all elements must be skipped ) then
!              imarknode(inode) := 1

         do j = kmshd1(inode)+1, kmshd1(inode+1)
            ielem = kmshd2(j)
            if ( imarkelem(ielem).le.0 ) go to 100
         end do
         imarknode(inode) = 1

100   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarkskipnods'

      end if
      call erclos ( 'prmarkskipnods' )

      end
