      subroutine seplevelkm2ad ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ad as well as the subarrays ipointobst and
!     ielemobst (2d only) with respect to level set function
!     Old subroutine mshlevelkm2ad
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     node
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ichoice

!     ichoice        i    Choice parameter: defines which part of kmesh part ad
!                         must be filled
!                         Possible values:
!                         0: the complete array
!                         1: part 1 (ipointobst)
!                         2: parts 1,2 (ielemobst)
!                         3: part 1-4 (icrossobst,rcrossobst)
!                         -2: parts 2 only (ielemobst)
!                         -3: part 3,4 only (icrossobst,rcrossobst)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPLEVCROSS2D  fill arrays icrossobst and rcrossobst (2d)
!     SEPLEVFLIELEM  Fill kmesh part ad in case of levelset functions
!                    Part ielemobst
!     SEPLEVFLPOINT  Fill kmesh part ad in case of levelset functions
!                    Part ipointobst
!     SEPPLOTOBST    Print and plot information about the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define level set fucntion as phi
!     numobst := numobst
!     for iobst := 1 (1) numobst do
!        Mark all nodes of mesh in region with phi>0
!        Mark all elements to be in/out or partly in region with phi>0
!        Mark all nodes that are in region with phi>0 and only in elements
!        that are completely in region with phi>0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevelkm2ad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevelkm2ad'
         write(irefwr,*) 'ichoice, kelmad ', ichoice, kelmad

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      numobst = max(1,numobst)
      nlevelsets = 1

      if ( ichoice>=0 ) then

!     --- ichoice>=0, fill kmesh ad

         call seplevflpoint ( iseq )

      else

!     --- ichoice<0, activate kmesh ad

         iseq = kmeshad(5)

      end if

!     --- Mark all elements to be in/out or partly in obstacle

      if ( ichoice==0 .or. ichoice>=2 .or. ichoice==-2 ) then

!     --- ichois = 0 or 2, fill array ielemobst
!         Mark all elements to be in/out or partly in obstacle

         call  seplevflielem ( iseq )

      end if

      if ( ichoice>=-2 .and. (debug .or. plotobst .or. printobst) ) &
         then

!     --- debug = true, plot nodes and elements with a colour

         call  sepplotobst( debug, ichoice )

      end if

      if ( ichoice==0 .or. ichoice>=3 .or. ichoice==-3 ) then

!     --- ichois = 0 or 3, fill array icrossobst,rcrossobst

         call seplevcross2d ( iseq )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplevelkm2ad'

      end if
      call erclos ( 'seplevelkm2ad' )

      end
