      subroutine sepupdobs3d
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in kmesh part x if it has already been filled
!     before
!     Old subroutine mshupdobs3d
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nx, ny, nz, iobst, j, inode
      double precision xmin, xmax, ymin, ymax, zmin, zmax, dx, dy, dz, &
                       x, y, z, unit, deltax

!     deltax         maximum unit length of element
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     i              Counting variable
!     inode          Node sequence number
!     iobst          Obstacle sequence number
!     j              Counting variable
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     unit           unit length
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Compute xmin, xmax, ymin, ymax, zmin, zmax
!      Check if obstacle is closed or open
!      unit := minimum length in elsize
!      dx := xmax-xmin; dy := ymax-ymin; dz := zmax-zmin;
!      xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!      ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!      zmax := zmax+0.005*dz  zmin := zmin-0.005*dz
!      nx := max(5,4*(xmax-xmin)/unit+1)
!      ny := max(5,4*(ymax-ymin)/unit+1)
!      nz := max(5,4*(zmax-zmin)/unit+1)
!      dx := (xmax-xmin)/nx;  dy := (ymax-ymin)/ny;  dz := (zmax-zmin)/nz
!      Fill information in arrays kmeshx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepupdobs3d' )

!     --- unit := minimum length in elsize

      unit = rinfin
      deltax= -rinfin
      do i = 1, ncurvs
         do j = 1, 2
            x = elsize(j,i)
            if ( x>0d0 ) unit = min(unit,x)
            if ( x>0d0 ) deltax = max(deltax,x)
         end do  ! j = 1, 2
      end do

!     --- Compute xmin, xmax, ymin, ymax, zmin, zmax

      xmin = rinfin
      xmax =-rinfin
      ymin = rinfin
      ymax =-rinfin
      zmin = rinfin
      zmax =-rinfin

      do iobst = 1, numobst

         if (kmeshx(9,iobst)/=kelmi) then
!            call seprmrbuf ( kmeshx(9,iobst), 'coorobst' )
            kmeshx(9,iobst) = kelmi
            coorobst => coor
         end if

         do j = 1, kmeshx(11,iobst)
            do i = 1, kmeshx(10,iobst)
               inode=kmeshcobst(i,j)
               x=coorobst(1,inode)
               y=coorobst(2,inode)
               z=coorobst(3,inode)

!              --- write(*,*)'inode, x,y,z',inode,  x,y,z

               xmin = min(xmin,x)
               xmax = max(xmax,x)
               ymin = min(ymin,y)
               ymax = max(ymax,y)
               zmin = min(zmin,z)
               zmax = max(zmax,z)
            end do
         end do

!        --- dx := xmax-xmin; dy := ymax-ymin

         if (abs(xmax-xmin)<1d-10) then
            deltax=deltax*0.5d0
            xmin=xmin-deltax
            xmax=xmax+deltax
         end if

         if (abs(ymax-ymin)<1d-10) then
            deltax=deltax*0.5d0
            ymin=ymin-deltax
            ymax=ymax+deltax
         end if

         if (abs(zmax-zmin)<1d-10) then
            deltax=deltax*0.5d0
            zmin=zmin-deltax
            zmax=zmax+deltax
         end if

!        --- dx := xmax-xmin; dy := ymax-ymin; dz := zmax-zmin

         dx = xmax-xmin
         dy = ymax-ymin
         dz = zmax-zmin

      if ( ierror/=0 ) go to 1000

!     --- xmax := xmax+0.005*dx  xmin := xmin-0.005*dx
!         ymax := ymax+0.005*dy  ymin := ymin-0.005*dy
!         zmax := zmax+0.005*dz  zmin := zmin-0.005*dz
!         nx := max(5,4*(xmax-xmin)/unit+1)
!         ny := max(5,4*(ymax-ymin)/unit+1)
!         nz := max(5,4*(zmax-zmin)/unit+1)
!         dx := (xmax-xmin)/nx; dy := (ymax-ymin)/ny; dz := (zmax-zmin)/nz

         xmax = xmax+0.005d0*dx
         xmin = xmin-0.005d0*dx
         ymax = ymax+0.005d0*dy
         ymin = ymin-0.005d0*dy
         zmax = zmax+0.005d0*dz
         zmin = zmin-0.005d0*dz
         dx   = xmax-xmin
         dy   = ymax-ymin
         dz   = zmax-zmin
         nx   = min(max(5,int(1.1d0*dx/unit+1d0)),100)
         ny   = min(max(5,int(1.1d0*dy/unit+1d0)),100)
         nz   = min(max(5,int(1.1d0*dz/unit+1d0)),100)

!        --- write(*,*)'nx, ny, nz', nx, ny, nz

         dx   = dx/nx
         dy   = dy/ny
         dz   = dz/nz

!        --- Fill information in arrays kmeshx
!            write(*,*)'nxo,nyo,nzo', kmeshx(3,iobst), kmeshx(4,iobst), &
!            +                            kmeshx(5,iobst)
!            write(*,*)'nxn,nyn,nzn', nx, ny, nz
!            write(*,*)'xmino', rmeshx(1,iobst),rmeshx(3,iobst), &
!            +                        rmeshx(5,iobst)
!            write(*,*)'xminn', xmin, ymin, zmin
!            write(*,*)'xmaxo', rmeshx(2,iobst),rmeshx(4,iobst), &
!            +                        rmeshx(6,iobst)
!            write(*,*)'xmaxn', xmax, ymax, zmax

         rmeshx(1,iobst) = xmin
         rmeshx(2,iobst) = xmax
         rmeshx(3,iobst) = ymin
         rmeshx(4,iobst) = ymax
         rmeshx(5,iobst) = zmin
         rmeshx(6,iobst) = zmax
         rmeshx(7,iobst) = dx
         rmeshx(8,iobst) = dy
         rmeshx(9,iobst) = dz
         kmeshx(3,iobst) = nx
         kmeshx(4,iobst) = ny
         kmeshx(5,iobst) = nz

      end do

!     --- icrossobst and rcrossobst need to be filled again

      iseqobst = iseqobst + 1

!     --- The filling of kmeshxa has not yet been implemented

1000  call erclos ( 'sepupdobs3d' )
      end
