      subroutine er0009func ( usolbn, coorbn, ndim, n, ichint, mcomp, &
                              ichfun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute function f and store usolbn * f in usolbn
!     Help routine for er0009
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, n, ichint, mcomp, ichfun
      double precision usolbn(mcomp,n), coorbn(ndim,n)

!     coorbn         i    array containing the co-ordinates of the curves
!     ichfun         i    indication of the value of f. Possibilities:
!                         0   f = 1
!                         >0  f = func ( ichfun, x, y, z )
!                             at this moment only the combination f#1 and
!                             ichint=1 is permitted
!     ichint         i    indicates the kind of integral to be computed.
!                         possibilities:
!                          1  /  f u  ds
!                          2  /  f  u.n  ds
!                          3  /  u.t  ds
!                          4  /  n. sigma .n  ds
!                          5  /  n. sigma .t  ds
!                          6  /  n. sigma     ds
!                          7  /  u  n  ds
!                          8  sum f(i) u(i)
!                          9  /  f ds
!     mcomp          i    number of components
!     n              i    number of points along the boundary
!     ndim           i    Dimension of the space.
!     usolbn        i/o   array containing the solution along the curves
!                         At output multiplied by f
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, dim2, dim3
      double precision f(n)

!     dim2           second dimension
!     dim3           third dimension
!     f              Contains the function f in all nodes of the boundary
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision func

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FUNC           Computes the value of the vector in a point as function of
!                    the co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0009func' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0009func'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichfun==0 .and. ichint==9 ) then

!     --- ichfun = 0, f = 1: store in usolbn

         usolbn = 1d0

      else if ( ichfun>0 ) then

!     --- ichfun>0, f = function

         dim2 = min(ndim,2)
         dim3 = min(ndim,3)
         do i = 1, n
            f(i) = func ( ichfun, coorbn(1,i), coorbn(dim2,i), &
                          coorbn(dim3,i) )
         end do  ! i = 1, n

         if ( ichint==9 ) then

!        --- ichint = 9, no u

            usolbn(1,:) = f(:)

         else

!        --- ichint # 9, multiply usolbn by f

            usolbn(1,:) = f(:)*usolbn(1,:)

         end if  ! ( ichint==9 )

      end if  ! ( ichfun==0 .and. ichint==9 )

1000  call erclos ( 'er0009func' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0009func'

      end if  ! ( debug )

      end
