      subroutine er00108 ( ichfun, ndim, n, usolbn, &
                           coorbn, value, m )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual computation of boundary integrals over curves
!     Special case: ichint = 8, summation over nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichfun, ndim, n, m
      double precision usolbn(m,n), coorbn(ndim,n), value

!     coorbn     i    array containing the co-ordinates of the curves
!     ichfun     i    indication of the value of f. Possibilities:
!                     0   f = 1
!                     >0  f = func ( ichfun, x, y, z )
!                         at this moment only the combination f#1 and ichint=1
!                         is permitted
!     m          i    number of components
!     n          i    number of points along the boundary
!     ndim       i    Dimension of the space.
!     usolbn     i    array containing the solution along the curves
!     value     i/o   Value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x1(3)

!     i              Counting variable
!     x1             coordinates of first point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision func

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FUNC           Computes the value of the vector in a point as function of the
!                    co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er00108' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er00108'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x1 = 0d0

      if ( ichfun==0 ) then

!     --- ichfun = 0, hence f = 1

         value = value+sum(usolbn)

      else

!     --- ichfun>0, compute f

         do i = 1, n
            x1(1:ndim)=coorbn(1:ndim,i)
            value = value+usolbn(1,i)*func(ichfun,x1(1),x1(2),x1(3))
         end do

      end if

1000  call erclos ( 'er00108' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er00108'

      end if  ! ( debug )

      end
