      subroutine er0035 ( ichois, nfill, idegfd, usol, routcp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-11-1991 Error correction, renumbering
!        version  2.0  date 13-12-1990 Complete revision, new parameter list
!        version  1.1  date 12-09-1988 Extension with ichois = 2,3
!        version  1.0  date 28-02-1986
!
!   copyright (c) 1986-1999  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     ER0035 may be used to copy one or all unknowns per point
!     from the solution vector into the output vector or vice versa
!     Real case, for the complex equivalent see ER0051
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     sepran_vector
!     user_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, idegfd, nfill
      double precision usol(*), routcp(*)

!     ichois    i    Choice parameter. Defines the actions to be eprformed
!                    Possible values:
!                    0: copy contents of usol into ROUTCP
!                    1: copy contents of ROUTCP into usol
!     idegfd    i    Degree of freedom, see IINCOP
!     nfill    i/o   Number of unknowns that have been filled in ROUTCP
!                    at output
!     routcp    o
!     usol      i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, inunkp, iunk, i

!    i        General loop variable
!    iunk     Position of degree of freedom in node
!    node     Nodal point number
!    inunkp   Maximum number of unknowns in node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY     Copy double precision dx to double precision dy
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( idegfd.eq.0 .and. indprpi.eq.0 ) then

!     --- idegfd = 0, Copy complete array. No renumbering so straight forward

         if ( ichois.eq.0 ) then
            call dcopy ( nfill, usol, 1, routcp, 1 )
         else
            call dcopy ( nfill, routcp, 1, usol, 1 )
         end if

      else if ( idegfd.eq.0 ) then

!     --- idegfd = 0, Copy complete array with renumbering

         if ( indprpi.ne.0 ) then

!        --- indprpi # 0, information in KPROB part p

            nfill = 0
            do 110 node = 1 , npoint
               do 100 i = 1, nphysi
                  iunk = kprobpi(node,i)
                  if ( iunk.ne.0 ) then

!                 --- Unknown found

                     nfill = nfill + 1
                     if ( ichois.eq.0 ) then
                        routcp(nfill) = usol(iunk)
                     else
                        usol(iunk) = routcp(nfill)
                     end if
                  end if
100            continue
110         continue
         end if
      else if ( indprpi.ne.0 ) then

!     --- indprpi # 0, information in KPROB part p, only one degree of freedom

         nfill = 0
         do 200 node = 1 , npoint
            iunk = kprobpi(node,idegfd)
            if ( iunk.ne.0 ) then

!           --- Unknown found

               nfill = nfill + 1
               routcp(nfill) = usol(iunk)
            end if
200      continue
      else if ( indprfi.ne.0 ) then

!     --- indprfi # 0, information in KPROB part f

         nfill = 0
         do 210 node = 1, npoint
            inunkp = kprobfi(node+1) - kprobfi(node)
            if ( idegfd.le.inunkp ) then

!           --- Unknown found

               nfill = nfill + 1
               routcp(nfill) = usol(kprobfi(node)+idegfd)
            end if
210      continue
      else

!     --- indprpi = indprfi = 0

         nfill = npoint
         call dcopy ( nfill, usol(idegfd), nphysi, routcp, 1 )
      end if
      end
