      subroutine er0068 ( idegfd, inode, usol, nfill, routcp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       ER0068 may be used to copy one or all unknowns in user
!       point INODE from the solution vector into the output vector
!       Complex equivalent of er0067
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idegfd, inode, nfill
      double complex usol(*), routcp(*)

!     idegfd   i  Degree of freedom, see IINCOP
!     inode    i  User point number, see IINCOP
!     nfill    o  Number of unknowns that have been filled in ROUTCP at output
!     routcp   o
!     usol     i  Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, i, iunk, inunkp

!    i        General loop variable
!    iunk     Position of degree of freedom in solution array
!    node     Nodal point number
!    inunkp   Maximum number of unknowns in node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     180   user point is not a nodal point
!     219   Degree of freedom is not available in point
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0068' )
      node = kmeshl(inode)
      nfill = 0
      if ( node.eq.0 ) then
         call errint ( inode, 1 )
         call errsub ( 180, 1, 0, 0 )
      else if ( indprpi.ne.0 ) then

!     --- indprpi # 0, information in KPROB part p

         if ( idegfd.eq.0 ) then

!        --- idegfd = 0, fill all values in node

            do 100 i = 1, nphysi
               iunk = kprobpi(node,i)
               if ( iunk.ne.0 ) then

!              --- Unknown found

                  nfill = nfill + 1
                  routcp(nfill) = usol(iunk)
               end if
100         continue
         else

!        --- idegfd > 0

            iunk = kprobpi(node,idegfd)
            if ( iunk.ne.0 ) then

!           --- Unknown found

               nfill = nfill + 1
               routcp(nfill) = usol(iunk)
            end if
         end if
      else if ( indprfi.ne.0 ) then

!     --- indprfi # 0, information in KPROB part f

         inunkp = kprobfi(node+1) - kprobfi(node)
         if ( idegfd.eq.0 ) then

!        --- idegfd = 0, fill all values in node

            do 110 i = 1, inunkp
               routcp(i) = usol(kprobfi(node)+i)
110         continue
            nfill = inunkp
         else

!        --- idegfd > 0

            if ( idegfd.le.inunkp ) then

!           --- Unknown found

               nfill = 1
               routcp(1) = usol(kprobfi(node)+idegfd)
            end if
         end if
      else

!     --- indprpi = indprfi = 0

         if ( idegfd.eq.0 ) then

!        --- idegfd = 0, fill all values in node

            do 130 i = 1, nphysi
               routcp(i) = usol((node-1)*nphysi+i)
130         continue
            nfill = nphysi
         else

!        --- idegfd > 0

            nfill = 1
            routcp(1) = usol((node-1)*nphysi+idegfd)
         end if
         if ( nfill.eq.0 ) then
            call errint ( idegfd, 1 )
            call errint ( inode , 2 )
            call errint ( inunkp, 3 )
            call errsub ( 219, 3, 0, 0 )
         end if
      end if
      call erclos ( 'er0068' )
      end
