      subroutine getucdinf(itype,icell,cellmap,ncellnod,cellname)
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute info for a given element type.  The number of cells
!   per element, the name of the cells, the mapping of one element
!   onto one or more cells.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itype, icell, cellmap(*), ncellnod
      character *(*)   cellname

!     cellmap        o    SEPRAN CELL map       - map of local node to cell
!     cellname       o    contains the name of the expanded cell
!     icell          i    current expanded cell
!     itype          i    contains the current standard element
!                         See UM pag. 2.2.17
!     ncellnod       o    number of nodes in a cell
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, NrPoints_DX(18), Line2(1,2), &
              Line3(2,2), Tri3(1,3), Tri6(4,3), Quad4(1,4), Quad9(4,4), &
              Tri7(6,3), Tri10(9,3), Quad5(4,3), Tri4(3,3), Tet4(1,4), &
              Tet10(8,4), Hex8(1,8), Hex27(8,8), Tet14(8,4), Tet15(8,4), &
              Hex9(12,4), Tet5(4,4)
      character*12  expanded_name(18)

!     expanded_name  contains the DX cells that will be
!                    used for each SEPRAN cell
!     hex27          cellmap of one hex27 onto 8 brick cells
!     hex8           cellmap of one hex8 onto 1 brick cell
!     hex9           cellmap of one hex9 onto 12 tet cells
!     i              Counting variable
!     j              Counting variable
!     line2          cellmap of one line2 onto 1 line cell
!     line3          cellmap of one line3 onto 2 line cells
!     nrpoints_dx    contains the nr. of the points per SEPRAN cell
!     quad4          cellmap of one quad4 onto 1 quad cell
!     quad5          cellmap of one quad5 onto 4 tria cells
!     quad9          cellmap of one quad9 onto 4 quad cells
!     tet10          cellmap of one quad10 onto 8 tet cells
!     tet14          cellmap of one quad14 onto 8 tet cells
!     tet15          cellmap of one quad15 onto 8 tet cells
!     tet4           cellmap of one quad4 onto 1 tet cells
!     tet5           cellmap of one quad5 onto 4 tet cells
!     tri10          cellmap of one tri10 onto 9 tet cells
!     tri3           cellmap of one tri3 onto 1 tria cells
!     tri4           cellmap of one tri4 onto 3 tria cells
!     tri6           cellmap of one tri6 onto 4 tria cells
!     tri7           cellmap of one tri7 onto 6 tria cells
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      DATA EXPANDED_NAME/2*'"line"',       2*'"triangles"', &
                           2*'"quads"',      4*'"triangles"', &
                           2*'"tetrahedra"', 2*'"cubes"', &
                           2*'"tetrahedra"', '"pyr"','"tetrahedra"'/

!      EXPANDED_NAME -
!        Shape SEPRAN              Expansion DX
!          1 line                     1  line
!          2 line 3 points            2  lines 2 points
!          3 triangle                 1  triangle
!          4 isopar. tri.             4  triangles
!          5 quad                     1  quad
!          6 isopar. quad             4  quads
!          7 isopar. tri.             6  triangles
!          8 isopar. tri.             9  triangles
!          9 quad2 (5 points)         4  triangles
!         10 triangle (4 points)      3  triangles
!         11 tetrahedron              1  tetrahedron
!         12 isopat. tetra.           8  tetrahedra
!         13 hexahedra                1  hexahedra
!         14 isopar. hexa(27 points)  8  hexahedra
!         15 isopar. tetra(14 points) 5  tetrahedra
!         16 isopar. tetra(15 points) 5  tetrahedra
!         17 hexahedra (9 points)     12 tetrahedra
!         18 tehtrahedra (5 points)   4  tetrahedra
      DATA NrPoints_DX/2,2,3,3,4,4,3,3,3,3,4,4,8,8, &
                            4,4,4,4/

!      Initialize the connectivity of the expanded cells
!     Line with 2 points
      DATA ((Line2(I,J),J=1,2),I=1,1)/1,2/

!     Line with 3 points
      DATA ((Line3(I,J),J=1,2),I=1,2) &
            /1,2, &
             2,3/

!     Triangle with 3 points
      DATA ((Tri3(I,J),J=1,3),I=1,1)/1,2,3/

!     Isoparametric triangle with 6 points
      DATA  ((Tri6(I,J),J=1,3),I=1,4) &
            /1,2,6, &
             2,3,4, &
             4,5,6, &
             2,4,6/

!      Quadrilateral with 4 points
      DATA  ((Quad4(I,J),J=1,4),I=1,1)/1,4,2,3/

!      Isoparametric Quadrilateral with 9 points
      DATA  ((Quad9(I,J),J=1,4),I=1,4) &
             /1,8,2,9, &
              2,9,3,4, &
              9,6,4,5, &
              8,7,9,6/

!      Isoparametric triangle with 7 points
      DATA  ((Tri7(I,J),J=1,3),I=1,6) &
             /1,2,7, &
              2,3,7, &
              7,3,4, &
              7,4,5, &
              7,5,6, &
              7,6,1/

!      Isoparametric Triangle with 10 points
      DATA  ((Tri10(I,J),J=1,3),I=1,9) &
             /10,1,2, &
              10,2,3, &
              10,3,4, &
              10,4,5, &
              10,5,6, &
              10,6,7, &
              10,7,8, &
              10,8,9, &
              10,9,1/

!      Quadrilateral with 5 points
      DATA  ((Quad5(I,J), J=1,3),I=1,4) &
             /5,1,2, &
              5,2,3, &
              5,3,4, &
              5,4,1/

!      Traingle with 4 points
      DATA  ((Tri4(I,J),J=1,3),I=1,3) &
             /4,1,2, &
              4,2,3, &
              4,3,1/

!      Tetrahedral with 4 points
      DATA  ((Tet4(I,J),J=1,4),I=1,1) /1,2,3,4/

!      Isoparametric tetrahedral with 10 points
      DATA  ((Tet10(I,J),J=1,4),I=1,8) &
             /1,2,6,7, &
              2,3,4,8, &
              4,5,6,9, &
              7,8,9,10, &
              2,4,6,8, &
              6,4,9,8, &
              2,6,7,8, &
              6,9,7,8/

!      Hexahedral with 8 points
      DATA  ((Hex8(I,J),J=1,8),I=1,1) &
             /1,2,5,6,4,3,8,7/

!      Hexahedral with 27 points
      DATA  ((Hex27(I,J),J=1,8),I=1,8) &
             /1,2,10,11,4,5,13,14, &
              2,3,11,12,5,6,14,15, &
              4,5,13,14,7,8,16,17, &
              5,6,14,15,8,9,17,18, &
              10,11,19,20,13,14,22,23, &
              11,12,20,21,14,15,23,24, &
              13,14,22,23,16,17,25,26, &
              14,15,23,24,17,18,26,27/

!      Isoparametric tetrahedron with 14 points
!      The points in the middle of the faces are ignored
      DATA  ((Tet14(I,J),J=1,4),I=1,8) &
             /1,2,6,7, &
              2,3,4,8, &
              4,5,6,9, &
              7,8,9,10, &
              2,4,6,8, &
              6,4,9,8, &
              2,6,7,8, &
              6,9,7,8/

!      Isoparametric etrahedron with 15 points
!      The points in the middle of the faces are ignored
!      Also the point in the middle of the element is ignored
      DATA  ((Tet15(I,J),J=1,4),I=1,8) &
             /1,2,6,7, &
              2,3,4,8, &
              4,5,6,9, &
              7,8,9,10, &
              2,4,6,8, &
              6,4,9,8, &
              2,6,7,8, &
              6,9,7,8/

!      Hexahedral element with 9 points
      DATA  ((Hex9(I,J),J=1,4),I=1,12) &
             /1,2,4,9, &
              2,3,4,9, &
              1,6,2,9, &
              1,5,6,9, &
              2,6,3,9, &
              3,6,7,9, &
              4,3,7,9, &
              4,7,8,9, &
              5,7,6,9, &
              5,8,7,9, &
              1,4,5,9, &
              5,4,8,9/

!      Tetrahedral with 5 points
      DATA  ((Tet5(I,J),J=1,4),I=1,4) &
             /1,3,4,5, &
              1,4,2,5, &
              2,4,3,5, &
              1,2,3,5/
! ======================================================================
!
      cellname  = Expanded_Name(itype)
      ncellnod = NrPoints_DX(itype)

!      --- get the connectivity of cell icell

      if(itype.EQ.1) then

!      --- Line1

         do i=1,ncellnod,1
            cellmap(i)=Line2(icell,i)
         end do
      else if(itype.EQ.2) then

!      --- Line3

         do i=1,ncellnod,1
            cellmap(i)=Line3(icell,i)
         end do
      else if(itype.EQ.3) then

!      --- Tri3

         do i=1,ncellnod,1
            cellmap(i)=Tri3(icell,i)
         end do
      else if(itype.EQ.4) then

!      --- Tri6

         do i=1,ncellnod,1
            cellmap(i)=Tri6(icell,i)
         end do
      else if(itype.EQ.5) then

!      --- Quad4

         do i=1,ncellnod,1
            cellmap(i)=Quad4(icell,i)
         end do

      else if(itype.EQ.6) then

!      --- Quad9

         do i=1,ncellnod,1
            cellmap(i)=Quad9(icell,i)
         end do

      else if(itype.EQ.7) then

!      --- Tri7

         do i=1,ncellnod,1
            cellmap(i)=Tri7(icell,i)
         end do
      else if(itype.EQ.8) then

!      --- Tri10

         do i=1,ncellnod,1
            cellmap(i)=Tri10(icell,i)
         end do
      else if(itype.EQ.9) then

!      --- Quad5

         do i=1,ncellnod,1
            cellmap(i)=Quad5(icell,i)
         end do
      else if(itype.EQ.10) then

!      --- Tri4

         do i=1,ncellnod,1
            cellmap(i)=Tri4(icell,i)
         end do
      else if(itype.EQ.11) then

!      --- Tet4

         do i=1,ncellnod,1
            cellmap(i)=Tet4(icell,i)
         end do
      else if(itype.EQ.12) then

!      --- Tet10

         do i=1,ncellnod,1
            cellmap(i)=Tet10(icell,i)
         end do
      else if(itype.EQ.13) then

!      --- Hex8

         do i=1,ncellnod,1
            cellmap(i)=Hex8(icell,i)
         end do
      else if(itype.EQ.14) then

!      --- Hex27

         do i=1,ncellnod,1
            cellmap(i)=Hex27(icell,i)
         end do
      else if(itype.EQ.15) then

!      --- Tet14

         do i=1,ncellnod,1
            cellmap(i)=Tet14(icell,i)
         end do
      else if(itype.EQ.16) then

!      --- Tet15

         do i=1,ncellnod,1
            cellmap(i)=Tet15(icell,i)
         end do
      else if(itype.EQ.17) then

!      --- Hex9

         do i=1,ncellnod,1
            cellmap(i)=Hex9(icell,i)
         end do
      else if(itype.EQ.18) then

!      --- Tet5

         do i=1,ncellnod,1
            cellmap(i)=Tet5(icell,i)
         end do
      end if

      end
