      subroutine intbndobst ( inpcop, ivectr, nparm, value, ichint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 23-09-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute an integral over obstacles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     integral
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpcop(*), ivectr, nparm, ichint
      double precision value(*)

!     ichint         i    indicates the kind of integral to be computed.
!                         possibilities:
!                          1  /  f u  ds
!                          2  /  f  u.n  ds
!                          3  /  f  u.t  ds
!                          4  /  f  n. sigma .n  ds
!                          5  /  f  n. sigma .t  ds
!                          6  /  f  n. sigma     ds
!                          7  /  f  u  n  ds
!                          8  sum f(i) u(i)
!     inpcop         i    Copy of integer input array extended with defaults
!                         See intbndbf
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     nparm          i    Number of parameters in array iintbn
!     value          o    Array containing the computed integrals at output
!                         If the vector to be integrated corresponds to a
!                         complex vector, VALUE is a complex array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmeshc
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     lenkmeshc      length of array kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERINTOBST      Compute an integral over the boundary of an obstacle
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2077   The vector containing the function u to be integrated has not been
!           filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intbndobst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intbndobst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      if ( kmeshad(9)==0 ) call sepfillobfaces
      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(37)==0 ) call sepfillnmeshg
      if ( ichint/=9 ) then

!     --- ichint # 9, function u required

         if ( ivectr==0 ) then

!        --- Vector usol has not been filled

            call errsub ( 2077, 0, 0, 0 )
            go to 1000

         end if

         usoltemp => ks(ivectr)%sol

      end if

!     --- Next compute space to store the elements

      lenkmeshc = 3*size ( nmeshg )
      compl = ks(ivectr)%compl==1
      call erintobst ( inpcop, usoltemp, &
                       inpcop(1+nparm), value, compl, lenkmeshc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intbndobst'

      end if  ! ( debug )
      call erclos ( 'intbndobst' )

      end
