      subroutine intbndsumsurf ( inpcop, ivectr, nparm, value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 23-09-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute an integral along a boundary
!     The integral consists of a summation over surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpcop(*), ivectr, nparm
      double precision value(*)

!     inpcop         i    Copy of integer input array extended with defaults
!                         See intbndbf
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     nparm          i    Number of parameters in array iintbn
!     value          o    Array containing the computed integrals at output
!                         If the vector to be integrated corresponds to a
!                         complex vector, VALUE is a complex array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0088         compute a sum over a number of surfaces (real case)
!     ER0089         compute a sum over a number of surfaces (complex case)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2001   Surfaces do not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intbndsumsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intbndsumsurf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kelmn<=0 ) then

!     --- Error there are no surface elements

         call errsub ( 2001, 0, 0, 0 )
         go to 1000

      end if

      usoltemp => ks(ivectr)%sol

      compl = ks(ivectr)%compl==1
      if ( compl ) then

!     --- complex case

         call er0089 ( inpcop, usoltemp, inpcop(1+nparm), value )

      else

!     --- real case

         call er0088 ( inpcop, usoltemp, inpcop(1+nparm), value )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intbndsumsurf'

      end if  ! ( debug )
      call erclos ( 'intbndsumsurf' )

      end subroutine intbndsumsurf
