      subroutine intmap3d2 ( usol, usolcop, nunkploc, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The unknowns are present in all points of mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkploc, number
      double precision usol(*), usolcop(nunkploc,*)

!     number         i    indicates which is the first degree of freedom
!                         that must be interpolated in each point
!     nunkploc       i    indicates the number of unknowns that must be
!                         interpolated in each point containing this
!                         number of unknowns counted from number
!     usol           i    Solution vector to be copied
!     usolcop        o    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap3d2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over nodes

      do i = 1, npoint

!     --- Loop over unknowns

         do j = 1, nunkploc
            usolcop(j,i) = usol(kprobpi(i,number-1+j))
         end do  ! j = 1, nunkploc

      end do  ! i = 1, npoint

1000  call erclos ( 'intmap3d2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap3d2'

      end if  ! ( debug )

      end
