      subroutine intmap3d411 ( kmeshc, inelem, kprobf, usol, &
                               usolcop, ielgrp, inpelm, nvertex )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Elements of shape 11, 13, 17, 18 or 21
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     3d
!     tetrahedron
!     linear
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), inelem, &
              kprobf(*), ielgrp, nvertex
      double precision usol(*), usolcop(*)

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    Number of nodes in elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         kmeshc is only related to this element group
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nvertex        i    Number of vertices
!     usol           i    Solution vector
!     usolcop       i/o   Solution vector to which the solution must be
!                         copied. usolcop has nunkp unknowns per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, node

!     i              Counting variable
!     ielem          Element number
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     583   The required degree of freedom is not available in the vertices
!           of the element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap3d411' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d411'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if unknown is present in all vertices

      do i = 1, nvertex

         node = kmeshc(i,1)

         if ( kprobf(node+1)-kprobf(node)==0 ) then

!        --- Unknown not present in vertex

            call errint ( 1, 1 )
            call errint ( 1, 2 )
            call errint ( ielgrp, 3 )
            call errwar ( 583, 3, 0, 0 )
            go to 1000

         end if  ! ( kprobf(node+1)-kprobf(node)==0 )

      end do  ! i = 1, nvertex

!     --- Copy solution in vertices

      do ielem = 1, inelem

         do i = 1, nvertex

            node = kmeshc(i,ielem)

            usolcop(node) = usol(kprobf(node)+1)

         end do  ! i = 1, nvertex

      end do  ! ielem = 1, inelem

1000  call erclos ( 'intmap3d411' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap3d411'

      end if  ! ( debug )

      end
