      subroutine intmsh2d ( meshin, meshout, isol1, isol2, ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 20-04-2005 New call to intmshm2
!        version  3.1  date 03-01-2001 New call to intsolmp
!        version  3.0  date 07-03-1998 Extra parameter ichois
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Interpolate the solution as stored in array isol1
!   from nodal points in array kmesh1 onto nodal points in array kmesh2
!   and store the result in array isol2     (2D) version
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, meshout, isol1(*), isol2(*), ichois

!    ichois   i    Choice parameter
!                  Possible values:
!                  0,1: All points are interpolated; Points outside the mesh are
!                       extrapolated
!                  2:   Only points inside the old mesh or very close to the
!                       old mesh are interpolated, the other points are left
!                       unchanged
!    isol1    i    Standard SEPRAN array containing information of the vector to
!                  be interpolated
!    isol2    o    Standard SEPRAN array containing information of the
!                  interpolated vector
!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinputmsh(2), icurvs(1)
      type (mapping) :: maplc

!     icurvs         Dummy array
!     iinputmsh      Input array for subroutine intmshm2
!     maplc          mapping information
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTMSHM2       Make a formal interpolation from one mesh to another (2D)
!     INTSOLMP       Interpolate array isol1 into array isol2
!     SEPRMIBUF      delete temporary array from ibuffr
!     SEPRMRBUF      Remove permanent space in array BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmsh2d' )

      call sepclear ( maplc )

      iinputmsh(1) = ichois  ! ichoice for intmshm2
      iinputmsh(2) = 0       ! idim for intmshm2
      call intmshm2 ( kmesh2d(meshin), kmesh2d(meshout), maplc, &
                      iinputmsh, icurvs )
      call intsolmp ( meshin, meshout, maplc, isol1, isol2, 0, 0 )
      deallocate ( maplc%imap, maplc%rmap )

      call erclos ( 'intmsh2d' )
      end subroutine intmsh2d
