      subroutine intsolmp1 ( isol1, isol2, map, inunkp, &
                             npointnw, inunk, minmax, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-06-2014 New definition of imap(5)
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 14-06-2004
!
!   copyright (c) 2004-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map contents of solution array "usol1" into array "usol2" in case the
!     number of degrees of freedom per array is constant
!     The mapping is stored in array imap
!     Help subroutine of intsolmp
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol1(*), isol2(*), inunkp, npointnw, inunk, minmax
      logical compl
      type (mapping) :: map

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     inunk          i    Number of degrees of freedom in isol2
!     inunkp         i    Number of degrees of freedom in isol1
!     isol1          i    Standard SEPRAN array containing information of the
!                         vector nterpolated
!     isol2          o    Standard SEPRAN array containing information of the
!                         interpolated vector
!     map            i    contains information of the mapping
!     minmax         i    If 1 the interpolated solution is restricted to the
!                         range uold_iun(min)<=unew_iun<=uold_iun(max)
!                         for each physical unknown iun, if 0 no restriction
!     npointnw       i    Estimate of new number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTCOPCOMPLUN  Interpolate usol1 into usol2 (complex case)
!     INTCOPUN       Interpolate usol1 into usol2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intsolmp1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intsolmp1'

      end if
      if ( ierror/=0 ) go to 1000

      vectin => ks(isol1(1))%sol
      vectout => ks(isol2(1))%sol
      imap => map%imap
      rmap => map%rmap

      if ( map%copy ) then

!     --- The first part of sol2 is identical to sol1 (copy)

         vectout(1:size(vectin)) = vectin
         vectin => vectout

      end if  ! ( map%copy )

      if ( compl ) then

!     --- Complex case

         call intcopcomplun ( vectin, vectout, inunkp, &
                              1, npointnw, npointnw, inunk )

      else

!     --- Real case

         call intcopun ( vectin, vectout, inunkp, &
                         1, npointnw, npointnw, minmax, inunk )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intsolmp1'

      end if
      call erclos ( 'intsolmp1' )

      end
