      subroutine pr0015dir
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-08-2010 Remove istore/store from parm list
!        version  1.0  date 14-08-2000
!
!   copyright (c) 2000-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print profile matrix as a compact matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulematr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, jdiag, jstart, m, icount, jlow, istore(nrusol)
      double precision :: a, store(nrusol)

!     a              Help variable to store quantity
!     i              Counting variable
!     icount         Counter
!     istore         Work array of length nrusol to store at most one row
!                    of the oclumn indices of the matrix
!     j              Counting variable
!     jdiag          Indicates if the basis functions satisfy phii(j = 0, &
!                    if i#j (jdiag=1) or not (jdiag=2)
!     jlow           Under boundary for j
!     jstart         starting position
!     m              number of values in one row to be printed
!     store          Work array of length nrusol to store at most one row
!                    of the matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1406   The storage scheme is not implemented in this routine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      write(irefwr,100)
100   format(' matrix first part, profile storage'//, &
             ' diagonal of matrix'/)
      write(irefwr,110) (i, S_ii(irowsii(i)), i = 1, nrusol)
110   format((4(1x,i6,':',ss,1pe12.5e2)))

!     --- jmethod = 2, non-symmetric profile matrix

         write(irefwr,130)
130      format(//' lower triangular part of the matrix with row and ', &
                  'column numbers'/)

         do i = 1, nrusol
            jlow   = icolsii(i)
            jdiag  = irowsii(i)
            jstart = jdiag-i
            icount = 0
            do j = jlow, i-1
               a = S_ii(jstart+j)
               if ( abs(a)>0d0 ) then

!              --- non-zero matrix element found

                  icount = icount+1
                  store(icount) = a
                  istore(icount) = j

               end if
            end do
            m=min(4,icount)
            write(irefwr,200) i,(istore(j),store(j), j = 1, m)
200         format(1x,i6,':',4(i5,':',ss,1pe12.5e2))
            if ( m<icount ) write(irefwr,210) &
                (istore(j),store(j),j=m+1,icount)
210         format((8x,4(i5,':',ss,1pe12.5e2)))

         end do

         write(irefwr,250)
250      format (/' upper triangular matrix with column and', &
                  ' row numbers'/)

         do i = 1, nrusol
            jlow   = icolsii(i)
            jdiag  = irowsii(i)
            jstart = jdiag+i
            icount = 0
            do j = jlow, i-1
               a = S_ii(jstart-j)
               if ( abs(a)>0d0 ) then

!              --- non-zero matrix element found

                  icount = icount+1
                  store(icount) = a
                  istore(icount) = j

               end if
            end do
            m=min(4,icount)
            write(irefwr,200) i,(istore(j),store(j), j = 1, m)
            if ( m<icount ) write(irefwr,210) &
                (istore(j),store(j),j=m+1,icount)

         end do

      end
