      subroutine pr2023 ( coor, xmin, ymin, dx, dy, ijcoor, minnode, &
                          maxnode, nx, ny )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date   28-02-1998 Extra parameters nx and ny
!        version 1.0    date   06-12-1996
!
!
!
!   copyright (c) 1996-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array ijcoor with the ij-cell number of the co-ordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!    2d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision coor(2,*), xmin, ymin, dx, dy
      integer ijcoor(2,*), minnode, maxnode, nx, ny

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     dx         i    Distance between x-coordinates of cells
!     dy         i    Distance between y-coordinates of cells
!     ijcoor     i    Contains the ij cell number of the co-ordinates
!                     ijcoor(1,k) contains the i number of coor_k
!                     ijcoor(2,k) contains the j number of coor_k
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     xmin       i    Minimum x-value
!     ymin       i    Minimum y-value
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp

!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For all nodes do
!      i := (x-xmin)/ds+1
!      j := (y-ymin)/ds+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Loop over nodes

      do nodp = minnode, maxnode
         ijcoor(1,nodp) = min(int((coor(1,nodp)-xmin)/dx)+1,nx)
         ijcoor(2,nodp) = min(int((coor(2,nodp)-ymin)/dy)+1,ny)
      end do
      end
