      subroutine pr2024 ( ijcoor, minnode, maxnode, npntij, nx, ny )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   06-12-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute number of nodes for each ij combination and store in array
!    npntij. This array is accumulated
! **********************************************************************
!
!                       KEYWORDS
!
!    2d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ijcoor(2,*), minnode, maxnode, npntij(*), nx, ny

!     ijcoor     i    Contains the ij cell number of the co-ordinates
!                     ijcoor(1,k) contains the i number of coor_k
!                     ijcoor(2,k) contains the j number of coor_k
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     npntij     o    Contains number of coordinates in ij-cell accumulated
!                     npntij(1) = 0
!                     npntij((i-1)*ny+j+1) - npntij((i-1)*ny+j) is number
!                     of points in cell ij
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp

!     i          Counting variable
!     j          Counting variable
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array npntij
!    For all nodes in range do
!       (i,j) := ijcoor(.,nodp)
!       npntij((i-1)*ny+j) := npntij((i-1)*ny+j)+1
!    Accumulate npntij
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array npntij

      do i = 1, nx*ny+1
         npntij(i) = 0
      end do

!     --- For all nodes in range do

      do nodp = minnode, maxnode

!     --- (i,j) := ijcoor(.,nodp)

         i = ijcoor(1,nodp)
         j = ijcoor(2,nodp)

!        --- npntij((i-1)*ny+j) := npntij((i-1)*ny+j)+1

         npntij((i-1)*ny+j+1) = npntij((i-1)*ny+j+1)+1

      end do

!     --- Accumulate npntij

      npntij(1) = 0
      do i = 2, nx*ny+1
         npntij(i) = npntij(i-1)+npntij(i)
      end do

      end
