      subroutine pr2027res1 ( kmeshtri, minnode, maxnode, ielem1, ielem2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a mesh consisting of triangles and/or quadrilaterals
!     into three-point triangles, for one surface only
!     The surface is marked by begin and end element
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtri(*), minnode, maxnode, ielem1, ielem2

!     ielem1         i    First element number in surface
!     ielem2         i    Last element number in surface
!     kmeshtri       o    Kmesh part c for triangular mesh
!     maxnode        i    Highest node number in part of mesh
!     minnode        i    Lowest node number in part of mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, ikelmctri, ielem, ielgrp

!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         Last position used in kmeshc
!     ikelmctri      Last position used in kmeshtri
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR2027SUB      Subdivide a mesh consisting of triangles and/or
!                    quadrilaterals into three-point triangles for one
!                    element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2027res1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2027res1'
         write(irefwr,100) 'ielem1, ielem2', ielem1, ielem2
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,100) 'nelem', inelem

!     --- ielem := 0

      ielem = 0             ! element number

!     --- For all element groups do

      ikelmc = 1
      ikelmctri = 0
      do ielgrp = 1, nelgrp

!     --- inpelm := number of nodes in element group
!         inelem := number of elements in element group

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( debug ) then
            write(irefwr,100) 'ielgrp, inpelm, inelem, ishape, ielem', &
                               ielgrp, inpelm, inelem, ishape, ielem
         end if  ! ( debug )

         if ( ielem1>ielem .and. ielem2<=ielem+inelem ) then

!        --- Element is in correct range
!            Raise kmeshc in case ielem1 is not first element of
!            element group

            ikelmc = ikelmc+(ielem1-ielem-1)*inpelm
            ielem = ielem+inelem         ! raise ielem before changing inelem
            inelem = ielem2-ielem1+1
            if ( debug ) then
               write(irefwr,100) 'ielem, inelem', ielem, inelem
            end if  ! ( debug )
            if ( kmeshc(ikelmc+1)>=minnode .and. &
                 kmeshc(ikelmc+1)<=maxnode ) then

!           --- element group belongs to sub part, subdivide

               call pr2027sub ( ishape, inelem, kmeshc(ikelmc), kmeshtri, &
                                ikelmctri, inpelm )

            end if

         else

!        --- Element is not in correct range
!            Raise ielem

            ielem = ielem+inelem

         end if  ! ( ielem1>ielem .and. ielem2<=ielem+nelem )

         ikelmc = ikelmc+inpelm*inelem

      end do
      inelem = ikelmctri/3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'nelem, ikelmctri', inelem, ikelmctri
         write(irefwr,*) 'End pr2027res1'

      end if
      call erclos ( 'pr2027res1' )

      end
