      subroutine pr2027sub ( ishape, inelem, kmeshc, kmeshtri, &
                             kelmctri, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a mesh consisting of triangles and/or quadrilaterals
!     into three-point triangles
!     Restricted to a submesh of one shape only
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, inelem, kmeshc(*), kmeshtri(*), kelmctri, inpelm

!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     kelmctri      i/o   Last position used in kmeshtri
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshtri      i/o   Kmesh part c for triangular mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1586   Subdivision of elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2027sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2027sub'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ishape==3 ) then

!     --- inpelm=3 Copy kmeshc to kmeshtri

         do i = 1, inelem*3
            kmeshtri(kelmctri+i) = kmeshc(i)
         end do
         kelmctri = kelmctri+3*inelem

      else if ( ishape==10 ) then

!     --- ishape=10 Skip centroid

         do i = 0, inelem-1
            do j = 1, 3
               kmeshtri(kelmctri+j+3*i) = kmeshc(inpelm*i+j)
            end do
         end do
         kelmctri = kelmctri+3*inelem

      else if ( ishape==5 .or. ishape==9 ) then

!     --- inpelm=4 Subdivide elements into two triangles

         do i = 0, inelem-1
            kmeshtri(kelmctri+1+6*i) = kmeshc(inpelm*i+1)
            kmeshtri(kelmctri+2+6*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+3+6*i) = kmeshc(inpelm*i+3)

            kmeshtri(kelmctri+4+6*i) = kmeshc(inpelm*i+1)
            kmeshtri(kelmctri+5+6*i) = kmeshc(inpelm*i+3)
            kmeshtri(kelmctri+6+6*i) = kmeshc(inpelm*i+4)
         end do
         kelmctri = kelmctri+6*inelem

      else if ( ishape==4 .or. ishape==7 ) then

!     --- inpelm = 6, 7 Subdivide elements into four triangles

         do i = 0, inelem-1
            kmeshtri(kelmctri+1+12*i) = kmeshc(inpelm*i+1)
            kmeshtri(kelmctri+2+12*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+3+12*i) = kmeshc(inpelm*i+6)

            kmeshtri(kelmctri+4+12*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+5+12*i) = kmeshc(inpelm*i+3)
            kmeshtri(kelmctri+6+12*i) = kmeshc(inpelm*i+4)

            kmeshtri(kelmctri+7+12*i) = kmeshc(inpelm*i+4)
            kmeshtri(kelmctri+8+12*i) = kmeshc(inpelm*i+5)
            kmeshtri(kelmctri+9+12*i) = kmeshc(inpelm*i+6)

            kmeshtri(kelmctri+10+12*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+11+12*i) = kmeshc(inpelm*i+4)
            kmeshtri(kelmctri+12+12*i) = kmeshc(inpelm*i+6)
         end do
         kelmctri = kelmctri+12*inelem

      else if ( ishape==6 ) then

!     --- inpelm=9 Subdivide elements into 8 triangles

         do i = 0, inelem-1
            kmeshtri(kelmctri+1+24*i) = kmeshc(inpelm*i+1)
            kmeshtri(kelmctri+2+24*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+3+24*i) = kmeshc(inpelm*i+9)

            kmeshtri(kelmctri+4+24*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+5+24*i) = kmeshc(inpelm*i+4)
            kmeshtri(kelmctri+6+24*i) = kmeshc(inpelm*i+9)

            kmeshtri(kelmctri+7+24*i) = kmeshc(inpelm*i+2)
            kmeshtri(kelmctri+8+24*i) = kmeshc(inpelm*i+3)
            kmeshtri(kelmctri+9+24*i) = kmeshc(inpelm*i+4)

            kmeshtri(kelmctri+10+24*i) = kmeshc(inpelm*i+9)
            kmeshtri(kelmctri+11+24*i) = kmeshc(inpelm*i+4)
            kmeshtri(kelmctri+12+24*i) = kmeshc(inpelm*i+5)

            kmeshtri(kelmctri+13+24*i) = kmeshc(inpelm*i+9)
            kmeshtri(kelmctri+14+24*i) = kmeshc(inpelm*i+5)
            kmeshtri(kelmctri+15+24*i) = kmeshc(inpelm*i+6)

            kmeshtri(kelmctri+16+24*i) = kmeshc(inpelm*i+8)
            kmeshtri(kelmctri+17+24*i) = kmeshc(inpelm*i+6)
            kmeshtri(kelmctri+18+24*i) = kmeshc(inpelm*i+7)

            kmeshtri(kelmctri+19+24*i) = kmeshc(inpelm*i+8)
            kmeshtri(kelmctri+20+24*i) = kmeshc(inpelm*i+9)
            kmeshtri(kelmctri+21+24*i) = kmeshc(inpelm*i+6)

            kmeshtri(kelmctri+22+24*i) = kmeshc(inpelm*i+1)
            kmeshtri(kelmctri+23+24*i) = kmeshc(inpelm*i+9)
            kmeshtri(kelmctri+24+24*i) = kmeshc(inpelm*i+8)
         end do
         kelmctri = kelmctri+24*inelem

      else if ( ishape>10 ) then

!     --- Other types have not yet been implemented

         call errint ( ishape, 1 )
         call errsub ( 1586, 1, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pr2027sub'

      end if
      call erclos ( 'pr2027sub' )

      end
