      subroutine pr2040 ( coor, xmin, xmax, ymin, ymax, zmin, zmax, ds, &
                          minnode, maxnode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-03-1998
!
!   copyright (c) 1998-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute xmin, xmax, ymin, ymax, zmin, zmax from array coor
!    Compute ds from coor
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), xmin, xmax, ymin, ymax, zmin, zmax, ds
      integer minnode, maxnode

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ds         o    Minimum distance between two points along a curve
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     xmax       o    Maximum x-value
!     xmin       o    Minimum x-value
!     ymax       o    Maximum y-value
!     ymin       o    Minimum y-value
!     zmax       o    Maximum z-value
!     zmin       o    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2
      double precision x1, y1, z1, x2, y2, z2

!     i1         node
!     i2         node
!     x1         x-coordinate of point i1
!     x2         x-coordinate of point i2
!     y1         y-coordinate of point i1
!     y2         y-coordinate of point i2
!     z1         z-coordinate of point i1
!     z2         z-coordinate of point i2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   initialize xmin, xmax, ymin, ymax, zmin, zmax, ds
!   For all curves do
!      For all nodes along the curve do
!         Compute xmin, xmax, ymin, ymax, zmin, zmax
!         Compute ds
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize xmin, xmax, ymin, ymax, dx

      xmin = coor(1,1)
      xmax = xmin
      ymin = coor(2,1)
      ymax = ymin
      zmin = coor(3,1)
      zmax = zmin
      ds = rinfin

!     --- For all curves do

      do i1 = minnode, maxnode

         x1 = coor(1,i1)
         y1 = coor(2,i1)
         z1 = coor(3,i1)

!        --- For all nodes along the curve do

         do i2 = minnode, maxnode

!        --- Compute ds

            x2 = coor(1,i2)
            y2 = coor(2,i2)
            z2 = coor(3,i2)
            if (i1.ne.i2) ds = min(ds,(x2-x1)**2+(y2-y1)**2+(z2-z1)**2)
         end do

!        --- Compute xmin, xmax, ymin, ymax, zmin, zmax

         xmin = min(xmin,x1)
         xmax = max(xmax,x1)
         ymin = min(ymin,y1)
         ymax = max(ymax,y1)
         zmin = min(zmin,z1)
         zmax = max(zmax,z1)

      end do

      ds = sqrt(ds)
      if ( minnode.eq.maxnode ) then
         xmin = xmin-1d-5
         xmax = xmax+1d-5
         ymin = ymin-1d-5
         ymax = ymax+1d-5
         zmin = ymin-1d-5
         zmax = ymax+1d-5
         ds = rinfin
      end if

      end
