      subroutine prbins (srtkey, kgrad, sentry, iout, NPOINT)
!**************************************************************************
!******
!        version 1.4  date   27-11-1992 Minor change in declaration
!        version 1.3  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.1  date   17-05-1988
!        version 1.0  date   01-07-1987
!******  Prog: Jos van Kan
!******
!*************************************************************************
!******
!****** BINS finds the highest element in a real array srtkey by binary search
!****** smaller than  a given value sentry. kgrad is the permutation that orders
!****** the elements in ascending order. To find kgrad subroutine sort must
!****** be used.
!****** IOUT contains the index of kgrad on output.
!****** in other words srtkey (kgrad (iout)) is the element looked for.
!****** WARNING: If sentry is smaller than all elements in srtkey iout has value
!****** 0 on exit.
!******
!*************************************************************************
!******
!****** Parameters      i/o       meaning
!******
!****** srtkey           i        array with sortkeys
!******
!****** kgrad            i        integer array containing the permutation
!******                           that orders sortkey (ascending)
!******
!****** sentry           i        real, entry to be searched in sortkey.
!******
!****** iout             o        integer. index of kgrad of the searched entry.
!******
!****** NPOINT           i        number of entries in sortkey
!******
!*************************************************************************
!******
!****** Local Variables
!******
!****** ilo           actual lower bound of the arraypart searched.
!******               (sentry >= srtkey (kgrad (ilo + 1)) at any time).
!******
!****** ihi           actual upperbound of the array part searched.
!******               (sentry < srtkey (kgrad (ihi)) at any time).
!******
!****** mid           (ilo + ihi) / 2
!******
!*************************************************************************
!******
!****** Algorithm:
!******
!****** Assuming sentry is between the highest and lowest value of srtkey
!****** the interval is halved. Then we test if sentry is in the left or
!****** the right part of the (sorted) array. We repeat the procedure
!****** with the appropriate array part until sentry is between successive
!****** points. The lower point of those two is output.
!******
!*************************************************************************
      implicit none
      integer NPOINT, kgrad(NPOINT), iout
      double precision srtkey(NPOINT), sentry
      integer ilo, ihi, mid
!******
!****** Exception handling : the list may be empty (error), sentry may
!****** be smaller than the smallest srtkey element (warning, iout = 0), &
!****** or larger than the largest srtkey element (regular, iout = NPOINT).
!******
      if (NPOINT.le.0) call ertrap ( 'prbins', 707, 1, 0, 0, 0 )
!******
!****** ERROR 1: BINS CALLED WITH EMPTY LIST, OR NPOINT NEGATIVE.
!******
      if (srtkey (kgrad (1)).gt.sentry)  then
         iout = 0
         return
      end if
!******
!****** Warning: Smallest sortkey higher than sentry
!******
      if (srtkey (kgrad (NPOINT)).le.sentry) then
        iout = NPOINT
        return
      end if
!******
!****** Done : sentry larger than or equal to largest sortkey.
!******
!******
!****** Now for the main algorithm:
!****** At the start we have
!******    srtkey (kgrad (ilo)) <= sentry < srtkey (kgrad (ihi)).    (*)
!****** We calculate mid = (ilo + ihi) / 2
!****** now if srtkey(kgrad (mid)) <= sentry we make ilo = mid, &
!****** otherwise ihi = mid.
!****** So inequality (*) is preserved during the loop. We stop
!****** as soon as ilo + 1 equls ihi.
!******
      ilo = 1
      ihi = NPOINT
!******
!****** While (ihi - ilo > 1)
!****** { Halve interval;
!******   test sentry against value in srtkey;
!******   update upper or lowerboud accordingly}
!****** end while
!******
!****** START WHILE
!******
100   if (ihi.eq.ilo + 1) goto 110
      mid = (ihi + ilo) / 2
      if (srtkey (kgrad (mid)).le.sentry) then
        ilo = mid
      else
        ihi = mid
      end if
      goto 100
!******
!****** END WHILE
!******
!****** output result
!******
110   iout = ilo
      return
      end
