      subroutine prcut1 (ielem, slevel, coor, fval, knodp, &
           c1o, c2o, f1o, f2o)
!************************************************************************
!******
!         version 1.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
!*******  Version 1.0
!*******
!*******  Date: 11-03-91
!*******
!*******  Prog: Jos van Kan
!*******
!************************************************************************
!******
!****** Complex version of prcutl
!****** prcut1 finds the intersection of a line with y-coordinate
!****** equal to slevel with element with number ielem. It exits with
!****** function values and coordinate values of the intersection points.
!****** in fio and cio respectively.
!******
!************************************************************************
!******
!****** DICTIONARY      i/o          meaning
!******
!****** ielem            i           element number of element that is
!******                              intersected
!****** slevel           i           value of y coordinate to intersect
!******
!****** coor             i           nodal coordinates
!******
!****** fval             i           function values in nodes
!******
!****** knodp            i           nodal point numbers
!******
!****** c1o, c2o         o           interpolated x coordinates
!******
!****** f1o, f2o         o           interpolated function values
!******
!************************************************************************
!******
!****** Local Variables
!******
!****** i1, i2, i3        indices of y-coordinates in coor of element
!******                   under consideration.
!******
!****** y1, y2, y3        value of y coordinates, shifted by slevel
!******
!****** ih, yh            auxiliary variables for swap
!******
!****** x1, x2, x3        value of x coordinates
!******
!****** cho, fho          auxiliary variables for swap
!******
!****** f1, f2, f3        value of function  in nodal points.
!******
!****** a1, b1, a2, b2    interpolation weights
!******
!************************************************************************
!******
      implicit none
      double precision slevel, coor(*), c1o, c2o
      double complex  fval(*), f1o, f2o
      integer ielem, knodp(*)
      double precision a1, a2, b1, b2, x1, x2, x3, y1, y2, y3, yh, cho
      double complex f1, f2, f3, fho
      integer i1, i2, i3, ih
!******
!****** Calculate indices of y-coordinates of this element in coor
!******
      i1 = knodp (3 * ielem - 1 + 1)
      i2 = knodp (3 * ielem - 2 + 1)
      i3 = knodp (3 * ielem - 3 + 1)
!******
!****** Calculate shifted y-coordinates
!******
      y1 = coor (2 * i1) - slevel
      y2 = coor (2 * i2) - slevel
      y3 = coor (2 * i3) - slevel

!******
!****** Not all of y1, y2 and y3 can have the same sign if the
!****** line y = slevel intersects the element. The final interpolation assumes
!****** that points 1 and 2 are on the same side of the line. If this is not
!****** the case some swaps have to be done.
!******
        ih = i3
        yh = y3
      if ( y1 * y2.lt.0) then
!******
!****** points 1 and 2 on different sides of line
!****** 3 has to be swapped with the point that lies alone.
!******
        if (y2 * y3.le.0) then
!******
!****** point 2 lies alone, swap 2 & 3
!******
          i3 = i2
          i2 = ih
          y3 = y2
          y2 = yh
        else
!******
!****** point 1 lies alone, swap 1 & 3
!******
          i3 = i1
          i1 = ih
          y3 = y1
          y1 = yh
        end if
      else if (y1 * y2.gt.0) then
        if (y2 * y3.gt.0) call ertrap ('prcut1', 704, 1, 0, 0, 0 )
!******
!****** ERROR 1: prcut1 called with slevel that does not intersect
!****** this element.
!******
!****** This ends the generic cases. What rests are the cases that the line
!****** passes exactly through a vertex of the triangle. (the case y3 = 0, y1 * &
!****** y2.ne.0 is already handled generically).
!******
      else if (y2 * y3.gt.0 .or. (y2 .eq.0 .and. y3.eq.0)) then
!******
!****** y1 = 0 and y2 & y3 lie on the same side of the line y = 0   OR
!****** y2 & y3 are both 0. In either case 1 & 3 have to be swapped.
!******
        i3 = i1
        i1 = ih
        y3 = y1
        y1 = yh
      else if (y1 * y3.gt.0  .or. (y1.eq.0 .and. y3 .eq.0)) then
!******
!****** y2 = 0 and y1 & y3 lie on the same side of the line y = 0   OR
!****** y1 & y3 are both 0. In either case 2 & 3 have to be swapped.
!******
        i3 = i2
        i2 = ih
        y3 = y2
        y2 = yh
!******
!****** What rests is the case y1 = 0, y3.ne.0, y2 * y3.le.0, that is y2 &
!****** y3 on different sides or y2 = 0. In that case nothing has to be done.
!****** The same holds for the case y2 = 0, y3.ne.0, y1 * y3.le.0. What we
!****** haven't considered yet is the possibility that all y's are 0. This is an
!****** error and should not occur.
!******
      end if
!******
!****** now comes the interpolation
!******
!****** Calculate interpolation weights
!******
      a1 = y1/(y1 - y3)
      b1 = 1d0 - a1
      a2 = y2/(y2 - y3)
      b2 = 1d0 - a2
!******
!****** Calculate x-coordinate and function values in nodal points
!******
      x1 = coor (2 * i1 - 1)
      x2 = coor (2 * i2 - 1)
      x3 = coor (2 * i3 - 1)
      f1 = fval (i1)
      f2 = fval (i2)
      f3 = fval (i3)
!******
!****** Interpolate and store
!******
      c1o = a1 * x3 + b1 * x1
      c2o = a2 * x3 + b2 * x2
      f1o = a1 * f3 + b1 * f1
      f2o = a2 * f3 + b2 * f2
!******
!****** Now order them in ascending x-coordinate order
!******
      if (c2o.lt.c1o) then
         cho = c1o
         c1o = c2o
         c2o = cho
         fho = f1o
         f1o = f2o
         f2o = fho
      end if
      return
      end
