      subroutine preli1 ( slevel, knodp, nelem, coor, iout, niout )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.6  date 29-12-2002 Replace error 1693 by 550
!        version  1.5  date 21-02-2000 layout
!        version  1.4  date 27-11-1992 Minor change in declarations
!        version  1.3  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.2  date 17-05-1988
!        version  1.0  date 01-07-1987
!
!   copyright (c) 1987-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  prelin finds in a list of unsorted elements all elements that
!  cut the line y = slevel. This list is stored in iout.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision slevel, coor(2,*)
      integer nelem, knodp(3,nelem), iout(nelem), niout

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iout           o    integer array with element numbers that
!                         intersect the line y = slevel.
!     knodp          i    array with element nodal point numbers
!     nelem          i    Number of elements
!     niout          o    number of elements in iout.
!     slevel         i    value of y coordinate to intersect the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem
      double precision y1, y2, y3

!     ielem          Element number
!     y1             y-coordinate of point
!     y2             y-coordinate of point
!     y3             y-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   Estimated length of array too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all elements
!        If element intersects line y = slevel
!        put element number in iout
!
!     Initialize number of elements on output
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      niout = 0
      do ielem = 1, nelem
         y1 = coor (2, knodp (1, ielem))
         y2 = coor (2, knodp (2, ielem))
         y3 = coor (2, knodp (3, ielem))
         if ((y1 - slevel) * (y2 - slevel).le.0 .or. (y1 - slevel) * &
            (y3 - slevel).le.0) then
               niout = niout + 1
               if ( niout.gt.nelem ) then
                  call eropen ( 'preli1' )
                  call errint ( niout, 1 )
                  call errint ( nelem, 2 )
                  call errchr ( 'iout', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  call erclos ( 'preli1' )
                  go to 1000
               end if
               iout (niout) = ielem
         end if
      end do
1000  end
