      subroutine prentropy1 ( npoint, pres, dens, entropy, ipost, &
                              rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.0  date 31-05-2000
!
!   copyright (c) 1999-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the entropy in all points of the mesh, from the given
!    density and pressure
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          npoint, ipost(*)
      double precision dens(npoint), pres(npoint), entropy(npoint), &
                       rpost(*)

!     dens           i    Array containing the density at every nodal point
!     entropy        o    Array containing the entropy at every
!                         nodal point
!     ipost          i    Integer array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     npoint         i    Number of nodal points
!     pres           o    Array containing the pressure at every nodal point
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i, mcom
      double precision spgamma, machno, const

!     const          Constant
!     i              Counting variable
!     machno         Mach number
!     mcom           If 0 the flow is incompressible; if 1 then the flow is
!                    compressible and the pressure-correction method is
!                    used;
!                    if 2 then we are dealing with multi-gas flow, and
!                    if 3 then the flow is compressible and the fully
!                    compressible method is used.
!     spgamma        Specific heat ratio
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The entropy is defined by
!
!                    spgamma
!     S = ln( p / rho        )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      spgamma = rpost(1)
      mcom = ipost(7)

      if ( mcom.eq.1 ) then

!     --- Mach uniform pressure-correction approach

         machno = rpost(2)
         const  = spgamma*machno**2
         do i = 1, npoint
            entropy(i) = log( (1.0d0 + const*pres(i))/dens(i)**spgamma )
         end do
      else if ( mcom.eq.3 ) then

!     --- Fully compressible approach

         do i = 1, npoint
            entropy(i) = log( pres(i)/(dens(i)**spgamma) )
         end do
      end if
      end
