      subroutine prfillgrid3 ( coor, xmin, ymin, zmin, dx, dy, dz, &
                               nstepx, nstepy, nstepz )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-01-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of equidistant grid
!     3D version
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     3d
!     grid
!     regular
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nstepx, nstepy, nstepz
      double precision coor(3,nstepz,nstepy,nstepx), xmin, dx, ymin, &
                       dy, zmin, dz

!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dx             i    step size in x-direction
!     dy             i    step size in y-direction
!     dz             i    step size in z-direction
!     nstepx         i    Number of steps in x-direction
!     nstepy         i    Number of steps in y-direction
!     nstepz         i    Number of steps in z-direction
!     xmin           i    Minimum x-value
!     ymin           i    Minimum y-value
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, nstepx
         do j = 1, nstepy
            do k = 1, nstepz
               coor(1,k,j,i) = xmin+dx*(i-1)
               coor(2,k,j,i) = ymin+dy*(j-1)
               coor(3,k,j,i) = zmin+dz*(k-1)
            end do
         end do
      end do
      end
