      subroutine prfillisol2d ( usol3d, usol2d, imap, rmap, nunkp, &
                                nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map 3d solution onto 2d mesh and fill
!     corresponding array isol2d
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     intersection
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imap(2,*), nunkp, nnodes
      double precision rmap(*), usol3d(nunkp,*), usol2d(nunkp,*)

!     imap           i    Mapping array of size 2 x nnodes
!                         Local node i lies between global nodes
!                         imap(1,i) and imap(2,i)
!     nnodes         i    Number of nodes in 2d mesh
!     nunkp          i    Maximum number of unknowns in a point
!     rmap           i    Mapping array of size nnodes
!                         The mapping of local node i is defined as
!                         rmap(i) x node imap(1,i) + &
!                         (1-rmap(i)) x node imap(2,i)
!     usol2d         o    Result of interpolation
!     usol3d         i    Array to be interpolated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, node1, node2
      double precision fact

!     fact           defines mapping (rmap(i))
!     i              Counting variable
!     j              Counting variable
!     node1          First node in imap(.,i)
!     node2          Second node in imap(.,i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillisol2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillisol2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      do i = 1, nnodes
         node1 = imap(1,i)
         node2 = imap(2,i)
         fact = rmap(i)
         do j = 1, nunkp
            usol2d(j,i) = fact*usol3d(j,node1)+ &
                          (1d0-fact)*usol3d(j,node2)
         end do  ! j = 1, nunkp
      end do  ! i = 1, nnodes

1000  call erclos ( 'prfillisol2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillisol2d'

      end if  ! ( debug )

      end
