      subroutine prgathercore2 ( usol, usolpart, map, ndeg, npoint )
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 13-10-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copies a local solution vector into a global solution vector by using
!     the mapping between the local and the global node numbers. This subroutine
!     was intended for solution vectors with a constant number of degrees of
!     freedom per point.
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer map(*), ndeg, npoint
      double precision usol(*), usolpart(*)

!     map            i    mapping local to global node numbers as read from
!                         meshoutput_par.000
!     ndeg           i    Number of degrees of freedom
!     npoint         i    Number of nodal points
!     usol           o    Solution vector
!     usolpart       i    Part of solution vector on one processor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, lnod, lpos, i, staddress

!     i              Counting variable
!     lnod           local node number
!     lpos           Starting position in usolpart array
!     node           Node number
!     staddress      starting address in usol array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgathercore2' )

      debug = .false.
      if (debug) print*, 'prgathercore2'

      lpos = 0
      do lnod = 1,npoint
         node = map(lnod)
         staddress = ndeg*(node-1)
         do i=1,ndeg
            usol(staddress+i) = usolpart(lpos+i)
            if (debug) print*, lnod, i, ' copying', node, &
                           usolpart(lpos+i)
         end do
         lpos = lpos + ndeg
      end do
      call erclos ( 'prgathercore2' )
      end
