      subroutine prgetname ( vectorname, ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 20-03-2015 No warning if ivecnm<0
!        version  1.5  date 20-04-2012 Indexed vectors
!        version  1.4  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 1999-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the vector with vector name
!     If the name does not exist ivecnm is not changed
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     vector_name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*), intent(in) :: vectorname
      integer, intent(inout) :: ivecnm

!    ivecnm      i/o  Sequence number of vector with respect to array names - 1
!    vectorname   i   Name of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nch, nch1
      character (len=80) :: vname1, vname2

!     i              General loop variable
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
!     vname1         Name of vector translated to capitals
!     vname2         Name of vector translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2340   The vector name does not correspond to a vector
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numvec do
!        if ( vectorname = names(i) ) then
!           ivecnm : i-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgetname' )
      nch1 = index ( vectorname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(vectorname)
      vname1 = vectorname(1:nch1)
      call askupc(vname1)
      do i = 1, numvec

         nch = ncharnames(i)

         if ( nch1==nch ) then
            vname2 = names(i)(1:nch1)
            call askupc(vname2)
            if ( vname1(1:nch1)==vname2(1:nch1) ) then

!           --- name of vector found

               if ( mainsubr==4 ) then
                  ivecnm = i-1
               else
                  ivecnm = i
               end if
               go to 1000

            end if
         end if
      end do

!     --- If the code enters this position, no vector has been found with
!         this name

      if ( ivecnm>=0 .and. mainsubr/=4 ) then

!     --- The warning is not printed in seppost

         call errchr ( vectorname, 1 )
         call errwar ( 2340, 0, 0, 1 )

      end if

1000  call erclos ( 'prgetname' )

      end
