      subroutine princurv ( ichois, icurvs, curves, ncurvs, ndim, iwork, &
                            text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 03-08-1999 Extra parameters ichois, iwork
!        version  1.0  date 13-09-1997
!
!   copyright (c) 1997-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Help subroutine to print the co-ordinates of the curves from the
!    array icurvs and curves
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     curve
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, icurvs(*), ncurvs, iwork(*), ichois
      double precision curves(ndim,*)
      character *(*) text

!     curves     i    array containing the coordinates of the curves
!     ichois     i    Choice parameter indicating what has to be printed
!                     by this subroutine
!                     ichois consists of two parts iprint and jmax
!                     according to ichois = iprint+10*jmax
!                     iprint may take one of the following values:
!                     0,1: print all curves
!                       2: print all curves as indicated by iwork
!                     3,4: See 1, 2, also print the distances along the curves
!                     if jmax=1 only the coordinates in the region xmin, xmax, &
!                     ymin, ymax are printed
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     iwork      i    Array of length ncurvs
!                     If iwork(i)=0 the curve must be printed, &
!                        iwork(i)=1 not
!     ncurvs     i    Number of curves in mesh
!     ndim       i    Dimension of the space.
!     text       i    Text to be printed in the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, istart, iprint, imax, k, icount, ihelp(3), kupp
      double precision coor(2,3), x, y, dx, work(5)
      logical first

!     coor           array of length ndim x 3 containing the
!                    co-ordinates of the nodal points to be printed
!     dx             length of vector in x-direction
!     first          If true it concerns the first step
!     i              Counting variable
!     icount         number of unknowns per element
!     ihelp          integer work array of length 3
!     imax           Parameter jmax in ichois
!     iprint         Parameter iprint in ichois
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kupp           Upper boundary for k
!     work           Work array
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      write ( irefwr, 100 ) text
100   format(1x,a)
      istart = 0
      iprint = mod(ichois,10)
      imax = ichois/10
      if ( imax.eq.0 .or. ndim.eq.3 ) then

!     --- Print all curves

         do i = 1, ncurvs
            if ( iprint.eq.2 .or. iprint.eq.4 ) then
               if ( iwork(i).eq.0 ) go to 200
            end if
            write ( irefwr,110 ) i
110         format( 'curve: ', i3 )
            if ( ndim.eq.2 ) then
               write(irefwr,120) (j-istart,curves(1,j),curves(2,j), &
                  j=istart+1,icurvs(i))
120            format((i4,':',ss,2(1pe10.3e2),i4,':',ss,2(1pe10.3e2), &
                       i4,':',ss,2(1pe10.3e2)))
            else
               write(irefwr,130) (j-istart,curves(1,j),curves(2,j), &
                                  curves(3,j),j=istart+1,icurvs(i))
130            format((i4,':',ss,3(1pe10.3e2),i4,':',ss,3(1pe10.3e2)))
            end if
            if ( iprint.gt.2 ) then

!           --- Print also the distances along the curves

               write ( irefwr,140 ) 'distances'
140            format(a)
               if ( ndim.eq.2 ) then
                  do j = istart+1, icurvs(i)-1, 5
                     kupp = min(j+4,icurvs(i)-1)
                     do k = j, kupp
                        dx = (curves(1,k+1)-curves(1,k))**2+ &
                             (curves(2,k+1)-curves(2,k))**2
                        work(k-j+1) = sqrt(dx)
                     end do
                     write(irefwr,150) (k-istart,work(k-j+1), k=j,kupp)
150                  format(5(i4,':',ss,1pe10.3e2))
                  end do
               else
                  do j = istart+1, icurvs(i)-1, 5
                     kupp = min(j+4,icurvs(i)-1)
                     do k = j, kupp
                        dx = (curves(1,k+1)-curves(1,k))**2+ &
                             (curves(2,k+1)-curves(2,k))**2+ &
                             (curves(3,k+1)-curves(32,k))**2
                        work(k-j+1) = sqrt(dx)
                     end do
                     write(irefwr,150) (k-istart,work(k-j+1), k=j,kupp)
                  end do
               end if
            end if
200         istart = icurvs(i)

         end do

      else

!     --- Print curves in selected region only

         do i = 1, ncurvs
            if ( iprint.ge.2 .or. iprint.eq.4 ) then
               if ( iwork(i).eq.0 ) go to 300
            end if
            icount = 0
            first = .true.
            do j = istart+1,icurvs(i)
               x = curves(1,j)
               y = curves(2,j)
               if ( x.ge.xmin .and. x.le.xmax .and. &
                    y.ge.ymin .and. y.le.ymax ) then

!              --- Region detected

                  icount = icount+1
                  ihelp(icount) = j-istart
                  coor(1,icount) = x
                  coor(2,icount) = y

               end if
               if ( icount.eq.3 ) then

!              --- 3 points stored, print

                  if ( first ) write ( irefwr,110 ) i
                  write(irefwr,120) ( ihelp(k), coor(1,k), coor(2,k), &
                                      k = 1, 3 )
                  first = .false.
                  icount = 0
               end if

            end do
            if ( icount.gt.0 ) then

!           --- still points stored, print

               if ( first ) write ( irefwr,110 ) i
               write(irefwr,120) ( ihelp(k), coor(1,k), coor(2,k), &
                                   k = 1, icount )
            end if

300         istart = icurvs(i)

         end do

      end if
      end
