      subroutine prindmat ( dmat, imat1, icol, n_p )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-05-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the D matrix in the Simple method
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     matrix
!     divergence_matrix
!     gradient_matrix
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision dmat(*)
      integer imat1(*), icol(*), n_p

!     dmat           i    d matrix to be printed
!     icol           i    Integer array containing the column numbers of the
!                         non-zero elements
!     imat1          i    Integer array of length n_p+1, containing the number
!                         of elements in each row that are non-zero accumulated
!     n_p            i    Number of pressure unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irow, ncol, jstart, jend, j

!     irow           Row sequence number
!     j              Counting variable
!     jend           end position of unknown in intmat
!     jstart         starting position
!     ncol           Help variable for columns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For irow := 1 (1) n_p do
!        write all non-zero elements in row
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prindmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prindmat'

      end if

!     --- For irow := 1 (1) n_p do
!            write all non-zero elements in row

      jstart = 1
      do irow = 1, n_p
         jend = imat1(irow+1)
         ncol = min(jstart+3,jend)
         write(irefwr,110) irow, ( icol(j), dmat(j), j = jstart, ncol )
110      format(1x,i6,':',4(i5,':',ss,1pe12.5e2))
         if ( ncol.lt.jend ) write(irefwr,120) &
              (icol(j), dmat(j), j = ncol+1, jend )
120      format((8x,4(i5,':',ss,1pe12.5e2)))
         jstart = jend+1
      end do  ! irow = 1, n_p

      if ( debug ) write(irefwr,*) 'End prindmat'
      call erclos ( 'prindmat' )

      end
