      subroutine pringlobnodes ( numnodesloc, globnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array globnodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     node
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numnodesloc(iacnodes), globnodes(*)

!     globnodes      i    Array of size npoint_glob containing the numbering of
!                         the global nodes under the condition that
!                         first all nodes of the first cpu are numbered, &
!                         then of the second one and so on.
!                         A global node has only one sequence number
!                         defined by the lowest cpu it is positioned on
!     numnodesloc    i    Contains the number of nodes that correspond
!                         to a cpu accumulated
!                         Nodes that belong to a cpu with lower number are
!                         excluded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pringlobnodes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pringlobnodes'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call prinin ( numnodesloc, iacnodes, 'numnodesloc' )
      call prinin ( globnodes, numnodesloc(iacnodes), 'globnodes' )

1000  call erclos ( 'pringlobnodes' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pringlobnodes'

      end if  ! ( debug )

      end subroutine pringlobnodes
