      subroutine prinintmat2 ( sendunknowns )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Print contents of array sendunknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     parallel
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

!     use sepmodulecomio

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: sendunknowns(0:*)

!     sendunknowns   i    Array to store the unknowns to be sent to other blocks
!                         See subrutine incommatpar8
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iblock, istart, nunks

!     iblock         Block sequence number
!     istart         First position used at a part
!     nunks          number of unknowns to be sent
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinintmat2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinintmat2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = sendunknowns(0)
      do iblock = 1, iacnodes

         nunks = sendunknowns(iblock)-istart

         if ( nunks>0 ) then

!        --- Some unknowns must be sent to iblock

            write(irefwr,1) 'block number', iblock
            call prinin ( sendunknowns(istart+1), nunks, 'unknowns' )
            istart = istart+nunks

         end if  ! ( nunks>0 )

      end do  ! iblock = 1, iacnodes

1000  call erclos ( 'prinintmat2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinintmat2'

      end if  ! ( debug )

      end subroutine prinintmat2
