      subroutine prinkmeshac ( ndim, nsplit, kmeshac, rmeshac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-03-2012 Use subroutines pringridxd
!        version  1.0  date 24-06-2000
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Print the contents of pat kmeshac in a readable form
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     sort
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, nsplit, kmeshac(*)
      double precision rmeshac(2*ndim,*)

!     kmeshac        i    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         kmeshac is organized as follows:
!                         Part 1: length (ndim+2) x nsplit
!                            For each discontinuous part ndim+2 positions
!                            Pos. 1 number of nodes in part
!                            pos. 2-ndim+1: nx,ny,nz i.e. number of cells in
!                                           each direction
!                            pos. ndim+2: starting address w.r.t. part 2
!                         For each discontinuous part we have a separate
!                         part 2 and part 3
!                         Part 2: length 1+nx ny nz starting address of
!                            nodes in part 3 for each cell
!                            So in 2d the number of nodes in cell (i,j) is
!                            kmeshac_2(k+1)-kmeshac_2(k), with k=(i-1)*ny+j
!                         Part 3: length nnodes contains the node numbers
!                            in the sequence defined by part 2
!     ndim           i    Dimension of the space.
!     nsplit         i    Number of parts into which the mesh has been splitted
!     rmeshac        i    Real array of length 2 x ndim x nsplit
!                         contains xmin, xmax, ymin, ymax, zmin zmax
!                         for each subpart
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isplit, nnodes, istart, nx, ny, nz
      double precision xmin, xmax, ymin, ymax, zmin, zmax

!     isplit         Sequence number of discontinuous part
!     istart         First position used at a part
!     nnodes         Number of nodes
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRINGRID2D     Print part of kmesh ac corresponding to 1 subpart of mesh
!                    (2d)
!     PRINGRID3D     Print part of kmesh ac corresponding to 1 subpart of mesh
!                    (3d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For isplit := 1 (1) nsplit do
!        get general information
!        print general information
!        print information per cell
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nz = 0
      zmin = 0d0
      zmax = 0d0
      write(irefwr,*) 'Contents of kmesh part ac'

!     --- Loop over all subparts

      do isplit = 1, nsplit

!     --- get general information

         nnodes = kmeshac((isplit-1)*(ndim+2)+1)
         nx = kmeshac((isplit-1)*(ndim+2)+2)
         ny = kmeshac((isplit-1)*(ndim+2)+3)
         xmin = rmeshac(1,isplit)
         xmax = rmeshac(2,isplit)
         ymin = rmeshac(3,isplit)
         ymax = rmeshac(4,isplit)
         if ( ndim>2 ) then
            nz = kmeshac((isplit-1)*5+4)
            zmin = rmeshac(5,isplit)
            zmax = rmeshac(6,isplit)
         end if
         istart = kmeshac(isplit*(ndim+2))

!        --- print general information

         write(irefwr,100) isplit, nnodes, nx, ny, nz, xmin, xmax, ymin, &
                           ymax, zmin, zmax
100      format ( 'Part ', i3, '   Number of nodes ', i6/ &
                  'nx = ', i6, '   ny = ', i6, '   nz = ', i6/ &
                  'xmin, xmax = ', 2(ss,1pe10.2e2) / &
                  'ymin, ymax = ', 2(ss,1pe10.2e2) / &
                  'zmin, zmax = ', 2(ss,1pe10.2e2) )
         if ( ndim==2 ) then

!        --- ndim = 2, print information per cell

            call pringrid2d ( nx, ny, istart, kmeshac, xmin, xmax, &
                              ymin, ymax )

         else

!        --- ndim = 3, print information per cell

            call pringrid3d ( nx, ny, nz, istart, kmeshac, xmin, xmax, &
                              ymin, ymax, zmin, zmax )

         end if

      end do

      end
