      subroutine prinkmeshcglob
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print array kmeshcglob containing global topology array
!     in parallel case
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     parallel
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, inpelm, istart, ielem, iielem, lenkmeshc

!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     istart         Starting address of array KMESH part c
!     lenkmeshc      length of array kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkmeshcglob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkmeshcglob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,*) 'global topology'

      ielem = 0
      istart = nelgrp+1

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = kmeshcglob(ielgrp)
         write(irefwr,100) ielgrp
100      format(' standard element sequence number', i6/ &
                ' absolute/relative el.number     nodes'/)

         do iielem = 1, inelem
            lenkmeshc = min(inpelm,10)
            write(irefwr,110) ielem+iielem, iielem, &
                              kmeshcglob(istart:istart+lenkmeshc-1)
            if ( inpelm>10 ) write(irefwr,120) &
               kmeshcglob(istart+lenkmeshc+1:istart+inpelm)
            istart = istart+inpelm
110         format(i8,i6,':',5x,10i6)
120         format(20x,10i6)

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinkmeshcglob'

      end if  ! ( debug )
      call erclos ( 'prinkmeshcglob' )

      end
