      subroutine prinkprobpar
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-10-2014 New parameter list
!        version  1.2  date 02-03-2004 Corection for 2 nodes
!        version  1.1  date 26-04-2003 New definition part 2
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the parallel arrays parinfo, sendinfo
!     and innercontr in case parallel computing is applied
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     print
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numneigh, i, iblock, isendinfo, nneigh, m_i, istartinfo, &
              j, ineigh, n_ij, ipinner

!     i              Counting variable
!     iblock         Block sequence number
!     ineigh         Neighbour of block
!     ipinner        Starting address of array innercontr
!     isendinfo      Position in array sendinfo
!     istartinfo     Position in array parinfo
!     j              Counting variable
!     m_i            The number of neighbor blocks of block i that are also
!                    neighbors of the present block
!     n_ij           nodes in the interface of iblock and ineigh that are also
!                    neighbor nodes of blocknr
!     nneigh         Number of neighbors of block
!     numneigh       number of neighbors of a block
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINKPROBPAR2  Print array kprobhsend
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinkprobpar' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinkprobpar'

      end if
      if ( .not. parallel ) go to 1000

      write(irefwr,*) 'Parallel information of block number ', inodenr

      numneigh = blockunkinfo(1)

!     --- For all neighbor blocks do

      do i = 1, numneigh

         iblock =  blockunkinfo(2*i)
         write(irefwr,*) 'Neighbour block ', iblock
         write(irefwr,*) 'Degrees of freedom on common sides of ', &
                         'blocks ', inodenr, ' and ', iblock
         isendinfo = blockunkinfo(2*i+1)
         nneigh = sendinfo(isendinfo)
         call prinin ( sendinfo(isendinfo+1), nneigh, 'unknowns' )

      end do  ! i = 1, numneigh

      if ( iacnodes>2 ) then

!     --- iacnodes>2, extra information

         write(irefwr,*) 'Information stored in second part'

!        --- For all neighbor blocks do

         do i = 1, numneigh

            iblock =  blockunkinfo(2*i)
            write(irefwr,*) 'Neighbour block ', iblock
            istartinfo = blockunkinfo(2*numneigh+i+1)
            m_i = blockunkinfo(istartinfo)

!           --- Loop over common neighbors

            do j = 1, m_i

               ineigh = blockunkinfo(istartinfo+2*j-1)
               isendinfo = blockunkinfo(istartinfo+2*j)
               n_ij = sendinfo(isendinfo)
               write(irefwr,*) 'Degrees of freedom on common sides of ', &
                               'blocks ', inodenr, ' and ', iblock, &
                               ' that are neighbors of block ', ineigh
               call prinin ( sendinfo(isendinfo+1), n_ij, &
                             'unknowns' )

            end do  ! j = 1, m_i

         end do  ! i = 1, numneigh

      end if  ! ( iacnodes>2 )

!     --- Print array innercontr

      nrusol = kprobloc(66)
      innercontr => kprobpart%innercontr
      call prinin ( innercontr, nrusol, 'innercontr part 1' )
      i = innercontr(nrusol+1)
      call prinin1 ( innercontr(nrusol+2), i, 2, &
                     'innercontr part 2' )

!     --- Print array kprobhsend

      if ( associated(kprobhsend) ) call prinkprobpar2

1000  if ( debug ) write(irefwr,*) 'End prinkprobpar'
      call erclos ( 'prinkprobpar' )

      end subroutine prinkprobpar
