      subroutine prinmtrowcomp ( Sii )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print Sii in case of amg row compact storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: Sii(*)

!     Sii            i    Internal matrix to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, ncols, jstart, m, jend, j

!                    they are not
!     i              Counting variable
!     istart         start address
!     j              Counting variable
!     jend           end address
!     jstart         starting position
!     m              number of items to be printed
!     ncols          number of non-zero icolsii in row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinmtrowcomp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinmtrowcomp'
         write(irefwr,1) 'nrusol', nrusol
         call prinin ( irowsii, nrusol+1, 'irowsii' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,100)
100   format(//'matrix with row and column numbers'/)

      istart = 0
      do i = 1, size(irowsii)-1
         ncols = irowsii(i+1)-istart
         jstart = istart+1
         jend = istart+ncols
         m = min(jstart+3,jend)
         write(irefwr,200) i, ( icolsii(j), Sii(j), j = jstart, m )
200      format(1x,i6,':',4(i5,':',ss,1pe12.5e2))
         if ( m<jend ) &
            write(irefwr,210) ( icolsii(j), Sii(j), j = m+1, jend )
210      format((8x,4(i5,':',ss,1pe12.5e2)))

         istart = istart+ncols

      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinmtrowcomp'

      end if  ! ( debug )
      call erclos ( 'prinmtrowcomp' )

      end
