      subroutine prinnodeelem ( nnodes, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 15-12-2005 New call to prinnodeelms3
!        version  1.0  date 12-11-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print all elements that are connected to the nodes stored in
!     array nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*)

!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers
!                         Only elements containing all these nodes are printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenstore
      parameter (lenstore=100)
      integer i, istore(lenstore), icount
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     icount         Actual number of elements in istore
!     istore         Array in whch the elements are stored
!     iwork          integer work array
!                    iwork(i) contains the number of nodes from array
!                    nodes connected to element i
!     lenstore       Declared length of array istore
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINNODEELMS2  Raise iwork for all elements corresponding to a node
!     PRINNODEELMS3  Print all elements corresponding to a node with
!                    iwork(ielem) = nnodes1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Create temporary array iwork
!     Compute starting addresses
!     Clear array iwork(1:nelem)
!     for i := 1 (1) nnodes-1 do
!        For all elements ielem corresponding to nodes(i) do
!           iwork(ielem) := iwork(ielem)+1
!     For all elements ielem corresponding to nodes(nnodes) do
!        Print element (iwork(ielem) = nnodes-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnodeelem' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnodeelem'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kelmd==0 )call sepfillkmeshd

!     --- Create temporary array iwork

      allocate ( iwork(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:nelem)

      iwork = 0

!     --- for i := 1 (1) nnodes-1 do
!            For all elements ielem corresponding to nodes(i) do
!               iwork(ielem) := iwork(ielem)+1

      do i = 1, nnodes-1
         call prinnodeelms2 ( kmeshd, nodes(i), npoint, iwork )
      end do  ! i = 1, nnodes-1

!     --- For all elements ielem corresponding to nodes(nnodes) do
!           Store element  when iwork(ielem) = nnodes-1

      call prinnodeelms3 ( kmeshd, nodes(nnodes), npoint, &
                           iwork, nnodes-1, istore, icount, lenstore )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Print elements

      call prinin ( nodes, nnodes, 'Set of nodes' )
      call prinin ( istore, icount, 'corresponding elements' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinnodeelem'

      end if
      call erclos ( 'prinnodeelem' )

      end subroutine prinnodeelem
