      subroutine prinnodeelms3 ( kmeshd, inode, npoint, iwork, nnodes1, &
                                 istore, icount, lenstore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-12-2005 Extra parameters
!        version  1.0  date 10-11-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements corresponding to a node with iwork(ielem) = nnodes1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshd(*), inode, npoint, iwork(*), nnodes1, istore(*), &
              icount, lenstore

!     icount         o    Number of elements in array istore
!     inode          i    Node sequence number
!                         All elements corresponding to this node are printed
!     istore         o    Contains the set of elements
!     iwork          i    integer work array
!                         iwork(i) contains the number of nodes from array
!                         nodes connected to element i
!     kmeshd         i    KMESH part d
!     lenstore       i    Declared length of array istore
!     nnodes1        i    number of nodes in array nodes minus 1
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, nelements, i, ielem

!     i              Counting variable
!     ielem          Element number
!     ipstart        Starting address in array
!     nelements      Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute starting position and number of elements
!     Raise iwork for all elements corresponding to inode
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinnodeelms3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinnodeelms3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting position and number of elements

      ipstart = kmeshd(inode)+npoint+1
      nelements = kmeshd(inode+1)-kmeshd(inode)

!     --- Store all elements corresponding to inode with iwork(ielem) = nnodes1

      icount = 0
      do i = 1, nelements
         ielem = kmeshd(ipstart+i)
         if ( iwork(ielem)==nnodes1 ) then
            icount = icount+1
            call inichkarlen ( icount, lenstore, 'istore' )
            if ( ierror/=0 ) go to 1000
            istore(icount) = ielem
         end if  ! ( iwork(ielem)==nnodes1 )
      end do  ! i = 1, nelements

1000  call erclos ( 'prinnodeelms3' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinnodeelms3'

      end if

      end
