      subroutine prinoutcrvs ( ioutercurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-02-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array ioutercurvs
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     curve
!     print
!     boundary
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ioutercurvs(*)

!     ioutercurvs    i    Contains information about the curves in the outer
!                         boundary (2D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         These curve numbers are consecutive, which means
!                         the end of one curve is the start of the new one
!                         The part itself is closed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparts, ipart, istart, nouter, i

!     i              Counting variable
!     ipart          Sequence number of part
!     istart         First position used at a part
!     nouter         Number of points outside the obstacle
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nparts := ioutercurvs(1)
!     for ipart := 1 (1) nparts do
!        istart := ioutercurvs(ipart+1)
!        nouter := ioutercurvs(istart)
!        write outer curves for part ipart
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinoutcrvs' )

!     --- nparts := ioutercurvs(1)

      nparts = ioutercurvs(1)

!     --- for ipart := 1 (1) nparts do

      do ipart = 1, nparts

!     --- istart := ioutercurvs(ipart+1)
!         nouter := ioutercurvs(istart)
!         write outer curves for part ipart

         istart = ioutercurvs(ipart+1)
         nouter = ioutercurvs(istart)
         write ( irefwr,100 ) ipart, (ioutercurvs(istart+i),i=1,nouter)
100      format ( 'Outer curves corresponding to part ', i3 / &
                  (10i6) )

      end do
      call erclos ( 'prinoutcrvs' )
      end
