      subroutine prinproprerin1 ( iinput, istart, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print the contents of array iinput for periodical boundary conditions
!     it concerns the part degrees of freedom for one case only
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), istart
      double precision rinput(*)

!     iinput         i    integer input array to be printed
!                         It concerns the part of periodical boundary conditions
!                         corresponding to one pair of points, curves or surfs
!     istart        i/o   Last position used in iinput
!     rinput         i    Real input array
!                         It may contain factor and constant depending on iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, nparts

!     j              Counting variable
!     nparts         number of parts with information of degrees of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinproprerin1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinproprerin1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = istart+1
      nparts = iinput(istart)

      do j = 1, nparts

         write(irefwr,1) 'bit pattern', iinput(istart+1)
         if ( iinput(istart+2)==0 ) then
            write(irefwr,2) 'constant', 0d0
         else
            write(irefwr,2) 'constant', rinput(iinput(istart+2))
         end if  ! ( iinput(istart+2)==0 )
         if ( iinput(istart+3)==0 ) then
            write(irefwr,2) 'factor', 1d0
         else
            write(irefwr,2) 'factor', rinput(iinput(istart+3))
         end if  ! ( iinput(istart+3)==0 )
         istart = istart+3

      end do  ! j = 1, nparts

1000  call erclos ( 'prinproprerin1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinproprerin1'

      end if  ! ( debug )

      end
