      subroutine prinsurfaces ( nsurfs, kmeshn, ifile, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 05-04-2004
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array kmeshn either to the standard output file
!     a specified file or a set of specified files
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     surface
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs, kmeshn(*), ifile
      character *(*) filename

!     filename       i    Name of file
!     ifile          i    Defines how the output must be written
!                         0:  write to standard output
!                         1:  write to file filename
!                         2:  write each surface to a separate file of
!                             name filenamexxxx, with xxxx a sequence number
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefwrsav, isurnr, inpelm, inelem, ielem, iielem, istart, &
              j, iref
      character (len=80) text
      save iref

!     ielem          Element number
!     iielem         Relative element number in element group
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     iref           File reference number
!     irefwrsav      Help parameter to store the value of irefwr
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinsurfaces' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinsurfaces'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifile>0 ) then

!     --- ifile>0, write to file

         irefwrsav = irefwr
         if ( ifile==1 ) then
            iref = inisetnewref()
            open ( irefwr, file = filename )
         end if  ! ( ifile==1 ) then
         irefwr = iref

      end if  ! ( ifile>0 )

      istart = 2*nsurfs
      ielem = 0
      do isurnr = 1, nsurfs

         if ( ifile==2 ) then

!        --- ifile = 2, new file

            write(text,100) filename, isurnr
100         format(a,i4.4)
            open ( irefwr, file = text )

         end if  ! ( ifile==2 )

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)
         write(irefwr,*) 'Surface number ', isurnr
         write(irefwr,*) 'Number of elements ', inelem
         write(irefwr,*) 'element   Rel. element   nodes '

         do iielem = 1, inelem
            ielem = ielem+1
            write(irefwr,200) ielem, iielem, &
                              (kmeshn(istart+j),j=1,inpelm)
200         format(2i6,':', 3x, 40i6)
            istart = istart+inpelm
         end do  ! iielem = 1, inelem

         if ( ifile==2 ) then
            close(irefwr)
            call inirmref ( iref )
         end if  ! ( ifile==2 )

      end do  ! isurnr = 1, nsurfs

      if ( ifile>0 ) then

!     --- ifile>0, write to file

         if ( ifile==1 ) then
            close ( irefwr )
            call inirmref ( iref )
         end if  ! ( ifile==1 )
         irefwr = irefwrsav

      end if  ! ( ifile>0 )

1000  call erclos ( 'prinsurfaces' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinsurfaces'

      end if

      end
