      subroutine prinsurfnorms ( nnodes, elemnodes, nodes, anorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-05-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Print the various norms for all the nodes in the surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     surface
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, elemnodes(*), nodes(nnodes)
      double precision anorm(3,*)

!     anorm          i    In this array the normal vectors corresponding
!                         to the nodes on the surface are stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector
!                         The storage in anorm is the same as in elemnodes, &
!                         except that part 1 is skipped
!                         The first position corresponding node is also
!                         not used. So for each node we have m normals
!                         if the node is in m elements, but we need (m+1)x3
!                         positions
!                         vector in the i-th integration point
!     elemnodes      i    Contains the elements in the surface corresponding
!                         to the nodes in array nodes
!                         Positions 1 to nnodes contain the starting addresses
!                         (istart) for each node
!                         position istart of node inode contains the number of
!                         elements connected to node inode
!                         The next positions contain the element numbers
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers in the surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, istart, numelms, j, even, nlines, iline

!     even           if 1 the number of elements is odd, otherwise even
!     i              Counting variable
!     iline          number of lines
!     inode          Node sequence number
!     istart         Starting position
!     j              Counting variable
!     nlines         number of parts where line elements are created
!     numelms        Number of elements connected to node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinsurfnorms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinsurfnorms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      write(irefwr,*) 'Normals corresponding to nodes in free surface'
      do i = 1, nnodes
         inode = nodes(i)
         istart = elemnodes(i)
         numelms = elemnodes(istart)
         even = mod(numelms,2)
         nlines = numelms/2
         istart = istart-nnodes

!        --- First line

         if ( numelms==1 ) then
            write(irefwr,100) inode, anorm(1:3,istart+1)
100         format( i8,': ', '(',f7.3,',',f7.3,',',f7.3,' ) ' )
         else
            write(irefwr,110) inode, (anorm(1:3,istart+j),j=1,2)
110         format( i8,': ', 2('(',f7.3,',',f7.3,',',f7.3,' ) ') )
         end if  ! ( numelms==1 )

!        --- Next lines

         do iline = 2, nlines
            write(irefwr,120) (anorm(1:3,istart+j),j=3,2*nlines)
120         format( 10x, 2('(',f7.3,',',f7.3,',',f7.3,' ) ') )
         end do  ! j = 2, nlines-1

!        --- Last line

         if ( even==1 .and. nlines>1 ) then
            write(irefwr,130) anorm(1:3,istart+numelms)
130         format( 10x, '(',f7.3,',',f7.3,',',f7.3,' ) ' )
         end if  ! ( even==1 .and. nlines>1 ) then

      end do  ! i = 1, nnodes

1000  call erclos ( 'prinsurfnorms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinsurfnorms'

      end if  ! ( debug )

      end
