      subroutine printcursur ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-01-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a set of curves or surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     curve
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         see subroutine askprinsurf
!     istart         i    First position used at a part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypcur

!                    they are not
!     itypcur        indicates type of item to be printed
!                    1:  curves
!                    2:  surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINTCURVS     Print a set of curves
!     PRINTSURFS     Print a set of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2764   Integer parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printcursur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printcursur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itypcur = inpsep(1)

      select case ( itypcur )
      case(1)

!     --- curves found

         call printcurvs ( inpsep, istart )

      case(2)

!     --- surfaces found

         call printsurfs ( inpsep, istart )

      case default

         call errint ( itypcur, 1 )
         call errchr ( 'itypcur', 1 )
         call errsub ( 2764, 1, 0, 1 )

      end select  ! case ( itypcur )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printcursur'

      end if  ! ( debug )
      call erclos ( 'printcursur' )

      end
