      subroutine printcurves ( kmeshlc, iprins )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 17-11-2004 ini060 --> ini060text
!        version  1.1  date 31-01-2003 New call to prinbn
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    this subroutine prints the curves or the user points of the mesh
!    when ndim = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     curve
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlc, iprins(10)

!    iprins    i   Integer array to be filled by the user with the following
!                  information:
!                  1:  Indication if user points (1) or curves must be
!                      printed (2)
!                  2:  IINNER Information of how the curves must be printed
!                      1      only the outer boundaries are printed
!                      2      only the inner boundaries are printed
!                      3      all boundaries are printed
!                      4      only one curve is printed, curve number stored
!                             in pos. 3
!                  3:  ICURNR Curve number in case IINNER = 4
!                  4-10:  Not yet used   (Fill zeros for future developments)
!     kmeshlc        i    Output of the mesh generator
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINTBN        print boundary of region
!     PRINTUSERP     print user points
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1007   Parameter of integer input array has illegal value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printcurves' )
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)

      if ( iprins(1)==1 ) then

!     --- iprins(1) = 1, print user points

         call printuserp

      else if ( iprins(1)==2 ) then

!     --- iprins(1) = 2, print curves

         call printbn ( iprins(2), iprins(3) )

      else

!     --- Error 825: iprins(1) has wrong value

         call errint ( 1, 1 )
         call errint ( iprins(1), 2 )
         call errchr ( 'iprins', 1 )
         call errsub ( 1007, 2, 0, 1 )
         ierror = 0

      end if

      call erclos ( 'printcurves' )
      end subroutine printcurves
