      subroutine printimehis3 ( work, iseq, iskip, num )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store contents of time history of one scalar in array work
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     time_history
!     scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseq, iskip, num
      double precision, intent(inout) :: work(num+1,*)

!     iseq           i    Sequence number of scalar
!     iskip          i    Indicates the number of time steps to be skipped
!     num            i    Number of scalars
!     work           o    Work array with the time history of all scalars
!                         to be printed/plotted as well as the time
!                         work(1,:) contains the time
!                         work(1+i,:) contains the i^th scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printimehis3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printimehis3'
         write(irefwr,1) 'iseq, iskip, istephiscal', &
                          iseq, iskip, istephiscal
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseq==1 ) then

!     --- First vector, copy time into work

         work(1,1:istephiscal-iskip) = rvectim(1,iskip+1:istephiscal)

      end if  ! ( iseq==1 )

!     --- Copy array into work

      work(1+iseq,1:istephiscal-iskip) = rvectim(2,iskip+1:istephiscal)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printimehis3'

      end if  ! ( debug )
      call erclos ( 'printimehis3' )

      end subroutine printimehis3
