      subroutine printpsi ( psi, kprobf, nodes, numnods, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution array psi in a set of points
!     The position of psi is given by kprobf
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), numnods, nodes(numnods)
      double precision psi(*)
      character (len=*) text

!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nodes          i    Contains the node numbers that must be printed
!     numnods        i    Number of nodes in array nodes
!     psi            i    Array to be printed
!                         At most 1 unknown per point
!                         Array is only filled for some points
!     text           i    String variable containing the heading to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode
      double precision psiloc(numnods)

!     i              Counting variable
!     inode          Node sequence number
!     psiloc         Contains the solution to be printed restricted to
!                    the set stored in nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printpsi' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printpsi'
         call prinin ( nodes, numnods, 'nodes' )
         call prinin ( kprobf, numnods, 'kprobf' )
         call prinrl ( psi, numnods, 'psi' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numnods
         inode = nodes(i)
         psiloc(i) = psi(kprobf(inode)+1)
      end do  ! i = 1, numnods

      write ( irefwr, 100 ) text, (nodes(i),psiloc(i),i=1,numnods)
100   format(1x,a/(5(1x,i4,':',ss,1pe10.3e2)))

1000  call erclos ( 'printpsi' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printpsi'

      end if  ! ( debug )

      end
