      subroutine printscalars ( inpsep, lentxt, nscals, textloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 22-03-2014 Allow expressions
!        version  2.1  date 08-12-2013 Extension with integer scalars
!        version  2.0  date 14-04-2013 Remove scalar from parameter list
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write scalar variable for subroutine TO0094
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), lentxt, nscals
      character (len=*) textloc

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine erscal
!     lentxt         i    length of text parameter, with trailing blanks deleted
!     nscals         i    Number of scalars
!     textloc       i/o   String to store a local text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, k, itypescal, iseqnr, ipos, ivalue, lengtext, &
              numscalars, i
      double precision rvalue
      logical :: integerfound

!                    they are not
!     i              Counting variable
!     integerfound   If true an integer has been found
!     ipos           Position in array
!     iseqnr         Sequence number of scalar
!     istart         First position used at a part
!     itypescal      Type of scalar:
!                    1:  integer
!                    2:  real
!                    3:  scalar
!     ivalue         Integer value
!     k              Counting variable
!     lengtext       Help parameter to indicate the length of the actual text
!     numscalars     Number of scalars
!     rvalue         Real value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSCALVAL      Fill value of scalar defined by ipos into either rvalue or
!                    ivalue
!     GETSCALINDEX   Return with adapted index of scalar
!     PRININ         print 1d integer array
!     PRINTSCALSTORE Fill name of variable in textloc
!                    Investigate if variable is integer or real
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printscalars' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printscalars'
         write(irefwr,1) 'nscals', nscals
         write(irefwr,*) 'textloc ', textloc(1:lentxt)
         call prinin ( inpsep, 12, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nscals = inpsep(1)
      lengtext = lentxt
      istart = 9

      do k = 1, nscals

         itypescal = inpsep(istart+1)
         if (itypescal==19 ) then

!        --- Expression found

            iseqnr = inpsep(istart+2)

         else

!        --- Scalar found

            iseqnr = getscalindex ( inpsep(istart+2) )

         end if  ! (itypescal==19 )

         if ( debug ) &
            write(irefwr,1) 'k, itypescal, iseqnr, istart', &
                             k, itypescal, iseqnr, istart
         numscalars = 1
         if ( iseqnr<0 ) then

!        --- iseqnr<0, complete array of scalars

            iseqnr = abs(iseqnr)            ! original sequence number
            numscalars = dimscals(iseqnr)   ! number of scalars to be printed
            if ( debug ) &
               write(irefwr,1) 'iseqnr, numscalars', iseqnr, numscalars

         end if  ! ( iseqn<0 )
         ipos = inpsep(istart+3)
         istart = istart+3
         if ( debug ) &
            write(irefwr,1) 'k, itypescal, iseqnr, ipos', &
                             k, itypescal, iseqnr, ipos
         if ( itypescal==1 ) then

!        --- integer from incons

            if ( ipos/=0 ) ipos = nint(scalars(ipos))-1
            ivalue = incons(iseqnr+ipos)
            rvalue = 0d0
            integerfound = .true.
            call printscalstore ( integerfound, lengtext, textloc, &
                                  lentxt, rvalue, ivalue )

         else if ( itypescal==2 ) then

!        --- real from rlcons

            if ( ipos/=0 ) ipos = nint(scalars(ipos))-1
            rvalue = rlcons(iseqnr+ipos)
            integerfound = .false.
            call printscalstore ( integerfound, lengtext, textloc, &
                                  lentxt, rvalue, ivalue )

         else

!        --- real or integer from scalars

            if ( ipos/=0 ) ipos = nint(scalars(ipos))-1

            do i = 1, numscalars

               call erscalval ( iseqnr+ipos+i-1, rvalue, ivalue, &
                                integerfound )
               if ( ierror/=0 ) go to 1000
               call printscalstore ( integerfound, lengtext, textloc, &
                                     lentxt, rvalue, ivalue )
               if ( ierror/=0 ) go to 1000

            end do  ! i = 1, numscalars

         end if

      end do
      write(irefwr,300) textloc(1:lengtext)
300   format(1x,a)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printscalars'

      end if  ! ( debug )
      call erclos ( 'printscalars' )

      end
