      subroutine printsurfs ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-01-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a set of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         see subroutine askprinsurf
!     istart         i    First position used at a part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numsurfs, isurfaces(1), nnodes
      integer, allocatable :: nodes(:), iwork(:), iwork1(:)
      character (len=11) text

!     i              Counting variable
!     isurfaces      Array containing the surfaces from which information
!                    must be found
!     iwork          integer work array used for sorting
!     iwork1         integer work array used for sorting
!     nnodes         Number of nodes in nodes
!     nodes          Contains the node numbers of the surface
!     numsurfs       number of surfaces in isurfaces
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printsurfs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printsurfs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- allocate work arrays

      allocate ( nodes(npoint), iwork(npoint), iwork1(npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 3*npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      numsurfs = inpsep(2)  ! number of surfaces to be printed

      text(1:8) = 'surface '

!     --- lopp over surfaces

      do i = 1, numsurfs

         isurfaces(1) = inpsep(i+2)  ! surface number

!        --- Get node numbers of surface

         call sepsurfsnod (  1, nodes, nnodes, isurfaces )

!        --- Sort array nodes, result in iwork1

         call chsort ( nodes, iwork, nnodes )
         iwork1(1:nnodes) = nodes(iwork(1:nnodes))

         write(text(9:11),100) isurfaces(1)  ! put surface number in text
100      format(i3)

         call prinin ( iwork1, nnodes, text )  ! print surface

      end do  ! i = 1, numsurfs

      istart = istart+3+numsurfs  ! raise istart

      deallocate ( nodes, iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printsurfs'

      end if  ! ( debug )
      call erclos ( 'printsurfs' )

      end
