      subroutine printvectors ( inpsep, rinsep, lentxt, textloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 05-06-2014 Extension for print coor
!        version  1.5  date 16-04-2014 Shift in inpsep
!        version  1.4  date 20-06-2013 New definition file ref numbers
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write vector for subroutine TO0094
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), lentxt
      double precision rinsep(*)
      character (len=*) textloc

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine erscal
!     lentxt         i    length of text parameter, with trailing blanks deleted
!     rinsep         i    Array containing the real input for this subroutine
!     textloc       i/o   String to store a local text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inppri(12), k, iseqnr, &
              isolloc, lenname, i, iname, irefwrsav, istrin, &
              numarray, istcurv
      double precision rvalue, rinpri(2)
      character (len=120) filename
      integer, pointer :: kmeshlc, kproblc

!                    they are not
!     filename       Name of file
!     i              Counting variable
!     iname          Sequence number of name in text array
!     inppri         Input array with respect to PRINVC (see PRINVC)
!     irefwrsav      Help variable to store the value of irefwr
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     isolloc        Standard SEPRAN array containing information of the
!                    solution arrays (same as isol, but local name)
!     istcurv        Startin address of curves etc in inpsep
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     k              Counting variable
!     kmeshlc        Subdivision of array kmesh2d
!     kproblc        Local array kprob (rectangular grid)
!     lenname        Length of name
!     numarray       Number of arrays
!     rinpri         dummy
!     rvalue         Real value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr
      double precision :: prsetvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICLOSEFILE   Close file and reset irefwr
!     INIOPENFILE    Open serial file and set irefwr
!     INIVECTR       Return with sequence number of vector
!     MSHMAPRECT     Map the solution defined on an arbitrary mesh onto a
!                    rectangular grid
!     PRINCOOR1      Print one of the coordinates
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINTVECT      Print the values of a SEPRAN solution vector or vector of
!                    special structure
!     PRMATLABSOL    Print solution vector in a form suitable for matlab
!     PRSETVAR       Fill value of variable
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2072   Vector has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printvectors' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printvectors'
         call prinin ( inpsep, 23, 'inpsep' )
         rinpri = 0d0
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmeshlc => kmesh2d(maxmesh+1)
      kproblc => kprob2d(maxmesh+1)
      isolloc = 0

      lenname = 0      ! no name found
      filename = ' '   ! start file name

      inppri(1) = 12
      do k = 2, 10
         if ( inpsep(3+k)<=100000000 ) then
            inppri(k) = inpsep(3+k)
         else
            iseqnr = inpsep(3+k)-100000000
            rvalue = scalars(iseqnr)
            inppri(k) = nint(rvalue)
         end if
      end do
      inppri(11:12) = 0

      if ( debug ) call prinin ( inppri, 10, 'inppri' )

      if ( inppri(7)/=0 ) then

!     --- iregion # 0, fill xmin, xmax, ... from rinsep

         if ( inpsep(4)<=100000000 ) then
            istrin = inpsep(4)
         else
            iseqnr = inpsep(4)-100000000
            rvalue = scalars(iseqnr)
            istrin = nint(rvalue)
         end if
         xmin = rinsep(istrin)
         xmax = rinsep(istrin+1)
         ymin = rinsep(istrin+2)
         ymax = rinsep(istrin+3)
         zmin = rinsep(istrin+4)
         zmax = rinsep(istrin+5)
         if ( debug ) then
            write(irefwr,1) 'istrin', istrin
            write(irefwr,2) 'xmin, xmax, ymin, ymax, zmin, zmax', &
                             xmin, xmax, ymin, ymax, zmin, zmax
         end if  ! ( debug )

      end if

      if ( inpsep(15)>0 ) iname = inpsep(15)  ! sequence number of file

      if ( inpsep(16)>0 ) then

!     --- inpsep(16)>0, file sequence number

         if ( lenname==0 ) then

!        --- No name found yet, use name of vector

            lenname = lentxt
            filename = textloc

         end if  ! ( lenname==0 )

!        --- Detect sequence number (may be scalar)

         k = mod(inpsep(16),100000000)  ! seq number
         if ( inpsep(16)>100000000 ) k = nint(scalars(k))  !scalar

!        --- Extend with _ sequence number (_ followed by 3 positions)

         write(filename(lenname+1:lenname+4),250) k
250      format('_',i3.3)
         lenname = lenname+4  ! new length
         iname = 0

      end if  ! ( inpsep(16)>0 )

      do i = 1, 2

         if ( inpsep(17+i)/=0 ) then

!        --- inpsep(17)#0, hence only values larger than minval are printed

            inppri(10+i) = 1
            rinpri(i) = prsetvar ( inpsep(17+i), rinsep )

         end if  ! ( inpsep(17+i)/=0 )

      end do  ! i = 1, 2

      if ( inpsep(20)<=100000000 ) then
         icurvs(1) = inpsep(20)
      else
         iseqnr = inpsep(20)-100000000
         rvalue = scalars(iseqnr)
         icurvs(1) = nint(rvalue)
      end if

      if ( icurvs(1)>100 ) then

!     --- Array length too small

         call errint ( icurvs(1)+1, 1 )
         call errint ( 101, 2 )
         call errchr ( 'icurvs', 1 )
         call errsub ( 550, 2, 0, 1 )

      else if ( icurvs(1)>0 ) then

!     --- There are curve of surfaces number given
!         Store these numbers in icurvs

         istcurv = inpsep(21)-1
         do i = 1, icurvs(1)
            if ( inpsep(istcurv+i)<=100000000 ) then
               icurvs(1+i) = inpsep(istcurv+i)
            else
               iseqnr = inpsep(istcurv+i)-100000000
               rvalue = scalars(iseqnr)
               icurvs(1+i) = nint(rvalue)
            end if
         end do
      end if

      if ( debug ) call prinin ( icurvs, icurvs(1)+1, 'icurvs' )

      iseqnr = inpsep(1)
      if ( debug ) write(irefwr,1) 'iseqnr/1', iseqnr

      numarray = 1   ! number of arrays to be printed

      if ( iseqnr>10000 ) then

!     --- iseqnr>10000, special case of indexed vector

         iseqnr = inivectr(iseqnr)
         textloc = names(iseqnr)
         lentxt = len_trim(textloc)

      else if ( iseqnr>5000 ) then

!     --- iseqnr>5000, hence set of indexed arrays

         iseqnr = iseqnr-5000           ! original sequence number
         numarray = dimnames(iseqnr)   ! number of arrays to be printed
         textloc = names(iseqnr)
         lentxt = len_trim(textloc)

      end if  ! ( iseqnr>10000 )

      if ( debug ) &
         write(irefwr,1) 'iseqnr, numarray', iseqnr, numarray
      if ( iseqnr<=-1000 ) then

!     --- Special case, iseqnr<=-1000, hence coordinates

         call princoor1 ( -iseqnr-1000, inppri, icurvs )
         go to 1000

      else if ( iseqnr>maxvc1 ) then

!     --- Special case, iseqnr>maxvc1, hence coordinates

         call princoor1 ( iseqnr-maxvc1-1, inppri, icurvs )
         go to 1000

      end if  ! ( iseqnr==-1000 )

      if ( debug ) then
         call prinin ( inppri, 12, 'inppri' )
         write(irefwr,1) 'iseqnr', iseqnr
         call prinin ( isolut2d(iseqnr), 5, 'isolut' )
         call prinrl ( rinpri, 2, 'rinpri' )
      end if  ! ( debug )

      do i = 0, numarray-1

         if ( isolut2d(iseqnr)==0 ) then

!        --- Solution array has not been filled

            call errchr ( textloc(1:lentxt), 1 )
            call errwar ( 2072, 0, 0, 1 )

         else if ( inpsep(14)==0 ) then

!        --- Solution array has been filled, print (standard)

            if ( inpsep(17)==1 ) then

!           --- inpsep(17) = 1, matlab output

               call prmatlabsol ( isolut2d(iseqnr), filename )

            else

!           --- inpsep(17) = 0, standard output

               if ( inpsep(15)>0 ) &
                  call iniopenfile ( iname, irefwrsav, inpsep(22), &
                                     filename )

!              --- Print

               call printvect ( inppri, isolut2d(iseqnr), icurvs, &
                                textloc(1:lentxt), rinpri )

               if ( inpsep(15)>0 ) call iniclosefile ( irefwrsav )

            end if  ! ( inpsep(17)==1 )

         else

!        --- Solution array has been filled,
!            must be mapped on rectangular grid

            call mshmaprect ( isolut2d(iseqnr), isolloc, &
                              rinsep(istrin+6), inpsep(6), icurvs )

!           --- Print

            if ( inpsep(15)>0 ) &
               call iniopenfile ( iname, irefwrsav, inpsep(22), &
                                  filename )

            inppri(3) = 0  ! idim is 0 here, has been used in mshmaprect
            kmesh => kmeshlc
            kprob => kproblc
            iseqkp = kprob
            iprob = 1
            kprobpart => kp(iseqkp)%kprobsub(iprob)
            kprobloc => kprobpart%kprobloc
            call sepfillkmesh(kmesh)
            call sepfillkprob
            call printvect ( inppri, isolloc, icurvs, textloc(1:lentxt), &
                             rinpri )

            if ( inpsep(15)>0 ) call iniclosefile ( irefwrsav )

!           --- Remove local arrays

            call seprmmeshinfo ( kmeshlc )
            call seprmprobinfo ( kproblc )
            call seprmsolinfo ( isolloc )
            kmesh => kmesh2d(meshnr)
            kprob => kprob2d(meshnr)
            iseqkp = kprob
            kprobpart => kp(iseqkp)%kprobsub(iprob)
            kprobloc => kprobpart%kprobloc
            call sepfillkmesh(kmesh)
            call sepfillkprob

         end if  ! ( isolut2d(iseqnr)==0 )

         if ( numarray>1 ) then

!        --- numarray>1, raise iseqnr

            iseqnr = iseqnr+1
            textloc = names(iseqnr)
            lentxt = len_trim(textloc)

         end if  ! ( numarray>1 )

         if ( debug ) &
            write(irefwr,1) 'numarray, iseqnr/2', numarray, iseqnr

      end do  ! i = 0, numarray-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printvectors'

      end if  ! ( debug )
      call erclos ( 'printvectors' )

      end subroutine printvectors
