      subroutine prmachno ( isolvel, isolenth, isolmach, ipost, rpost )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 08-03-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the mach number in all points of the mesh, from the given
!    velocity and enthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isolvel, isolenth, isolmach, ipost(*)
      double precision rpost(*)

!     ipost      i    Integer array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
!     isolenth   i    Array of the structure of isol referring to the enthalpy
!     isolmach  i/o   Array of the structure of isol referring to mach
!     isolvel    i    Array of the structure of isol referring to the velocity
!     rpost      i    Real array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRMACHN1       Actual computation of the mach number
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     Create or activate mach array
!     Compute starting addresses
!     Compute mach numbers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmachno' )

!     --- Create or activate mach array

      velocity => ks(isolvel)%sol
      enthalpy => ks(isolenth)%sol
      call sepcreasolbf1 ( isolmach, ks(isolenth)%typevec, &
                           ks(isolenth)%ivec, 'mach_no', 0 )
      machno => usoltemp

!     --- Compute Mach number in all vertices

      call prmachn1 ( npoint, velocity, enthalpy, machno, ipost, rpost )

      call erclos ( 'prmachno' )
      end
