      subroutine proutpsub1 ( icounter, isubract, &
                              iseq, isubr, numveclc, iavs, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 23-11-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set some parameters depending on the input list
!     with respect to subroutine proutp
! **********************************************************************
!
!                       KEYWORDS
!
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icounter, isubract, &
              iseq, isubr, numveclc, iavs, iseqnr

!     iavs           o    Indicates which kind of output must be created.
!                         Possible values:
!                         0:  Create output for SEPPOST only in the form of file
!                             sepcomp.out
!                         1:  Create output for AVS only in the form of a file
!                             sepavs_xxx.yyy
!                             With xxx a time sequence number of the shape 001, &
!                             002 etc and yyy a vector sequence number of the
!                             shape 001, 002 etc
!                         2:  Create output for SEPPOST in the form of file
!                             sepcomp.out
!                             and for AVS in the form of a file sepavs_xxx.yyy
!     icounter      i/o   Integer time sequence number, specific for proutp
!     iseq           i    Sequence number for array to be written
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!                         In case of TUDFINVOL it is the vector sequence number
!                         to be used counted from 0
!     isubr          i    Indicates the type of calling subroutine
!                         isubr consists of 2 parts isubr_act and nvect
!                         according to isubr = isubr_act + 10*nvect
!                         Possible values for isubr_act
!                         1:  OUTPUT
!                         2:  OUTTIM
!                         3:  OUTSOL
!                         4:  OUTPOS
!                         5:  TUDFINVOL
!                         nvect is only used if isubr_act=5 and it indicates the
!                         number of vectors to be written
!     isubract       o    Parameter isubr_act, see isubr
!     numveclc       o    Number of derivative vectors that have been written
!                         to file sepcomp.out
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'proutpsub1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutpsub1'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      icounter = icounter + 1
      iseq = 0
      isubract = mod(isubr,10)
      if ( debug ) &
         write(irefwr,100) 'icounter, isubract', icounter, isubract
      if ( isubract==5 ) iseq = iseqnr
      if ( debug ) &
         write(irefwr,100) 'iseq, iavs, ifoutp', iseq, iavs, ifoutp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End proutpsub1'

      end if
      call erclos ( 'proutpsub1' )

      end
