      subroutine prsepblockcons ( numlayers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write part constant to output file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numlayers

!     numlayers      o    Number of blocks that are created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer shapecur, shapesur, shapevol, nx, ny, nz
      double precision dx, dy, dz

!     dx             length of block in x-direction
!     dy             length of block in y-direction
!     dz             length of block in z-direction
!     nx             number of elements in x direction
!     ny             number of elements in y direction
!     nz             number of elements in z direction
!     shapecur       shape number of line elements
!     shapesur       shape number of surface elements
!     shapevol       shape number of volume elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getint
      double precision getconst

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETCONST       Get the value of a constant in the input block constants
!     GETINT         Get the value of an integer in the input block constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsepblockcons' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsepblockcons'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d14.6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Write block constants

      write(irefwr,*) 'constants'

      write(irefwr,*) '   reals'

      dx = getconst ('dx')
      if ( dx==0d0 ) dx = 1d0
      dy = getconst ('dy')
      if ( dy==0d0 ) dy = 1d0
      dz = getconst ('dz')
      if ( dz==0d0 ) dz = 1d0
      write(irefwr,2) '   dx =', dx
      write(irefwr,2) '   dy =', dy
      write(irefwr,2) '   dz =', dz

      write(irefwr,*) '   integers'

      nx = getint ('nx')
      ny = getint ('ny')
      nz = getint ('nz')
      write(irefwr,1) '   nx =', nx
      write(irefwr,1) '   ny =', ny
      write(irefwr,1) '   nz =', nz

      shapecur = getint('shape_cur')
      shapesur = getint('shape_sur')
      shapevol = getint('shape_vol')
      if ( shapecur==0 ) shapecur=1
      if ( shapesur==0 ) shapesur=5
      if ( shapevol==0 ) shapevol=13
      write(irefwr,1) '   shape_cur =', shapecur
      write(irefwr,1) '   shape_sur =', shapesur
      write(irefwr,1) '   shape_vol =', shapevol

      write(irefwr,*) 'end'

      numlayers = getint('num_layers')
      if ( numlayers==0 ) numlayers=1

1000  call erclos ( 'prsepblockcons' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsepblockcons'

      end if  ! ( debug )

      end
