      subroutine prsepblockpnts ( numlayers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write part points to output file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numlayers

!     numlayers      i    Number of blocks that are created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount
      character (len=80) text

!     i              Counting variable
!     icount         Counter
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsepblockpnts' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsepblockpnts'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Write block points

      write(irefwr,*) 'mesh3d'

      write(irefwr,*) 'points'
      write(irefwr,*) '   p1 = (  0,  0,  0)'
      write(irefwr,*) '   p2 = ( dx,  0,  0)'
      write(irefwr,*) '   p3 = ( dx, dy,  0)'
      write(irefwr,*) '   p4 = (  0, dy  ,0)'
      write(irefwr,*) '   p5 = (  0,  0, dz)'
      write(irefwr,*) '   p6 = ( dx,  0, dz)'
      write(irefwr,*) '   p7 = ( dx, dy, dz)'
      write(irefwr,*) '   p8 = (  0, dy, dz)'

      icount = 8
      do i = 2, numlayers

         write(text,100) icount+1, i
100      format ( '   p', i3, ' = (  0,  0, ' , i3, '*dz )' )
         write(irefwr,*) text
         write(text,110) icount+2, i
110      format ( '   p', i3, ' = ( dx,  0, ' , i3, '*dz )' )
         write(irefwr,*) text
         write(text,120) icount+3, i
120      format ( '   p', i3, ' = ( dx, dy, ' , i3, '*dz )' )
         write(irefwr,*) text
         write(text,130) icount+4, i
130      format ( '   p', i3, ' = (  0, dy, ' , i3, '*dz )' )
         write(irefwr,*) text
         icount = icount+4

      end do  ! i = 2, numlayers

1000  call erclos ( 'prsepblockpnts' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsepblockpnts'

      end if  ! ( debug )

      end
