      subroutine prstagpres1 ( npoint, pres, machno, stagpres, rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 31-05-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the stagnation pressure in all points of the mesh, from the given
!    mach number and pressure
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          npoint
      double precision machno(npoint), pres(npoint), stagpres(npoint), &
                       rpost(*)

!     machno         i    Array containing the mach number at every nodal point
!     npoint         i    Number of nodal points
!     pres           o    Array containing the pressure at every nodal point
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     stagpres       o    Array containing the stagnation pressure at every
!                         nodal point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i
      double precision const1, const2, const3, spgamma

!     const1         Helping constant
!     const2         Helping constant
!     const3         Helping constant
!     i              Counting variable
!     spgamma        Specific heat ratio
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The stagnation pressure, in dimensionless quantities, is defined by
!
!     P = p*( 1 + 0.5*(gamma-1)*mach^2 )^(gamma/gamma-1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      spgamma = rpost(1)
      const1  = 0.5d0*(spgamma - 1.0d0)
      const2  = spgamma/(spgamma - 1.0d0)

      do i = 1, npoint
         const3      = 1.0d0 + const1*machno(i)**2
         stagpres(i) = pres(i)*const3**const2
      end do

      end
