      subroutine prsw06 ( ichoice_print, number, isuppress, &
                          inodes, nnodes, iformat, heading, icoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2006 Replace body by subroutines
!        version  1.0  date 18-04-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     real case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_print, number, isuppress, inodes(*), &
              nnodes, iformat, icoor
      logical heading

!     heading        i    If true a heading must be printed
!     ichoice_print  i    0:   The nodal point number is printed followed by the
!                                 values in the nodal point
!                         1:   The nodal point number is printed followed by the
!                                 coordinates and then the values in the
!                                 nodal point
!                         2:   The nodal point number is printed followed by the
!                                 coordinates
!                         3:   See 2, however, preceded by the number of nodal
!                                 points to be printed
!     icoor          i    Type of coordinate system
!                         1: Cartesian
!                         2: Cylindrical
!                         3: Polar
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     isuppress      i    If 0 the node numbers are printed,
!                         if 1 they are suppressed
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSW061        Print the solution, do not print coordinates
!                    One unknown to be printed
!     PRSW0613       Print the solution
!                    Transformation to polar coordinates
!     PRSW0614       Print the solution
!                    Transformation to polar coordinates
!                    Do not print node numbers
!     PRSW062        Print the solution, do not print coordinates and node
!                    numbers. One unknown to be printed
!     PRSW063        Print the solution, do not print coordinates
!                    more than 1 unknown per point
!     PRSW064        Print the solution, do not print coordinates
!                    more than 1 unknown per point
!                    Do not print node numbers
!     PRSW065        Print the solution, do not print coordinates
!                    Non constant number of unknowns per point
!     PRSW066        Print the solution, do not print coordinates
!                    Non constant number of unknowns per point
!                    Do not print node numbers
!     PRSW067        Print the solution and coordinates
!                    One unknown to be printed
!     PRSW068        Print the solution and coordinates
!                    Do not print node numbers
!     SEPHEADING1    Print heading: coordinates values
!     SEPPRIRVEC     Print real vector usoltemp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06'
         write(irefwr,1) 'ichoice_print, number, nunkpo, isuppress', &
                          ichoice_print, number, nunkpo, isuppress
         write(irefwr,1) 'indprfo', indprfo
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice_print==0 ) then

!     --- ichoice_print = 0, do not print coordinates

         if ( number>0 .or. nunkpo==1 ) then

!        --- only one unknown

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               call prsw061 ( iformat, inodes, nnodes, usoltemp )

            else

!           --- isuppress = 1, do not print node numbers

               call prsw062 ( iformat, nnodes, usoltemp )

            end if

         else if ( indprfo<=0 ) then

!        --- more unknowns

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               call prsw063 ( iformat, inodes, nnodes, usoltemp, nunkpo, &
                              heading )

            else

!           --- isuppress = 1, do not print node numbers

               call prsw064 ( iformat, nnodes, usoltemp, nunkpo, heading )

            end if

         else

!        --- number of unknowns not constant

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               call prsw065 ( iformat, inodes, nnodes, usoltemp, kprobfo, &
                              heading )

            else

!           --- isuppress = 1, do not print node numbers

               call prsw066 ( iformat, nnodes, usoltemp, kprobfo, heading )

            end if

         end if

      else if ( icoor<=2 .or. ndim/=2 ) then

!     --- ichoice_print = 1, print coordinates

         if ( number>0 .or. nunkpo==1 ) then

!        --- only one unknown

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               call prsw067 ( iformat, inodes, nnodes, usoltemp, heading, &
                              coor, ndim )

            else

!           --- isuppress = 1, do not print node numbers

               call prsw068 ( iformat, inodes, nnodes, usoltemp, heading, &
                              coor, ndim )

            end if  ! ( isuppress==0 )

         else

!        --- more unknowns

            if ( heading ) call sepheading1 ( 1-isuppress )
            call sepprirvec ( iformat, inodes, nnodes, 1-isuppress )

         end if  ! ( number>0 .or. nunkpo==1 )

      else

!     --- ichoice_print = 1, print coordinates and transform to polar ones

         if ( isuppress==0 ) then

!        --- isuppress = 0, print node numbers

            call prsw0613 ( iformat, inodes, nnodes, usoltemp, heading, &
                            coor, ndim, kprobfo, number, nunkpo, indprfo )

         else

!        --- isuppress = 1, do not print node numbers

            call prsw0614 ( iformat, inodes, nnodes, usoltemp, heading, &
                            coor, ndim, kprobfo, number, nunkpo, indprfo )

         end if  ! ( isuppress==0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06'

      end if
      call erclos ( 'prsw06' )

      end
