      subroutine prsw065 ( iformat, inodes, nnodes, usol, kprobf, &
                           heading )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     real case
!     Special case, do not print coordinates
!     Non constant number of unknowns per point
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nnodes, iformat, kprobf(*)
      double precision usol(*)
      logical heading

!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nnodes         i    Number of nodes in inodes
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, istart

!     i              Counting variable
!     inunkp         Number of degrees of freedom in first point
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     Print of vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw065' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw065'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( heading ) write(irefwr,240)
240   format(' node       values'/)

      do i = 1, nnodes
         istart = kprobf(i)
         inunkp = kprobf(i+1)-istart
         if ( iformat==1 ) then
            write(irefwr,250) inodes(i), &
               (usol(j+istart), j=1,min(5,inunkp))
         else if ( iformat==2 ) then
            write(irefwr,251) inodes(i), &
               (usol(j+istart), j=1,min(5,inunkp))
         else if ( iformat==3 ) then
            write(irefwr,252) inodes(i), &
               (usol(j+istart), j=1,inunkp)
         else
            write(irefwr,253) inodes(i), &
               (usol(j+istart), j=1,inunkp)
         end if
         if ( iformat==1 ) then
            if ( inunkp>5 ) write(irefwr,260) &
               (usol(j+istart), j=6,inunkp)
         else if ( iformat==2 ) then
            if ( inunkp>5 ) write(irefwr,261) &
               (usol(j+istart), j=6,inunkp)
         end if
      end do

250   format(1x,i9,':',ss,5(1pe13.5e2))
251   format(1x,i9,':',ss,5(1pe16.8e2))
252   format(1x,i9,':',ss,99(1pe13.5e2))
253   format(1x,i9,':',ss,99(1pe16.8e2))
260   format((7x,ss,5(1pe13.5e2)))
261   format((7x,ss,5(1pe16.8e2)))

1000  call erclos ( 'prsw065' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw065'

      end if  ! ( debug )

      end
