      subroutine prsw06c ( ichoice_print, number, nunkp, isuppress, &
                           inodes, nnodes, usol, indprf, kprobf, coor, &
                           iformat, heading, ndim, icoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_print, number, nunkp, isuppress, inodes(*), &
              nnodes, indprf, kprobf(*), iformat, ndim, icoor
      double precision coor(ndim,*)
      complex *16 usol(*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     ichoice_print  i    0:   The nodal point number is printed followed by the
!                                 values in the nodal point
!                         1:   The nodal point number is printed followed by the
!                                 coordinates and then the values in the
!                                 nodal point
!                         2:   The nodal point number is printed followed by the
!                                 coordinates
!                         3:   See 2, however, preceded by the number of nodal
!                                 points to be printed
!     icoor          i    Type of coordinate system
!                         1: Cartesian
!                         2: Cylindrical
!                         3: Polar
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     isuppress      i    If 0 the node numbers are printed, &
!                         if 1 they are suppressed
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSW06C1       Print the solution, do not print coordinates
!                    One unknown to be printed
!                    complex case
!     PRSW06C10      Print the solution and coordinates
!                    more than 1 unknown per point
!                    Do not print node numbers
!                    complex case
!     PRSW06C11      Print the solution and coordinates
!                    Non constant number of unknowns per point
!                    complex case
!     PRSW06C12      Print the solution and coordinates
!                    Non constant number of unknowns per point
!                    Do not print node numbers
!                    complex case
!     PRSW06C13      Print the solution
!                    Transformation to polar coordinates
!                    complex case
!     PRSW06C14      Print the solution
!                    Transformation to polar coordinates
!                    Do not print node numbers
!                    complex case
!     PRSW06C2       Print the solution, do not print coordinates and node
!                    numbers. One unknown to be printed
!                    complex case
!     PRSW06C3       Print the solution, do not print coordinates
!                    more than 1 unknown per point
!                    complex case
!     PRSW06C4       Print the solution, do not print coordinates
!                    more than 1 unknown per point
!                    Do not print node numbers
!                    complex case
!     PRSW06C5       Print the solution, do not print coordinates
!                    Non constant number of unknowns per point
!                    complex case
!     PRSW06C6       Print the solution, do not print coordinates
!                    Non constant number of unknowns per point
!                    Do not print node numbers
!                    complex case
!     PRSW06C7       Print the solution and coordinates
!                    One unknown to be printed
!                    complex case
!     PRSW06C8       Print the solution and coordinates
!                    Do not print node numbers
!                    complex case
!     PRSW06C9       Print the solution and coordinates
!                    more than 1 unknown per point
!                    complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06c' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06c'
         write(irefwr,1) 'ichoice_print, number, nunkp, isuppress', &
                          ichoice_print, number, nunkp, isuppress
         write(irefwr,1) 'indprf', indprf
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice_print==0 ) then

!     --- ichoice_print = 0, do not print coordinates

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c1'
               call prsw06c1 ( iformat, inodes, nnodes, usol )

            else

!           --- isuppress = 1, do not print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c2'
               call prsw06c2 ( iformat, nnodes, usol )

            end if

         else if ( indprf<=0 ) then

!        --- more unknowns

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c3'
               call prsw06c3 ( iformat, inodes, nnodes, usol, nunkp, &
                              heading )

            else

!           --- isuppress = 1, do not print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c4'
               call prsw06c4 ( iformat, nnodes, usol, nunkp, heading )

            end if

         else

!        --- number of unknowns not constant

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c5'
               call prsw06c5 ( iformat, inodes, nnodes, usol, kprobf, &
                              heading )

            else

!           --- isuppress = 1, do not print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c6'
               call prsw06c6 ( iformat, nnodes, usol, kprobf, heading )

            end if

         end if

      else if ( icoor<=2 .or. ndim/=2 ) then

!     --- ichoice_print = 1, print coordinates

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c7'
               call prsw06c7 ( iformat, inodes, nnodes, usol, heading, &
                              coor, ndim )

            else

!           --- isuppress = 1, do not print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c8'
               call prsw06c8 ( iformat, inodes, nnodes, usol, heading, &
                              coor, ndim )

            end if  ! ( isuppress==0 )

         else if ( indprf<=0 ) then

!        --- more unknowns

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c9'
               call prsw06c9 ( iformat, inodes, nnodes, usol, heading, &
                              coor, ndim, nunkp )

            else

!           --- isuppress = 1, do not print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c10'
               call prsw06c10 ( iformat, inodes, nnodes, usol, heading, &
                               coor, ndim, nunkp )

            end if  ! ( isuppress==0 )

         else

!        --- number of unknowns not constant

            if ( isuppress==0 ) then

!           --- isuppress = 0, print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c11'
               call prsw06c11 ( iformat, inodes, nnodes, usol, heading, &
                               coor, ndim, kprobf )

            else

!           --- isuppress = 1, do not print node numbers

               if ( debug ) write(irefwr,*) 'prsw06c12'
               call prsw06c12 ( iformat, inodes, nnodes, usol, heading, &
                               coor, ndim, kprobf )

            end if  ! ( isuppress==0 )

         end if  ! ( number>0 .or. nunkp==1 )

      else

!     --- ichoice_print = 1, print coordinates and transform to polar ones

         if ( isuppress==0 ) then

!        --- isuppress = 0, print node numbers

            if ( debug ) write(irefwr,*) 'prsw06c13'
            call prsw06c13 ( iformat, inodes, nnodes, usol, heading, &
                            coor, ndim, kprobf, number, nunkp, indprf )

         else

!        --- isuppress = 1, do not print node numbers

            if ( debug ) write(irefwr,*) 'prsw06c14'
            call prsw06c14 ( iformat, inodes, nnodes, usol, heading, &
                            coor, ndim, kprobf, number, nunkp, indprf )

         end if  ! ( isuppress==0 )

      end if

1000  call erclos ( 'prsw06c' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06c'

      end if

      end
