      subroutine prsw06sup01 ( isuppress, inodes, usol, nnodes, &
                               iformat, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     real case
!     Special case ichoice_print=0, number>0 or nunkp=1
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isuppress, inodes(*), ind(*), iformat, nnodes
      double precision usol(*)

!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     ind            i    Integer array of length icount containing the
!                         indices of the nodes to be printed
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     isuppress      i    If 0 the node numbers are printed, &
!                         if 1 they are suppressed
!     nnodes         i    Number of nodes in inodes
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount

!     i              Counting variable
!     icount         Number of nodes to be printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06sup01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06sup01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0     ! number of non-zero entries

!     --- First store the indices of non-zero entries

      do i = 1, nnodes

         if ( abs(usol(i))>0d0 ) then

!        --- usol(i) # 0, store sequence number in ind

            icount = icount+1
            ind(icount) = i

         end if  ! ( abs(usol(i)>0d0 )

      end do  ! i = 1, nnodes

      if ( isuppress==0 ) then

!     --- isuppress = 0, print node numbers

         if ( iformat==1 ) then
            write(irefwr,220) (inodes(ind(i)), usol(ind(i)), i=1,icount)
         else if ( iformat==2 ) then
            write(irefwr,221) (inodes(ind(i)), usol(ind(i)), i=1,icount)
         else if ( iformat==3 ) then
            write(irefwr,222) (inodes(ind(i)), usol(ind(i)), i=1,icount)
         else
            write(irefwr,223) (inodes(ind(i)), usol(ind(i)), i=1,icount)
         end if

      else

!     --- isuppress = 1, do not print node numbers

         if ( iformat==1 ) then
            write(irefwr,230) (usol(ind(i)), i=1,icount)
         else if ( iformat==2 ) then
            write(irefwr,231) (usol(ind(i)), i=1,icount)
         else if ( iformat==3 ) then
            write(irefwr,232) (usol(ind(i)), i=1,icount)
         else
            write(irefwr,233) (usol(ind(i)), i=1,icount)
         end if

      end if
220   format((1x,4(i9,':',ss,1pe13.5e2)))
221   format((1x,4(i9,':',ss,1pe16.8e2)))
222   format((1x, (i9,':',ss,1pe13.5e2)))
223   format((1x, (i9,':',ss,1pe16.8e2)))
230   format((1x,4(ss,1pe13.5e2)))
231   format((1x,4(ss,1pe16.8e2)))
232   format((1x, (ss,1pe13.5e2)))
233   format((1x, (ss,1pe16.8e2)))

1000  call erclos ( 'prsw06sup01' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06sup01'

      end if  ! ( debug )

      end
