      subroutine prsw06sup07 ( inodes, nnodes, usol, coor, &
                               heading, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     real case
!     Special case ichoice_print=1,transform to polar coordinates
!     only one unknown
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nnodes, ndim
      double precision usol(*), coor(ndim,*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x, y, r, phi

!     i              Counting variable
!     phi            Angle
!     r              Radius
!     x              x coordinate
!     y              y coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06sup07' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06sup07'

      end if
      if ( ierror/=0 ) go to 1000

!     --- ichoice_print = 1, print coordinates and transform to polar ones
!         only one unknown

      if ( heading ) write(irefwr,350)
      do i = 1, nnodes
         x = coor(1,inodes(i))
         y = coor(2,inodes(i))
         r = sqrt(x**2+y**2)
         if ( x>0d0 ) then
            phi = atan(y/x)
         else
            phi = 0d0
         end if
         if ( abs(usol(i))>0d0 ) &
            write(irefwr,360) inodes(i), r, phi, usol(i)
      end do

350   format(' node       x            y         value')
360   format((1x,i9,ss,1pe13.5e2,1pe13.5e2,1pe13.5e2))

1000  call erclos ( 'prsw06sup07' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06sup07'

      end if

      end
