      subroutine prsw09 ( ichoice, nnodes, inodes, number, &
                          heading, icoor, iformat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 13-12-2007 Debug
!        version  4.1  date 11-12-2006 Extra debug
!        version  4.0  date 27-04-2005 Replace body by subroutines
!
!   copyright (c) 1985-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      This subroutine performs the actual printing for subroutine PRINVC
!      real case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, inodes(*), number, &
              nnodes, icoor, iformat
      logical heading

!     heading    i    If true a heading must be printed
!     ichoice    i    Choice parameter consisting of 3 parts:
!                     ichoice_print, isuppress and suppress_zeros according to
!                     ichoice = ichoice_print+10*isuppress+100*suppress_zeros
!                     with:
!                     ichoice_print
!                     0:   The nodal point number is printed followed by the
!                          values in the nodal point
!                     1:   The nodal point number is printed followed by the
!                          coordinates and then the values in the nodal point
!                     2:   The nodal point number is printed followed by the
!                          coordinates
!                     3:   See 2, however, preceded by the number of nodal
!                          points to be printed
!                     isuppress:
!                     if 1 the nodal point number is suppressed if 0 not
!                     suppress_zeros
!                     if 1 points in which the solution is zero are skipped, &
!                     if 0 not
!     icoor      i    Type of coordinate system
!                     1: Cartesian
!                     2: Cylindrical
!                     3: Polar
!     iformat    i    Format used for printing
!                     1: e13.5
!                     2: e16.8
!                     3: e13.5, one node per line
!                     4: e16.8, one node per line
!     inodes     i    Array containing the nodal point numbers for which
!                     printing must be performed
!     nnodes     i    Number of nodes in inodes
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered. If the array is complex number=2*i-1
!                     corresponds to the real part of array i and number=2*i
!                     to the imaginary part of array i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isuppress, ichoice_print, suppress_zeros

!     ichoice_print  0:   The nodal point number is printed followed by the
!                         values in the nodal point
!                    1:   The nodal point number is printed followed by the
!                         coordinates and then the values in the
!                         nodal point
!                    2:   The nodal point number is printed followed by the
!                         coordinates
!                    3:   See 2, however, preceded by the number of nodal
!                         points to be printed
!     isuppress      If 0 the node numbers are printed, if 1 they are suppressed
!     suppress_zeros if 1 points in which the solution is zero are skipped,
!                    if 0 not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSW05         Print the coordinates for subroutine PRINVC
!                    real case
!     PRSW06         Print the solution for subroutine PRINVC
!                    real case
!     PRSW06SUP      Print the solution for subroutine PRINVC
!                    Nodes with a zero solution are skipped
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw09' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw09'
         write(irefwr,1) 'ichoice', ichoice
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ichoice_print = mod(ichoice,10)
      isuppress = mod(ichoice,100)/10
      suppress_zeros = ichoice/100
      if ( debug ) write(irefwr,1) &
         'ichoice, ichoice_print, isuppress, suppress_zeros', &
          ichoice, ichoice_print, isuppress, suppress_zeros
      if ( ichoice_print==2 .or. ichoice_print==3 ) then

!     --- ichoice_print = 2, print coordinates

         if ( debug ) write(irefwr,*) 'before prsw05'
         call prsw05 ( ichoice_print, isuppress, nnodes, ndim, &
                       inodes, coor, iformat, heading )

      else if ( suppress_zeros==0 ) then

!     --- suppress_zeros = 0, print solution

         if ( debug ) write(irefwr,*) 'before prsw06'
         call prsw06 ( ichoice_print, number, isuppress, &
                       inodes, nnodes, iformat, heading, icoor )

      else

!     --- suppress_zeros = 1, print solution but suppress zeros

         if ( debug ) write(irefwr,*) 'before prsw06sup'
         call prsw06sup ( ichoice_print, number, nunkpo, isuppress, &
                          inodes, nnodes, usoltemp, indprfo, kprobfo, coor, &
                          iformat, heading, ndim, icoor )

      end if  ! ( ichoice_print==2 .or. ichoice_print==3 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw09'

      end if
      call erclos ( 'prsw09' )

      end
