      subroutine prtoavsext1 ( isol, usollc, npoint, &
                               namevec, iseq, nphyslc, nphysorig )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-04-2014 Extra parameter nphysorig
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 13-04-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution array with nphyslc degrees of freedom in each node
!     into solution array stored in usollc
!     This is meant for paraview input file
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     output
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iseq, nphyslc, npoint, nphysorig
      character (len=*) namevec
      double precision :: usollc(*)

!     iseq          i/o   Sequence number of component in output vector
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     namevec        i    Name of the vector
!     nphyslc          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     nphysorig      i    Number of physical degrees of freedom before reduction
!     npoint         i    Number of nodal points
!     usollc              temporary solution array organized per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOPYSEPUSER  Copy a sepran solution vector with one component per point
!                    into the iseq^th component of a user vector of size
!                    npoint x number_of_components
!     PRCOPYVECTRAN1 Copy solution from array usol stored in standard sepran
!                    sequence to array ucopy organized in columns of unknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2867   Value of inunkp not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtoavsext1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtoavsext1'
         write(irefwr,1) 'iseq, nphyslc, nphysorig',iseq, nphyslc, nphysorig
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nphyslc==1 ) then

!     --- One degree of freedom per point
!         Copy in vector

         call prcopysepuser ( isol, usollc, npoint, namevec, iseq )
         iseq = iseq+1

      else if ( nphyslc>=2 .and. nphyslc<=4 .or. nphyslc==6 ) then

!     --- More degrees of freedom per point
!         Copy in vector

         usoltemp => ks(isol(1))%sol

!        --- Copy vector

         call prcopyvectran1 ( usoltemp, npoint, &
                               usollc(1+(iseq-1)*npoint), nphyslc, nphysorig )
         iseq = iseq+nphyslc

      else

!     --- Situation not yet implemented

         call errint ( nphyslc, 1 )
         call errsub ( 2867, 1, 0, 0 )
         go to 1000

      end if  ! ( nphyslc==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtoavsext1'

      end if  ! ( debug )
      call erclos ( 'prtoavsext1' )

      end subroutine prtoavsext1
