      subroutine prvec0c ( vector, inppri, text, infovec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-06-2010 Change of layout
!        version  1.0  date 17-03-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print vector of special structure defined per element per integration point
!   It concerns vectors of type 129, complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     real
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), infovec(4,*)
      complex *16 vector(*)
      character *(*) text

!     infovec    i    Contains first integer part of vector of structure 129
!                     This part has length 4*nelgrp
!                     contents:
!                     (1,ielgrp): number of integration points
!                     (2,ielgrp): number of unknowns per point
!                     (3,ielgrp): number of elements in group
!                     (4,ielgrp): starting address of first element in
!                                 real part minus 1
!     inppri     i    Input array with respect to PRINVC (see PRINVC)
!     text       i    String variable containing some text
!     vector     i    Array of type 126 to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, ikelmc, ielem, irelem, &
              ielgrp, nunkp, m
      logical heading

!     heading    If true a heading must be printed
!     i          Counting variable
!     ielem      Element number
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     irelem     Relative element sequence number with respect to element group
!     k          Counting variable
!     m          number of integration points
!     nunkp      Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all element groups do
!       For all elements do
!          print part of vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      heading = inppri(8)==0

!     --- Write heading

      if ( heading ) then
         write(irefwr,100) text
100      format(/' vector ',a/)
      else
         write(irefwr,*)
      end if
      if ( heading ) then
         write(irefwr,110)
110      format ( 'element integrat. point                values ' )
      end if
      ikelmc = 0
      ielem = 0

!     --- For all element groups do

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         nunkp = infovec(2,ielgrp)
         m = infovec(1,ielgrp)

!        --- For all elements do

         do irelem = 1, inelem

            ielem = ielem + 1

!           --- print part of vector

            write(irefwr,200) ielem, 1, &
               (vector(ikelmc+k),k=1,min(2,nunkp))
200         format(i6, i4, ss,2('(',1pe13.5e2,',',1pe13.5e2,')'))
            if ( nunkp>2 ) write(irefwr,210) &
               (vector(ikelmc+k),k=3,nunkp)
210         format((10x, ss,2('(',1pe13.5e2,',',1pe13.5e2,')')))
            ikelmc = ikelmc + nunkp
            do i = 2, m
               write(irefwr,220) i, &
                  (vector(ikelmc+k),k=1,min(2,nunkp))
220            format(6x, i4, ss,2('(',1pe13.5e2,',',1pe13.5e2,')'))
               if ( nunkp>2 ) write(irefwr,210) &
                  (vector(ikelmc+k),k=3,nunkp)
               ikelmc = ikelmc + nunkp
            end do
         end do

      end do
      end
