      subroutine sepcompder
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes derived quantities of an input vector, either in the nodal points
!     or in the elements
!     Old subroutine to0047
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPASSEMSTREAC Reserve space in array IBUFFR with respect to
!                    subroutine sepcompreac
!                    Fill array ipoint in common CSUBWK
!     SEPDERBODY     Body of subroutine sepcompder
!     SEPDERSTART    Perform some tests for subroutine sepcompder
!     SEPREACSTART1  Start of assembly process
!                    Make all initializations, fill modules and declare arrays
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcompder' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcompder'
         write(irefwr,1) 'iprob', iprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kproba => kprobpart%kproba
      indpra = kprobloc(14)
      kprobc => kprobpart%kprobc

!     --- Perform some checks
!         Make initializations

      call sepderstart
      if ( ierror/=0 ) go to 1000

!     ---  Start of assembly process
!          Make some initializations, fill modules and declare arrays

      if ( debug ) write(irefwr,*) 'before sepreacstart1'
      call sepreacstart1
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for the element groups

      if ( debug ) write(irefwr,*) 'before sepassemstreac'
      call sepassemstreac

!     --- compute the output vector

      if ( debug ) write(irefwr,*) 'before sepderbody'
      call sepderbody

!     --- deallocate arrays

      if ( debug ) write(irefwr,*) 'before deallocate'
      deallocate ( istold, istold1, oldsols, indoldsols, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istoldlc' )

      call seprmmatstr2  ! nullify arrays in modulematstr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcompder'

      end if  ! ( debug )
      call erclos ( 'sepcompder' )

      end subroutine sepcompder
