      subroutine sepcreaskip ( iskip, iskpel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array skipelgrps depending on iskip and array iskpel
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesmf
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iskip, iskpel(nelgrp)

!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskpel         i    Array containing the element groups to be skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreaskip' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreaskip'
         write(irefwr,1) 'iskip', iskip
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iskip>0 ) then

!     --- iskip>0, Copy array iskpel in iskip (indskp)

         skipelgrps = iskpel

      else

!     --- iskip = 0, set array skip

         skipelgrps = 1

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( skipelgrps, nelgrp, 'skipelgrps' )
         write(irefwr,*) 'End sepcreaskip'

      end if  ! ( debug )
      call erclos ( 'sepcreaskip' )

      end subroutine sepcreaskip
