      subroutine sepderindex ( ivectr, nunkploc, indprfloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill index25 with positions of degrees of freedom in element ielem
!      as stored in vector denoted by ivectr
!      for ivectr (ks(ivectr)%typevec=110,115,116.126)
!      Old subroutine to0038
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivectr, nunkploc, indprfloc

!     indprfloc      i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     ivectr         i    standard sepran array referring to a vector
!     nunkploc       i    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jcount, i, inunkp, inode, inusol, j

!     i              Counting variable
!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jcount         Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderindex' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderindex'
         call ertrace
         write(irefwr,1) 'posinp, nunkploc, indprfloc', &
                          posinp, nunkploc, indprfloc
         write(irefwr,1) 'ivectr, itypvc', ivectr, ks(ivectr)%typevec
         call prinin ( kprobatmp, inpelm, 'kprobatmp' )
         call prinin ( index1, inpelm, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount = 0
      if ( ks(ivectr)%typevec==110 .or. ks(ivectr)%typevec==115 ) then

!     --- itypvc = 110 or 115

         if ( indprfloc==0 ) then

!        --- number of degrees of freedom is constant for itypvc = 110,115

            inunkp = ks(ivectr)%nusol/npoint
            if ( inunkp<=1 ) then

!           --- simplest case : 1 degree of freedom per nodal point

               index25(1:inpelm) = index1
               jcount = inpelm
               do i = 1, inpelm
                  indexh(i) = i
               end do  ! i = 1, inpelm

            else

!           --- number of degrees of freedom is constant but>1

               do i = 1, inpelm
                  inode = index1(i)
                  inusol = (inode-1)*inunkp
                  do j = 1,inunkp
                     index25(jcount+j) = inusol+j
                  end do
                  jcount = jcount+inunkp
                  indexh(i) = jcount
               end do

            end if

         else

!        --- number of degrees of freedom not constant

            if ( debug ) call prinin ( kprobflc, npoint, 'kprobflc' )
            do i = 1, inpelm
               inode = index1(i)
               inusol = kprobflc(inode)
               inunkp = kprobatmp(i)
               do j = 1,inunkp
                  index25(jcount+j) = inusol+j
               end do
               jcount = jcount+inunkp
               indexh(i) = jcount
            end do

         end if

      else if ( ks(ivectr)%typevec==116 .or. ks(ivectr)%typevec==136 ) then

!     --- vector of special structure is defined per element , itypvc = 116
!         number of degrees of freedom per element is also in ivectr(4)

         jcount = ks(ivectr)%ivec
         inusol = (ielem-1)*jcount
         do j = 1, jcount
            index25(j) = inusol+j
         end do
         indexh(1) = jcount

      else if ( ks(ivectr)%typevec==126 ) then

!     --- vector of special structure is defined per element per node,
!         itypvc = 126
!         number of degrees of freedom per element is also in ivectr(4)

         jcount = ks(ivectr)%n
         inusol = posinp*jcount
         do j = 1, jcount*inpelm
            index25(j) = inusol+j
         end do
         indexh(1) = jcount

      else

!     --- vector of special structure is defined per element per integration
!         point, itypvc = 129
!         number of degrees of freedom per element is also in ivectr(4)

         jcount = ks(ivectr)%n
         inusol = posinp*jcount
         do j = 1, jcount*m
            index25(j) = inusol+j
         end do
         indexh(1) = jcount

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( index25, jcount, 'index25' )
         write(irefwr,*) 'End sepderindex'

      end if  ! ( debug )
      call erclos ( 'sepderindex' )

      end subroutine sepderindex
