      subroutine sepderintbody
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutine sepcompderint
!     Compute integer quantities per element using one or more
!     input vectors
!     Old subroutine to0047int
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc, ipkproba, lenindex3, ih

!     ih             Contains last address used in output vector
!     ikelmc         position in kmesh part c
!     ipkproba       start address in kprob part a
!     lenindex3      length of index3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCSUBSINT     Call specific element subroutine, depending on itype
!                    The output vector is integer
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYELEM    Copy information of elements in type element
!     SEPDERELGRP    Initialize some parameters for one element group
!     SEPDERVDALLOC  De-allocate temporary arrays in for subroutine to0112
!     SEPDERVINDEX   Fill arrays index2, index3 and index4 for subroutine deriv
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     211   notvec = 1  for relative element number more than 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderintbody' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderintbody'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kproba => kprobpart%kproba
      indpra = kprobloc(14)
      kprobc => kprobpart%kprobc

      ielem  = 0
      ikelmc = 0
      posout = 0
      ipkproba = 1
      ih = 0

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         itype = kprobc(ielgrp)

!        --- Fill information for element group in E and allocate some
!            element arrays

         if ( itype<=0 .or. skipelgrps(ielgrp)==0 ) then

!        --- element group ielgrp must be skipped
!            Raise parameters

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem

         else

            ifirst = 0
            vector = .true.
            icouno = sepfillcountvc ( SMF%ioutvc, ielgrp )

            if ( debug ) &
               write(irefwr,1) 'ielgrp, ikelmc, icouno', &
                                ielgrp, ikelmc, icouno

            call sepderelgrp


!           --- Fill information for element group in E and allocate some
!               element arrays

            allocate ( ielemvc(icouno), stat=error )
            if ( error/=0 ) call eralloc ( error, icouno, 'ielemvc' )
            if ( ierror/=0 ) go to 1000

            ielemvc = 0

!           --- loop over all elements of this group

            do irelem = 1, inelem

               ielem  = ielem + 1
               E%ielem = ielem
               E%irelem = irelem
               E%first = ifirst

               index1 => kmeshc(ikelmc+1:ikelmc+inpelm)

!              --- Fill arrays index2/3/4

               call sepdervindex ( ikelmc, ipkproba, SMF%ioutvc )

!              --- Copy information into type elem

               call sepcopyelem

               call elcsubsint
               if ( ierror/=0 ) go to 1000

               if ( notvec/=0 ) then

!              --- skip other elements of this group since the element vector
!                  is zero

                  if ( irelem>1 ) then
                     call errint ( ielgrp, 1 )
                     call errint ( irelem, 2 )
                     call errsub ( 211, 2, 0, 0 )
                     go to 1000
                  end if
                  ikelmc = ikelmc+(inpelm-1)*inelem
                  ielem = ielem+inelem-1
                  go to 500

               end if

!              --- add element element vectorto large vector

               ivectout(ih+1:ih+icouno)=ivectout(ih+1:ih+icouno)+ielemvc
               ih = ih+icouno

               ikelmc = ikelmc+inpelm
               if ( ichtst/=7 ) then
                  posout = ikelmc
               else
                  posout = posout+m
               end if  ! ( ichtst/=7 )

            end do  ! irelem = 1, inelem

         end if  ! ( itype<=0 .or. skipelgrps(ielgrp)==0 )

!        --- Deallocate temporary arrays

500      ipkproba = ipkproba+inpelm

         call sepdervdalloc
         if ( ierror/=0 ) go to 1000

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderintbody'

      end if  ! ( debug )
      call erclos ( 'sepderintbody' )

      end subroutine sepderintbody
