      subroutine sepderperiod
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt the derivative for periodic boundary conditions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc

!     ikelmc         position in kmesh part c
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0042         apply averaging for periodical boundary conditions
!     TO0042C        apply averaging for periodical boundary conditions
!                    (complex)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderperiod' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderperiod'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ikelmc = 0
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- nelgrp>1 determine number of elements of this group
!                    and number of nodal points of an element

         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         itype = kprobc(ielgrp)
         if ( itype==-1 .and. skipelgrps(ielgrp)/=0 ) then

!        --- loop over all elements of this group

            do irelem = 1, inelem

               ielem = ielem+1
               index1 => kmeshc(ikelmc+1:ikelmc+inpelm)

!              --- element vector must be averaged

               if ( complo ) then

!              --- Complex case

                  call to0042c

               else

!              --- Real case

                  call to0042

               end if
               ikelmc = ikelmc+inpelm

            end do

         else

!        --- skip all element groups, except those with itype = -1 that
!            satisfy certain conditions (.not.skipel).or.(ielhlp==1)
!            revise pointers as a whole group of elements will be skipped

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderperiod'

      end if  ! ( debug )
      call erclos ( 'sepderperiod' )

      end subroutine sepderperiod
