      subroutine sepmapsol1d ( isol, func )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate one degree of freedom of sepran vector in user vector
!     Array imapfn and mapfn are used
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmoduleint2d
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol
      double precision, intent(out) :: func(numint)

!     func           o    contains computed function values
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPMAPSOL1D1   Interpolate one degree of freedom of sepran vector in
!                    user vector
!                    Constant number of degrees per point
!     SEPMAPSOL1D2   Interpolate one degree of freedom of sepran vector in
!                    user vector
!                    Variable number of degrees per point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmapsol1d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmapsol1d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and related arrays

      call sepactsolbf2 ( isol )

!     --- Compute starting addresses

      usol => ks(isol)%sol

      if ( indprfi==0 ) then

!     --- indprfi = 0, constant number of degrees per point

         call sepmapsol1d1 ( usol, nunkpi, func )

      else

!     --- indprfi>0, variable number of degrees per point

         call sepmapsol1d2 ( func, usol )

      end if  ! ( indprfi==0 )

!     --- deallocate mapfn and imapfn

      deallocate ( mapfn, imapfn, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'mapfn' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmapsol1d'

      end if  ! ( debug )
      call erclos ( 'sepmapsol1d' )

      end subroutine sepmapsol1d
