      subroutine sepprecprin1 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-02-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print minimum and maximum element of diagonal of matrix
!     Old subroutine precmkprin1
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imin, imax, i
      double precision amin, amax, ah

!     ah             hep variable to store elemnt of matrix
!     amax           maximal element of a matrix
!     amin           minimal element of a matrix
!     i              Counting variable
!     imax           index of the maximal element of a matrix
!     imin           index of the minimal element of a matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecprin1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecprin1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initializations

      imin = 1
      imax = 1
      amin = A%S(1)
      amax = A%S(1)
      do i = 2, A%neq

         ah = A%S(A%diag(i))
         if ( ah<amin) then
            imin = i
            amin = ah
         end if

         if ( ah>amax) then
            imax = i
            amax = ah
         end if

      end do  ! i = 2, A%neq
      write ( irefwr,100 ) amin, imin
      write ( irefwr,110 ) amax, imax

!     --- Next for absolute values

      imin = 1
      imax = 1
      amin = abs(A%S(1))
      amax = abs(A%S(1))

      do i = 2, A%neq

         ah = abs(A%S(A%diag(i)))
         if ( ah<amin) then
            imin = i
            amin = ah
         end if

         if ( ah>amax) then
            imax = i
            amax = ah
         end if

      end do

      write ( irefwr,120 ) amin, imin
      write ( irefwr,130 ) amax, imax

100   format ( //' The minimal diagonal  element of the matrix is ', &
               1pe10.2e2, ' in row ', i7 )
110   format (   ' The maximal diagonal  element of the matrix is ', &
               1pe10.2e2, ' in row ', i7 )

120   format (   ' The minimal abs(diag) element of the matrix is ', &
               1pe10.2e2, ' in row ', i7 )
130   format (   ' The maximal abs(diag) element of the matrix is ', &
               1pe10.2e2, ' in row ', i7  / )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprecprin1'

      end if  ! ( debug )
      call erclos ( 'sepprecprin1' )

      end
