      subroutine seppriglob ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print global vector in parallel case
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulevecs
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(0:*)
      double precision rinsep(*)

!     inpsep         i    See subroutine askprivec
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqnr, lentxt, ivec, typevec, i, irefwrsav, iname
      character (len=120) :: textloc
      integer, pointer :: inodes(:)
      character (len=120) filename

!     filename       Name of file
!     i              General loop variable
!     iname          Sequence number of name in text array
!     inodes         Contains the set of node numbers to be printed
!     irefwrsav      Help variable to store the value of irefwr
!     iseqnr         Sequence number with respect to the storage in
!                    array ISEPIN
!     ivec           Sequence number of vector of special structure
!     lentxt         length of text parameter, with trailing blanks deleted
!     textloc        String to store a local text
!     typevec        Type of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICLOSEFILE   Close file and reset irefwr
!     INIOPENFILE    Open serial file and set irefwr
!     PRININ         print 1d integer array
!     SEPGATHER      Compute global vector in case of parallel computations by
!                    combining parts of the vectors from the nodes
!     SEPHEADING1    Print heading: coordinates values
!     SEPPRIRVEC     Print real vector usoltemp
!     SEPSORTCOOR    Sort nodal points with respect to increasing co-ordinates
!     SEPSTORENEWSOL Find all degrees of freedom from the solution array or
!                    array of special structure as indicated by array inodes and
!                    store in output array. Real case
!     SEPTEXTVEC     Compute textloc and lentxt as function of iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppriglob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppriglob'
         write(irefwr,1) 'isolut, istartmn', isolut, istartmn
         call prinin ( inpsep(1), 20, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store global vector in usolglob

      iseqnr = inpsep(1)   ! sequence number read
      isolut => isolut2d(iseqnr)

      call sepgather ( isolut )

      if ( inodenr>1 ) go to 1000

!     --- Printing is only performed on node 1

      filename = ' '   ! start file name
      if ( inpsep(15)>0 ) then

!     --- File name found

         iname = inpsep(15)  ! sequence number of file
         call iniopenfile ( iname, irefwrsav, inpsep(22), &
                            filename )

      end if  ! ( inpsep(15)>0 )

      usol => ks(isolut)%solglob
      typevec = ks(isolut)%typevec
      iprob = ks(isolut)%iprob
      iseqkp = ks(isolut)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobfglobal => kprobpart%kprobfglobal
      kprobloc => kprobpart%kprobloc
      iseqkm = kp(iseqkp)%iseqkm
      ivec = ks(isolut)%ivec
      if ( debug ) &
         write(irefwr,1) 'typevec, iprob, iseqkp, ivec', &
                          typevec, iprob, iseqkp, ivec

!     --- Sort coordinates in increasing sequence

      coor => km(iseqkm)%coorglob
      allocate ( inodes(npointglob), usoltemp(size(usol)) )
      do i = 1, npointglob
         inodes(i) = i
      end do  ! i = 1, npointglob
      call sepsortcoor ( 1, npointglob, inodes )

!     --- Fill textloc and lentxt

      call septextvec ( iseqnr, textloc, lentxt )

!     --- Print vector

      write(irefwr,100) textloc(1:lentxt)
100   format(/' vector ',a/)

      call sepheading1 ( 1 )  ! heading
      if ( debug )  &
         write(irefwr,*) 'associated(kprobfglobal(ivec)%arr) ',&
                          associated(kprobfglobal(ivec)%arr)
      if ( associated(kprobfglobal(ivec)%arr) ) then
         indprfi = 1
         kprobfi => kprobfglobal(ivec)%arr
         indprfo = 1
      else
         indprfi = 0
         if ( typevec==110 ) then
            nunkpi = kprobloc(4)
         else
            kprobe2 => kprobpart%kprobe2
            nunkpi = kprobe2(ivec)
         end if  ! ( typevec==110 )
      end if  ! ( associated(kprobfglobal(ivec)%arr) )

      unew => usoltemp
      usol => usolglob
      nullify(istore)
      if ( indprfi>0 ) allocate ( istore(npointglob+1))
      call sepstorenewsol ( 0, 1, inodes, npointglob )
      kprobfo => istore
      nunkpo = kprobloc(4)
      indprfo = indprfi

      call sepprirvec ( 1, inodes, npointglob, 1 )

      if ( associated(istore) ) deallocate(istore)

      deallocate ( inodes, usoltemp )

      if ( inpsep(15)>0 ) call iniclosefile ( irefwrsav )
      coor => km(iseqkm)%coor

!     --- Raise istartmn

1000  istartmn = istartmn + 23 + inpsep(20)

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End seppriglob'

      end if  ! ( debug )
      call erclos ( 'seppriglob' )

      end subroutine seppriglob
