      subroutine sepprincompstr ( nrusol, intmt1, intmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-04-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print part of array intmat corresponding to internal unknowns
!     Compact storage
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nrusol, intmt1(*), intmt2(*)

!     intmt1   i  first part of array intmat part 1
!     intmt2   i  second part of array intmat part 1
!     nrusol   i  number of free degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, n, m, k

!     i              Counting variable
!     isum           Integer variable to compute a sum
!     k              Counting variable
!     m              number of items to be printed in one line
!     n              number of items to be printed for one row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprincompstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprincompstr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,200)
200   format( ' compact storage of matrix'/ &
              ' rows i coupled with internal degrees of freedom' )
      isum=intmt1(1)
      do i=1,nrusol
         n=intmt1(i+1)-isum
         m=min(10,n)
         write(irefwr,210) i,(intmt2(isum+k),k=1,m)
210      format(1x,i6,':',1x,10i6)
         if(n>m) write(irefwr,220) (intmt2(isum+k),k=m+1,n)
220      format(9x,10i6)
         isum=isum+n
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprincompstr'

      end if  ! ( debug )
      call erclos ( 'sepprincompstr' )

      end subroutine sepprincompstr
