      subroutine sepprinkmeshm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print array kmesh part m
!     Old subroutines prinkmeshm and pr0008
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, istar4, i, n, j

!     i          Counting variable
!     istar3     Starting address of KMESHM part 3 in KMESH part M
!     istar4
!     j          Counting variable
!     n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate array
!     compute starting address
!     print array kmeshm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkmeshm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinkmeshm'

      end if
      if ( ierror/=0 ) go to 1000

      ninner=kmeshm(1)
      istar3=kmeshm(2)
      istar4=kmeshm(3)
      write(irefwr,100)
100   format(' nodal point numbers of the curves')

      do i = 1,ncurvs
         n=kmeshm(5+i)-kmeshm(4+i)
         if ( n>0 ) then
            write(irefwr,110)i,(kmeshm(istar3+j-1),j=1,n)
         else
            write(irefwr,120)i
         end if  ! ( n>0 )
110      format(' curve number', i6/ &
                '  nodal point numbers:' /(1x,10i7) )
120      format(' curve number', i6, '  is empty' )
         istar3=istar3+n
      end do

      if ( ninner>0 ) then
         write(irefwr,210) (kmeshm(istar4+j-1),j=1,ninner)
210      format(' the following curves are internal:'/(1x,10i7))
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinkmeshm'

      end if
      call erclos ( 'sepprinkmeshm' )

      end
