      subroutine sepprinmat4 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print inverse matrix that is stored as profile complex matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat   ! definition of matrices
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, leftcl, nel, mult

!     i              Counting variable
!     istart         First position used at a part
!     leftcl         smallest column number in row
!     mult           multiplication factor
!     nel            number of elements in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0019         print row of complex matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinmat4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinmat4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,*) 'Inverse matrix'
      if ( A%symmetric ) then
         mult = 1
      else
         mult = 2
      end if  ! ( A%symmetric )

      istart = 1
      do i = 1, A%neq
         leftcl = A%Mcols(i)
         nel = mult*(i-leftcl)+1
         call pr0019(A%MD(istart), nel, i)
         istart = istart+nel
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinmat4'

      end if  ! ( debug )
      call erclos ( 'sepprinmat4' )

      end subroutine sepprinmat4
