      subroutine sepprintmatc ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print large complex matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr       i    Standard SEPRAN array containing information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      type (cmatrix), pointer :: Ac
      type (globmat), pointer :: A

!     A              Structure of type globmat containing the necessary
!                    information of the matrix
!     Ac             Structure of type rmatrix containing the necessary
!                    information of the matrix
!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINMAT8    Print complex matrix stored as row-compact complex matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprintmatc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprintmatc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      A => realmt(matr)
      IM => intmt(A%intmatref)
      S_ii => A%S_ii
      Ac => A%Sc_i

      if ( .not. IM%nosepmat ) call sepprinmat8 ( Ac )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprintmatc'

      end if  ! ( debug )
      call erclos ( 'sepprintmatc' )

      end subroutine sepprintmatc
