      subroutine sepreacoutvec
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create output vector for sepcompreac and
!     Activate input vector
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: ioutvc

!     ioutvc         Pointer referring to SMF%ioutvc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2058   input vector is not of the type solution vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreacoutvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreacoutvec'
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ioutvc => SMF%ioutvc

      if ( ks(isolut2d(iseqi1))%typevec/=110 ) then

!     --- Input vector is not of the type solution vector

         call errint ( iseqi1, 1 )
         call errint ( ks(isolut2d(iseqi1))%typevec, 2 )
         call errsub ( 2058, 2, 0, 0 )
         go to 1000

      end if

!     --- Compute array lengths

      compl = ks(isolut2d(iseqi1))%compl==1

!     --- Create output vector

      if ( debug ) call prinin ( ioutvc, 5, 'ioutvc' )

!     --- Activate input vector

      call sepactsolbf2 ( isolut2d(iseqi1) )

!     --- Create space for the output vector

      call sepcreasol110 ( ioutvc, 'outvec', &
                           ks(isolut2d(iseqi1))%compl )
      nusol = ks(ioutvc)%nusol

      if ( debug ) call prinin ( ioutvc, 5, 'ioutvc/2' )

      if ( nunkpo==1 ) ichnun = 1
      if ( nunkpo>1 ) ichnun = 3
      if ( indprfi>0 ) ichnun = ichnun+1  ! kprobf>0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nunkpo, indprfi, ichnun', nunkpo, indprfi, ichnun
         write(irefwr,*) 'End sepreacoutvec'

      end if  ! ( debug )
      call erclos ( 'sepreacoutvec' )

      end subroutine sepreacoutvec
