      subroutine sepreacstart
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some tests
!     First routine to be called in sepcompreac
!     Similar to sepbuildstart, without intmat
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

!     SMF            i    Contains all input for subroutine SEPBUILD
!                         See modules SEPMODULESMF
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTREAC     Activate and create arrays for module sepmodulebuild
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPFILLOLDSOL  Fill old solutions as well as array flipflop in
!                    sepmodulebuild
!     SEPINITBUILD   Allocate some arrays for subroutine to0050
!                    Initialize some parameters
!     SEPINITDER     Initialize parameters for subroutine sepcompreac
!     SEPREACOUTVEC  Create output vector for sepcompreac and
!                    Activate input vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreacstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreacstart'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      bufchanged = .true.

!     --- Perform some tests and store paramters in sepmodulebuild

      call sepinitder
      if ( debug ) write(irefwr,1) 'iprob, numold', iprob, numold
      if ( nusol-nbound-npboun==0 ) go to 1000  ! there are no unknowns

!     --- if usol exits, activate array and renumber if necessary

      if ( ierror/=0 ) go to 1000

      nusolcac = nusol
      oldsol = .false.

!     --- Allocate some arrays for subroutine sepbuild
!         Initialize some parameters
!         These parameters and arrays are all used by element subroutines

      call sepinitbuild
      if ( ierror/=0 ) go to 1000

!     --- Fill some parameters in sepmodulematstr

      call sepfillmatstr2

!     --- Activate and create arrays for module sepmodulebuild

      call sepactreac

!     --- Fill pointers in sepmodulebuild for subroutine build
!         Create relation between arrays and buffer

      call sepfilloldsol
      if ( ierror/=0 ) go to 1000

!     --- Create output vector

      call sepreacoutvec
      if ( ierror/=0 ) go to 1000

      addvec = .true.  ! no right hand side
      addmat = .true.

      vectin => ks(isolut2d(iseqi1))%sol

      vectout => ks(SMF%ioutvc)%sol

      if ( iinder(9)==0 ) vectout =0d0  ! clear vector

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'addmat, clrmat, addvec, clrvec, compl', &
                          addmat, clrmat, addvec, clrvec, compl
         write(irefwr,*) 'End sepreacstart'

      end if  ! ( debug )
      call erclos ( 'sepreacstart' )

      end subroutine sepreacstart
