      subroutine sepstream2 ( ichois, ielhlp, knodp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     yields the element and group in which knodp is lying
!     Old subroutine to0041
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecactl
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ielhlp(*), knodp

!     ichois  i    Choice parameter
!     ielhlp  i    This array of length nelgrp+5 is only used if allelements = 1
!                  ielhlp(ielgrp+5) = 0 means elements of element group ielgrp
!                  are skipped, ielhlp(ielgrp+5) = 1, means that they are
!                  assembled
!     knodp   i    Nodal point number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jtest, i

!     i       Counting variable
!     jtest   If 1 element and group have been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     215   Expected two-dimensional problem on two-dimensional mesh does
!           not exist or starting point is not lying in an accessible element
!           group.
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstream2' )

!     --- find the group and the shape of the elements with knodp
!         continue if a "good" group has been found

      jtest = 0

!     --- Loop over elements the node is lying in

      do i = kmshd1(knodp)+1, kmshd1(knodp+1)

         ielem = kmshd2(i)             ! element number
         ielgrp = sepelgrpelem(ielem)  ! element group

         itype = kprobc(ielgrp)        ! type number

         if ( itype>0 .and. kmeshf(ielgrp)>2 ) then

!        --- Type number > 0 and surface element

            if ( ichois==0 ) then

!           --- Do not skip element groups

               jtest = 1
               exit

            else if ( ielhlp(5+ielgrp)/=0 ) then

!           --- This element group should not be skipped

               jtest = 1
               exit

            end if  ! ( ichois==0 )

         end if  ! ( itype>0 .and. kmeshf(ielgrp)>2 )

      end do  ! i = kmshd1(knodp)+1, kmshd1(knodp+1)

      if ( jtest==0 ) call errsub ( 215, 0, 0, 0 )  ! No element found

      call erclos ( 'sepstream2' )

      end subroutine sepstream2
