      subroutine sepstreamchk ( ivec, ivctst, allelements, ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     information from backing storage (file 1), as stored in
!     istfl1 for standard elements with standard element number > 99
!     is analyzed : the parameter ivec for the stream function
!     must be equal for all itype of the problem (and equal to
!     ivec for type numbers less than 100
!     skip element groups with ishape<=2
!     Old subroutine to0034
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec, ivctst, allelements, ielhlp(*)

!     allelements    i    Choice parameter, see ielhlp
!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, ielhlp(ielgrp+5) = 1, means that
!                         they are assembled
!     ivctst         o    value of ivec for a standard problem (itype>=100)
!     ivec           i    Sequence number of vector of special structure
!                         for subroutine stream
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ifirst, indtvc, ipos, itype, ivcstr, &
              jtest, ncheld, istfl1(2*nrec1)
      logical skip

!     ielgrp         element group number
!     ifirst         ifirst=0  istfl1 is to be filled by subroutine to0025
!                         for the first time
!                    ifirst=1  is set by subroutine after processing istfl1
!     indtvc         Number of vectors of special structure
!     ipos           position in istfl1 where information about standard
!                    problem itype starts
!     istfl1         Array in which information from FILE 1 is stored
!     itype          type number of standard element
!     ivcstr         type of vector for stream function
!     jtest          = 0, all type numbers < 100
!                    > 99, at least one type number with type number jtest
!                    all with respect to one standard element
!     ncheld         Number of choices for derivative subroutine
!     skip           If true the element group must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TO0025         information from backing storage (file 1) is stored in
!                    array istfl1 for a standard problem with standard problem
!                    number > 99
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     228   Parameter ivec has different values for different element types
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstreamchk' )
      ifirst=0
      jtest=0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         itype = kprobc(ielgrp)
         skip = ishape<=2 .or. itype==0 .or. itype>=200 .and. itype<=300
         if ( .not. skip .and. allelements==1 ) &
            skip = ielhlp(ielgrp+5)==0
         if ( .not. skip ) then
            itype=kprobc(ielgrp)
            if (itype<100) then

!           --- set test values for itype < 100

              if (jtest<=0) then

!             --- jtest <= 0

                  ivctst=ivec
                  jtest=itype

               else

!              --- jtest > 0
!                  compare values

                  if (ivec/=ivctst) then
                     call errint ( ivec, 1 )
                     call errint ( ivctst, 2 )
                     call errint ( itype, 3 )
                     call errsub ( 228, 3, 0, 0 )
                  end if

               end if

            else

!           --- itype>=100

               call to0025(itype,ifirst,ipos,istfl1,ielgrp)
               indtvc=istfl1(ipos+1)
               ncheld=istfl1(ipos+4+indtvc)
               ivcstr=istfl1(ipos+6+indtvc+2*ncheld)
               if (jtest<=0) then

!              --- set test values for itype >= 100

                  ivctst=ivcstr
                  jtest=itype

               else

!              --- compare values
!                  ivec different for itype and itype1(=jtest)

                  if (ivcstr/=ivctst)then
                     call errint ( ivcstr, 1 )
                     call errint ( ivctst, 2 )
                     call errint ( itype, 3 )
                     call errsub ( 228, 3, 0, 0 )
                  end if

               end if

            end if

         end if

      end do

      call erclos ( 'sepstreamchk' )

      end subroutine sepstreamchk
