      subroutine to0071 ( xstart, ystart, zstart, coor, array, inpelm, &
                          ielem, nelem, ndim )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.1  date 21-03-1991 adjusted for NDIM = 3
!        version  1.0  date 14-11-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The number of the element in which the point ( xstart, ystart ) lies
!     is determined
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*), xstart, ystart, zstart
      integer array(*), ielem, inpelm, ndim, nelem

!     array    i  integer array containing KMESH part c (see PG 13.1.3)
!     coor     i  standard SEPRAN array, containing the coordinates of
!                 all nodal points
!     ielem    o  the number of the element in which (xstart,ystart) lies
!     inpelm   i  the number of nodal points per element
!     ndim     i  the dimension of space
!     nelem    i  the number of elements
!     xstart   i  real, the x-coordinate of the point
!     ystart   i  real, the y-coordinate of the point
!     zstart   i  real, the z-coordinate of the point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision bmat(3,3), j, binv(3,3), b(3), xloc(3), x(30), &
                       eps, xmin, xmax, ymin, ymax, zmin, zmax
      integer iloc, i

!     b           real vector, the translation vector of the mapping
!     binv        real matrix, the inverse of Bmat
!     bmat        real matrix, the mapping to a reference element
!     eps         real, for permitting a small error in the calculation
!                 of the local coordinates
!     i           Counting variable
!     iloc        integer, the number of the edge on which the point lies
!                 (not used)
!     j           real, the determinant of Bmat
!     x           real array, the coordinates of the midths of the edges
!                 of an element
!     xloc        real array, the local coordinates of (xstart,ystart)
!     xmax        real, the maximum value of the x-coordinates of the nodal
!                 points of one element
!     xmin        real, the minimum value of the x-coordinates of the nodal
!                 points of one element
!     ymax        real, the maximum value of the y-coordinates of the nodal
!                 points of one element
!     ymin        real, the minimum value of the y-coordinates of the nodal
!                 points of one element
!     zmax        real, the maximum value of the z-coordinates of the nodal
!                 points of one element
!     zmin        real, the minimum value of the z-coordinates of the nodal
!                 points of one element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELM651    fill coordinates of midths of edges into local array
!     ELM659    invert an NDIM*NDIM matrix
!     ELM660    determine matrix of the mapping to a reference element
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     TO0075    determine the translation vector of the mapping
!     TO0076    determine the local coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ---------------------------------------------------------------------- &
!   | while ( not found and not last_element )                           |
!   |   -----------------------------------------------------------------|
!   |   | take next element                                              |
!   |   |----------------------------------------------------------------|
!   |   | check if the point lies in the rectangle around this element   |
!   |   |----------------------------------------------------------------|
!   |   | if so then                                                     |
!   |   |   -------------------------------------------------------------|
!   |   |   | calculate local coordinates                                |
!   |   |   |------------------------------------------------------------|
!   |   |   |  \      0<=xlocal<=1-ylocal and 0<=ylocal<=1-xlocal      / |
!   |   |   |  T \      and 0<=zlocal<=1-xlocal-ylocal               / F |
!   |   |   |------------------------------------------------------------|
!   |   |   |   found = true               |                             |
!   |--------------------------------------------------------------------- &
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0071' )
      eps   = 1d-5
      i     = 0
      ielem = 0

!     --- while element is not found (i.e. ielem=0) and there are still elements

10    if ( ( ielem.eq.0 ) .and. ( i.lt.nelem ) ) then

!     --- take next element

         i = i + 1

!        --- determine if (xstart,ystart) lies in rectangle around this element

         if ( ndim.eq.2 ) then
            xmin = min ( coor(2*array(1+(i-1)*inpelm)-1), &
                         coor(2*array(3+(i-1)*inpelm)-1), &
                         coor(2*array(5+(i-1)*inpelm)-1) )
            ymin = min ( coor(2*array(1+(i-1)*inpelm)), &
                         coor(2*array(3+(i-1)*inpelm)), &
                         coor(2*array(5+(i-1)*inpelm)) )
            xmax = max ( coor(2*array(1+(i-1)*inpelm)-1), &
                         coor(2*array(3+(i-1)*inpelm)-1), &
                         coor(2*array(5+(i-1)*inpelm)-1) )
            ymax = max ( coor(2*array(1+(i-1)*inpelm)), &
                         coor(2*array(3+(i-1)*inpelm)), &
                         coor(2*array(5+(i-1)*inpelm)) )
            zmin = -rinfin
            zmax =  rinfin
         else if ( ndim.eq.3 ) then
            xmin = min ( coor(3*array( 1+(i-1)*inpelm)-2), &
                         coor(3*array( 3+(i-1)*inpelm)-2), &
                         coor(3*array( 5+(i-1)*inpelm)-2), &
                         coor(3*array(10+(i-1)*inpelm)-2) )
            xmax = max ( coor(3*array( 1+(i-1)*inpelm)-2), &
                         coor(3*array( 3+(i-1)*inpelm)-2), &
                         coor(3*array( 5+(i-1)*inpelm)-2), &
                         coor(3*array(10+(i-1)*inpelm)-2) )
            ymin = min ( coor(3*array( 1+(i-1)*inpelm)-1), &
                         coor(3*array( 3+(i-1)*inpelm)-1), &
                         coor(3*array( 5+(i-1)*inpelm)-1), &
                         coor(3*array(10+(i-1)*inpelm)-1) )
            ymax = max ( coor(3*array( 1+(i-1)*inpelm)-1), &
                         coor(3*array( 3+(i-1)*inpelm)-1), &
                         coor(3*array( 5+(i-1)*inpelm)-1), &
                         coor(3*array(10+(i-1)*inpelm)-1) )
            zmin = min ( coor(3*array( 1+(i-1)*inpelm)), &
                         coor(3*array( 3+(i-1)*inpelm)), &
                         coor(3*array( 5+(i-1)*inpelm)), &
                         coor(3*array(10+(i-1)*inpelm)) )
            zmax = max ( coor(3*array( 1+(i-1)*inpelm)), &
                         coor(3*array( 3+(i-1)*inpelm)), &
                         coor(3*array( 5+(i-1)*inpelm)), &
                         coor(3*array(10+(i-1)*inpelm)) )
         end if
         if ( ( xstart.ge.xmin ) .and. ( xstart.le.xmax ) .and. &
              ( ystart.ge.ymin ) .and. ( ystart.le.ymax ) .and. &
              ( zstart.ge.zmin ) .and. ( zstart.le.zmax ) ) then

!        --- if so then determine local coordinates

            call elm651 ( coor, inpelm, array(1+(i-1)*inpelm), x, ndim )
            call elm660 ( x, Bmat, J, ndim )
            call elm659 ( Bmat, Binv, ndim )
            call to0075 ( inpelm, x, b )
            call to0076 ( Binv, xstart, ystart, zstart, b, xloc, iloc, &
                          ndim )
            if ( ndim.eq.2 ) xloc(3) = 0d0

!           --- if local coordinates are okay then set this element will do
!                  and ielem is set to the number of this element

            if ( ( xloc(1).ge.-eps )       .and. &
                 ( xloc(1).le.1d0-xloc(2)+eps ) .and. &
                 ( xloc(2).ge.-eps )       .and. &
                 ( xloc(2).le.1d0-xloc(1)+eps ) .and. &
                 ( xloc(3).ge.-eps )       .and. &
                 ( xloc(3).le.1d0-xloc(1)-xloc(2)+eps ) ) then
               ielem = i
            end if
         end if
         goto 10
      end if
      call erclos ( 'to0071' )
      end
