      subroutine to0075 ( inpelm, x, b )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-21 adjusted for NDIM = 3
!        version 1.0   date   90-11-14
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   Compute the translation vector of the mapping to a reference element
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none
      integer inpelm
      double precision x(*), b(*)

!
!     inpelm   i  integer, the number of nodal points per element
!
!     x        i  real array, containing the coordinates of all nodal points
!                 of one element
!
!     b        o  real vector, the translation vector of the mapping
!
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
!     none
!
! ********************************************************************
!
!                              METHOD
!     _   _   _   _                         _
!     b = x + x - x    for a triangle where x is the midth of edge
!          2   3   1                         i                    i
!
!     _   _   _   _     _                           _
!     b = x + x + x - 2 x   for a tetrahedron where x is the midth of side
!          2   3   4     1                           i                    i
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!     none
!
! ********************************************************************

      call eropen ( 'to0075' )

!                                                       _
!     if element is a six point triangle then calculate b

      if ( inpelm.eq.6 ) then
         b(1) = x(3) + x(5) - x(1)
         b(2) = x(4) + x(6) - x(2)
         b(3) = 0d0
!                                                               _
!     if element is a fourteen point tetrahedron then calculate b

      else if ( inpelm.eq.14 ) then
         b(1) = x(4) + x(7) + x(10) - 2 * x(1)
         b(2) = x(5) + x(8) + x(11) - 2 * x(2)
         b(3) = x(6) + x(9) + x(12) - 2 * x(3)
      end if

      call erclos ( 'to0075' )

      end
