      subroutine to0100 ( iskpel, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.4  date 17-07-2015 New call to to0103
!        version  3.3  date 15-11-2012 Long integers
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes derived quantities of an input vector, either in the nodal points
!     or in the elements
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iskpel(*), icurvs(*)

!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral.
!                         A negative curve number indicates that the curve
!                         must be computed in the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array iinder
!     iskpel         i    This array of length nelgrp is only used if
!                         allelements>=1 iskpel(ielgrp) = 0 means elements of
!                         element group ielgrp are skipped, iskpel(ielgrp) = 1,
!                         means that they are assembled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iskip, ivec

!     iskip          parameter allelements see iinder(12)
!     ivec           parameter ivec see iinder(7)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOMPDER     computes derived quantities of an input vector, either
!                    in the nodal points or in the elements
!     SEPCOMPDERINT  Compute integer quantities per element
!     SEPCOMPREACCUR Compute reaction force along curves
!     SEPCREASKIP    Fill array skiparr depending on iskip and array iskpel
!     SEPDERINTEGRAL Compute integrals over derivatives
!     SEPDERTYPE     Analyze information from backing storage (file 1), as
!                    stored in istfl1 for standard elements with standard
!                    element number > 99 for given icheld
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPNULLIFYELM  Nullify all pointers associated with E
!     SEPPRINTV      Print solution vector or vector of special structure
!     TO0088         Compute the actual output vector for DERIV when ICHOIS=4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ichderiv := iinder(2)
!   if ( ichderiv=4 ) then
!      Fill mid points
!   else
!    Perform some checks
!    Make initializations
!    Compute array lengths
!    Reserve space
!    Construct list_of_element_groups and per element group
!       list_of_elements allowed
!    For ielgrp member of list_of_element_groups
!       For ielem member of list_of_elements in ielgrp
!          Compute derivative in ielem
!          add derivatives and weigths to output vectors
!          compute line integral and add to final integral
!    if ( ichderiv<2 ) then
!       Divide output vector by weight vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0100' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0100'
         call prinin ( iinder, 16, 'iinder' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Activate and fill subarrays and parameters corresponding to
!         kmesh

      iseqi1 = iinder(10)
      iprob  = ks(isolut2d(iseqi1))%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kmeshloc => kmesh
      call sepfillkmesh ( kmesh )
      call sepfillkprob
      kprobc => kprobpart%kprobc
      kprobj => kprobpart%kprobj
      bufchanged = .false.
      call sepnullifyelm  ! nullify all pointers in E

!     --- Copy information in cactl

      npointcac = npoint
      nelemcac  = nelem
      inpelm = npelm
      maxunk = 0
      numns = numnat+nelgrp

!     --- Create array skipelgrps

      iskip  = iinder(12)
      allocate ( skipelgrps(nelgrp) )
      call sepcreaskip ( iskip, iskpel )

!     --- Fill parameters from iinder

      ichderiv = iinder(2)
      icheld = iinder(4)

      if ( debug ) write(irefwr,1) 'ichderiv/1', ichderiv

      if ( ichderiv>=0 .and. ichderiv/=4 .and. ichderiv<7 ) then

!     --- Detect parameters

         call sepdertype
         ichderiv = ichtst-1
         if ( debug ) write(irefwr,1) 'ichderiv/2', ichderiv

      end if  ! ( ichderiv>=0 .and. ichderiv/=4 .and. ichderiv<7 )

      select case ( ichderiv )

      case ( -1 )

!     --- ichderiv = -1, no output vector required, just integrals

         call sepderintegral ( icurvs )

      case ( 0:3, 5:6 )

!     --- ichderiv = 0 to 3 or 5 to 6, standard case

         call sepcompder

      case ( 4 )

!     --- ichderiv = 4, special situation (fill mid points)

         call to0088

      case ( 7 )

!     --- ichderiv =  7, special situation, reaction forces over curves

         call sepcompreaccur ( icurvs )

      case ( 8 )

!     --- ichderiv = 8, compute integer vector defined per element

         call sepcompderint

      end select  ! case ( ichderiv )

      deallocate ( skipelgrps, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'skipelgrps' )

1000  if ( debug ) then
         if ( ichderiv/=8 ) call sepprintv ( SMF%ioutvc, 'outvec' )
         write(irefwr,*) 'End to0100'
      end if  ! ( debug )
      call erclos ( 'to0100' )

      end subroutine to0100
