      subroutine to0100reac1 ( clear, iinder, icurvs, &
                               skiparr, nodes, nnodes, listelms )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 10-10-2012 Extra parameter kprob
!        version  1.0  date 11-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store nodes in which reaction force has to be computed in nodes
!     Make a list active elements
! **********************************************************************
!
!                       KEYWORDS
!
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinder(*), icurvs(*), skiparr(nelgrp)
      integer, intent(out) :: nodes(*), nnodes, listelms(nelem)
      logical, intent(in) :: clear

!     clear          i    If true the output vector must be cleared before
!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral. A negative curve
!                         number indicates that the curve must be computed in
!                         the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array iinder
!     iinder         i    Array to pass information to subroutines of deriv
!                         For a description see subroutine TO0100
!     listelms       i    Array of length nelem, containing information about the
!                         elements
!                         if listelms(i) = 0, the element must be skipped,
!                         if listelms(i) = 1, the element must be taken into account
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be used
!     skiparr        i    if skiparr(i)=0, element group i must be skipped
!                          otherwise not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nparts, inumcv
      integer, allocatable :: ipoints(:)

!                    they are not
!     inumcv         Number of curves to be used
!     ipoints        Work array of length npoint containing a set of nodes
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
!     TO0114         Make list of element groups and elements for all nodal
!                    points in iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0100reac1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0100reac1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( ipoints(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'ipoints' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array nodes

      inumcv = iinder(15)   ! number of user points or curves in boundary
      nnodes = inumcv
      call sepfillnodes ( iinder(16), inumcv, icurvs, &
                          nodes, nnodes, ipoints, nparts )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'nnodes', nnodes
         call prinin ( nodes, min(100,nnodes), 'nodes')
      end if  ! ( debug )

      call to0114 ( nodes, listelms, skiparr, nnodes )

      deallocate ( ipoints, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         call prininnonzero ( listelms, nelem, 'list' )
         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End to0100reac1'

      end if  ! ( debug )
      call erclos ( 'to0100reac1' )

      end
