      subroutine topmaptrian2 ( kmeshcnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map 2d array kmeshc onto new array kmesh consisting of
!     linear triangles only
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*)

!     kmeshcnew      o    New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, istart, ielgrp, inelemnew, inpelmnew, istartnew

!     ielgrp         element group number
!     inelemnew      inelem for new mesh
!     inpelm         number of points per element
!     inpelmnew      number of points per element for new mesh
!     istart         First position used at a part
!     istartnew      starting address in new array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOPMAPTRIAN3   Copy elements of shape 3 or 18 into new elements of
!                    shape 3
!     TOPMAPTRIAN4   Copy elements of shape 4, 15 or 16 into new elements of
!                    shape 3
!     TOPMAPTRIAN5   Copy elements of shape 5 into new elements of shape 3
!     TOPMAPTRIAN6   Copy elements of shape 6 into new elements of shape 3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topmaptrian2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topmaptrian2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill new array kmeshc

      istart = 1             ! starting address in old kmeshc
      istartnew = 1          ! starting address in new kmeshc
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         if ( ishape<3 ) go to 100

         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem, inpelm, istart', &
                             ielgrp, ishape, inelem, inpelm, istart

         select case ( ishape )

            case(3, 10)

!           --- Linear triangle

               inelemnew = inelem
               inpelmnew = 3
               call topmaptrian3 ( kmeshc(istart), &
                                   kmeshcnew(istartnew), inpelm, &
                                   inelem, inpelmnew )

            case(4, 7)

!           --- Quadratic triangle

               inelemnew = 4*inelem
               inpelmnew = 3
               call topmaptrian4 ( kmeshc(istart), &
                                   kmeshcnew(istartnew), inpelm, &
                                   inelem )

            case(5, 9)

!           --- Bilinear quadrilateral

               inelemnew = 2*inelem
               inpelmnew = 3
               call topmaptrian5 ( kmeshc(istart), &
                                   kmeshcnew(istartnew), inpelm, &
                                   inelem )

            case(6)

!           --- Quadratic quadrilateral

               inelemnew = 8*inelem
               inpelmnew = 3
               call topmaptrian6 ( kmeshc(istart), &
                                   kmeshcnew(istartnew), inpelm, &
                                   inelem, ishape )

            case default

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

         end select  ! case ( ishape )
         istartnew = istartnew+inelemnew*inpelmnew
100      istart = istart+inelem*inpelm    ! raise starting address

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topmaptrian2'

      end if  ! ( debug )
      call erclos ( 'topmaptrian2' )

      end
