      subroutine er0028 ( icurvs, n, number, ndim, coor, vector, x, y, &
                          kprobf, indprf, nunkp, factor, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 19-11-1999 Replace number bu numb
!        version  2.2  date 23-11-1997 New layout
!        version  2.0  date 25-04-1991 Extension with icompl
!        version  1.0  date 09-05-1985
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays x and y with co-ordinates (arc length s) and function
!     values along a line, where the nodal point numbers along the
!     line are given
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer icurvs(*), kprobf(*), n, number, ndim, indprf, &
              nunkp
      double precision coor(*), vector(*), x(*), y(*), factor

!     compl      i    indication whether the arithmetic is real (false)
!                     or complex (true)
!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     factor     i    the function values are multiplied by factor
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     indprf     i    indication if kprob part f has been filled (>0) or not (0)
!     kprobf     i    KPROB part f: contains the number of unknowns per point
!                     accumulated
!     n         i/o   at input: number of nodal points in icurvs
!                     at output: number of points in x and y
!     ndim       i    Dimension of the space.
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered. If the array is complex number=2*i-1
!                     corresponds to the real part of array i and number=2*i
!                     to the imaginary part of array i
!     nunkp      i    Maximum number of unknowns in a point
!     vector     i    array containing the vector from which the function y
!                     must be computed
!     x          o    array containing the x-co-ordinates of the function to
!                     be plotted (arc length)
!     y          o    array containing the function values to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, numb, iextra, m, i, inodp, j, inusol
      double precision s, sum, x1(3), x2(3)

!     i          Counting variable
!     iextra     Extra space needed at both sides of the array for control
!                integers
!     inodp      Nodal point number
!     inunkp     Number of degrees of freedom in first point
!     inusol     Position of unknown in array USOL
!     j          Counting variable
!     m          number of points with the specified degree of freedom
!     numb       numb gives the number of the unknown to be taken in each
!                nodal point. numb is completely dependent on number
!                If number = 0 then numb = 1 else numb = number
!     s          Arc length
!     sum        Help parameter to compute a sum
!     x1         coordinates of first point
!     x2         coordinates of last point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inunkp=nunkp
      if ( compl ) then
         numb=(number+1)/2
         iextra = mod(number+1,2) - 1
      else
         numb=number
      end if
      m=0
      do i=1,n
         inodp=icurvs(i)
         do j=1,ndim
            x2(j)=coor(ndim*(inodp-1)+j)
         end do
         if(i.eq.1) then
            s=0d0
         else
            sum=0d0
            do j=1,ndim
               sum=sum+(x2(j)-x1(j))**2
         end do
            s=s+sqrt(sum)
         end if
         if(indprf.eq.0) then
            inusol=(inodp-1)*nunkp+numb
         else
            inusol=kprobf(inodp)
            inunkp=kprobf(inodp+1)-inusol
            inusol=inusol+numb
         end if
         if(inunkp.gt.0) then
            m=m+1
            x(m)=s
            if ( compl ) inusol = 2 * inusol + iextra
            y(m)=vector(inusol)*factor
         end if
         do j=1,ndim
            x1(j)=x2(j)
         end do
      end do
      n=m
      end
