      subroutine ini023 ( vector, number, amin, amax, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 31-12-1995 Complete revision, new parameter list
!        version  2.0  date 26-03-1992 Ad-hoc extension for complex
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1986-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute minimum and maximum value of sepran vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     maximum
!     minimum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number
      double precision vector(*), amin, amax
      logical compl

!     amax       o    Maximum value in solution array
!     amin       o    Minimum value in solution array
!     compl      i    indication whether the arithmetic is real (false)
!                     or complex (true)
!     number     i    The number-th unknown in each nodal point of the array
!                     is considered. If the array is complex number=2*i-1
!                     corresponds to the real part of array i and number=2*i
!                     to the imaginary part of array i
!     vector     i    sepran vector from which minimum and maximum value
!                     must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision u
      integer i, inunkp, iextra, iusol, numb

!     i          Counting variable
!     iextra     Extra position needed because of complex values
!     inunkp     Number of degrees of freedom in first point
!     iusol      Position in solution vector
!     numb       numb gives the number of the unknown to be taken in each
!                nodal point. numb is completely dependent on number
!                If number = 0 then numb = 1 else numb = number
!     u          solution in specific point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      amin=rinfin
      amax=-rinfin
      if ( compl ) then
         numb = (number+1)/2
         iextra = 1-mod(number,2)
      else
         numb = number
      end if
      do i = 1, npoint
         if ( indprpi>0 ) then
            iusol = kprobpi(i,numb)
         else if ( indprfi==0 ) then
            iusol = (i-1)*nphysi+numb
         else
            iusol = kprobfi(i)
            inunkp = kprobfi(i+1)-iusol
            if ( inunkp<numb ) then
               iusol = 0
            else
               iusol=iusol+numb
            end if
         end if
         if ( iusol>0 ) then
            if ( compl ) iusol=2*iusol-1+iextra
            u=vector(iusol)
            amin=min(amin,u)
            amax=max(amax,u)
         end if
      end do
      end
