      subroutine inits5 ( iuser, ncontr, jmark, imark, &
                          part, plboun, ichois, plrenm, fillcl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 15-11-2013 New definition ichois
!        version  4.1  date 14-11-1994 Extra possibility jmark
!        version  4.0  date 22-12-1991 Extension with fillcl
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   initialize some parameters for subroutine plotm2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical ncontr, plboun, part, plrenm, fillcl
      integer iuser(*), jmark, imark, ichois

!     fillcl  i    Indicates if the elements must be filled with a colour (true)
!                  or that only the boundaries must be plotted (false)
!                  If fillcl = true, the colour is given by dcolor, the
!                  boundaries get the default colour. Otherwise the colour of
!                  the boundaries is defined by dcolor
!     ichois  i    Parameter to distinguish between the following
!                  possibilities, see plotm2bf
!     imark   o    jmark modulo 10
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See PLOTM2
!     jmark   i    Indicates information about the plotting
!                  0  plot nodes and elements provided with numbers and marks
!                  1  nodal points are marked but no number is plotted
!                     elements are provided with a number
!                  2  nodal points are not marked nor numbers are plotted
!                     elements are provided with a number
!                  3  plot nodes provided with numbers and marks
!                     elements are not provided with a number
!                  4  nodal points are marked but no number is plotted
!                     elements are not provided with a number
!                  5  nodal points are not marked nor numbers are plotted
!                     elements are not provided with a number
!                  6  nodal points are not marked but numbers are plotted
!                     elements are provided with a number
!                  7  nodal points are not marked but numbers are plotted
!                     elements are not provided with a number
!                 >=10 <=15  See jmark-10, however renumbered nodes are plotted
!     ncontr  o    indication whether the elements are contracted (false)
!                  or not (true)
!     part    o    indication whether all elements are plotted (true)
!                  or that some are skipped (false)
!     plboun  o    indication whether the boundaries must be plotted with
!                  double lines (true) or not (false)
!     plrenm  o    Indicates if the renumbered nodes must be plotted (true)
!                  or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jchois, niuser
      double precision dummy

!     dummy   Dummy parameter
!     jchois  ?
!     niuser  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      15   jmark has in correct value
!      62   dimension of space (ndim) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inits5' )
      ncontr = .true.
      part = .true.
      jchois = mod(ichois,100)

      if ( jchois>=10 ) then
         plboun = .true.
         jchois = jchois-10
      else
         plboun = .false.
      end if
      if ( jchois>=5 ) then
         fillcl = .true.
         jchois = jchois-5
      else
         fillcl = .false.
      end if
      if ( jmark/10==1 ) then

!     --- jmark>10, i.e. plot renumbered nodal point numbers

         imark = jmark - 10
         if ( kelmj>0 ) then
            plrenm = .true.
         else
            plrenm = .false.
         end if
      else

!     --- Jmark<10, standard numbering

         plrenm = .false.
         imark = jmark

      end if
      if ( jchois==2 .or. jchois==3 ) ncontr = .false.
      if ( jchois==1 .or. jchois==3 ) part = .false.

!     --- tests

      if ( imark<0 .or. imark>7 ) then
         call errint ( jmark, 1 )
         call errint ( 5, 2 )
         call errsub ( 15, 2, 0, 0)
      end if
      if ( jchois<0 .or. jchois>5 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0)
      end if
      if ( .not. part ) then

!     --- not all element groups must be plotted

         niuser = nelgrp+5

!        --- jchois = 1 or 3 check length iuser

         call prinlg ( 'iuser', niuser, dummy, iuser)

      end if
      if ( ndim>2 ) then
         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0)
      end if
      call erclos ( 'inits5' )
      end
