      subroutine pl0001 ( bcord, mbcrd, nbndpt, xmin, xmax, ymin, ymax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 08-03-2010 Debug statements
!        version  1.3  date 21-05-2001 Extra parameter mbcrd
!        version  1.2  date 22-08-1996 Layout
!
!   copyright (c) 1983-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute xmin, xmax, ymin, ymax
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt, mbcrd
      double precision bcord(mbcrd,*), xmin, xmax, ymin, ymax

!     bcord      i    array containing the coordinates of the boundaries
!     mbcrd      i    First dimesnion of array bcord if considered as
!                     two-dimesional array
!     nbndpt     i    Number of nodal points in boundary
!     xmax       o    Maximum x-value
!     xmin       o    Minimum x-value
!     ymax       o    Maximum y-value
!     ymin       o    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x, y

!     i              Counting variable
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0001' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pl0001'
         write(irefwr,1) 'mbcrd, nbndpt', mbcrd, nbndpt
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xmin=rinfin
      ymin=xmin
      xmax=-xmin
      ymax=xmax
      do i = 1, nbndpt
         x=bcord(1,i)
         y=bcord(2,i)
         if(x<xmin) xmin=x
         if(y<ymin) ymin=y
         if(x>xmax) xmax=x
         if(y>ymax) ymax=y
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pl0001'

      end if  ! ( debug )
      call erclos ( 'pl0001' )

      end
