      subroutine pl0008 ( str, npnt, contln, ncntln, coor, indcol, &
                          ndim, imesh, iplotc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.7  date 25-04-2010 New call to plaf05
!        version  5.6  date 31-03-2009 Correct top level
!        version  5.5  date 20-02-2002 Correct number of levels
!        version  5.4  date 05-04-2001 Correct number of levels
!
!   copyright (c) 1982-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contour lines for one triangle
!     fill all contour levels in the triangle with colors
! **********************************************************************
!
!                       KEYWORDS
!
!     color
!     contour_level
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npnt(3), indcol(*), ndim, ncntln, imesh, iplotc
      double precision coor(ndim,*), str(3), contln(*)

!      contln    i   array containing the contour levels
!      coor      i   array containing the co-ordinates of the nodal points
!                    The x-coordinate of point i is stored in coor(1,i)
!                    The y-coordinate of point i is stored in coor(2,i)
!                    If ndim=3, coor(3,i) is not used
!      imesh     i   Color of the triangle boundaries, 0 = none
!      indcol    i   array containing the color numbers of the various
!                    contour levels
!      iplotc    i   Color of the contour lines, 0 = none
!      ncntln    i   number of contour levels
!      ndim      i   First parameter in the definition of coor
!      npnt          nodal point numbers of the three nodal points
!      str      i/o  array containing the function values in the triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(3), x2(3), cj, polyg(2,6), work(2,2,ncntln)
      integer iwork(3), iperm(3), i, npolyg, icolor, lcont, icount, &
              icolsv

!      cj            Contour value
!      i             Counting variable
!      icolor        Color sequence number of contour
!      icolsv        Help variable to store the value of dcolor temporarily
!      icount        Number of contour lines found in the triangle
!      iperm         Array containing the permutation produced by the sorting
!      iwork         Work array for the permutation
!      lcont         Sequence number of contour level
!      npolyg        number of polygons
!      polyg         work array of length 20
!                    The first 10 positions are used to store the polygon
!      work          Work array of length 4*ncntln. Is only used if iplotc>0
!                    In this array the initial and end points of all contour
!                    lines within the triangle stored as follows:
!                    work(1,1,i): x-co-ordinate first point of line i
!                    work(1,2,i): y-co-ordinate first point of line i
!                    work(2,1,i): x-co-ordinate last point of line i
!                    work(2,2,i): y-co-ordinate last point of line i
!      x1            x-co-ordinates of the three nodes
!      x2            y-co-ordinates of the three nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLINTP         Linear interpolation between two points on a line
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      sort the function values for increasing value
!      store the coordinates of the triangle in the same sequence
!      icount := 0
!      if ( no contour lines in triangle ) then
!         last_triangle := triangle
!         detect color
!      else if ( function nearly constant ) then
!         last_triangle := triangle
!         detect color
!      else
!         ???  must be filled
!      Fill last triangle
!      If ( imesh>0 ) then
!         Plot triangle
!      If ( iplotc>0 ) then
!         Plot icount contour lines
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0008' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pl0008'
         call prinin ( npnt, 3, 'npnt' )
         call prinrl ( str, 3, 'str' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
!     --- the function values are sorted for increasing value
!         the corresponding nodal point numbers are stored in npnt

      call prsort ( str, iperm, 3 )
      do i = 1, 3
         x1(i) = str(iperm(i))
         iwork(i) = npnt(iperm(i))
      end do
      do i = 1, 3
         str(i) = x1(i)
         npnt(i) = iwork(i)
      end do

!     --- determination of co-ordinates
!         the x- and y-co-ordinates are stored in the arrays x1 and x2
!         in the same sequence as the function values in str

      do i = 1, 3
         x1(i) = coor(1,npnt(i))
         x2(i) = coor(2,npnt(i))
      end do
      icount = 0

!     --- check if the triangle has only values smaller than contln(1)
!         or larger than contln(ncntln)
!         If so, the triangle is filled with one color

      if ( str(3)<=contln(1) .or. str(1)>=contln(ncntln) ) then

!     --- Only one color

         do i=1,3
            polyg(1,i)=x1(i)
            polyg(2,i)=x2(i)
         end do
         npolyg=3
         if(str(3)<=contln(1)) then
            icolor=indcol(1)
         else
            icolor=indcol(ncntln)
         end if
         goto 500

      else if (abs(str(3)-str(1))<=sqreps*(abs(str(3)+abs(str(1))))) &
           then

!     --- the function is nearly constant in the triangle, &
!         therefore only one color is plotted

         do i=1,3
            polyg(1,i)=x1(i)
            polyg(2,i)=x2(i)
         end do
         npolyg=3

!        --- find color by finding smallest value of contour larger than str(1)

         do i=1,ncntln
            lcont=i
            if(contln(i)>str(1)) goto 250
         end do
         lcont=ncntln
250      icolor=indcol(lcont)
         goto 500
      end if

!     --- find smallest contour (lcont) larger than str(1)

      do i=1,ncntln
         lcont=i
         if(contln(i)>str(1)) goto 270
      end do
      lcont=ncntln

!     --- next construct one-colored polygons in the triangle

270   cj=contln(lcont)
      if(str(3)<=cj) then

!     --- The contour level cj is larger than str(3), hence the triangle
!         is filled with one color

         do i=1,3
            polyg(1,i)=x1(i)
            polyg(2,i)=x2(i)
         end do
         npolyg=3
         icolor=indcol(lcont)
         goto 500
      end if

!     --- cj<str(3) hence there is at least one contour line in the triangle

      polyg(1,1)=x1(1)
      polyg(2,1)=x2(1)
      if(cj<=str(2)) then

!     --- contour level<str(2), hence the contour crosses the line
!          (x1,y1)-(x2,y2) in (x4,y4) and the line (x1,y1)-(x3,y3) in (x5,y5)
!          The triangle (x1,y1), (x4,y4), (x5,y5) gets the color indcol(lcont)

         call plintp(x1(1),x2(1),x1(2),x2(2),polyg(1,2),polyg(2,2), &
              str(1),str(2),cj)
         call plintp(x1(1),x2(1),x1(3),x2(3),polyg(1,3),polyg(2,3), &
              str(1),str(3),cj)
         if ( iplotc>0 ) then
            icount = icount + 1
            work(1,1,icount) = polyg(1,2)
            work(1,2,icount) = polyg(2,2)
            work(2,1,icount) = polyg(1,3)
            work(2,2,icount) = polyg(2,3)
         end if
         npolyg=3
         icolor=indcol(lcont)
         call plaf05(1,icolor,npolyg,polyg,2)

!        --- store (x5,y5)-(x4,y4) in polyg(1-4) in that sequence

         polyg(1,1)=polyg(1,3)
         polyg(2,1)=polyg(2,3)
         lcont=lcont+1
         goto 300
      else

!     --- contour level>str(2) and contour level<str(3), &
!         hence the contour crosses the line (x2,y2)-(x3,y3) in (x4,y4)
!         and the line (x1,y1)-(x3,y3) in (x5,y5)
!         The quadrilateral (x1,y1), (x2,y2), (x4,y4), (x5,y5)
!         gets the color indcol(lcont)

         call plintp(x1(2),x2(2),x1(3),x2(3),polyg(1,3),polyg(2,3), &
              str(2),str(3),cj)
         call plintp(x1(1),x2(1),x1(3),x2(3),polyg(1,4),polyg(2,4), &
              str(1),str(3),cj)
         if ( iplotc>0 ) then
            icount = icount + 1
            work(1,1,icount) = polyg(1,3)
            work(1,2,icount) = polyg(2,3)
            work(2,1,icount) = polyg(1,4)
            work(2,2,icount) = polyg(2,4)
         end if
         npolyg=4
         icolor=indcol(lcont)
         polyg(1,2)=x1(2)
         polyg(2,2)=x2(2)
         call plaf05(1,icolor,npolyg,polyg,2)

!        --- store (x5,y5)-(x4,y4) in polyg(1-4) in that sequence

         do i=1,2
            polyg(i,1)=polyg(i,4)
            polyg(i,2)=polyg(i,3)
         end do
         lcont=lcont+1
         goto 400
      end if

!     --- situation that already some part of the triangle has been colored, &
!         but that the preceding contour line is below str(2)
!         Hence the remaining part is a quadrilateral (x5,y5), (x4,y4), &
!         (x2,y2), (x3,y3)

300   if ( lcont>ncntln .or. str(3)<=contln(min(lcont,ncntln)) ) &
         then

!     --- largest contour level has been reached
!         or str(3)<=contln(lcont)
!         The quadrilateral gets one color

         npolyg=4
         lcont = min(lcont,ncntln)
         icolor=indcol(lcont)
         do i=1,2
            polyg(1,2+i)=x1(i+1)
            polyg(2,2+i)=x2(i+1)
         end do
         goto 500

      else

!     --- The last contour level has not been reached
!         and str(3)>contln(lcont)

         cj=contln(lcont)
         if(cj<=str(2)) then

!        --- contour level<str(2), hence the contour crosses the line
!            (x1,y1)-(x2,y2) in (x6,y6) and the line (x1,y1)-(x3,y3) in (x7,y7)
!            The quadrilateral (x5,y5), (x4,y4), (x6,y6), (x7,y7)
!            gets the color indcol(lcont)

            call plintp(x1(1),x2(1),x1(2),x2(2),polyg(1,3),polyg(2,3), &
                 str(1),str(2),cj)
            call plintp(x1(1),x2(1),x1(3),x2(3),polyg(1,4),polyg(2,4), &
                 str(1),str(3),cj)
            if ( iplotc>0 ) then
               icount = icount + 1
               work(1,1,icount) = polyg(1,3)
               work(1,2,icount) = polyg(2,3)
               work(2,1,icount) = polyg(1,4)
               work(2,2,icount) = polyg(2,4)
            end if
            npolyg=4
            icolor=indcol(lcont)
            call plaf05(1,icolor,npolyg,polyg,2)

!           --- store (x7,y7)-(x6,y6) in polyg(1-4) in that sequence

            do i=1,2
               polyg(i,1)=polyg(i,4)
               polyg(i,2)=polyg(i,3)
            end do
            lcont=lcont+1
            goto 300
         else

!        --- contour level>str(2) and contour level<str(3), &
!             hence the contour crosses the line (x2,y2)-(x3,y3) in (x6,y6)
!             and the line (x1,y1)-(x3,y3) in (x7,y7)
!             The 5-sided polygon (x5,y5), (x4,y4), (x2,y2), (x6,y6), (x7,y7)
!             gets the color indcol(lcont)

            polyg(1,3)=x1(2)
            polyg(2,3)=x2(2)
            call plintp(x1(2),x2(2),x1(3),x2(3),polyg(1,4),polyg(2,4), &
                 str(2),str(3),cj)
            call plintp(x1(1),x2(1),x1(3),x2(3),polyg(1,5),polyg(2,5), &
                 str(1),str(3),cj)
            if ( iplotc>0 ) then
               icount = icount + 1
               work(1,1,icount) = polyg(1,4)
               work(1,2,icount) = polyg(2,4)
               work(2,1,icount) = polyg(1,5)
               work(2,2,icount) = polyg(2,5)
            end if
            npolyg=5
            icolor=indcol(lcont)
            call plaf05(1,icolor,npolyg,polyg,2)

!           --- store (x7,y7)-(x6,y6) in polyg(1-4) in that sequence

            do i=1,2
               polyg(i,1)=polyg(i,5)
               polyg(i,2)=polyg(i,4)
            end do
            lcont=lcont+1
            goto 400
         end if
      end if

!     --- situation that already some part of the triangle has been colored, &
!         including the point (x2,y2)
!         Hence the remaining part is the triangle (x5,y5), (x4,y4), (x3,y3)

400   if ( lcont>ncntln .or. str(3)<=contln(min(lcont,ncntln)) ) &
         then

!     --- largest contour level has been reached
!         or str(3)<=contln(lcont)
!         The triangle gets one color

         npolyg=3
         lcont = min(lcont,ncntln)
         icolor=indcol(lcont)
         polyg(1,3)=x1(3)
         polyg(2,3)=x2(3)
         goto 500

      else

!     --- The last contour level has not been reached
!         and str(3)>contln(lcont)
!         The contour crosses the line (x2,y2)-(x3,y3) in (x6,y6)
!         and the line (x1,y1)-(x3,y3) in (x7,y7)
!         The quadrilateral (x5,y5), (x4,y4), (x6,y6), (x7,y7)
!         gets the color indcol(lcont)

         cj=contln(lcont)
         call plintp(x1(2),x2(2),x1(3),x2(3),polyg(1,3),polyg(2,3), &
              str(2),str(3),cj)
         call plintp(x1(1),x2(1),x1(3),x2(3),polyg(1,4),polyg(2,4), &
              str(1),str(3),cj)
         if ( iplotc>0 ) then
            icount = icount + 1
            work(1,1,icount) = polyg(1,3)
            work(1,2,icount) = polyg(2,3)
            work(2,1,icount) = polyg(1,4)
            work(2,2,icount) = polyg(2,4)
         end if
         npolyg=4
         icolor=indcol(lcont)
         call plaf05(1,icolor,npolyg,polyg,2)

!        --- store (x7,y7)-(x6,y6) in polyg(1-4) in that sequence

         do i=1,2
            polyg(i,1)=polyg(i,4)
            polyg(i,2)=polyg(i,3)
         end do
         lcont=lcont+1
         goto 400
      end if

!     --- Fill last part of triangle with color

500   call plaf05(1,icolor,npolyg,polyg,2)

!     --- Color triangle with color imesh

      if ( imesh>0 ) then
         icolsv = dcolor
         dcolor = imesh
         call plafep ( 4, x1(1), x2(1), 0d0, 0d0, 0 )
         call plafep ( 5, x1(2), x2(2), 0d0, 0d0, 0 )
         call plafep ( 5, x1(3), x2(3), 0d0, 0d0, 0 )
         call plafep ( 5, x1(1), x2(1), 0d0, 0d0, 0 )
         dcolor = icolsv
      end if

!     --- Plot contour lines with color iplotc

      if ( iplotc>0 ) then
         icolsv = dcolor
         dcolor = iplotc
         do i = 1, icount
            call plafep ( 4, work(1,1,icount), work(1,2,icount), 0d0, &
                          0d0, 0 )
            call plafep ( 5, work(2,1,icount), work(2,2,icount), 0d0, &
                          0d0, 0 )
         end do
         dcolor = icolsv
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pl0008'

      end if  ! ( debug )
      call erclos ( 'pl0008' )

      end subroutine pl0008
