      subroutine plaf051 ( npoint, npolyg, work, polyg, ndimpol, iside, &
                           jtest, ioutsd, icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array work with plot coordinates of polygon
!     Coordinates are given in user coordinates
!     Clipping is active
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     polygon
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, npolyg, ndimpol, iside, jtest, ioutsd, icount
      real work(2,2*npolyg)
      double precision polyg(ndimpol,npolyg)

!     icount        i/o   Number of points in work
!     ioutsd        i/o   See common cplaf
!     iside          i    Indicates at which side the cross section
!                         is positioned
!                         Possible values:
!                         0: There is no cross section point computed in this
!                            call
!                         1: The cross section point is at side y=ymint
!                         2: The cross section point is at side x=xmaxt
!                         3: The cross section point is at side y=ymaxt
!                         4: The cross section point is at side x=xmint
!     jtest         i/o   See common cplaf
!     ndimpol        i    Declared first parameter of polyg (>= 2)
!     npoint         i    Number of points in polygon (if closed)
!     npolyg         i    Number of points in polygon
!     polyg          i    Array containing the co-ordinates of the point in the
!                         polygon in the following way:
!                         polyg(1,i) contains the x-coordinate of the first
!                         point
!                         polyg(2,i) contains the y-coordinate of the first
!                         point
!     work           o    real single precision work aray of length 4*npolyg to
!                         store the co-ordinates of the points of the polygon
!                         in plot co-ordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, isidpr, isidpf
      double precision xp, yp, xpboun, ypboun

!     i              Counting variable
!     isidpf         Previous value of isidpr at a cross section point
!     isidpr         Previous value of iside at a cross section point
!     j              Counting variable
!     xp             x-coordinate in plotting units
!     xpboun         x-coordinate of point at the boundary of the plotting
!                    region
!     yp             y-coordinate in plotting units
!     ypboun         y-coordinate of point at the boundary of the
!                    plotting region
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF01         Translate user coordinates into plotter coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plaf051' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plaf051'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isidpr = 0
      isidpf = 0

      do i = 1, npoint
         if ( i<=npolyg ) then

!        --- i<=npolyg,  internal point

            j = i

         else

!        --- First point

            j = 1

         end if
         call plaf01 ( polyg(1,j), polyg(2,j), xp, yp, xpboun, ypboun, &
                       iside )
         if ( jtest==0 .or. ioutsd==0 ) then

!        --- jtest = 0 or ioutsd=0 both new and old point within rectangle
!            Fill work, raise icount

            icount = icount + 1
            work(1,icount) = sngl(xp)
            work(2,icount) = sngl(yp)

         else if ( ioutsd==1 ) then

!        --- ioutsd=1 new point within rectangle, old one outside rectangle
!            Fill work with boundary point and new point, raise icount

            if ( icount>0 ) then

!           --- There is a preceding point

               if ( iside/=isidpr ) then

!              --- New boundary point and preceding one on different sides
!                  Check if the side numbers differ not more than one

                  if ( mod(abs(iside-isidpr),2)==1 ) then

!                 --- Either the difference is 1 or 3

                     icount = icount + 1
                     if ( mod(iside,2)==0 ) then
                        work(1,icount) = sngl(xpboun)
                     else
                        work(2,icount) = sngl(ypboun)
                     end if
                     if ( mod(isidpr,2)==0 ) then
                        work(1,icount) = work(1,icount-1)
                     else
                        work(2,icount) = work(2,icount-1)
                     end if
                  end if
               end if
            end if
            icount = icount + 1
            work(1,icount) = sngl(xpboun)
            work(2,icount) = sngl(ypboun)
            isidpf = isidpr
            isidpr = iside
            icount = icount + 1
            work(1,icount) = sngl(xp)
            work(2,icount) = sngl(yp)

         else if ( ioutsd==2 .and. i>1 ) then

!        --- ioutsd=2 new point outside rectangle, old one inside rectangle
!            Fill work with boundary point, raise icount

            if ( j==1 ) then

!           --- Last point

               if ( iside/=isidpr ) then

!              --- New boundary point and preceding one on different sides
!                  Check if the side numbers differ not more than one

                  if ( mod(abs(iside-isidpr),2)==1 ) then

!                 --- Either the difference is 1 or 3

                     icount = icount + 1
                     if ( mod(iside,2)==0 ) then
                        work(1,icount) = sngl(xpboun)
                     else
                        work(2,icount) = sngl(ypboun)
                     end if
                     if ( mod(isidpr,2)==0 ) then
                        work(1,icount) = work(1,icount-1)
                     else
                        work(2,icount) = work(2,icount-1)
                     end if
                  end if
               end if
            end if
            icount = icount + 1
            work(1,icount) = sngl(xp)
            work(2,icount) = sngl(yp)
            isidpr = iside

         else if ( icount>1 .and. ioutsd==3 .and. j==1 ) then

!        --- ioutsd=3 and last point
!            In this situation the previous and the last point are outside
!            the region but there are points inside
!            Fill work with boundary point, raise icount

            if ( isidpr/=isidpf ) then

!           --- Last boundary point and preceding one on different sides
!               Check if the side numbers differ not more than one

               if ( mod(abs(isidpr-isidpf),2)==1 ) then

!              --- Either the difference is 1 or 3

                  icount = icount + 1
                  if ( mod(isidpf,2)==0 ) then
                     work(1,icount) = work(1,1)
                  else
                     work(2,icount) = work(2,1)
                  end if
                  if ( mod(isidpr,2)==0 ) then
                     work(1,icount) = work(1,icount-1)
                  else
                     work(2,icount) = work(2,icount-1)
                  end if
               end if
            end if
            icount = icount + 1
            work(1,icount) = work(1,1)
            work(2,icount) = work(2,1)

         end if

      end do

1000  call erclos ( 'plaf051' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plaf051'

      end if  ! ( debug )

      end
