      subroutine plafp3 ( ichois, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version 4.7   date   06-02-2004 Remove superfluous computers
!        version 4.6   date   20-04-1999 Inlcude commons
!        version 4.5   date   16-11-1994 no colors for postscript B/W
!
!
!   copyright (c) 1985-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                         DESCRIPTION
!
!    subroutine to plot symbol with center (x,y)
!    move with pen up or down to that point, depending on ichois
!
!    machine dependent subroutine
!    starbase/ hp-gl/ tektronix/ plot10/ apollo/ calcomp/ gks / cgi
!    PC with VGA or EGA
!
! **********************************************************************
!
!                      KEYWORDS
!
!    machine_dependent
!    plot
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleplot
      implicit none
      integer ichois
      double precision x, y

!    ichois   i   the pen is moved to x,y with pen down when
!                 1<=ichois<=10, and with pen up when 11<=ichois<=20
!                 the symbol to be printed is indicated by ichois
!                 (ichois<=10) or ichois-10 (ichois>10)
!    x        i   points to which the pen must be moved in cm.
!                 the symbol is plotted with center (x,y)
!    y        i   See x
! **********************************************************************
!
!                      COMMON BLOCKS
!

! **********************************************************************
!
!                    LOCAL PARAMETERS

      integer icode, isymb, imode, iref, irefgl
      real sx, sy, angle

!    angle        angle to be used for markers, depending on direction
!    icode        variable to indicate pen up/down for drawing
!    imode        first parameter for unformatted file
!    iref         Reference number for SEPRAN neutral plotting file
!    irefgl       Reference number for HPGL file
!    isymb        choice of marker type to be used
!    sx           new x coordinate, after addition of new origin
!    sy           new y coordinate, after addition of new origin
!
! ********************************************************************
!
!             SUBROUTINES CALLED
!   plafp6  Routine to go to coordinate
!   plcol0  Set default color for lines and markers
!   plstb3  Handle plotting through StarBase
!   plpos3  Make plot in PostScript, both B/W and color
!   plown3  "own" plotting routine, to be plugged in
!   pltek0, pltek2, pltek3  Tektronix 4010 support routines
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
!       device not supported
! **********************************************************************
!                 Machine-dependent actions
!
!    In order to get the correct package, the user should remove the
!    comments before the calls of the packages
!
!    The following specific comments are used
!
!    chps   HP-Unix   StarBase version
!    chgl   HP-GL plotter version
!    ctek   Tektronix 4010 version (direct)
!    cpl1   Plot-10 version (through library)
!    capo   Apollo version
!    calc   Calcomp version (with slight modifications for some versions)
!    cgks   GKS version
!    cvga   PC, running FTN77/386 with VGA or EGA card
!    ccgi   PC, running UNIX with CGI package
!    cpos   PostScript output
!    cown   Local version not supported by SEPRAN
!
! **********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
!
      if (idirec.eq.1) then
         angle=0.
      else
         angle=-90.
      end if
      if (ichois.le.10) then
!       *  pen down
         isymb=ichois
         icode=-2
      else
!       *  pen up
         isymb=ichois-10
         icode=-1
      end if
      iref = abs(irefpl)
      irefgl = iref+1

!    *  Move to point by subroutine PLAFP6 for some packages:

      if (posdev(1:1).eq.'a' .or. posdev(1:1).eq.'t' .or. &
          posdev(1:1).eq.'v' .or. posdev(1:2).eq.'p ' .or. &
          posdev(1:2).eq.'pb' .or. posdev(1:2).eq.'cg' ) &
          call plafp6 (icode+4, x, y)

      sx=x+xorig
      sy=y+yorig

!
! **********************************************************************
!   some implementations need explicit setting of the default color
!

      if ((posdev(1:2).ne.'ps' .and. posdev(1:2).ne.'pl') .and. &
          dcolor.gt.0) call plcol0 (dcolor)

!
! **********************************************************************
!
      if (iplotf.eq.1 .and. irefpl.lt.0 .or. posdev(1:2).eq.'fb') then
!
!       *  write binary (unformatted) to file 8
!
         imode=8
         iref = abs(irefpl)
         write (iref) imode,sx,sy,0.,0.,ichois + 100 * dcolor
      else if (iplotf.eq.1 .or. posdev(1:2).eq.'fa' .or. &
               posdev(1:2).eq.'f1' ) then
!
!       *  write ascii (formatted) to file 8
!
         if ( dcolor.le.9 ) then
            write (iref,1000) sx,sy,ichois + 100 * dcolor
1000        format ('8',2(1x,f8.3),' 0 0 ',i3)
         else
            write (iref,1001) sx,sy,ichois + 100 * dcolor
1001        format ('8',2(1x,f8.3),' 0 0 ',i5)
         end if
      end if
      if ( posdev(1:1).eq.'f') then
         return

! **********************************************************************
!
!   *  PostScript plotter output to file:
!
      else if ( posdev(1:2).eq.'ps' .or. posdev(1:2).eq.'pl' &
           .or. posdev(1:2).eq.'pc' .or. posdev(1:2).eq.'pw' ) then
!
         call plpos3 ( ichois, sx, sy )
         return
!
! **********************************************************************
!
!   *  Local version, not supported by SEPRAN
!
      else if ( posdev(1:2).eq.'ow' ) then
!
         call plown3 ( ichois, sx, sy )
         return
!
! **********************************************************************
!
!   *  HP-GL plotter output
!
      else if (posdev(1:1).eq.'p') then
         write (irefgl,2000) mod(dcolor,8)+1
2000     format (' sp',i1,';')
         call plgl1 (sx,sy,isymb)
         if (icode.eq.-2) call plafp6 (3, x, y)
         return
!
! **********************************************************************
!
!    Unknown choice, unsupported or commented out of compiled version
!
      else
         write (irefwr,9900) posdev
9900     format (' plafp3 - device ',a,' not supported in this version')
         call instop
      end if
!
      end
