      subroutine plafscan ( xminrd, xmaxrd, yminrd, ymaxrd, ierrtn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-05-2010 Read only the first record
!        version  1.0  date 17-05-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all records from a sepran neutral file and detect minimum and maximum
!     x and y values
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     plot
!     neutral_file
!     scan
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      real xminrd, xmaxrd, yminrd, ymaxrd
      integer ierrtn

!     ierrtn         i    Error return code with the following meaning:
!                         0:  No error has been found
!                         1:  The file with reference number |irefpl| is empty
!                         2:  An incorrect record has been read in the plot file
!     xmaxrd         o    Maximum x-value read in file
!     xminrd         o    Minimum x-value read in file
!     ymaxrd         o    Maximum y-value read in file
!     yminrd         o    Minimum y-value read in file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios, ichois, nchar, iref
      real x, y, angle, height

!     angle          angle of rotation
!     height         Height of a string
!     ichois         Choice parameter
!     ios            IOSTAT variable, returns from reading or writing
!     iref           File reference number
!     nchar          Number of characters in a word
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read first record
!     Check first record
!     finished := false
!     xminrd := infinity;  xmaxrd := -infinity
!     While ( not finished ) do
!        Read next record
!        Detect next record
!        if ( last record ) then
!           finished := true
!        else ( not last record )
!           Adapt xminrd, xmaxrd
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plafscan' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plafscan'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read first record

      iref = abs(irefpl)
      ierrtn = 0
      if ( irefpl<0 ) then
         read (iref,iostat=ios) ichois,x,y,angle,height,nchar
      else
         read (iref,*,iostat=ios) ichois,x,y,angle,height,nchar
      end if

      if ( debug ) then
         write(irefwr,1) 'ichois, nchar', ichois, nchar
         write(irefwr,2) 'x,y,angle,height', x,y,angle,height
      end if  ! ( debug )

      xminrd = 0
      xmaxrd = y
      yminrd = xminrd
      ymaxrd = xmaxrd

1000  call erclos ( 'plafscan' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'xminrd, xmaxrd, yminrd, ymaxrd, ierrtn ', &
                          xminrd, xmaxrd, yminrd, ymaxrd, ierrtn
         write(irefwr,*) 'End plafscan'

      end if

      end
