      subroutine plbc00 ( usol, contln, kmeshc, coor, &
                          ncoorc, coorcn, iplots, npoint, nelemt, &
                          ncont )
! ======================================================================
!
!        programmer    Niek Praagman
!        version 10.2  date 21-11-2012 New call to plbc01
!        version 10.1  date 30-03-2005 Correction array length iwork
!        version 10.0  date 05-03-1999 New parameter list
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot contour lines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), ncoorc(*), iplots(*), npoint, nelemt, ncont
      double precision usol(*), contln(*), coor(*), coorcn(*)

!    contln  i  User array of length ncntln + 5 (if used) in which the
!               user is supposed to have stored the contour levels from
!               position 6
!    coor    i  array containing the coordinates
!    coorcn  o  At output this array contains the co-ordinates of the
!               points on the contours in the sequnce as given in NCOORC.
!               These co-ordinates are stored from position 6 in the
!               sequence (x1,y1), (x2,y2) etc. COORCN is a variable length
!               array, hence the first position must be filled by the user.
!    iplots  i  Integer control array in which the user defines which choices
!               he wants for this subroutine
!               See subroutine PLCN00 for a description
!    kmeshc  i  kmesh part c
!    ncont   i  Actual number of contour levels
!    ncoorc  o  This variable length array contains at output for every
!               contour the following information from position 6
!               For any contour level n+1 positions are used, where n is
!               the number of parts the contour levels contain. A part is a
!               continuous line. The contours are stored sequentially, &
!               starting with contour level 1.
!               pos. 1  contains the number n for the contour level (n>=1)
!               pos. 2- n+1 contain of each part the numbe of points in the
!               contour level part of which the co-ordinates are stored in
!               array COORCN.
!               If the contour part is a closed contour, then minus the
!               number of points on the contour is stored.
!               So the information of contour level 1 is stored in NCOORC
!               from position 6 to position 6 + n1 etc.
!    nelemt  i  Number of triangles in new triangular mesh
!    npoint  i  Number of nodal points
!    usol    i  array containing the values of the function to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichois, jsmoot, icolor
      double precision, allocatable :: funk(:)

!     ichois         Choice parameter
!     icolor         Sequence number of colour
!     jsmoot         Parameter indicating the amount of smoothing
!                    Minimum value 0, maximum value 5
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     PLBC01    Plot 2D contour lines
!     PLBC23    Compute and plot 2D contour lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plbc00' )
      ichois = iplots(2)
      icolor = iplots(5)
      jsmoot = iplots(7)

      allocate ( funk(npoint) )
      funk = usol(1:npoint)

!     --- Call actual plotting subroutine

      if ( ichois==1 .or. ichois==4 ) then

!     --- ichois = 1, call from PLOTC1

         call plbc01 ( coor, npoint, kmeshc, nelemt, funk, &
                       contln, ncont, jsmoot, ichois, icolor )

      else if ( ichois==3 ) then

!     --- ichois = 3, call from PLOTC3

         call plbc23 ( coor, npoint, kmeshc, nelemt, funk, &
                       contln, ncont, &
                       jsmoot, ncoorc, coorcn, ichois, icolor )
      end if
      deallocate ( funk )
      call erclos ( 'plbc00' )
      end
