      subroutine plbc12 ( inodes, coorp, coorn, npoint )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 20-01-1996 Extra parameter npoint
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 29-07-1986
!
!   copyright (c) 1986-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to smooth contourlines using the shuman - technique
!   only lines with at least 3 points are taken into account
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), npoint
      double precision coorp(2,*), coorn(2,*)

!     coorn           help array for temporary storage of results
!     coorp     i/o   coordinates of points on line , at output all
!                     internal points have been smoothed
!     inodes     i    array containing information with respect to
!                     the contourlines ( number of nodes and whether
!                     line is closed or open )
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icoor, jkn, ital, icirc, istart
      double precision eps, x1, y1, x, y, x2, y2, s1, s2, xnew, ynew, &
                       dist

!     dist
!     eps        Local accuracy
!     i          Counting variable
!     icirc      indicator whether curve is open (icirc=0) or
!                closed (icirc=1)
!     icoor
!     istart
!     ital
!     jkn
!     s1
!     s2
!     x          x co-ordinate of point
!     x1
!     x2
!     xnew
!     y          y-coordinate
!     y1
!     y2
!     ynew
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1699   Array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      eps=10d0*epsmac
      icoor=0
      jkn=1
      ital=inodes(jkn)
100   if ( ital.eq.0 ) then
         do 110 i = 1, icoor
            coorp(1,i) = coorn(1,i)
            coorp(2,i) = coorn(2,i)
110      continue
         go to 1000
      end if
      if ( ital.lt.0 ) then
         ital=-ital
         icirc=1
      else
         icirc=0
      end if
      if ( ital.lt.3 ) then
         do 150 i=1,ital
            icoor=icoor+1
            coorn(1,icoor) = coorp(1,icoor)
            coorn(2,icoor) = coorp(2,icoor)
150      continue
         jkn=jkn+1
         ital=inodes(jkn)
         goto 100
      end if
      icoor=icoor+1
      istart=icoor
      x1=coorp(1,icoor)
      y1=coorp(2,icoor)
      x =coorp(1,icoor+1)
      y =coorp(2,icoor+1)
      coorn(1,icoor)=x1
      coorn(2,icoor)=y1
      do 200 i=2,ital-1
         icoor=icoor+1
         x2=coorp(1,icoor+1)
         y2=coorp(2,icoor+1)
         s1=sqrt((x-x1)*(x-x1)+(y-y1)*(y-y1))
         s2=sqrt((x-x2)*(x-x2)+(y-y2)*(y-y2))
         if ( s1+s2.lt.eps ) then
            s1=0.5d0
            s2=0.5d0
         end if
         xnew=s1*(x+x2)+s2*(x+x1)
         ynew=s1*(y+y2)+s2*(y+y1)
         dist=2*(s1+s2)
         if ( dist.lt.eps ) then
            coorn(1,icoor)=x
            coorn(2,icoor)=y
         else
            coorn(1,icoor)=xnew/dist
            coorn(2,icoor)=ynew/dist
         end if
         x1=x
         y1=y
         x=x2
         y=y2
200   continue
      icoor=icoor+1
      if ( icoor.gt.2*npoint ) then
         call eropen ( 'plbc12' )
         call errint ( 2*icoor, 1 )
         call errint ( 4*npoint, 2 )
         call errchr ( 'coorn', 1 )
         call errsub ( 1699, 2, 0, 1 )
         call erclos ( 'plbc12' )
         go to 1000
      end if
      coorn(1,icoor)=x2
      coorn(2,icoor)=y2
      if ( icirc.eq.1 ) then
         x2=coorp(1,istart+1)
         y2=coorp(2,istart+1)
         x1=coorp(1,icoor-1)
         y1=coorp(2,icoor-1)
         s1=sqrt((x-x1)*(x-x1)+(y-y1)*(y-y1))
         s2=sqrt((x-x2)*(x-x2)+(y-y2)*(y-y2))
         xnew=s1*(x+x2)+s2*(x+x1)
         ynew=s1*(y+y2)+s2*(y+y1)
         dist=2*(s1+s2)
         if ( dist.lt.eps ) then
            xnew=x
            ynew=y
         else
            xnew=xnew/dist
            ynew=ynew/dist
         end if
         coorn(1,istart)=xnew
         coorn(1,icoor)=xnew
         coorn(2,istart)=ynew
         coorn(2,icoor)=ynew
      end if
      jkn=jkn+1
      ital=inodes(jkn)
      goto 100
1000  continue
      end
