      subroutine plbc26 ( v   , xa   , ya   , xao  , yao    , zao  , &
                          xb  , yb   , xbo  , ybo  , zbo    , &
                          kp  , lp   , mp   , coor , procor , &
                          ivis, tst  , ten  , in   , nprcor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.2  date 21-04-1995 New call of plt354
!        version  2.1  date 18-11-1994 small adjustments for norms
!        version  2.0  date 30-12-1992 extra parameter nprcor
!        version  1.0  date 06-03-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether part of line A - B is covered by
!     triangle kp,lp,mp in projection-plane.
! **********************************************************************
!
!                       KEYWORDS
!
!     covering
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kp, lp, mp, ivis, in, nprcor
      double precision v(*), xa, ya, xao, yao, zao, xb, yb, &
                       xbo, ybo, zbo, tmid, coor(*), procor(nprcor,*), &
                       tst, ten

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     in      i    indicator whether background lines are considered
!                  (in=1) or not (in=0)
!     ivis    o    indicator whether line is
!                  visible                     :  ivis = 1
!                  not visible                 :  ivis = 2
!                  begin or end part visible   :  ivis = 3
!                  middle part not visible     :  ivis = 4
!     kp      i    nodes of triangle to be considered
!     lp      i    nodes of triangle to be considered
!     mp      i    nodes of triangle to be considered
!     nprcor  i    dimension of first index of procor
!                  if nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     ten     o    output in cases ivis = 3 or ivis = 4
!                  ivis = 3 : segment ( tst,ten ) is visible
!                  ivis = 4 : segment ( tst,ten ) is not visible
!     tst     o    See ten
!     v       i    array containing the co-ordinates of the view point
!     xa      i    coordinates beginpoint in projection plane
!     xao     i    begin point in original space
!     xb      i    coordinates endpoint projection plane
!     xbo     i    end point in original space
!     ya      i    coordinates beginpoint in projection plane
!     yao     i    begin point in original space
!     yb      i    coordinates endpoint projection plane
!     ybo     i    end point in original space
!     zao     i    begin point in original space
!     zbo     i    end point in original space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ityp, itypa, itypb, jes, isnij, inum, isn, ino(3)
      double precision x1, y1, x2, y2, x3, y3, surf, xmintr, xmaxtr, &
                       ymintr, ymaxtr, xminli, xmaxli, yminli, ymaxli, &
                       ski1a, ski2a, ski3a, ski1b, ski2b, ski3b, eps, &
                       aps, t1, t2, t3, tmax, tmin, xp, yp, zp, &
                       skmina, skminb

!     aps      accuracy from above to 1
!     eps      local accuracy
!     ino      array with ordering of coordinates
!     inum     indicator for start and end point
!     isn      counter for intersection points
!     isnij    indicator for intersection or not
!     ityp     indicator for type of line A-B
!     itypa    indicator type of A
!              1 : point is in projected triangle
!              2 : point is node of triangle
!              3 : point is outside projected triangle
!     itypb    indicator type of point B
!     jes      indicator whether point is visible
!     ski1a    first area coordinate of point A
!     ski1b    first area coordinate of point B
!     ski2a    second area coordinate of point A
!     ski2b    second area coordinate of point B
!     ski3a    third area coordinate of point A
!     ski3b    third area coordinate of point B
!     skmina   smallest area coordinate point A
!     skminb   smallest area coordinate point B
!     surf     area of projected triangle
!     t1       line coordinate of first intersection point
!     t2       line coordinate of second intersection point
!     t3       line coordinate of third intersection point
!     tmax     maximal value of the t-values
!     tmid     t-value between two intersection values
!     tmin     minimal value of the t-values
!     x1       x-coordinate projection first point
!     x2       x-coordinate projection second point
!     x3       x-coordinate projection third point
!     xmaxli   largest x-coordinate line
!     xmaxtr   largest x-coordinate triangle
!     xminli   smallest x-coordinate line
!     xmintr   smallest x-coordinate triangle
!     xp       x-coordinate of midpoint of line
!     y1       y-coordinate projection first point
!     y2       y-coordinate projection second point
!     y3       y-coordinate projection third point
!     ymaxli   largest y-coordinate line
!     ymaxtr   largest y-coordinate triangle
!     yminli   smallest y-coordinate line
!     ymintr   smallest y-coordinate triangle
!     yp       y-coordinate of midpoint of line
!     zp       z-coordinate of midpoint of line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLT354   check whether point is visible with respect to triangle
!     PLT355   compute area coordinates of point
!     PLT356   compute whether intersection point is in triangle
!     PLT357   compute intersection point of two lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     determine projected points of triangle kp,lp,mp
!     compute surface triangle in projection plane and if not
!     interesting return
!     check whether intersection is possible, consider coordinates
!     triangle and line; if these regions are disjunct return
!     compute area coordinates of A and B
!     check whether these indicate that no intersection is possible
!     determine typ of points A and B
!       1 : means in projected area
!       3 : means out of projected area
!     run through all possibilities and determine the visibility
!     if fully visible return
!     if line is partially (?) visible, compute intersection points
!     with all edges and (eventually) compute visible part of line
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set local accuracy

      eps =  10 * epsmac

!     --- Set default ordering

      ino(1) = 1
      ino(2) = 2
      ino(3) = 3

!     --- determine projected points of triangle kp,lp,mp

      x1 = procor( 1, kp )
      y1 = procor( 2, kp )
      x2 = procor( 1, lp )
      y2 = procor( 2, lp )
      x3 = procor( 1, mp )
      y3 = procor( 2, mp )

!     --- check for background lines

      if ( in.eq.1 ) goto 100

!     --- Compute surface triangle in projection plane

      surf = abs ( x1*(y2-y3) + x2*(y3-y1) + x3*(y1-y2) )
      if ( surf.lt.eps ) then

!     --- Nothing visible of triangle

           ivis = 1
           goto 1000
      end if

!     --- check whether intersection is possible

      xmintr = min( x1, x2, x3 )
      xmaxtr = max( x1, x2, x3 )
      ymintr = min( y1, y2, y3 )
      ymaxtr = max( y1, y2, y3 )
      ivis = 1
      xminli = min( xa,xb )
      if ( xminli.ge.xmaxtr ) goto 1000
      xmaxli = max( xa,xb )
      if ( xmaxli.le.xmintr ) goto 1000
      yminli = min( ya,yb )
      if ( yminli.ge.ymaxtr ) goto 1000
      ymaxli = max( ya,yb )
      if ( ymaxli.le.ymintr ) goto 1000

!     --- compute area coordinates of A and B

      call plt355( x1, x2, x3, y1, y2, y3, xa, ya, ski1a, ski2a, ski3a )
      call plt355( x1, x2, x3, y1, y2, y3, xb, yb, ski1b, ski2b, ski3b )
      aps = 1d0 + eps
      if ( ski1a.lt.-eps .and. ski1b.lt.-eps ) return
      if ( ski1a.gt. aps .and. ski1b.gt. aps ) return
      if ( ski2a.lt.-eps .and. ski2b.lt.-eps ) return
      if ( ski2a.gt. aps .and. ski2b.gt. aps ) return
      if ( ski3a.lt.-eps .and. ski3b.lt.-eps ) return
      if ( ski3a.gt. aps .and. ski3b.gt. aps ) return
      skmina = min (ski1a,ski2a,ski3a)
      skminb = min (ski1b,ski2b,ski3b)
      itypa = 1
      if ( skmina.lt.0 ) itypa = 3
      itypb = 1
      if ( skminb.lt.0 ) itypb = 3
      ityp  = itypa + 3 * itypb

!     --- Run through all possibilities

      if ( ityp.eq.4 ) then

!     --- A=1 and B=1, both are in the triangle and also both
!         points are visible or both points are not visible

           call plt354( v, coor, kp, lp, mp, 0, xao, yao, zao, jes, ino)
           ivis = 2 - jes
           goto 1000
      else if ( ityp.eq.10 ) then

!     --- A=1 and B=3, if A is visible then the whole line is visible

           call plt354( v, coor, kp, lp, mp, 0, xao, yao, zao, jes, ino)
           if ( jes.eq.1 ) then

!          --- The whole line is visible

                ivis = 1
                goto 1000
           end if
      else if ( ityp.eq.6 ) then

!     --- A=3 and B=1, if B is visible then the whole line is visible

           call plt354( v, coor, kp, lp, mp, 0, xbo, ybo, zbo, jes, ino)
           if ( jes.eq.1 ) then

!          --- The whole line is visible

                ivis = 1
                goto 1000
           end if
      end if
100   continue
      if ( in.eq.1 ) ityp = 12

!     --- Line is partially (?) visible, compute intersection points
!         Edge 1

      call plt357( x2, x3, y2, y3, xa, xb, ya, yb, t1 )

!     --- Edge 2

      call plt357( x3, x1, y3, y1, xa, xb, ya, yb, t2 )

!     --- Edge 3

      call plt357( x1, x2, y1, y2, xa, xb, ya, yb, t3 )
      tmax = max ( t1, t2, t3 )
      tmin = min ( t1, t2, t3 )
      if ( ityp.eq.10 ) then

!     --- A=1 and B=3, there is one and only one intersection point

           if ( t1.le.0d0 ) t1 = 1d3
           if ( t2.le.0d0 ) t2 = 1d3
           if ( t3.le.0d0 ) t3 = 1d3
           tst  = min( t1,t2,t3 )
           ivis = 3
           ten  = 1.d0
      else if ( ityp.eq.6 ) then

!     --- A=3 and B=1, there is one and only one intersection point

           if ( t1.ge.1d0 ) t1 = -1d3
           if ( t2.ge.1d0 ) t2 = -1d3
           if ( t3.ge.1d0 ) t3 = -1d3
           tst  = 0.d0
           ten  = max( t1,t2,t3 )
           ivis =  3
      else if ( ityp.eq.12 ) then

!     --- A=3 and B=3, compute real intersection points

           isnij = 0
           call plt356( t1, xa, xb, ya, yb, x1, x2, x3, y1, y2, y3, &
                        isnij )
           if ( isnij.eq.0 ) then
                t1  = -1d0
                isn = 0
           else
                isn = 1
           end if
           isnij = 0
           call plt356( t2, xa, xb, ya, yb, x1, x2, x3, y1, y2, y3, &
                        isnij )
           if ( isnij.eq.0 ) then
                t2 = -1d0
           else
                isn = isn + 1
           end if
           isnij = 0
           call plt356( t3, xa, xb, ya, yb, x1, x2, x3, y1, y2, y3, &
                        isnij )
           if ( isnij.eq.0 ) then
                t3 = -1d0
           else
                isn = isn + 1
           end if
           if ( isn.eq.0 ) then
                ivis = 1
                goto 1000
           else if ( isn.eq.1 ) then

!          --- " Impossible situation "
!                Probable cause : Collinear lines !

                ivis = 1
                goto 1000
           else if ( isn.eq.2 ) then
                ivis = 4
                inum = 0
                if ( t1.gt.-0.5d0 ) inum = 1
                if ( t2.gt.-0.5d0 ) then
                     if ( inum.eq.0 ) then
                          tst = min(t2,t3)
                          ten = max(t2,t3)
                     else
                          tst = min(t1,t2)
                          ten = max(t1,t2)
                     end if
                     inum = 2
                end if
                if ( inum.eq.1 ) then
                     tst = min(t1,t3)
                     ten = max(t1,t3)
                end if
           else if ( isn.eq.3 ) then

!          --- Very special case : Exactly through a cornerpoint

                ivis = 4
                tst  = tmin
                ten  = tmax
           end if

!          --- Check visibility of middle point

           if ( in.eq.1 ) goto 1000
           tmid = ( tst + ten ) / 2d0
           xp   =   xao + tmid * ( xbo - xao )
           yp   =   yao + tmid * ( ybo - yao )
           zp   =   zao + tmid * ( zbo - zao )
           call plt354(v,coor,kp,lp,mp,0,xp,yp,zp,jes,ino)
           if ( jes.eq.1 ) ivis = 1
      end if
1000  continue
      end
