      subroutine plbc32 ( line,iline,procor,ihelp,help,ispace,nprcor)
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.2  date 23-01-1995 New call to prsort
!        version  2.1  date 21-11-1994 Norms
!        version  2.0  date 30-12-1992 Extra parameter nprcor
!        version  1.0  date 09-04-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Select all lines with respect to their x-coordinate from small too
!     large
! **********************************************************************
!
!                       KEYWORDS
!
!     selection
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer line(*), iline, ihelp(*), ispace(*), nprcor
      double precision procor( nprcor, * ), help(*)

!     help    i    double precision help array of length iline
!     ihelp   i    integer helparray of length 2*iline
!     iline   i    number of lines
!     ispace  i    integer work array for sorting
!     line    i    line array
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1,i2, j
      double precision xa, xb

!     i       Counting variable
!     i1      Start point of line
!     i2      End point of line
!     j       Counting variable
!     xa      x-coordinate start point
!     xb      x-coordinate end point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from calling string
!     EROPEN  Concatenate name of subroutine to calling string
!     PRSORT  Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all lines and select coordinates
!     Sort all coordinates
!     Place sorted lines back in array line
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'plbc32' )

!     --- Run through all lines and place the smallest x-coordinate of begin
!         and end point in array help

      do 100 i = 1 , iline
          i1 = line( 2 * i - 1 )
          i2 = line( 2 * i     )
          xa = procor(1,i1)
          xb = procor(1,i2)
          if ( xa.lt.xb ) then
               help(i) = xa
          else
               help(i) = xb

!              --- Change order of points

               line( 2 * i - 1 ) = i2
               line( 2 * i     ) = i1
          end if
100   continue

!     --- Sort array help, i.e. the coordinates

      call prsort( help, ihelp, iline )

!     --- Copy array line in array space

      do 200 i = 1 , 2 * iline
          ispace(i) = line(i)
200   continue

!     --- Fill ordered line array in accordance with ordering of help

      do 300 i = 1 , iline
         j = ihelp( i )
         line( 2*i - 1 ) = ispace( 2*j - 1 )
         line( 2*i     ) = ispace( 2*j     )
300   continue
      call erclos( 'plbc32' )
      end
