      subroutine plbc33 ( kmeshc, nelem, procor, ihelp, help, ispace, &
                          nprcor )
! ======================================================================
!
!
!         programmer     Niek Praagman
!         version 2.2    date  23-01-1995 New call to prsort
!         version 2.1    date  21-11-1994 Norms
!         version 2.0    date  30-12-1992 Extra parameter nprcor
!         version 1.0    date  09-04-1990
!
!
!
!   copyright (c) 1990-1995   "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Select all lines with respect to their x-coordinate from small too
!     large
!
! ***********************************************************************
!
!     KEYWORDS
!
!     selection
!     triangle
! ********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer kmeshc(*), nelem, ihelp(*), ispace(*), nprcor
      double precision procor(nprcor,*), help(*)

!     help    i    double precision help array of length iline
!     ihelp   i    integer helparray of length 2*iline
!     ispace  i    integer work array for sorting
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     nelem   i    Number of elements
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     procor  i    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, i1, i2, i3, j
      double precision xa, xb, xc, xm

!     i       loop variable
!     i1      first corner node of triangle
!     i2      second corner node of triangle
!     i3      third corner node of triangle
!     j       loop variable
!     xa      x-coordinate first node
!     xb      x-coordinate second node
!     xc      x-coordinate third node
!     xm      computed minimal value of the x-coordinates
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from calling string
!     EROPEN  Concatenate name of subroutine to calling string
!     PRSORT  Sort real array for increasing sequence
! ********************************************************************
!
!     I/O
!
! ********************************************************************
!
!     ERROR MESSAGES
!
!
! ********************************************************************
!
!     PSEUDO CODE
!
!     Run through all triangles and compute for each the smallest
!     x-coordinate
!
!     Order the computed x-coordinates
!
!     Place the reordered triangles back in the original array
!
! ======================================================================
!
      call eropen( 'plbc33' )

!     --- Run through all triangles and place the smallest x-coordinate of
!         the three nodes in array help

      do 100 i = 1 , nelem

          i1 = kmeshc( 3 * i - 2 )
          i2 = kmeshc( 3 * i - 1 )
          i3 = kmeshc( 3 * i     )

          xa = procor(1,i1)
          xb = procor(1,i2)
          xc = procor(1,i3)

          xm = min(xa,xb,xc)

          help(i) = xm

100   continue

!     --- Sort array with x-coordinates

      call prsort( help, ihelp, nelem )

!     --- Copy triangle array in help array ispace

      do 200 i = 1 , 3 * nelem

          ispace(i) = kmeshc(i)

200   continue

!     --- Refill array kmeshc with ordered triangles

      do 300 i = 1 , nelem

          j = ihelp( i )

          kmeshc( 3*i - 2 ) = ispace( 3*j - 2 )
          kmeshc( 3*i - 1 ) = ispace( 3*j - 1 )
          kmeshc( 3*i     ) = ispace( 3*j     )

300   continue

      call erclos( 'plbc33' )

      end
