      subroutine plcn00 ( iplots, rplots, contln, input, text, &
                          ncoorc, coorcn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.16 date 20-01-2015 New call to plcon1
!        version  2.15 date 09-03-2013 Initialize pointers
!        version  2.14 date 25-11-2012 Long integers
!
!   copyright (c) 1992-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual general SEPRAN contour plotting subroutine
!     Both 2D and 3D contour lines or colored contour levels may be plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     color
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemesh
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplots(*), input(*), ncoorc(*)
      double precision rplots(*), coorcn(*), contln(*)
      character (len=*) text(*)

!     contln  i  User array of length ncntln + 5 (if used) in which the
!                user is supposed to have stored the contour levels from
!                position 6
!     coorcn  o  At output this array contains the co-ordinates of the
!                points on the contours in the sequnce as given in NCOORC.
!                These co-ordinates are stored from position 6 in the
!                sequence (x1,y1), (x2,y2) etc. COORCN is a variable length
!                array, hence the first position must be filled by the user.
!                COORCN is only filled if CHOICE (IPLOTS(2)) = 3
!     input   i  This array is only used when JINDCOL (IPLOTS(5)) >0 and
!                or ISKIP>0
!                In fact this array consists of three parts:
!                Part 1 are the first 5 positions. They have to be filled
!                by the user in the following way:
!                Pos. 1:  Declared length of array input
!                Pos. 2-5: 0
!                Part 2 contains information about the colors to be used.
!                Part 2 starts at position 6 and the length depends on JINDCOL
!                If JINDCOL=1, it must be filled by the user with the colors
!                of the various contour levels in increasing sequence.
!                The length is equal to the number of contour levels
!                If JINDCOL=2 it has length 1
!                In INDCOL(6) the user must store the minimum color required
!                If JINDCOL=3 it is has length 2
!                The minimum color must be stored in INDCOL(6) the maximum
!                one in INDCOL(7)
!                If the range of the colors is less than the available range
!                of colors the color numbering is repeated
!                Part 3 contains information about the element groups to be
!                skipped. Part 3 is only used if ISKIP>0
!                The value of ISKIP defines the first position in INPUT
!                with information about the element groups to be skipped.
!                Part 3 must be filled as follows:
!                If INPUT(ISKIP-1+i)=0, element group i must be skipped
!                If INPUT(ISKIP-1+i)=1, element group i must be taken into
!                account
!     iplots  i  Integer control array in which the user defines which choices
!                he wants for this subroutine
!                IPLOTS must be filled  by the user in the following way:
!                Position:
!                 1: Not used
!                 2: (CHOICE) defines the type of contour lines or levels
!                    required. Possible values for CHOICE are:
!                    1:  Contour lines will be drawn. These lines are
!                        provided with a numbered label.
!                        In the case of a 3D grid, contour lines are plotted
!                        on the outer surface only
!                    2:  Contour levels will be colored, i.e. the interior
!                        space between contour levels as well as the region
!                        smaller than the smallest contour level and the
!                        region larger than the largest contour level is
!                        filled with a specific color for that level.
!                    3:  See 1, however, not only are the contour levels
!                        plotted, but also the co-ordinates of these contour
!                        lines are outputed in the arrays COORCN and NCOORC
!                        This possibility is only available in 2D
!                    4:  See 1, however numbers are not plotted
!                 3: (NUMBER) the number'th physical unknown in each node is
!                    considered When this unknown does not appear in a nodal
!                    point, the point is skipped
!                    When array ISOL corresponds to a complex array, &
!                    NUMBER=2*i-1 corresponds to the real part of unknown i
!                    and NUMBER=2*i to the imaginary part of unknown i
!                 4: (NCNTLN) Parameter indicating the number of contour
!                    levels to be used. Possible values of ncntln are:
!                   >0:  There are ncntln contour levels that must be stored
!                          in array contln from position 6
!                    = 0:  The program computes the contour levels by
!                          dividing the range of the function between maximum
!                          and minimum value into 10 parts in the case of
!                          CHOICE = 1 or 3 and in 20 parts if CHOICE=2
!                   <0>-1000: The program computes the contour levels by
!                          dividing the range of the function between maximum
!                          and minimum value into |NCNTLN| - 1 parts. The
!                          values are chosen such that rounded values of the
!                          contour levels appear
!                   <-1000: The program computes the contour levels by
!                          dividing the range of the function between maximum
!                          and minimum value into |NCNTLN + 1000| - 1 parts.
!                    -10000: separate levels are used for the positive values
!                            as well as for the negative values
!                            The zero level is always plotted
!                          The number of positive levels is stored in pos. 23
!                          The number of negative levels is stored in pos. 24
!                    -10001: see -10000, however, now the values are rounded
!                            as well as for the negative values
!                 5: (JINDCOL) Parameter indicating the type of coloring to
!                    be used for the contour lines or levels:
!                    Possible values:
!                    0:  The standard colors are used, i.e. standard color
!                        contour lines in the case of CHOICE=1 or CHOICE=3
!                        and color numbers 1 to number_of_contour_levels in
!                        the case of CHOICE=2.
!                    1:  The user defines the color numbers explicitly
!                        through array INDCOL. This possibility is
!                        moment only available for CHOICE=2
!                    2:  The user defines the minimum color number explicitly
!                        through array INDCOL. This possibility is
!                        only available for CHOICE=2
!                    3:  The user defines the minimum and maximum color
!                        number explicitly through array INDCOL. This
!                        possibility is only available for CHOICE=2
!                    >0:  The visible parts of the contour lines get the
!                        color sequence number JINDCOL CHOICE=1 or 3
!                 6: (PROJECT) choice parameter for type of projection
!                    (3D only)
!                    0:  parallel projection plot (View point automatic)
!                    1:  perspective projection plot (View point automatic)
!                    10: parallel projection plot (view point given by the
!                        user in RPLOTS(4-6) )
!                    11: perspective projection plot (view point given by the
!                        user in RPLOTS(4-6) )
!                 7: (JSMOOTH) choice parameter for type of smoothing to be
!                    used for the contour lines. At this moment JSMOOTH is
!                    only implemented for CHOICE=1 or 3 in 2-dimensional
!                    grids.
!                    Possible values:
!                    0:  no smoothing
!                    1:  shuman smoothing
!                    2:  |
!                    3:  |  all spline smoothing : the higher the value the
!                    4:  |  more smoothing and the less interpolation of
!                    5:  |  the values.
!                 8: (IDOUBLE) indication whether the outer boundary is
!                    plotted as a single line (0), as a double line (1)
!                    (2D only) or is not plotted at all (2)
!                 9: (INNER) indication whether inner boundaries must be
!                    plotted (1) or not (0)
!                    (2D only)
!                10: (PLOT_CONTOUR) indication if the boundaries of the
!                    contourlevels must be plotted (>0) or not (0). The
!                    value of PLOT_CONTOUR defines the color.
!                    This possibility is only available in the case CHOICE=2.
!                11: (TYPE_LEGENDA) defines the type of legenda to be plotted
!                    Possible values:
!                    1:  No legenda
!                    2:  Standard legenda; the text is plotted at the right
!                        side of the picture.
!                    3:  Horizontal legenda; the text plotted below the
!                        picture. Only minimum and maximum values are given.
!                        Only available for CHOICE=2
!                12: (TYPE_SCALES) defines whether the scaling factors are
!                    plotted (1) or not (2)
!                13: (ISKIP) indicates if element groups must be excluded (>0)
!                    or not (0)
!                    If ISKIP>0, the value of ISKIP defines the first position
!                    in INPUT where information about the element groups
!                    to be excluded is stored
!                14: IDEFFM   Indicates if default sizes of the plot are used
!                    (IDEFFM=0) or that the user gives these values
!                    in RPLOTS(1) and RPLOTS(2) (IDEFFM=1)
!                    If IDEFFM=2 the user must also fill RPLOTS(3) with a
!                    premultiplication factor for the solution
!                15: (IMESH) indicates if the mesh must be plotted in the
!                    picture (>0) or not (0)
!                    If IMESH>0, the value of IMESH defines the color
!                    sequence number with which the mesh must be plotted.
!                    In 3D only the visible parts of the mesh are plotted
!                    with color IMESH
!                    If in 3D CHOICE=2 is used, plotting of the mesh means
!                    plotting of triangles. In the case of quadrilaterals
!                    or quadratic elements this may mean that the mesh is
!                    subdivided into smaller triangles
!                16: (INVIS_MESH) indicates if the invisible part of the
!                    mesh must be plotted in the picture (>0) or not (0)
!                    If INVIS_MESH>0, the value of INVIS_MESH defines the
!                    color sequence number with which the invisible parts
!                    of the mesh must be plotted. (CHOICE=1 and 3D only)
!                17: (INVIS_CONT) indicates if the invisible part of the
!                    contour lines must be plotted in the picture (>0) or not
!                    (0). If INVIS_CONT>0, the value of INVIS_CONT defines the
!                    color sequence number with which the invisible parts
!                    of the contours must be plotted. (CHOICE=1 and 3D only)
!                18: (BACK_COLOUR) indicates if back ground lines must
!                    plotted in the picture (>0) or not (0)
!                    If BACK_COLOUR>0, the value of BACK_COLOUR defines the
!                    color sequence number with which the back ground lines
!                    must be plotted. (CHOICE=1 and 3D only)
!                19: (CURV_COLOUR) indicates if curves must be
!                    plotted in the picture (>0) or not (0)
!                    If CURV_COLOUR>0 and<100, the value of CURV_COLOUR
!                    defines the color sequence number with which the
!                    visible part of the curves must be plotted.
!                    If CURV_COLOUR>100 also the invisible part of the
!                    curves is plotted, the value of
!                    CURV_COLOUR-100 defines the color sequence number
!                    with which the curves must be plotted.
!                    This possibility is only available for 3D
!                20: (ITEXT) indicates if text must be plotted in the
!                    picture.
!                    Possible values:
!                    0:  No text at all
!                    1:  Extra text at bottom
!                    2:  Extra text at bottom and top
!                21: (ICOMPT) indicates how the contour levels must be
!                    computed from NCNTLN in the case NCNTLN<=0
!                    0:  The minimum and maximum level are computed
!                    1:  The minimum level is given in contln(6), the
!                        maximum level must be computed
!                    2:  The maximum level is given in contln(7), the
!                        minimum level must be computed
!                    3:  The minimum level is given in contln(6), the
!                        maximum level in contln(7)
!                22: (IORIENT) indicates the orientation of the base vectors
!                    Possible values:
!                    1 : x - y - z system (also default)
!                    2 : z - x - y system
!                    3 : y - z - x system
!                23: (NLEV_POS) indicates the number of positive levels
!                    for ncntln = -10000 or -10001
!                24: (NLEV_NEG) indicates the number of negative levels
!                    for ncntln = -10000 or -10001
!     ncoorc  o  This variable length array contains at output for every
!                contour the following information from position 6
!                For any contour level n+1 positions are used, where n is
!                the number of parts the contour levels contain. A part is a
!                continuous line. The contours are stored sequentially,
!                starting with contour level 1.
!                pos. 1  contains the number n for the contour level (n>=1)
!                pos. 2- n+1 contain of each part the number of points in the
!                contour level part of which the co-ordinates are stored in
!                array COORCN.
!                If the contour part is a closed contour, then minus the
!                number of points on the contour is stored.
!                So the information of contour level 1 is stored in NCOORC
!                from position 6 to position 6 + n1 etc.
!     rplots  i  Double precision input array that must be filled as follows:
!                1:    plotfm
!                2:    yfact
!                3:    scale factor
!                      This parameter is only used if IDEFFM=2
!                4-6:  eye-point  (Only if PROJECT>9)
!     text    i  Text array containing the texts to be plotted
!                1:  Text at bottom of the picture
!                2:  Text at top of the picture
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl, firleg, bouncurv
      integer ibackl, ichoic, ichois, iconcl, ideffm, idoubl, ikader, &
              iline, ilplot(1), imesh, indlin, ipinpt,  &
              inner, iresul(5,3), iplotc, leng, &
              iskip, itext, ityplg, itypsc, jsmoot, jv, &
              ktimes, licrvs, ncont, nelgrpsav, nelemsav,  &
              nelemh, neleml, nelemt, nelglc, nomber(2), nplsub, &
              nprcor, number, special, iscolr, &
              ilevel, isolcop, plotzerolev
      integer :: kmeshalc(1), kmeshglc(1)
      double precision, allocatable :: help2(:), help3(:), &
                                       coornw(:), cont(:)
      integer, pointer :: arline(:), kmeshcquat(:)
      integer, allocatable :: incol(:), line(:)
      double precision xmean(3), vact(3)
      logical perspc, ncontr, part, fillcl, integervect
      save firleg

!     arline         array to store line pieces
!     bouncurv       if true the 2D boundaries are plotted using icurvs/curves
!                    otherwise using kmeshm and coor
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     cont           Array with the contour levels
!     coornw         coordinate array for projected points
!                    (1,i) contains the x-coordinate of the projected point i
!                    (2,i) contains the y-coordinate of the projected point i
!                    (3,i) contains the distance of point i to
!                          the view-point
!     fillcl         Indicates if the elements must be filled with a color
!                    (true) or that only the boundaries must be plotted (false)
!                    If fillcl = true, the color is given by dcolor, the
!                    boundaries get the default color. Otherwise the color
!                    of the boundaries is defined by dcolor
!     firleg         Indicates if this is the first time a contour plot in the
!                    present picture is made (true) or not (false)
!                    If true a legenda may be plotted
!     help2          double precision help array
!     help3          double precision help array
!     ibackl         See IPLOTS(18)
!     ichoic         Choice parameter for TOPQU0. With this parameter it is
!                    defined which actions will be performed. Possible values:
!                    1:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex only the real or
!                        imaginary part is used
!                    2:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex the copied solution
!                        vector is also complex
!                    3:  Create a submesh consisting of the surface elements
!                        only of each element only the vertices are stored, &
!                        hence the final grid consists of a grid of 3-point
!                        triangles or 4-point quadrilaterals only
!                        This possibility only uses KMESH
!                        It has only been implemented for 3D
!                    4:  Combination of 1 and 3, hence possibly 2 meshes may be
!                        created
!                    5:  Combination of 2 and 3, hence possibly 2 meshes may be
!                        created
!                    6:  Create a submesh consisting of 4-point tetraeders only
!                        Create also a copy of the solution array containing
!                        number(1) unknowns in all points of the new grid
!     ichois         Choice parameter See IPLOTS(2)
!     iconcl         See IPLOTS(19)
!     ideffm         See IPLOTS(14)
!     idoubl         See IPLOTS(8)
!     ikader         Help variable to store jkader temporarily
!     ilevel         dummy
!     iline          number of lines
!     ilplot         Local parameter representing array iplots for subroutine
!                    PLOTSF
!     imesh          See IPLOTS(15)
!     incol          This array is only used if isubr=2
!                    It is filled with the colors of the levels
!     indlin         Sequence number INFOR for "line-elements" array
!     inner          See IPLOTS(9)
!     integervect    if true dooouble points must be removed
!     ipinpt         Starting address of array ISKIP (IHELP) in INPUT
!     iplotc         See IPLOTS(10)
!     iresul         Output array of subroutine TOPQU0
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                            (ichois=1,3,4)
!                    Pos .,3 corresponds to the solution array (ichois=1,3,4)
!                    Pos. 1,1  INDVER  sequence number with respect to INFOR
!                                      for the "vertex" mesh array
!                    Pos. 2,1  NPELM   number of points per element for this
!                                      mesh
!                    Pos. 3,1  NELEM   number of points per element for this
!                                      mesh
!                    Pos. 4,1  NEWVER  indicates if the mesh is new (1) or is
!                                      in fact the standard mesh (0)
!                    Pos. 5,1  INDLIN  sequence number with respect to INFOR
!                                      for the "line-elements" array
!                                      In case of special=2, it contains the
!                                      elgrps array
!                    Pos. 1,2  INDTRI  sequence number with respect to INFOR
!                                      for the "triangle" mesh array
!                    Pos. 2,2  NPELM   number of points per element for this
!                                      mesh
!                    Pos. 3,2  NELEM   number of points per element for this
!                                      mesh
!                    Pos. 4,2  NEWTRI  indicates if the mesh is new (1) or is
!                                      in fact the standard mesh (0) or the mesh
!                                      created by the "vertex" mesh
!                    Pos. 5,2  NPHYS   Max. number of degrees of freedom
!                    Pos. 1,3  INDSOL  sequence number with respect to INFOR
!                                      for the new solution array
!                    Pos. 2,3  NEWSOL  indicates if the solution array is new
!                                      (1) or is in fact the standard solution
!                                      array (0)
!                    Pos. 3,3  INDPRF  sequence number of KPROBF
!                    Pos. 4,3  INDPRH  sequence number of KPROBH
!                    Pos. 5,3  INPPRP  sequence number of KPROBP
!     iscolr         Help variable to store the value of dcolor temporarily
!     iskip          See IPLOTS(13)
!     isolcop        Contains memory management number of copy of isol
!     itext          See IPLOTS(20)
!     ityplg         See IPLOTS(11)
!     itypsc         See IPLOTS(12)
!     jsmoot         Parameter indicating the amount of smoothing
!     jv             Indicates if the view point has been filled (1) or not (0)
!     kmeshalc       Local array KMESH part a for PLOTSF
!     kmeshcquat     Array kmeshc corresponding to quadratic elements
!     kmeshglc       Local array KMESH part g for PLOTSF
!     ktimes         mod(jtimes,10)
!     leng           size of array
!     licrvs         Number of lines stored in array for line pieces
!     line           line array
!     ncont          Actual number of contour levels
!     ncontr         indication whether the elements are contracted (false)
!                    or not (true)
!     nelemh         Help number of triangles
!     neleml         Number of "line"elements
!     nelemsav       Help parameter to save the old value of nelem
!     nelemt         Number of triangles in new triangular mesh
!     nelglc         Local number of element groups with respect to PLOTSF
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     nomber         Array where is stored which unknown has to be considered
!     nplsub         Number of points in the submesh consisting of elements
!                    with vertex nodes only
!     nprcor         Dimension of first index of procor
!                    If nprcor = 3, pl0014 is called by a color subroutine,
!                    which implies that also the distance to the view-point is
!                    computed, if nprcor = 2, only the projection is computed
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and number=2*i
!                    to the imaginary part of array i
!     part           indication whether all elements are plotted (false)
!                    or that some are skipped (true)
!     perspc         indicates if a perspective (true) or a parallel projection
!                    (false) must be applied
!     plotzerolev    if 1 then plot boundary of zero level set
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a colored 2d plot
!                          is required
!                    2     solution vector is of type 110 or 115 and a colored
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
!     vact           computed (corrected) view point
!     xmean          Array containing the mean values of the co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DSCAL          Blas routine to scale a vector
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPINT2DOUB Copy integer array to double precision array
!     PL0003         Open plot, plots frame work and computes plotting
!     PL0014         Compute projection of points onto projection plane
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLBC00         Actual contour plot
!     PLBC25         Compute some parameters and fill contour levels
!     PLBC31         Subroutine to place all lines of triangular elements in an
!                    array
!     PLBC32         Select all lines with respect to their x-coordinate
!                    from small too large
!     PLBC33         Select all lines with respect to their x-coordinate
!                    from small too large
!     PLBC34         Make colored contour plot of a function defined per
!                    element in 2D
!     PLBC35         Plot background lines in case of a 3-D projection
!     PLBC36         Plot curves in projection plane
!     PLBC37         Plot projection of 3-D object with hidden lines removed
!     PLBC38         Subroutine to plot projections of contour lines on a mesh
!                    in 3-D
!     PLBC39         Reset co-ordinates array after a call of PL0014
!     PLBC40         Subroutine to place all sides of quadrilaterals in line
!                    array
!     PLBC41         Fill array with the line pieces of the curves as given by
!                    the user
!     PLBC42         Select all (subtriangles of the) quadrilaterals with
!                    respect to their x-coordinate from small to large
!     PLCN01         Put the parameters from array IPLOTS into common celint
!                    Compute number of contour lines
!     PLCON1         Actual contour plot for colored levels
!     PLELM1         Plot elements in 2D mesh
!     PLLGCL         Plot legenda in colored level case
!     PLLGCN         Plot legenda
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTBN         Plot boundary of region
!     PLOTSF         Make a three-dimensional hidden line plot of a 3D surface,
!                    using colored surfaces. (Display only)
!     PLTOPBOTTEXT   Plot text at top or bottom of plot if required
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRPLOTZEROLEV  Plot zero level set curve
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TOPQ01         Subroutine to cancel all double lines in array line
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check arrays KMESH, KPROB, ISOL and compute IPROB
!     Put array IPLOTS into common CELINT and compute number of contour
!     levels
!     Activate standard arrays
!     Compute contour levels
!     Subdivide grid into triangles
!     Plot actual contour lines
!     Plot mesh if required
!     Plot texts and legenda
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data firleg /.true./
! ======================================================================
!
      call eropen ( 'plcn00' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plcn00'
         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,1) 'inelem, nelgrp', inelem, nelgrp
  1      format ( a, 1x, (10i6) )

      end if

!     --- Clear iresul

      iresul = 0

!     --- Put array IPLOTS into common CELINT and compute number of contour
!         levels

      ierror = 0
      iwarn  = 0

      call plcn01 ( iplots, ndim, ncont )
      ichois = iplots(2)
      number = iplots(3)
      jsmoot = iplots(7)
      idoubl = iplots(8)
      inner  = mod(iplots(9),10)
      plotzerolev = iplots(9)/10
      if ( inner==1 ) then
         inner = 3
      else
         inner = 1
      end if
      iplotc = iplots(10)
      ityplg = iplots(11)
      itypsc = iplots(12)
      iskip  = iplots(13)
      ideffm = iplots(14)
      imesh  = iplots(15)
      ibackl = iplots(18)
      iconcl = iplots(19)
      itext  = iplots(20)

      if ( debug ) then
         call prinin ( iplots, 21, 'iplots' )
         call prinin ( isolut, 5, 'isolut' )
         write(irefwr,1) 'plotzerolev, inner', plotzerolev, inner
      end if  ! ( debug ) then

      if ( ks(isolut)%typevec==136 ) then

!     --- Vector is of type 136, i.e. integer vector
!         isol is saved into isolcop and a double precision
!         variant of usol is stored in a new version of isol
!         in this way the subroutine does not have to be adapted
!         for integers

         integervect = .true.
         isolcop = isolut
         nusol = ks(isolut)%nusol
         isolut = 0      ! new array
         call sepcreasolbf1 ( isolut, 116,  nusol/nelem, 'usolcopy', 0 )
         usol => usoltemp
         isoltemp => ks(isolcop)%isol
         call inicopint2doub ( usol, isoltemp, nusol )

      else

!     --- Vector is of other type, standard case

         integervect = .false.

      end if  ! ( ks(isolut)%typevec==136 )

!     --- set special

      if ( debug ) write(irefwr,1) 'ndim', ndim
      if ( ndim==2 .and. ks(isolut)%typevec==116 .and. ichois==2 ) then

!     --- Coloured 2d plot of vectors defined per element

         special = 1

      else if ( ndim==2 .and. ichois==2 ) then

!     --- Coloured 2d plot of vectors of type 110 or 115
!         First special is set equal to 2.
!         If unknowns are present in the vertices special is reset to 0

         special = 2

      else

!     --- standard case

         special = 0

      end if

!     --- Subdivide grid into triangles, activate and check arrays

      if ( iskip==0 ) then
         ipinpt = 1
      else
         ipinpt = iskip
      end if
      if ( debug ) &
         write(irefwr,1) 'ndim, ichois, imesh, iconcl, special', &
                          ndim, ichois, imesh, iconcl, special

      if ( ndim==3 .and. ( ( ichois==1 .or. ichois==4 ) .and. &
           (iconcl>0 .and. iconcl<=100 .or. imesh>0) .or. &
           (ichois==2 .and.iconcl>0 .and. iconcl<=100) ) ) then
         ichoic = 4
      else
         ichoic = 1
      end if
      if ( debug ) write(irefwr,1) 'ichoic', ichoic

      compl = .false.
      nomber(1) = 1
      nomber(2) = number
      nelgrpsav = nelgrp
      nelemsav = nelem

      if ( special/=1 ) then

!     --- special = 0 or 2, subdivide mesh into sub elements if necessary

         ilevel = 1
         call topqu0 ( ichoic, nomber, iskip, input(ipinpt), iresul, &
                       compl, 0, special, ilevel )

      else

!     --- special = 1

         call sepactsolbf2 ( isolut )
         compl = ks(isolut)%compl==1
         if ( compl ) nphysi = nphysi*2
         if ( debug ) write(irefwr,1) 'nphysi', nphysi

      end if
      if ( ierror/=0 ) go to 1000

      bouncurv = .false.
      if ( ndim==2 .and. kelmh>0 ) then

!     --- ndim =2 and kmesh part h has been filled
!         Check if icurvs and curves have been filled

         if ( kmeshh(3,1)>0 .and. kmeshh(5,1)>0 ) bouncurv = .true.

      end if

!     --- Compute contour levels
!         Compute starting position of storage of contour levels in ibuffr
!         These positions are stored at the end of IBUFFR
!         Later on they are deleted

      allocate ( cont(ncont), incol(ncont+1), stat = error )
      if ( error/=0 ) call eralloc ( error, ncont, 'cont' )
      if ( ierror/=0 ) go to 1000

      if ( special==0 ) then
         indprfi = iresul(3,3)
         indprpi = iresul(5,3)
         indlin = iresul(5,1)
         nphysi  = iresul(5,2)
         usol => ks(isolut)%sol
         nelemt = 1

      else
         indprfi = 0
         indprpi = 0
         indlin = 0
         nelemt = nelem
         if ( special==1 ) then
            usol => ks(isolut)%sol
         else
            usol => usolt
            nphysi = 1
         end if
         if ( special==2 ) number = 1
      end if

      call plbc25 ( contln, usol, cont, &
                    compl, input, incol, &
                    rplots(3), special, iplots, ncont, nelemt )
      if ( debug ) &
         call prinrl ( cont, ncont, 'contour levels' )

      iline = 0
      if ( ichoic==4 ) then

!     --- Either the mesh or the curves array must be plotted in 3D
!         Fill the lines array

         nplsub = iresul(2,1)
         nelem  = iresul(3,1)

         if ( nplsub>0 .and. nelem>0 ) then


            allocate ( line(2*nplsub*nelem), stat = error )
            if ( error/=0 ) call eralloc ( error, 2*nplsub*nelem, 'line' )
            if ( ierror/=0 ) go to 1000
            if ( nplsub==3 ) then

!           --- Triangles

               call plbc31 ( kmeshcsub, nelem, line, iline )
            else

!           --- Quadrilaterals

               call plbc40 ( kmeshcsub, nelem, line, iline )

            end if

         end if

      end if

!     --- Reserve work space

      leng = 0
      nprcor = 3
      if ( special==0 ) nelemt = iresul(3,2)

!     --- Determine whether there are line elements

      if ( indlin/=0 ) then

!     --- line elements

         neleml = size ( kmeshcline ) / 2

      else

!     --- no line elements

         neleml = 0

      end if

      if ( ndim==3 ) then

!     --- ndim = 3
!         Array PROCOR (COORNW)

         if ( ichois/=2 ) nprcor = 2

      end if

      leng = 1
      if ( nplsub==4 ) leng = 6 * nelemt
      allocate ( help1(7*iline), help2(2*iline), help3(2*iline), &
                 arline(2*iline), kmeshcquat(leng), &
                 coornw(nprcor*npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, iline, 'help1' )
      if ( ierror/=0 .or. iwarn/=0 ) go to 1000

      if ( special==0 .or. special==2 ) then
         usol => usolt
         if ( iresul(1,2)>0 .and. iresul(1,2)/=kelmc ) then
            kmeshc => kmeshctmp
         end if
      end if

!     --- Open plot

      ktimes = mod(jtimes,10)
      ikader = jkader
      if ( debug ) write(irefwr,1) 'ktimes', ktimes
      if ( ktimes<2 .and. ndim==2 ) then

!     --- ktimes<2, hence open plot

         textx = 'x'
         texty = 'y'
         if ( bouncurv ) then

!        --- Use arrays curves and icurvs

            call pl0003 ( ikader, 1, rplots(1), rplots(2), 10 )

         else

!        --- Use arrays coor and kmeshm

            call pl0003 ( ikader, 1, rplots(1), rplots(2), 0 )

         end if

      end if

      if ( ndim==3 ) then

!     --- ndim = 3, compute the 2-D coordinates and distances of the projected
!         points

         jv = iplots(6)/10
         perspc = mod(iplots(6),10)==1

         call pl0014 ( coor, rplots(4), jv, coornw, &
                       npoint, xmean, perspc, nprcor, vact, 1, &
                       rplots(1), rplots(2), iplots(22) )

      end if

!     --- Plot contours

      if ( special>=1 ) then

!     --- Special case of vectors of type 116 with colored 2d plot
!         or vectors of type 110 or 115 only defined in the centroid

         if ( special==2 ) nphysi = 1
         call plbc34 ( usol, cont, ncont, &
                       max(number,1), nphysi, incol, special, &
                       kmeshcline )

      else if ( ichois/=2 ) then

!     --- Standard case

         if ( ndim==2 ) then

!        --- ndim = 2

            call plbc00 ( usol, cont, kmeshc, &
                          coor, ncoorc, coorcn, iplots, &
                          npoint, nelemt, ncont )

         else

!        --- ndim = 3

            call plbc38 ( kmeshc, coor, usol, &
                          nelemt, rplots(4), coornw, &
                          cont, ncont, &
                          iplots(5), iplots(17), nprcor )

         end if

      else

!     --- colored contour levels

         if ( indlin/=0 ) then

!        --- line elements

            neleml = size ( kmeshcline ) / 2

         else

!        --- no line elements

            neleml = 0

         end if

         if ( ndim==2 ) then

!        --- ndim = 2

            call plcon1 ( kmeshc, coor, &
                          usol, cont, ncont, &
                          nelemt, incol, iplotc, &
                          neleml, kmeshcline, npoint )

         else

!        --- ndim = 3

            kmeshalc(1) = 3
            kmeshglc(1) = nelemt
            nelglc = 1
            ilplot(1) = 2

            call plotsf ( coornw, npoint, kmeshalc, kmeshglc, &
                          nelglc, kmeshc, &
                          ilplot, usol, &
                          cont, ncont, incol, &
                          imesh, iplotc, neleml, kmeshcline )

         end if

      end if

      if ( ndim==2 .and. ktimes<=1 .and. idoubl/=2 ) then

!     --- Plot boundary of region

         call plotbn ( idoubl, inner, bouncurv )

      end if

!     --- Plot back ground if required

      if ( ndim==3 .and. ibackl>0 ) then

         call plbc35 ( coor, vact, coornw, xmin, xmax, &
                       ymin, ymax, rplots(1), kmeshc, nelemt, &
                       iplots(18), nprcor )

      end if

!     --- Plot mesh if required

      if ( imesh>0 ) then

         if ( ndim==2 ) then

!        --- ndim = 2, plot elements with color imesh

            ncontr = .false.
            fillcl = .false.
            if ( iskip==0 ) then
               part = .false.
            else
               part = .true.
            end if
            iscolr = dcolor
            dcolor = imesh
            call plelm1 ( input(ipinpt), coor, &
                          ncontr, part, fillcl )
            dcolor = iscolr

         else if ( ichois/=2 ) then

!        --- ndim = 3, plot of mesh with hidden lines

            call topq01 ( line, iline, npoint, help1, help2 )

!           --- Select all lines with respect to their x - coordinate

            call plbc32 ( line, iline, coornw, &
                          help1, help2, help3, nprcor )

!           --- Select all triangles with respect to their x-coordinate

            if ( nplsub==3 ) then

               call plbc33 ( kmeshcsub, nelem, coornw, &
                             help1, help2, help3, nprcor )

               nelemh = nelem

            else if ( nplsub==4 ) then

               call plbc42 ( kmeshcsub, nelem , coornw, &
                             help1, help2, help3, nprcor, kmeshcquat )

               kmeshcsub => kmeshcquat
               nelemh = 2 * nelem

            end if

!           --- Plot all lines

            call plbc37 ( coor, coornw, &
                          kmeshcsub, nelemh, help1, &
                          help2, line, iline, &
                          vact, iplots(15), iplots(16), nprcor )

            deallocate ( line, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'line' )
            if ( ierror/=0 ) go to 1000

         end if

      end if

      if ( allocated(line) ) then

         deallocate ( line, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'line' )
         if ( ierror/=0 ) go to 1000

      end if   ! ( allocated(line) )

!     --- Plot curves if required

      if ( ndim==3 .and. iconcl>0 ) then

         if ( iconcl>100 ) then

!        --- Plot all curves, visible and non-visible

            call plbc36 ( ncurvs, kmeshm, coornw, &
                          iconcl-100, nprcor )
         else

!        --- Plot visible curves only
!            Start by determining curves

            call plbc41( ncurvs, kmeshm, arline, licrvs)

!           --- Select all lines with respect to their x - coordinate

            call plbc32 ( arline, licrvs, coornw, &
                          help1, help2, help3, nprcor )

!           --- Select all triangles with respect to their x-coordinate

            if ( nplsub==3 ) then

!           --- Triangle mesh

               call plbc33 ( kmeshcsub, nelem, coornw, &
                             help1, help2, help3, nprcor )

               nelemh = nelem

            else if ( nplsub==4 ) then

!           --- Quadrilateral mesh

               call plbc42 ( kmeshcsub, nelem, coornw, &
                             help1, help2, help3, nprcor, kmeshcquat )

               kmeshcsub => kmeshcquat
               nelemh = 2 * nelem

            end if

!           --- Plot all lines

            call plbc37 ( coor, coornw, &
                          kmeshcsub, nelemh, help1, &
                          help2, arline, licrvs, &
                          vact, iconcl, 0, nprcor )

         end if

      end if

!     --- Plot texts and legenda

      if ( ktimes==0 .or. ktimes==1 ) firleg = .true.
      if ( firleg ) then
         if ( ideffm==2 ) call dscal ( ncont, rplots(3), &
              cont, 1 )
         if ( ichois/=2 ) then
            call pllgcn ( cont, ncont, itypsc, ityplg )
         else
            call pllgcl ( cont, ncont, incol, itypsc, &
                          ityplg )
         end if

!        --- Plot text at top and bottom if required

         call pltopbottext ( itext, text, rplots(1) )
         firleg = .false.

         if ( plotzerolev==1 ) then

!        --- plotzerolev=1, plot zero level set, with color red

            dcolor = 0  ! set color black
            call prplotzerolev

         end if  ! ( plotzerolev==1 ) then

      end if

!     --- Close plot

      if ( ktimes<1 .or. ktimes>2 ) then
         call plmore
         call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
         firleg = .true.
      end if

      jkader = ikader

!     --- Error and warning checks

1000  if ( ierror/=0 ) then

!     --- ierror # 0  Reset ierror and nerror

         ierror = 0
         nerror = 0

         if ( jtimes==11 ) jtimes = 1

      end if

      if ( iwarn/=0 ) then

!     --- iwarn # 0  Reset iwarn and nwarn

         iwarn = 0
         if ( nwarn<10 ) nwarn = 0

      end if

      deallocate ( help1, help2, help3, arline, kmeshcquat, coornw, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'help1' )
      if ( ierror/=0 ) go to 1100

!     --- Reset co-ordinates array

      if ( nprcor==2 ) call plbc39 ( coor, npoint, xmean )

      if ( iresul(2,3)==1 ) call seprmsolinfo ( iresul(1,3) )
      if ( iresul(4,2)==1 ) deallocate ( kmeshctmp )
      if ( iresul(4,1)==1 ) deallocate ( kmeshcsub )
      if ( iresul(5,1)>0 ) deallocate ( kmeshcline )
      if ( integervect ) then

!     --- Remove double precision array and copy old integer array back

         call seprmsolinfo ( isolut )
         isolut = isolcop
         ks(isolut)%typevec = 136

      end if  ! ( integervect )
      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem

      deallocate ( cont, incol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'cont' )
      if ( ierror/=0 ) go to 1000

      if ( iresul(1,2)>0 .and. associated(kmeshctmp) ) deallocate(kmeshctmp)

1100  if ( debug ) then
         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,*) 'End plcn00'
      end if  ! ( debug )
      call erclos ( 'plcn00' )

      end subroutine plcn00
