      subroutine plcompkies ( bmin, bmax, m, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-10-2011 Debug
!        version  1.0  date 04-10-2000
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      the parameters m and n are computed corresponding to bmin and bmax
!
! **********************************************************************
!
!                       KEYWORDS
!
!     rounding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bmin, bmax
      integer n, m

!    bmax      i   upper bound
!    bmin      i   lower bound
!    m         o   indication whether the result can be considered
!                  in fixed point format (m=1)
!                  or floating point format (m=0)
!    n         o   number of significant digits behind the decimal point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ismin, ismax, iexp, mantmn, istep, mx, mn, iextra, &
              ibmin1, ibmax1, mantmx
      double precision amstep, step, e

!     amstep
!     e
!     ibmax1
!     ibmin1
!     iexp
!     iextra
!     ismax
!     ismin
!     istep
!     mantmn
!     mantmx
!     mn
!     mx
!     step
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inipl0, inipl1, inipl2

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPL0         Find smallest nice number larger than argument
!     INIPL1         Find largest nice number smaller than argument * 100
!     INIPL2         Compute exponent of real number
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plcompkies' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plcompkies'
         write(irefwr,1) 'n, m', n, m
         write(irefwr,2) 'bmin, bmax', bmin, bmax
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute signs of the numbers

      if ( abs(bmax)<=0d0 .and. abs(bmin)<=0d0 ) then
         m = 1
         n = 1
      else
         ismin=0
         if (abs(bmin)>0d0) ismin=nint(sign(1d0,bmin))
         ismax=0
         if (abs(bmax)>0d0) ismax=nint(sign(1d0,bmax))
         amstep = 1d-5 * ( abs(bmax)+abs(bmin) )
         step=max(abs(bmax)+abs(bmin), amstep)
         iexp=inipl2(step)
         e=10d0**(-iexp)
         if ( ismax>0 ) then
            mantmx=inipl1(abs(bmax))
         else
            mantmx=inipl0(abs(bmax))
         end if
         if ( ismin>0 ) then
            mantmn=inipl0(abs(bmin))
         else
            mantmn=inipl1(abs(bmin))
         end if
         e=0.01d0/e
         istep=ismax*mantmx-ismin*mantmn
         mx=mod(mantmx,10)
         mn=mod(mantmn,10)
         iextra=0
         if(mx/=0 .or. mn/=0) iextra=1
         if(mx/=mn) iextra=iextra+1
         if(mod(istep,100)/=0) iextra=iextra+1
         n=max(0,-iexp+1+iextra)

         if( n<5 .and. iexp>-3 .and. iexp<4) then
             m      =  1
             ibmin1 = nint(10**n * bmin)
             ibmax1 = nint(10**n * bmax)
200          continue
             if ( mod(ibmax1-ibmin1,100)==0 .and. &
                  ibmax1/=ibmin1 ) then
                  n      = n - 1
                  ibmin1 = ibmin1 / 10
                  ibmax1 = ibmax1 / 10
                  go to 200
             end if
         else
             m=0
             e=max(mantmx,mantmn)*0.01d0
             n=inipl2(e)+2+iextra
             if ( n>5 ) n = 5
         end if
      end if
      n = n+1

1000  call erclos ( 'plcompkies' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plcompkies'

      end if  ! ( debug )

      end
