      subroutine plconvertflnum ( dnumbr, nchar, isig, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-06-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Convert real number to text with isig digits behind decimal point
!     Floating format
! **********************************************************************
!
!                       KEYWORDS
!
!     number
!     conversion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nchar, isig
      double precision dnumbr
      character (len=*) text

!     dnumbr         i    Number to be converted
!     isig           i    Number of points behind decimal point
!     nchar          o    Number of characters in text
!     text           o    String representation of dnumbr
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iexp, nchar1
      double precision rnum
      character (len=20) text1

!     iexp           Exponent
!     nchar1         Number of characters in text1
!     rnum           Real number
!     text1          String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inipl2

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPL2         compute exponent of real number
!     PLCONVERTNUM   Convert real number to text with isig digits behind
!                    decimal point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plconvertflnum' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plconvertflnum'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iexp = inipl2(dnumbr)  ! exponent of number

!     --- Part before exponent

      rnum = dnumbr*10d0**(-iexp)

!     --- Convert to text with nchar characters

      call plconvertnum ( rnum, isig, text, nchar, -1 )

      if ( debug ) then
         write(irefwr,1) 'iexp', iexp
         write(irefwr,2) 'rnum', rnum
         write(irefwr,*) 'text ', text(1:nchar)
      end if  ! ( debug )

!     --- Exponential part

      text(nchar+1:nchar+1) = 'e'
      write(text1,100) iexp
100   format ( i16 )
      text1 = adjustl(text1)            ! remove leading blanks
      nchar1 = index ( text1, ' ' )-1   ! number of non-blank characters
      text(nchar+2:nchar+1+nchar1) = text1(1:nchar1)
      nchar = nchar+1+nchar1

1000  call erclos ( 'plconvertflnum' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'text ', text(1:nchar)
         write(irefwr,*) 'End plconvertflnum'

      end if  ! ( debug )

      end
