      subroutine plelm1 ( iuser, coorlc1, ncontr, part, fillcl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 15-09-2010 Remove work arrays
!        version  3.8  date 14-02-2003 Debug statements
!        version  3.7  date 23-11-2000 Minor correction in comment
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot elements in a one or two-dimensional mesh
!     If this subroutine is changed also change plelm1par !!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     2d
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical ncontr, part, fillcl
      integer :: iuser(*)
      double precision coorlc1(*)

!     coorlc1        i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     fillcl         i    Indicates if the elements must be filled with a colour
!                         (true) or that only the boundaries must be plotted
!                         (false)
!                         If fillcl = true, the colour is given by dcolor, the
!                         boundaries get the default colour. Otherwise the
!                         colour of the boundaries is defined by dcolor
!     iuser          i    Integer user array to indicate if element group IELGRP
!                         must be skipped (iuser(ielgrp)=0) or not
!                         (iuser(ielgrp)=1)
!     ncontr         i    indication whether the elements are contracted (false)
!                         or not (true)
!     part           i    indication whether all elements are plotted (false)
!                         or that some are skipped (true)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscolr, inpelm, ilow, ielgrp, nisrf, ikelmc
      logical circle

!     circle         indication whether the element is two-dimensional
!                    (true) or not (false)
!                    when circle is true the first and last point are
!                    connected
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ilow           elements are numbered from element number ilow
!     inpelm         number of points per element
!     iscolr         Help variable to store the value of dcolor temporarily
!     nisrf          Help variable for number of extra nodes in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLELEM         Plot elements for one element group (2d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plelm1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plelm1'

      end if

      iscolr = dcolor
      ilow   = 1
      ikelmc = 1
      do ielgrp = 1, nelgrp

!     --- Determine shape number and check whether 2d element

         ishape = kmeshf(ielgrp)
         circle = .true.
         if ( ishape<=2 ) circle = .false.

!        --- Determine number of elements and number of nodes in element

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         nbounelm = sepgetbounelm(ielgrp)
         if ( kelmu>0 .and. ndim>1 .and. ishape>0 ) then

!        --- kelmu>0, Spectral elements
!            determine niedge and nisurf

            if ( nisurf==-1 ) then
               if ( ishape==3 ) then
                    nisrf = 1
               else if ( ishape==5 ) then
                    nisrf = niedge * niedge
               end if
            else
               nisrf = nisurf
            end if
            if ( ishape==1 ) then

!           --- Line mesh (Special situation)

               nbounelm  = inpelm
               circle = .false.

            else if ( ishape==3 ) then

!           --- Triangles, delete internal points

               nbounelm = inpelm - nisrf

            else if ( ishape==50 ) then

!           --- Interface elements (Special situation)

               nbounelm  = inpelm
               circle = .false.

            else

!           --- Quadrilaterals

               if ( nisrf==0 ) then

!              --- Row by row, no internal points

                  nbounelm = -1

               else

!              --- Internal points

                  nbounelm = -2

               end if

            end if

         end if

         if ( .not. part ) then

!        --- Skip element groups if iuser(ielgrp) # 1

           if ( iuser(ielgrp)/=1 ) goto 100

         end if
         if ( iscolr>0 .or. fillcl ) dcolor = iscolr + ielgrp

         call plelem ( ncontr, coorlc1, kmeshc(ikelmc), ilow, inelem, &
                       inpelm, nbounelm, circle, ndim, fillcl )

!        --- Raise number of elements and start position

100      ilow   = ilow   +          inelem
         ikelmc = ikelmc + inpelm * inelem

      end do
      dcolor = iscolr

      if ( debug ) write(irefwr,*) 'End plelm1'
      call erclos ( 'plelm1' )

      end
