      subroutine plhgl6 ( ichois, sx, sy )
! ======================================================================
!
!        programmer     Eef Hartman
!        version 1.1    date   20-04-1999 Inlcude commons
!        version 1.0    date   06-12-1991
!
!
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!    HPGL dependent calls of plotting subroutines
!    except for texts, special symbols and colours
!
!    machine dependent subroutine
!
! ********************************************************************
!
!                      KEYWORDS
!
!   machine_dependent
!   plot
!   hpgl
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulemach
      use sepmoduleplot
      implicit none
      integer ichois
      real sx, sy
!
!    ichois   i   The use of the various parameters depends on ichois.
!                 Possibilities:
!       1         Call plot start subroutine
!                 When X=1 small plotting paper is used, when X=2 wide
!                 plotting paper is used. (not supported in some versions)
!                 The length of the plot is given in Y (centimeters)
!       2         Plot with pen down to point (x,y)
!       3         Plot with pen up to point (x,y)
!       4         Change origin (will disappear in next major revision!)
!                 (x,y) are the values to be added to the current origin
!       5         Close plot data set
!                 When X=-1 the number for the next (w or u) output file
!                 will be reset to 1 again
!
!    sx       i   Single precision value of x
!
!    sy       i   Single precision value of y
!
! ********************************************************************
!
!                      COMMON BLOCKS
!

! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer pltnum, irefgl, ios
      logical check
      character*30 filenm
      save pltnum, irefgl
      data pltnum /0/
!
!    check        Indicates if a file exists (true) or not (false)
!
!    ios          File i/o status parameter
!
!    irefgl       file reference number for hpgl file
!
!    filenm       character field to build filename in
!
!    pltnum       file number of the plot last made, initialized to 0
!
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS:  Resets old name of previous subroutine of higher level
!     ERDUMP:  Print text from error message file
!     EROPEN:  Produces concatenated name of local subroutine
!     ERRCHR:  Put characters in error message
!     ERRINT:  Put integers in error message
!     ERRSUB:  Error messages
!     INSTOP:  Stop routine
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   1297:  Value of ichois not supported
!   1317:  Error in opening of the plot
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!    If ( ichois=1 and first picture ) then
!       delete all preceding plot files
!    if ( ichois=1 ) then
!       open plot
!       write heading of file
!    else if ( ichois=2 ) then
!       compute to where the "pen" must be moved to
!       write this position to the file
!    else if ( ichois=3 ) then
!       compute to where the "pen" must be drawn to
!       write this position to the file
!    else if ( ichois=5 ) then
!       close the plot
!
! ======================================================================
!

!    *  Make a choice depending on ichois

      go to ( 100, 200, 300, 1000, 500 ), ichois

!        *  Incorrect value of ichois

         call eropen ( 'plhgl6' )
         call errint ( ichois, 1 )
         call errsub ( 1297, 1, 0, 0 )
         call erclos ( 'plhgl6' )
         call instop
         go to 1000

! *  ichois = 1, open plot

100      if ( pltnum.eq.0 ) then

!    *  First call, *  delete existing files

            irefgl = abs(irefpl) + 1
110         pltnum=pltnum+1
            write ( filenm, 120 ) 'sephpgl', pltnum
120         format ( a, '.', i3.3 )

            inquire ( file=filenm, exist=check )
            if ( check ) then
               open ( unit=irefgl, file=filenm, status='old' )
               close ( unit=irefgl, status='delete' )
               go to 110
            end if
            pltnum = 0

         end if

         pltnum=pltnum+1
         write ( filenm, 120 ) 'sephpgl', pltnum
         open ( unit=irefgl, file=filenm, form='formatted', iostat=ios )
         if ( ios.ne.0 ) then

!     *  ios # 0

            call eropen ( 'plhgl6' )
            call errint ( ios, 1 )
            call errchr ( filenm, 1 )
            call errsub ( 1317, 1, 0, 1 )
            call erclos ( 'plhgl6' )
            call instop
            go to 1000

         end if

            if ( sx.le.2.0 .and. sx.ge.2.0 .or. &
     &           posdev(2:2).eq.'b' ) then
               write (irefgl,130) 38.0,24.9
            else if ( posdev(2:2).eq.'0' ) then
               write (irefgl,130) 100.0,76.0
            else
               write (irefgl,130) 24.9,18.0
            end if
130         format (' insc0,' ,f5.1, ',0,', f4.1, 'sp1pu;' )
         go to 1000

!  *  ichois = 2,  draw line:

200         write (irefgl,210) mod(dcolor,8)+1,sx,sy
210         format (' sp', i1, 'pd', f6.2, ',', f6.2, ';' )
         go to 1000

!  *  ichois = 3,  move pen:

300         write (irefgl,310) sx,sy
310         format (' pu', f6.2, ',', f6.2, ';' )
         go to 1000

!  *  ichois = 5,  end of plot:

500         write (irefgl,510)
510         format (' sppg;')
            close (unit=irefgl)
            if ( sx.le.-1.0 .and. sx.ge.-1.0 ) pltnum = 0
         go to 1000

1000  return
      end
