      subroutine pllgcl ( levels, nlevel, icolor, itypsc, ityplg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 27-09-2006 Make text size variable
!        version  2.7  date 01-09-2006 Make text levels variable
!        version  2.6  date 14-06-2006 Reverse sequence of legends
!
!   copyright (c) 1990-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        Plot subroutine to plot the levels for a coloured contour plot, &
!        as well as the scale factors
!        Help subroutine for subroutine PLOTC1/3
! **********************************************************************
!
!                       KEYWORDS
!
!     colour
!     contour_level
!     legenda
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlevel, icolor(nlevel+1), ityplg, itypsc
      double precision levels(nlevel)

!    icolor    i   Array containing the colours of the contour levels
!    ityplg    i   Choice parameter for type of legenda
!                  Possible values:
!                  1:  The legenda are not plotted
!                  2:  The colours are plotted vertical including all values
!                  3:  The colours are plotted horizontal only minimum
!                      and maximum values
!    itypsc    i   Indicates if scales must be plotted (1) or not (2)
!    levels    i   Contour levels. A list of these values must be plotted
!    nlevel    i   Number of contour levels.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=11) charac
      double precision xp, yp, minlev, maxlev, deltxp
      integer i, npolyg, ichois, nstep, nchar
      real polyg(2,4)
      logical floatp

!     charac         Help variable to store a number in character form
!     deltxp         Spacing in horizontal colour legenda
!     floatp         Indication if numbers must be plotted in fixed or in
!                    floating precision
!     i              General loop variable
!     ichois         Choice parameter for subroutine plafp8
!     maxlev         Maximal absolute value larger than zero
!     minlev         Minimal absolute value larger than zero
!     nchar          Number of characters in a string
!     npolyg         Defines the number of nodes necessary to define a polygon
!     nstep          Help parameter to indicate if all colours must be plotted
!                    (1) or only one of two (2)
!     polyg          Contains the co-ordinates of the polygon to be plotted
!                    polyg(1,.) defines the x-co-ordinate
!                    polyg(2,.) defines the y-co-ordinate
!     xp             Plotting x-position in cm's
!     yp             Plotting y-position in cm's
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLAFP8         Plot coloured levels (Low level plotting subroutine)
!     PLSCAL         Plot scales
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pllgcl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pllgcl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Plot scales:

      if ( itypsc==1 ) call plscal
      if ( ityplg==1 ) go to 1000

!     --- Plot levels

      ichois = 1
      npolyg = 4
      nstep = 1
      if ( nlevel>30 ) nstep = 2
      if ( ityplg==2 ) then

!     --- ityplg = 2, plot text LEVELS

         xp = max ( ale + 5.3d0, 16.3d0 )
         yp = 3.55d0+nlevel*0.5d0/nstep
         nchar = iinputpst(101)
         call plafp7 ( xp, yp, textid(2), 0d0, rinputpst(44), nchar )
         xp = max ( ale + 5.0d0, 16d0 )
         yp = yp - 0.6d0

      end if

!     --- First plot colours

      if ( ityplg==3 ) then

!     --- ityplg = 3, horizontal colours

         xp = .5d0
         yp = 1d0
         deltxp = 16d0/nlevel
         do i = nlevel+1, 1, -nstep
            polyg(1,1) = sngl(xp)
            polyg(1,2) = sngl(xp+deltxp)
            polyg(1,3) = sngl(xp+deltxp)
            polyg(1,4) = sngl(xp)
            xp = xp + deltxp
            polyg(2,1) = sngl(yp)
            polyg(2,2) = sngl(yp)
            polyg(2,3) = sngl(yp+0.5d0)
            polyg(2,4) = sngl(yp+0.5d0)
            if ( nlevel==21 .and. i==nlevel+1 ) then
               call plafp8 ( ichois, icolor(nlevel), npolyg, polyg )
            else
               call plafp8 ( ichois, icolor(i), npolyg, polyg )
            end if  ! ( i==nlevel+1 )

         end do
         xp = 0.5d0
         yp = 0.2d0
         call plafp7 ( xp, yp, 'Min, Max:', 0d0, 0.5d0, 9 )
         write ( charac(1:10), 120 ) levels(1)
         call plafp7 ( xp+3.5d0, yp, charac, 0d0, 0.5d0, 10 )
         write ( charac(1:10), 120 ) levels(nlevel)
         call plafp7 ( xp+10d0, yp, charac, 0d0, 0.5d0, 10 )
120      format(ss,1pe10.3e2)

      else

!     --- ityplg = 2, vertical colours

         do i = nlevel+1, 1, -nstep
            polyg(1,1) = sngl(xp)
            polyg(1,2) = sngl(xp+1.0d0)
            polyg(1,3) = sngl(xp+1.0d0)
            polyg(1,4) = sngl(xp)
            yp = yp - 0.5d0
            polyg(2,1) = sngl(yp)
            polyg(2,2) = sngl(yp)
            polyg(2,3) = sngl(yp+0.5d0)
            polyg(2,4) =sngl( yp+0.5d0)
            npolyg = 4
            if ( nlevel==21 .and. i==nlevel+1 ) then
               call plafp8 ( ichois, icolor(nlevel), npolyg, polyg )
            else
               call plafp8 ( ichois, icolor(i), npolyg, polyg )
            end if  ! ( i==nlevel+1 )

         end do
         yp = yp + (nlevel+1)*0.5d0/nstep
         if ( nstep==2 .and. mod(nlevel,2)==0 ) yp = yp + 0.25d0

!        --- Next plot levels
!            Check if levels should be plotted in fixed or floating format

         minlev = rinfin
         maxlev = -rinfin
         do i = 1, nlevel
            if ( abs(levels(i))>0d0 ) then
               minlev = min ( minlev, abs(levels(i)) )
               maxlev = max ( maxlev, abs(levels(i)) )
            end if
         end do
         floatp = .false.
         if ( minlev<1d-3 .or. minlev>1d3 .or. maxlev<1d-3 .or. &
              maxlev>1d3 .or. abs(maxlev-minlev)<1d-3 ) &
            floatp = .true.

         if ( floatp ) then

!        --- Plot in floating precision

            do i = nlevel, 1, -nstep
               yp = yp - 0.5d0
               write ( charac(1:11), 220 ) levels(i)
220            format(ss,1pe11.4e2)
               call plafp7 ( xp+1.5d0, yp, charac, 0d0, rinputpst(45), 11 )
            end do

         else

!        --- Plot in fixed precision

            do i = nlevel, 1, -nstep
               yp = yp - 0.5d0
               write ( charac(1:10), 280 ) levels(i)
280            format(f10.3)
               call plafp7 ( xp+1.5d0, yp, charac, 0d0, rinputpst(45), 10 )
            end do
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pllgcl'

      end if  ! ( debug )
      call erclos ( 'pllgcl' )

      end
