      subroutine pllgcn( levels, nlevel, itypsc, ityplg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 27-09-2006 Make text size variable
!        version  2.2  date 01-09-2006 Make text levels variable
!        version  2.1  date 14-06-2006 Reverse sequence of legends
!
!   copyright (c) 1990-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!        Plot subroutine to plot the levels for a contour plot, &
!        as well as the scale factors
!        Help subroutine for subroutine PLOTC1/3
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     legenda
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlevel, itypsc, ityplg
      double precision levels(nlevel)

!    ityplg    i   Choice parameter for type of legenda
!                  Possible values:
!                  1:  The legenda are not plotted
!                  2:  The colours are plotted vertical including all values
!                  3:  The colours are plotted horizontal only minimum
!                      and maximum values
!    itypsc    i   Indicates if scales must be plotted (1) or not (2)
!    levels    i   Contour levels. A list of these values must be plotted
!    nlevel    i   Number of contour levels.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=14) charac
      double precision xp, yp, minlev, maxlev
      integer i, nchar
      logical floatp

!     charac         Help variable to store a number in character form
!     floatp         Indication if numbers must be plotted in fixed or in
!                    floating precision
!     i              General loop variable
!     maxlev         Maximal absolute value larger than zero
!     minlev         Minimal absolute value larger than zero
!     nchar          Number of characters in a string
!     xp             Plotting x-position in cm's
!     yp             Plotting y-position in cm's
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLSCAL         Plot scales
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pllgcn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pllgcn'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Plot scales:

      if ( itypsc==1 ) call plscal
      if ( ityplg==1 ) go to 1000

!     --- Plot levels

      xp = max ( ale + 3.3d0, 16.3d0 )
      yp = 3.3d0+nlevel*0.5d0
      nchar = iinputpst(101)
      call plafp7 ( xp, yp, textid(2), 0d0, rinputpst(44), nchar )

      xp = max ( ale + 3.0d0, 16d0 )
      yp = yp - 0.6d0-(nlevel+1)*0.5d0

!     --- Check if levels should be plotted in fixed or floating format

      minlev = rinfin
      maxlev = -rinfin
      do i = 1, nlevel
         if ( abs(levels(i))>0d0 ) then
            minlev = min ( minlev, abs(levels(i)) )
            maxlev = max ( maxlev, abs(levels(i)) )
         end if
      end do
      floatp = .false.
      if ( minlev<1d-3 .or. minlev>1d3 .or. maxlev<1d-3 .or. &
           maxlev>1d3 ) floatp = .true.
      charac(3:4) = ': '
      if ( floatp ) then

!     --- Plot in floating precision

         do i = 1, nlevel
            write ( charac,110 ) i
110         format(i2)
            write ( charac(5:14), 120 ) levels(i)
120         format(ss,1pe10.3e2)
            call plafp7 ( xp, yp+i*0.5d0, charac, 0d0, rinputpst(45), 14 )
         end do
      else

!     --- Plot in fixed precision

         do i = 1, nlevel
            write ( charac,110 ) i
            write ( charac(5:14), 180 ) levels(i)
180         format(f10.3)
            call plafp7 ( xp, yp+i*0.5d0, charac, 0d0, rinputpst(45), 14 )
         end do

      end if

1000  call erclos ( 'pllgcn' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pllgcn'

      end if  ! ( debug )

      end
