      subroutine plot2dmesh ( kbndpt, nbndpt, coor, kmeshc, npelm, &
                              nelem, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-09-2010 New call to plelm1
!        version  1.0  date 18-02-2009
!
!   copyright (c) 2009-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot 2d submesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt, npelm, nelem, kbndpt(nbndpt), kmeshc(npelm,nelem), &
              npoint
      double precision coor(2,npoint)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nelem          i    Number of elements
!     npelm          i    Number of points per element
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikader, imark, i, inode
      double precision plotfm, yfact

!     i              Counting variable
!     ikader         Help variable to store jkader temporarily
!     imark          Indicates if markers must be set in the plot (>0)
!                    or not (0)
!     inode          Node sequence number
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELEM         Plot elements for one element group (2d)
!     PLPOIN         plot nodal points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plot2dmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plot2dmesh'
         write(irefwr,1) 'nbndpt, npoint, nelem, npelm', &
                          nbndpt, npoint, nelem, npelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      plotfm = 15d0
      yfact = 1d0

!     --- compute xmin, xmax, ymin, ymax, aleng and width

      xmin=rinfin
      ymin=xmin
      xmax=-xmin
      ymax=xmax
      do i = 1, nbndpt
         inode = kbndpt(i)
         xmin = min(xmin,coor(1,inode))
         xmax = max(xmax,coor(2,inode))
         ymin = min(ymin,coor(1,inode))
         ymax = max(ymax,coor(2,inode))
      end do  ! i = 1, nbndpt
      call pl0000 ( plotfm, yfact )

!     --- Store kader value

      ikader = jkader
      jkader = -1

!     --- Start picture

      call plafep(1, aleng, width, 0d0, 0d0, 0)
      jkader =  ikader
      imark  =  jmark
      jmark  =  2

!     --- Plot all the elements

      call plelem ( .true., coor, kmeshc, 1, nelem, npelm, &
                    npelm, .true., 2, .false. )

!     --- plot nodal points

      call plpoin ( coor, npoint, kmeshc, 2 )

!     --- Reset jmark

      jmark  =  imark

!     --- Close plot dataset

      call plafep(2, 0d0, 0d0, 0d0, 0d0, 0)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plot2dmesh'

      end if  ! ( debug )
      call erclos ( 'plot2dmesh' )

      end
