      subroutine plotaxis
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-05-2014 Extra debug
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot the axis of a function plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision axhulp, ayhulp

!     axhulp         Defines the step along the x-axis
!     ayhulp         Defines the step along the y-axis
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTXAXIS      plot the x-axis of a function plot
!     PLOTYAXIS      plot the y-axis of a function plot
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotaxis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotaxis'
         write(irefwr,1) 'nstepx, nstepy', nstepx, nstepy
         write(irefwr,2) 'axmin, axmax, aymin, aymax', &
                          axmin, axmax, aymin, aymax
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- definition of window

      axhulp = (axmax-axmin)/nstepx
      ayhulp = (aymax-aymin)/nstepy

      if ( debug ) write(irefwr,2) 'axhulp, ayhulp', axhulp, ayhulp

!     --- draw x-axis

      call plotxaxis ( axhulp )

!     --- draw y-axis

      call plotyaxis ( ayhulp )

1000  call erclos ( 'plotaxis' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotaxis'

      end if  ! ( debug )

      end
