      subroutine plotcolcont ( contln, ncntln, indcol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual filled colored level plot
!     Called by prplotcol
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     color
!     driving_subroutine
!     plot
!     plot_contour
!     contour_level
!     level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ncntln, indcol(*)
      double precision, intent(in) :: contln(*)

!     contln         i    Contains the contour levels
!     indcol         i    Contains the colors for the levels
!     ncntln         i    number of contour levels
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nline, kline(1), iplotc
      logical :: newkmeshc

!     iplotc         Color of the contour lines, 0 = none
!     kline          Array for extra line elements
!     newkmeshc      if true a new temporary kmeshc has been created
!     nline          number of line elements in kline
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLCON1         Actual contour plot for colored levels
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotcolcont' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotcolcont'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nline = 0
      iplotc = 0


      if ( associated(kmeshctmp) )then

!     --- subdivided mesh

         inelem = size ( kmeshctmp )/3
         newkmeshc = .true.

      else

!     --- original mesh

         inelem = size ( kmeshc )/3
         kmeshctmp => kmeshc
         newkmeshc = .false.

      end if  ! ( indkmeshc>0 )

      call plcon1 ( kmeshctmp, coor, usolt, &
                    contln, ncntln, inelem, indcol, iplotc, nline, &
                    kline, npoint )

      if ( newkmeshc ) nullify(kmeshctmp)
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotcolcont'

      end if  ! ( debug )
      call erclos ( 'plotcolcont' )

      end subroutine plotcolcont
