      subroutine plotcompscal1 ( exp1, valmin, valmax, minout, &
                                 maxout, step, ndec, exp3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute exponent and step for scales
!
! **********************************************************************
!
!                       KEYWORDS
!
!     exponent
!     scale
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer exp1, ndec, exp3
      double precision valmin, valmax, minout, maxout, step

!     exp1           o    Exponent to be used for these values
!     exp3           o    Exponent corresponding to valmax-valmin
!     maxout         o    Scaled valmax
!     minout         o    Scaled valmin
!     ndec           o    Number of digits to be used in step
!     step           o    Step along axis
!     valmax         i    Maximum value of function
!     valmin         i    Minimum value of function
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer exp2, num
      double precision diff, mult, eps

!     diff           Difference between valmax and valmin
!     eps            Local accuracy
!     exp2           Help exponent
!     mult           Mulitplication factor
!     num            Integer representation of scaled step times 100
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inipl1, inipl2

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPL1         find largest "nice" number larger than real number
!     INIPL2         compute exponent of real number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotcompscal1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotcompscal1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      diff = valmax - valmin   !difference
      exp1 = inipl2 ( diff )   ! compute exponent
      exp3 = exp1
      if ( debug ) write(irefwr,1) 'exp1', exp1

      if ( exp1>=-4 .and. exp1<=4 ) then

!     --- exponent between -3 and 4, do not use exponent

         exp2 = exp1
         exp1 = 0
         minout = valmin
         maxout = valmax

      else

!     --- exponent less than -3 or more than 4, use exponent

         mult = 10d0**(-exp1)
         diff = diff*mult
         minout = valmin*mult
         maxout = valmax*mult
         exp2 = inipl2 ( diff )   ! compute exponent

         if ( debug ) write(irefwr,2) 'diff', diff

      end if  ! ( exp1>=-3 .and. exp1<=4 ) then

      if ( debug ) write(irefwr,1) 'exp2', exp2

      eps = 1d-3

      if ( exp2<0 ) then

!     --- Numbers less than 0, but more than 0.001

         mult = 10d0**(-exp2)

         if ( diff*mult>6.001d0 ) then
            step = 1d0/mult
            ndec = -exp2
         else
            step = 0.5d0/mult
            ndec = -exp2+1
         end if  ! ( diff*mult>=6d0 )

         if ( debug ) then
            write(irefwr,1) 'exp1, exp2, ndec', exp1, exp2, ndec
            write(irefwr,2) 'mult, diff, step', mult, diff, step
         end if  ! ( debug )

      else if ( diff<=1.2d0+eps ) then

!     --- diff<=1.2

         ndec = exp2+1
         step = 0.1d0
         if ( debug ) then
            write(irefwr,*) 'diff<=1.2'
            write(irefwr,2) 'step', step
         end if  ! ( debug )

      else if ( diff>12d0+eps ) then

!     --- diff>12, i.e. step more than 1

         exp2 = inipl2 ( diff )   ! compute exponent
         num = inipl1(diff)/100
         mult = 10d0**(exp2)
         step = num/mult
         if ( step<=6d0 ) then
            step = 0.5d0*mult
         else
            step = mult
         end if  ! ( step<=3 )
         ndec = 0
         if ( debug ) then
            write(irefwr,*) 'diff>12'
            write(irefwr,1) 'exp2, num', exp2, num
            write(irefwr,2) 'step, mult', step, mult
         end if  ! ( debug )

      else

!     --- Make nice value:

         num = inipl1(diff)/100
         if ( num>6 ) then
            step = 1d0
            ndec = max(0,-exp2)
         else
            step = 0.5d0
            ndec = max(0,-exp2+1)
         end if  ! ( num>6 )
         if ( debug ) then
            write(irefwr,*) 'other case'
            write(irefwr,1) 'num', num
         end if  ! ( debug )

      end if  ! ( corrdiff<=1.2d0+eps ) then
      if ( debug ) write(irefwr,2) 'minout, maxout', minout, maxout

1000  call erclos ( 'plotcompscal1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndec', ndec
         write(irefwr,2) 'minout, maxout, step', minout, maxout, step
         write(irefwr,*) 'End plotcompscal1'

      end if  ! ( debug )

      end
