      subroutine plotfillcolors ( incol, indcol, ncont, jindcl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array with sequence numbers of colors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     color
!     plot
!     contour_level
!     level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: indcol(*), ncont, jindcl
      integer, intent(out) :: incol(ncont+1)

!     incol          o    Array with the colors of the levels
!     indcol         i    This array is only used when isubr=2 and jsmoot>0
!                         If JSMOOT=1 it is a variable length array of length:
!                         number of contour levels plus 5
!                         indcol must be filled by the user with the colors
!                         of the various contour levels in increasing sequence.
!                         indcol must be filled from position 6
!                         If JSMOOT=2 it is a variable length array of length 6
!                         In INDCOL(6) the user must store the minimum color
!                         required
!                         If JSMOOT=3 it is a variable length array of length 7
!                         The minimum color must be stored in INDCOL(6) the
!                         maximum one in INDCOL(7)
!                         If the range of the colors is less than the available
!                         range of colors the color numbering is repeated
!     jindcl         i    indicates how array indcol must be filled for routine
!                         plotc2.
!                         Possible values:
!                         0: The colors with numbers 1 to 1+ncntln are used
!                         1: The user defines the color numbers explicitly
!                         2: The user defines the minimum color
!                         3: The user defines the minimum and maximum color
!                         See indcol
!     ncont          i    Number of contour levels
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: dummy(1)

!     dummy          Dummy parameter
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfillcolors' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfillcolors'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jindcl==1 ) then

!     --- indcol given by user, fill by copying

         incol(1:ncont+1) = indcol(6:6+ncont)

      else if ( jindcl==0 ) then

!     --- compute standard array indcol in incol

         do i = 1, ncont+1
            incol(i) = i
         end do

      else if ( jindcl==2 ) then

!     --- Minimum color given
!         compute standard array indcol in incol from indcol(6)

         call prinlg ( 'indcol', 6, dummy, indcol )
         do i = 1, ncont+1
            incol(i) = i+indcol(6)-1
         end do

      else

!     --- Minimum and maximum color given
!         compute standard array indcol in incol from indcol(6) and indcol(7)

         call prinlg ( 'indcol', 7, dummy, indcol )
         do i = 1, ncont+1
            incol(i) = indcol(6)+mod(i-1,indcol(7)-indcol(6)+1)
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotfillcolors'

      end if  ! ( debug )
      call erclos ( 'plotfillcolors' )

      end subroutine plotfillcolors
