      subroutine plotintszc ( zc, x, y, z, xi, yi, nintersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of triangle with vertices stored in x, y and z
!     with plane z = zc and store the intersections in xi and yi
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     obstacle
!     grid
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision zc, x(3), y(3), z(3), xi(*), yi(*)
      integer nintersect

!     nintersect     o    Number of intersections found
!     x              i    x co-ordinates of points
!     xi             o    x-coordindates of interpolation
!     y              i    y co-ordinates of points
!     yi             o    y-coordindates of interpolation
!     z              i    z co-ordinates of points
!     zc             i    z-coordinate of the midth-point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision alpha, beta

!     alpha          factor
!     beta           factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The line z is constant is intersected with the triangle with coordinates
!     stored in x, y and z
!
!     The sides of the triangle are defined by:
!
!     x = x_1 + alpha (x_2-x_1) + beta (x_3-x_1)
!
!     alpha = 0, gives the side x1, x3  (0<= beta <= 1)
!     beta = 0, gives the side x1, x2   (0<= alpha <= 1)
!     alpha+beta = 1, gives the side x2, x3
!        In this case we have
!        x = x_1 + alpha (x_2-x_1) + (1-alpha) (x_3-x_1)
!        hence
!        x = x_3 + alpha (x_2-x_3)   (0<= alpha <= 1)
!
!     Intersection with two sides at most is possible
!     The intersection is only computed if the side is not parallel to z is cons
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nintersect = 0
      if ( abs(z(3)-z(1)).gt.sqreps ) then

!     --- z(3)#z(1)
!         Possible intersection with line alpha = 0

         beta = (zc - z(1))/(z(3)-z(1))
         if ( beta.ge.0d0 .and. beta.le.1d0 ) then

!        --- intersection found

            nintersect = nintersect+1
            xi(nintersect) = x(1)+beta*(x(3)-x(1))
            yi(nintersect) = y(1)+beta*(y(3)-y(1))

         end if

         if ( abs(z(2)-z(1)).gt.sqreps ) then

!        --- Intersection with line beta = 0

            alpha = (zc - z(1))/(z(2)-z(1))
            if ( alpha.ge.0d0 .and. alpha.le.1d0 ) then

!           --- intersection found

               nintersect = nintersect+1
               xi(nintersect) = x(1)+alpha*(x(2)-x(1))
               yi(nintersect) = y(1)+alpha*(y(2)-y(1))

            end if

         end if
         if ( nintersect.eq.1 .and. abs(z(2)-z(3)).gt.sqreps ) then

!        --- nintersect = 1, check with line alpha+beta=1

            alpha = (zc - z(3))/(z(2)-z(3))
            if ( alpha.ge.0d0 .and. alpha.le.1d0 ) then

!           --- intersection found

               nintersect = nintersect+1
               xi(nintersect) = x(3)+alpha*(x(2)-x(3))
               yi(nintersect) = y(3)+alpha*(y(2)-y(3))

            end if

         end if

      else

!     --- Intersection with line beta = 0

         alpha = (zc - z(1))/(z(2)-z(1))
         if ( alpha.ge.0d0 .and. alpha.le.1d0 ) then

!        --- intersection found

            nintersect = nintersect+1
            xi(nintersect) = x(1)+alpha*(x(2)-x(1))
            yi(nintersect) = y(1)+alpha*(y(2)-y(1))

         end if
         if ( nintersect.eq.1 .and. abs(z(2)-z(3)).gt.sqreps ) then

!        --- nintersect = 1, check with line alpha+beta=1

            alpha = (zc - z(3))/(z(2)-z(3))
            if ( alpha.ge.0d0 .and. alpha.le.1d0 ) then

!           --- intersection found

               nintersect = nintersect+1
               xi(nintersect) = x(3)+alpha*(x(2)-x(3))
               yi(nintersect) = y(3)+alpha*(y(2)-y(3))

            end if

         end if

      end if

      end
