      subroutine plotln1 ( iplots, rplots, ivalue, values, textx, texty )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 17-03-2011 Debug statements
!        version  2.0  date 12-10-1999 New parameter list
!        version  1.0  date 22-11-1997
!
!   copyright (c) 1997-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Intersects a two-dimensional mesh with a line and interpolates the
!     solution along this line
!     The interpolated function is plotted by means of subroutine PLOTFN
!     plotln1 replaces the subsequent calls of INTP2D and PLOTFN
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_function
!     intersection
!     intersection
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplots(*), ivalue(*), number
      double precision values(*), rplots(*)
      character *(*) textx, texty

!     iplots     i    Integer input array for subroutine PLOTFNBF
!                     IPLOTS must be filled as follows:
!                     1:   Highest entry that has been filled by the user
!                     2:   ipict  sequence number of the function to be plotted
!                                 Must be between 1 and npict
!                                 (inclusive end-points)
!                          Default value: 1
!                     3:   npict  number of functions to be plotted in one
!                                 picture
!                          Default value: 1
!                     4:   ichint corresponds to the parameter ichois in intp2d
!                          which indicates how the line is defined.
!                          Possibilities:
!                          1:  The user defines a line by defining a starting
!                              point and an angle with respect to the positive
!                              x-axis under which the line must be drawn in the
!                              positive x-direction.
!                              The values of the starting point resp. the angle
!                              must be stored into array VALUES as follows:
!                              VALUES (1) = xstart
!                              VALUES (2) = ystart
!                              VALUES (3) = angle of rotation with respect to
!                                           pos. x-axis
!                          2:  See 1, however the solution must be multiplied by
!                              a factor f, that is stored in values(4)
!                          Default value: 1
!                     5:   isymbl  Choice parameter indicating with what symbols
!                                 the functions are plotted. Isymbl may be
!                                 different for every call of plotfnbf.
!                                 Possibilities are:
!                                  0     Only straight lines are plotted
!                                  1-10  Points will be connected with a line
!                                        and marked with a symbol,
!                                        system depended
!                                 11-20  Only the same symbols are used,
!                                        no lines
!                          Default value: 0
!                     6:   jpicho   Indicator to use a second y-scale along the
!                                 right-hand side
!                                  0  Only one y-scale is used.
!                                 >0  A second scale will be made
!                          Default value: 0
!                     7:   number  the number th degree of freedom in each nodal
!                                 point is plotted
!                          Default value: 1
!                     8:   itypescales  type of scales used along each of the
!                                 axis. Is subdivided into 2 or 3 parts
!                                 according to itypescales = itype1+10*itype2
!                                 where itype1 refers to the x-axis, itype2 to
!                                 the y-axis
!                                 The following values for itypex are available
!                                  0  Linear scale
!                                  1  Logarithmic scale (base 10)
!                          Default value: 0
!     ivalue          Not yet used
!     rplots     i    Real input array for plot subroutine
!                     RPLOTS must be filled as follows:
!                     1:  plotfm   length of the plot in the x-direction in cm.
!                                  when plotfm<0 the length is equal to
!                                  -plotfm * (xmax-xmin)
!                     2:  height   height of the plot
!     textx      i    text to write along the x-axis
!     texty      i    text to put along (one of the) y-axis(es)
!     values     i    Double precision array in which the real information
!                     concerning the line is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icross, ichint, iskip, numb, idummy(1)
      double precision dummy(1), height, plotfm
      save icross

!     dummy          Dummy parameters that are not used
!     height         Height of a plot, when <0 then in cm, else
!                    dimensionless
!     ichint         Parameter ichpict as arising from ichois
!     icross         Array of length 5 in which the reference to the
!                    interpolation of the solution in array IBUFFR is stored
!     idummy         Dummy parameters that are not used
!     iskip          Input parameter for subroutine INTP2D.
!                    Indicates if submeshes must be skipped (1) or not
!     numb           Parameter number computed back for a complex field
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTFN1        Plot function
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPINTP2D      Interpolate 2D region with straight line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotln1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotln1'
         call prinin ( iplots, iplots(1), 'iplots' )
         call prinrl ( values, 3, 'values' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ichint = 1
      number = 1
      if ( iplots(1)>=4 ) ichint = iplots(4)
      if ( iplots(1)>=7 ) number = iplots(7)
      iskip  = 0
      iwarn  = 0

      if ( ks(isolut)%compl==0 ) then
         numb = number
      else
         numb = (number+1)/2
      end if
      call sepintp2d ( ichint, ivalue(1), values, numb, &
                       icross, iskip, idummy )
      if ( iwarn/=0 ) go to 1000

!     --- determine yfact for call of plotfn

      plotfm = rplots(1)
      height = rplots(2)
      if ( height>0d0 ) then
         rplots(2) = -height*plotfm
      else if ( height<0d0 ) then
         rplots(2) = -height
      end if

      if ( debug ) call prinrl ( rplots, 2, 'rplots' )

!     --- reset parameter ichois; parameter number is equal to 1 now

      if ( iplots(1)>=4 ) iplots(4) = 0
      call plotfn1 ( iplots, rplots, icross, &
                     idummy, dummy, dummy )
      rplots(2) = height
      if ( iplots(1)>=4 ) iplots(4) = ichint

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotln1'

      end if  ! ( debug )
      call erclos ( 'plotln1' )

      end
