      subroutine plotmarkels ( imarkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot information from array imarkelem
!     defining the status of elements in case of an obstacle
!     2d only
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     obstacle
!     2d
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: imarkelem(*)

!     imarkelem      i    See subroutine plotmarkelnods
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscolr, inpelm, ielgrp, nboun, ikelmc, ielem, iielem

!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iscolr         Help variable to store the value of dcolor temporarily
!     nboun          Number of boundary points at an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTELEM       Plot one element (2d) with color dcolor
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotmarkels' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotmarkels'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iscolr = dcolor

!     --- First the elements inside the obstacle are plotted

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( imarkelem(ielem)==0 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, 0, .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

!     --- Then the elements outside the boundary are plotted

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( imarkelem(ielem)==1 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, 1, .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

!     --- Then the active elements

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( imarkelem(ielem)==2 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, 10, .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

!     --- Finally the elements that are removed

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( imarkelem(ielem)==3 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, 20, .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

      dcolor = iscolr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotmarkels'

      end if  ! ( debug )
      call erclos ( 'plotmarkels' )

      end
