      subroutine plotnowbnd ( nowbnd, nloop, imxlps, layer, coor, &
                              text1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the actual boundary during the creation of the mesh in a specific
!     iteration
!     The boundary is defined by array nowbnd, which is only defined for mshpav
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     boundary
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imxlps, nowbnd(imxlps,*), nloop, layer
      double precision coor(4,*)
      character *(*) text1

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     imxlps         i    maximum loops in nowbnd in paver, also first dimension
!                         of nowbnd
!     layer          i    Present layer sequence number
!     nloop          i    number of closed boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     text1          i    Text to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nbndpt, node, j, lentext
      character *100 text

!     i              Counting variable
!     j              Counting variable
!     lentext        Number of characters in text
!     nbndpt         Number of nodal points in boundary, length of array
!                    kbndpt
!     node           Node number
!     text           String variable containing some text to be plotted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open plot
!     Plot each boundary
!     Close plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotnowbnd' )
      debug = .false.
      if ( nloop.le.0 .and. mod(jtimes,10).ne.1 .and. &
           mod(jtimes,10).ne.3 ) go to 1000
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotnowbnd'
         write(irefwr,*) 'nloop ', nloop

      end if

!     --- Open plot

      call pl0000 ( 15d0, 1d0 )
      call plafep( 1, aleng, width, 1d0, 15d0, 1 )

!     --- Plot each boundary

      do i = 1, nloop
         nbndpt = nowbnd(i,1)
         if ( debug ) write(irefwr,*) 'Loop ', i,'  nbndpt = ', nbndpt
         if ( nbndpt.le.0 ) go to 100

!        --- Go to first node with pen up

         node = abs(nowbnd(i,2))
         call plafep( 4, coor(1,node), coor(2,node), 0d0, 0d0, 0 )
         call plafep( 6, coor(1,node), coor(2,node), 0d0, 0d0, 0 )
         call plafep ( 7, coor(1,node), coor(2,node), -0.06d0, 0.06d0, &
                       node )
         if ( debug ) &
            write(irefwr,*) 'node ', node, coor(1,node), coor(2,node)

!        --- Go to all other nodes with pen down

         do j = 2, nbndpt
            node = abs(nowbnd(i,j+1))
            call plafep( 5, coor(1,node), coor(2,node), 0d0, 0d0, 0 )
            call plafep( 6, coor(1,node), coor(2,node), 0d0, 0d0, 63 )
            if ( mod(j,5).eq.0 ) &
               call plafep ( 7, coor(1,node), coor(2,node), -0.06d0, &
                             0.06d0, node )
            if ( debug ) &
               write(irefwr,*) 'node ', node, coor(1,node), coor(2,node)
         end do

!        --- Go to first node with pen down

         node = abs(nowbnd(i,2))
         call plafep( 5, coor(1,node), coor(2,node), 0d0, 0d0, 0 )

100   end do  !  i = 1, nloop

!     --- Close plot

      lentext = len(text1)+1
      text = text1
      text(lentext+1:lentext+18) = '        loop '
      write(text(lentext+19:lentext+24),200) layer
200   format(i6)
      call plafp7 ( 1d0, 1d0, text, 0d0, 0.25d0, lentext+24 )
      call plafep( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  call erclos ( 'plotnowbnd' )
      if ( debug ) write(irefwr,*) 'End plotnowbnd'

      end
