      subroutine plotnumber ( x, y, dnumbr, isig, sizetxt, ipos, xshift, &
                              yshift, iform, nchar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot a real number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isig, ipos, iform, nchar
      double precision x, y, dnumbr, xshift, yshift, sizetxt

!     dnumbr         i    number to be plotted
!     iform          i    Defines format to be used
!                         -1:  fixed format, non-significant zeros are removed
!                          0:  fixed format
!                          1:  floating format
!     ipos           i    Defines position of number with respect to
!                         given coordinates
!                         1:  centered
!                         2:  Left allignment of text
!                         3:  Right allignment of text
!     isig           i    Number of digits behind decimal point
!     nchar          o    Number of characters in text
!     sizetxt        i    Size of numbers (multiplied by 0.15)
!     x              i    x co-ordinate of point (standard coordinates)
!     xshift         i    Shift in x direction (plotting coordinates)
!     y              i    y co-ordinate of point (standard coordinates)
!     yshift         i    Shift in y direction (plotting coordinates)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iside
      double precision xp, yp, xpboun, ypboun
      character (len=20) text

!     iside          dummy
!     text           String variable containing some text
!     xp             x-coordinate in plotting units
!     xpboun         dummy
!     yp             y-coordinate in plotting units
!     ypboun         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF01         Translate user coordinates into plotter coordinates
!     PLCONVERTFLNUM Convert real number to text with isig digits behind
!                    decimal point
!                    Floating format
!     PLCONVERTNUM   Convert real number to text with isig digits behind
!                    decimal point
!     PLOTTEXT1      Plot a text
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotnumber' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotnumber'
         write(irefwr,1) 'isig, ipos, iform', &
                          isig, ipos, iform
         write(irefwr,2) 'dnumbr, sizetxt', dnumbr, sizetxt
         write(irefwr,2) 'x, y, xshift, yshift', x, y, xshift, yshift
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute position in plotting units xp, yp

      call plaf01 ( x, y, xp, yp, xpboun, ypboun, iside )
      if (  debug ) write(irefwr,2) 'xp, yp', xp, yp

      if ( jtest>0 .and. ioutsd>=2 ) go to 1000  ! plot outside window

!     --- Compute length of number and store number in a string

      if ( iform<=0 ) then

!     --- iform = 0, i.e. fixed format with isig digits behind decimal
!         point
!         If iform= -1, non-significant zeros are removed

         call plconvertnum ( dnumbr, isig, text, nchar, iform )
         if (  debug ) write(irefwr,*) 'text ', text(1:nchar)
         if ( ierror/=0 ) go to 1000

      else

!     --- iform>0, i.e. floating format with isig digits behind decimal
!         point

         call plconvertflnum ( dnumbr, nchar, isig, text )

      end if  ! ( iform==0 )

!     --- Plot number

      call plottext1 ( text, nchar, sizetxt, ipos, xp, yp, xshift, &
                       yshift, 0d0 )

1000  call erclos ( 'plotnumber' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotnumber'

      end if  ! ( debug )

      end
