      subroutine plotrennodes
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 21-07-2008
!
!   copyright (c) 2008-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot renumbered nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     plot
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinplt(8), idum(1)
      double precision rinplt(2)

!     idum           General dummy variable
!     iinplt         Input array for subroutine plotms:
!                    1:  Number of entries filled
!                    2:  Plot element numbers (1) or not (0)
!                    3:  Plot nodal point numbers (1) or not (0)
!                    4:  Mark nodal point numbers (1) or not (0)
!                    5:  Contract elements (1) or not (0) (2D only)
!                    6:  Plot outer boundaries double (1) or not (0)
!                    7:  Skip element groups (>0) or not (0)
!                    8:  Plot renumbered node numbers (1) or original ones (0)
!     rinplt         Input array for subroutine plotms:
!                    1:    plotfm
!                    2:    yfact
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPLOTMS      plot mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotrennodes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotrennodes'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Plot renumbered nodes

      iinplt(1) = 8
      iinplt(2) = 0
      iinplt(3) = 1
      iinplt(4) = 0
      iinplt(5) = 0
      iinplt(6) = 0
      iinplt(7) = 0
      iinplt(8) = 1
      rinplt(1) = rinputkm(1)
      rinplt(2) = rinputkm(2)
      jtimes = 0
      call sepplotms ( iinplt, rinplt, idum )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotrennodes'

      end if  ! ( debug )
      call erclos ( 'plotrennodes' )

      end subroutine plotrennodes
