      subroutine plotststart ( yfact, plotfm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start a new plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision yfact, plotfm

!     plotfm         i    Plotting factor. When PLOTFM<0, then the length of
!                         the plot is computed by - (xmax-xmin) * PLOTFM
!                         and the width by - (ymax-ymin) * YFACT * PLOTFM
!                         else the length of the plot is equal to PLOTFM cm
!                         When by some mistake the user has given | PLOTFM | a
!                         value smaller than 1 cm, then the length is chosen
!                         equal to the default length stored in common block
!                         consta
!     yfact          i    Multiplication for the y-co-ordinates. With this
!                         parameter the user can influence the scale in
!                         y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotststart' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotststart'
         write(irefwr,2) 'plotfm, yfact', plotfm, yfact
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute length and width of the plot

      if (plotfm>0) then
         aleng = plotfm
      else
         aleng = -plotfm*(axmax-axmin)
      end if
      if (yfact>=0d0) then
         width = aleng*(aymax-aymin)/(axmax-axmin)*yfact
      else
         width = -yfact
      end if

      if ( debug ) write(irefwr,2) 'aleng, width', aleng, width

      aleng = max(aleng,2.5d0)
      width = max(width,2.5d0)

      call plafep ( 1, aleng, width, 1d0, 0d0, 1 )

1000  call erclos ( 'plotststart' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotststart'

      end if  ! ( debug )

      end
