      subroutine plottext1 ( text, nchar, sizetxt, ipos, xp, yp, xshift, &
                             yshift, angle )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot a text
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nchar, ipos
      double precision xp, yp, xshift, yshift, sizetxt, angle
      character (len=*) text

!     angle          i    angle of rotation
!     ipos           i    Defines position of number with respect to
!                         given coordinates
!                         1:  centered
!                         2:  Left allignment of text
!                         3:  Right allignment of text
!     nchar          i    Number of characters in text
!     text           i    String variable containing the text to be plotted
!     xp             i    x-coordinate in plotting units
!     xshift         i    Shift in x direction (plotting coordinates)
!     yp             i    y-coordinate in plotting units
!     yshift         i    Shift in y direction (plotting coordinates)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision shiftx1, x1, y1

!     shiftx1        Shift in x-direction
!     sizetxt        Size of digits
!     x1             x-coordinate
!     y1             y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plottext1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plottext1'
         write(irefwr,1) 'ipos', ipos
         write(irefwr,2) 'xp, yp, xshift, yshift, sizetxt', &
                          xp, yp, xshift, yshift, sizetxt
         write(irefwr,*) 'text ', text(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define position of at most left character

      select case ( ipos)

      case(1)

!     --- Centered text, move nchar/2*sizetxt positions to the left

         shiftx1 = xshift-0.3d0*nchar*sizetxt

      case(2)

!     --- Left allignment of text, no movement

         shiftx1 = xshift

      case(3)

!     --- Right allignment of text, move nchar*sizetxt positions to the left

         shiftx1 = xshift-0.6*nchar*sizetxt

      end select  ! case ( ipos)
      if ( debug ) write(irefwr,2) 'shiftx1', shiftx1

      if ( idirec==1 ) then

!     --- Standard direction

         x1 = xp+shiftx1
         y1 = yp+yshift
         if ( debug ) write(irefwr,2) 'x1, y1', x1, y1
         call plafp7 ( x1, y1, text, angle, sizetxt, nchar )

      else

!     --- Rotated direction

         x1 = xp+yshift
         y1 = yp-shiftx1
         call plafp7 ( x1, y1, text, angle-90d0, sizetxt, nchar )

      end if  ! ( idirec==1 )

1000  call erclos ( 'plottext1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plottext1'

      end if  ! ( debug )

      end
