      subroutine plottimehis2 ( vectim, num, rinsep, inpsep, colours, &
                                numtim, iscalars )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot contents of time history array
!     At present the array is printed instead of plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer num, colours(*), inpsep(*), numtim, iscalars(num)
      double precision vectim(num+1,numtim), rinsep(*)

!     colours        i    Contains the set of colors to be used for each
!                         function
!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine readscplottim
!     iscalars       i    Contains the sequence numbers of the scalars
!                         to be used
!     num            i    Number of points where time history is stored
!     numtim         i    Number of time steps
!     rinsep         i    Array containing the real input for this subroutine
!                         For a description see subroutine readscplottim
!     vectim         i    Array containing the value of the time followed
!                         by the num node values for each time step
!                         in the time history
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision funcx(numtim), funcy(numtim,num)

!     funcx          Work array to store the time parameter
!     funcy          Work array to store the values in each time history
!                    point per time step
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTFUNCSCALAR Plot one or more time-histories of scalars in a
!                    function plot
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plottimehis2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plottimehis2'
         write(irefwr,1) 'numtim, num', numtim, num
         call prinrl1 ( vectim, numtim, num+1, 'vectim' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy vectim in funcx and funcy
!         This is necessary, since the sequence is changed
!         The first and second index are interchanged

      funcx(1:numtim) = vectim(1,1:numtim)

      if ( debug ) call prinrl ( funcx, numtim, 'funcx' )

      do i = 1, num
         funcy(1:numtim,i) = vectim(1+i,1:numtim)
      end do  ! i = 1, num

      if ( debug ) &
         call prinrl1 ( funcy, num, numtim, 'funcy' )

      call plotfuncscalar ( numtim, funcx, funcy, num, rinsep, &
                            inpsep, colours, iscalars )

1000  call erclos ( 'plottimehis2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plottimehis2'

      end if  ! ( debug )

      end
