      subroutine plotzerolev3d ( coor, ilev, nmeshc, iobstfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the faces (3d) of the zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ilev, nmeshc(*), iobstfaces(*)
      double precision coor(3,*)

!     iobstfaces     i    contains edge sequence numbers of boundary
!                         of levelset
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nnode_edge, jstart, numedges, iedge, kstart

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotzerolev3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotzerolev3d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kstart = iobstfaces(ilev)
      numedges = iobstfaces(kstart)

      do i = 1, numedges

         iedge = iobstfaces(kstart+i)       ! edge number
         jstart = nmeshc(iedge)
         nnode_edge = nmeshc(jstart)

         if ( debug ) then
            write(irefwr,1) 'iedge', i
            call prinin ( nmeshc(jstart+1), nnode_edge, 'nodes' )
         end if  ! ( debug )

      end do  ! i = 1, numedges

      if ( debug ) write(irefwr,1) 'nnode_edge', nnode_edge

1000  call erclos ( 'plotzerolev3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotzerolev3d'

      end if  ! ( debug )

      end
