      subroutine plpoin ( coor, npoint, kmeshj, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Use kmeshj
!        version  2.1  date 14-11-1994 Extra possibility jmark
!        version  2.0  date 15-02-1991 Extension for 1-D, extra parameter
!
!   copyright (c) 1983-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    plot nodal points with marks (x)
!    when jmark = 0, 3  nodal point numbers are plotted also
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kmeshj(*), ndim
      double precision coor(ndim,*)

!     coor    i    array containing the coordinates of the nodal points
!     kmeshj  i    see npoint
!     ndim    i    Dimension of the space.
!     npoint  i    |npoint| = number of nodal points
!                  when npoint>0 the nodal points are numbered from 1 to
!                  npoint.  when npoint<0 kmeshj contains the new nodal
!                  point numbering
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodp, imark, jnodp, np, i
      integer, allocatable :: mapar(:)
      double precision x, y
      logical plnode, plmark

!     i              Counting variable
!     imark          jmark '/' 10
!     inodp          Nodal point number
!     jnodp          Nodal point sequence number
!     mapar          Contains the new nodes numbers
!     np             |npoint|
!     plmark         If true plot markers
!     plnode         If true plot nodal point numbers
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plpoin' )

      if ( jmark>=10 ) then
         imark = jmark-10
      else
         imark = jmark
      end if
      plmark = imark==0 .or. imark==1 .or. imark==3 .or. &
               imark==4
      plnode = imark==0 .or. imark==3 .or. imark==7 .or. &
               imark==6
      np = abs(npoint)
      if ( npoint<0 ) then

!     --- npoint<0, fill mapar

         allocate ( mapar(np), stat = error )
         if ( error/=0 ) call eralloc ( error, np, 'mapar' )
         if ( ierror/=0 ) go to 1000

         do i = 1, np
            mapar(kmeshj(i)) = i
         end do  ! i = 1, np

      end if  ! ( npoint<0 )

      y = 0d0
      do inodp = 1, np
         x = coor(1,inodp)
         if ( ndim>1 ) y = coor(2,inodp)
         if ( plmark ) call plafep ( 6, x, y, 0d0, 0d0, 0 )
         if ( plnode ) then
            jnodp = inodp
            if ( npoint<0 ) jnodp = mapar(inodp)
            call plafep ( 7, x, y, -0.06d0, 0.06d0, jnodp )
         end if
      end do

      if ( npoint<0 ) then

!     --- npoint<0, fill mapar

         deallocate ( mapar, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'mapar' )

      end if  ! ( npoint<0 )

1000  call erclos ( 'plpoin' )

      end subroutine plpoin
