      subroutine plpos8 ( icolor, npolyg, polyg )
! ======================================================================
!
!        programmer    Jac Janssen
!        version  1.3  date 22-08-2013 New definition file ref numbers
!        version  1.2  date 16-05-2003 Increase colour number by 1
!        version  1.1  date 28-06-1994 add support for PostScript color
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Plot polygon, or fill polygon with colour depending on the
!    value of ichois
!    All parameters are single precision
!    machine dependent subroutine
!    PostScript
! **********************************************************************
!
!                       KEYWORDS
!
!     machine_dependent
!     plot
!     postscript
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icolor, npolyg
      real polyg(2,*)

!    icolor   i   number indicating the type of colour to be used
!                 0: do not fill the polygon
!    npolyg   i   number of points in the polygon
!    polyg    i   array of length 2*npolyg containing the co-ordinates
!                 of the points in the polygon:
!                 polyg(1,i) = xi
!                 polyg(2,i) = yi
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      real size, xoffset, yoffset
      parameter ( size = 72/3.0, xoffset = 15.0, yoffset = 20.0 )

!     i              General loop variable
!     size           Parameter to scale from program units to cm's on the paper
!                    This parameter is defined by a parameter statement
!                    One postscript unit = 1/72 inch
!     xoffset        offset in x direction
!     yoffset        offset in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLCOL0         Routine to look at color and set it if changed since last
!                    call
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (icolor.gt.0) call plcol0 (icolor+1)

!     --- Put definitions of number of polygon points
!         into postscript file

      write ( irefps, '( '' % polygon:'' )' )
      write ( irefps, 100 ) npolyg
100   format ( ' /numofpol  ', i2, '  def' )

!     --- The postscript command "/polygon is written to the file
!         This command has already been defined in plpos6

      write ( irefps, 110 ) ( size*polyg(1,i)+xoffset, &
                              size*polyg(2,i)+yoffset, &
                              i = npolyg, 1, -1 )
110   format ( ( 2(1x,f8.3) ) )
      write ( irefps, 120 )
120   format ( ' polygon' )
      if (icolor.gt.0) write ( irefps,130 )
130   format (' fill')
      call plcol0 (dcolor+1)
      end
