      subroutine plt303( number, compl, kprobf, &
                         nstart, amag, bmag, larray, &
                         angleh, linopt ,plotfm, factor, ispec, refval )
! ======================================================================
!
!        programmer    niek praagman
!        version  6.4  date 18-08-1992 remove float
!        version  6.3  date 17-01-1992 user may specify min and max of plot
!        version  6.2  date 16-09-1991 stop replaced by call instop
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine to make a 3-d plot of array amag
!    with hidden lines
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleconsts
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nstart(*), kprobf(*), ispec, larray, &
              linopt, number
      double precision amag(*), bmag(*), refval(2), &
                       angleh, factor, plotfm
      logical compl

!     amag       i    array containing the function values in all
!                     nodal points of the function to be plotted
!     angleh     i    direction from which the user wants to see
!                     the picture in degrees
!     bmag       i    help array of nstep*nstep*npoint positions
!     compl      i    indication whether the arithmetic is real (false)
!                     or complex (true)
!     factor     i    Multiplication factor for function to be plotted
!     ispec      i    parameter to indicate whether function values are
!                     given in all nodal points (ispec=0) or only in
!                     vertices (ispec=1).
!     kprobf     i    KPROB part f: contains the number of unknowns per point
!                     accumulated
!     larray     i    length of arrays for help in pltiso
!     linopt     i    see plot3d
!     nstart     i    special array containing all startaddresses
!                     of the elements
!     number     i    the number'th unknown in each point is considered
!     plotfm     i    Plotting factor. When PLOTFM < 0, then the length of
!                     the plot is computed by - (xmax-xmin) * PLOTFM
!                     and the width by - (ymax-ymin) * YFACT * PLOTFM
!                     else the length of the plot is equal to PLOTFM cm
!                     When by some mistake the user has given | PLOTFM | a value
!                     smaller than 1 cm, then the length is chosen equal to
!                     the default length stored in common block consta
!     refval    i     if base < 0 , refval contains the reference values
!                     position 1 refmax and
!                     position 2 refmin
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, i8, i9, iel, ielem, ielgrp, &
              ihelp, inpelm, inunkp, ipl, jmaxbf, &
              kwadrt, linold, nh, nstep, nx, ny
      double precision angle, delta, dx, dy, si, x00, x1, x2, x3, x4, x5, &
              x6, x7, x8, x9, xh, xma, xmaxi, xmi, xmini, y00, y1, y2, &
              y3, y4, y5, y6, y7, y8, y9, yh, yma, ymaxi, ymi, ymini, co

!     angle
!     co
!     delta
!     dx         length of vector in x-direction
!     dy         length of vector in y-direction
!     i          Counting variable
!     i1
!     i2
!     i3
!     i4
!     i5
!     i6
!     i7
!     i8
!     i9
!     iel
!     ielem      Element number
!     ielgrp     element group number
!     ihelp
!     inpelm     number of points per element
!     inunkp     Number of degrees of freedom in point
!     ipl
!     jmaxbf
!     kwadrt
!     linold
!     nh
!     nstep
!     nx         number of points in x direction
!     ny         number of points in y direction
!     si
!     x00
!     x1
!     x2
!     x3
!     x4
!     x5
!     x6
!     x7
!     x8
!     x9
!     xh
!     xma
!     xmaxi
!     xmi
!     xmini
!     y00
!     y1
!     y2
!     y3
!     y4
!     y5
!     y6
!     y7
!     y8
!     y9
!     yh
!     yma
!     ymaxi
!     ymi
!     ymini
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PLT304    determine function values on an equidistant grid
!     PLT305    compute factors for a 3-d plot and activate plotting system
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     605   The number of points in x-direction too small
!     606   The number of points in y-direction too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plt303' )

!     --- determine nh

      nh=number
      if ( compl ) nh=(nh+1)/2

!     --- compute angle in radians

      angle=(angleh-45d0)/180d0*pi

!     --- rotate over angle radians

      co=cos(angle)
      si=sin(angle)

      do 100 i=1,npoint
          xh=coor(1,i)
          yh=coor(2,i)
          coor(1,i)= xh*co+yh*si
          coor(2,i)=-xh*si+yh*co
100   continue

!     --- determine xmin,xmax,ymin,ymax if not already given

      jmaxbf = jmax
      if ( jmax==1 ) then
           xmini=xmin
           xmaxi=xmax
           ymini=ymin
           ymaxi=ymax
           xmi= xmin*co+ymin*si
           ymi=-xmin*si+ymin*co
           xma= xmax*co+ymax*si
           yma=-xmax*si+ymax*co
           xmin=xmi
           ymin=ymi
           xmax=xma
           ymax=yma
           if ( xmax<xmin ) then
                xmax=xmi
                xmin=xma
           end if
           if ( ymax<ymin ) then
                ymax=ymi
                ymin=yma
           end if
      else
           xmin=rinfin
           ymin=xmin
           xmax=-xmin
           ymax=xmax

           do 200 i=1,npoint
               x00=coor(1,i)
               y00=coor(2,i)
               if (x00<xmin) xmin=x00
               if (x00>xmax) xmax=x00
               if (y00<ymin) ymin=y00
               if (y00>ymax) ymax=y00
200        continue

      end if

!     --- Compute density of help-points

      nstep=int(linopt/10.)+1
      linold=linopt-10*(nstep-1)
      kwadrt=nstep*nstep

!     --- determine stepsize delta

      delta=(xmax-xmin)*(ymax-ymin)/(npoint*1d0*kwadrt)
      delta=sqrt(abs(delta))

      nx=nint((xmax-xmin)/delta)
      if ( nx>larray ) nx=larray
      ny=nint((ymax-ymin)/delta)
      if ( ny>larray ) ny=larray

300   continue

      if ((nx*ny)<=(kwadrt*npoint)) then
          goto 400
      else
          nx=nx-1
          if (nx<2) then
             call errint ( nx, 1 )
             call errint ( 2, 2 )
             call errsub ( 605, 2, 0, 0 )
             go to 1000
          end if
          ny=ny-1
          if (ny<2) then
             call errint ( ny, 1 )
             call errint ( 2, 2 )
             call errsub ( 606, 2, 0, 0 )
            go to 1000
          end if
          goto 300
      end if

400   continue

      dx=(xmax-xmin)/(nx-1d0)
      dy=(ymax-ymin)/(ny-1d0)

!     --- initialize bmag

      do 450 i=1,nx*ny
          bmag(i)=0d0
450   continue

!     --- determine values for all points

      ielem=0
      ipl=0

      do 550 ielgrp=1,nelgrp
          ishape=kmeshf(ielgrp)
          inpelm=Topgrp(ielgrp)%inpelm
          inelem=Topgrp(ielgrp)%inelem
          if (ishape<3.or.ishape>7) then
              ipl=ipl+inpelm*inelem
              ielem=ielem+inelem
              goto 550
          end if

      ihelp=0

      if ( inpelm==6 .and. ispec.eq.1 ) ihelp=1

      if ( inpelm==7 .or. inpelm.eq.9 ) then

           if ( ispec==1 ) then

                i2=kmeshc(ipl+2)
                inunkp=kprobf(i2+1)-kprobf(i2)
                if ( inunkp<nh ) ihelp=1

           end if

      end if

      do 500 iel=1,inelem
          ielem=ielem+1
          inpelm=nstart(ielem+1)-nstart(ielem)
          if (inpelm==3) then

!         --- inpelm=3 , triangle

              i1=kmeshc(ipl+1)
              i2=kmeshc(ipl+2)
              i3=kmeshc(ipl+3)
              ipl=ipl+3

              x1=coor(1,i1)
              x2=coor(1,i2)
              x3=coor(1,i3)

              y1=coor(2,i1)
              y2=coor(2,i2)
              y3=coor(2,i3)

              call plt304(i1,i2,i3,x1,x2,x3,y1,y2,y3,dx,dy,amag, &
                          bmag,nx,ny,xmin,ymin)

              goto 500
          end if

          if (inpelm==4) then

!         --- inpelm = 4 , two triangles

              i1=kmeshc(ipl+1)
              i2=kmeshc(ipl+2)
              i3=kmeshc(ipl+3)
              i4=kmeshc(ipl+4)
              ipl=ipl+4

              x1=coor(1,i1)
              x2=coor(1,i2)
              x3=coor(1,i3)
              x4=coor(1,i4)

              y1=coor(2,i1)
              y2=coor(2,i2)
              y3=coor(2,i3)
              y4=coor(2,i4)

              call plt304(i1,i2,i3,x1,x2,x3,y1,y2,y3,dx,dy,amag, &
                          bmag,nx,ny,xmin,ymin)

              call plt304(i1,i3,i4,x1,x3,x4,y1,y3,y4,dx,dy,amag, &
                          bmag,nx,ny,xmin,ymin)

              goto 500

          end if

          if ((inpelm==6).or.(inpelm.eq.7)) then

!         --- inpelm=6 or 7 , four triangles or one triangle

              i1=kmeshc(ipl+1)
              i2=kmeshc(ipl+2)
              i3=kmeshc(ipl+3)
              i4=kmeshc(ipl+4)
              i5=kmeshc(ipl+5)
              i6=kmeshc(ipl+6)

              ipl=ipl+inpelm

              x1=coor(1,i1)
              x2=coor(1,i2)
              x3=coor(1,i3)
              x4=coor(1,i4)
              x5=coor(1,i5)
              x6=coor(1,i6)

              y1=coor(2,i1)
              y2=coor(2,i2)
              y3=coor(2,i3)
              y4=coor(2,i4)
              y5=coor(2,i5)
              y6=coor(2,i6)

            if ( ispec==0 .or. ihelp.eq.0 ) then
              call plt304(i1,i2,i6,x1,x2,x6,y1,y2,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i2,i3,i4,x2,x3,x4,y2,y3,y4,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i2,i4,i6,x2,x4,x6,y2,y4,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i4,i5,i6,x4,x5,x6,y4,y5,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)

            else
              call plt304(i1,i3,i5,x1,x3,x5,y1,y3,y5,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
            end if

              goto 500

          end if

          if (inpelm==9) then

!         --- inpelm=9 , eight triangles , six triangles or two triangles

              i1=kmeshc(ipl+1)
              i2=kmeshc(ipl+2)
              i3=kmeshc(ipl+3)
              i4=kmeshc(ipl+4)
              i5=kmeshc(ipl+5)
              i6=kmeshc(ipl+6)
              i7=kmeshc(ipl+7)
              i8=kmeshc(ipl+8)
              i9=kmeshc(ipl+9)

              ipl=ipl+9

              x1=coor(1,i1)
              x2=coor(1,i2)
              x3=coor(1,i3)
              x4=coor(1,i4)
              x5=coor(1,i5)
              x6=coor(1,i6)
              x7=coor(1,i7)
              x8=coor(1,i8)
              x9=coor(1,i9)

              y1=coor(2,i1)
              y2=coor(2,i2)
              y3=coor(2,i3)
              y4=coor(2,i4)
              y5=coor(2,i5)
              y6=coor(2,i6)
              y7=coor(2,i7)
              y8=coor(2,i8)
              y9=coor(2,i9)

            if (ispec==0) then

!           --- eight triangles

              call plt304(i1,i2,i8,x1,x2,x8,y1,y2,y8,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i2,i3,i4,x2,x3,x4,y2,y3,y4,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i2,i9,i8,x2,x9,x8,y2,y9,y8,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i2,i4,i9,x2,x4,x9,y2,y4,y9,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i8,i9,i6,x8,x9,x6,y8,y9,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i6,i7,i8,x6,x7,x8,y6,y7,y8,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i4,i5,i6,x4,x5,x6,y4,y5,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i4,i6,i9,x4,x6,x9,y4,y6,y9,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
            else

              if ( ihelp==0 ) then

!             --- six triangles

              call plt304(i8,i1,i2,x8,x1,x2,y8,y1,y2,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i2,i3,i4,x2,x3,x4,y2,y3,y4,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i4,i5,i6,x4,x5,x6,y4,y5,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i6,i7,i8,x6,x7,x8,y6,y7,y8,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i8,i2,i4,x8,x2,x4,y8,y2,y4,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i8,i4,i6,x8,x4,x6,y8,y4,y6,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)

              else if ( ihelp==1 ) then

!             --- two triangles

              call plt304(i1,i3,i5,x1,x3,x5,y1,y3,y5,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              call plt304(i1,i5,i7,x1,x5,x7,y1,y5,y7,dx,dy,amag,bmag, &
                          nx,ny,xmin,ymin)
              end if

            end if

              goto 500

          end if

500      continue

550   continue

!     --- the values of the equidistant grid are stored in bmag ;
!                make picture

      call plt305(bmag,ny,nx,factor,plotfm,linold, &
                  larray,refval)

!     --- rotate back to original situation

      do 600 i=1,npoint
          xh=coor(1,i)
          yh=coor(2,i)
          coor(1,i)= xh*co-yh*si
          coor(2,i)= xh*si+yh*co
600   continue

      jmax = jmaxbf
      if ( jmax==1 ) then
           xmin= xmini
           ymin= ymini
           xmax= xmaxi
           ymax= ymaxi
      end if

1000  call erclos ( 'plt303' )

      end
