      subroutine plt305(ta,n,m,factor,plotfm,linopt,lar,refval)
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.5  date 17-01-1992 user may specify min and max of plot
!        version  3.4  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.3  date 17-02-1989  Deletion of superfluous parameters
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to compute factors for a 3-d plot and to activate
!   the plotting system
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision factor, plotfm, ta(*), refval(2)
      integer lar, linopt, m, n

!     factor     i    factor to multiply solution ta with in order
!                     to scale the plot
!     lar             help variable
!     linopt     i    see plot3d
!     m          i    number of grid points in x-direction
!     n          i    number of grid points in y-direction
!     plotfm     i    Plotting factor. When PLOTFM < 0, then the length of
!                     the plot is computed by - (xmax-xmin) * PLOTFM
!                     and the width by - (ymax-ymin) * YFACT * PLOTFM
!                     else the length of the plot is equal to PLOTFM cm
!                     When by some mistake the user has given | PLOTFM | a value
!                     smaller than 1 cm, then the length is chosen equal to
!                     the default length stored in common block consta
!     refval     i    reference value, if refval # 0 then the values
!                     of refval are used to define the plot instead
!                     of the extreme values of ta
!     ta         i    array of n*m positions containing the values
!                     of a funtion to be plotted on an equidistant grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical solid,base,xsect
      integer i, ikader, in, nm
      double precision alen, eps, sc, w, w1, xp, cfact, z, z2, zh
      double precision :: cmh(13*lar)

!     alen
!     base       Base of 3D plot
!     cfact
!     cmh         help array
!     eps        Local accuracy
!     i          Counting variable
!     ikader     Help variable to store jkader temporarily
!     in
!     nm
!     sc
!     solid
!     w
!     w1
!     xp
!     xsect
!     z          z co-ordinate of point
!     z2
!     zh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAFEP    General SEPRAN low level plotting subroutine
!     PLTISO    produce an isometric projection of a surface z =z(x,y)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set eps

      eps = 10. * epsmac


!     --- plot pictures

      ikader=jkader
      if(jkader.ge.0) jkader=1


      if ( linopt.eq.1 .or. linopt.eq.5 ) then

         zmin = ta(1)
         zmax = ta(1)
         nm   = n*m


!        --- Compute extremal values

         do 10 i=2,nm
             if (ta(i).gt.zmax) zmax=ta(i)
             if (ta(i).lt.zmin) zmin=ta(i)
10       continue


         if ( abs(refval(1) ).gt.eps ) then
             zmax = refval(1)
             zmin = refval(2)
         end if


         do 20 i=1,nm
             ta(i)=ta(i)-zmin
20       continue


         zmax=zmax-zmin


         w=sqrt(3d0)
         w1=1d0/w


!        --- Compute scale factor

         sc=(1d0-w1)/zmax
         xp=2*w1*(m-1)/((m+n-2)*sc)
         xmin=-w/2d0*xp*sc*(n-1d0)/(m-1d0)
         xmax=w/2d0*xp*sc
         zh=0.5d0*xp*sc*(n+m-2d0)/(m-1d0)+sc*zmax
         z=zh*factor
         cfact=(z-0.5d0*xp*sc*(n+m-2d0)/(m-1d0))/(sc*zmax)


!        --- Compute values of common block cplot , see pr.guide

         jmax=1
         ymin=-0d0
         ymax=factor*(xmax-xmin)


!        --- Start picture

         alen=plotfm*factor
         call plafep(1,plotfm,alen,1d0,z2,in)


         solid=.true.
         base=.true.
         zmin=-0d0


!        --- Scale with user-factor

         do 30 i=1,nm
             ta(i)=ta(i)*cfact
30       continue


         xsect=.false.
         call pltiso(m,n,sc,xp,ta,solid,base,zmin,xsect, &
                 cmh(1),cmh(lar+1),cmh(2*lar+1),cmh(3*lar+1), &
                 cmh(4*lar+1),cmh(5*lar+1),cmh(6*lar+1),cmh(7*lar+1), &
                 cmh(8*lar+1),cmh(9*lar+1),cmh(10*lar+1), &
                 cmh(11*lar+1),cmh(12*lar+1))


!        --- Scale back with user-factor

         do 40 i=1,nm
             ta(i)=ta(i)/cfact
40       continue


!        --- Close picture data set

         jmax = 0
         call plafep(2,0d0,0d0,0d0,0d0,0)


      end if


      if ( linopt.eq.2 .or. linopt.eq.5 ) then


         zmin = ta(1)
         zmax = ta(1)
         nm   = n*m


!        --- Compute extremal values

         do 50 i=2,nm
             if (ta(i).gt.zmax) zmax=ta(i)
             if (ta(i).lt.zmin) zmin=ta(i)
50       continue


         if ( abs(refval(1) ).gt.eps ) then
            zmax = refval(1)
            zmin = refval(2)
         end if


         do 60 i=1,nm
             ta(i)=ta(i)-zmin
60       continue


         zmax=zmax-zmin
         w=sqrt(3d0)
         w1=1d0/w
         sc=(1d0-w1)/zmax
         xp=2*w1*(m-1)/((m+n-2)*sc)
         xmin=-w/2d0*xp*sc*(n-1d0)/(m-1d0)
         xmax=w/2d0*xp*sc
         zh=0.5d0*xp*sc*(n+m-2d0)/(m-1d0)+sc*zmax
         z=zh*factor
         cfact=(z-0.5d0*xp*sc*(n+m-2d0)/(m-1d0))/(sc*zmax)
         ymin=-0d0
         ymax=factor*(xmax-xmin)
         jmax=1


!        --- Start picture

         alen=plotfm*factor
         call plafep(1,plotfm,alen,1d0,z2,in)


         solid=.true.
         base=.true.
         xsect=.true.
         zmin=-0d0


         do 70 i=1,nm
             ta(i)=ta(i)*cfact
70       continue


         call pltiso(m,n,sc,xp,ta,solid,base,zmin,xsect, &
                 cmh(1),cmh(lar+1),cmh(2*lar+1),cmh(3*lar+1), &
                 cmh(4*lar+1),cmh(5*lar+1),cmh(6*lar+1),cmh(7*lar+1), &
                 cmh(8*lar+1),cmh(9*lar+1),cmh(10*lar+1), &
                 cmh(11*lar+1),cmh(12*lar+1))


         do 80 i=1,nm
             ta(i)=ta(i)/cfact
80       continue


!        --- Close plot data set

         jmax=0
         call plafep(2,0d0,0d0,0d0,0d0,0)


      end if


      if ( linopt.eq.3 .or. linopt.eq.5 ) then
         zmin=ta(1)
         zmax=ta(1)
         nm=n*m


!        --- Compute extremal values

         do 90 i=2,nm
             if (ta(i).gt.zmax) zmax=ta(i)
             if (ta(i).lt.zmin) zmin=ta(i)
90       continue


         if ( abs(refval(1) ).gt.eps ) then
            zmax = refval(1)
            zmin = refval(2)
         end if


         do 100 i=1,nm
             ta(i)=ta(i)-zmin
100      continue


         zmax=zmax-zmin
         w=sqrt(3d0)
         w1=1d0/w
         sc=(1d0-w1)/zmax
         xp=2*w1*(m-1)/((m+n-2)*sc)
         xmin=-w/2d0*xp*sc*(n-1d0)/(m-1d0)
         xmax=w/2d0*xp*sc
         zh=0.5d0*xp*sc*(n+m-2d0)/(m-1d0)+sc*zmax
         z=zh*factor
         cfact=(z-0.5d0*xp*sc*(n+m-2d0)/(m-1d0))/(sc*zmax)
         jmax=1
         ymin=-0d0
         ymax=factor*(xmax-xmin)


!        --- Start plot

         alen=plotfm*factor
         call plafep(1,plotfm,alen,1d0,z2,in)


         solid=.true.
         base=.true.
         xsect=.true.
         zmin=-0d0


         do 110 i=1,nm
             ta(i)=ta(i)*cfact
110      continue


         call pltiso(m,n,sc,xp,ta,solid,base,zmin,xsect, &
                 cmh(1),cmh(lar+1),cmh(2*lar+1),cmh(3*lar+1), &
                 cmh(4*lar+1),cmh(5*lar+1),cmh(6*lar+1),cmh(7*lar+1), &
                 cmh(8*lar+1),cmh(9*lar+1),cmh(10*lar+1), &
                 cmh(11*lar+1),cmh(12*lar+1))
         xsect=.false.
         call pltiso(m,n,sc,xp,ta,solid,base,zmin,xsect, &
                 cmh(1),cmh(lar+1),cmh(2*lar+1),cmh(3*lar+1), &
                 cmh(4*lar+1),cmh(5*lar+1),cmh(6*lar+1),cmh(7*lar+1), &
                 cmh(8*lar+1),cmh(9*lar+1),cmh(10*lar+1), &
                 cmh(11*lar+1),cmh(12*lar+1))


         do 120 i=1,nm
             ta(i)=ta(i)/cfact
120      continue


         jmax=0


!        --- Close plot data set

         call plafep(2,0d0,0d0,0d0,0d0,0)


      end if


      jkader=ikader


      end
