      subroutine pltbn1 ( ichois, icurvs, curves, ncurvs, plotfm, yfact, &
                          nuspnt, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-01-1996 New layout
!        version  1.0  date 23-01-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    this subroutine plots the curves and the user points of the mesh
!    when ndim = 1
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     curve
!     plot
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, icurvs(ncurvs), nuspnt, ichois
      double precision curves(*), plotfm, yfact, rinput(nuspnt)

!     curves   i   array containing the co-ordinates of the curve points
!     ichois   i   Choice parameter indicating what has to be plotted
!                  by this subroutine
!                  ICHOIS consists of 3 parts iplcur, iplnod and iplusp
!                  according to
!                  ICHOIS = iplcur + 10*iplnod + 100*iplusp
!                  with:
!                  iplcur:  Information of the curves to be plotted
!                    0  No plot of the curve
!                    1  Plot of the curve without curve numbers
!                    2  Plot of the curve with curve numbers
!                  iplnod:  Information of the nodal points to be plotted
!                    0  No plot of the nodal points
!                    1  Plot of the nodal points using crosses
!                    >1 Plot of the curve with symbols (symbol number ichois-1)
!                  iplusp:  Information of the user points to be plotted
!                    0  No plot of the user points
!                    1  Plot of the user points without numbers
!                    2  Plot of the user points with numbers
!     icurvs   i   array containing the number of nodal points
!                  in the curves  ( accumulated )
!     ncurvs   i   number of curves
!     nuspnt   i   Number of user points
!     plotfm   i   length of the plot in centimeters
!     rinput   i   Array containing the coordinates of the user points
!     yfact    i   multiplication factor for the y-coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurve, j, ikader, ncount, iuppl, n, iplcur, iplnod, &
              iplusp, iscolr
      double precision x

!     icurve    Curve number
!     ikader    Old value of jkader (temporary storage)
!     iplcur    Indication of how the curve must be plotted, see ICHOIS
!     iplnod    Indication of how the nodes must be plotted, see ICHOIS
!     iplusp    Indication of how the user points must be plotted, see ICHOIS
!     iscolr    Help variable to store the old value of dcolor
!     iuppl     Indication if the pen must move to a point with pen up
!               (4) or down (#4)
!     j         Counting variable
!     n         Number of points in the curves that are plotted until now, &
!               plus present curve
!     ncount    Number of points in the curves that are plotted until now
!     x         Co-ordinates of a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     PL0000    Compute length and width of the plot
!     PLAFEP    General SEPRAN low level plotting subroutine
!     PLAFP3    Machine dependent plotting of symbols
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltbn1' )
      iscolr = dcolor

!     --- compute iplcur, iplnod, iplusp, xmin, xmax, ymin, ymax, &
!         aleng and width

      iplcur = mod ( ichois,10 )
      iplnod = mod ( ichois/10,10 )
      iplusp = mod ( ichois/100,10 )
      ikader = jkader
      if(jkader.ge.0) jkader = 4
      ncount = icurvs(ncurvs)
      if ( jmax.eq.0 ) then
         xmin = curves(1)
         xmax = xmin
         do 100 j = 2, ncount
            xmin = min ( xmin, curves(j) )
            xmax = max ( xmax, curves(j) )
100      continue
         ymax = 0.25d0 * ( xmax - xmin )
         ymin = - ymax
      end if
      call pl0000 ( plotfm, yfact )

!     --- start picture

      call plafep ( 1, aleng, width, 0d0, 0d0, 0 )

!     --- plot curves

      if ( iplcur.gt.0 .or. iplnod.gt.0 ) then

!     --- Plot curves and/or nodal points along curves

         ncount = 1
         do 300 icurve = 1, ncurvs
            n = icurvs(icurve)
            iuppl = 4
            do 200 j = ncount, n
               if ( iscolr.gt.0 ) dcolor = 1+iscolr
               if ( iplcur.gt.0 ) call plafep ( iuppl, curves(j), &
                    0d0, 0d0, 0d0, 0 )
               if ( iplnod.eq.1 ) then
                  if ( iscolr.gt.0 ) dcolor = 2+iscolr
                  call plafep ( 6, curves(j), 0d0, 0d0, 0d0, 0 )
               else if ( iplnod.gt.1 ) then
                  if ( iscolr.gt.0 ) dcolor = 2+iscolr
                  call plafp3 ( iplnod-1, curves(j), 0d0 )
               end if
               iuppl = 5
200         continue
            if ( iplcur.eq.2 ) then

!           --- iplcur = 2, Plot curve numbers

               dcolor = iscolr
               x = 0.5d0 * ( curves(ncount) + curves(n) )
               call plafep ( 9, x, 0d0, 0.2d0, 0.2d0, icurve )
            end if
            ncount = n+1
300      continue
      end if
      if ( iplusp.gt.0 ) then

!     --- Plot user points

         if ( iscolr.gt.0 ) dcolor = 4+iscolr
         do 350 j = 1, nuspnt
            call plafep ( 6, rinput(j), 0d0, 0d0, 0d0, 0 )
350      continue
      end if
      if ( iplusp.gt.1 ) then

!     --- Plot numbers at user points

         if ( iscolr.gt.0 ) dcolor = iscolr
         do 360 j = 1, nuspnt
            call plafep ( 9, rinput(j), 0d0, 0.2d0, 0.2d0, j )
360      continue
      end if

!     --- close plot file

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      jkader = ikader
      call erclos ( 'pltbn1' )
      dcolor = iscolr
      end
