      subroutine pltext ( x, y, text, angle, height )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 01-09-2006 Allow plot coordinates
!        version  1.2  date 09-01-1993 New call of plaf01
!        version  1.1  date 10-05-1990 changed layout of common cplaf
!
!   copyright (c) 1989-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Plot text at position given by the user in user co-ordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, angle, height
      character (len=*) text

!     angle     i   The text is plotted under an angle of ANGLE degrees.
!                   Double precision parameter
!     height    i   The height of the text is equal to HEIGHT centimeters.
!                   Double precision parameter
!     text      i   Text to be plotted. Character variable
!                   The text may contain at most 80 characters
!     x         i   x-co-ordinate of the at most left under point of
!                   the first character. The pen moves with pen up to (X,Y).
!                   Double precision parameters.
!                   x,y are given in user co-ordinates
!     y         i   y-co-ordinate see x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xp, yp, xpboun, ypboun
      integer nchar, iside

!     iside   Indicates at which side the cross section is positioned
!             Possible values:
!             0: There is no cross section point computed in this call
!             1: The cross section point is at side y=ymint
!             2: The cross section point is at side x=xmaxt
!             3: The cross section point is at side y=ymaxt
!             4: The cross section point is at side x=xmint
!     nchar   Number of characters in a word
!     xp      x-coordinate in plotting units of at most left character
!             (under side)
!     xpboun  x-coordinate of point at the boundary of the plotting region
!     yp      y-coordinate in plotting units of at most left character
!             (under side)
!     ypboun  y-coordinate of point at the boundary of the plotting region
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAF01  Translate user coordinates into plotter coordinates
!     PLAFP6  Standard machine dependent plotting
!     PLAFP7  Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute plotting co-ordinates and check if point is within plotting
!         region

      nchar = len ( text )
      if ( iinputpst(102)==0 ) then

!     --- Plot user coordinates

         call plaf01 ( x, y, xp, yp, xpboun, ypboun, iside )
         call plafp6 ( 3, xp, yp )

      else

!     --- Plot fixed coordinates

         xp = x
         yp = y
         ioutsd = 1
         idirec = 1

      end if  ! ( iinputpst(102)==0 ) then

      if ( ioutsd<=1 ) then

!     --- ioutsd<2, so new point is within plotting region

         if ( idirec==1 ) then

!        --- idirec = 1, normal direction

            call plafp7 ( xp, yp, text, angle, height, nchar )

         else

!        --- idirec = -1, rotated direction

            call plafp7 ( xp, yp, text, angle-90d0, height, nchar )

         end if

      end if
      end
