      subroutine pltintcv ( icurvs, numcrv, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-10-2002 New storage of composite curves
!        version  1.0  date 28-05-1999
!
!   copyright (c) 1999-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array iwork with a 0 for all curves that must be plotted and with
!    a 1 for all curves that must be skipped
!    A curve must be plotted if it is an outer curve or a free surface curve
!    Composite curves itself are skipped, but their subcurves may be plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     curve
!     plot
!     selection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), numcrv, iwork(*)

!     icurvs     i    array containing the curve numbers of the free boundary
!     iwork      o    integer array of length ncurvs
!                     If iwork(i)=0 the curve must be plotted, &
!                     iwork(i)=1 not
!     numcrv     i    number of curves in array icurvs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, icurnr, isubcv, nsubcv, istart

!     i          Counting variable
!     icurnr     Curve sequence number
!     istart     First position used at a part
!     isubcv     Subcurve sequence number
!     j          Counting variable
!     nsubcv     Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Set iwork equal to 1 for each composite curve
!     Skip internal curves, except for free surface curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array iwork

      iwork(1:ncurvs) = 0
      if ( kelmq>0 ) then

!     --- There are subcurves
!         Set iwork equal to 1 for each composite curve

         do i = 1, ncurvs
            if ( kmeshq(i+1)-kmeshq(i)>0 ) then

!           --- curve i contains subcurves

               iwork(i) = 1

            end if
         end do
      end if

!     --- Skip internal curves

      ninner = kmeshm(1)
      if ( ninner>0 ) then

!     --- ninner > 0, so there are internal curves

         istart = kmeshm(3)-1
         do i =1, ninner
            iwork(abs(kmeshm(istart+i))) = 1
         end do

!        --- However, curves in the free surface must be plotted

         do i = 1, numcrv
            icurnr = icurvs(i)
            if ( kelmq>0 ) then

!           --- There are subcurves

               nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
               if ( nsubcv==0 ) then

!              --- Curve is stand alone

                  iwork(icurnr) = 0

               else

!              --- Curve consists of subcurves

                  do j = 1, nsubcv
                     istart = kmeshq(icurnr)
                     isubcv = kmeshq(istart+j)
                     iwork(isubcv) = 0
                  end do

               end if

            else

!           --- There are no subcurves

               iwork(icurnr) = 0

            end if

         end do

      end if

      end
