      subroutine pltjrm ( hptr0, hptr1 )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjrm removes a segment and returns it to the free list
!       hptr0 points to hptr1 and has to be updated
!       if hptr0 = nil then hptr1 is the start of the list
!       On exit hptr1 contains the pltjca of hptr1 on entry. (that is the
!       pointer to the next segment.)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer hptr0, hptr1

!     hptr0   i    predecessor cell of segment to be removed. Its adrg will
!                  be updated.
!     hptr1  i/o   header cell of segment to be removed. If hptr0 = &
!                  nil hptr1 contains the new startaddress on exit
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer optr

!     optr    auxiliary variable to temporarily store a pointer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjca

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     PLTJAD  to change the segment pointer
!     PLTJCA  to get the next segment
!     PLTJFS  to return a segment to the freelist
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1268   pltjrm called with Empty segment.
!    1271   hptr1 is not the segment successor of hptr0
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjrm')
      if (hptr1.eq.nil) then
         call errsub (1268, 0, 0, 0)

!        --- Error pltjrm called with Empty segment.

      end if
      if (hptr0.eq.nil) then

!     --- This is the first segment of this list. The new start address
!         of the list will be in hptr1 on exit.

      call pltjfs (hptr1)
      else if (hptr1.eq.pltjca (hptr0)) then

!     --- Not the first segment of the list. hptr1 must be the (segment)
!         successor of hptr0 and adrg (hptr0) will be updated to adrg (hptr1).
!         NOTE: on exit from freesg hptr1 contains adrg (hptr1)

        call pltjfs (hptr1)
        call pltjad (hptr0, hptr1, optr)
      else
        call errint (hptr1, 1)
        call errint (pltjca (hptr0), 2)
        call errsub (1271, 2, 0, 0)

!       --- Error hptr1 is not the segment successor of hptr0

      end if
      call erclos ('pltjrm')
      end
