      subroutine prcontourcomp3 ( markelems, area, nelem, kmeshc, &
                                  coor, kmesho, plotpos, numelms, &
                                  activeelems, icolor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all neighboring elements of element corresponding to the
!     same region satisfying function value>specified value
!     Compute area and plot elements if necessary
!     The mesh must consist of linear trangles only
!     Special case of element completely in region
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     area
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, markelems(nelem), kmeshc(3), kmesho(3), &
              numelms, activeelems(nelem)
      double precision area, coor(2,*)
      logical plotpos

!     activeelems   i/o   Contains the list of still active elements
!                         These elements are neighbors of the elements in the
!                         bubble
!     area          i/o   Area of bubble
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icolor         i    Sequence number of colour
!     kmeshc         i    kmesh part c, contains the nodal point numers of the
!                         element
!     kmesho         i    kmesh part o, contains the neighbor elements of the
!                         element
!     markelems     i/o   See subroutine prcontourcomp
!     nelem          i    Number of elements
!     numelms       i/o   Number of elements in activeelems
!     plotpos        i    If true the bubble must be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ineigh, icolor
      double precision polyg(2,3), arealoc

!     arealoc        Area of local triangle
!     i              Counting variable
!     ineigh         neighbor
!     polyg          Array containing the co-ordinates of the point in the
!                    polygon in the following way:
!                    polyg(1,i) contains the x-coordinate of the first
!                    point
!                    polyg(2,i) contains the y-coordinate of the first
!                    point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshareatria

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHAREATRIA    Return with area of triangle, with nodes given in kmeshc
!     PLAF05         Plot polygon
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark element and all its non-marked neighbors with num_bubbles
!     Compute area
!     Plot element if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcontourcomp3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcontourcomp3'
         write(irefwr,2) 'area', area
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- All three nodes belong to bubble

      arealoc = mshareatria ( kmeshc, coor )
      area = area + arealoc
      if ( debug ) &
         write(irefwr,2) 'area, arealoc', area, arealoc

!     --- Find non-marked neighbors

      do i = 1, 3

         ineigh = mod(kmesho(i),nelem+1)
         if ( ineigh>0 ) then

!        --- neighbor found

            if ( markelems(ineigh)==0 ) then

!           --- new neighbor, add to list

               numelms = numelms+1
               activeelems(numelms) = ineigh
               markelems(ineigh) = -2

            end if  ! ( markelems(ineigh)==0 )

         end if  ! ( ineigh>0 )

      end do  ! i = 1, 3

      if ( plotpos ) then

!     --- Plot element with one colour

         polyg(1:2,1:3) = coor(1:2,kmeshc(1:3))
         call plaf05 ( 1, icolor, 3, polyg, 2 )

      end if  ! ( plotpos )

1000  call erclos ( 'prcontourcomp3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'area', area
         write(irefwr,*) 'End prcontourcomp3'

      end if  ! ( debug )

      end
