      subroutine prplotfn2 ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot function along line in surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     plot_function
!     driving_subroutine
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotfn
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     SEPFILLFUNCY   Fill array funcy  with function values for a series
!                    of sepran vectors
!     SEPFILLTEXT    Fill array loctext with the text to be used along axis
!     SEPINTPOLLINE  Compute intersection of line (p1,p2) with surface S1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     752   No intersection found of line with mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute intersection of line p1, p2 with surface s1
!     if ( intersection contains  more than 1 point ) then
!        Compute function values in intersection
!        Fill funcx and funcy
!        Make function plot
!     else
!        error: no intersection found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotfn2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotfn2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numelm = 0

!     --- Compute intersection of line p1, p2 with surface s1
!         Allocate funcx and funcy and fill funcx

      call sepintpolline ( inpsep, rinsep )

      if ( numelm==0 ) then

!     --- No intersection found

         call errwar ( 752, 0, 0, 0 )
         go to 1000

      else

!     --- intersection contains  more than 1 point
!         Compute function values in intersection
!         Fill funcy

         call sepfillfuncy ( inpsep(istartvecs) )

!        --- Fill text along axis

         call sepfilltext ( textaxis, inpsep, rinsep, numvecs, &
                            inpsep(istartvecs) )
         nnodes = numint

      end if  ! ( numint<=1 )

1000  call erclos ( 'prplotfn2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotfn2'

      end if  ! ( debug )

      end subroutine prplotfn2
