      subroutine sepmappnt3d2d ( x3d, trans, x2d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map coordinates stored in x3d onto surface and store in x2d
!     using the tranformation stored in trans
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: x3d(3), trans(3,3)
      double precision, intent(out) :: x2d(2)

!     trans          i    array containing the inverse of the transformation
!     x2d            o    transformed (2d) coordinates of point
!     x3d            i    coordinates of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmappnt3d2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmappnt3d2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do k = 1, 2
         x2d(k) = sum(x3d(:)*trans(k,:))
      end do  ! k = 1, 2

1000  call erclos ( 'sepmappnt3d2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmappnt3d2d'

      end if  ! ( debug )

      end subroutine sepmappnt3d2d
