      subroutine sepplot3dobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  subroutine sepplot3dobst is used to plot a three-dimensional mesh with
!  colors. In fact this subroutine creates a hidden line plot on a screen.
!  Remark:  this subroutine is only fitted for display plotting, not
!           for a pen plotter.
!           Subroutine PLAFP8 (fill colored surfaces) must have been
!           implemented for the plot package
!     Old subroutine plot3dobst
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     color
!     mesh
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jv, iplots(13), &
              iresul(5), iskip, ipline, neleml, iorien, i, irotation, &
              numrotations, jtimessav, iinput(4), ilevel, nelemlc, &
              nlevels, iscolor, npelmlc, idummy
      integer :: kmeshalc(1), kmeshglc(1)
      integer, allocatable :: elements(:), elmlevels(:)
      double precision dum(3), xmean(3), plotfm, yfact, &
                       viewpoint(3), viewp(3)
      logical perspc, uselevels, plotobstacle
      double precision, allocatable :: coornw(:)
      double precision :: timebefore(2)

!     coornw         coordinate array for projected points
!                    (1,i) contains the x-coordinate of the projected point i
!                    (2,i) contains the y-coordinate of the projected point i
!                    (3,i) contains the distance of point i to
!                          the view-point
!     dum            General dummy double precision array
!     elements       Array of length levels(nlevels) (<=nelem)
!                    containing the reordered element sequence numbers
!                    So elements(1) contains the first element number to
!                    be used, elements(2) the second one and so on
!     elmlevels      Array of length nlevels (<nelem) containing the
!                    levels in array elements
!                    The first level consists of the elements 1 to
!                    elmlevels(1) of array elements
!                    The second level contains the elements  elmlevels(1)+1
!                    to elmlevels(2) and so on
!     i              Counting variable
!     idummy         Dummy parameter
!     iinput         integer input array for subroutine TOPLEVELS
!     ilevel         Defines level sequence number for plotting
!     iorien         Direction of rotation:
!                     Possible values:
!                     1:  rotate in x-y plane
!                     2:  rotate in y-z plane
!                     3:  rotate in z-x plane
!     ipline         Starting address of local "line"-elements array
!     iplots         Integer array in which the following quantities must be
!                    stored:
!                     1:  Position of last item that has been filled by the user
!                     2:  choice parameter for type of projection
!                         Possible values:
!                          0 : parallel projection plot
!                          1 : perspective projection plot
!                         10 : parallel projection plot (View point automatic)
!                         11 : perspective projection plot (View point
!                              automatic)
!                     3:  color number of farthest triangle
!                     4:  color number of nearest triangle
!                         Default value: 16
!                     5:  Indication if boundaries must be plotted (1) or not
!                         (0)
!                     6:  Indication whether x-y is basic plane (1), z-x is
!                         basic plane (2) or y-z is basic plane (3)
!                     7:  iskip:  Indication if element groups must be skipped
!                                 (1) or not (0)
!                     8:  numrotations:  Number of rotations
!                     9:  Use specific volumes stored from position 15 (>0)
!                         or not (0)
!                    10:  Plot elements per layer (1) or not (0)
!                    11:  idefstart, defines how the start level must
!                         be computed
!                         1: Start with set of nodes with largest
!                            distance to view point
!                         2: Start with set of surfaces indicated in
!                            positions 12 and 13
!                    12:  ifirst, first surface number
!                    13:  ilast, last surface number
!                    15...14+nelgrp:  ihelp  Integer input array of length
!                                          NELGRP that is only used if ISKIP = 1
!                                          It indicates if element group ielgrp
!                                          must be skipped ihelp(ielgrp) = 0
!                                          or not ihelp(ielgrp) = 1
!     iresul         Output array of length 5
!                    Contents
!                    1: -
!                    2: npelmw
!                    3: nelemlc
!                    4: if > 0, array kmeshcsub has been allocated
!                    5: if > 0, array kmeshctri has been allocated
!     irotation      Loop variable for rotation
!     iscolor        Help variable to store dcolor
!     iskip          dummy
!     jtimessav      Help variable to save the value of jtimes
!     jv             Indicates if the view point has been filled (1) or not (0)
!     kmeshalc       kmesh part a
!     kmeshglc       kmesh part g
!     neleml         Number of "line"-elements in mesh
!     nelemlc        Local number of elements
!     nlevels        Number of entries in array elmlevels
!     npelmlc        Local number of nodes per element
!     numrotations   Number of rotations
!     perspc         indicates if a perspective (true) or a parallel projection
!                    (false) must be applied
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     plotobstacle   If true the obstacle will be or has been plotted
!     uselevels      If true plotting per level must be performed
!     viewp          Array containing the first view point
!     viewpoint      Array containing actual view point
!     xmean          Array containing the mean values of the co-ordinates
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0014         Compute projection of points onto projection plane
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLFILLVIEWP    Fill array viewpoint depending on the rotation
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTSFOBST     Make a three-dimensional colored plot of a mesh with an
!                    obstacle
!     SEPLEVELS      Reorder a restricted set of elements in an ordered
!                    level set
!     SEPOBST        Mark all elemets that are inside and outside the obstacle
!     SEPPLCOMPVIEW  Compute the viewpoint for the plotting of the obstacles by
!                    looking at the line through centroid of starting surface
!                    and centroid of volume
!     SEPQU0OBST     Check if a 3d mesh with obstacles must be subdivided
!                    into sub elements
!     TOPELEMSKIP    Fill array elemskip with information about elements to be
!                    skipped and elements to be plotted
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check input
!   Copy iplots into iplots and fill defaults (PL0011)
!   Activate permanent arrays
!   Create surface vertex elements only
!   Perform actual procedure (PLOTSF)
!   Plot the obstacle separately
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplot3dobst' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplot3dobst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define plot parameters

      iplots(1) = 13
      iplots(2) = 0
      iplots(3) = 20
      iplots(4) = 20
      iplots(5) = 1
      iplots(6) = 1
      iplots(7) = 0
      iplots(8) = num_of_rotat
      iplots(9) = 0
      iplots(10) = 1
      iplots(11) = 2
      iplots(12) = startsurface
      iplots(13) = startsurface
      plotfm = 15d0
      yfact = 1d0

!     --- Compute viewpoint

      if ( debug ) write(irefwr,*) 'before plcompview'
      call sepplcompview ( startsurface, viewpoint )

!     --- Define work space and activate permanent arrays

      if ( iplots(7)>0 ) then
         iskip = 1
      else if ( iplots(9)>0 ) then
         iskip = -1
      else
         iskip = 0
      end if
      uselevels = iplots(10)>0
      nlevels = 1

      allocate ( elemskip(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'elemskip' )
      if ( ierror/=0 ) go to 1000

      if ( uselevels ) then

!     --- Plot per level
!         Call subroutine toplevels to compute the levels

         allocate ( elmlevels(nelem), elements(nelem), stat = error )
         if ( error/=0 ) call eralloc ( error, 2*nelem, 'elmlevels' )
         if ( ierror/=0 ) go to 1000

!        --- Actual call

         iinput(1) = iplots(7)
         iinput(2) = iplots(11)
         iinput(3) = iplots(12)
         iinput(4) = iplots(13)
         if ( debug ) write(irefwr,*) 'before toplevels'
         call seplevels ( viewpoint, iskip, elements, &
                          elmlevels, nlevels, iinput )

!        --- Create array elemskip

         if ( debug ) write(irefwr,*) 'before topelemskip'
         call topelemskip ( nlevels, elmlevels, elements )

         deallocate ( elmlevels, elements, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'elmlevels' )

      end if
      jtimessav = jtimes
      plotobstacle = .false.

      allocate ( coornw(3*npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'coornw' )
      if ( ierror/=0 ) go to 1000

!     --- Loop over levels

100   do ilevel = 1, nlevels

         if ( debug ) write(irefwr,*) 'before topqu0obst'
         call sepqu0obst ( iskip, idummy, iresul, ilevel )
         if ( ierror/=0 ) go to 1000

!        --- Compute actual plot

         npelmlc = iresul(2)
         nelemlc = iresul(3)
         if ( nelemlc==0 ) go to 200
         nelgrp = 1

         kmeshalc(1) = npelmlc
         kmeshglc(1) = nelemlc
         jv = 1 - iplots(2)/10
         perspc = mod(iplots(2),10)==1
         iorien = iplots(6)
         numrotations = iplots(8)

!        --- Loop over the rotations

         if ( numrotations>1 .or. nlevels>1 ) then

!        --- More than one rotation, separate pictures required

            if ( jtimes>10 ) jtimes = jtimes-10
            if ( jtimes==2 .or. jtimes==3 ) then
               jtimes = 3
            else
               jtimes = 0
            end if

         end if

         do irotation = 1, numrotations

            if ( numrotations==1 .and. ilevel==nlevels ) then
               jtimes = jtimessav
            else if ( irotation==numrotations .and. &
                      jtimessav>=10 .and. ilevel==nlevels ) then
               jtimes = 11
            end if

            if ( debug ) write(irefwr,*) 'before plfillviewp'
            call plfillviewp ( irotation, jv, viewp, viewpoint, iorien, &
                               numrotations )

            if ( debug ) write(irefwr,*) 'before pl0014'
            call pl0014 ( coor, viewpoint, jv, coornw, &
                          npoint, xmean, perspc, 3, dum, 1, plotfm, &
                          yfact, iorien )
            if ( irotation==1 .and. jv==0 ) then

!           --- First step and view point has not yet been filled

               do i = 1, 3
                  viewp(i) = viewpoint(i)
               end do

            end if

!           --- PLot some texts

            iscolor = dcolor
            if ( plotobstacle ) then

               call plafp7 ( 15d0, 19.5d0, 'obstacle', 0d0, &
                             0.25d0, 8 )

            else

!           --- Standard plot

               call plafp7 ( 15d0, 19.5d0, 'one row of elements', 0d0, &
                             0.25d0, 19 )
               dcolor = 20
               call plafp7 ( 15d0, 19d0, '-- external', 0d0, 0.25d0, &
                             11 )
               dcolor = 15
               call plafp7 ( 15d0, 18.5d0, '-- internal', 0d0, 0.25d0, &
                             11 )
               dcolor = 63
               call plafp7 ( 15d0, 18d0, '-- boundary', 0d0, 0.25d0, &
                             11 )
               dcolor = iscolor

            end if

            neleml = 0

            if ( debug ) write(irefwr,*) 'before plotsfobst'
            call plotsfobst ( coornw, npoint, kmeshalc, &
                              kmeshglc, nelgrp, kmeshcsub, iplots, &
                              neleml, idummy, kmeshctri )

            if ( mod(jtimes,10)<1 .or. mod(jtimes,10)>2 ) then
               call plmore
               call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
            end if

         end do

200   end do

!     --- Next plot the obstacle separately
!         First fill elemskip with a 1 for all elements inside the obstacle

      if ( .not. plotobstacle ) then

!     --- Plot obstacle is false, obstacle has not yet been plotted
!         Set nlevels = 1

         nlevels = 1
         plotobstacle = .true.

!        --- Fill array elemskip with a 0 for all elements outside the
!            obstacle and a 1 inside the obstacle

         if ( debug ) write(irefwr,*) 'before topobst'
         call sepobst
         jtimes = 0

!        --- Repeat the plotting

         go to 100

      end if

      deallocate ( elemskip, coornw, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'elemskip' )

1000  if ( ierror/=0 ) then

!     --- ierror # 0  Reset ierror and nerror
!                     Return to main program

         ierror = 0
         nerror = 0
         if ( jtimes==11 ) jtimes = 1
         go to 1010

      end if

      if ( iwarn/=0 ) then

!     --- iwarn # 0  Reset iwarn and nwarn
!                    Return to main program

         iwarn = 0
         nwarn = 0

      end if

1010  if ( iresul(4)==1 ) deallocate ( kmeshcsub )
      if ( iresul(5)/=0 ) deallocate ( kmeshctri )
      jtimes = jtimessav
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplot3dobst'

      end if  ! ( debug )
      call erclmn ( 'sepplot3dobst', timebefore )

      end subroutine sepplot3dobst
