      subroutine sepplotcrossob
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the contents of the arrays icrossobst and rcrossobst
!     Old subroutine plotcrossobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
!     intersection
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, istarticross, numintersect, iedge, &
              numnodes, istart, inodp1, inodp2, dcolor
      double precision fact, x, y, x1, y1, x2, y2

!     dcolor         Colour sequence number
!     fact           factor
!     i              Counting variable
!     iedge          edge sequence number
!     inodp1         First node
!     inodp2         Second node
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     numintersect   Number of intersections
!     numnodes       Number of nodla points in array nodes
!     x              x co-ordinate of point
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     y              y co-ordinate of point
!     y1             y-coordinate of first point
!     y2             y-coordinate of last point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do iobst = 1, numobst

         istarticross = icrossobst(iobst)
         numintersect = icrossobst(istarticross)
         do i = 1, numintersect

            iedge = icrossobst(istarticross+1)
            fact = rcrossobst(icrossobst(istarticross+2))
            istarticross = istarticross+2
            istart = nmeshc(iedge)
            numnodes = nmeshc(istart)
            inodp1 = nmeshc(istart+1)
            inodp2 = nmeshc(istart+numnodes)
            x1 = coor(1,inodp1)
            y1 = coor(2,inodp1)
            x2 = coor(1,inodp2)
            y2 = coor(2,inodp2)
            x = x1 + fact*(x2-x1)
            y = y1 + fact*(y2-y1)
            dcolor = 14
            call plafep ( 6, x, y, 0d0, 0d0, dcolor )

         end do

      end do
      call plafp7 ( 5d0, 19d0, 'intersection points', 0d0, 0.25d0, 19 )
      end
