      subroutine sepplotcu ( iplots, rplots )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 23-09-1998 Extra parameters ibuffr, buffer
!        version  1.3  date 06-09-1997 New call to pl0003
!
!   copyright (c) 1989-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    this subroutine plots the curves or the user points of the mesh
!    when ndim = 2
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(10)
      double precision rplots(2)

!    iplots    i   Integer array to be filled by the user with the following
!                  information:
!                  1:  Indication if user points (1) or curves must be
!                      plotted (2)
!                  2:  IDOUBL
!                  3:  IINNER
!                  4:  NUMBER
!                  5:  ISYMBL
!                  6-10:  Not yet used   (Fill zeros for future developments)
!                  with:
!                  idoubl   indication whether the boundaries are plotted with
!                           double lines (idoubl = 1) or not (idoubl = 0)
!                  iinner   Information of how the curves must be plotted
!                    1      only the outer boundaries are plotted
!                    2      only the inner boundaries are plotted
!                    3      all boundaries are plotted
!                  number   Indication if the curves must be provided with
!                           a curve number (1) or not (0)
!                  isymbl   Indicates if the nodes along the curves must be
!                           marked with a cross (1) or not (=0)
!    rplots    i   Array to be filled by the user with the following quantities:
!                  1:   PLOTFM  (length of the plot in centimeters)
!                  2:   YFACT   (multiplication factor for the y-co-ordinates)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikader

!     ikader     Help variable to store jkader temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTBN         Plot boundary of region
!     PLOTUS         Plot user points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!     825   IPLOTS(1) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotcu' )

!     --- Compute starting addresses of arrays

      ikader = jkader
      if ( ndim/=2 ) then

!     --- error 62:  NDIM # 2

         call errint ( ndim, 1 )
         call errsub ( 62, 1, 0, 0 )
         ierror = 0

      end if

!     --- Start plot:

      call pl0003 ( ikader, 1, rplots(1), rplots(2), 0 )

      if ( iplots(1)==1 ) then

!     --- iplots(1) = 1, plot user points

         call plotus ( kmeshl, coor, nuspnt )

      else if ( iplots(1)==2 ) then

!     --- iplots(1) = 2, plot curves

         call plotbn ( iplots(2), &
                       iplots(3)+10*iplots(4)+100*iplots(5), .false. )

      else

!     --- Error 825: iplots(1) has wrong value

         call errint ( iplots(1), 1 )
         call errsub ( 825, 1, 0, 0 )
         ierror = 0

      end if

!     --- Close plot file

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      jkader = ikader

      call erclos ( 'sepplotcu' )
      end
