      subroutine creatmeanval ( usol, usolloc, ielem1, ielem2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute mean value of scalar in element and store in usolloc
!     At this moment only for vectors with 1 degree of freedom in each point
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector_defined_per_element
!     centre
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielem1, ielem2
      double precision usol(*), usolloc(*)

!     ielem1         i    First element number in group
!     ielem2         i    Last element number in group
!     usol           i    Solution vector with respect to all nodes
!     usolloc        o    Solution vector defined per element
!                         This is the result of the averaging
!                         The first position refers to element ielem1, &
!                         the last one to ielem2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inpelm,  ielgrp,  ielem, inode, j
      double precision val

!     ielem          Element number
!     ielgrp         element group number
!     inode          Node sequence number
!     inpelm         number of points per element
!     j              Counting variable
!     val            Sum of the solutoin over all nodes in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3021   elements in more than one element group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatmeanval' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatmeanval'
         write(irefwr,1) 'ielem1, ielem2', ielem1, ielem2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem = 0             ! element number

!     --- For all element groups do

      kelmc = 0             ! last address used in kmeshc
      do ielgrp = 1, nelgrp

!     --- inpelm := number of nodes in element group
!         inelem := number of elements in element group

         inpelm = sepgetinpelm ( ielgrp )
         inelem = sepgetinelem ( ielgrp )

         if ( debug ) &
            write(irefwr,1) 'inpelm, inelem, ielgrp, kelmc, ielem', &
                             inpelm, inelem, ielgrp, kelmc, ielem

         if ( ielem1<ielem+inelem ) then

!        --- Element is in correct range

            if ( ielem2>ielem+inelem ) then

!           --- Set of elements belongs to more than one element group
!               not yet implemented

               call errint ( ielem1, 1 )
               call errint ( ielem2, 2 )
               call errsub ( 3021, 2, 0, 0 )
               go to 1000

            end if  ! ( ielem2>ielem+inelem )

!           --- Raise kmeshc in case ielem1 is not first element of
!               element group

            kelmc = kelmc+(ielem1-ielem-1)*inpelm
            ielem = ielem+inelem         ! raise ielem before changing inelem
            inelem = ielem2-ielem1+1
            if ( debug ) &
               write(irefwr,1) 'inpelm, inelem, kelmc', &
                                inpelm, inelem, kelmc
            go to 100   ! jump since element group has been found

         else

!        --- Element group not yet reached

            kelmc = kelmc+inelem*inpelm
            ielem = ielem+inelem

         end if  ! ( ielem1>ielem )

      end do  ! ielgrp = 1, nelgrp

!     --- Compute mean value

100   do ielem = 1, inelem

         val = 0d0

         do j = 1, inpelm

            inode = kmeshc(kelmc+j)
            val = val + usol(inode)

         end do  ! j = 1, inpelm
         usolloc(ielem) = val/inpelm
         kelmc = kelmc+inpelm

      end do  ! ielem = 1, inelem

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usolloc, inelem, 'usolloc' )
         write(irefwr,*) 'End creatmeanval'

      end if  ! ( debug )
      call erclos ( 'creatmeanval' )

      end
