      subroutine elcsubsint
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 31-11-2016 Complete revision
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 11-10-2005
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call specific element subroutine, depending on itype
!     The output vector is integer
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element
!     element_group
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELD807INT   Fill integer element derivative vector for
!                    temperature equation
!     SEPELD810INT   Fill integer element derivative vector for
!                    enthalpy equation
!     SEPELDEFAULT   Set default values for parameters in module sepmodulebasefn
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elcsubsint' )

      E%symm = symm

      if ( ifirst==0 ) call sepeldefault

      select case ( itype )

         case (0)

!        --- Type 0, skip

         case (807)

!        --- Type 810 :

            call sepeld807int

         case (810)

!        --- Type 810 :

            call sepeld810int

         case default

!        --- Type<0 or>999
!            These types are not available

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 43, 2, 0, 0)

      end select  ! case ( itype )

      ifirst = 1
      call erclos ( 'elcsubsint' )
      end
