      subroutine er0000 ( usol, ichoice, &
                          icrv1, icrv2, idegfd, jstap, value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 14-10-2002 Extra parameters kmesh, kmeshq
!        version  7.5  date 15-10-1999 New call to er0026
!        version  7.4  date 26-12-1994 New commons cmessc and cmessg
!
!   copyright (c) 1984-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for curves      one component only
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer ichoice, icrv1, icrv2, idegfd, jstap
      double precision usol(*), value

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichoice         i    Choice parameter
!     icrv1          i    First curve number
!     icrv2          i    Last curve number
!     idegfd         i    Degree of freedom in VECTOR(IVEC) that must be used
!     jstap          i    ?
!     usol          i/o   Solution vector
!     value          i    Array of length m in which the value of the
!                         coefficient is stored if the coefficient is a constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jstapo, iexclu, kstap, inpelm, n, kstart, kend, i, &
              nodenr, j, inusol
      integer, allocatable :: iwork(:)

!     i       Counting variable
!     iexclu  Indicates which points are excluded
!     inpelm  number of points per element
!     inusol  Position in usol
!     iwork   integer work array containing the nodal point numbers
!             of the curves
!     j       Counting variable
!     jstapo  ?
!     kend    End value of parameter
!     kstap   ?
!     kstart  Start value of parameter
!     n
!     nodenr  Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
!     SEPINIBVL      actual filling of essential boundary condition for one point
!                    and one degree of freedom only
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       6   no nodal points are created along the curve.
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0000' )

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      jstapo = jstap
      if(jstapo>0) then

!     --- jstapo > 0,  compute iexclu

         iexclu=jstapo/100
         if(iexclu>3) then
            call errint ( iexclu, 1 )
            call errint ( jstapo, 2 )
            call errsub ( 558, 2, 0, 0 )
         end if
         jstapo=jstapo-100*iexclu
      else
         iexclu=0
      end if
      kstap=abs(jstapo)
      inpelm=kstap+1
      call seper0026 ( icrv1, icrv2, inpelm, n, iwork, .false., kmeshm )
      if ( jstapo<0 ) goto 200

!     --- jstapo >= 0   end points of intervals
!         Correct for initial and end point in the case that exclude > 0

      if(iexclu==0) then

!     --- iexclu = 0  No excluding of nodal points

         kstart=1
         kend=n
      else if(iexclu==1) then

!     --- iexclu = 1 Exclude initial point

         kstart=2
         kend=n
      else if(iexclu==2) then

!     --- iexclu = 2 Exclude end point

         kstart=1
         kend=n-1
      else if(iexclu==3) then

!     --- iexclu = 3 Exclude initial and end point

         kstart=2
         kend=n-1
      end if
      do i=kstart, kend, inpelm
         nodenr=iwork(i)
         if(nodenr==0) then
            call errint ( i, 1 )
            call errsub ( 6, 1, 0, 0 )
            go to 1000
         end if
         inusol = nodenr
         if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
         if ( inusol>0 ) &
            call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                             value, compl )
      end do
      go to 1000

!     --- jstapo < 0

200   do i=1, n-1, inpelm
        do j=1, kstap
           nodenr=iwork(i+j)
           if(nodenr==0) then
              call errint ( i+j, 1 )
              call errsub ( 6, 1, 0, 0 )
           go to 1000
           end if
           inusol = nodenr
           if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
           if ( inusol>0 ) &
              call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                               value, compl )
        end do
      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'er0000' )
      end
