      subroutine er0064 ( vector, n, rincre, kfun, ivec, kdeg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 16-05-1999 Extra parameters ivec, kdeg
!        version  2.0  date 19-11-1995 Extension with function, &
!                                      extra parameters
!        version  1.0  date 23-01-1992
!
!   copyright (c) 1992-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill value into array of special structure defined
!     per element   (Real case)
!     The complex equivalent of this subroutine is ER0073
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, kfun, ivec, kdeg
      double precision vector(*), rincre(*)

!     ivec    i    Number of degrees of freedom per element or node
!     kdeg    i    Indication which degrees of freedom must be filled
!                  Possibilities:
!                  0:    All degrees of freedom
!                  >0:   Only the KDEGth degree of freedom
!                  <0:   The -KDEG degrees of freedom stored in array IARDEG
!                        must be filled
!     kfun    i    Indication of how the degrees of freedom must be filled.
!                  Possibilities:
!                  0:  Not filled
!                 <0:  Constant value stored in RINCRE(-KFUN)
!                 >0:  Function subroutine
!     n       i    Number of elements in vector
!     rincre  i    Double precision array containing the values of the constant
!                  if KFUN < 0
!     vector i/o   Array of length n to be (partly) filled by this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,  inpelm, ikelmc, ielem, ielgrp, irelem, nodp
      double precision consta, xc, yc, zc, v

!     consta  Help variable to store the constant
!     i       Counting variable
!     ielem   Element number
!     ielgrp  element group number
!     ikelmc  position in kmesh part c
!     inpelm  number of points per element
!     irelem  Relative element sequence number with respect to element group
!     nodp    Nodal point sequence number
!     v       Help variable to store a value
!     xc      x-coordinate of the centroid
!     yc      y-coordinate of the centroid
!     zc      z-coordinate of the centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision func

!     FUNC    Computes the value of the vector in a point as function of the
!             co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( kfun.lt.0 ) then

!     --- kfun < 0, function is constant

         consta = rincre(-kfun)
         if ( kdeg.eq.0 ) then

!        --- All degrees of freedom

            do i = 1, n
               vector(i) = consta
            end do
         else

!        --- One degree of freedom

            do i = kdeg, n, ivec
               vector(i) = consta
            end do
         end if
      else

!     --- kfun > 0, hence function of the co-ordinates

         ikelmc = 0
         ielem  = 1
         do ielgrp = 1, nelgrp
            inpelm = sepgetinpelm ( ielgrp )
            inelem = sepgetinelem ( ielgrp )
            do irelem = 1, inelem
               xc = 0d0
               yc = 0d0
               zc = 0d0
               do i = 1, inpelm
                  nodp = kmeshc(ikelmc+i)
                  xc = xc + coor(1,nodp)
                  if ( ndim.gt.1 ) yc = yc + coor(2,nodp)
                  if ( ndim.gt.2 ) zc = zc + coor(3,nodp)
               end do
               v = func ( kfun, xc/inpelm, yc/inpelm, zc/inpelm )
               if ( kdeg.eq.0 ) then

!              --- All degrees of freedom

                  do i = 1, ivec
                     vector((ielem-1)*ivec+i) = v
                  end do
               else

!              --- None degrees of freedom

                  vector((ielem-1)*ivec+kdeg) = v
               end if
               ikelmc = ikelmc + inpelm
               ielem  = ielem  + 1
            end do
         end do
      end if
      end
