      subroutine ercreatstand1 ( kfill, nnodes, inpcre, jpstin, kfun, &
                                 rincre, ipstin, icompl, kdeg, kvec, &
                                 isolut, maxphys, mvec, &
                                 istartvc, islold, bndcon, ivecold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-03-2014 extension with vector expression
!        version  1.0  date 09-01-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one vector for one option only
!     Standard case, the nodes have been filled in nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfill, nnodes, &
              inpcre(*), jpstin, kfun, ipstin, maxphys, &
              icompl, kdeg, kvec, isolut(*), mvec, &
              islold(*), istartvc, ivecold
      double precision rincre(*)
      logical bndcon

!     bndcon         i    If true boundary conditions must be filled, otherwise
!                         a vector must be created
!     icompl         i    Indication if the vector to be created is real (0) or
!                         complex
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin         i    Starting address in array INPCRE
!     islold         i    User input array in which the user puts information
!                         of all preceding solutionss
!                         Integer array of length 5 x numold
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     istartvc       i    Starting position of sequence numbers of vectors in inpcre
!     ivecold        i    ivec for old vector
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     kvec           i    Loop variable over the vectors to be created
!     maxphys        i    Maximum number of unknowns per point
!     mvec           i    Number of old vectors to be used in FUNCVECT
!     nnodes         i    Number of nodes
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCREATQUAD    Fill boundary conditions that are defined by a quadratic
!                    polynomial
!     ERCREATSTAND3  Fill one vector for one option only
!                    kfun>3000, function of old vectors
!     ERCREATSTAND4  Fill one vector for one option only
!                    kfun>4000, vector
!     ERCREATSTAND5  Fill one vector for one option only
!                    kfun<3000, standard case
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLVEC      Fill vector in case of a vector expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatstand1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatstand1'
         write(irefwr,1) 'kfun', kfun
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( kfun )

      case (3001:3999)

!     --- kfun>3000, function of old vectors

         call ercreatstand3 ( nnodes, inpcre, kfun, ipstin, islold, &
                              kdeg, kvec, isolut, maxphys, mvec, istartvc )

      case (4001:4999)

!     --- kfun>4000, vector

         call ercreatstand4 ( nnodes, inpcre, ipstin, kdeg, kvec, &
                              isolut, islold, ivecold )

      case (9001:9004)

!     --- kfun = 9001-9004, quadratic functions along curves or surfaces

         usoltemp => ks(isolut(kvec))%sol
         call ercreatquad ( usoltemp, nodes, &
                            nnodes, kdeg, inpcre(ipstin+3), rincre, &
                            kfun, kfill, icompl, inpcre(jpstin) )

      case default

!     --- Other value of kfun

         if ( kfun>200000000 ) then

!        --- kfun>200000000, hence vector expression

            call prfillvec ( isolut, kvec, kfun, nnodes, kdeg )

         else if ( kfun/=0 ) then

!        --- kfun#0

            call ercreatstand5 ( nnodes, inpcre, kfun, rincre, ipstin, &
                                 icompl, kdeg, kvec, isolut, bndcon )

         end if  ! ( kfun/=0 )

      end select  ! case ( kfun )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatstand1'

      end if  ! ( debug )
      call erclos ( 'ercreatstand1' )

      end
