      subroutine ercreatstand5 ( nnodes, inpcre, kfun, rincre, ipstin, &
                                 icompl, kdeg, kvec, isolut, bndcon )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 09-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one vector for one option only
!     kfun<3000, standard case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nnodes,  inpcre(*), kfun, ipstin, &
                 icompl, kdeg, kvec, isolut(*)
      double precision :: rincre(*)
      logical bndcon

!     bndcon         i    If true boundary conditions must be filled, otherwise
!                         a vector must be created
!     icompl         i    Indication if the vector to be created is real (0) or
!                         complex
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin         i    Starting address in array INPCRE
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     kvec           i    Loop variable over the vectors to be created
!     nnodes         i    Number of nodes
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0048         Fill values in the vector in the points stored in array
!                    NODES  real case
!     ER0069         Fill values in the vector in the points stored in array
!                    NODES  complex case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatstand5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatstand5'
         write(irefwr,1) 'nphyso', nphyso
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      usoltemp => ks(isolut(kvec))%sol

      if ( icompl==0 ) then

!     --- Real case

         call er0048 ( usoltemp, nodes, nnodes, kdeg, &
                       inpcre(ipstin+2), kfun, rincre, bndcon )
      else

!     --- Complex case

         call er0069 ( usoltemp, nodes, nnodes, kdeg, &
                       inpcre(ipstin+2), kfun, rincre, bndcon )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatstand5'

      end if  ! ( debug )
      call erclos ( 'ercreatstand5' )

      end subroutine ercreatstand5
