      subroutine ermatrixbvec ( xvec, yvec, irows, icols, amat, compl, &
                                compvc, isign )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 21-04-2015 New parameter list
!        version  1.0  date 11-01-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Part of matrix vector multiplication with respect to the boundary
!     conditions
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     boundary_condition
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: irows(*), icols(*), isign
      double precision xvec(*), yvec(*), amat(*)
      logical compl, compvc

!     amat           i    part 2 of large matrix corresponding to bc's
!     compl          i    indication whether the matrix is real (false)
!                         or complex (true)
!     compvc         i    If true the vector to be multiplied with is complex
!     icols          i    Contains column numbers of non-zero elements
!                         of matrix in sequence defined by irows
!     irows          i    Contains pointers to rows in matrix and icols
!                         irows(1) = 0
!                         number of non-zeros per row is irows(i+1)-irows(i)
!     isign          i    Multiplication sign, if 1 the effect is added to the
!                         existing vector, if -1 it is subtracted
!     xvec           i    Vector to be multiplied
!     yvec           o    result of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0017         incorporate effect of boundary conditions, real case
!     ER0018         incorporate effect of boundary conditions, complex case
!     ER0056         incorporate effect of boundary conditions, matrix real, &
!                    vector complex
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ermatrixbvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ermatrixbvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( compl ) then

!     --- complex case

         call er0018 ( irows, icols, amat, xvec, yvec, nusol, &
                       nbound, isign )

      else

!     --- real case

         if ( compvc) then

!        --- real matrix and complex vector

            call er0056 ( irows, icols, amat, xvec, yvec, nusol, &
                          nbound, isign )

         else

!        --- real matrix and real vector

            call er0017 ( irows, icols, amat, xvec, yvec, nusol, &
                          nbound, isign )

         end if

      end if

1000  call erclos ( 'ermatrixbvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ermatrixbvec'

      end if  ! ( debug )

      end
