      subroutine erskipelgrp ( indelgrp, ioutpoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array ioutpoints for all points that are not lying in element groups
!    that must be skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indelgrp(*), ioutpoints(npoint)

!     indelgrp       i    If indelgrp(i) = 1, then element group i must be
!                         skipped, otherwise it must be taken into account
!     ioutpoints     o    Output array of length npoint
!                         If ioutpoints(i)=0, node i must be skipped, else
!                         ioutpoints(i)>0, node i must not be skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, inpelm, ielgrp, ikelmc, irelem

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array ioutpoints
!     For ielgrp := 1 (1) nelgrp do
!        if ( element group ielgrp does not have to be skipped ) then
!           set ioutpoitns(i) = 1 for all point in element group
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

!     --- Clear array ioutpoints

      ioutpoints = 0

!     --- Loop over element groups

      ikelmc = 0
      do ielgrp = 1, nelgrp
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         if ( indelgrp(ielgrp).eq.1 ) then

!        --- indelgrp(ielgrp) = 1, hence skip element group

            ikelmc = ikelmc + inelem*inpelm

         else

!        --- indelgrp(ielgrp) = 0, hence do not skip element group

            do irelem = 1, inelem
               do k = 1, inpelm
                  ioutpoints(kmeshc(ikelmc+k)) = 1
               end do
               ikelmc = ikelmc+inpelm
            end do
         end if
      end do

      end
