      subroutine ervl00 ( n, first, m, restrt, q1, u, q0, uw, &
                          alpha, beta, stiff, smas )
! ======================================================================
!
!        programmer    J.v.Katz and H.v.d.Vorst
!        version  2.1  date 27-06-1992 Initialise ichois
!        version  2.0  date 17-03-1991 Removal of intmas
!        version  1.1  date 19-02-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to carry out lanczos process for the e.v. problem
!           inv(smas) * stiff * inv(smas) * y = lambda y
!        renamed subroutine lsvlan of J.v.Katz and H.v.d.Vorst
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulecomio
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: n, m, first
      logical restrt
      double precision q1(n), q0(n), u(n), alpha(m), beta(m), &
                       smas(*),  uw(*)
      type (rmatrix) :: stiff

!     alpha      o    the diagonal of the tridiagonal matrix which is
!                     roughly similar to a.
!     beta       o    the superdiagonal of the tridiagonal matrix
!     first      i    the first column to be computed on this call to the
!                     the subroutine (on initial call, first will be 1)
!     m          i    the maximum number of columns of the process to be
!                     computed
!     n          i    the size of the matrix
!     q0              work vector of length n
!     q1         i    the first column of q, a real vector, not assumed
!                     to be of norm 1
!     restrt     i    a logical variable indicating whether we are
!                     restarting from a previous partial
!                     tridiagonalization or starting a new one
!     smas       i
!     stiff      i
!     u               work vector of length n
!     uw         i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichois, km1, i, k
      double precision temp, alphaj, betaj, betamx

!     alphaj
!     betaj
!     betamx
!     i          Counting variable
!     ichois     Choice parameter
!     k          Counting variable
!     km1
!     temp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision dnrm2, ddot

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DCOPY          Copy double precision dx to double precision dy
!     DDOT           (CDOT)     Inner product of two vectors
!     DNRM2          Blas routine to compute the 2-norm of a vector
!     DSCAL          Blas routine to scale a vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERVL11         solve   sqrt(m)^(-1) s sqrt(m)^(-1)    a  =  b
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   process used: this subroutine implements a refined version of the
!   paige-style lanczos process.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ervl00' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ervl00'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if (restrt) go to 60

!     --- i n i t i a l i z a t i o n
!         step 1:  normalize q1 to unit length

      temp = dnrm2(n,q1,1)
      if ( debug ) write(irefwr,2) 'temp', temp
      call dscal(n,1d0/temp,q1,1)

!     --- compute q2

      call ervl11(q1, u, n, stiff, smas, uw)
      alphaj = ddot(n,u,1,q1,1)
      if ( debug ) write(irefwr,2) 'alphaj', alphaj
      call daxpy(n,-alphaj,q1,1,u,1)

!     --- for use in an iterative lanczos process, since beta may be
!         very small, we reorthogonalize q2 with respect to q1 in order
!         to avoid loss of a lot of our orthogonality right off.

      temp = ddot(n,u,1,q1,1)
      call daxpy(n,-temp,q1,1,u,1)
      if ( debug ) write(irefwr,2) 'temp/2', temp

!     --- now normalize q2

      betaj = dnrm2(n,u,1)
      temp = 1d0/betaj
      call dcopy(n,q1,1,q0,1)
      if ( debug ) write(irefwr,2) 'temp/3', temp
      do 50 i=1,n
         q1(i) = u(i)*temp
50    continue

!     --- compute initial alpha and beta.

      alpha(1) = alphaj
      beta(1) = betaj
      betamx = betaj
      k = 2
      go to 80

!     --- r e s t a r t i n g
!             the user must have restored q1 and q0, alpha, beta and
!             kappa as they were at the end of the last call.  betaj is
!             stored in beta(first-1)

60    k = first
      betamx = 0d0
      km1 = k - 1
      do 70 i=1,km1
         if (beta(i)>betamx) betamx = beta(i)
70    continue
      betaj = beta(k-1)

!     --- now carry out lanczos process for k = 3, 4, ..., m;
!         for k = 2 to m-1
!               < compute succeeding columns of orthogonal matrix q >

80    if (k>m) go to 1000

!     --- compute previous alpha and direction of next vector

      call ervl11(q1, u, n, stiff, smas, uw)
      call daxpy(n,-betaj,q0,1,u,1)
      alphaj = ddot(n,u,1,q1,1)
      call daxpy(n,-alphaj,q1,1,u,1)
      alpha(k) = alphaj

!     --- normalize next vector

      betaj = dnrm2(n,u,1)
      if (betaj<epsmac*betamx) then
         m=k
         return
      end if
      temp = 1d0/betaj
      beta(k) = betaj
      if (betaj>betamx) betamx = betaj
      call dcopy(n,q1,1,q0,1)
      do 110 i=1,n
         q1(i) = u(i)*temp
110   continue

      k = k + 1
      go to 80

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ervl00'

      end if  ! ( debug )
      call erclos ( 'ervl00' )

      end subroutine ervl00
