      subroutine prsolcheck ( matr, irhsd, compl, jcart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-12-2017 New storage of intmat
!        version  1.1  date 04-06-2014 Allow -102 in intmat(2)
!        version  1.0  date 07-04-2014
!
!   copyright (c) 2014-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some checks with respect to the input for linear solvers
!     Compute some relevant parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
!     mumps
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulematstr
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr, irhsd
      integer, intent(out) :: compl, jcart

!     compl          o    Indication if the right-hand side is real (1)
!                         or complex (2)
!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector
!     jcart          o    indication whether the element is a triangle with
!                         Cartesian coordinates (1)
!                         or a circular triangle with cylinder symmetric
!                         coordinates (2)
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: complexmat, filled

!     complexmat     If true the matrix is complex
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      96   array irhsd has not been filled
!     750   The problem number corresponding to array INTMAT is
!           not the same as the problem number corresponding to IRHSD.
!     777   The right-hand side is a real vector, while the matrix is complex
!     778   The right-hand side is a complex vector, while the matrix is real
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsolcheck' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsolcheck'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      filled = irhsd>0 .and. irhsd<=lastsol
      if ( filled ) filled = solarr(irhsd)==1
      if ( filled ) filled = ks(irhsd)%typevec==110
      if ( .not. filled ) then
         call errint ( irhsd, 1 )
         call errsub ( 96, 1, 0, 0 )
      end if
      compl = ks(irhsd)%compl+1
      jcart = ks(irhsd)%jcart

      if ( matr>0 ) then

!     --- the matrix has been filled

         iseqmt = matr

         iprob = realmt(iseqmt)%iprob
         complexmat = realmt(iseqmt)%complexmat

         if ( .not. realmt(iseqmt)%diagonal ) then
            intmat = realmt(iseqmt)%intmatref
            call sepfillmatstr
         end if  ! ( .not. realmt(iseqmt)%diagonal )
         if ( compl==1 .and. complexmat ) then
            call errsub ( 777, 0, 0, 0 )
         end if
         if ( compl==2 .and. .not. complexmat ) then
            call errsub ( 778, 0, 0, 0 )
         end if

      end if

      if ( ks(irhsd)%iprob/=iprob ) then
         call errint ( ks(irhsd)%iprob, 1 )
         call errint ( iprob, 2 )
         call errsub ( 750, 2, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsolcheck'

      end if  ! ( debug )
      call erclos ( 'prsolcheck' )

      end
