      subroutine sepassemelem
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create element matrices and vectors for one element
!     Assemble in large matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ifirsv

!     ifirsv         Help variable to save ifirst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILDDALLOC De-allocate temporary arrays in for subroutine build
!     TO0050TRANS    Transform element matrix and vector in case of
!                    local transformations
!     TO0051         add element matrix to large matrix, element vector to large
!                    vector and element mass matrix to mass matrix
!                    (diagonal or not)
!     TOELMATVEC     Multiply element matrix by part of solution vector and
!                    add to right-hand side vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepassemelem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepassemelem'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ifirsv = ifirst

      if ( .not. vector .and. .not. matrix .and. notmas==1 ) then

!     --- Skip rest if element matrix nor element vector must be
!         computed

         call sepbuilddalloc  ! deallocate arrays
         skip = .true.
         go to 1000

      end if

!     --- If npltra>0  and ielgrp<=nelgrp, i.e. local tranformation possible
!         Transform internal element

      if ( transform .and. npltra>0 .and. ielgrp<=nelgrp ) call to0050trans

!     --- Add element matrix and element vector to large matrix and vector

      ifirst = ifirsv
      if ( iinbld(2)==6 ) then

!     --- iinbld(2) = 6, i.e. boundary elements are multiplied by
!         solution to get a part of the reaction force

         call toelmatvec

      else

!     --- iinbld(2) # 6, standard situation

         call to0051

      end if  ! ( iinbld(2)==6 )
      ifirst = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepassemelem'

      end if  ! ( debug )
      call erclos ( 'sepassemelem' )

      end subroutine sepassemelem
