      subroutine sepbvalue ( ichoice, jdimpr, isol, &
                             value, icrv1, icrv2, idegfd, jstap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-05-2014 Extension of timebefore
!        version  1.2  date 05-12-2012 Long integers
!        version  1.1  date 14-10-2002 New call to er0024
!
!   copyright (c) 1999-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill essential boundary conditions in the solution vector
!     one component and one part of the boundary only
!     This is an old-fashioned subroutine and it is not longer recommended to
!     use it
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, jdimpr, icrv1, icrv2, idegfd, jstap, isol
      double precision value

!     ichoice    i   choice parameter.  possibilities:
!                    0      the degree of freedom gets the constant value of the
!                           parameter value
!                    >0     variable values of the degrees of freedom.
!                           subroutine funcbc is called.
!                    <0     complex values of the degrees of freedom
!                           subroutine cfuncb is called
!     icrv1      i   boundary conditions must be filled in the boundaries
!                    icrv1 to icrv2
!     icrv2      i   See icrv1
!     idegfd     i   see programmer's guide
!     isol       o   Integer array of length 5 containing information of the
!                    solution vector
!     jdimpr     i   Choice parameter consisting of two parts idim and iprob
!                    according to the following rules:
!                    jdimpr = idim + 1000 * (iprob-1) with:
!                    idim
!                    iprob    Problem number
!     jstap      i   see programmer's guide
!     value      i   value of the constant when the boundary condition
!                    is constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl, clear
      integer mult, ivec, k
      double precision :: timebefore(2)

!     clear          Indicates if solution array must be cleared (true) or not
!     compl          Indicates if the solution array is complex (true) or not
!     ivec           Sequence number of vector of special structure
!     k              Counting variable
!     mult
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0000
!     ER0008
!     ER0011
!     ER0012
!     ER0024
!     ER0036
!     ER0037
!     ER0038
!     ER0046
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     TOBVALZC       Clear part of usol corresponding to boundary conditions
!                    (comp)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   182   idim has incorrect value
!   183   idegfd incorrect
!   185   icrv2<icrv1
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbvalue' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbvalue'
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- tests

      iprob  = jdimpr / 1000
      idim   = jdimpr - 1000 * iprob
      iprob  = iprob + 1

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobp => kprobpart%kprobp
      indprp = kprobloc(35)
      call sepfillkprob
      if ( idim<0 .or. (idim>5 .and. idim<11) .or. &
           idim>10+nelgrp ) then
         call errint ( idim, 1 )
         call errsub ( 182, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- end tests

      if ( ichoice<0 ) then
         compl = .true.
      else
         compl = .false.
      end if
      if ( idim/=0 .and. icrv2<icrv1 ) then
         call errint ( icrv1, 1 )
         call errint ( icrv2, 2 )
         call errsub ( 185, 2, 0, 0 )
      end if

!     --- reserve space for arrays

      npointcac = npoint

      if ( compl ) then
         mult = 2
      else
         mult = 1
      end if

      clear = isol<1 .or. isol>lastsol
      if ( .not. clear ) clear = solarr(isol)==0
      if ( .not. clear ) clear = ks(isol)%typevec/=110
      call sepcreasol110 ( isol, 'usol', mult-1 )

      nusolcac = nusol
      usol => usoltemp

      if ( debug ) write(irefwr,*) 'clear ', clear

      if ( clear ) then

!     --- ks(isol)%typevec # 110,  new array;  clear array

         usol(1:mult*nusolcac) = 0d0

      else if ( idim==0 ) then

!     --- clear boundary conditions

         if ( nbound>0 ) then
            if ( compl ) then
               kprobhinvo => kprobhinv
               call tobvalzc ( usol )
            else
               usol(kprobhinv(nrusol+1:nrusol+nbound)) = 0d0
            end if
         end if
      end if
      if ( idim==0 ) goto 1000

      if ( idegfd<=0 .or. idegfd>nphys ) then
         call errint ( idegfd, 1 )
         call errint ( nphys, 2 )
         call errsub ( 183, 2, 0, 0 )
      end if

      if ( idim==1 ) then

!     --- idim  =  1  point boundary conditions

         call er0008 ( usol, ichoice, icrv1, icrv2, &
                       idegfd, jstap, value, compl )

      else if ( idim==2 ) then

!     --- idim  =  2  curve boundary conditions

         if ( abs(ichoice)<1000) then

!        --- |ichoice|<1000  standard method

            call er0000 ( usol, ichoice, icrv1, icrv2, &
                          idegfd, jstap, value, compl )
         else

!        --- |ichoice|>1000  special method

            call er0024( usol, &
                         ichoice, icrv1, icrv2, jstap, compl )
         end if

      else if ( idim==3 ) then

!     --- idim  =  3   surface boundary conditions

         if ( abs(ichoice)<1000 ) then

!        --- |ichoice|<1000  standard method

            call er0011 ( usol, ichoice, icrv1, icrv2, &
                          idegfd, jstap, value, compl )
         else

!        --- |ichoice|>1000   special method

           call er0012 ( usol, ichoice, icrv1, icrv2, compl )

         end if

      else if ( idim==4 ) then

!     --- idim  =  4   nodes icrv1 to icrv2

         call er0036 ( usol, ichoice, icrv1, icrv2, &
                       idegfd, value, compl )

      else if ( idim==5 ) then

!     --- idim  =  5   elements icrv1 to icrv2,  relative nodal point jstap

         call er0037 ( usol, ichoice, icrv1, icrv2, &
                       idegfd, value, jstap, compl )

      else if ( idim>10 ) then

!     --- idim  =  10+ielgrp   element ielgrp
!                  nodal points icrv1 to icrv2,  step: jstap

         ielgrp = idim-10
         if ( abs(ichoice)<1000 ) then

!        --- |ichoice|<1000  standard method

            call er0038 ( usol, ichoice, icrv1, icrv2, &
                          idegfd, value, jstap, ielgrp, compl )

         else

!        --- |ichoice|>1000  special method

            call er0046 ( usol, ichoice, ielgrp, compl )

         end if

      end if


1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbvalue'

      end if  ! ( debug )
      call erclmn ( 'sepbvalue', timebefore )

      end subroutine sepbvalue
