      subroutine sepcgeps ( A, repeat1, bnorm, bsnorm, r0norm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute A%eps as function of A%eps1 A%eps2 and norms
!     Help subroutine for Krylov subspace methods
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     accuracy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(in) :: bnorm, bsnorm, r0norm
      logical, intent(out) :: repeat1

!     A             i/o   Matrix A
!     bnorm          i    Norm of the right-hand side (b,b)
!     bsnorm         i    Preconditioned norm of the right-hand side
!                         (M^(-1)b,M^(-1)b)
!     r0norm         i    norm of start residual
!     repeat1        o    Logical variable indicating if the accuracy check must
!                         be carried out twice (true) or once (false), because
!                         the norm of the solution could not be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgeps' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgeps'
         write(irefwr,2) 'eps, eps1, eps2', A%eps, A%eps1, A%eps2
         write(irefwr,1) 'irelerr', A%irelerr
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%eps2<=0d0 ) then

         A%eps = A%eps1
         repeat1 = .false.

      else

         if ( A%iaccuracy==0 ) then

            A%eps = A%eps1 + sqrt(bsnorm) * A%eps2
            repeat1 = .true.

         else if ( A%irelerr==1 ) then

            A%eps = A%eps1 + sqrt(r0norm) * A%eps2
            repeat1 = .false.

         else if ( A%irelerr==2 ) then

            A%eps = A%eps2
            repeat1 = .false.

         else if ( A%irelerr==3 ) then

            A%eps = A%eps1 + sqrt(bnorm)  * A%eps2
            repeat1 = .false.

         end if

      end if

      A%eps = A%eps ** 2
      if ( debug ) write(irefwr,2) 'A%eps', A%eps

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgeps'

      end if  ! ( debug )
      call erclos ( 'sepcgeps' )

      end
