      subroutine sepcgprint ( res, rhotest, bnorm, bsnorm, r0norm, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform extended print for subroutine sepcg
!     iprint>=2
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: rhotest, bnorm, bsnorm, r0norm
      double precision, intent(in) :: res(A%neq)

!     A              i    Matrix A
!     bnorm          i    Norm of the right-hand side (b,b)
!     bsnorm         i    Preconditioned norm of the right-hand side
!                         (M^(-1)b,M^(-1)b)
!     r0norm         i    norm of start residual
!     res            i    residual vector
!     rhotest        i    norm of residual used to terminate CG
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: reprint

!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgprint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgprint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      reprint = norm(res, A)

      if ( A%iprint==1 ) then

!     --- Print text in case iprint = 1

         write ( irefwr, 110 ) reprint
110      format ( ' The 2-norm of the initial residual', ss, 1pe17.2e2 )
         go to 1000

      end if  ! ( A%iprint==1 )

!     --- The rest of the print only in case iprint>1

      if ( A%nproj>0 ) then

         if ( A%deflation ) then

            write ( irefwr, 115 ) A%nproj
115         format ( 'Deflation is used with ', i4, ' vectors', //)

         else

            write ( irefwr, 116 ) A%nproj
116         format ( 'Coarse Grid Correction is used with ', &
                     i4, ' vectors', //)

         end if

      end if

      if ( parallel ) then
         write ( irefwr, 117 ) iacnodes
117      format ( 'The problem is solved using', i4, &
                  ' parallel computers', // )
      end if

      if ( A%iaccuracy==0 ) then

         write ( irefwr, 120 ) 0, reprint/sqrt(bnorm), &
                sqrt(rhotest/bsnorm), '-', '-'
120      format ( '  All errors and residuals are relative.'// &
                  '  Iter   residual     residual    lambda', &
                  '     error'/ &
                  ' number   |r_k|_2  |M^-1r_k|_2          ', &
                  '      est.'// &
                    i7, ss, 1pe10.2e2, 1pe13.2e2, 9x, a, &
                    9x, a)
      else if ( A%irelerr==-1 ) then

         write ( irefwr, 121 )
         write ( irefwr, 125 ) 0, reprint/sqrt(bnorm)
121      format ( ' The residual is measured relative to the', &
                  ' right-hand side.'//)
125      format ( '  Iter   residual'/ &
                  ' number   |r_k|_2' // &
                    i7, ss, 1pe10.2e2)

      else if ( A%irelerr==1 ) then

         write ( irefwr, 122 )
         write ( irefwr, 125 ) 0, reprint/sqrt(r0norm)
122      format ( ' The residual is measured relative to the', &
                  ' initial residual.'//)

      else  if ( A%irelerr==2 ) then

         write ( irefwr, 125 ) 0, reprint

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgprint'

      end if  ! ( debug )
      call erclos ( 'sepcgprint' )

      end subroutine sepcgprint
