      subroutine sepcmaver1 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2019
!
!   copycight (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication in case ichoice = 1
!     y_i = S_ii*x_i;  y_b = x_b
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulecmat
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(in) :: A
      double complex, intent(in) :: x(A%nrow)
      double complex :: y(A%nrow)

!     A              i    matrix of type mat to be used in multiplication
!     x              i    vector to be multiplied
!     y              o    result vector of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (cmatrix) :: B

!     B              internal matrix A_ii stored as type cmatrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCOPMATMULR  Copy information from A with type mat into B with
!                    type rmatrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcmaver1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcmaver1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%diagonal ) then

!     --- Matrix is diagonal matrix, treat separately

            y = A%S_ii*x

      else

!     --- Matrix is standard matrix
!         First fill modules sepmodulematr and sepmodulematstr

  !       call sepfillmatr ( matr )

         if ( A%complexmat ) then

!        --- Complex matrix
call ertrace
call instop

  !          call sepmatveccompl ( ichoice, x, y )

         else

!        --- Real matrix, real vector

  !          call sepmatvecreal ( ichoice, x, y )
  !          call sepcopmatmulr ( A, B )
            yci = B*xci

         end if  ! ( compl )

         if ( indprh>0 ) then
            ycompl(kprobhinv(1:nrusol)) = yci(1:nrusol)
            deallocate(xci, yci)
         end if  ! ( indprh>0 .and. ichoice<=5 )

         if ( parallel ) then

  !          deallocate ( xglob, sendwork, stat = error )
  !          if ( error/=0 ) call erdealloc ( error, 'sendwork' )
call ertrace
print *,'parallel'
call instop
         end if  ! ( parallel )

      end if  ! ( ks(matr)%typevec==103 )


1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcmaver1'

      end if  ! ( debug )
      call erclos ( 'sepcmaver1' )

      end subroutine sepcmaver1
