      subroutine sepcmaxacc ( A, b, work, bnorm, rferr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute maximum accuracy possible in iterative solver
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: b(A%neq)
      double precision, intent(in) :: bnorm
      double complex, intent(inout) :: work(A%neq)
      double precision, intent(out) :: rferr

!     A              i    Matrix A
!     b              i    Right-hand side
!     bnorm          i    Norm of the right-hand side (b,b)
!     rferr          o    Maximal accuracy possible for this matrix, due to
!                         round-off
!     work                Work array to store a vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h

!                    they are not
!     h              Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcmaxacc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcmaxacc'
         write(irefwr,*) 'eistat ', A%eistat
         write(irefwr,2) 'bnorm ', bnorm
         call prinrl ( b, 2*A%neq, 'b' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%eistat ) then

!     --- Use cnorm of L^(-1) * b.

         work = A/b
         h = cnorm(work, A%neq)

      else

         h = bnorm

      end if

      if ( debug ) write(irefwr,2) 'h', h

      rferr = 1d2*h*epsmac

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'rferr', rferr
         write(irefwr,*) 'End sepcmaxacc'

      end if  ! ( debug )
      call erclos ( 'sepcmaxacc' )

      end subroutine sepcmaxacc
