      subroutine sepcopinpsol ( inpsl1, rinsl1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!     into sepmodulesolve
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsl1(*)
      double precision :: rinsl1(*)

!     inpsl1         i    Integer input array for subroutine solvel
!                         See solvelbf for a description
!     rinsl1         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: iinsol(:)

!     iinsol         Refers to inpsol(:,1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPINITINPSOL  initialize arrays inpsol and rinsol for seplinsol
!     SEPUPDINPSOL   Update inpsol for special situations
!                    Put parameters in sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopinpsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopinpsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepinitinpsol  ! initialize inpsol/rinsol

      iinsol => inpsol(:,1)

      length = min(inpsl1(1),ninpsol1)
      iinsol(1:length) = inpsl1(1:length)

      rinsol(1:nrinsol1,1) = rinsl1(1:nrinsol1)

      call sepupdinpsol  ! Update inpsol for special situations

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopinpsol'

      end if  ! ( debug )
      call erclos ( 'sepcopinpsol' )

      end subroutine sepcopinpsol
