      subroutine sepcopinpsolin ( iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy input for subroutine seplinsol from sepmoduleinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulesolve
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iread

!     iread          i    if >0 the input is copy from sepmoduleinput
!                         Otherwise defaults are used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqnr, jpos, maxcall, i

!     i              general loop variable
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     jpos           Position of input arrays in sepmoduleinput
!     maxcall        Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     SEPFILLINPSOL  Fill arrays inpsol and rinsol with default values
!                    extended with given values in the form of scalars or reals
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPINITINPSOL  initialize arrays inpsol and rinsol for seplinsol
!     SEPUPDINPSOL   Update inpsol for special situations
!                    Put parameters in sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopinpsolin' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopinpsolin'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepinitinpsol  ! initialize inpsol/rinsol

      iseqnr = mod(iread,1000)
      call sepgetinpfinf ( 8, iseqnr, jpos, maxcall )

      if ( in(jpos)%used ) then

!     --- iinsol>0, copy

         do i = 1, size(in(jpos)%iinput)/ninpsol1
            inpsol(1:ninpsol1,i) = &
               in(jpos)%iinput((i-1)*ninpsol1+1:i*ninpsol1)
         end do  ! i = 1, 3


!        --- Adapt variables

         call inichinpsol ( inpsol, ninpsol-1 )

         length = size(re(jpos)%rinput)
         call dcopy ( length, re(jpos)%rinput, 1, rinsol, 1 )

      else

!     --- iinsol=0, Fill inpsol with defaults extended with predefined
!         variables

         call sepfillinpsol

      end if
      if ( iread>1000 ) inpsol(9,1) = 1
      call sepupdinpsol  ! Update inpsol for special situations

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopinpsolin'

      end if  ! ( debug )
      call erclos ( 'sepcopinpsolin' )

      end subroutine sepcopinpsolin
