      subroutine sepcopyelem
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy information of elements in type element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     element
!     assembly
!     mass_matrix
!     matrix
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulebuild
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      logical :: lapshift, simple

!     IM             Refers to intmt(intmat)
!     lapshift       If true a laplace shift is applied in the solver
!     simple         If true simple is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopyelem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopyelem'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseqim>0 ) then
         IM => intmt(iseqim)
         lapshift = IM%lapshift
         simple = IM%simple
      else
         lapshift = .false.
         simple = .false.
      end if  ! ( iseqim>0 )

!     --- Get parameters

      E%mat => elemmt                     ! element matrix
      E%vec => elemvc                     ! element vector
      E%mmat => elemms                    ! element mass matrix
      E%ielvec => ielemvc                 ! integer element vector

      if ( simple .or. lapshift ) E%matshift => elemmt2(:,1)

      E%coor => coor                      ! coordinates
      E%nodes => index1(1:inpelm)         ! node numbers of element

      if ( inpelm>1 ) then
         E%ishape = ishape                ! shape of element
      else
         E%ishape = 0                     ! point element
      end if  ! ( inpelm>1 )
      E%npelm = inpelm                    ! number of nodes in element
      E%ndim = ndim                       ! dimension of space
      E%nunkp_pel = icount                ! number of degrees of freedom
                                          ! in the element
      E%itype = itype                     ! type number of element

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ishape, inpelm', E%ishape, E%npelm
         write(irefwr,*) 'End sepcopyelem'

      end if  ! ( debug )
      call erclos ( 'sepcopyelem' )

      end subroutine sepcopyelem
