      subroutine sepcopymatq
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy real Q part of simple matrix in type matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     real
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM, IMp

!     IM             Refers to intmt(intmat)
!     IMP            Refers to intmt(iseqimpre)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYINPSOL  Put information of inpsol and rinsol in matrix structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopymatq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopymatq'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)
      IMp => intmt(iseqimpre)

      Q%rows => IM%rowsQ
      Q%cols => IM%colsQ
      Q%Mrows => IMp%rowsQ
      Q%Mcols => IMp%colsQ
      Q%diag => IM%diagQ
      Q%Mdiag => IMp%diagQ

      Q%S => S_Q
      Q%type_prec = 0
      Q%scaling = 1          ! matrix is scaled (standard)
      Q%deflation = .false.
      Q%iseqmt = iseqmt
      Q%iseqim = iseqim

!     --- Store type of matrix

      if ( IM%Qsymmetric ) then
         Q%type_storage = 5
      else
         Q%type_storage = 6
      end if  ! ( IM%Qsymmetric )
      Q%symmetric = IM%Qsymmetric
      Q%neq = nrusolvel
      Q%nglob = nrusolvel
      Q%nproj = 0
      Q%matmultsp = 0
      call sepcopyinpsol ( Q, 2 )
      Q%simple = .false.

      if ( Q%type_prec>0 ) then
         Q%scaling = 2
         if ( .not. Q%symmetric .and. Q%type_prec==3 ) Q%scaling = 3
      end if  ! ( iprec>0 )

      if ( parallel ) then

!     --- Parallel case

         Q%innercontr => innercontru
         Q%sendinfo => sendinfou
         Q%nglob = nusolglob

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopymatq'

      end if  ! ( debug )
      call erclos ( 'sepcopymatq' )

      end subroutine sepcopymatq
