      subroutine sepcreavc ( ichoice, ichvc, iproblc, iveclc, &
                             ivectr, iuvec1, uvec1 )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  1.3  date 29-05-2014 Extension of timebefore
!        version  1.2  date 05-12-2012 Long integers
!        version  1.1  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill vectors itypvc=110(usol),115(vector) with given function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ivectr, ichoice, ichvcp, iveclc, iproblc, iuvec1(*), ichvc
      double precision uvec1(*)

!     ichoice i    Choice parameter: -1 complex, 0 real vector
!     ichvc   i    Choice parameter with
!                  ichvc  =1 ivectr must be connected with a vector of type usol
!                         =2 ivectr must be connected with a vector of special
!                            structure with sequence number ivec
!     iproblc i    Problem number
!     iuvec1  i    arrays containing information concerning the values of
!                  the unknowns in the nodal points
!                  iuvec1(i) corresponds to the i th unknown
!                  when iuvec1(i)=0 then the unknown is a constant whose value
!                  is stored in uvec1(i)
!                  when iuvec1(i)>0 subroutine func is called.
!                  the value of iuvec1(i) gives the parameter ikeuze in func
!     ivec    i    Sequence number of vector of special structure
!     ivectr i/o   standard sepran array denoting usol or vector of special
!                  structure defined in nodal points
!     uvec1   i    See uvec1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer k, itypvc, icompl, ivec
      double precision :: timebefore(2)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0006         Actual filling of function in vector
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRSPRCRE       Fill part of USOL corresponding to "right" periodical
!                    boundary conditions (complex case)
!     PRSPRRRE       Fill part of USOL corresponding to "right" periodical
!                    boundary conditions ( real case )
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     205   Vector to be created is of illegal type
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreavc' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ierror/=0 ) go to 1000
      if ( ichoice<0 ) then
         compl=.true.
         icompl = 1
      else
         compl=.false.
         icompl = 0
      end if
      iprob  =iproblc
      if ( ichvc==1 ) then

!     --- ichvc = 110

         itypvc=110

      else if ( ichvc==2 ) then

!     --- ichvc = 115

         itypvc=115

      else

!     --- Error 205: ichvc is incorrect

         call errint ( ichvc, 1 )
         call errsub ( 205, 1, 0, 0 )
         go to 1000

      end if

!     --- end tests

      ivec = iveclc
      call sepcreasolbf1 ( ivectr, itypvc, ivec, 'vector-new', icompl )
      if ( ichvc==1 ) indprho = kprobloc(50)
      if ( itypvc==110 ) then
         nunkpo = kprobloc(4)
      else
         nunkpo = nphyso
      end if
      if (ierror/=0) go to 1000

      usoltemp => ks(ivectr)%sol
      indprho = 0

!     --- fill vector in er0006

      call er0006 ( usoltemp, iuvec1, uvec1, compl )
      nusol  = kprobloc(5)
      nbound = kprobloc(10)
      npboun = kprobloc(34)
      if ( itypvc==110 .and. npboun>0 ) then

!     --- npboun>0
!         Periodical boundary conditions
!         Compute effect of periodical boundary conditions

         kprobs => kprobpart%kprobs
         rprobs => kprobpart%rprobs
         indprho = kprobloc(50)
         if ( compl ) then

!        --- complex case

            call prsprcre ( usoltemp, kprobhinvo, indprho )

         else

!        --- real case

            call prsprrre ( usoltemp, kprobhinvo, indprho )

         end if

      end if

1000  call erclmn ( 'sepcreavc', timebefore )
      end subroutine sepcreavc
