      subroutine sepfillsolmod ( matr, irhsd, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 24-02-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill several modules for subroutine seplinsol
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematr
      use sepmodulesolve
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr, irhsd(*)
      integer, intent(inout) :: isol(*)

!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, lengL
      logical :: compl
      type (rmatrix), pointer :: A
      type (cmatrix), pointer :: B
      type (intmatrix), pointer :: IM

!     A              Structure of type rmatrix containing the necessary
!                    information of the real matrix
!     B              Structure of type cmatrix containing the necessary
!                    information of the complex matrix
!     IM             Refers to intmt(intmat)
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     lengL          Length of lower tridiagonal of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLMATR    Fill parameters in module sepmodulematr
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPFILLSOLMOD1 Create arrays usol and rhsd
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolmod' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolmod'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then
         call sepfillmatstr2
      end if  ! ( parallel )
      if ( nrusol==0 ) go to 1000

      intmat = realmt(matr)%intmatref
      IM => intmt(intmat)
      iseqim = intmat
      iseqimpre = iseqim

!     --- Make correction of nrusol in case of parallel computing

      if ( parallel ) then

!     --- Parallel case

         nrusol = IM%nrusol
         indprh = kprobloc(50)
         if ( typecoupl==1 ) indprh = kprobpar(4) ! Dirichlet-Dirichlet coupling

      end if  ! ( parallel )

      call sepfillmatr ( matr )

      if ( iseqpre/=iseqintmat ) then

!     --- iseqpre differs from iseqintmat

         iseqimpre = intmatar(iseqpre)

         IM => intmt(iseqimpre)

      end if  ! ( iseqpre/=iseqintmat )

      if ( iseqqpre/=iseqintmat ) then

!     --- iseqqpre differs from iseqintmat

         iseqimpre = intmatar(iseqqpre)
         IM => intmt(iseqimpre)

      end if  ! ( iseqpre/=iseqintmat )

      if ( iseqdgpre/=iseqintmat ) then

!     --- iseqdgpre differs from iseqintmat

         iseqimpre = intmatar(iseqdgpre)
         IM => intmt(iseqimpre)

      end if  ! ( iseqpre/=iseqintmat )

      if ( iseqpmpre/=iseqintmat ) then

!     --- iseqpmpre differs from iseqintmat

         iseqimpre = intmatar(iseqpmpre)
         IM => intmt(iseqimpre)

      end if  ! ( iseqpre/=iseqintmat )

      if ( keep==2 .and. .not. IM%direct ) then

!     --- The preconditioning matrix exists and must be reused

         Pre_ii => realmt(iseqmt)%matinv

      end if  ! ( keep==2 .and. .not. IM%direct )

      rhsdbuf => ks(irhsd(1))%sol

      usolbuf => ks(isol(iseqsol))%sol
      if ( ks(isol(iseqsol))%typevec/=110 .and. &
           ks(isol(iseqsol))%typevec/=115 ) usolbuf = 0d0

      if ( iseqreac/=0 ) reac => ks(isol(abs(iseqreac)))%sol

      if ( iseq_exact>0 ) usolex => ks(isol(iseq_exact))%sol

      if ( iseqres_exact>0 ) resusolex => ks(isol(iseqres_exact))%sol

      if ( iseq_start_res>0 ) startres => ks(isol(iseq_start_res))%sol
      if ( iseq_end_res>0 ) endres => ks(isol(iseq_end_res))%sol
      if ( numdef_corr>0 ) rhsdbuf2 => ks(irhsd(2))%sol

      if ( ilimit>0 ) then

!     --- ilimit>0, compute minimum and maximum of input solution
!         Store in valmin, valmax

         valmin = minval(usolbuf(1:nusol))
         valmax = maxval(usolbuf(1:nusol))

      end if  ! ( ilimit>0 )

!     --- Create usol and rhsd

      compl = ks(irhsd(1))%compl==1
      call sepfillsolmod1 ( compl )
      if ( .not. IM%direct ) then

!     --- Compact matrix

         if ( realmt(matr)%complexmat ) then
            B => realmt(matr)%Sc_i
            B%scaling = 1          ! matrix is scaled (standard)
            B%prlump = inpsol(33,1)
            B%deflation = .false.

            if ( .not. IM%simple ) then
               B%Mrows => IM%rowsii
               B%Mcols => IM%colsii
            end if  ! ( .not. IM%simple )
         else
            A => realmt(matr)%S_i
            A%scaling = 1          ! matrix is scaled (standard)
            A%prlump = inpsol(33,1)
            A%deflation = .false.
            if ( IM%nested ) then
               A%Mrows => IM%rowsiiprec
               A%Mcols => IM%colsiiprec
               A%Mdiag => IM%diagprec
            else if ( .not. IM%simple ) then
               A%Mrows => IM%rowsii
               A%Mcols => IM%colsii
            end if  ! ( .not. IM%simple )
         end if  ! ( realmt(matr)%complex )

      end if  ! ( .not. IM%direct )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillsolmod'

      end if  ! ( debug )
      call erclos ( 'sepfillsolmod' )

      end subroutine sepfillsolmod
