      subroutine sepfillsolmods
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill several modules for subroutine seplinsol
!     Special case: simple
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulevecs
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipstart
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(iseqimpre)
!     ipstart        Starting address of array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     SEPCOPYMATC    Copy real C part of simple matrix in type matrix
!     SEPCOPYMATD    Copy real D part of simple matrix in type matrix
!     SEPCOPYMATQ    Copy real Q part of simple matrix in type matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2444   Exact solution in simple not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsolmods' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsolmods'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      np = nrusolpress
      nu = nrusolvel
      uvels => usol(1:nu)
      pvels => usol(nu+1:nrusol)
      fu => rhsd(1:nu)
      fp => rhsd(nu+1:nrusol)
      algamma = 0d0
      IM => intmt(iseqimpre)

      allocate ( worksim(nrusol), resup(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'ru' )
      if ( ierror/=0 ) go to 1000

      ru => resup(1:nu)
      rp => resup(nu+1:nrusol)

      if ( schur ) IM%Gsymmetric = .true.

      sizeqmatpre = 0
      if ( iprecq==1 .or. iprecq==2 ) then

!     --- Only a diagonal is used for the preconditioner

         sizeqmatpre = nu

      else if ( iprecq==3 .or. iprecq==4 ) then

!     --- The complete ILU is used for the preconditioner

         sizeqmatpre = size(IM%colsQ)

      end if

      sizedgmatpre = 0
      if ( iprecr==1 .or. iprecr==2 ) then

!     --- Only a diagonal is used for the preconditioner

         sizedgmatpre = np

      else if ( iprecr==3 .or. iprecr==4 ) then

!     --- The complete ILU is used for the preconditioner

         sizedgmatpre = size(IM%colsDG)

      end if

      sizepmmatpre = 0
      if ( iprecr==1 .or. iprecr==2 ) then

!     --- Only a diagonal is used for the preconditioner

         sizepmmatpre = np

      else if ( iprecr==3 .or. iprecr==4 ) then

!     --- The complete ILU is used for the preconditioner

         sizepmmatpre = size(IM%colsPM)

      end if

      irowsQpre => IM%rowsQ
      icolsQpre => IM%colsQ
      irowsDGpre => IM%rowsDG
      icolsDGpre => IM%colsDG
      irowsPMpre => IM%rowsPM
      icolsPMpre => IM%colsPM

!     --- Fill matrices C and Q

      call sepcopymatc
      call sepcopymatq
      call sepcopymatd

      if ( iseq_exact>0 ) then

!     --- At this moment it is not possible to do something with the exact
!         solution

         call errwar ( 2444, 0, 0, 0 )
         iseq_exact = 0

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillsolmods'

      end if  ! ( debug )
      call erclos ( 'sepfillsolmods' )

      end subroutine sepfillsolmods
