      subroutine sepmaverp2 ( ix, iy, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Last part of Matrix vector multiplication
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulemat
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ix, iy, ichoice

!     ichoice   i choice parameter. See sepmaver
!     ix       i  standard sepran array containing information of the
!                 vector x
!     iy       i  sepran array containing information of the vector y
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numrows

!     numrows        number of rows to be used for renumbering
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmaverp2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmaverp2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Add contributions of all cpu's to common nodes

  !       length = size(Smat%sendinfo)
         call sepparglob ( Smat%S_i, yri, nusol )

      end if  ! ( parallel )

      if ( ichoice<=1 ) then
         numrows = nrusol
      else
         numrows = nusol
      end if  ! ( ichoice<=1 )

      if ( indprh>0 ) then
         if ( ks(ix)%compl==1 ) then
            ycompl(kprobhinv(1:numrows)) = yci(1:numrows)
            deallocate(xci, yci)
         else
            yreal(kprobhinv(1:numrows)) = yri(1:numrows)
            deallocate(xri, yri)
         end if  ! ( ks(ix)%compl==1 )
      end if  ! ( indprh>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmaverp2'

      end if  ! ( debug )
      call erclos ( 'sepmaverp2' )

      end subroutine sepmaverp2
