      subroutine sepparprec1 ( A, y, m, inumblocks )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-10-2014 New parameter list
!        version  1.0  date 01-11-2008
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform parallel part of matrix vector multiplication
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     multiplication
!     parallel
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      integer m, inumblocks(2,*)
      double precision y(A%neq)

!     A              o    Structure of type matrix containing the necessary
!                         information of the matrix
!     inumblocks     i    unknown number and number of subdomains
!     m              i    The number of interface points
!     y             i/o   vector y in matrix-vector multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icontext

!                    they are not
!     icontext       Tag for sending and receiving, is used as check
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPARGLOB     Send information to neighbouring processors in order to
!                    give all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepparprec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepparprec1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      y(inumblocks(1,1:m)) = y(inumblocks(1,1:m))/ &
                             inumblocks(2,1:m)

      length = size(A%sendinfo)
      call sepparglob ( A, y, A%neq )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepparprec1'

      end if  ! ( debug )
      call erclos ( 'sepparprec1' )

      end
