      subroutine sepprecusymgs9 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix for the CGS-process
!     ipreco = 4,  Symmetric Gauss-Seidel
!     Old subroutine prcg02_4
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, n

!                    they are not
!     i              Counting variable
!     k              Counting variable
!     n              number of rows in matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecusymgs9' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecusymgs9'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = A%neq

!     --- ipreco = 4,  Symmetric Gauss-Seidel

      if ( skipzeros ) then

!     --- Zero diagonal elements are allowed

         A%adapt = max(A%adapt,epsmac)

      end if  ! ( skipzeros )

      do i = 1, n
         if ( A%adapt<=0d0 .and. A%S(A%diag(i))<=0d0 ) then
            call erreal ( max(A%adapt,A%S(A%diag(i))), 1 )
            call errint ( i, 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

!        --- Pre en post multiply the preconditioning matrix by sqrt(D^-1)
!            which implies  sqrt(D^-1) L U sqrt(D^-1) gets diagonal element 1
!            (scaling)

         A%MD(A%diag(i)) = sqrt ( max(A%S(A%diag(i)),A%adapt) )
         do k = A%rows(i)+1, A%diag(i)-1
            A%MD(k) = A%S(k) / A%MD(A%diag(i))
         end do  ! k = A%rows(i)+1, A%diag(i)-1
         do k = A%diag(i)+1,A%rows(i+1)
            A%MD(k) = A%S(k) / A%MD(A%diag(i))
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprecusymgs9'

      end if  ! ( debug )
      call erclos ( 'sepprecusymgs9' )

      end subroutine sepprecusymgs9
