      subroutine sepprintcond ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print information about the condition number of the large matrix
!     Only the real case is allowed
!     Old subroutine princnd1
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulemat
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A              i    Matrix A
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, imin, imax
      double precision :: h, amin, amax, rcond
      integer, pointer :: diag(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     amax       Maximum value in solution array
!     amin       Minimum value in solution array
!     diag       Refers to positions of diagonal elements of the matrix
!     h          Help variable to store quantity
!     i          Counting variable
!     imax       Index for which maximum value is reached
!     imin       Index for which minimum value is reached
!     rcond      Inverse of condition of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprintcond' )
      amin = rinfin
      amax = 0d0
      imin = 0
      imax = 0

      IM => intmt(A%iseqim)
      if ( IM%nested ) then
         diag => A%Mdiag
      else
         diag =>A%Mrows
      end if  ! ( IM%nested )

      do i = 1, nrusol
         h = abs(A%MD(diag(i)))
         if ( amin>h ) then
            amin = h
            imin = i
         end if
         if ( amax<h ) then
            amax = h
            imax = i
         end if
      end do
      if ( amax>0d0 ) then
         rcond = amin/amax
      else
         rcond = 0d0
      end if
      write(irefwr,100) rcond
100   format ( ' The inverse of the condition of the matrix is ', &
               d12.3 )
      if ( A%iprint>=2 ) write(irefwr,110) amin, imin, amax, imax
110   format ( ' The minimum value of the diagonal of the LU-matrix is', &
               d13.3/ ' for diagonal element ', i6/ &
               ' The maximum value of the diagonal of the LU-matrix is', &
               d13.3/ ' for diagonal element ', i6 )

      if ( A%iprint>=3 ) then
         write(irefwr,120) (i,A%MD(diag(i)), i = 1, nrusol )
120   format(1x,'Pivots of the large matrix'/ &
             (5(1x,i4,':',ss,1pe10.3e2)))
      end if
      call erclos ( 'sepprintcond' )

      end subroutine sepprintcond
