      subroutine sepscmatdivU ( A, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = L^(-1) x, where A is a preconditioning matrix
!     a symmetric compact storage is used, hence only D and L
!     The preconditioning matix has been created before
!     In other words a part of the ILU is executed (The L part)
!     This routine is meant for eistenstat preconditioning
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision :: y(A%neq)

!     A              i    Matrix A
!     y              o    Result  y = A^(-1) x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, mj
      double precision :: yi

!     i              General loop variable
!     j              General loop variable
!     mj             column number (index in vector)
!     yi             help variable to store y(i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepscmatdivU' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepscmatdivU'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Finally backward substitution U y = y

      do i = A%neq, 1, -1

         do j = A%rows(i)+2,A%rows(i+1)

            y(i) = y(i)-A%S(j)*y(A%cols(j))

         end do  ! j = A%rows(i)+1,A%rows(i+1)

      end do  ! i = n, 2, -1

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y')
         write(irefwr,*) 'End sepscmatdivU'

      end if  ! ( debug )
      call erclos ( 'sepscmatdivU' )

      end subroutine sepscmatdivU
