      subroutine sepsimpleconvg ( iter, rferr, converged, &
                                  niteru, niterp, rnorm, normprec, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-09-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute norm of residual and adapt accuracy if necessary
!     Check if process has converged
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulesolve
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iter, niteru, niterp
      double precision, intent(in) :: rnorm, normprec
      double precision, intent(inout) :: rferr
      logical, intent(out) :: converged
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     converged      o    If true the iteration process has been converged
!     iter           i    Iteration sequence number
!     niterp         i    Number of iterations to solve the p-equations
!     niteru         i    Number of iterations to solve the u-equations
!     normprec       i    Norm of preconditioned residual
!     rferr         i/o   Maximal accuracy possible for this matrix, due to
!                         round-off
!     rnorm          i    Norm of residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: normu, normp

!                    they are not
!     normp          norm of p-vector
!     normu          norm of u-vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimpleconvg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimpleconvg'
         write(irefwr,1) 'ireler, iter', A%irelerr, iter
         write(irefwr,2) 'eps, rferr', A%eps, rferr
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%iprint>=2 ) then

!     --- iprint>=2,  Maximal output

         normu = norm ( ru, Q )
         normp = norm ( rp, C )
         write ( irefwr, 400 ) iter, rnorm, normu, normp, normprec, &
                               niteru, niterp
400      format ( i5, 3x, ss, 4(1pe10.2e2), 2i8, 1pe10.2e2 )

      end if

!     --- In case of stokes and relative accuracy the convergence
!         may depend on the viscosity
!         To avoid that dependence we also have to check the norm
!         of the velocity part

      if ( A%irelerr==1 ) then
         converged = normprec<=A%eps
      else
         converged = rnorm<=A%eps
      end if  ! ( A%irelerr==1 ) then

      if ( converged ) go to 1000
      if ( rnorm<rferr .and. iter>3 ) then

!     --- maximum accuracy has been reached

         call erreal ( rferr, 1 )
         call errwar ( 677, 0, 1, 0 )
         converged = .true.

      end if  ! ( rnormu< rferr )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsimpleconvg'

      end if  ! ( debug )
      call erclos ( 'sepsimpleconvg' )

      end
