      subroutine sepsimplematpm ( simple, iseqinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create the preconditioning matrix for PM depending on iprec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     preconditioning
!     krylov
!     linear_solver
!     schur_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: simple
      integer, intent(in) :: iseqinp

!     iseqinp        i    Sequence number of input array inpsol
!     simple         i    if true the matrix R from simple is used,
!                         otherwise the pressure mass matrix PM
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lmax, istart, iend

!     iend           Last position used at a part
!     istart         First position used at a part
!     lmax           Maximal length of the blocks
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMAKEPREC    Create preconditioning matrix if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimplematpm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplematpm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create space for the preconditioning matrix for R

      if ( debug ) write(irefwr,1) 'sizedgmatpre/2', sizedgmatpre

      if ( sizedgmatpre>0 ) then
         allocate ( S_Rpre(sizedgmatpre), stat = error )
         if ( error/=0 ) call eralloc ( error, sizedgmatpre, 'S_Rpre' )
         if ( ierror/=0 ) go to 1000
      else if ( simple ) then
         S_Rpre => S_R
      else
         S_Rpre => S_PM
      end if  ! ( infmatvec(33)>0 )

!     --- create the preconditioning matrix for R depending on iprec

      lmax = 0

      call sepmakeprec ( R )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsimplematpm'

      end if  ! ( debug )
      call erclos ( 'sepsimplematpm' )

      end subroutine sepsimplematpm
