      subroutine sepsimplematq
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create the preconditioning matrix for Q depending on iprec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     matrix
!     preconditioning
!     krylov
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lmax, istart, iend
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iend           Last position used at a part
!     istart         First position used at a part
!     lmax           Maximal length of the blocks
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCREAPREC    computes the preconditioning matrix for the CG-
!                    CGS- and Bi-CGSTAB process. (real case)
!     SEPPRECAL5     Copy augmented lagrangian matrix into preconditioning
!                    matrix (symmetric storage)
!     SEPPRECAL9     Copy augmented lagrangian matrix into preconditioning
!                    matrix (row-compact storage)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimplematq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplematq'
         write(irefwr,1) 'nu, np', nu, np
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

!     --- Create space for the preconditioning matrix for Q

      if ( sizeqmatpre>0 ) then
         allocate ( S_Qpre(sizeqmatpre), stat = error )
         if ( error/=0 ) call eralloc ( error, sizeqmatpre, 'S_Qpre' )
         if ( ierror/=0 ) go to 1000
      else
         S_Qpre => S_Q
         go to 1000
      end if  ! ( sizeqmatpre>0 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'sizeqmatpre/1', sizeqmatpre

!     --- Make relation between S_Qpre and Q%M

      Q%MD => S_Qpre

      if ( algamma>0d0 ) then

!     --- Copy Q + gamma G M^-1 D into the preconditioning
!         matrix for those positions that fall into the preconditioning
!         matrix
!         First copy diagonal of matrix into premat

          if ( Q%symmetric ) then
             call sepprecal5
          else
             call sepprecal9
          end if  ! ( Q%symmetric )
          copyprec = .false.

      else if ( sizeqmatpre/=sizeqmat ) then


  !       call preccopy ( nu, irowsQ, irowsQpre, Q%L, Q%U, &
  !                       Q%ML, Q%MU, icolsQ, icolsQpre, S_Qpre, S_Q, jmethodq )
  !       inpsol(40,2) = -1

      end if  ! ( algamma/=0d0 )

!     --- create the preconditioning matrix for Q depending on iprec

      lmax = 0

      call sepcreaprec ( Q )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsimplematq'

      end if  ! ( debug )
      call erclos ( 'sepsimplematq' )

      end subroutine sepsimplematq
