      subroutine sepsimpleres ( normrhsd, A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the residuals r_u and r_p to be used at the start of the simple
!     algorithm
!     Old subroutine prsimpleres
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     residual
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(inout) :: normrhsd
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     normrhsd       o    Norm of right-hand side vector
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: work(:)
      double precision, pointer :: worku(:), workp(:)

!                    they are not
!     work           Work array
!     workp          Work array of length np to store intermediate vectors
!     worku          Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The residuals are defined by:
!     r_u = f_u - Q u - G p
!     r_p = f_p - D u - C p
!
!     Code:
!
!     Create array r_u
!     Clear du and dp
!     Compute r_u
!     Compute r_p
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimpleres' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimpleres'
         write(irefwr,1) 'nu, np, nrusol', nu, np, nrusol
         call prinrl ( rhsd, nrusol, 'rhsd' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      allocate(work(nu+np), stat = error)
      if ( error/=0 ) call eralloc ( error, nu+np, 'work' )
      if ( ierror/=0 ) go to 1000

      worku => work(1:nu)
      workp => work(nu+1:nu+np)

!     --- Compute r_u = f_u - Q u - G p
!         First r_u := f_u

      work = A*usol
      resup = rhsd(1:nu+np)-work

      if ( algamma>0d0 ) then

!     --- In case of AL we have to subtract algamma G M^-1 D u from r_u, where
!         M^-1 is the inverse of the diagonal pressure mass matrix
!         Du is stored in work
!         Multiply work by M^-1

         workp = workp*S_PMD

!        --- Next work(2) := G work

         worku = G*workp
         if ( debug ) call prinrl ( worku, nu, 'G M^-1 D u' )

!        --- subtract from r_u

         ru = ru - algamma * worku

      end if  ! ( algamma>0d0 )

!     --- Compute norm

      normrhsd = norm ( rhsd, A )

1000  deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( ru, nu, 'u-residual' )
         call prinrl ( rp, np, 'p-residual' )
         write(irefwr,2) 'normrhsd ', normrhsd
         write(irefwr,*) 'End sepsimpleres'

      end if
      call erclos ( 'sepsimpleres' )

      end subroutine sepsimpleres
