      subroutine sepsoldiagmat ( amat, y, x )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of equations amat u = f, where amat is a
!     diagonal matrix
!     Old subroutine prsdia
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: amat(nusol), y(nusol)
      double precision, intent(out) :: x(nusol)

!     amat       i    matrix to be inverted
!     x          o    Solution vector
!     y          i    Right-hand-side vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ih, imin, imax, n1
      double precision :: a, amin, amax, rcond

!     a          Help variable to store quantity
!     amax       Maximum value in solution array
!     amin       Minimum value in solution array
!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     imax       Index for which maximum value is reached
!     imin       Index for which minimum value is reached
!     n1             number of non-prescribed unknowns
!     rcond      Inverse of condition of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   1535   matrix singular
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsoldiagmat' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nrusol
         if ( abs(amat(i))<=0d0 ) then
            call errint ( i, 1 )
            call errsub ( 1535, 1, 0, 0 )
            go to 1000
         end if

      end do
      if ( indprh==0 ) then
         x(1:nrusol) = y(1:nrusol)/amat(1:nrusol)
      else
         n1 = nusol-nbound
         x(kprobhinv(1:n1)) = y(kprobhinv(1:n1))/amat(1:n1)
      end if  ! ( indprh==0 )
      if ( printlev>0 ) then

!     --- printlev>0, print information about the matrix

         amin = rinfin
         amax = 0d0
         imin = 0
         imax = 0
         do i = 1, nusol-nbound
            a = abs(amat(i))
            if ( amin>a ) then
               amin = a
               imin = i
            end if
            if ( amax<a ) then
               amax = a
               imax = i
            end if
         end do  ! i = 1, nrusol
         if ( amax>0d0 ) then
            rcond = amin/amax
         else
            rcond = 0d0
         end if
         write(irefwr,100) rcond
100      format ( ' The inverse of the condition of the matrix is ', d12.3 )
         if ( printlev>=2 ) write(irefwr,110) amin, imin, amax, imax
110      format ( ' The minimum value of the diagonal of matrix is ', &
                  d12.3/ ' for diagonal element ', i6/ &
                  ' The maximum value of the diagonal of matrix is ', &
                  d12.3/ ' for diagonal element ', i6 )

      end if

      if ( npboun>0 ) then

!     --- npboun>0; compute effect of periodical boundary conditions

         x(nrusol+1:nrusol+npboun) = &
            rprobs(:,2)*x(kprobs)+rprobs(:,1)

      end if  ! ( npboun>0 )

1000  call erclos ( 'sepsoldiagmat' )

      end
