      subroutine sepzprofiledec ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute lU decomposition for system of complex linear equations
!     Use Gauss elimination
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     complex
!     direct_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecmat
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex, allocatable :: work(:)
      double precision :: rcond
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     rcond          Inverse of condition of matrix
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPROFCS      compute the decomposition of a hermitian complex profile
!                    matrix
!     SEPPROFCU      compute the decomposition of a asymmetric complex profile
!                    matrix
!     SEPZCOPYTOPRFS Initialize the LU-decomposition matrix of A for a profile
!                    method
!                    This is done by copying the matrix from row-compact
!                    storage to profile storage (complex symmetric)
!     SEPZCOPYTOPROF Initialize the LU-decomposition matrix of A for a profile
!                    method
!                    This is done by copying the matrix from row-compact
!                    storage to profile storage (complex asymmetric)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepzprofiledec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzprofiledec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

!     --- Translate row-compact data structure to profile structure

      nrusol = A%neq

      if ( A%symmetric ) then

!     --- jmethod = 3    symmetric complex matrix
!         Copy matrix into "inverse matrix"

         call sepzcopytoprfs ( A )

!        --- Compute LDL^T decomposition, store in inverse matrix

         call sepprofcs ( A )

      else

!     --- jmethod = 4    unsymmetric complex matrix
!         Copy matrix into "inverse matrix"

         call sepzcopytoprof ( A )

!        --- Compute LU decomposition, store in inverse matrix

         call sepprofcu ( A )

      end if  ! ( A%symmetric )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepzprofiledec'

      end if  ! ( debug )
      call erclos ( 'sepzprofiledec' )

      end subroutine sepzprofiledec
