      subroutine sepzprofilesol ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of complex linear equations by Gaussian elimination
!     The matrix is supposed to be decomposed before
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     complex
!     direct_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPROFSUBCS   solve system of linear equations by gaussian elimination
!                    where the matrix has been decomposed by SEPPROFCS
!     SEPPROFSUBCU   solve system of linear equations by gaussian elimination
!                    where the matrix has been decomposed by SEPPROFCU
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepzprofilesol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzprofilesol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%symmetric ) then

!     --- jmethod = 3  symmetric complex profile matrix

         call sepprofsubcs ( A )

      else

!     --- jmethod = 4  asymmetric complex profile matrix

         call sepprofsubcu ( A )

      end if  ! ( A%symmetric )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepzprofilesol'

      end if  ! ( debug )
      call erclos ( 'sepzprofilesol' )

      end subroutine sepzprofilesol
