      subroutine to0039 ( ichois, vec1, vec2, vec3, a, nusol, nbound, &
                          indprh, iconst, kprobh )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version  4.0  date 02-01-1998 The solution arrays are not longer
!                                      renumbered, newparameter list
!        version  3.3  date 16-10-1993 Extra ICHOIS=22, 23
!        version  3.2  date 12-03-1993 New layout
!        version  3.1  date 12-09-1992 Correction sign in nbound
!        version  3.0  date 11-04-1990 Extra parameter iconst
!        version  2.1  date 13-07-1989 ICHOIS =9, multiplication by a
!        version  2.0  date 16-04-1989 Complete revision, new meaning
!        version  1.0  date 03-12-1985
!
!   copyright (c) 1985-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     algebraic manipulations with complex vectors  ( ichois = 8, 9, 22, 23)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex vec1(*)
      double precision a, vec2(*), vec3(*)
      integer ichois, nusol , nbound, indprh, iconst, kprobh(*)

!     a       i    Multiplication factor of modulus
!                  If zero no multiplication takes place
!     ichois  i    Choice parameter. Possible values
!                  8  modulus of vec1 (*a)           : a*sqrt(vec1*vec1)
!                  9  argument of vec2 (*a)          : a*atan2(im,re)
!                  22 real part of vec2 (*a)         : a*re
!                  23 imaginary part of vec2 (*a)    : a*im
!     iconst  i    Indication if the result for ichois=9 must be multiplied by
!                  180/pi or not
!     indprh  i    indication if kprob part h has been filled (>0) or not (0)
!     kprobh  i    KPROB part h
!     nbound  i    Number of essential boundary conditions
!     nusol   i    Total number of unknowns
!     vec1    i    The complex vector from which the modulus must be computed
!                  (ichois=8)
!     vec2    i    The complex vector from which the argument must be computed
!                  vec2 is treated as a real array, i.e. 2 positions for each
!                  complex number (ichois=9)
!     vec3    o    output vector with result
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!    i         Loop variable
!    j         Help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DSCAL   Blas routine to scale a vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( indprh.eq.0 ) then

!     --- No renumbering required

         if ( ichois.eq.8 ) then

!        --- modulus of vec1

            do i = 1, nusol
               vec3(i) = abs(vec1(i))
            end do

         else if ( ichois.eq.9 ) then

!        --- ichois = 9,  argument of vec1

            do i = 1, nusol
               if ( abs(vec2(2*i-1)).le.sqreps .and. &
                    abs(vec2(2*i)).le.sqreps ) then
                  vec3(i) = 0.0d0
               else
                  vec3(i) = atan2(vec2(2*i),vec2(2*i-1))
               end if
            end do
            if ( iconst.gt.0 ) then

!           --- iconst = 1, muliply by 180/pi

               call dscal ( nusol, 180d0/pi, vec3, 1 )

            end if

         else if ( ichois.eq.22 ) then

!        --- ichois = 22,  real part of vec1

            do i = 1,nusol
               vec3(i) = dble(vec1(i))
            end do

          else if ( ichois.eq.23 ) then

!         --- ichois = 23, imaginary part of vec1

            do i = 1,nusol
               vec3(i) = dimag(vec1(i))
            end do

         end if

         if ( abs(a).gt.0d0 .and. (abs(a-1d0).gt.epsmac)) then

!        --- a # 0, multiply by a

            call dscal ( nusol, a, vec3, 1 )

         end if

      else

!     --- Renumbering required

         if ( ichois.eq.8 ) then

!        --- modulus of vec1

            do i = 1, nusol-nbound
               j = kprobh(i)
               vec3(j) = abs(vec1(j))
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = abs(vec1(j))
            end do

         else if ( ichois.eq.9 ) then

!        --- ichois = 9,  argument of vec1

            do i = 1, nusol-nbound
               j = kprobh(i)
               if ( abs(vec2(2*j-1)).le.sqreps .and. &
                    abs(vec2(2*j)).le.sqreps ) then
                  vec3(j) = 0.0d0
               else
                  vec3(j) = atan2(vec2(2*j),vec2(2*j-1))
               end if
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               if ( abs(vec2(2*j-1)).le.sqreps .and. &
                    abs(vec2(2*j)).le.sqreps ) then
                  vec3(j) = 0.0d0
               else
                  vec3(j) = atan2(vec2(2*j),vec2(2*j-1))
               end if
            end do
            if ( iconst.gt.0 ) then

!           --- iconst = 1, muliply by 180/pi

               a = 180d0/pi
               do i = 1, nusol-nbound
                  j = kprobh(i)
                  vec3(j) = vec3(j)*a
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  vec3(j) = vec3(j)*a
               end do

            end if

         else if ( ichois.eq.22 ) then

!        --- ichois = 22,  real part of vec1

            do i = 1,nusol-nbound
               j = kprobh(i)
               vec3(j) = dble(vec1(j))
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = dble(vec1(j))
            end do

          else if ( ichois.eq.23 ) then

!         --- ichois = 23, imaginary part of vec1

            do i = 1,nusol-nbound
               j = kprobh(i)
               vec3(j) = dimag(vec1(j))
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = dimag(vec1(j))
            end do

         end if

         if ( abs(a).gt.0d0 .and. (abs(a-1d0).gt.epsmac)) then

!        --- a # 0, multiply by a

            do i = 1, nusol-nbound
               j = kprobh(i)
               vec3(j) = vec3(j)*a
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = vec3(j)*a
            end do

         end if
      end if
      end
