      subroutine to0050trans
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 24-07-2015 Use sepmodulebuild
!        version  3.0  date 23-03-2011 Remove icount from list
!        version  2.0  date 27-10-2010 Remove ihelp
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform element matrix and vector in case of local transformations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: matrixsav, vectorsav

!                    they are not
!     matrixsav      Help parameter to store the value matrix
!     vectorsav      Help parameter to store the value vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TO0036         transformation of element matrix and element vector if
!                    necessary complex case
!     TO0053         transformation of element matrix and element vector if
!                    necessary real case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0050trans' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0050trans'
         write(irefwr,*) 'compl ', compl
         write(irefwr,1) 'idefcr, imas, itype', idefcr, imas, itype
         write(irefwr,1) 'icount, maxtrn', icount, maxtrn
         call prinin ( index5, icount, 'index5' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( compl ) then

!     --- Complex case

         call to0036 ( elemmt, elemvc1 )
         if ( idefcr==1 ) call to0036 ( elemmt2, elemvc2 )

      else

!     --- Real case

         call to0053 ( elemmt, elemvc )
         if ( idefcr==1 ) call to0053 ( elemmt2, elemvc2 )

      end if
      if ( imas>0 ) then
         matrixsav = matrix
         vectorsav = vector
      end if  ! ( imas>0 )

      if ( imas==1 ) then

!     --- Diagonal mass matrix

         matrix = .false.
         vector = .true.

      else if ( imas>=2 ) then

!     --- Non-diagonal mass matrix

         matrix = .true.
         vector = .false.

      end if
      if ( imas>0 ) then

!     --- Transform mass matrix

         call to0053 ( elemms, elemms )
         matrixsav = matrix
         vectorsav = vector

      end if  ! ( imas>0 )

1000  call erclos ( 'to0050trans' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0050trans'

      end if

      end
