      subroutine tofict ( iinbld, ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.2  date 21-04-2015 New definition intmat
!        version  1.1  date 05-12-2012 Long integers
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill matrix with respect to fictitious unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     fictitious_domain
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinbld(*), ielhlp(*)

!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, &
!                         ielhlp(ielgrp+5) = 1, means that they are assembled
!     iinbld         i    User input array for subroutine BUILD, see buildbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lnindex1, lnindex2, lnelemmt
      double precision, allocatable :: elemmtloc(:)

!     elemmtloc      Local element matrix
!     lnelemmt       Length of array elemmt
!     lnindex1       Length of array index1
!     lnindex2       Length of array index2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     TOFICT00       Fill part of matrix corresponding to fictitious unknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofict' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofict'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lnindex1 = npelm*2
      lnindex2 = lnindex1*ndim
      lnelemmt = 27*lnindex2

      allocate ( iwork(nusol), elemmtloc(lnelemmt), index1(lnindex1), &
                 index2(lnindex2), index5(lnindex2), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nusol+lnelemmt+lnindex2*2, 'iwork' )
      if ( ierror/=0 ) go to 1000

      call tofict00 ( iinbld, ielhlp, elemmtloc )

      error = 0
      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      deallocate ( index1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index1' )
      deallocate ( index2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index2' )
      deallocate ( index5, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index5' )
      deallocate ( elemmtloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'elemmtloc' )

1000  call erclos ( 'tofict' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tofict'

      end if  ! ( debug )

      end subroutine tofict
