      subroutine tofillqmat ( elemmp, jwork, icnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.4  date 24-07-2015 Extra debug
!        version  3.3  date 21-09-2011 Extra debug
!        version  3.2  date 27-10-2010 array bounds jwork
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the matrix Qmat for the SIMPLE algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmoduleintmat
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: jwork(nrusolvel)
      integer, intent(out) :: icnt
      double precision, intent(in) :: elemmp(icount,icount)

!     elemmp         i    element matrix after mapping
!                         rows and columns corresponding to prescribed degrees
!                         of freedom are eliminated
!                         elemmp is stored row-wise (mathematical sequence)
!                         So in fortran we have to use the transpose fo elemmp
!     icnt           o    Number of non-prescribed unknowns in element
!     jwork         i/o   integer work array of size nrusolvel
!                         In jwork the sequence numbers of the columns of the
!                         non-zero elements in one row of the matrix are stored
!                         At input jwork must be 0
!                         At output jwork is reset to 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, inusol, jnusol, iamat, nusolmax, nusolmin
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     iamat          Position in array AMAT
!     inusol         Position of unknown in array USOL (row number)
!     j              Counting variable
!     jnusol         Position of unknown in array USOL (column number)
!     nusolmax       Maximum column number in a row
!                    In symmetrical case the row number
!     nusolmin       Minimum column number in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( Qsymmetric ) then
!        For i := 1 (1) icount do
!           inusol := indexren(i)
!           if ( inusol<=nrusolvel ) then
!              Fill jwork with sequence numbers
!              Raise diagQ
!              For j := 1 (1) icount do
!                 if ( j # i ) then
!                    jnusol := indexren(j)
!                    if ( jnusol = inusol ) then
!                       Add element to diagQ
!                    else if ( jnusol<=nrusolvel ) then
!                       Add element to Qmatl
!     else
!        For i := 1 (1) icount do
!           inusol := indexren(i)
!           if ( inusol<=nrusolvel ) then
!              Fill jwork with sequence numbers
!              Raise diagQ
!              For j := 1 (1) icount do
!                 if ( j # i ) then
!                    jnusol := indexren(j)
!                    if ( jnusol = inusol ) then
!                       Add element to diagQ
!                    else if ( jnusol<=nrusolvel ) then
!                       Add element to Qmatl
!                       Add element to Qmatu
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofillqmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofillqmat'
         write(irefwr,1) 'nrusolvel, icount', nrusolvel, icount
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)
      nusolmin = 1

      icnt = 0

!     --- For i := 1 (1) icount do

      do i = 1, icount

!     --- inusol := indexren(i)

         inusol = indexren(i)
         if ( inusol<=nrusolvel ) then

!        --- inusol<=nrusolvel, i.e. row corresponds to q matrix
!            Fill jwork with sequence numbers
!            Raise diagQ

            do j = irowsQ(inusol)+1, irowsQ(inusol+1)
               jwork(icolsQ(j)) = j
            end do  ! j = irowsQ(inusol)+1, irowsQ(inusol+1)
            nusolmax = icolsQ(irowsQ(inusol+1))

            if ( IM%Qsymmetric ) nusolmin = inusol

            icnt = icnt+1

!           --- For j := 1 (1) icount do

            do j = 1, icount

               jnusol = indexren(j)
               if ( jnusol>=nusolmin .and. jnusol<=nusolmax ) then
                  iamat = jwork(jnusol)
                  if ( iamat>0 ) S_Q(iamat) = S_Q(iamat)+elemmp(j,i)
               end if  ! ( jnusol<=nusolmax )

            end do  ! j = 1, icount

!           --- Reset jwork to 0

            do j = irowsQ(inusol)+1, irowsQ(inusol+1)
               jwork(icolsQ(j)) = 0
            end do  ! j = irowsQ(inusol)+1, irowsQ(inusol+1)

         end if  ! ( inusol<=nrusolvel )

      end do   ! i = 1, icount

1000  if ( debug ) then
         call sepprinmat6 ( Q )
         write(irefwr,*) 'End tofillqmat'
      end if  ! ( debug )
      call erclos ( 'tofillqmat' )

      end
