      subroutine tomanvec46_1 ( vecin, vecout, indprf, kprobf, nphys, &
                                npoint, norm, weight, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 21-09-2002 Extra parameter ichoice
!        version  1.0  date 06-10-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subtract the normal component of a vector defined on one surface (3D)
!     or curve (2D) and store the result in another vector
!     The input vector must be a vector with ndim components
!     The normal on this curve or surface is computed and the normal
!     component of u in the n direction is subtracted from u
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     normal_vector
!     3d
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecin(*), vecout(*), norm(3,*), weight(*)
      integer indprf, kprobf(*), nphys, npoint, ichoice

!     ichoice        i    Choice parameter, see manvecbf
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     norm                Work array of length ndim * npoint containing the
!                         normal in the nodal points
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     npoint         i    Number of nodal points
!     vecin          i    Input vector containing u
!     vecout         i    Output vector, result of the manipulation
!     weight              Work array of legth npoint to store weights for
!                         the computation of normals
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision udotn
      integer i, inode, inunkp, istart

!     i              Counting variable
!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
!     udotn          Inner product of vector and normal in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2283   number of degrees of freedom in node #0 or 3
!    2284   number of degrees of freedom in node = 3, but node not on surface
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For inode := 1 (1) npoints do
!        if ( there are three unknowns in node ) then
!           Check if weight is nonzero
!           Store u - u.n n into output vector
!        else if ( there are unknowns in node ) then
!           error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec46_1' )

!     --- For inode := 1 (1) npoints do

      inunkp = nphys
      do inode = 1, npoint

!     --- Compute number of unknowns per point

         if ( indprf.gt.0 ) then
            istart = kprobf(inode)
            inunkp = kprobf(inode+1)-istart
         else
            istart = (inode-1)*nphys
         end if

         if ( inunkp.ge.3 ) then

!        --- there are three unknowns in node
!            Check if weight is nonzero
!            Store u - u.n n into output vector

            if ( weight(inode).le.0d0 ) then

!           --- weight in node = 0, so not on surface

               call errint ( inode, 1 )
               call errsub ( 2284, 1, 0, 0 )

            end if
            udotn = 0d0
            do i = 1, 3
               udotn = udotn+vecin(istart+i)*norm(i,inode)
            end do
            if ( ichoice.eq.46 ) then

!           --- ichoice = 46, subtract normal component

               do i = 1, 3
                  vecout(istart+i) = vecin(istart+i)-udotn*norm(i,inode)
               end do

            else

!           --- ichoice = 47, store normal component

               do i = 1, 3
                  vecout(istart+i) = udotn*norm(i,inode)
               end do

            end if

         else if ( inunkp.gt.0 ) then

!        --- error there are unknowns in node

            call errint ( inode, 1 )
            call errint ( inunkp, 2 )
            call errsub ( 2283, 2, 0, 0 )

         end if

      end do
      call erclos ( 'tomanvec46_1' )
      end
