      subroutine tomanvec48_1 ( vec1, vec2, vecout, indprf, kprobf, &
                                nphys, npoint, idegfd, indprf2, kprobf2, &
                                nphys2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply the idegfd-th component of vector vec1 by the idegfd-th
!     component of vector vec1 and the unit vector in idegfd direction
!     The output vector has the same structure as vec1 and vec2 must be
!     defined in at least the same points as vec1
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vec1(*), vec2(*), vecout(*)
      integer indprf, kprobf(*), nphys, indprf2, kprobf2(*), idegfd, &
              nphys2, npoint

!     idegfd         i    Degree of freedom sequence number
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     indprf2        i    indication if kprob2 part f has been filled (>0)
!                         or not (0)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     kprobf2        i    KPROB part f: contains the number of unknowns per
!                         point accumulated (corresponds to vec2)
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     nphys2         i    Number of physical unknowns, or maximal degrees of
!                         freedom per point in second input vector
!     npoint         i    Number of nodal points
!     vec1           i    first input vector
!     vec2           i    second input vector
!     vecout         o    Output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, inode, inunkp2, istart, istart2

!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in point
!     inunkp2        Number of degrees of freedom in point (vector 2)
!     istart         First position used at a part
!     istart2        First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2283   number of degrees of freedom in node #0 or 3
!    2349   Second input vector contains too few unknowns in point
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For inode := 1 (1) npoints do
!        if ( there are three unknowns in node ) then
!           Check if weight is nonzero
!           Store u - u.n n into output vector
!        else if ( there are unknowns in node ) then
!           error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec48_1' )

!     --- For inode := 1 (1) npoints do

      inunkp = nphys
      inunkp2 = nphys2
      do inode = 1, npoint

!     --- Compute number of unknowns per point

         if ( indprf.gt.0 ) then
            istart = kprobf(inode)
            inunkp = kprobf(inode+1)-istart
         else
            istart = (inode-1)*nphys
         end if

         if ( inunkp.ge.3 ) then

!        --- there are three unknowns in node
!            Check if vec2 contains three unknowns

            if ( indprf2.gt.0 ) then
               istart2 = kprobf2(inode)
               inunkp2 = kprobf2(inode+1)-istart2
            else
               istart2 = (inode-1)*nphys2
            end if
            if ( inunkp2.le.3 ) then

!           --- Second input vector does not contain three degrees of freedom

               call errint ( inunkp2, 1 )
               call errint ( inode, 2 )
               call errint ( 3, 3 )
               call errsub ( 2349, 3, 0, 0 )
               go to 1000

            end if

!           --- Store result into output vector

            vecout(istart+idegfd) = &
                   vec1(istart+idegfd)*vec2(istart2+idegfd)

         else if ( inunkp.gt.0 ) then

!        --- error there are unknowns in node

            call errint ( inode, 1 )
            call errint ( inunkp, 2 )
            call errsub ( 2283, 2, 0, 0 )

         end if

      end do
1000  call erclos ( 'tomanvec48_1' )
      end
