      subroutine tomanvec52 ( ivec1, iresvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 29-10-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map vector of type 110 or 115 onto vector of type 116, i.e. map a vector
!     defined per node onto a vector defined per element
!     Line elements and connection elements are skipped
!     At this moment the mapping is restricted to linear triangles only
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     map
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, iresvc

!     iresvc         o    Standard SEPRAN array containing information of the
!                         output solution vector
!     ivec1          i    Standard SEPRAN array containing information of the
!                         input solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compli

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TOMAPSOL116    Map vector of type 110 or 115 onto vector of type 116, i.e.
!                    map a vector defined per node onto a vector
!                    defined per element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1393   Not constant number of degrees of freedom not available in intmsh.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Detect number of degrees of freedom per point in input vector
!     Get number of elements
!     Detect whether input vector is real or complex
!     Compute length of output array
!     Create output array
!     Fill output array by mapping
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec52' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tomanvec52'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Detect number of degrees of freedom per point in input vector
!         and activate input array and related arrays

      call sepactsolbf2 ( ivec1 )
      iprob = ks(ivec1)%iprob
      if ( indprfi/=0 ) then

!     --- Number of degrees of freedom per node not constant, &
!         not yet implemented

         call errint ( ks(ivec1)%typevec, 1 )
         call errchr ( 'manvec', 1 )
         call errsub ( 1393, 1, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Detect whether input vector is real or complex

      compli = ks(ivec1)%compl/=0

!     --- Create output array

      call sepcreasolbf1 ( iresvc, 116, nphysi, 'solout_elem', &
                           ks(ivec1)%compl )
      vectout => usoltemp

!     --- Compute starting addresses

      vectin => ks(ivec1)%sol

!     --- Clear output array

      vectout = 0d0

!     --- Fill output array by mapping

      call tomapsol116 ( vectin, vectout, nphysi, compli )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tomanvec52'

      end if
      call erclos ( 'tomanvec52' )

      end subroutine tomanvec52
