      subroutine tomapsol116 ( vecin, vecout, ndegfd, compli )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map vector of type 110 or 115 onto vector of type 116, i.e. map a vector
!     defined per node onto a vector defined per element
!     Line elements and connection elements are skipped
!     At this moment the mapping is restricted to linear triangles only
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     map
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndegfd
      double precision vecin(*), vecout(*)
      logical compli

!     compli         i    Indication if IVEC1 corresponds to a complex array
!                         (true) or to a real one (false)
!     ndegfd         i    Number of degrees of freedom per point
!     vecin          i    input vector containing the solution to be mapped
!     vecout         o    Output vector, result of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielgrp, ikelmc, ielem, iielem

!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOMAPSOLELM    Map vector of type 110 or 115 onto vector of type 116, &
!                    i.e. map a vector defined per node onto a vector defined
!                    per element for one element (Real case)
!     TOMAPSOLELMC   Map vector of type 110 or 115 onto vector of type 116, &
!                    i.e. map a vector defined per node onto a vector defined
!                    per element for one element (Complex case)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ikelmc := 1; ielem := 0
!     For ielgrp := 1 (1) nelgrp do
!        ishape := shape of elements in element group ielgrp
!        inpelm := number of nodes per element in ielgrp
!        inelem := number of elements in ielgrp
!        if ( ishape<3 ) then
!           raise ikelmc and ielem
!        else if ( ishape=3 ) then
!           for iielem := 1 (1) inelem do
!              ielem := ielem+1
!              Fill vecout for one element
!        else
!           error: not yet implemented
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomapsol116' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tomapsol116'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- ikelmc := 1; ielem := 0

      ikelmc = 1
      ielem = 0

!     --- For ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- ishape := shape of elements in element group ielgrp
!         inpelm := number of nodes per element in ielgrp
!         inelem := number of elements in ielgrp

         ishape = kmeshf(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         if ( ishape.lt.3 ) then

!        --- ishape<3, i.e. line lements
!            raise ikelmc and ielem

            ielem = ielem+inelem
            ikelmc = ikelmc + inpelm*inelem

         else if ( ishape.eq.3 ) then

!        --- ishape=3, linear triangles
!            for iielem := 1 (1) inelem do
!               ielem := ielem+1
!               Fill vecout for one element

            do iielem = 1, inelem

               ielem = ielem+1
               if ( compli ) then

!              --- Complex case

                  call tomapsolelmc ( vecout((ielem-1)*ndegfd*2+1), &
                                      vecin, kmeshc(ikelmc), ndegfd )

               else  ! ( .not. compli )

!              --- Real case

                  call tomapsolelm ( vecout((ielem-1)*ndegfd+1), &
                                     vecin, kmeshc(ikelmc), ndegfd )

               end if  ! ( compli )
               ikelmc = ikelmc+inpelm

            end do  ! iielem = 1, inelem

         else  ! ( ishape.gt.3 )

!        --- error: shape number not yet implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if  ! ( ishape.lt.3 )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tomapsol116'

      end if
      call erclos ( 'tomapsol116' )

      end
