      subroutine tomaptovec116 ( vecinp, vecout, nphys, ivec, idegfd, indpfi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-03-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map vector defined per node onto a vector defined per element
!     Values per element are computed as mean value
!     Real case
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     vector_defined_per_element
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nphys, ivec, idegfd, indpfi
      double precision vecinp(*), vecout(*)

!     idegfd         i    Degree of freedom sequence number for input
!                         vector
!                         At this moment only 0 or 1 allowed
!     indpfi         i    indication if kprob part f has been filled (>0)
!                         or not (0)  input vector
!     ivec           i    Sequence number of vector of special structure
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     vecinp         i    Input vector to be mapped
!     vecout         o    Output vector, result of the mapping
!                         vector is of type 116
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, jdegfd, inunkp, ielem, &
              iunk, k, inode
      double precision sum

!     ielem          Element number
!     ikelmc         last position used in kmesh part c
!     inode          Node sequence number
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     iunk           Position of degree of freedom in node
!     jdegfd         Sequence number of degree of freedom
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2829   Number of degrees of freedom per point not constant for input vector
!    2834   nelgrp>1
!    2868   Value of jdegfd not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get constants
!     For all elements do
!        For all unknowns per element do
!           Compute mean value
!           Store in array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomaptovec116' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tomaptovec116'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      inpelm = npelm
      inelem = nelem
      ikelmc = 0            ! last position used in kmeshc
      jdegfd = max(1,idegfd)  ! position of first degree of freedom

!     --- Perform some checks

      if ( nelgrp>1 ) then

!     --- Number of element groups>1, not yet implemented

         call errint ( nelgrp, 1 )
         call errsub ( 2834, 1, 0, 0 )

      end if  ! ( nelgrp>1 )
      if ( jdegfd/=1 ) then

!     --- jdegfd#1, not yet implemented

         call errint ( jdegfd, 1 )
         call errsub ( 2868, 1, 0, 0 )

      end if  ! ( jdegfd/=1 )
      if ( ierror/=0 ) go to 1000

      if ( indpfi==0 ) then

!     --- indpfi = 0, number of unknowns per point is constant
!         For all elements do

         inunkp = min(nphys,ivec)
         do ielem = 1, inelem

!        --- For all unknowns per element do

            do iunk = 1, inunkp

!           --- Compute mean value
!               Store in array

               sum = 0d0
               do k = 1, inpelm
                  inode = kmeshc(ikelmc+k)
                  sum = sum + vecinp((inode-1)*nphys+iunk)
               end do  ! k = 1, inpelm

               vecout((ielem-1)*ivec+iunk) = sum/inpelm

            end do  ! iunk = 1, nphys
            ikelmc = ikelmc+inpelm

         end do  ! ielem = 1, inelem

      else

!     --- indpfi>0, number of unknowns per point is not constant
!         Not yet implemented

         call errsub ( 2829, 0, 0, 0 )

      end if  ! ( indpfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tomaptovec116'

      end if  ! ( debug )
      call erclos ( 'tomaptovec116' )

      end
