      subroutine tost02 ( ilow, ihigh, cmat, celemmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 27-10-2010 remove jwork
!        version  2.0  date 06-12-2008 New parameter list
!        version  1.2  date 28-01-1993 Clear array jwork
!
!   copyright (c) 1989-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of element matrix corresponding to periodical boundary
!     conditions to second part of large matrix.
!     For this subroutine we need index5 instead of index5, since we only
!     need the connection with respect to the "right" unknowns. Real case
!     It is assumed that an element does not contain both "left" and
!     "right" unknowns corresponding to one point, which means that
!     periodical boundary conditions should be separated by at least
!     two elements.
!     Compare with to0007.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     matrix
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ilow, ihigh
      double complex, intent(inout) :: cmat(*), celemmt(icount,icount)

!     celemmt        i    Element matrix.
!     cmat          i/o   Part of matrix corresponding to essential and
!                         periodical boundary conditions.
!                         By this subroutine only the part with respect to
!                         periodical boundary conditions is filled
!     ihigh          i    Highest position number in USOL with respect to
!                         "right" periodical boundary conditions
!                         ihigh = nusol-nbound
!     ilow           i    Lowest position number in USOL with respect to "right"
!                         periodical boundary conditions
!                         ilow = nusol-nbound-ndepen+1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k, irel, icmat, jwork(nusolcac)

!     icmat          Position in array cmat
!     irel           Relative number of degrees of freedom with respect to
!                    periodical boundary conditions
!     j              Counting variable
!     jwork          Work array of length nusol
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array jwork
!     for j := 1(1)icount do
!        if (index5(j)>=ilow and<=ihigh) then
!           fill jwork with sequence numbers of unknowns in row index5(j)
!           with respect to matrix
!           for k := 1(1)icount do
!              if (index5(k)<=ihigh) then
!                 icmat := jwork(index5(k))
!                 cmat(icmat) := cmat(icmat)+Ski
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tost02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tost02'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array jwork

      do j = 1, icount
         if ( index5(j)<=ihigh ) jwork(index5(j)) = 0
      end do

!     --- Loop over degrees of freedom

      do j=1,icount
        if (index5(j)>=ilow .and. index5(j)<=ihigh) then

!       --- ilow<=index5(j)<=ihigh, degrees of freedom is "right"
!           periodical boundary condition
!           fill array jwork

           irel = index5(j)-ilow+1
           do k = irowsic(irel)+1,irowsic(irel+1)
              jwork(icolsic(k)) = k
           end do
           do k = 1,icount
              if (index5(k)<=ihigh) then

!             --- index5(k)<=ihigh, non-prescribed degrees of freedom

                 icmat = jwork(index5(k))
                 if ( icmat>0 ) cmat(icmat) = cmat(icmat)+celemmt(j,k)
              end if
           end do
         end if
      end do

1000  call erclos ( 'tost02' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tost02'

      end if  ! ( debug )

      end
