      subroutine tost03 ( ilow, amatloc, isign )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-07-2015 Use sepmodulebuild
!        version  2.4  date 21-04-2015 Extra debug
!        version  2.3  date 15-11-2012 Long integers
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute effect of periodical boundary conditions to right-hand side
!    in the case of real arithmetic
!    Compare with subroutine to0010
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     real
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isign, ilow
      double precision, intent(in) :: amatloc(*)

!   amatloc     i    Second part of large matrix corresponding to boundary
!                    conditions
!   ilow        i
!   isign       i    sign of multiplication, for right-hand side isign should
!                    be -1, in matrix-vector multiplication isign = 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k

!                    they are not
!     i              General loop variable
!     j              General loop variable
!     k              index in array rhsd
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   -
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The right-hand side must be updated in the following way
!               f = f + isign S   c
!                              ir  r
!    The matrix S   is stored in the sequence S
!                ir                            ri
!    Pseudo code
!    For i := 1(1)npboun do
!        for j := irowsic(i)+1,irowsic(i+1)
!           rhsd(icolsic(j)) := rhsd(icolsic(j))+isign*amatloc(j)*c(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tost03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tost03'
         write(irefwr,1) 'ilow, npboun, isign', ilow, npboun, isign
         call prinrl ( rprobs, npboun, 'rprobs' )
  1      format ( a, 1x, (10i6) )
      end if

      do i = 1, npboun
         do j = irowsic(i)+1, irowsic(i+1)
            k = icolsic(j)
            if ( k>=ilow ) k = kprobs(k-ilow+1)
            rhsd(k) = rhsd(k)+isign*amatloc(j)*rprobs(i,1)
         end do  ! i = 1, npboun
      end do  ! j = irowsic(i)+1, irowsic(i+1)

      if ( debug ) write(irefwr,*) 'End tost03'
      call erclos ( 'tost03' )
      end
