      subroutine tost05 ( ilow, ihigh, relemmt, relemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 24-07-2015 Use sepmodulebuild
!        version  3.1  date 18-01-2013 Remove unsymmetric
!        version  3.0  date 06-12-2008 New parameter list
!
!   copyright (c) 1992-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine tost05 multiplies the rows and columns of the element
!     matrix corresponding to the "right" periodical boundary conditions
!     by the corresponding constants
!     Real case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     periodical
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ilow, ihigh
      double precision, intent(inout) :: relemmt(icount,icount), &
                                         relemvc(icount)

!     ihigh          i    Highest position number in USOL with respect to
!                         "right" periodical boundary conditions
!                         ihigh = nusol-nbound
!     ilow           i    Lowest position number in USOL with respect to "right"
!                         periodical boundary conditions
!                         ilow = nusol-nbound-npboun+1
!     relemmt       i/o   Element matrix
!     relemvc       i/o   Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k
      double precision :: rmult

!     j              general loop variable
!     k              general loop variable
!     rmult          Multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for j := 1(1)icount do
!        if (index5(j)>=ilow and<=ihigh) then
!           mult := rprobsmult(index5(j)-ilow+1)
!           for k := 1(1)icount do
!              relemmt(i,j) := relemmt(i,j) * mult
!              relemmt(j,i) := relemmt(j,i) * mult
!           relemvc(j) := relemvc(j) * mult
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tost05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tost05'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over degrees of freedom

      do j = 1, icount
         if (index5(j)>=ilow .and. index5(j)<=ihigh) then

!        --- ilow<=index5(j)<=ihigh, degrees of freedom is "right"
!            periodical boundary condition

            rmult = rprobs(index5(j)-ilow+1,2)
            if ( matrix ) then
              do k = 1,icount
                 relemmt(j,k) = relemmt(j,k) * rmult
                 relemmt(k,j) = relemmt(k,j) * rmult
              end do
            end if
            if ( vector ) relemvc(j) = relemvc(j) * rmult
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tost05'

      end if  ! ( debug )
      call erclos ( 'tost05' )

      end
