      subroutine tost06 ( ilow, ihigh, relemmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-07-2015 Use sepmodulebuild
!        version  2.0  date 06-12-2008 New parameter list
!        version  1.0  date 12-08-1992
!
!   copyright (c) 1992-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine tost06 divides the columns of the element
!     matrix corresponding to the "right" periodical boundary conditions
!     by the corresponding constants
!     Real case
! **********************************************************************
!
!                       KEYWORDS
!
!     periodical
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ilow, ihigh
      double precision, intent(inout) :: relemmt(icount,icount)

!     ihigh          i    Highest position number in USOL with respect to
!                         "right" periodical boundary conditions
!                         ihigh = nusol-nbound
!     ilow           i    Lowest position number in USOL with respect to "right"
!                         periodical boundary conditions
!                         ilow = nusol-nbound-npboun+1
!     relemmt       i/o   Element matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k
      double precision :: rmult

!     j              general loop variable
!     k              general loop variable
!     rmult          Multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for j := 1(1)icount do
!        if (index5(j)>=ilow and<=ihigh) then
!           mult := rprobsmult(index5(j)-ilow+1)
!
!           for k := 1(1)icount do
!              relemmt(j,i) := relemmt(j,i) * mult
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tost06' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tost06'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over degrees of freedom

      do j=1,icount

         if (index5(j)>=ilow .and. index5(j)<=ihigh) then

!        --- ilow<=index5(j)<=ihigh, degrees of freedom is "right"
!            periodical boundary condition

            rmult = rprobs(index5(j)-ilow+1,2)

           do k = 1,icount
              relemmt(j,k) = relemmt(j,k) / rmult
           end do

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tost06'

      end if  ! ( debug )
      call erclos ( 'tost06' )

      end
