      subroutine prbearmasscon3 ( p_cav, nset1a, nset1b, nchanged )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2006
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Detect set 2 of all points with pressure<p_cav (may be empty)
!     Make set 1a equal to the rest of the nodes and 1b equal
!     to the empty set
! **********************************************************************
!
!                       KEYWORDS
!
!     lubrication
!     mass_conservation
!     incompressible
!     bearing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nset1a, nset1b, nchanged
      double precision p_cav

!     nchanged      i/o   Number of points that have moved from one set to
!                         another
!     nset1a         o    Number of nodes in set 1a
!     nset1b         o    Number of nodes in set 1b
!     p_cav          i    Cavity pressure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array nodes
!     Create set 3 (boundary conditions)
!     Create set 2
!     Make set 1a equal to the rest of the nodes and 1b equal
!     to the empty set
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbearmasscon3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbearmasscon3'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create set 2

      nchanged = 0
      nset1b = 0
      do i = 1, npoint
         if ( nodes(i)==0 .and. pressure(i)<p_cav ) then

!        --- pressure<p_cav, hence point belongs to set 2

            nset1b = nset1b+1
            nodes(i) = 2
            pressure(i) = p_cav
            nchanged = nchanged+1

         else if ( nodes(i)==2 .and. reac(i)>0d0 ) then

!        --- pressure<p_cav, hence point belongs to set 2

            nset1b = nset1b+1
            nodes(i) = 2
            pressure(i) = p_cav

         else if ( nodes(i)==2 ) then

!        --- point goes to set1a form 1b

            nodes(i) = 0
            nchanged = nchanged+1

         end if  ! ( pressure(i)<p_cav )

      end do  ! i = 1 , npoint

!     --- Make set 1a equal to the rest of the nodes and 1b equal
!         to the empty set

      nset1a = nrusol-nset1b

1000  call erclos ( 'prbearmasscon3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nset1a, nset1b, nset1b, nchanged', &
                          nset1a, nset1b, nset1b, nchanged
         call prinin ( nodes, npoint, 'nodes' )
         write(irefwr,*) 'End prbearmasscon3'

      end if

      end
