      subroutine prcheckphase ( tm, rewind )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 08-02-2006
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether the phase of an element has passed from fully solid
!     to fully liquid or vice versa
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision tm
      logical rewind

!     rewind        i/o   When true, the current time level is rewinded
!     tm             i    melting temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical prcheckphase1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCHECKPHASE1  Actual body of routine prcheckphase
!     PRCREATEPHASE1 Actual body of routine prcreatephase
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcheckphase' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcheckphase'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill phasenew

      call prcreatephase1 ( phasenew, temperature, tm )

!     --- Check difference between phase new and phase old

      rewind = prcheckphase1 ( phasenew, phaseold, nelem )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcheckphase'

      end if
      call erclos ( 'prcheckphase' )

      end subroutine prcheckphase
