      subroutine prcompenthalp3 ( numveclc, iinenthalpy, irhsdspecial, dt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the special part of the right-hand side of the newton iteration,
!     given the temperature and enthalpy in the previous iteration
!     Newton approach, i.e. enthalpy is defined per integration point
!     This is the part denoted by M H^i - mu M (T^i- tau(h^i))
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     temperature
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numveclc, iinenthalpy(*), irhsdspecial(*)
      double precision dt

!     dt             i    Time step
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     irhsdspecial   o    Special part of right-hand side
!     numveclc        i/o   Number of vectors stored in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iread(2), inpder(10), idummy(1), iprob, iseqtemp, &
              iseqenth, iseqcoef
      double precision output(1)

!     idummy         Dummy parameter
!     inpder         Input array for DERIVS
!     iprob          Actual problem number
!     iread          Defines how the input must be read
!                    Possible values:
!                    -1:  No input is required
!                     0:  All SEPRAN input has been read by
!                         subroutine SEPSTN until
!                         END_OF_SEPRAN_INPUT or
!                         end of file has been found
!                         This input is used
!                     1:  The input is read as described for DERIV
!     iseqcoef       Sequence number of coefficients input
!     iseqenth       Sequence number of enthalpy with respect to array isol
!     iseqtemp       Sequence number of temperature in solution vector
!     output         Dummy parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPDERIVS      Compute derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters from iinenthalpy
!     Fill specific arrays for derivs
!     Actual filling of enthalpy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompenthalp3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompenthalp3'
         call prinin ( iinenthalpy, 7, 'iinenthalpy' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Get parameters from iinenthalpy

      iseqenth = iinenthalpy(2)
      iseqtemp = iinenthalpy(3)
      iseqcoef = iinenthalpy(7)
      numveclc = max(numveclc,iseqenth)

!     --- Fill specific arrays for derivs

      iread(1) = -1   ! input stored in inpder
      iread(2) = iseqcoef ! sequence number of coefficients

      inpder(1) = 9   ! number of positions filled
      inpder(2) = 0   ! output vector of type 110 (solution vector)
      inpder(3) = 0   ! real vector
      inpder(4) = 42  ! icheld
      inpder(5) = 1   ! ix
      inpder(6) = 1   ! jdegfd
      inpder(7) = 0   ! ivec
      inpder(8) = numveclc  ! Number of vectors in isolut2d
      inpder(9) = 0   ! clear vector
      iprob = ks(isolut2d(iseqtemp))%iprob

      if ( debug ) then

!     --- Debug information

         call prinin ( inpder, 9, 'inpder' )
         call prinin ( iread, 2, 'iread' )
         write(irefwr,*) 'iseqenth, iseqtemp, iseqcoef, numveclc, iprob ', &
                          iseqenth, iseqtemp, iseqcoef, numveclc, iprob

      end if  ! ( debug )

!     --- Actual filling of special right-hand side

      call sepderivs ( inpder, irhsdspecial, &
                       isolut2d, iread, output, iprob, idummy )
      if ( ierror.ne.0 ) go to 1000

!     --- subdivide irhsdspecial by dt

      usoltemp => ks(irhsdspecial(1))%sol
      usoltemp = usoltemp/dt

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompenthalp3'

      end if
      call erclos ( 'prcompenthalp3' )

      end
