      subroutine prcompinv ( iinputinv, rinputinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Solve an inverse problem using a simple linearization method
!
! **********************************************************************
!
!                       KEYWORDS
!
!     inverse_problem
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputinv(*)
      double precision rinputinv(*)

!     iinputinv      i    Integer input array for subroutine prcompinvmat
!                         Contents
!                          1: Largest entry that has been filled by the user
!                          2: iseqsl:  sequence number of input for linear
!                                      solver
!                          3: iseqcf:  sequence number of input for coefficients
!                          4: iveccapacity: sequence number of the capacity
!                                           vector in array isolut
!                          5: ibegincv: curve number of first electrode
!                          6: iendcv : curve number of last electrode
!                          7: ivecsol: sequence number of the solution
!                                      vector in array isolut
!                          8: ivecreact: sequence number of the reaction
!                                        vector in array isolut
!                          9: ielgrploc: element group sequence number
!                                        corresponding to inverse problem
!                         10: iveceps: sequence number in array solut of
!                                      array in which the values of epsilon
!                                      are stored
!                         11: method: type of method to solve the
!     rinputinv      i    Double precision input array for subroutine
!                         prcompinvmat
!                         Contents
!                          1: value of delta_eps
!                          2: value of regularization parameter mu
!                          3: value of eps_ref
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iveccapacity
      double precision, pointer :: capacitycp(:)

!     capacitycp     Copy of capacity
!     iveccapacity   Sequence number of capacity vector in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOMPINV01    Solve the system of equations arising in the inverse
!                    problem
!     PRCOMPINVMAT   Compute the matrix S for the inverse problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2133   Sequence number of vector too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy capacity vector into icapacity
!     Build matrix S
!     Copy capacity vector back
!     Solve inverse problem depending on the method used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompinv' )

!     --- Save capacity vector (measured values) into icapacity

      iveccapacity = 3
      if ( iinputinv(1)>3 ) iveccapacity = iinputinv(4)
      if ( iveccapacity>maxvc1 ) then

!     --- error: iveccapacity too large

         call errint ( iveccapacity, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'capacity', 1 )
         call errsub ( 2133, 2, 0, 1 )
         go to 1000

      end if

      capacity => ks(isolut2d(iveccapacity))%sol
      allocate ( capacitycp(size(capacity)) )
      capacitycp = capacity

!     --- Build matrix S

      call prcompinvmat ( iinputinv, rinputinv )

!     --- Solve inverse problem depending on the method used
!         First copy capacity vector back

      capacity = capacitycp
      deallocate ( capacitycp )

      call prcompinv01 ( iinputinv, rinputinv )

!     --- delete arrays c0 and c1

      deallocate ( vecc0, vecc1 )

1000  call erclos ( 'prcompinv' )
      end
