      subroutine prcompinver ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 25-10-1999
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Help subroutine to be called by sepcomp or sepfree
!    This subroutine calls the actual inverse subroutine prcompinv
! **********************************************************************
!
!                       KEYWORDS
!
!     inverse_problem
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmoduleinput
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput     i    Array containing information about the eigenvalue
!                     problem
!                     Contents
!                     1: contains the sequence number for the input
!                     2: IPROB, problem sequence number
!                     3: IVECTOR, vector sequence number of output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivectr, iseqnr, i, ipos, maxcal

!     i              Counting variable
!     ipos           Position of input arrays in sepmoduleinput
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     ivectr         standard sepran array denoting usol or vector of
!                    special structure defined in nodal points
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRCOMPINV      Solve an inverse problem using a simple linearization
!                    method
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2132   No input for solution of inverse problem found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompinver' )
      iseqnr = iinput(1)
      ivectr = iinput(3)

      if ( ioutp>=0 .and. itimloop<=1 ) then

!     --- Pint information

         write(irefwr,100) ivectr
100      format ( ' Fill vector',i3,' by solving inverse problem ' )

      end if
      call sepgetinpfinf ( 25, iseqnr, ipos, maxcal )
      if ( maxcal==0 ) then

!     --- No input for inverse problem found

         call errsub ( 2132, 0, 0, 0 )
         go to 1000

      end if

      call prcompinv ( in(ipos)%iinput, re(ipos)%rinput )

1000  call erclos ( 'prcompinver' )

      istartmn = istartmn + 4
      end
