      subroutine prcomppsisolv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-05-2014 New call to sepsolvel
!        version  1.0  date 16-02-2009
!
!   copyright (c) 2009-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system corresponding to pseudo stream function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     2d
!     3d
!     stream_function
!     free_surface
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulemain
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPINITINPSOL  initialize arrays inpsol and rinsol for seplinsol
!     SEPLINSOL      Body of linear solver
!     SEPLINSOLKPROB Fill relevant parameters into sepmodulekprob for
!                    subroutine linsol
!     SEPUPDINPSOL   Update inpsol for special situations
!                    Put parameters in sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomppsisolv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomppsisolv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Solve system of equations

      call sepinitinpsol  ! initialize inpsol/rinsol
      inpsol(1,1) = ninpsol    ! Number of positions filled
      inpsol(3,1) = 10         ! idr
      inpsol(4,1) = 3          ! ilu preconditioning
      inpsol(5,1) = 100        ! maxiter
      inpsol(6,1) = -1         ! iprint
      inpsol(14,1) = 1         ! s-dimension
      rinsol(1,1) = 1d-3       ! accuracy

      iseqsol = 1
      iseqreac = 0
      iseqintmat = 1
      keepmat = 0
      iseqsolut = 1
      iseqmt = stiffmat(1)
      call sepupdinpsol  ! Update inpsol for special situations
      call seplinsolkprob ( irhsd )
      call seplinsol ( stiffmat, isolut, irhsd )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcomppsisolv'

      end if  ! ( debug )
      call erclos ( 'prcomppsisolv' )

      end
