      subroutine prcompvol ( volumes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-06-2000 New call to elp624
!        version  1.0  date 22-10-1999
!
!   copyright (c) 1999-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute volume of all elements and store in array volumes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision volumes(*)

!     volumes        o    Array of length nelem containing the volumes of all
!                         elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision w(27), loccoor(81), dummy(9*27)
      integer ikelmc, i

!     dummy          Dummy parameter
!     i              Counting variable
!     ikelmc         position in kmesh part c
!     loccoor        array containing the local coordinates of the elements
!     w              array of length m containing the weights for
!                    integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!    nelgrp := number of element groups
!    ielem := 0
!    for ielgrp := 1 (1) nelgrp do
!       inelem := number of elements in element group ielgrp
!       inpelm := number of nodes in elements of group ielgrp
!       ishape := shape number of group ielgrp
!       for irelem := 1 (1) inelem do
!          ielem := ielem+1
!          volumes(ielem) := volume of element ielem
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompvol' )

!     --- initializations

      call elcomm
      jcart  = 1
      ideriv = 0
      ikelmc = 1
      ielem  = 0

!     --- for ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group ielgrp
!         inpelm := number of nodes in elements of group ielgrp
!         ishape := shape number of group ielgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         ifirst = 0
         n = inpelm
         m = n

!        --- for irelem := 1 (1) inelem do

         do irelem = 1, inelem

!        --- ielem := ielem+1
!            volumes(ielem) := volume of element ielem

            ielem = ielem+1
            if ( ishape<=0 ) then

!           --- ishape <= 0, set volume=0

               volumes(ielem) = 0d0

            else if ( ishape>21 ) then

!           --- ishape > 21, shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               do i = 1, inelem
                  volumes(ielem+i-1) = 0d0
               end do
               ielem = ielem+inelem-1
               ikelmc = ikelmc+inpelm*inelem
               go to 600

            else

!           --- 0 < ishape < 22, standard case

               call elm800basefn ( coor, loccoor, w, dummy, dummy, &
                                   dummy, kmeshc(ikelmc), dummy, dummy, &
                                   ishape, dummy, dummy, dummy, dummy )

            end if

!           --- add weights in order to get volume

            volumes(ielem) = 0d0
            do i = 1, m
               volumes(ielem) = volumes(ielem)+w(i)
            end do
            ikelmc = ikelmc+inpelm

         end do

600   end do
      call erclos ( 'prcompvol' )
      end
