      subroutine prfillgridnod ( interf, ninterf, &
                                 grid, nx, origin, xmaxar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 24-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information of a set of nodes in rectangular 2d or 3d grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     rectangle
!     grid
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ninterf, interf(ninterf), nx(*), grid(*)
      double precision origin(*), xmaxar(*)

!     grid           o    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the following way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     interf         i    Contains node numbers of interface
!     ninterf        i    number of points in ndim directions of the grid
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
!     xmaxar         i    Contains the coordinates xmax, ymax ..
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRFLGRIDNOD2D  Fill information of a set of nodes in rectangular 2d grid
!     PRFLGRIDNOD3D  Fill information of a set of nodes in rectangular 3d grid
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillgridnod' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillgridnod'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array grid

      if ( ndim==2 ) then

!     --- ndim = 2

         call prflgridnod2d ( grid, nx(1), nx(2), ninterf, &
                              coor, interf, 1, origin(1), &
                              xmaxar(1), origin(2), xmaxar(2) )

      else

!     --- ndim = 3

         call prflgridnod3d ( grid, nx(1), nx(2), nx(3), ninterf, &
                              coor, interf, 1, origin(1), &
                              xmaxar(1), origin(2), xmaxar(2), &
                              origin(3), xmaxar(3) )

      end if  ! ( ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillgridnod'

      end if  ! ( debug )
      call erclos ( 'prfillgridnod' )

      end subroutine prfillgridnod
