      subroutine prfillintfnmsl ( nmeshl, lennmeshl, nmeshc, &
                                  numedgeintf, numnodeintf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nmeshl for interface (2d)
!     Edge numbers of edges connected to a node
!     The local numbering of the interface is used
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     node
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshc(*), lennmeshl, nmeshl(lennmeshl), numedgeintf, &
              numnodeintf

!     lennmeshl     i/o   Declared length of array nmeshl
!                         At output the computed length
!     nmeshc         i    nmesh part c, contains local node numbers
!                         corresponding to local edges of interface
!     nmeshl         o    nmesh part l, contains local edge numbers
!                         corresponding to local nodes of interface
!     numedgeintf    i    Number of edges at the interface
!     numnodeintf    i    Number of nodes at the interface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istnmeshc, istart, i, j, inode, nnodes
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     inode          Node sequence number
!     istart         First position used at a part
!     istnmeshc      Starting address in array nmeshc
!     iwork          integer work array, see comments
!     j              Counting variable
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINNMESHL     print nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillintfnmsl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillintfnmsl'
         write(irefwr,1) 'lennmeshl, numedgeintf, numnodeintf', &
                          lennmeshl, numedgeintf, numnodeintf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(numnodeintf), stat = error )
      if ( error/=0 ) call eralloc ( error, numnodeintf, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork = 0  ! clear array iwork

!     --- For all edges do
!         Fill array iwork with number of edges per node

      do i = 1, numedgeintf

         istnmeshc = nmeshc(i)
         nnodes = nmeshc(istnmeshc)  ! number of "local" nodes
         do j = 1, nnodes
            inode = nmeshc(istnmeshc+j)  ! "local" node number
            iwork(inode) = iwork(inode)+1 ! raise iwork
         end do  ! j = 1, nedges
         istnmeshc = istnmeshc+nnodes

      end do  ! i = 1, numelms

      if ( debug ) call prinin ( iwork, numnodeintf, 'iwork/1' )

!     --- Fill starting addresses in nmeshl

      istart = numnodeintf+1
      do i = 1, numnodeintf
         nmeshl(i) = istart
         nmeshl(istart) = iwork(i)
         istart = istart+iwork(i)+1
      end do  ! i = 1, numnodeintf

      call inichkarlen ( istart-1, lennmeshl, 'nmeshl' ) ! check array length
      if ( ierror/=0 ) go to 1000

      iwork(1:numnodeintf) = nmeshl(1:numnodeintf)  ! copy starting addresses

      if ( debug ) call prinin ( iwork, numnodeintf, 'iwork/2' )

!     --- Actual filling of nmeshl

      do i = 1, numedgeintf

         istnmeshc = nmeshc(i)
         nnodes = nmeshc(istnmeshc)  ! number of "local" nodes
         do j = 1, nnodes
            inode = nmeshc(istnmeshc+j)  ! "local" node number
            iwork(inode) = iwork(inode)+1 ! raise iwork
            nmeshl(iwork(inode)) = i
         end do  ! j = 1, nedges
         istnmeshc = istnmeshc+nnodes

      end do  ! i = 1, numelms

      lennmeshl = istart-1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinnmeshl ( numedgeintf, nmeshl )
         write(irefwr,*) 'End prfillintfnmsl'

      end if  ! ( debug )
      call erclos ( 'prfillintfnmsl' )

      end
