      subroutine princip03 ( ichoice, stress, outvec1, outvec2, nnodes )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Computes the principal stresses from a 3d stress tensor
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     stress
!     principal_stress
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, nnodes
      double precision stress(6,*), outvec1(3,*), outvec2(9,*)

!     ichoice        i    Integer choice parameter with the following meaning
!                         1: Compute the eigenvalues only
!                         2: Compute the eigenvectors only
!                         3: Compute the scaled eigenvectors only
!                         4: Compute the eigenvectors  and eigenvalues
!                         5: Compute the scaled eigenvectors and eigenvalues
!     nnodes         i    Number of nodes
!     outvec1        i    In this vector of size ndim x nnodes the eigenvalues
!                         are stored
!     outvec2        o    In this vector of size ndim^2 x nnodes the
!                         eigenvectors are stored
!     stress         i    Array of size 6 x nnodes containing the stress tensor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, nrot, j, i, k
      double precision stress3d(3,3), eigvect(9), eigval(3), &
                       amatcopy(9), bhelp(3), zhelp(3)

!     amatcopy       Work array of size ndim x ndim for subroutine prcompeig
!     bhelp          Work array of size ndim for subroutine prcompeig
!     eigval         Array of size ndim where the eigenvalues are stored
!     eigvect        Array of size ndim  x ndimwhere the eigenvalues are stored
!     i              Counting variable
!     inode          Node sequence number
!     j              Counting variable
!     k              Counting variable
!     nrot           Number of rotations applied
!     stress3d       Local stress tensor
!     zhelp          Work array of size ndim for subroutine prcompeig
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOMPEIG      Compute all eigenvalues and eigenvectors of a small
!                    symmetric matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For inodes := 1 (1) nnodes do
!        Copy local stress tensor from global one (with rearranging)
!        compute the eigenvalues and eigen vectors
!        Store eigenvalues and eigenvectors depending on ichoice
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'princip03' )
      do inode = 1, nnodes

!     --- for every node fill the local stress tensor

         do j = 1, 3
            stress3d(j,j) = stress(j,inode)
         end do
         stress3d(1,2) = stress(4,inode)
         stress3d(2,3) = stress(5,inode)
         stress3d(1,3) = stress(6,inode)
         stress3d(2,1) = stress3d(1,2)
         stress3d(3,1) = stress3d(1,3)
         stress3d(3,2) = stress3d(2,3)

!        --- compute the eigenvalues and eigen vectors

         call prcompeig ( stress3d, 3, eigval, eigvect, nrot, amatcopy, &
                          bhelp, zhelp )

         if ( ichoice.eq.1 .or. ichoice.ge.4 ) then

!        --- store the eigenvalues

            do j = 1, 3
               outvec1(j,inode) = eigval(j)
            end do

         end if

         if ( ichoice.eq.2 .or. ichoice.eq.4 ) then

!        --- store the eigenvectors

            do j = 1, 9
               outvec2(j,inode) = eigvect(j)
            end do

         else if ( ichoice.eq.3 .or. ichoice.eq.5 ) then

!        --- store the scaled eigenvectors

            do j = 1, 3
               do i = 1, 3
                  k = (j-1)*3+i
                  outvec2(k,inode) = eigvect(k)*abs(eigval(j))
               end do
            end do

         end if

      end do

      call erclos ( 'princip03' )
      end
