      subroutine printiter ( iprint, iter, rnorm, priter, residu, mult )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-09-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays priter and residu as function of iprint and iter
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iprint, iter, priter(*), mult
      double precision rnorm, residu(*)

!     iprint         i    Indication of the amount of print output required.
!                         Possible values see prcgst or pridr
!     iter           i    Iteration sequence number
!     mult           i    Number of iteration steps between calls to this
!                         subroutine
!     priter        i/o   Output array to store iteration numbers
!     residu        i/o   Array to store norms of resiuals
!     rnorm          i    Norm of residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, jiter

!     j              Counting variable
!     jiter          Iteration number, adapted to mult
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printiter' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printiter'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      jiter = 1+(iter-1)/mult

      if ( iprint==2 ) then

!     --- iprint = 2, print the first and last residuals

         if ( jiter<=10 ) then

            write ( irefwr, 150 ) iter, rnorm
            priter(jiter) = iter
            residu(jiter) = rnorm

         else

            do j = 2, 10

               priter(j-1) = priter(j)
               residu(j-1) = residu(j)
            end do

            priter(10) = iter
            residu(10) = rnorm

         end if
      end if
      if ( iprint>=3 ) then

!     --- iprint>=3,  Maximal output

         write ( irefwr, 150 ) iter, rnorm
150      format( i10, ss,1pe15.2e2 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printiter'

      end if  ! ( debug )
      call erclos ( 'printiter' )

      end
