      subroutine prmarkintfedge ( iobstfaces, markintfedge, nmeshb )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all edges in 3d interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     edge
!     3d
!     mark
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markintfedge(*), iobstfaces(*), nmeshb(*)

!     iobstfaces     i    contains edge sequence numbers of boundary
!                         of levelset
!     markintfedge   o    Array of size nedges to mark the edges
!                         of the interface (1) else 0
!     nmeshb         i    nmesh part b
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilev, kstart, numedges, numfaces, i, ipstart, iface

!     i              Counting variable
!     iface          Face sequence number
!     ilev           Sequence number of level set being considered
!     ipstart        Starting address in array nmeshb
!     kstart         Starting address in iobstfaces
!     numedges       number of edges
!     numfaces       number of faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarkintfedge' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarkintfedge'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilev = 1  ! level set sequence number
      kstart = iobstfaces(ilev)      ! start of edges in iobstfaces
      numfaces = iobstfaces(kstart)  ! number of faces in interface

      do i = 1, numfaces
         iface = iobstfaces(kstart+i)  ! face number
         ipstart = nmeshb(iface)       ! starting address of face in nmeshb
         numedges = nmeshb(ipstart)    ! number of edges in face
         markintfedge(abs(nmeshb(ipstart+1:ipstart+numedges))) = 1  ! mark edges
      end do  ! i = 1, numfaces

1000  call erclos ( 'prmarkintfedge' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarkintfedge'

      end if  ! ( debug )

      end
