      subroutine prmarklev ( iobst, interf, ninterf, iwork, isig )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 14-11-2013 Extra debug
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 24-10-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iwork and interf with interface points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, interf(*), ninterf, iwork(*), isig

!     interf         o    Contains node numbers of interface
!     iobst          i    Obstacle sequence number
!     isig           Sign of level set inside obstacle
!     iwork          o    integer work array of length npoint
!                          -1:  node with phi<0
!                           0:  node on interface
!                           1:  node with phi>0
!     ninterf        o    Number of nodes in interf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRMARKLEV1     Fill arrays iwork and interf with interface points
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarklev' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarklev'
         write(irefwr,1) 'isig, iobst, numobst', &
                          isig, iobst, numobst
         call sepprinipobst ( ipointobst )

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize output

      ninterf = 0           ! number of nodes in interface
      iwork(1:npoint) = -isig  ! all points outside obstacle

      if ( numobst==0 ) go to 1000  ! no obstacle found

!     --- actual call

      call prmarklev1 ( ipointobst, iwork, interf, ninterf, &
                        isig, iobst )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( interf, ninterf, 'interf' )
         call prinin ( iwork, npoint, 'iwork' )
         write(irefwr,*) 'End prmarklev'

      end if  ! ( debug )
      call erclos ( 'prmarklev' )

      end
