      subroutine prmarklevset ( ipointobst, ilev, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-08-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set iwork(i)=1 for all nodes i with level set 0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), ilev, iwork(*)

!     ilev           i    Sequence number of level being considered
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     iwork          o    integer work array of size npoint
!                         if iwork(i) = 0 point is not on the interface, &
!                         if 1 it is
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, numnodes, istart, inodp

!     inodp          Nodal point number
!     istart         First position used at a part
!     j              Counting variable
!     numnodes       Number of nodal points in obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarklevset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarklevset'

      end if
      if ( ierror/=0 ) go to 1000

      istart = ipointobst(ilev)
      numnodes = ipointobst(istart+2)  ! nodes on boundary obst
      istart = istart+3+ipointobst(istart)+ipointobst(istart+1)

      do j = 1, numnodes

         inodp = ipointobst(istart+j)
         iwork(inodp) = 1

      end do  ! j = 1, numnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarklevset'

      end if
      call erclos ( 'prmarklevset' )

      end
