      subroutine prmatlabrmat ( n, amat, intmt1, intmt2, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 03-01-2008 Extra parameter filename
!        version  1.0  date 06-05-2003
!
!   copyright (c) 2003-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print a rectangular compact matrix in a form suitable to be read by matlab
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
!     matlab
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, intmt1(*), intmt2(*)
      double precision amat(*)
      character (len=*) filename

!     amat           i    matrix to be printed
!     filename       i    Name of file to which the matlab output must be
!                         written
!     intmt1         i    first part of array intmat contains number of
!                         non-zero columns per row accumulated
!     intmt2         i    part 2 of array intmat, contains column numbers of
!                         non-zero elements per row
!     n              i    number of rows in matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefwrsav, i, jend, j, jstart

!     i          Counting variable
!     irefwrsav      Help variable to store the value of irefwr
!     j          Counting variable
!     jend       end position of a row
!     jstart     starting position of row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabrmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabrmat'

      end if

!     --- open file

      irefwrsav = irefwr    ! save irefwr
      irefwr = 19           ! new definition (temporary)
      open ( unit = irefwr, file = filename )  ! open file

!     --- Write matrix

      do i=1,n
         jstart=intmt1(i)+1
         jend=intmt1(i+1)
         write(irefwr,110) (i,intmt2(j),amat(j),j=jstart,jend)
110      format(2i6,ss,1pe17.8e2)
      end do

!     --- close file

      close( unit = irefwr )  ! close file
      irefwr = irefwrsav      ! reset irefwr

      call erclos ( 'prmatlabrmat' )
      if ( debug ) write(irefwr,*) 'End prmatlabrmat'

      end
