      subroutine prmatlabvector ( irhsd, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 31-03-2015 Replace er0061 by inicopindarr1
!        version  1.4  date 11-12-2012 Long integers
!        version  1.3  date 20-10-2010 Extra debug
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a sepran vector in a form suitable for reading
!     in matlab
!     reading in matlab may be done in the following way:
!     sepget make_matrix.m
!     give the command make_matrix in the matlab command window
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     real
!     print
!     matlab
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irhsd
      character (len=*) filename

!     filename       i    Name of file to which the matlab output must be
!                         written
!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icompl
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRVEC2MATLAB   Print a sepran vector in a form suitable for reading
!                    in matlab
!     PRVEC2MATLABC  Print a complex sepran vector in a form suitable for
!                    reading in matlab
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
!     The vector is written to the file with name filename
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabvector' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabvector'
         call prinin ( irhsd, 5, 'irhsd' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepactsolbf1 ( irhsd )
      compl = ks(irhsd)%compl==1

      usoltemp => ks(irhsd)%sol
      if ( compl ) then

!     --- complex case

         call prvec2matlabc ( nrusol, usoltemp, filename )

      else

!     --- real case

         call prvec2matlab ( nrusol, usoltemp, filename )

      end if  ! ( compl )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlabvector'

      end if  ! ( debug )
      call erclos ( 'prmatlabvector' )

      end subroutine prmatlabvector
