      subroutine prmindist1 ( numfound, nodes, ndim, mindist, coor, &
                              node, markintf, nmeshc, nmeshl, &
                              coorinter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find minimum distance of node to set of nodes in interface
!     It concerns the case of a non-adapted mesh
!     This subroutine is the equivalent of prmindist
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     node
!     distance
!     minimum
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numfound, nodes(numfound), ndim, markintf(*), nmeshc(*), &
              node, nmeshl(*)
      double precision coor(ndim,*), mindist, coorinter(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorinter      i    Contains coordinates of the interface nodes
!                         in the local numbering of the interface
!     markintf      i/o   Array of size nedges or nfaces to mark the edges or
!                         faces of the interface (1) else 0
!     mindist       i/o   Minimum distance of node wrt interface
!     ndim           i    Dimension of the space.
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     node           i    Node number
!     nodes          i    Contains node numbers in interface that are part
!                         of the set of cells under consideration
!     numfound       i    Number of nodes in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x(ndim), xintf(ndim), dx(ndim)

!     dx             x-xintf
!     i              Counting variable
!     x              co-ordinates of point
!     xintf          co-ordinates of point in interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRMINDIST2D    Find minimum distance of node to set of nodes in edges of
!                    interface
!                    It concerns the case of a non-adapted mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmindist1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmindist1'
         write(irefwr,1) 'node, numfound', node, numfound
         write(irefwr,2) 'mindist', mindist
         call prinin ( nodes, numfound, 'local interface nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store coordinates of node in x

      x = coor(:,node)

!     --- Find shortest distance from points in array nodes to node

      do i = 1, numfound

         xintf = coorinter(:,nodes(i))
         dx = x-xintf
         mindist = min(mindist,dot_product(dx,dx))

      end do  ! i = 1, numfound

      if ( debug ) write(irefwr,2) 'mindist', mindist

!     --- Find shortest distance from node to edges in interface

      call prmindist2d ( markintf, x, coorinter, numfound, nodes, &
                         nmeshl, mindist, nmeshc, ndim )

1000  call erclos ( 'prmindist1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'mindist', mindist
         write(irefwr,*) 'End prmindist1'

      end if  ! ( debug )

      end
