      subroutine prnodecellij ( indi, nx, origin, dx, ndim, x )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find ij cell index of node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     cell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, indi(ndim), nx(ndim)
      double precision origin(ndim), dx(ndim), x(ndim)

!     dx             i    contains the difference between xmax, xmin etc
!     indi           o    contains ijk-index of cell in which point is lying
!     ndim           i    Dimension of the space.
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
!     x              i    co-ordinates of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prnodecellij' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prnodecellij'
         write(irefwr,1) 'ndim', ndim
         call prinin ( nx, ndim, 'nx' )
         call prinrl ( x, ndim, 'x' )
         call prinrl ( dx, ndim, 'dx' )
         call prinrl ( origin, ndim, 'origin' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indi = min(nx,int((x-origin)/dx)+1)

1000  call erclos ( 'prnodecellij' )
      if ( debug ) then

!     --- Debug information

         call prinin ( indi, ndim, 'indi' )
         write(irefwr,*) 'End prnodecellij'

      end if  ! ( debug )

      end
