      subroutine prob10cv1 ( n, kstep, iwork1, jdgfd1, &
                             jdgfd2, jdgfd3, iwork, iwork2, &
                             jtrans, ifirst, ilast, &
                             iinp, mappinglc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-11-2007 Give warning in case of double transform
!        version  1.0  date 01-10-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n for curves with degrees of freedom
!     and transformation factors (coortr)
!     jstep<0
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  n, kstep, iwork1(*), jdgfd1, &
               jdgfd2, jdgfd3, iwork(*), iwork2(*), &
               jtrans, ifirst, ilast, &
               iinp, mappinglc(*)

!     ifirst         i    First  curve number
!     iinp          i/o   last position used in array iinputkp
!     ilast          i    Last curve number
!     iwork          i    integer work array of length npoint
!     iwork1        i/o   integer work array length max(npoint, ncurvs)
!     iwork2        i/o   integer work array of length npoint
!                         iwork2 is used to mark nodes that have been
!                         transformed already
!                         iwork2(i) contains the number of times a node
!                         is transformed
!     jdgfd1         i    Degree of freedom
!     jdgfd2         i    Degree of freedom
!     jdgfd3         i    Degree of freedom
!     jtrans         i    Type of local transformation
!                         1:  standard transformation
!                         2:  symmetric transformation
!                         3:  non-symmetric transformation
!     kstep          i    Step in loop
!     mappinglc        i    Array of size npointglobal
!                         Contains the local nodal point numbers corresponding
!                         to nodes in the global mesh
!                         If gobal node is not present in local mesh the
!                         corresponding value in mappinglc is 0
!     n              i    number of nodes in curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, nodp, inusol, inunkp, ideg, l, iprec, inext, j, ifunc, &
              nodpglob
      double precision z

!     ideg           Degree of freedom
!     ifunc          Choice parameter with respect to function calls
!     inext          Next point in curve
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     iprec          Prior point in curve
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     nodp           Nodal point sequence number (local mesh)
!     nodpglob       Nodal point sequence number (Global mesh)
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision functr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     FUNCTR         User function subroutine to compute the local
!                    transformation
!     PROB10TR       Compute local transformation for points on a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2909   Double transformation
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10cv1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10cv1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      z = 0d0

      n = n-1
      do k = 1, n, kstep
         do j = 1, kstep-1

            if ( ipstkmeshpar>0 ) then

!           --- parallel case, use array mappinglc

               nodpglob = iwork1(k+j)
               nodp = mappinglc(nodpglob)

            else

!           --- serial case

               nodp = iwork1(k+j)
               nodpglob = nodp

            end if  ! ( ipstkmeshpar>0 )
            iprec = iwork1(k+j-1)
            inext = iwork1(k+j+1)

            if ( nodp<=0 ) go to 300  ! skip non-positive nodes

            if ( iwork(nodp)==0 ) then

!           --- node not used in this part

               if ( iwork2(nodp)>0 ) then

!              --- Node has been tranformed before, give warning and skip

                  call errint ( nodp, 1 )
                  call errwar ( 2909, 1, 0, 0 )
                  go to 300

               end if  ! ( iwork2(nodp)>0 )

               if ( indprf>0) then

!              --- indprf>0  kprob part f contains the
!                  number of unknowns

                  inusol = kprobf(nodp)
                  inunkp = kprobf(nodp+1)-inusol
                  if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 300
                  if ( ndim==3 .and. jdgfd3>inunkp ) goto 300
               else
                  inusol = nunkp*(nodp-1)
               end if

               iwork(nodp) = 1
               iwork2(nodp) = iwork2(nodp)+1

               npltra = npltra+1
               ideg = inusol+jdgfd1
               if ( indprh/=0) ideg = kprobh(ideg)
               kprobn(1,npltra) = ideg
               ideg = inusol+jdgfd2
               if ( indprh/=0 ) ideg = kprobh(ideg)
               kprobn(2,npltra) = ideg
               if ( ndim==3 ) then
                  ideg = inusol+jdgfd3
                  if ( indprh/=0) ideg = kprobh(ideg)
                  kprobn(3,npltra) = ideg
               end if
               if ( jtrans==1 .or. jtrans==4 ) then

!              --- Fill array coortr

                  call prob10tr ( maxtrn, jtrans, coortr, npltra, &
                                  ndim, coorglob, iprec, inext, &
                                  nodpglob, k, ifirst, ilast )
                  if ( ierror/=0 ) go to 1000

               else

!              --- compute R and V as given by the user

                  do l = 1, 4*(jtrans-1)
                     ifunc = iinputkp(iinp+l)
                     if ( ifunc<0 ) then
                        if ( ndim==3 ) z = coorglob(3,nodpglob)
                        coortr(l,npltra) = functr ( -ifunc, &
                              coorglob(1,nodpglob), coorglob(2,nodpglob), z )
                     else
                        coortr(l,npltra) = rinputkp(ifunc)
                     end if
                  end do
               end if
               if ( jtrans==2 .and. maxtrn==8 ) then
                  do l = 1, ndim**2
                     coortr(ndim**2+l,npltra) = coortr(l,npltra)
                  end do
               end if
            end if

300      end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10cv1'

      end if  ! ( debug )
      call erclos ( 'prob10cv1' )

      end
