      subroutine prob10sfusp ( kmeshl, iinp, jtrans, inpelm, coor, t1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-10-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the user points P1 and P2
!     fill the tangential direction in t1
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshl(*), iinp, jtrans, inpelm
      double precision coor(3,*), t1(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iinp          i/o   last position used in array iinputkp
!     inpelm         i    number of points per element
!     jtrans         i    Type of local transformation
!                         1:  standard transformation
!                         2:  symmetric transformation
!                         3:  non-symmetric transformation
!     kmeshl         i    KMESH part l, contains nodal point numbers of the
!                         user points
!     t1             o    First tangential vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iuserp1, iuserp2, k
      double precision h

!     h              Length of vector
!     iuserp1        First user point
!     iuserp2        Second user point
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1503   User point is not a nodal point
!    2061   The same point is used for both parts of the tangential vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10sfusp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10sfusp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iuserp1 = kmeshl(iinputkp(iinp+1+(jtrans-1)*9+inpelm))
      iuserp2 = kmeshl(iinputkp(iinp+2+(jtrans-1)*9+inpelm))
      if ( iuserp1==0 ) then
         call errint ( iinputkp(iinp+1+(jtrans-1)*9+inpelm), 1 )
         call errsub ( 1503, 1, 0, 0 )
      end if
      if ( iuserp2==0 ) then
         call errint ( iinputkp(iinp+2+(jtrans-1)*9+inpelm), 1 )
         call errsub ( 1503, 1, 0, 0 )
      end if
      if ( iuserp1==iuserp2 ) then
         call errint ( iuserp1, 1 )
         call errsub ( 2061, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      do k = 1, 3
         t1(k) = coor(k,iuserp2) - coor(k,iuserp1)
      end do
      h = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
      do k = 1, 3
         t1(k) = t1(k)*h
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10sfusp'

      end if  ! ( debug )
      call erclos ( 'prob10sfusp' )

      end
