      function prpythagoras ( a, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     finds sqrt(a^2+b^2) without overflow or destructive underflow
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision prpythagoras, a, b

!     a              i    parameter a
!     b              i    parameter b
!     prpythagoras   o    sqrt(a^2+b^2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision p,r,s,t,u

!     p              ?
!     r              ?
!     s              ?
!     t              ?
!     u              ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpythagoras' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpythagoras'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      prpythagoras = 0d0
      p = max ( abs(a), abs(b) )
      if ( abs(p)<=0d0 ) go to 1000

      r = (min(abs(a),abs(b))/p)**2
      do while ( abs(r)>0d0 )
         t = 4.0d0 + r
         s = r/t
         u = 1.0d0 + 2.0d0*s
         p = u*p
         r = (s/u)**2 * r
      end do ! while ( absr)>0d0 )
      prpythagoras = p

1000  call erclos ( 'prpythagoras' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpythagoras'

      end if  ! ( debug )

      end
