      subroutine sepfillkmeshv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshv with element groups coupled to element groups
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: congrp(nelgrp,nelgrp), ncongrp(nelgrp+1), &
                 i, j, ipos, ielgrp

!     congrp         2D Integer array, containing information on element
!                    groups coupled to element groups.
!                    CONGRP( i,j ) = 1 means that element group i and element
!                    group j are coupled.
!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     ipos           Position in array
!     j              Counting variable
!     ncongrp        Integer array containing information on the number of
!                    element groups coupled to each element group.
!                    NCONGRP( 1 ) = 0
!                    NCONGRP(ielgrp+1) = number of element groups coupled
!                    to element group IELGRP. Note that at output nCONGRP is
!                    an index array:
!                    NCONGRP( 1 ) = 0
!                    NCONGRP(ielgrp+1) - NCONGRP(ielgrp) = number
!                    of groups connected to iELGRP.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCONELGRPS   Determine what element groups are coupled to element groups
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays congrp and ncongrp with element groups that are coupled
!         as well as number of element groups that are coupled (accumulated)

      call sepconelgrps ( congrp, ncongrp )

!     --- Reserve space for an array containing information
!         on element groups coupled to element groups.

      length = ncongrp(nelgrp+1)+nelgrp+1
      if ( associated( km(iseqkm)%kmeshv) ) deallocate( km(iseqkm)%kmeshv )
      allocate(km(iseqkm)%kmeshv(length))
      km(iseqkm)%kmesh(45) = length
      kelmv => km(iseqkm)%kmesh(45)
      kmeshv =>km(iseqkm)%kmeshv

!     --- Fill array kmeshv

      kmeshv(1:nelgrp+1) = ncongrp(1:nelgrp+1)
      ipos = nelgrp+1
      do ielgrp = 1, nelgrp
         do j = 1, nelgrp
            if ( congrp(ielgrp,j)==1 ) then
               ipos = ipos + 1
               kmeshv(ipos) = j
            end if
         end do
      end do

      kmshv1 => kmeshv(1:nelgrp+1)
      kmshv2 => kmeshv(nelgrp+2:length)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshv, nelgrp+1+ncongrp(nelgrp+1), 'kmeshv' )
         write(irefwr,*) 'End sepfillkmeshv'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshv' )

      end subroutine sepfillkmeshv
