      subroutine sepfillmatsimp ( S )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill pointers in sepmodulematr for subroutine build
!     Create relation between arrays and buffer
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulematr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
     type (globmat) :: S

!     S              pointer to realmt(iseqmt)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, istartsav, lencols
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     istart         First position used at a part
!     istartsav      Help variable to save the value of istart
!     lencols        length of the part columns of the matrix structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillmatsimp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillmatsimp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Get parameters and put into sepmodulematstr

      nrusolvel   = IM%nu
      nrusolpress = IM%np
      lencols = size(IM%colsQ)

      S_Q => S%S_Q

!     --- Next D and G-matrix

      S_DIV => S%S_DIV

      if ( IM%Gsymmetric ) then
         S_G => S_DIV
      else
         S_G => S%S_G
      end if  ! ( IM%Gsymmetric )

!     --- C-matrix

      S_C => S%S_C

!     --- Scaled pressure mass matrix

      S_PM => S%S_PM

!     --- Diagonal pressure mass matrix

      S_PMD => S%S_PMD

!     --- Diagonal velocity mass matrix

      S_UD => S%S_UD

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillmatsimp'

      end if  ! ( debug )
      call erclos ( 'sepfillmatsimp' )

      end subroutine sepfillmatsimp
