      subroutine sepfilltiminp ( inpsep, isubr, ivector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill module sepmoduletime with information stored in inpsep/rinsep
!     and ibuffr
!     Old subroutine prtimefr01
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     copy
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleinput
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*), isubr, ivector

!     inpsep         i    The integer input read is stored in array INPSEP
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values:
!                         1:  timprb
!                         2:  prtimd
!                         3:  timefrbf
!     ivector        i    Sequence number of first vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos, iseqnr, maxcal

!     ipos           Position of input arrays in sepmoduleinput
!     iseqnr         Sequence number with respect to the storage in
!                    sepmoduleinput
!     maxcal         Maximum number of coefficient inputs read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
!     PRININ         print 1d integer array
!     SEPCOPTIMINP   Copy input for time solution from arrays iintim and rintim into
!                    module sepmoduletime
!     SEPFILLTIMDEF  Find input parameters for time integration in structure
!                    block and extend with default values
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilltiminp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilltiminp'
         write(irefwr,1) 'ivector', ivector
         call prinin ( inpsep, 4, 'inpsep' )
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpsep(1)==-10 ) then

!     --- Input directly from the structure block as parameters from the
!         call to solve_time_dependent_problem

          print *,'inpsep(1)==-10, not yet implemented'
          call instop

      else

!     --- Input not from the structure block

         if ( inpsep(1)==-1 ) then

!        --- No sequence number given

            call sepgetinpfinf ( 15, 1, ipos, maxcal )
            if ( maxcal>0 ) then

!           --- There is an input block time_integration

               iseqnr = 1

            else

               iseqnr = 0
               print *,'inpsep(1)==-1, not yet implemented'
               call instop

            end if

         else

!        --- inpsep(1) > 0, sequence number found

            iseqnr = inpsep(1)

         end if  ! ( inpsep(1)==-1 )

         if ( debug ) write(irefwr,1) 'iseqnr', iseqnr

         if ( iseqnr>0 ) then

!        --- Copy input from inpsep and rinsep into module sepmoduletime

            call sepgetinpfinf ( 15, iseqnr, ipos, maxcal )

            if ( debug ) write(irefwr,*) 'in(ipos)%used ', in(ipos)%used

            if ( in(ipos)%used ) then

!           --- in(ipos)%used true, hence input block found

               call sepcoptiminp ( in(ipos)%iinput, re(ipos)%rinput, &
                                   ndim, ivector )
            else

!           --- in(ipos)%used false, no input block found
!               Find parameters from input and extend with defaults

               call sepfilltimdef ( ndim, ivector )

            end if  ! ( in(ipos)%used )

         end if  ! ( iseqnr>0 )

      end if  ! ( inpsep(1)==-10 )

      if ( isubr==3 ) icall_outtim = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'tinit, tstep, tend', t0, tstep, tend
         write(irefwr,*) 'End sepfilltiminp'

      end if  ! ( debug )
      call erclos ( 'sepfilltiminp' )

      end subroutine sepfilltiminp
