      subroutine seppjspar1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates projection vectors for the deflated iccg algorithm. The
!     space spanned by the projection vectors approximates the space
!     spanned by the small eigenvectors.
!     Old subroutine prpjspar1
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenprojv, lenintprj

!                    they are not
!     lenintprj      Length of array intprj
!     lenprojv       Total length of the temporary array RTSPROJV.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPPJDSET      Combines element groups into sets
!     SEPPJDSTS      Copy the non-sparse array CONSET into the sparse array
!                    SPCONSET
!     SEPPJNODE      Determines which node belongs to which set
!     SEPPJNODE3     Determines which node belongs to which set
!     SEPPJSHAP      Determines the shape of the projection vectors
!     SEPPJSHAP3     Determines the shape of the projection vectors
!     SEPPJSPAR      Makes the projection vectors sparse
!     SEPPJSUBP      Solves subsystems for each projection vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1664   No integer properties are available for the mesh
!    1665   No projection vectors could be constructed
!    1666   Too few sets could be constructed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjspar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjspar1'
         write(irefwr,1) 'nrusol, proj_type', nrusol, proj_type
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

!     --- ASSUMED IS THAT INTEGER PROPERTIES HAVE BEEN GIVEN FOR EACH SET

      if ( kelmw==0 ) call errsub ( 1664, 0, 0, 0 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'print_lv, proj_type', print_lv, proj_type

      if ( print_lv>=1 ) write(irefwr,120)
120   format ( '  [2] grouping element groups into sets' )

      allocate ( groupset(nelgrp), sandset(nelgrp), inclusn(nelgrp), &
                 conset(nelgrp,nelgrp), nconset(nelgrp+1), &
                 nsetnode(nelgrp+1), projsand(nelgrp), nsol(nelgrp), &
                 solshale(nelgrp), sandproj(nelgrp), dirichlt(nelgrp), &
                 nodeset(npoint), setnode(nrusol), stat = error )

!     --- Combine element groups into sets

      call seppjdset

!     --- IF THE NUMBER OF SETS IS ONE THEN DEFLATION HAS NO POINT

      if ( nset<2 ) then
         call errsub ( 1666, 0, 0, 0 )
         go to 1000
      end if

!     --- activate arrays in IBUFFR, and compute starting adresses

      allocate ( spconsetsh(nset+1), spconsetsa(nconset(nset+1)), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nset+1+nconset(nset+1), 'spconset' )
      if ( ierror/=0 ) go to 1000

      call seppjdsts

      if ( print_lv>=1 ) write(irefwr,130)
130   format ( '  [3] collecting all nodes for each set' )

!     --- Collect all nodes for each set

      if ( proj_type==2 ) then

!     --- proj_type = 2

         call seppjnode3

      else

!     --- proj_type = 1

         call seppjnode

      end if

!     --- At this point the number of projection vectors is known

      if ( print_lv>=1 ) write(irefwr,140) nproj
140   format ( '      - ', I4, ' projection vectors have been found')

      if ( print_lv>=1 ) write(irefwr,150)
150   format ( '  [4] determining the shape of the', &
                 ' projection vectors' )

!     --- IF THE NUMBER OF PROJECTION VECTORS IS ZERO THEN CONTINUING
!         HAS NO USE.

      if ( nproj==0 ) then
         call errwar ( 1665, 0, 0, 0 )
         go to 1000
      end if

      allocate ( itsprojv(nproj,nset), stat = error )
      if ( error/=0 ) call eralloc ( error, nproj*nset, 'itsprojv' )
      if ( ierror/=0 ) go to 1000

!     --- Determine the shape of the projection vectors

      if ( proj_type==2 ) then

         call seppjshap3 ( lenprojv )

      else

         call seppjshap ( lenprojv )

      end if

      if ( debug ) &
         write(irefwr,1) 'nproj, nnproj, lenprojv', nproj, nnproj, lenprojv

      if ( print_lv>=1 ) write(irefwr,160)
160   format ( '  [5] solve subsystems for each projection vector' )

!     --- Reserve space for array RTSPROJV with the subsolutions

      lenprojv = max(1,lenprojv)
      allocate( rtsprojv(lenprojv), stat = error )
      if ( error/=0 ) call eralloc ( error, lenprojv, 'rtsprojv' )
      if ( ierror/=0 ) go to 1000

!        --- Solve subsystems for each projection vector

      if ( proj_type==1 ) call seppjsubp

      if ( print_lv>=1 ) write(irefwr,170)
170   format ( '  [6] making the projection vectors sparse' )

      if ( debug ) &
         write(irefwr,1) 'nproj, nnproj, lenprojv', nproj, nnproj, lenprojv

      allocate ( projrows(nproj+1), projcolstmp(nnproj), projUtmp(nnproj), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, nproj+nnproj, 'projrows' )
      if ( ierror/=0 ) go to 1000

!     --- Make the projection vectors sparse.

      call seppjspar

      if ( debug ) &
         write(irefwr,1) 'nproj, nnproj, lenprojv', nproj, nnproj, lenprojv

!     --- Make arrays permanent

      allocate ( projcols(nnproj), projU(nnproj), stat = error )
      if ( error/=0 ) call eralloc ( error, nproj+nnproj, 'projrows' )
      if ( ierror/=0 ) go to 1000

      projcols(1:nnproj) = projcolstmp(1:nnproj)
      projU(1:nnproj) = projUtmp(1:nnproj)

!     --- Clear arrays that are no longer needed

      deallocate ( groupset, sandset, inclusn, conset, &
                   nconset, projsand, nsetnode, nsol, solshale, &
                   sandproj, dirichlt, nodeset, projcolstmp, &
                   projUtmp, setnode, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'groupset' )
      if ( associated(spconsetsa) ) deallocate ( spconsetsa, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'spconsetsa' )
      if ( associated(spconsetsh) ) deallocate ( spconsetsh, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'spconsetsh' )
      if ( associated(itsprojv) ) deallocate ( itsprojv, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'itsprojv' )
      if ( associated(rtsprojv) ) deallocate ( rtsprojv, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'rtsprojv' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'nproj, nnproj', nproj, nnproj
         write(irefwr,*) 'End seppjspar1'

      end if  ! ( debug )
      call erclos ( 'seppjspar1' )

      end
