      subroutine seppjsubf ( iproj, ishale, idxprojv )
! ======================================================================
!
!        programmer     Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine copies the solution of the subsystem in arrays
!     ITSPROJV and RTSPROJV.
!     Old subroutine prpjsubf
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      use sepmodulekprob
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer iproj, ishale, idxprojv

!     idxprojv   i/o    Index into array RTSPROJV.
!     iproj       i     Number of the current projection vector.
!     ishale      i     Dummy variable, set number of a shale set.
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, first, last, inodp, idegfd, ilocdgfd

!     first       Start of loop.
!     i           Loop variable.
!     idegfd      Dummy variable, degree of freedom.
!     ilocdgfd    Dummy variable, local degree of freedom.
!     inodp       Dummy variable, nodal point number.
!     last        End of loop.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Copying of the solution of the sub-system in array
!           RTSPROJV
!
!     Semi PSD: seppjsubf
!
!     Set the index array ITSPROJV for this shale set equal
!     to the next free place in array RTSPROJV:
!     RTSPROJV( iPROJ, iSHALE ) = - ( iDXPROJV+1 ). The
!     minus sign is used to indicate that this number is an
!     index into array RTSPROJV.
!
!     LOOP over all degrees of freedom in the shale set
!        Get the local degree of freedom from DMAP.
!        Increase the pointer IDXPROJV with one.
!        Copy the solution in the local degree of freedom
!        to array RTSPROJV( IDXPROJV ).
!     END LOOP
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      itsprojv(iproj,ishale) = -(idxprojv+1)

!     --- Substitute solution into the projection vectorc

      do i = nsetnode(ishale)+1, nsetnode(ishale+1)

         inodp = setnode(i)
         idegfd = kprobh(inodp)
         ilocdgfd = dmap(idegfd)

!        --- Fill part of the projection vector

         idxprojv = idxprojv+1
         rtsprojv(idxprojv) = locsol(ilocdgfd)

      end do

      end
