      subroutine septimcheck ( first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some checks with respect to time integration input
!     Fill array iinbldtim
!     Old subroutine prtm02
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletime
      use sepmodulecons
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: first

!     first         i/o   If true it concerns the first time step
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: x, y, z

!     i              Counting variable
!     x              x-coordinate of point
!     y              y-coordinate of point
!     z              z-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     FINDXY         Find nodal point closest to a specific point
!     PRFILLIINBLD   Fill arrays iinbld for building of matrix with defaults and
!                    some paramters from the input file
!     PRFILLINPSOL   Fill arrays inpsol and rinsol for linear solver depending
!                    on input in input file or defaults
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     452   The parameter TSTEP has an incorrect value
!     600   theta <= 0 or theta > 1
!     623   tout < t
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimcheck' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimcheck'
         write(irefwr,2) 'toutinit, toutend, toutstep', &
                          toutinit, toutend, toutstep
         write(irefwr,2) 'tinit, tstep, tend', t0, tstep, tend
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( first .or. complrun ) then

!     --- check input

         if ( tout<=t-100d0*epsmac ) then
            call erreal ( t, 1 )
            call erreal ( tout, 2 )
            call errsub ( 623, 0, 2, 0 )
         end if
         if ( ichtim>4 ) then

!        --- step size given by the user

            if ( tstep<100d0*epsmac ) then
               call erreal ( tstep, 1 )
               call erreal ( 100d0*epsmac, 2 )
               call errsub ( 452, 0, 2, 0 )
            end if

         end if  ! ( ichtim>4 )

      end if  ! ( first .or. complrun )

      if ( ierror/=0 .or. nocomputationtim ) go to 1000

      if ( ichtim==9 ) then

!     --- theta given by the user

         if ( theta<=0d0 .or. theta>1d0 ) then
            call erreal ( theta, 1 )
            call errsub ( 600, 0, 1, 0 )
         end if
         if ( debug ) write(irefwr,2) 'theta', theta

      end if

      massrule = Teq(1)%imas  ! Set global imas for integration rule

!     --- Fill array iinbldtim with part of information with respect to prtm00
!         Kan vereenvoudigd worden

      call prfilliinbld ( iinbldtim, Teq(1)%iconsrhs, Teq(1)%imas, &
                          0, numoldtim, 1, Teq(1)%isubdiv )

      if ( debug ) then
         call prinin ( iinbldtim,20, 'iinbldtim' )
         write(irefwr,1) 'ireadsl, leninpsol, lenrinsol', &
                          Teq(1)%ireadsl, ninpsol, nrinsol
      end if  ! ( debug )

!     --- Fill array inpsoltim and rinsoltim if necessary

      call prfillinpsol ( Teq(1)%ireadsl, &
                          inpsoltim, rinsoltim, &
                          ninpsol, nrinsol, epstimsol  )

      if ( debug ) call prinin ( inpsoltim, 10, 'inpsoltim' )

      if ( t<=t0+1d3*epsmac .and. nhist>0 ) then

!     --- Fill array nodhist

         do i = 1, nhist

            x = coorhist(1,i)
            if ( ndim>1 ) y = coorhist(2,i)
            if ( ndim>2 ) z = coorhist(3,i)
            call findxy ( coor, x, y, z, npoint, nodhist(i), ndim, 3 )

         end do  ! i = 1, nhist

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimcheck'

      end if  ! ( debug )
      call erclos ( 'septimcheck' )

      end subroutine septimcheck
