      subroutine septimd ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 13-12-2017 New storage of intmat
!        version  3.2  date 06-02-2012 Allow indexed vector.
!        version  3.1  date 05-06-2011 Save and reset common ctimen
!
!   copyright (c) 1995-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutine TIMPRB for the essential boundary conditions
!    Fill array IINTIM with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         1  ISEQNR     Sequence number corresponding to input
!                                       If iseqnr = -1, the next one will
!                                       be used
!                         2  IPROB      Problem sequence number
!                                       Not used
!                         3  IVECTOR    Vector sequence number (first vector. &
!                                       If ivector = -1, the same value as iprob
!                                       will be used
!     rinsep         i    The real input read is stored in array RINSEP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqintmat, isubr, ivector
      logical :: first, finished, onerun

!     finished       Indicates if the time loop has been finished (true) or not
!                    (false)
!     first          If true it concerns the first time step
!     iseqintmat     Sequence number with respect to array intmat
!     isubr          Indicates the type of calling subroutine
!     ivector        Sequence number of first vector
!     onerun         if true a complete run is carried out
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PRININ         print 1d integer array
!     PRRESETCTIMEN  Reset the parameters in ctimen
!     PRSAVECTIMEN   Save the parameters in ctimen
!     SEPFILLTIMINP  Fill module sepmoduletime with information stored in
!                    inpsep/rinsep and ibuffr
!     SEPTIMPRB      solves time-dependent problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimd' )

      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from septimd'
         call prinin ( inpsep, 3, 'inpsep' )
         call prinin ( isolut2d, 5, 'isolut2d' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Save parameters from ctimen

      onerun = itimloop==0
      if ( .not. onerun ) call prsavectimen

      first = .true.
      finished = .false.
      complrun = .true.
      isubr = 2

      if ( debug ) then
         call prinin ( kprob, 20, 'kprob' )
      end if  ! ( debug )

!     --- Copy input concerning time time_integration into module sepmoduletime

      ivector = max(1,(inivectr(inpsep(3))))
      call sepfilltiminp ( inpsep, isubr, ivector )

      numoldtim = max(1,numvecmn)
      maxoldtim = maxvec

      iprobtim = ks(isolut2d(Teq(1)%iseqfr))%iprob
      iseqim = intmat1d(iprobtim)
      if ( intmt(iseqim)%iprob==iprobtim ) then
         iseqintmat = iprobtim
      else
         iseqintmat = 1
      end if

      length = size(intmat1d)
      intmatar => intmat1d(iseqintmat:length)
      call septimprb ( finished, first, isubr )
      numvecmn = max(numvecmn,Teq(1)%iseqfr+ncopeqtim-1)

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend  deallocate arrays

         if ( associated(ireadcftim) ) then

            deallocate ( ireadcftim, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'ireadcftim' )
            if ( ierror/=0 ) go to 1000

            if ( nhist>0 ) deallocate ( coorhist, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'coorhist' )
            if ( ierror/=0 ) go to 1000

            deallocate ( Teq, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'timeequation' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( .not. associated(ireadcftim) )

      end if  ! ( t>=tend-0.00001*tstep )

!     --- Reset parameters from ctimen

      if ( .not. onerun ) call prresetctimen

      istartmn = istartmn+4  ! raise istartmn

1000  if ( debug ) then

!     --- debug information

         write(irefwr,*) 'End septimd'

      end if
      call erclos ( 'septimd' )

      end
