      subroutine septimenthal1 ( first, finished, iseqenthalpy, &
                                 iinenthalpy, rinenthalpy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     time integration for enthalpy equation using overrelaxation
!     Old subroutine prtimenthal1
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqenthalpy, iinenthalpy(*)
      double precision, intent(in) :: rinenthalpy(*)
      logical, intent(inout) :: first, finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not (false)
!     first         i/o   If true it concerns the first time step
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iseqenthalpy   i    Sequence number of input for block enthalpy
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: joutp, jtime

!     joutp          Help variable to store the value of ioutp
!     jtime          Help variable to store the value of itime
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPTIMCHECK    Perform some checks with respect to time integration input
!                    Fill array iinbldtim
!     SEPTIMENTH1EU  time integration for enthalpy equation using overrelaxation
!                    a theta method is used
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimenthal1' )

!     --- set ioutp and itime temporarily

      joutp = ioutp
      ioutp = min(0,ioutp)
      jtime = itime
      itime = 0

!     --- Define parameters and fill arrays

      call septimcheck ( first )
      if ( keeptim>1 ) t = t - tstep

!     --- Implicit theta method (Euler, Crank Nicolson, fractional step)
!         Stepsize is given by user.
!         This is the only option where enthalpy is available

      call septimenth1eu ( finished, iseqenthalpy, iinenthalpy, rinenthalpy )

!     --- reset ioutp and itime

      first = .false.
      ioutp = joutp
      itime = jtime

      call erclos ( 'septimenthal1' )
      end
