      subroutine septimreset ( irhsd, isol, isolprec, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     reset all arrays with respect to time integration that are no longer
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduletime
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irhsd(5,*), isol(5,*), isolprec(5,*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
!     irhsd         i/o   Standard SEPRAN array, containing information of the
!                         large vector. &
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     isolprec      i/o   Solution at preceding level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, jmat

!     j              Counting variable
!     jmat           j+icalltim-1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRDELMATS      Delete matrices and right-hand-side vector for time
!                    dependent problem
!     PRININ1        print 2d integer array
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimreset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimreset'
         write(irefwr,1) 'ncopeqtim', ncopeqtim
  1      format ( a, 1x, (10i6) )
         call prinin1 ( mattot, ncopeqtim, 5, 'mattot/begin' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      do j = 1, ncopeqtim

         jmat = j
         call prdelmats ( massmt(jmat), stiffm(jmat), &
                          mattot(jmat), irhsd(1,jmat) )
         call seprmsolinfo ( isol(1,j) )
         if ( isolprec(1,j)>0 ) call seprmsolinfo ( isolprec(1,j) )

      end do
      finished = .true.
      if ( complrun ) itimloop = 0
      massrule = 0    ! reset massrule

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( mattot, ncopeqtim, 5, 'mattot/end' )
         write(irefwr,*) 'End septimreset'

      end if
      call erclos ( 'septimreset' )

      end subroutine septimreset
