      subroutine septimstart ( first, eps, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start part of subroutine septimprb
!     Fill input arrays and constants and write initial condition
!     Old subroutine prtimprb01
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr
      double precision, intent(out) :: eps
      logical, intent(in) :: first

!     eps            o    Local accuracy
!     first          i    Indicates if the loop has been finished (true) or not
!                         (false)
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values:
!                         1:  timprb
!                         2:  septimd
!                         3:  timefrbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nch, i

!     i              Loop variable
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROUTP         Performs the actual write actions for OUTPUT
!     SEPTIMEHIS2    Create real arrays for time history and fill partly
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimstart'
         write(irefwr,1) 'itimloop', itimloop
         write(irefwr,*) 'complrun, first ', complrun, first
         write(irefwr,1) 'keeptim, icall_outtim', keeptim, icall_outtim
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      eps = 1d3*epsmac
      itimdep = 1
      if ( complrun ) itimloop = 1
      if ( isubr==3 .and. icalltim>1 ) keeptim = 2

      if ( toutstep<0d0 ) toutstep = arrtimestep(1)
      if ( toutend<t0 ) toutend = arrendtime(1)

      if ( isubr==2 .and. ioutp>=0 ) then
         do i = 1, ncopeqtim
            nch = index(names(Teq(i)%iseqfr),' ')-1
            if ( nch<=0 ) nch = len(names(Teq(i)%iseqfr))
            write ( irefwr, 10 ) names(Teq(i)%iseqfr)(1:nch)
10          format ( ' Compute vector ', a, &
                     ' by solving a system of time-dependent', &
                     ' equations' )
         end do
      end if

      if ( keeptim<=1 .and. first ) then

!     --- Only for the first time step

         if ( .not. oldmethod ) t = t0
         itiminterval = 1

         call septimehis2
         toutnext = toutinit

      end if  ! ( keeptim<=1 .and. first )

      if ( debug ) then
          write(irefwr,2) 'toutinit, t0, eps', toutinit, t0, eps
          write(irefwr,2) 'toutend, toutstep', toutend, toutstep
      end if  ! ( debug )

!     --- Write initial condition depending on toutinit

      if ( icall_outtim==1 .and. abs(toutinit-t0)<eps .and. &
           toutend>=toutinit .and. first .and. complrun ) then

         write(irefwr,50) t
 50      format( ' time = ',f12.5 )
         call proutp ( 3, t, 1, iseqouttim, 1, .false., &
                       Teq(1)%iseqfr )
100      toutnext = toutnext+toutstep
         if ( toutstep>0d0 .and. toutnext<t+2d0*eps ) go to 100
         if ( toutnext>toutend+0.5d0*toutstep ) toutnext = tend + 100d0
         tendnext = arrendtime(1)

      else if ( icall_outtim==0 ) then

         toutnext = tend
         tendnext = tend

      end if  ! ( icall_outtim==1 .and. abs(toutinit-t0)<eps .and. ... )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'toutinit, toutend, toutstep', &
                          toutinit, toutend, toutstep
         write(irefwr,2) 'toutnext, tendnext', toutnext, tendnext
         write(irefwr,*) 'End septimstart'

      end if  ! ( debug )
      call erclos ( 'septimstart' )

      end subroutine septimstart
