      subroutine toassemblecmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 24-07-2015 Use sepmodulebuild
!        version  3.0  date 21-04-2015 New parameter list
!        version  2.0  date 27-10-2010 remove iwork
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble one element matrix in the large matrix
!     Complex matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idown, ihigh, icnt
      logical :: shiftedlapl
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     icnt           Number of non-prescribed unknowns in element
!     idown          Lowest position number in USOL with respect to "right"
!                    periodical boundary conditions
!                    idown = nusol-nbound-npboun+1
!     ihigh          Highest position number in USOL with respect to
!                    "right" periodical boundary conditions
!                    ihigh = nusol-nbound
!     shiftedlapl    If true a shifted laplace preconditioning is applied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TO0213         add element matrix to large matrix part 2, 3, 4
!     TOADDCROWCOMP  Assemble one element matrix in the large matrix
!                    complex matrix with compact row storage
!     TOADDMATSHIFTC Add diagonal element matrix to extra diagonal of
!                    complex matrix
!     TOST02         Add contributions of periodical boundary conditions to
!                    second part of matrix in the case of complex arithmetic
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3064   Shifted Laplace preconditioning not available for type number $
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toassemblecmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toassemblecmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      shiftedlapl = IM%lapshift

      if ( matrix ) then

!     --- jmethod = 8  complex row-compact matrix

         Smat => realmt(stiffmat(1))
         call toaddcrowcomp ( elemmt, icnt )
         if ( idefcr==1 ) then
            Smat => realmt(stiffmat(2))
            call toaddcrowcomp ( elemmt2, icnt )
            Smat => realmt(stiffmat(1))
         end if  ! ( idefcr==1 )

         if ( shiftedlapl .and. ielgrp<=nelgrp ) then

!        --- shiftedlapl = true, store extra diagonal matrix

            if ( itype>99 .and. itype/=150 ) then

!           --- Type number not available

               call errint ( itype, 1 )
               call errsub ( 3064, 1, 0, 0 )
               go to 1000

            end if  ! ( itype/=800 )

            call toaddmatshiftc ( elemmt2 )

         end if  ! ( shiftedlapl )

!        --- add element matrix to second part of matrix

         if ( icnt>0 .and. icnt<icount ) then

            nrusol = nusol-nbound
            call to0213 ( S1_ip, S1_pi, S1_pp, elemmt1 )
            if ( idefcr==1 )  call to0213 ( S2_ip, S2_pi, S2_pp, elemmt2 )
            nrusol = IM%nrusol

         end if  ! ( icnt>0 .and. icnt<icount )

         if ( npboun>0 ) then

!        --- npboun>0, fill part of second part of large matrix
!            corresponding to periodical boundary conditions
!            First divide the columns of the element matrix by the
!            multiplication factor

            idown = nusol-nbound-npboun+1
            ihigh = nusol-nbound

            call tost02 ( idown, ihigh, S1_ip, elemmt )
            if ( idefcr==1 ) &
               call tost02 ( idown, ihigh, S2_iib, elemmt2d(1,1,2) )

         end if  ! ( npboun>0 )

      end if  ! ( matrix .and. icnt>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toassemblecmat'

      end if  ! ( debug )
      call erclos ( 'toassemblecmat' )

      end
