      function zcjdot ( n, x, incrx, y, incry )
! ======================================================================
!
!        programmer     Zdenek Vlasek
!        version 1.1    date   23-03-1996 Layout
!        version 1.0    date   17-10-1990
!
!   copyright (c) 1990-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                         DESCRIPTION
!
!        algorithm:  complex summ x(ix)*y(iy), &
!                    where ix=1, ... step incrx
!                          iy=1, ... step incry
!
!        This "wrong" complex dot product is used in some complex
!        solvers, instead of ZDOT product  summ conjg(x(ix)) * y(iy)
! **********************************************************************
!
!                      KEYWORDS
!
! **********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer n, incrx, incry
      complex*16 zcjdot, x(n), y(n)

!     incrx      i    Increment for x vector
!     incry      i    Increment for y vector
!     n          i    number of entries of vectors
!     x          i    x vector
!     y          i    y vector
!     zcjdot     o    Result of dot product
! **********************************************************************
!
!                      COMMON BLOCKS
!
! **********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i
      complex*16 sum

!     i          Counting variable
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!             SUBROUTINES CALLED
!
! **********************************************************************
!
!                    I/O
!
! **********************************************************************
!
!                    ERROR MESSAGES
!
! **********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
!
      sum = (0d0,0d0)

      if ( incrx.eq.1 .and. incry.eq.1 ) then

        do 10 i=1,n
          sum = sum + x(i) * y(i)
   10   continue

      else

        do 20 i=1, n
          sum = sum + x (1+(i-1)*incrx) * y (1+(i-1)*incry)
   20   continue

      end if

      zcjdot = sum

      end
