      subroutine chtmc4 ( ksetnp, mset, ifirst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 28-05-2003 Lay-out
!        version  3.0  date 22-11-1996 Extra parameter ifirst
!        version  2.0  date 22-12-1994 New parameter list
!
!   copyright (c) 1983-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   The rest of the levels is computed according to the Cuthill-Mckee algorithm
!   They are partly sorted
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ksetnp(*), mset, ifirst

!     ifirst  i    The nodes from ifirst must be renumbered
!     ksetnp  o    array containing the set of nodal points
!     mset    o    number of nodal points in ineigh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jcount, ithres, nset
      integer, target :: itemp(npoint), istore1(npoint)

!     i            Counting variable
!     istore1      Contains the last level set
!     itemp        integer work array
!     ithres       Integer threshold value
!     jcount       Counter
!     nset         Number of elements in a set of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHTMC3         The starting level set is sorted using
!                    the Cuthill-Mckee algorithm
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Free array marknodes except for already numbered nodes
!    Store starting set in array istore1
!    While ( mset > 0 ) do
!       find neigbors of last set
!       Sort last set into array ksetnp
!       Store last set into istore1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmc4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmc4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- reset marknodes

      jcount = mset
      ithres = -3
      do i = 1, npoint
         if ( marknodes(i)>-10 ) marknodes(i) = -2
      end do
      do i = 1, jcount
         marknodes(ksetnp(i)) = -10
      end do

!     --- find neigbors of level sets

      do i = 1, jcount
         istore1(i) = ksetnp(i)
      end do
      nset = jcount

200   if ( mset>0 ) then

!     --- Find neighbours of set in istore1, put in itemp

         lastlevel => istore1(1:nset)
         nextlevel => itemp(1:npoint)
         call sepnextlevel ( ithres, mset )
         if ( mset>0 ) then

!        --- mset > 0, sort last set and put into ksetnp

            call chtmc3 ( ksetnp(jcount+1), itemp, mset, npoint, ifirst )
            do i = 1, mset
               marknodes(ksetnp(jcount+i)) = -10
            end do
            do i = 1, mset
               istore1(i) = ksetnp(jcount+i)
            end do
            jcount = jcount+mset
            nset = mset
         end if
         go to 200
      end if
      mset = jcount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End chtmc4'

      end if  ! ( debug )
      call erclos ( 'chtmc4' )

      end subroutine chtmc4
