      subroutine chtmck ( istart, lenstr, meshsplt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 28-05-2003 Allow disjoint mesh parts
!        version  2.4  date 22-11-1996 New call to chtmc4
!        version  2.3  date 22-12-1994 New call to chtmc2
!
!   copyright (c) 1982-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute renumbering according to the Cuthill-Mckee algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istart(*), lenstr, meshsplt(*)

!     istart         i    Contains the starting set
!     lenstr         i    Length of istart
!     meshsplt      i/o   In this array the largest node number of the various
!                         disconnected parts of the mesh are stored
!                         Length: number of parts (<npoint)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, mset, ifirst, isplit
      integer :: ksetnp(npoint)

!     i              Counting variable
!     ifirst         First node for renumbering
!     isplit         Sequence number of discontinuous part
!     ksetnp         array containing the set of nodal points
!     mset           number of nodal points in last set
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer sepstartnode

!     CHTMC4         The rest of the levels is computed according to the
!                    Cuthill-Mckee algorithm
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPSTARTNODE   Find start node in a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     552   Mesh consists of unconnected parts
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy starting set
!     Compute rest of levels
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmck'
         call prinin ( istart, lenstr, 'start set' )

      end if

!     --- Copy starting set into ksetnp

      mset = lenstr
      ksetnp(1:lenstr) = istart(1:lenstr)

!     --- Compute rest of levels

      call chtmc4 ( ksetnp, mset, 1 )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'mset, npoint ', mset, npoint
         call prinin ( ksetnp, mset, 'ksetnp' )
         call prinin ( marknodes, npoint, 'marknodes' )
      end if
      kmeshj(1:mset) = ksetnp(1:mset)
      isplit = 1

      do while ( mset<npoint )

!     --- Not all nodes have been renumbered, which means that the
!         region consists of disjunct parts
!         The next part is renumbered

         isplit = isplit+1
         ifirst = mset+1
         do i = ifirst, meshsplt(isplit)
             kmeshj(i) = i
         end do  ! i = ifirst, ilast
         set => kmeshj(ifirst:meshsplt(isplit))
         kmeshj(ifirst) = sepstartnode()
         mset = 1

!        --- Compute rest of levels

         call chtmc4 ( ksetnp, mset, ifirst )
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'mset, npoint, ifirst ', &
                             mset, npoint, ifirst
            call prinin ( ksetnp, mset, 'ksetnp' )
            call prinin ( marknodes, npoint, 'marknodes' )
         end if
         do i = 1, mset
            kmeshj(ifirst+i-1) = ksetnp(i)
         end do
         mset = ifirst+mset

      end do  ! while ( mset<npoint )

      if ( mset<npoint ) then
         call errint ( mset, 1 )
         call errint ( npoint, 2 )
         call errsub ( 552, 2, 0, 0 )
      end if

      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshj, npoint, 'kmeshj' )
         write(irefwr,*) 'End chtmck'
      end if
      call erclos ( 'chtmck' )
      end
