      subroutine incm011loop ( kmeshc2d, iwork, ihelp, maxchn, jnelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 28-06-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark positions in array iwork corresponding to periodical boundary
!     conditions and creat chain of nodes
!     Sub case for one element group
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer:: jnelem, kmeshc2d(2,jnelem), iwork(3,*), ihelp(*), maxchn

!     ihelp         i/o   work array to compute the chain
!                         Futhermore it is used to indicate if a point belongs
!                         to the set of periodical points corresponding to a
!                         specific periodic point in the following way:
!                         ihelp(i) = 1, point i corresponds to the periodic
!                         point
!                         ihelp(i) = 0, it does not
!     iwork         i/o   work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incm011 for a description
!     jnelem         i    Number of elements in a specific element group
!     kmeshc2d       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     maxchn        i/o   maximum length of chain
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iielem, nodp1, nodp2, ichain, nodalp, i

!     i              Counting variable
!     ichain         length of chain
!     iielem         Relative element number in element group
!     nodalp         Nodal point sequence number
!     nodp1          Node sequence number (first in element)
!     nodp2          Node sequence number (second in element)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm011loop' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm011loop'
         write(irefwr,1) 'jnelem', jnelem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do iielem = 1, jnelem

         nodp1 = kmeshc2d(1,iielem)
         nodp2 = kmeshc2d(2,iielem)

         if ( debug ) write(irefwr,1) 'nodp1, nodp2', nodp1, nodp2

         if ( iwork(1,nodp2)==0 ) then

!        --- nodp2 has not been connected to another node yet

            iwork(1,nodp2) = nodp1

         else if ( iwork(1,nodp2)==nodp1 .or. &
                   iwork(1,nodp1)==nodp2 ) then

!        --- Connection has already been found: skip

         else if ( iwork(1,nodp2)==0 ) then

!        --- nodp1 has not been connected to another node yet

            iwork(1,nodp1) = nodp2

         else

!        --- both nodp1 and nodp2 have been connected to another node
!            Compute the chain from nodp2

            ihelp(1) = nodp2
            ichain = 1
            nodalp = iwork(1,nodp2)

!           --- while loop

150         if ( nodalp/=0 ) then

!           --- still a node in the chain

               ichain = ichain + 1
               ihelp(ichain) = nodalp
               nodalp = iwork(1,nodalp)
               go to 150

            end if

!           --- update iwork

            do i = ichain, 2, -1
               iwork(1,ihelp(i)) = ihelp(i-1)
            end do

            iwork(1,ihelp(1)) = nodp1
            maxchn = max ( maxchn, ichain )

         end if  ! ( iwork(1,nodp2)==0 )

      end do  ! iielem = 1, jnelem

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iwork, 10, 'iwork' )
         write(irefwr,*) 'End incm011loop'

      end if  ! ( debug )
      call erclos ( 'incm011loop' )

      end
