      subroutine incommatdiag
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store position of diagonal elements of array icolsii into intdia
!     in case of row-compact storage
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, jstart, jend, istart, ncols
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     istart         Last position used in icolsii
!     j              Counting variable
!     jend           end position of row in icolsii
!     jstart         start position of row in icolsii
!     ncols          number of non-zero columns in row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatdiag' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatdiag'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( associated(IM%diag) ) deallocate(IM%diag)
      allocate ( IM%diag(nrusol) )
      intdia => IM%diag
      istart = 0
      do i = 1, nrusol

         ncols = irowsii(i+1)-istart
         jstart = istart+1
         jend = istart+ncols
         if ( debug ) then
            write(irefwr,1) 'i, ncols, jstart, jend', i, ncols, jstart, jend
            call prinin ( icolsii(jstart), ncols, 'icolsii' )
         end if  ! ( debug )
         do j = jstart, jend
            if ( icolsii(j)==i ) then
               intdia(i) = j
               exit
            end if  ! ( icolsii(j)==i )
         end do  ! j = jstart, jend

         istart = istart+ncols

      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         call prinin ( intdia, nrusol, 'intdia' )
         write(irefwr,*) 'End incommatdiag'

      end if  ! ( debug )
      call erclos ( 'incommatdiag' )

      end subroutine incommatdiag
