      subroutine incommatpar5
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 13-05-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill row information for the part of the rows in a block
!     that refers to unknowns outside the block
!     Hence the global matrix Sif (Siout)
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ncols, k, istart2, icol, &
                 istart, ninterf, irow, j, istart1, ilow, ihigh

!     i              Counting variable
!     icol           column number for the global system
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     irow           Row number for the global system
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     k              Counting variable
!     ncols          number of non-zero columns in row
!     ninterf        Number of unknowns in interf (interface)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar5', inodenr
         call prinin ( nrusollocs, iacnodes, 'nrusollocs' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ilow = 0
      if ( inodenr>1 ) ilow = nrusollocs(inodenr-1)
      ihigh = nrusollocs(inodenr)

      if ( debug ) &
         write(irefwr,1) 'nrusol, ninterface, ilow, ihigh', &
                          nrusol, ninterface, ilow, ihigh

!     --- First store information from local matrix Sif

      if ( irowsif(nrusol)>0 ) then

!     --- Matrix if contains unknowns
!         Copy irowsif into irowsout

         do i = 1, nrusol
            irowsout(i) = irowsif(i)-irowsif(i-1)
         end do  ! i = 1, nrusol
         irowsout(0) = 0

      else

!     --- Initialize irowsout

         irowsout(0:nrusol) = 0

      end if  ! ( irowsif(nrusol)>0 )

      if ( debug ) call prinin ( irowsout(1), nrusol, 'irowsout' )

!     --- Next store information from matrices Sif and sff of other blocks
!         as stored in storeinfo
!         In fact we create an overestimation of the size

      do i = 1, iacnodes

         istart = storeinfo(i)  ! start address of neighbor block information

         if ( istart>0 ) then

!        --- istart>0, information available

            ninterf = storeinfo(istart)  ! number of interface unknowns
            istart1 = istart+ninterf+1

            do j = 1, ninterf

               irow = kprobhglobinv(storeinfo(istart+j))
               ncols = storeinfo(istart1+j)-storeinfo(istart1+j-1)
               istart2 = istart1+storeinfo(istart1+j-1)+ninterf

               do k = 1, ncols

                  icol = storeinfo(istart2+k)

                  if ( icol<=ilow .or. icol>ihigh ) then

!                 --- Column number outside present block

                    irowsout(irow) = irowsout(irow)+1

                  end if  ! ( icol<=ilow .or. icol>ihigh )

               end do  ! k = 1, ncols

            end do  ! j = 1, ninterf

         end if  ! ( istart>0 )

      end do  ! i = 1, numupp

      if ( debug ) call prinin ( irowsout(1), nrusol, 'irowsout' )

!     --- accumulate irowsout

      do i = 1, nrusol
         irowsout(i) = irowsout(i)+irowsout(i-1)
      end do  ! i = 1, nrusol

      length = irowsout(nrusol)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'length', length
         call prinin ( irowsout, nrusol+1, 'irowsout' )
         write(irefwr,*) 'End incommatpar5', inodenr

      end if  ! ( debug )
      call erclos ( 'incommatpar5' )

      end subroutine incommatpar5
