      subroutine incommatpar6
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill column information for the part of the rows in a block
!     that refers to unknowns outside the block
!     Hence the global matrix Sif (Siout)
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ncols, istart, ninterf, irow, &
                 j, istart1, istart2, k, icol, icount, &
                 ilow, ihigh, icountsav

!     i              Counting variable
!     icol           column number for the global system
!     icount         Counter
!     icountsav      Help variable to store the previous value of icount
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     irow           Row number for the global system
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     k              Counting variable
!     ncols          number of non-zero columns in row
!     ninterf        Number of unknowns in interf (interface)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0005         Print INTMAT corresponding to boundary conditions
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar6'
         write(irefwr,1) 'length icolsout', 2*irowsout(nrusol)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ilow = 0
      if ( inodenr>1 ) ilow = nrusollocs(inodenr-1)
      ihigh = nrusollocs(inodenr)

      if ( debug ) &
         write(irefwr,1) 'nrusol, ninterface, ilow, ihigh', &
                          nrusol, ninterface, ilow, ihigh

!     --- Reset irowsout

      do i = nrusol, 1, -1
         irowsout(i) = irowsout(i-1)
      end do  ! i = 1, nrusol

      if ( debug ) call prinin ( irowsout(1), nrusol, 'irowsout' )

!     --- First store information from local matrix Sif
!         This requires the structure of Sfi

      if ( irowsif(nrusol)>0 ) then

!     --- Matrix if contains unknowns
!         Copy icolsif into icolsout and translate in global numbering

         istart = 0
         do i = 1, nrusol

            ncols = irowsif(i)-istart

            istart1 = irowsout(i)

            if ( debug ) &
               write(irefwr,1) 'i, ncols, istart1, istart', &
                                i, ncols, istart1, istart

            do j = 1, ncols
               icol = icolsif(istart+j)
               icolsout(istart1+j) = kprobhglob(icol)
            end do  ! j = 1, ncols

            irowsout(i) = irowsout(i)+ncols
            istart = istart+ncols

         end do  ! i = 1, nrusol

      end if  ! ( irowsif(nrusol)>0 )

      if ( debug ) call prinin ( irowsout, nrusol+1, 'irowsout' )

!     --- Next store information from matrices Sif of other blocks
!         as stored in storeinfo
!         In fact we create an overestimation of the size

      do i = 1, iacnodes

         istart = storeinfo(i)  ! start address of neighbor block information

         if ( debug ) write(irefwr,1) 'i, istart', i, istart

         if ( istart>0 ) then

!        --- istart>0, information available

            ninterf = storeinfo(istart)  ! number of interface unknowns
            istart1 = istart+ninterf+1

            if ( debug ) &
               write(irefwr,1) 'ninterf, istart1', ninterf, istart1

            do j = 1, ninterf

               irow = kprobhglobinv(storeinfo(istart+j))
               istart2 = istart1+storeinfo(istart1+j-1)+ninterf
               ncols = storeinfo(istart1+j)-storeinfo(istart1+j-1)

               if ( debug ) then
                  write(irefwr,1) 'j, irow, istart2, ncols', &
                                   j, irow, istart2, ncols
                  call prinin ( storeinfo(istart2+1), ncols, 'storeinfo' )
               end if  ! ( debug )

               do k = 1, ncols
                  icol = storeinfo(istart2+k)

                  if ( icol<=ilow .or. icol>ihigh ) then

!                 --- Column number outside present block

                     irowsout(irow) = irowsout(irow)+1
                     icolsout(irowsout(irow)) = icol

                  end if  ! ( icol<=ilow .or. icol>ihigh )

               end do  ! k = 1, ncols

            end do  ! j = 1, ninterf

         end if  ! ( istart>0 )

      end do  ! i = 1, iacnodes

      if ( debug ) then
         call prinin ( irowsout, nrusol+1, 'irowsout' )
         call prinin ( icolsout, irowsout(nrusol), 'icolsout' )
      end if  ! ( debug )

!     --- Remove double entries from icolsout and adapt irowsout

      iwork = 0

      istart = 0
      icount = 0
      do i = 1, nrusol

         ncols = irowsout(i)-istart


         if ( ncols>0 ) then

!        --- There are columns in row i

            icountsav = icount
            do k = 1, ncols

               icol = icolsout(istart+k)

               if ( iwork(icol)==0 ) then

!              --- column number not yet in use

                  icount = icount+1
                  iwork(icol) = 1
                  icolsout(icount) = icol

               end if  ! ( iwork(icol)==0 )

            end do  ! k = 1, ncols

            iwork(icolsout(icountsav+1:icount)) = 0  ! reset iwork
            istart = istart+ncols

         end if  ! ( ncols>0 )
         irowsout(i) = icount

      end do  ! i = 1, nrusol

      length = irowsout(nrusol)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'length', irowsout(nrusol)
         write(irefwr,*) 'structure Sif_complete'
         call pr0005 ( nrusol, irowsout, icolsout, 0 )
         write(irefwr,*) 'End incommatpar6'

      end if  ! ( debug )
      call erclos ( 'incommatpar6' )

      end subroutine incommatpar6
