      subroutine incommatrows ( iwork, lenim1, lenim2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmodulematstr
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.0  date 21-04-2015 New parameter list
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill structure of matrix in case of a compact matrix
!     Part 1a, number of non-zero columns per row
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     iterative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), lenim1, lenim2

!     iwork               work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incommatcomp
!     lenim1         o    Length of the array imat part 1
!     lenim2         o    length of the array imat part 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leng1
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     leng1          Length of local co-ordinates array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCM01         Compute first part of contact matrix
!     PRININ         print 1d integer array
!     SEPPETSCSTART  Start petsc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatrows' )
      debug = .false. .and. ioutp>=0

      IM => intmt(intmat)
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatrows'
         write(irefwr,*) 'symmetric', IM%symmetric
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      if ( debug ) &
         write(irefwr,1) 'npoint, nelem, nrusol', &
                          npoint, nelem, nrusol

!     --- Reserve temporary space for subr incm01

      leng1 = nrusol+1

      if ( debug ) write(irefwr,1) 'nusol', nusol

      allocate ( irowsii(leng1), stat = error )
      if ( error/=0 ) call eralloc ( error, leng1, 'irowsii' )
      if ( ierror/=0 ) go to 1000

!     --- remark: kprob(1,1) is always defined, e.g. 1500

      if ( ierror/=0 ) go to 1000

!     --- Fill intmat part 1a: number of columns per row

      call incm01 ( iwork )

      if ( IM%petsc ) then

!     --- Petsc is used
!         Start petsc if not already started

         call seppetscstart

      end if  ! ( IM%petsc  )

!     --- Compute length of imat1

      lenim1 = nrusol+1
      lenim2 = irowsii(lenim1)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsii, nrusol+1, 'imat1/1' )
         write(irefwr,1) 'lenim2, lenim1', lenim2, lenim1
         write(irefwr,*) 'End incommatrows'

      end if  ! ( debug )
      call erclos ( 'incommatrows' )

      end subroutine incommatrows
