      subroutine ini014
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 06-01-2016 Use sepmodulematstr
!        version  6.0  date 17-12-2015 New parameter list
!        version  5.0  date 21-04-2015 New parameter list
!
!   copyright (c) 1985-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute length of arrays for subroutine ini011
!     arrays ihelp and intmat part b
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inusol, nhelp1, jnusol, jnunkp, inunkp, &
              nhelp2, knusol, nfnbr, nbr, iknbr, nodp, j, ibr

!     ibr            Loop variable for neighbors
!     iknbr          Nodal point number of neighbor
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jnunkp         Number of degrees of freedom in first point
!     jnusol         Position of unknown in array USOL
!     knusol         Position of unknown in array USOL
!     nbr            number of neighbors of nodal point
!     nfnbr          help parameter to compute nbr
!     nhelp1         number of prescribed degrees of freedom in node
!     nhelp2         number of free degrees of freedom coupled to
!                    the prescribed degrees of freedom in node
!     nodp           Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INI014FICT     Fill part of imat1 corresponding to fictitious unknown
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini014' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini014'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      irowsip = 0

      if ( debug ) &
         write(irefwr,1) 'nusol, nbound, nrusolloc', nusol, nbound, nrusolloc

      do nodp = 1, npoint

!     --- for every nodal point in the mesh

         inusol = sepgetinusol ( nodp, inunkp ) ! last unknown before nodp
         nhelp1 = 0      ! number of prescribed dofs
         nhelp2 = 0      ! number of free dofs

         do j = 1,inunkp

            knusol = kprobr(inusol+j)       ! number with respect to matrix
            if ( knusol<=nrusolloc ) then

!           --- knusol<=  nrusolloc (free d.o.f.)

              nhelp2 = nhelp2+1

            else

!           --- knusol>nrusolloc  (prescribed d.o.f.)

              nhelp1 = nhelp1+1

            end if

         end do

         if ( debug ) &
            write(irefwr,1) 'nodp, inunkp, nhelp1, nhelp2', &
                             nodp, inunkp, nhelp1, nhelp2

         if ( nhelp1>0 ) then

!        --- nhelp1>0, so there is at least one prescribed unknown
!            in nodp
!            Add the number of free degrees of freedom in neighbors
!            to nhelp2
!            get the number of neighboring nodes and put in nbr

            nfnbr = kmshe1(nodp)
            nbr = kmshe1(nodp+1)-nfnbr

            do ibr = 1, nbr

               iknbr = kmshe2(nfnbr+ibr)

!              --- get the number of d.o.f.'s in this neighboring node

               jnusol = sepgetinusol ( iknbr, jnunkp )

               do j = 1, jnunkp
                  knusol = kprobr(jnusol+j)
                  if(knusol<=nrusolloc) nhelp2 = nhelp2+1
               end do

            end do

            if ( debug ) write(irefwr,1) 'nhelp2', nhelp2

            do j = 1, inunkp
               knusol = kprobr(inusol+j)
               if ( knusol>nrusolloc ) irowsip(knusol-nrusolloc+1) = nhelp2
            end do

         end if

      end do

      if ( indpry>0 ) then

!     --- indpry>0, fictitious unknowns

         call ini014fict ( irowsip )

      end if

!     --- Accumulate irowsip

      do j = 2, nbound+1
         irowsip(j) = irowsip(j)+irowsip(j-1)
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsip, nbound+1, 'irowsip' )
         write(irefwr,*) 'End ini014'

      end if  ! ( debug )
      call erclos ( 'ini014' )

      end subroutine ini014
