      subroutine ininodeelms ( elements, numelms, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-02-2015 Use allocate
!        version  1.0  date 20-01-2003
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all nodes of the elements stored in array elements into array nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer elements(*), numelms, nodes(*), nnodes

!     elements       i    Array of length numelms containing the element
!                         numbers of the elements of which the nodes must
!                         be detected
!     nnodes         o    Number of nodes in array nodes
!     nodes          o    Contains the node numbers of the elements in array
!                         elements
!     numelms        i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inpelm, ielem, istart, ielgrp, iielem, i
      integer, allocatable :: elementsact(:), iwork(:)

!     elementsact    Array of size nelem indicating which elements are
!                    stored in array elements (elementsact(i) = 1) or not
!                    (elementsact(i) = 0)
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     istart         First position used at a part
!     iwork          integer work array of length npoint
!                    If a node i is in one of the elements of array elements
!                    iwork(i) is made equal to 1, otherwise 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays elementsact and iwork
!     Make elementsact equal to 1 for all elements in elements
!     Set iwork = 1 for all nodes in the elements in elements
!     Fill array nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ininodeelms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ininodeelms'

      end if

      allocate ( elementsact(nelem), iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays elementsact and iwork

      elementsact = 0
      iwork = 0

!     --- Make elementsact equal to 1 for all elements in elements

      do i = 1, numelms
         elementsact(elements(i)) = 1
      end do

!     --- Set iwork = 1 for all nodes in the elements in elements
!         For all elements ielem do

      ielem  = 0
      istart = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

         do iielem = 1, inelem
            ielem = ielem + 1
            if ( elementsact(ielem)==1 ) then

!           --- Element belongs to list of elements
!               Set iwork equal to 1 for all nodes in the element

               do i = 1, inpelm
                  iwork(kmeshc(istart+i)) = 1
               end do

            end if  ! ( elementsact(ielem)==1 )
            istart = istart+inpelm

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

!     --- Fill array nodes

      nnodes = 0
      do i = 1, npoint

         if ( iwork(i)==1 ) then

!        --- Node found

            nnodes = nnodes+1
            nodes(nnodes) = i

         end if

      end do  ! i = 1, npoint

      deallocate ( iwork, elementsact, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) write(irefwr,*) 'End ininodeelms'
      call erclos ( 'ininodeelms' )

      end
