      subroutine init07comsrf ( iinp, ielshape, natcomsrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for
!     boundary groups of type complete surface in array ielshape
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, ielshape(2,*), natcomsrf

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     iinp          i/o   last position used in array iinputkp
!     natcomsrf      i    number of parts where complete_surface elements are
!                         defined for natural boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ibngrp, inum, inpelm, i
      integer, allocatable :: nodes(:)

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
!     inum           Number of surfaces
!     j              Counting variable
!     nodes          integer work array of length npoint to store
!                    the nodes along the surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     271   boundary element group number has wrong value
!    1083   Shape number for boundary group different from before
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init07comsrf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07comsrf'
         write(irefwr,1) 'iinp, natcomsrf', iinp, natcomsrf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- numcomsrf>0  complete surface boundary elements

      do j = 1, natcomsrf
         ibngrp = iinputkp(iinp+1)
         if ( ibngrp<=0 .or. ibngrp>numnat ) then
            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errint ( j, 3 )
            call errchr ( 'surface', 1 )
            call errsub ( 271, 3, 0, 1 )
         end if
         inum = iinputkp(iinp+2)
         if ( debug ) write(irefwr,1) 'j, ibngrp, inum', j, ibngrp, inum

!        --- Store all nodes of the surface into array nodes

         ishape = 0

         allocate ( nodes(inpelm), stat = error )
         if ( error/=0 ) call eralloc ( error, inpelm, 'nodes' )
         if ( ierror/=0 ) go to 1000

         call sepsurfsnod ( inum, nodes, inpelm, iinputkp(iinp+3) )

         deallocate ( nodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nodes' )

         if ( ielshape(2,ibngrp+nelgrp)/=0 ) then

!        --- check if shapes coincide

            if ( ielshape(2,ibngrp+nelgrp)/=ishape ) then
               call errint ( ibngrp, 1 )
               call errint ( ishape, 2 )
               call errint ( ielshape(2,ibngrp+nelgrp), 3 )
               call errint ( i, 4 )
               call errchr ( 'surface', 1 )
               call errsub ( 1083, 4, 0, 1 )
            end if

         end if
         ielshape(1,ibngrp+nelgrp) = inpelm
         ielshape(2,ibngrp+nelgrp) = ishape
         iinp = iinp+2+inum

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         write(irefwr,*) 'End init07comsrf'

      end if  ! ( debug )
      call erclos ( 'init07comsrf' )

      end
