      subroutine init07contpt ( iinp, ielshape, natconpt, natnocpt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for
!     boundary groups of type contact points in array ielshape
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, ielshape(2,*), natconpt, natnocpt

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     iinp          i/o   last position used in array iinputkp
!     natconpt       i    number of parts where contact points are defined as
!                         bound els.
!     natnocpt       i    number of parts where no_contact points are defined as
!                         boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ibngrp, inpelm

!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     271   boundary element group number has wrong value
!    1040   Contact elements have only been implemented for surfaces
!    1083   Shape number for boundary group different from before
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init07contpt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07contpt'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- natconpt>0  contact point boundary elements or
!         natnocpt>0  no_contact point boundary elements

      if ( kelmn==0 ) then
         call errsub ( 1040, 0, 0, 0 )
         go to 1000
      end if
      do j = 1, natconpt+natnocpt
         ibngrp = iinputkp(iinp+1)
         if ( ibngrp<=0 .or. ibngrp>numnat ) then
            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errint ( j, 3 )
            call errchr ( '(no)contact points', 1 )
            call errsub ( 271, 3, 0, 1 )
         end if
         iinp = iinp+3
         inpelm = 1
         ishape = -10
         if ( ielshape(2,ibngrp+nelgrp)/=0 ) then

!        --- check if shapes coincide

            if ( ielshape(2,ibngrp+nelgrp)/=ishape ) then
               call errint ( ibngrp, 1 )
               call errint ( ishape, 2 )
               call errint ( ielshape(2,ibngrp+nelgrp), 3 )
               call errint ( j, 4 )
               call errchr ( '(no)contact points', 1 )
               call errsub ( 1083, 4, 0, 1 )
            end if

         end if
         ielshape(1,ibngrp+nelgrp) = inpelm
         ielshape(2,ibngrp+nelgrp) = ishape

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End init07contpt'

      end if  ! ( debug )
      call erclos ( 'init07contpt' )

      end
