      subroutine init09 ( istfl1, ielshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 14-10-2011 New parameter list
!        version  3.0  date 28-12-1998 Extra parameter nphys
!        version  2.0  date 19-10-1998 Extra parameter ielshape
!
!   copyright (c) 1998-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information from iinputkp concerning the element groups corresponding
!     to global unknowns into array kprob part x
! **********************************************************************
!
!                       KEYWORDS
!
!     file_1
!     read
!     problem_definition
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istfl1(*), ielshape(2,*)

!     ielshape   i    Array of length 2*(nelgrp+numnat) containing:
!                     (1,i) number of nodes in element group i
!                     (2,i) shape number of elements in element group i
!                     if i>nelgrp: i-nelgrp = boundary group number
!                     otherwise it is the element group number
!     istfl1    i/o   Array in which information from FILE 1 is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical linear
      integer kh1, ipos, kh2, i, meshtp, iinp10, &
              loctype, iglgrp, iinp, iinp8, iinp9, itype, &
              inunkp, indtvc, ielgrp, numglall, numglcrv, &
              numglsrf, numglvlm
      integer, allocatable :: iwork(:,:)

!     i          Counting variable
!     ielgrp     element group number
!     iglgrp     global element group number
!     iinp       last position used in array iinputkp
!     iinp10     first pos. iinputkp definition global group renumbering
!     iinp8      first pos. iinputkp definition global group type numbers
!     iinp9      first pos. iinputkp definition global group elements
!     indtvc     Number of vectors of special structure
!     inunkp     Number of degrees of freedom in first point
!     ipos       Position in array
!     itype      Type number of standard element
!     iwork      integer work array of length 2* nglgrp
!                iwork(1,i) contains the starting address of group i in
!                           iinputkp
!                iwork(2,i) contains the type of elements to be used
!                           for group i
!                           1 = curves
!                           2 = surfaces
!                           3 = volumes
!                           4 = all
!     kh1        Help parameter to compute a position in KPROB
!     kh2        Help parameter to compute a position in KPROB
!     linear     Indicates whether the element is a linear simplex (true)
!                or not (false)
!     loctype    mod(itype,10000), i.e. the actual type number
!     meshtp     Shape number of element as stored on file 1
!                If meshtp = -1, the shape of the element is free and exactly
!                10 positions are filled in file 1 with information concerning
!                the unknowns for each inunkp>0
!     numglall   number of parts where all elements are defined
!                for global elements
!     numglcrv   number of parts where curve elements are defined
!                for global elements
!     numglsrf   number of parts where surface elements are defined
!                for global elements
!     numglvlm   number of parts where volume elements are defined
!                for global elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    input from file 1 is read
!    File 1 is filled as follows:
! part 1)  records 1 up to krec2-1:  pointers for each standard problem
!          with 99<itype<maxtyp  (maximum type number) to part 2
!          For a certain type the pointer may be to another type
!          only if all the information is identical
! part 2)  records krec2,...,9 information on standard problems
!  for each type:
!  With respect to global unknowns:
!    pos. 1   -inunkp   - number of global degrees of freedom, &
!         2   -1        indicating that it concerns a global unknown
!         3   meshtp    shape number of the element (must be -10)
!         4   0
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
!    1944   Type number does not correspond to global group
!    2029   Number of coupled degrees of freedom larger than nphys
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read information concerning type numbers from file 1 if necessary
!     Compute starting address of information concerning the type in array
!     istfl1
!     Read information concerning information corresponding to type numbers
!     from file 1 if necessary
!     Compute starting address of information concerning the type in array
!     istfl1
!     Put information into istore (read information if necessary)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init09' )

!     --- Read information concerning type numbers from file 1 if necessary

      iinp8 = intarprb(8)
      iinp9 = intarprb(9)
      iinp10 = intarprb(10)
      if ( iinputkp(15)<=1 ) then
         nglgrp = 0
      else
         nglgrp = iinputkp(29)
      end if
      nglunk = 0

!     --- First detect if all surface elements are linear or quadratic

      linear = .false.
      do i = 1, nelgrp
         ishape = ielshape(2,i)
         if ( ishape==3 .or. ishape==5 .or. ishape==9 ) linear = .true.
      end do

      allocate ( iwork(2,nglgrp),stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*nglgrp, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Find starting addresses of groups in part 9

      iwork = 0

      numglcrv = iinputkp(30)
      numglsrf = iinputkp(31)
      numglvlm = iinputkp(32)
      numglall = iinputkp(33)
      iinp = iinp9
      do i = 1, numglcrv
         iglgrp = iinputkp(iinp)
         iwork(1,iglgrp) = iinp
         iwork(2,iglgrp) = 1
         iinp = iinp+4
      end do
      do i = 1, numglsrf
         iglgrp = iinputkp(iinp)
         iwork(1,iglgrp) = iinp
         iwork(2,iglgrp) = 2
         iinp = iinp+3
      end do
      do i = 1, numglvlm
         iglgrp = iinputkp(iinp)
         iwork(1,iglgrp) = iinp
         iwork(2,iglgrp) = 3
         iinp = iinp+3
      end do
      do i = 1, numglall
         iglgrp = iinputkp(iinp)
         iwork(1,iglgrp) = iinp
         iwork(2,iglgrp) = 4
         iinp = iinp+1
      end do
      do iglgrp = 1, nglgrp
         itype = iinputkp(iinp8+2*(iglgrp-1))
         iinp = iwork(1,iglgrp)
         if ( itype>=100 ) then

!        --- Standard element

            loctype = mod(itype,10000)
            kh1 = (loctype-100)/nrec1+1
            if ( kh1/=irecp1 ) &
               call readfl ( kh1, 1, nrec1, 1, 1, istfl1 )

!           --- Compute starting address of information concerning the type in
!               array istfl1

            ipos = istfl1(loctype-99-nrec1*(kh1-1))
            if ( ipos==0 ) then

!           --- Error 43: type number not available

               call errint ( loctype, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 43, 2, 0, 0)
               go to 1000

            end if

!           --- Read information concerning information corresponding to type
!               numbers from file 1 if necessary

            kh2 = (ipos-1)/nrec1+1
            if ( kh2/=irecp2 ) &
                call readfl ( kh2, nrec1+1, 2*nrec1, 2, 1, istfl1 )

!           --- Compute starting address of information concerning the type in
!               array istfl1

            ipos   = ipos-(kh2-2)*nrec1
            inunkp = abs(istfl1(ipos))

            indtvc = istfl1(ipos+1)
            meshtp = istfl1(ipos+2)

            if ( indtvc/=-1 .and. meshtp/=-10 ) then

!           --- error 1944: element does not correspond to global element

               call errint ( iglgrp, 1 )
               call errint ( itype, 2 )
               call errsub ( 1944, 2, 0, 0 )

            end if

         else

!        --- Number of unknowns stored in iinputkp

            inunkp = iinputkp(iinp8+2*(iglgrp-1)+1)

         end if
         nglunk = nglunk+inunkp

         kprobx(3+(iglgrp-1)*6+1) = itype
         kprobx(3+(iglgrp-1)*6+2) = inunkp
         kprobx(3+(iglgrp-1)*6+3) = iwork(2,iglgrp)
         if ( iwork(2,iglgrp)==1 ) then
            ishape = iinputkp(iinp+1)
            if ( ishape==0 ) then
               if ( linear ) then
                  ishape = 1
               else
                  ishape = 2
               end if
            end if
            kprobx(3+(iglgrp-1)*6+4) = ishape
            kprobx(3+(iglgrp-1)*6+5) = iinputkp(iinp+2)
            kprobx(3+(iglgrp-1)*6+6) = iinputkp(iinp+3)
         else if ( iwork(2,iglgrp)<=3 ) then
            kprobx(3+(iglgrp-1)*6+4) = 0
            kprobx(3+(iglgrp-1)*6+5) = iinputkp(iinp+1)
            kprobx(3+(iglgrp-1)*6+6) = iinputkp(iinp+2)
         else
            kprobx(3+(iglgrp-1)*6+4) = 0
            kprobx(3+(iglgrp-1)*6+5) = 0
            kprobx(3+(iglgrp-1)*6+6) = 0
         end if

      end do

!     --- Compute number of coupled degrees of freedom

      nglcoupl = iinputkp(iinp10)
      kprobx(1) = nglgrp
      kprobx(2) = nglunk
      kprobx(3) = nglcoupl
      if ( nglcoupl>nphys ) then

!     --- Error: nglcoupl>nphys

         call errint ( nglcoupl, 1 )
         call errint ( nphys, 2 )
         call errsub ( 2029, 2, 0, 0 )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'init09' )

      end
