      subroutine printprobinp04 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the global elements part of array
!     iinput with respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
!     global
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nglgrp, iglgrp, i, numcrv, numsrf, numvlm, numall

!     i              Counting variable
!     iglgrp         global element group number
!     nglgrp         number of element groups corresponding to global
!                    unknowns
!     numall         Number of global groups on all parts
!     numcrv         number of global groups on curves
!     numsrf         number of global groups on surface
!     numvlm         number of global groups volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp04'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nglgrp = iinput(29)

      if ( nglgrp==0 ) go to 1000

      write(irefwr,*) 'global groups'

      do iglgrp = 1, nglgrp

         call printposint ( iinput(iinp), 'itype' )
         call printposint ( iinput(iinp+1), 'numdegfd' )
         iinp = iinp+2

      end do  ! iglgrp = 1, nglgrp

      numcrv = iinput(30)
      do i = 1, numcrv

         call printposint ( iinput(iinp), 'iglgrp' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call printposint ( iinput(iinp+2), 'icrv1' )
         call printposint ( iinput(iinp+3), 'icrv2' )
         iinp = iinp+4

      end do  ! i = 1, numcrv

      numsrf = iinput(31)
      do i = 1, numsrf

         call printposint ( iinput(iinp), 'iglgrp' )
         call printposint ( iinput(iinp+1), 'isrf1' )
         call printposint ( iinput(iinp+2), 'isrf2' )
         iinp = iinp+3

      end do  ! i = 1, numsrf

      numvlm = iinput(32)
      do i = 1, numvlm

         call printposint ( iinput(iinp), 'iglgrp' )
         call printposint ( iinput(iinp+1), 'ivlm1' )
         call printposint ( iinput(iinp+2), 'ivlm2' )
         iinp = iinp+3

      end do  ! i = 1, numvlm

      numall = iinput(33)
      call prinin ( iinput(iinp), numall, 'global groups on all parts' )
      iinp = iinp + numall

1000  call erclos ( 'printprobinp04' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp04'

      end if  ! ( debug )

      end
