      subroutine printprobinp08 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the unknown constants part of array
!     iinput with respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, numcrv, numsrf

!     i              Counting variable
!     inpelm         number of points per element
!     numcrv         number of parts where curves are given for generation
!                    of unknown constants
!     numsrf         number of parts where surfaces are given for generation
!                    of unknown constants
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp08' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp08'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iinput(13)==0 ) go to 1000

      write(irefwr,*) 'unknown constants'

      numcrv = iinput(iinp)
      numsrf = iinput(iinp+1)
      iinp = iinp+2

      if ( numcrv>0 ) write(irefwr,*) 'on curves'
      do i = 1, numcrv

         call printposint ( iinput(iinp), 'degfd' )
         call printposint ( iinput(iinp+1), 'jstep' )
         call printposint ( iinput(iinp+2), 'ifirst' )
         call printposint ( iinput(iinp+3), 'ilast' )
         iinp = iinp+4

      end do  ! i = 1, numcrv

      if ( numsrf>0 ) write(irefwr,*) 'on surfaces'
      do i = 1, numsrf

         call printposint ( iinput(iinp), 'degfd' )
         call printposint ( iinput(iinp+1), 'ifirst' )
         call printposint ( iinput(iinp+2), 'ilast' )
         inpelm = iinput(iinp+3)
         call printposint ( inpelm, 'inpelm' )
         call prinin ( iinput(iinp+4), inpelm, 'local nodes' )
         iinp = iinp+4+inpelm

      end do  ! i = 1, numsrf

1000  call erclos ( 'printprobinp08' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp08'

      end if  ! ( debug )

      end
