      subroutine prob04elgrp ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for element groups
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numgrp, l, ielgrp, inpelm, k

!     ielgrp         element group number
!     inpelm         number of points per element
!     k              Counting variable
!     l              General loop variable
!     numgrp         Number of groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     443   element group number in essential boundary conditions incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04elgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04elgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numgrp = iinput(12)
      do l = 1,numgrp
         ielgrp = iinput(iinp+2)
         iinp = iinp+2
         if ( ielgrp<=0 .or. ielgrp>nelgrp ) then
            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errsub ( 443, 2, 0, 0 )
         end if
         if ( ierror/=0 ) goto 1000

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         do k = 1,inpelm
            if ( iinput(iinp+k)/=0 ) nkbnd = nkbnd+inelem
         end do
         iinp = iinp+inpelm
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04elgrp'

      end if  ! ( debug )
      call erclos ( 'prob04elgrp' )

      end
