      subroutine prob04inters ( iinput, iinp, &
                                nodes, ipoint, nesintersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for intersections
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     essential_boundary_condition
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nesintersect, &
              nodes(npoint,nesintersect), ipoint(nesintersect)

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     ipoint         o    Array of length nesintersect containing the
!                         number of nodes for each intersection
!     nesintersect   i    Number of intersections where essential boundary
!                         conditions are given
!     nodes          o    Array of size nesintersect x npoint
!                         nodes(i,1:ipoint(i)) contains the nodal points
!                         corresponding to intersection i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numsrf, nparts, nnodes

!     i              Counting variable
!     nnodes         Number of nodes
!     nparts         number of parts the boundary consists of (dummy)
!     numsrf         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04inters' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04inters'
         write(irefwr,1) 'nkbnd, nesintersect', nkbnd, iinput(56)
         write(irefwr,1) 'iinp, npoint', iinp, npoint
         call prinin ( iinput(iinp+1), 5, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nesintersect = iinput(56)
      do  i = 1, nesintersect

         numsrf = iinput(iinp+3)       ! number of surfaces
         if ( debug ) write(irefwr,1) 'i, numsrf', i, numsrf

!        --- Find all nodes in intersection and store in nodes

         call sepfillnodes ( 26, 0, iinput(iinp+3), &
                             nodes(1,i), nnodes, ipoint, nparts )
         ipoint(i) = nnodes  ! number of nodes
         nkbnd = nkbnd+nnodes
         if ( debug ) then
            write(irefwr,1) 'nnodes, nkbnd', nnodes, nkbnd
            call prinin ( nodes(1,i), nnodes, 'nodes' )
         end if  ! ( debug )
         iinp = iinp+3+numsrf

      end do  ! i = 1, nesintersect

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nkbnd/end', nkbnd
         write(irefwr,*) 'End prob04inters'

      end if  ! ( debug )
      call erclos ( 'prob04inters' )

      end
