      subroutine prob04levset ( iinput, iinp, nkbnd, intarprb )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodes with essential boundary conditions in the
!     case of level set functions
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     obstacle
!     essential_boundary_condition
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd, intarprb(*)

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     intarprb       i    Array containing parameters to be transported between
!                         lower subroutines of PROBDF.
!                         See PROBDFBF for a description
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numlevelset, iinp14, ilev, iskip, numnodes, i, istart

!     i              Counting variable
!     iinp14         first position in array iinput corresponding to level set
!     ilev           Sequence number of level set function
!     iskip          Defines how the level set nodes must be treated
!                    -1: all nodes outside phi>0 are prescribed
!                     0: no nodes are prescribed
!                     1: all nodes inside phi>0 are prescribed (not the
!                        boundary)
!     istart         First position used at a part
!     numlevelset    Number of different level set functions
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all level set functions ilev do
!        iskip := action to be performed for level set function ilev
!        if ( iskip = -1 ) then
!           Set all points outside "obstacle" as prescribed
!        else if ( iskip = 1 ) then
!           Set all points inside "obstacle" as prescribed
!     If zero_level is given then
!        set corresponding nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04levset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04levset'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Get parameters

      numlevelset = iinput(48)
      iinp14 = intarprb(18)

!     --- For all level set functions ilev do

      do ilev = 1, numlevelset

!     --- iskip := action to be performed for level set function ilev

         istart = ipointobst(ilev)
         iskip = iinput(iinp14+ilev-1)
         if ( debug ) write(irefwr,*) 'ilev, iskip ', ilev, iskip
         numnodes = ipointobst(istart)+ &     ! nodes in inner obstacle
                    ipointobst(istart+1)      ! nodes in boun obstacle
         if ( iskip.eq.-1 ) then

!        --- iskip = -1
!            Set all points inside "obstacle" as prescribed

            nkbnd = nkbnd+numnodes

         else if ( iskip.eq.1 ) then

!        --- iskip = 1
!            Set all points outside "obstacle" as prescribed

            numnodes = numnodes+ipointobst(istart+2)  ! nodes on boundary obst
            nkbnd = nkbnd+npoint-numnodes

         end if  ! ( iskip.eq.-1 )

      end do  ! ilev = 1, numlevelset

      if ( iinput(47).gt.0 ) then

!     --- zero_level is given
!         set corresponding nodes

         do i = 1, iinput(47)

            ilev = iinput(iinp+2)
            iinp = iinp+2
            istart = ipointobst(ilev)
            numnodes = ipointobst(istart+2)  ! nodes on boundary obst
            nkbnd = nkbnd+numnodes

         end do  ! i = 1, iinput(47)

      end if  ! ( iinput(47).gt.0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04levset'

      end if
      call erclos ( 'prob04levset' )

      end
