      subroutine prob04nodal ( iinput, nkbnd, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nkbnd, iinp

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nesnodal, i, numnodal, idummy(1), idum(1)
      double precision dummy(1)
      character (len=80) filename

!     dummy          Dummy parameter
!     filename       Name of file
!     i              Counting variable
!     idum           General dummy variable
!     idummy         Dummy parameter
!     nesnodal       Number of groups of nodal points with essential
!                    boundary conditions
!     numnodal       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFILE       Read information from an ASCII file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1673   Wrong number of nodes found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04nodal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04nodal'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nesnodal = iinput(28)

      do i = 1, nesnodal
         numnodal = iinput(iinp+2)
         if ( numnodal>0 ) then

!        --- numnodal>0, point numbers given

            iinp = iinp+2+numnodal
            nkbnd = nkbnd+numnodal

         else if ( numnodal<0 ) then

!        --- numnodal<0, point numbers in file
!            Read number from file

            filename = texts(-numnodal)
            call readfile ( filename, .true., numnodal, &
                            idum, idummy, dummy, 1 )
            if ( ierror/=0 ) go to 1000
            if ( numnodal>npoint ) then

!           --- Too many numbers in file

               call errint ( numnodal, 1 )
               call errint ( npoint, 2 )
               call errchr ( filename, 1)
               call errsub ( 1673, 2, 0, 1)

            end if
            iinp = iinp+3
            nkbnd = nkbnd+numnodal

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04nodal'

      end if  ! ( debug )
      call erclos ( 'prob04nodal' )

      end
