      subroutine prob04up ( iinp, iinput, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for user points
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, iinput(*), nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numpnt, j, npnt, i, ipoint, jnodp

!     i              Counting variable
!     ipoint         Nodal point sequence number
!     j              Counting variable
!     jnodp          Nodal point sequence number
!     npnt           Number of points
!     numpnt         number of parts where points are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     179   point number of user point is incorrect
!     180   user point is not a nodal point
!     283   The number of user points required in the creation of essential
!           boundary conditions is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04up' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04up'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numpnt = iinput(6)

      do j = 1, numpnt
         npnt = iinput(iinp+2)
         if ( npnt<=0 .or. npnt>nuspnt ) then
            call errint ( npnt, 1 )
            call errint ( nuspnt, 2 )
            call errsub ( 283, 2, 0 , 0 )
         end if
         iinp = iinp+2
         do i = 1, npnt
            ipoint = iinput(iinp+i)
            if ( ipoint<=0 .or. ipoint>nuspnt ) then
               call errint ( ipoint, 1 )
               call errint ( nuspnt, 2 )
               call errint ( 0, 3 )
               call errint ( 0, 4 )
               call errsub ( 179, 4, 0 , 0 )
            end if
            jnodp = kmeshl(ipoint)
            if ( jnodp>0 ) then

!           --- point belongs to present block

               ipoint = kmeshl(ipoint)
               nkbnd = nkbnd+1
               if ( debug ) &
                  write(irefwr,1) 'nkbnd, jnodp', nkbnd, jnodp

            else if ( .not. parallel ) then

!           --- point is not a user point

               call errint ( ipoint, 1 )
               call errwar ( 180, 1, 0, 0)

            end if

         end do

         iinp = iinp+npnt

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04up'

      end if  ! ( debug )
      call erclos ( 'prob04up' )

      end
