      subroutine prob05cont ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     contact
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nesconct, nesnocon, icontc, ibit

!     i              Counting variable
!     ibit           Actual bit pattern
!     icontc         Sequence number of contact
!     nesconct       Number of contacts at which essential boundary
!                    conditions are given
!     nesnocon       Number of "no"-contacts with essential boundary
!                    conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PROB05CONT1    Fill essential boundary conditions for contact points
!     PROB05CONT2    Fill essential boundary conditions for no-contact points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05cont' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05cont'
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxcontc = kprobloc(48)
      nesconct = iinputkp(19)
      nesnocon = iinputkp(22)
      if ( debug ) &
         write(irefwr,1) 'maxcontc, nesconct, nesnocon', &
                          maxcontc, nesconct, nesnocon

      do i = 1, nesconct
         ibit = iinputkp(iinp+1)
         icontc = iinputkp(iinp+3)
         iinp = iinp+3
         if ( maxcontc>0 .and. kprobloc(46)>0 ) &
            call prob05cont1 ( icontc, ibit )
         if ( debug ) &
            write(irefwr,1) 'i, icontc, nkbnd', i, icontc, nkbnd
      end do

      do i = 1, nesnocon
         ibit = iinputkp(iinp+1)
         icontc = iinputkp(iinp+3)
         iinp = iinp+3
         if ( maxcontc>0 .and. kprobloc(46)>0 ) &
            call prob05cont2 ( icontc, ibit  )
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
         write(irefwr,*) 'End prob05cont'

      end if  ! ( debug )
      call erclos ( 'prob05cont' )

      end
