      subroutine prob05elgrp ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for element groups
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numgrp, l, ielgrp, inpelm, k, &
              ibit, ibit2, ih, ikelmc, nodalp, inunkp, inodp, j

!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ielgrp         element group number
!     ih             Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     inodp          Nodal point number
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     nodalp         Nodal point sequence number
!     numgrp         Number of groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      77   You try to prescribe the wrong degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05elgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05elgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      ibit2  = inibt0(inunkp)

      numgrp = iinputkp(12)
      do l = 1,numgrp
         ielgrp = iinputkp(iinp+2)
         ibit = iinputkp(iinp+1)
         iinp = iinp+2
         ikelmc = 0
         do k = 1, ielgrp-1
            inelem = Topgrp(k)%inelem
            inpelm = Topgrp(k)%inpelm
            ikelmc = ikelmc+inpelm*inelem
         end do
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         do j = 1, inelem

            do k = 1, inpelm

               if ( iinputkp(iinp+k)/=0 ) then

                  nodalp = kmeshc(ikelmc+k)
                  if ( indprf>0 ) then

!                 --- indprf>0  kprob part f contains the number of unknowns

                      inunkp = kprobf(nodalp+1)-kprobf(nodalp)
                      ibit2 = inibt0(inunkp)
                  end if

                  if ( ibit>ibit2 .or. ibit<0 ) then
                     call errint ( ibit, 1 )
                     call errint ( ibit2, 2 )
                     call errint ( inodp, 3 )
                     call errwar ( 77, 3, 0, 0 )
                  end if

                  nkbnd = nkbnd+1
                  ih = ibit
                  if ( ih==0 ) ih = ibit2
                  kprobg(1,nkbnd) = nodalp
                  kprobg(2,nkbnd) = ih

               end if

            end do

            ikelmc = ikelmc+inpelm

         end do

         iinp = iinp+inpelm

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05elgrp'

      end if  ! ( debug )
      call erclos ( 'prob05elgrp' )

      end
