      subroutine prob05nodal2 ( ibit, numnodal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for nodes in a file
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodal, ibit

!     ibit           i    Actual bit pattern
!     numnodal       i    Minus number of nodal points in file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, ih, k, inodp, ibit2, idum
      double precision dummy(1)
      integer, allocatable :: iwork(:)

!     dummy          Dummy parameter
!     ibit2          Maximal bit pattern
!     idum           General dummy variable
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     iwork          integer work array of length -numnodal to store the
!                    nodes
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
!     READFILE       Read information from an ASCII file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1672   Nodal point number out of range
!    1851   Wrong unknown prescribed in user point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05nodal2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05nodal2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      numnodal = abs(numnodal)

      allocate ( iwork(numnodal), stat = error )
      if ( error/=0 ) call eralloc ( error, numnodal, 'iwork' )
      if ( ierror/=0 ) go to 1000

      call readfile ( texts(numnodal), .false., numnodal, &
                      idum, iwork, dummy, 1 )
      if ( ierror/=0 ) go to 1000

      do k = 1, numnodal
         inodp = iwork(k)
         if ( inodp<=0 .or. inodp>npoint ) then
            call errint ( k, 1 )
            call errint ( inodp, 2 )
            call errint ( npoint, 3 )
            call errsub ( 1672, 3, 0, 0 )
         end if
         if ( indprf>0 ) then

!        --- indprf>0  kprob part f contains the number of unknowns

            inunkp = kprobf(inodp+1)-kprobf(inodp)
            ibit2 = inibt0(inunkp)

         end if
         if ( ibit>ibit2 .or. ibit<0 ) then
            call errint ( inibt1(ibit), 1 )
            call errint ( inodp, 2 )
            call errint ( inunkp, 3 )
            call errwar ( 1851, 3, 0 , 0 )
         end if
         nkbnd = nkbnd+1
         ih = ibit
         if ( ih==0 ) ih = ibit2
         kprobg(1,nkbnd) = inodp
         kprobg(2,nkbnd) = ih
      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05nodal2'

      end if  ! ( debug )
      call erclos ( 'prob05nodal2' )

      end
