      subroutine prob05sfouter ( iinp, inunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for outer surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, inunkp

!     iinp          i/o   last position used in array iinput
!     inunkp        i/o   Number of degrees of freedom in point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nnodes, nodalp, nparts, isrf1, ibit, ibit2, ielem, ih, &
              isrf2, istart, j, nsurfaces, &
              inodp, ipart, isurnr, k
      integer, allocatable :: nodes(:), iwork(:)

!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     ipart          Sequence number of part
!     isrf1          First surface number
!     isrf2          Last surface number
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array
!     j              Counting variable
!     k              Counting variable
!     nnodes         Number of nodes
!     nodalp         Nodal point sequence number
!     nodes          Contains the node numbers that must be filled
!     nparts         number of parts the boundary consists of
!     nsurfaces      Number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1862   Wrong unknown prescribed at surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05sfouter' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05sfouter'
         write(irefwr,1) 'inunkp', inunkp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( nodes(npoint), iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      ibit2  = inibt0(inunkp)
      ibit = ibit2

      nparts = outerboun(1)
      if ( debug ) write(irefwr,1) 'nparts', nparts

!     --- Clear iwork

      iwork = 0

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         nsurfaces = outerboun(istart)

         do j = 1, nsurfaces

            isurnr = abs(outerboun(istart+j))
            nnodes = npoint
            call sepsurnod ( isurnr, isurnr, nodes, nnodes )
            do k = 1, nnodes
               inodp = nodes(k)

               if ( inodp==0 ) go to 200  ! jump for empty nodes

               if ( iwork(inodp)==0 ) then

!              --- iwork(inodp) = 0:  new nodal point

                  iwork(inodp) = 1
                  if ( indprf>0 ) then

!                 --- indprf>0  kprob part f contains
!                     the number of unknowns

                     inunkp = kprobf(inodp+1)-kprobf(inodp)
                     ibit2 = inibt0(inunkp)

                  end if
                  if ( ibit>ibit2 .or. ibit<0 ) then
                     call errint ( inibt1(ibit), 1 )
                     call errint ( nodalp, 2 )
                     call errint ( ielem, 3 )
                     call errint ( isrf1, 4 )
                     call errint ( isrf2, 5 )
                     call errint ( inunkp, 6 )
                     call errwar ( 1862, 6, 0, 0 )
                  end if
                  nkbnd = nkbnd+1
                  ih = ibit
                  if ( ih==0 ) ih = ibit2
                  kprobg(1,nkbnd) = inodp
                  kprobg(2,nkbnd) = ih
                  if ( debug ) &
                     write(irefwr,1) 'nkbnd, inodp', nkbnd, inodp
               end if

200         end do  ! k = 1, nnodes

         end do  ! j = 1, nsurfaces

      end do  ! ipart = 1, nparts

      deallocate ( nodes, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

      iinp = iinp+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05sfouter'

      end if  ! ( debug )
      call erclos ( 'prob05sfouter' )

      end
