      subroutine prob07 ( numsrf, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kprob part i with boundary elements corresponding to
!     surface
!     This subroutine is the counterpart of subroutine PROB11, &
!     if PROB11 is changed also PROB07 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numsrf, iinp, iposgrp(*)

!     iinp      i/o   last position used in array iinputkp
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     At input the last postions used for an element group
!                     are stored, at output they are updated
!     numsrf     i    number of parts where surfaces are given for generation
!                     of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, k, inpelm, j, istart, &
              ibngrp, ikprbi, nelemlc, ielem, inum
      logical present

!     i              Counting variable
!     ibngrp         boundary element group number
!     ielem          Loop variable for elements
!     ikprbi         Last position used in kprobi for this group
!     inpelm         number of points per element
!     inum           Number of surfaces
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     j              Counting variable
!     k              Counting variable
!     nelemlc        Number of active boundary elements in surface
!     present        If true the element is lying in the processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     SEPIINFSURF    Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob07' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob07'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'nsurfs, numsrf', nsurfs, numsrf
      do i = 1, numsrf

         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         inum = iinputkp(iinp+2)
         nelemlc = 0

         do j = 1, inum

            isurnr = iinputkp(iinp+2+j)

!           --- Find information of surface in kmeshn

            call sepiinfsurf ( istart, inpelm, isurnr )

            if ( debug ) &
               write(irefwr,1) 'j, ibngrp, inpelm, iinp', &
                                j, ibngrp, inpelm, iinp

            do ielem = 1, inelem

               present = .true.  ! indicates whether element is in processor

               do k = 1, inpelm
                  if ( kmeshn(istart+k)<=0 ) present = .false. !not in processor
               end do  ! k = 1, inpelm

               if ( present ) then

!              --- Element is completely in processor

                  do k = 1, inpelm
                     kprobi(ikprbi+k) = kmeshn(istart+k)
                  end do  ! k = 1, inpelm
                  ikprbi = ikprbi+inpelm
                  nelemlc = nelemlc+1

               end if  ! ( present )
               istart = istart+inpelm

            end do  ! ielem = 1, inelem

         end do  ! j = 1, inum

         if ( debug ) then
            ikprbi = iposgrp(ibngrp)
            call prinin1 ( kprobi(ikprbi+1), nelemlc, inpelm, 'kprobi' )
         end if  ! ( debug )

         iposgrp(ibngrp) = iposgrp(ibngrp)+nelemlc*inpelm

         iinp = iinp+2+inum

      end do  ! i = 1, numsrf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob07'

      end if  ! ( debug )
      call erclos ( 'prob07' )

      end
