      subroutine prob08 ( numcrv, ikprbi, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 15-03-2012 Extra parameter ivers
!        version  6.2  date 14-03-2007 New call to probcurvnumelm
!        version  6.1  date 28-06-2004 Extra parameter debug
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute length of array kprob part i with respect to boundary
!    elements corresponding to curves
!    This subroutine is the counterpart of subroutine PROB03, &
!    if PROB08 is changed also PROB03 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcrv, ikprbi, iinp, iposgrp(*)

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         In this subroutine the length with respect to an
!                         element group is added to the existing array
!     kprobloc       i    Standard SEPRAN array, containing information of the
!                         problem
!     numcrv         i    number of parts where curves are given for generation
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibngrp, inpelm

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
!     PROB08NEW      Compute length of KPROB part i with respect to curve
!                    boundary elements New input
!     PROB08OLD      Compute length of KPROB part i with respect to curve
!                    boundary elements Old input
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract constants from kmesh and kprob
!     For all groups of curves with boundary conditions do
!        Get information from iinputkp
!        For all curves in group do
!           Compute number of nodes along curve
!           Compute length of arrays and number of elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob08' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob08'
         call pr0008 ( kmeshm, ncurvs )

      end if

!     --- For all groups of curves with boundary conditions do

      do i = 1, numcrv

!     --- Get information from iinputkp

         ibngrp = iinputkp(iinp+1)
         ishape = kprobj(ibngrp,2)
         if ( kelmu>0 ) ishape = 1+kmeshu(2)
         inpelm = ishape+1

         call prob08new ( iinp, inpelm, ibngrp, ikprbi, iposgrp )

      end do  ! i = 1, numcrv

      call erclos ( 'prob08' )

      end
