      subroutine prob10sf01 ( inusol, jdgfd1, jdgfd2, jdgfd3, &
                              jtrans, norm, isurnr, ielem, nodp, j, &
                              iinp, itang, coor, &
                              inpelm, iwork, tangent, t1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-10-2006 Correction for itang<5
!        version  1.1  date 16-03-2005 New definition of itang
!        version  1.0  date 20-12-2004
!
!   copyright (c) 2004-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n with degrees of freedom with respect to surfaces
!     and transformation factors (coortr) in one specific point
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     local_transform
!     transformation
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inusol, jdgfd1, jdgfd2, jdgfd3, inpelm, &
              jtrans, isurnr, ielem, &
              nodp, j, iinp, itang, iwork(*)
      double precision norm(3,*), coor(3,*), tangent(3,*), t1(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ielem          i    Element number
!     iinp          i/o   last position used in array iinputkp
!     inpelm         i    number of points per element
!     inusol         i    Position of unknown in array USOL
!     isurnr         i    Surface sequence number
!     itang          i    Defines how the first tangential vector must be
!                         defined
!                         0: arbitrary
!                         1: By a line through points p1 and p2
!                         3: By a curve in the mesh
!     iwork          i    integer work array of length npoint
!                         if iwork(i)<0, node i must be skipped
!                         if ( iwork(i) = 0, node i is a standard node
!                         if ( iwork(i)>0, node i cooresponds to the special
!                         curve with given tangential vectors
!                         The value of iwork refers to the position in tangent
!     j              i    Loop variable in PROB10SF (used in error message)
!     jdgfd1         i    First degree of freedom
!     jdgfd2         i    Second degree of freedom
!     jdgfd3         i    Third degree of freedom
!     jtrans         i    Type of local transformation
!                         1:  standard transformation
!                         2:  symmetric transformation
!                         3:  non-symmetric transformation
!     nodp           i    Nodal point sequence number
!     norm           i    Array of length ndim * npoint containing the normal
!                         in the nodal points
!     t1            i/o   First tangential vector (muts be filled if itang=1)
!     tangent        i    Contains the tangential vectors along the given curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ideg, raise(4), imax, ifunc, k, l
      double precision locnorm(3), t2(3), t3(3), h

!     h              Help variable to store a constant temporarily
!     ideg           Degree of freedom
!     ifunc          Choice parameter with respect to function calls or
!                    in cae tang = 3: defines position in array tangent
!     imax           Entry with largest element in absolute value in vector
!     k              Counting variable
!     l              General loop variable
!     locnorm        Contains the norm in a node
!     raise          Raise-operator
!     t2             Second tangential vector
!     t3             First tangential vector, but now orthogonal to n and t2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer idamax
      double precision functr

!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCTR         User function subroutine to compute the local
!                    transformation
!     IDAMAX         Blas subroutine defining the index of the maximum
!                    element in an array
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2242   tangential vector in normal direction
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data raise /2, 3, 1, 2/
! ======================================================================
!
      call eropen ( 'prob10sf01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10sf01'
         write(irefwr,1) 'npltra, inusol, jdgfd1, jdgfd2, jdgfd3', &
                          npltra, inusol, jdgfd1, jdgfd2, jdgfd3
         write(irefwr,1) 'jtrans, itang, nodp', jtrans, itang, nodp
  1      format ( a, 1x, (10i7) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npltra = npltra+1
      ideg = inusol+jdgfd1
      if ( indprh/=0) ideg = kprobh(ideg)
      kprobn(1,npltra) = ideg
      ideg = inusol+jdgfd2
      if ( indprh/=0 ) ideg = kprobh(ideg)
      kprobn(2,npltra) = ideg
      ideg = inusol+jdgfd3
      if ( indprh/=0 ) ideg = kprobh(ideg)
      kprobn(3,npltra) = ideg
      if ( debug ) call prinin ( kprobn(1,npltra), 3, 'kprobn' )
      if ( jtrans==1 ) then

!     --- Standard situation, for the time being two
!         assumptions are made:
!         tang = 1
!         First the normal is detected

         do k = 1, 3
            locnorm(k) = norm(k,nodp)
         end do
         if ( debug ) call prinrl ( locnorm, 3, 'locnorm' )

!        --- Next the tangential vector t2 perpendicular to
!            the normal and t1

         ifunc = itang
         if ( itang==10 ) then

!        --- itang = 10, tangential vector is given along the curve
!            If node corresponds to such a vector iwork(nodp) must be positive

            ifunc = iwork(nodp)  ! ifunc defines position in tangent

         end if  ! ( itang==3 )
         if ( debug ) write(irefwr,1) 'ifunc', ifunc

         if ( ifunc==0 ) then

!        --- itang = 0, find vector not in the direction of
!            the normal vector
!            First find index largest of largest element

            imax = idamax ( 3, locnorm, 1 )

!           --- Replace this elemnt with the next one and
!               change sign

            t1(imax) = -locnorm(raise(imax))
            t1(raise(imax)) = locnorm(imax)
            t1(raise(imax+1)) = locnorm(raise(imax+1))

         else if ( itang==10 ) then

!        --- itang = 10, tangential vector is given along the curve
!            this must be the third vector, and the other 2 the normal vectors

            do k = 1, 3
               t1(k) = tangent(k,ifunc)
            end do  ! k = 1, 3

         end if  ! ( itang==0 )
         if ( debug ) then
            call prinrl ( t1, 3, 't1' )
         end if  ! ( debug )

!        --- Create vector t2 perpendicular to t1 and n

         call eloutr ( t1, locnorm, t2 )
         h = sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
         if ( h<=sqreps ) then

!        --- error: tangential vector is the same as normal
!            vector

            call errint ( isurnr, 1 )
            call errint ( ielem, 2 )
            call errint ( j, 3 )
            call errint ( nodp, 4 )
            call errsub ( 2242, 4, 0, 0 )
            go to 1000

         end if
         h = 1d0/h
         do k = 1, 3
            t2(k) = t2(k)*h
         end do
         if ( debug ) call prinrl ( t2, 3, 't2' )

         if ( ifunc>0 .and. itang==10 ) then

!        --- ifunc>0 and itang = 10, use n, t2, t1 in that sequence

            do k = 1, 3
               coortr(3*k-2,npltra) = locnorm(k)
               coortr(3*k-1,npltra) = t2(k)
               coortr(3*k  ,npltra) = t1(k)
            end do
            if ( debug ) then
               write(irefwr,1) 'npltra', npltra
               call prinrl1 ( coortr(1,npltra), 3, 3, 'coortr' )
            end if  ! ( debug )

         else if ( itang>1 .and. itang<5 ) then

!        --- 1<itang = 5, use n, t1, t2 in that sequence

            do k = 1, 3
               coortr(3*k-2,npltra) = locnorm(k)
               coortr(3*k-1,npltra) = t1(k)
               coortr(3*k  ,npltra) = t2(k)
            end do
            if ( debug ) then
               write(irefwr,1) 'npltra', npltra
               call prinrl1 ( coortr(1,npltra), 3, 3, 'coortr' )
            end if  ! ( debug )

         else

!        --- ifunc = 0
!            Finally the tangential vector t3 perpendicular to
!            the normal and t2 is found
!            For a straight surface we have t1 = t3

            call eloutr ( locnorm, t2, t3 )
            h = 1d0/sqrt(t3(1)**2+t3(2)**2+t3(3)**2)
            do k = 1, 3
               t3(k) = t3(k)*h
            end do
            if ( debug ) call prinrl ( t3, 3, 't3' )

!           --- The three vectors are stored in coortr

            do k = 1, 3
               coortr(3*k-2,npltra) = locnorm(k)
               coortr(3*k-1,npltra) = t3(k)
               coortr(3*k  ,npltra) = t2(k)
            end do
            if ( debug ) &
               call prinrl1 ( coortr(1,npltra), 3, 3, 'coortr' )

         end if  ! ( ifunc>0 )

      else

!     --- compute R and V as given by the user

         do l = 1, 9*(jtrans-1)
            ifunc = iinputkp(iinp+l+inpelm)
            if ( ifunc<0 ) then
               coortr(l,npltra) = &
                   functr ( -ifunc,coor(1,nodp), &
                            coor(2,nodp), &
                            coor(3,nodp) )
            else
               coortr(l,npltra) = rinputkp(ifunc)
            end if
         end do

      end if
      if ( jtrans<=2 .and. maxtrn==18 ) then
         do k = 1, 9
            coortr(k+9,npltra) = coortr(k,npltra)
         end do
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10sf01'

      end if
      call erclos ( 'prob10sf01' )

      end
