      subroutine prob10tr01 ( ndim, coor, anx, any, anz, iprec, inext, &
                              nodp, ifirst, ilast, h, k )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-09-2007 Remove array curves
!        version  1.0  date 23-10-2006
!
!   copyright (c) 2006-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the components anx, any and anz of the tangential
!     vector along a curve
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     tangential_vector
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, inext, nodp, ifirst, ilast, iprec, k
      double precision coor(ndim,*), anx, any, anz, h

!     anx            o    x-component of tangential vector
!     any            o    y-component of tangential vector
!     anz            o    z-component of tangential vector
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     h              o    length of tangential vector
!     ifirst         i    First curve number
!     ilast          i    Last curve number
!     inext          i    Next point in curve
!     iprec          i    Prior point in curve
!     k              i    Sequence number along curve (only used for error
!                         message)
!     ndim           i    Dimension of the space.
!     nodp           i    Nodal point sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xp, yp, zp, xn, yn, zn

!     xn             next x coordinate
!     xp             previous x coordinate
!     yn             next y coordinate
!     yp             previous y coordinate
!     zn             next z coordinate
!     zp             previous z coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     339   Length of line between two points is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10tr01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10tr01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute nx and ny from tangential vector

      anz = 0d0
      if ( iprec>0 ) then
         xp = coor(1,iprec)
         yp = coor(2,iprec)
         if ( ndim==3 ) zp = coor(3,iprec)
      else
         xp = coor(1,nodp)
         yp = coor(2,nodp)
         if ( ndim==3 ) zp = coor(3,nodp)
      end if

      if ( inext>0 ) then
         xn = coor(1,inext)
         yn = coor(2,inext)
         if ( ndim==3 ) zn = coor(3,inext)
      else
         xn = coor(1,nodp)
         yn = coor(2,nodp)
         if ( ndim==3 ) zn = coor(3,nodp)
      end if

      if ( ndim==3 ) then
         anx = xn-xp
         any = yn-yp
         anz = zn-zp
         h = sqrt(anx*anx+any*any+anz*anz)
      else
         anx = yn-yp
         any = xn-xp
         h = sqrt(anx*anx+any*any)
      end if
      if ( h<=0 ) then
         call errint ( nodp, 1 )
         call errint ( iprec, 2 )
         call errint ( inext, 3 )
         call errint ( ifirst, 4 )
         call errint ( ilast, 5 )
         call errint ( k, 6 )
         call errsub ( 339, 6, 0, 0 )
      end if

1000  call erclos ( 'prob10tr01' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10tr01'

      end if  ! ( debug )

      end
