      subroutine prob14 ( leniinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.25 date 13-10-2013 Extension with leniinpfp
!        version  5.24 date 06-06-2012 Extension with periodical boundary conds
!        version  5.23 date 15-03-2012 New type of input
!
!   copyright (c) 1987-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting positions of various parts in array iinputkp
!     corresponding to subroutine PROBDF
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: leniinp

!     leniinp    o    Last position used of array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, itype, ibngrp, numcrv, numsrf, &
              k, ifirst, numcurvs, jtrans, &
              ielgrp, iinp1, iinp2, iinp3, iinp4, iinp5, &
              iinp6, iinp7, indtvc, inpelm, inunkp, &
              iinp8, iinp9, iinp10, itang, &
              numlocobst, numloccont, iinp11, iinp12, &
              ncurvsact, iinp13, numpart, itrans, leniinpfp, &
              iinp14, iinp15, numobstlc

!     ibngrp         Actual boundary element group number
!     ielgrp         element group number
!     ifirst         First curve (surface) number
!     iinp           last position used in array iinputkp
!     iinp1          first pos. iinputkp definition type numbers
!     iinp10         first pos. iinputkp definition global group renumbering
!     iinp11         first pos. iinputkp definition fictitious group type numbers
!     iinp12         first pos. iinputkp definition fictitious group elements
!     iinp13         first pos. iinputkp definition skip elements
!     iinp14         first pos. iinputkp definition levelset functions
!     iinp15         first pos. iinputkp definition periodical boundary conditions
!     iinp2          first pos. iinputkp definition boundary group type numbers
!     iinp3          first pos. iinputkp definition boundary elements
!     iinp4          first pos. iinputkp definition type of essential boundary
!                                      conditions
!     iinp5          first pos. iinputkp definition position of essential boundary
!                                      conditions
!     iinp6          first pos. iinputkp definition boundary conditions of type
!                                      u constant
!     iinp7          first pos. iinputkp definition renumbering
!     iinp8          first pos. iinputkp definition global group type numbers
!     iinp9          first pos. iinputkp definition global group elements
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in first point
!     itang          Defines how the first tangential vector must be
!                    defined
!                    0: arbitrary
!                    1: By a line through points p1 and p2
!     itrans         Defines type of local transformation
!     itype          Type number of element
!     jtrans         Type of local transformation
!     k              Counting variable
!     leniinpfp      Length of first part of iinputkp for probdf
!     ncurvsact      Actual number of curves to be used
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numcurvs       Number of curves
!     numloccont     Number of contacts with local transforms
!     numlocobst     Number of obstacles with local transforms
!     numobstlc      Local number of obstacles
!     numpart        Number of parts read
!     numsrf         number of parts where surfaces are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROB14BELMS    Raise iinp for boundary elements
!     PROB14ESS      Raise iinp for essential boundary conditions
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob14' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob14'
         call prinin ( iinputkp,70, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      ntype  = iinputkp(1)
      numnat = iinputkp(2)
      nglgrp = iinputkp(29)
      nfictgrp = iinputkp(36)
      leniinpfp = 60
      iinp = leniinpfp+1
      iinp1 = iinp
      if ( debug ) write(irefwr,1) 'iinp', iinp

!     --- Loop over element groups

      do ielgrp  =  1,nelgrp
         itype  =  iinputkp(iinp)
         if ( itype>0 ) then
            do k  =  2,ntype
               itype = max (itype,iinputkp(iinp+k-1))
            end do
         end if
         iinp = iinp+ntype
         if ( itype>0 .and. itype<100 ) then
            inunkp = iinputkp(iinp)
            indtvc = iinputkp(iinp+1)
            nphys  = iinputkp(iinp+2)
            iinp = iinp+3
            inpelm = Topgrp(ielgrp)%inpelm
            if ( inunkp==0 ) iinp = iinp+inpelm
            if ( indtvc>0 ) iinp = iinp+indtvc*inpelm
            if ( nphys>0 ) iinp = iinp+nphys*inpelm
         else if ( itype>10000 ) then
            indtvc = iinputkp(iinp)
            iinp = iinp + 1
            if ( indtvc>0 ) iinp = iinp+indtvc*4
         else if ( itype==-1 ) then
            if ( iinputkp(15)>=8 ) then
               iinp = iinputkp(iinp)*2+iinp+1
            else if ( iinputkp(15)>=1 ) then
               iinp = iinp+1
            end if
         end if
      end do
      if ( debug ) write(irefwr,1) 'iinp1, iinp', iinp1, iinp

!     --- Boundary element groups

      iinp2  =  iinp
      do ibngrp = 1,numnat

         itype  =  0
         do k  =  1,ntype
            itype = max (itype,iinputkp(iinp+k-1))
         end do
         iinp = iinp+ntype
         if ( itype<100 ) then
            inpelm = iinputkp(iinp)
            iinp = iinp+1
            if ( itype>0 ) then
               inunkp = iinputkp(iinp)
               indtvc = iinputkp(iinp+1)
               nphys  = iinputkp(iinp+2)
               iinp = iinp+3
               if ( inunkp==0 ) iinp = iinp+inpelm
               if ( indtvc>0 ) iinp = iinp+indtvc*inpelm
               if ( nphys>0 ) iinp = iinp+nphys*inpelm
            end if
         else if ( itype>10000 ) then
            indtvc = iinputkp(iinp)
            iinp = iinp + 1
            if ( indtvc>0 ) iinp = iinp+indtvc*4
         end if

      end do
      if ( debug ) write(irefwr,1) 'iinp2, iinp', iinp2, iinp

!     --- Boundary elements

      call prob14belms ( iinp, iinp3 )

!     --- Global element groups

      iinp8  =  iinp
      do ibngrp = 1, nglgrp
         iinp = iinp+2
      end do

!     --- Global elements

      iinp9 = iinp
      numcrv = iinputkp(30)
      numsrf = iinputkp(31)+iinputkp(32)
      iinp = iinp+numcrv*4+numsrf*3+iinputkp(33)
      iinp10 = iinp
      if ( nglgrp>0 ) iinp = iinp+iinputkp(iinp)+1

!     --- Fictitious element groups

      iinp11  =  iinp
      do ibngrp = 1, nfictgrp
         iinp = iinp+2
      end do

!     --- Fictitious elements

      iinp12 = iinp
      numcrv = iinputkp(37)
      numsrf = iinputkp(38)
      numobstlc = iinputkp(40)
      iinp = iinp+numcrv*7+numsrf*7+numobstlc*4

!     --- Essential boundary conditions

      iinp4  =  iinp
      if ( debug ) write(irefwr,1) 'iinp4, iinp', iinp4, iinp
      call prob14ess ( iinputkp, iinp )
      iinp5 = iinp
      if ( debug ) write(irefwr,1) 'iinp5, iinp', iinp5, iinp

!     --- local transform

      if ( iinputkp(9)/=0 ) then
         numcrv = iinputkp(iinp)
         numsrf = iinputkp(iinp+1)
         iinp = iinp+2

!        --- local transforms on curves

         if ( ndim==2 ) then
            do k = 1, numcrv
               jtrans = iinputkp(iinp+5)
               iinp = iinp+6
               if ( jtrans==2 ) then
                  iinp = iinp+4
               else if ( jtrans==3 ) then
                  iinp = iinp+8
               end if
            end do
         else
            do k = 1, numcrv
               iinp = iinp+7
               if ( iinputkp(iinp)==2 ) then
                  iinp = iinp+9
               else if ( iinputkp(iinp)==3 ) then
                  iinp = iinp+16
               end if
            end do
         end if

!        --- local transforms on surfaces

         do k = 1, numsrf
            ifirst = iinputkp(iinp+3)
            itrans = iinputkp(iinp+5)
            itang = iinputkp(iinp+7)
            numcurvs = iinputkp(iinp+8)
            ncurvsact = 0
            if ( mod(iinputkp(iinp+6),10)>1 ) ncurvsact = -1
            iinp = iinp + kmeshn(ifirst) + 8
            if ( itang>0 .and. itang<10 ) then

!           --- 1<=itang<=9, 2 extra positions

               iinp = iinp+2

            else if ( itang>9 ) then

!           --- itang>=10, numcurvs+1 extra positions

               numcurvs = iinputkp(iinp)
               iinp = iinp+1+numcurvs

            end if  ! ( itang>0 .and. itang<10 )
            if ( itrans==2 ) then
               iinp = iinp+9
            else if ( itrans==3 ) then
               iinp = iinp+16
            end if
            if ( ncurvsact==-1 ) then

!           --- ncurvsact = -1, i.e. there are curves to be excluded or included

               ncurvsact = iinputkp(iinp)
               iinp = iinp + ncurvsact+1

            end if

         end do

!        --- local transforms on obstacles and contacts

         numlocobst = iinputkp(18)
         numloccont = iinputkp(21)
         iinp = iinp + 6*(numlocobst+numloccont)

      end if
      iinp6 = iinp
      if ( debug ) write(irefwr,1) 'iinp6, iinp', iinp6, iinp

      if ( iinputkp(13)/=0 ) then

!     --- iinputkp(13)>0, i.e. there are boundary conditions of
!                         the type u equals unknown constant

         numcrv = iinputkp(iinp)
         numsrf = iinputkp(iinp+1)
         iinp = iinp+2
         if ( numcrv>0 ) iinp = iinp+numcrv*4
         if ( numsrf>0 ) then

!        --- numsrf>0: surfaces

            do k = 1, numsrf
               inpelm = iinputkp(iinp+3)
               iinp = iinp+4+inpelm
            end do

         end if

      end if

      iinp7 = iinp
      if ( debug ) write(irefwr,1) 'iinp7, iinp', iinp7, iinp

      iinp = iinp + abs(iinputkp(14))
      iinp13 = iinp
      if ( debug ) write(irefwr,1) 'iinp13, iinp', iinp13, iinp

      intarprb(20) = 0
      if ( iinputkp(44)/=0 ) then

!     --- iinputkp(44)>0, i.e. there are elements to be skipped

         numpart = iinputkp(iinp)
         iinp = iinp+1+5*numpart
         intarprb(20) = 1

      end if
      iinp14 = iinp
      if ( debug ) write(irefwr,1) 'iinp14, iinp', iinp14, iinp
      if ( iinputkp(48)/=0 ) then

!     --- iinputkp(48)>0, i.e. level set functions

         iinp = iinp+iinputkp(48)

      end if
      iinp15 = iinp
      if ( debug ) write(irefwr,1) 'iinp15', iinp
      if ( iinputkp(52)/=0 ) then

!     --- iinputkp(52)>0, i.e. periodical boundary conditions

         iinp = iinp+iinputkp(iinp)

      end if
      if ( debug ) write(irefwr,1) 'iinp', iinp

      leniinp = iinp - 1
      intarprb(1) = iinp1
      intarprb(2) = iinp2
      intarprb(3) = iinp3
      intarprb(4) = iinp4
      intarprb(5) = iinp5
      intarprb(6) = iinp6
      intarprb(7) = iinp7
      intarprb(8) = iinp8
      intarprb(9) = iinp9
      intarprb(10) = iinp10
      intarprb(11:14) = 0
      intarprb(15) = iinp11
      intarprb(16) = iinp12
      intarprb(17) = iinp13
      intarprb(18) = iinp14
      intarprb(19) = iinp15

1000  if ( debug ) then
         call prinin ( iinputkp, leniinp, 'iinputkp' )
         call prinin ( intarprb, 20, 'intarprb' )
         write(irefwr,*) 'End prob14'
      end if  ! ( debug )
      call erclos ( 'prob14' )

      end
