      subroutine prob14esscurv ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise iinp for essential boundary conditions (subroutine PROBDF)
!     Only curve part
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     initialization
!     essential_boundary_condition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array corresponding to subroutine probdf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcrv, k, inum

!     inum           Number of curves found
!     k              Counting variable
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob14esscurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob14esscurv'
         write(irefwr,1) 'iinp', iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numcrv = iinput(7)
      if ( debug ) then
         write(irefwr,1) 'numcrv', numcrv
         call prinin ( iinput(iinp), 4*numcrv, 'iinput' )
      end if  ! ( debug )

!     --- new version

      do k = 1, numcrv
         inum = iinput(iinp+2)
         if ( debug ) write(irefwr,1) 'k, inum', k, inum
         if ( inum<0 ) then
            iinp = iinp+3
         else
            iinp = iinp+inum+3
         end if  ! ( inum<0 )
      end do  ! k = 1, numcrv

      if ( debug ) write(irefwr,1) 'iinp/1a', iinp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob14esscurv'

      end if  ! ( debug )
      call erclos ( 'prob14esscurv' )

      end
