      subroutine prob14essvolm ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise iinp for essential boundary conditions (subroutine PROBDF)
!     Only volume part
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     initialization
!     essential_boundary_condition
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array corresponding to subroutine probdf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numvlm, k, ncurvsact, ifirst, inpelm, istart

!     ifirst         First volume number
!     inpelm         number of points per element
!     istart         First position used at a part
!     k              Counting variable
!     ncurvsact      Actual number of curves to be used
!     numvlm         number of parts where volumes are given for
!                    generation of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob14essvolm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob14essvolm'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numvlm = iinput(39)
      if ( numvlm==0 ) go to 1000

      if ( debug ) then

         write(irefwr,1) 'numvlm', numvlm
         call prinin ( iinput(iinp), 4*numvlm, 'iinput' )

      end if  ! ( debug )

      do k = 1, numvlm

         ifirst = iinput(iinp+1)
         if ( debug ) write(irefwr,1) 'iinp, ifirst', iinp, ifirst

         if ( debug ) &
            call prinin ( iinput(iinp), iinput(iinp+1)+3+ &
                          iinput(iinp+2), 'iinput' )
         iinp = iinp + iinput(iinp+1) + iinput(iinp+2) + 3


!        --- skip boundary

         if ( debug ) then
            write(irefwr,1) 'iinp/vol_2', iinp
            call prinin ( iinput(iinp), 2, 'iinput' )
         end if  ! ( debug )
         ncurvsact = iinput(iinp+1)
         iinp = iinp+2+ncurvsact

         if ( debug ) write(irefwr,1) 'iinp/vol_3', iinp

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob14essvolm'

      end if  ! ( debug )
      call erclos ( 'prob14essvolm' )

      end
