      subroutine prob21 ( n, iobst, jstep, kmeshm, icurvsobs, ncurvs, &
                          iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-08-1999 New definition icurvsobs
!        version  1.0  date 29-09-1997
!
!   copyright (c) 1997-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the number of nodes with local transformations at
!   obstacle iobst and store the node numbers in iwork
!   This subroutine is the counterpart of subroutine PROB22, &
!   if PROB22 is changed also PROB19 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     local_transform
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jstep, n, iobst, kmeshm(*), icurvsobs(2,*), ncurvs, &
              iwork(*)

!     icurvsobs  i    Array containing information of he active obstacles.
!                     The array corresponds to all curves separately
!                     icurvsobs(1,i) = 1 means that the corresponding point is
!                     active; 0 means not active
!     iobst      i    Obstacle sequence number
!     iwork      o    Nodal point numbers with local transformations are
!                     stored in iwork positions 1 to n
!     jstep      i    Indicates which points must be taken into account
!                     and which points must be skipped (See PG)
!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     n          o    Number of points with local transformations at the
!                     obstacle
!     ncurvs     i    Number of curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, k, i1, i2, icurnr

!     i1         First point of curve in icurvsobs
!     i2         Last point of curve in icurvsobs
!     icurnr     Curve sequence number
!     istar3  Starting address of KMESHM part 3 in KMESH part M
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     428   The option jstep = ... has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters
!    n := 0
!    For all curves do
!       For all nodes along the curve do
!          if ( node is at obstacle iobst ) then
!             n := n + 1
!             iwork(n) := node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      istar3 = kmeshm(2)-1
      n = 0
      do icurnr = 1, ncurvs
         i1 = kmeshm(icurnr+4)
         i2 = kmeshm(icurnr+5)-1

         if ( jstep.eq.1 ) then

!        --- all nodes must be used

            do k = i1, i2-1
               if ( icurvsobs(1,k).eq.iobst ) n = n+1
               iwork(n) = kmeshm(istar3+k)
            end do

         else

!        --- not yet implemented

            call eropen ( 'prob21' )
            call errint ( jstep, 1 )
            call errsub ( 428, 1, 0, 0 )
            call erclos ( 'prob21' )

         end if

      end do

      end
