      subroutine prob24 ( natconct, ikprbi, iinp, iposgrp )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date   06-05-1998 Remove error message
!                                         Extra parameter iposgrp
!        version 1.1    date   12-12-1997 Update because of new definition
!                                         of kprobw
!        version 1.0    date   07-10-1997
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute length of array kprob part i with boundary elements
!    corresponding to contact surfaces
!    This subroutine is the counterpart of subroutine PROB25, &
!    if PROB24 is changed also PROB25 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!   problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natconct, ikprbi, iinp, iposgrp(*)

!     iinp       i    last position used in array iinputkp
!     ikprbi    i/o   last position used in array kprob part i
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     In this subroutine the length with respect to an
!                     element group is added to the existing array
!     natconct   i    Number of contacts where natural boundary conditions
!                     are given
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iconct, ibngrp, inpelm, numconct, isurnr, nelcon

!     i          Counting variable
!     ibngrp     Boundary element group number read
!     iconct     Sequence number of contact
!     inpelm     number of points per element
!     isurnr     Surface sequence number
!     nelcon     Number of contact elements
!     numconct   Maximum number of contacts in problem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all natural boundary contact elements do
!       ibngrp := boundary element group
!       iconct := contact sequence number
!       inpelm = number of nodes per element
!       nelcon = number of contact elements
!       update kprobj
!       raise ikprbi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Initializations

      numconct = kprobloc(48)

!     --- For all natural boundary contact elements do

      do i = 1, natconct
         ibngrp = iinputkp(iinp+1)
         iconct = iinputkp(iinp+3)
         iinp = iinp+3
         inpelm = 0
         if ( iconct<=numconct ) then

            isurnr = kprobwp1(iconct,3)
            inpelm = kmeshn(isurnr)
            nelcon = kprobwp1(iconct,1)
            if ( nelcon>0 ) then

!           --- Contact elements have been found

               kprobb(ibngrp) = kprobb(ibngrp)+nelcon

               ikprbi = ikprbi+nelcon*inpelm
               iposgrp(ibngrp) = iposgrp(ibngrp)+nelcon*inpelm

            end if

         end if

      end do
      end
