      subroutine probfict ( newelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 12-02-2003 Replace ini060 by ini060text
!        version  2.0  date 03-04-2001 Include parameter newelm (J de Hart)
!        version  1.1  date 25-08-2000 Minor error correction
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the fluid elements corresponding to each lambda point in array
!     kprobfict
! **********************************************************************
!
!                       KEYWORDS
!
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: newelm

!     newelm         o    Parameter indicating if different fluid elements are
!                         found that contain lambda points. Hence, a new call
!                         to commat should be made.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idummy(1)
      integer, allocatable :: iwork(:), iwork1(:), iwork2(:)

!     idummy         Dummy parameter
!     iwork          integer work array
!     iwork1         integer work array
!     iwork2         integer work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSORTELM     Fill array kmesh part ac with information of in which part
!                    of the region an element is positioned
!     PROBFICT02     Find fluid elements in which the fictitious unknowns are
!                    stored (2D)
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPIACTIVATE   Reactivate existing array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Sort the nodal points of the mesh in a rectangular grid
!     Get some constants
!     Activate arrays
!     Define work space
!     Define starting addresses of arrays
!     Fill fluid elements in kprobfict
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probfict' )
      if ( ierror/=0 ) go to 1000

      kmesh => kmeshloc
      call sepfillkmesh ( kmesh )

!     --- Sort the nodal points of the mesh in a rectangular grid

      if ( kelmac==0 ) call mshsortelm

!     --- Define work space

      allocate ( iwork(npoint), iwork1(nelem), iwork2(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nelem+npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill fluid elements in kprobfict

      call probfict02 ( iwork, iwork1, iwork2, newelm )

      deallocate ( iwork, iwork1, iwork2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'probfict' )
      end subroutine probfict
