      subroutine probloctrser ( ih )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-10-2013 Remove ipkmsh
!        version  1.2  date 09-03-2013 Set pointers
!        version  1.1  date 26-11-2007 Check only in case of simalex
!
!   copyright (c) 2007-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute local transformations and store in array kprobn
!     Serial computation
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     local_transform
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ih

!     ih             i    Help variable to store the value of kprobloc(2)
!                         temporarily
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      integer, allocatable :: mappinglc(:)

!     k              Counting variable
!     mappinglc      contains global node numbers for old mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIMAP2        Compute mapping from global to local array
!                    Points that are not present in local array get the value 0
!     PRININ         print 1d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
!     PROB10         Fill KPROB part n with information concerning local
!                    transforms
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probloctrser' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probloctrser'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobloc(2) = ih
      maxtrn = kprobloc(45)

      if ( debug )write(irefwr,1) 'maxtrn, npltra, ndim', maxtrn, npltra, ndim

!     --- Create array

      allocate ( kp(iseqkp)%kprobsub(iprob)%kprobn(ndim,npltra) )
      allocate ( kp(iseqkp)%kprobsub(iprob)%coortr(maxtrn,npltra) )
      kprobn => kprobpart%kprobn
      coortr => kprobpart%coortr

!     --- Activate array

      allocate ( mappinglc(npointglob), stat = error )
      if ( error/=0 ) call eralloc ( error, npointglob, 'mappinglc' )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case, use global arrays coor, kmesl, kmeshm and kmeshn
!         Also make mapping from global to local

         if ( debug ) then
            call prinin ( kmeshpar(2), 5, 'kmesh_par' )
         end if  ! ( debug )

      end if  ! ( parallel )

      if ( parallel ) then

!     --- Parallel case, use global arrays coor, kmesl, kmeshm and kmeshn

         call inimap2 ( npointglob, mappinglc, npoint, mappingpar )
         if ( .false. .and. debug ) then
            call prininnonzero ( mappinglc, npointglob, 'map' )
            call prininnonzero ( mappingpar, npoint, 'mappinglc' )
         end if  ! ( debug )

      end if  ! ( parallel )

!     --- Fill kprob part n

      call prob10 ( mappinglc )
      kprobloc(2) = 101

      deallocate ( mappinglc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'mappinglc' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probloctrser'

      end if  ! ( debug )
      call erclos ( 'probloctrser' )

      end subroutine probloctrser
