      subroutine probnatblevs2 ( natlevset, iinp, &
                                 nmeshc, iposgrp, ikprbi, iobstfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-05-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part i with elements for
!     natural boundary conditions defined on zero levelset points
!     2 dimensions only
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     natural
!     problem_definition
!     level_set
!     natural_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natlevset, iinp, &
              nmeshc(*), iposgrp(*), ikprbi, iobstfaces(*)

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iobstfaces     i    contains edge sequence numbers of boundary
!                         of levelset
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!     natlevset      i    number of parts where zero level set elements
!                         are defined
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer igrp, ibngrp, inum, i, nnode_edge, numbounelms, &
              jstart, numedges, iedge, kstart

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     iedge          Edge sequence number
!     igrp           Boundary element group number read
!     inum           Number of numbers read in a record
!     jstart         First position used at a part
!     kstart         First position used at a part
!     nnode_edge     number of nodes corresponding to edge
!     numbounelms    Number of elements that have all points on the
!                    boundary of the obstacle or a set of curves
!     numedges       Number of edges in boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnatblevs2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnatblevs2'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinputkp(iinp+1), 6, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      do igrp = 1, natlevset

         ibngrp = iinputkp(iinp+1)         ! boundary group
         inum = max(1,iinputkp(iinp+5) )   ! number of curves
         iinp = iinp+6+inum
         ikprbi = iposgrp(ibngrp)        ! starting address in kprob part i
         if ( debug ) &
              write(irefwr,1) 'igrp, ibngrp, ikprbi, inum, iinp', &
                               igrp, ibngrp, ikprbi, inum, iinp

!        --- Find all edges with all points on boundary

         kstart = iobstfaces(igrp)
         numedges = iobstfaces(kstart)
         numbounelms = 0
         do i = 1, numedges
            iedge = iobstfaces(kstart+i)       ! edge number
            jstart = nmeshc(iedge)
            nnode_edge = nmeshc(jstart)

!           --- Fill kprob part i

            numbounelms = numbounelms+1
            kprobi(ikprbi+1:ikprbi+nnode_edge) = &
                nmeshc(jstart+1:jstart+nnode_edge)
            ikprbi = ikprbi+nnode_edge

            if ( debug ) then
               write(irefwr,1) 'iedge', i
               call prinin ( nmeshc(jstart+1), nnode_edge, 'nodes' )
            end if  ! ( debug )

         end do  ! i = 1, numedges

         if ( debug ) then
            call prinin1 ( kprobi(iposgrp(ibngrp)+1), numbounelms, &
                           nnode_edge, 'kprob nat elements' )
            write(irefwr,1) 'numbounelms, nnode_edge', &
                             numbounelms, nnode_edge

         end if  ! ( debug )

!        --- Raise iposgrp

         iposgrp(ibngrp) = iposgrp(ibngrp)+numbounelms*nnode_edge

      end do  ! igrp = 1, natlevset

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobb, kprobloc(12), 'kprob part b' )
         write(irefwr,*) 'End probnatblevs2'

      end if  ! ( debug )
      call erclos ( 'probnatblevs2' )

      end
