      subroutine probparglob
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 03-05-2015 New parameter list
!        version  1.1  date 12-08-2014 Extension with kprobpar part 5
!        version  1.0  date 26-03-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write some quantities from local node to node 1
!     Parallel case
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenkprobhsend, i

!     i              General loop variable
!     lenkprobhsend  Length of array kprobhsend
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBCASTMPI    Wrapper for MPI_BCAST
!     PRINKPROBPAR1  Print array kprob par
!     PROBPARGLOB1   Compute global quantities with respect to problem
!                    definition Parallel case (node 1)
!     PROBPARGLOB2   Compute global quantities with respect to problem
!                    definition Parallel case (nodes > 1)
!     PROBPARGLOB6   copy data from kprobhsend to kprobhglob on host
!     PROBPARGLOB7   receive data from kprobhsend in node and
!                    store in kprobhglob
!     PROBPARGLOB8   Fill array nrusollocs with largest degree of freedom
!                    per cpu
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( blocknr==1 ) then
!        create and clear array numunknowns(1:npoint_global)
!        Fill array numunknowns for all local nodes
!        For blocks := 2, number_of_blocks do
!           Read mapping array as well as kprobf and kprobh from block
!           Fill array numunknowns for all local nodes
!        Compute nusol_glob and fill array kprobf_glob if necessary
!        if ( kprobh must be filled ) then
!           create and clear array kprobh_glob
!           For all blocks do
!              Fill array kprobh from local kprobf/h and local mapping
!     else
!        Send dimension quantities as well as mapping and kprobf/h to block 1
!     end if
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob'
         write(irefwr,1) 'inodenr', inodenr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipstkprobpar = kprobloc(65)
      kprobpar => kprobpart%kprobpar
      ndtvec = kprobloc(6)

      allocate ( kprobpart%kprobfglobal(0:ndtvec) )
      kprobfglobal => kprobpart%kprobfglobal
      do i = 0, ndtvec
         nullify ( kprobfglobal(i)%arr )
      end do  ! i = 0, ndtvec

      if ( inodenr==1 ) then

!     --- Node number 1, receive data from other nodes and create arrays

         lenkprobhsend = 2*iacnodes*npointglob*nunkp  ! overestimation

         allocate ( kprobhsend(lenkprobhsend), stat = error )
         if ( error/=0 ) &
             call eralloc ( error, lenkprobhsend, 'kprobhsend' )
         if ( ierror/=0 ) go to 1000

         call probparglob1 ( lenkprobhsend )

      else

!     --- Node number>1, send data from other nodes

         call probparglob2

      end if  ! ( inodenr==1 )

!     --- Send first 2 positions of kprobpar from host to other nodes

      call inibcastmpi ( kprobpar, 2, SPINT )

!     --- Fill mapping from local to global degrees of freedom per node

      if ( associated(kprobpart%kprobhglob) ) &
           deallocate(kprobpart%kprobhglob)
      allocate(kprobpart%kprobhglob(nrusol))
      kprobhglob => kprobpart%kprobhglob

      if ( inodenr==1 ) then

!     --- Node number 1, copy data from kprobhsend to kprobhglob

         call probparglob6

      else

!     --- Node number >1, receive data from kprobhsend and copy to kprobhglob

         call probparglob7

      end if  ! ( inodenr==1 )

!     --- Compute maximum degree of freedom per cpu and store in
!         nrusollocs of length iacnodes

      if ( associated(kprobpart%nrusollocs) ) &
           deallocate(kprobpart%nrusollocs)
      allocate(kprobpart%nrusollocs(iacnodes))
      nrusollocs => kprobpart%nrusollocs

      if ( inodenr==1 ) then

!     --- Fill array nrusollocs

         call probparglob8
         deallocate ( kprobhsend )

      end if  ! ( inodenr==1 )

!     --- Send nrusollocs  from host to other nodes

      call inibcastmpi ( nrusollocs, iacnodes, SPINT )

1000  if ( debug ) then

!     --- Debug information

         call prinkprobpar1
         write(irefwr,*) 'End probparglob'

      end if  ! ( debug )
      call erclos ( 'probparglob' )

      end
