      subroutine probparglob8
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-08-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nrusollocs with largest degree of freedom per cpu
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
!     petsc
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, n

!     i              General loop variable
!     istart         First position used at a part
!     n              number of degrees of freedom on cpu
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob8' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob8'
         write(irefwr,1) 'iacnodes', iacnodes
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array nrusollocs

      do i = 1, iacnodes
         istart = kprobhsend(i)
         n = kprobhsend(istart)
         nrusollocs(i) = maxval(kprobhsend(istart+1:istart+n))
      end do  ! i = 1, iacnodes

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nrusollocs, iacnodes, 'nrusollocs' )
         write(irefwr,*) 'End probparglob8'

      end if  ! ( debug )
      call erclos ( 'probparglob8' )

      end subroutine probparglob8
