      subroutine probparkprobh
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rearrange array map, recompute nrusol and fill array kprobh
!     New numbering is used
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: globnodesp(:)

!     globnodesp     Second part of array globnodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROBPARKPROBH1 Define interface unknowns and create new kprobh
!     PROBREORSEND   Fill new array sendinfo with numbering corresponding to
!                    dirichlet-dirichlet ordering of unknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparkprobh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparkprobh'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( associated(kprobpart%ninterfdeg) ) &
           deallocate(kprobpart%ninterfdeg)
      allocate(kprobpart%ninterfdeg(iacnodes+1))
      ninterfdeg => kprobpart%ninterfdeg

!     --- Create new array kprobh for Dirichlet-Dirichlet coupling

      if ( associated(kprobpart%kprobhdd) ) &
           deallocate(kprobpart%kprobhdd, &
                      kprobpart%kprobhddinv )
      allocate(kprobpart%kprobhdd(nusol), &
               kprobpart%kprobhddinv(nusol) )
      kprobhdd => kprobpart%kprobhdd
      kprobhddinv => kprobpart%kprobhddinv
      kprobpar(4) = nusol

!     --- compute starting addresses

      globnodesp => globnodes(iacnodes+1:iacnodes+npointglob)

!     --- Copy old mapping back into mappingpar
!         This is necessary since in case of more problems the mappingpar has
!         been changed by the statements below

      mappingpar(1:npoint) = mappingparcp(1:npoint)

!     --- Renumber mappingpar to new numbering

      mappingpar(1:npoint) = globnodesp(mappingpar(1:npoint))

      if ( debug ) then
         call prinin ( mappingpar, npoint, 'mappingpar' )
         call prinin ( kprobh, nusol, 'kprobh')
         call prinin ( kprobhinv, nusol, 'kprobhinv')
      end if  ! ( debug )

!     --- Copy kprobh into kprobhdd

      kprobhdd = kprobh
      kprobhddinv = kprobhinv

!     --- Define interface unknowns and create new kprobh

      if ( inodenr>1 ) then

!     --- slave

         call probparkprobh1

!        --- Reorder sendinfo for new numbering (store in parinfo)

         call probreorsend

      else

!     --- host: clear ninterfdeg

         ninterfdeg = 0

      end if  ! ( inodenr>1 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobhdd, nusol, 'kprobhdd')
         call prinin ( kprobhinv, nusol, 'kprobhinv')
         write(irefwr,1) 'nrusol, ninterf', kprobloc(66:67)
         write(irefwr,*) 'End probparkprobh'

      end if  ! ( debug )
      call erclos ( 'probparkprobh' )

      end subroutine probparkprobh
