      subroutine probskippoint ( iskippoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-02-2001 Extension with debugging
!        version  1.0  date 15-12-2000
!
!   copyright (c) 2000-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Detect which points are completely in the set of elements that are skipped
!    and store these nodes in array skippoints
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iskippoint(*)

!     iskippoint     o    Array of length nskip+1 containing the points to
!                         be skipped
!                         Position 1 contains nskip (number of points to be
!                                    skipped
!                         The next positions contain the node numbers to be
!                         skipped
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inode, nskip, ielem

!     ielem          Element number
!     inode          Node sequence number
!     j              Counting variable
!     nskip          Number of points that must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nskip := 0
!     For all nodes inode do
!        find all elements corresponding to node
!           if ( all elements must be skipped ) then
!              nskip := nskip+1
!              iskippoint(nskip+1) := inode
!     iskippoint(1) := nskip
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- nskip := 0

      nskip = 0

!     --- For all nodes do

      do inode = 1, npoint

!     --- find all elements corresponding to node
!           if ( all elements must be skipped ) then
!              nskip := nskip+1
!              iskippoint(nskip+1) := inode

         do j = kmshd1(inode)+1, kmshd1(inode+1)
            ielem = kmshd2(j)
            if ( kprobaa(ielem)<=0 ) go to 100
         end do
         nskip = nskip+1
         iskippoint(nskip+1) = inode

100   end do

      iskippoint(1) = nskip
      if ( debug_probskippoint ) then

!     --- debug on, print information

         call prinin ( iskippoint(2), nskip, 'Points that are skipped' )

      end if

      end
