      subroutine probstruc ( meshin, meshout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 15-12-2017 New storage of intmat
!        version  1.1  date 04-06-2014 Allow -102 in intmat(2)
!        version  1.0  date 31-05-2013
!
!   copyright (c) 2013-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create problem definition and matrix structure for new mesh
!     It is supposed that the old mesh exists
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     matrix_structure
!     refine
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulecons
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, meshout

!     meshin         i    Sequence number of the input mesh
!     meshout        i    Sequence number of output mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum(1), ichprb, iseqprob
      logical :: allociin

!     allociin       If true iinputtmp and rinputtmp have been allocated
!     ichprb         Choice parameter for subroutine PROBDF
!     idum           General dummy variable
!     iseqprob       Sequence number of problem input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probstruc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probstruc'
         write(irefwr,1) 'meshin, meshout', meshin, meshout
         call prinin ( intmat2d(1,meshin), 5, 'intmat_in' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Remove kprob2d corresponding to meshout if it already exists
!         The part with respect to iinput/rinput is excluded

      kprob => kprob2d(meshout)
      ichprb = 202  ! Default problem definition
      allociin = .false.
      if ( kprob>0 .and. kprob<=lastprob ) then

!     --- Use problem definition that is stored in iinput/rinput

         ichprb = 1
         iinputkp => kp(kprob)%iinputkp
         rinputkp => kp(kprob)%rinputkp
         length = size(iinputkp)
         allocate ( iinputtmp(length) )
         iinputtmp = iinputkp
         length = size(rinputkp)
         allocate ( rinputtmp(length) )
         rinputtmp = rinputkp
         allociin = .true.

      end if  ! ( kprob>0 .and. kprob<=lastprob )

!     --- Fill kprob2d corresponding to meshout

      iseqprob = 1
      kmesh => kmesh2d(meshout)
      kprob => kprob2d(meshout)
      call sepfillkmesh ( kmesh )

      call probdfext ( ichprb, iseqprob )

      if ( allociin ) deallocate ( iinputtmp, rinputtmp )

!     --- Fill array intmat for new mesh
!         Remark:  iread = 1, might be incorrect

      if ( intmat2d(1,meshin)>0 .and. intmat2d(1,meshin)<=lastintmat ) then

!     --- intmat has been filled for old mesh, fill new mesh

         iseqkp = kprob2d(meshin)
         nprob = kp(iseqkp)%nprob
         intmat2d(1:nprob,meshout) = intmat2d(1:nprob,meshin)
         intmat1d => intmat2d(:,meshout)
         call incomgen ( -1, intmat2d(1,meshin) )

      end if  ! ( abs(intmat2d(2,meshin))==102 )

      meshnr = meshout  ! change mesh number

1000  if ( debug ) then

!     --- Debug information

         call prinin ( intmat2d(1,meshout), 5, 'intmat_out' )
         write(irefwr,1) 'meshnr', meshnr
         write(irefwr,*) 'End probstruc'

      end if  ! ( debug )
      call erclos ( 'probstruc' )

      end subroutine probstruc
