      subroutine prord0type ( iinplc, iinputlc, &
                              ntype, iwork, inum, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 24-07-2015 Extension with type 220
!        version  1.2  date 05-03-2015 Extension with type 912
!        version  1.1  date 16-02-2015 Extension with type 814
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information of type numbers
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: inum, ielgrp
      integer, intent(inout) :: iinplc, iinputlc(*), ntype, iwork(*)

!     ielgrp         o    element group number
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   See subroutine prord0
!     inum           o    Number of numbers read in a record
!     iwork         i/o   See subroutine prord0
!     ntype         i/o   Number of different type numbers to be used for one
!                         group
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=18
      integer :: ncode2(nncode2), ncodmx2(nncode2)
      integer :: ielgrpfirst, ielgrplast
      character (len=25) :: code2(nncode2)

!     code2          Table with keywords
!     ielgrpfirst    first element group number
!     ielgrplast     last element group number
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      67   ielgrp has wrong value
!     162   Number of type numbers read in record is incorrect
!     163   element group number read is not the next element group number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'ELGRP', 'LAPLACE', 'POISSON', 'DIFFUSION_EQUATION', &
                  'CONVECTION_DIFFUSION', 'REYNOLDS', 'ELASTICITY', &
                  'TYPE', 'GENERAL_ELLIPTIC_EQUATION', &
                  'NAVSTOKES_PENALTY', 'NAVSTOKES_REDUCED', &
                  'NAVSTOKES_CR', 'NAVSTOKES_TH', 'BOUSSINESQ', &
                  'MINIMAL_SURFACE', 'NON_LINEAR_DIFFUSION', &
                  'MASS_FLUX', 'BEAM' /
      data ncode2  / 3, 4, 4, 4, 6,  5, 5, 4,11,11, &
                   11,11,11, 4, 5,  7, 4, 4 /
      data ncodmx2 / 5, 7, 7,18,20,  8,10, 4,25,17, &
                   17,12,12,10,15, 20, 9, 4 /

!     code2     Table of local keywords
!              1: ELGRP
!              2: LAPLACE
!              3: POISSON
!              4: DIFFUSION_EQUATION
!              5: CONVECTION_DIFFUSION
!              6: REYNOLDS
!              7: ELASTICITY
!              8: TYPE
!              9: GENERAL_ELLIPTIC_EQUATION
!             10: NAVSTOKES_PENALTY
!             11: NAVSTOKES_REDUCED
!             12: NAVSTOKES_CR
!             13: NAVSTOKES_TH
!             14: BOUSSINESQ
!             15: MINIMAL_SURFACE
!             16: NON_LINEAR_DIFFUSION
!             17: MASS_FLUX
!             18: BEAM
! ======================================================================
!
      call eropen ( 'prord0type' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord0type'
         write(irefwr,1) 'ntype', ntype
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      itype(1:nncode) = 2
      tabmin(1:nncode) = 0d0
      tabmax(1:nncode) = 0d0
      table(1:nncode) = -1d0

!     --- Read information of types

      itype(1) = 41       ! elgrp i to j
      tabmin(1) = 21d0
      tabmax(1) = 1d6
      table(1) = -3 ! type

      itype(8) = 11       ! type
      tabmin(8) = 1d0
      tabmax(8) = 999d0

      nmain = -nncodemain
      iretur = icode

      if ( debug ) write(irefwr,*) 'before inxt02/1'
      call sepreadsetkeyw
      icode = iretur
      if ( debug ) then
         write(irefwr,1) 'icode, ierror', icode, ierror
         call prinrl ( table, nncode, 'table' )
      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      if ( icode>0 ) then

!     --- Main keyword found

         if ( icode==7 ) then

!        --- Problem found, get next item

            call asknxt

         end if
         go to 1000  ! repeat

      end if

      if ( table(1)<0d0 ) then

!     --- not an element group number found

         ielgrpfirst = 1
         ielgrplast = nelgrp
         iposut = 0
         inum = 0

         if ( table(8)>0d0 ) then

!        --- type found

            iposut = nint(table(8))
            inum = nint(tabmax(8))

         end if  ! ( table(8)>0d0 )

      else

!     --- element group number found

         iposut = nint(table(1))
         inum = nint(tabmax(1))-2
         ielgrpfirst = nint(tabout(iposut))
         ielgrplast = nint(tabout(iposut+1))

      end if  ! ( table(1)<0 )

      if ( debug ) &
         write(irefwr,1) 'ielgrpfirst, ielgrplast, inum, iposut', &
                          ielgrpfirst, ielgrplast, inum, iposut
      do ielgrp = ielgrpfirst, ielgrplast

         if ( debug ) write(irefwr,1) 'ielgrp', ielgrp

         if ( ielgrp<1 .or. ielgrp>nelgrp ) then

!        --- ielgrp out of range

            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errsub ( 67, 2, 0, 0 )

         else if ( iwork(ielgrp)/=0 ) then

!        --- ielgrp has been used already

            call errint ( ielgrp, 1 )
            call errsub ( 163, 1, 0, 0 )

         else

            iwork(ielgrp) = iinplc+1

         end if

      end do  ! ielgrp = ielgrpfirst, ielgrplast
      ielgrp = ielgrpfirst
      iinputlc(iinplc+1) = ielgrpfirst
      iinputlc(iinplc+2) = ielgrplast
      iinplc = iinplc+2
      iinputlc(iinplc+1:iinplc+20) = 0

      if ( inum>0 ) then

         if ( table(8)<0d0 ) iposut = iposut+2
         ntype = max(ntype,inum)  ! number of types found

         iinputlc(iinplc+1:iinplc+inum) = &
            nint(tabout(iposut:iposut+inum-1))

         if ( debug ) then
            write(irefwr,1) 'iinplc, inum, iposut', iinplc, inum, iposut
            call prinrl ( tabout(iposut-2), inum+2, 'tabout' )
         end if  ! ( debug )

      else if ( table(2)>0d0 ) then

!     --- table(2)>0d0, Laplace

         inum = 1
         iinputlc(iinplc+1) = 700

      else if ( table(3)>0d0 ) then

!     --- table(3)>0d0, Poisson

         inum = 1
         iinputlc(iinplc+1) = 701

      else if ( table(4)>0d0 ) then

!     --- table(4)>0d0, diffusion

         inum = 1
         iinputlc(iinplc+1) = 702

      else if ( table(5)>0d0 ) then

!     --- table(5)>0d0, convection diffusion

         inum = 1
         iinputlc(iinplc+1) = 703

      else if ( table(6)>0d0 ) then

!     --- table(6)>0d0, Reynolds

         inum = 1
         iinputlc(iinplc+1) = 325

      else if ( table(7)>0d0 ) then

!     --- table(7)>0d0, Elasticity

         inum = 1
         iinputlc(iinplc+1) = 250

      else if ( table(9)>0d0 ) then

!     --- table(9)>0d0, General elliptic equation

         inum = 1
         iinputlc(iinplc+1) = 705

      else if ( table(10)>0d0 ) then

!     --- table(10)>0d0, Navier-Stokes equation (penalty, CR)

         inum = 1
         iinputlc(iinplc+1) = 900

      else if ( table(11)>0d0 ) then

!     --- table(11)>0d0, Navier-Stokes equation (reduced, CR)

         inum = 1
         iinputlc(iinplc+1) = 901

      else if ( table(12)>0d0 ) then

!     --- table(12)>0d0, Navier-Stokes equation (CR)

         inum = 1
         iinputlc(iinplc+1) = 902

      else if ( table(13)>0d0 ) then

!     --- table(13)>0d0, Navier-Stokes equation (TH)

         inum = 1
         iinputlc(iinplc+1) = 903

      else if ( table(14)>0d0 ) then

!     --- table(14)>0d0, Boussinesq equation

         inum = 1
         iinputlc(iinplc+1) = 420

      else if ( table(15)>0d0 ) then

!     --- table(15)>0d0, minimal_surface equation

         inum = 1
         iinputlc(iinplc+1) = 814

      else if ( table(16)>0d0 ) then

!     --- table(16)>0d0, non_linear_diffusion equation

         inum = 1
         iinputlc(iinplc+1) = 803

      else if ( table(17)>0d0 ) then

!     --- table(17)>0d0, mass flux given

         inum = 1
         iinputlc(iinplc+1) = 912

      else if ( table(18)>0d0 ) then

!     --- table(18)>0d0, beam given

         inum = 1
         iinputlc(iinplc+1) = 220

      else

!     --- No type number found

         call errint ( inum, 1 )
         call errint ( 20, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 162, 3, 0, 0 )
         call errcrd

      end if  ! ( inum>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinputlc(iinplc+1), 20, 'iinputlc' )
         write(irefwr,1) 'ntype', ntype
         write(irefwr,*) 'End prord0type'

      end if  ! ( debug )
      call erclos ( 'prord0type' )

      end subroutine prord0type
