      subroutine prord3 ( iinput, maxlen, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version 15.2  date 13-10-2013 Extension with intersection
!        version 15.1  date 08-03-2013 Correction do-loop
!        version 15.0  date 03-04-2012 Split in subroutines
!
!   copyright (c) 1982-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards for essential boundary conditions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp, iinputlc(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         the essential boundary conditions
!     iinputlc            integer work array where the information read is
!                         stored before processing
!     maxlen         i    Maximal space available for array IINPUT
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nespnt, nescrv, nessrf, nesnod, neselm, lencrv, &
              nesgrp, nesobst, nesconct, nesnocon, iinplc, &
              lengrp, lenpnt, lensrf, nesnodal, &
              nesvlm, lenvlm, nesallobst, lennodal, &
              nesinnobst, nesbnobst, nesonbnobst, leninters, &
              nesinonbnobst, neszerolev, nescav, nesintersect
      integer, parameter :: ncodsb=5, nncode2=28
      integer nscods(ncodsb), ncode2(nncode2), ncodmx2(nncode2)
      character (len=18) code2(nncode2)
      character (len=1) codesb(ncodsb)

!     code2          Table with keywords
!     codesb         Table with subkeywords
!     iinplc         Last position used in array iinputlc
!     lencrv         Number of entries in array curves
!     lengrp         Length of group part
!     leninters      Length of intersection part
!     lennodal       Length of nodal_point part
!     lenpnt         Length of points part
!     lensrf         Length of surfaces part
!     lenvlm         Length of volumes part
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncodsb         Number of entries in codesb
!     nesallobst     Number of obstacles where all nodes are prescribed
!     nesbnobst      Number of obstacles where all but the inner nodes are
!                    prescribed
!     nescav         Number of parts where cavity bondary conditions are given
!     nesconct       Number of contacts whith essential boundary conditions
!     nescrv         Number of parts where curve essential boundary
!                    conditions are given
!     neselm         Number of parts where element essential boundary
!                    conditions are given
!     nesgrp         Number of groups of element groups with essential boundary
!                    boundary conditions
!     nesinnobst     Number of obstacles where all inner nodes are prescribed
!     nesinonbnobst  Number of obstacles where all he nodes on the
!                    obstacle which are only part of elements completely
!                    inside the obstacle are prescribed
!     nesintersect   Number of intersections where essential boundary
!                    conditions are given
!     nesnocon       Number of "no"-contacts whith essential
!                    boundary conditions
!     nesnod         Number of groups of nodes with essential
!                    boundary conditions
!     nesnodal       Number of groups of nodal point with essential boundary
!                    conditions
!     nesobst        Number of obstacles whith essential boundary conditions
!     nesonbnobst    Number of obstacles where all the nodes on the boundary of
!                    the obstacle are prescribed
!     nespnt         Number of parts where point essential boundary
!                    conditions are given
!     nessrf         Number of parts where surface essential boundary
!                    conditions are given
!     nesvlm         Number of parts where volume essential boundary
!                    conditions are given
!     neszerolev     Number of parts where essential boundary conditions of
!                    the type zero_levelset i are given
!     nncode2        Number of entries in code2
!     nscods         Number of sub keywords
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRORD3COPY     Copy all input read by prord3 in the correct sequence
!                    into iinput
!     PRORD3DETECT   Interprete information read by prord3 and store in iinputlc
!     PRORD3TABLE    Fill tables for subroutine prord3 and read input line
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognised:
!     [degrees of freedom] [location part]
!     in arbitrary order
!     Degrees of freedom has the following shape
!     DEGFD k [,DEGFD l [, DEGFD m ....] ]
!     If omitted DEGFD 0 is assumed
!     The location part has one of the following structures:
!     POINTS ( Pk, Pl, . . . , Pm)
!     CURVES [l] (Cj [to Cm] )
!     OUTER_CURVES
!     SURFACES [i1, i2, i3, ... ] (Sj, Sk, ...] ) [ SKIP_BOUNDARY ]//
!         [SKIP_CURVES (Ci, Cj, ... )], [INCLUDE_CURVES (Ci, Cj, ... )]
!     OUTER_SURFACES
!     VOLUMES [i1, i2, i3, ... ] (Vj, Vk, .. ), [SKIP_SURFACES (Si, Sj, ... )]
!     NODES (Nj [to Nm ] )
!     ELEMENTS i [to j] [(RN1, RN2, RN3, ... )]
!     GROUP ielgrp [(RN1, RN2, RN3, ... )]
!     OBSTACLE i
!     CONTACT i
!     NO_CONTACT i
!     NODAL_POINTS ( i1, i2, i3, ... )
!     FILE_NODAL_POINTS = 'name_of_file'
!     IN_ALL_OBSTACLE i
!     IN_INNER_OBSTACLE i
!     IN_BOUN_OBSTACLE i
!     ON_BOUN_OBSTACLE i
!     INON_BOUN_OBSTACLE i
!     ZERO_LEVELSET i
!     CAVITATION i
!     INTERSECTION (Si, Sj, Sk, ... )
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     172   too much record input for essential boundary conditions
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'DEGFD', 'POINTS', 'CURVES', 'TYPE', &
                  'SURFACES', 'VOLUMES', 'NODES', 'ELEMENTS', &
                  'GROUP', 'OBSTACLE', 'CONTACT', 'NO_CONTACT', &
                  'FILE_NODAL_POINTS', 'SKIP_BOUNDARY', &
                  'SKIP_CURVES', 'INCLUDE_CURVES', &
                  'NODAL_POINTS', 'IN_ALL_OBSTACLE', &
                  'IN_INNER_OBSTACLE', 'IN_BOUN_OBSTACLE', &
                  'ON_BOUN_OBSTACLE', 'INON_BOUN_OBSTACLE', &
                  'SKIP_SURFACES', 'OUTER_CURVES', &
                  'OUTER_SURFACES', 'ZERO_LEVELSET', 'CAVITATION', &
                  'INTERSECTION' /
      data ncode2 /  3, 4, 4, 4, 5,    5, 5, 5, 4, 4, &
                    4, 6, 8, 9, 9,   12, 8,10,12,12, &
                   12,12, 9, 9, 9,    8, 5, 6 /
      data ncodmx2/  5, 6, 6, 4, 8,    7, 5, 8, 5, 8, &
                    7,10,17,13,11,   14,12,15,17,16, &
                   16,18,13,12,14,   13,10,12 /
      data codesb/ 'V', 'P', 'C', 'S', 'N' /
      data nscods /5*1/

!    code2      Contains keywords that are recognized
!              Possible values:
!              1:  DEGFD
!              2:  POINTS
!              3:  CURVES
!              4:  TYPE
!              5:  SURFACES
!              6:  VOLUMES
!              7:  NODES
!              8:  ELEMENTS
!              9:  GROUP
!             10:  OBSTACLE
!             11:  CONTACT
!             12:  NO_CONTACT
!             13:  FILE_NODAL_POINTS
!             14:  SKIP_BOUNDARY
!             15:  SKIP_CURVES
!             16:  INCLUDE_CURVES
!             17:  NODAL_POINTS
!             18:  IN_ALL_OBSTACLE
!             19:  IN_INNER_OBSTACLE
!             20:  IN_BOUN_OBSTACLE
!             21:  ON_BOUN_OBSTACLE
!             22:  INON_BOUN_OBSTACLE
!             23:  SKIP_SURFACES
!             24:  OUTER_CURVES
!             25:  OUTER_SURFACES
!             26:  ZERO_LEVELSET
!             27:  CAVITATION
!             28:  INTERSECTION
! ======================================================================
!
      call eropen ( 'prord3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3'
         write(irefwr,1) 'ncodsb', ncodsb
  1      format ( a, 1x, (10i9) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters

      nespnt = 0
      nescrv = 0
      nessrf = 0
      nesvlm = 0
      nesnod = 0
      neselm = 0
      nesgrp = 0
      nesobst = 0
      nesconct = 0
      nesnocon = 0
      nesnodal = 0
      nesallobst = 0
      nesinnobst = 0
      nesbnobst = 0
      nesonbnobst = 0
      nesinonbnobst = 0
      neszerolev = 0
      nescav = 0
      nesintersect = 0

      lenpnt = 0
      lencrv = 0
      lensrf = 0
      lenvlm = 0
      lengrp = 0
      lennodal = 0      ! length of part corresponding to nodal_points
      iinplc = 0
      ityprd = 1

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

!     --- Set itype

      itype(1:nncode) = 0
      itype(1) = 14   ! DEGFD
      itype(2) = 18   ! POINTS
      itype(3) = 31   ! CURVES
      itype(5:6) = 32 ! SURFACES, VOLUMES
      itype(7) = 18   ! NODES
      itype(8) = 37   ! ELEMENTS
      itype(9) = 33   ! GROUP
      itype(13) = 6   ! FILE_NODAL_POINTS
      itype(14) = 2   ! SKIP_BOUNDARY
      itype(15:16) = 18  ! SKIP_CURVES, INCLUDE_CURVES
      itype(17) = 28  ! NODAL_POINTS
      itype(23) = 18  ! SKIP_SURFACES
      itype(24:25) = 2 ! OUTER_CURVES, OUTER_SURFACES
      itype(28) = 18  ! INTERSECTION

!     --- Copy extra information into codemain

      call sepcopycodesub ( codesb, ncodsb, nscods )

      icode = 0
      iinplc = 0
      leninters = 0

!     --- Loop

      do while ( icode==0 )

!     --- Fill tables and read record

         call prord3table
         if ( ierror/=0 ) go to 1000

         if ( iinp>maxlen ) then

!        --- iinp too large:  stop

           call errint ( iinp, 1 )
           call errint ( maxlen, 2 )
           call errsub ( 172, 2, 0, 0 )
           go to 1000

         end if

         if ( icode/=0 ) exit

!        --- interpret contents of table and store in iinputlc

         call prord3detect ( iinputlc, iinplc, table, tabout, tabmin, &
                             tabmax, nespnt, lenpnt, &
                             nescrv, nessrf, code, ncodmx, lensrf, &
                             nesvlm, lenvlm, nesnod, neselm, nesgrp, &
                             lengrp, nesobst, nesconct, nesnocon, &
                             nesnodal, lennodal, nesallobst, &
                             nesinnobst, nesbnobst, nesonbnobst, &
                             nesinonbnobst, neszerolev, nescav, &
                             strout, lencrv, nesintersect, leninters )
         if ( ierror/=0 ) go to 1000

      end do  ! while ( icode==0 )

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy all input in the correct sequence into iinput

      if ( iinplc>0 ) &
         call prord3copy ( iinp, lenpnt, lensrf, lenvlm, &
                           nesnod, neselm, lengrp, nesobst, &
                           nesallobst, nesinnobst, nesbnobst, &
                           nesonbnobst, nesinonbnobst, nesconct, &
                           nesnocon, lennodal, neszerolev, nescav, &
                           iinput, iinplc, iinputlc, npoint, lencrv, &
                           leninters )
      if ( ierror/=0 ) go to 1000

      if ( iinp>=maxlen ) then

!     --- iinp>maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if

      iinput(6) = nespnt
      iinput(7) = nescrv
      iinput(8) = nessrf
      iinput(10) = nesnod
      iinput(11) = neselm
      iinput(12) = nesgrp
      iinput(16) = nesobst
      iinput(19) = nesconct
      iinput(22) = nesnocon
      iinput(28) = nesnodal
      iinput(39) = nesvlm
      iinput(45) = nesallobst
      iinput(41) = nesinnobst
      iinput(42) = nesbnobst
      iinput(43) = nesonbnobst
      iinput(46) = nesinonbnobst
      iinput(47) = neszerolev    ! number of parts with zero level set
      iinput(49) = nescav        ! number of cavity parts
                                 ! mark that position 48 is already in use
      iinput(56) = nesintersect  ! number of intersection parts

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, iinp, 'iinput' )
         write(irefwr,*) 'End prord3'

      end if
      call erclos ( 'prord3' )

      end
