      subroutine prord3copysf ( iinput, iinputlc, jinp, istsrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy all input read by prord3 in the correct sequence into iinput
!     surface part
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     essential_boundary_condition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinputlc(*), jinp, istsrf

!     iinput        i/o   integer array to be filled with information concerning
!                         the natural boundary conditions
!     iinputlc       i    integer work array where the information read is
!                         stored before processing
!     istsrf         i    Starting address of surfaces in iinput
!     jinp          i/o   last position used in array iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jnpelm, inum, iskipbn, ncurves

!     inum           Number of numbers read in a record
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     jnpelm         Number of nodes in an element
!     ncurves        Number of curves that must be adapted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3copysf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3copysf'
         write(irefwr,1) 'jinp, istsrf', jinp, istsrf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jnpelm = iinputlc(jinp+3)
      iinput(istsrf+1) = iinputlc(jinp+1)  ! bit pattern

      if ( debug ) &
         write(irefwr,1) 'ibit, jnpelm', iinputlc(jinp+1), jnpelm

      if ( jnpelm==-10 ) then

!     --- Special case of outer surfaces

         iinput(istsrf+2) = -1
         iinput(istsrf+3) = -1
         jinp = jinp+3
         istsrf = istsrf+3

      else  ! ( iinputlc(jinp+3)/=-10 )

!     --- Standard case
!         Storage:
!           1: bit pattern
!           2: jnpelm (number of nodes per element)
!           3: number of surfaces
!           next:  set of surface numbers
!           followed by jnpelm positions inidicating if local node
!                    must be used
!           next positions: iskipbn and ncurves
!           followed by ncurves curve numbers if iskipbn>1

         inum = iinputlc(jinp+4)+jnpelm+2 ! 2+number of curves+local nodes

!        --- Copy curve numbers and local nodes info in that sequence

         iinput(istsrf+2:istsrf+1+inum) = iinputlc(jinp+3:jinp+2+inum)
         if ( debug ) &
            call prinin ( iinput(istsrf+1), inum+1, 'iinput_sur/1' )

         istsrf = istsrf+inum+1
         jinp = jinp+inum+2

         if ( debug ) write(irefwr,1) 'istsrf, jinp', istsrf, jinp

!        --- Next part with respect to skip boundary

         iskipbn = iinputlc(jinp+1)
         ncurves = iinputlc(jinp+2)

         if ( debug ) &
            write(irefwr,1) 'iskipbn, ncurves', iskipbn, ncurves

         iinput(istsrf+1) = iskipbn
         iinput(istsrf+2) = ncurves
         if ( debug ) &
            call prinin ( iinput(istsrf+1), 2, 'iinput_sur/2' )
         jinp = jinp+2
         istsrf = istsrf+2
         if ( iskipbn>1 ) then

!        --- iskipbn>1,  copy curves from work into iinput

            iinput(istsrf+1:istsrf+ncurves) = &
               iinputlc(jinp+1:jinp+ncurves)
            if ( debug ) &
               call prinin ( iinput(istsrf+1), ncurves, &
                             'iinput_sur/3' )

         end if
         jinp = jinp+ncurves
         istsrf = istsrf+ncurves

      end if  ! ( iinputlc(jinp+3)==-10 )

1000  call erclos ( 'prord3copysf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord3copysf'

      end if  ! ( debug )

      end
