      subroutine prord3table
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-10-2013 Extension with intersection
!        version  1.0  date 30-03-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill tables for subroutine prord3 and read input line
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3table' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3table'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set table values

      table(1:nncode) = -1d20
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 0d0
      table(2) = -2
      table(3) = -3
      table(5) = -4
      table(6) = -1
      table(7) = -5
      table(15) = -3
      table(16) = -3
      table(23) = -4
      table(28) = -4  ! S

!     --- Initialize some parameters

      tabmin(1) = 1d2
      tabmax(1) = 11d0
      tabmin(2) = 20d0
      tabmax(2) = nuspnt
      tabmin(3) = ncurvs
      tabmax(3) = ncurvs
      tabmin(4) = npelm
      tabmax(4) = ncurvs
      tabmin(5:6) = npelm
      tabmax(5) = nsurfs
      tabmax(6) = nvolms
      tabmin(7) = 2d0
      tabmax(7) = npoint
      tabmin(8) = 3d0
      tabmax(8) = npelm
      tabmin(9) = nelgrp
      tabmax(9) = npelm
      tabmax(10) = 1d2
      tabmax(11) = 1d1
      tabmax(12) = 1d1
      tabmin(15) = 97d0
      tabmax(15) = ncurvs
      tabmin(16) = 97d0
      tabmax(16) = ncurvs
      tabmin(17) = 1d2
      tabmax(17) = nelem
      tabmin(18) = 1d0
      tabmax(18) = numobst
      tabmin(19) = 1d0
      tabmax(19) = numobst
      tabmin(20) = 1d0
      tabmax(20) = numobst
      tabmin(21) = 1d0
      tabmax(21) = numobst
      tabmin(22) = 1d0
      tabmax(22) = numobst
      tabmin(23) = 97d0
      tabmax(23) = nsurfs
      tabmin(26) = 1d0
      tabmax(26) = 1d0
      tabmin(27) = 1d0
      tabmax(27) = 1d0
      tabmin(28) = 97d0        ! maximum number of surfaces (INTERSECT)
      tabmax(28) = nsurfs   ! maximum surface number
      nmain = nncodemain
      iretur = icode
      call sepreadsetkeyw
      icode = iretur

      if ( debug ) then
         write(irefwr,1) 'icode', icode
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord3table'

      end if  ! ( debug )
      call erclos ( 'prord3table' )

      end
