      subroutine prord5 ( iinput, maxlen, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.1  date 10-02-2012 use getnpelmelgrp
!        version  7.0  date 30-04-1998 Extra parameters icode, iinp
!        version  6.1  date 14-07-1996 Correction for surfaces
!
!   copyright (c) 1987-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards for boundary conditions of the type u equals unknown
!   constant along a part of the boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp

!     iinp      i/o   Last position used in IINPUT
!     iinput    i/o   integer array to be filled with information concerning
!                     the local transformations
!                     iinput must have been filled before partly
!     maxlen     i    Maximal space available for array IINPUT
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nloccv, nlocsf, ncount, i, jinp, nodp, jnodp, inpelm
      integer, parameter :: nncode2=6
      integer itype2(nncode2), ncode2(nncode2), ncodmx2(nncode2)
      double precision tabmin2(nncode2), tabmax2(nncode2)
      character (len=11) :: code2(nncode2)

!     code2      Table with keywords
!     i          Counting variable
!     inpelm     Number of points in an element
!     itype2     Table containing information about the numbers behind
!                the keywords in the table code2
!     jinp       position used in array iinput
!     jnodp      Nodal point sequence number
!     ncode2     Table containing the number of significant characters in
!                the keyword table code2
!     ncodmx2    Table containing the maximal number of  characters in
!                the keyword table code2
!     ncount     Counter
!     nloccv     Number of curves with boundary conditions u=constant
!     nlocsf     Number of surfaces with boundary conditions u=constant
!     nncode2    Number of entries in code2
!     nodp       Nodal point sequence number
!     tabmax2    Contains maximum values allowed in table
!     tabmin2    Contains minimum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRTS00         Check icurv1, icurv2
!     PRTS01         Check isurf1, isurf2
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognised:
!     [degrees of freedom] [location part]
!     in arbitrary order
!     Degrees of freedom has the following shape
!     DEGFD k
!     If omitted DEGFD 1 is assumed
!     The location part has one of the following structures:
!     CURVES [l] (Cj [to Cm] )
!     SURFACES [i1, i2, i3, ... ] (Sj [to Sm] )
!     All these items must be in one line
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     170   records with curves found whereas already records with surfaces
!           have been found
!     269   Relative nodal point number in surface record incorrect
!     280   Double point found in surface record
!     329   Array to store the input too small
!     892   No number or more than two numbers found after CURVES [i] = (
!    1136   Number of surfaces found incorrect
!    1522   Both curves and surfaces found
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'DEGFD', 'CURVES', 'SURFACES', 'C', 'S', 'TO' /
      data ncode2 /  3, 4,  4,  1,  1,  2 /
      data ncodmx2/  5, 6,  8,  1,  1,  2 /
      data itype2 /  0, 4, 11, 15, 15, -1 /
      data tabmin2 / 1d0, 0d0, 0d0, 2d0, 2d0, 0d0 /
      data tabmax2 / 1d2, 5d2, 0d0, 1d3, 1d3, 0d0 /

!     code2     Table of local keywords
!              1: DEGFD
!              2: CURVES
!              3: SURFACES
!              7: C
!              8: S
!              9: TO
! ======================================================================
!
      call eropen ( 'prord5' )
      jinp = iinp
      iinp = iinp+2
      nloccv = 0
      nlocsf = 0
      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmin2, tabmax2 )

      icode = 0

!     --- while loop (icode=0)

200   if ( icode==0 ) then
         ityprd = 1
         table(1:nncode) = -1d20
         table(4) = 0d0
         table(5) = 0d0
         tabmax(1) = 100d0
         tabmin(3) = npelm
         tabmax(3) = npelm
         tabmax(4) = ncurvs
         tabmax(5) = nsurfs
         nmain = nncodemain
         iretur = icode
         call sepreadsetkeyw
         icode = iretur
         if ( icode/=0 ) go to 200

!        --- interpret curves part

         if ( table(2)>-1d18 ) then

!        --- curves record found

            nloccv = nloccv+1
            if ( nlocsf>0 ) then
               call errsub ( 170, 0, 0, 0 )
            end if
            iposut = nint(table(4))
            ncount = nint(tabmax(4))
            if ( ncount==0 ) then
               call errint ( ncount, 1 )
               call errsub ( 892, 1, 0, 0 )
            else
               iinput(iinp+3) = nint(tabout(iposut))
               if ( ncount==1 ) then
                  iinput(iinp+4) = iinput(iinp+3)
               else
                  iinput(iinp+4) = nint(tabout(iposut+1))
               end if
               call prts00 ( iinput(iinp+3), iinput(iinp+4), ncurvs )

!              --- Fill default values in iinput

               iinput(iinp+1) = 1
               if ( table(2)>0 ) then

!              --- integer found after curves

                  iinput(iinp+2) = nint(tabout(nint(table(2))))

               else

!              --- no integer found after curves

                  iinput(iinp+2) = 0

               end if

            end if

         end if

!        --- interpret surfaces part

         if ( table(3)>-1d18 ) then

!        --- surfaces record found

            nlocsf = nlocsf+1
            if ( table(2)>-1d-18 ) then
               call errsub ( 1522, 0, 0, 0 )
            end if
            iposut = nint(table(5))
            ncount = nint(tabmax(5))
            if ( ncount==0 ) then
               call errint ( ncount, 1 )
               call errsub ( 1136, 1, 0, 0 )
            else
               iinput(iinp+2) = nint(tabout(iposut))
               if ( ncount==1 ) then
                  iinput(iinp+3) = iinput(iinp+2)
               else
                  iinput(iinp+3) = nint(tabout(iposut+1))
               end if
               call prts01 ( iinput(iinp+2), iinput(iinp+3), nsurfs )
               inpelm = sepgetinpelsurf ( iinput(iinp+2), 0 )
               iinput(iinp+4) = inpelm

!              --- Fill default values in iinput

               iinput(iinp+1) = 1
               do i = 1,inpelm
                  iinput(iinp+4+i) = 1
               end do
               if ( tabmax(3)>0 ) then

!              --- integers found after surfaces

                  iposut = nint(table(3))
                  ncount = nint(tabmax(3))
                  do i = 1,inpelm
                     iinput(iinp+4+i) = 0
                  end do
                  do i = 1, ncount
                     nodp = nint(tabout(iposut+i-1))
                     if ( nodp<=0 .or. nodp>inpelm ) then
                        call errint ( nodp, 1 )
                        call errint ( inpelm, 2 )
                        call errsub ( 269, 2, 0, 0 )
                     end if
                     jnodp = iinput(iinp+3+i)
                     if ( jnodp>0 ) then

!                    --- jnodp >0  double point

                        call errint ( i, 1 )
                        call errint ( nodp, 2 )
                        call errsub ( 280, 2, 0 , 0)

                     else

!                    --- jnodp   =  0

                        iinput(iinp+3+i) = 1

                     end if

                  end do

               end if

            end if

         end if

!        --- interpret degfd part

         if ( table(1)>0d0 ) iinput(iinp+1) = nint(table(1))
         iinp = iinp+4
         if ( nlocsf>0 ) iinp=iinp+inpelm

         if ( iinp>maxlen ) then

!        --- iinp too large:  stop

            call errint ( iinp, 1 )
            call errint ( maxlen, 2 )
            call errsub ( 329, 2, 0, 0 )
            icode = -1

         end if
         go to 200

      end if

      iinput(13) = nloccv+nlocsf
      iinput(jinp+1) = nloccv
      iinput(jinp+2) = nlocsf

      call erclos ( 'prord5' )
      end
