      subroutine sepcommatdiadg
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store position of diagonal elements of array cols into idiag
!     in case of row-compact storage
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, jstart, jend, istart, ncols
      integer, pointer :: rows(:), cols(:), idiag(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     cols           Refers to column npmbers
!     i              Counting variable
!     idiag          Positions of diagonal elements in matrix
!     istart         Last position used in cols
!     j              Counting variable
!     jend           end position of row in cols
!     jstart         start position of row in cols
!     ncols          npmber of non-zero columns in row
!     rows           Refers to start of column npmbers per row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatdiadg' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatdiadg'
         write(irefwr,1) 'np', np
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( associated(IM%diagDG) ) deallocate(IM%diagDG)
      allocate ( IM%diagDG(np) )
      idiag => IM%diagDG
      rows => IM%rowsDG
      cols => IM%colsDG
      istart = 0
      do i = 1, np

         ncols = rows(i+1)-istart
         jstart = istart+1
         jend = istart+ncols
         if ( debug ) then
            write(irefwr,1) 'i, ncols, jstart, jend', i, ncols, jstart, jend
            call prinin ( cols(jstart), ncols, 'cols' )
         end if  ! ( debug )
         do j = jstart, jend
            if ( cols(j)==i ) then
               idiag(i) = j
               exit
            end if  ! ( cols(j)==i )
         end do  ! j = jstart, jend

         istart = istart+ncols

      end do  ! i = 1, np

1000  if ( debug ) then

!     --- Debug information

         call prinin ( idiag, np, 'idiag' )
         write(irefwr,*) 'End sepcommatdiadg'

      end if  ! ( debug )
      call erclos ( 'sepcommatdiadg' )

      end subroutine sepcommatdiadg
