      subroutine sepcommatreac ( isum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parts of array intmat with respect to boundary conditions
!     Part reaction forces
!     Old subroutine incommatreac
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     boundary_condition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: isum

!     isum          i/o   Integer variable to compute a sum
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: iwork(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iwork          integer work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INI011REAC     compute integer information of matrix corresponding to
!                    essential boundary conditions
!                    It concerns the part with respect to reaction forces
!     INI014REAC     compute length of arrays for subroutine ini011
!                    arrays ihelp and intmat part b
!                    It concerns the part with respect to reaction forces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatreac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatreac'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- compute second part of array INTMAT corresponding to boundary
!         conditions

      allocate ( IM%rowspp(nbound+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nbound+1, 'irowsp' )
      if ( ierror/=0 ) go to 1000
      irowsp => IM%rowspp

      call ini014reac

      isum = irowsp(nbound+1)
      if ( debug ) write(irefwr,*) 'isum ', isum

!     --- Reserve space for INTMAT part 2 and temporary space
!         First compute length of subarrays
!         Part 2a: isum

      allocate ( IM%colspp(isum), stat = error )
      if ( error/=0 ) call eralloc ( error, isum, 'icolsp' )
      if ( ierror/=0 ) go to 1000
      icolsp => IM%colspp

!     --- Copy intmat part 1a, positions 1 to nbound to
!         iwork so that it can be changed in ini011

      allocate ( iwork(nbound), stat = error )
      if ( error/=0 ) call eralloc ( error, nbound, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork(1:nbound) = irowsp(1:nbound)

!     --- Fill intmat part 2

      call ini011reac ( iwork )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcommatreac'

      end if  ! ( debug )
      call erclos ( 'sepcommatreac' )

      end subroutine sepcommatreac
