      subroutine sepcommatrnes
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     If necessary renumber nodes for nested dissection
!     The result is stored in array km(iseqkm)%rennested
!     Array kprobh/r is adapted
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     renumbering
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPNESTEDDIS   Renumbering with nested dissection
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatrnes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatrnes'
         write(irefwr,*) 'associated(km(iseqkm)%rennested) ', &
                          associated(km(iseqkm)%rennested)

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( associated(km(iseqkm)%rennested) ) then

!     --- nested dissection has already been performed

         kmeshj => km(iseqkm)%rennested

      else

!     --- nested dissection has not been performed

         nullify ( kmeshj )
         allocate ( km(iseqkm)%rennested(npoint) )
         kmeshj => km(iseqkm)%rennested
         kelmj = 1

         call sepnesteddis

      end if  ! ( associated(km(iseqkm)%rennested) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcommatrnes'

      end if  ! ( debug )
      call erclos ( 'sepcommatrnes' )

      end subroutine sepcommatrnes
