      subroutine sepedg04 ( inodes, iedge )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates nmeshd part 2 actual edge labels
!     Old subroutine topedg04
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inodes(*),  iedge

!     iedge      i    edge sequence number
!     inodes     i    array with information about the edge nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n1, ne, ie, ielem, ielgrp, jedge, nn, &
              ihelp(1000), kc, in, ipkc, ipd

!     ie         relative element label
!     ielem      Element number
!     ielgrp     element group number
!     ihelp      help variable
!     in         node number
!     ipd        pointer in kmeshaad
!     ipkc       pointer in kmeshc
!     jedge      edge number
!     kc         indicator
!     n1         node labe first node
!     ne         number of elements connected with node n1
!     nn         number of nodes per edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TOPEDG03       Get local node numbers of an edges
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepedg04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepedg04'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fetch node label of first node of this edge

      n1 = inodes(1)

!     --- Loop over all elements connected with this node

      ne = kmeshd(n1+1) - kmeshd(n1)

      if ( debug ) &
         write(irefwr,1) 'n1, ne, npoint, iedge', n1, ne, npoint, iedge

      do ie = 1, ne

!     --- Fetch element label

         ielem = kmeshd(npoint+1+kmeshd(n1)+ie)

!        --- Fetch starting position in kmeshc of this element

         ipkc = sepstkmeshc ( ielem, npelm )-1

!        --- Fetch the element group of this element

         ielgrp = sepelgrpelem(ielem)

!        --- Fetch the number of edges of this element

         nedpel = sepgetnedpel(ielgrp)

!        --- Fetch the mesh type of this element

         ishape = sepgetshape ( ielgrp )

         if ( debug ) &
            write(irefwr,1) 'ie, ielem, ielgrp, nedpel', &
                             ie, ielem, ielgrp, nedpel

!        --- Loop over the edges of this element

         do jedge = 1, nedpel

!        --- Get local nodes of this edge

            call topedg03( jedge, ishape, nn, ihelp, nspec )
            if ( nn==nndped ) then

!           --- Check the other nodes first in one way

               kc = 0
               do in = 1, nn
                  if ( kmeshc(ipkc + ihelp(in))==inodes(in) ) &
                     kc = kc + 1
               end do
               if ( kc/=nn ) then

!              --- Try the other way around

                  kc = 0
                  do in = 1, nn
                     if ( kmeshc(ipkc + ihelp(in)) == &
                        inodes(nn+1-in) ) &
                        kc = kc -1
                  end do
               end if
               if ( debug ) write(irefwr,1) 'kc, nn', kc, nn
               if ( abs(kc)==nn ) then

!              --- Element found with edge the same as input edge

                  ipd = nmeshd(ielem)
                  nmeshd(ipd+jedge) = iedge*sign(1,kc)
                  goto 400
               end if
            end if
         end do
400   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepedg04'

      end if  ! ( debug )
      call erclos ( 'sepedg04' )

      end subroutine sepedg04
