      subroutine sepfillcommat ( iincommt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill modules for subroutine incommat
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iincommt(*)

!     iincommt   i    Integer input array containing information concerning
!                     the structure of the large matrix
!                     See subroutine read00 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLKPROBN  Fill parameters in module sepmodulekprob
!     SEPFILLKPROBPC Fill parameters wrt kprobpar in module sepmodulekprob
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPRMMATSTR1   nullify arrays in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillcommat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillcommat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill sepmodulematstr

      nunkp = kprobloc(4)
      nusol = kprobloc(5)
      nbound = kprobloc(10)
      indprx = kprobloc(11)
      numnat = kprobloc(12)
      indpra = kprobloc(14)
      nphys  = kprobloc(33)
      npboun = kprobloc(34)
      indpry = kprobloc(51)
      numpairs = kprobloc(62)
      nrusol = kprobloc(66)
      ninterface = kprobloc(67)
      iseqkm = kp(iseqkp)%iseqkm

      nrusolloc = nusol-nbound
      kproby => kprobpart%kproby
      ipstkprobpar = kprobloc(65)  ! starting address

!     --- intmat may have been filled before
!         If so, nullify arrays

      call seprmmatstr1

      call sepfillkprobn
      call sepfillkprobpc

      kprobloc(80) = 0     ! mark as new

      if ( kelme==0 ) call sepfillkmeshe

!     --- Fill parameters in module sepmodulematstr

      call sepfillmatstr2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillcommat'

      end if  ! ( debug )
      call erclos ( 'sepfillcommat' )

      end subroutine sepfillcommat
