      subroutine sepfilledges ( mnoped )
! ======================================================================
!
!        programmer    Frans van de Vosse/Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the edges part 2 actual edge labels and nmeshc
!     This subroutine is used in case nspec = 0
!     See also topedg02
!     Old subroutine topfilledges
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mnoped

!     mnoped         i    maximum number of nodes per edge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, nedge_elem, ielem, i, ipd, ipc, &
              irelem, jedge, ikelmc, inpelm, nnode_edge, iedge, &
              j, jelem, jelgrp, jkelmc, jnedge_elem, jnode_edge, &
              jshape, icount, ledge, nodp1, jpd, &
              jnpelm, nside, ihelp(mnoped), jhelp(mnoped)
      logical knownshape
      integer, allocatable :: infoelms(:,:)

!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     ihelp          integer work array of length mnoped
!     ikelmc         position in kmesh part c
!     infoelms       integer work array of length 2*nelem
!                    infoelms(1,i) contains the starting address of
!                       element i in kmeshc (minus 1)
!                    infoelms(1,i) contains the element group number of
!                       element i
!     inpelm         number of points per element
!     ipc            Starting address in array
!     ipd            pointer in kmeshaad
!     irelem         Relative element sequence number with respect to
!                    element group
!     j              Counting variable
!     jedge          edge sequence number
!     jelem          do loop variable with respect to the elements
!     jelgrp         Actual element group number
!     jhelp          integer work array of length mnoped
!     jkelmc         position in kmesh part c
!     jnedge_elem    number of edges in one element
!     jnode_edge     number of nodes in one edge
!     jnpelm         number of points per element
!     jpd            pointer in kmeshaad
!     jshape         Shape number of elements
!     knownshape     If true the shpae is known in this subroutine
!     ledge          edge sequence number
!     nedge_elem     Number of edges in element
!     nnode_edge     number of nodes in edge
!     nodp1          Node sequence number
!     nside          number of points along edge except vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ININODEEDGES   Fill array nodesedges with the local node numbers of the
!                    nodes on the iedge-th edge of an element with shape number
!                    ishape
!     PRININ         print 1d integer array
!     SEPPRINKMESHC  print kmesh part c
!     SEPPRINNMESHC  print nmesh part c
!     SEPPRINNMESHD  print nmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen  ('sepfilledges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilledges'
         call prinin ( nmeshd, nelem, 'nmeshd' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      nside = max(0,nspec-2)

      allocate ( infoelms(2,nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'infoelms' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array infoelms with starting addresses in kmeshc and element
!         group numbers

      ielem = 0
      ikelmc = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!         and number of elements in this group (inelem)
!         and number of nodes per element in his group

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         do irelem = 1, inelem
            ielem = ielem+1
            infoelms(1,ielem) = ikelmc
            infoelms(2,ielem) = ielgrp
            ikelmc = ikelmc+inpelm
         end do
      end do

      ielem = 0
      nedges = 0
      ikelmc = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!         and number of elements in this group (inelem)
!         and number of edges per element in this group (nedge_elem)
!         and number of nodes per element in his group

         ishape = kmeshf ( ielgrp )
         knownshape = ishape<19 .or. ishape==21 .or. ishape==22 .or. &
                      ishape>=50 .and. ishape<60
         if ( .not. knownshape ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         if ( ishape==-10001 ) then

!        --- Connection elements must be skipped

            ikelmc = ikelmc+inelem*inpelm
            ielem = ielem +inelem
            go to 500

         end if

         nedge_elem =  sepgetnedgeshape(ishape)
         if ( ierror/=0 ) go to 1000
         if ( ishape==-9 ) nedge_elem = inpelm-1
         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, inelem, nedge_elem', &
                             ielgrp, inpelm, inelem, nedge_elem

!        --- Loop over the elements in this group

         do irelem = 1, inelem
            ielem = ielem + 1
            ipd = nmeshd(ielem)

!           --- Loop over the edges

            do jedge = 1, nedge_elem

               if ( nmeshd(ipd+jedge)==0 ) then

!              --- New edge found

                  nedges = nedges + 1
                  nmeshc(nedges) = 0
                  nnode_edge = sepgetnumnodedge ( ishape )
                  if ( ierror/=0 ) go to 1000
                  if ( nside>0 .and. ishape>0 ) &
                     nnode_edge = nnode_edge + nside
                  if ( debug ) &
                   write(irefwr,1) 'irelem, jedge, nedges, nnode_edge', &
                                    irelem, jedge, nedges, nnode_edge

!                 --- Get the local nodes of this edge and store in ihelp

                  call ininodeedges ( ishape, jedge, ihelp, nspec )
                  if ( ierror/=0 ) go to 1000
                  if ( ishape==-9 ) then

!                 --- Special case of long line element
!                     The element is subdivided in inpelm-1 edges

                     ihelp(1) = jedge
                     ihelp(2) = jedge+1

                  end if

!                 --- Store this edge in ihelp for a while

                  do i = 1, nnode_edge
                     ihelp(i) = kmeshc(ikelmc+ihelp(i))
                  end do
                  if ( debug ) &
                     call prinin ( ihelp, nnode_edge, 'nodes of edge' )

!                 --- Find all elements with this edge and fill nmeshd part 2

                  nodp1 = ihelp(1)
                  do j = kmshd1(nodp1)+1, kmshd1(nodp1+1)

!                 --- Fetch element label

                     jelem = kmshd2(j)

!                    --- Fetch starting position in kmeshc of this element
!                        Fetch the element group of this element
!                        Fetch the number of edges of this element
!                        Fetch the mesh type of this element

                     jkelmc = infoelms(1,jelem)
                     jelgrp = infoelms(2,jelem)
                     jshape = kmeshf(jelgrp)
                     jnpelm = sepgetinpelm ( jelgrp )
                     if ( jshape==-9 ) then
                        jnedge_elem = jnpelm-1
                     else if ( jshape==-10001 .or. jshape==50 .or. &
                               jshape==109 .or.jshape==134 ) then
                        jnedge_elem = 0
                     else
                        jnedge_elem = sepgetnedge_elem ( jshape )
                     end if
                     if ( debug ) then
                        write(irefwr,1) 'nodp1, j, jelem, jelgrp', &
                                         nodp1, j, jelem, jelgrp
                        write(irefwr,1) 'jshape, jnedge_elem', &
                                         jshape, jnedge_elem
                     end if

!                    --- Loop over the edges of this element

                     do ledge = 1, jnedge_elem

!                    --- Get local nodes of this edge and store in jhelp

                        jnode_edge = sepgetnumnodedge ( jshape )
                        if ( nside>0 .and. ishape>0 ) &
                           jnode_edge = jnode_edge + nside
                        if ( debug ) &
                           write(irefwr,1) 'ledge, jnode_edge', &
                                            ledge, jnode_edge
                        if ( jnode_edge==nnode_edge ) then

!                       --- Only when both edges have the same number of nodes

                           call ininodeedges ( jshape, ledge, jhelp, nspec )
                           if ( jshape==-9 ) then

!                          --- Special case of long line element
!                              The element is subdivided in inpelm-1 edges

                              jhelp(1) = ledge
                              jhelp(2) = ledge+1

                           end if

!                          --- Check the other nodes first in one way

                           icount = 0
                           do i = 1, nnode_edge
                              if ( kmeshc(jkelmc+jhelp(i))==ihelp(i) ) &
                                 icount = icount + 1
                           end do
                           if ( icount/=nnode_edge ) then

!                          --- Try the other way around

                              icount = 0
                              do i = 1, nnode_edge
                                 if ( kmeshc(jkelmc+jhelp(i)) == &
                                    ihelp(nnode_edge+1-i) ) &
                                    icount = icount -1
                              end do

                           end if
                           if ( debug ) &
                              write(irefwr,1) 'nnode_edge', nnode_edge
                           if ( abs(icount)==nnode_edge ) then

!                          --- Element found with edge the same as input edge

                              jpd = nmeshd(jelem)
                              nmeshd(jpd+ledge) = nedges*sign(1,icount)
                              goto 400
                           end if

                        end if

                     end do

400               end do

               end if

            end do

!           --- update pointer in kmesh part c

            ikelmc = ikelmc + inpelm

         end do ! irelem = 1, inelem

500   end do  ! ielgrp = 1, nelgrp

!     --- Redo this but now fill nmeshc

      ikelmc = 0
      ipc = nedges + 1
      ielem = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Fetch type of the elements in this group (ishape)
!         and number of elements in this group (inelem)
!         and number of edges per element in this group (nedge_elem)
!         and number of nodes per element in his group

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         if ( ishape==-10001 ) then

!        --- Connection elements must be skipped

            ikelmc = ikelmc+inelem*inpelm
            ielem = ielem+inelem
            go to 600

         else if ( ishape==-9 ) then
            nedge_elem = inpelm-1
         else
            nedge_elem = sepgetnedgeshape ( ishape )
         end if

!        --- Loop over the elements in this group

         do irelem = 1, inelem
            ielem = ielem + 1
            ipd = nmeshd(ielem)

!           --- Loop over the edges

            do jedge = 1, nedge_elem

!           --- Get the label of this edge from nmeshd

               iedge = abs(nmeshd(ipd+jedge))

               if ( nmeshc(iedge)==0 ) then

!              --- New edge found: get the local nodes of this edge and store
!                  in ihelp

                  nnode_edge = sepgetnumnodedge ( ishape )
                  if ( nside>0 .and. ishape>0 ) &
                     nnode_edge = nnode_edge + nside
                  call ininodeedges ( ishape, jedge, ihelp, nspec )
                  if ( ishape==-9 ) then

!                 --- Special case, large line element

                     ihelp(1) = jedge
                     ihelp(2) = jedge+1

                  end if

!                 --- Store this edge in nmeshc

                  nmeshc(iedge) = ipc
                  nmeshc(ipc) = nnode_edge
                  do i = 1, nnode_edge
                     nmeshc(ipc+i) = kmeshc(ikelmc+ihelp(i))
                  end do
                  ipc = ipc + nnode_edge + 1
                  if ( ipc>length ) go to 1000

               end if

            end do

!           --- update pointer in kmesh part c

            ikelmc = ikelmc + inpelm

         end do

600   end do

1000  if ( ipc-1>length ) then

!     --- length too short

         call errint ( ipc, 1 )
         call errint ( length, 2 )
         call errchr ( 'nmeshc', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      length = ipc - 1
      nmesh(4) = nedges

      deallocate ( infoelms, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'infoelms' )

      if ( debug ) then
         write(irefwr,1) 'nedges', nedges
         call sepprinkmeshc
         call sepprinnmeshd
         call sepprinnmeshc
         write(irefwr,*) 'End sepfilledges'
      end if
      call erclos  ('sepfilledges' )

      end
