      subroutine sepfillkmadl3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh ad  part 3: icrossobst,rcrossobst in case of level set
!     Old subroutine topfillkmadl3
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     topology
!     mesh
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLKMESHAA Fill kmesh part aa
!     SEPFILLNMESHC  Fill nmesh part c
!     SEPFILLNMESHE  Fill nmesh part e
!     SEPFILLNMESHH  Fill nmesh part h
!     SEPLEVELKM1AD  Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (1d only) with respect to level
!                    set function
!     SEPLEVELKM2AD  Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (2d only) with respect to level
!                    set function
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmadl3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmadl3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate old arrays nmesha

      if ( debug ) &
         write(irefwr,1) 'kelmaa, kelmad, ndim', &
                          kelmaa, kelmad, ndim

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc

      if ( debug ) write(irefwr,1) 'nmesh(35)', nmesh(35)

      if ( nmesh(35)==0 ) call sepfillnmeshe
      if ( debug ) write(irefwr,1) 'nmesh(38)', nmesh(38)

      if ( nmesh(38)==0 ) call sepfillnmeshh

!     --- Array kmesh ad 3,4 does not exist, create

      if ( debug .and. kelmad>0 ) call prinin ( kmeshad, 7, 'kmesh-ad' )

      if ( kelmad>0 ) then

!     --- kelmad > 0

         if ( kmeshad(2)==0 .or. kmeshad(5)/=kmeshad(7) ) &
            call seplevelkm2ad ( 3 )

      else

!     --- kelmad = 0

         call seplevelkm2ad ( 3 )

      end if  ! ( kelmad>0 )
      if ( debug ) write(irefwr,1) 'ndim', ndim
      select case ( ndim )

      case (1)

!     --- ndim = 1

         call seplevelkm1ad ( 3 )

      case (2, 3)

!     --- ndim = 2, 3

         call seplevelkm2ad ( 3 )

      end select  ! case ( ndim )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmadl3'

      end if
      call erclos ( 'sepfillkmadl3' )
      end subroutine sepfillkmadl3
