      subroutine sepintmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform symmetric compact storage scheme to row-compact storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: rows(:), cols(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     cols           array wirh column numbers
!     rows           Refers to array irowsii
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPINTMAT1     Transform symmetric compact storage scheme to row-compact
!                    storage
!     SEPINTMAT2     Fill data structure in case of symmetric compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmat'
         write(irefwr,1) 'nrusol', nrusol
         call prinin ( irowsii, nrusol+1, 'irowsii' )
         call prinin ( icolsii, size(icolsii), 'icolsii' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      allocate ( cols(size(icolsii)), rows(nrusol+1) )

!     --- Copy icolsii in cols and irowsii in rows

      cols = icolsii
      rows = irowsii

      deallocate ( icolsii )

!     --- Refill array irowsii and icolsii

      if ( .not. IM%symmetric .or. IM%mumps ) then

!     --- Non-symmetric storage

         if ( debug ) write(irefwr,*) 'before sepintmat1'
         allocate ( icolsii(2*size(cols)+nrusol) )
         call sepintmat1 ( nrusol, irowsii, icolsii, size(icolsii), &
                           rows, cols, size(cols) )
      else

!     --- symmetric storage

         if ( associated(IM%diag) ) deallocate(IM%diag)
         allocate ( icolsii(size(cols)+nrusol), IM%diag(nrusol) )
         call sepintmat2 ( nrusol, irowsii, icolsii, size(icolsii), &
                           rows, cols, size(cols), IM%diag )

      end if  ! ( .not. IM%symmetric .or. IM%mumps )
      deallocate ( cols, rows )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintmat'

      end if  ! ( debug )
      call erclos ( 'sepintmat' )

      end subroutine sepintmat
