      subroutine sepprobloop2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     information with respect to the problem is computed
!     and filled in array kprob
!     Only the part that depends on renumbering of nodes is carried out
!     Old subroutine probdfloop
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulecpack
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lensend2, lensendinfo, lentransinfo

!     lensend2       Length of first part of sendinfo
!     lensendinfo    Length of array sendinfo
!                    At input: estimated length
!                    At output: computed length
!     lentransinfo   Length of array transinfo
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROBLOCALTRANS Check if there are any local transformations
!                    If so store information in part n
!     PROBPARBLOCKS  Fill some information about blocks in array blockinfor
!     PROBPARGLOB    Write some quantities form local node to node 1
!     PROBPARINFO    Prepare arrays for solver with respect to parallel solution
!     PROBPARKPROBH  Rearrange array map, recompute nrusol and fill array kprobh
!                    New numbering is used
!     SEPFILLKPROBPC Fill parameters wrt kprobpar in module sepmodulekprob
!     TOPBV1         Fill array KPROB parts h, r and s
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprobloop2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprobloop2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobloc => kprobpart%kprobloc
      iinputkp => kprobpart%iinputloc
      rinputkp => kprobpart%rinputloc
      intarprb => kprobpart%intarprb

!     --- Fill arrays KPROB part h, r and s
!         Renumber KPROB part p
!         Subroutine TOPBV1 uses memory management subroutines

      call topbv1
      nrusol = kprobloc(66)

!     --- Check if there are any local transformations
!         If so store information in part n

      call problocaltrans

      if ( parallel ) then

!     --- parallel computing, prepare readinfo and sendinfo
!         First activate arrays

         lentransinfo = size(transinfo)
         lensendinfo = max(nphys,nunkp)*lentransinfo

!        --- Next compute starting addresses

         call probparinfo ( lensend2 )

!        --- Fill some information about blocks

         call probparblocks

!        --- Make arrays sendinfo, blockunkinfo and innercontr permanent
!            Fill global arrays kprobh and kprobhnew

         call probparglob

!        --- Rearrange array map, recompute nrusol and fill array kprobh
!            Also renumber kprobhglob

         call probparkprobh

         call sepfillkprobpc

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobloc, kprobloc(3), 'kprob' )
         write(irefwr,*) 'End sepprobloop2'

      end if  ! ( debug )
      call erclos ( 'sepprobloop2' )

      end subroutine sepprobloop2
