      subroutine sepprobstart ( iseqprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill problem input read first into isepi2/3
!     Old subroutine erprobstart
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqprob

!     iseqprob       i    Sequence number of problem input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos, niinp, nrinp

!     ipos           Position of input arrays in sepmoduleinput
!     niinp          Length of integer input array
!     nrinp          Length of real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPERR1556     Check if sequence number for reading is in the right range
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make input arrays permanent
!     Extract length
!     Activate problem input arrays
!     Copy contents to final positions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprobstart' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprobstart'
         write(irefwr,1) 'iseqprob', iseqprob
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extract length

      niinp = size(iinputkp)
      nrinp = 0

      if ( associated(rinputkp) ) nrinp = size(rinputkp)
      if ( debug ) write(irefwr,1) 'nrinp', nrinp

!     --- Make input arrays permanent
!         5 extra positions, since these are user arrays

      call seperr1556 ( 27, iseqprob, 'PROBLEM', ipos )
      if ( ierror/=0 ) go to 1000

!     --- Create array iinput in sepmoduleinput (iin)

      allocate ( in(ipos)%iinput(niinp+5), stat = error )
      if ( error/=0 ) call eralloc ( error, niinp+5, 'iinput_prob' )
      if ( ierror/=0 ) go to 1000

!     --- Fill positions 1 to 5 and copy rest of positions

      in(ipos)%iinput(1) = niinp+5    ! length
      in(ipos)%iinput(2:5) = 0
      in(ipos)%iinput(3) = niinp+5    ! length
      in(ipos)%iinput(5) = nprob  ! nprob
      in(ipos)%used = .true.

      in(ipos)%iinput(6:niinp+5) = iinputkp(1:niinp)
      if ( debug ) call prinin ( in(ipos)%iinput, niinp+5, 'iinput' )

      if ( nrinp>0 ) then

!     --- nrinp>0, also write rinput

         allocate ( re(ipos)%rinput(nrinp+5), stat = error )
         if ( error/=0 ) call eralloc ( error, nrinp+5, 'rinput_prob' )
         if ( ierror/=0 ) go to 1000

!        --- Fill positions 1 to 5 and copy rest of positions

         re(ipos)%rinput(1) = dble(nrinp+5)    ! length
         re(ipos)%rinput(2:5) = 0d0
         re(ipos)%rinput(3) = dble(nrinp+5)    ! length

         re(ipos)%rinput(6:nrinp+5) = rinputkp(1:nrinp)
         if ( debug ) call prinrl( re(ipos)%rinput, nrinp+5, 'rinput' )

      end if  ! ( nrinp>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprobstart'

      end if
      call erclos ( 'sepprobstart' )

      end
