      subroutine topelmfacessd ( nouterfaces, outerfaces, kmeshc, &
                                 inpelm, nmeshg, ntria )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 22-05-2014 Extension with shape 22
!        version  1.1  date 11-03-2013 Correction declaration
!        version  1.0  date 08-01-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshc with surface elements corresponding to the faces
!     stored in array outerfaces
!     In this case not all faces have the same number of nodes and as
!     a consequence all quadrilaterals are subdivided into two triangles
!     This subroutine can only be used for three and four node faces
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     face
!     outer_boundary
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nouterfaces, outerfaces(*), inpelm, nmeshg(*), &
              kmeshc(inpelm,*), ntria

!     inpelm         i    number of points per element
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nmeshg         i    nmesh part g
!     nouterfaces    i    Number of outer faces (length of outerfaces)
!     ntria          o    number of triangles in kmeshc
!     outerfaces     i    Array containing the face numbers of the outer faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, i, j, ipg, npelm

!     i              Counting variable
!     iface          Face sequence number
!     ipg            pointer in kmeshaag
!     j              Counting variable
!     npelm          Number of points per face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1382   Value of NPELM is not yet implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all faces in array outerfaces do
!        Store element in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topelmfacessd' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topelmfacessd'
         write(irefwr,1) 'nouterfaces, inpelm', nouterfaces, inpelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpelm/=3 ) then

!     --- inpelm has wrong value

         call errint ( inpelm, 1 )
         call errsub ( 1382, 1, 0, 0 )
         go to 1000

      end if

!     --- For all faces in array outerfaces do

      ntria = 0
      do i = 1, nouterfaces

!     --- Store element in kmeshc

         iface = outerfaces(i)
         ipg = nmeshg(iface)
         npelm = nmeshg(ipg)
         ntria = ntria+1

         if ( debug ) write(irefwr,1) 'i, iface, npelm', i, iface, npelm

         select case ( npelm )
         case(3)

!        --- npelm = 3, i.e. face is a triangle

            do j = 1, inpelm
               kmeshc(j,ntria) = nmeshg(ipg+j)
            end do

         case(4)

!        --- npelm = 4, i.e. face is a quadrilateral, subdivide into 2 triangles

            do j = 1, inpelm
               kmeshc(j,ntria) = nmeshg(ipg+j)
            end do
            ntria = ntria+1
            kmeshc(1,ntria) = nmeshg(ipg+3)
            kmeshc(2,ntria) = nmeshg(ipg+4)
            kmeshc(3,ntria) = nmeshg(ipg+1)

         case(6)

!        --- npelm = 6, i.e. face is a quadratic triangle

            do j = 1, inpelm
               kmeshc(j,ntria) = nmeshg(ipg+2*j-1)
            end do

         case(9)

!        --- npelm = 9, i.e. face is a quadrilateral, subdivide into 2 triangles

            do j = 1, inpelm
               kmeshc(j,ntria) = nmeshg(ipg+2*j-1)
            end do
            ntria = ntria+1
            kmeshc(1,ntria) = nmeshg(ipg+7)
            kmeshc(2,ntria) = nmeshg(ipg+5)
            kmeshc(3,ntria) = nmeshg(ipg+1)

         case default

!        --- wrong value of npelm

            call errint ( npelm,1 )
            call errsub ( 1382, 1, 0, 0 )
            go to 1000

         end select  ! case ( npelm )

      end do

1000  call erclos ( 'topelmfacessd' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topelmfacessd'

      end if  ! ( debug )

      end
