      subroutine topelmfacesvt ( nouterfaces, outerfaces, kmeshc, &
                                 nmeshg, nvertx, ishapesur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshc with surface elements corresponding to
!     the vertices of the faces stored in array outerfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     face
!     outer_boundary
!     topology
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nouterfaces, outerfaces(*), nmeshg(*), nvertx, &
              kmeshc(nvertx,nouterfaces), ishapesur

!     ishapesur      i    shape number of surface element
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nmeshg         i    nmesh part g
!     nouterfaces    i    Number of outer faces (length of outerfaces)
!     nvertx         i    Number of vertices per element
!     outerfaces     i    Array containing the face numbers of the outer faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, i, j, ipg, istep

!     i              Counting variable
!     iface          Face sequence number
!     ipg            pointer in kmeshaag
!     istep          Step size of elemmtloc in elemmt
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all faces in array outerfaces do
!        Store element in kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topelmfacesvt' )

      if ( ishapesur.eq.8 ) then

!     --- triangles with 4 points at a side

         istep = 3

      else

!     --- elements with 3 points at a side

         istep = 2

      end if

!     --- For all faces in array outerfaces do

      do i = 1, nouterfaces

!     --- Store element in kmeshc

         iface = outerfaces(i)
         ipg = nmeshg(iface)
         do j = 1, nvertx
            kmeshc(j,i) = nmeshg(ipg+istep*(j-1)+1)
         end do

      end do

      call erclos ( 'topelmfacesvt' )
      end
