      subroutine topfa07 ( iface, ishape, nndpfa, nodes, nspecp )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.3  date 22-05-2014 Extension with shape 22
!        version  1.2  date 17-07-2012 Fortran 90
!        version  1.1  date 18-12-2002 Extension with prisms
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     return local node numbers for local face iface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iface, ishape, nndpfa, nodes(*), nspecp

!     iface      i    Face sequence number
!     ishape     i    Type number of elements in the mesh or element group
!     nndpfa     o    number of nodes in one face
!     nodes      o    Contains the node numbers that must be filled
!     nspecp     i    number of spectral points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer    dle3(3,1), dle4(6,1), dle5(4,1), &
                 dle6(9,1), dle7(7,1), dle8(10,1), dle9(5,1), &
                 dle10(4,1), dle11(3,4), dle21(18), istart, &
                 dle12(6,4), dle13(4,6), dle14(9,6), dle15(7,4), &
                 dle16(7,4), dle17(4,6), dle18(3,4), dndpfa(18), &
                 i, in, nndped, j, dle22(39)

!     dle10      Data nodes per face for element 10
!     dle11      Data nodes per face for element 11
!     dle12      Data nodes per face for element 12
!     dle13      Data nodes per face for element 13
!     dle14      Data nodes per face for element 14
!     dle15      Data nodes per face for element 15
!     dle16      Data nodes per face for element 16
!     dle17      Data nodes per face for element 17
!     dle18      Data nodes per face for element 18
!     dle21      Data nodes per face for element 21
!     dle22      Data nodes per face for element 22
!     dle3       Data nodes per face for element 3
!     dle4       Data nodes per face for element 4
!     dle5       Data nodes per face for element 5
!     dle6       Data nodes per face for element 6
!     dle7       Data nodes per face for element 7
!     dle8       Data nodes per face for element 8
!     dle9       Data nodes per face for element 9
!     dndpfa     Data nodes per face
!     i          Counting variable
!     in         (i-1) x lrow
!     istart     Starting address
!     j          Counting variable
!     nndped     number of nodes in one edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1692   type of spectral element has not been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data dndpfa / 0, 0, 3, 6, 4, 9, 7, 10, 5, 4, 3, 6, 4, 9, 7, 7, &
                    4, 3 /
      data dle3 / 1, 2, 3 /
      data dle4 / 1, 2, 3, 4, 5, 6 /
      data dle5 / 1, 2, 3, 4 /
      data dle6 / 1, 2, 3, 4, 5, 6, 7, 8, 9 /
      data dle7 / 1, 2, 3, 4, 5, 6, 7 /
      data dle8 / 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 /
      data dle9 / 1, 2, 3, 4, 5 /
      data dle10/ 1, 2, 3, 4 /
      data dle11/ 1, 2, 4,     2, 3, 4,     3, 1, 4,     1, 3, 2 /
      data dle12/ 1, 2, 3, 8, 10, 7, &
                  3, 4, 5, 9, 10, 8, &
                  5, 6, 1, 7, 10, 9, &
                  1, 6, 5, 4,  3, 2 /
      data dle13/ 1, 2, 6, 5, &
                  3, 4, 8, 7, &
                  2, 1, 4, 3, &
                  5, 6, 7, 8, &
                  1, 5, 8, 4, &
                  2, 3, 7, 6 /
      data dle14/  1,  2,  3, 12, 21, 20, 19, 10, 11, &
                   9,  8,  7, 16, 25, 26, 27, 18, 17, &
                   3,  2,  1,  4,  7,  8,  9,  6,  5, &
                  19, 20, 21, 24, 27, 26, 25, 22, 23, &
                   1, 10, 19, 22, 25, 16,  7,  4, 13, &
                   3,  6,  9, 18, 27, 24, 21, 12, 15 /
      data dle15/ 1, 2, 3, 8, 10, 7, 12, &
                  3, 4, 5, 9, 10, 8, 13, &
                  5, 6, 1, 7, 10, 9, 14, &
                  1, 6, 5, 4,  3, 2, 11 /
      data dle16/ 1, 2, 3, 8, 10, 7, 12, &
                  3, 4, 5, 9, 10, 8, 13, &
                  5, 6, 1, 7, 10, 9, 14, &
                  1, 6, 5, 4,  3, 2, 11 /
      data dle17/ 1, 2, 6, 5, &
                  3, 4, 8, 7, &
                  2, 1, 4, 3, &
                  5, 6, 7, 8, &
                  1, 5, 8, 4, &
                  2, 3, 7, 6 /
      data dle18/ 1, 2, 4,     2, 3, 4,     3, 1, 4,     1, 3, 2 /
      data dle21 / 1, 3, 2,   4, 5, 6, &
                   1, 2, 5, 4,  2, 3, 6, 5,  3, 1, 4, 6 /
      data dle22 / 1, 6, 5, 4, 3, 2,  13, 14, 15, 16, 17, 18, &
                   1, 2, 3, 9,15,14,13, 7, 8, &
                   3, 4, 5,11,17,16,15, 9,10, &
                   5, 6, 1, 7,13,18,17,11,12 /
! ======================================================================
!
      call eropen  ('topfa07' )

      if ( ishape== -10001 .and. nspecp==0 ) then

         nndpfa = 0

      else if ( nspecp==0 .and. ishape<=18 ) then

!     --- Standard two or three-dimensional element

         nndpfa = dndpfa(ishape)

      else if ( ishape==21 ) then

!     --- linear prism

         if ( iface<=2 ) then

!        --- First two faces are triangles

            nndpfa = 3

         else

!        --- Last three faces are quadrilaterals

            nndpfa = 4

         end if

      else if ( ishape==22 ) then

!     --- quadratic prism

         if ( iface<=2 ) then

!        --- First two faces are triangles

            nndpfa = 6

         else

!        --- Last three faces are quadrilaterals

            nndpfa = 9

         end if

      else if ( nspecp>0 .and. &
                (ishape==5 .or. ishape==13) ) then
         nndpfa = nspecp*nspecp
         nndped = nspecp

      else

         call errint ( ishape, 1 )
         call errint ( nspecp, 2 )
         call errsub ( 1692, 2, 0, 0 )
         go to 1000

      end if
      if ( nspecp==0 ) then

!     --- No spectral elements

         select case ( ishape )

         case(3)

!        --- ishape = 3, linear triangles

            nodes(1:nndpfa) = dle3 (1:nndpfa,iface)

         case(4)

!        --- ishape = 4, quadratic triangles

            nodes(1:nndpfa) = dle4 (1:nndpfa,iface)

         case(5)

!        --- ishape = 5, bilinear quadrilaterals

            nodes(1:nndpfa) = dle5 (1:nndpfa,iface)

         case(6)

!        --- ishape = 6, biquadratic quadrilaterals

            nodes(1:nndpfa) = dle6 (1:nndpfa,iface)

         case(7)

!        --- ishape = 7, extended quadratic triangles

            nodes(1:nndpfa) = dle7 (1:nndpfa,iface)

         case(8)

!        --- ishape = 8, triangles with 4 nodes per edge

            nodes(1:nndpfa) = dle8 (1:nndpfa,iface)

         case(9)

!        --- ishape = 9, extended bilinear quadrilaterals

            nodes(1:nndpfa) = dle9 (1:nndpfa,iface)

         case(10)

!        --- ishape = 10, extended linear triangles

            nodes(1:nndpfa) = dle10(1:nndpfa,iface)

         case(11)

!        --- ishape = 11, linear tetrahedron

            nodes(1:nndpfa) = dle11(1:nndpfa,iface)

         case(12)

!        --- ishape = 12, quadratic tetrahedron

            nodes(1:nndpfa) = dle12(1:nndpfa,iface)

         case(13)

!        --- ishape = 13, trilinear hexahedron

            nodes(1:nndpfa) = dle13(1:nndpfa,iface)

         case(14)

!        --- ishape = 14, triquadratic hexahedron

            nodes(1:nndpfa) = dle14(1:nndpfa,iface)

         case(15)

!        --- ishape = 15, extended quadratic tetrahedron

            nodes(1:nndpfa) = dle15(1:nndpfa,iface)

         case(16)

!        --- ishape = 16, extended quadratic tetrahedron + &

            nodes(1:nndpfa) = dle16(1:nndpfa,iface)

         case(17)

!        --- ishape = 17, extended trilinear hexahedron

            nodes(1:nndpfa) = dle17(1:nndpfa,iface)

         case(18)

!        --- ishape = 18, extended linear tetrahedron

            nodes(1:nndpfa) = dle18(1:nndpfa,iface)

         case(21)

!        --- ishape = 21, linear prism

            if ( iface<=2 ) then

!           --- First two faces are triangles

               istart = 3*(iface-1)

            else

!           --- Last three faces are quadrilaterals

               istart = 6+4*(iface-3)

            end if
            do i = 1, nndpfa
               nodes(i) = dle21(istart+i)
            end do

         case(22)

!        --- ishape = 22, quadratic prism

            if ( iface<=2 ) then

!           --- First two faces are triangles

               istart = 6*(iface-1)

            else

!           --- Last three faces are quadrilaterals

               istart = 12+9*(iface-3)

            end if
            do i = 1, nndpfa
               nodes(i) = dle22(istart+i)
            end do

         end select  ! case ( ishape )

      else if ( ishape==5 ) then
         do i=1,nndpfa
            nodes(i) = i
         end do
      else if ( ishape==13 ) then
         in = 0
         do i=1,nndped
            do j=1, nndped
               in = in + 1
               if (iface==1) then
                  nodes(in) = j+(i-1)*nndped*nndped
               else if (iface==2) then
                  nodes(in) = 1-j+(i)*nndped*nndped
               else if (iface==3) then
                  nodes(in) = 1-j+(i)*nndped
               else if (iface==4) then
                  nodes(in) = (nndped-1)*nndped*nndped+ &
                             j+(i-1)*nndped
               else if (iface==5) then
                  nodes(in) = 1+(i-1)*nndped+(j-1)*nndped*nndped
               else if (iface==6) then
                  nodes(in) = j*nndped+(i-1)*nndped*nndped
               end if
            end do
         end do
      end if

1000  call erclos  ('topfa07' )

      end
