      subroutine topfillkmeshe1 ( kmeshc, kmeshd1, kmeshd2, kmeshe, &
                                  maxim, inpelm, nelem, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute length of kmesh part e2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, nelem, kmeshc(inpelm,nelem), kmeshd1(*), &
              kmeshd2(*), kmeshe(*), maxim, npoint

!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshd1        i    KMESH part d1
!     kmeshd2        i    contains the rest of kmesh part d
!                         i.e. the nodal point numbers
!     kmeshe         i    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     maxim          o    computed length of kmeshd2
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lenkneigh, &
              isum, ikelm, ih, neli, &
              isum1, j, ielem, k, nodp
      integer, allocatable :: iwork(:), kneigh(:)

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ikelm          Last position used in kmeshc
!     isum           Integer variable to compute a sum
!     isum1          Actual length of array kneigh
!     iwork          integer work array
!     j              Counting variable
!     k              Counting variable
!     kneigh         Contains neigboring nodes
!     lenkneigh      Declared length of array kneigh
!     neli           number of elements coupled to a node
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1381   Too few space available in array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topfillkmeshe1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topfillkmeshe1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenkneigh = npoint
      allocate ( iwork(npoint), kneigh(lenkneigh), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+lenkneigh, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- iwork will be cleared

      iwork=0

!     --- fill kmesh parts e1 and e

      isum=0
      ikelm=0
      do i=1,npoint
         kmeshe(i)=isum
         ih=kmeshd1(i+1)
         neli=ih-ikelm
         isum1=0
         iwork(i)=2
         do j=1,neli
            ielem=kmeshd2(ikelm+j)
            do k=1,inpelm
               nodp=kmeshc(k,ielem)
               if(iwork(nodp)==0) then

!              --- new nodal point

                  iwork(nodp)=1
                  isum=isum+1
                  isum1=isum1+1
                  if(isum1>lenkneigh) then
                     call errint ( isum, 1 )
                     call errint ( lenkneigh, 2 )
                     call errsub ( 1381, 2, 0, 0 )
                     go to 1000
                  end if
                  kneigh(isum1)=nodp
               end if
            end do
         end do

!        --- positions in iwork with iwork(i)>0 will be cleared

         iwork(i)=0
         iwork(kneigh(1:isum1))=0
         ikelm=ih

      end do
      kmeshe(npoint+1)=isum
      maxim=isum

      deallocate ( iwork, kneigh, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topfillkmeshe1'

      end if  ! ( debug )
      call erclos ( 'topfillkmeshe1' )

      end
