      subroutine topfillkmeshe2 ( kmeshc, kmeshd1, kmeshd2, kmeshe1, &
                                  kmeshe2, maxim, inpelm, nelem, &
                                  npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute nodal points connected with nodal points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     neighbour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, nelem, kmeshc(inpelm,nelem), kmeshd1(*), &
              kmeshd2(*), kmeshe1(*), kmeshe2(*), maxim, npoint

!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshd1        i    KMESH part d
!     kmeshd2        i    contains the rest of kmesh part d
!                         i.e. the nodal point numbers
!     kmeshe1        o    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     kmeshe2        o    kmesh e part 2
!     maxim          i    computed length of kmeshd2
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, ikelm, ih, neli, i1, &
              j, ielem, k, nodp, ih1
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     i1
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ikelm          position used in kmeshc
!     isum           Integer variable to compute a sum
!     iwork          integer work array
!     j              Counting variable
!     k              Counting variable
!     neli           number of elements coupled to a node
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1381   Too few space available in array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topfillkmeshe2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topfillkmeshe2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- iwork will be cleared

      iwork=0

!     --- fill kmesh parts e1 and e

      isum=0
      ikelm=0
      do i=1,npoint
         kmeshe1(i)=isum
         ih=kmeshd1(i+1)
         neli=ih-ikelm
         i1=isum
         iwork(i)=2
         do j=1,neli
            ielem=kmeshd2(ikelm+j)
            do k=1,inpelm
               nodp=kmeshc(k,ielem)
               if(iwork(nodp)==0) then

!              --- new nodal point

                  iwork(nodp)=1
                  isum=isum+1
                  kmeshe2(isum)=nodp
               end if
            end do
         end do

!        --- positions in iwork with iwork(i)>0 will be cleared

         iwork(i)=0
         ih1=i1+1
         iwork(kmeshe2(ih1:isum))=0
         ikelm=ih
         if(isum>maxim) then
            call errint ( isum, 1 )
            call errint ( maxim, 2 )
            call errsub ( 1381, 2, 0, 0 )
            go to 1000
         end if
      end do
      kmeshe1(npoint+1)=isum
      maxim=isum

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'topfillkmeshe2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topfillkmeshe2'

      end if  ! ( debug )

      end
