      subroutine topfillkmeshmp ( kmeshm, ncurvs, kmeshm2, kmeshm3, &
                                  ncurvs1, kmeshm21, kmeshm31 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Create array kmesh part m from separate arrays kmeshm part 2 and part 3
!
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     curve
!     interpolation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), ncurvs, kmeshm2(*), kmeshm3(*), &
              ncurvs1, kmeshm21(*), kmeshm31(*)

!     kmeshm         o    KMESH part m, contains nodal point numbers of the
!     curve
!     kmeshm2        i    Part 2 of array kmesh part m (outer curves)
!     kmeshm21       i    Part 2 of array kmesh part m (inner curves)
!     kmeshm3        i    Part 3 of array kmesh part m (outer curves)
!     kmeshm31       i    Part 3 of array kmesh part m (inner curves)
!     ncurvs         i    Number of outer curves in mesh
!     ncurvs1        i    Number of inner curves in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart3, istart4, ninner, i

!     i              Counting variable
!     istart3        starting address of part iii in kmeshm
!     istart4        starting address of part iv in kmeshm
!     ninner         Number of inner curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0008         Print information of curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topfillkmeshmp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topfillkmeshmp'

      end if

!     --- Fill first 4 positions of kmeshm

      istart3 = ncurvs+ncurvs1+6
      istart4 = istart3+kmeshm2(ncurvs+1)-1
      if ( ncurvs1.gt.0 ) istart4 = istart4+kmeshm21(ncurvs1+1)-1
      ninner = ncurvs1
      kmeshm(1) = ninner    ! ninner
      kmeshm(2) = istart3   ! starting address of part iii
      kmeshm(3) = istart4   ! starting address of part iv
      kmeshm(4) = ninner    ! length of part iv

!     --- Copy kmeshm part 2

      do i = 1, ncurvs+1
         kmeshm(4+i) = kmeshm2(i)
      end do

      if ( ncurvs1.gt.0 ) then

!     --- There also curves in kmeshm2

         do i = 1, ncurvs1
            kmeshm(5+i+ncurvs) = kmeshm21(i+1)-1+kmeshm2(ncurvs+1)
         end do

      end if

!     --- Copy kmeshm part 3

      do i = 1, kmeshm2(ncurvs+1)-1
         kmeshm(istart3-1+i) = kmeshm3(i)
      end do
      istart3 = istart3+kmeshm2(ncurvs+1)-1
      if ( ncurvs1.gt.0 ) then

!     --- Inner curves

         do i = 1, kmeshm21(ncurvs1+1)-1
            kmeshm(istart3-1+i) = kmeshm31(i)
         end do
         istart3 = istart3+kmeshm21(ncurvs1+1)-2

      end if

!     --- Fill inner curves

      if ( ninner.gt.0 ) then

!     --- ninner > 0

         do i = 1, ninner
            kmeshm(istart4-1+i) = ncurvs+i

         end do

      end if
      ncurvs = ncurvs+ncurvs1
      call erclos ( 'topfillkmeshmp' )
      if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'End topfillkmeshmp'

      end if

      end
