      subroutine topnodfaces1 ( npoint, nfaces, nmeshg, nmeshk, lennmeshk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-05-2004 Extra parameter iwork
!        version  1.0  date 27-02-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmeshk with all faces connected to the nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nfaces, nmeshg(*), lennmeshk, nmeshk(lennmeshk)

!     lennmeshk     i/o   Length of array nmeshk
!                         At input estimated length
!                         At output computed length
!     nfaces         i    Number of faces
!     nmeshg         i    nmesh part g
!     nmeshk         o    nmesh part k
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, iface, istart, nnodes, inode, ipstart, &
              iwork(npoint)

!     i              Counting variable
!     icount         Counter
!     iface          Face sequence number
!     inode          Node sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     iwork               integer work array of length npoint
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topnodfaces1' )
      debug = .false.

!     --- First compute number of faces connected to nodes and store in iwork:
!         icount := 0
!         Clear array iwork

      icount = 0
      do i = 1, npoint
         iwork(i) = 0
      end do

!     --- For all faces iface do

      do iface = 1, nfaces

!     --- For all nodes inode connected to face iface do

         istart = nmeshg(iface)
         nnodes = nmeshg(istart)
         do i = 1, nnodes

            inode = nmeshg(istart+i)

!           --- iwork(inode) := iwork(inode)+1
!               icount := icount+1

            iwork(inode) = iwork(inode)+1
            icount = icount+1

         end do

      end do

!     --- Check length nmeshk

      if ( lennmeshk.lt.icount+2*npoint ) then

!     --- lennmeshk too small

         call errint ( icount+2*npoint, 1 )
         call errint ( lennmeshk, 2 )
         call errchr ( 'nmeshk', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      lennmeshk = icount+2*npoint

!     --- Fill starting addresses and number of faces per node in nmeshk:
!         ipstart := npoint+1
!         For inode := 1 (1) npoint do

      ipstart = npoint+1
      do inode = 1, npoint

!     --- nmeshk(inode) := ipstart; nmeshk(ipstart) := iwork(inode)
!         ipstart := ipstart+iwork(inode)+1

         nmeshk(inode) = ipstart
         nmeshk(ipstart) = iwork(inode)
         ipstart = ipstart+iwork(inode)+1

      end do

!     --- Fill rest of nmeshk:
!         Set iwork equal to nmeshk(1:npoint)

      do i = 1, npoint
         iwork(i) = nmeshk(i)
      end do

!     --- For all faces iface do

      do iface = 1, nfaces

!     --- For all nodes inode connected to face iface do

         istart = nmeshg(iface)
         nnodes = nmeshg(istart)
         do i = 1, nnodes

            inode = nmeshg(istart+i)

!           --- iwork(inode) := iwork(inode)+1
!               nmeshk(iwork(inode)) := inode

            iwork(inode) = iwork(inode)+1
            nmeshk(iwork(inode)) = iface

         end do

      end do

      if ( debug ) then

!     --- debug = true, print nmeshk

         write ( irefwr, 100 )
100      format ( ' Face numbers corresponding to the nodes'/ )
         call prinnmeshx ( npoint, nmeshk )

      end if

1000  call erclos ( 'topnodfaces1' )

      end
