      subroutine topol00
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-07-1997
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array kmesh part v with information about element groups coupled
!    with element groups
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: nnelgr(:), nnodel(:), nnelgn(:), nodel(:), &
                              nelgno(:)

!     nelgno         Contains the nodes corresponding to the element
!                    group sequence numbers
!     nnelgn         Contains the number of nodes corresponding
!                    to the element groups
!     nnelgr         Temporary array kmeshv
!     nnodel         Contains the number of element groups corresponding
!                    to the nodes
!     nodel          Contains the element group sequence numbers corresponding
!                    to the nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     TOPOL01        Fill array nnodelgr with number of element groups
!                    corresponding to nodes
!     TOPOL02        Fill array nodelgrp with element groups corresponding to
!                    nodes
!     TOPOL03        Fill array kmesh part v
!     TOPOL04        Fill array nnelgnod with number of nodes corresponding
!                    to an element group
!     TOPOL05        Fill array nelgrnod with nodes corresponding to an element
!                    group
!     TOPOL06        Fill array irenum with new numbering
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Standard initializations
!     Fill array nnodelgr with number of element groups corresponding to nodes
!     Fill array nodelgrp with element groups corresponding to nodes
!     Fill array kmesh part v
!     Make array kmesh part v permanent
!     Fill array nnelgnod with number of nodes corresponding to element groups
!     Fill array nelgrnod with nodes corresponding to element groups
!     Renumber nodes in relation to element groups
!     Reset nbuffr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill array nnodelgr with number of element groups corresponding to
!         node

      allocate ( iwork(nelem), ihelp(nelgrp), nnodel(npoint+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem+nelgrp+npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      call topol01 ( nnodel )

!     --- Fill array nodelgrp with element groups corresponding to nodes

      length = nnodel(npoint+1)

      allocate ( nodel(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'nodel' )
      if ( ierror/=0 ) go to 1000

      call topol02 ( nnodel, nodel )

!     --- Fill array kmesh part v

      allocate ( nnelgr(nelgrp+1+2*nelgrp**2), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp+1+2*nelgrp**2, 'nnelgr' )
      if ( ierror/=0 ) go to 1000

      call topol03 ( nnodel, nodel, nnelgr(nelgrp+2+nelgrp**2), &
                     nnelgr, nnelgr(nelgrp+2) )

!     --- Make array kmesh part v permanent

      if ( associated(kmeshv) ) deallocate(kmeshv)
      kelmv = nelgrp+1+nnelgr(1+nelgrp)
      allocate ( kmeshv(kelmv) )
      kmeshv = nnelgr(1:kelmv)

      deallocate ( nnelgr, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nnelgr' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array nnelgnod with number of nodes corresponding to
!         element groups

      allocate ( nnelgn(nelgrp+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp+1, 'nnelgn' )
      if ( ierror/=0 ) go to 1000

      call topol04 ( nnodel, nodel, nnelgn )

!     --- Fill array nelgrnod with nodes corresponding to element groups

      length = nnelgn(1+nelgrp)

      allocate ( nelgno(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'nelgno' )
      if ( ierror/=0 ) go to 1000

      call topol05 ( nnodel, nodel, nnelgn, nelgno )

!     --- Renumber nodes corresponding to element groups

      allocate ( km(iseqkm)%kmeshj(npoint) )
      kmeshj => km(iseqkm)%kmeshj
      kelmj = npoint
      call topol06 ( nnelgn, nelgno, nnodel )

      deallocate ( nnodel, nnelgn, nodel, nelgno, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nnodel' )

1000  end subroutine topol00
