      subroutine topol03 ( nnodelgr, nodelgrp, iwork, kmeshv1, kmeshv2 )
! ======================================================================
!
!        programmer     Guus Segal
!        version  2.0  date 25-02-2018
!        version  1.0  date 22-07-1997
!
!   copyright (c) 1997-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array kmeshv1 with number of element groups corresponding
!    to an element group and kmeshv2 with element group numbers
! **********************************************************************
!
!                       KEYWORDS
!
!  topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nnodelgr(*), nodelgrp(*), &
              iwork(nelgrp,nelgrp), kmeshv1(*), kmeshv2(*)

!     iwork           integer work array of size nelgrp x nelgrp
!                     iwork(i,j) = 1 means that element group i is coupled
!                     to element group i
!                     iwork(i,j) = 0 means not coupled
!     kmeshv1     o    Kmesh part v, first part
!                     Contains number of element groups corresponding
!                     to element groups
!     kmeshv2     o    Kmesh part v, second part
!                     Contains element groups corresponding to element groups
!     nnodelgr   i    Contains the number of element groups corresponding
!                     to the nodes
!     nodelgrp   i    Contains the element group sequence numbers corresponding
!                     to the nodes
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, j, iend, istart, nodp

!     i          General loop parameter
!     iend       Last position used at a part
!     istart     First position used at a part
!     isum       Integer variable to compute a sum
!     j          General loop parameter
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Fill array iwork:
!     for nodp := 1 (1) npoint do
!        istart := nnodelgr(nodp)+1
!        iend := nnodelgr(nodp+1)
!        for i := istart, iend do
!           for j := istart (1) iend do
!              iwork(nodelgrp(i),nodelgrp(j)) = 1
!     Fill array kmeshv1:
!     kmeshv1(1) := 0
!     For i := 1 (1) nelgrp do
!         iwork(i,i) = 1
!     For i := 1 (1) nelgrp do
!        isum := 0
!        For j := 1 (1) nelgrp do
!           if ( iwork(i,j)=1 ) then
!              isum := isum+1
!        kmeshv1(i+1) := isum+kmeshv1(i)
!     Fill array kmeshv2:
!     For i := 1 (1) nelgrp do
!        For j := 1 (1) nelgrp do
!           if ( iwork(i,j)=1 ) then
!              kmeshv1(i) := kmeshv1(i)+1
!              kmeshv2(kmeshv1(i)) := j
!     reset kmeshv1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Clear array iwork

      iwork = 0

!     --- Fill array iwork

      do nodp = 1, npoint
         istart = nnodelgr(nodp)+1
         iend = nnodelgr(nodp+1)
         do i = istart, iend
            do j = istart, iend
               iwork(nodelgrp(i),nodelgrp(j)) = 1
            end do
         end do
      end do

!     --- Make diagonal equal to 0

      do i = 1, nelgrp
         iwork(i,i) = 0
      end do

!     --- Fill array kmeshv1

      kmeshv1(1) = 0
      do i = 1, nelgrp
         isum = 0
         do j = 1, nelgrp
            if ( iwork(i,j).eq.1 ) isum = isum+1
         end do
         kmeshv1(i+1) = isum+kmeshv1(i)
      end do

!     --- Fill array kmeshv2

      do i = 1, nelgrp
         do j = 1, nelgrp
            if ( iwork(i,j).eq.1 ) then
               kmeshv1(i) = kmeshv1(i)+1
               kmeshv2(kmeshv1(i)) = j
            end if
         end do
      end do

!     --- reset kmeshv1

      do i = nelgrp, 1, -1
         kmeshv1(i+1) = kmeshv1(i)
      end do
      kmeshv1(1) = 0

      end
