      function dnrm2(n, dx, incx)
      implicit double precision (a-h,o-z)
      double precision dx(*)
      data zero, one /0d0,1d0/
!
!        version 1.1    date   22-08-1991 (Zdenek: no dimension, cdc, ce)
!
!     euclidean norm of the n-vector stored in dx() with storage
!     increment incx .
!     if    n.le.0 return with result = 0.
!     if n.ge.1 then incx must be .ge. 1
!
!           c.l.lawson, 1978 jan 08
!
!     four phase method     using two built-in constants that are
!     hopefully applicable to all machines.
!         cutlo = maximum of  sqrt(u/eps)  over all known machines.
!         cuthi = minimum of  sqrt(v)      over all known machines.
!     where
!         eps = smallest no. such that eps + 1..gt.1.
!         u   = smallest positive no.   (underflow limit)
!         v   = largest  no.            (overflow  limit)
!
!     brief outline of algorithm..
!
!     phase 1    scans zero components.
!     move to phase 2 when a component is nonzero and.le.cutlo
!     move to phase 3 when a component is.gt.cutlo
!     move to phase 4 when a component is.ge.cuthi/m
!     where m = n for x() real and m = 2*n for complex.
!
!     values for cutlo and cuthi..
!     from the environmental parameters listed in the imsl converter
!     document the limiting values are as follows..
!     cutlo, s.p.   u/eps = 2**(-102) for  honeywell.  close seconds are
!                   univac and dec at 2**(-103)
!                   thus cutlo = 2**(-51) = 4.44089e-16
!     cuthi, s.p.   v = 2**127 for univac, honeywell, and dec.
!                   thus cuthi = 2**(63.5) = 1.30438e19
!     cutlo, d.p.   u/eps = 2**(-67) for honeywell and dec.
!                   thus cutlo = 2**(-33.5) = 8.23181d-11
!     cuthi, d.p.   same as s.p.  cuthi = 1.30438d19
!     data cutlo, cuthi / 8.232d-11,  1.304d19 /
!     data cutlo, cuthi / 4.441e-16,  1.304e19 /
      data cutlo, cuthi /8.232d-11,1.304d19/
!
      if (n.gt.0) go to 10
      dnrm2 = zero
      go to 140
!
   10 assign 30 to next
      sum = zero
      nn = n*incx
!                                                 begin main loop
      i = 1
   20 go to next, (30, 40, 70, 80)
   30 if (abs(dx(i)).gt.cutlo) go to 110
      assign 40 to next
      xmax = zero
!
!                        phase 1.  sum is zero
!
   40 if (dx(i).eq.zero) go to 130
      if (abs(dx(i)).gt.cutlo) go to 110
!
!                                prepare for phase 2.
      assign 70 to next
      go to 60
!
!                                prepare for phase 4.
!
   50 i = j
      assign 80 to next
      sum = (sum/dx(i))/dx(i)
   60 xmax = abs(dx(i))
      go to 90
!
!                   phase 2.  sum is small.
!                             scale to avoid destructive underflow.
!
   70 if (abs(dx(i)).gt.cutlo) go to 100
!
!                     common code for phases 2 and 4.
!                     in phase 4 sum is large.  scale to avoid overflow.
!
   80 if (abs(dx(i)).le.xmax) go to 90
      sum = one + sum*(xmax/dx(i))**2
      xmax = abs(dx(i))
      go to 130
!
   90 sum = sum + (dx(i)/xmax)**2
      go to 130
!
!
!                  prepare for phase 3.
!
  100 sum = (sum*xmax)*xmax
!
!
!     for real or d.p. set hitest = cuthi/n
!     for complex      set hitest = cuthi/(2*n)
!
  110 hitest = cuthi/float(n)
!
!                   phase 3.  sum is mid-range.  no scaling.
!
      do 120 j=i,nn,incx
         if (abs(dx(j)).ge.hitest) go to 50
         sum = sum + dx(j)**2
  120 continue
      dnrm2 = sqrt(sum)
      go to 140
!
  130 continue
      i = i + incx
      if (i.le.nn) go to 20
!
!              end of main loop.
!
!              compute square root and adjust for scaling.
!
      dnrm2 = xmax*sqrt(sum)
  140 continue
      return
      end
