      subroutine fnmooney ( eps, matpar, s, se )
! ======================================================================
!
!        programmer     Marcel Rutten
!        version 1.0    date   21-01-1998
!
!
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
! fill estimate of 2nd Piola Kirchoff stress plus tangential stiffness
! matrix for one integration point.
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      solid
!      material_behaviour
!      neo_hookean
!      mooney_rivlin
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision eps(6), matpar(21), s(6), se(6,6)

!     eps      i     Green-Lagrange strain
!     matpar   i     material properties
!     s        o     2nd Piola Kirchhoff stress estimate
!     se       o     tangential stiffness matrix
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision g, k

!     g
!     k
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      G = matpar(1)
      K = matpar(2)

      if ( K.ne.0d0 ) then

!     --- Mooney-Rivlin material

        se(1,2) = 2d0*K
        se(1,3) = 2d0*K
        se(2,1) = 2d0*K
        se(2,3) = 2d0*K
        se(3,1) = 2d0*K
        se(3,2) = 2d0*K
        se(4,4) = -K
        se(5,5) = -K
        se(6,6) = -K

        s(1) = G + K * 2d0 * ( eps(2) + eps(3) + 1d0 )
        s(2) = G + K * 2d0 * ( eps(1) + eps(3) + 1d0 )
        s(3) = G + K * 2d0 * ( eps(1) + eps(2) + 1d0 )
        s(4) = -K * 2d0 * eps(4)
        s(5) = -K * 2d0 * eps(5)
        s(6) = -K * 2d0 * eps(6)

      else

!     --- Neo Hookean material

        s(1) = G
        s(2) = G
        s(3) = G

!       --- yes, se is completely zero in this case

      end if

      end
