      subroutine own_curve ( ifunc, icurnr, coor, npoints, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     User written subroutine that fills the coordinates along a curve
!     The user creates the coordinates
!     npoints is defined in the input file and may not be changed
! **********************************************************************
!
!                       KEYWORDS
!
!     curves
!     mesh_generation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifunc, icurnr, npoints, ndim
      double precision coor(ndim,*)

!     coor           o    array of length ndim x npoints containing the
!                         co-ordinates of the nodal points
!                         coor must be filled by the user
!     icurnr         i    Curve sequence number
!     ifunc          i    Choice parameter
!                         The value is set in the input file
!     ndim           i    Dimension of the space.
!     npoints        i    Number of points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision width, dx

!     dx             step size in x-direction
!     i              Counting variable
!     width          Length of the line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getconst

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     GETCONST       Get the value of a constant in the input block constants
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     349   Subroutine not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'own_curve' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from own_curve'

      end if
      if ( ierror.ne.0 ) go to 1000

      if ( ifunc.eq.1 ) then

!     --- ifunc = 1, example of filling a straight line
!         This line goes from (0,0) to ($width,0)

         width = getconst ( 'width' )  ! end point of line
         dx = width/(npoints-1)        ! step size in x-direction
         do i = 1, npoints
            coor(1,i) = (i-1)*dx       ! all x-coordinates
            coor(2,i) = 0d0            ! all y-coordinates are 0
         end do  ! i = 1, npoints

      else

!     --- ifunc # 1, not implemented

         call errchr ('own_curve',1 )
         call errsub ( 349, 0, 0, 1 )
         call instop

      end if  ! ( ifunc.eq.1 )

1000  call erclos ( 'own_curve' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End own_curve'

      end if

      end
