      subroutine el0606(iuser, user, kfirst, jelp, isub, isup, array, &
                        icheld, crstrm, jrhs1, jrhs2, method)
! ======================================================================
!
!        programmer    Luc Timmermans
!        version  2.2  date 23-08-2001 New call to elusr3
!        version  2.1  date 03-09-2000 Extension for scalars
!        version  2.0  date 19-09-1995  Addition of crossterm. Patrick Anderson
!        version  1.0  date 05-08-1992  based on el0602
!
!   copyright (c) 1992-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELM606
!     or ELD606 in the case that IFIRST = 0
!     the parameters in the common blocks celint and celiar get a start
!     value
!     array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), jelp, kfirst, isub, isup, icheld, jrhs1, jrhs2, &
              method, crstrm
      double precision user(*), array(*)

!     array   o      real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    1: alpha11
!                    2: alpha12
!                    3: alpha22
!                    4: u1
!                    5: u2
!                    6: beta
!                    7: gamma
!                    8: f1
!                    9: f2
!                    the coefficients 1-9 require exact n (number of
!                    nodal points) positions
!     crstrm   i     ?
!     icheld   i     choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    0:   Call by ELM606
!                    >0:  Call by ELD606 (not yet done)
!     isub    o      integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup    o      integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     user array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jelp    o      Indication of the type of basis function subroutine must
!                    be called by ELM606, possible vaues:
!                    1:  subroutine ELP640  (one-dimensional)
!                    (more values not yet done)
!     jrhs1   o      information of the weight function w and the function to
!                    be integrated f1, possibilities:
!                    0   f1 = 0
!                    1   f1 constant, w variable
!                    2   general
!     jrhs2   o      same as jrhs1 for f2
!     kfirst  o      This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     method  o      choice parameter, see ELM606
!     user    i      user array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, kchois, ishift
      double precision rn

!     i             counting variable
!     ishift        Shift in coefficient sequence number
!     isubcf        under bound for coefficients check
!     j             counting variable
!     kchois        indication what type of parameters must be computed
!                   possible values:
!                   0:  standard method, the calling subroutine is ELM606
!                       it is supposed that an element matrix must be computed
!                   1:  subroutine ELD606 calls this subroutine, only
!                       derivatives must be computed
!                   2:  subroutine ELD606 calls this subroutine, &
!                       a user function defined as first parameter in
!                       IUSER/USER must be filled in elemvc (icheld=9)
!                   3:  subroutine ELD606 calls this subroutine, &
!                       alpha times the gradient must be stored in ELEMVC
!                       the first 3 parameters of IUSER/USER are used
!     ncoef         number of coefficients in the equation
!     rn            double precision value of inpelm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!      70   alpha11 = 0 or alpha22 = 0
!    1605   method has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('el0606')
      call elcomm
      ncomp = 2
      ishift = 0

!     --- compute kchois

      if (icheld.eq.0) then

!     --- call by elm606, all coefficients required

         kchois = 0
      end if

!     --- set parameters depending on the type numbers

      n = inpelm
      m = inpelm
      rn = n
      mone = int(sqrt(rn))
      jelp = 1
      jdercn = 1
      nvert = 4
      ndimlc = 2

!     --- fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ncoef =9
      if ( ncoef.gt.0 ) then
         ipos = iuser(ielgrp + 5)
         if (ipos.lt.7) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub (42, 5, 0, 0)
            go to 1000
         end if
         nuser = 0
         do i = 1 , ncoef
            call elusr3 (i, array(1+(i-1)*n), iuser, user, i+ishift)
         end do

!        --- compute method

         method = inirepscal ( iuser(ipos) )

!        --- check value of method

         if ( method.lt.0 .or. method.gt.1 ) then
            call errint ( method, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1605, 4, 0, 0)
            go to 1000
         end if

!        --- test array lengths, number of nodes and number of degrees of
!            --- freedom

         call prinlg ( 'iuser', ipos-1, user, iuser)
         call prinlg ( 'user', -nuser, user, iuser)
      end if

!     --- next consider all terms carefully in order to get the cheapest
!         computation

      if ( kchois.eq.0 .and. matrix .and. &
           (method.eq.0 .or. method.eq.1) ) then

!     --- compute jsecnd, idim and jzero
!         first check if alpha11 and alpha22 are unequal to zero

         if ( ind(1).eq.0 .or. ind(3).eq.0 ) then

!        --- alpha11 or alpha22 is zero

            call errwar (70, 0, 0, 0)
         end if

!        --- second derivative terms:
!            compute jsecnd

         if (ind(2).eq.0) then

!        --- alpha 12 = 0

            if ( ind(1).eq.0 .and. ind(3).eq.0 ) then

!           --- alpha = 0

               jsecnd = 0
            else if ( ind(1).le.0 ) then

!           --- alpha 11 is constant

               if (ind(3).eq.ind(1)) then

!              --- alpha 11 = alpha 22

                  jsecnd = 1
               else if (ind(3).le.0) then

!              --- alpha 22 is constant

                  if ( abs(array(1)-array(1+2*n)).gt.sqreps* &
                       abs(array(1)) ) then

!                 --- alpha 11 # alpha 22

                     jsecnd = 4
                  else

!                 --- alpha 11 = alpha 22

                     jsecnd = 1
                  end if
               else

!              --- alpha 22 is variable

                  jsecnd = 4
               end if
            else if ( ind(1).eq.ind(3) .and. &
                      mst(1).eq.mst(3) ) then

!           --- alpha 11 = alpha 22  variable

               jsecnd = 2
            else

!           --- general, except for alpha 12 = 0

               jsecnd = 4
            end if
         else

!        --- alpha 12 # 0

            if ( ind(1).le.0 ) then

!           --- alpha 11 = constant

               if ( ind(3).eq.ind(1) ) then

!              --- alpha 11 = alpha 22

                  jsecnd = 3
               else if ( ind(3).le.0 ) then

!              --- alpha 22 is constant

                  if ( abs(array(1)-array(1+2*n)).gt.sqreps* &
                       abs(array(1)) ) then

!                 --- alpha 11 # alpha 22

                     jsecnd = 4
                  else

!                 --- alpha 11 = alpha 22

                     jsecnd = 3
                  end if
               else

!              --- alpha 22 is variable

                  jsecnd = 4
               end if
            else

!           --- alpha 11 is variable

               if ( ind(1).eq.ind(3) .and. &
                    mst(1).eq. mst(3) ) then

!              --- alpha 11 = alpha 22  variable

                  jsecnd = 3
               else

!              --- general

                  jsecnd = 4
               end if
            end if
         end if

!        --- first derivative terms:
!            compute idim and first component idim

         kfirst = 1
         if ( ind(4).eq.0 ) then

!        --- u1 = 0

            if (ind(5).eq.0) then

!           --- u2 = 0

               idim = 0
            else

!           --- u1 = 0, u2 # 0

               idim = 1
               kfirst = 2
            end if
         else if (ind(5).eq.0) then

!        --- u1 # 0, u2 =0

            idim = 1
         else

!        --- general

            idim = 2
         end if

!        --- constant part of the equation:
!            compute jzero

         if (ind(6).eq.0) then

!        --- beta = 0

            jzero = 0
         else if ( ind(6).lt.0 .or. ind(6).eq.2002 ) then

!        --- beta is constant (per element)

            jzero = 1
         else

!        --- general

            jzero = 2
         end if
      end if
      if  ( kchois.eq.0 ) then
         if ( ind(7).eq.0 ) then

!        --- Crossterm = 0

            crstrm = 0
         else if ( ind(7).lt.0 .or. ind(7).eq.2002 ) then

!        --- gamma is constant (per element)

            crstrm = 1
         else

!        --- general

            crstrm = 2
         end if
      end if
      if ( kchois.eq.0 .and. vector .and. &
          (method.eq.0 .or. method.eq.1) ) then

!     --- compute jrhs1

         if ( ind(8).eq.0 ) then

!        --- f1 = 0, no 1st component of the right-hand side function

            jrhs1 = 0
         else if ( ind(8).lt.0 .or. ind(8).eq.2002 ) then

!        --- f1 is constant (per element)

            jrhs1 = 1
         else

!        --- general

            jrhs1 = 2
         end if

!        --- compute jrhs2

         if ( ind(9).eq.0 ) then

!        --- f2 = 0, no 2nd component of the right-hand side function

            jrhs2 = 0
         else if ( ind(9).lt.0 .or. ind(9).eq.2002 ) then

!        --- f2 is constant (per element)

            jrhs2 = 1
         else

!        --- general

            jrhs2 = 2
         end if
      end if

!     --- check which parameters depend on the space coordinates
!         compute isub and isup

      if ( kchois.eq.0 .and. matrix .and. &
          (method.eq.0 .or. method.eq.1) ) then
         isubcf = 1
      else
         isubcf = 9
      end if
      do i = isubcf, ncoef
         isub = i

!        --- if ind(i) > 0 then paramater i corresponds to a function

         if (ind(i).gt.0) goto 300
      end do
      isub = ncoef + 1
300   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if (ind(i).gt.0) goto 500
      end do

!     --- check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         set ind(lengthmst)

500   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1, isup
               if (ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
      end do
1000  if (ierror.ne.0) call instop
      call erclos ('el0606')
      end
