      subroutine el0800subp ( isub, isup, ncoef, jelp, iqmat1, metup )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameters isub and isup defining the lowest and largest
!     coefficient depending on space
!     Adapt some paramaters
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isub, ncoef, jelp, iqmat1, metup, isup

!     iqmat1         i    Indication whether the right-hand side has to be
!                         transformed
!     isub          i/o   Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     metup          i    Upwind parameter
!     ncoef          i    Number of coefficients in the equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isubcf, i, j
      logical funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     isubcf         Under bound for coefficients check
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800subp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800subp'
         call prinin ( ind, ncoef, 'ind/1' )
         call prinin ( ivecnums, ncoef, 'ivecnums/1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isubcf = isub
      do i = isubcf, ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) exit
         if ( i==ncoef ) isub = ncoef + 1

      end do

      isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) exit
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

      ist(isub:isup) = 0
      funchk = .false.
      do i = isub, isup

         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
                  ivecnums(j) = 0
               end if
            end do

         end if

         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do
      if ( mod(metup,20)>=7 .and. mod(metup,20)<=11 ) then
         jsol = 1
         jderiv = 1
      end if
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2
      if ( jelp==4 .and. jcart==1 .and. metup==0 .and. &
           irule==1 ) kstep = 2
      if ( iqmat1==1 ) iweigh = iweigh + 20
      if ( igausp==1 .and. mod(iweigh,10)==0 ) iweigh = iweigh+1

1000  call erclos ( 'el0800subp' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         call prinin ( ivecnums, ncoef, 'ivecnums' )
         write(irefwr,*) 'End el0800subp'

      end if  ! ( debug )

      end
