      subroutine el100116 ( icheld, dudx, elemvc, array, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 16-04-2014 Debug statements
!        version  1.0  date 08-08-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     with respect to subroutine ELD800
!     icheld = 6, 16, 26  Compute - A gradient
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ndim
      double precision  dudx(m,ndimlc), elemvc(ndimlc,*), array(m,*)

!     array    i     Array containing the coefficients of the second order
!                    part of the elliptic equation
!                    At this moment array is only used for ICHELD = 6 or 16
!                    Array contains the coefficientsin the integration points
!                    in the sequence:
!                         1D                  2D            3D
!                    1:   alpha 11         alpha 11       alpha 11
!                    2:                    alpha 12       alpha 12
!                    3:                    alpha 22       alpha 13
!                    4:                                   alpha 22
!                    5:                                   alpha 23
!                    6:                                   alpha 33
!                    Array(.,1) contains the coefficient alpha 11
!                    Array(.,2) contains the coefficient alpha 12
!                    etc.
!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    Definitions:
!                    6:   compute - A grad u, where A is the matrix
!                         corresponding to the second order term of the elliptic
!                         differential equation
!                   16:   See 6, however, now defined per element
!                   26:   See 6, however, now derivatives are computed
!                         with general weight 1
!     ndim     i     Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision som

!     i              Counting variable
!     som            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el100116' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el100116'
         write(irefwr,1) 'jsecnd, ndim, jdercn', jsecnd, ndim, jdercn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==1 ) then

!     --- NDIM = 1, 1 dimensional space
!         The formula to be computed is:
!         g1 = - a11 ux

         if ( jdercn==0 ) then

!        --- Derivatives are constant

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = -dudx(1,1) * array(i,1)
               end do
            else
               som = 0d0
               do i = 1, n
                  som = som-dudx(1,1)*array(i,1)
               end do
               elemvc(1,1) = som/n
            end if

         else

!        --- General case

            if ( icheld==6 .or. icheld==36 ) then
               do i = 1, n
                  elemvc(1,i) = -dudx(i,1) * array(i,1)
               end do
            else
               som = 0d0
               do i = 1, n
                  som = som-dudx(i,1) * array(i,1)
               end do
               elemvc(1,1) = som/n
            end if

         end if

      else if ( ndim==2 ) then

!     --- NDIM = 2, 2 dimensional space
!         The formula to be computed is:
!         g1 = - a11 ux - a12 uy
!         g2 = - a21 ux - a22 uy

         if ( jdercn==0 ) then

!        --- Derivatives are constant

            if ( jsecnd<=2 ) then

!           --- jsecnd = 1,2 i.e. a12 = 0, a11 = a22 = a

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = -dudx(1,1) * array(i,1)
                     elemvc(2,i) = -dudx(1,2) * array(i,1)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som-dudx(1,1) * array(i,1)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som-dudx(1,2) * array(i,1)
                  end do
                  elemvc(2,1) = som/n
               end if

            else

!           --- jsecnd = 3,4 i.e. general

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(1,1) * array(i,1) &
                                   - dudx(1,2) * array(i,2)
                     elemvc(2,i) = - dudx(1,1) * array(i,2) &
                                   - dudx(1,2) * array(i,3)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,1) * array(i,1) &
                               - dudx(1,2) * array(i,2)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,1) * array(i,2) &
                               - dudx(1,2) * array(i,3)
                  end do
                  elemvc(2,1) = som/n
               end if

            end if

         else

!        --- General element

            if ( jsecnd<=2 ) then

!           --- jsecnd = 1,2 i.e. a12 = 0, a11 = a22 = a

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(i,1) * array(i,1)
                     elemvc(2,i) = - dudx(i,2) * array(i,1)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,1)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,2) * array(i,1)
                  end do
                  elemvc(2,1) = som/n
               end if

            else

!           --- jsecnd = 3,4 i.e. general

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(i,1) * array(i,1) &
                                   - dudx(i,2) * array(i,2)
                     elemvc(2,i) = - dudx(i,1) * array(i,2) &
                                   - dudx(i,2) * array(i,3)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,1) &
                               - dudx(i,2) * array(i,2)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,2) &
                               - dudx(i,2) * array(i,3)
                  end do
                  elemvc(2,1) = som/n
               end if

            end if

         end if

      else

!     --- NDIM = 3, 3 dimensional space
!         The formula to be computed is:
!         g1 = - a11 ux - a12 uy - a13 uz
!         g2 = - a21 ux - a22 uy - a23 uz
!         g3 = - a31 ux - a32 uy - a33 uz

         if ( jdercn==0 ) then

!        --- Derivatives are constant

            if ( jsecnd<=2 ) then

!           --- jsecnd = 1,2 i.e. aij = 0, aii = a

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(1,1) * array(i,1)
                     elemvc(2,i) = - dudx(1,2) * array(i,1)
                     elemvc(3,i) = - dudx(1,3) * array(i,1)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,1) * array(i,1)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,2) * array(i,1)
                  end do
                  elemvc(2,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,3) * array(i,1)
                  end do
                  elemvc(3,1) = som/n
               end if

            else

!           --- jsecnd = 3,4 i.e. general

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(1,1) * array(i,1) &
                                   - dudx(1,2) * array(i,2) &
                                   - dudx(1,3) * array(i,3)
                     elemvc(2,i) = - dudx(1,1) * array(i,2) &
                                   - dudx(1,2) * array(i,4) &
                                   - dudx(1,3) * array(i,5)
                     elemvc(3,i) = - dudx(1,1) * array(i,3) &
                                   - dudx(1,2) * array(i,5) &
                                   - dudx(1,3) * array(i,6)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,1) * array(i,1) &
                               - dudx(1,2) * array(i,2) &
                               - dudx(1,3) * array(i,3)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,1) * array(i,2) &
                               - dudx(1,2) * array(i,3) &
                               - dudx(1,3) * array(i,5)
                  end do
                  elemvc(2,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(1,1) * array(i,3) &
                               - dudx(1,2) * array(i,5) &
                               - dudx(1,3) * array(i,6)
                  end do
                  elemvc(3,1) = som/n
               end if

            end if

         else

!        --- General element

            if ( jsecnd<=2 ) then

!           --- jsecnd = 1,2 i.e. a12 = 0, a11 = a22 = a

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(i,1) * array(i,1)
                     elemvc(2,i) = - dudx(i,2) * array(i,1)
                     elemvc(3,i) = - dudx(i,3) * array(i,1)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,1)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,2) * array(i,1)
                  end do
                  elemvc(2,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,3) * array(i,1)
                  end do
                  elemvc(3,1) = som/n
               end if

            else

!           --- jsecnd = 3,4 i.e. general

               if ( icheld==6 .or. icheld==36 ) then
                  do i = 1, n
                     elemvc(1,i) = - dudx(i,1) * array(i,1) &
                                   - dudx(i,2) * array(i,2) &
                                   - dudx(i,3) * array(i,3)
                     elemvc(2,i) = - dudx(i,1) * array(i,2) &
                                   - dudx(i,2) * array(i,4) &
                                   - dudx(i,3) * array(i,5)
                     elemvc(3,i) = - dudx(i,1) * array(i,3) &
                                   - dudx(i,2) * array(i,5) &
                                   - dudx(i,3) * array(i,6)
                  end do
               else
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,1) &
                               - dudx(i,2) * array(i,2) &
                               - dudx(i,3) * array(i,3)
                  end do
                  elemvc(1,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,2) &
                               - dudx(i,2) * array(i,3) &
                               - dudx(i,3) * array(i,5)
                  end do
                  elemvc(2,1) = som/n
                  som = 0d0
                  do i = 1, n
                     som = som - dudx(i,1) * array(i,3) &
                               - dudx(i,2) * array(i,5) &
                               - dudx(i,3) * array(i,6)
                  end do
                  elemvc(3,1) = som/n
               end if

            end if

         end if

      end if

1000  call erclos ( 'el100116' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el100116'

      end if  ! ( debug )
      end
