      subroutine el1010 ( s, u, f, rmult, if, istep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-1993
!
!   copyright (c) 1993-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute matrix vector multiplication rmult s u = f, where
!     rmult is a multiplication factor, s the matrix and u and f vectors
!     The matrices and vectors have all size n (see common celint)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istep, if
      double precision s(n,n), u(n), f(istep,n), rmult

!     f       o    right-hand-side vector
!     if      i    first position in right-hand-side vector to be filled
!     istep   i    step to be used in right-hand-side vector
!     rmult   i    multiplication factor
!     s       i    matrix S
!     u       i    vector to be multiplied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision sum

!     i       Counting variable
!     j       Counting variable
!     sum     Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   f(i) = rmult s(i,j) u(j) j=1,n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( jadd.eq.0 ) then

!     --- jadd = 0, fill f

         do i = 1, n
            sum = 0d0
            do j = 1, n
               sum = sum + s(i,j) * u(j)
            end do
            f(if,i) = rmult * sum
         end do
      else

!     --- jadd = 1, add to f

         do i = 1, n
            sum = 0d0
            do j = 1, n
               sum = sum + s(i,j) * u(j)
            end do
            f(if,i) = f(if,i) + rmult * sum
         end do
      end if
      end
