      subroutine el1014 ( u, dudx, x, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 12-05-1999 Minor correction
!        version  1.1  date 12-02-1999 Include commons
!        version  1.0  date 25-01-1997
!
!   copyright (c) 1997-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The values of u, dudx and x are reset to the values in the centroid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision  dudx(m,ndimlc,*), u(m,*), x(n,*)
      integer ishape

!     dudx    i/o    Real m x ndimlc x ncomp array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du_i/ dx_j(x_k) = dudx ( k, j, i )
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*,*) is filled because dudx is then a constant
!                    per element
!                    At output the value in the centroid is filled in dudx(1,..)
!     ishape   i     Shape number of elements
!     u       i/o    array of length m x ncomp in which the values of
!                    u in the integration points are stored the sequence:
!                    u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                     i   j
!                    At output the value in the centroid is filled in u(1,..)
!     x       i/o    array containing the coordinates of the nodal points
!                    according to:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                     i              i              i
!                    At output the value in the centroid is filled in x(1,..)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision h

!     h        help variable to compute a constant
!     i        General loop variable
!     j        General loop variable
!     k        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ndimlc.eq.2 ) then

!     --- ndim = 2

         if ( ishape.eq.3 .or. ishape.eq.5 ) then

!        --- Linear triangle or quadrilateral

            do i = 2, n
               x(1,1) = x(1,1) + x(i,1)
               x(1,2) = x(1,2) + x(i,2)
               do k = 1, ncomp
                  u(1,k) = u(1,k) + u(i,k)
                  dudx(1,1,k) = dudx(1,1,k) + dudx(i,1,k)
                  dudx(1,2,k) = dudx(1,2,k) + dudx(i,2,k)
               end do
            end do
            h = 1d0/n
            x(1,1) = x(1,1) * h
            x(1,2) = x(1,2) * h
            do k = 1, ncomp
               u(1,k) = u(1,k) * h
               dudx(1,1,k) = dudx(1,1,k) * h
               dudx(1,2,k) = dudx(1,2,k) * h
            end do

         else if ( ishape.eq.4 ) then

!        --- Quadratic triangle

            do i = 3, n, 2
               x(1,1) = x(1,1) + x(i,1)
               x(1,2) = x(1,2) + x(i,2)
               do k = 1, ncomp
                  u(1,k) = u(1,k) + u(i,k)
                  dudx(1,1,k) = dudx(1,1,k) + dudx(i,1,k)
                  dudx(1,2,k) = dudx(1,2,k) + dudx(i,2,k)
               end do
            end do
            h = 1d0/3d0
            x(1,1) = x(1,1) * h
            x(1,2) = x(1,2) * h
            do k = 1, ncomp
               dudx(1,1,k) = dudx(1,1,k) * h
               dudx(1,2,k) = dudx(1,2,k) * h
            end do
            do k = 1, ncomp
               u(1,k) = -1d0/9d0*u(1,k) + 4d0/9d0*(u(2,k)+u(4,k)+u(6,k))
            end do

         else

!        --- centroid available

            x(1,1) = x(n,1)
            x(1,2) = x(n,2)
            do k = 1, ncomp
               u(1,k) = u(n,k)
               dudx(1,1,k) = dudx(n,1,k)
               dudx(1,2,k) = dudx(n,2,k)
            end do

         end if

      else

!     --- ndim = 3

         if ( ishape.eq.11 .or. ishape.eq.13 ) then

!        --- Linear tetrahedron or hexahedron

            do i = 2, n
               x(1,1) = x(1,1) + x(i,1)
               x(1,2) = x(1,2) + x(i,2)
               x(1,3) = x(1,3) + x(i,3)
               do k = 1, ncomp
                  u(1,k) = u(1,k) + u(i,k)
                  dudx(1,1,k) = dudx(1,1,k) + dudx(i,1,k)
                  dudx(1,2,k) = dudx(1,2,k) + dudx(i,2,k)
                  dudx(1,3,k) = dudx(1,3,k) + dudx(i,3,k)
               end do
            end do
            h = 1d0/n
            x(1,1) = x(1,1) * h
            x(1,2) = x(1,2) * h
            x(1,3) = x(1,3) * h
            do k = 1, ncomp
               u(1,k) = u(1,k) * h
               dudx(1,1,k) = dudx(1,1,k) * h
               dudx(1,2,k) = dudx(1,2,k) * h
               dudx(1,3,k) = dudx(1,3,k) * h
            end do

         else if ( ishape.eq.12 ) then

!        --- Quadratic tetrahedron

            x(1,1) = x(1,1) + x(3,1) + x(5,1) + x(10,1)
            x(1,2) = x(1,2) + x(3,2) + x(5,2) + x(10,2)
            x(1,3) = x(1,3) + x(3,3) + x(5,3) + x(10,3)
            do k = 1, ncomp
               u(1,k) = u(1,k) + u(3,k) + u(5,k) + u(10,k)
               dudx(1,1,k) = dudx(1,1,k) + dudx(3,1,k) + &
                             dudx(5,1,k) + dudx(10,1,k)
               dudx(1,2,k) = dudx(1,2,k) + dudx(3,2,k) + &
                             dudx(5,2,k) + dudx(10,2,k)
               dudx(1,3,k) = dudx(1,3,k) + dudx(3,3,k) + &
                             dudx(5,3,k) + dudx(10,3,k)
            end do
            h = 0.25d0
            x(1,1) = x(1,1) * h
            x(1,2) = x(1,2) * h
            x(1,3) = x(1,3) * h
            do k = 1, ncomp
               u(1,k) = u(1,k) * h
               dudx(1,1,k) = dudx(1,1,k) * h
               dudx(1,2,k) = dudx(1,2,k) * h
               dudx(1,3,k) = dudx(1,3,k) * h
            end do

         else

!        --- centroid available

            j = n
            if ( ishape.eq.14 ) j = 14
            x(1,1) = x(j,1)
            x(1,2) = x(j,2)
            x(1,3) = x(j,3)
            do k = 1, ncomp
               u(1,k) = u(j,k)
               dudx(1,1,k) = dudx(j,1,k)
               dudx(1,2,k) = dudx(j,2,k)
               dudx(1,3,k) = dudx(j,3,k)
            end do

         end if

      end if

      end
