      subroutine el2001(jfi,jdiagloc,u,s,w,phi,phider,nloc,mloc, &
                        idimloc,lrowloc, work,econ)
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 03-04-1999 Include commons
!        version  4.0  date 05-02-1992 Removal of delta,dksidx and phidph
!        version  3.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.0  date 08-10-1986
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of element matrix corresponding to first derivatives
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical econ
      double precision u(*), s(*), w(*), phider(*), phi(*), work(*)
      integer jfi,jdiagloc,nloc,mloc,idimloc,lrowloc

!     econ       i    indication whether some integer parameters may be saved
!                     for the next calls of el2001 (true) or not (false)
!                     When el2001 is called more than once in an element
!                     subroutine econ = .false. should be used
!     idimloc       i    number of components of the vector u
!     jdiagloc      i    Indicates if the basis functions satisfy phi i(j) = 0, &
!                     if i#j (jdiagloc=1) or not (jdiagloc=2)
!     jfi        i    choice parameter. Possibilities:
!                     jfi = jfirst + 10000 * ispec
!                     ispec:
!                       0    normal case
!                       1    quadratic triangle with 3-point newton cotes rule
!                     jfirst:
!                       1    the element matrix s is added to an already
!                            existing element matrix s
!                       2    the element matrix s is a new matrix
!                       3,4  not yet available
!                       5    see 1, however, the derivatives of the basis
!                            functions are constant
!                       6    see 2, however, the derivatives of the basis
!                            functions are constant
!                     > 100  jfirst = jf1 + 100 x ni
!                            where jf1 has the same meaning as jfirst for
!                            jfirst<100 and ni is the number of basis functions
!                            psi
!                            the integral to be computed in that case is
!                              / u . grad( phi j )  psi i
!     lrowloc       i    number of rows and columnlocs of matrix s
!     m          i    number of integration points
!     n          i    number of basis functions
!     phi        i    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!                     when jfirst>100 the function psi must be stored in phi
!     phider     i    array containing the values of the derivatives in the
!                     integration points
!                     dphi i/dxk( xj ) = phider((i-1)*m+j+(k-1)*m*n)
!     s         i/o   element matrix  ( lrowloc x lrowloc )
!                     s(i,j)=s((i-1)*lrowloc+j)  1-d storage
!                     only the positions i,j=1,...,n are filled
!     u          i    array containing the velocities
!                     in the integration points
!                     the first m positions correspond to the u-component
!                     the second m to the v-component etc.
!     w          i    array of length m containing the weights for integration
!     work            Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jfirst,kbot,ksteploc,lrow1,instar,ni,jf1, im, in, j, jm, &
              jm1, jm2, k, m2, mnloc, n1, n2, n3, nn, i
      double precision h1, h2, h3, ph, sum, sum1, sum2
      save jfirst,kbot,ksteploc,lrow1,instar,ni,jf1

!     h1         Help variable to compute the element matrix
!     h2
!     h3
!     i          Counting variable
!     im         (i-1) x m
!     in         (i-1) x lrowloc
!     instar
!     j          Counting variable
!     jf1
!     jfirst
!     jm         (j-1) x m
!     jm1
!     jm2
!     k          Counting variable
!     kbot
!     ksteploc
!     lrow1
!     m2
!     mnloc
!     n1
!     n2
!     n3
!     ni
!     nn
!     ph
!     sum        Help parameter to compute a sum
!     sum1       Parameter to compute a sum
!     sum2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if(.not. econ .or. ifirst.eq.0) then

!     --- ifirst = 0  compute jfirst, kbot and ksteploc

         if(jfi.gt.10000) then
            jfirst=jfi-10000
            kbot=2
            ksteploc=2
            lrow1=2*lrowloc
            instar=lrowloc
         else
            jfirst=jfi
            kbot=1
            ksteploc=1
            lrow1=lrowloc
            instar=0
         end if
         ni=jfirst/100
         jf1=jfirst-100*ni
         if(ni.eq.0) ni=nloc
      end if
      goto ( 100, 400, 100, 400, 700, 1000 ), jf1

!     --- jf1 = 1  add matrix s to an already existing matrix

100   if(jdiagloc.eq.1) then
         in=instar
         goto ( 110, 140, 170 ), idimloc

!        --- idimloc = 1

110         do 130 i=kbot,ni,ksteploc
               h1=w(i)*u(i)
               n1=i-nloc
               do 120 j=1,nloc
120               s(in+j)=s(in+j)+phider(nloc*j+n1)*h1
130            in=in+lrow1
            return

!           --- idimloc = 2

140         mnloc=mloc*nloc
            do 160 i=kbot,ni,ksteploc
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               n1=i-nloc
               n2=n1+mnloc
               do 150 j=1,nloc
150               s(in+j)=s(in+j)+phider(nloc*j+n1)*h1 &
                         +phider(nloc*j+n2)*h2
160            in=in+lrow1
            return

!           --- idimloc = 3

170         mnloc=mloc*nloc
            nn=ni+ni
            do 190 i=kbot,ni,ksteploc
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               h3=w(i)*u(i+nn)
               n1=i-nloc
               n2=n1+mnloc
               n3=n2+mnloc
               do 180 j=1,nloc
180               s(in+j)=s(in+j)+phider(nloc*j+n1)*h1 &
                         +phider(nloc*j+n2)*h2+phider(nloc*j+n3)*h3
190            in=in+lrow1
            return
      else
        in=0
        im=0
        goto ( 210, 260, 310 ), idimloc

!       --- idimloc = 1

210         do 220 k=1,mloc
220            work(k)=u(k)*w(k)
            do 250 i=1,ni
               jm=0
               do 240 j=1,nloc
                  sum=0d0
                  do 230 k=1,mloc
230                 sum=sum+(work(k)*phider(jm+k))*phi(im+k)
                     s(in+j)=s(in+j)+sum
240               jm=jm+mloc
               im=im+mloc
250         in=in+lrow1
            return

!           --- idimloc = 2

260         mnloc=mloc*nloc
            m2=mloc+mloc
            do 270 k=1,mloc
               work(k)=u(k)*w(k)
270            work(k+mloc)=u(k+mloc)*w(k)
            do 300 i=1,ni
               jm=0
               do 290 j=1,nloc
                  jm1=jm+mnloc
                  sum=0d0
                  do 280 k=1,mloc
280                 sum=sum+(work(k)*phider(jm+k)+ &
                        phider(jm1+k)*work(k+mloc))*phi(im+k)
                     s(in+j)=s(in+j)+sum
290               jm=jm+mloc
               im=im+mloc
300         in=in+lrow1
            return

!           --- idimloc = 3

310         mnloc=mloc*nloc
            m2=mloc+mloc
            do 320 k=1,mloc
               work(k)=u(k)*w(k)
               work(k+mloc)=u(k+mloc)*w(k)
320            work(k+m2)=u(k+m2)*w(k)
            do 350 i=1,ni
               jm=0
               do 340 j=1,nloc
                  jm1=jm+mnloc
                  jm2=jm1+mnloc
                  sum=0d0
                  do 330 k=1,mloc
330                 sum=sum+(work(k)*phider(jm+k)+ &
                        phider(jm1+k)*work(k+mloc)+ &
                        phider(jm2+k)*work(k+m2))*phi(im+k)
                     s(in+j)=s(in+j)+sum
340               jm=jm+mloc
               im=im+mloc
350         in=in+lrow1
            return
      end if

!     --- jf1 = 2  construct a new matrix s

400   if(jdiagloc.eq.1) then
         in=instar
         goto ( 410, 440, 470 ), idimloc

!        --- idimloc = 1

410         do 430 i=1,ni
               h1=w(i)*u(i)
               n1=i-nloc
               do 420 j=1,nloc
420               s(in+j)=phider(nloc*j+n1)*h1
430            in=in+lrow1
            return

!           --- idimloc = 2

440         mnloc=mloc*nloc
            do 460 i=kbot,ni,ksteploc
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               n1=i-nloc
               n2=n1+mnloc
               do 450 j=1,nloc
450               s(in+j)=phider(nloc*j+n1)*h1 &
                         +phider(nloc*j+n2)*h2
460            in=in+lrow1
            return

!           --- idimloc = 3

470         mnloc=mloc*nloc
            nn=ni+ni
            do 490 i=1,ni
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               h3=w(i)*u(i+nn)
               n1=i-nloc
               n2=n1+mnloc
               n3=n2+mnloc
               do 480 j=1,nloc
480               s(in+j)=phider(nloc*j+n1)*h1 &
                         +phider(nloc*j+n2)*h2+phider(nloc*j+n3)*h3
490            in=in+lrow1
            return
      else
        in=0
        im=0
        goto ( 510, 560, 610 ), idimloc

!       --- idimloc = 1

510         do 520 k=1,mloc
520            work(k)=u(k)*w(k)
            do 550 i=1,ni
               jm=0
               do 540 j=1,nloc
                  sum=0d0
                  do 530 k=1,mloc
530                 sum=sum+(work(k)*phider(jm+k))*phi(im+k)
                     s(in+j)=sum
540               jm=jm+mloc
               im=im+mloc
550         in=in+lrow1
            return

!           --- idimloc = 2

560         mnloc=mloc*nloc
            m2=mloc+mloc
            do 570 k=1,mloc
               work(k)=u(k)*w(k)
570            work(k+mloc)=u(k+mloc)*w(k)
            do 600 i=1,ni
               jm=0
               do 590 j=1,nloc
                  jm1=jm+mnloc
                  sum=0d0
                  do 580 k=1,mloc
580                 sum=sum+(work(k)*phider(jm+k)+ &
                        phider(jm1+k)*work(k+mloc))*phi(im+k)
                     s(in+j)=sum
590               jm=jm+mloc
               im=im+mloc
600         in=in+lrow1
            return

!           --- idimloc = 3

610         mnloc=mloc*nloc
            m2=mloc+mloc
            do 620 k=1,mloc
               work(k)=u(k)*w(k)
               work(k+mloc)=u(k+mloc)*w(k)
620            work(k+m2)=u(k+m2)*w(k)
            do 650 i=1,ni
               jm=0
               do 640 j=1,nloc
                  jm1=jm+mnloc
                  jm2=jm1+mnloc
                  sum=0d0
                  do 630 k=1,mloc
630                 sum=sum+(work(k)*phider(jm+k)+ &
                        phider(jm1+k)*work(k+mloc)+ &
                        phider(jm2+k)*work(k+m2))*phi(im+k)
                     s(in+j)=sum
640               jm=jm+mloc
               im=im+mloc
650         in=in+lrow1
            return
      end if

!     --- jf1 = 5  add matrix s to an already existing matrix

700   if(jdiagloc.eq.1) then
         in=instar
         goto ( 710, 740, 770 ), idimloc

!        --- idimloc = 1

710         do 730 i=1,ni
               h1=w(i)*u(i)
               do 720 j=1,nloc
720               s(in+j)=s(in+j)+phider(j)*h1
730            in=in+lrow1
            return

!           --- idimloc = 2

740         do 760 i=kbot,ni,ksteploc
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               do 750 j=1,nloc
750               s(in+j)=s(in+j)+phider(j)*h1 &
                         +phider(j+nloc)*h2
760            in=in+lrow1
            return

!           --- idimloc = 3

770         nn=ni+ni
            do 790 i=1,ni
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               h3=w(i)*u(i+nn)
               do 780 j=1,nloc
780               s(in+j)=s(in+j)+phider(j)*h1 &
                         +phider(j+nloc)*h2+phider(j+nn)*h3
790            in=in+lrow1
            return
      else
        in=0
        im=0
        goto ( 810, 860, 910 ), idimloc

!       --- idimloc = 1

810         do 820 k=1,mloc
820            work(k)=u(k)*w(k)
            do 850 i=1,ni
               sum=0d0
               do 830 k=1,mloc
830              sum=sum+work(k)*phi(im+k)
               do 840 j=1,nloc
840               s(in+j)=s(in+j)+sum*phider(j)
               im=im+mloc
850         in=in+lrow1
            return

!           --- idimloc = 2

860         mnloc=mloc*nloc
            m2=mloc+mloc
            do 870 k=1,mloc
               work(k)=u(k)*w(k)
870            work(k+mloc)=u(k+mloc)*w(k)
            do 900 i=1,ni
               sum=0d0
               sum1=0d0
               do 880 k=1,mloc
                  ph=phi(im+k)
                  sum=sum+work(k)*ph
880               sum1=sum1+work(k+mloc)*ph
               do 890 j=1,nloc
890               s(in+j)=s(in+j)+sum*phider(j)+sum1*phider(j+nloc)
               im=im+mloc
900         in=in+lrow1
            return

!           --- idimloc = 3

910         m2=mloc+mloc
            do 920 k=1,mloc
               work(k)=u(k)*w(k)
               work(k+mloc)=u(k+mloc)*w(k)
920            work(k+m2)=u(k+m2)*w(k)
            do 950 i=1,ni
                  sum=0d0
                  sum1=0d0
                  sum2=0d0
                  do 930 k=1,mloc
                    ph=phi(im+k)
                    sum=sum+work(k)*ph
                    sum1=sum1+work(k+mloc)*ph
930                 sum2=sum2+work(k+m2)*ph
               do 940 j=1,nloc
940               s(in+j)=s(in+j)+sum*phider(j)+sum1*phider(j+nloc)+ &
                          sum2*phider(j+m2)
               im=im+mloc
950         in=in+lrow1
            return
      end if

!     --- jf1 = 6  construct a new matrix s

1000   if(jdiagloc.eq.1) then
         in=0
         goto ( 1010, 1040, 1070 ), idimloc

!        --- idimloc = 1

1010         do 1030 i=1,ni
               h1=w(i)*u(i)
               do 1020 j=1,nloc
1020               s(in+j)=phider(j)*h1
1030            in=in+lrow1
            return

!           --- idimloc = 2

1040         do 1060 i=kbot,ni,ksteploc
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               do 1050 j=1,nloc
1050               s(in+j)=phider(j)*h1 &
                         +phider(j+nloc)*h2
1060            in=in+lrow1
            return

!           --- idimloc = 3

1070         nn=ni+ni
            do 1090 i=1,ni
               h1=w(i)*u(i)
               h2=w(i)*u(i+ni)
               h3=w(i)*u(i+nn)
               do 1080 j=1,nloc
1080               s(in+j)=phider(j)*h1 &
                         +phider(j+nloc)*h2+phider(j+nn)*h3
1090            in=in+lrow1
            return
      else
        in=0
        im=0
        goto ( 1110, 1160, 1210 ), idimloc

!       --- idimloc = 1

1110         do 1120 k=1,mloc
1120            work(k)=u(k)*w(k)
            do 1150 i=1,ni
               sum=0d0
               do 1130 k=1,mloc
1130              sum=sum+work(k)*phi(im+k)
               do 1140 j=1,nloc
1140               s(in+j)=sum*phider(j)
               im=im+mloc
1150         in=in+lrow1
            return

!           --- idimloc = 2

1160         mnloc=mloc*nloc
            m2=mloc+mloc
            do 1170 k=1,mloc
                work(k)=u(k)*w(k)
1170            work(k+mloc)=u(k+mloc)*w(k)
            do 1200 i=1,ni
               sum=0d0
               sum1=0d0
               do 1180 k=1,mloc
                  ph=phi(im+k)
                  sum=sum+work(k)*ph
1180               sum1=sum1+work(k+mloc)*ph
               do 1190 j=1,nloc
1190               s(in+j)=sum*phider(j)+sum1*phider(j+nloc)
               im=im+mloc
1200         in=in+lrow1
            return

!           --- idimloc = 3

1210         m2=mloc+mloc
            do 1220 k=1,mloc
                work(k)=u(k)*w(k)
                work(k+mloc)=u(k+mloc)*w(k)
1220            work(k+m2)=u(k+m2)*w(k)
            do 1250 i=1,ni
                  sum=0d0
                  sum1=0d0
                  sum2=0d0
                  do 1230 k=1,mloc
                    ph=phi(im+k)
                    sum=sum+work(k)*ph
                    sum1=sum1+work(k+mloc)*ph
1230                 sum2=sum2+work(k+m2)*ph
               do 1240 j=1,nloc
1240               s(in+j)=sum*phider(j)+sum1*phider(j+nloc)+ &
                          sum2*phider(j+m2)
               im=im+mloc
1250         in=in+lrow1
      end if
      end
