      subroutine el2002(jzer,jdiagloc,b,s,w,phi,nloc,mloc,lrowloc, &
                        work,phiphi,delta,econ)
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.4  date 03-04-1999 Include commons
!        version  4.3  date 22-07-1993 correction jzer = 4
!        version  4.2  date 22-12-1991 improve errors for jdiagloc=22;JvdZ
!        version  4.1  date 18-12-1991 add jdiagloc=22, use phiphi; JvdZ
!        version  4.0  date 4-12-1991 add jdiagloc=12, use phiphi; JvdZ
!        version  3.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.1  date 01-07-1991 improved for jzer<0 if
!                                      jdiagloc=2 by jaap vd zanden
!        version  3.0  date 08-10-1986
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute part of matrix corresponding to zeroth order derivatives
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical econ
      double precision s(*), b(*), w(*), work(*), phi(*), phiphi(*), &
                       delta
      integer jzer,jdiagloc,nloc,mloc,lrowloc

!     b          i    array containing the function b
!                     in the integration points
!     delta           not yet used
!     econ       i    indication whether some integer parameters may be saved
!                      for the next calls of el2001 (true) or not (false)
!                     When el2001 is called more than once in an element
!                     subroutine econ = .false. should be used
!     jdiagloc      i    Indicates if the basis functions satisfy phi i(j) = 0, &
!                     if i#j (jdiagloc=1) or not (jdiagloc=2)
!                     12   general; use phi    as basis function
!                                   and phiphi as test function
!                     22   special; use phi    as basis function
!                                   and phiphi as test function
!     jzer       i    choice parameter. Possibilities:
!                     jzer = jzeroloc + 100 * ispec
!                     ispec:
!                       0  normal case
!                       1  quadratic triangle with 3-point newton cotes rule
!                     jzeroloc:
!                       1    b and w constant
!                       2    general
!                       3,4  see 1,2   no addition takes place
!                       5    b constant
!                       6    see 5  no addition takes place
!                     < 0    see 6, however the subroutine starts with the
!                            -jzerolocth row
!     lrowloc       i    number of rows in s
!     m          i    number of integration points
!     n          i    number of basis functions
!     phi        i    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!     phiphi     i    see phi and jdiagloc=12, 22
!     s         i/o   element matrix  ( lrowloc x lrowloc )
!                     only a n x n part is filled
!                     s(i,j)=s((i-1)*lrowloc+j)  1-d storage
!     w          i    array of length m containing the weights for integration
!     work            Work array of length m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jzeroloc,kbot,ksteploc,n1,nstart,istart,kzero, i, i1, ii, im, &
              in, j, jm, k
      double precision h, sum
      save jzeroloc,kbot,ksteploc,n1,nstart,istart,kzero

!     h          Help variable to store a constant temporarily
!     i          Counting variable
!     i1
!     ii
!     im         (i-1) x m
!     in         (i-1) x lrowloc
!     istart
!     j          Counting variable
!     jm         (j-1) x m
!     jzeroloc
!     k          Counting variable
!     kbot
!     ksteploc
!     kzero
!     n1
!     nstart
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if(jzer.eq.0 .or.jzer.eq.100) return
      if(.not. econ .or. ifirst.eq.0) then

!     --- ifirst = 0  compute jzeroloc, kbot and ksteploc

         if(jzer.gt.100) then
            jzeroloc=jzer-100
            kbot=2
            ksteploc=2
            n1=2*(lrowloc+1)
         else
            jzeroloc=jzer
            kbot=1
            ksteploc=1
            n1=lrowloc+1
            istart=1
         end if
         if(jzeroloc.lt.0) then
            kzero=6
            nstart=-jzeroloc
            istart=1+(nstart-1)*n1
            if (jdiagloc.eq.2 .or. jdiagloc.eq.12 .or. jdiagloc.eq.22) n1=lrowloc
         else
            kzero=jzeroloc
            nstart=kbot
            istart=1+(kbot-1)*(lrowloc+1)
         end if
      end if
      if(kzero.gt.2 .and. kzero.ne.5) then
         do 50 i=1,lrowloc*lrowloc
 50         s(i)=0d0
      end if
      if(jdiagloc.eq.1) then

!     --- jdiagloc = 1   Newton Cotes rule

         if(kzero.eq.1) then

!        --- kzero = 1  b and w constant

            h=b(1)*w(1)
            ii=istart
            do 100 i=nstart,nloc,ksteploc
               s(ii)=s(ii)+h
100            ii=ii+n1
         else if(kzero.eq.2) then

!        --- kzero = 2

            ii=istart
            do 200 i=nstart,nloc,ksteploc
               s(ii)=s(ii)+w(i)*b(i)
200            ii=ii+n1
         else if(kzero.eq.3) then

!        --- kzero = 3  b and w constant

            h=b(1)*w(1)
            ii=istart
            do 210 i=nstart,nloc,ksteploc
               s(ii)=h
210            ii=ii+n1
         else if(kzero.eq.4) then

!        --- kzero = 4

            ii=istart
            do 220 i=nstart,nloc,ksteploc
               s(ii)=w(i)*b(i)
220            ii=ii+n1
         else if(kzero.eq.5) then
            ii=istart
            do 230 i=nstart,nloc,ksteploc
               s(ii)=s(ii)+w(i)*b(1)
230            ii=ii+n1
         else
            ii=istart
            do 240 i=nstart,nloc
               s(ii)=w(i)*b(1)
240            ii=ii+n1
         end if
      else if (jdiagloc.eq.2) then

!     --- jdiagloc = 2   Gauss rule

         if(kzero.eq.1) then

!        --- kzero = 1  b and w constant

            h=b(1)*w(1)
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 280 i=nstart,nloc
               jm=0
               i1=i-lrowloc
               do 270 j=1,i
                  sum=0d0
                  do 260 k=1,mloc
260                  sum=sum+phi(im+k)*phi(jm+k)
                  sum=h*sum
                  s(in+j)=s(in+j)+sum
                  if(i.ne.j) s(j*lrowloc+i1)=s(j*lrowloc+i1)+sum
270               jm=jm+mloc
               im=im+mloc
280            in=in+lrowloc
         else if(kzero.eq.2 .or. kzero.eq.5) then

!        --- kzero = 2 or 5 general or b constant

            if(kzero.eq.2) then
               do 310 k=1,mloc
310               work(k)=w(k)*b(k)
            else
               do 320 k=1,mloc
320               work(k)=w(k)*b(1)
            end if
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 380 i=nstart,nloc
               jm=0
               i1=i-lrowloc
               do 370 j=1,i
                  sum=0d0
                  do 360 k=1,mloc
360                  sum=sum+work(k)*phi(im+k)*phi(jm+k)
                  s(in+j)=s(in+j)+sum
                  if(i.ne.j) s(j*lrowloc+i1)=s(j*lrowloc+i1)+sum
370               jm=jm+mloc
               im=im+mloc
380            in=in+lrowloc
         else if(kzero.eq.3) then

!        --- kzero = 3  b and w constant

            h=b(1)*w(1)
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 480 i=nstart,nloc
               jm=0
               i1=i-lrowloc
               do 470 j=1,i
                  sum=0d0
                  do 460 k=1,mloc
460                  sum=sum+phi(im+k)*phi(jm+k)
                  sum=h*sum
                  s(in+j)=sum
                  if(i.ne.j) s(j*lrowloc+i1)=sum
470               jm=jm+mloc
               im=im+mloc
480            in=in+lrowloc
         else

!        --- kzero = 2  general

            if(kzero.eq.2 .or. kzero.eq.4) then
               do 510 k=1,mloc
510               work(k)=w(k)*b(k)
            else
               do 520 k=1,mloc
520               work(k)=w(k)*b(1)
            end if
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 580 i=nstart,nloc
               jm=0
               i1=i-lrowloc
               do 570 j=1,i
                  sum=0d0
                  do 560 k=1,mloc
560                  sum=sum+work(k)*phi(im+k)*phi(jm+k)
                  s(in+j)=sum
                  if(i.ne.j) s(j*lrowloc+i1)=sum
570               jm=jm+mloc
               im=im+mloc
580            in=in+lrowloc
         end if
      else if (jdiagloc.eq.12) then

!     --- jdiagloc = 12   Gauss rule phi is shape function;
!                                 phiphi test  function

         if(kzero.eq.1) then

!        --- kzero = 1  b and w constant

            h=b(1)*w(1)
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 680 i=nstart,nloc
               jm=0
               do 670 j=1,nloc
                  sum=0d0
                  do 660 k=1,mloc
660                  sum=sum+phi(im+k)*phiphi(jm+k)
                  sum=h*sum
                  s(in+j)=s(in+j)+sum
670               jm=jm+mloc
               im=im+mloc
680            in=in+lrowloc
         else if(kzero.eq.2 .or. kzero.eq.5) then

!        --- kzero = 2 or 5 general or b constant

            if(kzero.eq.2) then
               do 710 k=1,mloc
710               work(k)=w(k)*b(k)
            else
               do 720 k=1,mloc
720               work(k)=w(k)*b(1)
            end if
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 780 i=nstart,nloc
               jm=0
               do 770 j=1,nloc
                  sum=0d0
                  do 760 k=1,mloc
760                  sum=sum+work(k)*phi(im+k)*phiphi(jm+k)
                  s(in+j)=s(in+j)+sum
770               jm=jm+mloc
               im=im+mloc
780            in=in+lrowloc
         else if(kzero.eq.3) then

!        --- kzero = 3  b and w constant

            h=b(1)*w(1)
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 880 i=nstart,nloc
               jm=0
               do 870 j=1,nloc
                  sum=0d0
                  do 860 k=1,mloc
860                  sum=sum+phi(im+k)*phiphi(jm+k)
                  sum=h*sum
                  s(in+j)=sum
870               jm=jm+mloc
               im=im+mloc
880            in=in+lrowloc
         else

!        --- kzero = 2,4  general

            if(kzero.eq.2) then
               do 910 k=1,mloc
910               work(k)=w(k)*b(k)
            else
               do 920 k=1,mloc
920               work(k)=w(k)*b(1)
            end if
            in=(nstart-1)*n1
            im=(nstart-1)*mloc
            do 980 i=nstart,nloc
               jm=0
               do 970 j=1,nloc
                  sum=0d0
                  do 960 k=1,mloc
960                  sum=sum+work(k)*phi(im+k)*phiphi(jm+k)
                  s(in+j)=sum
970               jm=jm+mloc
               im=im+mloc
980            in=in+lrowloc
         end if
      else if (jdiagloc.eq.22) then

!     --- jdiagloc = 22   Gauss rule phi is shape function;
!                                 phiphi test  function
!           kzero = 2  general

            if (kzero.eq.2) then
               do 1210 k=1,mloc
1210              work(k)=w(k)*b(k)
            else
               do 1220 k=1,mloc
1220              work(k)=w(k)*b(1)
            end if
            if (mloc.ne.nloc .or. nloc.eq.3)  then
               ksteploc=1
            else
               ksteploc=2
            end if
            in=nloc*(ksteploc-1)
            im=mloc*(ksteploc-1)
            do 1280 i=ksteploc,nloc,ksteploc
               jm=0
               do 1270 j=1,nloc
                  sum=0d0
                  do 1260 k=1,mloc
1260                 sum=sum+work(k)*phi(jm+k)*phiphi(im+k)
                  s(in+j)=s(in+j)+sum
1270              jm=jm+mloc
               im=im+mloc*ksteploc
1280           in=in+lrowloc*ksteploc
      end if
      end
