      subroutine el2018 ( icheld, array, dudx, smagin, smagno, &
                          mdecl, ndecl, ndimdc, w, phi, &
                          elemvc, elemwg )
! ======================================================================
!
!        programmer    Zdenek Vlasek and Guus Segal
!        version  1.3  date 24-12-1993 Correction error messages
!        version  1.2  date 23-06-1992 New common cmessg
!        version  1.1  date 13-12-1989 case icheld=12 dropped
!        version  1.0  date 01-12-1989 Experimental version, &
!                                      not much time-efffective
!
!   copyright (c) 1989-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill elemvc for a-posteriori error estimations
!     Fill also elemwg ( for icheld=10 or 11 only )
!     This subroutine is ready only for itype = 100 or 104
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, mdecl, ndecl, ndimdc
      double precision array(mdecl,8), dudx(mdecl,ndimdc), &
                       smagin(mdecl,ndimdc), smagno(ndecl,ndimdc), &
                       w(mdecl), phi(ndecl,mdecl), &
                       elemvc(*), elemwg(*)

!     array   i    array m * 8,  coefficients of the equation in all
!                  integration points
!     dudx   i/o   Real m x ndim  array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j )
!                    j     k
!                  If jdercn = 0 in common block celint, then only
!                  dudx (1,*) is filled because dudx is then a constant
!                  per element
!                  In that case this subroutines filles them all
!                  dudx is only filled if jderiv > 0
!     elemvc  o    element vector to be filled:
!                  icheld =10, 11:  length: ndim * n
!                          13:              2
!     elemwg  o    element weights to be filled when icheld=10 or 11:
!                  icheld =10, 11:  length:  n
!                          13:               0      ( not used )
!     icheld  i    choice parameter to distinguish between different
!                  possibilities of this subroutine, possible values are:
!                  10:   the first smoothing of  A * grad u
!                  11:   iteration step in smoothing of  A * grad u
!                        in smagin, smagno is old smooth  A * grad u
!                  13:   in smagin, smagno is last smooth  A * grad u
!                        the first "degree of freedom"
!                        a-posteriori local error estimations
!                        attached to element
!                        the second "degree of freedom"
!                        local semi-energy norm of  u
!     mdecl   i    number of integration points in element, &
!                  same as  m  in common /celint/
!     ndecl   i    number of nodal points in element, &
!                  same as  n  in common /celint/
!     ndimdc  i    number of dimensions of the space
!                  same as  ndim  in common /celint/
!     phi     i    array of lenght ndecl * mdecl, values of shape
!                  functions in integration points
!     smagin  i    array of length mdecl * ndimdc, values of smooth
!                  A * grad u  in integration points
!     smagno  i    array of length mdecl * ndimdc, values of smooth
!                  A * grad u  in nodal points
!     w       i    array of length mdecl containing the weights for integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mxm, mxn, nd, nc, &
              i, j, idi, jdi, k, itempo, inarr(2,2)
      parameter ( mxm=9, mxn=9, nd=2, nc=3 )
      double precision sum, adetel, b(mxm), s(mxn,mxn), diter(mxn), &
                       adu(mxm,nd), rhsid(mxn), &
                       exm100(3,3), exm104(6,6), &
                       work(mxm), arrinv(mxm,nc), errsig(mxm,2), &
                       det, sumjdi
      data b      / mxm * 1d0 / &
           inarr  / 1,2, &
                    2,3 / &
           exm100 / 2d0, 1d0, 1d0, &
                    1d0, 2d0, 1d0, &
                    1d0, 1d0, 2d0 / &
           exm104 /  6d0,  0d0, -1d0, -4d0, -1d0,  0d0, &
                     0d0, 32d0,  0d0, 16d0, -4d0, 16d0, &
                    -1d0,  0d0,  6d0,  0d0, -1d0, -4d0, &
                    -4d0, 16d0,  0d0, 32d0,  0d0, 16d0, &
                    -1d0, -4d0, -1d0,  0d0,  6d0,  0d0, &
                     0d0, 16d0, -4d0, 16d0,  0d0, 32d0 /

!     adetel  absolute value of determinant of the element
!             which is simplex, &
!             multiplied by a constant specific for type of element
!             ( only simplex elements supposed )
!     adu     array mxm * ndim of which only part m * ndim is used, &
!             values of the old  A * grad u  in integration points
!     arrinv  array mxm * nc of which only part m * nc is used, &
!             inverse of matrix A in integration points, &
!             only three coefficients of inv(A) are stored, &
!             ( as it is in array(*,*) )
!     b       array of length m, values of unit function in
!             integration points, for some integrals
!     det     determinant of matrix A
!     diter   array of length n, diagonal of the iteration matrix
!     errsig  array mxm * ndim of which only part m * ndim is used, &
!             estimation of error of A * grad u in integration points
!     exm100  exact mass matrix for itype=100, multiplied by 24, &
!             when element determinant is equal to 1  ?
!     exm104  exact mass matrix for itype=104, multiplied by 360, &
!             when element determinant is equal to 1
!     i       Counting variable
!     idi     subscript for ax
!     inarr   array ndim * ndim of which only part ndimdc * ndimdc is
!             used, it tells that the coefficient A(idi,jdi) is item
!             number inarr(idi,jdi) in the list of all coefficients
!     itempo  to store temporary overwritten value
!     j       Counting variable
!     jdi     subscript for ax
!     k       Counting variable
!     mxm     max values of m
!     mxn     max values of n
!     nc      maximum number of coefficents of matrix A, stored in
!             array resp. invarr
!     nd      the same as ndim and ndimdc
!     rhsid   array of size n, righthand side for one spacial
!             component in smoothing process
!     s       array mxn * mxn of which only part n * n is used, &
!             first: element mass matrix, &
!             later:    "     "     "    minus a diagonal matrix
!             used for iterations
!     sum     temporary, for summations
!             element mass matrix
!     sumjdi  temporary, for summ over axis
!     work    work array of size m for subroutine el3003
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003  Compute integrals of the shape / f(x) phi (x) dx
!                                            e         i
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!    1200   Energy norm not positive
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check  itype, icheld
!         When du/dx is constant, eld100 has filled values for the first
!         integration point only, expand them to all integration points

      if ( jdercn.eq.0  ) then
         do idi = 1, ndimlc
            do k = 2, m
               dudx ( k, idi ) = dudx ( 1, idi )
            end do
         end do
      end if

!     --- fill array adu(m,ndim) by matrix-vector product  A * grad u
!         ( in integration points ), &
!         this value may be discontinuous across element interfaces
!         adu(k,idi) = /  A(k,idi,jdi)  * dudx(k,jdi)
!                     jdi

      do idi = 1, ndimlc
         do  k = 1, m
            sum = 0d0
            do jdi = 1, ndimlc
               sum = sum + &
                  array ( k, inarr(idi,jdi) ) * dudx ( k, jdi )
            end do
            adu ( k, idi ) = sum
         end do
      end do

!     --- rewrite jadd ( temporary ) for call el3003

      itempo = jadd
      jadd = 0
      if ( icheld.le.11 ) then

!     --- icheld = 10 or 11
!         goal: fill  h,  fill diter(*)
!         diagonal of lumped mass matrix is
!         diter(i) = sum  integ ( phi(i) * phi(j) ) = &
!                     j
!                  = integ ( phi(i) * sum phi(j) ) = &
!                                      j
!                  = integ ( phi(i) * 1 ) = integ phi(i), &
!         which is approximated exactly by  sum w(k) * phi(k,i)
!                                            k
!         ( also for gauss integration ):

         call el3003 ( b, diter, w, phi, m, n )
         if ( itype.eq.100 ) then

!        --- adetel is the abs.determinant divided by 24, &
!            so diter(1) = adetel * (2+1+1)

            adetel = diter(1) / 4d0
         else if ( itype.eq.104 ) then

!        --- diagonal of the lumped mass matrix is singular, &
!            but also weights w(1)=w(3)=w(5)=0
!            to calculate integrals of error we need only
!            the non-vertex values
!            adetel is the abs.determinant divided by 360, &
!            so  diter(2) = adetel * (32+16-4+16) = adetel * 60

            adetel = diter(2) / 60d0

!           --- replace the zero values by adetel, to allow assembling

            do i = 1, n, 2
               diter ( i ) = adetel
            end do
         end if

!        --- Fill elemwg, now treated as  ndim * n, &
!            duplicate the values diter to ndim dimensions

         do i = n, 1, -1
            do idi = 1, ndimlc
               elemwg ( idi + ndimlc * ( i - 1 ) ) = diter ( i )
            end do
         end do
         if ( icheld.eq.11 ) then

!        --- One iteration, prepare in s(*,*) the exact mass matrix

            if ( itype.eq.100 ) then

!           --- adetel is the abs.determinant divided by 24

               do i = 1, n
                  do j = 1, n
                     s(i,j) = adetel * exm100 ( i, j )
                  end do
               end do
            else if ( itype.eq.104 ) then

!           --- adetel is the abs.determinant divided by 360

               do i = 1, n
                  do j = 1, n
                     s(i,j) = adetel * exm104 ( i, j )
                  end do
               end do
            end if

!           --- subtract from (s*,*) the diagonal matrix

            do i = 1, n
               s(i,i) = s(i,i) - diter(i)
            end do
         end if

!        --- elemwg was filled above, &
!            fill elemvc, now treated as  ndim * n
!            elemvc ( idi, i ) = &
!                 /  phi  *  /    A        * dudx
!                 e     i   jdi    idi,jdi       jdi

         do idi = 1, ndimlc

!        --- integrate, result is  rhsid

            call el3003 ( adu ( 1, idi ), rhsid, w, phi, m, n )
            if ( icheld.eq.11 ) then

!           --- in previous call ( icheld=10 )
!               instead of exact mass matrix a diagonal matrix was used
!               their difference is now in s(*,*), &
!               make iterative correction:
!                  elemvc ( idi, i ) = elemvc ( idi, i ) - &
!                  - /  s * sm ( A * du/dx    )
!                    j              i,j                 idi  j

               do i = 1, n
                  sum = 0d0
                  do j = 1, n
                     sum = sum - s ( i, j ) * smagno ( j, idi )
                  end do
                  rhsid ( i ) = rhsid ( i ) - sum
               end do
            end if
            if ( itype.eq.104 ) then

!           --- for non-gauss integration is rhsid(1,3,5)=0, &
!                   m=n, and we smooth by some other way
!               ??? what about Gauss integration

               do i = 1, n, 2
                  rhsid ( i ) = adetel * adu ( i, idi )
               end do
            end if

!           --- fill elemvc(*) for this idi by rhsid(*)

            do i = 1, n
               elemvc ( idi + ndimlc * ( i - 1 ) ) = rhsid ( i )
            end do
         end do
      else

!     --- icheld = 13, use A * grad u  in integration points
!                      and smagin      in integration points
!                      to estimate local error
!                      by Zienkiewicz & Zhu
!                      "simple error estimator"
!                      calculate also semi-energy norm of  u
!          invert matrix  A  in each integration point

         do k = 1, m
            det = array(k,1) * array(k,3) - array(k,2)**2
            if ( det.le.0d0 ) then
               call eropen ( 'el2018' )
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
               call erclos ( 'el2018' )
            end if
            arrinv(k,1) = array(k,3) / det
            arrinv(k,2) = -array(k,2) / det
            arrinv(k,3) = array(k,1) / det
         end do

!        --- Fill  errsig  by estimation of error of vector  A * grad u
!            in all integration points

         do idi = 1, ndimlc
            do k = 1, m
               errsig ( k, idi ) = adu ( k, idi ) - smagin ( k, idi )
            end do
         end do

!        --- Integrate   transp(errsig) * inverse(A) * errsig

         sum = 0d0
         do idi = 1, ndimlc
            do k = 1, m
               sumjdi = 0d0
               do jdi = 1, ndimlc
                  sumjdi = sumjdi + &
                     arrinv ( k, inarr(idi,jdi) ) * errsig ( k, jdi )
               end do
               sum = sum + w(k) * errsig ( k, idi ) * sumjdi
            end do
         end do
            if ( sum.lt.0d0 ) then
               call eropen ( 'el2018' )
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 1200, 3, 0, 0 )
               call erclos ( 'el2018' )
            end if
         sum = sqrt ( sum )

!        --- elemwg was set to 1 in eld100
!            attach the error estimation to element

         elemvc ( 1 ) = sum

!        --- calulate semi-energy norm of  u
!            Fill  errsig  by  A * grad u
!            in all integration points

         do idi = 1, ndimlc
            do k = 1, m
               errsig ( k, idi ) = adu ( k, idi )
            end do
         end do

!        --- Integrate   transp(errsig) * inverse(A) * errsig

         sum = 0d0
         do idi = 1, ndimlc
            do k = 1, m
               sumjdi = 0d0
               do jdi = 1, ndimlc
                  sumjdi = sumjdi + &
                     arrinv ( k, inarr(idi,jdi) ) * errsig ( k, jdi )
               end do
               sum = sum + w(k) * errsig ( k, idi ) * sumjdi
            end do
         end do

!        --- store it as second component of elemvc

         if ( sum.lt.0d0 ) then
            call eropen ( 'el2018' )
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 1200, 3, 0, 0 )
            call erclos ( 'el2018' )
         end if
         elemvc(2) = sqrt ( sum )
      end if

!     --- Restore  jadd

      jadd = itempo
      end
