      subroutine el3013 ( elemmt, elemmtloc, nelarge, nesmall, induv, &
                          number, transp, rmul )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-06-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill small matrix elemmtloc of size nesmall x nesmall into larger matrix
!    elemmt of size nelarge x nelarge.
!    Also the transpose of a matrix may be filled
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelarge, nesmall, induv(ncomp,*), number
      double precision elemmtloc(nesmall,nesmall), &
                       elemmt(nelarge,nelarge), rmul
      logical transp

!     elemmt        i/o   Element matrix
!     elemmtloc      i    Local element matrix to be put into elemmt
!     induv          i    Index array of size ncomp x inpelm that contains
!                         the positions of the degrees of freedom 1 to ncomp
!                         in the element
!                         Hence induv(k,i) contains the position of the k^th
!                         degree of freedom in the i^th node of the element
!                         with respect to the element vector
!     nelarge        i    Size of large matrix elemmt
!     nesmall        i    Size of small matrix elemmtloc
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     rmul           i    Multiplication factor
!     transp         i    Indicates if the transpose of the matrix must be
!                         filled (true) or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      logical mult

!     i              Counting variable
!     j              Counting variable
!     mult           Indication whether multiplication must take place or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   elemmt(induv(number,i),induv(number,j)) := elemmtloc(i,j)  ,i,j,=1(1)nesmall
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      mult = abs(1d0-rmul).gt.1d-8
      if ( jadd.eq.0 .and. .not. mult ) then

!     --- Standard sitation, no multiplication, no adding

         if ( transp ) then

!        --- transpose of matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmtloc(j,i)
               end do
            end do

         else

!        --- matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmtloc(i,j)
               end do
            end do

         end if

      else if ( jadd.eq.1 .and. .not. mult ) then

!     --- no multiplication, but matrix is added to existing one

         if ( transp ) then

!        --- transpose of matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmt(induv(number,i),induv(number,j)) + &
                  elemmtloc(j,i)
               end do
            end do

         else

!        --- matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmt(induv(number,i),induv(number,j)) + &
                  elemmtloc(i,j)
               end do
            end do

         end if

      else if ( jadd.eq.0 ) then

!     --- multiplication by constant, no adding

         if ( transp ) then

!        --- transpose of matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmtloc(j,i)*rmul
               end do
            end do

         else

!        --- matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmtloc(i,j)*rmul
               end do
            end do

         end if

      else

!     --- multiplication, and matrix is added to existing one

         if ( transp ) then

!        --- transpose of matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmt(induv(number,i),induv(number,j)) + &
                  elemmtloc(j,i)*rmul
               end do
            end do

         else

!        --- matrix

            do i = 1, nesmall
               do j = 1, nesmall
                  elemmt(induv(number,i),induv(number,j)) = &
                  elemmt(induv(number,i),induv(number,j)) + &
                  elemmtloc(i,j)*rmul
               end do
            end do

         end if

      end if

      end
