      subroutine el6001(u, s, w, dphidx, mdecl, work)
!
!***********************************************************************
!
!     programmer Luc Timmermans
!     version 4.0   date  18-01-1992  change in definition of dphidx
!     version 3.0   date  17-01-1991  source
!     version 2.0   date  07-12-1990  test version
!     version 1.0   date  11-06-1990  first draft
!
!***********************************************************************
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the convection part of the equation, i.e.
!
!          s   =  s   + / u . grad phi  phi  de
!           ij     ij   e             j    i
!
!                          i
!              =  s   + u(x ) dphidx(i,j) w(i)
!                  ij
!
!            i
!     where x  is the integration point
!
!     EL6001 is called by the element subroutine ELM600
!            it must be preceded by the subroutines ELP640 and ELP641
!
!***********************************************************************
!
!     INPUT/OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none
      integer mdecl
      double precision u(mdecl), s(mdecl,mdecl), w(mdecl), &
                       dphidx(mdecl,mdecl), work(mdecl)
!
!     u        i     array of length mdecl containing the values of the
!                    second coefficient in the integration points
!     s       i/o    array of length mdecl x mdecl containing the spectral
!                    element stiffness matrix, at input s may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!                    since SEPRAN assumes an one-dimensional storage, s
!                    must be stored transposed
!     w        i     array of length mdecl containing the weights for
!                    numerical integration
!     dphidx   i     array of length mdecl x mdecl containing the values of
!                    the derivatives of the basisfunctions in the
!                    integration points
!                                                 k
!                        dphidx(k,j) = dphi / dx(x )
!                                          j
!                                                k
!                    for each integration point x
!     mdecl    i     number of integration/interpolation points (Legendre
!                    Gauss-Lobatto points)
!     work           work array of length mdecl, to store products of
!                    u x w temporarily
!
!***********************************************************************
!
!     PARAMETERS IN COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j
!
!     i, j    counting variables
!
!***********************************************************************

!     * check if convection matrix must be added to an existing matrix
!       or not, if not set s equal to zero

      if (jadd.eq.0) then
!        * jadd = 0, set s equal to zero

         do 200 i = 1, mdecl

            do 100 j = 1, mdecl

               s(i,j) = 0d0
100         continue
200      continue

      end if

!     * add convection matrix to s

!     * first compute u(i) * w(i)

      do 300 i = 1, mdecl

         work(i) = u(i)*w(i)
300   continue

      do 500 i = 1, mdecl

         do 400 j = 1, mdecl

            s(j,i) = s(j,i) + work(i)*dphidx(i,j)
400      continue
500   continue

      end
