      subroutine el6003 ( alpha, s, w1, w2, d1, d2, det, p1, p2, p3, &
                          p4, work, modecl, mdecl, ichois )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.0    date  20-11-1996
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the diffusive part of the equation
!     EL6019 is called by the element subroutine ELM606
!            it must be preceded by the subroutines ELP640 and ELP642
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer modecl, mdecl, ichois
      double precision alpha(mdecl), s(mdecl,mdecl), w1(modecl), &
                       w2(modecl),  d1(modecl,modecl), &
                       d2(modecl,modecl), det(modecl,modecl), &
                       p1(modecl,modecl), p2(modecl,modecl), &
                       p3(modecl,modecl), p4(modecl,modecl), &
                       work(mdecl)

!     alpha    i   array of length mdecl x 3 containing the values of the
!                  first three coefficients in the integration points
!     d1       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     d2       i   array of length modecl x modecl contaning the values of the
!                  derivatives of the basisfunctions in the y-direction
!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     ichois   i
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     p1       i   arrays of length modecl x modecl containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   see p1
!     p3       i   see p1
!     p4       i   see p1
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     w2       i   array of length modecl containing the weights for
!                  numerical integration in the y-direction corresponding
!                  to the reference element
!     work         work array of length mdecl, to store products temporarily
! **********************************************************************
!
!                       COMMON BLOCKS
!
      integer ierror, nerror
      common /cconst/ ierror, nerror
      save /cconst/

!    /cconst/
!    error codes and number of errors to print
!    ierror  i/o  error number of last error occurred
!        0        no error present
!       >0        last run-time error
!    nerror   -   number of error messages printed
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, ij, kl, kj, lj, il, ik
      double precision sum, h1, h2, h3

!     h1         help variable to compute a product
!     h2         help variable to compute a product
!     h3         help variable to compute a product
!     i          Counting variable
!     ij         position in the matrix s
!     ik         position in the matrix s
!     il         help parameter
!     j          Counting variable
!     k          Counting variable
!     kj         position in the matrix s
!     kl         position in the matrix s
!     l          General loop variable
!     lj         position in the matrix s
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INSTOP    Stop the program
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- first initialize s as zero

      do 200 i = 1, mdecl
         do 100 j = 1, mdecl
            s(i,j) = 0d0
 100     continue
 200  continue

      if (ichois.eq.1) then

!     --- compute contribution to deformation D_u matrix
!         | 2 eta   0  |
!         |    0   eta |
!         summation over the integration points
!         store first help factor in work

         do 300 i = 1, modecl

            ij = i
            do 250 j = 1, modecl

               h1 =   p3(i,j)*p3(i,j)
               h2 =   p4(i,j)*p4(i,j)
               h3 = 0.0d0
               work(ij) = (h1*alpha(ij) + 2*h2*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 250        continue
 300     continue

!        --- add first factor of diffusion matrix to s

         do 700 i=1, modecl

            ij = i
            do 600 j = 1, modecl
               do 500 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 400 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
 400              continue

                  s(ij,kj) = s(ij,kj) + sum
 500           continue

               ij = ij + modecl
 600        continue
 700     continue

!        --- store second help factor in work

         do 900 i = 1, modecl

            ij = i
            do 800 j = 1, modecl

               h1 = - p3(i,j)*p1(i,j)
               h2 = - p4(i,j)*p2(i,j)
               h3 = 0.0d0
               work(ij) = (h1*alpha(ij)+2*h2*alpha(ij)) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 800        continue
 900     continue

!        --- add second factor of diffusion matrix to s

         do 1300 i = 1, modecl

            ij = i
            do 1200 j = 1, modecl

               do 1100 k = 1, modecl

                  kl = k
                  do 1000 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
 1000             continue
 1100          continue

               ij = ij + modecl
 1200       continue
 1300    continue

!        --- store third help factor in work

         do 1500 i = 1, modecl

            ij = i
            do 1400 j = 1, modecl

               h1 = - p1(i,j)*p3(i,j)
               h2 = - p2(i,j)*p4(i,j)
               h3 = 0.0d0
               work(ij) = (h1*alpha(ij)+2*h2*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 1400       continue
 1500    continue

!        --- add third factor of diffusion matrix to s

         do 1900 i = 1, modecl

            ij = i
            do 1800 j = 1, modecl
               do 1700 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 1600 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
 1600             continue
 1700          continue

               ij = ij + modecl
 1800       continue
 1900    continue

!        --- store fourth help factor in work

         do 2100 i = 1, modecl

            ij = i
            do 2000 j = 1, modecl

               h1 =   p1(i,j)*p1(i,j)
               h2 =   p2(i,j)*p2(i,j)
               h3 = 0.0d0
               work(ij) = (h1*alpha(ij)+2*h2*alpha(ij)) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 2000       continue
 2100    continue

!        --- add fourth factor of diffusion matrix to s

         do 2500 i = 1, modecl

            ij = i
            do 2400 j = 1, modecl

               do 2300 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 2200 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
 2200             continue

                  s(ij,ik) = s(ij,ik) + sum
 2300          continue

               ij = ij + modecl
 2400       continue
 2500    continue

      end if

      if (ichois.eq.2) then

!     --- compute contribution to deformation D_u matrix
!         |  eta    0  |
!         |   0  2 eta |
!         summation over the integration points
!         store first help factor in work

         do 2700 i = 1, modecl

            ij = i
            do 2600 j = 1, modecl

               h1 =   p3(i,j)*p3(i,j)
               h2 =   p4(i,j)*p4(i,j)
               h3 = 0.0d0
               work(ij) = (2*h1*alpha(ij)+h2*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 2600       continue
 2700    continue

!        --- add first factor of diffusion matrix to s

         do 3100 i=1, modecl

            ij = i
            do 3000 j = 1, modecl
               do 2900 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 2800 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
 2800             continue

                  s(ij,kj) = s(ij,kj) + sum
 2900          continue

               ij = ij + modecl
 3000       continue
 3100    continue

!        --- store second help factor in work

         do 3300 i = 1, modecl

            ij = i
            do 3200 j = 1, modecl

               h1 = - p3(i,j)*p1(i,j)
               h2 = - p4(i,j)*p2(i,j)
               h3 = 0.0d0
               work(ij) = (2*h1*alpha(ij)+h2*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 3200       continue
 3300    continue

!        --- add second factor of diffusion matrix to s

         do 3700 i = 1, modecl

            ij = i
            do 3600 j = 1, modecl

               do 3500 k = 1, modecl

                  kl = k
                  do 3400 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
 3400             continue
 3500          continue

               ij = ij + modecl
 3600       continue
 3700    continue

!        --- store third help factor in work

         do 3900 i = 1, modecl

            ij = i
            do 3800 j = 1, modecl

               h1 = - p1(i,j)*p3(i,j)
               h2 = - p2(i,j)*p4(i,j)
               h3 = 0.0d0
               work(ij) = (2*h1*alpha(ij)+h2*alpha(ij)) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 3800       continue
 3900    continue

!        --- add third factor of diffusion matrix to s

         do 4300 i = 1, modecl

            ij = i
            do 4200 j = 1, modecl
               do 4100 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 4000 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
 4000             continue
 4100          continue

               ij = ij + modecl
 4200       continue
 4300    continue

!        --- store fourth help factor in work

         do 4500 i = 1, modecl

            ij = i
            do 4400 j = 1, modecl

               h1 =   p1(i,j)*p1(i,j)
               h2 =   p2(i,j)*p2(i,j)
               h3 = 0.0d0
               work(ij) = (2*h1*alpha(ij)+h2*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 4400       continue
 4500    continue

!        --- add fourth factor of diffusion matrix to s

         do 4900 i = 1, modecl

            ij = i
            do 4800 j = 1, modecl

               do 4700 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 4600 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
 4600             continue

                  s(ij,ik) = s(ij,ik) + sum
 4700          continue

               ij = ij + modecl
 4800       continue
 4900    continue

      end if

!     --- compute contribution to deformation D_u matrix
!         | 0   0 |
!         | eta 0 |
!         summation over the integration points
!         store first help factor in work

      if (ichois.eq.3) then
         do 5100 i = 1, modecl

            ij = i
            do 5000 j = 1, modecl

               h1 =   0.0d0
               h2 =   0.0d0
               h3 = - p4(i,j)*p3(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 5000       continue
 5100    continue

!        --- add first factor of diffusion matrix to s

         do 5500 i=1, modecl

            ij = i
            do 5400 j = 1, modecl
               do 5300 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 5200 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
 5200             continue

                  s(ij,kj) = s(ij,kj) + sum

 5300          continue

               ij = ij + modecl
 5400       continue
 5500    continue

!        --- store second help factor in work

         do 5700 i = 1, modecl

            ij = i
            do 5600 j = 1, modecl

               h1 = 0.0d0
               h2 = 0.0d0
               h3 =   p4(i,j)*p1(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 5600       continue
 5700    continue

!        --- add second factor of diffusion matrix to s

         do 6000 i = 1, modecl

            ij = i
            do 5900 j = 1, modecl

               do 5800 k = 1, modecl

                  kl = k
                  do 5701 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
 5701             continue
 5800          continue

               ij = ij + modecl
 5900       continue
 6000    continue

!        --- store third help factor in work

         do 6200 i = 1, modecl

            ij = i
            do 6100 j = 1, modecl

               h1 = 0.0d0
               h2 = 0.0d0
               h3 = p2(i,j)*p3(i,j)
               work(ij) = (h3*alpha(ij)) &
                    * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 6100       continue
 6200    continue

!        --- add third factor of diffusion matrix to s

         do 6600 i = 1, modecl

            ij = i
            do 6500 j = 1, modecl
               do 6400 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 6300 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
 6300             continue
 6400          continue

               ij = ij + modecl
 6500       continue
 6600    continue

!        --- store fourth help factor in work

         do 6800 i = 1, modecl

            ij = i
            do 6700 j = 1, modecl

               h1 =   0.0d0
               h2 =   0.0d0
               h3 = - p2(i,j)*p1(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 6700       continue
 6800    continue

!        --- add fourth factor of diffusion matrix to s

         do 7200 i = 1, modecl

            ij = i
            do 7100 j = 1, modecl

               do 7000 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 6900 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
 6900             continue

                  s(ij,ik) = s(ij,ik) + sum
 7000          continue

               ij = ij + modecl
 7100       continue
 7200    continue
      end if

      if (ichois.eq.4) then

!     --- compute contribution to deformation D_u matrix
!         | 0   eta |
!         | 0     0 |
!         summation over the integration points
!         store first help factor in work

         do 7600 i = 1, modecl

            ij = i
            do 7500 j = 1, modecl

               h1 =   0.0d0
               h2 =   0.0d0
               h3 = - p3(i,j)*p4(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 7500       continue
 7600    continue

!        --- add first factor of diffusion matrix to s

         do 8000 i=1, modecl

            ij = i
            do 7900 j = 1, modecl
               do 7800 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  sum = 0d0
                  do 7700 l = 1, modecl

                     lj = l + (j - 1)*modecl
                     sum = sum + work(lj)*d1(l,k)*d1(l,i)
 7700             continue

                  s(ij,kj) = s(ij,kj) + sum
 7800          continue

               ij = ij + modecl
 7900       continue
 8000    continue

!        --- store second help factor in work

         do 8200 i = 1, modecl

            ij = i
            do 8100 j = 1, modecl

               h1 = 0.0d0
               h2 = 0.0d0
               h3 =   p3(i,j)*p2(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 8100       continue
 8200    continue

!        --- add second factor of diffusion matrix to s

         do 8600 i = 1, modecl

            ij = i
            do 8500 j = 1, modecl

               do 8400 k = 1, modecl

                  kl = k
                  do 8300 l = 1, modecl

                     il = i + (l - 1)*modecl
                     s(ij,kl) = s(ij,kl) + work(il)*d1(i,k)*d2(l,j)
                     kl = kl + modecl
 8300             continue
 8400          continue

               ij = ij + modecl
 8500       continue
 8600    continue

!        --- store third help factor in work

         do 8800 i = 1, modecl

            ij = i
            do 8700 j = 1, modecl

               h1 = 0.0d0
               h2 = 0.0d0
               h3 =   p1(i,j)*p4(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 8700       continue
 8800    continue

!        --- add third factor of diffusion matrix to s

         do 9200 i = 1, modecl

            ij = i
            do 9100 j = 1, modecl
               do 9000 k = 1, modecl

                  kj = k + (j - 1)*modecl
                  kl = k
                  h1 = work(kj)*d1(k,i)
                  do 8900 l = 1, modecl

                     s(ij,kl) = s(ij,kl) + h1*d2(j,l)
                     kl = kl  + modecl
 8900             continue
 9000          continue

               ij = ij + modecl
 9100       continue
 9200    continue

!        --- store fourth help factor in work

         do 9400 i = 1, modecl

            ij = i
            do 9300 j = 1, modecl

               h1 =   0.0d0
               h2 =   0.0d0
               h3 = - p1(i,j)*p2(i,j)
               work(ij) = (h3*alpha(ij)) &
                          * (w1(i)*w2(j)/det(i,j))
               ij = ij + modecl
 9300       continue
 9400    continue

!        --- add fourth factor of diffusion matrix to s

         do 9800 i = 1, modecl

            ij = i
            do 9700 j = 1, modecl

               do 9600 k = 1, modecl

                  ik = i + (k - 1)*modecl
                  sum = 0d0
                  do 9500 l = 1, modecl

                     il = i + (l - 1)*modecl
                     sum = sum + work(il)*d2(l,k)*d2(l,j)
 9500             continue

                  s(ij,ik) = s(ij,ik) + sum
 9600          continue

               ij = ij + modecl
 9700       continue
 9800    continue

      end if

      if (ierror.ne.0) call instop

      end
