      subroutine el6011(u, f, old, w, dphidx, mdecl, work)
! ======================================================================
!
!        programmer     Luc Timmermans
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  08-06-1992
!
!
!   copyright (c) 1992-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the convection part of the equation in the
!     case of finite element preconditioning
!
!     EL6011 is called by the element subroutine ELM600
!            it must be preceded by the subroutines ELP640 and ELP641
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mdecl
      double precision u(mdecl), f(mdecl), old(mdecl), w(mdecl), &
                       dphidx(mdecl,mdecl), work(mdecl)

!     dphidx   i     array of length mdecl x mdecl containing the values of
!                    the derivatives of the basisfunctions in the
!                    integration points
!                                                 k
!                        dphidx(k,j) = dphi / dx(x )
!                                          j
!                                                k
!                    for each integration point x
!     f       i/o    array of length mdecl containing the spectral
!                    element residue vector, at input f may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!     mdecl    i     number of integration/interpolation points (Legendre
!                    Gauss-Lobatto points)
!     old      i     array of length mdecl containing the solution of
!                    the previous iteration step in the nodal points
!     u        i     array of length mdecl containing the values of the
!                    second coefficient in the integration points
!     w        i     array of length mdecl containing the weights for
!                    numerical integration
!     work           work array of length mdecl, to store products of
!                    u x w temporarily
!***********************************************************************
!
!                  COMMON BLOCKS
!
!***********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision sum

!     i       counting variable
!     j       counting variable
!     sum     help parameter to compute a sum
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- check if convection part must be added to an existing part
!         or not, if not set f equal to zero

      if (jadd.eq.0) then

!     --- jadd = 0, set f equal to zero

         do 100 i = 1, mdecl
            f(i) = 0d0
100      continue

      end if

!     --- add convection part to f
!         first compute u(i) * w(i)

      do 300 i = 1, mdecl
         work(i) = u(i)*w(i)
300   continue

      do 500 i = 1, mdecl

         sum = 0d0
         do 400 j = 1, mdecl
            sum = sum + dphidx(j,i)*old(j)
400      continue

         f(i) = f(i) + sum*work(i)
500   continue

      end
