      subroutine el6016 ( alpha, s, w, dphidx, mdecl, work, det )
! ======================================================================
!
!        programmer     Luc Timmermans/ Peter Minev
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  24-04-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the diffusive part of the equation, i.e.
!
!          s   =  s   + /  alpha grad phi . grad phi de
!           ij     ij   e                j          i
!
!                                   k
!              =  s   +  sum alpha(x ) dphidx(k,j) dphidx(k,j) w(k)/ det
!                  ij     k
!
!             k
!     where  x  is the integration point
!
!     EL6016 is called by the element subroutine ELM607
!            it must be preceded by the subroutines ELP640 and ELP647
!
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
!***********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none
      integer mdecl
      double precision alpha(mdecl), s(mdecl,mdecl), w(mdecl), &
                       dphidx(mdecl,mdecl), work(mdecl), det

!     alpha    i   array of length mdecl containing the values of the
!                  first coefficient in the integration points
!     det      i   the jacobian of the transform
!     dphidx   i   array of length mdecl x mdecl containing the values of
!                  the derivatives of the basisfunctions in the
!                  integration points
!                                                  k
!                        dphidx(k,j) = dphi  / dx(x )
!                                          j
!                                              k
!                  for each integration point x
!     mdecl    i   number of integration/interpolation points
!     s        o   array of length mdecl x mdecl containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w        i   array of length mdecl containing the weights for
!                  numerical integration
!     work         work array of length mdecl, to store products of
!                  alpha x w temporarily
!***********************************************************************
!
!               COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, k
      double precision  sum, acon

!     acon       constant value of alpha
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     sum        Help parameter to compute a sum
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      if (jsecnd.eq.1) then

!     --- jsecnd = 1, alpha constant

         acon = alpha(1)

!        --- summation over the integration points

         do 300 i = 1, mdecl
            do 200 j = 1, i
               sum = 0d0
               do 100 k = 1, mdecl

                  sum = sum + w(k)*dphidx(k,i)*dphidx(k,j)
100            continue

               s(i,j) = sum*acon/det
200         continue
300      continue
      end if

      if (jsecnd.eq.2) then

!     --- jescnd = 2, alpha general
!         summation over the integration points
!         first compute alpha(k) * w(k)

         do 400 k = 1, mdecl
            work(k) = alpha(k)*w(k)
400      continue

         do 700 i = 1, mdecl
            do 600 j = 1, i
               sum = 0d0
               do 500 k = 1, mdecl
                  sum = sum + work(k)*dphidx(k,j)*dphidx(k,i)
500            continue
               s(i,j) = sum/det
600         continue
700      continue

      end if

!     --- the upper triangle of s is computed by transposition

      do 900 i = 1, mdecl
         do 800 j = 1, i-1
            s(j,i) = s(i,j)
800      continue
900   continue

      end
