      subroutine el6033(g, f, w1, det, modecl, mdecl)
! ======================================================================
!
!        programmer    Luc Timmermans
!        version 2.0   date  26-09-1994  equal number of nodes in each
!                                       direction) - P. Minev
!        version 1.0   date  14-04-1992  test version
!
!   copyright (c) 1992-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!***********************************************************************
!
!                         DESCRIPTION
!     compute contributions to the spectral element residue vector
!     corresponding to the righthand side of the equation in the
!     case of finite element preconditioning
!
!     EL6033 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!                      KEYWORDS
!
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision g(mdecl), f(mdecl), w1(modecl), &
                       det(modecl,modecl)

!     det      i   array of length modecl x modecl containing the
!                  transformation determinant
!     f       i/o  array of length mdecl containing the spectral element
!                  residue vector, at input f may already have been filled
!     g        i   array of length mdecl containing the values of the
!                  fourth coefficient (righthand side) in the integration
!     node
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in 1 direction corresponding
!                  to the reference element
!***********************************************************************
!
!                      COMMON BLOCKS
!
!***********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer i, j, ij
      double precision gcon, help

!     gcon   constant value in the case that  g is a constant function
!     help   help parameter
!     i      general loop variable
!     ij     position variable
!     j      general loop variable
!***********************************************************************
!
!               SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- check if righthanside part must be added to an existing vector
!         or not, if not set f equal to zero
!
      if (jadd.eq.0) then

!     --- jadd = 0, set f equal to zero

         do 100 i = 1, mdecl
            f(i) = 0d0
100      continue

      end if

!     --- add right-hand-side part to f

      if (jconsf.eq.1) then

!     --- g is constant (per element)

         gcon = g(1)
         do 400 i = 1, modecl

            help = gcon*w1(i)
            ij = i
            do 300 j = 1, modecl

               f(ij) = f(ij) - help*w1(j)*det(i,j)
               ij = ij + modecl
300         continue
400      continue

      else

!     --- g is variable

         do 600 i = 1, modecl

            ij = i
            do 500 j = 1, modecl

               f(ij) = f(ij) - w1(i)*w1(j)*det(i,j)*g(ij)
               ij = ij + modecl
500         continue
600      continue

      end if

      end
