      subroutine el6050(beta, s, w1, det, modecl, mdecl)
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  29-11-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to the spectral element vector
!     corresponding to the helmholtz part of the equation in 3-D case
!
!     EL6050 is called by the element subroutine elm600
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision beta(mdecl), s(mdecl,mdecl), w1(modecl), &
                       det(modecl,modecl,modecl)

!     beta     i   array of length m containing the values of the
!                  third coefficient in the integration points
!     det      i   array of length modecl x modecl x modecl containing the
!                  transformation determinant
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     s        o   Element matrix to be filled
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!***********************************************************************
!
!                        COMMON BLOCKS
!
!***********************************************************************
!
!                           LOCAL PARAMETERS
!
      integer i, j, ijk, k, mtdecl

!     i          Counting variable
!     ijk        Counting variable
!     j          Counting variable
!     k          Counting variable
!     mtdecl
!***********************************************************************
!
!                      SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
!***********************************************************************
!
!     ERROR MESSAGES
!
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      mtdecl = modecl*modecl

!     --- check if Helmholtz matrix must be added to an existing matrix
!         or not, if not set s equal to zero

      if (jadd.eq.0) then

!     --- jadd = 0, set s equal to zero

         do 200 i = 1, mdecl
            do 100 j = 1, mdecl

               s(i,j) = 0d0
100         continue
200      continue

      end if

!     --- add Helmholtz matrix to s


         do 500 i = 1, modecl
            do 600 j = 1, modecl
               do 700 k = 1, modecl

               ijk = i + (j-1)*modecl + (k-1)*mtdecl
               s(ijk,ijk) =s(ijk,ijk)+ &
                           beta(ijk)*w1(i)*w1(j)*w1(k)*det(i,j,k)
700            continue
600         continue
500      continue

      end
