      subroutine el6052 ( f, elemwg,old, dx, p1, p2, p3, p4, &
                          det, work, mone )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.2    date  25-06-1996  Correction in use elemwg
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  10-11-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the strong gradient of a scalar in the case of spectral
!     elements;
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer mone
      double precision f(2,mone,mone), elemwg(2,mone,mone), &
                       old(mone,mone), work(mone,mone), &
                       dx(mone,mone), det(mone,mone), &
                       p1(mone,mone), p2(mone,mone), &
                       p3(mone,mone), p4(mone,mone)

!     det      i   jacobian of the transform of the element
!     dx       i   array of length mone x mone contaning the values of the
!                  derivatives of the basisfunctions
!     elemwg   o   element vector containing the weights for subroutine deriva
!     f        o   array of length 2*mone*mone containing the gradient
!     mone     i   number of velocity points in the one dimension
!     old      i   array of length mone*mone containing the scalar
!     p1       i   arrays of length mone x mone containing the partial
!                  derivatives of the coordinates due to transformation for
!                  the pressure
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     work     -   work array, length mone*mone
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i            counting variable
!     j            counting variable
!     k            counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do 600 i = 1, mone

         do 500 j = 1, mone

!        --- calculating the first sum

         work(i,j) = 0d0
         do 400 k = 1, mone
            work(i,j) = work(i,j) + old(k,j)*dx(i,k)
400      continue

!        --- first component of the gradient

         f(1,i,j) = work(i,j)*p4(i,j)

!        --- second component of the gradient

         f(2,i,j) = -work(i,j)* p3(i,j)

!        --- calculating the second sum

         work(i,j) = 0d0
         do 300 k = 1, mone
            work(i,j) = work(i,j) + old(i,k)*dx(j,k)
300      continue

!        --- first component of the gradient

         f(1,i,j) = (f(1,i,j) - work(i,j)*p2(i,j))/det(i,j)
         elemwg(1,i,j) = 1.0d0

!        --- second component of the gradient

         f(2,i,j) = (f(2,i,j) + work(i,j)*p1(i,j))/det(i,j)
         elemwg(2,i,j) = 1.0d0

500      continue
600   continue

      end
