      subroutine el6084 ( alpha, s, w1, d1, det, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, work, mod, md )
! ======================================================================
!
!        programmer     Peter Minev / Patrick Anderson
!        version 2.0    date   28-02-1997 Completely rewritten (PdA)
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   29-09-1994
!
!   copyright (c) 1994-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the diffusive part of the equation - 3-D case
!     EL6084 is called by the element subroutine ELM600
!            it must be preceded by the subroutines ELP640 and ELP646
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mod, md
      double precision alpha(mod,mod,mod,6), s(md,md), w1(mod), &
                       d1(mod,mod), det(mod,mod,mod), &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), work(mod,mod,mod)

!     alpha    i   array of length md  containing the values of the
!                  first 3 coefficients in the integration points
!     d1       i   array of length mod x mod contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     det      i   array o length md containing the
!                  transformation determinant
!     md       i   number of integration points in the element
!     mod      i   number of integration points in one dimension
!     p1       i   arrays of length mod x mod containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     s        o   array of length md x md containing the spectral
!                  element stiffness matrix, since SEPRAN assumes an
!                  one-dimensional storage, s must be stored transposed
!     w1       i   array of length mod containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work         work array of length md, to store products temporarily
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, mloc, nloc, ijk, imn, ljn, lmk, &
              imk, ijn, ljk, mtd
      double precision sum, h1, h2, h3, h4, h5, h6, a11con

!     a11con     Constant value of alpha
!     h1         Help variable to compute a product
!     h2         Help variable to compute a product
!     h3         Help variable to compute a product
!     h4         Help variable to compute a product
!     h5         Help variable to compute a product
!     h6         Help variable to compute a product
!     i          Counting variable
!     ijk        position in the matrix s
!     ijn        position in the matrix s
!     imk        position in the matrix s
!     imn        position in the matrix s
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     ljk        position in the matrix s
!     ljn        position in the matrix s
!     lmk        position in the matrix s
!     mloc       Counting variable
!     mtd        mtd = mod*mod
!     nloc       Counting variable
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      mtd = mod*mod

!     --- first initialize s as zero, if necessary

      if (jadd.eq.0) then
         do 200 i = 1, md
            do 100 j = 1, md
               s(i,j) = 0d0
100         continue
200      continue
      end if

      if (jsecnd.eq.1) then

!     --- jsecnd = 1, alpha11 = constant

         a11con = alpha(1,1,1,1)

!        --- store first help factor in work

         do 400 i = 1, mod
            h1 = a11con*w1(i)
            do 300 j = 1, mod
               do 350 k = 1, mod
                  h2 = p1(i,j,k)*p1(i,j,k) + p2(i,j,k)*p2(i,j,k) + &
                       p3(i,j,k)*p3(i,j,k)
                  work(i,j,k) = h1*h2*w1(j)*w1(k)/det(i,j,k)
350            continue
300         continue
400      continue

!        --- add first factor of diffusion matrix to s

         do 800 i = 1, mod
            do 700 j = 1, mod
               do 600 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 500 l = 1, mod
                     sum = 0d0
                     do 550 mloc = 1, mod
                        sum = sum + work(mloc,j,k)*d1(mloc,i)*d1(mloc,l)
550                  continue
                     ljk = l + (j-1)*mod + (k-1)*mtd
                     s(ijk,ljk) = s(ijk,ljk) + sum
500               continue
600            continue
700         continue
800      continue

!        --- store second help factor in work

         do 1000 i = 1, mod
            h1 = a11con*w1(i)
            do 900 j = 1, mod
               do 950 k = 1, mod
                  h2 =  p1(i,j,k)*p4(i,j,k) + p2(i,j,k)*p5(i,j,k) + &
                        p3(i,j,k)*p6(i,j,k)
                  work(i,j,k) = h1*h2*w1(j)*w1(k)/det(i,j,k)
950            continue
900         continue
1000     continue

!        --- add second and third factor of diffusion matrix to s

         do 1400 i = 1, mod
            do 1300 j = 1, mod
               do 1200 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 1100 l = 1, mod
                     do 1150 mloc = 1, mod
                        lmk = l + (mloc-1)*mod + (k-1)*mtd

!                       --- add second factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(i,mloc,k)*d1(i,l)*d1(mloc,j)

!                       --- add third factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(l,j,k)*d1(j,mloc)*d1(l,i)
1150                 continue
1100              continue
1200           continue

1300        continue
1400     continue

!        --- store third help factor in work

         do 1005 i = 1, mod
            h1 = a11con*w1(i)
            do 905 j = 1, mod
               do 955 k = 1, mod
                  h2 =  p1(i,j,k)*p7(i,j,k) + p2(i,j,k)*p8(i,j,k) + &
                        p3(i,j,k)*p9(i,j,k)
               work(i,j,k) = h1*h2*w1(j)*w1(k)/det(i,j,k)
955            continue
905         continue
1005     continue

!        --- add fourth and fifth factor of diffusion matrix to s

         do 1500 i = 1, mod
            do 1600 j = 1, mod
               do 1700 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 1800 l = 1, mod
                     do 1850 nloc = 1, mod
                        ljn = l + (j-1)*mod + (nloc-1)*mtd

!                       --- add fourth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(i,j,nloc)*d1(i,l)*d1(nloc,k)

!                       --- add fifth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(l,j,k)*d1(k,nloc)*d1(l,i)
1850                 continue
1800              continue
1700           continue
1600        continue
1500     continue

!        --- store fourth help factor in work

         do 1900 i = 1, mod
            h1 = a11con*w1(i)
            do 2000 j = 1, mod
               do 2100 k = 1, mod
                  h2 = p4(i,j,k)*p4(i,j,k) + p5(i,j,k)*p5(i,j,k) + &
                       p6(i,j,k)*p6(i,j,k)
               work(i,j,k) = h1*h2*w1(j)*w1(k)/det(i,j,k)
2100           continue
2000        continue
1900     continue

!        --- add sixth factor of diffusion matrix to s

         do 2200 i = 1, mod
            do 2300 j = 1, mod
               do 2400 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 2500 mloc = 1, mod
                  sum = 0d0
                  do 2550 l = 1, mod
                     sum = sum + work(i,l,k)*d1(l,mloc)*d1(l,j)
2550              continue
                  imk = i + (mloc-1)*mod + (k-1)*mtd
                  s(ijk,imk) = s(ijk,imk) + sum
2500              continue
2400           continue
2300        continue
2200     continue

!        --- store fifth help factor in work

         do 2600 i = 1, mod
            h1 = a11con*w1(i)
            do 2700 j = 1, mod
               do 2800 k = 1, mod
                  h2 = p4(i,j,k)*p7(i,j,k) + p5(i,j,k)*p8(i,j,k) + &
                       p6(i,j,k)*p9(i,j,k)
                  work(i,j,k) = h1*h2*w1(j)*w1(k)/det(i,j,k)
2800           continue
2700        continue
2600     continue

!        --- add seventh and eighth factor of diffusion matrix to s

         do 2900 i = 1, mod
            do 3000 j = 1, mod
               do 3100 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 3200 mloc = 1, mod
                     do 3300 nloc = 1, mod
                        imn = i + (mloc-1)*mod + (nloc-1)*mtd

!                       --- add seventh factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,j,nloc)*d1(j,mloc)*d1(nloc,k)

!                       --- add eighth factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,mloc,k)*d1(k,nloc)*d1(mloc,j)
3300                 continue
3200              continue
3100           continue
3000        continue
2900     continue

!        --- store sixth help factor in work

         do 3400 i = 1, mod

            h1 = a11con*w1(i)
            do 3500 j = 1, mod
               do 3600 k = 1, mod
                  h2 = p7(i,j,k)*p7(i,j,k) + p8(i,j,k)*p8(i,j,k) + &
                       p9(i,j,k)*p9(i,j,k)
               work(i,j,k) = h1*h2*w1(j)*w1(k)/det(i,j,k)
3600           continue
3500        continue
3400     continue

!        --- add ninth factor of diffusion matrix to s

         do 3700 i = 1, mod

            do 3800 j = 1, mod

               do 3900 k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd

                  do 4000 nloc = 1, mod

                  sum = 0d0
                  do 4100 l = 1, mod
                     sum = sum + work(i,j,l)*d1(l,nloc)*d1(l,k)
4100              continue

                  ijn = i + (j-1)*mod + (nloc-1)*mtd
                  s(ijk,ijn) = s(ijk,ijn) + sum
4000              continue
3900           continue
3800        continue
3700     continue

      else

!     --- jsecnd greater than 1
!         --- store first help factor in work

         do 401 i = 1, mod
            do 301 j = 1, mod
               do 351 k = 1, mod
                  h1 = p1(i,j,k)*p1(i,j,k)
                  h2 = p2(i,j,k)*p1(i,j,k) +  p1(i,j,k)*p2(i,j,k)
                  h3 = p3(i,j,k)*p1(i,j,k) +  p1(i,j,k)*p3(i,j,k)
                  h4 = p2(i,j,k)*p2(i,j,k)
                  h5 = p3(i,j,k)*p2(i,j,k) +  p2(i,j,k)*p3(i,j,k)
                  h6 = p3(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
351            continue
301         continue
401      continue

!        --- add first factor of diffusion matrix to s

         do 801 i = 1, mod
            do 701 j = 1, mod
               do 601 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 501 l = 1, mod
                     sum = 0d0
                     do 551 mloc = 1, mod
                        sum = sum + work(mloc,j,k)*d1(mloc,i)*d1(mloc,l)
551                  continue
                     ljk = l + (j-1)*mod + (k-1)*mtd
                     s(ijk,ljk) = s(ijk,ljk) + sum
501               continue
601            continue
701         continue
801      continue

!        --- store second help factor in work

         do 1001 i = 1, mod
            do 901 j = 1, mod
               do 951 k = 1, mod
                  h1 = p1(i,j,k)*p4(i,j,k)
                  h2 = p2(i,j,k)*p4(i,j,k) +  p1(i,j,k)*p5(i,j,k)
                  h3 = p3(i,j,k)*p4(i,j,k) +  p1(i,j,k)*p6(i,j,k)
                  h4 = p2(i,j,k)*p5(i,j,k)
                  h5 = p3(i,j,k)*p5(i,j,k) +  p2(i,j,k)*p6(i,j,k)
                  h6 = p3(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
951            continue
901         continue
1001     continue

!        --- add second factor of diffusion matrix to s

         do 1401 i = 1, mod
            do 1301 j = 1, mod
               do 1201 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 1101 l = 1, mod
                     do 1151 mloc = 1, mod
                        lmk = l + (mloc-1)*mod + (k-1)*mtd

!                       --- add second factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(i,mloc,k)*d1(i,l)*d1(mloc,j)

1151                 continue
1101              continue
1201           continue

1301        continue
1401     continue

!        --- store third help factor in work

         do 1002 i = 1, mod
            do 902 j = 1, mod
               do 952 k = 1, mod
                  h1 = p4(i,j,k)*p1(i,j,k)
                  h2 = p5(i,j,k)*p1(i,j,k) +  p4(i,j,k)*p2(i,j,k)
                  h3 = p6(i,j,k)*p1(i,j,k) +  p4(i,j,k)*p3(i,j,k)
                  h4 = p5(i,j,k)*p2(i,j,k)
                  h5 = p6(i,j,k)*p2(i,j,k) +  p5(i,j,k)*p3(i,j,k)
                  h6 = p6(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
952            continue
902         continue
1002     continue

!        --- add third factor of diffusion matrix to s

         do 1402 i = 1, mod
            do 1302 j = 1, mod
               do 1202 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 1102 l = 1, mod
                     do 1152 mloc = 1, mod
                        lmk = l + (mloc-1)*mod + (k-1)*mtd

!                       --- add third factor of diffusion matrix to s

                        s(ijk,lmk)=s(ijk,lmk)+ &
                                   work(l,j,k)*d1(j,mloc)*d1(l,i)
1152                 continue
1102              continue
1202           continue

1302        continue
1402     continue

!        --- store fourth help factor in work

         do 1003 i = 1, mod
            do 903 j = 1, mod
               do 953 k = 1, mod
                  h1 = p1(i,j,k)*p7(i,j,k)
                  h2 = p2(i,j,k)*p7(i,j,k) +  p1(i,j,k)*p8(i,j,k)
                  h3 = p3(i,j,k)*p7(i,j,k) +  p1(i,j,k)*p9(i,j,k)
                  h4 = p2(i,j,k)*p8(i,j,k)
                  h5 = p3(i,j,k)*p8(i,j,k) +  p2(i,j,k)*p9(i,j,k)
                  h6 = p3(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
953            continue
903         continue
1003     continue

!        --- add fourth factor of diffusion matrix to s

         do 1501 i = 1, mod
            do 1601 j = 1, mod
               do 1701 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 1801 l = 1, mod
                     do 1851 nloc = 1, mod
                        ljn = l + (j-1)*mod + (nloc-1)*mtd

!                       --- add fourth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(i,j,nloc)*d1(i,l)*d1(nloc,k)

1851                 continue
1801              continue
1701           continue
1601        continue
1501     continue

!        --- store fifth help factor in work

         do 1004 i = 1, mod
            do 904 j = 1, mod
               do 954 k = 1, mod
                  h1 = p7(i,j,k)*p1(i,j,k)
                  h2 = p8(i,j,k)*p1(i,j,k) +  p7(i,j,k)*p2(i,j,k)
                  h3 = p9(i,j,k)*p1(i,j,k) +  p7(i,j,k)*p3(i,j,k)
                  h4 = p8(i,j,k)*p2(i,j,k)
                  h5 = p9(i,j,k)*p2(i,j,k) +  p8(i,j,k)*p3(i,j,k)
                  h6 = p9(i,j,k)*p3(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
954            continue
904         continue
1004     continue

!        --- add fifth factor of diffusion matrix to s

         do 1502 i = 1, mod
            do 1602 j = 1, mod
               do 1702 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 1802 l = 1, mod
                     do 1852 nloc = 1, mod
                        ljn = l + (j-1)*mod + (nloc-1)*mtd

!                       --- add fifth factor of diffusion matrix to s

                        s(ijk,ljn)=s(ijk,ljn)+ &
                                   work(l,j,k)*d1(k,nloc)*d1(l,i)
1852                 continue
1802              continue
1702           continue
1602        continue
1502     continue

!        --- store sixth help factor in work

         do 1901 i = 1, mod
            do 2001 j = 1, mod
               do 2101 k = 1, mod
                  h1 = p4(i,j,k)*p4(i,j,k)
                  h2 = p5(i,j,k)*p4(i,j,k) +  p4(i,j,k)*p5(i,j,k)
                  h3 = p6(i,j,k)*p4(i,j,k) +  p4(i,j,k)*p6(i,j,k)
                  h4 = p5(i,j,k)*p5(i,j,k)
                  h5 = p6(i,j,k)*p5(i,j,k) +  p5(i,j,k)*p6(i,j,k)
                  h6 = p6(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
2101           continue
2001        continue
1901     continue

!        --- add sixth factor of diffusion matrix to s

         do 2201 i = 1, mod
            do 2301 j = 1, mod
               do 2401 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 2501 mloc = 1, mod
                  sum = 0d0
                  do 2551 l = 1, mod
                     sum = sum + work(i,l,k)*d1(l,mloc)*d1(l,j)
2551              continue
                  imk = i + (mloc-1)*mod + (k-1)*mtd
                  s(ijk,imk) = s(ijk,imk) + sum
2501              continue
2401           continue
2301        continue
2201     continue

!        --- store seventh help factor in work

         do 2601 i = 1, mod
            do 2701 j = 1, mod
               do 2801 k = 1, mod
                  h1 = p4(i,j,k)*p7(i,j,k)
                  h2 = p5(i,j,k)*p7(i,j,k) +  p4(i,j,k)*p8(i,j,k)
                  h3 = p6(i,j,k)*p7(i,j,k) +  p4(i,j,k)*p9(i,j,k)
                  h4 = p5(i,j,k)*p8(i,j,k)
                  h5 = p6(i,j,k)*p8(i,j,k) +  p5(i,j,k)*p9(i,j,k)
                  h6 = p6(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
2801           continue
2701        continue
2601     continue

!        --- add seventh factor of diffusion matrix to s

         do 2901 i = 1, mod
            do 3001 j = 1, mod
               do 3101 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 3201 mloc = 1, mod
                     do 3301 nloc = 1, mod
                        imn = i + (mloc-1)*mod + (nloc-1)*mtd

!                       --- add seventh factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,j,nloc)*d1(j,mloc)*d1(nloc,k)

3301                 continue
3201              continue
3101           continue
3001        continue
2901     continue

!        --- store eighth help factor in work

         do 2602 i = 1, mod
            do 2702 j = 1, mod
               do 2802 k = 1, mod
                  h1 = p7(i,j,k)*p4(i,j,k)
                  h2 = p8(i,j,k)*p4(i,j,k) +  p7(i,j,k)*p5(i,j,k)
                  h3 = p9(i,j,k)*p4(i,j,k) +  p7(i,j,k)*p6(i,j,k)
                  h4 = p8(i,j,k)*p5(i,j,k)
                  h5 = p9(i,j,k)*p5(i,j,k) +  p8(i,j,k)*p6(i,j,k)
                  h6 = p9(i,j,k)*p6(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
2802           continue
2702        continue
2602     continue

!        --- add eighth factor of diffusion matrix to s

         do 2902 i = 1, mod
            do 3002 j = 1, mod
               do 3102 k = 1, mod
                  ijk = i + (j-1)*mod + (k-1)*mtd
                  do 3202 mloc = 1, mod
                     do 3302 nloc = 1, mod
                        imn = i + (mloc-1)*mod + (nloc-1)*mtd

!                       --- add eighth factor of diffusion matrix to s

                        s(ijk,imn)=s(ijk,imn)+ &
                                   work(i,mloc,k)*d1(k,nloc)*d1(mloc,j)
3302                 continue
3202              continue
3102           continue
3002        continue
2902     continue

!        --- store ninth help factor in work

         do 3401 i = 1, mod

            do 3501 j = 1, mod
               do 3601 k = 1, mod
                  h1 = p7(i,j,k)*p7(i,j,k)
                  h2 = p8(i,j,k)*p7(i,j,k) +  p7(i,j,k)*p8(i,j,k)
                  h3 = p9(i,j,k)*p7(i,j,k) +  p7(i,j,k)*p9(i,j,k)
                  h4 = p8(i,j,k)*p8(i,j,k)
                  h5 = p9(i,j,k)*p8(i,j,k) +  p8(i,j,k)*p9(i,j,k)
                  h6 = p9(i,j,k)*p9(i,j,k)

                  work(i,j,k) = h1*alpha(i,j,k,1) + h2*alpha(i,j,k,2) + &
                                h3*alpha(i,j,k,3) + h4*alpha(i,j,k,4) + &
                                h5*alpha(i,j,k,5) + h6*alpha(i,j,k,6)

                  work(i,j,k) = work(i,j,k)*w1(i)*w1(j)*w1(k)/det(i,j,k)
3601           continue
3501        continue
3401     continue

!        --- add ninth factor of diffusion matrix to s

         do 3701 i = 1, mod

            do 3801 j = 1, mod

               do 3901 k = 1, mod
               ijk = i + (j-1)*mod + (k-1)*mtd

                  do 4001 nloc = 1, mod

                  sum = 0d0
                  do 4101 l = 1, mod
                     sum = sum + work(i,j,l)*d1(l,nloc)*d1(l,k)
4101              continue

                  ijn = i + (j-1)*mod + (nloc-1)*mtd
                  s(ijk,ijn) = s(ijk,ijn) + sum
4001              continue
3901           continue
3801        continue
3701     continue

      end if

      end
