      subroutine el6085 ( alpha, f, w1, d1, det, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, work, old, mod )
! ======================================================================
!
!        programmer     Peter Minev
!        version 2.1    date   28-01-1996 Layout
!        version 2.0    date   24-10-1995  (completely rewritten)
!        version 1.0    date   18-01-1995
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element stiffness residual
!     in the case of matrix-free iterative solver - 3-D case
!
!     EL6085 is called by the element subroutine ELM607
!            it must be preceded by the subroutines ELP640 and ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mod
      double precision alpha(mod,mod,mod), f(mod,mod,mod), w1(mod), &
                       d1(mod,mod), det(mod,mod,mod), &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), &
                       work(mod,mod,mod), old(mod,mod,mod)

!     alpha    i   array of length mod**3  containing the values of the
!                  first 3 coefficients in the integration points
!     d1       i   array of length mod x mod contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     det      i   array o length mod**3 containing the
!                  transformation determinant
!     f        o   array of length mod**3 containing the spectral
!                  element stiffness residual
!     mod      i   number of integration points in one dimension
!     old      i   array of length mod**3 containing the old solution
!     p1       i   arrays of length mod x mod containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     w1       i   array of length mod containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work         work array of length mod**3, to store products temporarily
!***********************************************************************
!
!                   COMMON BLOCKS
!
!***********************************************************************
!
!                         LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision sum, h1, h2, h3, a11con

!     a11con     Constant value of alpha
!     h1         help variable to compute a product
!     h2         help variable to compute a product
!     h3         help variable to compute a product
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     sum        Help parameter to compute a sum
!***********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRSUB    Error messages
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1602   these values of alpha not done
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      if (jsecnd.eq.1) then

!     --- jsecnd = 1, alpha11 = constant

         a11con = alpha(1,1,1)

!        --- store first help factor in work

         do 400 i = 1, mod
            do 300 j = 1, mod
               do 350 k = 1, mod
                  sum = 0d0
                  do 360 l = 1, mod
                     sum = sum + d1(i,l)*old(l,j,k)
360               continue
                  work(i,j,k) = sum*a11con
350            continue
300         continue
400      continue

!        --- add first factor of diffusion residual to f

         do 800 i = 1, mod
            do 700 j = 1, mod
               do 600 k = 1, mod
                  sum = 0d0
                  do 500 l = 1, mod
                     h1 = (p1(l,j,k)*p1(l,j,k) + p2(l,j,k)*p2(l,j,k) + &
                           p3(l,j,k)*p3(l,j,k))*w1(l)*w1(j)*w1(k)/ &
                           det(l,j,k)
                     h2 = (p1(i,l,k)*p4(i,l,k) + p2(i,l,k)*p5(i,l,k) + &
                           p3(i,l,k)*p6(i,l,k))*w1(i)*w1(l)*w1(k)/ &
                           det(i,l,k)
                     h3 = (p1(i,j,l)*p7(i,j,l) + p2(i,j,l)*p8(i,j,l) + &
                           p3(i,j,l)*p9(i,j,l))*w1(i)*w1(j)*w1(l)/ &
                           det(i,j,l)
                     sum = sum + h1*d1(l,i)*work(l,j,k) + &
                                 h2*d1(l,j)*work(i,l,k) + &
                                 h3*d1(l,k)*work(i,j,l)
500               continue
                  f(i,j,k) = sum
600            continue
700         continue
800      continue

!        --- store second help factor in work

         do 1000 i = 1, mod
            do 900 j = 1, mod
               do 950 k = 1, mod
                  sum = 0d0
                  do 960 l = 1, mod
                     sum = sum + d1(i,l)*old(j,l,k)
960               continue
                  work(i,j,k) = sum*a11con
950            continue
900         continue
1000     continue

!        --- add second factor of diffusion residual to f

         do 1400 i = 1, mod
            do 1300 j = 1, mod
               do 1200 k = 1, mod
                  sum = 0d0
                  do 1100 l = 1, mod
                     h1 = (p1(l,j,k)*p4(l,j,k) + p2(l,j,k)*p5(l,j,k) + &
                           p3(l,j,k)*p6(l,j,k))*w1(l)*w1(j)*w1(k)/ &
                           det(l,j,k)
                     h2 = (p4(i,l,k)*p4(i,l,k) + p5(i,l,k)*p5(i,l,k) + &
                           p6(i,l,k)*p6(i,l,k))*w1(i)*w1(l)*w1(k)/ &
                           det(i,l,k)
                     h3 = (p4(i,j,l)*p7(i,j,l) + p5(i,j,l)*p8(i,j,l) + &
                           p6(i,j,l)*p9(i,j,l))*w1(i)*w1(j)*w1(l)/ &
                           det(i,j,l)
                     sum = sum + h1*d1(l,i)*work(j,l,k) + &
                                 h2*d1(l,j)*work(l,i,k) + &
                                 h3*d1(l,k)*work(j,i,l)
1100              continue
                  f(i,j,k) = f(i,j,k) + sum
1200           continue
1300        continue
1400     continue

!        --- store third help factor in work

         do 1001 i = 1, mod
            do 901 j = 1, mod
               do 951 k = 1, mod
                  sum = 0d0
                  do 961 l = 1, mod
                     sum = sum + d1(i,l)*old(j,k,l)
961               continue
                  work(i,j,k) = sum*a11con
951            continue
901         continue
1001     continue

!        --- add third factor of diffusion matrix to s

         do 1500 i = 1, mod
            do 1600 j = 1, mod
               do 1700 k = 1, mod
                  sum = 0d0
                  do 1800 l = 1, mod
                     h1 = (p1(l,j,k)*p7(l,j,k) + p2(l,j,k)*p8(l,j,k) + &
                           p3(l,j,k)*p9(l,j,k))*w1(l)*w1(j)*w1(k)/ &
                           det(l,j,k)
                     h2 = (p4(i,l,k)*p7(i,l,k) + p5(i,l,k)*p8(i,l,k) + &
                           p6(i,l,k)*p9(i,l,k))*w1(i)*w1(l)*w1(k)/ &
                           det(i,l,k)
                     h3 = (p7(i,j,l)*p7(i,j,l) + p8(i,j,l)*p8(i,j,l) + &
                           p9(i,j,l)*p9(i,j,l))*w1(i)*w1(j)*w1(l)/ &
                           det(i,j,l)
                     sum = sum + h1*d1(l,i)*work(k,l,j) + &
                                 h2*d1(l,j)*work(k,i,l) + &
                                 h3*d1(l,k)*work(l,i,j)
1800              continue
                  f(i,j,k) = f(i,j,k) + sum
1700           continue
1600        continue
1500     continue

      else

!     --- jsecnd = 2, jsecnd = 3 or jsecnd = 4, not done

         call eropen('el6085')
         call errsub ( 1602, 0, 0, 0)
         call erclos('el6085')

      end if

      end
