      subroutine el6088(old, old2, old3, alpha, w1, d1, q1, q2, q3, q4, &
                        q5, q6 , q7, q8, q9 , det, &
                        work, work2, work3 ,m1, f, ichois)
! ======================================================================
!
!        programmer    M.Speetjens, P. Anderson
!        version 1.1   date  28-08-1996  Correction, Layout
!        version 1.0   date  14-04-1996
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the diffusive part of the equation in the
!     case of non-constant diffusion
!     el6088 is called by the element subroutine ELD607
!            it must be preceded by the subroutine ELP646
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!     spectral
!    diffusion
!    residual
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer m1, ichois
      double precision alpha(m1,m1,m1,3), f(m1,m1,m1), &
                       old(m1,m1,m1),old2(m1,m1,m1), &
                       old3(m1,m1,m1), w1(m1), d1(m1,m1), &
                       det(m1,m1,m1), &
                       q1(m1,m1,m1), q2(m1,m1,m1), &
                       q3(m1,m1,m1), q4(m1,m1,m1), &
                       q5(m1,m1,m1), q6(m1,m1,m1), &
                       q7(m1,m1,m1), q8(m1,m1,m1), &
                       q9(m1,m1,m1), work(m1,m1,m1), &
                       work2(m1,m1,m1), work3(m1,m1,m1)

!     alpha    i   array of length m x m x m x 3 containing the values of
!                  the first three coefficients in the integration points
!     d1       i   array of length m x m contaning the values of the
!                  derivatives of the basisfunctions in the 1 direction
!     det      i   array of length m x m x m containing the
!                  transformation determinant
!     f        o   array of length m x m x m containing the spectral
!                  element residue vector
!     ichois  i   indicates which component of the 3-D diffusion
!                  residual is to be computed, viz.
!                  ichois = i (i = 1..3) : compute i-th component
!     m1       i   number of integration points in one direction
!     old      i   arrays of length m x m x m containing the first, second
!                  and third component, respectively, of the old solution
!     old2     i   See old
!     old3     i   See old
!     q1       i   array of length m x m x m containing the partial
!                  derivatives of the coordinates due to transformation
!     q2       i   See q1
!     q3       i   See q1
!     q4       i   See q1
!     q5       i   See q1
!     q6       i   See q1
!     q7       i   See q1
!     q8       i   See q1
!     q9       i   See q1
!     w1       i   array of length m containing the weights for
!                  numerical integration in the 1 direction corresponding
!                  to the reference element
!     work         work arrays of length m x m x m, to store products
!                  temporarily
!     work2        See work
!     work3        See work
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, p, r, s, t
      double precision sum, sum2, sum3, h(20,20,20), &
                       a11con

!     a11con     constant value of alpha
!     h
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     p          Counting variable
!     r          Counting variable
!     s          Counting variable
!     sum        help variable to compute a sum
!     sum2       help variable to compute a sum
!     sum3       help variable to compute a sum
!     t          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1602   these values of alpha not done
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('el6088')

      if (jsecnd.eq.1) then

!     --- jsecnd = 1, alpha11 = alpha22 = alpha33 = constant = a11con, &
!         off-diagonal elements of matrix alpha are zero
!                                     -1
!         --- compute alpha*nabla(old) = M  * D(old), with f = d(old)

!         defining constant viscosity 'a11con'

         a11con = alpha(1,1,1,1)

!        --- compute multiplication factor containing determinant and weight
!            factors

         do 105 i = 1 , m1
            do 116 j = 1 , m1
               do 129 k = 1 , m1

                  h(i,j,k) = w1(i)*w1(j)*w1(k)/det(i,j,k)

129            continue
116         continue
105      continue

!        --- swap between 'old' and 'old2', 'old3' in case of computing
!            the second and third component of the diffusion residual, all done
!            according to the same procedure

         if(ichois.eq.2) then

            do 1001 i = 1,m1
               do 1002 j = 1,m1
                  do 1003 k = 1,m1

                     old(i,j,k) = old2(i,j,k)

1003              continue
1002           continue
1001        continue

         end if

         if(ichois.eq.3) then

            do 1004 i = 1,m1
               do 1005 j = 1,m1
                     do 1006 k = 1,m1

                        old(i,j,k) = old3(i,j,k)

1006                 continue
1005           continue
1004        continue

         end if

!        --- store first help factor in work

         do 35 i = 1 , m1
            do 36 j = 1 , m1
               do 37 k = 1 , m1

                  sum = 0.0d0

                  do 38 p = 1 , m1

                     sum = sum + old(p,i,j)*d1(k,p)

38                continue

                  work(i,j,k) = a11con*sum

37             continue
36          continue
35       continue

!        --- compute first factor of the residual

         do 410 r = 1, m1
            do 411 s = 1, m1
               do 412 t = 1, m1

                  sum = 0.0d0

                  do 413 i = 1 , m1

                     sum = sum + h(i,s,t)*d1(i,r)* &
                          (q1(i,s,t)**2 + q2(i,s,t)**2 + &
                           q3(i,s,t)**2)*work(s,t,i) &
                         + h(r,i,t)*d1(i,s)* &
                          (q1(r,i,t)*q4(r,i,t) + q2(r,i,t)*q5(r,i,t) &
                         + q3(r,i,t)*q6(r,i,t))*work(i,t,r) &
                         + h(r,s,i)*d1(i,t)* &
                          (q1(r,s,i)*q7(r,s,i) + q2(r,s,i)*q8(r,s,i) &
                         + q3(r,s,i)*q9(r,s,i))*work(s,i,r)

413               continue

                  f(r,s,t) = -sum

412            continue
411         continue
410      continue

!        --- store second help factor in work

         do 25 i = 1 , m1
            do 26 j = 1 , m1
               do 27 k = 1 , m1

                   sum = 0.0d0

                     do 28 p = 1 , m1

                        sum = sum + old(i,p,j)*d1(k,p)

28                   continue

                     work(i,j,k) = a11con*sum

27             continue
26          continue
25       continue

!        --- compute second factor of the residual

         do 125 r = 1, m1
            do 126 s = 1, m1
               do 127 t = 1, m1

                  sum = 0.0d0

                  do 128 i = 1 , m1

                     sum = sum + h(i,s,t)*d1(i,r)* &
                          (q1(i,s,t)*q4(i,s,t) + q2(i,s,t)*q5(i,s,t) &
                        + q3(i,s,t)*q6(i,s,t))*work(i,t,s) &
                        + h(r,i,t)*d1(i,s)* &
                         (q4(r,i,t)**2 + q5(r,i,t)**2 + &
                          q6(r,i,t)**2)*work(r,t,i) &
                        + h(r,s,i)*d1(i,t)* &
                         (q7(r,s,i)*q4(r,s,i) + q8(r,s,i)*q5(r,s,i) &
                        + q9(r,s,i)*q6(r,s,i))*work(r,i,s)

128               continue

                  f(r,s,t) = f(r,s,t) - sum

127            continue
126         continue
125      continue

!        --- store third help factor in work

         do 45 i = 1 , m1
            do 46 j = 1 , m1
               do 47 k = 1 , m1

                  sum = 0.0d0

                  do 48 p = 1 , m1

                     sum = sum + old(i,j,p)*d1(k,p)

48                continue

                  work(i,j,k) = a11con*sum

47              continue
46           continue
45        continue

!         --- compute third factor of the residual

          do 135 r = 1, m1
             do 136 s = 1, m1
                do 137 t = 1, m1

                   sum = 0.0d0

                   do 138 i = 1 , m1

                      sum = sum + h(i,s,t)*d1(i,r)* &
                            (q1(i,s,t)*q7(i,s,t) + &
                             q2(i,s,t)*q8(i,s,t) + &
                             q3(i,s,t)*q9(i,s,t))*work(i,s,t) &
                              + h(r,i,t)*d1(i,s)* &
                             (q7(r,i,t)*q4(r,i,t) + &
                              q8(r,i,t)*q5(r,i,t) &
                            + q9(r,i,t)*q6(r,i,t))*work(r,i,t) &
                            + h(r,s,i)*d1(i,t)* &
                             (q7(r,s,i)**2 + q8(r,s,i)**2 + &
                              q9(r,s,i)**2)*work(r,s,i)

138                continue

                   f(r,s,t) = f(r,s,t) - sum

137             continue
136          continue
135       continue

      end if

      if (jsecnd.eq.2) then

!     --- jsecnd = 2, alpha11 = alpha22 = alpha33 = variable, &
!         off-diagonal elements of matrix alpha are zero

!                                      T
!         --- compute grad.(alpha (grad(u)) + alpha grad(u))
!                    -1
!                 = M  * D(u), where
!                       T
!         u = (old,old2,old3) , &
!         f1 = D(u)*e , f2 = D(u)*e , f3 = D(u)*e
!                    x             y             z
!         the first, second and third component of the residual.

!         compute multiplication factor containing determinant, weight
!         factors and viscosity

         do 201 i = 1 , m1
            do 211 j = 1 , m1
               do 221 k = 1 , m1

                  h(i,j,k) = alpha(i,j,k,1)*w1(i)*w1(j)*w1(k)/det(i,j,k)

221            continue
211         continue
201      continue

!        --- compute FIRST component of 3-D diffusion residual (f1)

         if(ichois.eq.1) then

!        --- store first help factor in work, work2, work3

            do 10 i = 1 , m1
               do 11 j = 1 , m1
                  do 12 k = 1 , m1

                     sum = 0.0d0
                     sum2 = 0.0d0
                     sum3 = 0.0d0

                     do 13 p = 1 , m1

                        sum = sum + old(p,i,j)*d1(k,p)
                        sum2 = sum2 + old2(p,i,j)*d1(k,p)
                        sum3 = sum3 + old3(p,i,j)*d1(k,p)

13                   continue

                     work(i,j,k) = sum
                     work2(i,j,k) = sum2
                     work3(i,j,k) = sum3

12                continue
11             continue
10          continue

!           --- compute first factor of the residual

            do 110 r = 1, m1
               do 111 s = 1, m1
                  do 112 t = 1, m1

                     sum = 0.0d0

                     do 113 i = 1 , m1

                        sum = sum + h(i,s,t)*d1(i,r)*( &
                             (2.0d0*q1(i,s,t)**2 + &
                             q2(i,s,t)**2 + q3(i,s,t)**2) &
                             *work(s,t,i) &
                              + q1(i,s,t)*(q2(i,s,t)*work2(s,t,i) &
                              + q3(i,s,t)*work3(s,t,i)) ) &
                              + h(r,i,t)*d1(i,s)*( &
                               (2.0d0*q1(r,i,t)*q4(r,i,t) + &
                               q2(r,i,t)*q5(r,i,t) &
                             + q3(r,i,t)*q6(r,i,t))*work(i,t,r) &
                             + q1(r,i,t)*(q5(r,i,t)*work2(i,t,r) &
                             + q6(r,i,t)*work3(i,t,r)) ) &
                             + h(r,s,i)*d1(i,t)*( &
                              (2.0d0*q1(r,s,i)*q7(r,s,i) + &
                               q2(r,s,i)*q8(r,s,i) &
                             + q3(r,s,i)*q9(r,s,i))*work(s,i,r) &
                             + q1(r,s,i)*(q8(r,s,i)*work2(s,i,r) &
                             + q9(r,s,i)*work3(s,i,r))   )

113                  continue

                     f(r,s,t) = -sum

112               continue
111            continue
110         continue

!           --- store second help factor in work, work2, work3

            do 20 i = 1 , m1
               do 21 j = 1 , m1
                  do 22 k = 1 , m1

                     sum = 0.0d0
                     sum2 = 0.0d0
                     sum3 = 0.0d0

                     do 23 p = 1 , m1

                        sum = sum + old(i,p,j)*d1(k,p)
                        sum2 = sum2 + old2(i,p,j)*d1(k,p)
                        sum3 = sum3 + old3(i,p,j)*d1(k,p)

23                   continue

                     work(i,j,k) = sum
                     work2(i,j,k) = sum2
                     work3(i,j,k) = sum3

22                continue
21             continue
20          continue

!           --- compute second factor of the residual

            do 120 r = 1, m1
               do 121 s = 1, m1
                  do 122 t = 1, m1

                     sum = 0.0d0

                     do 123 i = 1 , m1

                        sum = sum + h(i,s,t)*d1(i,r)*( &
                             (2.0d0*q1(i,s,t)*q4(i,s,t) + &
                              q2(i,s,t)*q5(i,s,t) &
                         +    q3(i,s,t)*q6(i,s,t))*work(i,t,s) &
                           + q4(i,s,t)*(q2(i,s,t)*work2(i,t,s) &
                           + q3(i,s,t)*work3(i,t,s)) ) &
                           + h(r,i,t)*d1(i,s)*( &
                           (2.0d0*q4(r,i,t)**2 + q5(r,i,t)**2 &
                          + q6(r,i,t)**2) &
                          *work(r,t,i)  + q4(r,i,t)*(q5(r,i,t)* &
                           work2(r,t,i) + q6(r,i,t)*work3(r,t,i))  ) &
                         + h(r,s,i)*d1(i,t)*( &
                          (2.0d0*q7(r,s,i)*q4(r,s,i) + &
                           q8(r,s,i)*q5(r,s,i) &
                         + q9(r,s,i)*q6(r,s,i))*work(r,i,s) &
                         + q4(r,s,i)*(q8(r,s,i)*work2(r,i,s) &
                         + q9(r,s,i)*work3(r,i,s)) )

123                  continue

                     f(r,s,t) = f(r,s,t) - sum

122               continue
121            continue
120         continue

!           --- store third help factor in work, work2, work3

            do 30 i = 1 , m1
               do 31 j = 1 , m1
                  do 32 k = 1 , m1

                     sum = 0.0d0
                     sum2 = 0.0d0
                     sum3 = 0.0d0

                     do 33 p = 1 , m1

                        sum = sum + old(i,j,p)*d1(k,p)
                        sum2 = sum2 + old2(i,j,p)*d1(k,p)
                        sum3 = sum3 + old3(i,j,p)*d1(k,p)

33                   continue

                     work(i,j,k) = sum
                     work2(i,j,k) = sum2
                     work3(i,j,k) = sum3

32                continue
31             continue
30          continue

!           --- compute third factor of the residual

            do 130 r = 1, m1
               do 131 s = 1, m1
                  do 132 t = 1, m1

                     sum = 0.0d0

                        do 133 i = 1 , m1

                           sum = sum + h(i,s,t)*d1(i,r)*( &
                                 (2.0d0*q1(i,s,t)*q7(i,s,t) + &
                                  q2(i,s,t)*q8(i,s,t) &
                                + q3(i,s,t)*q9(i,s,t))*work(i,s,t) &
                                + q7(i,s,t)*(q2(i,s,t)*work2(i,s,t) &
                                + q3(i,s,t)*work3(i,s,t)) ) &
                                + h(r,i,t)*d1(i,s)*( &
                                  (2.0d0*q7(r,i,t)*q4(r,i,t) + &
                                   q8(r,i,t)*q5(r,i,t) &
                                + q9(r,i,t)*q6(r,i,t))*work(r,i,t) &
                                + q7(r,i,t)*(q5(r,i,t)*work2(r,i,t) &
                                + q6(r,i,t)*work3(r,i,t)) ) &
                                + h(r,s,i)*d1(i,t)*( &
                                  (2.0d0*q7(r,s,i)**2 + q8(r,s,i)**2 + &
                                   q9(r,s,i)**2) *work(r,s,i) &
                                + q7(r,s,i)*(q8(r,s,i)*work2(r,s,i) &
                                + q9(r,s,i)*work3(r,s,i))  )

133                  continue

                  f(r,s,t) = f(r,s,t) - sum

132               continue
131            continue
130         continue

         end if

!        --- compute SECOND component of 3-D diffusion residual (f2)

         if(ichois.eq.2) then

!        --- store first help factor in work, work2, work3

            do 50 i = 1 , m1
               do 51 j = 1 , m1
                  do 52 k = 1 , m1

                     sum = 0.0d0
                     sum2 = 0.0d0
                     sum3 = 0.0d0

                     do 53 p = 1 , m1

                        sum = sum + old(p,i,j)*d1(k,p)
                        sum2 = sum2 + old2(p,i,j)*d1(k,p)
                        sum3 = sum3 + old3(p,i,j)*d1(k,p)

53                   continue

                     work(i,j,k) = sum
                     work2(i,j,k) = sum2
                     work3(i,j,k) = sum3

52                continue
51             continue
50          continue

!           --- compute first factor of the residual

            do 510 r = 1, m1
               do 511 s = 1, m1
                  do 512 t = 1, m1

                     sum = 0.0d0

                     do 513 i = 1 , m1

                        sum = sum + h(i,s,t)*d1(i,r)*( &
                             (q1(i,s,t)**2 + 2.0d0*q2(i,s,t)**2 + &
                              q3(i,s,t)**2) *work2(s,t,i) &
                            + q2(i,s,t)*(q1(i,s,t)*work(s,t,i) &
                            + q3(i,s,t)*work3(s,t,i)) ) &
                            + h(r,i,t)*d1(i,s)*( &
                              (q1(r,i,t)*q4(r,i,t) + &
                               2.0d0*q2(r,i,t)*q5(r,i,t) &
                             + q3(r,i,t)*q6(r,i,t))*work2(i,t,r) &
                             + q2(r,i,t)*(q4(r,i,t)*work(i,t,r) &
                             + q6(r,i,t)*work3(i,t,r)) ) &
                            + h(r,s,i)*d1(i,t)*( &
                            (q1(r,s,i)*q7(r,s,i) + &
                             2.0d0*q2(r,s,i)*q8(r,s,i) &
                            + q3(r,s,i)*q9(r,s,i))*work2(s,i,r) &
                            + q2(r,s,i)*(q7(r,s,i)*work(s,i,r) &
                            + q9(r,s,i)*work3(s,i,r))   )

513                  continue

                     f(r,s,t) = -sum

512               continue
511            continue
510         continue

!           --- store second help factor in work, work2, work3

            do 60 i = 1 , m1
               do 61 j = 1 , m1
                  do 62 k = 1 , m1

                     sum = 0.0d0
                     sum2 = 0.0d0
                     sum3 = 0.0d0

                     do 63 p = 1 , m1

                        sum = sum + old(i,p,j)*d1(k,p)
                        sum2 = sum2 + old2(i,p,j)*d1(k,p)
                        sum3 = sum3 + old3(i,p,j)*d1(k,p)

63                   continue

                     work(i,j,k) = sum
                     work2(i,j,k) = sum2
                     work3(i,j,k) = sum3

62                continue
61             continue
60          continue

!           --- compute second factor of the residual

            do 620 r = 1, m1
               do 621 s = 1, m1
                  do 622 t = 1, m1

                     sum = 0.0d0

                     do 623 i = 1 , m1

                        sum = sum + h(i,s,t)*d1(i,r)*( &
                             (q1(i,s,t)*q4(i,s,t) + &
                              2.0d0*q2(i,s,t)*q5(i,s,t) &
                             + q3(i,s,t)*q6(i,s,t))*work2(i,t,s) &
                             + q5(i,s,t)*(q1(i,s,t)*work(i,t,s) &
                             + q3(i,s,t)*work3(i,t,s)) ) &
                             + h(r,i,t)*d1(i,s)*( &
                               (q4(r,i,t)**2 + 2.0d0*q5(r,i,t)**2 + &
                              q6(r,i,t)**2) *work2(r,t,i) &
                             + q5(r,i,t)*(q4(r,i,t)*work(r,t,i) &
                             + q6(r,i,t)*work3(r,t,i)) ) &
                             + h(r,s,i)*d1(i,t)*( &
                              (q7(r,s,i)*q4(r,s,i) + &
                               2.0d0*q8(r,s,i)*q5(r,s,i) &
                             + q9(r,s,i)*q6(r,s,i))*work2(r,i,s) &
                             + q5(r,s,i)*(q7(r,s,i)*work(r,i,s) &
                             + q9(r,s,i)*work3(r,i,s)) )

623                  continue

                     f(r,s,t) = f(r,s,t) - sum

622               continue
621            continue
620         continue

!           --- store third help factor in work, work2, work3

            do 70 i = 1 , m1
               do 71 j = 1 , m1
                  do 72 k = 1 , m1

                     sum = 0.0d0
                     sum2 = 0.0d0
                     sum3 = 0.0d0

                     do 73 p = 1 , m1

                        sum = sum + old(i,j,p)*d1(k,p)
                        sum2 = sum2 + old2(i,j,p)*d1(k,p)
                        sum3 = sum3 + old3(i,j,p)*d1(k,p)

73                   continue

                     work(i,j,k) = sum
                     work2(i,j,k) = sum2
                     work3(i,j,k) = sum3

72                continue
71             continue
70          continue

!           --- compute third factor of the residual

            do 730 r = 1, m1
               do 731 s = 1, m1
                  do 732 t = 1, m1

                     sum = 0.0d0

                        do 733 i = 1 , m1

                           sum = sum + h(i,s,t)*d1(i,r)*( &
                                (q1(i,s,t)*q7(i,s,t) + &
                                 2.0d0*q2(i,s,t)*q8(i,s,t) &
                               + q3(i,s,t)*q9(i,s,t))*work2(i,s,t) &
                               + q8(i,s,t)*(q1(i,s,t)*work(i,s,t) &
                               + q3(i,s,t)*work3(i,s,t))  ) &
                               + h(r,i,t)*d1(i,s)*( &
                                 (q7(r,i,t)*q4(r,i,t) + &
                                  2.0d0*q8(r,i,t)*q5(r,i,t) &
                               + q9(r,i,t)*q6(r,i,t))*work2(r,i,t) &
                               + q8(r,i,t)*(q4(r,i,t)*work(r,i,t) &
                               + q6(r,i,t)*work3(r,i,t)) ) &
                               + h(r,s,i)*d1(i,t)*( &
                                 (q7(r,s,i)**2 + 2.0d0*q8(r,s,i)**2 + &
                                  q9(r,s,i)**2) &
                                 *work2(r,s,i) &
                               + q8(r,s,i)*(q7(r,s,i)*work(r,s,i) &
                               + q9(r,s,i)*work3(r,s,i))   )

733                     continue

                        f(r,s,t) = f(r,s,t) - sum

732               continue
731            continue
730         continue

         end if

      end if

!     --- compute THIRD component of 3-D diffusion residual (f3)

      if(ichois.eq.3) then

!     --- store first help factor in work, work2, work3

         do 80 i = 1 , m1
            do 81 j = 1 , m1
               do 82 k = 1 , m1

                  sum = 0.0d0
                  sum2 = 0.0d0
                  sum3 = 0.0d0

                  do 83 p = 1 , m1

                     sum = sum + old(p,i,j)*d1(k,p)
                     sum2 = sum2 + old2(p,i,j)*d1(k,p)
                     sum3 = sum3 + old3(p,i,j)*d1(k,p)

83                continue

                  work(i,j,k) = sum
                  work2(i,j,k) = sum2
                  work3(i,j,k) = sum3

82             continue
81          continue
80       continue

!        --- compute first factor of the residual

         do 810 r = 1, m1
            do 811 s = 1, m1
               do 812 t = 1, m1

                  sum = 0.0d0

                  do 813 i = 1 , m1

                     sum = sum + h(i,s,t)*d1(i,r)*( &
                          (q1(i,s,t)**2 + q2(i,s,t)**2 + &
                           2.0d0*q3(i,s,t)**2) *work3(s,t,i) &
                        + q3(i,s,t)*(q1(i,s,t)*work(s,t,i) &
                        + q2(i,s,t)*work2(s,t,i)) ) &
                        + h(r,i,t)*d1(i,s)*( &
                         (q1(r,i,t)*q4(r,i,t) + &
                          q2(r,i,t)*q5(r,i,t) &
                        + 2.0d0*q3(r,i,t)*q6(r,i,t))*work3(i,t,r) &
                        + q3(r,i,t)*(q4(r,i,t)*work(i,t,r) &
                        + q5(r,i,t)*work2(i,t,r)) ) &
                        + h(r,s,i)*d1(i,t)*( &
                         (q1(r,s,i)*q7(r,s,i) + q2(r,s,i)*q8(r,s,i) &
                        + 2.0d0*q3(r,s,i)*q9(r,s,i))*work3(s,i,r) &
                        + q3(r,s,i)*(q7(r,s,i)*work(s,i,r) &
                        + q8(r,s,i)*work2(s,i,r))  )

813               continue

                  f(r,s,t) = -sum

812            continue
811         continue
810      continue

!        --- store second help factor in work, work2, work3

         do 90 i = 1 , m1
            do 91 j = 1 , m1
               do 92 k = 1 , m1

                  sum = 0.0d0
                  sum2 = 0.0d0
                  sum3 = 0.0d0

                  do 93 p = 1 , m1

                     sum = sum + old(i,p,j)*d1(k,p)
                     sum2 = sum2 + old2(i,p,j)*d1(k,p)
                     sum3 = sum3 + old3(i,p,j)*d1(k,p)

93                continue

                  work(i,j,k) = sum
                  work2(i,j,k) = sum2
                  work3(i,j,k) = sum3

92             continue
91          continue
90       continue

!        --- compute second factor of the residual

         do 920 r = 1, m1
            do 921 s = 1, m1
               do 922 t = 1, m1

                  sum = 0.0d0

                  do 923 i = 1 , m1

                     sum = sum + h(i,s,t)*d1(i,r)*( &
                          (q1(i,s,t)*q4(i,s,t) + &
                           q2(i,s,t)*q5(i,s,t) &
                          + 2.0d0*q3(i,s,t)*q6(i,s,t))*work3(i,t,s) &
                          + q6(i,s,t)*(q1(i,s,t)*work(i,t,s) &
                          + q2(i,s,t)*work2(i,t,s)) ) &
                          + h(r,i,t)*d1(i,s)*( &
                           (q4(r,i,t)**2 + q5(r,i,t)**2 + &
                            2.0d0*q6(r,i,t)**2)*work3(r,t,i) &
                           + q6(r,i,t)*(q4(r,i,t)*work(r,t,i) &
                           + q5(r,i,t)*work2(r,t,i)) ) &
                          + h(r,s,i)*d1(i,t)*( &
                          (q7(r,s,i)*q4(r,s,i) + q8(r,s,i)*q5(r,s,i) &
                         + 2.0d0*q9(r,s,i)*q6(r,s,i))*work3(r,i,s) &
                         + q6(r,s,i)*(q7(r,s,i)*work(r,i,s) &
                         + q8(r,s,i)*work2(r,i,s)) )

923               continue

                  f(r,s,t) = f(r,s,t) - sum

922            continue
921         continue
920      continue

!        --- store third help factor in work, work2, work3

         do 705 i = 1 , m1
            do 716 j = 1 , m1
               do 727 k = 1 , m1

                  sum = 0.0d0
                  sum2 = 0.0d0
                  sum3 = 0.0d0

                  do 738 p = 1 , m1

                     sum = sum + old(i,j,p)*d1(k,p)
                     sum2 = sum2 + old2(i,j,p)*d1(k,p)
                     sum3 = sum3 + old3(i,j,p)*d1(k,p)

738               continue

                  work(i,j,k) = sum
                  work2(i,j,k) = sum2
                  work3(i,j,k) = sum3

727            continue
716         continue
705      continue

!        --- compute third factor of the residual

         do 735 r = 1, m1
            do 736 s = 1, m1
               do 737 t = 1, m1

                  sum = 0.0d0

                  do 739 i = 1 , m1

                     sum = sum + h(i,s,t)*d1(i,r)*( &
                          (q1(i,s,t)*q7(i,s,t) + q2(i,s,t)*q8(i,s,t) &
                         + 2.0d0*q3(i,s,t)*q9(i,s,t))*work3(i,s,t) &
                         + q9(i,s,t)*(q1(i,s,t)*work(i,s,t) &
                         + q2(i,s,t)*work2(i,s,t)) ) &
                         + h(r,i,t)*d1(i,s)*( &
                           (q7(r,i,t)*q4(r,i,t) + q8(r,i,t)*q5(r,i,t) &
                         + 2.0d0*q9(r,i,t)*q6(r,i,t))*work3(r,i,t) &
                         + q9(r,i,t)*(q4(r,i,t)*work(r,i,t) &
                         + q5(r,i,t)*work2(r,i,t))   ) &
                         + h(r,s,i)*d1(i,t)*( &
                          (q7(r,s,i)**2 + q8(r,s,i)**2 + &
                           2.0d0*q9(r,s,i)**2) *work3(r,s,i) &
                         + q9(r,s,i)*(q7(r,s,i)*work(r,s,i) &
                         + q8(r,s,i)*work2(r,s,i))   )

739               continue

                  f(r,s,t) = f(r,s,t) - sum

737            continue
736         continue
735      continue

      end if

      if(jsecnd.ge.3) then

!     --- jsecnd = 3 or jsecnd = 4, not done

         call eropen('el6088')
         call errsub ( 1602, 0, 0, 0)
         call erclos('el6088')
      end if

      if (ierror.ne.0) call instop
      call erclos('el6088')

      end
