      subroutine el6092 ( f, old, wxp, dxp, work, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, mone, numr )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   03-10-1994
!
!   copyrigth (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the residual of the first derivative with respect to
!     the numr - th coordinate, numr =1, 2, 3; 3-D case;
!     it is used to compute the pressure gradient with integration
!     by parts.
!
!     EL6092 is called by the element subroutine ELd607
!            it must be preceded by the subroutines ELP640, ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer mone, numr
      double precision f(mone,mone,mone), wxp(mone), &
                       old(mone,mone,mone), work(mone,mone,mone), &
                       p1(mone,mone,mone), p2(mone,mone,mone), &
                       p3(mone,mone,mone), p4(mone,mone,mone), &
                       p5(mone,mone,mone), p6(mone,mone,mone), &
                       p7(mone,mone,mone), p8(mone,mone,mone), &
                       p9(mone,mone,mone), dxp(mone,mone)

!     dxp      i   array of length mone x mone contaning the values of the
!                  derivatives of the basisfunctions for the pressure in the
!     f       i/o  array of length mone x mone x mone
!                  containing the rigthhandside or the residual
!     mone     i   number of points in one dimension
!     numr     i   indicates which component of the gradient to be computed
!                  numr = i indicates the i-th component
!     old      i   array of length m containing the old pressure
!     p1       i   arrays of length m containing the elements of the invert
!                  jacobian of the local transform onto the standard
!                  quadrilateral
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     work    i/o  work array for temporary storage
!     wxp      i   array of length mone containing the pressure weigths for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!                     LOCAL PARAMETERS
!
      integer i, j, k, l

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!***********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1619   Component of gradient can not be computed
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- first initialize f and load the work array
!         change the sign of the final vector because integration by parts
!         is supposed
!
      do 10 i = 1, mone
         do 11 j = 1, mone
            do 12 k = 1, mone
               work(i,j,k) = - wxp(i)*wxp(j)*wxp(k)*old(i,j,k)
               f(i,j,k) = 0d0
12          continue
11       continue
10    continue

      if ( numr.lt.1 .or. numr.gt.3 ) then
         call eropen ( 'el6092' )
         call errint ( numr, 1 )
         call errsub ( 1619, 1, 0, 0 )
         call erclos ( 'el6092' )
         go to 5000
      end if
      go to ( 1, 2, 3) numr

!     --- first component of the gradient

  1   do 400 i = 1, mone
         do 500 j = 1, mone
            do 600 k = 1, mone
               do 700 l = 1, mone
                  f(i,j,k) = f(i,j,k) + dxp(l,i)*p1(l,j,k)*work(l,j,k) &
                                      + dxp(l,j)*p4(i,l,k)*work(i,l,k) &
                                      + dxp(l,k)*p7(i,j,l)*work(i,j,l)
700            continue
600         continue
500      continue
400   continue

      go to 5000

!     --- second component of the gradient

  2   do 1000 i = 1, mone
         do 1100 j = 1, mone
            do 1200 k = 1, mone
               do 1300 l = 1, mone
                  f(i,j,k) = f(i,j,k) + dxp(l,i)*p2(l,j,k)*work(l,j,k) &
                                      + dxp(l,j)*p5(i,l,k)*work(i,l,k) &
                                      + dxp(l,k)*p8(i,j,l)*work(i,j,l)
1300           continue
1200        continue
1100     continue
1000  continue

      go to 5000

!     --- third component of the gradient

  3   do 2000 i = 1, mone
         do 2100 j = 1, mone
            do 2200 k = 1, mone
               do 2300 l = 1, mone
                  f(i,j,k) = f(i,j,k) + dxp(l,i)*p3(l,j,k)*work(l,j,k) &
                                      + dxp(l,j)*p6(i,l,k)*work(i,l,k) &
                                      + dxp(l,k)*p9(i,j,l)*work(i,j,l)
2300           continue
2200        continue
2100     continue
2000  continue

5000  continue

      end
