      subroutine el6094 ( u1, u2, u3, work, w1, d1, p1, p2, p3, p4, &
                          p5, p6, p7, p8, p9, det, mod, f, numr )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   02-11-1994
!
!   copyrigth (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the gradient of divergence of velocity residual
!     in the case of spectral elements; integration by parts is assumed.
!     3-D case
!
!     EL6094 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640 and ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer mod, numr
      double precision u1(mod,mod,mod),u2(mod,mod,mod),u3(mod,mod,mod), &
                       work(mod,mod,mod), w1(mod), d1(mod,mod), &
                       p1(mod,mod,mod), p2(mod,mod,mod), &
                       p3(mod,mod,mod), p4(mod,mod,mod), &
                       p5(mod,mod,mod), p6(mod,mod,mod), &
                       p7(mod,mod,mod), p8(mod,mod,mod), &
                       p9(mod,mod,mod), f(mod,mod,mod), &
                       det(mod,mod,mod)

!     d1       i   array of length mod x mod contaning the values of the
!                  derivatives of the basisfunctions in the x-direction
!     det     i/o  array of length mod**3 containing the Jacobian
!     f       i/o  array of length mod**3 containing the rigthhand-side vector
!     mod      i   number of integration points in the one dimension
!     numr     i   number of component to be computed
!     p1       i   arrays of length mod x mod containing the partial
!                  derivatives of the coordinates due to transformation
!     p2       i   See p1
!     p3       i   See p1
!     p4       i   See p1
!     p5       i   See p1
!     p6       i   See p1
!     p7       i   See p1
!     p8       i   See p1
!     p9       i   See p1
!     u1       i   array of length mod x mod x mod containing the values
!                  of the first velocity component in the integration
!     node
!     u2       i   array of length mod x mod x mod containing the values
!                  of the second velocity component in the integration
!     node
!     u3       i   array of length mod x mod x mod containing the values
!                  of the third velocity component in the integration
!     node
!     w1       i   array of length mod containing the weigths for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!     work    i/o  work array for storage of the total weigth
! **********************************************************************
!
!                     COMMON BLOCKS
!
! **********************************************************************
!
!                     LOCAL PARAMETERS
!
      integer i, j, k, l, m
      double precision sum, sum1, sum2, sum3, sum4, &
                       sum5, sum6, sum7, sum8, sum9

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     m          General loop variable
!     sum        Help parameter to compute a sum
!     sum1       Help parameter to compute a sum
!     sum2       Help parameter to compute a sum
!     sum3       Help parameter to compute a sum
!     sum4       Help parameter to compute a sum
!     sum5       Help parameter to compute a sum
!     sum6       Help parameter to compute a sum
!     sum7       Help parameter to compute a sum
!     sum8       Help parameter to compute a sum
!     sum9       Help parameter to compute a sum
! **********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1619   Component of gradient can not be computed
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- first load the total weigth

      do 10 i = 1, mod
         do 11 j = 1, mod
            do 12 k = 1, mod
               work(i,j,k) = w1(i)*w1(j)*w1(k)/det(i,j,k)
12          continue
11       continue
10    continue

      if ( numr.lt.1 .or. numr.gt.3 ) then
         call eropen ( 'el6094' )
         call errint ( numr, 1 )
         call errsub ( 1619, 1, 0, 0 )
         call erclos ( 'el6094' )
         go to 5000
      end if
      go to ( 1, 2, 3) numr

!     --- compute the first component

  1   do 1700 i = 1, mod
         do 1600 j = 1, mod
            do 1500 k = 1, mod

!           --- first sum

               sum = 0d0
               do 2000 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 1550 m = 1, mod
                     sum1 = sum1 + d1(l,m)*u1(m,j,k)
                     sum2 = sum2 + d1(j,m)*u1(l,m,k)
                     sum3 = sum3 + d1(k,m)*u1(l,j,m)
                     sum4 = sum4 + d1(l,m)*u2(m,j,k)
                     sum5 = sum5 + d1(j,m)*u2(l,m,k)
                     sum6 = sum6 + d1(k,m)*u2(l,j,m)
                     sum7 = sum7 + d1(l,m)*u3(m,j,k)
                     sum8 = sum8 + d1(j,m)*u3(l,m,k)
                     sum9 = sum9 + d1(k,m)*u3(l,j,m)
1550              continue
                  sum = (sum1*p1(l,j,k) + &
                         sum2*p4(l,j,k) + &
                         sum3*p7(l,j,k) + &
                         sum4*p2(l,j,k) + &
                         sum5*p5(l,j,k) + &
                         sum6*p8(l,j,k) + &
                         sum7*p3(l,j,k) + &
                         sum8*p6(l,j,k) + &
                         sum9*p9(l,j,k)) * &
                         p1(l,j,k)*work(l,j,k)*d1(l,i)
2000           continue
               f(i,j,k) = sum

!              --- second sum

               sum = 0d0
               do 2100 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 1650 m = 1, mod
                     sum1 = sum1 + d1(i,m)*u1(m,l,k)
                     sum2 = sum2 + d1(l,m)*u1(i,m,k)
                     sum3 = sum3 + d1(k,m)*u1(i,l,m)
                     sum4 = sum4 + d1(i,m)*u2(m,l,k)
                     sum5 = sum5 + d1(l,m)*u2(i,m,k)
                     sum6 = sum6 + d1(k,m)*u2(i,l,m)
                     sum7 = sum7 + d1(i,m)*u3(m,l,k)
                     sum8 = sum8 + d1(l,m)*u3(i,m,k)
                     sum9 = sum9 + d1(k,m)*u3(i,l,m)
1650              continue
                  sum = (sum1*p1(i,l,k) + &
                         sum2*p4(i,l,k) + &
                         sum3*p7(i,l,k) + &
                         sum4*p2(i,l,k) + &
                         sum5*p5(i,l,k) + &
                         sum6*p8(i,l,k) + &
                         sum7*p3(i,l,k) + &
                         sum8*p6(i,l,k) + &
                         sum9*p9(i,l,k)) * &
                         p4(i,l,k)*work(i,l,k)*d1(l,j)
2100           continue
               f(i,j,k) = f(i,j,k) + sum

!              --- third sum

               sum = 0d0
               do 2200 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 1750 m = 1, mod
                     sum1 = sum1 + d1(i,m)*u1(m,j,l)
                     sum2 = sum2 + d1(j,m)*u1(i,m,l)
                     sum3 = sum3 + d1(l,m)*u1(i,j,m)
                     sum4 = sum4 + d1(i,m)*u2(m,j,l)
                     sum5 = sum5 + d1(j,m)*u2(i,m,l)
                     sum6 = sum6 + d1(l,m)*u2(i,j,m)
                     sum7 = sum7 + d1(i,m)*u3(m,j,l)
                     sum8 = sum8 + d1(j,m)*u3(i,m,l)
                     sum9 = sum9 + d1(l,m)*u3(i,j,m)
1750              continue
                  sum = (sum1*p1(i,j,l) + &
                         sum2*p4(i,j,l) + &
                         sum3*p7(i,j,l) + &
                         sum4*p2(i,j,l) + &
                         sum5*p5(i,j,l) + &
                         sum6*p8(i,j,l) + &
                         sum7*p3(i,j,l) + &
                         sum8*p6(i,j,l) + &
                         sum9*p9(i,j,l)) * &
                         p7(i,j,l)*work(i,j,l)*d1(l,k)
2200           continue
               f(i,j,k) = f(i,j,k) + sum
1500        continue
1600     continue
1700  continue

      go to 5000

!     --- compute the second component

  2   do 2700 i = 1, mod
         do 2600 j = 1, mod
            do 2500 k = 1, mod

!           --- first sum

               sum = 0d0
               do 3000 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 1850 m = 1, mod
                     sum1 = sum1 + d1(l,m)*u1(m,j,k)
                     sum2 = sum2 + d1(j,m)*u1(l,m,k)
                     sum3 = sum3 + d1(k,m)*u1(l,j,m)
                     sum4 = sum4 + d1(l,m)*u2(m,j,k)
                     sum5 = sum5 + d1(j,m)*u2(l,m,k)
                     sum6 = sum6 + d1(k,m)*u2(l,j,m)
                     sum7 = sum7 + d1(l,m)*u3(m,j,k)
                     sum8 = sum8 + d1(j,m)*u3(l,m,k)
                     sum9 = sum9 + d1(k,m)*u3(l,j,m)
1850              continue
                  sum = (sum1*p1(l,j,k) + &
                         sum2*p4(l,j,k) + &
                         sum3*p7(l,j,k) + &
                         sum4*p2(l,j,k) + &
                         sum5*p5(l,j,k) + &
                         sum6*p8(l,j,k) + &
                         sum7*p3(l,j,k) + &
                         sum8*p6(l,j,k) + &
                         sum9*p9(l,j,k)) * &
                         p2(l,j,k)*work(l,j,k)*d1(l,i)
3000           continue
               f(i,j,k) = sum

!              --- second sum

               sum = 0d0
               do 2300 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 1950 m = 1, mod
                     sum1 = sum1 + d1(i,m)*u1(m,l,k)
                     sum2 = sum2 + d1(l,m)*u1(i,m,k)
                     sum3 = sum3 + d1(k,m)*u1(i,l,m)
                     sum4 = sum4 + d1(i,m)*u2(m,l,k)
                     sum5 = sum5 + d1(l,m)*u2(i,m,k)
                     sum6 = sum6 + d1(k,m)*u2(i,l,m)
                     sum7 = sum7 + d1(i,m)*u3(m,l,k)
                     sum8 = sum8 + d1(l,m)*u3(i,m,k)
                     sum9 = sum9 + d1(k,m)*u3(i,l,m)
1950              continue
                  sum = (sum1*p1(i,l,k) + &
                         sum2*p4(i,l,k) + &
                         sum3*p7(i,l,k) + &
                         sum4*p2(i,l,k) + &
                         sum5*p5(i,l,k) + &
                         sum6*p8(i,l,k) + &
                         sum7*p3(i,l,k) + &
                         sum8*p6(i,l,k) + &
                         sum9*p9(i,l,k)) * &
                         p5(i,l,k)*work(i,l,k)*d1(l,j)
2300           continue
               f(i,j,k) = f(i,j,k) + sum

!              --- third sum

               sum = 0d0
               do 2400 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 2050 m = 1, mod
                     sum1 = sum1 + d1(i,m)*u1(m,j,l)
                     sum2 = sum2 + d1(j,m)*u1(i,m,l)
                     sum3 = sum3 + d1(l,m)*u1(i,j,m)
                     sum4 = sum4 + d1(i,m)*u2(m,j,l)
                     sum5 = sum5 + d1(j,m)*u2(i,m,l)
                     sum6 = sum6 + d1(l,m)*u2(i,j,m)
                     sum7 = sum7 + d1(i,m)*u3(m,j,l)
                     sum8 = sum8 + d1(j,m)*u3(i,m,l)
                     sum9 = sum9 + d1(l,m)*u3(i,j,m)
2050              continue
                  sum = (sum1*p1(i,j,l) + &
                         sum2*p4(i,j,l) + &
                         sum3*p7(i,j,l) + &
                         sum4*p2(i,j,l) + &
                         sum5*p5(i,j,l) + &
                         sum6*p8(i,j,l) + &
                         sum7*p3(i,j,l) + &
                         sum8*p6(i,j,l) + &
                         sum9*p9(i,j,l)) * &
                         p8(i,j,l)*work(i,j,l)*d1(l,k)
2400           continue
               f(i,j,k) = f(i,j,k) + sum
2500        continue
2600     continue
2700  continue

       go to 5000

!      --- compute the third component

  3   do 3700 i = 1, mod
         do 3600 j = 1, mod
            do 3500 k = 1, mod

!           --- first sum

               sum = 0d0
               do 4000 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 2150 m = 1, mod
                     sum1 = sum1 + d1(l,m)*u1(m,j,k)
                     sum2 = sum2 + d1(j,m)*u1(l,m,k)
                     sum3 = sum3 + d1(k,m)*u1(l,j,m)
                     sum4 = sum4 + d1(l,m)*u2(m,j,k)
                     sum5 = sum5 + d1(j,m)*u2(l,m,k)
                     sum6 = sum6 + d1(k,m)*u2(l,j,m)
                     sum7 = sum7 + d1(l,m)*u3(m,j,k)
                     sum8 = sum8 + d1(j,m)*u3(l,m,k)
                     sum9 = sum9 + d1(k,m)*u3(l,j,m)
2150              continue
                  sum = (sum1*p1(l,j,k) + &
                         sum2*p4(l,j,k) + &
                         sum3*p7(l,j,k) + &
                         sum4*p2(l,j,k) + &
                         sum5*p5(l,j,k) + &
                         sum6*p8(l,j,k) + &
                         sum7*p3(l,j,k) + &
                         sum8*p6(l,j,k) + &
                         sum9*p9(l,j,k)) * &
                         p3(l,j,k)*work(l,j,k)*d1(l,i)
4000           continue
               f(i,j,k) = sum

!              --- second sum

               sum = 0d0
               do 4100 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 2250 m = 1, mod
                     sum1 = sum1 + d1(i,m)*u1(m,l,k)
                     sum2 = sum2 + d1(l,m)*u1(i,m,k)
                     sum3 = sum3 + d1(k,m)*u1(i,l,m)
                     sum4 = sum4 + d1(i,m)*u2(m,l,k)
                     sum5 = sum5 + d1(l,m)*u2(i,m,k)
                     sum6 = sum6 + d1(k,m)*u2(i,l,m)
                     sum7 = sum7 + d1(i,m)*u3(m,l,k)
                     sum8 = sum8 + d1(l,m)*u3(i,m,k)
                     sum9 = sum9 + d1(k,m)*u3(i,l,m)
2250              continue
                  sum = (sum1*p1(i,l,k) + &
                         sum2*p4(i,l,k) + &
                         sum3*p7(i,l,k) + &
                         sum4*p2(i,l,k) + &
                         sum5*p5(i,l,k) + &
                         sum6*p8(i,l,k) + &
                         sum7*p3(i,l,k) + &
                         sum8*p6(i,l,k) + &
                         sum9*p9(i,l,k)) * &
                         p6(i,l,k)*work(i,l,k)*d1(l,j)
4100           continue
               f(i,j,k) = f(i,j,k) + sum

!              --- third sum

               sum = 0d0
               do 4200 l = 1, mod

!              --- compute the "internal" sums

                  sum1 = 0d0
                  sum2 = 0d0
                  sum3 = 0d0
                  sum4 = 0d0
                  sum5 = 0d0
                  sum6 = 0d0
                  sum7 = 0d0
                  sum8 = 0d0
                  sum9 = 0d0
                  do 2350 m = 1, mod
                     sum1 = sum1 + d1(i,m)*u1(m,j,l)
                     sum2 = sum2 + d1(j,m)*u1(i,m,l)
                     sum3 = sum3 + d1(l,m)*u1(i,j,m)
                     sum4 = sum4 + d1(i,m)*u2(m,j,l)
                     sum5 = sum5 + d1(j,m)*u2(i,m,l)
                     sum6 = sum6 + d1(l,m)*u2(i,j,m)
                     sum7 = sum7 + d1(i,m)*u3(m,j,l)
                     sum8 = sum8 + d1(j,m)*u3(i,m,l)
                     sum9 = sum9 + d1(l,m)*u3(i,j,m)
2350              continue
                  sum = (sum1*p1(i,j,l) + &
                         sum2*p4(i,j,l) + &
                         sum3*p7(i,j,l) + &
                         sum4*p2(i,j,l) + &
                         sum5*p5(i,j,l) + &
                         sum6*p8(i,j,l) + &
                         sum7*p3(i,j,l) + &
                         sum8*p6(i,j,l) + &
                         sum9*p9(i,j,l)) * &
                         p9(i,j,l)*work(i,j,l)*d1(l,k)
4200           continue
               f(i,j,k) = f(i,j,k) + sum
3500        continue
3600     continue
3700  continue

5000  continue

      end
