      subroutine el6202 ( xgl, crn )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   07-06-1995
!
!   copyrigth (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the transformation of a rectangle element to the
!     standard element [-1,-1] x [1,1]
!
!     computed are:  given global coordinates in physical space
!                    the local coordiantes
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      double precision    xgl(1,2),  crn(3,2)

!     crn        i     input corners (a,b), (c,d) and (e,f)
!
!                         |-----------------| (c,d)
!                         |                 |
!                         |                 |
!                         |                 |
!                         |                 |
!                         |-----------------|
!                       (a,b)              (e,f)
!
!
!                         (a,b)   ->  (-1,-1)   (c,d) -> (1,1)  (e,f) -> (1,-1)
!
!                         xi = (a + c - 2 x) /( a - c)
!
!                         eta = (b + d - 2 y) /( b - d)
!
!     xgl       i/o    input global coordinates
!                      output local coordinates
!***********************************************************************
!
!                     COMMON BLOCKS
!
! **********************************************************************
!
!                     LOCAL PARAMETERS

      double precision rdum

!     rdum      help variable to store a constant
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      xgl(1,1) = ( crn(1,1) + crn(2,1) - 2 * xgl(1,1) ) &
                / (crn(1,1) - crn(2,1))
      xgl(1,2) = ( crn(1,2) + crn(2,2) - 2 * xgl(1,2) ) &
                / (crn(1,2) - crn(2,2))

!     --- test if applied rotation is correct

      if ( abs(crn(1,1)-crn(3,1)).lt.1d-2 ) then
         rdum = xgl(1,2)
         xgl(1,2) = xgl(1,1)
         xgl(1,1) = rdum
      end if

      end
