      subroutine el6206 ( coor, index1, x, d1, det, p1, p2, p3, p4, &
                          xref, wref, d1ref, d2ref, sol, user, &
                          mone, m )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   04-07-1995
!
!   copyrigth (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rigths reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute L2 norm for 2D element
!***********************************************************************
!
!                      KEYWORDS
!     spectral
!    norm
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer index1(*), mone, m
      double precision coor(2,*), x(m,2), d1(mone,mone), &
                       det(mone,mone), p1(mone,mone), p2(mone,mone), &
                       p3(mone,mone), p4(mone,mone),xref(m),wref(m), &
                       d1ref(m,m),d2ref(m,m), sol(m), user(*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     d1         i    array of length modecl x modecl contaning the values of
!                     the derivatives of the basisfunctions in the x-direction
!     d1ref
!     d2ref
!     det        i    array of length modecl x modecl containing the
!                     transformation determinant
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     m          i    number of integration points
!     mone       i    number of integration points in one dimension
!     p1         i    arrays of length m containing the elements of the invert
!                     jacobian of the local transform onto the standard
!                     quadrilateral
!     p2         i    See p1
!     p3         i    See p1
!     p4         i    See p1
!     sol        i
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     wref       i    array of length m containing the Legendre
!                     Gauss-Lobatto weights for numerical integration
!     x          i    x co-ordinate of point
!     xref       i    array of length m containing the coordinates of
!                     Legendre-Gauss-Lobatto points
!***********************************************************************
!
!                     COMMON BLOCKS
!
! **********************************************************************
!
!                     LOCAL PARAMETERS
!
      integer i, j, k

!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
!***********************************************************************
!
!             SUBROUTINES CALLED
!
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP642    compute the integration points, determinant and
!               partial derivatives, two-dimensional case
!               (element dependent)
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- store global integration points
!         calculate the determinant
!
      call elp642 ( coor, index1, x, d1, det, p1, p2, p3, p4, &
                    mone, m )

!     --- calculate the weights

      call elp640(0, xref, wref, d1ref, d2ref, mone)
      k=1
      do 10, j=1,mone
         do 20, i=1,mone
            user(50) = user(50) + wref(i)*wref(j)*det(i,j)* &
                       (sol(k))**2
            k = k + 1
   20    continue
   10 continue
      end
