      subroutine el6212 ( coor, index1, x, sol, phix, phiy, &
                          inter, iinter, numold, mone, m )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 2.2    date   30-05-1996 Multiple solutions
!        version 2.1    date   25-01-1996 Layout
!        version 2.0    date   10-09-1995 quadrilateral element
!        version 1.0    date   30-06-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!       Interpolation of (multiple) solution(s) in an arbitrary point
!       ONLY WORKS WITH ODD NUMBER OF SIDEPOINTS
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer index1(*), numold, mone, m, iinter(*)
      double precision coor(2,*), x(m,2), sol(mone*mone, numold), &
                       phix(mone), phiy(mone), inter(4,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     iinter i/o   Integer array containing information of points
!                  are already interpolated or not
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     inter  i/o   Array containing on input the (x,y) co-ordinates of the
!                  points to be interpolated such that
!                  x = inter(1,*)
!                  y = inter(2,*)
!                  Output is given in fourth column of inter
!     m       i    number of integration points
!     mone    i    number of integration points in one dimension
!     numold  i    Number of vectors that are stored in VECOLD
!     phix    i    Array of length mone containing the derivatives
!                  of the basis functions in one dimension
!     phiy    i    Array of length mone containing the derivatives
!                  of the basis functions in one dimension
!     sol     i    Solution array.
!     x       i    Array containing the co-ordinate of the nodal points
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k,ij, ii, indx, ntrial
      double precision lox, loy, rbx, rby, tolf, &
                       crn(8,2), xlocal(1,2), sum(2), rdum(50), &
                       crnini(3,2), f1, f2, j11, j12, j21, j22, errorf, &
                       errorx, delta1, delta2, n, z, y, bndval

!     bndval
!     crn
!     crnini
!     delta1     Newton-update
!     delta2     Newton-update
!     errorf
!     errorx
!     f1
!     f2
!     i          Counting variable
!     ii         help parameter
!     ij         help parameter
!     indx
!     j          Counting variable
!     j11
!     j12
!     j21
!     j22
!     k          Counting variable
!     lox
!     loy
!     n
!     ntrial     Maximum number of Newton-iterations
!     rbx
!     rby
!     rdum       dummy variable
!     sum        Help parameter to compute a sum
!     tolf       Tolerance for local co-ordinates
!     xlocal
!     y
!     z          z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision fncf1, fncf2, fncj11, fncj12, fncj21, fncj22

!     EL6202    compute the transformation of a rectangle element to the
!               standard element [-1,-1] x [1,1]
!     ELP648    compute quantities for one-dimensional spectral reference
!               element [-1,1]
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     FNCF1     Local statement function
!     FNCF2     Local statement function
!     FNCJ11    Local statement function
!     FNCJ12    Local statement function
!     FNCJ21    Local statement function
!     FNCJ22    Local statement function
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1620   Number of side points even
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                    STATEMENT FUNCTIONS
!
      fncf1(n,z,y) = y + (1 - n)*(1 - z)*(1 + n + z)*crn(1,1)/4 - &
         (1 - n)*(1 - z**2)*crn(2,1)/2 - &
         (1 - n)*(1 + z)*(-1 - n + z)*crn(3,1)/4 - &
         (1 - n**2)*(1 + z)*crn(4,1)/2 - &
         (1 + n)*(1 + z)*(-1 + n + z)*crn(5,1)/4 - &
         (1 + n)*(1 - z**2)*crn(6,1)/2 + &
         (1 + n)*(1 - z)*(1 - n + z)*crn(7,1)/4 - &
         (1 - n**2)*(1 - z)*crn(8,1)/2

      fncf2(n,z,y) = y + (1 - n)*(1 - z)*(1 + n + z)*crn(1,2)/4 - &
         (1 - n)*(1 - z**2)*crn(2,2)/2 - &
         (1 - n)*(1 + z)*(-1 - n + z)*crn(3,2)/4 - &
         (1 - n**2)*(1 + z)*crn(4,2)/2 - &
         (1 + n)*(1 + z)*(-1 + n + z)*crn(5,2)/4 - &
         (1 + n)*(1 - z**2)*crn(6,2)/2 + &
         (1 + n)*(1 - z)*(1 - n + z)*crn(7,2)/4 - &
         (1 - n**2)*(1 - z)*crn(8,2)/2

      fncj11(z,n) = (1 - n)*(1 - z)*crn(1,1)/4 - &
         (1 - z)*(1 + n + z)*crn(1,1)/4 + &
         (1 - z**2)*crn(2,1)/2 + (1 - n)*(1 + z)*crn(3,1)/4 + &
         (1 + z)*(-1 - n + z)*crn(3,1)/4 + n*(1 + z)*crn(4,1) - &
         (1 + n)*(1 + z)*crn(5,1)/4 - (1 + z)*(-1 + n + z)*crn(5,1)/4 - &
         (1 - z**2)*crn(6,1)/2 - (1 + n)*(1 - z)*crn(7,1)/4 + &
         (1 - z)*(1 - n + z)*crn(7,1)/4 + n*(1 - z)*crn(8,1)

      fncj12(z,n) = (1 - n)*(1 - z)*crn(1,1)/4 - &
         (1 - n)*(1 + n + z)*crn(1,1)/4 + &
         (1 - n)*z*crn(2,1) - (1 - n)*(1 + z)*crn(3,1)/4 - &
         (1 - n)*(-1 - n + z)*crn(3,1)/4 - (1 - n**2)*crn(4,1)/2 - &
         (1 + n)*(1 + z)*crn(5,1)/4 - (1 + n)*(-1 + n + z)*crn(5,1)/4 + &
         (1 + n)*z*crn(6,1) + (1 + n)*(1 - z)*crn(7,1)/4 - &
         (1 + n)*(1 - n + z)*crn(7,1)/4 + (1 - n**2)*crn(8,1)/2

      fncj21(z,n) = (1 - n)*(1 - z)*crn(1,2)/4 - &
         (1 - z)*(1 + n + z)*crn(1,2)/4 + &
         (1 - z**2)*crn(2,2)/2 + (1 - n)*(1 + z)*crn(3,2)/4 + &
         (1 + z)*(-1 - n + z)*crn(3,2)/4 + n*(1 + z)*crn(4,2) - &
         (1 + n)*(1 + z)*crn(5,2)/4 - (1 + z)*(-1 + n + z)*crn(5,2)/4 - &
         (1 - z**2)*crn(6,2)/2 - (1 + n)*(1 - z)*crn(7,2)/4 + &
         (1 - z)*(1 - n + z)*crn(7,2)/4 + n*(1 - z)*crn(8,2)

      fncj22(z,n) = (1 - n)*(1 - z)*crn(1,2)/4 - &
         (1 - n)*(1 + n + z)*crn(1,2)/4 + &
         (1 - n)*z*crn(2,2) - (1 - n)*(1 + z)*crn(3,2)/4 - &
         (1 - n)*(-1 - n + z)*crn(3,2)/4 - (1 - n**2)*crn(4,2)/2 - &
         (1 + n)*(1 + z)*crn(5,2)/4 - (1 + n)*(-1 + n + z)*crn(5,2)/4 + &
         (1 + n)*z*crn(6,2) + (1 + n)*(1 - z)*crn(7,2)/4 - &
         (1 + n)*(1 - n + z)*crn(7,2)/4 + (1 - n**2)*crn(8,2)/2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- check if mone is odd

      if ( mod(mone,2).eq.0 ) then
         call eropen ( 'el6212' )
         call errint ( mone, 1 )
         call errsub ( 1620, 1, 0, 0 )
         call erclos ( 'el6212' )
         go to 1000
      end if

      ntrial = 100
      tolf   = 1.0d-6
      bndval = 1.0d0 + 1.0d-6

!     --- store global integration points

      do 150 i = 1, mone
         ij = i
         do 100 j = 1, mone
            x(ij,1) = coor(1, index1(ij))
            x(ij,2) = coor(2, index1(ij))
            ij = ij + mone
100      continue
150   continue

!     --- get corners of bounding box correspondig to numbering
!         of a standard quadratic element

      crn(1,1) = x(1,1)
      crn(1,2) = x(1,2)
      crn(2,1) = x((mone+1)/2,1)
      crn(2,2) = x((mone+1)/2,2)
      crn(3,1) = x(mone,1)
      crn(3,2) = x(mone,2)
      crn(8,1) = x((mone*(mone-1)+2)/2,1)
      crn(8,2) = x((mone*(mone-1)+2)/2,2)
      crn(4,1) = x((mone*(mone+1))/2,1)
      crn(4,2) = x((mone*(mone+1))/2,2)
      crn(7,1) = x((mone*(mone-1))+1,1)
      crn(7,2) = x((mone*(mone-1))+1,2)
      crn(6,1) = x((2*mone*mone-mone+1)/2,1)
      crn(6,2) = x((2*mone*mone-mone+1)/2,2)
      crn(5,1) = x(mone*mone,1)
      crn(5,2) = x(mone*mone,2)

!     --- determine bounding box of element

      lox  = min(crn(1,1),crn(2,1),crn(3,1),crn(4,1), &
                 crn(5,1),crn(6,1),crn(7,1),crn(8,1))
      rbx  = max(crn(1,1),crn(2,1),crn(3,1),crn(4,1), &
                 crn(5,1),crn(6,1),crn(7,1),crn(8,1))
      loy  = min(crn(1,2),crn(2,2),crn(3,2),crn(4,2), &
                 crn(5,2),crn(6,2),crn(7,2),crn(8,2))
      rby  = max(crn(1,2),crn(2,2),crn(3,2),crn(4,2), &
                 crn(5,2),crn(6,2),crn(7,2),crn(8,2))

      indx=10

!     --- check of any points have to be interpolated

200   if ( (iinter(indx).eq.0) .and. (iinter(1).gt.iinter(2)) &
           .and. ((indx-10).lt.iinter(1)) ) then
         xlocal(1,1)=inter(1,indx)
         xlocal(1,2)=inter(2,indx)

!        --- check if point is in bounding box

         if ( xlocal(1,1).ge.lox .and. &
              xlocal(1,1).le.rbx .and. &
              xlocal(1,2).ge.loy .and. &
              xlocal(1,2).le.rby ) then
            crnini(1,1)=crn(1,1)
            crnini(1,2)=crn(1,2)
            crnini(2,1)=crn(5,1)
            crnini(2,2)=crn(5,2)
            crnini(3,1)=crn(3,1)
            crnini(3,2)=crn(3,2)

!           --- get initial guess local coordinates

            call el6202(xlocal,crnini)

            do 210 k = 1, ntrial

!           --- f1 is x - (sum over x_i N_i)

               f1 = fncf1(xlocal(1,1),xlocal(1,2),inter(1,indx))

!              --- f2 is y - (sum over x_i N_i)

               f2 = fncf2(xlocal(1,1),xlocal(1,2),inter(2,indx))

!              --- determine derivative of (f1 f2)

               j11 = fncj11(xlocal(1,1),xlocal(1,2))
               j12 = fncj12(xlocal(1,1),xlocal(1,2))
               j21 = fncj21(xlocal(1,1),xlocal(1,2))
               j22 = fncj22(xlocal(1,1),xlocal(1,2))

!              --- determine the error in (f1 f2)

               errorf = abs(f1)+abs(f2)

!              --- print *,'Error in f on step ',k,'=', errorf

!                  --- determine the Newton-update

               delta1 = f2*j12/(-(j12*j21) + j11*j22) - &
                        f1*j22/(-(j12*j21) + j11*j22)
               delta2 = -(f2*j11/(-(j12*j21) + j11*j22)) + &
                        f1*j21/(-(j12*j21) + j11*j22)

               errorx = abs(delta1)+abs(delta2)

!              --- correct the local coordiates

               xlocal(1,1) = xlocal(1,1) + delta1
               xlocal(1,2) = xlocal(1,2) + delta2

!              --- decide whether to stop iterating

               if ( errorf.le.tolf .or. errorx.le.tolf ) goto 220
               if (k.eq.ntrial) then

!              --- print *,'maximum number of iters'

                  indx = indx + 1
                  goto 200
               end if
210         continue

!           --- determine if the point really is in the element

220         if ( abs(xlocal(1,1)).gt.bndval .or. &
                  abs(xlocal(1,2)).gt.bndval) then

!           --- print *,'=============================='
!               print *,'local coordinates not correct'
!               print *,xlocal(1,1),xlocal(1,2),errorf
!               print *,'=============================='

               indx = indx + 1
               goto 200
            end if

!           --- calculate basis functions

            call elp648(xlocal(1,1),rdum,phix,mone)
            call elp648(xlocal(1,2),rdum,phiy,mone)

!           --- calculate interpolant

            sum(1)=0.0d0
            sum(2)=0.0d0
            k=1
            do 250 i=1,mone
               do 240 j=1,mone
                  do 230 ii=1, numold
                    sum(ii) = sum(ii) + sol(k,ii)*phix(i)*phiy(j)
230               continue
                  k=k+1
240            continue
250         continue

            do 300, ii=1, numold
              inter(2+ii,indx)=sum(ii)
300         continue

            iinter(indx)=1
            iinter(2) = iinter(2) + 1
            indx = indx + 1
         else
         indx = indx + 1
         goto 200
         end if
      else
         indx = indx + 1
         if ( (indx-10).lt.iinter(1) ) goto 200
      end if
      if ( (indx-10).lt.iinter(1) ) goto 200

1000  continue
      end
