      subroutine elchkintcoef ( iseqcoef, itype, ielgrp, ielem, ival, &
                                namevar, ivalmin, ivalmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if integer coefficient is between two limits
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqcoef, itype, ielgrp, ielem, ival, ivalmin, ivalmax
      character (len=*) namevar

!     ielem          i    Element number
!     ielgrp         i    element group number
!     iseqcoef       i    Sequence number of coefficients input
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     ival           i    Reference to position in rincof
!     ivalmax        i    Maximum integer value
!     ivalmin        i    Minimum integer value
!     namevar        i    Name of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELERRINTCOEF   Give error message if integer coefficient out of range
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elchkintcoef' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elchkintcoef'

      end if  ! ( debug )

      if ( ival<ivalmin .or. ival>ivalmax ) then

!     --- error: coefficient has wrong value

          call elerrintcoef ( iseqcoef, itype, ielgrp, ielem, ival, &
                              namevar )

      end if  ! ( ival<ivalmin .or. ival>ivalmax )

      call erclos ( 'elchkintcoef' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elchkintcoef'

      end if  ! ( debug )

      end
