      subroutine elcompdet ( djacob, e, x, lderiv )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 03-02-2005
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute determinant of tetrahedron in 3d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     area
!     element
!     tetrahedron
!     determinant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision djacob, e(n,ndim), x(n,ndim)
      integer lderiv

!     djacob  o      jacobian of element
!     e       o      Work array of length 4 x 3, to store the factors
!                     ij
!                    e.        See method
!     lderiv  i      Indication if derivatives must be computed (1) or not (0)
!     x       i      array of length 3 x 4 containing the x- y- and
!                    z-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                     i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision delta(4,3)
      integer i, k, iven, ni, indlc(5), ind1(4)

!     delta          Work array of length 4 x 3, to store the factors
!                         ij
!                    delta.    See method
!     i              Counting variable
!     ind1           Array of length 4 to store the cyclic permutation of 1 - 3
!     indlc          Integer array containing the cyclic permutation of
!                    1,2,3,4,5
!     iven           help parameter to compute (-1)
!     k              Counting variable
!     ni             Upper boundary in the computation for the loop variable i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indlc / 2,3,4,1,2 /, ind1 / 2,3,1,2 /
! ======================================================================
!
      call eropen ( 'elcompdet' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elcompdet'
110      format ( a, 1x, (5d12.4) )
         call prinrl2 ( x, 4, 3, 'x' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- lderiv = 1  Compute derivatives or iweigo = 1: weights
!         Compute the quantities delta^(ij) ,  e^(ij) and djacob

      do k = 1,3
         do i = 1,4
            delta(i,k) = x(indlc(i),k) - x(i,k)
         end do
      end do

      if ( lderiv==1 ) then

!     --- lderiv = 1,  compute all factors e^(ij)

         ni = 4

      else

!     --- lderiv = 0,  compute only the factors e^(1j)

         ni = 1

      end if

      iven = -1
      do i = 1, ni
         do k = 1, 3
            e (i,k) = iven * ( delta ( indlc(i), ind1(k) ) * &
                      delta ( indlc(i+1), ind1(k+1) ) - &
                      delta ( indlc(i), ind1(k+1) ) * &
                      delta ( indlc(i+1), ind1(k) ) )
         end do
         iven = - iven
      end do

      djacob = - delta(1,1) * e(1,1) - delta(1,2) * e(1,2) &
               - delta(1,3) * e(1,3)
      if ( checksign .and. djacob<0d0 ) then
         print *,'area of element ', ielem, ' is negative'
      end if

1000  call erclos ( 'elcompdet' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,110) 'djacob', djacob
         write(irefwr,*) 'End elcompdet'

      end if

      end
