      subroutine elflr5 ( ichois, ivalue, ivalh, user, result, x, y, z, &
                          uold, nloc, mloc, k, index1, phi, index, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-02-2003 Layout
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 10-11-1984
!
!   copyright (c) 1984-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill function values in array result for the integration points
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ivalue, ivalh, index1(*), index(*), nloc, mloc, k
      double precision result(*), user(*), uold(*), x(*), y(*), z(*), &
                       work(*), phi(*)

!     ichois         i    Choice parameter
!     index          i    special index array of length m that is only used
!                         when uold is used
!                         index(i) contains the starting point of the nodal
!                         point in uold
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     ivalh          i    result of subroutine eluser when ivalue>2000
!                         the function values will be evaluated in the
!                         integration points
!     ivalue         i    result of subroutine eluser (ivalue>0)
!     k              i    number of degrees of freedom in the points in uold
!                         hence uold((i-1)*k+j) is jth degree of freedom in
!                         nodal point i
!     m              i    number of integration points
!     n              i    number of basis functions
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     result         o    array containing the function in the integration
!     node
!     uold           i    Array containing the previous iteration (old solution)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     work                Work array of length nunkp
!     x              i    x co-ordinates of integration points
!     y              i    y co-ordinates of integration points
!     z              i    z co-ordinates of integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xc, yc, zc, f, h
      integer i, j, ihi, ihst

!     f
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ihi
!     ihst
!     j              Counting variable
!     xc             x-coordinate of the midth-point
!     yc             y-coordinate of the midth-point
!     zc             z-coordinate of the midth-point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision funcc1, funccf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FUNCC1         User function subroutine to compute the value of a
!                    coefficient in a point depending on a preceding solution
!     FUNCCF         User function subroutine to compute the value of a
!                    coefficient in a point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elflr5' )
      if(ivalue.le.1000) then

!     --- 0 < ivalue < 1000

          do i=1,mloc
             xc=x(i)
             yc=y(i)
             zc=z(i)
             result(i)=funccf(ivalue,xc,yc,zc)
          end do

      else if(ivalue.le.2000) then

!     --- 1000 < ivalue < 2000

          ihst=1
          do i=1,mloc
             xc=x(i)
             yc=y(i)
             zc=z(i)
             do j=ihst,index(i)
                work(j-ihst+1)=uold(j)
             end do
             ihst=index(i)+1
             result(i)=funcc1(ivalue-1000,xc,yc,zc,work)
          end do
      else if(ivalue.eq.2001) then

!     --- ivalue = 2001

          if(ivalh.gt.0) then

!         --- ivalh > 0

             do i=1,nloc
                result(i)=user(ivalh+index1(i))
             end do
          else

!         --- ivalh < 0

             do i=1,mloc
                result(i)=0d0
             end do
             ihi=0
             do i=1,inpelm
                f=user(-ivalh+index1(i))
                do j=1,mloc
                   result(j)=result(j)+f*phi(ihi+j)
                end do
                ihi=ihi+nloc
             end do
          end if
      else if(ivalue.eq.2002) then

!     --- ivalue = 2002

          h=user(ivalh+irelem)
          do i=1,mloc
             result(i)=h
          end do
      end if
      call erclos ( 'elflr5' )
      end
