      subroutine elfmulttang ( f, fmult, qmat, idim, m, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 10-03-2003 Extension of shapes
!        version  2.0  date 21-01-2003 Extra parameter ishape
!        version  1.0  date 11-09-2002
!
!   copyright (c) 2002-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply the right-hand side by (e_zeta,e_dim)
!     At this moment it is assumed that e_zeta is constant
!     e_zeta must be stored in qmat(1,.)
!     Later on this must be replaced by qmat(1,.,k)
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     element_vector
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idim, m, ishape
      double precision f(m), fmult(m), qmat(3,3,*)

!     f              i    Contains the right-hand side vector in the integration
!     node
!     fmult          o    New right-hand side vector after multiplication
!     idim           i    defines the direction of the base vector e_idim
!     ishape         i    Shape number of elements
!     m              i    number of integration points
!     qmat           i    transformation matrix global -> local coordinates
!                         in case of a 3D element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1025   shape number has not yet been implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfmulttang' )
      if ( ishape.eq.1 ) then

!     --- Linear line element, transformation is constant

         do k = 1, m
            fmult(k) = f(k)*qmat(1,idim,1)
         end do

      else if ( ishape.eq.3 ) then

!     --- Linear triangles, transformation is constant

         do k = 1, m
            fmult(k) = f(k)*qmat(1,idim,1)
         end do

      else if ( ishape.eq.2 .or. ishape.eq.5 ) then

!     --- Other types of elements, transformation not constant

         do k = 1, m
            fmult(k) = f(k)*qmat(1,idim,k)
         end do

      else

!     --- Other type of shape number possibly incorrect

          call errint ( ishape, 1 )
          call errchr ( 'elfmulttang', 1 )
          call errsub ( 1025, 1, 0, 1 )

      end if

      call erclos ( 'elfmulttang' )
      end
