      subroutine elgetunknown ( iseq, nu, uold, namevec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-12-2012 Long integers
!        version  1.0  date 13-04-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get unknown vector from all vectors stored vecold and
!     store in uold
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
!     extract_unknown
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, nu
      double precision uold(nu)
      character (len=*) namevec

!     iseq           i    Sequence number
!     namevec        i    Name of vector for error messages
!     nu             i    Number of unknowns
!     uold           o    Array containing the vector to be extracted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision, pointer :: vecoldlc(:)

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2763   number of degrees of freedom per element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elgetunknown' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elgetunknown'
         write(irefwr,100) 'nu, iseq, inpelm, numold', &
                            nu, iseq, inpelm, numold
         call prinin1 ( index4, inpelm, numold, 'index4' )
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iseqtype = oldsols(iseq)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      index4n => indoldsols(iseqtype)%index4
      vecoldlc => oldsols(iseq)%oldvecs

      if ( nu/=index4n(inpelm) ) then

!     --- Number of degrees of freedom incorrect

         call errchr ( namevec, 1 )
         call errint ( iseq, 1 )
         call errint ( index4n(inpelm), 2 )
         call errint ( nu, 3 )
         call errsub ( 2763, 3, 0, 1 )
         go to 1000

      end if  ! ( nu/=index4n(inpelm) )

!     --- Copy solution

      do i = 1, nu
         uold(i) = vecoldlc(index3n(i))
      end do  ! i = 1, nu

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elgetunknown'

      end if
      call erclos ( 'elgetunknown' )

      end
