      subroutine eli003 ( ichint, ichfun, mvec, usollc, coor, index1, &
                          value, phiksi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-07-2013 Extra parameter phiksi
!        version  1.8  date 29-07-2013 New call to elp621
!        version  1.7  date 20-06-2013 Set parameter n
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      subroutine eli003 is used to compute the boundary integral over
!      a bilinear surface quadrilateral
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichint, mvec, index1(4), ichfun
      double precision usollc(mvec,4), coor(3,*), value(*), &
                       phiksi(4,4,2)

!     coor     i    Array containing the co-ordinates of the mesh
!     ichfun   i    indication of the value of f.
!                   Possibible values:
!                   0         f = 1
!                   >0        f = func ( ichfun, x, y, z ) (real case)
!                             or
!                             f = cfunc ( ichfun, x, y, z ) (complex
!                                 case)
!                             at this moment only the combination f#1
!                             and ichint=1 is permitted
!     ichint   i    indicates the kind of integral to be computed.
!                   possibilities:
!                   1          /  f u  ds
!                   2          /  f  u.n  ds
!                   9          /  f ds
!     index1   i    Array containing the nodal point numbers of the element
!     mvec     i    Number of components of the solution vector
!     phiksi  i/o   contains derivatives in reference element for gauss
!                   rule.
!     usollc   i    Local solution vector
!     value    o    Contains the value of the computed integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision som, x(4,3), w(4), dphidx(48), xgauss(12), &
                       f(4), qmat(3,27), phi(16)

!     dphidx         Array of length n x m x ndim  containing the
!                    derivative
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     f              array of length m containing the rhs
!     i              Counting variable
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     som            Help parameter to compute a sum
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision func

!     ELP621  Compute basis functions and weights for bilinear quadrilateral
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     FUNC    Computes the value of the vector in a point as function of the
!             co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli003' )

!     --- Compute the weight functions

      m = 4
      n = 4
      iweigh = 1
      ideriv = 0
      irule = 1
      igausp = 0
      icoor = 1
      call elp621 ( coor, x, w, dphidx, xgauss, phi, index1, qmat, &
                    phiksi )

      if ( ichfun==0 ) then
         do i = 1, m
            f(i) = 1d0
         end do
      else
         do i = 1, m
            f(i) = func ( ichfun, x(i,1), x(i,2), x(i,3) )
         end do
      end if
      if ( ichint==1 ) then
         som = 0d0
         do i = 1, m
            som = som + w(i)*f(i)*usollc(1,i)
         end do
         value(1) = som
      else if ( ichint==9 ) then
         som = 0d0
         do i = 1, m
            som = som + w(i)*f(i)
         end do
         value(1) = som
      end if

      call erclos ( 'eli003' )
      end
