      subroutine elm800cons ( metup, metupc, w, phi, elemmt, eldiag, &
                              idefcr, beta, psiupw, elemmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for zeroth order part of PDE
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_matrix
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer metup, metupc, idefcr
      double precision w(m), phi(*), elemmt(n,n), eldiag(*), &
                       beta(*), psiupw(*), elemmt2(*)

!     beta           i    array of length m containing the values of the
!                         third coefficient in the integration points
!     eldiag         o    Contains the lumped element matrix (as vector)
!                         in case of shifted Laplace
!     elemmt        i/o   Element matrix
!     elemmt2       i/o   Second element matrix for defect correction
!     idefcr         i    If 1 defect correction is applied, if 0 not
!     metup          i    Upwind parameter
!     metupc         i    If 0 the complete upwinding must be taken into account
!                         otherwise (1) only the convective terms
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision selem(n,n), sum
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     j              Counting variable
!     selem          Contains part of element matrix corresponding to
!                    "zeroth order" term
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3006         Compute integrals of the shape / beta(x) phi (x) psi (x) dx
!                                                   e            i       j
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800cons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800cons'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      if ( metup==0 .or. metupc==1 ) then

         jadd = 0
         call el3002 ( beta, selem, w, phi, m, n, n )
         jadd = 1

         elemmt = elemmt+selem

         if ( IM%lapshift ) then

!        --- Laplace shift

            do i = 1, n
               sum = 0d0
               do j = 1, n
                  sum = sum+selem(i,j)
               end do
               eldiag(i) = sum
            end do

         end if  ! ( IM%lapshift )

      else

         call el3006 ( beta, elemmt, w, phi, psiupw )

      end if

      if ( idefcr==1 ) then

!     --- Build central difference matrix in elemmt(.,2)

         call el3002 ( beta, elemmt2, w, phi, m, n, n )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm800cons'

      end if  ! ( debug )
      call erclos ( 'elm800cons' )

      end
