      subroutine elm811vec ( velocity )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element right-hand side vector for computation of
!     pseudo stream function
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision velocity(n,*)

!     velocity       i    Contains the velocity in the nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision un(m), work(m)

!     un             normal velocity
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003         Compute integrals of the shape / f(x) B%lam (x) dx
!                                                   e         i
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm811vec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm811vec'
         call prinrl1 ( B%qmat, m, 9, 'B%qmat' )
         call prinrl ( velocity(1,1), m, 'u' )
         call prinrl ( velocity(1,2), m, 'v' )
         call prinrl ( velocity(1,3), m, 'w' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      un = velocity(1:m,1)*B%qmat(1,1,1:m) + &
           velocity(1:m,2)*B%qmat(1,2,1:m) + &
           velocity(1:m,3)*B%qmat(1,3,1:m)
      if ( debug ) call prinrl ( un, m, 'un' )
      call el3003 ( un, E%vec, B%w, B%psi, m, n )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, n, 'E%vec' )
         write(irefwr,*) 'End elm811vec'

      end if  ! ( debug )
      call erclos ( 'elm811vec' )

      end
