      subroutine elmmulturho ( u, rho, ind, kstep, ndim, m, ustore, &
                               ichoice, indrho )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-05-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply the velocity u by rho in case both are constant
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     velocity
!     density
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ind(*), kstep, ndim, m, ichoice, indrho
      double precision u(m,*), rho(*), ustore(*)

!     ichoice        i    Integer choice parameter, defining what actions
!                         to be preformed
!                         1:  Call in element independent part of
!                             element subroutine
!                             The velocity is stored in ustore and
!                             if u and rho are constant, u is multiplied by rho
!                         2:  Call in element dependent part of
!                             element subroutine
!                             if u or rho are not constant, u is multiplied by
!                             rho
!     ind            i    In this array of length 50, of each coefficient it
!                         is stored whether the coefficient is 0 (ind(i) = 0), &
!                         or something else as described in common celiar
!                         In this case ind must start with the first entry
!                         where information concerning the velocity is stored, &
!                         for example entry 6
!     indrho         i    Entry in array ind corresponding to density rho
!     kstep          i    Step in integration loop, see common celint
!     m              i    number of integration points
!     ndim           i    Dimension of the space.
!     rho            i    density rho
!     u              i    Array containing the velocity components per
!                         integration point
!                         u(k,i) contains the i^th component in integration
!                         point k
!     ustore        i/o   array of length 3 containing the values of the
!                         velocity if it is constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmmulturho' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmmulturho'
         write(irefwr,1) 'ichoice, ndim, indrho', ichoice, ndim, indrho
         write(irefwr,1) 'kstep, m', kstep, m
         call prinin ( ind, ndim, 'ind' )
         call prinrl ( rho, m, 'rho' )
         call prinrl1 ( u, ndim, m, 'u' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice==1 ) then

!     --- ichoice = 1, general part

         do i = 1, ndim

            ustore(i) = u(1,i)
            if ( ind(i)<0 .and. indrho<0 ) then

!           --- ind(i)<0, hence u constant and rho constant

               do k = kstep, m, kstep
                  u(k,i) = u(k,i)*rho(k)
               end do

            end if  ! ( ind(i)<0 )

         end do  ! i = 1, ndim

      else

!     --- ichoice = 2, each element

         do i = 1, ndim

            if ( ind(i)>0 ) then

!           --- ind(i)>0, hence u variable

               do k = kstep, m, kstep
                  u(k,i) = u(k,i)*rho(k)
               end do

            else if ( ind(i)<0 .and. indrho>0 ) then

!           --- ind(i)<0, hence u constant, but rho variable

               do k = kstep, m, kstep
                  u(k,i) = ustore(i)*rho(k)
               end do

            end if  ! ( ind(i)<0 )

         end do  ! i = 1, ndim

      end if  ! ( ichoice==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( u, ndim, m, 'u' )
         write(irefwr,*) 'End elmmulturho'

      end if
      call erclos ( 'elmmulturho' )

      end
