      subroutine elp121(kchois,coor,x,w,phis,index1,phi,xgaus,tang)
!
! ======================================================================
!
!
!        programmer   Guus Segal
!
!        version 4.2  date   22-12-1992 remove name from heading;
!                                       expl. decl.; replace ertrap
!        version 4.1  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 4    date   13-02-1988 Factor 2 pi for axisymmetric co-ord.
!        version 3    date   27-07-1987
!
!
!
!   copyright (c) 1982,1992  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!   DESCRIPTION
!
!     basis functions quadratic line element
!
! **********************************************************************
      use sepmodulecomio
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
      integer kchois, index1(*)
      double precision coor(*), x(*), phi(*), w(*), phis(*), xgaus(*), &
             tang(*)
! **********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
!     kchois   choice parameter, possibilities:
!
!             0:  coordinates and weights for the integration are stored
!                 cartesian co-ordinates
!                 three-point newton cotes rule
!
!             1:  coordinates and weights for the integration are stored
!                 derivatives are computed as well
!                 cartesian co-ordinates
!                 three-point newton cotes rule
!
!             2:  see 0, however for axisymmetric co-ordinates
!
!             3:  see 1, however for axisymmetric co-ordinates
!
!             4-7:  See 0-3, however a three-point gauss rule is applied
!                            arrays phi and xgaus are filled
!
!             >=100:  See kchois-100, array tang is filled also
!
!     coor        array containing the coordinates
!
!     name        name of calling subroutine (character constant)
!
!     x           array containing the co-ordinates of the nodal points
!                 sequence:  1-3: x-co-ordinates
!                            4-6: y-co-ordinates
!
!     w           array containing the weights for the numerical integration
!
!     phis        array containing the values of the derivatives of the
!                 basis functions
!                 sequence:  dphi /ds (x ) = phis((i-1)*3+j)
!                                i      j
!     index1      see to0000
!
!     phi         array containing the values of the basis functions
!                 sequence:  phi (x ) = phi ((i-1)*3+j)
!                               i    j
!                 array phi is only filled when gauss integration is applied
!
!     xgaus       array containing the co-ordinates of the gauss points
!                 sequence:  1-3: x-co-ordinates
!                            4-6: y-co-ordinates
!                 array xgaus is only filled when gauss integration is applied
!
!     tang        array containing the tangential vector in the integration
!     node
!                 sequence:   1-3: x-component
!                             4-6: y-component
!
! **********************************************************************
!
!   COMMON  BLOCKS
!

! **********************************************************************
!
!     ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!     CALLED SUBROUTINES
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!   LOCAL  PARAMETERS
!
! **********************************************************************
      integer i, ichois, ih, j, jchois
      logical deriv,gauss,axisym,tangvc
      double precision delta, dx, dy, t1, t2, x2, y2
      double precision wh(3), alpha(3), phider(9), xksi(3), yksi(3)
      save deriv,gauss,axisym,wh,alpha,phider,tangvc
! ======================================================================
      call eropen( 'elp121' )

      if(ifirst.eq.0) then
!
!      *  ifirst = 0  First time element group is called
!                     Compute some initial values
!
         if(kchois.ge.100) then
!
!         *  kchois >= 100  compute tangential vector
!
            tangvc=.true.
            ichois=kchois-100
         else
            tangvc=.false.
            ichois=kchois
         end if
         if(ichois.ge.4) then
!
!         *  ichois >= 4  gauss integration
!
            gauss=.true.
            jchois=ichois-4
            wh(1)=2.5d0/9d0
            wh(2)=4d0/9d0
            wh(3)=wh(1)
            alpha(1) = 0.5d0 * 0.225403330758517d0
            alpha(2) = 0.5d0
            alpha(3) = 1d0 - alpha(1)
!
            do 10 j=1,3
               phi(j)=(1d0-alpha(j))*(2*(1d0-alpha(j))-1d0)
               phi(j+6)=alpha(j)*(2*alpha(j)-1d0)
 10            phi(j+3)=4d0*alpha(j)*(1d0-alpha(j))
         else
!
!         *  ichois < 4  Newton Cotes integration
!
            gauss=.false.
            jchois=ichois
            wh(1)=1d0/6d0
            wh(2)=4d0/6d0
            wh(3)=wh(1)
            alpha(1)=0d0
            alpha(2)=.5d0
            alpha(3)=1d0
         end if
         if(mod(ichois,2).eq.1) then
!
!         *  ichois is odd,  compute derivatives
!
            deriv=.true.
            do 20 j=1,3
               phider(j)=-(4d0*(1d0-alpha(j))-1d0)
               phider(j+3)=4d0*(1d0-2d0*alpha(j))
 20            phider(j+6)=4d0*alpha(j)-1d0
         else
!
!         *  ichois is even, no derivatives
!
            deriv=.false.
         end if
         if(jchois.ge.2) then
            axisym=.true.
         else
            axisym=.false.
         end if
      end if
!
!      variable part
!
      do 100 i=1,3
         ih=2*index1(i)
         x(i)=coor(ih-1)
100      x(i+3)=coor(ih)
!
      if(.not. gauss) then
!
!      *  gauss is false  Newton Cotes integration rule
!
!      *  check whether the element is straight
!
         x2=2d0*x(2)
         y2=2d0*x(5)
         if(abs(x2-x(1)-x(3)).lt.sqreps*(abs(x(1))+abs(x(3))) &
           .and. abs(y2-x(4)-x(6)).lt.sqreps*(abs(x(4))+abs(x(6)))) then
!
!          *  straight element
!
            dx=x(3)-x(1)
            dy=x(6)-x(4)
            delta=sqrt(dx*dx+dy*dy)
            if(delta.le.epsmac) then
               call errint(itype,1)
               call errint(ielem,2)
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            end if
            w(1)=delta
            w(2)=delta
            w(3)=delta
            if(tangvc) then
!
!            *  compute tangential vector
!
               t1=dx/delta
               t2=dy/delta
               do 110 i=1,3
                  tang(i)=t1
110               tang(i+3)=t2
            end if
         else
!
!         *  curved element
!
            dx=-1.5d0*x(1)+x2-0.5d0*x(3)
            dy=-1.5d0*x(4)+y2-0.5d0*x(6)
            delta=sqrt(dx*dx+dy*dy)
            if(tangvc) then
               tang(1)=dx/delta
               tang(4)=dy/delta
            end if
            w(1)=2d0*delta
            dx=0.5d0*(x(3)-x(1))
            dy=0.5d0*(x(6)-x(4))
            delta=sqrt(dx*dx+dy*dy)
            if(tangvc) then
               tang(2)=dx/delta
               tang(5)=dy/delta
            end if
            w(2)=2d0 * delta
            dx=0.5d0*x(1)-x2+1.5d0*x(3)
            dy=0.5d0*x(4)-y2+1.5d0*x(6)
            delta=sqrt(dx*dx+dy*dy)
            if(tangvc) then
               tang(3)=dx/delta
               tang(6)=dy/delta
            end if
            w(3)=2d0 * delta
!
            do 150 i=1,3
               if(abs(w(i)).le.epsmac) then
                  call errint(itype,1)
                  call errint(ielem,2)
                  call errint ( ielgrp, 3 )
                  call errsub ( 105, 3, 0, 0)
               end if
150         continue
         end if
      else
!
!       *  Gauss integration
!
!       *  compute dx/dksi, dy/dksi
!
         do 210 j=1,3
            xksi(j)=0d0
            do 200 i=1,3
200            xksi(j)=xksi(j)+x(i)*phider((i-1)*3+j)
            yksi(j)=0d0
            do 210 i=1,3
210            yksi(j)=yksi(j)+x(i+3)*phider((i-1)*3+j)
!
!       *  Compute Jacobian, store temporarily in w
!
         do 220 i=1,3
220         w(i)=sqrt(xksi(i)**2+yksi(i)**2)
         do 225 i=1,3
            if(abs(w(i)).le.epsmac) then
               call errint(itype,1)
               call errint(ielem,2)
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0)
            end if
225      continue
         if(tangvc) then
!
!         *  compute tangential vector
!
            do 228 i=1,3
               tang(i)=xksi(i)/w(i)
228            tang(i+3)=yksi(i)/w(i)
            end if
!
!       *  Fill array xgaus
!
         do 240 j=1,3
            xgaus(j)=0d0
            do 230 i=1,3
230            xgaus(j)=xgaus(j)+x(i)*phi((i-1)*3+j)
            xgaus(j+3)=0d0
            do 240 i=1,3
240            xgaus(j+3)=xgaus(j+3)+x(i+3)*phi((i-1)*3+j)
      end if
      if(deriv) then
!
!       *  Compute derivatives
!
         do 250 i=1,3
         do 250 j=1,3
250         phis((i-1)*3+j)=phider((i-1)*3+j)/w(j)
      end if
!
!   *  Compute weights
!
      do 280 i=1,3
280      w(i)=w(i)*wh(i)
      if(axisym) then
!
!        * axisymmetric co-ordinates  correct weights
!
         if(gauss) then
            do 300 i=1,3
300            w(i)=w(i)*xgaus(i)*2d0*pi
         else
            do 310 i=1,3
310            w(i)=w(i)*x(i)*2d0*pi
         end if
      end if
      call erclos( 'elp121' )
      end
