      subroutine elp210 ( ichois, coor, sign, x, y, w, phix, phiy, &
                          index1 )
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 3.0    date   26-12-1994 Removal of naam
!        version 2.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 2.0    date   04-04-1986
!        version 3.0    date   13-02-1988 Factor 2 pi for axisymmetric co-ord.
!
!
!
!   copyright (c) 1982-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     basis functions linear triangle
!
! ********************************************************************
!
!                      KEYWORDS
!
!    2d
!    basis_function
! ********************************************************************
!
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
      integer index1(*), ichois
      double precision coor(*), x(*), y(*), w(*), phix(*), phiy(*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ichois  i    Choice parameter
!                  Possibilities:
!                  0:  coordinates and weights for the integration are stored
!                      cartesian coordinates (Newton-cotes rule)
!                  1:  coordinates and weights for the integration are stored
!                      values of the derivatives of the basis functions in
!                      the nodal points are computed (Newton-cotes rule)
!                  2:  see 0, however for axisymmetric coordinates
!                  3:  see 1, however for axisymmetric coordinates
!                  4:  coordinates and weights for the integration are stored
!                      cartesian coordinates (one-point gauss rule)
!                  5:  coordinates and weights for the integration are stored
!                      values of the derivatives of the basis functions in
!                      the nodal points are computed (one-point gauss rule)
!                 10:  compute the derivatives in the arrays phix and phiy
!                      arrays x and y must be filled
!                 11:  compute the values of the basis functions in a point
!                      xp,yp and store these values in array w
!                      the arrays x,y,phix and phiy must have been filled before
!                      the co-ordinates xp,yp must have been filled in
!                      coor(1), coor(2)
!                 12:  compute the values of the basis functions in a point
!                      xp,yp and store these values in array w
!                      fill the arrays phix and phiy
!                      the arrays x and y must have been filled before
!                      the co-ordinates xp,yp must have been filled in
!                      coor(1), coor(2)
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     phix    o    Array of length n x m x ndim  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = phix (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = phix (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = phix (i,k,3);
!                       i        k
!                  If the element is a linear tetrahedron (n=4), then the
!                  derivatives are constant and only k = 1 is filled.
!     phiy    i    ?
!     sign    o    sign of the jacobian
!     w       o    array of length m containing the weights for integration
!     x       o    x co-ordinates of point
!     y       o    y co-ordinates of point
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, ih
      double precision h, e1(3), e2(3), delta, rc, sign, d, dx, dy

!     d
!     delta   jacobian of element
!     dx      length of vector in x-direction
!     dy      length of vector in y-direction
!     e1      factors e i,j  diktaat num. analyse c2/b3
!     e2      factors e i,j  diktaat num. analyse c2/b3
!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     ih      Help variable to store a constant temporarily
!     rc      r(1)+r(2)+r(3)
! ********************************************************************
!
!                      SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! ********************************************************************
!
!                      I/O
!
!    none
! ********************************************************************
!
!                      ERROR MESSAGES
!
!     105   Jacobian of element is zero
! ********************************************************************
!
!                      PSEUDO CODE
!
!   trivial
! ======================================================================
!
      if ( ichois.ge.10 ) then

!     --- ichois >= 10  special situation

         if ( ichois.ne.11 ) then

!        --- ichois = 10,12  compute the derivatives

            e1(1) = y(2)-y(3)
            e1(3) = y(1)-y(2)
            e2(1) = x(3)-x(2)
            e2(3) = x(2)-x(1)
            d = 1d0/(e1(3)*e2(1)-e1(1)*e2(3))
            phix(1) = e1(1)*d
            phix(3) = e1(3)*d
            phix(2) = -phix(1)-phix(3)
            phiy(1) = e2(1)*d
            phiy(3) = e2(3)*d
            phiy(2) = -phiy(1)-phiy(3)

         end if
         if ( ichois.eq.10 ) return

!        --- ichois = 11,12  compute lambda i (xp,yp)

         dx = coor(1)-x(3)
         dy = coor(2)-y(3)
         w(1) = phix(1)*dx+phiy(1)*dy
         w(2) = phix(2)*dx+phiy(2)*dy
         w(3) = 1d0-w(1)-w(2)
         return

      end if

!        --- compute  e1,e2 and delta

      do 310 i = 1, 3
         ih = 2*index1(i)
         x(i) = coor(ih-1)
         y(i) = coor(ih)
310   continue
      e1(1) = y(2)-y(3)
      e1(3) = y(1)-y(2)
      e2(1) = x(3)-x(2)
      e2(3) = x(2)-x(1)
      delta = e1(3)*e2(1)-e1(1)*e2(3)

      if ( delta) 330,320,340

!     --- error 105:  delta = 0

320      call eropen ( 'elp210' )
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         call erclos ( 'elp210' )
         return

!     --- delta < 0

330      sign = -1
         goto 350

!     --- delta > 0

340      sign = 1

!     --- fill weights

350   if ( ichois.eq.0 .or. ichois.eq.1 ) then

!     --- ichois = 0 or 1  cartesian coordinates

         h = abs(delta)/6d0
         w(1) = h
         w(2) = h
         w(3) = h
         if ( ichois.eq.0) return
      else if ( ichois.eq.2 .or. ichois.eq.3 ) then

!     --- ichois = 2 or 3  axi-symmetric coordinates

         rc = x(1)+x(2)+x(3)
         h = abs(delta)/12d0*pi
         w(1) = h*(x(1)+rc)
         w(2) = h*(x(2)+rc)
         w(3) = h*(x(3)+rc)
         if ( ichois.eq.2 ) return

      else if ( ichois.eq.4 .or. ichois.eq.5 ) then

!     --- ichois = 4 or 5  cartesian coordinates  1 point rule

         h = abs(delta)/2d0
         w(1) = h
         h = 1d0/3d0
         x(1) = h*(x(1)+x(2)+x(3))
         y(1) = h*(y(1)+y(2)+y(3))
         if ( ichois.eq.4 ) return

      end if

!     --- ichois = 1 ,3 or 5   derivatives

      d = 1d0/delta
      phix(1) = e1(1)*d
      phix(3) = e1(3)*d
      phix(2) = -phix(1)-phix(3)
      phiy(1) = e2(1)*d
      phiy(3) = e2(3)*d
      phiy(2) = -phiy(1)-phiy(3)
      end
