      subroutine elp212(jchois,coor,bsign,name,x,y,w,phix,phiy,index1, &
                        work)
      use sepmodulecactl
      use sepmoduleconsts
      implicit double precision (a-h,o-z)
      character *(*) name
      integer index1(*)
      double precision coor(*), x(*), y(*), w(*), phix(*), phiy(*), &
             work(*), phiksi(36), phieta(36)
      save  jweigh,itcoor,ivertx,ichois,phiksi,phieta
      data phiksi / -3d0, -1d0,  1d0,  1d0,  1d0, -1d0, &
                     4d0,  0d0, -4d0, -2d0,  0d0,  2d0, &
                    -1d0,  1d0,  3d0,  1d0, -1d0, -1d0, &
                     0d0,  0d0,  0d0,  2d0,  4d0,  2d0, &
                     0d0,  0d0,  0d0,  0d0,  0d0,  0d0, &
                     0d0,  0d0,  0d0, -2d0, -4d0, -2d0/
      data phieta / -3d0, -1d0,  1d0,  1d0,  1d0, -1d0, &
                     0d0, -2d0, -4d0, -2d0,  0d0,  0d0, &
                     0d0,  0d0,  0d0,  0d0,  0d0,  0d0, &
                     0d0,  2d0,  4d0,  2d0,  0d0,  0d0, &
                    -1d0, -1d0, -1d0,  1d0,  3d0,  1d0, &
                     4d0,  2d0,  0d0, -2d0, -4d0,  0d0/
!
!
! ********************************************************************
!
!
!     basis functions quadratic triangle
!
! ********************************************************************
!
!
!        programmer   Guus Segal
!        version 5.2  date   21-12-1991 Save common, phiksi,..; JvdZ
!        version 5.1  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 5    date   09-09-1987
!        version 6    date   18-11-1987  Correction for ichois=4, curved mesh
!        version 7    date   13-02-1988 Factor 2 pi for axisymmetric co-ord.
!
!
!
!   copyright (c) 1982,1991  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!
!       input and output parameters
!
!      jchois   Choice parameter. jchois = ichois + 10 * ivertx
!               with:
!      ichois   choice parameter, possibilities:
!
!             0:  coordinates and weights for the integration are stored
!                 cartesian coordinates
!
!             1:  coordinates and weights for the integration are stored
!                 values of the derivatives of the basis functions in
!                 the nodal points are computed
!
!             2:  see 0, however for axisymmetric coordinates
!
!             3:  see 1, however for axisymmetric coordinates
!
!             4:  co-ordinates are stored as well as derivatives in vertices
!
!     ivertx     1:  Derivatives of basis functions are also computed in
!     vertex
!
!     coor        array containing the coordinates
!
!     bsign       sign of the jacobian
!
!     name        name of calling subroutine (literal constant)
!
!     x,y         arrays containing the coordinates of the nodal points
!
!     w           array containing the weights for the numerical integration
!
!     phix,phiy   arrays containing the values of the derivatives of the
!                 basis functions ( constant in element )
!
!     index1      see to0000
!
!     work        work array of length 30
!
! ********************************************************************
!
!    *local parameters
!
!    work      array work is filled as follows:
!
!              cartesian co-ordinates:
!
!              pos. 1-3   e1      d phi/ dx  *  delta
!                   4-6   e2      d phi/ dy  *  delta
!                   7-9   e3      d phi/ dx
!                   10-12  e4     d phi/ dy
!
!             axi-symmetric co-ordinates
!
!             pos.   1- 6  dx / dxi
!             pos.   7-12  dx / deta
!             pos.  13-18  dy / dxi
!             pos.  19-24  dy / deta
!             pos.  25-30  jacobian
!
!    delta    jacobian of element
!
!    phieta   phieta(i,j) = dphi (i) / deta (xj)    (i,j)=(i-1)*6+j
!
!    jweigh   =0  weights must not be computed
!             =1  weights must be computed
!
!    itcoor   =0  cartesian co-ordinates
!             =1  axi-symmetric co-ordinates
!
!    jderiv   =0  derivatives must not be computed
!             =1  derivatives must be computed
!
!    jcurve   =0  element is straight
!             =1  element is curved
!
! ********************************************************************
!
!
      if (ifirst.eq.0) then
         ivertx = jchois/10
         ichois = jchois - 10 * ivertx
         if(ichois.eq.4) then
            jweigh=0
            jderiv=1
            ivertx = 1
         else
            jweigh=1
            if(ichois.le.1) then
               itcoor=0
            else
               itcoor=1
            end if
         end if
         if(ichois.eq.0 .or. ichois.eq.2) then
            jderiv=0
         else
            jderiv=1
         end if
      end if
!
!    *compute  co-ordinates e1,e2,e3,e4 and delta
!     check whether element is straight or curved
!
      call elp900(coor,x,y,index1,jcurve,6,work,work(4),delta,bsign, &
                  work(7),work(10),name,ichois)
!
      if(jcurve.eq.0 .or. ichois.eq.4) then
!
!     * element is straight, computation is cheaper
!
         if(jweigh.eq.1) then
!
!         *  jweigh =1   fill weights
!
            if(itcoor.eq.0) then
!
!            * itcoor = 0  cartesian co-ordinates
!
               h=delta/6d0
               do 100 i=1,5,2
                  w(i)=0d0
100               w(i+1)=h
            else
!
!            * itcoor = 1  axi-symmetric co-ordinates
!
               h=delta/60d0*pi
               h1=x(1)+x(3)+x(5)
               do 110 i=1,5,2
110               w(i)=h*(3d0*x(i)-h1)
               h1=2d0*h1
               h=4d0*h
               w(2)=h*(h1-x(5))
               w(4)=h*(h1-x(1))
               w(6)=h*(h1-x(3))
            end if
         end if
!
         if(jderiv.eq.1) then
!
!          *  jderiv = 1   fill derivatives
!
            jtcoor = itcoor
            if ( ivertx.eq.1 ) itcoor = 1
            call elp950(work(7),phix,itcoor)
            call elp950(work(10),phiy,itcoor)
            itcoor = jtcoor
         end if
!
      else
!
!       *  triangle is curved
!          store dx/dxi, dx/deta, dy/dxi, dydeta in work
!
         if(ivertx.eq.0 .and. itcoor.eq.0) then
!
!         *  cartesian co-ordinates
!
            kbot=2
            kstep=2
         else
!
!         *  axi-symmetric co-ordinates
!
            kbot=1
            kstep=1
         end if
!
         do 250 k=kbot,6,kstep
            sum=0d0
            do 200 j=1,6
200            sum=sum+x(j)*phiksi((j-1)*6+k)
            work(k)=sum
            sum=0d0
            do 210 j=1,6
210            sum=sum+x(j)*phieta((j-1)*6+k)
            work(k+6)=sum
            sum=0d0
            do 220 j=1,6
220            sum=sum+y(j)*phiksi((j-1)*6+k)
            work(k+12)=sum
            sum=0d0
            do 230 j=1,6
230            sum=sum+y(j)*phieta((j-1)*6+k)
            work(k+18)=sum
250      continue
!
!      *  fill jacobian
!
         do 260 k=kbot,6,kstep
260         work(k+24)=work(k)*work(k+18)-work(k+12)*work(k+6)
!
!        * check sign of jacobian
!
         ksign=sign(1d0,work(26))
         do 270 k=kbot,6,kstep
            jsign=sign(1d0,work(k+24))
            if(ksign.ne.jsign) call ertrap(name,230,15,itype,ielem,0)
270      continue
         bsign=ksign
!
!      *  fill weights
!
         if(itcoor.eq.0) then
!
!          * cartesian co-ordinates
!
            h=bsign/6d0
            do 280 i=1,5,2
               w(i)=0d0
280            w(i+1)=h*work(25+i)
         else
!
!          * axi-symmetric co-ordinates
!
            h=bsign/60d0*pi
            h1=x(1)+x(3)+x(5)
            do 290 i=1,5,2
290            w(i)=h*(3d0*x(i)-h1)*work(24+i)
            h1=2d0*h1
            h=4d0*h
            w(2)=h*(h1-x(5))*work(26)
            w(4)=h*(h1-x(1))*work(28)
            w(6)=h*(h1-x(3))*work(30)
         end if
!
         if(jderiv.eq.1) then
!
!         *  fill derivatives
!
            do 300 i=1,6
               ih=(i-1)*6
               do 300 k=kbot,6,kstep
                  phix(ih+k)=(phiksi(ih+k)*work(18+k)- &
                        phieta(ih+k)*work(12+k))/work(24+k)
                  phiy(ih+k)=(-phiksi(ih+k)*work(6+k)+ &
                        phieta(ih+k)*work(k))/work(24+k)
300         continue
         end if
      end if
      end
