      subroutine elp620jacob ( e, x, delta, isubr, index1, &
                               ielem, ielgrp, itype )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-06-2013 Change accuracy
!        version  1.0  date 02-02-2006
!
!   copyright (c) 2006-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian of a triangle
!     Also the factors ^ij, necessary for the derivatives are computed
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     determinant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, index1(3), ielem, ielgrp, itype
      double precision e(3,2), x(3,2), delta

!     delta          o    jacobian of element
!     e              o    array of length 3 x 2, to store the factors e^ij
!     ielem          i    Element number
!     ielgrp         i    element group number
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     isubr          i    Indicates the type of calling subroutine
!     itype          i    Type number of equation
!     x              i    array with the coodinates of the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision h

!     h              Help variable to store a constant temporarily
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp620jacob' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp620jacob'
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      do k = 1, 2
         e(1,k) = x(3,k) - x(2,k)
         e(3,k) = x(2,k) - x(1,k)
      end do
      h = max(abs(e(1,1)),abs(e(1,2)),abs(e(3,1)),abs(e(3,2)))
      h = h**2*1d-7

      if ( debug ) then

!     --- Debug information

         write(irefwr,110) 'e1 ', e(1,1), e(1,2)
         write(irefwr,110) 'e3 ', e(3,1), e(3,2)
         write(irefwr,110) 'h', h

      end if   ! ( debug )

      e(1,2) = -e(1,2)
      e(3,2) = -e(3,2)
      delta = e(3,2)*e(1,1) - e(1,2)*e(3,1)
      if ( checksign .and. delta<h ) then
         write(irefwr,*) 'area of element ', ielem, ' is negative '
         write(irefwr,*) 'area = ', delta
         call prinin ( index1, 3,  'nodes' )
         call prinrl1 ( x, 2, 3, 'coordinates' )

      end if

!     --- Check if delta is non-zero

      if ( abs(delta)<=h ) then

!     --- delta<=0

         if ( isubr==0 ) then
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
         else
            call errint ( ielgrp, 1 )
            call errint ( ielem, 2 )
            call errchr ( 'surface', 1 )
            call errsub ( 1540, 2, 0, 1 )
         end if
         call prinin(index1,3,'node numbers')
         call prinrl2(x,3,2,'co-ordinates')
         go to 1000
      end if

1000  call erclos ( 'elp620jacob' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp620jacob'

      end if

      end
