      subroutine elp622first ( intrule, w, dpdxi, iweigo, phi, &
                               psi, comppsi, weight, lambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 22-05-2014 Use elp622first1
!        version  2.0  date 10-06-2013 Remove parameters
!        version  1.0  date 27-02-2009
!
!   copyright (c) 2009-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a quadratic triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intrule, iweigo
      double precision w(m), weight(m), &
                       dpdxi(6,m,2), phi(6,m), psi(3,m), lambda(3,m)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     dpdxi          o    contains derivatives in reference element for gauss
!                         rule.
!                         dpdxi(i,k,l) = d phi_i  / d xsi_l (x^k)
!     intrule        i    Actual integration rule
!     iweigo         i    if 1 the weights must be filled
!     lambda         o    Array containing the linear basis functions in the
!                        integration points
!     phi            o    array of length 6 x m containing the values of the
!                         basis functions in the gauss points in the sequence:
!                                k
!                         phi (xg ) = phi (i,k)
!                            i
!                         array phi is only filled when ifirst = 0 and gauss
!                         integration is applied
!     psi            o    array of length 3 x m containing the values of the
!                         linear basis functions in the gauss points
!     w              i    array of length m containing the weights for
!                         integration
!     weight         o    Contains the weights for the integration rule in the
!                         reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=18) text

!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP622FIRST1   Fill some element independent quantities for a quadratic
!                    triangle
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Quadratic triangle' /
! ======================================================================
!
      call eropen ( 'elp622first' )
      debug = .false. .and. ioutp>= 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp622first'

      end if  ! ( debug )
      if ( ierror/= 0 ) go to 1000

      if ( iweigo/= 0 ) w = 0d0
      if ( jcart<1 .or. jcart>3 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>3 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( intrule==1 .and. m/=6 .or. &
           intrule==2 .and. m/=4 .or. &
           intrule==3 .and. m/=7 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

      call elp622first1 ( intrule, dpdxi, phi, weight, lambda, m )

      if ( comppsi ) then

!     --- comppsi true, hence fill array psi

         psi(1:3,1:m) = lambda(1:3,1:m)

      end if  ! ( comppsi )

1000  call erclos ( 'elp622first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp622first'

      end if  ! ( debug )

      end
