      subroutine elp646 ( coor, index1, x1, x2, x3, d1, det, p1, p2, &
                          p3, p4, p5, p6, p7, p8, p9, mone )
! ======================================================================
!
!        programmer    Peter Minev
!        version  1.2  date 24-05-2001 Layout
!        version  1.1  date 28-01-1996 Layout
!        version  1.0  date 28-09-1994
!
!   copyright (c) 1994-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     store global integration points
!     compute transformation determinant and partial derivatives  in 3-D
!     ELP646 is called by the element subroutine ELM674
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), mone
      double precision coor(3,*),x1(mone,mone,mone),x2(mone,mone,mone), &
                       x3(mone,mone,mone), d1(mone,mone), &
                       det(mone,mone,mone), p1(mone,mone,mone), &
                       p2(mone,mone,mone), p3(mone,mone,mone), &
                       p4(mone,mone,mone), p5(mone,mone,mone), &
                       p6(mone,mone,mone), p7(mone,mone,mone), &
                       p8(mone,mone,mone), p9(mone,mone,mone)

!     coor      i   array of length npoint containing the coordinates
!                   of the nodal points with respect to the global
!                   nummering
!                       x  = coor(1,i); y  = coor(2,i), z = coor(3,i)
!                        i               i
!     d1        i   array of length mone x mone contaning the values of the
!                   derivatives of the basisfunctions in the x-direction
!     det       o   array of length m containing the transformation
!                   determinant
!     index1    i   array of length inpelm containing the point numbers
!                   of the nodal points in the element
!     mone      i   number of intregation points in one dimension
!     p1        o   array of length mone x mone containing the partial
!                   derivatives of the coordinates due to transformation
!     p2        o   See p1
!     p3        o   See p1
!     p4        o   See p1
!     p5        o   See p1
!     p6        o   See p1
!     p7        o   See p1
!     p8        o   See p1
!     p9        o   See p1
!     x1        o   array containing the coordinates of the integration
!                   nummering, length m x 3
!                   xl(i,j,k) = coor(l,index1(i,j,k))
!     x2        o   See x1
!     x3        o   See x1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, ijk, mtwo
      double precision h1, h2, h3, h4, h5, h6, h7, h8, h9, swap

!     h1         Help variable
!     h2         Help variable
!     h3         Help variable
!     h4         Help variable
!     h5         Help variable
!     h6         Help variable
!     h7         Help variable
!     h8         Help variable
!     h9         Help variable
!     i          Counting variable
!     ijk        Help variable
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     mtwo
!     swap       swap parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  print error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1603   transformation determinant equal to zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elp646')
      mtwo = mone*mone

!     --- store global integration points

      do i = 1, mone
         do j = 1, mone
            do k = 1, mone
               ijk = i + (j-1)*mone + (k-1)*mtwo
               x1(i,j,k) = coor(1, index1(ijk))
               x2(i,j,k) = coor(2, index1(ijk))
               x3(i,j,k) = coor(3, index1(ijk))
            end do
         end do
      end do
      if (mone.eq.2) then

!     --- mone = 2, different nodal numbering

         swap = x1(1,2,1)
         x1(1,2,1) = x1(2,2,1)
         x1(2,2,1) = swap
         swap = x2(1,2,1)
         x2(1,2,1) = x2(2,2,1)
         x2(2,2,1) = swap
         swap = x3(1,2,1)
         x3(1,2,1) = x3(2,2,1)
         x3(2,2,1) = swap
         swap = x1(1,2,2)
         x1(1,2,2) = x1(2,2,2)
         x1(2,2,2) = swap
         swap = x2(1,2,2)
         x2(1,2,2) = x2(2,2,2)
         x2(2,2,2) = swap
         swap = x3(1,2,2)
         x3(1,2,2) = x3(2,2,2)
         x3(2,2,2) = swap
      end if

!     --- compute transformation determinant
!         first compute partial derivatives

      do i = 1, mone
         do j = 1, mone
            do k = 1, mone

!           --- ksi derivatives

               h1 = 0d0
               h2 = 0d0
               h3 = 0d0
               do l = 1, mone
                  h1 = h1 + x1(l,j,k)*d1(i,l)
                  h2 = h2 + x2(l,j,k)*d1(i,l)
                  h3 = h3 + x3(l,j,k)*d1(i,l)
               end do
               p1(i,j,k) = h1
               p2(i,j,k) = h2
               p3(i,j,k) = h3

!              --- eta derivatives

               h1 = 0d0
               h2 = 0d0
               h3 = 0d0
               do l = 1, mone
                  h1 = h1 + x1(i,l,k)*d1(j,l)
                  h2 = h2 + x2(i,l,k)*d1(j,l)
                  h3 = h3 + x3(i,l,k)*d1(j,l)
               end do
               p4(i,j,k) = h1
               p5(i,j,k) = h2
               p6(i,j,k) = h3

!              --- dzeta derivatives

               h1 = 0d0
               h2 = 0d0
               h3 = 0d0
               do l = 1, mone
                  h1 = h1 + x1(i,j,l)*d1(k,l)
                  h2 = h2 + x2(i,j,l)*d1(k,l)
                  h3 = h3 + x3(i,j,l)*d1(k,l)
               end do
               p7(i,j,k) = h1
               p8(i,j,k) = h2
               p9(i,j,k) = h3

!              --- then the determinant

               det(i,j,k) = p1(i,j,k)*p5(i,j,k)*p9(i,j,k) + &
                            p2(i,j,k)*p6(i,j,k)*p7(i,j,k) + &
                            p4(i,j,k)*p8(i,j,k)*p3(i,j,k) - &
                            p3(i,j,k)*p5(i,j,k)*p7(i,j,k) - &
                            p2(i,j,k)*p4(i,j,k)*p9(i,j,k) - &
                            p6(i,j,k)*p8(i,j,k)*p1(i,j,k)

!              --- check if determinant equal to zero

               if ( det(i,j,k).lt.1d-12 ) then
                  call eropen ('elp646')
                  call errsub ( 1603, 0, 0, 0)
                  call erclos ('elp646')
                  go to 1000
               end if

!              --- loading in p1-p9 the inversed of the jacobian

               h1 = p1(i,j,k)
               h2 = p2(i,j,k)
               h3 = p3(i,j,k)
               h4 = p4(i,j,k)
               h5 = p5(i,j,k)
               h6 = p6(i,j,k)
               h7 = p7(i,j,k)
               h8 = p8(i,j,k)
               h9 = p9(i,j,k)
               p1(i,j,k) = h5*h9 - h6*h8
               p2(i,j,k) = -h4*h9 + h7*h6
               p3(i,j,k) = h4*h8 - h5*h7
               p4(i,j,k) = -h2*h9 + h3*h8
               p5(i,j,k) = h1*h9 - h3*h7
               p6(i,j,k) = -h1*h8 + h2*h7
               p7(i,j,k) = h2*h6 - h3*h5
               p8(i,j,k) = -h1*h6 + h3*h4
               p9(i,j,k) = h1*h5 - h2*h4
            end do
         end do
      end do
1000  call erclos ('elp646')
      end
