      subroutine elp647( coor, index1, x, trans, m)
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   29-03-1995
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     store integration points
!     transform weights and derivatives from the reference element
!
!     ELP647 is called by the subroutine ELM607
!***********************************************************************
!
!                      KEYWORDS
!
!     spectral
!***********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer index1(*), m
      double precision coor(*), x(m), trans

!     coor     i     array of length npoint containing the co-ordinates of
!                    the nodal points with respect to the global numbering
!                    x  = coor (1,i)
!                     i
!     index1   i     array containing the point numbers of the nodal points
!                    in the elements, length inpelm
!     m        i     number of integration points
!     trans    o     the jacobian of the transform
!     x        o     array containing the coordinates of the integration
!                    points in the element, length m x 1
!                    x  = x(i)  i = 1(1)m
!                     i
!***********************************************************************
!
!                   COMMON BLOCKS
!
!***********************************************************************
!
!                   LOCAL PARAMETERS

      integer i

!     i          counting variable
!***********************************************************************
!
!                 SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRSUB  print error messages
!***********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     1603   transformation determinant equal to zero
!***********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- compute kchois
!         store coordinates of the nodal points in array x

      do 100 i = 1, m

         x(i) = coor(index1(i))
100   continue

!     --- compute transformation Jacobian

      trans = (x(m) - x(1))/2d0

!     --- check value of Jacobian

      if (trans.lt.1d-10) then

         call eropen ( 'elp647' )
         call errsub ( 1603, 0, 0, 0)
         call erclos ( 'elp647' )

      end if

      end
