      subroutine elpcoor ( icoor, coor, index1, x, n, ndim, nx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill co-ordinates of nodes in array x
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nx, ndim, icoor, index1(n)
      double precision coor(ndim,*), x(nx,ndim)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icoor          i    Type of co-ordinate system
!                         1: Cartesian
!                         2: Cylindrical
!                         3: Polar
!     index1         i    Array of length n containing the point numbers
!                         of the nodal points in the element
!     n              i    number of nodes in element
!     ndim           i    Dimension of the space.
!     nx             i    first dimension of array x
!     x              o    Contains the coordinates of the nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpcoor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpcoor'
         write(irefwr,1) 'icoor, n, ndim', icoor, n, ndim
         call prinin ( index1, n, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( mod(icoor,10)==2 ) then

         do k = 1, ndim
            x(1:n,k) = coor(k,1:n)
         end do  ! k = 1, ndim

      else

         do k = 1, ndim
            x(1:n,k) = coor(k,index1(1:n))
         end do  ! k = 1, ndim

      end if  ! ( mod(icoor,10)==2 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( x, n, ndim, 'coordinates' )
         write(irefwr,*) 'End elpcoor'

      end if  ! ( debug )
      call erclos ( 'elpcoor' )

      end
