      subroutine elpgauss ( ndim, intrule, x, xgauss, phi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill Gauss coordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, intrule
      double precision x(n,ndim), phi(n,m), xgauss(m,ndim)

!     intrule        i    Actual integration rule
!     ndim           i    Dimension of the space.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     x              i    contains coordinates of vertices
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpgauss' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpgauss'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( intrule==1 ) then

!     --- irule = 1  Newton Cotes rule

         do k = 1, ndim
            xgauss(1:n,k)=x(1:n,k)
         end do

      else

!     --- intrule>1  Gauss rule

         do i = 1, ndim
            do k = 1, m
               xgauss(k,i) = dot_product ( x(:,i),phi(:,k) )
            end do  ! k = 1, m
         end do  ! i = 1, ndim

      end if  ! ( intrule==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( xgauss, m, ndim, 'Gauss coordinates' )
         write(irefwr,*) 'End elpgauss'

      end if  ! ( debug )
      call erclos ( 'elpgauss' )

      end
