      subroutine elpup3 ( u, dphidx, h, unorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 15-01-2003 Complete new subroutine
!        version  2.2  date 30-01-2002 Include commons
!        version  2.1  date 22-05-1995 Rename ctime in ctimen
!        version  2.0  date 29-01-1995 Extra parameter dcdx;
!                                      Extra possibilities:DC1-2)
!        version  1.0  date 09-09-1993
!
!   copyright (c) 1993-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute distance h for modified basis functions for upwind elements
!    Three dimensional elements (linear tetrahedrons)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision h(*), u(m,3), unorm(*), dphidx(n,m,3)

!     dphidx         i    Array of length n x m x ndim  containing the
!     derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     u              i    Array of length m x 3 containing the velocity in the
!                         integration points
!     unorm          i    Length of velocity vector in the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision phimax

!     i           Counting variable
!     phimax      Max_j( |u_i . grad phi_j| )
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       h = max distance in element in the direction of u
!                                                       - &
!       For a tetrahedron:  h_i  = ||u_i ||_j / max (|u_i . grad_j lambda|)
!       in the vertices
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, m
         phimax = max ( abs(u(i,1)*dphidx(1,1,1)+ &
                            u(i,2)*dphidx(1,1,2)+ &
                            u(i,3)*dphidx(1,1,3)), &
                        abs(u(i,1)*dphidx(2,1,1)+ &
                            u(i,2)*dphidx(2,1,2)+ &
                            u(i,3)*dphidx(2,1,3)), &
                        abs(u(i,1)*dphidx(3,1,1)+ &
                            u(i,2)*dphidx(3,1,2)+ &
                            u(i,3)*dphidx(3,1,3)), &
                        abs(u(i,1)*dphidx(4,1,1)+ &
                            u(i,2)*dphidx(4,1,2)+ &
                            u(i,3)*dphidx(4,1,3)) )
         if ( phimax.gt.epsmac ) then
            h(i) = unorm(i)/phimax
         else
            h(i) = 0d0
         end if
      end do

      end
