      subroutine elpup31 ( u, h, unorm, dlamdx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 20-06-2013 New call to elp631
!        version  2.0  date 04-07-2003 Extra parameter dlamdx
!        version  1.0  date 10-01-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute distance h for modified basis functions for upwind elements
!    Three dimensional elements (quadratic tetrahedrons)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision h(*), u(m,3), unorm(*), dlamdx(4,3)

!     dlamdx         o    Contains derivatives of linear basis functions
!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     u              i    Array of length m x 3 containing the velocity in the
!                         integration points
!     unorm          i    Length of velocity vector in the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision phimax

!     i              Counting variable
!     phimax         Max_j( |u_i . grad phi_j| )
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       h = max distance in element in the direction of u
!                                                       - &
!       For a tetrahedron:  h_i  = ||u_i ||_j / max (|u_i . grad_j lambda|)
!       in the vertices
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpup31' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpup31'
         call prinrl ( unorm, m, 'unorm' )
         call prinrl1 ( dlamdx, 3, 4, 'dlamdx' )

      end if

      do i = 1, m
         phimax = max(abs(u(i,1)*dlamdx(1,1)+ &
                          u(i,2)*dlamdx(1,2)+ &
                          u(i,3)*dlamdx(1,3)), &
                      abs(u(i,1)*dlamdx(2,1)+ &
                          u(i,2)*dlamdx(2,2)+ &
                          u(i,3)*dlamdx(2,3)), &
                      abs(u(i,1)*dlamdx(3,1)+ &
                          u(i,2)*dlamdx(3,2)+ &
                          u(i,3)*dlamdx(3,3)), &
                      abs(u(i,1)*dlamdx(4,1)+ &
                          u(i,2)*dlamdx(4,2)+ &
                          u(i,3)*dlamdx(4,3)) )
         if ( phimax>epsmac ) then
            h(i) = unorm(i)/phimax
         else
            h(i) = 0d0
         end if
      end do

      call erclos ( 'elpup31' )
      if ( debug ) then
         call prinrl ( h, m, 'h' )
         write(irefwr,*) 'End elpup31'
      end if

      end
