      subroutine sepadaprule250
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt integration rule in case of elements for linear elasticity
!     See old subroutine el0250
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     numerical_integration
!! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: massparm, mrule, igprob
      logical :: plane

!     igprob         E%stress_strain+1
!     massparm       Indicates the largest value of massrule during the
!                    time integration
!     mrule          Type of integration rule as given by the user
!     plane          If true we are dealing with plane stress
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepadaprule250' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepadaprule250'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      massparm = max(E%imas,massrule)
      mrule = B%rule
      igprob = E%stress_strain+1
      plane = igprob==1 .or. igprob==2 .or. igprob==4.or. igprob==5 .or. &
              igprob==7
      if ( debug ) then
        write(irefwr,1) 'mrule, ishape, n, ndim', B%rule, E%ishape, B%n, E%ndim
        write(irefwr,1) 'massrule', massrule
      end if  ! ( debug )

      if ( mrule==0 ) then

!     --- Default integration rule

         B%rule = 3
         select case (E%ishape)
            case(3, 4, 7)
               if ( plane ) B%rule = 2
            case(5)
               B%rule = 3
            case(11, 12, 21)
               B%rule = 1
         end select  ! case (E%ishape)

      end if  ! ( mrule==0 )

      if ( debug ) write(irefwr,1) 'B%rule/1', B%rule

!     --- Adapt irule in case of mass matrix in order to get
!         a positive definite mass matrix

      if ( massparm==2 .and. mrule==0 ) then

!     --- Non-diagonal mass matrix
!         Integration rule must be at least 3 to avoid non-positive
!         definite mass matrices

         if ( E%ishape==2 ) then

!        --- Quadratic 1d, use irule = 4

            B%rule = 4

         else

!        --- Other cases, use irule = 3

            B%rule = max ( B%rule, 3 )

         end if

      end if  ! ( massparm==2 .and. mrule==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'B%rule', B%rule
         write(irefwr,*) 'End sepadaprule250'

      end if  ! ( debug )
      call erclos ( 'sepadaprule250' )

      end subroutine sepadaprule250
