      subroutine sepel0150cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients for elements of type 150
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     coefficient
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPELALLOCCCF  Allocate complex coeffs and related arrays for element
!     SEPELCTYPECONV Fill quantities with respect to convection (complex)
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPELTYPCRHOCP Fill information about density rho and rho cp (complex)
!     SEPELTYPECDIFF Compute typediff in case typediff = 0 (complex)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0150cf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0150cf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%indvel = 7
      E%indbeta = E%indvel+3
      E%indsource = E%indbeta+1
      E%indrho = E%indsource+1

      if ( E%itype==800 ) then
         E%indcp = 18              ! temporary
         E%indgamma = E%indrho+1
      else
         E%indcp = E%indrho+1
         E%indgamma = E%indrho+2
      end if  ! ( itype==800 ) then
      E%indrhocp = E%indgamma+3

      if ( debug ) &
         write(irefwr,1) 'indvel, indbeta, indsource, indrho, indgamma', &
                          E%indvel, E%indbeta, E%indsource, E%indrho, E%indgamma

      call sepelallocccf ( 3 ) ! allocate coefficient arrays (complex)

!        --- Define rho cp

      if ( E%ncoef>6 ) call sepeltypcrhocp

      do i = 1, E%ncoef
         E%ccoeffsnode(1:B%n,i) = E%ccoeffs(1,i)
      end do  ! i = 1, E%ncoef

!     --- Skip three coefficients

      E%ipos = E%ipos+3
      E%ishift = E%ishift+3
      if ( debug ) write(irefwr,1) 'E%ipos/2', E%ipos

!     --- Fill extra information concerning diffusion and convection

      if ( E%ncoef>1 ) call sepeltypecdiff
      if ( E%ncoef>6 ) call sepelctypeconv

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

      if ( E%ncoef>6 ) then

!     --- More than 6 real coefficients

         E%cgamma => E%ccoeffs(:,E%indgamma:E%indgamma+2)

         if ( E%ndim<2 ) E%ind(E%indgamma+1) = 0
         if ( E%ndim<3 ) E%ind(E%indgamma+2) = 0
         E%gammaterm = E%ind(E%indgamma)/=0 .or. &
                       E%ind(E%indgamma+1)/=0 .or. &
                       E%ind(E%indgamma+2)/=0
         E%sourceterm = E%ind(E%indsource)/=0
         E%zeroterm =  E%ind(E%indbeta)/=0

      end if  ! ( E%ncoef>6 ) then

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ich, E%ncoef, 'ich' )
         call prinin ( E%ind, E%ncoef, 'ind' )
         write(irefwr,*) 'End sepel0150cf'

      end if  ! ( debug )
      call erclos ( 'sepel0150cf' )

      end subroutine sepel0150cf
