      subroutine sepel0250
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Performs initialization for subroutine sepelm250
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     PRININ         print 1d integer array
!     SEPADAPRULE250 Adapt integration rule in case of elements for
!                    linear elasticity
!     SEPEL0250CF    Fill real coefficients for elements of type 250
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETRULE     Get the integration rule as stored in iuser
!     SEPGETSTRESSRL Read stress-strain relations and store information in E
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0250' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0250'
         call prinin ( E%iuser, 10, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Elements with extra points are treated as elements without

      if ( E%ishape==7 ) E%ishape=4
      if ( E%ishape==17 ) E%ishape=13

!     --- initialize ipos and some other quantities

      call sepeliposset
      E%ipos = E%ipos+1          ! skip first position
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get information of coefficients

      call sepgetstressrl  ! stress-strain relation
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      if ( ierror/=0 ) go to 1000

      B%cart = 1
      if ( E%stress_strain==2 .or. E%stress_strain==5 .or. &
           E%stress_strain==7 ) B%cart = 2

!     --- ioption(3) is passed to fnmateri as a choice variable

      E%choicefnmater = inirepscal ( E%iuser(E%ipos) )
      E%ipos = E%ipos+1          ! skip one position
      E%ishift = E%ishift+1

      E%special = inirepscal ( E%iuser(E%ipos) )
      E%ipos = E%ipos+1          ! skip one position
      E%ishift = E%ishift+1
      E%plane = E%ndim==2 .and. B%cart==1

!     --- Find size of D matrix

      if ( E%ndim==3 ) then
         E%dimD = 6
      else if ( B%cart==1 ) then
         E%dimD = 3
      else
         E%dimD = 4
      end if  ! ( E%ndim==3 )

!     --- Adapt integration rule for specific cases

      call sepadaprule250

!     --- Get information of basis functions

      call sepelbasefnch
      if ( ierror/=0 ) go to 1000
      n = B%n
      m = B%m
      E%b2filled = .false.

!     --- Allocate D-matrix and B-matrix

      allocate ( E%Dmat(E%dimD,E%dimD,m), E%Bmat(4,2,n,m), E%wr(n), &
                 E%detf(27), E%detfr(27), &
                 E%bl(6,3,B%n,B%n), E%blr(6,3,B%n,B%n), E%pksest(6,B%n), &
                 E%pksr(6,B%n), E%ser(6,6,B%n) )
      E%Dmat = 0d0
      E%Bmat = 0d0

!     --- Fill all real coefficients

      E%ncoef = 40
      call sepel0250cf

      if ( E%modelelastnl>0 ) then
         E%compold = max(E%compold,1)
         E%notvec = 0
         if ( indvector(iseqprevsol)==0 ) then
            indvector(iseqprevsol) = 1
            E%filloldderiv = .true.
            call sepelallocvecs
         end if  ! ( indvector(iseqprevsol)==0 )
      end if  ! ( E%modelelastnl>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0250'

      end if  ! ( debug )
      call erclos ( 'sepel0250' )

      end subroutine sepel0250
