      subroutine sepel0302
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 302
!
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     elasticity_equation
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     SEPEL0801CF    Fill real coefficients for elements of type 801
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0302' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0302'
         call prinin ( E%iuser, 10, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get information of basis functions

      B%rule = 1
      B%cart = 1
      call sepelbasefnch

      E%compold = 1
      indvector(iseqprevsol) = 1

!     --- Get information of coefficients

      E%ncoef = 5
      call sepel0801cf

      E%typelinear = inirepscal ( E%iuser(E%ipos) )
      E%tangstress = E%typelinear/10
      E%typelinear = E%typelinear-10*E%tangstress-1

      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

      E%newton = E%typelinear==1 .or. E%typelinear==3
      E%timedep = E%typelinear>1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0302'

      end if  ! ( debug )
      call erclos ( 'sepel0302' )

      end subroutine sepel0302
