      subroutine sepel0703
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 703
!     convection-diffusion equation
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELGETIQMAT  Get the parameter iqmat as stored in iuser
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPELRDTYPCONV Read information about type_convection
!                    Adapt methodupw if necessary
!     SEPELTYPECONV  Fill quantities with respect to convection
!     SEPELTYPEDIFF  Compute typediff in case typediff = 0
!     SEPELTYPERHOCP Fill information about density rho and rho cp
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0703' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0703'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

!     --- Get information of coefficients

      call sepelgetiqmat  ! iqmat
      E%methodupw = sepelgetintcf ( codeupw, nncodeupw, &
                                    ncodeupw, 1, 'supg' )

      if ( debug ) then
         write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift
         write(irefwr,1) 'methodupw', E%methodupw
      end if  ! ( debug )

!     --- Next quadrature rule and coordinate system

      call sepgetrule

      call sepgetcoorsys

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Read type_convection
!         Adapt methodupw if necessary

      call sepelrdtypconv

!     --- Fill all real coefficients

      E%ncoef = 13
      E%indvel = 7
      E%indsource = E%indvel+3
      E%indrho = E%indsource+1
      E%indcp = E%indrho+1
      E%indrhocp = E%indcp+1

      call sepelalloccf ( 3, 0 ) ! allocate coefficient arrays

!     --- Define rho cp

      call sepeltyperhocp

      do i = 1, E%ncoef
         E%coeffsnode(1:B%n,i) = E%coeffs(1,i)
      end do  ! i = 1, E%ncoef

!     --- Skip two coefficients

      E%ipos = E%ipos+2
      E%ishift = E%ishift+2

!     --- Fill extra information concerning diffusion and convection

      call sepeltypediff
      call sepeltypeconv

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

      E%sourceterm = E%ind(E%indsource)/=0

!     --- Check if derivatives of basis functions are required in nodes

      B%fillderivn = B%type_basisfunc/=3 .and. B%type_basisfunc/=10 .and. &
                     B%type_basisfunc/=1 .and. B%type_basisfunc/=11

!     --- Check if element vector is zero
!         If so set notvec to 1

      if ( .not. E%sourceterm .and. E%typeconv/=3 ) E%notvec = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'diffusion, convection', E%diffusion, E%convection
         write(irefwr,*) 'End sepel0703'

      end if  ! ( debug )
      call erclos ( 'sepel0703' )

      end subroutine sepel0703
