      subroutine sepel0705
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 705
!     general elliptic equation (one unknown per point)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPEL0705CF    Fill real coefficients for elements of type 705
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELGETIQMAT  Get the parameter iqmat as stored in iuser
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELRDTYPCONV Read information about type_convection
!                    Adapt methodupw if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0705' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0705'
         call prinin ( E%iuser, 10, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get information of coefficients

      call sepelgetiqmat

      E%methodupw = sepelgetintcf ( codeupw, nncodeupw, &
                                    ncodeupw, 1, 'supg' )
      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule

      call sepgetcoorsys

!     --- Read type_convection
!         Adapt methodupw if necessary

      call sepelrdtypconv

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch
      n = B%n

!     --- Fill all real coefficients

      E%ncoef = 18
      call sepel0705cf

!     --- Check if derivatives of basis functions are required in nodes

      B%fillderivn = B%type_basisfunc/=3 .and. B%type_basisfunc/=10 .and. &
                     B%type_basisfunc/=1 .and. B%type_basisfunc/=11

!     --- Check if element vector is completely zero

      if ( .not. E%sourceterm .and. .not. E%gammaterm .and. &
           E%typeconv/=3 ) E%notvec = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,1) 'B%m, B%n, B%cart', B%m, B%n, B%cart
         write(irefwr,1) 'B%rule', B%rule
         write(irefwr,*) 'diffusion, convection', E%diffusion, E%convection
         write(irefwr,*) 'End sepel0705'

      end if  ! ( debug )
      call erclos ( 'sepel0705' )

      end subroutine sepel0705
