      subroutine sepel0806
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine SEPELM806
!     in the case that IFIRST = 0
!     Old subroutine el0806
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPEL0801CF    Fill real coefficients for elements of type 801
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  Type of natural boundary condition
!         Possible values:
!         0,2: natural boundary condition of type 2
!         3: natural boundary condition of type 3
!         4: natural boundary condition of type 4
!         5: natural boundary condition of type 5
!     2:  Not yet in use:  set equal to 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  Not yet in use:  set equal to 0
!     Parameters with respect to the first unknown
!     6:  sigma
!     7:  h
!     8:  alpha 11  (1D/2D/3D)  (The parameters 8-13 are only used for
!     9:  alpha 12  (2D/3D)      boundary conditions of type 4)
!    10:  alpha 13  (3D)
!    11:  alpha 22  (2D/3D)
!    12:  alpha 23  (3D)
!    13:  alpha 33  (3D)
!    14-15: Not yet in use (must be zero)
!    16-25: See 6-15, now with respect to the second unknown
!    In case of boundary conditions of type 5:
!     6:  u1
!     7:  u2
!     8:  u3
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0806' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0806'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

!     --- Get information of coefficients

      E%typenatbnd = sepelgetintcf ( codenatbnd,  nncodenatbnd,&
                                     ncodenatbnd, 1, 'typenatbnd' )
      if ( E%typenatbnd==0 ) E%typenatbnd = 2
      if ( debug ) write(irefwr,1) 'typenatbnd', E%typenatbnd
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1
      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule

      call sepgetcoorsys
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Fill all real coefficients

      E%ncoef = 20
      call sepel0801cf

      if ( E%typenatbnd==2 ) then

!     --- Boundary conditions of type 2

         E%zeroterm1 = E%ind(1)/=0
         E%zeroterm2 = E%ind(11)/=0
         if ( .not. E%zeroterm1 .and. .not. E%zeroterm2 ) E%notmat = 1
         if ( E%ind(2)==0 .and. E%ind(12)==0 ) E%notvec = 1

      else if ( E%typenatbnd==3 .or. E%typenatbnd==5 ) then

!     --- Boundary conditions of type 3, 5. Use convection

         E%typeconv = 0

      else if ( E%typenatbnd==4 .or. E%typenatbnd==6 ) then

!     --- Boundary conditions of type 4, 6. Use convection

         E%notmat = 1

      end if  ! ( E%typenatbnd==3 )

      if ( E%typenatbnd==3 )  B%fillderivn = .true.

      E%sourceterm1 = E%ind(2)/=0
      E%sourceterm2 = E%ind(12)/=0

      if ( .not. E%sourceterm1 .and. .not. E%sourceterm2 ) notvec = 1

      notmas = 1

      allocate ( E%subvec(B%n), E%submat(B%n,B%n) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0806'

      end if  ! ( debug )
      call erclos ( 'sepel0806' )

      end subroutine sepel0806
