      subroutine sepelallocvcol
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Allocate locvecs in oldsols for one vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulefunc
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelallocvcol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelallocvcol'
         call prinin ( indvector, numoldvecs, 'indvector' )
         write(irefwr,1) 'iseqprevsol, ndim, iseqi2', &
                          iseqprevsol, ndim, iseqi2
         write(irefwr,*) 'filloldderiv ', E%filloldderiv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      k = iseqprevsol
      indvector(k) = 1
      nunkp = oldsols(k)%nunkp
      if ( debug ) &
         write(irefwr,1) 'allocate: k, m, maxunk', k, B%m, E%maxunk
      allocate ( oldsols(k)%locvecs(B%m,E%maxunk), &
                 oldsols(k)%locvecsnode(B%n,E%maxunk), &
                 oldsols(k)%loccvecs(B%m,E%maxunk), &
                 oldsols(k)%loccvecsnode(B%n,E%maxunk), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, (B%m+B%n)*E%maxunk, 'locvecs' )

      if ( iseqi2/=iseqprevsol ) then
         k = iseqi2
         allocate ( oldsols(k)%locvecs(B%m,E%maxunk), &
                    oldsols(k)%locvecsnode(B%n,E%maxunk), &
                    oldsols(k)%loccvecs(B%m,E%maxunk), &
                    oldsols(k)%loccvecsnode(B%n,E%maxunk), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, (B%m+B%n)*E%maxunk, 'locvecs' )
         indvector(iseqi1) = 1
         indvector(iseqi2) = 1
      else if ( E%compold==1 ) then
         if ( icheld==9 ) then
            k = iseqprevsol+1
            indvector(iseqprevsol:iseqprevsol+1) = 1
         else if ( icheld==10 ) then
            k = numold
            indvector(numold) = 1
         end if  ! ( icheld==9 )
         allocate ( oldsols(k)%locvecs(B%m,E%maxunk), &
                    oldsols(k)%locvecsnode(B%n,E%maxunk), &
                    oldsols(k)%loccvecs(B%m,E%maxunk), &
                    oldsols(k)%loccvecsnode(B%n,E%maxunk), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, (B%m+B%n)*E%maxunk, 'locvecs' )
      end if  ! ( iseqi2/=iseqprevsol )
      if ( ierror/=0 ) go to 1000

      if ( E%filloldderiv ) then

!     --- The derivatives of the old vector are also used

         allocate ( gradoldvec(B%m,ndim,nunkp), &
                    gradoldcvec(B%m,ndim,nunkp), &
                    stat = error )
         if ( error/=0 ) &
            call eralloc ( error, B%m*nunkp*ndim, 'gradoldvec' )
         if ( ierror/=0 ) go to 1000

!        --- quantities for module sepmodulefunc

         gradprev => gradoldvec(:,:,1)
         prevsolution => oldsols(k)%locvecs(:,1)

      end if  ! ( E%filloldderiv )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( indvector, numold, 'indvector' )
         write(irefwr,*) 'End sepelallocvcol'

      end if  ! ( debug )
      call erclos ( 'sepelallocvcol' )

      end subroutine sepelallocvcol
