      subroutine sepelbasefnch
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Define parameters for choice of basis functions
!     Based on integration rule and shape of element
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELNUMINT    Detect parameters n and m based on shape of element
!                    and quadrature rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbasefnch' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbasefnch'
         write(irefwr,1) 'ishape, n, m, nvert', E%ishape, B%n, B%m, B%nvert
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill n and m depending on shape and quadrature rule

      call sepelnumint

      B%type_basisfunc = E%ishape  ! type of element
      n = B%n
      m = B%m
      nvert = B%nvert  ! number of vertices

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ishape, n, m, nvert', E%ishape, n, m, nvert
         write(irefwr,*) 'End sepelbasefnch'

      end if  ! ( debug )
      call erclos ( 'sepelbasefnch' )

      end subroutine sepelbasefnch
