      subroutine sepelconvect6 ( veloc, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays veloc and ind in case of convection based on
!     sum of absolute values of derivatives
! ***********************************a***********************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: ind(3)
      double precision, intent(out) :: veloc(B%m,3)

!     ind            o    indicates if the "new" velocity components are
!                         constant of variable
!     veloc          o    Contains the new velocity components for this
!                         term. They are -1 or 1 depending on the sign of the
!                         derivatives
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: dudx(:,:)

!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the quadrature
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv > 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvect6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvect6'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldvec(:,:,1)

      ind(1:B%nd) = 1
      ind(B%nd+1:3) = 0
      veloc(1:B%m,1:B%nd) = sign(1d0,dudx(1:B%m,1:B%nd))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvect6'

      end if  ! ( debug )
      call erclos ( 'sepelconvect6' )

      end subroutine sepelconvect6
