      subroutine sepeld0300
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for derivatives of element type 300
!     Bearing
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: kchoice

!     kchoice        Defines type of coefficients required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf, inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     SEPEL0300CF    Fill real coefficients for elements of type 300
!     SEPELALLOCVCOL Allocate locvecs in oldsols for one vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld0300' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld0300'
         write(irefwr,1) 'icheld, itype', icheld, itype
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute kchoice as function of icheld

      select case ( icheld )
      case ( 22:25 )
         kchoice = 0
         E%ncoef = 18
      case ( 21 )
         kchoice = 4
         E%ncoef = 1
      case ( 6, 8:10, 16, 26, 28, 29, 36 )
         kchoice = 3
         E%ncoef = 6
      case default
         kchoice = 1
         E%ncoef = 0
      end select  ! case ( icheld )

      if ( debug ) write(irefwr,1) 'kchoice, ncoef', kchoice, E%ncoef

      E%imas = 0
      massrule = 0
      B%rule = 0
      E%methodupw = 0
      E%applyupw = 0
      B%cart = 1
      B%rule = 1
      E%compold = 0
      B%fillqmat = .true.  ! fill transformation matrix
      E%iqmat = 2     ! transformation parameter

      if ( icheld==10 .and. E%ndim>1 ) then
         E%filloldderiv = .false.
      else
         E%filloldderiv = .true.
      end if  ! ( icheld==10 .and. E%ndim>1 )

      if ( E%ncoef>0 ) call sepeliposset

      if ( kchoice==0 ) then

!     --- At least some coefficients must be filled
!         initialize ipos and some other quantities

         E%typebear = sepelgetintcf ( codebear, nncodebear, ncodebear, &
                                      1, 'type_of_bearing' )
         E%ipos = E%ipos+1  ! skip linerization

         if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!        --- Next quadrature rule and transformation

         call sepgetrule

         B%rule = max(1,B%rule)

      else

!     --- Standard case

         E%ipos = E%ipos+3

      end if  ! ( E%ncoef>0 )

!     --- Get information of basis functions

      call sepelbasefnch
      B%kstep = 1  ! to avoid jumping over vertices
      if ( E%ndim>B%nd ) then
         E%jtransfo = inirepscal ( E%iuser(E%ipos) )
         if ( E%jtransfo>=10 ) E%jtransfo = E%jtransfo/10
         if ( E%jtransfo==0 ) E%jtransfo = 2
      else
         E%jtransfo = 0
      end if  ! ( E%ndim>B%nd )

      E%ipos = E%ipos+2
      if ( debug ) write(irefwr,1) 'ipos, jtransfo', E%ipos, E%jtransfo

      if ( E%ncoef>0 ) call sepel0300cf  ! fill coefficients

      if ( E%compold==0 ) then
         if ( icheld==10 .or. icheld==9 ) E%compold = 1
         call sepelallocvcol! allocate oldsols%locvecs
      end if  ! ( E%compold==0 )

      B%fillderivn = .true.  ! Compute derivatives also in vertices
      B%callbyderiv = .true.

      n = B%n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icheld, ncoef', icheld, E%ncoef
         write(irefwr,1) 'irule, cart', B%rule, B%cart
         call prinin ( E%ind, E%ncoef, 'ind' )
         call prinrl1 ( E%coeffs, E%ncoef, B%m, 'coeffs' )
         write(irefwr,*) 'End sepeld0300'

      end if  ! ( debug )
      call erclos ( 'sepeld0300' )

      end subroutine sepeld0300
