      subroutine sepeld084 ( elemwg, elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in E%vec
!     with respect to subroutine sepeld705
!     icheld =  smoother for to0090 (2d)
!     Old subroutine eld100 (linear elements)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: elemwg(ndimlc,B%n), elemvc(ndimlc,B%n)

!     elemvc         o    Element vector
!     elemwg         o    element vector containing the weights
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: h, adu(B%n,E%ndim), wh(B%n)
      double precision, pointer :: a11(:), a12(:), a22(:), dudx(:), &
                                   dudy(:)

!     a11            First component of diffusion tensor
!     a12            12 component of diffusion tensor
!     a22            22 component of diffusion tensor
!     adu            A * grad(u)
!                    they are not
!     dudx           Contains du/dx
!     dudy           Contains du/dy
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     wh             Copy of the weights, but the zeros replace by a
!                    positive value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld084' )
      debug = .false.

      a11 => E%coeffs(:,1)
      a12 => E%coeffs(:,2)
      a22 => E%coeffs(:,4)
      dudx => gradoldvec(:,1,1)
      dudy => gradoldvec(:,2,1)
      n = B%n

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld084'
         call prinrl ( a11, n, 'a11' )
         call prinrl ( a12, n, 'a12' )
         call prinrl ( a22, n, 'a22' )
         call prinrl ( dudx, n, 'dudx' )
         call prinrl ( dudy, n, 'dudy' )
         call prinrl ( B%w, n, 'w' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      adu(:,1) = a11*dudx+a12*dudy
      adu(:,2) = a12*dudx+a22*dudy
      if ( debug ) then
         call prinrl ( adu(1,1), n, 'adux' )
         call prinrl ( adu(1,2), n, 'aduy' )
      end if  ! ( debug )

      wh(2:6:2) = B%w(2:6:2)
      h= wh(2)/60d0
      wh(1:5:2) = h

      elemwg(1,:) = wh
      elemwg(2,:) = wh

      do i = 1, 2
         elemvc(i,2:6:2) = adu(2:6:2,i)*B%w(2:6:2)
         elemvc(i,1:5:2) = h*adu(1:5:2,i)
      end do  ! i = 1, 2

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemvc, n, 2, 'elemvc' )
         call prinrl1 ( elemwg, n, 2, 'elemwg' )
         write(irefwr,*) 'End sepeld084'

      end if  ! ( debug )
      call erclos ( 'sepeld084' )

      end subroutine sepeld084
