      subroutine sepeld093 ( uold2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in E%vec
!     with respect to subroutine sepeld705
!     icheld = 9:local estimators for to0090 (attached to elements) (2d)
!     Old subroutine el2018
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: uold2(B%n,2)

!     uold2          i    Contains second input vector per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: adu(B%n,E%ndim), errsig(B%n,E%ndim)
      double precision :: ainv11(B%n), ainv12(B%n), ainv22(B%n), det(B%n)
      double precision :: help(B%n,E%ndim)
      double precision, pointer :: a11(:), a12(:), a22(:), dudx(:), &
                                   dudy(:)

!     a11            First component of diffusion tensor
!     a12            12 component of diffusion tensor
!     a22            22 component of diffusion tensor
!     adu            A * grad(u)
!     ainv11         First component of inverse of diffusion tensor
!     ainv12         12 component of inverse of diffusion tensor
!     ainv22         22 component of inverse of diffusion tensor
!                    they are not
!     det            determinant of diffusion tensor
!     dudx           Contains du/dx
!     dudy           Contains du/dy
!     errsig         estimation of error of A * grad u
!     help           help array to store temporary values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld093' )
      debug = .false.

      a11 => E%coeffs(:,1)
      a12 => E%coeffs(:,2)
      a22 => E%coeffs(:,4)
      dudx => gradoldvec(:,1,1)
      dudy => gradoldvec(:,2,1)
      n = B%n

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld093'
         call prinrl ( dudx, n, 'dudx' )
         call prinrl ( dudy, n, 'dudy' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      adu(:,1) = a11*dudx+a12*dudy
      adu(:,2) = a12*dudx+a22*dudy

      if ( debug ) then
         call prinrl ( adu(1,1), n, 'adudx' )
         call prinrl ( adu(1,2), n, 'adudy' )
      end if  ! ( debug )

!     --- invert matrix A, store in Ainv

      det = a11*a22-a12**2
      ainv11 = a22/det
      ainv12 = -a12/det
      ainv22 = a11/det

      if ( debug ) then
         call prinrl ( ainv11, n, 'a11^-1' )
         call prinrl ( ainv12, n, 'a12^-1' )
         call prinrl ( ainv22, n, 'a22^-1' )
      end if  ! ( debug )

!     --- Fill  errsig  by estimation of error of vector  A * grad u
!         in all integration points

      errsig = adu-uold2

      if ( debug ) then
         call prinrl1 ( uold2, 2, n, 'uold2')
         call prinrl ( errsig(1,1),m,'errsig1')
         call prinrl ( errsig(1,2),m,'errsig2')
      end if  ! ( debug )

!     --- Integrate   transp(errsig) * inverse(A) * errsig

      help(:,1) = ainv11*errsig(:,1)+ainv12*errsig(:,2)
      help(:,2) = ainv12*errsig(:,1)+ainv22*errsig(:,2)
      E%vec(1) = sqrt(sum(B%w*(errsig(:,1)*help(:,1)+errsig(:,2)*help(:,2))))

!     --- calulate semi-energy norm of  u

      help(:,1) = ainv11*adu(:,1)+ainv12*adu(:,2)
      help(:,2) = ainv12*adu(:,1)+ainv22*adu(:,2)
      E%vec(2) = sqrt(sum(B%w*(adu(:,1)*help(:,1)+adu(:,2)*help(:,2))))

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, 2, 'elemvc' )
         write(irefwr,*) 'End sepeld093'

      end if  ! ( debug )
      call erclos ( 'sepeld093' )

      end subroutine sepeld093
