      subroutine sepeld101
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives in E%vec
!     with respect to subroutine sepeld705
!     icheld = 10: local estimators for to0090 (distributed to vertices)
!     Old subroutine eld118
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h, adu(B%n), h1, du
      double precision, pointer :: u(:), a11(:), dudx(:), x(:)

!     a11            First component of diffusion tensor
!     adu            A * grad(u)
!                    they are not
!     du             u(end)-u(start) in element
!     dudx           Contains du/dx
!     h              Length of element
!     h1             Help variable to compute the element matrix
!     u              Reference to uprev
!                    Old solution in nodes
!     x              coordinates of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld101' )
      debug = .false.

      u => uprev(:,1)
      a11 => E%coeffs(:,1)
      dudx => uprev2(:,1)
      x => B%xyz(:,1)
      n = B%n

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld101'
         call prinrl ( a11, n, 'a11' )
         call prinrl ( u, n, 'u' )
         call prinrl ( dudx, n, 'dudx' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%derivconst ) then

!     --- Derivatives are constant

         h = x(2)-x(1)
         du = u(2)-u(1)
         h1 = du/h
         adu = h1*a11

      else

!     --- Derivatives are variable

         h = x(inpelm)-x(1)
         adu(1) = a11(1) * ( -3*u(1) +4*u(2) -  u(3) ) / h
         adu(2) = a11(2) * ( -  u(1)         +  u(3) ) / h
         adu(3) = a11(3) * (    u(1) -4*u(2) +3*u(3) ) / h

      end if  ! ( B%derivconst )
      E%vec(1) = sqrt(sum(B%w*(dudx-adu)**2/a11))
      E%vec(2) = E%vec(1)
      E%wvec = 1d0

1000  call erclos ( 'sepeld101' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, n, 'elemvc' )
         call prinrl ( E%wvec, n, 'elemwg' )
         write(irefwr,*) 'End sepeld101'

      end if  ! ( debug )

      end subroutine sepeld101
