      subroutine sepeld150
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weight vector for general
!     complex elliptic equations with one unknown
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icounw

!     icounw         length of elemwg(*)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
!     PRINRL         Print 1d real vector
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELD0150     Fill element independent quantities for derivatives
!                    of element type 150
!     SEPELD150DER   Actual computing of derivatives for complex elliptic
!                    equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELM150CF    Fill coefficients for complex general elliptic equation
!     SEPELOLDCVECS  Fill old complex vectors for element
!     SEPELOLDCVECSK Fill old complex vectors for element (only one vector)
!     SEPELUCDERIV   Compute the derivative of a given complex function
!                    in one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld150' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld150'
         write(irefwr,1) 'icheld, numoldvecs', icheld, numoldvecs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepeld0150

      if ( icheld==4 ) then
         E%vec = 0              ! real vector
      else
         E%cvec = 0             ! complex vector
      end if  ! ( icheld==4 )

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( debug ) &
         write(irefwr,1) 'E%compold, iseqi1, iseqi2, iseqprevsol', &
                          E%compold, iseqi1, iseqi2, iseqprevsol
      if ( E%compold>0 ) then

!     --- All old vectors are required

         call sepeloldcvecs

      else

!     --- Only one vector required

         call sepeloldcvecsk ( iseqprevsol )
         if ( iseqi2/=iseqprevsol ) call sepeloldcvecsk ( iseqi2 )
         call sepelucderiv

      end if  ! ( E%compold>0 )

      if ( icheld==9 ) then
         ucprev2 => oldsols(2)%loccvecsnode(1:B%n,1:nunkp)
      else if ( icheld==10 ) then
         ucprev2 => oldsols(numold)%loccvecsnode(1:B%n,1:nunkp)
      end if  ! ( icheld==9 )

!     --- Fill all coefficients

      if ( E%ncoef>0 ) call sepelm150cf

      if ( icheld==9 .or. icheld>10 .and. icheld<=20 .or. &
           icheld==29 .or. icheld>=41 ) then
         icounw = 0
      else
         icounw = icouno
      end if  ! ( icheld>10 .and. icheld<=20 ... )

!     --- Initialize weight vector, if any

      if ( icounw>0 ) E%wvec = 1d0

!     --- Compute derivatives

      call sepeld150der

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icheld', icheld
         if ( icheld==4 ) then
            call prinrl ( E%vec, icouno, 'elemvc')
         else
            call princompl ( E%cvec, icouno, 'elemvc')
         end if  ! ( icheld==4 )
         if ( icounw>0 ) call prinrl ( E%wvec, icounw, 'elemwg')
         write(irefwr,*) 'End sepeld150'

      end if  ! ( debug )
      call erclos ( 'sepeld150' )

      end subroutine sepeld150
