      subroutine sepeld150der
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual computing of derivatives for complex elliptic equation
!     One unknown per point
!     Type number 150
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELDC01      actual filling of derivatives and weights in elemvc and
!                    elemwg with respect to subroutine sepeld150
!                    icheld = 1: d c /d_ix
!     SEPELDC02      actual filling of derivatives and weights in elemvc and
!                    elemwg with respect to subroutine sepeld150
!                    icheld = 2, 3, 5: gradient
!     SEPELDC04      actual filling of derivatives and weights in elemvc and
!                    elemwg with respect to subroutine sepeld150
!                    icheld = 4  k ||grad phi ||^2
!     SEPELDC06      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 6, 16, 26  Compute - A gradient
!     SEPELDC07      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 7: div u
!     SEPELDC11      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 11: d c /d_ix
!     SEPELDC12      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 12, 13, 15: gradient
!     SEPELDC15      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 15: curl (2d)
!     SEPELDC31      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 31: d c /d_ix
!     SEPELDC32      actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld150
!                    icheld = 32, 33, 35: gradient
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld150der' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld150der'
         write(irefwr,1) 'icheld', icheld
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( icheld )

      case(1)

!     --- icheld = 1, compute derivative du/d ix

         call sepeldc01

      case(2, 3, 5)

!     --- icheld = 2,3,5  Compute gradient

         call sepeldc02 ( E%wvec, E%cvec )

      case(4)

!     --- icheld = 4, compute k * || grad(u)^2 ||

         call sepeldc04

      case(6,16,36)

!     --- icheld = 6, 16, 26  Compute - A gradient

         call sepeldc06 ( E%cvec )

      case(7)

!     --- icheld = 7, compute div u

         call sepeldc07

      case(11)

!     --- icheld = 11, See icheld = 1, however, without weights

         call sepeldc11

      case(12:13)

!     --- icheld = 12, 13, See icheld = 2, 3, however, without weights
!                          Defined per element

         call sepeldc12 ( E%cvec )

      case(15)

!     --- icheld = 15, See icheld = 5, however, without weights
!                      Defined per element

         call sepeldc15

      case(31)

!     --- icheld = 31, compute derivative du/d ix, without weights

         call sepeldc31

      case(32, 33, 35)

!     --- icheld = 32,33,35  Compute gradient, without weights

         call sepeldc32 ( E%cvec )

      case default

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      end select  ! case ( icheld )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeld150der'

      end if  ! ( debug )
      call erclos ( 'sepeld150der' )

      end subroutine sepeld150der
