      subroutine sepeld705der
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual computing of derivatives for elliptic equation
!     One unknown per point
!     Type number 705
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELD01       actual filling of derivatives and weights in elemvc and
!                    elemwg with respect to subroutine sepeld705
!                    icheld = 1: d c /d_ix
!     SEPELD02       actual filling of derivatives and weights in elemvc and
!                    elemwg with respect to subroutine sepeld705
!                    icheld = 2, 3, 5: gradient
!     SEPELD06       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 6, 16, 26  Compute - A gradient
!     SEPELD07       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 7: div u
!     SEPELD08       Compute smoother for to0090
!     SEPELD09       Compute local estimators for to0090
!     SEPELD10       Compute local estimators for to0090
!     SEPELD11       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 11: d c /d_ix
!     SEPELD12       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 12, 13, 15: gradient
!     SEPELD15       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 15: curl (2d)
!     SEPELD22       Fill element vector for flow computation of
!                    Reynolds equation
!     SEPELD23       Compute flow vector for Reynolds
!     SEPELD24       Compute shear stress for bottom surface for Reynolds
!     SEPELD25       Compute shear stress for top surface for Reynolds
!     SEPELD31       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 31: d c /d_ix
!     SEPELD32       actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705
!                    icheld = 32, 33, 35: gradient
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld705der' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld705der'
         write(irefwr,1) 'icheld', icheld
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( icheld )

      case(1)

!     --- icheld = 1, compute derivative du/d ix

         call sepeld01

      case(2, 3, 5)

!     --- icheld = 2,3,5  Compute gradient

         call sepeld02 ( E%wvec, E%vec )

      case(6,16,36)

!     --- icheld = 6, 16, 26  Compute - A gradient

         call sepeld06 ( E%vec )

      case(7)

!     --- icheld = 7, compute div u

         call sepeld07

      case(8)

!     --- icheld = 8, smoothing for to0090

         call sepeld08 ( E%wvec, E%vec )

      case(9)

!     --- icheld = 9, local estimators for to0090 (attached to elements)

         call sepeld09

      case(10)

!     --- icheld = 10, local estimators for to0090 (distributed to vertices)

         call sepeld10

      case(11)

!     --- icheld = 11, See icheld = 1, however, without weights

         call sepeld11

      case(12:13)

!     --- icheld = 12, 13, See icheld = 2, 3, however, without weights
!                          Defined per element

         call sepeld12 ( E%vec )

      case(15)

!     --- icheld = 15, See icheld = 5, however, without weights
!                      Defined per element

         call sepeld15

      case(22)

!     --- icheld = 22, flow for Reynolds equation

         call sepeld22

      case(23)

!     --- icheld = 23, flow for Reynolds equation

         call sepeld23 ( E%vec )

      case(24)

!     --- icheld = 24, shear stress at bottom surface for Reynolds equation

         call sepeld24 ( E%vec )

      case(25)

!     --- icheld = 25, shear stress at top surface for Reynolds equation

         call sepeld25 ( E%vec )

      case(31)

!     --- icheld = 31, compute derivative du/d ix, without weights

         call sepeld31

      case(32, 33, 35)

!     --- icheld = 32,33,35  Compute gradient, without weights

         call sepeld32 ( E%vec )

      case(45)

!     --- icheld = 45, compute vector of elements to refine
!         only available for integers

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      case default

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      end select  ! case ( icheld )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeld705der'

      end if  ! ( debug )
      call erclos ( 'sepeld705der' )

      end subroutine sepeld705der
