      subroutine sepeldealloc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Deallocate coefficients and basis functions for element subroutines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     allocate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldealloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldealloc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'associated(E%coeffs)', associated(E%coeffs)
      if ( associated(E%coeffs) ) deallocate ( E%coeffs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coeffs' )

      if ( debug ) &
         write(irefwr,*) 'associated(E%coeffsnode)', associated(E%coeffsnode)
      if ( associated(E%coeffsnode) ) deallocate ( E%coeffsnode, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coeffsnode' )
      if ( debug ) &
         write(irefwr,*) 'associated(E%coeffcp)', &
                          associated(E%coeffcp)
      if ( associated(E%coeffcp) ) &
                      deallocate ( E%coeffcp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coeffcp' )

      if ( debug ) &
         write(irefwr,*) 'associated(E%ccoeffs)', associated(E%ccoeffs)
      if ( associated(E%ccoeffs) ) deallocate ( E%ccoeffs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ccoeffs' )

      if ( debug ) &
         write(irefwr,*) 'associated(E%ccoeffsnode)', &
                          associated(E%ccoeffsnode)
      if ( associated(E%ccoeffsnode) ) &
           deallocate ( E%ccoeffsnode, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ccoeffsnode' )

      if ( debug ) &
         write(irefwr,*) 'associated(E%ich)', associated(E%ich), size(E%ich)
      if ( associated(E%ich) ) deallocate ( E%ich, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ich' )
      if ( debug ) write(irefwr,*) 'associated(E%ist)', associated(E%ist)
      if ( associated(E%ist) ) deallocate ( E%ist, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ist' )
      if ( debug ) write(irefwr,*) 'associated(E%mst)', associated(E%mst)
      if ( associated(E%mst) ) deallocate ( E%mst, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'mst' )
      if ( debug ) write(irefwr,*) 'associated(E%ind)', associated(E%ind)
      if ( associated(E%ind) ) deallocate ( E%ind, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ind' )
      if ( debug ) &
         write(irefwr,*) 'associated(E%ivecnums)', associated(E%ivecnums)
      if ( associated(E%ivecnums) ) deallocate ( E%ivecnums, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ivecnums' )
      if ( debug ) write(irefwr,*) 'associated(B%Jac)', associated(B%Jac)
      if ( associated(B%Jac) ) deallocate ( B%Jac, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'Jac' )

      if ( debug ) write(irefwr,*) 'associated(B%w)', associated(B%w)
      if ( associated(B%w) ) deallocate ( B%w, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'w' )
      if ( associated(B%w2) ) deallocate ( B%w2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'w2' )
      if ( debug ) write(irefwr,*) 'associated(B%xyz)', associated(B%xyz)
      if ( associated(B%xyz) ) deallocate ( B%xyz, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'xyz' )
      if ( debug ) write(irefwr,*) 'associated(B%lam)', associated(B%lam)
      if ( associated(B%lam) ) deallocate ( B%lam, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'lam' )
      if ( debug ) write(irefwr,*) 'associated(B%phic)', associated(B%phic)
      if ( associated(B%phic) ) deallocate ( B%phic, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phic' )
      if ( debug ) write(irefwr,*) 'associated(B%wc)', associated(B%wc)
      if ( associated(B%wc) ) deallocate ( B%wc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'wc' )
      if ( debug ) write(irefwr,*) 'associated(B%phider)', associated(B%phider)
      if ( associated(B%phider) ) deallocate ( B%phider, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phider' )
      if ( debug ) write(irefwr,*) 'associated(B%phiderc)',associated(B%phiderc)
      if ( associated(B%phiderc) ) deallocate ( B%phiderc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phiderc' )

      if ( associated(B%phidern) ) deallocate ( B%phidern, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phidern' )
      if ( debug ) &
         write(irefwr,*) 'associated(B%phidercn)',associated(B%phidercn)
      if ( associated(B%phidercn) ) deallocate ( B%phidercn, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phidercn' )

      if ( debug ) write(irefwr,*) 'associated(B%phider2)',associated(B%phider2)
      if ( associated(B%phider2) ) deallocate ( B%phider2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phider2' )

      if ( debug ) write(irefwr,*) 'associated(B%xyzgc)', associated(B%xyzgc)
      if ( associated(B%xyzgc) ) deallocate ( B%xyzgc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'xyzgc' )

      if ( debug ) write(irefwr,*) 'associated(B%xyzgcn)', associated(B%xyzgcn)
      if ( associated(B%xyzgcn) ) deallocate ( B%xyzgcn, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'xyzgcn' )

      if ( debug ) write(irefwr,*) 'associated(B%)dxidx', associated(B%dxidx)
      if ( associated(B%dxidx) ) deallocate ( B%dxidx, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'dxidx' )
      if ( debug ) write(irefwr,*) 'associated(B%phi)', associated(B%phi)
      if ( associated(B%phi) ) deallocate ( B%phi, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'phi' )

      if ( debug ) write(irefwr,*) 'associated(B%qmat)', associated(B%qmat)
      if ( associated(B%qmat) ) deallocate ( B%qmat, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'qmat' )

      if ( debug ) write(irefwr,*) 'associated(B%xyzg)', associated(B%xyzg)
      if ( associated(B%xyzg) ) deallocate ( B%xyzg, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'xyzg' )

      if ( B%rule/=1 .or. B%n/=B%m ) then

!     --- Gauss rules

         if ( debug ) &
            write(irefwr,*) 'associated(B%dxdxig)', associated(B%dxdxig)
         if ( associated(B%dxdxig) ) deallocate ( B%dxdxig, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'dxdxig' )

      end if  ! ( B%rule/=1 )

      do k = 1,  numoldvecs

         if ( debug ) write(irefwr,1) 'k, indvector(k)', k, indvector(k)

         if ( indvector(k)>0 ) then

!        --- Vector has been filled

            if ( debug ) &
               write(irefwr,*) 'associated(oldsols(k)%locvecs )', &
                                associated(oldsols(k)%locvecs )
            if ( associated(oldsols(k)%locvecs ) ) &
                 deallocate ( oldsols(k)%locvecs, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'locvecs' )
            if ( associated(oldsols(k)%loccvecs ) ) &
                 deallocate ( oldsols(k)%loccvecs, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'loccvecs' )

            if ( associated(oldsols(k)%locvecsnode ) ) &
                 deallocate ( oldsols(k)%locvecsnode, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'locvecsnode' )

            if ( associated(oldsols(k)%loccvecsnode ) ) &
                 deallocate ( oldsols(k)%loccvecsnode, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'loccvecsnode' )

         end if  ! ( indvector(k)>0 )

      end do  ! k = 1,  numoldvecs

      if ( debug ) write(irefwr,*) 'after loop'

      if ( debug ) &
         write(irefwr,*) 'associated(gradoldvec)', associated(gradoldvec)
      if ( associated(gradoldvec)) deallocate ( gradoldvec, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'gradoldvec' )

      if ( B%fillpsi ) then
         if ( debug ) &
            write(irefwr,*) 'associated(B%psi)', associated(B%psi)
         if ( associated(B%psi) ) deallocate ( B%psi, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'psi' )
      end if  ! ( B%fillpsi )

      if ( E%ktang>0 ) then
         if ( debug ) write(irefwr,1) 'E%ktang', E%ktang
         deallocate ( B%tang, stat = error )
      end if  ! ( E%ktang>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldealloc'

      end if  ! ( debug )
      call erclos ( 'sepeldealloc' )

      end subroutine sepeldealloc
