      subroutine sepeldiffup01
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute part of upwind discretization related to diffusion
!     1d quadratic elements, Cartesian coordinates
!     See old subroutine elpu01
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     upwind
!     diffusion
!     convection_diffusion
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, n, m
      double precision :: som, dalpdx(3,6)
      double precision, pointer :: phix(:,:)
      double precision, pointer :: alpha(:)

!     alpha          Diffusion coefficient D_11
!     dalpdx         Work array to store d/dx alpha d phi(j)/dx in the
!                    integration points in the following way:
!                    dalpdx(j,k) = d/dx alpha d phi(j)/dx (xk)
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     m              Number of quadrature points
!     n              Number of basis functions
!     phix           Contains x-derivatives of basis functions in quadrature
!                    points
!     som            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffup01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffup01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      alpha => E%difftensor(:,1)
      n = B%n
      m = B%m

!     --- First store d/dx alpha dphi(j)/dx in array dalpdx

      do j = 1, n
         do i = 1, m
            som = 0d0
            do k = 1, n
               som = som + alpha(k)*phix(j,k)*phix(k,i)
            end do
            dalpdx(j,i) = som
         end do
      end do

!     --- second derivative part

      do i = 1, n
         do j = 1, n
            som = 0d0
            do k = 1, m
               som = som + dalpdx(j,k)*B%w(k)*(B%psi(i,k)-B%phi(i,k))
            end do
            E%mat(j,i) = E%mat(j,i) - som
         end do
      end do

1000  call erclos ( 'sepeldiffup01' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiffup01'

      end if  ! ( debug )

      end subroutine sepeldiffup01
