      subroutine sepeldiffup02
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add contribution of second derivative part to matrix in the case of
!     upwind
!     Two-dimensional quadratic or bilinear elements; Gauss rule
!     Old subroutines elup02 and elup03
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     upwind
!     diffusion
!     convection_diffusion
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, i, j, n, k, kstep, l
      double precision :: a1(B%n,B%n), a2(B%n,B%n), som, som1
      double precision, pointer :: dphidx(:,:), dphidy(:,:)
      double precision, pointer :: eps11(:), eps12(:), eps22(:), w(:)
      double precision, pointer :: phi(:,:), psi(:,:)

!     a1             Work array to store parts of the computation of the
!                    second derivative term
!     a2             Work array to store parts of the computation of the
!                    second derivative term
!     dphidx         Array of length n x m x 2 containing the values of the
!                    derivatives of the basis functions in the integration
!                    node
!     eps11          Work array in which the values of alpha 11 in the
!                    nodal points are stored
!     eps12          Work array in which the values of alpha 12 in the
!                    nodal points are stored
!     eps22          Work array in which the values of alpha 22 in the
!                    nodal points are stored
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kstep          Step for quadrature points
!     l              Counting variable
!     m              Number of quadrature points
!     n              Number of basis functions
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!     psi            upwind basis functions
!     som            Help variable to compute a sum
!     som1           Help variable to compute a sum
!     w              weights for integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffup02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffup02'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m = B%m
      n = B%n
      kstep = B%kstep

      if ( debug ) write(irefwr,1) 'n, m, kstep', n, m, kstep

      dphidx => B%phidern(:,:,1)
      dphidy => B%phidern(:,:,2)
      eps11 => E%coeffsnode(:,1)
      eps12 => E%coeffsnode(:,2)
      eps22 => E%coeffsnode(:,4)
      phi => B%phi
      psi => B%psi
      w => B%w

!     --- Fill arrays a1 and a2

      do i = 1, n
         do k = 1, n
            a1(k,i) = eps11(k)*dphidx(i,k)+eps12(k)*dphidy(i,k)
            a2(k,i) = eps12(k)*dphidx(i,k)+eps22(k)*dphidy(i,k)
         end do
      end do

      if ( debug ) then
         call prinrl1 ( a1, n, n, 'a1')
         call prinrl1 ( a2, n, n, 'a2')
      end if  ! ( debug )

      dphidx => B%phider(:,:,1)
      dphidy => B%phider(:,:,2)

      do i = 1, n

!     --- second derivative part

         do j = 1, n
            som = 0d0
            do k = kstep, m, kstep
               som1 = 0d0
               do l = 1, n
                  som1 = som1 + a1(l,j)*dphidx(l,k)+a2(l,j)*dphidy(l,k)
               end do
               som = som - (psi(i,k)-phi(i,k)) * som1 * w(k)
            end do
            E%mat(j,i) = E%mat(j,i) + som
         end do

      end do

1000  call erclos ( 'sepeldiffup02' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, n, n, 'matrix' )
         write(irefwr,*) 'End sepeldiffup02'

      end if  ! ( debug )

      end subroutine sepeldiffup02
