      subroutine sepelinstrain1 ( eps0, Dw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector due to initial strain (2d)
!     Old subroutine el8103
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: eps0(m,E%dimD), Dw(E%dimD,E%dimD,m)

!     Dw             i    Dmat * w
!     eps0           i    initial strain in quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: v(:)
      integer :: ip, iq, i, l, nd
      double precision :: som

!     i              Counting variable
!     ip             Loop variable
!     iq             Loop variable
!     l              General loop variable
!     nd             Dimension of Matrix D
!     som            Help parameter to compute a sum
!     v              Sub element vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelinstrain1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelinstrain1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nd = E%dimD

      if (  B%derivconst .and. nd==3 ) then

!     --- Derivatives of basis functions constant

         do l = 1, 2

            v => E%vec(l:2*n:2)

            do i = 1, n
               som = 0d0
               do ip = 1, nd
                  do iq = 1, nd
                     som = som+E%Bmat(ip,l,i,1)*sum(Dw(ip,iq,:)*eps0(:,iq))
                  end do  ! iq = 1, nd
               end do  ! ip = 1, nd
               v(i) = v(i)+som
            end do  ! i = 1, n

         end do  ! l = 1, 2

      else

!     --- Derivatives of basis functions not constant

         do l = 1, 2

            v => E%vec(l:2*n:2)

            do i = 1, n
               som = 0d0
               do ip = 1, nd
                  do iq = 1, nd
                     som = som+sum(E%Bmat(ip,l,i,:)*Dw(ip,iq,:)*eps0(:,iq))
                  end do  ! iq = 1, nd
               end do  ! ip = 1, nd
               v(i) = v(i)+som
            end do  ! i = 1, n

         end do  ! l = 1, 2

      end if  ! (  B%derivconst .and. nd==3 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelinstrain1'

      end if  ! ( debug )
      call erclos ( 'sepelinstrain1' )

      end subroutine sepelinstrain1
