      subroutine sepelmlinelasb
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for boundary element corresponding to
!     linear elastic problem
!     Old subroutine el8000
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: s(:,:)
      double precision :: s1(n,n)

!     s              Submatrix of element matrix to be filled
!     s1             Submatrix of element matrix to be filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELMASSMAT3  Fill the element mass matrix in case of ndim unknowns
!                    per node
!                    Consistent mass matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmlinelasb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmlinelasb'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ind(5)/=0 ) then

!     --- alpha 11 # 0

         s => E%mat(1:2*n:2,1:2*n:2)
         call sepelmassmat3 ( E%coeffs(:,5), s )

      end if

      if ( E%ind(6)/=0 ) then

!     --- alpha 12 # 0

         call sepelmassmat3 ( E%coeffs(:,6), s1 )
         E%mat(1:2*n:2,2:2*n:2) = s1
         E%mat(2:2*n:2,1:2*n:2) = s1

      end if

      if ( E%ind(7)/=0 ) then

!     --- alpha 13 # 0

         call sepelmassmat3 ( E%coeffs(:,7), s1 )
         E%mat(1:2*n:2,3:2*n:2) = s1
         E%mat(3:2*n:2,1:2*n:2) = s1

      end if

      if ( E%ind(8)/=0 ) then

!     --- alpha 22 # 0

         s => E%mat(2:2*n:2,2:2*n:2)
         call sepelmassmat3 ( E%coeffs(:,8), s )

      end if

      if ( E%ind(9)/=0 ) then

!     --- alpha 23 # 0

         call sepelmassmat3 ( E%coeffs(:,9), s1 )
         E%mat(2:2*n:2,3:2*n:2) = s1
         E%mat(3:2*n:2,2:2*n:2) = s1

      end if

      if ( E%ind(10)/=0 ) then

!     --- alpha 33 # 0

         s => E%mat(3:2*n:2,3:2*n:2)
         call sepelmassmat3 ( E%coeffs(:,10), s )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmlinelasb'

      end if  ! ( debug )
      call erclos ( 'sepelmlinelasb' )

      end subroutine sepelmlinelasb
