      subroutine sepeloldvecs
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill old vectors for element
!     Only those vectors needed for coefficients are filled
!     Replaces old subroutine EL3009
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coefficient
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelem
      use sepmoduleelm
      use sepmoduleoldvecs
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivec

!     ivec           Sequence number of vector in set of old vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPELOLDVECSK  Fill old vectors for element (only one vector)
!     SEPELUDERIV    Compute the derivative of a given function in one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1792   FUNCC3 may only be called in the combination with NUMOLD>0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeloldvecs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeloldvecs'
         write(irefwr,1) 'maxunk, jdiag, irule', E%maxunk, B%jdiag, irule
         write(irefwr,*) 'filloldderiv', E%filloldderiv
         write(irefwr,1) 'ishape, m, n, kstep', E%ishape, m, n, B%kstep
         write(irefwr,1) 'iseqin', iseqin
         call prinin ( indvector, numoldvecs, 'indvector' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%maxunk==0 ) then

!     --- Error 1792:  FUNCC3 may only be used if numold>0 in the call of
!                      BUILD

         call errsub ( 1792, 0, 0, 0 )
         go to 1000

      end if

      if ( indvector(iseqin)>0 ) then
         E%ugauss => oldsols(iseqin)%locvecs
         uprev => oldsols(iseqprevsol)%locvecsnode
      end if  ! ( indvector(iseqin)>0 )

      do ivec = 1, numoldvecs

         if ( debug ) &
            write(irefwr,1) 'ivec, indvector(ivec)', ivec, indvector(ivec)

         if ( indvector(ivec)>0 .and. oldsols(ivec)%iseqtype>0 ) then

!        --- Vector is used

            call sepeloldvecsk ( ivec )

         end if  ! ( indvector(ivec)==0 )

      end do  ! ivec = 1, numoldvecs

!     --- Fill derivative of prior vector and store in gradoldvec

      if ( E%filloldderiv ) call sepeluderiv

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeloldvecs'

      end if  ! ( debug )
      call erclos ( 'sepeloldvecs' )

      end subroutine sepeloldvecs
