      subroutine sepelp02
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Quadratic 1d elements
!     Old subroutine elp611
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     1d
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m
      double precision :: h, hloc
      double precision, pointer :: x(:), dphidx(:,:), phider(:,:), w(:), xg(:)

!     dphidx         Contains values of dphi/dx in quadrature points
!     h              Help variable to store a constant temporarily
!     hloc           Local weight
!     m              number of quadrature points
!     phider         Contains values of dphi/dx in quadrature points in
!                    reference element
!     w              Contains weights
!     x              x-coordinates of nodal points
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR0    Print error message errornr for element
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELP02CURV   Fill dxdxi and dphidx for a curved 1d element
!     SEPELP02FIRST  compute some element independent quantities
!                    Quadratic 1d elements
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp02'
         write(irefwr,1) 'ndim, ifirst, jcart', E%ndim, E%first, B%cart
         write(irefwr,1) 'irule', B%rule
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x  => B%xyz(:,1)
      dphidx => B%phider(:,:,1)
      phider => B%phiderc(:,:,1)
      w => B%w

      m = B%m

      if ( E%first==0 ) call sepelp02first

!     --- Compute length of element

      if ( E%ndim==1 ) then

!     --- Standard: 1d

         h = x(3)-x(1)
         w = abs(h)*B%wc
         hloc = abs(h)

         if ( hloc<=0d0 ) then

!        --- Error 105: h is zero

            call errorelvar0 ( 105 )
            call prinrl2(B%xyz,2,E%ndim,'co-ordinates')
            go to 1000

         end if  ! ( abs(h)<=0d0 )

         if ( B%fillderiv ) dphidx = phider/h
         if ( B%comppsi ) B%lamder = B%lamderc/h

      else

!     --- ndim>1: line element

         call sepelp02curv

      end if  ! ( E%ndim==1 )

!     --- Fill Gauss points

      call sepelpgauss

      if ( B%cart>1 ) w = w*B%xyzg(:,1)*pi2

1000  call erclos ( 'sepelp02' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( B%w, m, 'w' )
         write(irefwr,*) 'End sepelp02'

      end if  ! ( debug )

      end subroutine sepelp02
