      subroutine sepelp02first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Quadratic 1d elements
!     Old subroutine elp611first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     1d
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision lambda(B%m), d, lam1(B%m)
      double precision, pointer :: wh(:), phi(:,:), phider(:,:)
      character (len=22), parameter :: text = 'quadratic line element'

!     d              help parameter to store a constant
!     lam1           1-lambda (lin basis function corresponding to first node)
!     lambda         Contains values of lambda_1 in quadrature points
!     phi            Contains values of basis functions in quadrature points
!     phider         Contains values of dphi/dx in quadrature points in
!                    reference element
!     text           String variable containing some text
!     wh             Contains the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp02first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp02first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phi => B%phi
      phider => B%phiderc(1:B%n,1:B%m,1)
      wh => B%wc

      if ( B%cart>2 ) call errorelvar ( B%cart-1, 1769, text, 0 )

!     --- Fill array lambda and wh

      if ( B%rule==1 ) then

!     --- irule = 1  Newton Cotes rule

         lambda(1) = 0d0
         lambda(2) = 0.5d0
         lambda(3) = 1d0

         d = 1d0/6d0
         wh(1) = d
         wh(2) = 4d0*d
         wh(3) = d

      else

!     --- irule>2  one-point Gauss rule

         call sepelplambda1d ( lambda, text, B%rule, B%m, B%wc )

      end if  ! ( irule==1 )

!     --- Fill array phi

      lam1 = 1d0-lambda
      phi(1,:) = lam1(:)*(2d0*lam1(:)-1d0)
      phi(2,:) = 4d0*lambda(:)*lam1(:)
      phi(3,:) = lambda(:)*(2d0*lambda(:)-1d0)

      phider(1,:) = -(4d0*lam1(:)-1d0)
      phider(2,:) = 4d0*(lam1(:)-lambda(:))
      phider(3,:) = 4d0*lambda(:)-1d0
      B%lam(1,:) = lambda
      B%lam(2,:) = lam1
      B%lamderc(1,:,1) = -1d0
      B%lamderc(2,:,1) = 1d0

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( phider, B%m, 3, 'phider' )
         write(irefwr,*) 'End sepelp02first'

      end if  ! ( debug )
      call erclos ( 'sepelp02first' )

      end subroutine sepelp02first
