      subroutine sepelp03transf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute x as function of jtransfo
!     Old subroutine elp620transfo
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: kx, ky, k, n
      double precision :: alpha(3), rr
      double precision, pointer :: x(:,:)

!     alpha          Contains angles in case of transformation 4
!     k              Counting variable
!     kx             integer help parameter to determine the correct angle
!     ky             integer help parameter to determine the correct angle
!     n              Number of basis functions
!     rr             Radius of cylinder in case of transformation 4
!     x              co-ordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp03transf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp03transf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz
      n = B%n

      select case (E%jtransfo)
      case(2)

!     --- Transform: xsi := y, eta := z

         x(:,1) = x(:,2)
         x(:,2) = x(:,3)

      case(3)

!     --- Transform: xsi := x, eta := z

         x(:,2) = x(:,3)

      case(4)

!     --- Transform: xsi := R*atan2(y,x), eta := z, R := sqrt(x**2+y**2)
!         Elements with nodes approx. (-1,0,z) can cause trouble because of
!         atan2(e,-1) = pi and atan2(-e,-1) = -pi !!!

         kx = 0
         ky = 0
         do k = 1, n
            if ( x(k,1)>0.0d0 ) then
               kx = kx + 1
            else
               kx = kx - 1
            end if
            if ( x(k,2)>=0.0d0 ) then
               ky = ky + 1
            else
               ky = ky - 1
            end if
            alpha(k) = atan2(x(k,2), x(k,1))
         end do
         if ( kx==-n .and. abs(ky)<n ) then
            do k = 1, n
               if (alpha(k)<0.0d0) then
                  alpha(k) = alpha(k) + 2d0*pi
               end if
            end do
         end if

         do k = 1, n
             rr = sqrt(x(k,1)**2 + x(k,2)**2)
             x(k,1) = rr*alpha(k)
             x(k,2) = x(k,3)
         end do

      case default

!     --- Incorrect value of E%jtransfo

            call errint ( E%jtransfo, 1 )
            call errint ( 2, 2 )
            call errint ( 4, 3 )
            call errchr ( 'E%jtransfo', 1 )
            call errsub ( 2147, 3, 0, 1 )

      end select  ! case (E%jtransfo)

1000  call erclos ( 'sepelp03transf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp03transf'

      end if  ! ( debug )

      end
