      subroutine sepelp05jacder
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Bi-linear quadrilateral (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: jac(:)

!     jac            Jacobian in the quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELFILLQMAT  Fill the transformation matrix qmat for the mapping of a
!                    3d surface onto a 2d surface
!     SEPELP03TRANSF Compute x as function of jtransfo
!     SEPELP05JACOB  Compute Jacobian and derivatives of basis functions
!                    Bi-linear quadrilateral (4 nodes)
!     SEPELP3D2DDER  Compute the derivatives dphi/dx for a 2d element in 3d
!     SEPELP3D2DJAC  Compute the jacobian delta and dx/dxi for a 2d element
!                    in 3d
!     SEPELTR3DQUAD  Map coordinates from a bilinear quadrilateral surface in
!                    R^3 onto a two-dimensional representation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp05jacder' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp05jacder'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ndim>2 ) then

!     --- Quadrilateral in three-dimensional space
!         transform co-ordinates

         if ( E%iqmat>5 ) call sepeltr3dquad

      end if

      if ( E%ndim==2 .or. E%iqmat==1 .or. E%iqmat>5 .or. E%jtransfo>0 ) then

!     --- standard situation

         if ( E%jtransfo>1 .and. E%jtransfo<5 ) call sepelp03transf

         call sepelp05jacob

      else

!     --- E%ndim = 3

         call  sepelp3d2djac

         B%w = B%wc*B%jac

         if ( B%fillderivn ) call sepelp3d2dder

         if ( E%iqmat>1 ) then

!        --- Fill array qmat
!            Loop over integration points

            if ( E%iqmat==2 ) then

!           --- E%iqmat = 2, standard transformation t1, t2, n

               B%inorm = 1
               B%itang = 3

            else

!           --- E%iqmat = 3, transformation in elm250 n, t1, t2

               B%inorm = 1
               B%itang = 3

            end if

            call sepelfillqmat

         end if

      end if

1000  call erclos ( 'sepelp05jacder' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp05jacder'

      end if  ! ( debug )

      end subroutine sepelp05jacder
