      subroutine sepelp05nc ( phider )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the derivatives of the basis functions in the nodes
!     of an element
!     Bi-linear quadrilateral (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: phider(B%n,B%n,B%nd)

!     phider         o    Contains derivatives of basis functions in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: dxidx(:,:,:)

!     dxidx          work array of length 9x2x3 to store the values of
!                    dxi/dx, dxi/dy, dxi/dz and deta/dx ... according to:
!                    dxi/dx(x^k) = dxdxi(k,1,1)  dxi/dy(x^k) = dxdxi(k,1,2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp05nc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp05nc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dxidx => B%dxidx


      phider(1,1,1:2) = -dxidx(1,1,1:2)-dxidx(1,2,1:2)
      phider(2,1,1:2) = dxidx(1,1,1:2)
      phider(3,1,1:2) = 0d0
      phider(4,1,1:2) = dxidx(1,2,1:2)

      phider(1,2,1:2) = -dxidx(2,1,1:2)
      phider(2,2,1:2) = dxidx(2,1,1:2)-dxidx(2,2,1:2)
      phider(3,2,1:2) = dxidx(2,2,1:2)
      phider(4,2,1:2) = 0d0

      phider(1,3,1:2) = 0d0
      phider(2,3,1:2) = -dxidx(3,2,1:2)
      phider(3,3,1:2) = dxidx(3,1,1:2)+dxidx(3,2,1:2)
      phider(4,3,1:2) = -dxidx(3,1,1:2)

      phider(1,4,1:2) = -dxidx(4,2,1:2)
      phider(2,4,1:2) = 0d0
      phider(3,4,1:2) = dxidx(4,1,1:2)
      phider(4,4,1:2) = -dxidx(4,1,1:2)+dxidx(4,2,1:2)

1000  call erclos ( 'sepelp05nc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp05nc'

      end if  ! ( debug )

      end subroutine sepelp05nc
