      subroutine sepelp06
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Bi-quadratic quadrilateral (9 nodes)
!     Old routine: elp623
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: jac(B%m)
      double precision, pointer :: w(:)
      integer :: i

!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
!     w              Contains weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELP06FIRST  compute some element independent quantities
!                    Bi-quadratic quadrilateral (9 nodes)
!     SEPELP06JACOB  Compute Jacobian and derivatives of basis functions
!                    Bi-quadratic quadrilateral (9 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     local numbering used in this subroutine with respect to jndex1:
!     7----8----9
!     |         |
!     4    5    6    numbering 1
!     |         |
!     1----2----3
!     local numbering with respect to index1:
!     7----6----5
!     |         |
!     8    9    4    numbering 2
!     |         |
!     1----2----3
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp06' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp06'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      w => B%w

      if ( E%first==0 ) call sepelp06first

!     --- Fill Gauss points

      call sepelpgauss ( B%phi, B%n )

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp06jacob ( jac )

!     --- Fill weights

      w = B%wc*abs(jac(1:B%m))

      if ( B%cart==2 .or. B%cart==3 ) then

!     --- Axi-symmetric co-ordinates

         w = w * B%xyzg(:,1)
         if ( B%cart==2 ) w = w*pi2

      end if  ! ( B%cart==2 .or. B%cart==3 )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( w, B%m, 'w' )
         write(irefwr,*) 'End sepelp06'

      end if  ! ( debug )
      call erclos ( 'sepelp06' )

      end subroutine sepelp06
