      subroutine sepelp06jacob ( jac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Bi-quadratic quadrilateral (9 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: jac(*)

!     jac            o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: eps, som
      double precision, pointer :: x(:,:)
      logical :: curved

!     eps            Local accuracy
!     som            Help parameter to compute a sum
!     x              coordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELP06JACCUR Compute basis functions and related quantities for
!                    quadratic quadrilateral (curved element)
!     SEPELP06JACSTR Compute basis functions and related quantities for
!                    quadratic quadrilateral (straight element)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp06jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp06jacob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz

!     --- check if element is straight or curved

      if ( E%ndim>2 ) then

!     --- Standard 3D case. Treat as curved

         curved = .true.

      else

!     --- 2D case, compute curved

         curved = .false.
         eps = sqreps * max(abs(x(1,1)), abs(x(3,1)), abs(x(5,1)), &
                            abs(x(7,1)), abs(x(1,2)), abs(x(3,2)), &
                            abs(x(5,2)), abs(x(7,2)) )
         curved = .false.
         som = x(1,1) + x(3,1) + x(5,1) + x(7,1)
         if ( abs(4d0*x(9,1)-som)>eps .or. &
              abs(x(2,1)+x(4,1)+x(6,1)+x(8,1)-som)>eps ) then
            curved = .true.
         else
            som = x(1,2)+x(3,2)+x(5,2)+x(7,2)
            if ( abs(4d0*x(9,2)-som)>eps .or. &
                 abs(x(2,2)+x(4,2)+x(6,2)+x(8,2)-som)>eps ) curved = .true.
         end if

      end if
      if ( debug ) write(irefwr,*) 'curved ', curved

      if ( .not. curved ) then

!     --- curved = false,  straight triangle

         call sepelp06jacstr ( jac )

      else

!     --- curved = true,  curved triangle

         call sepelp06jaccur ( jac )

      end if  ! ( .not. curved )

1000  call erclos ( 'sepelp06jacob' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phider(1,1,1), B%m, 9, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), B%m, 9, 'dphidy' )
         write(irefwr,*) 'End sepelp06jacob'

      end if  ! ( debug )

      end subroutine sepelp06jacob
