      subroutine sepelp10first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Linear triangle (3 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: lambda(:,:), phi(:,:)
      double precision :: phicen
      character (len=15), parameter :: text = 'Linear triangle'
      integer :: k, i

!     i              Loop variable
!     k              Loop variable
!     lambda         Contains values of lambda in quadrature points
!     phi            contains basis functions
!     phicen         Value of alam1*alam2*alam3 in integration point
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELPLAMBDA2D Fill the two-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp10first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp10first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lambda => B%lam
      phi => B%phi

      if ( B%cart>2 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( B%m/=4 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000
         lambda = 0d0
         phi = 0d0
         do k = 1, 3
            lambda(k,k) = 1d0
            phi(k,k) = 0d0
         end do  ! k = 1, 3
         phi(4,4) = 1d0
         lambda(:,4) = 1d0/3d0
         B%wc(1:3) = 11d0/120d0
         B%wc(4) = 27d0/120d0

         B%jdiag = 1   ! basis functions satisfy phi_i(x_j) = delta_ij

      case (2:3)

!     --- Gauss rules

         call sepelplambda2d ( lambda, text, B%rule+1, B%m, B%wc )
         do k = 1, B%m
            phicen = lambda(1,k)*lambda(2,k)*lambda(3,k)
            phi(4,k) = 27d0 * phicen
            do i = 1, 3
               phi(i,k) = lambda(i,k) - 9d0*phicen
            end do
         end do

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

      B%derivconst = .false.  ! derivatives of basis functions .not. constant

!     --- Fill basis functions in quadrature points

      B%phic = B%phi

1000  call erclos ( 'sepelp10first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp10first'

      end if  ! ( debug )

      end subroutine sepelp10first
