      subroutine sepelp13jacob ( w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     3d Tri-linear hexahedron (8 nodes)
!     Old subroutine elp632
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     hexahedron
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: w(B%m)

!     w              o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, n, i, j, k
      double precision, pointer :: dphidxi(:,:,:), dphidx(:,:,:)
      double precision, pointer :: psixi(:,:,:), dpsidx(:,:,:)
      double precision wn(B%n), detinv(B%m,3,3), detinvn(B%n,3,3)

!     detinv         Work array of length m x 3 x 3, to store the inverse
!                    of the jacobian matrix as follows:
!                     -1    k
!                    J   ( x ) =  detinv ( k, i, j )
!                     ij
!     detinvn        inverse of determinant in nodes
!     dphidx         Contains values of dphi/dx in quadrature points
!     dphidxi        Contains values of dphi/dxi in quadrature points
!     dpsidx         Contains values of dpsi/dx in quadrature points,
!                    with psi the linear basis functions
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     n              number of basis functions
!     psixi          Contains the values of the derivatives of the
!                    linear basis functions in the Gauss points in
!                    the reference element.
!     wn             See w, but now for the nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELP13JACOB1 Compute dphidx and w, defining the Jacobian of the element
!                    Quadratic tetrahedrons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp13jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp13jacob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidxi => B%phiderc
      dphidx => B%phider
      psixi => B%lamderc
      dpsidx => B%lamder

      m = B%m
      n = B%n

!     --- First compute dx/dxi
!         The first component refers to the integration point
!         The second one to the derivative
!         The third one to the coordinate direction

      call sepelp13jacob1 ( m, n, w, dphidxi, dphidx, detinv )
      if ( B%comppsi ) then
         do i = 1, B%nvert
            do k = 1, m
               do j = 1, 3
                  dpsidx(i,k,j) = dot_product(psixi(i,k,1:3), &
                                              detinv(k,1:3,j))/w(k)
               end do  ! j = 1, 3
            end do  ! k = 1, m
         end do  ! i = 1, n

      end if  ! ( comppsi )

      if ( B%fillderivn .and. B%rule>1 ) then

!     --- fillderivn = true, fill first derivatives in nodes

         dphidx => B%phidern
         dphidxi => B%phidercn

         m = n

         call sepelp13jacob1 ( m, n, wn, dphidxi, dphidx, detinvn )

      else if ( B%fillderivn ) then

!     --- fillderivn = true, fill first derivatives in nodes

         B%phidern = B%phider

      end if  ! ( B%fillderiv )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( w, B%m, 'w' )
         write(irefwr,*) 'End sepelp13jacob'

      end if  ! ( debug )
      call erclos ( 'sepelp13jacob' )

      end subroutine sepelp13jacob
