      subroutine sepelp16phixi ( phixi, lambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute dphi/dxi in quadrature points
!     Extended quadratic tetrahedron (15 nodes)
!     Old subroutine elp637
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: phixi(15,B%m,3), lambda(4,B%m)

!     lambda         i    Array containing the linear basis functions in a point
!     phixi          i    Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      double precision :: lam

!     k              Counting variable
!     lam            Help variable.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp16phixi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp16phixi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do k = 1, B%m
         lam = lambda(1,k)*lambda(3,k)*lambda(4,k)- &
            lambda(2,k)*lambda(3,k)*lambda(4,k)

         phixi(1,k,1) = -4d0*lam + 3d0*(lambda(1,k)*lambda(3,k) + &
            lambda(1,k)*lambda(4,k) - lambda(2,k)*lambda(3,k) - &
            lambda(2,k)*lambda(4,k) - lambda(3,k)*lambda(4,k)) - &
            4d0*lambda(1,k) + 1d0
         phixi(3,k,1) = -4d0*lam + 3d0*(lambda(1,k)*lambda(3,k) + &
            lambda(1,k)*lambda(4,k) + lambda(3,k)*lambda(4,k) - &
            lambda(2,k)*lambda(4,k) - lambda(2,k)*lambda(3,k)) + &
            4d0*lambda(2,k) - 1d0
         phixi(5,k,1) = -4d0*lam + 3d0*(lambda(1,k)*lambda(3,k) - &
            lambda(2,k)*lambda(3,k))
         phixi(10,k,1) = -4d0*lam + 3d0*(lambda(1,k)*lambda(4,k) - &
            lambda(2,k)*lambda(4,k))
         phixi(2,k,1) = 32d0*lam + 12d0*(lambda(2,k)*lambda(4,k) + &
            lambda(2,k)*lambda(3,k) - lambda(1,k)*lambda(4,k) - &
            lambda(1,k)*lambda(3,k)) + 4d0*(lambda(1,k)-lambda(2,k))
         phixi(4,k,1) = 32d0*lam + 12d0*(lambda(2,k)*lambda(3,k) - &
            lambda(1,k)*lambda(3,k) - lambda(3,k)*lambda(4,k)) + &
            4d0*lambda(3,k)
         phixi(6,k,1) = 32d0*lam + 12*(lambda(2,k)*lambda(3,k) + &
            lambda(3,k)*lambda(4,k) - lambda(1,k)*lambda(3,k)) - &
            4d0*lambda(3,k)
         phixi(7,k,1) = 32d0*lam + 12d0*(lambda(2,k)*lambda(4,k) + &
            lambda(3,k)*lambda(4,k) - lambda(1,k)*lambda(4,k)) - &
            4d0*lambda(4,k)
         phixi(8,k,1) = 32d0*lam + 12d0*(lambda(2,k)*lambda(4,k) - &
            lambda(1,k)*lambda(4,k) - lambda(3,k)*lambda(4,k)) + &
            4d0*lambda(4,k)
         phixi(9,k,1) = 32d0*lam
         phixi(11,k,1) = -108d0*lam + 27d0*(lambda(1,k)*lambda(3,k) - &
            lambda(2,k)*lambda(3,k))
         phixi(12,k,1) = -108d0*lam + 27d0*(lambda(1,k)*lambda(4,k) - &
            lambda(2,k)*lambda(4,k))
         phixi(13,k,1) = -108d0*lam + 27d0*lambda(3,k)*lambda(4,k)
         phixi(14,k,1) = -108d0*lam - 27d0*lambda(3,k)*lambda(4,k)
         phixi(15,k,1) = 256d0*lam

         lam = lambda(1,k)*lambda(2,k)*lambda(4,k)- &
            lambda(2,k)*lambda(3,k)*lambda(4,k)

         phixi(1,k,2) = -4d0*lam + 3d0*(lambda(1,k)*lambda(2,k) + &
            lambda(1,k)*lambda(4,k) - lambda(2,k)*lambda(3,k) - &
            lambda(2,k)*lambda(4,k) - lambda(3,k)*lambda(4,k)) - &
            4d0*lambda(1,k) + 1d0
         phixi(3,k,2) = -4d0*lam + 3d0*(lambda(1,k)*lambda(2,k) - &
            lambda(2,k)*lambda(3,k))
         phixi(5,k,2) = -4d0*lam + 3d0*(lambda(1,k)*lambda(2,k) + &
            lambda(1,k)*lambda(4,k) + lambda(2,k)*lambda(4,k) - &
            lambda(3,k)*lambda(4,k) - lambda(2,k)*lambda(3,k)) + &
            4d0*lambda(3,k) - 1d0
         phixi(10,k,2) = -4d0*lam + 3d0*(lambda(1,k)*lambda(4,k) - &
            lambda(3,k)*lambda(4,k))
         phixi(2,k,2) = 32d0*lam + 12d0*(lambda(2,k)*lambda(3,k) + &
            lambda(2,k)*lambda(4,k) - lambda(1,k)*lambda(2,k)) - &
            4d0*lambda(2,k)
         phixi(4,k,2) = 32d0*lam + 12d0*(lambda(2,k)*lambda(3,k) - &
            lambda(1,k)*lambda(2,k) - lambda(2,k)*lambda(4,k)) + &
            4d0*lambda(2,k)
         phixi(6,k,2) = 32d0*lam + 12d0*(lambda(2,k)*lambda(3,k) + &
            lambda(3,k)*lambda(4,k) - lambda(1,k)*lambda(2,k) - &
            lambda(1,k)*lambda(4,k)) + 4d0*(lambda(1,k)-lambda(3,k))
         phixi(7,k,2) = 32d0*lam + 12d0*(lambda(2,k)*lambda(4,k) + &
            lambda(3,k)*lambda(4,k) - lambda(1,k)*lambda(4,k)) - &
            4d0*lambda(4,k)
         phixi(8,k,2) = 32d0*lam
         phixi(9,k,2) = 32d0*lam + 12d0*(lambda(3,k)*lambda(4,k) - &
            lambda(2,k)*lambda(4,k) - lambda(1,k)*lambda(4,k)) + &
            4d0*lambda(4,k)
         phixi(11,k,2) = -108d0*lam + 27d0*(lambda(1,k)*lambda(2,k) - &
            lambda(2,k)*lambda(3,k))
         phixi(12,k,2) = -108d0*lam - 27d0*lambda(2,k)*lambda(4,k)
         phixi(13,k,2) = -108d0*lam + 27d0*lambda(2,k)*lambda(4,k)
         phixi(14,k,2) = -108d0*lam + 27d0*(lambda(1,k)*lambda(4,k) - &
            lambda(3,k)*lambda(4,k))
         phixi(15,k,2) = 256d0*lam

         lam = lambda(1,k)*lambda(2,k)*lambda(3,k)- &
            lambda(2,k)*lambda(3,k)*lambda(4,k)

         phixi(1,k,3) = -4d0*lam + 3d0*(lambda(1,k)*lambda(2,k) + &
            lambda(1,k)*lambda(3,k) - lambda(2,k)*lambda(3,k) - &
            lambda(2,k)*lambda(4,k) - lambda(3,k)*lambda(4,k)) - &
            4d0*lambda(1,k) + 1d0
         phixi(3,k,3) = -4d0*lam + 3d0*(lambda(1,k)*lambda(2,k) - &
            lambda(2,k)*lambda(4,k))
         phixi(5,k,3) = -4d0*lam + 3d0*(lambda(1,k)*lambda(3,k) - &
            lambda(3,k)*lambda(4,k))
         phixi(10,k,3) = -4d0*lam + 3d0*(lambda(1,k)*lambda(2,k) + &
            lambda(2,k)*lambda(3,k) + lambda(1,k)*lambda(3,k) - &
            lambda(2,k)*lambda(4,k) - lambda(3,k)*lambda(4,k)) + &
            4d0*lambda(4,k) - 1d0
         phixi(2,k,3) = 32d0*lam + 12d0*(lambda(2,k)*lambda(3,k) + &
            lambda(2,k)*lambda(4,k) - lambda(1,k)*lambda(2,k)) - &
            4d0*lambda(2,k)
         phixi(4,k,3) = 32d0*lam
         phixi(6,k,3) = 32d0*lam + 12d0*(lambda(2,k)*lambda(3,k) + &
            lambda(3,k)*lambda(4,k) - lambda(1,k)*lambda(3,k)) - &
            4d0*lambda(3,k)
         phixi(7,k,3) = 32d0*lam + 12d0*(lambda(2,k)*lambda(4,k) + &
            lambda(3,k)*lambda(4,k) - lambda(1,k)*lambda(2,k) - &
            lambda(1,k)*lambda(3,k)) + 4d0*(lambda(1,k)-lambda(4,k))
         phixi(8,k,3) = 32d0*lam + 12d0*(lambda(2,k)*lambda(4,k) - &
            lambda(1,k)*lambda(2,k) - lambda(2,k)*lambda(3,k)) + &
            4d0*lambda(2,k)
         phixi(9,k,3) = 32d0*lam + 12d0*(lambda(3,k)*lambda(4,k) - &
            lambda(1,k)*lambda(3,k) - lambda(2,k)*lambda(3,k)) + &
            4d0*lambda(3,k)
         phixi(11,k,3) = -108d0*lam - 27d0*lambda(2,k)*lambda(3,k)
         phixi(12,k,3) = -108d0*lam + 27d0*(lambda(1,k)*lambda(2,k) - &
            lambda(2,k)*lambda(4,k))
         phixi(13,k,3) = -108d0*lam + 27d0*lambda(2,k)*lambda(3,k)
         phixi(14,k,3) = -108d0*lam + 27d0*(lambda(1,k)*lambda(3,k) - &
            lambda(3,k)*lambda(4,k))
         phixi(15,k,3) = 256d0*lam
      end do  ! k = 1, B%m

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp16phixi'

      end if  ! ( debug )
      call erclos ( 'sepelp16phixi' )

      end subroutine sepelp16phixi
