      subroutine sepelp21phixi ( m, phixi, xi, eta, zeta )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute dphi/dxi in quadrature points
!     linear wedge element
!     Old subroutine elp638first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: m
      double precision, intent(in) :: xi(m), eta(m), zeta(m)
      double precision, intent(out) :: phixi(6,m,3)

!     eta            i    array that contains the values of eta in the
!                         integration points
!     m              i    number of quadrature points
!     phixi          o    Array of length 10 x m x 3 containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     xi             i    array that contains the values of xi in the
!                         integration points
!     zeta           i    array that contains the values of xeta in the
!                         integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp21phixi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp21phixi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phixi(1,:,1) = -(1d0-zeta(:))
      phixi(2,:,1) =  1d0-zeta(:)
      phixi(3,:,1) =  0d0
      phixi(4,:,1) = -zeta(:)
      phixi(5,:,1) =  zeta(:)
      phixi(6,:,1) =  0d0

      phixi(1,:,2) = -(1d0-zeta(:))
      phixi(2,:,2) =  0d0
      phixi(3,:,2) =  1d0-zeta(:)
      phixi(4,:,2) = -zeta(:)
      phixi(5,:,2) =  0d0
      phixi(6,:,2) =  zeta(:)

      phixi(1,:,3) = -(1d0-xi(:)-eta(:))
      phixi(2,:,3) = -xi(:)
      phixi(3,:,3) = -eta(:)
      phixi(4,:,3) = 1d0-xi(:)-eta(:)
      phixi(5,:,3) = xi(:)
      phixi(6,:,3) = eta(:)

1000  call erclos ( 'sepelp21phixi' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp21phixi'

      end if  ! ( debug )

      end subroutine sepelp21phixi
