      subroutine sepelrdtypconv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information about type_convection
!     Adapt methodupw if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     convection
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulestrings
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelrdtypconv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelrdtypconv'
         write(irefwr,1) 'E%methodupw, E%ipos, E%ishift', &
                          E%methodupw, E%ipos, E%ishift
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read type_convection

      E%typeconv = sepelgetintcf ( codecompr, nncodecompr, &
                                   ncodecompr, 1, 'type_convection' )

      if ( E%methodupw>=1000 ) then

         if ( E%typeconv==0 ) E%typeconv = E%methodupw/1000
         E%methodupw = mod(E%methodupw,1000)

      end if  ! ( E%methodupw>=1000 )

      if ( E%methodupw>=100 ) then

         if ( E%applyupw==0 ) E%applyupw = 2
         E%methodupw = E%methodupw-100

      end if  ! ( E%methodupw>=100 )

      if ( E%methodupw>=20 ) then

         if ( E%applyupw==0 ) E%applyupw = 1
         E%methodupw = E%methodupw-20

      end if  ! ( E%methodupw>=1000 )

1000  call erclos ( 'sepelrdtypconv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ipos, ishift/3', E%ipos, E%ishift
         write(irefwr,1) 'typeconv, methodupw', E%typeconv, E%methodupw
         write(irefwr,1) 'E%applyupw', E%applyupw
         write(irefwr,*) 'End sepelrdtypconv'

      end if  ! ( debug )

      end subroutine sepelrdtypconv
