      subroutine sepgetcoorsys
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the coordinate system as stored in iuser
!     raise ipos and E%ishift
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulestrings
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1767   Type of co-ordinates incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetcoorsys' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetcoorsys'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      B%cart = sepelgetintcf ( codecsys, nncodecsys, ncodecsys, &
                              1, codegen(3) )+1

      E%jtransfo = B%cart/10
      B%cart = mod(B%cart,10)
      jcart = B%cart

      if ( B%cart<1 .or. B%cart>3 ) then

!     --- B%cart has wrong value

         call errint ( B%cart, 1 )
         call errint ( E%ielem, 3 )
         call errint ( E%itype, 4 )
         call errint ( E%ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )

      end if  ! ( B%cart<B%cartmin .or. B%cart>B%cartmax )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgetcoorsys'

      end if  ! ( debug )
      call erclos ( 'sepgetcoorsys' )

      end subroutine sepgetcoorsys
