      subroutine sepgetrule
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!     Get the integration rule as stored in iuser
!     raise ipos and E%ishift
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     numerical_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: mrule

!     mrule          Type of integration rule
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1766   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetrule' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetrule'
         write(irefwr,1) 'ipos, ishift', E%ipos, E%ishift
         write(irefwr,1) 'mrule/1', E%iuser(E%ipos)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mrule = inirepscal ( E%iuser(E%ipos) )
      if ( debug ) write(irefwr,1) 'mrule/2', mrule
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

      if ( mrule<0 .or. mrule>6 ) then

!     --- mrule has wrong value

         call errint ( mrule, 1 )
         call errint ( E%ielem, 3 )
         call errint ( E%itype, 4 )
         call errint ( E%ielgrp, 5 )
         call errsub ( 1766, 6, 0, 0 )

      end if

      B%rule = mrule

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgetrule'

      end if  ! ( debug )
      call erclos ( 'sepgetrule' )

      end subroutine sepgetrule
