      subroutine el0903 ( iuser, user, jelp, jtime, cn, clamb, &
                          mconv, mcont, modelv, isub, isup, &
                          coeffintpnts, thetdt, nveloc, iseqvel, &
                          iseqpres, mdiv, modelrhsd, coeffnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 06-01-2016 Use sepmoduleelem
!        version  6.0  date 27-02-2010 Extra parameters
!        version  5.0  date 21-12-2008 Remove parameter ipenalty
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM903 or
!     ELD903 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array coeffintpnts is initialized with the values of the coefficients
!     If this subroutine is changed, also adapt el0260
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), coeffintpnts(m,*), cn, clamb, thetdt, &
                       coeffnodes(n,*)
      integer iuser(*), jelp, jtime, isub, isup, mconv, modelv, &
              mcont, nveloc, iseqvel(*), iseqpres(*), mdiv, modelrhsd

!     clamb          o    Parameter lambda with respect to viscosity model
!     cn             o    Power in power law model
!     coeffintpnts   o    Real array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                            In the case of eld903 or elm903:
!                                 2D                  3D
!                            1:   rho              rho
!                            2:   omega            omega
!                            3:   f1               f1
!                            4:   f2               f2
!                            5:                    f3
!                            6:   eta              eta
!                            7:   rho in nodal points (mcont=1 only)
!                            In the case of eli903:
!                            1:   f
!     coeffnodes     o    Real array in which the values of the
!                         coefficients in the nodal points are stored
!     iseqpres       o     Contains the sequence numbers of the pressure in the
!                          element vector
!     iseqvel        o     Array containing the positions of the velocities in
!                          the element vector in the sequence required by el2005
!     isub           o     Integer help parameter for do-loops to indicate the
!                          smallest coefficient number that is dependent on
!                          space
!     isup           o     Integer help parameter for do-loops to indicate the
!                          largest coefficient number that is dependent on space
!     iuser          i     Integer user array to pass user information from
!                          main program to subroutine. See STANDARD PROBLEMS
!     jelp           o     Indication of the type of basis function subroutine
!                          must be called by ELM100. Possible vaues:
!                           1:  Subroutine ELP610  (Linear line element)
!                           2:  Subroutine ELP611  (Quadratic line element)
!                           3:  Subroutine ELP620  (Linear triangle)
!                           4:  Subroutine ELP622  (Quadratic triangle)
!                           5:  Subroutine ELP621  (Bilinear quadrilateral)
!                           6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                          11:  Subroutine ELP630  (Linear tetrahedron)
!                          12:  Subroutine ELP631  (Quadratic tetrahedron)
!                          13:  Subroutine ELP632  (Trilinear brick)
!                          14:  Subroutine ELP633  (Triquadratic brick)
!     jtime          o     Integer parameter indicating if the mass matrix for
!                          the time-dependent equation must be computed
!                          (>0) or not ( = 0)
!                          Possible values:
!                            0:  no mass matrix
!                            1:  diagonal mass matrix with constant coefficient
!                            2:  diagonal mass matrix with variable coefficient
!                           11:  full mass matrix with constant coefficient
!                           12:  full mass matrix with variable coefficient
!                          101:  refers to the special possibility of the theta
!                                method directly applied
!                          102:  refers to the special possibility of the theta
!                                method directly applied with variable rho
!     mcont          o     Defines the type of continuity equation
!                          Possible values:
!                          0: incompressible flow
!                          1: compressible flow: div (rho u ) = 0
!                          5: special option for pressure mass matrix
!                             see elm903
!     mconv          o     Defines the treatment of the convective terms
!                          Possible values:
!                          0: convective terms are skipped (Stokes flow)
!                          1: convective terms are linearized by Picard
!                             iteration
!                          2: convective terms are linearized by Newton
!                             iteration
!                          3: convective terms are linearized by the incorrect
!                             Picard iteration
!                          4: convective terms are linearized by successive
!                             substitution
!     mdiv           o     Defines if divergence right-hand side is used (1)
!                          or not (0)
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                                  to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                                  tensor components stored in positions 19-24
!     modelv         o     Defines the type of viscosity model
!                          Possible values:
!                             1:  newtonian liquid
!                             2:  power law liquid
!                             3:  carreau liquid
!                             4:  plastico viscous liquid
!                           100:  user provided model depending on grad u
!                           101:  user provided model depending on the second
!                                 invariant
!                           102:  user provided model depending on the second
!                                 invariant, &
!                                 the co-ordinates and the velocity
!                           103:  user provided model depending on the second
!                                 invariant, &
!                                 the co-ordinates, the velocity, and old
!                                 solutions
!     nveloc         o     Number of velocity parameters
!     thetdt         o     1/(theta dt) for the case of the theta method
!                          immediately applied
!     user           i     Real user array to pass user information from
!                          main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, kchois, mrule, jcheld, ishift, &
              ioldm, mcoefconv, interpol, imesh
      double precision penalp
      logical compress

!     compress       If true the pressure must be computed
!     imesh          dummy
!     interpol       dummy
!     ioldm          dummy
!     ishift         Shift in coefficient sequence number
!     jcheld         Value of icheld corrected for different storage schemes
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM903
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD903 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELI903 calls this subroutine, only
!                        integrals must be computed
!     mcoefconv      Defines how the convective term is treated:
!                    0: standard case
!                    1: special case: the convective term is defined as
!                          c_conv_1 u du/dx + c_conv_2 v du/dy
!                          c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
!     penalp         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0900COEFF    Initialize coefficients for elements of type 900-902
!                    and 916-918
!     EL0900CONS     Set some constants for subroutines el0900/3/5
!     EL0900FLCOEFFS Initialize coefficients for elements of type 900-902
!                    and 916-918
!     EL0900INIT     Make some initialization for subroutines el0900/3/5
!                    depending on icheld
!     EL0903RULE     ?
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutine elm903
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0903' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0903'
         write(irefwr,1) 'imas, inpelm', imas, inpelm
  1      format ( a, 1x, (10i6) )
         call prinin ( iuser, 50, 'iuser' )
         call prinrl ( user, 20, 'user' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters in commons

      call elcomm

!     --- initialize some other parameters

      call el0900cons ( jtime, jconsf, ncomp, ndim, kchois, &
                        modelv, modelrhsd, mcont, mrule, jcheld, &
                        icheld, compress )
      if ( ierror/=0 ) go to 1000
      jsecnd = 2   ! is only used in case of upwind
                   ! viscosity is a scalar

!                  --- extra initializations

      jind = 0
      n = inpelm
      jtrans = 0
      jelp = ishape
      nvert = n
      ishift = 0

!     --- Compute kchois

      call el0900init ( icheld, ncoef, compress, itype, jcheld, &
                        ielgrp, ix, ndim, jdegfd, ncomp, jderiv, &
                        kchois, icount, inpelm )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ncoef, kchois, jelp, n', &
                          ncoef, kchois, jelp, n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      call el0900coeff ( ncoef, kchois, iuser, user, jtime, &
                         thetdt, jcheld, compress, penalp, &
                         ioldm, mcoefconv, mconv, mcont, &
                         modelrhsd, modelv, interpol, itype, ishift, &
                         mrule )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      call el0903rule ( mrule, irule, compress, ishape, icheld, &
                        n, m, jtrans, jelp, nveloc, ndim, &
                        iphi, igausp, jdiag, nvert, ncoef, &
                        jcart, mn, jdercn, ielem, ielgrp, &
                        itype, kchois, imas, iseqvel, iseqpres, &
                        npsi )
      if ( ierror/=0 ) go to 1000

!     --- Fill coefficients in coeffintpnts and do some checks

      call el0900flcoeffs ( ncoef, modelv, mcont, mcoefconv, &
                            coeffintpnts, jtime, jcheld, cn, clamb, &
                            kchois, iuser, modelrhsd, mconv, &
                            imesh, isub, isup, mdiv, &
                            user, ishift, coeffnodes )

      irule = irule + 100*interpol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'irule, n, m', irule, n, m
         write(irefwr,*) 'End el0903'

      end if
      call erclos ( 'el0903' )

      end
