      subroutine el2720 (B, dphidx, nd, phi, xg)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 13-04-2005 Extra parameters for axi-symmetry (RvL)
!        version  1.1  date 01-11-2002 Extension for 2D shapes (RvL)
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill matrix B for general nonlinear solid element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nd
      double precision B(nd,ndim,n,m), dphidx(n,m,ndim), phi(n,m), &
                       xg(m,ndim)

!     B              o    Contains the called B matrix
!     dphidx         i    Array of length n x m x 2  containing the derivatives
!                         of the basis functions in the sequence:
!                                      k
!                         d phi / dx (x )= dphidx (i,k,1);
!                              i
!                                      k
!                         d phi / dy (x )= dphidx (i,k,2);
!                              i
!     nd             i    Number of rows in B
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     xg             i    Array of length 4x2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l

!    i       General loop variable
!    j       General loop variable
!    k       General loop variable
!    l       General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (ifirst==0) then

!     --- set B equal to zero in the first call

         do l = 1, nd
            do j = 1, ndim
               do i = 1, n
                  do k = 1, m
                     B(l,j,i,k) = 0d0
                  end do
              end do
            end do
         end do
      end if

!     --- Fill in B-matrix

      if (ndim==2) then
       do i = 1, n
          do k = 1, m
             B(1,1,i,k) = dphidx(i,k,1)
             B(2,2,i,k) = dphidx(i,k,2)
             B(3,1,i,k) = dphidx(i,k,2)
             B(4,2,i,k) = dphidx(i,k,1)

!            --- axisymmetric case

             if (jcart==2) then
                B(5,1,i,k) = phi(i,k)/xg(k,1)
             end if

          end do
       end do
      else if (ndim==3) then
       do i = 1, n
         do k = 1, m
            B(1,1,i,k) = dphidx(i,k,1)
            B(2,2,i,k) = dphidx(i,k,2)
            B(3,3,i,k) = dphidx(i,k,3)
            B(4,1,i,k) = dphidx(i,k,2)
            B(5,2,i,k) = dphidx(i,k,3)
            B(6,3,i,k) = dphidx(i,k,1)
            B(7,2,i,k) = dphidx(i,k,1)
            B(8,3,i,k) = dphidx(i,k,2)
            B(9,1,i,k) = dphidx(i,k,3)
         end do
       end do
      end if

      end
