      subroutine el4021 (ichois,ncomploc,solold,index2,indloc, &
          uold,uoldg,duoldx,phig,dphidx,psig,dpsidx, &
          truc,work,nloc,intpnt,ns)
!
! ======================================================================
!
!        programmer   jaap van der zanden
!
!        version  1.5  date 12-12-2012 Long integers
!        version 1.4  date   22-12-1992 minor improvement
!        version 1.3  date   22-08-1991 Zdenek: no dimension, cdc, ce
!
!
!   copyright (c) 1985,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      double precision solold(*), uold(*), uoldg(*), duoldx(*), phig(*), &
             dphidx(*), psig(*), dpsidx(*), truc(*), work(*)
      integer ichois,ncomploc,nloc,intpnt,ns, index2(*)
      integer indloc(*)
! **********************************************************************
!
!   DESCRIPTION
!
!    the old solution is retrieved from solold with help of index2;
!    the old solution is transformed back to the situation with
!    uc,vc in the centroid with help of truc,trvc.
!
!    the derivatives of the old solution are calculated on request
!    (ichois=-1)
!
!    6-point triangle with --internally-- 7 points for u_x,u_y  /  u_r,u_z
!            with 7 point integration rule for all --ncomploc-- variables
!
! **********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
!     ichois=0   no derivatives of solold
!            -1  derivatives of uold,uold with respect to x1 and x2
!
!     solold   solution from previous iteration
!
!     index2   pointers to unknowns in nodal points
!
!     indloc      pointers to unknowns in h as most easy to build the
!              system of equations for an element
!
!     uold     transformed previous solution (also in centroid)
!
!     uoldg      previous solution in integration points
!
!     duoldx,duoldy   derivatives of uold in integration points
!     dvoldx,dvoldy
!     dwoldx,dwoldy
!     dtoldx,dtoldy
!     etcetera
!
!     phig       basis functions in integration points
!
!     dphidx,dphidy   derivatives of basis functions in integr. points
!
!     psig       basis functions in integration points
!
!     dpsidx,dpsidy   derivatives of basis functions in integr. points
!
!     truc,trvc  transformation matrices (columns) for velocities in
!                centroid
!
!     work       help array for intermediate storage of solold , e.g.
!                 work(i)=uold(i), i=1,..,n,
!                 work(i+nu)=uold(n+i), i=1,..,n;
!                 work(i+nu+nv)=uold(n2+i), i=1,..,ns;
!                 work(i+n+n+ns)=told(i), i=1,..,nt.
!
!                at output work(i)=solold in nodal points
!
!     n          number of nodal points including the centroid
!                2*(n-1)+ns = number of degees of freedom for velocity
!
!     intpnt     number of integration points
!
! **********************************************************************
!
!   ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!     None.
! **********************************************************************
!
!   LOCAL  VARIABLES
!
! **********************************************************************
      integer n1,n2,nm,nms,jcount,n22,n2i,n3i
      double precision sum1,sum2,sum3,sum4
      integer i,ik,jcomp,k,nh
      save n1,n2,nm,nms,jcount,n22,n2i,n3i
! ======================================================================
!
      if (ifirst.eq.0) then
         n1=nloc-1
         n2=nloc+nloc
         nm=nloc*intpnt
         nms=ns*intpnt
         if (ncomploc.ge.2) then
            jcount=2*n1+ns*(ncomploc-2)
         else
            jcount=2*n1
         end if
         n22=n2-2
         n2i=intpnt+intpnt
         n3i=n2i+intpnt
      end if
!
!*  fill work temporarily with == reordered == solution
!
      do 10 i=1,jcount
         work(indloc(i))=solold(index2(i))
10    continue
!
!*    compute uc,vc in centroid and store in uold(nloc),uold(n+n)
!
      sum1=0.0d0
      sum2=0.0d0
      do 110 i=1,n22
         if (abs(work(i)).gt.epsmac) then
            if (abs(truc(i)).gt.epsmac) &
               sum1=sum1+truc(i)*work(i)
            if (abs(truc(n22+i)).gt.epsmac) &
               sum2=sum2+truc(n22+i)*work(i)
         end if
110   continue
      uold(nloc)=sum1
      uold(nloc+nloc)=sum2
      do 120 i=1,n1
         uold(i)=work(i)
         uold(nloc+i)=work(i+n1)
120   continue
      if (ncomploc.ge.3) then
         do 121 i=1,(ncomploc-2)*ns
            uold(n2+i)=work(n22+i)
121      continue
      end if
!
!*  compute uoldg and uoldg in integration points
!
      do 160 i=1,intpnt
         ik=i
         sum1=0.0d0
         sum2=0.0d0
         do 150 k=1,nloc
            sum1=sum1+phig(ik)*uold(k)
            sum2=sum2+phig(ik)*uold(nloc+k)
            ik=ik+intpnt
150      continue
         uoldg(i)=sum1
         uoldg(intpnt+i)=sum2
160   continue
      if (ncomploc.ge.3) then
         nh=n2-ns
         do 163 jcomp=3,ncomploc
            nh=nh+ns
            do 162 i=1,intpnt
               ik=i
               sum1=0.0d0
               do 161 k=1,ns
                  sum1=sum1+psig(ik)*uold(nh+k)
                  ik=ik+intpnt
161            continue
               uoldg((jcomp-1)*intpnt+i)=sum1
162         continue
163      continue
      end if
      if (ichois.eq.0) return
!
!*  newton-cotes integration or gauss integration
!*  derivatives of uold,vold in integration points
!   store them in duoldx
!
      do 220 i=1,intpnt
         sum1=0.0d0
         sum2=0.0d0
         sum3=0.0d0
         sum4=0.0d0
!
!*    dphidx(ik) = dphi_i / dx in point k
!
         ik=i
         do 210 k=1,nloc
            if (abs(uold(k)).gt.epsmac) then
               sum1=sum1+uold(k)*dphidx(ik)
               sum2=sum2+uold(k)*dphidx(nm+ik)
            end if
            if (abs(uold(nloc+k)).gt.epsmac) then
               sum3=sum3+uold(nloc+k)*dphidx(ik)
               sum4=sum4+uold(nloc+k)*dphidx(nm+ik)
            end if
            ik=ik+intpnt
210      continue
         duoldx(i)=sum1
         duoldx(intpnt+i)=sum2
         duoldx(n2i+i)=sum3
         duoldx(n3i+i)=sum4
220   continue
      if (ncomploc.ge.3) then
         nh=n2-ns
         do 223 jcomp=3,ncomploc
            nh=nh+ns
            do 222 i=1,intpnt
               sum1=0.0d0
               sum2=0.0d0
!
!*    dpsidx(ik) = dpsi_i / dx in point k
!
               ik=i
               do 221 k=1,ns
                  if (abs(uold(nh+k)).gt.epsmac) then
                     sum1=sum1+uold(nh+k)*dpsidx(ik)
                     sum2=sum2+uold(nh+k)*dpsidx(nms+ik)
                  end if
                  ik=ik+intpnt
221            continue
               duoldx(2*(jcomp-1)*intpnt+i)=sum1
               duoldx((2*jcomp-1)*intpnt+i)=sum2
222         continue
223      continue
      end if
      end
