      subroutine el4024 (ichois,sm,sv,truc,b,ndim,nloc,lrowloc)
!
! ======================================================================
!
!        programmer   jaap van der zanden
!
!        version 3.0  date   04-06-1993 improve error for lrowloc>ndim*n;
!                                      improve comment)
!        version 2.3  date   22-12-1992 expl. decl.
!        version 2.2  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 2.1  date   19-01-1987 save
!        version 2    date   04-04-1985
!
!
!   copyright (c) 1984,1993  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
      use sepmodulecactl
      implicit none
      integer ichois, ndim, nloc, lrowloc
      double precision sm(*), sv(*), truc(*), b(*)
! **********************************************************************
!
!   DESCRIPTION
!
!    the element matrix sm (lrowloc)*(lrowloc) is transformed to
!        a matrix (lrowloc-2)*(lrowloc-2) ; i.e the velocities in the
!        centroid of the element are eliminated (ichois=1,3)
!
!    the element vector sv (k*n+m) is transformed to s (k*n-k+m)
!        if required (ichois=2,3)
!
! **********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
!     b       i    help array (lrowloc*lrowloc) or help vector(lrowloc)
!     ichois  i    Choice parameter
!                  ichois=1  transform array
!                        =2  transform vector
!     lrowloc    i    length of row of input array sm
!     n       i    number of basis functions for first ndim components
!     ndim    i    dimension of the space
!     sm      i/o  input/output array (to be transformed)
!     sv      i/o  input/output vector (to be transformed)
!     truc    i    2n-ndim elements of n^th row of transformation matrix and
!                  2n-ndim elements of 2n^th row
!
!    the transpose of the transformation matrix has the form (2-dim) (u,v,T)
!
! column
!     1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
! r 1 1           t             t
! o 2   1         r             r
! w 3     1       u             v
!   4       1     c             c
!   5         1   t             t
!   6           1 r             r
!   7             u 1           v
!   8             c   1         c
!   9             t     1       t
!   0             r       1     r                  = R (transpose)
!   1             u         1   u
!   2             c           1 c
!   3                             1
!   4                               1
!   5                                 1
!   6                                   1
!   7                                     1
!   8                                       1
!     1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
! column
!
!   Blanks are ZERO!  truc and trvc define the transformation from
!                     7 points to 6 points (i.e. n=7, ndim=2, lrowloc=20)
!
!   If the problem is a (u,v) problem only, then this would be lrowloc=14
!
!   The velocities in the centroid of the element are eliminated
!
!    The new variables depend on the old by the inverse of the relations
!    u_c=truc(i)*u_new(i), i=1,n22; v_c=...
!    the new u-variables now also depend on the v-variables, and via v
!    on, e.g. the temperature (if lrowloc>2*n)
!
! **********************************************************************
!
!     ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!     SUBROUTINES  CALLED
!
!     None.
! **********************************************************************
!
!     LOCAL PARAMETERS
!
!     i       counting variable
!     ilr     counting variable
!     ilr1    counting variable
!     ilr2    counting variable
!     ilr3    counting variable
!     j       counting variable
!     lr2     lrowloc-2 (lrowloc-ndim) new length of row and column
!     m       lrowloc-ndim*n
!     n1      nloc-1
!     n2      2*n
!     n21     2*nloc-1
!     n22     2*n-2
!     n2lr2   n21*lr2 pointer to n^th row
!     nlr2    n1*lr2  pointer to 2n^th row
!     sinn1   help variable
!     sinn2   help variable
!     trui    help variable
!     trvi    help variable
!     vn      help variable
!     vn2     help variavle
!
! **********************************************************************
      integer i, j, ilr, ilr1, ilr2, ilr3, nlr2, n2lr2
      double precision sinn1, sinn2, trui, trvi, vn, vn2
      integer n1,n2,mloc,n21,n22,lr2
      save    n1,n2,mloc,n21,n22,lr2
! ======================================================================
!
      if (ifirst.eq.0) then
         n1=nloc-1
         n2=nloc+nloc
         mloc=lrowloc-n2
         n21=n2-1
         n22=n2-2
         lr2=lrowloc-2
      end if
      if (ichois.eq.2) goto 100
!
!*    first : b (lrowloc)*(lrowloc-2) = sm (lrowloc)*(lrowloc) * r (lrowloc)*(lrowloc-2)
!             r(n,j)=truc(j) , r(2n,j)=truc(n22+j)
!
!  the storage is row-wise (1-d)
!
!  b(i,j)=sm(i,j)+sm(i,n)*truc(j)+sm(i,2n)*trvc(j) , i=1,lrowloc; j=1,n1
!
!  b(i,j)=sm(i,n)*truc(j)+sm(i,j+1)+sm(i,2n)*trvc(j) , i=1,lrowloc; j=n,n22
!
!  b(i,j)=                sm(i,j+2)                  , i=1,lrowloc; j=n21,lr2
!
      ilr=0
      ilr2=0
      do 20 i=1,lrowloc
         sinn1=sm(ilr+nloc)
         sinn2=sm(ilr+n2)
         do 10 j=1,n1
            b(ilr2+j)=sm(ilr+j)+sinn1*truc(j)+sinn2*truc(n22+j)
10       continue
         ilr1=ilr+1
         do 11 j=nloc,n22
            b(ilr2+j)=sinn1*truc(j)+sm(ilr1+j)+sinn2*truc(n22+j)
11       continue
         if (mloc.ne.0) then
            ilr1=ilr+2
            do 12 j=n21,lr2
               b(ilr2+j)=sm(ilr1+j)
12          continue
         end if
         ilr=ilr+lrowloc
         ilr2=ilr2+lr2
20    continue
!
!*    second : sm (lrowloc-2)*(lrowloc-2) = r(transp) * b (lrowloc)*(lrowloc-2)
!
!  the storage is row-wise (1-dim)
!
!  sm(i,j)=b(i,j)+truc(i)*b(n,j)+trvc(i)*b(2n,j) , i=1,n1; j=1,lr2
!
!  sm(i,j)=truc(i)*b(n,j)+b(i+1,j)+trvc(i)*b(2n,j) , i=nloc,n22; j=1,lr2
!
!  sm(i,j)=                b(i+2,j)                  , i=n21,lr2; j=1,lr2
!
      nlr2=n1*lr2
      n2lr2=n21*lr2
      ilr2=0
      do 40 i=1,n1
         trui=truc(i)
         trvi=truc(n22+i)
         do 30 j=1,lr2
            sm(ilr2+j)=b(ilr2+j)+trui*b(nlr2+j)+trvi*b(n2lr2+j)
30       continue
         ilr2=ilr2+lr2
40    continue
      do 60 i=nloc,n22
         trui=truc(i)
         trvi=truc(n22+i)
         ilr3=ilr2+lr2
         do 50 j=1,lr2
            sm(ilr2+j)=trui*b(nlr2+j)+b(ilr3+j)+trvi*b(n2lr2+j)
50       continue
         ilr2=ilr2+lr2
60    continue
      if (mloc.gt.0) then
         do 80 i=n21,lr2
            ilr3=ilr2+2*lr2
            do 70 j=1,lr2
               sm(ilr2+j)=b(ilr3+j)
70          continue
            ilr2=ilr2+lr2
80       continue
      end if
      if (ichois.eq.1) return
!
!*   transform vector  s = r (transp) * s
!
100   vn=sv(nloc)
      vn2=sv(n2)
      do 110 i=1,n1
         sv(i)=sv(i)+truc(i)*vn+truc(n22+i)*vn2
         sv(i+n1)=truc(i+n1)*vn+sv(i+nloc)+truc(n22+i+n1)*vn2
110   continue
!
!*    shift last elements if present
!
      if (mloc.eq.0) return
      do 120 i=n21,lr2
         sv(i)=sv(i+2)
120   continue
      end
