      subroutine el4028(ichois,itcoor,mloc,ndim, &
                        modelv,etha,cn,cl,param,work, &
                        xg,u,dudx,etheff,secinv,detdsc)
!
! ======================================================================
!
!        programmer   jaap van der zanden
!    version 4.0  date   12-12-1995 add modelv=5,6,7
!    version 3.9  date   08-04-1994 improve estimate etheff for
!                                   modelv=4 and small secinv
!    version 3.8  date   11-01-1994 improve `save'
!
!
!   copyright (c) 1984-1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!   DESCRIPTION
!
!
!     this subroutine tests and computes model-dependent
!     quantities for a generalized newtonian liquid
!                    and plastico-viscous liquids
!     available are :  modelv=1, newtonian model
!                      modelv=2, power law model
!                      modelv=3, 3 parameter carreau model
!                      modelv=4, 3 parameter plastico-viscous liquid
!                               cn=1 : Bingham model
!                               cn=2 : Casson model
!                      modelv=5, Papanastasiou plastico-viscous liquid
!                      modelv=6, bi-viscosity plastico-viscous liquid
!                      modelv=7, Carreau-Yasuda model
!                      modelv=100 , user defined effective viscosity
!                                   based on gradv
!                      modelv=101 , user defined effective viscosity
!                                   based on second invariant
!                      modelv=102 , user defined effective viscosity
!                                   based on x,u,second invariant
!
!            modelv=2,3 initiated by Hans Damsteegt ( 1-10-1983)
!            modelv=5,6 initiated by Jacques Vink   (30-08-1994)
!
! **********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer ichois, itcoor, mloc, ndim, modelv
      double precision cn, cl
      double precision etha(*), param(*), work(*), xg(*), u(*), dudx(*), &
             etheff(*), secinv(*), detdsc(*)

!     cl      i    carreau model parameter lambda
!                  bingham (plastico-viscous) model parameter s
!     cn      i    power (law model) parameter n
!                  plastico-viscous model parameter n
!     detdsc  i    2*(derivative of etheff to second invariant)
!                  for newton linearization  (for modelv=2,3 only)
!     dudx    i    Real m x ndim  array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j )
!                    j     k
!                  If jdercn = 0 in common block celint, then only
!                  dudx (1,*) is filled because dudx is then a constant
!                  per element
!                  dudx is only filled if jderiv > 0
!     etha    i    Array containing the viscosity in the integration points
!     etheff  i    Array containing the computed effective viscosity in
!                  the integration points
!     ichois  i    Choice parameter
!     itcoor  i    ?
!     m       i    number of integration points
!     modelv  i    Defines the type of viscosity model
!                  Possible values:
!                    1:  newtonian liquid
!                    2:  power law liquid
!                    3:  carreau liquid
!                    4:  plastico viscous liquid
!                    5:  bi-viscosity plastico-viscous liquid
!                  100:  user provided model depending on grad u
!                  101:  user provided model depending on the second invariant
!                  102:  user provided model depending on the second invariant, &
!                        the co-ordinates and the velocity
!     ndim    i    Dimension of the space.
!     param   i    Extra parameters for the model
!     secinv  i    Array containing the second invariant in the integration
!     node
!     u       i    array of length mdecl containing the values of the
!                  second coefficient in the integration points
!     work    i    Work array of length m x 3, to store products of
!                  alpha   x w temporarily
!                  ij
!     xg      i    Array of length 4x2 containing the co-ordinates of the Gauss
!                  points in the reference element for a four point Gauss rule
!
! **********************************************************************
!
!   COMMON  BLOCKS
!
! **********************************************************************
!
!   LOCAL  PARAMETERS
!
!
      integer i, k
      double precision cfac, expn, factor, fnv000, fnv001, fnv002, &
                       gradv(9), secnd, secndc, term, u1, u2, u3, &
                       xgk, x1, x2, x3
      integer m2,m3,m4,m5,m6,m7,m8
      save    m2,m3,m4,m5,m6,m7,m8, expn, factor, secndc
!
!     cfac    ?
!     expn    Value of an exponent
!     factor  Multiplication factor
!     gradv   Array containing the gradient vector in
!             the integration points
!     i       Counting variable
!     k       Counting variable
!     m2      ?
!     m3      ?
!     m4      ?
!     m5      ?
!     m6      ?
!     m7      ?
!     m8      ?
!     secnd   ?
!     secndc  ?
!     term    Value of a term in an expression
!     u1      ?
!     u2      Second component of velocity vector
!     u3      Third component of velocity vector
!     x1      x-coordinate of first point in curve
!     x2      x-coordinate of last point in curve
!     x3      Third component of co-ordinate vector
!     xgk     ?
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!   ERCLOS  reset old name of calling tree
!   EROPEN  add name of routine to calling tree
!   ERREAL  Put real in error message
!   ERRINT  Put integer in error message
!   ERRSUB  report error
!   FNV000  user-defined viscosity model (modelv=100)
!   FNV001  user-defined viscosity model (modelv=101)
!   FNV002  user-defined viscosity model (modelv=102)
!
! **********************************************************************
!
!   I / O
!
!   none
!
! **********************************************************************
!
!   ERROR  MESSAGES
!
!   234  computed effective viscosity is less than or equal to zero
!
! **********************************************************************
!
!   PSEUDO  CODE
!
!   trivial
! ======================================================================
!
!     --- computation of effective viscosity

      if (modelv.eq.1) then

!     --- newtonian shear-independent viscosity (modelv=1)

         do 5 i=1,mloc
            etheff(i)=etha(i)
5        continue
         goto 500
      else if ((modelv.ne.100).and.(modelv.ne.101).and.(modelv.ne.102)) &
        then

!     --- store parameter etha in work array for other models

         do 8 i=1,mloc
            work(i)=etha(i)
8        continue
      end if

!     --- computation of effective viscosity from first kinematic tensor
!                     or grad(v) (for modelv=100), or secinv (modelv=101,102)

      if (ifirst.eq.0) then
         m2=mloc+mloc
         m3=m2+mloc
         if (itcoor.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
         else if (ndim.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
            m6=m5+mloc
            m7=m6+mloc
            m8=m7+mloc
         end if
      end if
      if ((itcoor.eq.0).and.(ndim.eq.2)) then

!     --- 2-dim cartesian coordinates

         do 20 k=1,mloc
            gradv(1)=dudx(k)
            gradv(2)=dudx(m2+k)
            gradv(3)=dudx(mloc+k)
            gradv(4)=dudx(m3+k)
            if (modelv.eq.100) then
               etheff(k)=fnv000(gradv)
               if (etheff(k).le.0) then
                  call eropen('el4028')
                  call erreal(etheff(k),1)
                  call errint(ielem,1)
                  call errint(itype,2)
                  call errint(ielgrp,3)
                  call errsub ( 234, 3, 1, 0)
                  call erclos('el4028')
               end if
            else
               secinv(k)=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4)) &
                         +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
            end if
20       continue
      else if ((itcoor.eq.1).and.(ndim.eq.2)) then

!     --- axisymmetrical coordinates (no swirl)

         do 21 k=1,mloc
            gradv(1)=dudx(k)
            gradv(2)=dudx(m2+k)
            gradv(3)=dudx(mloc+k)
            gradv(4)=dudx(m3+k)
            gradv(5)=u(k)/xg(k)
            if (modelv.eq.100) then
               etheff(k)=fnv000(gradv)
               if (etheff(k).le.0) then
                  call eropen('el4028')
                  call erreal(etheff(k),1)
                  call errint(ielem,1)
                  call errint(itype,2)
                  call errint(ielgrp,3)
                  call errsub ( 234, 3, 1, 0)
                  call erclos('el4028')
               end if
            else
               secinv(k)=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4) &
                           +gradv(5)*gradv(5)) &
                         +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
            end if
21       continue
      else if ((itcoor.eq.2).and.(ndim.eq.2)) then

!     --- 2-dim polar coordinates

         do 22 k=1,mloc
            xgk=xg(k)
            gradv(1)=dudx(k)
            gradv(2)=dudx(m2+k)
            gradv(3)=(dudx(mloc+k)-u(mloc+k))/xgk
            gradv(4)=(dudx(m3+k)+u(k))/xgk
            if (modelv.eq.100) then
               etheff(k)=fnv000(gradv)
               if (etheff(k).le.0) then
                  call eropen('el4028')
                  call erreal(etheff(k),1)
                  call errint(ielem,1)
                  call errint(itype,2)
                  call errint(ielgrp,3)
                  call errsub ( 234, 3, 1, 0)
                  call erclos('el4028')
               end if
            else
               secinv(k)=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4)) &
                         +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
            end if
22       continue
      else if ((itcoor.eq.3).and.(ndim.eq.2)) then

!     --- axisymmetrical coordinates (with swirl)

         do 23 k=1,mloc
            xgk=xg(k)
            gradv(1)=dudx(k)
            gradv(2)=dudx(m4+k)
            gradv(3)=dudx(m2+k)
            gradv(4)=-u(m2+k)/xgk
            gradv(5)=u(k)/xgk
            gradv(6)=0.0d0
            gradv(7)=dudx(mloc+k)
            gradv(8)=dudx(m5+k)
            gradv(9)=dudx(m3+k)
            if (modelv.eq.100) then
               etheff(k)=fnv000(gradv)
               if (etheff(k).le.0) then
                  call eropen('el4028')
                  call erreal(etheff(k),1)
                  call errint(ielem,1)
                  call errint(itype,2)
                  call errint(ielgrp,3)
                  call errsub ( 234, 3, 1, 0)
                  call erclos('el4028')
               end if
            else
               secinv(k)=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                           +gradv(9)*gradv(9)) &
                         +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                         +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                         + gradv(8)*gradv(8)
            end if
23       continue
      else if ((itcoor.eq.0).and.(ndim.eq.3)) then

!     --- 3-dim cartesian coordinates

         do 30 k=1,mloc
            gradv(1)=dudx(k)
            gradv(2)=dudx(m3+k)
            gradv(3)=dudx(m6+k)
            gradv(4)=dudx(mloc+k)
            gradv(5)=dudx(m4+k)
            gradv(6)=dudx(m7+k)
            gradv(7)=dudx(m2+k)
            gradv(8)=dudx(m5+k)
            gradv(9)=dudx(m8+k)
            if (modelv.eq.100) then
               etheff(k)=fnv000(gradv)
               if (etheff(k).le.0) then
                  call eropen('el4028')
                  call erreal(etheff(k),1)
                  call errint(ielem,1)
                  call errint(itype,2)
                  call errint(ielgrp,3)
                  call errsub ( 234, 3, 1, 0)
                  call erclos('el4028')
               end if
            else
               secinv(k)=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                           +gradv(9)*gradv(9)) &
                          +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                          +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                          +(gradv(6)+gradv(8))*(gradv(6)+gradv(8))
            end if
30       continue
      else if ((itcoor.eq.4).and.(ndim.eq.3)) then

!     --- 3-dim cylindrical coordinates

         do 31 k=1,mloc
            xgk=xg(k)
            gradv(1)=dudx(k)
            gradv(2)=dudx(m3+k)
            gradv(3)=dudx(m6+k)
            gradv(4)=(dudx(mloc+k)-u(mloc+k))/xgk
            gradv(5)=(dudx(m4+k)+u(k))/xgk
            gradv(6)=dudx(m7+k)/xgk
            gradv(7)=dudx(m2+k)
            gradv(8)=dudx(m5+k)
            gradv(9)=dudx(m8+k)
            if (modelv.eq.100) then
               etheff(k)=fnv000(gradv)
               if (etheff(k).le.0) then
                  call eropen('el4028')
                  call erreal(etheff(k),1)
                  call errint(ielem,1)
                  call errint(itype,2)
                  call errint(ielgrp,3)
                  call errsub ( 234, 3, 1, 0)
                  call erclos('el4028')
               end if
            else
               secinv(k)=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                           +gradv(9)*gradv(9)) &
                          +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                          +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                          +(gradv(6)+gradv(8))*(gradv(6)+gradv(8))
            end if
31       continue
      end if
      if (modelv.eq.101) then
         do 51 k=1,mloc
            secnd=secinv(k)
            etheff(k)=fnv001(secnd)
            if (etheff(k).le.0) then
               call eropen('el4028')
               call erreal(etheff(k),1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 234, 3, 1, 0)
               call erclos('el4028')
            end if
51       continue
      else if (modelv.eq.102) then
         do 52 k=1,mloc
            secnd=secinv(k)
            x1=xg(k)
            x2=xg(mloc+k)
            if (ndim.eq.3) x3=xg(m2+k)
            u1=u(k)
            u2=u(mloc+k)
            if ((itcoor.eq.3).and.(ndim.eq.2)) then
               u2=u(m2+k)
               u3=u(mloc+k)
            else if (ndim.eq.3) then
               u3=u(m2+k)
            end if
            etheff(k)=fnv002(x1,x2,x3,u1,u2,u3,secnd)
            if (etheff(k).le.0) then
               call eropen('el4028')
               call erreal(etheff(k),1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 234, 3, 1, 0)
               call erclos('el4028')
            end if
52       continue
      end if
      if (modelv.eq.100.or.modelv.eq.101.or.modelv.eq.102) goto 500

!     --- for various models:
!          ichois = 0,1 computation of etheff
!          ichois = 2, computation of etheff and detdsc=2*deteff/dsecinv

      if (modelv.eq.2) then

!     --- powerlaw model
!          ichois=0,1,...

         expn=(cn-1.0d0)/2.0d0
      if (expn.le.0) then
         do 121 i=1,mloc
            if (secinv(i).lt.epsmac) secinv(i)=epsmac
121      continue
      end if
      do 122 i=1,mloc
         etheff(i)=work(i)*(secinv(i)**expn)
122   continue
      if (ichois.eq.2) then

!     --- ichois=2

            expn=(cn-3.0d0)/2.0d0
            if (expn.le.0) then
               do 126 i=1,mloc
                  if (secinv(i).lt.epsmac) secinv(i)=epsmac
126            continue
            end if
            do 127 i=1,mloc
               cfac=work(i)*(cn-1.0d0)
               detdsc(i)=cfac*(secinv(i)**expn)
127         continue
         end if
      else if (modelv.eq.3) then

!     --- 3 parameter carreau liquid
!          ichois=0,1,...

         expn=(cn-1.0d0)/2.0d0
         do 131 i=1,mloc
            term=1.0d0+cl*secinv(i)
         etheff(i)=work(i)*(term**expn)
131      continue
         if (ichois.eq.2) then

!        --- ichois=2

            expn=(cn-3.0d0)/2.0d0
            cfac=work(i)*(cn-1.0d0)*cl
            do 132 i=1,mloc
               term=1.0d0+cl*secinv(i)
               detdsc(i)=cfac*(term**expn)
132         continue
         end if
      else if (modelv.eq.4) then

!     --- 3 parameter plastico-viscous liquid
!          ichois=0,1,...

         if (ifirst.eq.0) then
            expn=0.5d0/cn
            factor=1.0d7

!           --- below a critical value of the rate of deformation a Newtonian
!               approximation is used with etha_0 = factor * etha_pv
!               according to:
!               C.R. Beverly, R.I. Tanner, &
!               Numerical analysis of three-dimensional Bingham plastic flow, &
!               J. non-Newt. Fluid Mech., vol 42 (1992) 85-115
!               a factor of 1000 appears to be optimal for channel and pipe
!               flow.
!               (this is confirmed by my test-problems)
!               a factor of 10000 appears to be better for outflow in open
!               vessels
!               especially for large values of the yiels stress.
!               For an ideal Bingham model the factor is as high as possible
!               secndc is the square of the critical rate of deformation where
!               the stress t=etha_0 (grad v + (grad v)^T) is equal to the
!               stress according to the model

            secndc=( cl/(work(1) * (factor**(1d0/cn)-1d0)**cn) )**2
         end if
         do 141 i=1,mloc
            if (secinv(i).lt.secndc) then
              etheff(i)=factor*work(i)
            else
               term=(cl*cl/(work(i)*work(i)*secinv(i)))**expn
               etheff(i)=work(i)*(1.0d0+term)**cn
            end if
141      continue
      else if (modelv.eq.5) then

!     --- 3 parameter Papanastasiou plastico-viscous liquid
!         T.C. Papanastasiou, Flow of materials with yield, &
!         J. Rheology, vol 31, pp 385-404 (1987).
!          ichois=0,1,...

         do 151 i=1,mloc
            term=sqrt(secinv(i))
            if (term.gt.epsmac) then
               etheff(i)=work(i)+cl*(1.d0-exp(-cn*term))/term
            else
               etheff(i)=work(i)+cl*cn
            end if
151      continue
      else if (modelv.eq.6) then

!     --- 4 parameter bi-viscosity plarico-viscous model

         if (ifirst.eq.0) then
            expn=0.5d0/cn
            factor=param(1)

!           --- compute critical (transition) value of secinv

            secndc=( cl/(work(1) * (factor**(1d0/cn)-1d0)**cn) )**2
         end if
         do 161 i=1,mloc
            if (secinv(i).lt.secndc) then
              etheff(i)=factor*work(i)
            else
               term=(cl*cl/(work(i)*work(i)*secinv(i)))**expn
               etheff(i)=work(i)*(1.0d0+term)**cn
            end if
161      continue
      else if (modelv.eq.7) then

!     --- 5 parameter Carreau-Yasuda model
!         see R.B. Bird, R.C. Armstrong, O. Hassager, &
!           Dynamics of polymeric liquids, Vol 1, &
!           2^nd ed., Wiley, 1987, pp 171-172.
!          ichois=0,1,...

         if (ifirst.eq.0) then
            expn=(cn-1.0d0)/param(2)
         end if
         do 171 i=1,mloc
            term=(1.0d0+(cl*sqrt(secinv(i)))**param(2))**expn
            etheff(i)=param(1)+(work(i)-param(1))*term
171      continue
      end if
500   continue
      end
