      subroutine el4029 ( elemvc, xgauss, u, dudx, secinv, detdsc, work, &
                          dphidx, w )
! ======================================================================
!
!
!        programmer     Guus Segal/Jaap van der Zanden
!        version 1.1    date   02-03-1994 Extension with 2D
!        version 1.0    date   31-07-1993 Copied from el4037
!
!
!
!   copyright (c) 1993,1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     This subroutine computes the effect of the newton linearization of
!     the generalized newtonian model to the element right-hand-side vector
!
! ********************************************************************
!
!                      KEYWORDS
!
!    element_vector
!    viscosity
! ********************************************************************
!
      use sepmodulecactl
      use sepmodulecactl
      implicit none
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      double precision xgauss(m,*), u(m,*), dudx(m,ndimlc,*), &
                       secinv(*), detdsc(*), work(*), elemvc(ncomp,*), &
                       dphidx(n,m,ndimlc), w(m)

!     detdsc  i    ?
!     dphidx  i    Array of length n x m x ndimlc  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = dphidx (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = dphidx (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = dphidx (i,k,3);
!                       i        k
!                  If the element is a linear simplex, the
!                  derivatives are constant and only k = 1 is filled.
!     dudx    i    Real m x ndimlc  array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j )
!                    j     k
!                  dudx (1,*) is filled because dudx is then a constant
!                  per element
!                  dudx is only filled if jderiv > 0
!     elemvc i/o   Element vector
!     secinv  i    Array containing the second invariant in the integration
!     node
!     u       i    array of length m x ncomp in which the values of
!                  u in the integration points are stored the sequence:
!                  u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                   i   j
!     w       i    array of length m containing the weights for integration
!     work         work array of length m
!                  In this array w*secinv*detdsc in the integration points is
!                  stored
!     xgauss  i    Double precision array of length m x ndimlc in which the
!                  co-ordinates of the integration points are stored
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
! ********************************************************************
!
!                    LOCAL PARAMETERS

      integer k, i
      double precision sum1, sum2, sum3

!     i       Counting variable
!     k       Counting variable
!     sum1    Parameter to compute a sum
!     sum2    Parameter to compute a sum
!     sum3    Parameter to compute a sum
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! ********************************************************************
!
!                    I/O
!
!     none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!     226   Possibility for power-law model has not yet been implemented
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- Fill w * secinv * detdsc in work

      do 100 k = 1, m
         work(k) = w(k) * detdsc(k) * secinv(k)
100   continue

      if ( jcart.eq.1 .and. ndimlc.eq.2 ) then

!     --- ndimlc = 2 and jcart = 1, Cartesian co-ordinates

         do 120 i = 1, n
            sum1 = 0d0
            sum2 = 0d0
            do 110 k = 1, m
               sum1 = sum1 + work(k)*(2d0*dphidx(i,k,1)*dudx(k,1,1)+ &
                             dphidx(i,k,2)*(dudx(k,1,2)+dudx(k,2,1)) )
               sum2 = sum2 + work(k)*(2d0*dphidx(i,k,2)*dudx(k,2,2)+ &
                             dphidx(i,k,1)*(dudx(k,1,2)+dudx(k,2,1)) )
110         continue
            elemvc(1,i) = elemvc(1,i) + sum1
            elemvc(2,i) = elemvc(2,i) + sum2
120      continue

      else if ( ndimlc.eq.3 ) then

!     --- ndimlc = 3 and jcart = 1, Cartesian co-ordinates

         do 220 i = 1, n
            sum1 = 0d0
            sum2 = 0d0
            sum3 = 0d0
            do 210 k = 1, m
               sum1 = sum1 + work(k)*(2d0*dphidx(i,k,1)*dudx(k,1,1)+ &
                             dphidx(i,k,2)*(dudx(k,1,2)+dudx(k,2,1)) + &
                             dphidx(i,k,3)*(dudx(k,1,3)+dudx(k,3,1)) )
               sum2 = sum2 + work(k)*(2d0*dphidx(i,k,2)*dudx(k,2,2)+ &
                             dphidx(i,k,1)*(dudx(k,1,2)+dudx(k,2,1)) + &
                             dphidx(i,k,3)*(dudx(k,2,3)+dudx(k,3,2)) )
               sum3 = sum3 + work(k)*(2d0*dphidx(i,k,3)*dudx(k,3,3)+ &
                             dphidx(i,k,1)*(dudx(k,1,3)+dudx(k,3,1)) + &
                             dphidx(i,k,2)*(dudx(k,2,3)+dudx(k,3,2)) )
210         continue
            elemvc(1,i) = elemvc(1,i) + sum1
            elemvc(2,i) = elemvc(2,i) + sum2
            elemvc(3,i) = elemvc(3,i) + sum3
220      continue

      else

!     --- This possibility has not yet been implemented

         call errint ( jcart, 1 )
         call errint ( ndimlc, 2 )
         call errsub ( 226, 2, 0, 0 )

      end if

      end
