      subroutine el4038(jcheld,ix,jd,itcoor,nloc,ndim, &
                        modelv,etha,cn,cl,param, &
                        x,u,dudx,elemvc,elemwg,work)
!
! =====================================================================
!
!   programmer   jaap van der zanden
!
!   version 4.4  date   29-12-1992 adapt for ndim=3
!   version 4.3  date   22-12-1992 remove name, m
!   version 4.2  date   29-10-1992 Add jcheld>=10
!
!
!   copyright (c) 1984,1992  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! *********************************************************************
!
!   DESCRIPTION
!
!     this subroutine computes derived quantities with help
!          of el4010, el4026/el4027
!
! *********************************************************************
      use sepmodulecactl
      implicit none
      integer jcheld,ix,jd,itcoor,nloc,ndim,modelv
      double precision cn, cl, etha(*), param(*), x(*), u(*), dudx(*), &
             elemvc(*), elemwg(*), work(*)
! *********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
!
!   jcheld  i  choice parameter with respect to subroutine DERIVA
!
!   ix      i  parameter with same meaning as for DERIVA
!
!   jd      i  parameter (jdegfd) with same meaning as for DERIVA
!
!   itcoor  i  type of coordinates (see eld400, eld406)
!
!   n       i  number of nodal points in the element
!
!   m       i  not used
!
!   ndim    i  space dimension of the problem
!
!   modelv  i  number of the viscosity model
!
!  etha,cn,cl i parameters of the viscosity model
!
!   param   i  array for other parameters (not yet used)
!
!   x       i  coordinates of nodal points
!
!   u       i  velocities in nodal points
!
!   dudx    i  derivatives of u with respect to x
!
!   elemvc  o  element vector of derived quantity
!
!   elemwg  o  weight vector for derived quantity
!
!   work    i  work array, its length is at most ncomp*ncomp=9
!
! *********************************************************************
!
!   COMMON  BLOCKS
!
! *********************************************************************
!
!    LOCAL  VARIABLES
!
! *********************************************************************
      double precision secinv,viscos
      integer icheld,ig,ij,ip,nh,nstep(27,2:3),nt
      save icheld,nh,nstep
! *********************************************************************
!
!   SUBROUTINES  CALLED
!
!     EL4010  Compute d v_i/d x_j, divergence, curl
!     EL4026  Compute  quantities for a generalized newtonian liquid
!     ERCLOS  Reset old name of calling tree
!     EROPEN  Add name of routine to calling tree
!
! *********************************************************************
!
!   I / O
!
!   none
!
! *********************************************************************
!
!   ERROR  MESSAGES
!
!   none
!
! *********************************************************************
!
!   PSEUDO  CODE
!
!   trivial
!
! =====================================================================
!
!*  derived quantities in vertices only (except for icheld>=10)
!   (set nt for weights)
!
      if (ifirst.eq.0) then
         if (jcheld.ge.10) then
            nh=2*(nloc/2)
            if (ndim.eq.3) nh=nloc
            if (jcheld.gt.10) then
               icheld=jcheld-10
            else
               icheld=jcheld
            end if
            do 1 ig=1,nh
               nstep(ig,ndim)=1
1           continue
         else
            if (ndim.eq.2) then
               nh=nloc/2
               do 2 ig=1,nh
                  nstep(ig,ndim)=2
2              continue
            else if (ndim.eq.3) then
               nh=8
               if (nloc.eq.27) then
                  nstep(1,3)=2
                  nstep(3,3)=4
                  nstep(7,3)=2
                  nstep(9,3)=10
                  nstep(19,3)=2
                  nstep(21,3)=4
                  nstep(25,3)=2
                  nstep(27,3)=0
               end if
            end if
            icheld=jcheld
         end if
         if (( icheld.ge.2 .and. icheld.le.4 ) .or. &
             ( icheld.ge.7 .and. icheld.le.9 )) then
            if (( icheld.eq.2 .or. icheld.eq.3 ) .or. &
                ( icheld.eq.7 .or. icheld.eq.9 )) then
               nt=nh
            else if ( icheld.eq.4 .or. icheld.eq.8 ) then
               if ( ndim.eq.2 .and. itcoor.ne.3 ) then
                  nt=nh
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  nt=nh*3
               end if
            end if
         else if ( icheld.ge.5 .and. icheld.le.6 ) then
            if (icheld.eq.5) then
               if ( ndim.eq.2 .and. &
                  ( itcoor.eq.0 .or. itcoor.eq.2 )) then
                  nt=nh*3
               else if ( ndim.eq.2 .and. itcoor.eq.1 ) then
                  nt=nh*4
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  nt=nh*6
               end if
            else if (icheld.eq.6) then
               if ( ndim.eq.2 .and. &
                  ( itcoor.eq.0 .or. itcoor.eq.2 )) then
                  nt=nh*4
               else if ( ndim.eq.2 .and. itcoor.eq.1 ) then
                  nt=nh*5
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  nt=nh*9
               end if
            end if
         end if
         do 9 ig=1,nt
            elemwg(ig)=1.0d0
9        continue
      end if
      call eropen('el4038')
      ij=1
      ip=1
      do 10 ig=1,nh
         if ( icheld.ge.2 .and. icheld.le.4 ) then
            call el4010(icheld,ix,jd,itcoor,x,u,dudx, &
                        ip,nloc,ndim,elemvc(ij))
            if ( icheld.eq.2 .or. icheld.eq.3 ) then
               ij=ij+1
            else if (icheld.eq.4) then
               if ( ndim.eq.2 .and. itcoor.ne.3 ) then
                  ij=ij+1
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  ij=ij+3
               end if
            end if
         else if ( icheld.ge.5 .and. icheld.le.9 ) then
!
!*  the order of operations in el4026 is such that work can be used
!       for the computation of gradv, from which the derived quantity
!       is computed, unless gradv itself is the derived quantity
!
            if (icheld.eq.6) then
!
!*  gradv is the derived quantity
!
               call el4026(icheld-4,itcoor,ip,nloc,ndim, &
                        modelv,etha,cn,cl,param,x,u,dudx, &
                        viscos,secinv,elemvc(ij),work)
            else
               call el4026(icheld-4,itcoor,ip,nloc,ndim, &
                        modelv,etha,cn,cl,param,x,u,dudx, &
                        viscos,secinv,work,elemvc(ij))
            end if
            if (icheld.eq.5) then
               if ( ndim.eq.2 .and. &
                  ( itcoor.eq.0 .or. itcoor.eq.2 )) then
                  ij=ij+3
               else if ( ndim.eq.2 .and. itcoor.eq.1 ) then
                  ij=ij+4
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  ij=ij+6
               end if
            else if (icheld.eq.6) then
               if ( ndim.eq.2 .and. &
                  ( itcoor.eq.0 .or. itcoor.eq.2 )) then
                  ij=ij+4
               else if ( ndim.eq.2 .and. itcoor.eq.1 ) then
                  ij=ij+5
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  ij=ij+9
               end if
            else if ( icheld.eq.7 .or. icheld.eq.9 ) then
               ij=ij+1
            else if (icheld.eq.8) then
               if ( ndim.eq.2 .and. itcoor.ne.3 ) then
                  ij=ij+1
               else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
                  ij=ij+3
               end if
            end if
         end if
         ip=ip+nstep(ip,ndim)
10    continue
      call erclos('el4038')
      end
