      subroutine el4914 ( elemmt, phi, rho, w, omega, swork, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 20-10-2010 new call to el3002
!        version  2.2  date 02-03-1994 Extension for 2D
!        version  2.1  date 19-08-1993 Extension for variable omega
!        version  2.0  date 16-07-1993 Complete revision
!
!   copyright (c) 1985-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element matrix for navier-stokes
!     and store in matrix s
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rho(m), w(m), omega(m), &
                       elemmt(ndimlc*n,ndimlc*n), &
                       swork(n,n), work(*), phi(n,m)

!     elemmt  i    Element matrix
!     omega   i    angular velocity
!     phi     i    Array of length n x m containing the values of the basis
!                  functions in the integration points if a Gauss rule is
!                  used.
!                  phi  (xg ) = phi (i,k)
!                     i    k
!     rho     i    density rho
!     swork        Work array in which a sub element matrix is stored
!     w       i    array of length m containing the weights for integration
!     work    i    Work array of length m x 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, jzeros

!     jzeros  Help variable to save the value of jzero
!     k       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3002  Compute integrals of the shape / beta(x) phi (x) dx
!                                            e            i
!     EL3010  Fill small matrix elemmt into larger matrix s
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the rotation matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!
!            |   0       S      0     |
!            |            uv          |
!            |                        |
!            |   S       0      0     |
!     S =    |    vu                  |
!            |                        |
!            |   0       0      0     |
!            |                        |
!
!
!     The submatrices have the following shape:
!
!     S uv =  -2 rho omega phi(j) phi(i)
!     S vu =   2 rho omega phi(j) phi(i)
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

!     --- Compute the matrix 2 rho omega phi(j)*phi(i)

      do 100 k = 1, m
         work(k) = 2d0*w(k)*omega(k)
100   continue

      jadd = 0
      jzeros = jzero
      jzero  = 2
      call el3002 ( rho, swork, work, phi, m, n, n )
      jzero = jzeros

!     --- Add matrix Suv

      jadd = 1
      call el3010 ( elemmt, swork, ndimlc*n, m, 2, 1, ndimlc, .false., &
                    -1d0 )

!     --- Add matrix Svu

      jadd = 1
      call el3010 ( elemmt, swork, ndimlc*n, m, 1, 2, ndimlc, .false., &
                    1d0 )
      end
