      subroutine el8201 (D, s, nd)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 13-04-2005 Extension for axi-symmetry (RvL)
!        version  1.1  date 01-11-2002 Extension for 2D shapes (RvL)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   computes stress matrix for general nonlinear solid element
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nd
      double precision D(nd,nd), s(*)

!     D       o    Stress matrix to be filled
!     nd      i    Number of rows in D
!     s       i    Stress vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (ndim==2) then

         D(1,1) = -s(1)
         D(1,3) = -s(3)
         D(2,2) = -s(2)
         D(2,4) = -s(3)
         D(3,2) = -s(3)
         D(3,4) = -s(1)
         D(4,1) = -s(3)
         D(4,3) = -s(2)

!        --- axisymmetric case

         if (jcart==2) then
            D(5,5) = -s(4)
         end if

      else if (ndim==3) then

         D(1,1) = -s(1)
         D(1,4) = -s(4)
         D(1,9) = -s(6)

         D(2,2) = -s(2)
         D(2,5) = -s(5)
         D(2,7) = -s(4)

         D(3,3) = -s(3)
         D(3,6) = -s(6)
         D(3,8) = -s(5)

         D(4,2) = -s(4)
         D(4,5) = -s(6)
         D(4,7) = -s(1)

         D(5,3) = -s(5)
         D(5,6) = -s(4)
         D(5,8) = -s(2)

         D(6,1) = -s(6)
         D(6,4) = -s(5)
         D(6,9) = -s(3)

         D(7,1) = -s(4)
         D(7,4) = -s(2)
         D(7,9) = -s(5)

         D(8,2) = -s(5)
         D(8,5) = -s(3)
         D(8,7) = -s(6)

         D(9,3) = -s(6)
         D(9,6) = -s(1)
         D(9,8) = -s(4)

      end if

      end
