      subroutine el915vec ( mloc, nloc, tang, elemvc, dphidx, sigma, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2002
!
!   copyright (c) 2002-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector for quadratic line element corresponding to
!     type 915: element for update of free boundary according to the
!     method described in
!     N.P. Kruyt, C. Cuvelier, A. Segal, J. van der Zanden, &
!     A total linearisation method for solving viscous free boundary flow
!     problems by the finite element method,<BR>
!     Int. J. for Num. Methods in Fluids, Vol. 8, pp. 351-363, 1988
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_vector
!     free_boundary
!     free_surface
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mloc, nloc
      double precision tang(mloc,2), elemvc(icount), dphidx(nloc,mloc), sigma(mloc), &
                       w(mloc)

!     dphidx         i    Array of length n x m x ndim  containing the
!     derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemvc         o    Element vector
!     m              i    number of integration points
!     n              i    number of basis functions for each component
!     sigma          i    surface tension in integration points
!     tang           i    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, l, i
      double precision sum

!     i              Counting variable
!     k              Counting variable
!     l              General loop variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute element vector corresponding to surface tension
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el915vec' )

!     --- Compute element vector corresponding to surface tension

      do l = 1, 2
         do i = 1, 3
            sum = 0d0
            do k = 1, 3
               sum = sum - w(k)*tang(k,l)*dphidx(i,k)*sigma(k)
            end do
            elemvc(l+3*(i-1)) = sum
         end do
      end do

      call erclos ( 'el915vec' )
      end
