      subroutine elcontangle ( elemvc, gamma, t, x, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-04-2003 Extra parameter w
!        version  1.0  date 07-03-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Computes contribution of contact angle to element vector
!     This is suited for a point element only
! **********************************************************************
!
!                       KEYWORDS
!
!     surface_tension
!     contact_angle
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemvc(*), gamma, t(*), x(*), w(*)

!     elemvc         o    Element vector corresponding to the the point element
!     gamma          i    surface tension coefficient
!     t              i    tangential vector in point defining the contact angle
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    co-ordinates of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1769   Type of co-ordinate system incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elcontangle' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elcontangle'
         write(irefwr,*) 'jcart, gamma, r ', jcart, gamma, x(1)
         call prinrl ( t, 2, 't' )

      end if
      if ( jcart.eq.1 .or. jcart.eq.2 ) then

!     --- Cartesian coordinates

         elemvc(1) = gamma*t(1)*w(1)
         elemvc(2) = gamma*t(2)*w(1)

      else

!     --- Type of coordinate system not available

         call errint ( jcart-1, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errchr ( 'point element', 1 )
         call errsub ( 1769, 4, 0, 1 )

      end if

      call erclos ( 'elcontangle' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc, 2, 'elemvc' )
         write(irefwr,*) 'End elcontangle'

      end if

      end
