      subroutine eld204 ( coor2d, iuser, user, islold, vecloc, work )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.9  date 29-05-2014 remove call to time subroutine
!        version  1.8  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weights for nonlinear
!     solid elements.
!     ELD200 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eldsubs
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELDSUBS
!             - ELD200
!    The subroutine is able to handle the following element shapes and cases:
!    3D: 27 node brick (Type 200)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor2d(*), user(*), vecloc(*), work(*)
      integer iuser(*), islold(*)

!     coor2d  i    array of length ndim x npointcac containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!                  x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i);
!                   i                 i                 i
!     islold  i    User input array in which the user puts information
!                  of all preceding solutions
!                  Integer array of length 5 x numold
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     vecloc  i    Work array in which all old solution vectors for the
!                  integration points are stored
!     work         work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnp, maxnu, maxnmu, maxnksi, maxusr, maxcoef, iseqnr
      parameter (maxnu = 27, maxnp = 8,  maxnmu = 3, maxnksi = 1, &
                 maxusr = 100, maxcoef = 45, iseqnr = 1)
      integer isub, isup, icounw, jelp, mult
      save isub, isup, icounw, jelp, mult
      integer i, j, igprob, imatlaw, iusrvc, iusrflg, ncoef, nd, &
              ndisp, npres
      integer isequ(3*maxnu), iseqp(maxnp), iseqmu(maxnmu*maxnp), &
              iseqksi(maxnksi*maxnp)
      save igprob, imatlaw, iusrvc, iusrflg, ncoef, nd, ndisp, npres, &
           isequ, iseqp, iseqmu, iseqksi
      double precision coor2d0(3,maxnu), coor2dn(3,maxnu), &
                       coor2d1(3,maxnu)
      integer ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
              ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
              ipu, ipug, ipFn, &
              ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipmu, ipmu8, &
              ipmun, ipmun8, ipN, ipN8, ipend, &
              ipNn, ipNn8, ipc, ipc8, ipcn, ipcn8, ipz, ipusrvc, &
              ipwork
      save ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
           ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
           ipu, ipug, ipFn, &
           ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipmu, ipmu8, &
           ipmun, ipmun8, ipN, ipN8, &
           ipNn, ipNn8, ipc, ipc8, ipcn, ipcn8, ipz, ipusrvc, &
           ipwork

!     coor2d0          Coordinates in undeformed configuration
!     coor2d1          Coordinates in current configuration
!     coor2dn          Coordinates in reference configuration
!     i              Counting variable
!     icounw         length of elemwg(*)
!     igprob         ?
!     imatlaw        Type of material constitutive equation to be used
!     ipc
!     ipc8           ?
!     ipcn           ?
!     ipcn8          ?
!     ipcoef         Pointer to coefficients
!     ipdphi         Pointer to dphi/dx on current configuration
!     ipdphi0        Pointer to dphi/dx on undeformed configuration
!     ipdphin        Pointer to dphi/dx on reference configuration
!     ipdpsi         Pointer to dpsi/dx on reference configuration
!     ipend          Last position in array
!     ipfd           Pointer to F_delta
!     ipfn           Pointer to F_n
!     ipjd           Pointer to J_delta
!     ipjn           Pointer to J_n
!     ipmu           ?
!     ipmu8          ?
!     ipmun          ?
!     ipmun8         ?
!     ipn
!     ipn8           ?
!     ipnn           ?
!     ipnn8          ?
!     ipp            Pointer to hydrostatic pressure
!     ipp8           Pointer to hydrostatic pressure in vertices
!     ipphi          Pointer to basisfunctions on current configuration
!     ipphi0         Pointer to basisfunctions on undeformed configuration
!     ipphin         Pointer to basisfunctions on reference configuration
!     ipphix         Pointer to phiksi
!     ippn           Pointer to hydrostatic pressure on reference configuration
!     ippn8          Pointer to hydrostatic pressure on reference configuration
!                    in the vertices
!     ippsi          Pointer to linear basis functions
!     ippsix         Pointer to psiksi
!     ipu            Pointer to nodal displacements
!     ipug           Pointer to displacements in integration points
!     ipusrvc        Pointer to user vector
!     ipw            Pointer to weighing functions
!     ipwork         Pointer to work array
!     ipx            Pointer to element coordinates
!     ipxg           Pointer to integration point coordinates
!     ipz            ?
!     iseqksi        Sequence for ksi dof's
!     iseqmu         Sequence for chemical potential dof's
!     iseqnr         Sequence number for first old solution vector
!     iseqp          Sequence for pressure dof's
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iusrflg        ?
!     iusrvc         if>0, a user defined vector is used
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     maxcoef        Maximum number of coefficients
!     maxnksi        ?
!     maxnmu         ?
!     maxnp          Maximum number of pressure dof's
!     maxnu          Maximum number of displacement dof's
!     maxusr         Maximum number of uservector dof's
!     mult           ?
!     ncoef          Number of coefficients in the equation
!     nd             Dimension of tangent stiffness matrix
!     ndisp          Number of displacement dof's
!     npres          Number of pressure dof's
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0204         performs the initializations for subroutine ELM204/ELD204
!     EL1007         Fill element vector for vertices only
!     EL2604         Get old displacement vector and deformation gradient for
!                    general nonlinear solid
!     EL2614         Add displacement field to element coordinates in buffer
!     EL2708         Add displacement field to element coordinates in buffer
!     EL8246         compute the stress for general nonlinear solid element
!     EL8248         compute the stress for general nonlinear solid element
!     EL8249         compute deformation gradient for general nonlinear solid
!     EL8250         compute the fiber direction in case of composite elements
!     EL8251         ?
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP634         Compute basis functions and related quantities for
!                    triquadratic brick
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+ &
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+ &
!    |   1   | information about the  time-integration                     |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |    | not used                                     |
!    +-------+---------+----+----------------------------------------------+ &
!    |   2   | type of stress-strain relation                              |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | full 3D                                      |
!    +-------+---------+----+----------------------------------------------+ &
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+ &
!    |                 |  0 | default value, defined by element            |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss                                        |
!    +-------+---------+----+----------------------------------------------+ &
!    |   4   | constitutive law  coef = imatlaw                            |
!    |       +---------+----+----------------------------------------------+ &
!    |         imatlaw |  1 | compressible elastic solid                   |
!    |                 |  2 | incompressible elastic solid                 |
!    |                 |  4 | incompressible Mooney-Rivlin                 |
!    |                 | 10 | composite material                           |
!    |                 | 11 | composite material                           |
!    |                 | 99 | user-defined material                        |
!    +-------+---------+----+----------------------------------------------+ &
!    |   5   | user flags    coef = iusrvc + 100*iusrflg                   |
!    |       +---------+----+----------------------------------------------+ &
!    |         iusrvec |  0 | user vector is not filled                    |
!    |                 |  * | user is vector is filled and vector has      |
!    |                 |    | iusrvec degrees of freedom                   |
!    |         iusrflg |  * | user defined flags                           |
!    +-------+---------+----+----------------------------------------------+ &
!    |   6   | density                                                     |
!    +-------+-------------------------------------------------------------+ &
!    |   7   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   8   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!    |   9   | not used                                                    |
!    +-------+-------------------------------------------------------------+ &
!
!    +---------------------------------------------------------------------+ &
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+ &
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+ &
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec>=1)            |
!    +----+-----------+----------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    CONSTITUTIVE EQUATION PARAMETERS                                 |
!    +---------------------------------------------------------------------+ &
!
!    +---------------------------------------------------------------------+ &
!    |    compressible elastic solid (COEF4 = 1)                           |
!    +-------+-------------------------------------------------------------+ &
!    |  10   | shear modulus G                                             |
!    |  11   | bulk modulus K                                              |
!    +-------+-------------------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    incompressible elastic solid (COEF4 = 2)                         |
!    +-------+-------------------------------------------------------------+ &
!    |  10   | shear modulus G                                             |
!    +-------+-------------------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    composite (COEF4 = 10, 11)                                       |
!    +-------+-------------------------------------------------------------+ &
!    | 10-29 | see above for matrix material                               |
!    +-------+-------------------------------------------------------------+ &
!    | 30-34 | composite and fiber type                                    |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 30 | type of constitutive equation for matrix           |
!    |           | 31 | type of constitutive equation for fiber            |
!    |           | 32 | number of layers                                   |
!    |           | 33 | stiffness relaxation factor                        |
!    |           | 34 | fiber volume fraction                              |
!    +-------+---+----+----------------------------------------------------+ &
!    | 35-45 | fiber material parameters                                   |
!    +-------+-------------------------------------------------------------+ &
!
!
!    +---------------------------------------------------------------------+ &
!    |    2-phase material (COEF4 = 4)                                     |
!    +-------+-------------------------------------------------------------+ &
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+ &
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+ &
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+ &
!    | 28    | misc                                                        |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 28 | fluid volume fraction                              |
!    +-----------+----+----------------------------------------------------+ &
!
!
!    +--------------------------------------------------------------------+ &
!    |    4-phase material (COEF4 = 5)                                     |
!    +-------+-------------------------------------------------------------+ &
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+ &
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+ &
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+ &
!    | 28-34 | ion properties                                              |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 28 | D+  diffusion coefficient positive ions            |
!    |           | 29 | D-  diffusion coefficient negative ions            |
!    |           | 30 | r                                                  |
!    |           | 31 | V+  molar volume fraction pos. ions                |
!    |           | 32 | V-  molar volume fraction neg. ions                |
!    |           | 33 | z+  valent pos. ions                               |
!    |           | 34 | z-  valent neg. ions                               |
!    +-------+---+----+----------------------------------------------------+ &
!    | 35-39 | misc                                                        |
!    |       +---+----+----------------------------------------------------+ &
!    |           | 35 | cfc    fixed charged density                       |
!    |           | 36 | gamma  osmotic coefficient                         |
!    |           | 37 | phif   volume fraction fluid                       |
!    |           | 38 | cext+  external concentration pos. ions            |
!    |           | 39 | cext-  external concentration neg. ions            |
!    +-----------+----+----------------------------------------------------+ &
!
!
!     The following derived quantitites can be computed:
!c    +--------+-----------------------+----+------------------------+------+ &
!    | icheld | derived quantity      | ix |                        | ivec |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    1   | du/dx                 |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    2   | grad u                |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    3   | -grad u               |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    4   | div u                 |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    5   | curl u                |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    6   | stress                                                 4   |
!    |        +-----------------------+----+------------------------+------+ &
!    |                                |  1 | sigma                  |      |
!    |                                |  2 | tau                    |      |
!    |                                |  3 | fiber stress (global)  |      |
!    |                                |  4 | fiber stress (local)   |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    7   | pressure              |    |                        |  1   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    8   | strain                |    |                        |  4   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    9   | deformation gradient                                   13  |
!    |        +-----------------------+----+------------------------+------+ &
!    |                                |  1 | nodal points           |      |
!    |                                |  2 | integration points     |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |   10   | local direction       |    |                        |  3   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |   11   | fluid velocity        |    |                        |  3   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |   12   | oldvector for 4-phase |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!
!     21-31 Only in vertices
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('eld204')

      if (ifirst==0)  then

!     --- ifirst = 0,  compute element independent quantities

         call el0204 (iuser, user, jelp, igprob, imatlaw, &
                      iusrvc, iusrflg, nd, isub, isup, work, &
                      npres, ndisp, isequ, iseqp, iseqmu, iseqksi, &
                      ncoef)

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser)

!        --- Length of elemwg to be filled

         goto (599, 599, 599, 599, 599, 506, 507, 508, 509, 510, &
               511, 512, 599, 599, 599, 599, 599, 599, 599, 599, &
               599, 599, 599, 599, 599, 599, 527, 599, 599, 599, &
               599, 532, 599, 599, 599, 599, 599, 599, 599, 599) icheld

 506     mult = 6
         goto 599
 507     mult = 1
         goto 599
 508     mult = 6
         goto 599
 509     mult = 9
         goto 599
 510     mult = 9
         goto 599
 511     mult = 3
         goto 599
 512     mult = 3
         goto 599
 527     mult = 1
         goto 599
 532     mult = 7
         goto 599

 599     if (icheld<=20) then

!        --- unknowns in all nodes

            icounw = inpelm * mult
         else

!        --- unknowns in vertices only

            icounw = nvert * mult
         end if

!        --- Initialize weight vector, if any

         do i = 1, icounw
            elemvc1(i) = 0d0

!           --- no weight for icheld=9, because F is stored elementwise

            if (icheld/=9) then
               elemwg(i) = 1d0
            end if
         end do

!        --- Fill pointers in array work

         ipcoef  = 1
         ipx     = ipcoef  + maxnu*maxcoef
         ipxg    = ipx     + maxnu*ndim
         ipw     = ipxg    + maxnu*ndim
         ipphix  = ipw     + maxnu
         ippsix  = ipphix  + maxnu*maxnu*ndim
         ipphi0  = ippsix  + maxnp*maxnu*ndim
         ipphin  = ipphi0  + maxnu*maxnu
         ipphi   = ipphin  + maxnu*maxnu
         ipdphi0 = ipphi   + maxnu*maxnu
         ipdphin = ipdphi0 + maxnu*maxnu*ndim
         ipdphi  = ipdphin + maxnu*maxnu*ndim
         ippsi   = ipdphi  + maxnu*maxnu*ndim
         ipdpsi  = ippsi   + maxnp*maxnu
         ipu     = ipdpsi  + maxnp*maxnu*ndim
         ipug    = ipu     + maxnu*ndim
         ipFn    = ipug    + maxnu*ndim
         ipJn    = ipFn    + maxnu*ndim*ndim
         ipFd    = ipJn    + maxnu
         ipJd    = ipFd    + maxnu*ndim*ndim
         ipp     = ipJd    + maxnu
         ipp8    = ipp     + maxnu
         ippn    = ipp8    + maxnp
         ippn8   = ippn    + maxnu
         ipmu    = ippn8   + maxnp
         ipmu8   = ipmu    + maxnu*maxnmu
         ipmun   = ipmu8   + maxnp*maxnmu
         ipmun8  = ipmun   + maxnu*maxnmu
         ipN     = ipmun8  + maxnp*maxnmu
         ipN8    = ipN     + maxnu*maxnmu
         ipNn    = ipN8    + maxnp*maxnmu
         ipNn8   = ipNn    + maxnu*maxnmu
         ipc     = ipNn8   + maxnp*maxnmu
         ipc8    = ipc     + maxnu*maxnmu
         ipcn    = ipc8    + maxnp*maxnmu
         ipcn8   = ipcn    + maxnu*maxnmu
         ipz     = ipcn8   + maxnp*maxnmu
         ipusrvc = ipz     + maxnp*maxnmu
         ipwork  = ipusrvc + maxnu*maxusr
         ipend   = ipwork  + max((n+1)*m,n**2+n+m)

!        --- Check length of array work (declared in elmgrinf)

         call inichkarlen ( ipend, 71830, 'work' )

      end if

!     --- Copy coordinates of reference mesh

      do i = 1, n
         do j = 1 , ndim
            coor2d0(j,i) = coor2d(j+(index1(i)-1)*ndim)
         end do
      end do

!     --- Compute coordinates of deformed mesh on increment n (coor2dn) and
!         on current configuration (coor2d1)

      call el2708 (coor2d0, coor2dn, coor2d1, isequ, iseqnr)

!     --- Compute basis functions on reference and deformed mesh

      goto (199, 199, 199, 199, 199, 199, 199, 199, 199, 199, &
            199, 199, 199, 114), jelp

!     --- Triquadratic brick

 114     call elp634 (coor2d0, work(ipx), work(ipw), work(ipdphi0), &
                      work(ipxg), work(ipphi0), index1, work(ipphix), &
                      work(ippsix), work(ippsi), work(ipdpsi))
         call elp634 (coor2dn, work(ipx), work(ipw), work(ipdphin), &
                      work(ipxg), work(ipphin), index1, work(ipphix), &
                      work(ippsix), work(ippsi), work(ipdpsi))
         call elp634 (coor2d1, work(ipx), work(ipw), work(ipdphi), &
                      work(ipxg), work(ipphi), index1, work(ipphix), &
                      work(ippsix), work(ippsi), work(ipdpsi))
         goto 199

 199  if (ierror/=0) goto 1000

!     --- Get all old vectors

      call el2604 (work(ipphi0), work(ipphin), work(ipphi), &
                   work(ipdphi0), work(ipdphin), &
                   work(ippsi), work(ipu), work(ipug), &
                   work(ipFd), work(ipJd), work(ipFn), work(ipJn), &
                   work(ipp), work(ipp8), work(ippn), work(ippn8), &
                   work(ipmu), work(ipmu8), work(ipmun), work(ipmun8), &
                   work(ipusrvc), iusrvc, iseqnr, isequ, iseqp, iseqmu, &
                   npres, work(ipc), work(ipcn), ncoef, &
                   work(ipcoef) )

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if (ind(i)>0 .or. ist(i)>0) then

!        --- Coefficient is variable

            call elflr71 (i, user, work, work(ipx), &
                         work(ipxg), m, n, work(ipphi), 1, index1, &
                         iuser, work(ipug), vecloc )
         end if
      end do

!     --- Compute quantities such as momentaneous volume fraction
!         and concentration

      call el2614 (work(ipN), work(ipNn), work(ipc), work(ipcn), &
                   work(ipJd), work(ipJn), work(ipz), work(ipcoef), &
                   ncoef, npres)

!     --- Fill elemvc1. Content depends on icheld

      if (icheld<=20) then
         goto (1000,1000,1000,1000,1000, 206, 207, 208, 209, 210, &
                211, 212,1000,1000,1000,1000,1000,1000,1000,1000),icheld
      else
         goto (1000,1000,1000,1000,1000, 206, 207, 208, 209, 210, &
                211, 212,1000,1000,1000,1000,1000,1000,1000,1000), &
               icheld-20
      end if

!     --- icheld = 6
!         Compute the stress tensor and store in elemvc1

 206  call el8246 ( work(ipwork), work(ipxg), coor2d0, work(ipFd), &
                    work(ipJd), work(ipFn), work(ipJn), work(ipp), &
                    work(ipusrvc), imatlaw, iusrvc, iusrflg, &
                    work(ipcoef), ncoef, index1, nd, work(ipxg), 0 )
      goto 999

!     --- icheld = 7
!         Compute the pressure and store in elemvc1

 207  do i = 1, n
         work(ipwork+i-1) = work(ipp+i-1)
      end do
      goto 999

!     --- icheld = 8
!         Compute the strain tensor and store in elemvc1

 208  call el8248 (work(ipwork), work(ipFd), work(ipFn), &
                    work(ipxg), work(ipxg) )
      goto 999

!     --- icheld = 9
!         Compute the deformation tensor and store in elemvc1

 209  call el8249 (work(ipwork), work(ipFd), work(ipFn), &
                   work(ipxg), work(ipxg), ndim+jcart-1 )
      goto 999

!     --- icheld = 10
!         Compute fiber direction in case of composite material law
!         and store in elemvc1

 210  call el8250 ( work(ipwork), work(ipxg), coor2d0, work(ipFd), &
                    work(ipFn), imatlaw, work(ipusrvc), iusrvc, iusrflg, &
                    work(ipcoef), ncoef, index1 )
      goto 999

!     --- icheld = 11
!         Compute fluid velocity and store in elemvc1

 211  call el8251 (work(ipwork), work(ipp8), work(ipdpsi), &
                   work(ipcoef), ncoef, npres)
      goto 999

!     --- icheld = 12
!         Compute old vector in case of 4-phase element and store in elemvc1

 212  do i = 1, n
         work(ipwork+(i-1)*7) = work(ipp+i-1)
         do j = 1, 3
            work(ipwork+j+(i-1)*7) = work(ipmu+j-1+(i-1)*3)
            work(ipwork+3+j+(i-1)*7) = work(ipN+j-1+(i-1)*3)
         end do
      end do
      goto 999

!     --- If icheld<=20 then work(ipwork) is just copied, otherwise
!         only the unknowns in the vertices are copied

 999  if (icheld<=20) then
         do i = 1, n*mult
            elemvc1(i) = work(ipwork+i-1)
         end do
      else
         call el1007 (work(ipwork), elemvc1, mult, ishape)
      end if

1000  call erclos ('eld204')

      end
