      subroutine elfillrhsdiv ( w, rhsdiv, fdiv, psipres )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-02-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill vector rhsdiv i.e. the right-hand side of the continuity equation
!     with respect to Crouzeix Raviart elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision w(m),  rhsdiv(*), fdiv(m), psipres(npsi,m)

!     fdiv           i    Contains values of divergence right-hand side to be
!                         integrated (if mdiv=1)
!     psipres        i    Contains the pressure basis functions
!                         psipres(i,k) refers to i-th basis function in
!                         integration point k
!     rhsdiv         o    Divergence right-hand side (if mdiv=1)
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k
      double precision sum

!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillrhsdiv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillrhsdiv'
         call prinrl ( fdiv, m, 'fdiv' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- psi = 1

      sum = 0d0
      do k = 1, m
         sum = sum + w(k)*fdiv(k)
      end do
      rhsdiv(1) = sum

!     --- psi = x  - xcent
!                j        j

      do j = 2, npsi

         sum = 0d0
         do k = 1, m
            sum = sum + w(k)*fdiv(k)*psipres(j,k)
         end do
         rhsdiv(j) = sum

      end do

1000  call erclos ( 'elfillrhsdiv' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( rhsdiv, npsi, 'rhsdiv' )
         write(irefwr,*) 'End elfillrhsdiv'

      end if  ! ( debug )

      end
