      subroutine elm203
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.13 date 29-05-2014 remove call to time subroutine
!        version  1.12 date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for general nonlinear solids
!     ELM200 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine elmsubs
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMSUBS
!             - ELM200
!    The subroutine is able to handle the following element shapes and cases:
!    3D:  27 node brick
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnp, maxnu, maxcoef, maxusr, iseqnr
      parameter (maxnu = 27, maxnp = 8, maxcoef = 45, maxusr = 1000, &
                 iseqnr = 1)
      integer isub, isup, jelp, nsig
      double precision rdum(1)
      save isub, isup, jelp
      integer i, j, imatlaw, iusrvc, iusrflg, ncoef, &
              nd, ndisp, npres, mdiv, ifsi
      integer isequ(3*maxnu), iseqp(maxnp)
      save imatlaw, iusrvc, iusrflg, ncoef, nd, ndisp, &
           npres, isequ, iseqp
      double precision coor0(ndim,maxnu), coorn(ndim,maxnu), &
                       coor1(ndim,maxnu),qmat(3*maxnu), dum(27), &
                       pp(8,8), rhsdiv(8), fdiv(27), kappa(27)
      integer, pointer :: index3n1(:)
      double precision, pointer :: vecoldlc(:), vecoldlc1(:)
      integer ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
              ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
              ipu, ipug, ipFn, ipxg0, ichfib, ipend, &
              ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipusrvc, ipB, &
              ipelS, ipelR, ipelL, ipelK, ipelU, ipelT, ipwork
      save ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
           ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
           ipu, ipug, ipFn, ipxg0, ichfib, &
           ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipusrvc, ipB, &
           ipelS, ipelR, ipelL, ipelK, ipelU, ipelT, ipwork

!     coor0          Coordinates in undeformed configuration
!     coor1          Coordinates in current configuration
!     coorn          Coordinates in reference configuration
!     dum            Dummy
!     fdiv           Dummy
!     i              Counting variable
!     ichfib         Indicates which fiber subroutine has to be used
!                    See el0200
!     ifsi           Dummy parameter
!     imatlaw        Type of material constitutive equation to be used
!     ipb            Pointer to B-matrix
!     ipcoef         Pointer to coefficients
!     ipdphi         Pointer to dphi/dx on current configuration
!     ipdphi0        Pointer to dphi/dx on undeformed configuration
!     ipdphin        Pointer to dphi/dx on reference configuration
!     ipdpsi         Pointer to dpsi/dx on reference configuration
!     ipelk          Pointer to element permeability matrix
!     ipell          Pointer to element divergence matrix
!     ipelr          Pointer to element rhs
!     ipels          Pointer to element material stiffness matrix
!     ipelt          Pointer to element permeability rhs
!     ipelu          Pointer to element divergence rhs
!     ipend          Last position in array
!     ipfd           Pointer to F_delta
!     ipfn           Pointer to F_n
!     ipjd           Pointer to J_delta
!     ipjn           Pointer to J_n
!     ipp            Pointer to hydrostatic pressure
!     ipp8           Pointer to hydrostatic pressure in vertices
!     ipphi          Pointer to basisfunctions on current configuration
!     ipphi0         Pointer to basisfunctions on undeformed configuration
!     ipphin         Pointer to basisfunctions on reference configuration
!     ipphix         Pointer to phiksi
!     ippn           Pointer to hydrostatic pressure on reference configuration
!     ippn8          Pointer to hydrostatic pressure on reference configuration
!                    in the vertices
!     ippsi          Pointer to linear basis functions
!     ippsix         Pointer to psiksi
!     ipu            Pointer to nodal displacements
!     ipug           Pointer to displacements in integration points
!     ipusrvc        Pointer to user vector
!     ipw            Pointer to weighing functions
!     ipwork         Pointer to work array
!     ipx            Pointer to element coordinates
!     ipxg           Pointer to integration point coordinates
!     ipxg0          ?
!     iseqnr         Sequence number for first old solution vector
!     iseqp          Sequence for pressure dof's
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iusrflg        ?
!     iusrvc         if>0, a user defined vector is used
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     kappa          Dummy
!     maxcoef        Maximum number of coefficients
!     maxnp          Maximum number of pressure dof's
!     maxnu          Maximum number of displacement dof's
!     maxusr         Maximum number of uservector dof's
!     mdiv           Defines if divergence right-hand side is used (1)
!                    or not (0)
!     ncoef          Number of coefficients in the equation
!     nd             Dimension of tangent stiffness matrix
!     ndisp          Number of displacement dof's
!     npres          Number of pressure dof's
!     nsig           ?
!     pp             dummy
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rdum           dummy variable
!     rhsdiv         Divergence right-hand side (if mdiv=1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0200         performs the initializations for subroutine ELM200/ELD200
!     EL1004         Compute gradient and divergence matrix (Taylor-Hood)
!     EL2603         Get old displacement vector and deformation gradient for
!                    general nonlinear solid
!     EL2720         Fill matrix B for general nonlinear solid element
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3010         Fill small matrix elemmt into larger matrix s
!     EL8200         compute the stress part of the element matrix for general
!                    nonlinear solid elements
!     EL8203         Compute the part of the element matrix, that is correlated
!                    to the fluid flux out and into the element and its
!                    right-hand side
!     EL8210         compute the stress part of the element vector for general
!                    nonlinear solid element
!     EL8211         compute the divergence part of the element vector
!     EL8212         Compute the part of the element matrix, that is correlated
!                    to the fluid flux out and into the element
!     EL8232         Assemble element stiffness matrix for general nonlinear
!                    solid
!     EL8233         Assemble element vector for general nonlinear solid
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!     See subroutine el0200 for a description
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The following system is build:
!     | S     -L |  | du |   |   R   |
!     |          |  |    | = |       |
!     |   T      |  |    |   |       |
!     | -L     K |  | dp |   | U + T |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('elm203')

      if (ifirst==0) then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0200 ( E%iuser, E%user, jelp, imatlaw, &
                       iusrvc, iusrflg, nd, isub, isup, coeffarr, &
                       npres, ndisp, isequ, iseqp, ncoef, ifsi, &
                       nsig, maxusr, ichfib )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser)

!        --- Fill pointers in array coeffarr

         ipcoef  = 1
         ipx     = ipcoef  + maxnu*maxcoef
         ipx     = ipcoef  + maxnu*maxcoef
         ipxg0   = ipx     + maxnu*ndim
         ipxg    = ipxg0   + maxnu*ndim
         ipw     = ipxg    + maxnu*ndim
         ipphix  = ipw     + maxnu
         ippsix  = ipphix  + maxnu*maxnu*ndim
         ipphi0  = ippsix  + maxnp*maxnu*ndim
         ipphin  = ipphi0  + maxnu*maxnu
         ipphi   = ipphin  + maxnu*maxnu
         ipdphi0 = ipphi   + maxnu*maxnu
         ipdphin = ipdphi0 + maxnu*maxnu*ndim
         ipdphi  = ipdphin + maxnu*maxnu*ndim
         ippsi   = ipdphi  + maxnu*maxnu*ndim
         ipdpsi  = ippsi   + maxnp*maxnu
         ipu     = ipdpsi  + maxnp*maxnu*ndim
         ipug    = ipu     + maxnu*ndim
         ipFn    = ipug    + maxnu*ndim
         ipJn    = ipFn    + maxnu*ndim*ndim
         ipFd    = ipJn    + maxnu
         ipJd    = ipFd    + maxnu*ndim*ndim
         ipp     = ipJd    + maxnu
         ipp8    = ipp     + maxnu
         ippn    = ipp8    + maxnp
         ippn8   = ippn    + maxnu
         ipusrvc = ippn8   + maxnp
         ipB     = ipusrvc + maxnu*maxusr
         ipelS   = ipB     + (ndim*ndim)*ndim*maxnu*maxnu
         ipelL   = ipelS   + (maxnu*ndim)**2
         ipelK   = ipelL   + (maxnu*ndim)*maxnp
         ipelR   = ipelK   + maxnp*maxnp
         ipelU   = ipelR   + maxnu*ndim
         ipelT   = ipelU   + maxnp
         ipwork  = ipelT   + maxnp
         ipend   = ipwork  + max((n+1)*m,n**2+n+m)

!        --- Check length of array coeffarr (declared in elmgrinf)

         call inichkarlen ( ipend, 69696, 'coeffarr' )

      end if
      mdiv = 0

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n1 => indoldsols(iseqtype)%index3
      vecoldlc1 => oldsols(iseqnr+1)%oldvecs
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         do j = 1 , ndim
            coorn(j,i) = coor(j,index1(i))
            coor1(j,i) = coorn(j,i) + &
                         vecoldlc(index3n(isequ((i-1)*ndim+j)))
            coor0(j,i) = coorn(j,i) - &
                         vecoldlc1(index3n1(isequ((i-1)*ndim+j)))
         end do
      end do

!     --- Compute basis functions on reference and deformed mesh

      call elm800basefn ( coor0, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphi0), &
                          coeffarr(ipxg), coeffarr(ipphi0), index1, qmat, rdum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix) )
      call elm800basefn ( coorn, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphin), &
                          coeffarr(ipxg), coeffarr(ipphin), index1, qmat, rdum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix) )
      call elm800basefn ( coor1, coeffarr(ipx), coeffarr(ipw), coeffarr(ipdphi), &
                          coeffarr(ipxg), coeffarr(ipphi), index1, qmat, rdum, &
                          jelp, coeffarr(ipphix), coeffarr(ippsi), coeffarr(ipdpsi), &
                          coeffarr(ippsix) )
      if (ierror/=0) goto 1000

!     --- Get all old vectors

      call el2603 ( coeffarr(ipphi0), coeffarr(ipphin), coeffarr(ipphi), &
                    coeffarr(ipdphi0), coeffarr(ipdphin), coeffarr(ippsi), coeffarr(ipu), &
                    coeffarr(ipug), coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), &
                    coeffarr(ipJn), coeffarr(ipp), coeffarr(ipp8), coeffarr(ippn), &
                    coeffarr(ippn8),coeffarr(ipusrvc), iseqnr, iusrvc, isequ, &
                    iseqp, npres )

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if (ind(i)>0 .or. ist(i)>0) then

!        --- Coefficient is variable

            call elflr71 (i, E%user, coeffarr, coeffarr(ipx), &
                         coeffarr(ipxg), m, n, coeffarr(ipphi), &
                         coeffarr(ipwork), 1, index1, E%iuser, coeffarr(ipug), work )
         end if
      end do

!     --- Compute B-matrix

      call el2720 (coeffarr(ipB), coeffarr(ipdphi), nd, coeffarr(ipphi), coeffarr(ipxg))

!     --- Fill parts of matrix if necessary

      if (matrix) then

!     --- Compute stiffness matrix

         call el8200 ( coeffarr(ipelS), coeffarr(ipxg), coor0, coeffarr(ipw), &
                       coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                       coeffarr(ipp), coeffarr(ipB), coeffarr(ipusrvc), nd, imatlaw, &
                       iusrvc, iusrflg, coeffarr(ipcoef), coeffarr(ipxg0), &
                       ncoef, index1, nsig, ichfib, maxusr )

!        --- Compute divergence and gradient matrix

         call el1004 ( coeffarr(ipelL), rdum, coeffarr(ippsi), coeffarr(ipdphi), &
                       coeffarr(ipphi), coeffarr(ipw), coeffarr(ipxg), 0, rdum, &
                       pp, kappa, rhsdiv, fdiv, mdiv, dum )

!        --- Compute mass matrix

         call el8203 (coeffarr(ipelK), coeffarr(ipw), coeffarr(ipdpsi), &
                      coeffarr(ipcoef), ncoef, npres)

      end if

!     --- Fill parts of vector if necessary

      if (vector .and. notvec==0) then

!     --- First set element vector equal to zero

         do i = 1, icount
            elemvc(i) = 0d0
         end do
         do i = 1, 3*n
            coeffarr(ipelR+i-1) = 0d0
         end do
         do i = 1, npres
            coeffarr(ipelU+i-1) = 0d0
            coeffarr(ipelT+i-1) = 0d0
         end do

!        --- Compute stiffness vector

         call el8210 ( coeffarr(ipelR), coeffarr(ipxg), coor0, coeffarr(ipw), &
                       coeffarr(ipFd), coeffarr(ipJd), coeffarr(ipFn), coeffarr(ipJn), &
                       coeffarr(ipp), coeffarr(ipB), coeffarr(ipusrvc), nd, imatlaw, &
                       iusrvc, iusrflg, coeffarr(ipcoef), coeffarr(ipxg0), &
                       ncoef, index1, nsig, ichfib, maxusr )

!        --- Compute the divergence vector

         call el8211 (coeffarr(ipelU), coeffarr(ipw), coeffarr(ippsi), &
                      coeffarr(ipJd), coeffarr(ipJn), npres, 1)

!        --- Compute mass vector

         call el8212 (coeffarr(ipelT), coeffarr(ipw), coeffarr(ipelK), &
                      coeffarr(ipdpsi), coeffarr(ipFd), coeffarr(ipJd), &
                      coeffarr(ipp8), coeffarr(ippn8), &
                      coeffarr(ipcoef), ncoef, npres)
      end if

!     --- Fill parts of mass matrix if necessary

      if (notmas==0) then

!     --- notmas = 0,  compute mass matrix

         if (imas>0) then
            elemms = 0d0
         else
            jadd = 0
         end if
         if (imas>1) then

!        --- non-diagonal mass matrix

            jzero = 2
            call el3002 (coeffarr, coeffarr(ipwork), coeffarr(ipw), coeffarr(ipphi), &
                         m, n, n )
         else

!        --- diagonal mass matrix

            jconsf = imas
            call el3003 (coeffarr, coeffarr(ipwork+n*n), coeffarr(ipw), coeffarr(ipphi), &
                         m, n )
         end if
         if (imas==1) then

!        --- imas = 1, build diagonal mass matrix

            do i = 1, n
               elemms(1,i) = coeffarr(ipwork+n*n+i-1)
               elemms(2,i) = coeffarr(ipwork+n*n+i-1)
               elemms(3,i) = coeffarr(ipwork+n*n+i-1)
            end do
         else

!        --- imas = 2, build non-diagonal mass matrix

            jadd = 0
            call el3010 (elemms, coeffarr(ipwork), 3*n, n, 1, 1, 3, &
                         .false., 1d0)
            call el3010 (elemms, coeffarr(ipwork), 3*n, n, 2, 2, 3, &
                         .false., 1d0)
            call el3010 (elemms, coeffarr(ipwork), 3*n, n, 3, 3, 3, &
                         .false., 1d0)
         end if
      end if

!     --- Assemble total element stiffness matrix

      if (matrix) then
         call el8232 (elemmt, coeffarr(ipelS), coeffarr(ipelL), coeffarr(ipelK), &
                      isequ, iseqp, npres)
      end if

!     --- Assemble total element vector

      if (vector) then
         call el8233 (elemvc, coeffarr(ipelR), coeffarr(ipelU), coeffarr(ipelT), &
                      isequ, iseqp, npres)
      end if

1000  call erclos ('elm203')

      end
