      subroutine elm420 (vecloc,array, elemmt1d )
! ======================================================================
!
!        programmer    Guus Segal/ Jaap van der Zanden
!        version  6.5  date 12-12-2012 Long integers
!        version  6.4  date 22-05-1995 Rename ctime in ctimen
!        version  6.3  date 26-12-1994 New commons cmessc and cmessg
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     2-dimensional boussinesq equations
!     fill element matrix and vector
!     extended quadratic triangle for velocities (itype=420)
!              quadratic triangle for temperature
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision theta,dt,ratime
      integer iatime
      common /ctimen/ theta,dt,ratime(8),iatime(10)
      save /ctimen/

!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(numold,maxunk,*),array(*), elemmt1d(*)

!     array        Work array for the element subroutines
!     vecloc       Work array in which all old solution vectors for the
!                  integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer method,itcoor,jtcoor,ndimloc,ncomplc,jcomp,n7,n14,lrowlc,nstart, &
           lmh,lmd,ipx,ipde,ipdl,ipw,ipxg,ipphi,ipdpx,iptr,ipt3,ipq1, &
           ipfg,ipet,ipu,ipug,ipdux,ipmt,ipvc,ipvsc,ipinv,ipdsc, &
           jtime,linear,irotat,modelv, &
           indeth,ippsi,ipdpsx,iche,ipbeta,iptem0,ipcp,ipcapp,mloc, &
           indf1,indf2,indf3,indt0,induv(18),indold,ipdpsr,ialpha,iptim
      double precision sigmalc,rho,omega,cn,cl,ct,temp1
      save method,itcoor,jtcoor,ndimloc,ncomplc,jcomp,n7,n14,lrowlc,nstart, &
           lmh,lmd,ipx,ipde,ipdl,ipw,ipxg,ipphi,ipdpx,iptr,ipt3,ipq1, &
           ipfg,ipet,ipu,ipug,ipdux,ipmt,ipvc,ipvsc,ipinv,ipdsc, &
           jtime,sigmalc,rho,linear,irotat,omega,modelv,cn,cl,ct,temp1, &
           indeth,ippsi,ipdpsx,iche,ipbeta,iptem0,ipcp,ipcapp,mloc, &
           indf1,indf2,indf3,indt0,induv,indold,ipdpsr,ialpha,iptim
      integer i,ih,ich1,ich220,ich221,ichh,ichint,ichiv,icurve,isign, &
              jdiaglc,jderivlc,jtranslc,junkp,jdercnlc,jindlc,jsollc,k,numveclc
      integer indexh1(18)
      double precision de,delta,data(7),ddatdx(14),datag(7), &
                       funcc3,rhoh(1)
      double precision, pointer :: vecoldlc(:), mat2(:,:)
      save rhoh

!     cl      ?
!     cn      Power in power law model
!     ct      ?
!     data    ?
!     datag   ?
!     ddatdx  ?
!     de      ?
!     delta   jacobian of element
!     i       Counting variable
!     ialpha
!     ich1    ?
!     ich220  ?
!     ich221  ?
!     iche    ?
!     ichh    ?
!     ichint  ?
!     ichiv   ?
!     icurve  Type of curve
!     ih      Help variable to store a constant temporarily
!     indeth  ?
!     indexh1  ?
!     indf1   ?
!     indf2   ?
!     indf3   ?
!     indold  ?
!     indt0   ?
!     induv   ?
!     ipbeta  ?
!     ipcapp  ?
!     ipcp    ?
!     ipde    ?
!     ipdl    ?
!     ipdpsr  ?
!     ipdpsx  ?
!     ipdpx   ?
!     ipdsc   ?
!     ipdux   ?
!     ipet    ?
!     ipfg    ?
!     ipinv   ?
!     ipmt    ?
!     ipphi   ?
!     ippsi   Starting address in array work of psi
!     ipq1    ?
!     ipt3    ?
!     iptem0  ?
!     iptim   ?
!     iptr    ?
!     ipu     ?
!     ipug    ?
!     ipvc    ?
!     ipvsc   ?
!     ipw     ?
!     ipx     ?
!     ipxg    ?
!     irotat
!     isign   Help parameters to check the sign of the Jacobian in the
!             integration points
!     itcoor  ?
!     jtime   ?
!     jcomp   ?
!     jdercnlc  ?
!     jderivlc  ?
!     jdiaglc   Indicates if the basis functions satisfy phi i(j) = 0, &
!             if i#j (jdiaglc=1) or not (jdiaglc=2)
!     jindlc    Array of length 4 to store the cyclic permutation of 1 - 3
!     jsollc    ?
!     jtcoor  ?
!     jtranslc  ?
!     junkp   ?
!     k       Counting variable
!     linear  Indicates whether the element is a linear simplex (true)
!             or not (false)
!     lmd     ?
!     lmh     ?
!     lrowlc    ?
!     m       number of integration points
!     method  see USER INPUT in subroutine ELM600
!     modelv  Defines the type of viscosity model
!             Possible values:
!               1:  newtonian liquid
!               2:  power law liquid
!               3:  carreau liquid
!               4:  plastico viscous liquid
!             100:  user provided model depending on grad u
!             101:  user provided model depending on the second invariant
!             102:  user provided model depending on the second invariant, &
!                   the co-ordinates and the velocity
!     n14     ?
!     n7      ?
!     ncomplc   ?
!     ndimloc    Dimension of the space.
!     nstart
!     numveclc  Number of vectors stored in isol
!     omega
!     rho     density rho
!     rhoh    ?
!     sigmalc
!     temp1   ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DAXPY   Overwrite double precision dy with double precision da*dx + dy
!     DCOPY   Copy double precision dx to double precision dy
!     EL2002  Compute part of matrix corresponding to zeroth order derivatives
!     EL2003  Compute right-hand side term for an element vector (one comp.)
!     EL2004  compute second derivative part for a 2-d elliptic equation
!     EL2008  Add matrix sh to matrix s, where sh is of smaller size than s
!     EL2009  Reorder the contents of the element matrix or element vector
!     EL2012  compute part of matrix corresponding to zeroth order derivatives
!             the integral to be computed is:  /  phi (i)  psi (j)  dx
!     EL2013  compute contribution of convective term to Boussinesq equations
!             for Picard or Newton iteration   2D only  (Temperature part)
!     EL2015  Compute matrix vector multiplication
!     EL2016  Compute sum of to matrices
!     EL4021  the old solution is retrieved from solold with help of index2;
!             uc,vc in the centroid with help of truc,trvc.
!     EL4022  Compute viscous matrix for 2-dim incomplcressible flow
!     EL4023  Compute convective and/or coriolis acceleration (matrix)
!     EL4024  Eliminate velocity from centroid (uc, vc)
!     EL4025  Compute (transformed) penalty vector
!     EL4033  Compute convective acceleration for RHS vector
!     EL4036  Compute matrix due to newton linearization of viscosity
!     EL4037  Compute vector due to newton linearization of viscosity
!     EL4040  Compute viscosity for a non-isothermal newtonian liquid
!     EL4041  Fetch parameters for boussinesq type problems
!     EL4402  extract old vectors from sol, solold and form derivatives
!     ELP220  Shape functions for 7-point triangle (7 integration points)
!     ELP221  Shape functions for 6 point triangle (7 integration points)
!     ELP222  (re)compute derivatives of basis functions for polar co-ordinates
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     FUNCC3  User function subroutine to compute the value of a coefficient
!             in a point depending on a series of preceding solutions
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      74   Incorrect number of degrees of freedom for this type
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data induv/1,7,13, 2,8,14, 3,9,15, 4,10,16, 5,11,17, 6,12,18/
! ======================================================================
!
      call eropen( 'elm420' )
      if (ifirst==1) goto 1000

!     --- ifirst = 0,  initialize some parameters and arrays
!         n7    number of points for velocity u1,u2 including centroid
!         nstart  number of test function to start with
!         lrowlc    total number of degrees of freedom in element
!         itype=  420      itcoor = 0  cartesian coordinates
!            422               2  polar coordinates
!            421               1  axisymmetric coordinates
!          ( all  with penalty function method (method=1) and )
!          (      stationary                                  )

      method=1
      itcoor=itype-420
      mloc=7
      ndimloc=2
      ncomplc=3
      jcomp=2
      n7=inpelm+1
      n14=n7+n7
      lrowlc=n14+inpelm
      nstart=1
      mat2 => E%mat(n7+1:,n7+1:)

!     --- test if element is quadratic (triangle,...) for the mesh generator

      if (inpelm/=6) then
         call errint ( inpelm, 1 )
         call errint ( 6, 2 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 5 )
         call errsub ( 3, 5, 0, 0 )
      end if
      if (icount/=ncomplc*inpelm) then
         call errint ( icount, 1 )
         call errint ( ncomplc*inpelm, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 74, 5, 0, 0 )
      end if

!     --- array /  first part to be created by elp220 (total length ...)
!         ----- &

      lmh=n7*n7
      lmd=lrowlc*lrowlc
      ipx=1
      ipde=ipx+ndimloc*n7
      ipdl=ipde+6
      ipw=ipdl+6
      ipxg=ipw+3*n7
      ipphi=ipxg+ndimloc*n7
      ipdpx=ipphi+lmh
      iptr=ipdpx+ndimloc*lmh
      ipt3=iptr+ndimloc*ndimloc*inpelm
      ipq1=ipt3

!     --- isign : sign of the jacobian
!         icurve =0 : element boundary is straight; =2 : curved
!         delta = 2 * area of element
!         ipx      x(i) = x1-coordinates of nodal points
!                  y(i) = y1-coordinates of nodal points
!         ipde     e1(i) see lecture notes van kan/segal numerieke analyse c2
!                  e2(i) see lecture notes van kan/segal numerieke analyse c2
!         ipdl     dldx1(i)  derivative of linear basis function to x1
!                  dldx2(i)  derivative of linear basis function to x2
!         ipw      w(i) = weight factors for integration (any itcoor)
!         ipxg     x1g(i) x1 coordinates in integration points
!                  x2g(i) x2 coordinates in integration points
!         ipphi     phig(ij) basis functions in integration points
!         ipdpx    dphidx(ij) derivatives of basis functions to x1 (integr)
!                  dphidy(ij) derivatives to x2 (in integration points)
!         iptr     truc(i) transformation/elimination due to uc
!                  trvc(i) transformation/elimination due to vc (centroid)
!         ipt3     trpsut(i) (u,v,psi,ut)=trpsut*(u,v) : solenoidal basis
!         ipq1     qphixx(i,j) integrals on reference element
!                  qphixe(i,j) integrals on reference element
!                  qphiee(i,j) integrals on reference element
!                  second part local parameters:
!                  array(ipmt),array(ipvc) are used for intermediate storage
!                  as well as for element matrix and element vector

      ipfg=ipq1+3*lmh
      ipet=ipfg+ncomplc*mloc
      ipu=ipet+mloc
      ipug=ipu+ncomplc*n7
      ipdux=ipug+ncomplc*mloc
      ipmt=ipdux+ndimloc*ncomplc*mloc
      ipvc=ipmt+lmd
      ipvsc=ipvc+lmd
      ipinv=ipvsc+mloc
      ipdsc=ipinv+mloc
      ippsi=ipdsc+mloc
      ipdpsx=ippsi+mloc*inpelm
      ipbeta=ipdpsx+2*inpelm*mloc
      iptem0=ipbeta+mloc
      ipcp=iptem0+mloc
      ipcapp=ipcp+mloc
      if(itcoor==2) then
         ipdpsr=ipcapp+n7*mloc*2
      else
         ipdpsr=ipdpsx
      end if
      iptim = n7 ** 2 + inpelm ** 2 + 1

!     --- for time dependent problems the following pointers and arrays
!         are used:  ipu,iptim,  E%mat
!         ipu:  the solution at the preceding time level is stored
!               in array from pos. ipu (solution in nodal points)
!               afterwards the result of m x uold is stored in array from ipu
!         E%mat:  the mass matrix m is stored in E%mat pos. 1,..,n7 x n7
!                  (u/v part), resp n7*n7+1, n7*n7+inpelm*inpelm (T-part)
!                  the result of m x uold is stored in E%mat from pos. iptim
!                  iptim = n7*n7+inpelm*inpelm+1
!         ipfg     f1(i)  external body force (to be multiplied with rho)
!                f2(i)  external body force (must be non-driving)
!                f3(i)  heat source per unit mass
!         ipet     etha(i) viscosity (parameter) in integration points
!         ipu      uold(i) previous u1-velocity in nodal points
!                vold(i) previous u2-velocity in nodal points
!                told(i) previous temperature in nodal points
!         ipug     uoldg(i) previous u1-velocity in integration points
!                voldg(i) previous u2-velocity in integration points
!                toldg(i) previous temperature in integration points
!         ipdux    duoldx(i) derivative of uold to x1 in integration points
!                duoldy(i) derivative of uold to x2 in integration points
!                dvoldx(i) derivative of vold to x1 in integration points
!                dvoldy(i) derivative of vold to x2 in integation points
!                dtoldx(i) derivative of told to x1 in integration points
!                dtoldy(i) derivative of told to x2 in integation points
!         ipmt     s(ifjf) help array for element matrix, &
!                        (icount+2)*(icount+2)
!         ipvc     v(if) help array for vector (icount+2) or
!                      matrix((icount+2)*(icount+2))
!         ipvsc    etheff(i) effective viscosity in integration points
!         ipinv    secinv(i) second invariant of a1 in integration points
!         ipdsc    dvisd2(i) derivative of visc to secinv (integr. points)
!         fetch parameters (the constants only)

      call el4041(0,array(ipxg),array(ipug),array(ipphi), &
                  index1,n7,n7,ncomplc,E%iuser,E%user, &
                  jtime,method,sigmalc,rho,linear,irotat,omega, &
                  array(ipfg),modelv,array(ipet),cn,cl,ct,temp1,indeth, &
                  array(ipvc),array(ipbeta),array(iptem0),array(ipcp), &
                  array(ipcapp),indf1,indf2,indf3,indt0,indold,ialpha)
      if ( modelv==2 .and. cn>1.0d0 ) then
         iche=2
      else
         iche=1
      end if
      if(jtime==1 .or. jtime==4) then
         rhoh(1) = rho/(theta*dt)
      else if(jtime==3) then
         rhoh(1) = rho
      end if

!     --- jtcoor=3  cartesian coordinates with Gauss integration

1000  if ( itcoor==0 .and. &
          (modelv/=1 .or. ct>0.0d0) ) then
          jtcoor=3
      else
         jtcoor=itcoor
      end if
      ich220=jtcoor+30
      if (method==0) ich220=ich220+40

!     --- compute basis functions, derivatives of basis functions, etc.

      call elp220(ich220,ichint,coor,index1,isign,icurve, &
          delta,array(ipx),array(ipxg),array(ipde),array(ipdl), &
          array(ipw),array(ipphi),array(ipdpx), &
          array(iptr),array(ipt3),array(ipq1))

!     --- temperature : quadratic basis functions

      ich221=10+jtcoor
      call elp221(ich221,ichh,inpelm,coor,index1,isign, &
          icurve,delta,array(ipx),array(ipxg), &
          array(ipw),array(ippsi),array(ipdpsx))

!     --- for polar co-ordinates the phi-derivatives of the temperature basis
!         functions are divided by r and stored in array array from position
!         ipdpsr

      if (itcoor==2) call elp222(array(ipdpsx),array(ipdpsr), &
          array(ipxg),inpelm,mloc)

!     --- fetch previous solution and derivatives , if required

      if ( jtime==1 .or. modelv>1 .or. linear>0 .or. &
              ialpha==1 ) then
         ich1=-1
      else
         ich1=0
      end if
      if (numold<=1) then
         indexh1(1:icount)=index2(1:icount)
         call el4021(ich1,ncomplc,vecold,indexh1,induv, &
           array(ipu),array(ipug),array(ipdux), &
           array(ipphi),array(ipdpx),array(ippsi),array(ipdpsr), &
           array(iptr),array(ipmt),n7,n7,inpelm)

!        --- get other vectors if present

      else if (numold>1) then
         iseqtype = oldsols(1)%iseqtype
         index3n => indoldsols(iseqtype)%index3
         vecoldlc => oldsols(1)%oldvecs

         call el4021(ich1,ncomplc,vecoldlc,index3n,induv, &
           array(ipu),array(ipug),array(ipdux), &
           array(ipphi),array(ipdpx),array(ippsi),array(ipdpsr), &
           array(iptr),array(ipmt),n7,n7,inpelm)
         jdiaglc  = 2
         jderivlc = 0
         jtranslc = 0
         jdercnlc = 0
         jindlc   = 1
         jsollc   =-1

         if ( .false. ) then
         do 100 numveclc = 2, numold

!        --- temporarily: constant number of degrees of freedom (no warning)

            iseqtype = oldsols(numveclc)%iseqtype
            index4n => indoldsols(iseqtype)%index4
            junkp  = index4n(1)
            if (junkp>1) then
               write(*,'('' junkp>1 in ELM420 not implemented'')')
               call instop
            end if
            call el4402 ( data , ddatdx , &
                array(ippsi), array(ipdpsr), &
                n7, inpelm, ndimloc, jcomp, index1, array(iptr), &
                datag , jtranslc, numold, numveclc, junkp, jdercnlc, &
                jderivlc, jdiaglc, jindlc, jsollc)
         do 90  i = 1, junkp
            do 89 k=1,n7
               ih=(i-1)*n7
               vecloc(numveclc,i,k)=datag((i-1)*n7+k)
89          continue
90       continue
100      continue
        end if
      end if
      if (.false. .and. numold>=2) then
         do 120  i = 1, ncomplc
            ih = (i-1)*n7-1
            do 119 k=1,n7
               vecloc(1,i,k)=array(ipug+ih+k)
119         continue
120      continue
      end if

!     --- if not constant compute the external (driving+conservative) forces
!         and the viscosity as function of (x,y)

      call el4041(1,array(ipxg),array(ipug),array(ipphi), &
                  index1,n7,n7,ncomplc,E%iuser,E%user, &
                  jtime,method,sigmalc,rho,linear,irotat,omega, &
                  array(ipfg),modelv,array(ipet),cn,cl,ct,temp1,indeth, &
                  array(ipvc),array(ipbeta),array(iptem0),array(ipcp), &
                  array(ipcapp),indf1,indf2,indf3,indt0,indold,ialpha)
      if ( indeth/=0 .or. ct>0.0d0 ) then
         ichiv=1
      else
         ichiv=ichint
      end if
      if (indeth>10000 .and. indeth<100000000) then
         do 130 k = 0, n7-1
            array(ipet+k) = funcc3 ( indeth-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
130      continue
      end if
      if (indf1>10000 .and. indf1<100000000) then
         do 140 k = 0, n7-1
            array(ipfg+k) = funcc3 ( indf1-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
140      continue
      end if
      if (indf2>10000 .and. indf2<100000000) then
         do 150 k = 0, n7-1
            array(ipfg+k+n7) = funcc3 ( indf2-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
150      continue
      end if
      if (indf3>10000 .and. indf3<100000000) then
         do 160 k = 0, n7-1
            array(ipfg+k+2*n7) = funcc3 ( indf3-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
160      continue
      end if

!     --- compute effective viscosity

         if ( modelv==2 .and. cn>1.0d0 ) then
            iche=2
         else
            iche=0
         end if
         if(ialpha==1) iche=iche+10
         if(.not. matrix .and. ialpha==0) goto 2000
         call el4040(iche,itcoor,n7,n7,ndimloc, &
             modelv,array(ipet),cn,cl,ct,temp1,array(ipvc), &
             array(ipxg),array(ipug),array(ipdux), &
             array(ipvsc),array(ipvc+n7),array(ipinv),array(ipdsc))
      if(jtime>=1) then

!     --- jtime>=1, time-dependent solution
!                    Compute the mass-matrix
!                    The part corresponding to the velocity component
!                    is stored in E%mat from position 1
!                    The temperature part is stored in E%mat from
!                    position 1 + n7 ** 2

         call el2002( 6, ichint+1, rhoh, E%mat, array(ipw), &
                      array(ipphi), n7, n7, n7, array(ipvc), array, &
                      de, .false. )

!        --- To compute the mass matrix for the temperature part the
!            expression rho cp / theta dt   is stored into array from position
!            ipmt
!            temporarily

         do 500 i = 0,mloc-1
            array ( ipmt+i ) = rho * array ( ipcp+i ) / ( theta*dt )
500      continue
         call el2002( 4, 2, array(ipmt), mat2, array(ipw), &
                      array(ippsi), inpelm, mloc, inpelm, array(ipvc), &
                      array, de, .false. )

!        --- Store M uold / theta dt in E%mat from position iptim
!            (temporarily)

         call el2015(1,ichint+1,E%mat,array(ipu),1d0,elemmt1d(iptim),n7, &
                n7,1)
         call el2015(1,ichint+1,E%mat,array(ipu+n7),1d0,elemmt1d &
              (iptim+n7),n7,n7,1)
         call el2015(1,2,mat2,array(ipu+n7*2),1d0,elemmt1d &
              (iptim+n7*2),inpelm,inpelm,1)

!        --- copy m uold in array from pos. ipu

         call dcopy(lrowlc,elemmt1d(iptim),1,array(ipu),1)
      end if

!     --- if (matrix) compute element matrix

      if (.not.matrix) goto 2000

!     --- compute viscous stresses and initialize element matrix
!             array(ipmt)
!         --- set matrix equal to zero

      do 1010 i=1,lrowlc*lrowlc
         array(ipmt+i-1)=0d0
1010  continue
      call el4022(ichiv,itcoor,array(ipvsc),array(ipmt),array(ipw), &
          array(ipxg),array(ipphi),array(ipdpx), &
          nstart,n7,n7,lrowlc,array(ippsi),array(ipdpsx),inpelm, &
          array(ipdl),array(ipq1),delta,array(ipvc))

!     --- compute terms due to newton linearization of viscosity
!             array(ipmt) for special cases only

      if ( modelv==2 .and. cn>1.0d0 ) then
         call el4036(itcoor,array(ipw),array(ipw+n7), &
             array(ipxg),array(ipphi),array(ipdpx), &
             array(ipug),array(ipdux),array(ipdsc), &
             array(ipvc),ndimloc,n7,n7,array(ippsi), &
             array(ipdpsx),inpelm)
         call el2008(array(ipmt),array(ipvc),n14,lrowlc)
      end if

!     --- compute penalty matrix if required, and add it

      if (method==1) &
          call el4025(itcoor,ichint,1,array(ipmt),array(ipvc), &
          sigmalc,isign,delta,array(ipde),array(ipw),array(ipxg), &
          array(ipphi),array(ipdpx),n7,n7,jcomp,lrowlc)

!     --- add convective and/or coriolis acceleration if required

      if ( linear>0 .or. irotat==1 ) then
         call el4023(ichint,itcoor,linear,rho,irotat,omega, &
            array(ipug),array(ipdux),array(ipw),array(ipxg), &
            array(ipphi),array(ipdpx),array(ipmt), &
            n7,n7,lrowlc,array(ippsi),array(ipdpsx),inpelm)
      end if

!     --- next the temperature part
!         --- first compute matrix   S   = - rho g beta  / psi  phi  de
!                                 vt                 e    j    i

      if(ichint==0) then
         jdiaglc=1
      else
         jdiaglc=3
      end if
      call el2012(2,jdiaglc,array(ipbeta),array(ipmt+n7*lrowlc+2*n7), &
            array(ipw),array(ipphi),array(ippsi),n7,inpelm,mloc,lrowlc, &
            array(ipvc))

!     --- next compute the matrix S   =  / k  grad psi  .  grad psi   de
!                                  tt    e            i            j

      call el2004(2,array(ipcapp),array,array(ipcapp),array(ipmt+ &
           2*n7*(lrowlc+1)),array(ipw),array(ipdpsr),array(ipdpsr+ &
           inpelm*mloc),inpelm,mloc,lrowlc,array(ipvc),.true.)

!     --- finally compute the convective terms corresponding to the temperature

      if(linear>0) call el2013(linear,ichint+1,array(ipmt),mloc,n7, &
           inpelm,lrowlc,array(ipphi),array(ippsi),array(ipdpsr), &
           array(ipw),array(ipug),array(ipdux),array(ipvc),rho, &
           array(ipcp))
      if(jtime==1 .and. theta>0d0) then

!     --- jtime = 1: time dependence
!                   Add mass matrix to element matrix

         call el2016(1,ichint+1,array(ipmt),E%mat,1d0,1d0,n7, &
            lrowlc)
         call el2016(1,ichint+1,array(ipmt+n7*lrowlc+n7),E%mat, &
           1d0,1d0,n7,lrowlc)
         call el2016(1,2,array(ipmt+n14*lrowlc+n14),mat2, &
           1d0,1d0,inpelm,lrowlc)
      end if

!     --- eliminate velocity from centroid (uc,vc) in point n7
!               i.e. transform matrix : r(transp)*smatrix*r

      call el4024(1,array(ipmt),array(ipvc),array(iptr),array(ipvc), &
                  ndimloc,n7,lrowlc)

!     --- reorder the contents of the element matrix and put it into E%mat

      call el2009(1,E%mat,E%vec,array(ipmt),array(ipvc), &
                  induv,icount,0)

!     --- compute element vector

2000  if (.not.vector) then
         call erclos( 'elm420' )
         return
      end if

!     --- fill (driving) external forces into vector (initialize it)

      do 2100 i=1,2*n7+inpelm
         array(ipvc+i-1)=0.0d0
2100  continue
      if(indf1/=0 .or. indf2/=0 .or. indf3/=0 .or. ialpha==1) &
            then
         do 2110 i=0,mloc-1
            array(ipmt+i)=rho*array(ipw+i)
2110     continue
         if(indf1/=0) call el2003(2,ichint+1,array(ipfg),array(ipvc), &
            array(ipmt),array(ipphi),n7,mloc,array(ipmt+mloc),.false.)
         if(indf2/=0) call el2003(2,ichint+1,array(ipfg+mloc), &
            array(ipvc+n7),array(ipmt),array(ipphi),n7,mloc,array(ipmt+mloc), &
            .false.)

!        --- compute contribution due to viscous dissipation eta * secinv

         if(ialpha==1) then
            do 2120 i=0,mloc-1
               array(ipmt+mloc+i)=array(ipinv+i)*array(ipvsc+i)/rho
2120        continue
         else
            do 2130 i=0,mloc-1
               array(ipmt+mloc+i)=0d0
2130        continue
         end if

!        --- add right-hand-side term for temperature equation

         if(indf3/=0) then
            do 2140 i=0,mloc-1
               array(ipmt+mloc+i)=array(ipmt+mloc+i)+array(ipfg+2*mloc+i)
2140        continue
         end if
         if(indf3/=0 .or. ialpha==1) call el2003(2,2,array(ipmt+mloc), &
            array(ipvc+2*n7),array(ipmt),array(ippsi),inpelm,mloc, &
            array(ipmt+2*mloc),.false.)
      end if

!     --- add terms due to newton linearization of viscosity
!             array(ipvc)

      if ( modelv==2 .and. cn>1.0d0 ) &
         call el4037(itcoor,array(ipw),array(ipmt), &
             array(ipxg),array(ipphi),array(ipdpx), &
             array(ipug),array(ipdux),array(ipinv),array(ipdsc), &
             array(ipvc),ndimloc,n7,n7, &
             array(ippsi),array(ipdpsx),inpelm)

!     --- compute influence of convective acceleration for linear = 2

      if (linear==2) then
         call el4033(ichint,itcoor,rho,array(ipug),array(ipdux), &
          array(ipw),array(ipxg),array(ipphi),array(ipvc), &
          nstart,n7,n7,ndimloc,array(ippsi),inpelm)
      end if

!     --- compute contribution to T :      /  - rho g beta T   phi   de
!                                  0       e                0     i

      if(indt0/=0) then
         do 2150 i=1,mloc
            array(ipmt+i-1)=array(ipw+i-1)*array(ipbeta+i-1)
2150     continue
         call el2003(5,2,array(iptem0),array(ipvc+n7),array(ipmt), &
            array(ipphi),n7,mloc,array(ipmt+mloc),.false.)
      end if

!     --- compute contribution due to convective term corresponding to
!         temperature
!                  n           n
!         /  rho cp  u   .  grad T   psi   de
!                                    i

      if(linear==2) then
         if(itcoor<2) then
            do 2160 i=1,mloc
               array(ipmt+i-1)=rho*array(ipcp+i-1)* &
                 (array(ipug+i-1)*array(ipdux+4*mloc+i-1)+ &
                  array(ipug+mloc+i-1)*array(ipdux+5*mloc+i-1))
2160         continue
            call el2003(5,2,array(ipmt),array(ipvc+2*n7),array(ipw), &
            array(ippsi),inpelm,mloc,array(ipmt+mloc),.false.)
         end if
      end if

!     --- Add M uold / theta dt to element vector in the time-dependent
!         case

      if(jtime==1) call daxpy(lrowlc,1d0,array(ipu),1,array(ipvc),1)

!     --- eliminate velocity basis functions from centroid from vector

      call el4024(2,array(ipmt),array(ipvc),array(iptr),array(ipmt), &
                  ndimloc,n7,lrowlc)

!     --- reorder the contents of the element vector and put it into E%vec

      call el2009(2,E%mat,E%vec,array(ipmt),array(ipvc), &
                  induv,icount,0)
      call erclos( 'elm420' )
      end
