      subroutine elm900address1 ( indkappa, indcoeffgrad, indcoeffdiv, &
                                  indetaspecial, indomega, indetha, &
                                  indgoertler, indcconv, mcont, &
                                  indfdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-02-2010 Complete revision
!        version  1.1  date 17-09-2008 Extension with pressure mass matrix
!        version  1.0  date 13-02-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting addresses of subarrays in array wrk1
!     Help subroutine for elm900
! **********************************************************************
!
!                       KEYWORDS
!
!     navier_stokes_equation
!     initialization
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indkappa, indcoeffgrad, indcoeffdiv, indfdiv, &
              indetaspecial, indomega, indetha, &
              indgoertler, indcconv, mcont

!     indcconv       o    Sequence number of array cconv
!     indcoeffdiv    o    Sequence number of array coeffdiv
!     indcoeffgrad   o    Sequence number of array coeffgrad
!     indetaspecial  o    Sequence number of array etaspecial
!     indetha        o    Sequence number of array etha
!     indfdiv        o    Sequence number of right-hand side continuity equation
!     indgoertler    o    Sequence number of array goertler
!     indkappa       o    Sequence number of array kappa
!     indomega       o    Sequence number of array omega
!     mcont          i    Defines the type of continuity equation
!                         See subroutine elm900
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900address1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900address1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set some special addresses

      indomega = 2          ! omega
      indetha = 6         ! etha
      indgoertler = 7     ! goertler number
      indkappa = 8
      indfdiv = 9         !rhs divergence equation
      indcoeffdiv = 11
      indcoeffgrad = 14
      indcconv = 15
      indetaspecial = 24  ! special for Goertler equations

!     --- In case the pressure mass matrix must be used for an iterative method
!         we store etha in kappa
!         This is the case for mcont = 5

      if ( mcont==5 ) indkappa = indetha

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm900address1'

      end if
      call erclos ( 'elm900address1' )

      end
