      subroutine elmassdiag ( elemms, elemcomp, iseqvel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 10-11-2007
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy components-wise defined diagonal element mass matrix
!     into diagonal element mass matrix
!     The original matrix is defined for one component only
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_matrix
!     mass_matrix
!     diagonal_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvel(*)
      double precision elemms(*), elemcomp(n)

!     elemcomp       i    Contains diagonal element mass matrix for one
!                         component
!     elemms         o    Diagonal element mass matrix
!     iseqvel        i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmassdiag' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmassdiag'
         write(irefwr,1) 'n, ndim, itype, ishape', &
                          n, ndim, itype, ishape
         if ( itype<900 .or. itype>902 ) &
            call prinin ( iseqvel, n*ndim, 'iseqvel' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itype>=900 .and. itype<=902 ) then

!     --- Type numbers 900 to 902, iseqvel does not exist

         if ( ishape==27 ) then

!        --- shape = 27, pressure in 14-th point

            do i = 1, 14
               do j = 1, ndim
                  elemms(ndim*(i-1)+j) = elemcomp(i)
               end do
            end do

            do i = 15, 27
               do j = 1, ndim
                  elemms(ndim*(i-1)+j+4) = elemcomp(i)
               end do
            end do

         else

!        --- shape #  27, pressure in last point

            do i = 1, n
               do j = 1, ndim
                  elemms(ndim*(i-1)+j) = elemcomp(i)
               end do
            end do

         end if  ! ( ishape/=27 )

      else

!     --- Other type numbers, use iseqvel

         do i = 1, n
            do j = 1, ndim
               elemms(iseqvel(ndim*(i-1)+j)) = elemcomp(i)
            end do
         end do

      end if  ! ( itype>=900 .and. itype<=902 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemms, icount ,'elemms' )
         write(irefwr,*) 'End elmassdiag'

      end if  ! ( debug )
      call erclos ( 'elmassdiag' )

      end
