      subroutine elmatmultrectp ( div, uold, elemvc, nrow, ncol, jadd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply transpose of rectangular matrix div of size (nrow x ncol) by
!     vector of size nrow to get vector of size ncol
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     element
!     full_matrix
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrow, jadd, ncol
      double precision div(nrow,ncol), uold(nrow), elemvc(ncol)

!     div            i    Divergence matrix
!     elemvc        i/o   Result of multiplication
!     jadd           i    if 0 no addition takes place
!     ncol           i    Number of rows in sp
!     nrow           i    Number of columns in sp
!     uold           i    old velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision sum

!     i              Counting variable
!     j              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmatmultrectp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmatmultrectp'

      end if
      if ( ierror/=0 ) go to 1000
      if ( jadd==0 ) then

!     --- jadd = 0, do not add to existing vector

         do i = 1, ncol
            sum = 0d0
            do j = 1, nrow
               sum = sum + div(j,i)*uold(j)
            end do
            elemvc(i) = sum
         end do

      else

!     --- jadd # 0, add to existing vector

         do i = 1, ncol
            sum = 0d0
            do j = 1, nrow
               sum = sum + div(j,i)*uold(j)
            end do
            elemvc(i) = elemvc(i)+sum
         end do

      end if  ! ( jadd==0 )

1000  call erclos ( 'elmatmultrectp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmatmultrectp'

      end if

      end
