      subroutine elp221 (ichois,ichint,nloc,coor,index1,isg,icrv, &
                         delta,x,xg,w,phi,dphidx)
      use sepmodulecactl
      implicit none
      integer ichois,ichint,nloc,index1(*),isg,icrv
      double precision delta, coor(*), x(*), xg(*), w(*), phi(*), &
             dphidx(*)
!
! --------------------------------------------------------------------
!
!     programmer   Jaap van der Zanden
!
!   version 4.1  date   29-05-1996 Improve accuracy of data
!   version 4.0  date   26-06-1992 Removal of name
!   version 3.1  date   22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1984,1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! --------------------------------------------------------------------
!
!       INPUT and OUTPUT parameters
!
! --------------------------------------------------------------------
!
!      ichois   choice parameter, possibilities:
!
!             0:  cartesian coordinates      |  icoor=0
!             1:  axi-symmetric coordinates  |       =1
!             2:  polar coordinates          |       =2
!             3:  cartesian coord/gauss integ|       =3
!             for curved elements gauss integration is used !!
!
!           +10:  + dphidx            |  iderivloc=1
!
!     n           number of basis functions (6 or 7)
!
!     coor        array containing the coordinates
!
!     index1      see to0000
!
!     isg         isg of the jacobian (anti-clockwise : >0 )
!
!     icrv        icrv=0,2 (triangle is not curved or is curved)
!
!     delta       jacobian (2*area)
!
!     x(14)       arrays containing the coordinates of the nodal points
!
!     w(7)        array containing the weights for the integration
!
!     xg(14)      value of x(1:7),y(1:7) (x1,x2) in integration points
!
!     phi(n*7)    value of basis functions in integration points
!
!     dphidx(2*n*7)  array containing the values of the derivatives
!                 ofthe basis functions in the integration points
!
! -------------------------------------------------------------------
!
!  SUBROUTINES  CALLED
!
! --------------------------------------------------------------------
!
!  ELP272  get coordinates and check if element is curved or not
!
!  ELP273  compute derivatives of coordinates, &
!          compute area of straight triangle, revise weights, &
!          compute coordinates of integration points (itcoor>0)
!
!  ELP277  compute derivatives of tranfornation basis functions for
!          curved triangles
!
!  ELP278  compute area of curved triangle; revise weights
!
!  ELP279  compute derivatives of basis functions for curved triangle
!
!  ELP280  compute derivatives of basis functions for straight triangle
!
!  ELP282  compute basis functions in integration points, &
!          compute derivatives of transformation basis functions
!
! ---------------------------------------------------------------------
!
!  COMMON  BLOCKS
!
! ---------------------------------------------------------------------
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - &
!
!   LOCAL  VARIABLES
!
! ----------------------------------------------------------------------

      double precision wnc(7), wng(7), detjac(7), e1(6), dldx1(6), &
             dpdks1(49), dpdet1(49), phigt(42), dptdks(42), dptdet(42), &
             dxdksi(7), dydksi(7), dxdeta(7), dydeta(7), baricc(21), &
             phinc(49), phig(49)
      integer inum,itcoor,jchint,jtcoor,n7,nlast,iderivloc
      save inum,itcoor,jchint,jtcoor,n7,nlast,iderivloc, &
           wnc,wng,phig,phigt,phinc, &
           dpdks1,dpdet1,dptdks,dptdet,baricc
      integer i
      double precision curved,sign
!
!     baricc(21)  baricentric coordinates of integration points
!                 (itcoor>0)
!
!    itcoor  = 0,1,2,3   type of coordinates (see ichois)
!    iderivloc  = 0,1   computation of dphidx    (=1) or not (=0)
!
!    e1(6)     differences (see lecture notes van kan/segal)
!                      (only for icrv=0)
!    dldx1(6)  derivatives of linear baricentric functions
!                      (only for icrv=0)
!
!    dpdks1(49)  = derivatives of basis functions in integration points
!    dpdet1(49)  = derivatives of basis functions in integration points
!
!    phigt(42)   = transformation functions in integration points
!    dptdks(42)  = derivatives of transformation in integration points
!    dptdet(42)  = derivatives of transformation in integration points
!
!    dxdksi(7)  = derivative of x with respect to ksi
!    dydksi(7)
!    dxdeta(7)
!    dydeta(7)
!    detjac(7)  = determinant of jacobi in nodal points
!
! --------------------------------------------------------------------
!
!
!*   initialize wnc : weights for newton-cotes integration on
!                     reference triangle with area 0.5
      data wnc/.025d0,0.06666666666666666d0,.025d0, &
               .06666666666666666d0, &
               .025d0,.06666666666666666d0,.225d0/
!
!*   initialize wng : weights for gauss integration on
!                     reference triangle with area 0.5
      data wng/.0629695902724135d0, .0661970763942532d0, &
               .0629695902724135d0, .0661970763942532d0, &
               .0629695902724135d0, .0661970763942532d0, &
               .11250d0/
!
!*   initialize phinc : basis functions in nodal points
!
      data phinc/1.0d0,7*0.0d0,1.0d0,7*0.0d0,1.0d0,7*0.0d0, &
                 1.0d0,7*0.0d0,1.0d0,7*0.0d0,1.0d0,7*0.0d0, &
                 1.0d0/
      data nlast/0/
!
!------------------------------------------------------------------* &
!
      call eropen( 'elp221' )
!
!*   ifirst=0, nlast.ne.n  compute parameters
!              and  (itcoor>0) baricc,phig,dptdks,...
!
      if ((ifirst.eq.0).or.(nlast.ne.nloc)) then
!
!*   ifirst=0  compute parameters itcoor,iderivloc
!
         inum=ichois/10
         itcoor=ichois-10*inum
         jtcoor=itcoor
         if (itcoor.eq.0) then
            jchint=0
         else
            jchint=1
         end if
         iderivloc=0
         if (inum.ne.0) iderivloc=1
         n7=nloc*7
         call elp282(baricc,phig,phigt, &
                     dptdks,dptdet,dpdks1,dpdet1,nloc)
      end if
!
!*    fill coordinates in x,x(8)  determine whether element is curved
!     if n=6 fill 7'th coordinate for integration
!
      call elp272(coor,index1,7,x,x(8),curved)
      icrv=nint(curved)
      if (icrv.lt.1) then
! --------------------------------------------------------------------
!
!
!*    boundaries of element are straight : special treatment
!
!         compute e1,dldx1,w,isg,delta,xg
!                 dphidx    if iderivloc=1
!
! --------------------------------------------------------------------
         if (itcoor.eq.0) then
            call elp273(itcoor,x,x(8),wnc,e1,e1(4),dldx1,dldx1(4), &
                        sign,delta,w,xg,xg(8),baricc,7)
         else
            call elp273(itcoor,x,x(8),wng,e1,e1(4),dldx1,dldx1(4), &
                        sign,delta,w,xg,xg(8),baricc,7)
         end if
         if (iderivloc.eq.1) then
            call elp280(itcoor,dldx1,dphidx,baricc,nloc)
            call elp280(itcoor,dldx1(4),dphidx(n7+1),baricc,nloc)
         end if
      else
! --------------------------------------------------------------------
!
!
!*    boundaries of element are curved (if itcoor=0 reset itcoor)
!
!         compute w,isg,delta,xg, &
!                 dphidx    if iderivloc=1
!
! --------------------------------------------------------------------
         if (itcoor.eq.0) itcoor=3
         call elp277(itcoor,x,x(8),dxdksi,dydksi,dxdeta,dydeta, &
                     xg,xg(8),phigt,dptdks,dptdet,7)
         call elp278(itcoor,wng,dxdksi,dydksi,dxdeta,dydeta, &
                     xg,detjac,sign,delta,w,7)
         if (iderivloc.eq.1) then
            call elp279(dxdksi,dydksi,dxdeta,dydeta,detjac, &
                        dpdks1,dpdet1,dphidx,dphidx(n7+1),nloc,7)
         end if
      end if
!
!*  set ichint and fill phi
!
      isg=nint(sign)
      if ((itcoor.eq.0).and.(icrv.eq.0)) then
         ichint=0
         if ((ifirst.eq.0).or.(ichint.ne.jchint).or. &
              (nlast.ne.nloc)) then
            jchint=0
            nlast=nloc
            do 100 i=1,n7
               phi(i)=phinc(i)
100         continue
            if(nloc.eq.6) then
               phi(7)=-1d0/9d0
               phi(21)=phi(7)
               phi(35)=phi(7)
               phi(14)=-4d0*phi(7)
               phi(28)=phi(14)
               phi(42)=phi(14)
            end if
         end if
         do 101 i=1,14
            xg(i)=x(i)
101      continue
      else
         ichint=1
         if ((ifirst.eq.0).or.(ichint.ne.jchint).or. &
             (nlast.ne.nloc)) then
            jchint=1
            nlast=nloc
            do 110 i=1,n7
               phi(i)=phig(i)
110         continue
         end if
      end if
      itcoor=jtcoor
      call erclos( 'elp221' )
      end
