      subroutine sepadaprule900
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt integration rule in case of elements for type 900
!     Stokes equation, penalty approach
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     numerical_integration
!! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: mrule

!     mrule          Type of integration rule as given by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepadaprule900' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepadaprule900'
         write(irefwr,1) 'npelm', E%npelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mrule = B%rule
      if ( debug ) &
        write(irefwr,1) 'mrule, E%ishape, n, ndim', B%rule, E%ishape, B%n, E%ndim

      if ( mrule==0 ) B%rule = 3   ! Default integration rule

      if ( debug ) write(irefwr,1) 'B%rule/1', B%rule

      if ( E%npelm==6 .and. itype==900 ) then

!     --- Special case of 6 point element

         B%n = 7
         E%ishape = 7
         jtrans = 1
         E%nveloc = 12

      else if ( itype==901 .and. E%npelm==7 ) then

!     --- Special case of 7 point element and type 901

         B%n = 7
         E%ishape = 7
         jtrans = 1
         E%nveloc = 12

      else if ( itype==901 .and. E%npelm==9 ) then

!     --- Special case of 9 point element and type 901

         B%n = 9
         E%ishape = 6
         jtrans = 1
         E%nveloc = 16

      else if ( itype>900 .and. E%npelm==5 ) then

         n = 4
         E%ishape = 5
         E%nveloc = 8

      else if ( itype>900 .and. E%ishape==17 ) then
         n = 8
         E%ishape = 13
         E%nveloc = 24
      else if ( itype==902 .and. (E%npelm==27 .or. E%ishape==6 .or. &
                E%ishape==16) .or. E%npelm==7 ) then
         n = E%npelm
         jtrans = 0
         E%nveloc = E%ndim*n

      else if ( itype==900 .and. (E%ishape>=4 .and. E%ishape<7 .or. &
                E%ishape==13 ) .or. itype==901 .and. &
                E%ishape==7 .or. E%ishape==14 ) then
         n = E%npelm
         E%nveloc = E%ndim*n

      else

!     --- Standard case

         E%nveloc = B%n*E%ndim

      end if  ! ( E%npelm==6 )
      E%matelim = .false.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'B%rule', B%rule
         write(irefwr,1) 'B%n, E%ishape, E%nveloc, jtrans', &
                          B%n, E%ishape, E%nveloc, jtrans
         write(irefwr,*) 'End sepadaprule900'

      end if  ! ( debug )
      call erclos ( 'sepadaprule900' )

      end subroutine sepadaprule900
