      subroutine sepel900conv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill convection tensor for Navier-Stokes into matrix S
!     Old subroutine elm900matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: umesh(:,:), rho(:)
      double precision :: work(n**2)
      integer :: iseq

!     iseq           Sequence number
!     rho            density rho
!     umesh          mesh velocity
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INSTOP         Stop the program
!     PRINRL1        Print 2d real vector
!     SEPEL900CONVCT compute the convective part of the element matrix for
!                    navier-stokes and store in matrix s
!     SEPEL900CONVSP compute the convective part of the element matrix for
!                    navier-stokes and store in matrix s
!                    Special form of the navier-stokes equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900conv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900conv'
         write(irefwr,1) 'seq_meshvel, typecontinuity, modelconv', &
                          E%seq_meshvel, E%typecontinuity, E%modelconv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rho => E%coeffs(:,E%indrho)

      if ( E%seq_meshvel>0 .and. E%typecontinuity/=2 ) then

!     --- Subtract mesh velocity from velocity and put in E%ugaussloc

         iseq = E%seq_meshvel
         umesh => oldsols(iseq)%locvecs
         allocate ( E%ugaussloc(size(umesh,1), size(umesh,2)) )
         E%ugaussloc = E%ugauss-umesh

      else

!     --- Use E%ugauss

         E%ugaussloc => E%ugauss

      end if

      if ( E%modelconv>0 ) then

!     --- compute convective terms where the coefficients are stored
!         in the special way

         E%cconv => E%coeffs(:,E%indcconv:E%indcconv+E%ndim**2-1)
         call sepel900convsp
         if ( E%seq_meshvel/=0 .and. E%typecontinuity/=2 ) &
            deallocate(E%ugaussloc)

      else if ( E%typecontinuity==2 ) then

!     --- compute convective terms in case of Goertler equations

         print *,'E%typecontinuity==2 '
         call ertrace
         call instop

!         call el2010g ( E%submat, B%phider, B%phi, rho, B%w, E%modelconv, &
!                        E%ugauss, gradoldvec, B%xyzg, work, &
!                        E%ugaussloc, du0dx, E%coeffs(:,E%indgoertler), &
!                        E%coeffs(:,E%indkappa) )
!           deallocate(E%ugaussloc)

      else

!     --- compute convective terms in standard case

         if ( debug ) call prinrl1 ( E%ugaussloc, E%ndim, m, 'u' )
         call sepel900convct
         if ( debug ) call prinrl1 ( E%submat, E%ndim*n, E%ndim*n, &
                                     'E%submat/2' )

         if ( E%seq_meshvel/=0 .and. E%typecontinuity/=2 ) &
            deallocate(E%ugaussloc)

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%submat, size(E%submat,1), size(E%submat,2), 'submat' )
         write(irefwr,*) 'End sepel900conv'

      end if  ! ( debug )
      call erclos ( 'sepel900conv' )

      end subroutine sepel900conv
