      subroutine sepel900finish
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Deallocate arrays with respect to type 900
!     Last call in sepelm900
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900finish' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900finish'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepeldealloc

      if ( debug ) write(irefwr,*) 'deallocate ( B%phipress )'
      deallocate ( B%phipress )
      if ( debug ) write(irefwr,*) 'deallocate ( E%gradmat )'
      deallocate ( E%gradmat )
      if ( debug ) write(irefwr,*) 'deallocate ( E%rhsdiv )'
      deallocate ( E%rhsdiv )
      if ( debug ) write(irefwr,*) 'deallocate ( E%pressmat )'
      deallocate ( E%pressmat )
      if ( debug ) write(irefwr,*) 'deallocate ( E%transdiv )'
      deallocate ( E%transdiv )
      if ( debug ) write(irefwr,*) 'deallocate ( E%rhsdivtr )'
      deallocate ( E%rhsdivtr )
      if ( debug ) write(irefwr,*) 'deallocate ( E%gradv )'
      deallocate ( E%gradv )
      if ( debug ) write(irefwr,*) 'deallocate ( E%a1 )'
      deallocate ( E%a1 )
      if ( debug ) write(irefwr,*) 'deallocate ( E%secinv )'
      deallocate ( E%secinv )
      if ( debug ) write(irefwr,*) 'deallocate ( E%submat )'
      deallocate ( E%submat )
      if ( debug ) write(irefwr,*) 'deallocate ( E%submass )'
      deallocate ( E%submass )
      if ( E%typecontinuity/=0 .and. E%typecontinuity/=4 ) then
         if ( debug ) write(irefwr,*) 'deallocate ( E%divmat )'
         deallocate ( E%divmat )
         if ( debug ) write(irefwr,*) 'deallocate ( E%transpress )'
         deallocate ( E%transpress )
      end if  ! ( E%typecontinuity/=0 .and. E%typecontinuity/=4 )

      if ( E%useetaeff ) deallocate ( E%etaeff )

      if ( jtrans==1 .or. itype==902 .and. E%npelm==27 ) deallocate ( E%subvec )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900finish'

      end if  ! ( debug )
      call erclos ( 'sepel900finish' )

      end subroutine sepel900finish
