      subroutine sepel900rhsstr
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector for Navier-Stokes element due to newton
!     Old subroutine elm900rhsd
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     SEPELSTRRHS    Compute right-hand side contribution of a given initial
!                    stress tensor
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2873   upwind not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900rhsstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900rhsstr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%methodupw==0 ) then

!     --- No upwind required

         call sepelstrrhs ( E%coeffs(:,E%indstress), E%subvec  )

      else

!     --- Upwind not yet implemented

         call errchr ( 'Stress right-hand side', 1 )
         call errsub ( 2873, 0, 0, 1 )

      end if  ! ( metupw==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900rhsstr'

      end if  ! ( debug )
      call erclos ( 'sepel900rhsstr' )

      end subroutine sepel900rhsstr
