      subroutine sepelgrad
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill sp matrix, ie. the gradient of the pressure matrix
!     with respect to Crouzeix Raviart elements
!     Old subroutine elfillsp
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     gradient
!     pressure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: work(m)
      double precision, pointer :: vec(:)

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     vec            work array to store a vector
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgrad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelgrad'
         write(irefwr,1) 'cart, npress, ndim', B%cart, E%npress, E%ndim
         write(irefwr,1) 'ishape', E%ishape
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( B%cart )

      case(1)

!     --- case 1: Cartesian coordinates
!         First psi = 1

         do i = 1, E%ndim
            vec => E%gradmat(i:n*E%ndim:E%ndim,1)
            do j = 1, n
               vec(j) = sum(B%w*B%phider(j,:,i))
            end do  ! j = 1, n
         end do  ! i = 1, E%ndim

         if ( E%ishape==4 .or. E%ishape==6 .or. E%ishape==7 .or. &
              E%ishape==14 .or. E%ishape==16 ) then

!        --- psi = x  - xcent
!                   j        j

            do j = 1, E%ndim
               work = B%w*B%phipress(j+1,:)
               do i = 1, E%ndim
                  vec => E%gradmat(i:n*E%ndim:E%ndim,j+1)
                  do k = 1, n
                     vec(k) = sum(work*B%phider(k,:,i))
                  end do  ! k = 1, n
               end do  ! i = 1, E%ndim
            end do  ! j = 1, E%ndim

         end if  ! ( E%ishape==4 .or. E%ishape==6 .or. E%ishape==7 ... )

      case(3)

!     --- case 3: polar coordinates

         do i = 1, n
            E%gradmat(2*i-1,1) = &
               sum(B%w*(B%phider(i,:,1)+B%phi(i,:)/B%xyzg(:,1)))
            E%gradmat(2*i,1) = sum(B%w*B%phider(i,:,2)/B%xyzg(:,1))

            if ( E%npelm>5 ) then

               do j = 1, 2
                  work = B%w*B%phipress(j+1,:)
                  E%gradmat(2*i-1,j+1) = &
                     sum(work*(B%phider(i,:,1)+B%phi(i,:)/B%xyzg(:,1)))
                  E%gradmat(2*i,j+1) = sum(work*B%phider(i,:,2)/B%xyzg(:,1))

               end do  ! j = 1, 2

            end if  ! ( E%npelm>5 )

         end do

      case default

!     --- case 2: axi-symmetric coordinates

         do i = 1, n
            E%gradmat(2*i-1,1) = &
               sum(B%w*(B%phider(i,:,1)+B%phi(i,:)/B%xyzg(:,1)))
            E%gradmat(2*i,1) = sum(B%w*B%phider(i,:,2))

            if ( E%npelm>5 ) then

               do j = 1, 2

                  work = B%w*B%phipress(j+1,:)
                  E%gradmat(2*i-1,j+1) = &
                     sum(work*(B%phider(i,:,1)+B%phi(i,:)/B%xyzg(:,1)))
                  E%gradmat(2*i,j+1) = sum(work*B%phider(i,:,2))

               end do

            end if  ! ( E%npelm>5 )

         end do  ! i = 1, n

      end select  ! case ( B%cart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelgrad'

      end if  ! ( debug )
      call erclos ( 'sepelgrad' )

      end subroutine sepelgrad
