      subroutine sepelm900mat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for the incompressible Navier-Stokes equations
!     Old subroutine elm900matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INSTOP         Stop the program
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPEL900CONV   Fill convection tensor for Navier-Stokes into matrix S
!     SEPEL900CORR   compute the rotatio part of the element matrix for
!                    navier-stokes and store in matrix s
!     SEPEL900VISCST Fill stress tensor for Navier-Stokes into matrix S
!     SEPELM900SPV   compute the viscosity part of the element matrix for
!                    navier stokes and store in matrix s for 2d elements
!                    Special form of Navier-Stokes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900mat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900mat'
         write(irefwr,1) 'modelvisc', E%modelvisc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%submat = 0d0

!     --- First viscous terms

      if ( E%modelvisc==5 ) then

!     --- Special case the coefficients are given explicitly

         call sepelm900spv

      else

!     --- Standard case: Compute the viscous terms

         call sepel900viscst

      end if
      if ( debug ) &
         call prinrl1 ( E%submat, E%ndim*n, E%ndim*n, 'E%submat/1' )

!     --- Next part due to Coriolis acceleration

      if ( E%ind(E%indangvel)/=0 ) call sepel900corr

!     --- Finally convective terms

      if ( E%typeconvect>0 .and. E%typeconvect/=4 ) then

!     --- compute convective terms

         call sepel900conv

      else if ( E%seq_meshvel>0 ) then

!     --- No convection, only mesh velocity
      print *,'E%seq_meshvel ', E%seq_meshvel
      call ertrace
      call instop

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( E%submat, E%ndim*n, E%ndim*n, 'submat' )
         write(irefwr,*) 'End sepelm900mat'

      end if  ! ( debug )
      call erclos ( 'sepelm900mat' )

      end subroutine sepelm900mat
