      subroutine sepelm900vec
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector for Navier-Stokes element
!     Old subroutine elm900rhsd
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPEL900RHSCON Compute element vector for Navier-Stokes element
!                    due to convection
!     SEPEL900RHSD   Compute element vector for Navier-Stokes element due
!                    to external forces
!     SEPEL900RHSNWT element vector for Navier-Stokes element due to newton
!     SEPEL900RHSSTR element vector for Navier-Stokes element due to newton
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900vec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900vec'
         write(irefwr,1) 'metupw, jdiag, ndim, n, m', &
                          E%methodupw, B%jdiag, E%ndim, n, m
         write(irefwr,1) 'mconv, mcoefconv, jtime, modelv, mcont', &
                          E%typeconvect, E%modelconv, E%typetime, &
                          E%modelvisc, E%typecontinuity
         write(irefwr,2) 'cn', E%coeffs(1,E%indpower)
         call prinrl1 ( E%coeffs(:,E%indsource), E%ndim, m, 'f' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%subvec = 0d0

!     --- Compute contribution due to external forces

      if ( E%sourceterm ) call sepel900rhsd


!     --- Stress term corresponding to right-hand side

      if ( E%ind(E%indstress)/=0 ) call sepel900rhsstr

!     --- compute contribution of convective terms

      if ( E%typeconvect==2 .or. E%typeconvect ==4 ) &
         call sepel900rhscon

!     --- E%modelvisc = 2, and cn>1:  Newton linearization

      if ( E%modelvisc==2 .and. E%coeffs(1,E%indpower)>1d0 )&
         call sepel900rhsnwt

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%subvec, n, E%ndim, 'elemvc/end' )
         write(irefwr,*) 'End sepelm900vec'

      end if  ! ( debug )
      call erclos ( 'sepelm900vec' )

      end subroutine sepelm900vec
