      subroutine sepelmodelv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill modelvisc for Navier-Stokes equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPERR1305     Print error message 1305
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmodelv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmodelv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%modelvisc = &
         sepelgetintcf ( codemodel, ncodemodel, nscodemodel, 0, 'model' )

      E%modelrhsd = E%modelvisc/1000
      E%modelvisc = mod(E%modelvisc,1000)
      if ( E%modelvisc==6 ) E%modelvisc = 10
      if ( E%modelvisc<0 .or. E%modelvisc>5 .and. E%modelvisc<100 .and. &
           E%modelvisc/=10 .or. E%modelvisc>103 ) &
         call seperr1305 ( 2, E%modelvisc, 'modelvisc' )
      E%modelvisc = max ( E%modelvisc, 1 )
      if ( E%modelrhsd<0 .or. E%modelrhsd>1 ) &
         call seperr1305 ( 2, E%modelrhsd, 'modelrhsd' )

1000  call erclos ( 'sepelmodelv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmodelv'

      end if  ! ( debug )

      end subroutine sepelmodelv
