      subroutine sepelnsvisdisp ( elemvc, t, a1, &
                                  nelemvc, mcont, ndcomp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute viscous dissipation for Navier-Stokes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
     integer, intent(in) :: nelemvc, mcont, ndcomp
     double precision, intent(in) :: t(9,m), a1(9,m)
     double precision, intent(inout) :: elemvc(ndcomp,nelemvc)

!     a1             i    A1 tensor defined by gradv + gradv^T
!     elemvc         o    Element vector
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!     ndcomp         i    Indicates the number of quantities that must be filled
!                         in elemvc and elemwg per nodal point
!     nelemvc        i    Length parameter for elemvc
!     t              i    stress tensor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The viscous disspation is defined as 1/2 t :: A1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsvisdisp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsvisdisp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 .and. (jcart==1 .or. jcart==3) ) then

!     --- ndimlc = 2, Cartesian or Polar co-ordinates

         elemvc(1,:) = 0.5d0*(a1(1,:)*t(1,:)+a1(2,:)*t(2,:)+a1(4,:)*t(4,:))

      else if ( ndimlc==2 .and. jcart==2 ) then

!     --- ndimlc = 2, Cylinder co-ordinates

         elemvc(1,:) = 0.5d0*(a1(1,:)*t(1,:)+a1(2,:)*t(2,:)+a1(3,:)*t(3,:)+ &
                              a1(4,:)*t(4,:))

      else

!     --- ndimlc = 3

         elemvc(1,:) = 0.5d0*(a1(1,:)*t(1,:)+a1(2,:)*t(2,:)+a1(3,:)*t(3,:)+ &
                              a1(4,:)*t(4,:)+a1(5,:)*t(5,:)+a1(6,:)*t(6,:))

      end if  ! ( ndimlc==2 .and. (jcart==1 .or. jcart==3) )

1000  call erclos ( 'sepelnsvisdisp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsvisdisp'

      end if  ! ( debug )

      end subroutine sepelnsvisdisp
