      subroutine sepflindex4
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array index4 starting position for node in index2
!     Old subroutine to00641
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     topology
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, inunrf, inusol, inunkp, isum
      logical consta

!     consta         If true the number of unknowns per point is constant
!     inunkp         Number of degrees of freedom in point
!     inunrf         Reference value for inunkp
!     inusol         Position of unknown in array USOL
!     isum           Integer variable to compute a sum
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( indprf=0 ) then
!       index4(1,j) := j*inunkp
!    else ( indprf>0 )
!       Fill index4 using kprobf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflindex4' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflindex4'
         write(irefwr,1) 'indprf', indprf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      index4(1,1:inpelm) = index4tmp(1:inpelm)

      if ( indprf<=0 ) then

!     --- indprf = 0, number of unknowns constant

         inunkp = icount/inpelm
         if ( inunkp==1 ) then
            index4(1,inpelm+1) = 1
         else
            index4(1,inpelm+1) = 2
         end if

      else

!     --- indprf > 0, number of unknowns not constant

         isum = 0
         consta = .true.
         inusol = 0
         do j = 1, inpelm
            inunkp = index4tmp(j)-inusol
            if ( j==1 ) then
               inunrf = inunkp
            else
               if ( inunrf/=inunkp ) consta = .false.
            end if
            isum = isum + inunkp
            inusol = isum
         end do
         if ( consta .and. inunkp==1 ) then
            index4(1,inpelm+1) = 1
         else if ( consta ) then
            index4(1,inpelm+1) = 2
         else if ( isum>0 ) then
            index4(1,inpelm+1) = -1
         else
            index4(1,inpelm+1) = -2
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflindex4'

      end if  ! ( debug )
      call erclos ( 'sepflindex4' )

      end subroutine sepflindex4
