      subroutine el8004 ( x, u, p, dudx, gradv, ndcomp, elemvc, elast, &
                          anu, work )
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 1.0    date   03-09-1994 Subroutine extracted from el4917
!
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     This subroutine computes the stress tensor for rubber elements
!
! ********************************************************************
!
!                      KEYWORDS
!
!     derivative
!    rubber_element
! ********************************************************************
!
!                      COMMON BLOCKS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      integer ndcomp
      double precision p(*), x(n,*), u(n,*), dudx(n,ndimlc,*), work(m), &
                       gradv(9,n), elemvc(ndcomp,*), elast(m), anu(m)

!     anu     i    Poisson parameter nu
!     dudx    i    Real m x n x ncomp array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j, i )
!                    i    j   k
!                  If jdercn = 0 in common block celint, then only
!                  dudx (1,*,k) is filled because dudx is then a constant
!                  per element
!                  dudx is only filled if jderiv > 0
!     elast   i    Elasticity modulus E
!     elemvc  o    Element vector
!     gradv   i    Array containing the gradient vector in
!                  the integration points
!     ndcomp  i    Indicates the number of quantities that must be filled
!                  in elemvc and elemwg per nodal point
!     p       i    Array containing the pressure
!                  p(1) contains the pressure in the centroid
!                  p(2-4) contains the gradient of the pressure in the centroid
!     u       i    array of length m containing the values of the
!                  second coefficient in the integration points
!     work         Work array
!     x       i    array containing the coordinates of the nodal points
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
! ********************************************************************
!
!                    LOCAL PARAMETERS
!

      integer k, i, npres
      logical incomp
      save incomp, npres
      data incomp /.true./

!     i       Counting variable
!     incomp  If true the element is incompressible otherwise compressible
!     k       Counting variable
!     npres   Number of pressure parameters per element
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!     none
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   For the Cartesian case the following formulas are used:
!
!   sigma_xx = 2G eps_xx -2/3 G eps_v + p
!   sigma_yy = 2G eps_yy -2/3 G eps_v + p
!   sigma_zz = 2G eps_zz -2/3 G eps_v + p
!   sigma_xy =  G eps_xy
!   sigma_yz =  G eps_yz
!   sigma_zx =  G eps_zx
!
!   eps_v = eps_xx + eps_yy + eps_zz
!   In the incompressible case (nu=0.5) eps_v=0
! ======================================================================
!
!     --- Compute the gradient tensor and store in gradv

      if ( ifirst.eq.0 .or. ind(2).gt.0 ) then

!     --- First call or nu variable
!         Check nu and set incomp

         do 100 k = 1, m
            anu(k) = min ( anu(k), 0.49999d0 )
100      continue
         if ( ind(2).gt.0 ) then
            incomp = .false.
         else if ( anu(1).lt.0.499989d0 ) then
            incomp = .false.
         end if
         if ( n.eq.4 ) then
            npres = 1
         else
            npres = ndimlc+1
         end if

      end if

      if ( jcart.eq.1 ) then

!     --- Cartesian co-ordinates

         if ( ndimlc.eq.2 ) then

!        --- ndimlc = 2

            do 210 k = 1, m
               gradv(1,k) = dudx(k,1,1)
               gradv(2,k) = dudx(k,1,2)
               gradv(3,k) = dudx(k,2,1)
               gradv(4,k) = dudx(k,2,2)
210         continue


         else

!        --- ndimlc = 3

            do 230 k = 1, m
               gradv(1,k) = dudx(k,1,1)
               gradv(2,k) = dudx(k,1,2)
               gradv(3,k) = dudx(k,1,3)
               gradv(4,k) = dudx(k,2,1)
               gradv(5,k) = dudx(k,2,2)
               gradv(6,k) = dudx(k,2,3)
               gradv(7,k) = dudx(k,3,1)
               gradv(8,k) = dudx(k,3,2)
               gradv(9,k) = dudx(k,3,3)
230         continue

         end if

      else if ( jcart.eq.2 ) then

!     --- Cylinder co-ordinates

         if ( ndimlc.eq.2 ) then

!        --- ndimlc = 2

            do 250 k = 1, m
               gradv(1,k) = dudx(k,1,1)
               gradv(2,k) = dudx(k,1,2)
               gradv(3,k) = dudx(k,2,1)
               gradv(4,k) = dudx(k,2,2)
               if ( abs(x(k,1)).gt.epsmac ) then
                  gradv(5,k) = u(k,1)/x(k,1)
               else
                  gradv(5,k) = dudx(k,1,1)
               end if
250         continue

         else

!        --- ndimlc = 3, store in sequence r, z, phi
!                      It is assumed that the solution is stored in the
!                      sequence: ur, uz, uphi
!                      Remark: in the old version it is stored in the
!                              sequence r, phi, z
!                      the co-ordinate system should be in the same sequence

            do 270 k = 1, m
               gradv(1,k) = dudx(k,1,1)
               gradv(2,k) = dudx(k,1,2)
               gradv(3,k) = dudx(k,1,3)
               gradv(4,k) = dudx(k,2,1)
               gradv(5,k) = dudx(k,2,2)
               gradv(6,k) = dudx(k,2,3)
               if ( abs(x(k,1)).gt.epsmac ) then
                  gradv(7,k) = (dudx(k,3,1)-u(k,3))/x(k,1)
                  gradv(8,k) = dudx(k,3,2)/x(k,1)
                  gradv(9,k) = (dudx(k,3,3)+u(k,1))/x(k,1)
               else
                  gradv(7,k) = -dudx(k,1,3)
                  gradv(8,k) = 0d0
                  gradv(9,k) = dudx(k,1,1)
               end if
270         continue

         end if

      else if ( jcart.eq.3 ) then

!     --- Polar co-ordinates

         do 290 k = 1, m
            gradv(1,k) = dudx(k,1,1)
            gradv(2,k) = dudx(k,1,2)
            if ( abs(x(k,1)).gt.epsmac ) then
               gradv(3,k) = (dudx(k,2,1)-u(k,2))/x(k,1)
               gradv(4,k) = (dudx(k,2,2)+u(k,1))/x(k,1)
            else
               gradv(3,k) = -dudx(k,1,2)
               gradv(4,k) = dudx(k,1,1)
            end if
290      continue

      else if ( jcart.eq.4 ) then

!     --- Cylindrical co-ordinates with swirl (2D), &
!         store in sequence r, phi, z
!         It is assumed that the solution is stored in the sequence:
!         ur, uz, uphi

         do 310 k = 1, m
            gradv(1,k) = dudx(k,1,1)
            gradv(2,k) = dudx(k,1,2)
            gradv(3,k) = dudx(k,1,3)
            gradv(4,k) = dudx(k,2,1)
            gradv(5,k) = dudx(k,2,2)
            gradv(6,k) = dudx(k,2,3)
            gradv(8,k) = 0d0
            if ( abs(x(k,1)).gt.epsmac ) then
               gradv(7,k) = -u(k,3)/x(k,1)
               gradv(9,k) = u(k,1)/x(k,1)
            else
               gradv(7,k) = -dudx(k,1,3)
               gradv(9,k) = dudx(k,1,1)
            end if
310      continue

      end if

!     --- compute stress tensor

      do 380 k = 1, m
         do 370 i = 1, 6
            elemvc(i,k) = 0d0
370      continue
380   continue

!     --- Fill G in work

      do 400 k = 1, m
         work(k) = elast(k)/(2d0*(1d0+anu(k)))
400   continue

      if ( ndimlc.eq.2 .and. (jcart.eq.1 .or. jcart.eq.3) ) then

!     --- ndimlc = 2, Cartesian or Polar co-ordinates

         do 410 k = 1, m
            elemvc(1,k) = 2d0*work(k)*gradv(1,k)
            elemvc(2,k) = 2d0*work(k)*gradv(4,k)
            elemvc(4,k) = work(k)*(gradv(2,k)+gradv(3,k))
410      continue

!        --- correct with eps_v in case of compressible elements

         if ( .not. incomp ) then
            do 420 k = 1, m
               work(k) = -2d0/3d0*work(k)*(gradv(1,k)+gradv(4,k))
420         continue
         else
            do 430 k = 1, m
               work(k) = 0d0
430         continue
         end if

!        --- Compute contribution due to pressure

         if ( npres.eq.1 ) then
            do 440 k = 1, m
               work(k) = work(k) + p(1)
440         continue
         else
            do 450 k = 1, m
               work(k) = work(k) + (p(1)+p(2)*(x(k,1)-x(n,1)) + &
                                         p(3)*(x(k,2)-x(n,2)))
450         continue
         end if

         do 460 k = 1, m
            elemvc(1,k) = elemvc(1,k)+work(k)
            elemvc(2,k) = elemvc(2,k)+work(k)
460      continue

      else if ( ndimlc.eq.2 .and. jcart.eq.2 ) then

!     --- ndimlc = 2, Cylinder co-ordinates

         do 520 k = 1, m
            elemvc(1,k) = 2d0*work(k)*gradv(1,k)
            elemvc(2,k) = 2d0*work(k)*gradv(4,k)
            elemvc(3,k) = 2d0*work(k)*gradv(5,k)
            elemvc(4,k) = work(k)*(gradv(2,k)+gradv(3,k))
520      continue
      else

!     --- ndimlc = 3

         do 600 k = 1, m
            elemvc(1,k) = 2d0*work(k)*gradv(1,k)
            elemvc(2,k) = 2d0*work(k)*gradv(5,k)
            elemvc(3,k) = 2d0*work(k)*gradv(9,k)
            elemvc(4,k) = work(k)*(gradv(2,k)+gradv(4,k))
            elemvc(5,k) = work(k)*(gradv(6,k)+gradv(8,k))
            elemvc(6,k) = work(k)*(gradv(3,k)+gradv(7,k))
600      continue

!        --- correct with eps_v in case of compressible elements

         if ( .not. incomp ) then
            do 610 k = 1, m
               work(k) = -2d0/3d0*work(k)* &
                         (gradv(1,k)+gradv(5,k)+gradv(9,k))
610         continue
         else
            do 620 k = 1, m
               work(k) = 0d0
620         continue
         end if

!        --- Compute contribution due to pressure

         if ( npres.eq.1 ) then
            do 630 k = 1, m
               work(k) = work(k) + p(1)
630         continue
         else
            do 640 k = 1, m
               work(k) = work(k) + (p(1)+p(2)*(x(k,1)-x(n,1)) + &
                                         p(3)*(x(k,2)-x(n,2)) + &
                                         p(4)*(x(k,3)-x(n,3)))
640         continue
         end if

         do 650 k = 1, m
            elemvc(1,k) = elemvc(1,k)+work(k)
            elemvc(2,k) = elemvc(2,k)+work(k)
            elemvc(3,k) = elemvc(3,k)+work(k)
650      continue

      end if
      end
