      subroutine el8302 (Fd, Fn, p, coef, D_t, sigma, &
                         ichoice, nd, xcoor, xcoor0)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!        version  2.0  date 01-11-2002 extra parameters
!                                      extra parameter for elmul01/03
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Incompressible Neo-Hookean constitutive law
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, nd
      double precision D_t(nd,nd), Fd(ndim,ndim), Fn(ndim,ndim), p, &
                       coef(*), sigma(*), xcoor(ndim), xcoor0(ndim)

!     coef           i    Element coefficients
!     d_t            o    Matrix to be filled
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    Deformation gradient at start of timestep
!     ichoice        i    Calculate tang. stifness matrix+stress(0)
!                         or just stress(1)
!     nd             i    Dimension of tangent stiffness matrix
!     p              i    Hyrostatic pressure
!     sigma          o    Stress to be filled
!     xcoor          i    array of x coordinates in the listpack.
!     xcoor0         i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,j
      double precision F(ndim,ndim), B(3*ndim-3), G, B4

!     b              Finger strain
!     b4             ?
!     f              Deformation gradient
!     g              Shear modulus
!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2730         ?
!     ELMUL01        ?
!     ELMUL03        ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute Finger strain
!                          c      c
!         B = Fd * (Fn * Fn ) * Fd

      call elmul03 (Fd, Fn, F, 0, ndim)
      call elmul01 (F, B, 0, ndim)

!     --- axisymmetric case

      if (jcart==2) then
         B4=(xcoor(1)/xcoor0(1))**2d0
      end if

!     --- Get material parameters

      G = coef(10-5)

!     --- Compute tangential stiffness

      if (ichoice==0) then

!     --- D_t = G * delta(B)

         call el2730 (D_t, B, nd, B4)

         do i = 1, nd
            do j = 1, nd
               D_t(i,j) = G*D_t(i,j)
            end do
         end do

      end if

!     --- Compute stress
!         sigma = -pI + G*(B-I)

      if (ndim==2) then

       sigma(1) = -p + G*(B(1) - 1d0)
       sigma(2) = -p + G*(B(2) - 1d0)
       sigma(3) = G*B(3)

!      --- axisymmetric case

       if (jcart==2) then
          sigma(4) = -p + G*(B4 - 1d0)
       end if

      else if (ndim==3) then

       do i = 1, 3
          sigma(i) = -p + G*(B(i) - 1d0)
       end do
       do i = 4, 6
          sigma(i) = G*B(i)
       end do

      end if

      end
