      subroutine el8306 (Fd, Fn, p, coef, D_t, sigma, ichoice, nd)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 01-11-2002 extra parameters
!                                      extra parameter for elmul01/03
!        version  1.0  date 18-06-2001
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Incompressible Yeoh constitutive law
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, nd
      double precision D_t(nd,nd), Fd(ndim,ndim), Fn(ndim,ndim), &
                       p, coef(*), sigma(*)

!     coef           i    element coefficients
!     d_t            o    matrix to be filled
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    deformation gradient at start of timestep
!     ichoice        i    calculate tang. stifness matrix+stress(0)
!                         or just stress(1)
!     nd             i    Dimension of tangent stiffness matrix
!     p              i
!     sigma          o    stress to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision B(6), trB, trB3, c1, c2, c3, b4, &
                       D1(nd,nd), D2(nd,nd), F(ndim,ndim)

!     b              B-matrix
!     b4             ?
!     c1             Cash-Karp parameters (see Press et al. (1992))
!     c2             Helping constant
!     c3             Cash-Karp parameters (see Press et al. (1992))
!     d1             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    x-direction
!     d2             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    y-direction
!     f              ?
!     i              Counting variable
!     j              Counting variable
!     trb            ?
!     trb3           ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2730         ?
!     ELMUL01        ?
!     ELMUL03        ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute Finger strain
!                         c     c
!         B = Fd * Fn * Fn  * Fd

      call elmul03 (Fd, Fn, F, 0, ndim)
      call elmul01 (F, B, 0, ndim)

!     --- Compute tr(B)

      trB = B(1) + B(2) + B(3)
      trB3 = trB - 3d0

!     --- Get material parameters

      c1 = coef(10-5)
      c2 = coef(11-5)
      c3 = coef(12-5)

!     --- Compute tangential stiffness

      if (ichoice==0) then

!     --- fill in D_t
!         D_t = (c1 + 2*c2*(trB-3) + 3*c3*(trB-3)^2)*delta(B) + &
!               (2*c2 + 6*c3*(trB-3))*tr(delta B)*B

         do i = 1, 9
            do j = 1, 9
               D1(i,j) = 0d0
               D2(i,j) = 0d0
            end do
         end do

!        --- D1 = delta(B)

         B4 = 0d0
         call el2730 (D1, B, nd, B4)

!        --- second part

         do i = 1, 6
            do j = 1, 6
               D2(i,j) = 2d0*B(j)*B(i)
            end do
            do j = 7, 9
               D2(i,j) = 2d0*B(j-3)*B(i)
            end do
         end do
         do i = 7, 9
            do j = 1, 6
               D2(i,j) = 2d0*B(j)*B(i-3)
            end do
            do j = 7, 9
               D2(i,j) = 2d0*B(j-3)*B(i-3)
            end do
         end do

!        --- assemble

         do i = 1,9
            do j = 1, 9
               D_t(i,j) = (c1+2d0*c2*trB3+3d0*c3*(trB3**2))*D1(i,j) + &
                          (2d0*c2 + 6d0*c3*trB3)*D2(i,j)
            end do
         end do

      end if

!     --- Compute stress
!         sigma = -pI + ( c1 + 2*c2*(trB-3) + 3*c3*(trB-3)^2 ) B

!         write(*,*) c1*B(1), p

      do i = 1, 3
         sigma(i) = -p - c1 + &
                    (c1 + 2d0*c2*trB3 + 3d0*c3*(trB3**2))*B(i)
      end do
      do i = 4, 6
         sigma(i) = (c1 + 2d0*c2*trB3 + 3d0*c3*(trB3**2))*B(i)
      end do

!     --- write(*,*) 'B(1) =', B(1)
!         write(*,*) 'sigma = ', sigma(1), ' p =', p

      end
