      subroutine el8308 (Fd, Fn, p, coef, D_t, sigma, ichoice, nd)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 01-11-2002 extra parameters
!                                      extra parameter for elmul01/03
!        version  1.0  date 13-08-2001
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Incompressible general hyperelastic material
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, nd
      double precision D_t(nd,nd), Fd(ndim,ndim), Fn(ndim,ndim), &
                       p, coef(*), sigma(*)

!     coef           i    element coefficients
!     d_t            o    matrix to be filled
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    deformation gradient at start of timestep
!     ichoice        i    calculate tang. stifness matrix+stress(0)
!                         or just stress(1)
!     nd             i    Dimension of tangent stiffness matrix
!     p              i
!     sigma          o    stress to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision B(6), B2(6), I1, I2, W1, W2, W11, &
                       D1(9,9), D2(9,9), b4, &
                       F(3,3), c1, c2

!     b              B-matrix
!     b2             constant
!     b4             ?
!     c1             Cash-Karp parameters (see Press et al. (1992))
!     c2             Helping constant
!     d1             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    x-direction
!     d2             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    y-direction
!     f              array of length m containing the spectral element
!                    vector
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     j              Counting variable
!     w1             array of length m containing the weights for
!                    numerical integration in the x-direction corresponding
!                    to the reference element
!     w11            ?
!     w2             array of length m containing the weights for
!                    numerical integration in the y-direction corresponding
!                    to the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2730         ?
!     ELMUL01        ?
!     ELMUL03        ?
!     ELMUL04        ?
!     FNHYPER        ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute Finger strain
!                         c     c
!         B = Fd * Fn * Fn  * Fd

      call elmul03 (Fd, Fn, F, 0, ndim)
      call elmul01 (F, B, 0, ndim)

!     --- Compute B^2

      call elmul04 (B, B, B2, 0)

!     --- Compute I1 = tr(B)

      I1 = B(1) + B(2) + B(3)
      I2 = 0d0

!     --- Get material parameters

      c1 = coef(10-5)
      c2 = coef(11-5)

!     --- Get strain energy function

      call fnhyper (I1, I2, W1, W2, W11, c1, c2)

!     --- Compute tangential stiffness

      if (ichoice==0) then

!     --- fill in D_t
!         D_t = (c1 + c2*tr(B))*delta(B) + c2*tr(delta(B))B - &
!                c2*(delta(B)*B + B*delta(B))

         do i = 1, 9
            do j = 1, 9
               D1(i,j) = 0d0
               D2(i,j) = 0d0
            end do
         end do

!        --- D1 = delta(B)

         B4 = 0d0
         call el2730 (D1, B, ndim, B4)

!        --- second part

         do i = 1, 6
            do j = 1, 6
               D2(i,j) = 2d0*B(j)*B(i)
            end do
            do j = 7, 9
               D2(i,j) = 2d0*B(j-3)*B(i)
            end do
         end do
         do i = 7, 9
            do j = 1, 6
               D2(i,j) = 2d0*B(j)*B(i-3)
            end do
            do j = 7, 9
               D2(i,j) = 2d0*B(j-3)*B(i-3)
            end do
         end do

!        --- third part D4 = D3*D1

!            call el2731 (D3, idx, 6, B)
!            call dgemm ('n', 'n', 9, 9, 9, 1d0, D3, 9, D1, 9, 0d0, &
!            +               D4, 9)

!            --- assemble

         do i = 1,9
            do j = 1, 9
               D_t(i,j) = 2d0*W1*D1(i,j) + W11*D2(i,j)
            end do
         end do

      end if

!     --- Compute stress
!         sigma = -pI + 2 [ (W1 + W2*I1)*B - W2*B^2 ]

      do i = 1, 3
         sigma(i) = -p - 2d0*(W1 + 2d0*W2) + &
                    2d0*((W1 + W2*I1)*B(i) - W2*B2(i))
      end do
      do i = 4, 6
         sigma(i) = 2d0*((W1 + W2*I1)*B(i) - W2*B2(i))
      end do

      end
