      subroutine elnlav ( elemvc, blwork, pksest, w, &
                          ndim, icount, nnode, nip, nsc )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  2.1  date 14-09-2005 Roy Kerckhoffs, changed
!                                      blwork(6,3,8,*) into (6,3,nnode,nip)
!        version  2.0  date 28-07-1997 Marcel Rutten,changed declaration of
!                                      arguments
!        version  1.0  date 16-03-1997
!
!   copyright (c) 1997-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     assemble vector for nonlinear solid analysis
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          icount, ndim, nip, nnode, nsc
      double precision elemvc(icount), blwork(6,3,nnode,nip), &
                       pksest(6,nip), w(nip)

!     blwork     i    linear strain-displacement matrix: blwork(i,j,k,l)
!                     i: direction
!                     j: direction
!                     k: node
!                     l: integration point
!     elemvc     o    Element vector
!     icount     i    number of unknowns per element
!     ndim       i    Dimsion of the space.
!     nip        i    number of integration points per elementen
!     nnode      i    number of nodal points per element
!     nsc        i    number of stress components ( = ndim * 2)
!     pksest     i    Piola-Kirchoff stress estimate
!     w          i    array of length m containing the weights for integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer eln1, di1, t1, ix1, ipn
      double precision tmp1

!     di1
!     eln1
!     ipn
!     ix1
!     t1
!     tmp1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do eln1=1,nnode

!     --- loop over dimensions

         do di1=1,ndim

!        --- loop over integration points

            tmp1 = 0d0

            do ipn = 1, nip

!           --- sufficient loops to perform right multiplications

               do t1 =1, nsc

!              --- F = B  * S
!                       L

                  tmp1 = tmp1 + pksest(t1,ipn) * &
                         blwork(t1,di1,eln1,ipn) * &
                         w(ipn)

               end do
            end do

            ix1 = (eln1-1)*ndim + di1
            elemvc( ix1 ) = elemvc(ix1) - tmp1

         end do
      end do

      end
