      subroutine sepelmplate2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute shear part element matrix for linear elasticity element
!     Plate element
!     Old subroutine el8112
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     parabolic_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, l1, l2, lm, ip, iq
      double precision :: h, w(m), som
      double precision, pointer :: height(:), d(:,:,:), bs(:,:,:,:), s(:,:)

!     bs             Matrix B^s
!     d              Matrix D^s
!     h              Help variable to store a constant temporarily
!     height         Thickness of plate
!     i              Counting variable
!     ip             Counting variable
!     iq             Counting variable
!     j              Counting variable
!     l1             general loop variable
!     l2             Counting variable
!     lm             Upper part of loop
!     s              Element matrix to be filled
!     som            Help parameter to compute a sum
!     w              array of length m containing the weights for
!                    quadrature
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmplate2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmplate2'
         call prinrl1 ( E%Dmats, 2, 2, 'ds' )
         call prinrl1 ( E%Bs, 3, 2, 'bs' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      s => E%mat
      bs => E%Bs
      d => E%Dmats
      height => E%coeffs(:,22)

!     --- Multiply weight by h

      w = B%w*height
      if ( debug ) call prinrl ( w, m, 'w' )

      do i = 1, n
         do j = 1, i
            do l1 = 1, 3
               if(i.eq.j) then
                  lm = l1
               else
                  lm = 3
               end if
               do l2 = 1, lm
                  som = 0d0
                  do ip = 1, 2
                     do iq = 1, 2
                        som = som+sum(w(:)*bs(ip,l1,i,:)*d(ip,iq,:)* &
                                      bs(iq,l2,j,:))
                     end do
                  end do
                  s(3*i-3+l1,3*j-3+l2) = s(3*i-3+l1,3*j-3+l2) + som
               end do
            end do
         end do
      end do

!     --- Store upper part

      do i = 1, icount
         do j = 1, i-1
            s(j,i) = s(i,j)
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( s, icount, icount, 's' )
         write(irefwr,*) 'End sepelmplate2'

      end if  ! ( debug )
      call erclos ( 'sepelmplate2' )

      end subroutine sepelmplate2
