      subroutine sepelnlam ( se, w, ndim, icounG, nnode, nip, nsc, jcart )
! ======================================================================
!
!        programmer    Sander Gielen
!        version  2.1  date 14-09-2005 Roy Kerckhoffs, changed
!                                      blwork(6,3,8,*) into (6,3,nnode,nip)
!        version  2.0  date 21-01-1998 Marcel Rutten, added axisymmetry,
!                                      changed declaration of arguments
!                                      call extended
!        version  1.0  date 16-03-1997
!
!   copyright (c) 1997-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     assemble matrix for nonlinear solid analysis
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulebasefn
      use sepmoduleelm
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          icounG, ndim, nip, nnode, nsc, jcart
      double precision se(6,6,nip), w(nip)

!     icounG     i    size of elemmt (square)
!     jcart      i    indication whether the element is a triangle with
!                     cartesian coordinates (1)
!                     or a circular triangle with cylinder symmetric
!                     coordinates (2)
!     ndim       i    dimension of space
!     nip        i    number of integration points per element
!     nnode      i    number of nodal points per element
!     nsc        i    number of stress components ( = ndim * 2)
!     phi        i    array of length n * m, values of shape
!                     functions in integration points in the sequence:
!                            k
!                     phi (xg ) = phi (i,k)
!                        i
!                     array phi is only filled when gauss integration is applied
!     se         i    Tangential matrix
!     w          i    weights
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer eln1, eln2, di1, di2, idx(3,3), t1, t2, &
              ix1, ix2, ipn
      double precision st, tmp1, tmp2
      double precision, pointer :: phi(:,:), xgauss(:,:), pksest(:,:), &
                                   blwork(:,:,:,:), dphidx(:,:,:)

!     di1
!     di2
!     eln1
!     eln2
!     idx
!     ipn
!     ix1
!     ix2
!     st
!     t1
!     t2
!     tmp1
!     tmp2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data    idx /  1, 4, 6, 4, 2, 5, 6, 5, 3 /
! ======================================================================
!

      blwork => E%blloc
      dphidx => E%dphidx
      phi => B%phi
      xgauss => B%xyzg
      pksest => E%pksestloc

!     --- loop over all element nodes twice
!         use symmetry

      do eln1 = 1, nnode
         do eln2 = eln1, nnode

!        --- loop over degrees of freedom (=ndim) twice

            do di1 = 1, ndim
               do di2 = 1, ndim

!              --- compute the entry number in the matrix

                  ix1 = (eln1-1)*ndim + di1
                  ix2 = (eln2-1)*ndim + di2
                  st = 0d0

!                 --- symmetry component skip check

                  if ( ix1<=ix2 ) then

!                 --- loop over the integration points:

                     do ipn = 1, nip
                        tmp1 = 0d0
                        tmp2 = 0d0

!                       --- contribution to stiffness from stress

!                           See Klaus-Juergen Bathe: Finit element procedures, &
!                           Prentice Hall 1996
!                           pp. 542:
!                           t       /  t T   t     t
!                           K    = |   B     S     B  dV0
!                           0 NL    /  0 NL  0     0 NL
!                           V0

                        if ( di1==di2 ) then

                           do t1=1,ndim
                              do t2=1,ndim

                                 tmp1=tmp1+pksest(idx(t1,t2),ipn) &
                                      *dphidx(eln1,ipn,t1) &
                                      *dphidx(eln2,ipn,t2) &
                                      *0.5d0

!                                --- my assumption: the 0.5d0 in the last line
!                                    --- is because of the zeros in the i
!                                    Bnl matrix according to Bathe:
!                                    the number of flops is doubled in this way!
!                                    Anyway, as long as it works...  ( MarcelR!)

                              end do
                           end do

!                          --- add circumferential component for axisymmetric
!                              analysis:

                           if ( jcart==2 ) &
                           tmp1 = tmp1 + pksest(idx(3,3),ipn) &
                                * phi(eln1,ipn)/xgauss(ipn,1) &
                                * phi(eln2,ipn)/xgauss(ipn,1)

                        end if

!                       --- tangential contribution to stiffness

!                           See Klaus-Juergen Bathe: Finite element procedures, &
!                           Prentice Hall 1996
!                           pp. 542:
!                           t       /  t T         t
!                           K    = |   B     C     B  dV0
!                           0 L     /  0 L   0     0 L
!                           V0

                        do t1=1,nsc
                           do t2=1,nsc
                              tmp2=tmp2+ &
                                   blwork(t1,di1,eln1,ipn) &
                                   *se(t1,t2,ipn) &
                                   *blwork(t2,di2,eln2,ipn)
                           end do
                        end do
!                       --- sum contribution per integration point

                        st = st + w(ipn)*(tmp1+tmp2)
                     end do

                  end if

!                 --- add to matrix

                  E%mat(ix1,ix2) = E%mat(ix1,ix2) + st
!                 --- symmetry

                  if ( ix1/=ix2 ) then
                     E%mat(ix2,ix1) = E%mat(ix2,ix1) + st
                  end if

               end do
            end do

!           --- end dimension loop

         end do
      end do

!     --- end nodal loop

      end
