      subroutine sepfilldmats
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill elasticity matrix D^s for bending problem, depending on the
!     values of the elasticity E and the poisson ratio nu
!     The matrix D^s is the elasticity matrix for shear stiffness
!     Old subroutine el8110
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: elas(:), anu(:)
      double precision :: fac1, factor(m)

!     anu            Poisson parameter nu
!     elas           Elasticity parameter E
!     fac1           Multiplication factor
!     factor         Multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     400   The value of stress-strain is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilldmats' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilldmats'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      fac1 = 5d0/6d0

      select case ( E%stress_strain)
      case(0)

!     --- igprob = 1

         elas => E%coeffs(:,1)
         anu => E%coeffs(:,2)
         factor = elas*fac1 / (2d0*(1d0+anu) )
         E%Dmats(1,1,:) = factor
         E%Dmats(2,2,:) = factor

      case default

!     --- Not yet implemented

         call errint ( E%stress_strain+1, 1 )
         call errint ( 2, 1 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 400, 5, 0, 0 )

      end select  ! case ( E%stress_strain)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfilldmats'

      end if  ! ( debug )
      call erclos ( 'sepfilldmats' )

      end subroutine sepfilldmats
