      program practicum7_1

!     --- Sample program for the example in the introduction Section 7.1

      call startsepcomp
      end

!     --- It is necessary to define your own element subroutine

      subroutine elemsubr ( ndim, npelm, x, nunk_pel, elem_mat,
     +                      elem_vec, elem_mass, uold, itype )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date 15-10-1998
!
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Special user element subroutine to be used by SEPRAN in the case of
!     the Numerical Analysis Lab of Delft University of Technology
! **********************************************************************
!
!                       KEYWORDS
!
!    matrix
!    vector
!    element
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ndim, npelm, nunk_pel, itype
      double precision x(npelm,ndim), elem_mat(nunk_pel,nunk_pel),
     +                 elem_vec(nunk_pel), elem_mass(nunk_pel),
     +                 uold(nunk_pel)

!     elem_mass   o   In this two-dimensional array the student must store the
!                     element mass matrix, provided the mass matrix must be
!                     computed, in the following way:
!                     elem_mass(i,j) = s_{ij} ; i,j = 1(1)nunk_pel
!                     This matrix should only be filled if a mass matrix is
!                     required, for example for time-dependent problems.
!     elem_mat    o   In this two-dimensional array the student must store the
!                     element matrix, in the following way:
!                     elem_mat(i,j) = s_{ij} ; i,j = 1(1)nunk_pel.
!                     The degrees of freedom in an element are stored
!                     sequentially, first all degrees of freedom corresponding
!                     to the first point, then to the second, etcetera.
!     elem_vec    o   In this array the student must store the element vector,
!                     in the following way:
!                     elem_vec(i) = f_i; i = 1(1)nunk_pel
!                     It concerns the derived quantity that must be computed
!     itype       i   Type number of element.
!                     This parameter is defined in the input block PROBLEM
!     ndim        i   Dimension of the space.
!     npelm       i   Number of points per element
!     nunk_pel    i   number of degrees of freedom in the element
!     uold        i   In this array the old solution, as indicated by V1,
!                     is stored. This solution may contain the boundary
!                     conditions only, if the array has been created by
!                     prescribe_boundary_conditions, but also a starting vector
!                     if V1 has been created by create or even the previous
!                     solution in an iteration process if nonlinear_equations
!                     is used.
!     x           i   Contains the coordinates of the nodes of the element using
!                     the local node numbering
!                     x(i,1) contains the x-coordinate of the i-th node in the
!                     element and x(i,2) the y-coordinate of this node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision mu, e(3,2), delta, gradphi(3,2)
      integer i, j

!     delta       Jacobian delta of the element
!     e           Contains the factors e^ij according to
!                 e(i,j) = e^ij, i=1,2,3; j=1,2
!     gradphi     Contains the gradient of the basis function phi_i according to
!                 gradphi(i,j) = dphi_i/dx_j, i=1,2,3; j=1,2
!     i           General loop variable
!     j           General loop variable
!     mu          Parameter mu in the differential equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!    none
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    none
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The element matrix, element right-hand side and if the problem so
!    requires the element mass matrix are filled by the user, depending on
!    the parameter itype
!
!    The element matrix and element vector are defined in the Lecture Notes
!    "Numerieke methoden voor partiele differentiaalvergelijkingen"
!    See also the description in the manual for the formulas
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( itype.eq.1 ) then

!     --- Type = 1: mu = 1

         mu = 1d0

      else

!     --- Type = 2: mu = 2

         mu = 2d0

      end if

!     --- Compute the factors e_ij and delta as defined in the Lecture Notes
!         (pages 97,98)

      e(1,1) = x(2,2) - x(3,2)
      e(2,1) = x(3,2) - x(1,2)
      e(3,1) = x(1,2) - x(2,2)

      e(1,2) = x(3,1) - x(2,1)
      e(2,2) = x(1,1) - x(3,1)
      e(3,2) = x(2,1) - x(1,1)

      delta = e(3,1) * e(1,2) - e(3,2) * e(1,1)

!     --- Compute the gradient of the basis functions as defined in the
!         Lecture Notes

      do j = 1, 2
         do i = 1, 3
            gradphi(i,j) = e(i,j) / delta
         end do
      end do

!     --- Fill the element matrix as defined in the Lecture Notes

      do j = 1, 3
         do i = 1, 3
            elem_mat(i,j) = mu * 0.5d0 * abs(delta) *
     +         ( gradphi(i,1)*gradphi(j,1) + gradphi(i,2)*gradphi(j,2) )
         end do
      end do

!     --- The element vector is zero

      do i = 1, 3
         elem_vec(i) = 0d0
      end do

      end
