      program examtim1
! ======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.0    date   30-09-92
!        version 1.1    date   06-05-89
! ********************************************************************
!
!                         DESCRIPTION
!
!   This program computes the heat equation with self-selecting step size
!
! ********************************************************************
!
!                      KEYWORDS
!
!  heat_equation
!  main_program
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmoduleoldrouts
      use sepmoduletime
      implicit none
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer kmesh, kprob, intmat, isol, iuser(100),
     +        icount, itytim
      double precision user(100), epsusr

!    epsusr  Required accuracy of the time-solver
!    icount  General loop variable
!    intmat  Standard SEPRAN array containing information of the storage
!            scheme of the matrix
!    isol    Standard SEPRAN array containing information of the solution
!    itytim  Defines the type of problem to be considered
!            1 means linear problem
!    iuser   User array in which information with respect to the coefficients
!            of the differential equation is stored
!    kmesh   Standard SEPRAN array containing information of the mesh
!    kprob   Standard SEPRAN array containing information of the problem
!    user    double precision user array in which information with respect to
!            the coefficients of the differential equation is stored
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     create  Create a SEPRAN solution vector
!     finish  Stop SEPRAN
!     outtim  Write solution to file sepcomp.out
!     sepstr  Start SEPRAN and read mesh
!     soltim  Solve time-dependent problem
! ********************************************************************
!
!                    I/O
!
!    This program reads the standard SEPRAN files.
!    It assumes that the mesh has been generated by program SEPMESH
!    with input file meshext1.dat and output file meshoutput
!    It reads input from the standard input file (probext1.dat)
!    It produces one file: sepcomp.inf for program SEPPOST
!    Program SEPPOST requires the input file postext1.dat
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   -
! ********************************************************************
!
!                    PSEUDO CODE
!
!    Perform standard SEPRAN initialisations
!    Start SEPRAN and read mesh
!    Create initial condition
!    t := 0
!    Write solution for post processing purposes
!    For tout := 0.1 (0.1) 1 do
!       Solve heat equation from t to tout
!       t := tout
!       Write solution for post processing purposes
!    Stop SEPRAN
! ======================================================================
!
!    *  Standard SEPRAN initialisations

      user(1)  = 100
      iuser(1) = 100
      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0
      user(3)  = 0
      iuser(3) = 0

!    *  Start SEPRAN  (read mesh etc.)

      call sepstr( kmesh, kprob, intmat )

! *   fill initial temperature in isol :

      t      = 0d0
      call create ( 0, kmesh, kprob, isol )

!    *  Write initial condition to file sepcomp.out

      call outtim ( t, kmesh, kprob, isol )
!
! *******************************************************************
!
! *   set parameters for subroutine soltim:
!
! *******************************************************************
!
      itytim = 1
      ichtim = 1
      epsusr = 1d-3
      oldmethod = .true.
!
! *******************************************************************
!
! *   compute solution for t = 0.1, 0.2, . . . , 1.0
!
! *******************************************************************
!
!
      tend = 1d0
      do 200 icount = 1, 10
         tout = icount*0.1d0
         call soltim ( itytim, ichtim, kmesh, kprob, intmat, isol,
     +                 epsusr, user, iuser)

!     *    write solution to file sepcomp.out

         call outtim ( t, kmesh, kprob, isol )

200   continue

!    *  Close SEPRAN

      call finish(0)
      end
! *******************************************************************
! *
! *   function func for the initial condition :
! *
! *******************************************************************
      function func ( ichois, x, y, z )
      implicit none
      double precision func, x, y, z
      integer ichois

      func = 16d0*y*(1d0-x)*(1d0-y)*x

      end
! *******************************************************************
! *
! *   function for essential boundary conditions :
! *
! *******************************************************************
      function funcbc ( ichois, x, y, z )
      use sepmoduletime
      implicit none
      double precision funcbc, x, y, z
      integer ichois

      if ( ichois.eq.1 ) then
         funcbc = (1d0-x*x)*(1d0-exp(-10d0*t))
      else if ( ichois.eq.2 ) then
         funcbc = (y*y)*(1d0-exp(-10d0*t))
      else if ( ichois.eq.3 ) then
         funcbc = (x*x)*(1d0-exp(-10d0*t))
      else if ( ichois.eq.4 ) then
         funcbc = (1d0-y*y)*(1d0-exp(-10d0*t))
      end if

      end
! **********************************************************************
! *
! *   subroutine filtim for the control of the time-dependent process
! *
! **********************************************************************
      subroutine filtim ( kmesh, kprob, intmat, isol, matrs, matrm,
     +                    irhsd, islold, user, iuser )
      use sepmoduleoldrouts
      implicit none
      double precision user(*)
      integer kmesh, kprob, intmat, isol, islold,
     +        matrs, matrm, irhsd, iuser(*)
      integer ictime, iprob, iinbld(4)
      save ictime
      data ictime /0/
!
! *******************************************************************
! *
! *   User written subroutine for the incorporation of boundary
! *   conditions and the determination of the matrices M and S
! *   Example : Heat equation on the unit square .
! *
! *******************************************************************
!
! *   essential boundary conditions :
!

      call prestm ( ictime, kmesh, kprob, isol )
!
! *******************************************************************
! *
! *   Fill matrices M and S and right-hand side f
! *   Since the matrices are time-independent this is only
! *   necessary at t = 0
! *
! *******************************************************************
      if ( ictime.eq.0 ) then

!      *  Information is only read the first step
!         The parameter ictime is used to distinguish between first and
!         next step

         iprob = 1
         call filcof ( iuser, user, kprob, kmesh, iprob )

         iinbld(1) = 4
         iinbld(2) = 1
         iinbld(3) = 0
         iinbld(4) = 1
         call build ( iinbld, matrs, intmat, kmesh, kprob,
     +                irhsd, matrm, isol, islold, iuser, user )
         ictime = 1

      end if
      end
