      program examtimdep
! *******************************************************************
! *
! *   example for time dependent integration  (diagonal mass matrix)
! *   Second order (wave) equation
! *   programmer Niek Praagman
! *   date 06-05-89
! *
! *******************************************************************
!
      use sepmoduleoldrouts
      use sepmoduleconsts
      use sepmoduletime
      integer kmesh, kprob, intmat, isol(2), iuser(100),
     +        iu1(10), ichois, ichvc, icount, itytim, iu2,
     +        ivec
      integer iex(5)
      double precision user(100), u1(10), epsusr, u2, difmax

!    *  Standard SEPRAN initialisations

      user(1)  = 100
      iuser(1) = 100
      user(3)  = 0
      iuser(3) = 0
      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0

!    *  Start SEPRAN  (read mesh etc.)

      call sepstr( kmesh, kprob, intmat )
!
! *******************************************************************
!
! *   fill initial conditions in isol :
!
! *******************************************************************
!
      t      = 0d0
      t0     = 0d0
      ichois = 0
      ichvc  = 1
      iu1(1) = 1
!
      call creavc ( ichois, ichvc, ivec, isol(1), kmesh, kprob,
     +              iu1, u1, iu2, u2 )

!   *  Initial derivatives:

      iu1(1) = 0
      u1(1)  = 0d0
      call creavc ( ichois, ichvc, ivec, isol(2), kmesh, kprob,
     +              iu1, u1, iu2, u2 )

!    *  Write initial condition to file sepcomp.out

      call outtim ( t, kmesh, kprob, isol )
!
! *******************************************************************
!
! *   set parameters for subroutine soltim:
!
! *******************************************************************
!
      itytim = 1
      ichtim = 11
      epsusr = 0d0
      oldmethod = .true.
!
! *******************************************************************
!
! *   compute solution for t = 0.1, 0.2, . . . , 1.0
!
! *******************************************************************
!
!
      tend = 1d0

!    *  Read time-step

      read ( irefre,* ) tstep
      do 200 icount = 1, 10
         tout = icount*0.1d0
         call soltim ( itytim, ichtim, kmesh, kprob, intmat, isol,
     +                 epsusr, user, iuser)

!     *    write solution to file sepcomp.out

         call outtim ( t, kmesh, kprob, isol )

!     *  Compare with exact solution

         ichois = 0
         ichvc  = 1
         iu1(1) = 1
!
         call creavc ( ichois, ichvc, ivec, iex, kmesh, kprob,
     +                 iu1, u1, iu2, u2 )
         call diffvc ( 0, iex, isol, kprob, difmax )
         write ( irefwr, 100 ) t, difmax
100      format( 'Maximal difference at t = ',f5.1,' = ', d12.2 )

200   continue

!    *  Close SEPRAN

      call finish(0)
      end
! *******************************************************************
! *
! *   function func for the initial condition :
! *
! *******************************************************************
      function func ( ichois, x, y, z )
      use sepmoduletime
      double precision func, x, y, z
      integer ichois
!
      func =  cos(t) * ( x + 3d0 * y )
!
      end
! *******************************************************************
! *
! *   function funccf for the right-hand side
! *
! *******************************************************************
      function funccf ( ichois, x, y, z )
      use sepmoduletime
      double precision funccf, x, y, z
      integer ichois
!
      funccf = -cos(t) * ( x + 3d0 * y )
!
      end
! *******************************************************************
! *
! *   function for essential boundary conditions :
! *
! *******************************************************************
      function funcbc ( ichois, x, y, z )
      use sepmoduletime
      double precision funcbc, x, y, z
      integer ichois
!
!   *  Fill boundary conditions at new time-level (i.e. t+tstep)
!

      funcbc = cos(t+tstep) * ( x + 3d0 * y )
!
      end
! **********************************************************************
! *
! *   subroutine filtim for the control of the time-dependent process
! *
! **********************************************************************
      subroutine filtim ( kmesh, kprob, intmat, isol, matrs, matrm,
     +                    irhsd, islold, user, iuser )
      use sepmoduleoldrouts
      use sepmoduletime
      double precision user(*)
      integer kmesh, kprob, intmat, isol, islold,
     +        matrs, matrm, irhsd, iuser(*)
      integer ictime, ichois, imas, ielhlp, iprob
      save ictime
      data ictime /0/
!
! *******************************************************************
! *
! *   User written subroutine for the incorporation of boundary
! *   conditions and the determination of the matrices M and S
! *   Example : Wave equation on the unit square .
! *
! *******************************************************************
!
! *   essential boundary conditions :

      call prestm ( ictime, kmesh, kprob, isol )
!
! *******************************************************************
! *
! *   Fill matrices M and S
! *   Since the matrices are time-independent this is only
! *   necessary at t = 0
! *
! *******************************************************************
      if ( ictime.eq.0 ) then

!      *  Information is only read the first step
!         The parameter ictime is used to distinguish between first and
!         next step

         iprob = 1
         call filcof ( iuser, user, kprob, kmesh, iprob )
!
         ichois = 14
         imas = 1
         call systm1 ( ichois, imas, matrs, intmat, kmesh, kprob,
     +                 irhsd, matrm, isol, iuser, user, islold, ielhlp)
         ictime = 1
      end if
! *******************************************************************
! *
! *   Fill right-hand-side vector at time t, without the effect of
! *   boundary conditions
! *
! *******************************************************************
!
      ichois = 12
      imas = 0
      call systm1 ( ichois, imas, matrs, intmat, kmesh, kprob,
     +              irhsd, matrm, isol, iuser, user, islold, ielhlp)
      end
