! ********************************************************************
! *
! *   example for the solution of the Navier-Stokes equations
! *   channel flow in t-configuration  (only one half used)
! *   programmer Guus Segal
! *   date 01-10-86
! *
! ********************************************************************
! *
      program navsto
      use sepmoduleoldrouts
      integer kmesh, kprob, iuser(100), intmat, isol(2)
      integer itertp, maxitr, iprint, ichois, icheld, ix, iwork, ielhlp,
     .        jdegfd, ivec
      double precision user(100), eps, work
      iuser(1) = 100
      user(1)  = 100
      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0
      user(3)  = 0
      iuser(3) = 0

!
!    Array isol is used to store both the velocity and the pressure in the
!    following way:
!
!    isol(.,1):  velocity
!    isol(.,2):  pressure
!
! ********************************************************************
!
!   *  start of program  (standard for each sepran program)
!
! ********************************************************************
!
      call sepstr ( kmesh, kprob, intmat )
!
! ********************************************************************
!
!   *  fill initial guess and boundary conditions in solution array
!
! ********************************************************************
!
      ictimen = 0
      call create ( ictimen, kmesh, kprob, isol )
!
! ********************************************************************
!
!   *    solve the system of non-linear equations
!
! ********************************************************************
!
      itertp = 1
      maxitr = 10
      eps = 1d-4
      iprint = 1
      ichois = -1
      call nonlin ( itertp, maxitr, eps, iprint, iwork, work, intmat,
     .              ichois, iuser, user, kmesh, kprob, isol, ielhlp )
!
!*********************************************************************
!
!*   compute the pressure
!
!*********************************************************************
!
!     iuser, user as required by nonlin (is still present)
!
      icheld = 1
      call deriva ( ichois, icheld, ix, jdegfd, ivec, isol(2), kmesh,
     .              kprob, isol, isol, iuser, user, ielhlp )
!
!*********************************************************************
!
!*   write both the velocity and the pressure to the file sepcomp.out
!    They are postprocessed by SEPPOST
!
!*********************************************************************
!
      call output ( kmesh, kprob, isol )
      call finish(0)
      end




      subroutine filnln ( niter, iuser, user, kmesh, kprob, isol,
     .                    relax, differ )
      integer niter, iuser(*), kmesh, kprob, isol(*), iprob
      double precision user(*), relax, differ
!
!**********************************************************************
!
!   *  No relaxation
!
      relax = 0d0
      iprob = 1
!
!*********************************************************************
!
!*   Fill coefficients
!    In the first iteration the linear Stokes problem is solved
!    In the second iteration the non-linear Navier-Stokes equations with
!    a Picard linearization
!    In the next iterations the non-linear Navier-Stokes equations with
!    a Newton linearization
!
!*********************************************************************
!
      if ( niter.eq.0 ) then

!      *  niter = 0, Stokes problem

         call filcof ( iuser, user, kprob, kmesh, iprob )

      else if ( niter.eq.1 ) then

!      *  niter = 1, Navier-Stokes problem, Picard linearization

         call filcof ( iuser, user, kprob, kmesh, iprob )

      else if ( niter.eq.2 ) then

!      *  niter = 2, Navier-Stokes problem, Newton linearization

         call filcof ( iuser, user, kprob, kmesh, iprob )

      end if

      end
