      program nstoktim

!     Example of time-dependent solution of the Navier-Stokes equations
!     In fact a stationary problem is solved by time-stepping
!     The difference between two succeeding time-steps is printed

      use sepmoduleoldrouts
      use sepmoduleconsts
      use sepmoduletime
      implicit none
      integer kmesh, kprob, iuser(100), intmat,
     +        isol(3), matr, irhsd, iinvec(10), iinbld(10),
     +        massmt
      double precision user(100), rinvec(10)
      double precision ratime, thetan
      integer istep, irtime, iprob

!
!   *  Contents of array isol
!
!      isol(.,1):  un
!      isol(.,2):  un+1
!      isol(.,3):  p
!

      integer ictimen, nstep, nprint, ielhlp, ipoint, ichois, icheld,
     +        ivec, ix, jdegfd
      double precision p, q
      data iinvec / 10*0/

      iuser(1)=100
      user(1)=100
      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0
      matr = 0
      irhsd = 0
      massmt = 0

!     --- Start SEPRAN

      call sepstr ( kmesh, kprob, intmat )

!     --- boundary conditions

      call presdf ( kmesh, kprob, isol(2) )

!     --- Fill coefficients

      iprob = 1
      call filcof ( iuser, user,  kprob, kmesh, iprob )

!     --- Read parameters concerning the time-stepping process from the
!         input file

      read(irefre,*) thetan,tend,nstep,nprint
      write(irefwr,10) thetan,tend,nstep,nprint
 10   format(' theta,tend,nstep,nprint',2f10.2,2i5)
      dt=tend/nstep
      t = 0d0

!     --- fill initial condition  (u=0)

      ictimen = 0
      call create ( ictimen, kmesh, kprob, isol(1) )

!     --- loop over time steps

      do 300 istep=1,nstep

         if ( istep.le.1 ) then
            theta = 1d0
         else
            theta = thetan
         end if
         t=t+theta*dt

!        --- Build matrix

         iinbld(1) = 10
         iinbld(2) = 1
         iinbld(3) = 0
         iinbld(4) = 0
         iinbld(5) = 0
         iinbld(6) = 0
         iinbld(7) = 0
         iinbld(8) = 0
         iinbld(9) = 0
         iinbld(10) = 1
         call build ( iinbld, matr, intmat, kmesh, kprob, irhsd, massmt,
     +                isol(2), isol(1), iuser, user )

!        --- Solve system of equations u(n+theta)

         call solve(0,matr,isol(2),irhsd,intmat,kprob)

!        --- correction to compute u(n+1)

         if ( abs(theta-1d0).gt.1d-6 ) then

!        --- theta # 1

            iinvec(1) = 2
            iinvec(2) = 27
            rinvec(1) = 1d0/theta
            rinvec(2) = (theta-1d0)/theta
            call manvec ( iinvec, rinvec, isol(2), isol(1),
     +                    isol(2), kmesh, kprob )

         end if
         t=t+(1-theta)*dt

!        --- Compute difference between two succeeding time steps

         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(5) = 1
         iinvec(6) = 3
         call manvec ( iinvec, rinvec, isol(2), isol(1), isol,
     +                 kmesh, kprob )
         write(irefwr,120) istep, t, rinvec(1)
120      format('At time step',i4,'    time =',f10.4,
     +          '  max difference = ',ss,1pe11.3e2)

!        --- Copy u(n+1) into un

         call copyvc(isol(2),isol(1))

         if ( istep.eq.nstep .or. mod(istep,nprint).eq.0 ) then

!        --- Write result to file sepcomp.out
!            each nprint steps
!            First compute the pressure

            ichois = 0
            icheld = 1
            call deriva ( ichois, icheld, ix, jdegfd, ivec, isol(3),
     +                    kmesh, kprob, isol(2), isol(1), iuser,
     +                    user, ielhlp )
            call outtim ( t, kmesh, kprob, isol(2) )

         end if
300   continue
      call finish(0)
      end

!     --- Boundary conditions

      function funcbc( ifunc, x, y, z )
      implicit none
      double precision funcbc, x, y, z
      integer ifunc
      if( ifunc .eq. 1) then
        funcbc = 4d0 * x * ( 1d0 - x )
      else if( ifunc .eq. 2 ) then
        funcbc = 4d0 * (y-3) * ( 1d0 - (y-3) )
      endif
      end
