      program testeigv
! ======================================================================
!
!
!        programmer     Guus Segal
!        version 2.0    date   03-10-92 (Complete revision)
!        version 1.3    date   03-10-92 (Adapation of plots)
!        version 1.2    date   30-01-92 (Zdenek: eval(3))
!        version 1.1    date   22-08-91 (Zdenek: no dimension, cdc, ce)
!        version 1.0    date   01-10-86
!
! ********************************************************************
!
!                         DESCRIPTION
!
! *   example for the computation of eigenvalues
!
! ********************************************************************
!
!                      KEYWORDS
!
!     eigenvalue
!     eigenvector
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmoduleoldrouts
      use sepmodulecomio
      implicit none
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer kmesh, kprob, intmat, iinbld(10),
     .        istiff, mass, icheig(10), ivec(3),
     .        iprob, idummy, i, numeig, iuser(100), irhsd
      double precision user(100), eval(10)

!    eval     Variable length array in which the eigevalues are stored
!    i        General loop variable
!    icheig   Input array for subroutine EIGVAL
!             ICHEIG is filled as follows:
!             Pos. 1:  1, i.e. eigenvectors must be computed
!                  2:  1, i.e. information about the iteration must be printed
!                  3-8: 0, these psitions are not yet used
!    idummy   Dummy parameter
!    iinbld   Input array for subroutine BUILD
!             IINBLD is filled as follows:
!             Pos. 1:  4, i.e. positions 1-4 are filled
!                  2: 14, matrix is build, but not the right-hand side
!                  3:  1, the effect of boundary conditions is not taken into
!                         account
!                  4:  1, the mass matrix is diagonal
!    intmat   Standard SEPRAN array containing information of the storage
!             scheme of the matrix
!    iprob    problem sequence number
!    irhsd    Array containing information about the right-hand side
!    iuser    Variable length user array to be filled by FILCOF which
!             contains information about the coefficients
!    istiff   Contains information about the stiffness matrix
!    ivec     Contains information about the three eigenvectors
!             ivec(j,i), j=1,5 relates to eigenvector i
!    kmesh    Standard SEPRAN array containing information of the mesh
!    kprob    Standard SEPRAN array containing information of the problem
!    mass     Contains information about the mass matrix
!    numeig   Number of eigenvalues and eigenvectors to be computed
!    user     Variable length double precision user array to be filled by FILCOF
!             which contains information about the coefficients
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     build   Build matrices
!     eigval  Comute eigenvalues and eigenvectors
!     filcof  Fill parameters for equations
!     finish  Stop SEPRAN
!     output  Write eigenvectors to backing storage
!     sepstr  Start SEPRAN and read mesh
!
! ********************************************************************
!
!                    I/O
!
!    This program reads the standard SEPRAN files.
!    It assumes that the mesh has been generated by program SEPMESH
!    with input file eigvm.dat and output file meshoutput
!    It reads input from the standard input file (eigvp.dat)
!    It produces one file: sepcomp.inf for program SEPPOST
!    Program SEPPOST requires the input file eigval.pst
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   -
! ********************************************************************
!
!                    PSEUDO CODE
!
!   Fill length of variable length arrays
!   Start SEPRAN and read mesh
!   Fill coefficients
!   Build matrices
!   Compute eigenvalues and eigenvectors
!   Print eigenvalues
!   Write eigenvectors for postprocessing
! ======================================================================
!
!   *  Fill length of variable length arrays

      user(1)  = 100
      iuser(1) = 100
      eval(1)  = 10
      kmesh = 0
      kprob = 0
      intmat = 0
      istiff = 0
      mass = 0
      ivec = 0
      user(3)  = 0
      iuser(3) = 0
      eval(3)  = 0

!    * Start SEPRAN and read mesh

      call sepstr ( kmesh, kprob, intmat )

! *   fill parameters for Laplace equation

      iprob = 1
      call filcof ( iuser, user, kprob, kmesh, iprob )

! *   build mass matrix and stiffness matrix
!     not the right-hand side!

      iinbld(1) = 4
      iinbld(2) = 14
      iinbld(3) = 1
      iinbld(4) = 1
      call build ( iinbld, istiff, intmat, kmesh, kprob, irhsd, mass,
     .             ivec(1), idummy, iuser, user )

! *   compute eigenvalues and eigenvectors

      icheig(1) = 1
      icheig(2) = 1
      do 110 i = 3, 8
         icheig(i) = 0
110   continue
      numeig = 3
      call eigval ( eval, numeig, ivec, istiff, intmat, mass, intmat,
     .              kprob, icheig )

! *   output

      do 200 i = 1, numeig
         write ( irefwr, 120 ) i, eval(i+5)
120      format(' eigenvalue',i4,':',e12.5)
200   continue

!    *  write eigenvectors to "sepcomp.out"

      call output ( kmesh, kprob, ivec )

!    *  Stop SEPRAN

      call finish(0)
      end
