      program enthalpy1d_4

!     --- Main program for 1d stefan problem with different parameters
!         in both phases
!         The enthalpy method is applied
!         Solution by quasi-newton (Nedjar)

      implicit none
      call startsepcomp
      end

!     --- To define kappa as function of the temperature we
!         use funcc3

      function funcc3 ( ifunc, x, y, z, numold, maxunk, uold )
      implicit none
      integer ifunc, numold, maxunk
      double precision funcc3, x, y, z, uold(numold,maxunk)

!     --- local parameters

      double precision kappa_s, kappa_l, temperature,
     +                 melt_temp
      integer itemperature, ifirst
      save kappa_s, kappa_l, melt_temp, itemperature

      double precision getconst
      data ifirst /0/

      if ( ifirst==0 ) then

!     --- First call of subroutine
!         Get parameters from input file and increase ifirst
!         Since these parameters are saved, they are kept during the computation

         kappa_s = getconst('kappa_s')   ! kappa_s (solid)
         kappa_l = getconst('kappa_l')   ! kappa_l (liquid)
         melt_temp = getconst('melt_temp')   ! melting temperature
         call prgetname('temperature', itemperature)  ! sequence number of
                                         ! temperature in solution vector
         ifirst = 1                      ! raise ifirst in order to avoid
                                         ! repeating this part

      end if  ! ( ifirst==0 )

      if ( ifunc.eq.1 ) then

!     --- ifunc = 1, hence compute kappa

         temperature = uold(itemperature,1)   ! temperature in point

!        --- set kappa according to temperature

         if ( temperature>=melt_temp ) then

!        --- temperature above melting temperature use kappa_l

            funcc3 = kappa_l

         else

!        --- temperature below melting temperature use kappa_s

            funcc3 = kappa_s

         end if ! ( temperature )


      else

!     --- Error, this option has not yet been programmed

         call errchr('funcc3',1)
         call errint ( ifunc, 1 )
         call errsub ( 1930, 1, 0, 1 )
         call instop
         funcc3 = 0d0

      end if ! ( ifunc )

      end
