      program examtim3
      use sepmoduletime
      use sepmodulecomio
      implicit none

      integer kmesh(200), kprob(200), intmat(5), isol(5,2), iuser(100),
     +        icount, itytim, iincrt(3), iinout(1), iinstr(2),
     +        iinvec(6), iex(5)
      double precision user(100), rinvec(1), epsusr

!     --- Standard SEPRAN initialisations

      kmesh(1) = 200
      kprob(1) = 200
      user(1)  = 100
      iuser(1) = 100
      kmesh(3) = 0
      kprob(3) = 0
      user(3)  = 0
      iuser(3) = 0

!     --- Start SEPRAN  (read mesh etc.)

      iinstr(1) = 2
      iinstr(2) = 1
      call sepstn ( kmesh, kprob, intmat, iinstr )

!     --- fill initial field in isol

      t      = 0d0
      iincrt(1) = 0
      call creatn ( iincrt, kmesh, kprob, isol )

!     --- Write initial condition to file sepcomp.out

      iinout(1) = 0
      call outsol ( kmesh, kprob, isol, iinout, t, iuser, user )

!     --- set parameters for subroutine soltim:

      itytim = 1
      ichtim = 11
      tstep = 0.05d0
      epsusr = 0d0
      oldmethod = .true.

!     --- compute solution for t = 0.1, 0.2, . . . , 1.0

      tend = 1d0
      do 200 icount = 1, 10
         tout = icount*0.1d0
         call soltim ( itytim, ichtim, kmesh, kprob, intmat, isol,
     +                 epsusr, user, iuser)

!      --- write solution to file sepcomp.out

         call outsol ( kmesh, kprob, isol, iinout, t, iuser, user )

!        --- Compare with exact solution

         iincrt(1) = 3
         iincrt(2) = 0
         iincrt(3) = 2
         call creatn ( iincrt, kmesh, kprob, iex )
         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(3) = 0
         iinvec(4) = 1
         iinvec(5) = 1
         iinvec(6) = 3
         call manvec ( iinvec, rinvec, isol, iex, isol, kmesh, kprob )
         write ( irefwr, 100 ) t, rinvec(1)
100      format( 'Maximal difference at t = ',f5.1,' = ', d12.2 )

200   continue

!     --- Close SEPRAN

      call finish(0)
      end


!     --- function func for the initial condition

      function func ( ichoice, x, y, z )
      use sepmoduletime
      implicit none
      double precision func, x, y, z
      integer ichoice

      func =  cos(t) * ( x + 3d0 * y )

      end


!     --- function funccf for the right-hand side:

      function funccf ( ichoice, x, y, z )
      use sepmoduletime
      implicit none
      double precision funccf, x, y, z
      integer ichoice

      funccf = -cos(t) * ( x + 3d0 * y )

      end


!     --- function funcbc for essential boundary conditions


      function funcbc ( ichoice, x, y, z )
      use sepmoduletime
      implicit none
      double precision funcbc, x, y, z
      integer ichoice

      funcbc = cos(t+tstep) * ( x + 3d0 * y )

      end


!     --- subroutine filtim for the control of the time-dependent process


      subroutine filtim ( kmesh, kprob, intmat, isol, matrs, matrm,
     +                    irhsd, islold, user, iuser )
      implicit none
      double precision user(*)
      integer kmesh(*), kprob(*), intmat(*), isol(*), islold(*),
     +        matrs(*), matrm(*), irhsd(*), iuser(*)
      integer ictime, iinbld(4), iinprs(1), iinfil(1)
      save ictime
      data ictime /0/

!     --- User written subroutine for the incorporation of boundary
!         conditions and the determination of the matrices M and S
!         Example : Heat equation on the unit square.


!     --- essential boundary conditions

      iinprs(1) = 0
      call presbc ( kmesh, kprob, isol, iinprs )

!     --- Fill matrices M and S and right-hand side f
!         Since the matrices are time-independent the matrices
!         will only be filled at t = 0

      if ( ictime==0 ) then

!     --- Information is only read the first step
!         The parameter ictime is used to distinguish between first and
!         next step

         iinfil(1) = 0
         call fillcf ( iuser, user, kprob, kmesh, iinfil )

!        --- Do not take boundary conditions into account
!            Do not compyte the right-hand side
!            Mass matrix is diagonal

         iinbld(1) = 4
         iinbld(2) = 14
         iinbld(3) = 1
         iinbld(4) = 1
         call build ( iinbld, matrs, intmat, kmesh, kprob,
     +                irhsd, matrm, isol, islold, iuser, user )
         ictime = 1

      end if

!     --- Build the time-dependent right-hand side

      iinbld(1) = 3
      iinbld(2) = 2
      iinbld(3) = 1
      call build ( iinbld, matrs, intmat, kmesh, kprob,
     +             irhsd, matrm, isol, islold, iuser, user )
      end
