      program plume2d_dir

!     --- Main program for 2d  developing plume problem in the Earth's mantle
!         A direct linear solver is used
!         penalty function approach
!         The heat equation and Stokes equation are solved in a decoupled way
!         In each time step corresponding to the solution of the heat equation,
!         the stationary Stokes equations are solved
!         The main program is necessary because function subroutines are used

      integer, allocatable, dimension (:) :: ibuffr
      integer pbuffr, error
      parameter ( pbuffr=100000000)
      allocate(ibuffr(pbuffr), stat = error)
      if (error /= 0) then
         ! space for these arrays could not be allocated
         print *, "error: (sepcompexe) could not allocate space."
         stop
      end if  ! (error /= 0)
      call sepcombf ( ibuffr, ibuffr, pbuffr )

      end

!     --- Function func is used to define the initial temperature along the
!         lower wall

      function func ( ichoice, x, y, z )
      implicit none
      integer ichoice
      double precision func, x, y, z

      if ( ichoice==1 ) then

!     --- initial temperature distribution:

         func = max(0d0, 1d0-2*x)

      else

!     --- Other values of ichoice not permitted

         print *,'This value of ichoice (',ichoice,') has not been ',
     +           'implemented in function func'

      end if

      end


!     --- Function funcc3 is used to define the viscosity as function
!         of the temperature and the viscosity contrast for T=0 and T=1

      function funcc3 ( ifunc, x, y, z, numold, maxunk, uold )
      implicit none
      integer ifunc, numold, maxunk
      double precision funcc3, x, y, z, uold(numold,maxunk)

      integer iold, icount
      double precision deta, getconst

!     --- Make the following variables valid in all calls
!         Their values are kept

      save iold, icount, deta

!     --- Initialize icount to 0

      data icount /0/

      if ( ifunc==1) then

!     --- ifunc = 1, compute viscosity

         if ( icount==0 ) then

!        --- First step, get values of iold and deta
!            Since these parameters are saved, they have to computed only once
!            Get sequence number of temperature in solution vector
!            and store in iold
!            Get deta from the input file

            call prgetname ( 'temperature', iold )
            deta=getconst('deta')

!           --- change icount in order to prevent recomputing these constants

            icount = 1

         end if

!        --- Compute viscosity

         funcc3 = exp(-log(deta) * uold(iold,1) )

      else

!     --- Other values of ichoice not permitted

         print *,'This value of ifunc (',ifunc,') has not been ',
     +           'implemented in function funcc3'

      end if


      end

!     --- Subroutine userout is used to store the computed maximum
!         velocity into the file vmax.dat for postprocessing purposes

      subroutine userout ( kmesh, kprob, isol, isequence, numvec )
      implicit none
      include 'SPcommon/comcons1'
      include 'SPcommon/cuscons'
      include 'SPcommon/ctimen'

      integer kmesh(*), kprob(*), numvec, isol(5,*), isequence(*)

      double precision getvar

      integer icount, iref
      double precision vmax

!     --- Save icount so that the value is kept for a subsequent call

      save icount

!     --- Initialize icount to 0

      data icount /0/

      iref = 31
      if ( icount==0 ) then

!     --- Open the file vmax.dat in the first step (icount=0)
!         Since icount is saved, this is an allowed construction

         open ( unit=iref, file='vmax.dat' )
         icount = 1

      end if
      vmax = getvar('normvel')
      write(iref,*) t, vmax

      write(*,100)' time=', t,': Vmax=', vmax
100   format(a,f10.5,a,f10.5)

      end
