      subroutine inireceivempi ( ibufptr, icount, itype, isource, &
                                 icontext )
! ======================================================================
!
!        Programmer     Guus Segal
!        version 1.0    date   17-09-2001
!
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Wrapper for MPI_RECV
!     This implementation is completely based on MPI
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     receive
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecpack
      implicit none
      integer ibufptr, icount, itype, isource, icontext

!     ibufptr   i    pointer to first element of array to be received
!     icontext  i    the communication context, (message tag)
!     icount   i/o   the length of array to be received
!                    At input: the available length (this must be sufficient)
!                    At output: the actual number of data received
!                    NOTE: THIS VALUE WILL BE MODIFIED.
!                    It must be a variable on input
!     isource  i/o   rank of the source process.
!                    NOTE: THIS VALUE WILL BE MODIFIED.
!                    It must be a variable on input
!     itype     i    MPI datatype

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
!mpi      include 'mpif.h'
!mpi      integer ierr, istatus(MPI_STATUS_SIZE)

!     ierr      error status of an MPI call ( .eq.MPI_SUCCESS if ok)
!     istatus   MPI status array
! **********************************************************************
!
!                       I/O
!
!     Information is received on host or node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer value in error message file
!     ERRSUB         Error print routine
!     MPI_GET_COUNT  Determine how many items were actually received
!     MPI_RECV       Immediately receives the data in the active MPI message
!                    buffer
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1750   MPI error
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inireceivempi' )

!     --- Wait for a message

!mpi      call MPI_PROBE ( MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &
!mpi                       istatus, ierr )

!mpi      if ( ierr.ne.MPI_SUCCESS ) then
!mpi         call errint ( ierr, 1 )
!mpi         call errsub ( 1750, 1, 0, 0 )
!mpi      end if

!     --- Receive regular message

!mpi      isource = MPI_ANY_SOURCE
!mpi      call MPI_RECV ( ibufptr, icount, itype, isource, icontext, &
!mpi                      MPI_COMM_WORLD, istatus, ierr )

!mpi      if ( ierr.ne.MPI_SUCCESS ) then
!mpi         call errint ( ierr, 1 )
!mpi         call errsub ( 1750, 1, 0, 0 )
!mpi      end if

!mpi      call MPI_GET_COUNT ( istatus, itype, icount, ierr )

!mpi      if ( ierr.ne.MPI_SUCCESS ) then
!mpi         call errint ( ierr, 1 )
!mpi         call errsub ( 1750, 1, 0, 0 )
!mpi      end if

!mpi      isource = istatus (MPI_SOURCE)+1

      call erclos ( 'inireceivempi' )
      end
