/*
 *  Copyright (c) 1991  Philips Export B.V.
 *
 *  Philips Research Laboratories
 *  Eindhoven, The Netherlands
 *
 *  Author    : G.F.M. Haenen
 *              F.W.G.M. Wouters
 *
 *  Date      : July, 1991
 *
 *  Modified  : Kees Lemmens, July 5, 1999
 *            - string length computation failed for strings with colors
 *              fixed by using nchar%100 instead nchar.
 * 
 *            - Kees Lemmens, June 24, 1999
 *            - Removed all the Mrm stuff and the need for .uid file
 *              Widgets are created instead in mrmplugin.c. It works now
 *              also with the standard Lesstif under Linux, and without 
 *              producing the irritating UIL output garbage.
 *            - Added the text height to sepscan boundary detection
 *
 *            - Kees Lemmens, June 9, 1999
 *            - Now really rewritten to ANSI-C (prototypes) :-)
 *            - Cleaned up the grubby UNDERSCORE and GRAY stuff
 *            - several small bug fixes 
 *            - removed the global variables for window size and filename
 *            - pop renamed to wpop to fix a name conflict with Mrm-2.0
 * 
 *            : Eef Hartman, 15 feb 1995
 *            - Adaptations to binary input file and TUD situation
 *            - Complete rewrite to ANSI-C (prototypes) and Motif-types
 *
 *            : Dirk De Wachter, 28 jun 1995
 *            - Adaptation for new user interface modules
 *            - Change in sepscan for color usage
 *
 *            : Guus Segal, 20 jul 1995
 *            - 2.5% margin at both sides for xmin, xmax, ymin and ymax
 *           
 *            : Marcel Rutten, 17 dec 1996
 *            - changed background colours to more conservative ones
 *            - viewport is adapted to largest available when playing
 *              continuously: comes in handy with moving meshes
 *
 *            : Sander Gielen, march 1998
 *            - added fig output
 *
 *  Function  : Main sepview program
 *              It reads and scans the input file and calls the
 *              plotting routines
 *
 *  Comment   :
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/file.h>
#include <Xm/MessageB.h>
#include <Xm/DrawingA.h>
#include <Xm/Text.h>   /* XmTextSetString */
#include <Xm/Xm.h>
#include <X11/Xlib.h>
#include "plotfun.h"
#include "sepview.h"
/* #include "icon.h" */

#define nFiles(x)       (sizeof(x)/sizeof(char *))

/* Global variables */

Widget          graphics;

/* Local static variables */

static struct window_t usr;
static struct filnam_t filename;
static int             output = X_OUTPUT;
static int             playing = 0;   /* backward = -1; forward = 1 */

static int             ifirst=0;      /* first call for sepscan, added 17-12-96  */
static int             scaling=1;     /* flag for autoscaling */
static int             zoomstat=0;    /* flag for zooming */
static Widget          top;
static double xmin, xmax, ymin, ymax;

static XtAppContext  apco;
static Display       *dpy;
static Window        win;
static Widget        widgar[sepLast];


/* initSepview */


void initSepview(int argc,char *argv[])
{
   int c;
   extern char * optarg;
   extern int optind, optopt;
    
   strcpy(filename.basedir,  "");
   strcpy(filename.name,     "");
   filename.version = 0;
   filename.new     = TRUE;
   
   New_Colortable(NULL,filename.name[0]); /* first set default colors */
   
   while ((c=getopt(argc,argv,":hc:"))!= -1)
   {
      switch(c) {
       case 'c': New_Colortable(optarg,filename.name[0]);
	 break;
       case ':':
	 fprintf(stderr, "%s: option -c requires an argument\n", 
		 argv[0]);
       case '?':
       case 'h':
	 fprintf(stderr, "version: %s\n", VERSION);
	 fprintf(stderr, "usage:   %s -h\n", argv[0]);
	 fprintf(stderr, "usage:   %s [ -c <sepran color file>] [<sepran plot file>]\n", argv[0]);
	 exit(0);
      }
    }
   if (optind < argc) setDataFile(argv[optind++]);
}


/* check */


void check(int num)
{
  char   s1[MAXCOMMAND];
  Widget w;
  int r;
  
  switch (num - filename.version)
    {
    case -1:
      w = widgar[sepPreviousButton];
      break;
    case 1:
      w = widgar[sepNextButton];
      break;
    default:
      return;
    }
  
  sprintf(s1, "%s%s%03d", filename.basedir, filename.name, num);
  XtVaSetValues(w, XmNsensitive, r=access(s1, R_OK) == F_OK, NULL);
}


/* putXString */


void putXString(char *buf, int sepWid)
{
   XmString str;
  
  str = XmStringCreate(buf, XmSTRING_DEFAULT_CHARSET);
  XtVaSetValues(widgar[sepWid], XmNlabelString, str, NULL);
  XmStringFree (str);
  /* XmTextSetString(widgar[sepWid], buf);*/
}



/* modifyStatus */


void modifyStatus(int status)
{
  static char buf[MAXSTATUSLINE/2];

  if (status == 0)
    sprintf(buf, "Datafile: %s%03d", filename.name, filename.version);
  else
    sprintf(buf, "Datafile: ");
  
  putXString(buf, sepStatus);
}



/* showColorfile */


void showColorfile(char *name)
{
  static char buf[MAXSTATUSLINE/2];
  
  if (!name)
    sprintf(buf, "Colors: Default");
  else {
    char *p = strrchr(name,'/');
    if (!p) p=name;
    else p++;
    sprintf(buf, "Colors: %s", p);
  }

  putXString(buf, sepColorStatus);
}



/* waitForEverything */


void waitForEverything(Widget widget)
{
  XEvent event;
  
  while (!XCheckTypedWindowEvent(dpy, XtWindow(widget), Expose, &event));
  XtDispatchEvent(&event);
  XFlush(dpy);
}



/* errorMessage */


void errorMessage(char *message)
{
  static   int first = TRUE;
  XmString str;
  
  if (first)
    {
      first = FALSE;
      
      XtUnmanageChild(XmMessageBoxGetChild(widgar[sepErrorMessage], 
					   XmDIALOG_CANCEL_BUTTON));
      XtUnmanageChild(XmMessageBoxGetChild(widgar[sepErrorMessage], 
					   XmDIALOG_HELP_BUTTON));
    }

  str = XmStringCreate(message, XmSTRING_DEFAULT_CHARSET);

  XtVaSetValues(widgar[sepErrorMessage], XmNmessageString, str, NULL);
  XmStringFree(str);
  
  XtManageChild(widgar[sepErrorMessage]);
}


/* infoMessage */


void infoMessage(char *message)
{
    static int first = TRUE;
    XmString str;

    if (first)
    {
      first = FALSE;

      XtUnmanageChild(XmMessageBoxGetChild(widgar[sepInfoMessage], 
					   XmDIALOG_CANCEL_BUTTON));
      XtUnmanageChild(XmMessageBoxGetChild(widgar[sepInfoMessage], 
					   XmDIALOG_HELP_BUTTON));
      XtUnmanageChild(XmMessageBoxGetChild(widgar[sepInfoMessage], 
					   XmDIALOG_OK_BUTTON));
      XtUnmanageChild(XmMessageBoxGetChild(widgar[sepInfoMessage], 
					   XmDIALOG_SEPARATOR));
    }

    if (*message == '\0')
    {
        sleep(1);
        XtUnmanageChild(widgar[sepInfoMessage]);
    }
    else
    {
        str = XmStringCreate(message, XmSTRING_DEFAULT_CHARSET);

        XtVaSetValues(widgar[sepInfoMessage], XmNmessageString, str, NULL);
        XmStringFree(str);

        XtManageChild(widgar[sepInfoMessage]);

        waitForEverything(widgar[sepInfoMessage]);
    }
}


/* sepscan */



Boolean sepscan(int ifirst, int scaling)
/*Boolean sepscan()  argument added 17-12-96 */
{
   int    i, nchar, command;
   int    ios;			/* Fortran I/O status field */
   float  x, y, a, h, polyg[2*MAXPOLY];
   char   string[256];
   if (ifirst==0)
   {
      xmin = 1E10; xmax = -xmin; ymin = xmin; ymax = xmax;
      ifirst = 1;
   }
   
   plrd01 (&command, &x, &y, &a, &h, &nchar, &ios);
   while (ios == 0)
   {
      switch (command)
      {
       case 2: /* PLOT */
	 add_color(nchar);
       case 3: /* MOVE */
	 if ( zoomstat == 0 )
	 {
	    if (x < xmin) { xmin = x; };
	    if (x > xmax) { xmax = x; };
	    if (y < ymin) { ymin = y; };
	    if (y > ymax) { ymax = y; };
	 }
	 break;
       case 6: /* TEXT */
	 i = nchar % 100;                /* strip dcolor out of value */
	 plrd06 (&i, string, &ios);
	 
	 if (ios != 0) { return FALSE; };
	 if ( zoomstat == 0 )
	 {
            if (x < xmin) { xmin = x; };
            if (x + i*h > xmax) { xmax = x + i*h; };
            if (y < ymin) { ymin = y; };
            if (y + h > ymax) { ymax = y + h; };
	 }
	 add_color(nchar/100);
	 break;
       case 7: /* POLYLINE */
	 
	 plrd07 (&nchar, polyg, &ios);
	 
	 if (ios != 0) { return FALSE; };
	 if ( zoomstat == 0 )
	 {
	    for (i = 0; i < nchar; i++) 
	    {
	       if (polyg[2*i] < xmin) { xmin = polyg[2*i]; };
	       if (polyg[2*i] > xmax) { xmax = polyg[2*i]; };
	       if (polyg[2*i+1] < ymin) { ymin = polyg[2*i+1]; };
	       if (polyg[2*i+1] > ymax) { ymax = polyg[2*i+1]; };
	    }
	 }
	 add_color((int) x);
	 break;
       case 8: /* SYMBOL */
	 if ( zoomstat == 0 )
	 {
	    if (x < xmin) { xmin = x; };
	    if (x > xmax) { xmax = x; };
	    if (y < ymin) { ymin = y; };
	    if (y > ymax) { ymax = y; };
	 }
	 add_color(nchar / 100);
	 break;
       case 9: /* ????????? */
	 add_color((int) x);
	 break;
      }
      
      plrd01 (&command, &x, &y, &a, &h, &nchar, &ios);
      
   }
   if (command == 5) /* CLOSE */
   {
      /*   dx = xmax-xmin;  dy = ymax-ymin;
       xmin = xmin-0.025*dx;  xmax = xmax+0.025*dx;
       ymin = ymin-0.025*dy;  ymax = ymax+0.025*dy; */
      return TRUE; 
   }               /* last record was plot close */
   else              
     return FALSE;
}



/* sepplot */



void sepplot(void)
{
  int    i, col, command, nchar, ios, fillen;
  float  x, y, a, h, polyg[2*MAXPOLY];  /* REAL's from Fortran files */
  double xcoord, ycoord, angle, height;
  double xstart = 0, ystart = 0;
  double polyx[MAXPOLY], polyy[MAXPOLY];
  char   string[256], fileid[MAXFILENAME];
  static char prevfile[MAXFILENAME] = {'\0'};

  sprintf(fileid, "%s%s%03d", filename.basedir, filename.name, 
	  filename.version);

  /*   for (fillen=0; *(fileid+fillen); fillen++) ; */
  fillen = strlen(fileid);
  plrdop (fileid, &fillen, &ios);

  if (ios != 0)
    {
      switch (playing) {
      case 0:
	sprintf(string, "Unable to open file : %s", fileid);
	errorMessage(string);
	/* the next command is to avoid a redraw */
	filename.version = 0;
	break;
      case 1:
	playing = 1;
	filename.version = 0;
	/* These two statements were: playing=-1;*/
	break;
      case -1:
	playing = 1;
	break;
      }
      return;
    }
  else {
    clear_colors();
    
    if ( sepscan( ifirst, scaling ) ) {
      /*        if ( sepscan() ) {  same here */

      plrdcl (&ios);
      plrdop (fileid, &fillen, &ios);

    }
    else {
      fprintf(stderr,"error scanning the file\n");
      return;
    }
  }
   
   if (strcmp(fileid, prevfile))
   {
      strcpy(prevfile, fileid);
      filename.new = TRUE;
   }
   
   check(filename.version - 1);
   check(filename.version + 1);
   modifyStatus(0);
   
   plrd01 (&command, &x, &y, &a, &h, &nchar, &ios);
/* 
 fprintf(stderr,"min: (%f,%f) ; max: (%f,%f) \n",xmin,ymin,xmax,ymax); 
*/   
   while (ios == 0)
   {
      switch (command)
        {
        case 1: /* START */
	  if ( filename.new )
            {
	       usr.xmin = xmin; usr.ymin = ymin;
	       usr.xmax = xmax; usr.ymax = ymax;
	       usr.width  = usr.xmax - usr.xmin;
	       usr.height = usr.ymax - usr.ymin;
            }
	  if ( scaling == 0 )
            {
              ifirst = 0; 
            }
	  else
            {
              ifirst = 1;
            }
	  if (!init(&usr,&filename,output))
	    goto error;
	  color(-1);
	  break;
        case 2: /* PLOT */
	  color(nchar);
	  xcoord = x;
	  ycoord = y;
	  line(xstart, ystart, xcoord, ycoord);
	  xstart = xcoord;
	  ystart = ycoord;
	  break;
        case 3: /* MOVE */
	  xstart = x;
	  ystart = y;
	  break;
        case 5: /* CLOSE */
	  pl_finish();
	  break;
        case 6: /* TEXT */
	  xcoord = x;
	  ycoord = y;
	  angle = a;
	  height = h;
	  col = nchar/100;
	  nchar = nchar%100;

	  plrd06 (&nchar, string, &ios);

	  *(string + nchar) = '\0';
	  color(col);
	  text(xcoord, ycoord, height, angle, string);
	  break;
        case 7: /* POLYLINE */
	  col = (int) x;

	  plrd07 (&nchar, polyg, &ios);

	  for (i = 0; i < nchar; i++)
            {
	      polyx[i] = polyg[2*i];
	      polyy[i] = polyg[2*i+1];
	    }
	  color(col);
	  polyfill(polyx, polyy, nchar);
	  break;
        case 8: /* SYMBOL */
	  xcoord = x;
	  ycoord = y;
	  color(nchar / 100);
	  symbol(xcoord, ycoord, nchar % 100);
	  break;
        case 9: /* ????????? */
	  col = (int) x;

	  plrd07 (&nchar, polyg, &ios);

	  for (i = 0; i < nchar; i++)
            {
	      polyx[i] = polyg[2*i];
	      polyy[i] = polyg[2*i+1];
	    }
	  color(col);
	  polyline(polyx, polyy, nchar);
	  break;
        default:
	  fprintf(stderr, "Unknown command: %d\n", command);
	  break;
        }

      plrd01 (&command, &x, &y, &a, &h, &nchar, &ios);

    }
error:

  plrdcl (&ios);

  filename.new = FALSE;
}



/* setDataFile */


void setDataFile(char *fn)
{
   /* check whether it is a colorfile or a plotfile:
      case colorfile, if file is OK, enable "Default colors"*/
    char *sp;
    char s1[MAXCOMMAND];

    sp = strrchr(fn, '/');
    if (sp != NULL)
    {
        strcpy(s1, sp + 1);
        *(sp+1) = '\0';
        strcpy(filename.basedir, fn);
    }
    else
    {
        strcpy(s1, fn);
        strcpy(filename.basedir, "");
    }

    filename.version = 0;

    if (sscanf(s1, "%[^.].plot%3d", filename.name, &filename.version) == 2)
        strcat(filename.name, ".plot");
    else if (sscanf(s1, "%[^.].%3d", filename.name, &filename.version) == 2)
        strcat(filename.name, ".");
    else {
        sprintf(filename.name, "%s.plot", s1);
        filename.version = 1;
    }

    sepplot();
    XtVaSetValues(widgar[sepPlayButton], XmNsensitive, TRUE, NULL);

}



/* mark */


void mark(Widget widget, int tag)
{
    widgar[tag] = widget;

   /* fprintf(stderr,"mark called with tag : %d\n",tag);
    */
   switch (tag)
    {
    case sepStatus:
        modifyStatus(1);
        break;
    case sepDrawArea:
        graphics = widget;
        break;
    }
}



/* 
zoom_status 
sets zoom buttons active after zooming operation
*/


void zoom_status(int s)
{
   /* XtSetSensitive(widgar[sepZoomReset], s); //KL:  always enabled
   */
   XtSetSensitive(widgar[sepZoomOut], s);
}

void color_status(int s)
{
    XtSetSensitive(widgar[sepColorDefault], s);
}


/* wpop */


 
void wpop(Widget widget, XtPointer ptag, XtPointer reason)
{
   int tag = (int )ptag;
   
   /* fprintf(stderr,"wpop called with tag : %d\n",tag);
   */
   switch (tag)
    {
    case sepManForm:
        displayMan();
        break;
    }
    XtManageChild(widgar[tag]);
}



/* unwpop */


void unwpop(Widget widget, XtPointer ptag,XtPointer reason)
{
   char *fn;
   int tag = (int)ptag;
   
   /* fprintf(stderr,"unwpop called with tag : %d\n",tag);
   */
   switch (tag)
   {
      
    case sepMainWindow:
      exit(0);
    case sepFileOpen:
      XtUnmanageChild(widgar[sepFileOpen]);
      XmStringGetLtoR(((XmSelectionBoxCallbackStruct *)reason)->value,
		      XmSTRING_DEFAULT_CHARSET, &fn);
      setDataFile(fn);
      XtFree(fn);
      break;
    case sepNoFileOpen:
      XtUnmanageChild(widgar[sepFileOpen]);
      break;
    case sepColorOpen:
      XtUnmanageChild(widgar[sepColorOpen]);
      XmStringGetLtoR(((XmSelectionBoxCallbackStruct *)reason)->value,
		      XmSTRING_DEFAULT_CHARSET, &fn);
      New_Colortable(fn,filename.name[0]);
      XtFree(fn);
      break;
    case sepNoColorOpen:
      XtUnmanageChild(widgar[sepColorOpen]);
      break;
    default:
      XtUnmanageChild(widgar[tag]);
   }
}



/* 
scanbutton 

Operates the text of the stop/play button
*/



Boolean scanbutton(mode)
int *mode;
{
    char *text;
    XmString str;

    switch(*mode)
    {
    case  0:
       text="PLAY";
       str = XmStringCreate(text, XmSTRING_DEFAULT_CHARSET);
       XtVaSetValues(widgar[sepPlayButton], XmNlabelString, str, NULL);
       XmStringFree(str);
       return TRUE;
    case -1:
       --filename.version;
       sepplot();
       return FALSE;
    case  1:
       ++filename.version;
       sepplot();
       return FALSE;
    }
    return TRUE;
}



/* 
job 

Does all the user's fiddling with the knobs.
*/



void job(Widget widget, XtPointer ptag,XtPointer reason)
{
   XmDrawingAreaCallbackStruct *cb;
   char *text;
   XmString str;
   static Boolean firstExpose = TRUE;

   int tag = (int)ptag;
   
   /* fprintf(stderr,"job called with tag : %d\n",tag);
    */
   switch (tag)
   {
      /* reverse, forward and play */
      
    case sepPlayButton:
      playing = !playing;
      if (playing)
      {
	 text="STOP";
	 str = XmStringCreate(text, XmSTRING_DEFAULT_CHARSET);
	 XtVaSetValues(widgar[sepPlayButton], XmNlabelString, str, NULL);
	 XmStringFree(str);
	 
	 XtAppAddWorkProc(apco, (XtWorkProc) scanbutton, (XtPointer) &playing);
          }
      break;
    case sepPreviousButton:
      --filename.version;
      sepplot();
      break;
    case sepNextButton:
      ++filename.version;
      sepplot();
      break;
      
      /* The hardcopies */
      
    case sepPsOutputButton:
      output = PS_OUTPUT;
      sepplot();
      output = X_OUTPUT;
      break;
    case sepHpglOutputButton:
      output = HPGL_OUTPUT;
      sepplot();
      output = X_OUTPUT;
      break;
    case sepGIFOutputButton:
      {
	 write_gif();
	 break;
      }
      
    case sepGIFOutputall:
      {
	 char  * filnam;
	 filename.version = 1;
	 filnam = (char *) malloc(80*sizeof(char));
	 sprintf(filnam,"%s%03d",filename.name,filename.version);
	 while ( (access(filnam, F_OK) )  !=(-1) )
         {
	    sepplot();
	    write_gif();
	    ++filename.version;
	    sprintf(filnam,"%s%03d",filename.name,filename.version);
         }
	 free(filnam);
	 break;
      }
      
    case sepPPMOutputButton:
      {
	 write_ppm();
	 break;
      }
      
    case sepPPMOutputall:
      {
	 char  * filnam;
	 filename.version = 1;
	 filnam = (char *) malloc(80*sizeof(char));
	 sprintf(filnam,"%s%03d",filename.name,filename.version);
	 while ( (access(filnam, F_OK) )  !=(-1) )
         {
	    sepplot();
	    write_ppm();
	    ++filename.version;
	    sprintf(filnam,"%s%03d",filename.name,filename.version);
         }
	 free(filnam);
	 break;
      }
      /* new fig button */
    case sepFigOutputButton:
      output = FIG_OUTPUT;
        sepplot();
      output = X_OUTPUT;
      break;
      
      /* Viewport settings */
      
    case sepAutoScale:
      {
	 if ( scaling == 0 )
	 {
	    scaling = 1; 
	    infoMessage("Autoscaling OFF");
	    infoMessage("");
	 }
	 else
	 {
	    scaling = 0;
	    infoMessage("Autoscaling ON");
	    infoMessage("");
	 }
	 break;
      }
    case sepDrawArea:
      if (firstExpose) {
	 if (( (XmAnyCallbackStruct *)reason)->reason == XmCR_RESIZE) break;

	 /*KL Next causes a BadDrawable (invalid Pixmap or Window parm) error
	  */
	 /* XtVaSetValues(top, XmNiconPixmap,
			  XCreateBitmapFromData(dpy, win,(char *)icon_bits,
						icon_width, icon_height),
			  NULL);
	  */
	 firstExpose = FALSE;
      }
      
      if (filename.version > 0)
      {
	 cb = (XmDrawingAreaCallbackStruct *) reason;
	 /*if (cb->event == NULL)*/
	 if ((cb->event == NULL)||(cb->event->type != Expose))
	   sepplot();
	 else {
	    redraw(cb->event->xexpose.x, cb->event->xexpose.y,
		   cb->event->xexpose.width, cb->event->xexpose.height);
	 }
      }
      break;
      
    case sepZoomOut:
      x_init(&usr,3);
      sepplot();
      zoomstat = 0;
      break;
    case sepZoomReset:
      while (x_init(&usr,3) > 1) ;
      sepplot();
      zoomstat = 0;
      break;
      
      /* Colormaps */
      
    case sepColorDefault:
      New_Colortable(NULL,filename.name[0]);
      break;
    default:
      break;
   }
}

void write_gif(void)
{
   XImage *im;
   extern int R[], G[], B[];
   int Ncol, w,h, *ScLine;
   extern XImage * ImgPickImage(int *, int *);
   
   im = ImgPickImage(&w,&h);
   if (im) 
   {
      ImgPickPalette(&Ncol,R,G,B);
      GIF_init(&filename);
      ScLine = (int *) malloc(w*sizeof(int));
      GIFencode(w,h,Ncol,R,G,B,ScLine,im);
      free(ScLine);
      ImgFreeImage(im);
      GIF_finish();
   }
}

void write_ppm(void)
/* Write ppm ( Portable Pix Map ) file: */

{
   FILE *fid;
   XImage *frame;
   extern int R[], G[], B[];
   int Ncol, w, h, i, j, k; 
   int *ScLine;
   extern XImage * ImgPickImage(int *, int *);
   char string[80], PPMfile[80];
   
   frame = ImgPickImage(&w, &h);
   
   if (frame) 
   {
      
      /* set filename and colormap: */
      
      sprintf(PPMfile, "%s%03d.ppm",filename.name,filename.version);
      
      ScLine = (int *) malloc(w*sizeof(unsigned long));
      ImgPickPalette(&Ncol,R,G,B);
      if ( (fid = fopen(PPMfile,"w+")) != NULL)
      {
	 sprintf(string, "Creating PPM file: %s...", PPMfile);
	 infoMessage(string);
	 
	 /* write magic number + size and number of colors: */
	 
	 fprintf(fid,"%c%c\n%d %d\n%d\n",'P','6',w,h,Ncol-1);
	 
	 /* write pixmap: */
	 
	 for  (i = 0; i < h; i++)
	 {
	    for ( k = 0; k < w; k++ )
	    {
	       ScLine[k] = XGetPixel(frame, k, i); 
	    }
	    for ( j = 0; j < w; j++ )
	    {
	       putc((unsigned char)(R[(ScLine[j])]>>8),fid);
	       putc((unsigned char)(G[(ScLine[j])]>>8),fid);
	       putc((unsigned char)(B[(ScLine[j])]>>8),fid);
	    }
	 }
	 fclose(fid);
	 infoMessage("");
      }
      else
      {
	 sprintf(string,"Unable to open file: %s",PPMfile);
	 errorMessage(string);
      } 
      free(ScLine);
   }
}


/*  displayMan : displays the man page for sepview  */

int displayMan(void)
{
   static char *text;
   static int first = TRUE;
   
   if (first)
   {
      int  fd,pathlen;
      long fsize;
      long i, j;
      char manpage[256];
      char pathname[MAXFILENAME];
      
      first = FALSE;
      
      XtVaSetValues
	(widgar[sepManText],
	 XmNtopAttachment         , XmATTACH_FORM,
	 XmNtopOffset             , NICESPACEV,
	 XmNleftAttachment        , XmATTACH_FORM,
	 XmNrightAttachment       , XmATTACH_FORM,
	 XmNrightOffset           , NICESPACE,
	 XmNleftOffset            , NICESPACE,
	 XmNcolumns               , 80,
	 XmNrows                  , 45,
	 XmNcursorPositionVisible , FALSE,
	 XmNeditMode              , XmMULTI_LINE_EDIT,
	 XmNeditable              , FALSE,
	 XmNfontList              , GetFont("8x13",XtDisplay(top)),
	 XmNforeground            , GetColor("black",top),
	 XmNscrollHorizontal      , FALSE,
	 XmNscrollLeftSide        , TRUE,
	 NULL);
      
      gethom (pathname, &pathlen); /* SPHOME variable for manual page */
      
      strcpy (manpage, pathname);
      strcat (manpage, "/");
      strcat (manpage, MANPAGE);
      
      if ((fd = open(manpage, O_RDONLY, 0)) == -1)
      {
	 errorMessage("error in displayMan (open man)");
	 return -1;
      }
      
      if ((fsize = lseek(fd, 0L, SEEK_END)) == -1)
      {
	 errorMessage("error in displayMan (seek end)");
	 close(fd);
	 return -1;
      }
      
      if (lseek(fd, L_SET, 0L) == -1)
      {
	 errorMessage("error in displayMan (seek start)");
	 close(fd);
	 return -1;
      }
      
      if ((text = (char *)malloc((size_t) fsize+1)) == NULL)
      {
	 errorMessage("error allocating memory in displayMan");
	 close(fd);
	 return -1;
      }
      
      if (read(fd, text, fsize) != fsize)
      {
	 errorMessage("error reading text in displayMan");
	 close(fd);
	 return -1;
      }
      text[fsize] = '\0';
      close(fd);
      
      for (i = j = 0; i < fsize; i++)
      {
	 if (text[i] == '_' && text[i+1] == 0x08)
	   i++;
	 else
	   text[j++] = text[i];
      }
      text[j] = '\0';
      
      XmTextSetString(widgar[sepManText], text);
   }
   return 0;
}

/* main */

int main(int argc,char *argv[])
{
   static String fallback_resources[] =
   {    "*highlightThickness: 0",
	"*background: gray",
	"*drawArea*background: white",
	
/* Colours for display changed into somewhat more conservative ones
 * however, these don't give you a terrible headache after several minutes,
 * which the original did to me:
 * 
         "*background: medium turquoise",
	 "*drawArea*background: turquoise",
 * 
 * and otherwise: ugly, ugly, ugly ( Marcel Rutten, 17-12-96 )
 */
	"*secondaryColorSetId: 8",
	"*primaryColorSetId: 8",
	NULL
   };
   
   int istart, irotat, ioutp, itime;
   
   top = XtVaAppInitialize(&apco, CLASS_NAME, NULL, 0,
			   &argc, argv,
			   fallback_resources,
			   XmNtitle, "Sepview - sepran plot interface",
			   XmNminAspectX,   1, XmNminAspectY,   1,
			   XmNmaxAspectX,   4, XmNmaxAspectY,   3,
			   XmNminHeight,  200, XmNminWidth,   100,
			   NULL);
   
   istart = 110100;
   irotat = 1;
   ioutp = -1;
   itime = 0;
   ifirst = 0;
   
   initmd ();
   start  (&istart, &irotat, &ioutp, &itime);
   
   createAllWidgets(top);

   dpy = XtDisplay(top);
   win = XtWindow(widgar[sepDrawArea]);

   XtManageChild(widgar[sepMainWindow]);
   XtRealizeWidget(top);

   setNiceColors(widgar,top);

   initSepview(argc, argv);

   if (playing)
   {
      XmString str = XmStringCreate("STOP", XmSTRING_DEFAULT_CHARSET);
      XtVaSetValues(widgar[sepPlayButton],
		    XmNlabelString, str,
		    XmNsensitive, TRUE,
		    NULL);
      
      XmStringFree(str);
      
      XtAppAddWorkProc(apco, (XtWorkProc) scanbutton, (XtPointer) &playing);
   }
   
   XtAppMainLoop(apco);
   return 0;
}

