      subroutine sepmumps ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a linear system of equation by MUMPS
!     Old subroutine prmumps
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
!     mumps
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: isendarr(5), istop

!                    they are not
!     isendarr       this array is distributed over all processors
!                    It is sent by processor 0 (inodenr=1)
!                    Contents:
!                    1: istop  (if -1 the process is stopped)
!                    2: jmethod
!                    3: iseqsolut
!                    4: keep_matrix
!     istop          See isendarr(1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBCASTMPI    Wrapper for MPI_BCAST
!     INSTOP         Stop the program
!     PRININ         print 1d integer array
!     SEPMUMPSMAT    Solve system of linear equations by a direct method using
!                    mumps
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmumps' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmumps'
         write(irefwr,*) 'skipmumps', skipmumps
         write(irefwr,1) 'inodenr, iseqsolut', inodenr, iseqsolut
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( skipmumps ) then

!     --- Skip mumps, hence this is a slave

         if ( debug ) write(irefwr,*) 'before inireceivempi'
         call inibcastmpi ( isendarr, 5, SPINT )

         if ( debug ) then
            write(irefwr,*) 'after inireceivempi'
            call prinin ( isendarr, 5, 'isendarr' )
         end if  ! ( debug )

         istop = isendarr(1)
         iseqim = isendarr(2)
         iseqsolut = isendarr(3)
         keep = isendarr(4)
         A%newmumps = isendarr(5)

         if ( istop<0 ) call instop
         go to 500

      else

         if ( inodenr==1 ) then

!        --- Parallel mumps, send information to slaves

            isendarr(1) = 1
            isendarr(2) = A%iseqim
            isendarr(3) = iseqsolut
            isendarr(4) = keep
            isendarr(5) = A%newmumps

            if ( debug ) then
               write(irefwr,*) 'before inibcastmpi'
               call prinin ( isendarr, 5, 'isendarr' )
            end if  ! ( debug )

            call inibcastmpi ( isendarr, 5, SPINT )
            if ( debug ) write(irefwr,*) 'after inibcastmpi'

         end if  ! ( inodenr==0 )

      end if  ! ( skipmumps )

      if ( debug ) write(irefwr,1) 'nusol, nbound', nusol, nbound

!     --- Decomposition and solution of system of equations

500   call sepmumpsmat ( A, usol, rhsd )
      if ( skipmumps ) go to 1000

   !   if ( debug ) call prinrl ( usol, nrusol, 'usol' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmumps'

      end if  ! ( debug )
      call erclos ( 'sepmumps' )

      end
