      subroutine seppetscsol2 ( Amat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sets the type of preconditioner to be used by petsc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulesolve
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: Amat

!     Amat          i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PCFACTORSETLEVELS  See petsc manual
!     PCFACTORSETMATSOLVERPACKAGE  See petsc manual
!     PCSETTYPE      See petsc manual
!     PETSCOPTIONSSETVALUE  See petsc manual
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscsol2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscsol2'
         write(irefwr,1) 'iprecpetsc, printlev', iprecpetsc, Amat%iprint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( iprecpetsc )
         case(0)
            call PCSetType ( seppc, 'none', ierrpetsc )
         case(3)
            call PCSetType ( seppc, 'ilu', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'ILU preconditioner'
            if ( num_levels>0 ) &
                call pcfactorsetlevels ( seppc, num_levels, ierrpetsc )
         case(2)
            call PCSetType ( seppc, 'eisenstat', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'eisenstat preconditioner'
         case(6)
            if ( parallel ) then
               call PCSetType ( seppc, 'bjacobi', ierrpetsc )
               if ( Amat%iprint>0 ) write(irefwr,*) 'block jacobi preconditioner'
            else
               call PCSetType ( seppc, 'ilu', ierrpetsc )
               if ( Amat%iprint>0 ) write(irefwr,*) 'ILU preconditioner'
               if ( num_levels>0 ) &
                   call pcfactorsetlevels ( seppc, num_levels, ierrpetsc )
            end if  ! ( parallel )
          case(8)
            call PCSetType ( seppc, 'jacobi', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'jacobi preconditioner'
         case(9)
            call PCSetType ( seppc, 'bjacobi', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'block jacobi preconditioner'
         case(10)
            call PCSetType ( seppc, 'sor', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'sor preconditioner'
         case(11)
            call PCSetType ( seppc, 'icc', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'icc preconditioner'
         case(12)
            call PCSetType ( seppc, 'asm', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'asm preconditioner'
         case(13)
            call PCSetType ( seppc, 'gamg', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'gamg preconditioner'
         case(14)
            call PCSetType ( seppc, 'lu', ierrpetsc )
! not in version 3.11.3            
!            call PCFactorSetMatSolverPackage (seppc, 'mumps', ierrpetsc )
!  version 3.5.3            
!            call PetscOptionsSetValue('-mat_mumps_icntl_7', '2', &
!                                    ierrpetsc )
!           call PetscOptionsSetValue('-mat_mumps_icntl_14', '400', &
!                                       ierrpetsc )
!  version 3.7.7        
!            call PetscOptionsSetValue( PETSC_NULL_OBJECT,'-mat_mumps_icntl_7', &
!                                       '2', ierrpetsc )
!            call PetscOptionsSetValue( PETSC_NULL_OBJECT, &
!                                      '-mat_mumps_icntl_14', '400', ierrpetsc )
!  version 3.11.3      
            call PetscOptionsSetValue( PETSC_NULL_OPTIONS,'-mat_mumps_icntl_7', &
                                       '2', ierrpetsc )
            call PetscOptionsSetValue( PETSC_NULL_OPTIONS, &
                                      '-mat_mumps_icntl_14', '400', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'mumps lu preconditioner'
         case(15)
            call PCSetType ( seppc, 'ml', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'ml preconditioner'
         case(16)
            call PCSetType ( seppc, 'hypre', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'hypre preconditioner'
      end select  ! case ( iprecpetsc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppetscsol2'

      end if  ! ( debug )
      call erclos ( 'seppetscsol2' )

      end subroutine seppetscsol2
