      subroutine sepsolvepetsc ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call petsc linear solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    or complex (2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPPETSCSOL    Actual call to petsc solver
!     SEPPETSCVECS   Create right-hand-side vector for petsc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvepetsc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvepetsc'
         call prinrl ( rhsd, A%neq, 'rhsd' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call seppetscvecs ( rhsd, A%neq, kprobhglob, nusolglob )

!     --- solve

      call seppetscsol ( A, usol, rhsd )
      if ( debug ) call prinrl ( usol, A%neq, 'usol' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsolvepetsc'

      end if  ! ( debug )
      call erclos ( 'sepsolvepetsc' )

      end subroutine sepsolvepetsc
