      subroutine toaddpetsc1 ( rowindex, nrow, relemmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add part of element matrix corresponding to real unknowns
!     to petsc matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     petsc
! **********************************************************************
!
!                       MODULES USED
!
      use seppetscmod   ! defines all petsc stuff
      use sepmodulebuild
      use sepmodulekprob
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrow, rowindex(nrow)
      double precision, intent(in) :: relemmt(icount,icount)

!     nrow           i    Number of rows of element matrix corresponding to
!                         non-prescribed unknowns
!     relemmt        i    Element matrix
!     rowindex       i    Contains the sequence numbers of the free unknowns
!                         in the element in petsc style (i.e. starting from 0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, jcount, kcount, inusol, jnusol, nrusolglob
      double precision :: elemmtcp(nrow,nrow)

!     elemmtcp       Copy of element matrix with removal of prescribed
!                    unknowns
!     i              Counting variable
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jcount         Counter
!     jnusol         Position of unknown in array USOL
!     kcount         Counter
!     nrusolglob     Global number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MATSETVALUES   Inserts or adds a block of values into a matrix.
!                    These values may be cached, so MatAssemblyBegin()
!                    and MatAssemblyEnd() MUST be called after all calls
!                    to MatSetValues() have been completed.
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddpetsc1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddpetsc1'
         write(irefwr,1) 'inodenr', inodenr
         write(irefwr,1) 'icount, nrow, iseqmat', icount, nrow, iseqmat
         write(irefwr,1) 'ielem, nrusol', ielem, nrusol
         call prinin ( rowindex, nrow, 'rowindex' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount = 0
      nrusolglob = nrusol+ninterface
      do i = 1, icount

         inusol=indexren(i)  ! sequence number of row
         if ( inusol<=nrusolglob ) then

!        --- Real unknown

            jcount = jcount+1
            kcount = 0
            do j = 1, icount

               jnusol=indexren(j)  ! sequence number of column
               if ( jnusol<=nrusolglob ) then

!              --- Real unknown

                  kcount = kcount+1
                  elemmtcp(jcount,kcount) = relemmt(i,j)

               end if  ! ( jnusol<=nrusol )

            end do  ! j = 1, icount

         end if  ! ( inusol<=nrusol )

      end do  ! i = 1, icount

      if ( debug ) call prinrl1 ( elemmtcp, nrow, nrow, 'elemmtcp' )
      call MatSetValues ( A(iseqmat), nrow, rowindex, nrow, rowindex, &
                          elemmtcp, ADD_VALUES, ierrpetsc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddpetsc1'

      end if  ! ( debug )
      call erclos ( 'toaddpetsc1' )

      end subroutine toaddpetsc1
