      subroutine to0017 ( indprf, nunkp, ivectr, kmesh, kprob, jcount )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.8  date 12-03-1999 Extension with type 129
!        version  2.7  date 12-12-1998 Correction for type 126
!        version  2.6  date 06-05-1998 New definition kelgrp
!        version  2.5  date 25-01-1997 Extension with type 126
!        version  2.4  date 19-08-1993 Allow zero unknowns in element
!        version  2.3  date 22-12-1992 improved error report; expl. decl.
!        version  2.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.1  date 22-02-1991 minor correction errint
!        version  2.0  date 08-02-1991 Deletion of parameter NAME
!        version  1.1  date 17-03-1989 Extension for more problems
!        version  1.0  date 25-06-1984
!
!   copyright (c) 1984-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   determine number of degrees of freedom jcount in an element
!   of ielgrp for existing ivectr (ivctr(2)=110,115,116,126,129)
!   (for some vector jcount=icount in common /cactl/)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cactlf90'
      include 'SPcommon/celintf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprf, nunkp, ivectr(*), kmesh(*), kprob(*), jcount

!     indprf     i    indication if kprob part f has been filled (>0) or not (0)
!     ivectr     i    standard sepran array denoting usol or vector of special
!                     structure defined in nodal points
!     jcount     o    Number of quantities to be stored in element vector
!     kmesh      i    Standard sepran array containing information of the mesh
!     kprob      i    Standard sepran array, containing information of
!                     the problem
!     nunkp      i    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kelma, ivec

!     i          Counting variable
!     ivec       Sequence number of vector of special structure
!     kelma      Indicates if KMESH part a has been filled (>0) or not (0)
!                If kelma>0, kelma is the starting address of KMESH part a
!                in KMESH
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     200   First input vector is of illegal type
!     206   Input vector with illegal sequence number ivec
!     208   Solution vector (type=110) with zero degrees of freedom
!     209   Vector of special structure (type=115) with zero degrees of freedom
!     210   Vector defined per element (type=116) with zero degrees of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- test if vector exists (itypvc=110,115,116,126,129 allowed)

      call eropen ( 'to0017' )
      jcount=0
      select case ( ivectr(2) )

         case ( :109, 111:114, 117:125, 127, 128, 130:135, 137: )

!        --- error : illegal type of vector (unequal to 110,115,116,126,129)

            call errint ( ivectr(2), 1 )
            call errsub ( 200, 1, 0, 0)

         case ( 110 )

!        --- vector is of type usol , itypvc = 110

            if ( indprf==0 ) then

!           --- indprf = 0

               jcount=nunkp*inpelm

            else

!           --- number of degrees of freedom not constant

               if ( kmesh(15)==0 ) then

!              --- kelma = 0

                  kelma=(ielgrp-1)*inpelm

               else

!              --- kelma>0

                  kelma=kmesh(kmesh(15)+ielgrp-1)

               end if
               kelma=kelma+kprob(14)-1
               do i = 1, inpelm
                  jcount=jcount+kprob(kelma+i)
               end do

            end if
            if ( jcount<0 ) then
               call errint ( irelem, 1 )
               call errint ( ielgrp, 2 )
               call errsub ( 208, 2, 0, 0 )
            end if

         case ( 115 )

!        --- vector of special structure , itypvc = 115 , ivectr(4)=ivec

            ivec=mod(ivectr(4),1000)
            if ( ivec<=0 .or. ivec>kprob(6) ) then

!           --- error : vector with sequence number ivec not defined

               call errint ( ivec, 1 )
               call errint ( kprob(6), 2 )
               call errsub ( 206, 2, 0, 0 )
               go to 1000

            end if
            if ( indprf==0 ) then

!           --- error : number of degrees of freedom = 0

               jcount=nunkp*inpelm

            else

!           --- number of degrees of freedom not constant

               if ( kmesh(15)==0 ) then

!              --- kelma = 0

                  kelma=(ielgrp-1)*inpelm

               else

!              --- kelma>0

                  kelma=kmesh(kmesh(15)+ielgrp-1)

               end if
               kelma=kelma+(ivec-1)*kprob(39)+kprob(18)-1
               do i=1,inpelm
                  jcount=jcount+kprob(kelma+i)
               end do

            end if

            if ( jcount<0 ) then
               call errint ( ivec, 1 )
               call errint ( irelem, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 209, 3, 0, 0 )
            end if

         case ( 116, 136 )

!        --- vector of special structure is defined per element , itypvc = 116

            jcount=nunkp
            if ( jcount==0 ) then
               call errint ( ielgrp, 1 )
               call errsub ( 210, 1, 0, 0 )
            end if

         case ( 126 )

!        --- vector of special structure is defined per element per node, &
!            itypvc = 126

            jcount=nunkp*inpelm
            if ( jcount==0 ) then
               call errint ( ielgrp, 1 )
               call errsub ( 210, 1, 0, 0 )
            end if

         case ( 129 )

!        --- vector of special structure is defined per element per integration
!            point, itypvc = 129

            jcount=nunkp*m
            if ( jcount==0 ) then
               call errint ( ielgrp, 1 )
               call errsub ( 210, 1, 0, 0 )
            end if

      end select  ! case ( ivectr(2) )

1000  call erclos ( 'to0017' )
      end
