      subroutine to0022 ( ivec1, ivec2, iseq1, iseq2 )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  3.1  date 30-08-2006 New definition of cpost5
!        version  3.0  date 30-11-2002 Extra parameters iseq, iseq2
!        version  2.4  date 08-11-1999 Allow type 127
!        version  2.3  date 17-08-1999 Remove some error messages
!        version  2.2  date 16-07-1994 New definition of ivec1(3)
!        version  2.1  date 12-04-1993 Better error messages
!        version  2.0  date 06-10-1991 New parameter list
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 22-08-1984
!
!   copyright (c) 1984-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     test if vectors are of equal type
!
! **********************************************************************
!
!                       KEYWORDS
!
!     compare
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1(*), ivec2(*), iseq1, iseq2

!     iseq1      i    Sequence number of first input vector in isol
!     iseq2      i    Sequence number of second input vector in isol
!     ivec1      i    vectors to be compared (must be of equal type)
!     ivec2      i    vectors to be compared (must be of equal type)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nch, ih1, ih2

!     ih1            Help variable to store a constant temporarily
!     ih2            Help variable to store a constant temporarily
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     201   Two SEPRAN vectors are not of equal type
!     202   One of the input vectors is real the other one complex
!     759   Second input vector is of illegal type
!    2103   Length of two input vectors is not the same
!    2370   First input vector is of illegal type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- tests

      call eropen ( 'to0022' )
      if ( ivec1(2)/=110 .and. ivec1(2)/=115 .and. &
           ivec1(2)/=116 .and. ivec1(2)/=119 .and. &
           ivec1(2)/=127 ) then
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), 1 )
         call errint ( iseq1, 1 )
         call errint ( ivec1(2), 2 )
         call errsub ( 2370, 2, 0, 1 )
      end if
      if ( ivec2(2)/=110 .and. ivec2(2)/=115 .and. &
           ivec2(2)/=116 .and. ivec2(2)/=119 .and. &
           ivec1(2)/=127 ) then
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), 1 )
         call errint ( iseq2, 1 )
         call errint ( ivec2(2), 2 )
         call errsub ( 759, 2, 0, 1 )
      end if
      if ( ivec1(2)==116 .and. ivec2(2)/=116 .or. &
           ivec1(2)/=116 .and. ivec2(2)==116 .or. &
           ivec1(2)==119 .and. ivec2(2)/=119 .or. &
           ivec1(2)/=119 .and. ivec2(2)==119 .or. &
           ivec1(2)==127 .and. ivec2(2)/=127 .or. &
           ivec1(2)/=127 .and. ivec2(2)==127 ) then
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), 1 )
         call errint ( iseq1, 1 )
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), 2 )
         call errint ( iseq2, 2 )
         call errint ( ivec1(2), 3 )
         call errint ( ivec2(2), 4 )
         call errsub ( 201, 4, 0, 2 )
      end if
      if ( mod(ivec1(3),10)/=mod(ivec2(3),10) ) then
         if ( mod(ivec1(3),10)==0 ) then
            ih1 = 1
            ih2 = 3
            call errchr ( 'real', 2 )
            call errchr ( 'complex', 4 )
         else
            ih1 = 3
            ih2 = 1
            call errchr ( 'real', 4 )
            call errchr ( 'complex', 2 )
         end if
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), ih1 )
         call errint ( iseq1, ih1 )
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), ih2 )
         call errint ( iseq2, ih2 )
         call errsub ( 202, 2, 0, 4 )
      end if
      if ( ivec1(5)/=ivec2(5) ) then
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), 1 )
         call errint ( iseq1, 1 )
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), 2 )
         call errint ( iseq2, 3 )
         call errint ( ivec1(5), 2 )
         call errint ( ivec2(5), 4 )
         call errsub ( 2103, 4, 0, 2 )
      end if
      call erclos ( 'to0022' )
      end
