      subroutine topq03 ( kmesh, nplsub, nelem, neleml )

! ======================================================================
!
!        programmer   Guus Segal
!
!        version 1.1    date   04-05-1994 Add possibility line elements
!        version 1.0    date   18-01-1993
!
!
!
!   copyright (c) 1993-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!     DESCRIPTION
!
!  Check if a three-dimensional mesh of surface elements consists of quads
!  only or that a subdivision into triangular elements must be made
!  The number of elements of the subdivided mesh is computed as well as the
!  number of points per element in the new mesh. Compute also the number
!  of line elements.
! ********************************************************************
!
!     KEYWORDS
!
!     3d
!     mesh
!     subdivision
!     surface
!
! ********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS

      implicit none
      integer kmesh(*), nelem, neleml, nplsub

!     kmesh   i    Standard SEPRAN array containing information of the mesh
!     nelem   o    Number of elements in the new submesh
!     neleml  o    Number of line elements to be used
!     nplsub  o    Number of points in the submesh consisting of elements
!                  with vertex nodes only
!
! ********************************************************************
!
!     COMMON BLOCKS
!
! ********************************************************************
!
!     LOCAL PARAMETERS

      integer ishape, inelem, kelmf, kelmg, ielgrp, nelgrp
      logical triang

!     ielgrp  element group number
!
!     inelem  Number of elements in a specific element group
!
!     ishape  Shape number of elements
!
!     kelmf   Indicates if KMESH part f has been filled (>0) or not (0)
!             If kelmf>0, kelmf is the starting address of KMESH part f in KMESH
!
!     kelmg   Indicates if KMESH part g has been filled (>0) or not (0)
!             If kelmg>0, kelmg is the starting address of KMESH part g in KMESH
!
!     nelgrp  Number of element groups
!
!     triang  Indicates if triangular elements have been found (true) or not
!
! ********************************************************************
!
!     SUBROUTINES CALLED
!
! ********************************************************************
!
!     I/O
!
! ********************************************************************
!
!     ERROR MESSAGES
!
! ********************************************************************
!
!     PSEUDO CODE
!
!
!     triangles := false
!     For ielgrp := 1 (1) nelgrp do
!        ishape := kmeshf(ielgrp)
!        if ( ishape = 3, 4 or 7 ) then
!           triangles := true
!     nelem := 0
!     neleml:= 0
!     For ielgrp := 1 (1) nelgrp do
!        ishape := kmeshf(ielgrp)
!        if ( ishape = 1 or 2 ) then
!           neleml := neleml + kelmg(ielgrp)
!        else if ( ishape = 3, 4 or 7 ) then
!           nelem  := nelem + kelmg(ielgrp)
!        else if ( ishape = 5 or 6 ) then
!           if ( triangles ) then
!              nelem := nelem + kelmg(ielgrp)*2
!           else ( quadrilaterals )
!              nelem := nelem + kelmg(ielgrp)
!
! ======================================================================
!
!     Check if there are triangles

      triang = .false.
      nelgrp = kmesh(5)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

      do 100 ielgrp = 1, nelgrp

         ishape = kmesh(kelmf+ielgrp-1)
         if ( ishape.eq.3 .or. ishape.eq.4 .or. ishape.eq.7 ) &
              triang = .true.

100   continue

!     Compute nelem and set nplsub

      if ( triang ) then
         nplsub = 3
      else
         nplsub = 4
      end if

      nelem  = 0
      neleml = 0

      do 200 ielgrp = 1, nelgrp

         ishape = kmesh(kelmf+ielgrp-1)
         inelem = kmesh(kelmg+ielgrp-1)

         if ( ishape.eq.1 .or. ishape.eq.2 ) then

            neleml = neleml + inelem

         else if ( ishape.eq.3 .or. ishape.eq.4 .or. ishape.eq.7 ) then

            nelem = nelem + inelem

         else if ( ishape.eq.5 .or. ishape.eq.6 ) then

            if ( triang ) then
               nelem = nelem + inelem*2
            else
               nelem = nelem + inelem
            end if

         end if

200   continue

      end
